/*
 * cgitest.c -- CGI Test example for the GoAhead WebServer
 *
 * Copyright (c) GoAhead Software Inc., 1995-2000. All Rights Reserved.
 *
 * See the file "license.txt" for usage and redistribution license requirements
 */

/******************************** Description *********************************/

/*
 *	Tests the CGI environment variables generated by the GoAhead Webserver
 */
 
/********************************* Includes ***********************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>


#define DEBUG TRUE              

/******** Global veriable                    */
char			fname[81];

char *getfilename(char *fname){

	char *result, *ptr, name[81];
        int i;

        ptr = result = fname;
        
        while ((ptr = strstr(ptr, "\\")) != NULL ) {
             ptr++;
             result = ptr;
        } 
        ptr = result;
        
        while ((ptr = strstr(ptr, "/")) != NULL ) {
             ptr++;
             result = ptr;
        } 

        i = 0;
        while ( result[i] != '"') {
              i++;
        }
        result[i] = '\0';

        return result;
}

/***********************************************************
Function to read n characters from standard input
breaks of encounters newline.
*/
int readStdin( char charbuf[], int intbuf[], int n){
     int ch, j;

     j = 0;
     while ( j < n) {
           ch = getc(stdin);
           if (ch == EOF) {
              break;
           }
           if (ch == '\n') {
              charbuf[j++] = '\n';
              break;
           }
           intbuf[j] = ch;
           charbuf[j] = ch;
           j++;
     }
           intbuf[j] = '\0';
           charbuf[j] = '\0';
     return (j-1);
}

/*********************************************************
Function to write characters to datafile
*/

void writeDatafile( FILE *fptr, int intbuf[], int n) {
     int j;

     j = 0;
     while ( j < n) {
          fputc(intbuf[j++], fptr);
     }

}

/******************************************************************************/
/*
 * Compare strings, ignoring case:  normal strcmp return codes.
 *
 *	WARNING: It is not good form to increment or decrement pointers inside a
 *	"call" to tolower et al. These can be MACROS, and have undesired side
 *	effects.
 */

int strcmpci(char *s1, char *s2)
{
	int		rc;

	if (s1 == NULL || s2 == NULL) {
		return 0;
	}

	if (s1 == s2) {
		return 0;
	}

	do {
		rc = tolower(*s1) - tolower(*s2);
		if (*s1 == '\0') {
			break;
		}
		s1++;
		s2++;
	} while (rc == 0);

	return rc;
}

/*************************************************************************/
/*
 *	gogetenv wraps getenv so that (NULL) is returned if the string is
 *	NULL.
 */

char *gogetenv(char *varname)
{
	char *result;

	if ((result = getenv(varname)) == NULL) {
		result = "(NULL)";
	}
	return result;
}

/*************************************************************************/
/*
 *	Convert a two-char hex string into the char it represents
 */

char x2c(char *what) 
{
   register char digit;

   digit = (what[0] >= 'A' ? ((what[0] & 0xdf) - 'A') + 10 : (what[0] - '0'));
   digit *= 16;
   digit += (what[1] >= 'A' ? ((what[1] & 0xdf) - 'A') + 10 : (what[1] - '0'));

   return digit;
}

/*************************************************************************/
/*
 *	Reduce any %xx escape sequences to the characters they represent
 */

void unescape_url(char *url) 
{
    register int i,j;

    for(i = 0, j = 0; url[j]; ++i,++j) {
        if((url[i] = url[j]) == '%') {
            url[i] = x2c(&url[j + 1]) ;
            j += 2 ;
        }
    }

    url[i] = '\0' ;
}

/*************************************************************************/
/*                                                                     
 *	getCGIvars()	Routine to read CGI input variables into an     
 *					array of strings.
 *
 *	Written in 1996 by James Marshall, james@jmarshall.com, except
 *	that the x2c() and unescape_url() routines were lifted directly
 *	from NCSA's sample program util.c, packaged with their HTTPD.
 *
 *	For the latest, see http://www.jmarshall.com/easy/cgi/ .
 *
 *	Read the CGI input and place all name/val pairs into list.
 *	Returns list containing name1, value1, name2, value2, ... , NULL
 */

char **getCGIvars() 
{
	register int	i;
	char			*request_method, *cgiinput, *nvpair, *eqpos,
				*content_type, *bptr, *boundary, *fptr,
				*end_of_input;
	int			content_length, paircount, filepresent,
				c, EndofInput , j, handleTransition,
                                end_of_file_data, charsRead;
        char			**cgivars, **pairlist;
	char			mybuf[81], fullpathfilename[81];
        int 			intbuf[81];
        FILE                    *fout, *datafile;

/*      Set flag to false                  */
        EndofInput = 0;

/*
 *	Depending on the request method, read all CGI input into cgiinput
 *	(really should produce HTML error messages, instead of exit()ing)
 */

	filepresent = 0;
    	request_method = gogetenv("REQUEST_METHOD") ;

	if (!request_method) {
		request_method = "NOT-CGI";
	}

    if (!strcmp(request_method, "GET") || !strcmp(request_method, "HEAD")) {
        cgiinput= strdup(gogetenv("QUERY_STRING"));
    } else if (!strcmp(request_method, "POST")) {

        if (!(content_length = atoi(gogetenv("CONTENT_LENGTH")))) {
            printf("getcgivars(): No Content-Length was sent with the POST request.\n");
            return NULL;
        }

        if (!(cgiinput = (char *) malloc(content_length + 1))) {
            printf("getcgivars(): Could not malloc for cgiinput.\n") ;
            return NULL;
        }

        content_type = gogetenv("CONTENT_TYPE") ;

/*	IF WE ARE GETTING REQUEST TO UPLOAD FILE    */

	if ( strstr(content_type,"multipart/form-data") != NULL) {

#ifdef DEBUG 
           fout = fopen("../cgistuff.txt","wb");
#endif

		bptr = strstr(content_type,"boundary=");
		bptr += 9;
                boundary = (char *) malloc (strlen(bptr) + 2);
                sprintf(boundary,"--%s", bptr);

/*      Here we got pointer to actual boundary string     */

/*      Build end of input string by padding -- to boundary   */
                end_of_input = (char *) malloc (strlen(bptr) + 5);
                end_of_input[0] = '\0';
		sprintf(end_of_input,"%s--", boundary);

           handleTransition = 1;


	   while (EndofInput == 0) {
              mybuf[0] = '\0';
              charsRead = readStdin( mybuf, intbuf, sizeof(mybuf)); 
#ifdef DEBUG 
		   fprintf(fout,"charsread %d\n", charsRead);
#endif
              if (charsRead < 0) {
                 EndofInput = 1;
                 continue;
              }

#ifdef DEBUG 
		   fprintf(fout,"read>");
                 /*  fwrite(mybuf, strlen(mybuf), 1, fout); */
                   fwrite(intbuf, charsRead, 1, fout); 
#endif

               if ( !strncmp(mybuf, boundary,strlen(boundary)) ) {
                  handleTransition = 1;
                  continue;
               }

/*	Handling transition to new field                  */
               if( handleTransition == 1) {
#ifdef DEBUG 
                 fprintf(fout,"Handling transition\n");
#endif

		  if ( (fptr = strstr(mybuf, "filename=")) != NULL) {
/*      At this point we have got a file name i.e. current field 
	contain file data. Need to skip two lines to reach 
	beginning of file data.				  */
		     sprintf(fname,"%s", getfilename(fptr+10)); 
		     sprintf(fullpathfilename,"../jbp/%s", fname); 

#ifdef DEBUG 
                 fprintf(fout,"File name :%s\n", fname);
#endif
                 datafile = fopen(fullpathfilename,"wb");

			filepresent = 1;
                  }
		  else {
			filepresent = 0;
		  }

               handleTransition = 0;
               } /* End of transition to new field    */

               if (filepresent == 0) {
	           continue;
                  } 
	       else {    /*  Read the next two lines and the file data   */
                    mybuf[0] = '\0';
                    charsRead = readStdin( mybuf, intbuf, sizeof(mybuf)); 
                    if (charsRead < 0) {
                      EndofInput = 1;
                      continue;
                    }
#ifdef DEBUG 
		   fprintf(fout,"read1>%s",mybuf);
#endif
                   mybuf[0] = '\0';
                   charsRead = readStdin( mybuf, intbuf, sizeof(mybuf)); 
                   if (charsRead < 0) {
                     EndofInput = 1;
                     continue;
                   }
#ifdef DEBUG 
		   fprintf(fout,"read2>%s",mybuf);
                   fflush(fout);
#endif
               end_of_file_data = 0;
               while (end_of_file_data < 1) {

                      mybuf[0] = '\0';
                      charsRead = readStdin( mybuf, intbuf, sizeof(mybuf)); 

#ifdef DEBUG 
		   fprintf(fout,"chars read>%d",charsRead);
#endif

                    if (!strncmp(mybuf, end_of_input, strlen(end_of_input)) ) {
#ifdef DEBUG 
                         fprintf(fout,"End of input\n");
#endif
	                 EndofInput == 1;
                         break;
                    }
                    else if (!strncmp(mybuf, boundary, strlen(boundary)) ) {
#ifdef DEBUG 
                         fprintf(fout,"\nread at end of input>%s",mybuf);
#endif
                         end_of_file_data = 1;
                    }
                    else {
#ifdef DEBUG 
                         fprintf(fout,"%s",mybuf);
#endif
                         fprintf(datafile,"%s",mybuf);     
/*                           writeDatafile( datafile, intbuf, charsRead);  */
                    }


               } /* End of file data    */

               }
              }  /* End of  data on standard input    */

#ifdef DEBUG 
           fclose(fout);
#endif
           fclose(datafile);
           }


    } else {
        printf("getcgivars(): unsupported REQUEST_METHOD\n") ;
        return NULL;
    }

/*
 *	Change all plusses back to spaces
 */
    for (i = 0; cgiinput[i]; i++) {
		if (cgiinput[i] == '+') {
			cgiinput[i] = ' ';
		}
	}

/*
 *	First, split on "&" to extract the name-value pairs into pairlist
 */
    pairlist = (char **) malloc(256 * sizeof(char **));
    paircount = 0 ;
    nvpair = strtok(cgiinput, "&");

    while (nvpair) {
        pairlist[paircount++] = strdup(nvpair);
        if (!(paircount % 256)) {
            pairlist= (char **) realloc(pairlist, (paircount + 256)
				* sizeof(char **));
		}

        nvpair = strtok(NULL, "&") ;
    }

/*	
 *	Terminate the pair list with NULL
 */
    pairlist[paircount] = 0;

/*
 *	Then, from the list of pairs, extract the names and values
 */
    cgivars = (char **) malloc((paircount * 2 + 1) * sizeof(char **));
    for (i = 0; i < paircount; i++) {
        if (eqpos = strchr(pairlist[i], '=')) {
            *eqpos = '\0' ;
            unescape_url(cgivars[i * 2 + 1] = strdup(eqpos + 1));
        } else {
            unescape_url(cgivars[i * 2 + 1] = strdup(""));
        }
        unescape_url(cgivars[i * 2] = strdup(pairlist[i]));
    }

/* 
 *	Terminate the cgi var list with NULL
 */
    cgivars[paircount * 2] = 0;	
    
/*
 *	Free anything that needs to be freed
 */
    free(cgiinput);

    for (i = 0; pairlist[i]; i++) {
		free(pairlist[i]);
	}

    free(pairlist);

/*
 *	Return the list of name-value strings
 */
    return cgivars;
    
}

/*************************************************************************/
/*
 *	Test program entry point
 */

int main(int argc, char *argv[])
{
	char	**pVars;
	int		loopCount = 1000;
	time_t	second;

/*
 *	Print the CGI response header, required for all HTML output.
 *	Note the extra \n, to send the blank line.
 */
    printf("Content-type: text/html\n\n") ;
/*	
 *	Print the HTML header to STDOUT.
 */
    printf("<html>\n");
    printf("<head><title>CGI Output</title></head>\n");
    printf("<body>\n");

    printf("<H1>CGI Test Program</H1>\n");
    printf("<P>This program displays the CGI Enviroment</P>\n");

/*	
 *	Print the CGI environment variables
 */
    printf("<H2>Environment Variables<A NAME=\"env\"></A></H2>\n");
    printf("<P>REQUEST METHOD = %s</P>\n", gogetenv("REQUEST_METHOD"));
    printf("<P>QUERY STRING = %s</P>\n", gogetenv("QUERY_STRING"));
    printf("<P>GATEWAY INTERFACE = %s</P>\n", gogetenv("GATEWAY_INTERFACE"));
    printf("<P>SCRIPT NAME = %s</P>\n", gogetenv("SCRIPT_NAME"));

    printf("<P>PATH TRANSLATED = %s</P>\n", gogetenv("PATH_TRANSLATED"));
    printf("<P>PATH INFO = %s</P>\n", gogetenv("PATH_INFO"));

    printf("<P>HTTP HOST = %s</P>\n", gogetenv("HTTP_HOST"));
    printf("<P>HTTP ACCEPT = %s</P>\n", gogetenv("HTTP_ACCEPT"));
    printf("<P>HTTP CONNECTION = %s</P>\n", gogetenv("HTTP_CONNECTION"));
    printf("<P>CONTENT LENGTH  = %s</P>\n", gogetenv("CONTENT_LENGTH"));
    printf("<P>CONTENT TYPE  = %s</P>\n", gogetenv("CONTENT_TYPE"));

    printf("<P>REMOTE USER = %s</P>\n", gogetenv("REMOTE_USER"));
    printf("<P>REMOTE HOST = %s</P>\n", gogetenv("REMOTE_HOST"));
    printf("<P>REMOTE ADDR = %s</P>\n", gogetenv("REMOTE_ADDR"));

    printf("<P>SERVER NAME = %s</P>\n", gogetenv("SERVER_NAME"));
    printf("<P>SERVER PORT = %s</P>\n", gogetenv("SERVER_PORT"));
    printf("<P>SERVER HOST = %s</P>\n", gogetenv("SERVER_HOST"));
    printf("<P>SERVER PROTOCOL = %s</P>\n", gogetenv("SERVER_PROTOCOL"));
    printf("<P>SERVER ADDR = %s</P>\n", gogetenv("SERVER_ADDR"));
    printf("<P>SERVER URL = %s</P>\n", gogetenv("SERVER_URL"));
    printf("<P>SERVER SOFTWARE = %s</P>\n", gogetenv("SERVER_SOFTWARE"));

/*	
 *	Print the parsed query string variables
 */
    printf("<H2>Query String Variables<A NAME=\"query\"></A></H2>\n");
    pVars = getCGIvars();
	if (pVars) {
		while (*pVars && *(pVars + 1)) {
			printf("<p>%s = <b>%s</b></p>\n", *pVars, *(pVars + 1));
			pVars += 2;
		}
	} else {
	    printf("<P>No query variables found</P>\n");
	}

/*	
 *	Print 1000 lines of text, 100 lines at a time.
 *	Pause for 5 seconds every 100 line chunk
 */

   printf( "<a href=\"http://gandalf.uccs.edu:8000/jbp/%s\">click
here to see %s</a>", fname, fname);
#ifdef PRINT_BIG_CHUNK_SLOWLY
    printf("<H2>Large chunk of meaningless text<A NAME=\"chunk\"></A></H2>\n");
	while (loopCount) {
		printf("<p>Meaningless line of text number %d.</p>\n", loopCount);
		if ((loopCount % 100) == 0) {
			second = time(NULL);
			while ((time(NULL) - second) < 5) {
			}
		}
		loopCount--;
	}
#endif

    printf("</body>\n");
    printf("</html>\n");

    return 0;
}

/*************************************************************************/
