/* boot.s
 * Startup code for OS-independent module
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: BootMgr
 * author: jdg 05/03/00
 * revisions:
 * $History: boot.s $
; 
; *****************  Version 4  *****************
; User: Jdguilfo     Date: 10/20/00   Time: 2:27p
; Updated in $/dev_1200/BoardSupport/BootFlash
; Merge Boot Manager & Flash Utility, add HPC
 * 
 * ------------------------------------------------------------------------
 */

	.data
	
/* Exported internal functions */

	.globl	_romInit	 /* start of system code */
	.globl	_sdata		 /* start of data */

/* externals */

	.extern	_romStart	/* system initialisation routine */

_sdata:
	.asciz	"dat"
	.balign	4


	.text
	.balign 4

_romInit:
	/* Initialize LED's */
	
	LDR	r1, L$_IXP1200_LED_ADDR
#ifndef LED_MEM_MAPPED
	
	/* 
	 * Set GPIO's to core ownership, outputs, and output a zero
	 * IXP1200_LED_ADDR is GPIO_DATA 
	 */
	
	MOV	r2, #(GPIO_OWN_CCCC | GPIO_OUTPUT(0xF))
	STR	r2, [r1,#-(IXP1200_LED_ADDR-IXP1200_GPIO_EN)]
#endif /* LED_MEM_MAPPED */
	MOV	r2, #0x30
	STR	r2, [r1]
	
	
	/*
	 * Set core clock speed in Power Manager PLL Configuration Register.
	 */

	LDR	r1, L$_IXP1200_PPL_CFG	/* PLL Configuration Reg*/
	MOV	r2, #CCCR_INIT_VALUE    /* 200 MHz for B0 */
	STR	r2, [r1]

	/*
	 * Put the CPU in its normal (fast) clock switching mode
	 */

	MCR	p15, 0, r14, c15, c1, 2
	
	/*
	 * Delay for a while to let the SDRAM clock PLL stabalize after
	 * the clock was changed above.
	 */
	
	MOV	r1, #0x40000
loop1:	SUBS	r1, r1, #1
	BNE	loop1

	/* 
	 * Program SRAM Controller
	 * Note that we need to change SRAM_FLASH_CONFIG *before* 
	 * SRAM_SLOW_CONFIG, otherwise, the timing parameters become 
	 * inconsistent 
	 */

	ADR	r1, L$_SRAM_CONFIG_PARAMS
	LDMIA	r1, {r2-r3}	/* get parameters from table */
	STR	r3, [r2]	/* Write SRAM CSR */
	

	LDR	r1, L$_IXP1200_LED_ADDR
	MOV	r2, #0x42
	STR	r2, [r1]
	/* Bring other functional units out of reset */

	LDR	r1, L$_IXP1200_RESET
	MOV	r2, #RESET_MICROENGINE
	STR	r2, [r1]

	/* Set up the SDRAM 
	 * Note that the SDRAM CSR must be the last one set
	 * r7 contains GPIO's, bit 3 
	 */
	
	ADR	r1, L$_SDRAM_CONFIG_PARAMS
	LDMIA	r1, {r2-r6}	/* get parameters from table */
	STMIA	r2, {r4-r6}	/* store MEMCTL0, MEMCTL1, MEMINIT */
	STR	r3, [r2,#-(IXP1200_SDRAM_MEMCTL0 - IXP1200_SDRAM_CSR)]
	
	/* 
	 * Read the Stack address and the boot rom address before we
	 * remap the MMU, so that we don't have to copy them into SDRAM 
	 */
	
	LDR	r3, L$_rStrtInRom
	LDR	sp, L$_STACK_ADDR
	MOV	fp, #0			/* zero frame pointer */

	LDR	r1, L$_IXP1200_LED_ADDR
	MOV	r2, #0x64
	STR	r2, [r1]
	/* jump to C entry point in ROM: routine - entry point + ROM base */

	MOV	pc, r3		/* L$_rStrtInRom, r0 = boot type */
	
/******************************************************************************/

	.balign	4
	

	/*
	 * PC-relative-addressable pointers - LDR Rn,=sym was (is?) broken
	 * note "_" after "$" to stop preprocessor performing substitution
	 */

L$_rStrtInRom:
	.long	ROM_TEXT_ADRS + _romStart - _romInit

L$_STACK_ADDR:
	.long	STACK_ADRS

L$_IXP1200_PPL_CFG:	
	.long IXP1200_PLL_CFG

L$_SRAM_CONFIG_PARAMS:	
	.long IXP1200_SRAM_CSR	/* CSR address */
	.long SRAM_CSR

	/* IXP1200_LED_ADDR is either in memory or is the same as GPIO_DATA */
	
L$_IXP1200_LED_ADDR:	
	.long IXP1200_LED_ADDR
	
L$_IXP1200_GPIO_EN:
	.long IXP1200_GPIO_EN

L$_SDRAM_CONFIG_PARAMS:
	.long IXP1200_SDRAM_MEMCTL0  /* SDRAM MEMCTL0 address */
	.long SDRAM_CSR
	.long SDRAM_MEMCTL0
	.long SDRAM_MEMCTL1
	.long SDRAM_MEMINIT
	
L$_IXP1200_RESET:	
	.long IXP1200_RESET
