/* chip.h
 * Definition of chip related constants
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: chip.h $
 * 
 * *****************  Version 5  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 10/26/00   Time: 3:38p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge serial and HPC versions into one version, see bootMgr.txt for
 * details.
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 2:27p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge Boot Manager & Flash Utility, add HPC
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:37p
 * Created in $/dev_1200/BoardSupport/FlashUtil
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 * This code defines a simple malloc interface using a fixed pool. It is
 * intended for OS-independent code where is no native malloc.
 * ------------------------------------------------------------------------
 */

#ifndef __CHIP_H__
#define __CHIP_H__

#ifndef _ASMLANGUAGE
typedef int UINT32;
typedef unsigned char UINT8;
typedef enum {
    OK,
    ERROR
} STATUS ;
#endif

#define VERSION "1.2.76"

#define IXP1200_REG_WRITE(a,val) (*(volatile UINT32 *)(a) = (val))
#define IXP1200_REG_READ(a,val) ((val) = *(volatile UINT32 *)(a))
#define IXP1200_REG_READ1(a) (*(volatile UINT32*)(a))

/* Register addresses */

#define UART_SR  0x90003400    /* UART Status Register */
#define UART_CR  0x90003800    /* UART Control Register */
#define UART_DR  0x90003C00    /* UART Data Register */

/* bit definitions within UART_SR */

#define UART_TX_FULL  (1 << 7)  /* Transmit fifo full */
#define UART_TX_EMPTY (1 << 5)  /* Transmit fifo empty */
#define UART_TX_RDY   (1 << 2)  /* Transmit fifo ready (not full) */
#define UART_RX_RDY   (1 << 4)  /* Receive fifo ready (not empty) */

/* bit definitions within UART_CR */

#define UART_IRQ_SEL   (1 << 9)  /* IRQ select      (Rev B) */
#define UART_INT_CLR   (1 << 9)  /* Interrupt clear (Rev A) */
#define UART_TX_INT_EN (1 << 8)  /* Transmit interupt enable */
#define UART_EN        (1 << 7)  /* UART Enable */
#define UART_RX_INT_EN (1 << 4)  /* Receive interupt enable */

#define UART_BDR_OFFSET 16
#define UART_BDR_MASK (0x3FF << 16) /* Baud Rate Mask */

#define UART_DSS_5BIT (0 << 5)   /* Data size select */
#define UART_DSS_6BIT (1 << 5)
#define UART_DSS_7BIT (2 << 5)
#define UART_DSS_8BIT (3 << 5)

#define UART_SBS_1BIT (0 << 3)   /* Stop Bit Select */
#define UART_SBS_2BIT (1 << 3)

#define IXP1200_GPIO_EN      0x90001800 /* Address of GPIO_EN register */
#define IXP1200_GPIO_DATA    0x90001C00 /* Address of GPIO_DATA register */

#define GPIO_OWN_CCCC    (0<<4)        /* Core owns all GPIO */
#define GPIO_OWN_CCCF    (1<<4)        /* Fbi owns GPIO[0] */
#define GPIO_OWN_FFFC    (2<<4)        /* Fbi owns GPIO[3:1] */
#define GPIO_OWN_FFFF    (3<<4)        /* Fbi owns GPIO[3:0] */
#define GPIO_OUTPUT(mask) (mask)       /* Which GPIO's are outputs */

#define SRAM_CSR ((1 << 14) | /* 8 Mbyte Flash */ \
                  (1 <<  9))  /* 2 Mbyte sram */
#define IXP1200_SRAM_CSR          0x38000000 /* SRAM CSR */
#define SRAM_8MF_BIT    14
#define SRAM_4MF_BIT    13
#define SRAM_2MF_BIT    12

#define IXP1200_SDRAM_CSR     0xFF000000 /* SDRAM CSR */
#define IXP1200_SDRAM_MEMCTL0 0xFF000004 /* SDRAM timing info */

#define FLASH_WIDTH 4

#define LED_MEM_MAPPED

#ifdef LED_MEM_MAPPED
#define IXP1200_LED_ADDR   (0x38400000+0x108000)  /* Address of LED display */
#else
#define IXP1200_LED_ADDR   IXP1200_GPIO_DATA
#endif

/* SDRAM Config */
/* for 176.95 MHz */

#define SDRAM_EN_FTST   0 /* Enable FIFO RAM test */
#define SDRAM_EN_INT    0 /* Enable Parity Interrupt */
#define SDRAM_EN_IRQ    0 /* Select IRQ */
#define SDRAM_EN_FIQ    0 /* Select FIQ */
#define SDRAM_EN_CPE    0 /* Enable StrongARM Core parity error */
#define SDRAM_EN_PCIPE  0 /* Enable PCI parity error */
#define SDRAM_EN_UEPE   0 /* Enable Microengine parity error */
#define SDRAM_GENPAR    0 /* Enable parity generation */
#define SDRAM_EVENPAR   0 /* Generate even parity */
#define SDRAM_TF_BENDN  0 /* 1 = big endian mode */
#define SDRAM_INIT      1 /* Initialize SDRAM unit */
#define SDRAM_CSR ((SDRAM_EN_FTST   << 15) | (SDRAM_EN_INT    << 10) | \
                   (SDRAM_EN_IRQ    <<  9) | (SDRAM_EN_FIQ    <<  8) | \
                   (SDRAM_EN_CPE    <<  6) | (SDRAM_EN_PCIPE  <<  5) | \
                   (SDRAM_EN_UEPE   <<  4) | (SDRAM_GENPAR    <<  3) | \
                   (SDRAM_EVENPAR   <<  2) | (SDRAM_TF_BENDN  <<  1) | \
                   (SDRAM_INIT))

#define SDRAM_RFRSH         0x0566 /* Refresh Count */
#define SDRAM_BURSTL        0x8    /* Burst Length */
#define SDRAM_CASL          0x2    /* CAS Latency */
#ifdef SPECTACLE_ISLAND
#define SDRAM_ROW_ADR_WIDTH 0xD    /* Row Address Width */
#define SDRAM_COL_ADR_WIDTH 0x9    /* Column Address Width */
#else
#define SDRAM_ROW_ADR_WIDTH 0xC    /* Row Address Width */
#define SDRAM_COL_ADR_WIDTH 0x8    /* Column Address Width */
#endif
#define SDRAM_MEMCTL0 ((SDRAM_RFRSH << 16) | (SDRAM_BURSTL << 12) | \
                       (SDRAM_CASL  <<  8) | (SDRAM_ROW_ADR_WIDTH << 4) | \
                       (SDRAM_COL_ADR_WIDTH))

#define SDRAM_TRWT     0x1 /* Read/Write turnaround time */
#define SDRAM_TDPL     0x2 /* Data In to Precharge time */
#define SDRAM_TDQZ     0x3 /* DQM Data Out Disable latency */
#define SDRAM_TRC      0x6 /* Bank Cycle time */
#define SDRAM_TRRD     0x1 /* Bank to Bank delay time */
#define SDRAM_TRCD     0x1 /* RAS to CAS delay */
#define SDRAM_TRASMIN  0x4 /* Active Command Period */
#define SDRAM_TRP      0x1 /* Precharge Time */

#define SDRAM_MEMCTL1 ((SDRAM_TRWT << 28) | (SDRAM_TDPL << 24) | \
                       (SDRAM_TDQZ << 20) | (SDRAM_TRC  << 16) | \
                       (SDRAM_TRRD << 12) | (SDRAM_TRCD <<  8) | \
                       (SDRAM_TRASMIN << 4) | (SDRAM_TRP))

#define SDRAM_TRSC       0x1     /* Mode register set delay */
#define SDRAM_INIT_RFRSH 0x2     /* Initialization Refresh Count */
#define SDRAM_INIT_DLY   0x2292  /* Initialization Delay */
#define SDRAM_MEMINIT ((SDRAM_TRSC << 20) | (SDRAM_INIT_RFRSH << 16) | \
                       (SDRAM_INIT_DLY))

#define IXP1200_RESET                 (IXP1200_PCI_CSR_BASE + 0x07C)
#define RESET_MICROENGINE   0x0000003F   /* Reset 6 microengines */

#define SDRAM_PHYS_BASE  0xC0000000 /* Physical address of start of SDRAM */

/*
 * RESERVED_LOW_MEM_MB is the number of MBytes reserved from low SDRAM.
 * This memory will not be mapped to virtual address 0 and will be used
 * to communicate with the microgengines.
 */
#ifdef SPECTACLE_ISLAND
#define RESERVED_LOW_MEM_MB 112 /* Number of MBytes reserved from low SDRAM */
#else
#define RESERVED_LOW_MEM_MB 24  /* Number of MBytes reserved from low SDRAM */
#endif
#define RESERVED_LOW_MEM    (0x00100000 * RESERVED_LOW_MEM_MB)

#ifdef SPECTACLE_ISLAND
#define LOCAL_MEM_SIZE_ACTUAL      0x08000000  /* 128 Mbytes */ 
#else
#define LOCAL_MEM_SIZE_ACTUAL      0x02000000  /* 32 Mbytes */ 
#endif

#define IXP1200_PCI_CSR_BASE 0x42000000 /* Base of PCI Unit CSR's */
#define IXP1200_PCI_MEM_BAR           (IXP1200_PCI_CSR_BASE + 0x010)
#define IXP1200_PCI_IO_BAR            (IXP1200_PCI_CSR_BASE + 0x014)
#define IXP1200_PCI_DRAM_BAR          (IXP1200_PCI_CSR_BASE + 0x018)
#define IXP1200_OUTB_INT_MASK         (IXP1200_PCI_CSR_BASE + 0x034)
#define IXP1200_PCI_INT_LAT           (IXP1200_PCI_CSR_BASE + 0x03c)
#define IXP1200_DOORBELL              (IXP1200_PCI_CSR_BASE + 0x060)
#define IXP1200_DOORBELL_SETUP        (IXP1200_PCI_CSR_BASE + 0x064)
#define IXP1200_CSR_BASE_ADDR_MASK    (IXP1200_PCI_CSR_BASE + 0x0F8)
#define IXP1200_DRAM_BASE_ADDR_MASK   (IXP1200_PCI_CSR_BASE + 0x100)
#define IXP1200_SA_CONTROL            (IXP1200_PCI_CSR_BASE + 0x13C)
#define IXP1200_PCI_ADDR_EXT          (IXP1200_PCI_CSR_BASE + 0x140)
#define IXP1200_DBELL_PCI_MASK        (IXP1200_PCI_CSR_BASE + 0x150)
#define IXP1200_DBELL_SA_MASK         (IXP1200_PCI_CSR_BASE + 0x154)

/* 16 bit PCI registers */

#define IXP1200_PCI_COMMAND (IXP1200_PCI_CSR_BASE + 0x04)

#define IXP1200_SA_CONTROL_PNR (1 << 9)  /* PCI Not Reset bit of SA_CONTROL */
#define IXP1200_SA_CONTROL_PCF (1 << 31) /* PCI Centrl Function bit */

/* Define PCI_COMMAND bits */

#define IXP1200_PCI_COMMAND_IOSE (1 << 0) /* I/O Space Enable */
#define IXP1200_PCI_COMMAND_MSE  (1 << 1) /* Mem Space Enable */
#define IXP1200_PCI_COMMAND_ME   (1 << 2) /* Master Enable */
#define IXP1200_PCI_COMMAND_MWI  (1 << 4) /* Master Write And Invalidate Enable */
#define IXP1200_PCI_COMMAND_PEC  (1 << 6) /* Parity Error Control */
#define IXP1200_PCI_COMMAND_SE   (1 << 8) /* SERR# enable */

#define IXP1200_PLL_CFG      0x90000C00 /* PLL Configuration Reg */
#define CCCR_INIT_VALUE      17         /* 176.95 MHz */

#define HPC_CHANNEL 2

/* Boot Flags */
#define BOOT_DONT_INIT_PCI   (1 << 30)
#define BOOT_DONT_INIT_HPC   (1 << 29)
#define BOOT_IO_SERIAL       (1 << 27)
#define BOOT_IO_HPC          (1 << 26)

/* Flash Utility Magic cookie */
#define BOOTFLASH_COOKIE      0x4274466c
#define BOOTFLASH_COOKIE_ADDR 0x00000014

#endif /* ifndef __CHIP_H__ */
