/* flashMain.c
 * Main code for updating flash based on directions from host
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: flashMain.c $
 * 
 * *****************  Version 11  *****************
 * User: Jdguilfo     Date: 11/16/00   Time: 1:20p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Clean up 16-bit mode
 * 
 * *****************  Version 10  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 9  *****************
 * User: Jdguilfo     Date: 10/26/00   Time: 3:38p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge serial and HPC versions into one version, see bootMgr.txt for
 * details.
 * 
 * *****************  Version 8  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 2:35p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge bootmgr and flashutility into bootflash
 * 
 * *****************  Version 7  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 2:27p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge Boot Manager & Flash Utility, add HPC
 * 
 * *****************  Version 4  *****************
 * User: Tclancyx     Date: 6/13/00    Time: 10:52a
 * Updated in $/dev_1200/BoardSupport/FlashUtil
 * change version to 1.1.NoBldNum
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 5/10/00    Time: 4:42p
 * Updated in $/dev_1200/BoardSupport/FlashUtil
 * Add BootMgr feature
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 5/05/00    Time: 4:39p
 * Updated in $/dev_1200/BoardSupport/FlashUtil
 * Add 16-bit flash support
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:37p
 * Created in $/dev_1200/BoardSupport/FlashUtil
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 */

#include "chip.h"

#include "genericIO.h"
#include "flashUart.h"
#include "smalloc.h"
#include "utils.h"
#include "zlib.h"

#include "flashUtil.h" /* from VxWorks BSP */

#ifndef NULL
#define NULL ((void*)0)
#endif

#define NUM_CE 4

#define CBUFF_RAW_SIZE (8*1024*1024)
#define CBUFF_SIZE (CBUFF_RAW_SIZE + CBUFF_RAW_SIZE/20)
#define UBUFF_RAW_SIZE (8*1024*1024)
#define UBUFF_SIZE (UBUFF_RAW_SIZE + 0)

#define BAUD 38400

extern unsigned char flashType;
extern int flashModeSwap;
int flash32BitMode;

#define FLASH_BASE 0

unsigned char *cbuff = NULL;
unsigned char *ubuff = NULL;

struct INFO {
    UINT32 cpu_id;
    UINT32 width; /* Flash width: 16 or 32 */
    UINT32 size;  /* 2, 4, or 8 MB */
    UINT32 bank_size;
    UINT32 type[4];
} info;

void getInfo();

extern int sysCpuRev;
extern unsigned int bootFlags;

static int marquee[] = {0xF1, 0x1F};

void
endCmd()
{
    while ('\n' != genericInChar());
}

void 
outStrX(char *str, unsigned int x)
{
  genericOutStr(str);
  genericOutStr(" ");
  genericOutStr(hex(x, 8));
  genericOutStr("\n");
}

/* read 32 bits in little-endian format */
UINT32
read32()
{
    UINT32 val;
    val = genericInChar();
    val |= genericInChar() << 8;
    val |= genericInChar() << 16;
    val |= genericInChar() << 24;
    return val;
}

void 
handle_default(UINT32 cmd)
{
    outStrX("Error: Unknown cmd",cmd);
    endCmd();
}

/*
void
handle_loss()
{
    int i,j;
    extern char loss_buf[];
    extern int loss_buf_ptr;

    for (j=0, i=loss_buf_ptr;
         j<64;
         j++, i = ((i+1)&63)) {
        if ((j & 15) == 0)
            genericOutStr("*");
        genericOutStr(" ");
        genericOutStr(hex(loss_buf[i],2));
        if ((j & 15) == 15)
            genericOutStr("\n");
    }
}
*/

void 
handle_helo()
{
    genericOutStr("FlashUtil, Version " VERSION "\r\n");
}

void
handle_info()
{
    /* okin cpu_rev_ wi s t0 t1 t2 t3 */
    /*           11111111112222222222 */
    /* 012345678901234567890123456789 */

    int i;

    endCmd();
    genericOutStr("okin ");
    genericOutStr(hex(info.cpu_id, 8));
    genericOutStr(" ");
    genericOutStr(hex(info.width, 2));
    genericOutStr(" ");
    genericOutStr(hex(info.size, 1));
    for (i=0; i<4; i++) {
        genericOutStr(" ");
        genericOutStr(hex(info.type[i], 2));
    }
    genericOutStr("\n");
}

void
handle_data()
{
    /* DATA<addr*4><size*4>\n<data>\n */
    UINT32 addr,size;
    UINT32 a,s;
    UINT32 crc;
    unsigned char c;

    addr = read32();
    size = read32();
    if (genericInChar() != '\n') {
        genericOutStr("Error\n");
        endCmd();
        return;
    }

    if (addr + size > CBUFF_SIZE) {
        genericOutStr("Too Big\n");
    } else {
        genericOutStr("ok\n");
        a = addr;
        s = size;
        crc = crc32(0, Z_NULL, 0);
        while (s-- > 0) {
            cbuff[a++] = c = genericInChar();
            crc = crc32(crc, &c, 1);
        }
        if (genericInChar() != '\n') {
            genericOutStr("Error\n");
            endCmd();
        } else {
            genericOutStr("okda ");
            genericOutStr(hex(crc,8));
            genericOutStr("\n");
        }
    }
}

void
handle_infl()
{
    /* INFL<s_addr*4><s_len*4><d_addr*4>\n */
    UINT32 s_addr, s_len, d_addr, d_len;
    int result;

    s_addr = read32();
    s_len  = read32();
    d_addr = read32();
    if (genericInChar() != '\n') {
        genericOutStr("Error\n");
        endCmd();
        return;
    }

    if (s_addr + s_len > CBUFF_SIZE) {
        genericOutStr("Too Big\n");
        return;
    }

    d_len = UBUFF_SIZE - d_addr;
    result = uncompress(ubuff + d_addr,
                        (uLongf*)&d_len,
                        cbuff + s_addr,
                        s_len);
    if (result == Z_OK) {
        genericOutStr("okin\n");
    } else {
        outStrX("Error", result);
    }
}

void
handle_writ()
{
    /* WRIT<b_addr><b_len><f_addr><32BitMode>\n */
    UINT32 b_addr, b_len, f_addr, f32bm, oldmode;
    unsigned char *buff;
    UINT32 nextBlock1, nextBlock2;
    UINT32 f_end_addr, addr;
    UINT32 bankSize;

    b_addr = read32();
    b_len  = read32();
    f_addr = read32();
    f32bm  = read32();
    if (genericInChar() != '\n') {
        genericOutStr("Error\n");
        endCmd();
        return;
    }

    if (b_addr & (1<<31)) {
        buff = cbuff;
        b_addr &= ~(1<<31);
    } else {
        buff = ubuff;
    }

    bankSize = info.bank_size;
    if (f32bm && !flash32BitMode)
        bankSize *= 2;
    else if (!f32bm && flash32BitMode)
        bankSize /= 2;
    if (f_addr & (1<<31)) {
        f_addr += bankSize * 2;
    }
    if (f_addr & (1<<30)) {
        f_addr += bankSize;
    }
    f_addr &= ~(3<<30);

    if (f_addr & 3) {
        outStrX("Error: flash alignment",f_addr);
        return;
    }
    if (b_len & 3) {
        outStrX("Error: length alignment",b_len);
        return;
    }
    if (b_addr & 3) {
        outStrX("Error: buffer alignment",b_addr);
        return;
    }


    oldmode = flash32BitMode;
    flash32BitMode = (f32bm != 0);
    flashModeSwap = oldmode ^ flash32BitMode;

    flashType = FTYPE_NONE;

    nextBlock1 = f_addr;
    flashNextBlock(&nextBlock1);
    if (f_addr != 0) {
        nextBlock2 = f_addr-1;
        flashNextBlock(&nextBlock2);
        if (nextBlock1 == nextBlock2) {
            genericOutStr("Error: flash start addr ");
            genericOutStr(hex(f_addr,8));
            genericOutStr(" ");
            genericOutStr(hex(nextBlock1,8));
            genericOutStr(" ");
            genericOutStr(hex(nextBlock2,8));
            genericOutStr(" ");
            genericOutStr(hex(flashType,8));
            genericOutStr("\n");
            goto ret;
        }
    }

    nextBlock1 = f_addr + b_len;
    nextBlock2 = nextBlock1-1;
    flashNextBlock(&nextBlock1);
    flashNextBlock(&nextBlock2);
    if (nextBlock1 == nextBlock2) {
        outStrX("Error: flash end addr",f_addr+b_len);
        goto ret;
    }

    f_end_addr = f_addr + b_len;

    buff += b_addr;
    for (addr = f_addr; addr < f_end_addr; addr = nextBlock1) {
        nextBlock1 = addr;
        flashNextBlock(&nextBlock1);

        if (OK != flashErase(addr)) {
            genericOutStr("Error: erase ");
            genericOutStr(hex(addr,8));
            genericOutStr(" ");
            genericOutStr(hex(flashType,2));
            genericOutStr(" ");
            genericOutStr(hex(f_addr + b_len,8));
            genericOutStr("\n");
            goto ret;
        }

        if (OK != flashWrite((FLASH_TYPE*)(buff + addr - f_addr),
                             nextBlock1 - addr,
                             addr,
                             0)) {
            outStrX("Error: writing block",addr);
            goto ret;
        }
        if (oldmode == flash32BitMode) {
            if (0 != bcmp(buff + addr - f_addr,
                          (unsigned char*)(FLASH_BASE + addr),
                          nextBlock1 - addr)) {
                outStrX("Error: miscompare on block", addr);
                goto ret;
            }
        }

        genericOutStr("stat ");
        genericOutStr(hex(nextBlock1,8));
        genericOutStr("\n");
        
    } /* end for addr */
    genericOutStr("okwr\n");

 ret:
    flash32BitMode = oldmode;
    flashModeSwap = 0;
}


void
handle_fill()
{
    /* FILL<b_addr><b_len><fill>\n */
    UINT32 b_addr, b_len, fill;
    UINT32* buff;

    b_addr = read32();
    b_len  = read32();
    fill   = read32();
    if (genericInChar() != '\n') {
        genericOutStr("Error\n");
        endCmd();
        return;
    }

    if (b_addr & (1<<31)) {
        buff = (UINT32*) cbuff;
        b_addr &= ~(1<<31);
    } else {
        buff = (UINT32*) ubuff;
    }

    if (b_len & 3) {
        outStrX("Error: length alignment", b_len);
        return;
    }
    if (b_addr & 3) {
        outStrX("Error: buffer alignment", b_addr);
        return;
    }

    buff += (b_addr >> 2);

    b_len >>= 2;
    while (b_len-- > 0) {
        *buff++ = fill;
    }
    genericOutStr("okfi\n");
}


#define CMD(a,b,c,d) (((a)<<24) | ((b)<<16) | ((c)<<8) | (d))

void
flashMainLoop()
{
    UINT32 cmd;
    char c;

    /* first sync with sender */
    while ((c = genericInChar()) != '\n');

    while (1) {
        /* read command */
        c = genericInChar();
        if (c == '\n') continue;
        cmd = c << 24;
        c = genericInChar();
        if (c == '\n') continue;
        cmd |= c << 16;
        c = genericInChar();
        if (c == '\n') continue;
        cmd |= c << 8;
        c = genericInChar();
        if (c == '\n') continue;
        cmd |= c;

        switch (cmd) {
        case CMD('H','E','L','O'): endCmd();
                                   handle_helo(); break;
        case CMD('I','N','F','O'): handle_info(); break;
        case CMD('D','A','T','A'): handle_data(); break;
        case CMD('I','N','F','L'): handle_infl(); break;
        case CMD('W','R','I','T'): handle_writ(); break;
        case CMD('F','I','L','L'): handle_fill(); break;
/*
        case CMD('L','O','S','S'): handle_loss(); break;
        case CMD('','','',''): handle_(); break;
*/
        default:                  handle_default(cmd); break;
        }
    } /* end while 1 */
}

void
flash_main(unsigned int bootflags)
{
    bootFlags = bootflags;

    set_marquee(marquee);

    if (bootFlags & BOOT_IO_SERIAL) {
        uartInit(BAUD);
    }

    handle_helo();

    if (cbuff == NULL)
        cbuff = malloc(CBUFF_SIZE);
    if (ubuff == NULL)
        ubuff = malloc(UBUFF_SIZE);

    getInfo();

    flashMainLoop();
}

void
getInfo()
{
    UINT32 gpio_en, val, addr;
    int i;

    info.cpu_id = sysCpuIDGet();

    gpio_en = IXP1200_REG_READ1(IXP1200_GPIO_EN);
    IXP1200_REG_WRITE(IXP1200_GPIO_EN, GPIO_OWN_CCCC | GPIO_OUTPUT(0));
    val = IXP1200_REG_READ1(IXP1200_GPIO_DATA);
    IXP1200_REG_WRITE(IXP1200_GPIO_EN, gpio_en);

    if ((val & (1<<3)) && ((info.cpu_id & 0xF) > 0)) {
        info.width = 16;
        flash32BitMode = 0;
    } else {
        info.width = 32;
        flash32BitMode = 1;
    }

    val = IXP1200_REG_READ1(IXP1200_SRAM_CSR);
    if (val & (1<<SRAM_2MF_BIT))
        info.size = 2;
    else if (val & (1<<SRAM_4MF_BIT))
        info.size = 4;
    else if (val & (1<<SRAM_8MF_BIT))
        info.size = 8;

    for (i=0; i<4; i++)
        info.type[i] = 0xFF;
    info.bank_size = info.size * ((1024*1024)/4);
    if (! flash32BitMode)
        info.bank_size /= 2;
    for (i=0, addr = FLASH_BASE;
         i<NUM_CE;
         i++, addr += info.bank_size ) {
        info.type[i] = flashTypeGet(addr);
    }
}
