/* flashUart.c
 * Simple Uart interface for polled-mode operation
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: flashUart.c $
 * 
 * *****************  Version 7  *****************
 * User: Jdguilfo     Date: 10/26/00   Time: 3:38p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge serial and HPC versions into one version, see bootMgr.txt for
 * details.
 * 
 * *****************  Version 6  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 2:27p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge Boot Manager & Flash Utility, add HPC
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 5/11/00    Time: 10:15a
 * Updated in $/dev_1200/BoardSupport/FlashUtil
 * Flush output before booting OS
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 5/10/00    Time: 4:42p
 * Updated in $/dev_1200/BoardSupport/FlashUtil
 * Add BootMgr feature
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:37p
 * Created in $/dev_1200/BoardSupport/FlashUtil
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 * This code defines a simple uart interface for polled-mode operation
 * ------------------------------------------------------------------------
 */

#include "chip.h"
#include "genericIO.h"

extern int sysCpuRev;

UINT32 uartCR;

void 
uartInit0(UINT32 baud_div)
{
    UINT32      cr;

    if (sysCpuRev < 1) {
        /* Clear any outstanding interrupts and disable the uart */
        IXP1200_REG_WRITE( UART_CR, UART_INT_CLR);
    } else {
        /* Disable uart */
        IXP1200_REG_WRITE( UART_CR, 0);
    }

    /* Compute baud rate divisor */
    cr = baud_div << 16;

    /* Set word format: 8 bits, 1 stop bit, no parity */
    /* Leave interrupts disabled */
    cr |=
         UART_DSS_8BIT |
         UART_SBS_1BIT |
         UART_EN;


    uartCR = cr;
    IXP1200_REG_WRITE( UART_CR, 0);
    IXP1200_REG_WRITE( UART_CR, cr );
}

int 
uartPollOutput( char c )
{
    UINT32 pollStatus;

    IXP1200_REG_READ (UART_SR, pollStatus);

    /* is the transmitter ready to accept a character? */

    if (pollStatus & UART_TX_FULL)
        return -1;

    /* write out the character */

    IXP1200_REG_WRITE (UART_DR, c);        /* transmit character */

    return 0;
}

int 
uartPollInput( char *c )
{
    UINT32 pollStatus;

    IXP1200_REG_READ (UART_SR, pollStatus);

    if (!(pollStatus & UART_RX_RDY))
        return -1;

    /* got a character */

    IXP1200_REG_READ (UART_DR, *c);
#ifdef IXP1200_A0
    IXP1200_REG_READ (UART_DR, *c); /* pass-1 bug */
#endif

    return 0;
}

void 
outStr(char *str)
{
    while (*str) {
        while (uartPollOutput(*str) != 0);
        str++;
    }
}

unsigned char 
inChar()
{
    char c;
    while (uartPollInput(&c) != 0)
        scroll_marquee();
    return (unsigned char) c;
}

void
uartFlush()
{
    UINT32 pollStatus;

    while (1) {
        IXP1200_REG_READ (UART_SR, pollStatus);
        if (pollStatus & UART_TX_EMPTY)
            break;
    }
}
