/* file.cpp
 * Routines to read files
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: file.cpp $
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:39p
 * Created in $/dev_1200/BoardSupport/FlashUtil/host
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 * This code reads files. Presently it only supports binary files
 * ------------------------------------------------------------------------
 */

#include <stdio.h>
#include <string.h>

#include "file.h"

FILESTATUS
readFile(char *filename, unsigned char * buffer, int bsize,
         unsigned long int& fsize)
{
    FILE *fp;

    fp = fopen(filename, "rb");
    if (fp == NULL) {
        fprintf(stderr,"Error: Can't open file: '%s'", filename );
        return FS_ERROR;
    }
    memset(&buffer[0], 0, bsize);
    fsize = fread(buffer, sizeof(unsigned char), bsize, fp);
    if (ferror(fp)) {
        fprintf(stderr,"Error reading file\n");
        fclose(fp);
        return FS_ERROR;
    }
    if (EOF != getc(fp)) {
        fprintf(stderr,"Error, file is longer than max size (%d)\n",
                bsize);
        fclose(fp);
        return FS_ERROR;
    }
    fclose(fp);
    return FS_BIN;
}

bool 
sanityCheck(unsigned char *buffer, int fsize)
{
    unsigned int *buff = (unsigned int*) buffer;

    if ((((*buff) & 0xFF000000) != 0xEA000000) &&
        (((*buff) & 0xFF00F000) != 0xE500F000)) {
        fprintf(stderr,"The first instruction is not a branch instruction.\n");
        return false;
    }

    return true;
}
