/* flash.h
 * Classes to deal with flash information
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: flash.h $
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 11/16/00   Time: 3:53p
 * Updated in $/dev_1200/BoardSupport/BootFlash/host
 * Add support for 28F160 parts (untested)
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash/host
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:39p
 * Created in $/dev_1200/BoardSupport/FlashUtil/host
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 */

#ifndef __FLASH_H__
#define __FLASH_H__

#include <stdio.h>

#define FLASH_28F160T           0x90          /* device code 28F160-T */
#define FLASH_28F160B           0x91          /* device code 28F160-B */
#define FLASH_28F800T           0x92          /* device code 28F800-T */
#define FLASH_28F800B           0x93          /* devide code 28F800-B */
#define FLASH_28F320T           0x96          /* device code 28F320-T */
#define FLASH_28F320B           0x97          /* device code 28F320-B */

class FLASH_INFO {
private:
    DWORD m_cpu_rev; // ID of chip
    DWORD m_width;   // Flash width: 16 or 32
    DWORD m_size;    // Max flash size, 2, 4, or 8 MB. Bank size is 1/4 of that
    DWORD m_bank_size;
    DWORD m_type[4]; // Flash type per bank (FF for not checked)
public:
    void set_cpu_rev(DWORD rev) {m_cpu_rev = rev;}
    void set_bank_size() {m_bank_size = m_size * m_width * (2*1024*1024/(8*32));}
    void set_width(DWORD width) {m_width = width; set_bank_size();}
    void set_size(DWORD size) {m_size = size; set_bank_size();}
    void set_type(int index, DWORD type) {m_type[index] = type;}
    DWORD cpu_rev() {return m_cpu_rev;}
    DWORD width() {return m_width;}
    DWORD size()  {return m_size;}
    DWORD type(int i) {return ((i >= 0) && (i < 4)) ? m_type[i] : 0xFF; }
    DWORD bankSize() {return m_bank_size;}
    DWORD flashSize(int bank);

    DWORD validVector();
    bool isValid(int bank);
    int lowestValid();
    DWORD nextBlock(int bank, DWORD addr);
    bool startOfBlock(int bank, DWORD addr);
    void printSummary(FILE *fp);
};


#endif // ifdef __FLASH_H__
