/* main.cpp
 * Main routines for communicating with FUtil app on 1200
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: main.cpp $
 * 
 * *****************  Version 5  *****************
 * User: Jdguilfo     Date: 11/16/00   Time: 1:20p
 * Updated in $/dev_1200/BoardSupport/BootFlash/host
 * Clean up 16-bit mode
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 1:38p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add HPC support
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 5/11/00    Time: 2:55p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add version info
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 5/05/00    Time: 4:38p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add 16-bit flash support
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:39p
 * Created in $/dev_1200/BoardSupport/FlashUtil/host
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <string.h>

#include "sport.h"
#include "userinfo.h"
#include "flash.h"
#include "span.h"

#include "zlib.h"

user_info ui;

#define CBUFF (1 << 31)

// do an integer division, rounding the result
#define RDIV(num,den) (((num) + ((den)/2)) / (den))

void mainLoop(int argc, char* argv[]);
main(int argc, char* argv[])
{
    printf("Flash Utility, Version " VERSION "\n");

    mainLoop(argc, argv);

    return 0;
}

void
myerror(char *str)
{
    fprintf(stderr, "ERROR: %s, GetLastError = %d\n",str,GetLastError());
    exit(0);
}

DWORD
fromHex(unsigned char *buf, int size)
{
    static unsigned char h[256];
    static bool inited = false;
    DWORD result;
    unsigned int c;

    if (! inited) {
        inited = true;
        for (c = 0; c < 256; c++)
            h[c] = 0;
        for (c = '0'; c <= '9'; c++)
            h[c] = c - '0';
        for (c = 'a'; c <= 'f'; c++)
            h[c] = c - 'a' + 10;
        for (c = 'A'; c <= 'F'; c++)
            h[c] = c - 'A' + 10;
    }

    result = 0;
    while (size > 0) {
        result = (result * 16) + h[*buf++];
        size--;
    }
    return result;
}

bool
sync(HANDLE hCom, FLASH_INFO *finfo)
{
    unsigned char buf[80];
    int read;
    bool timeout;
    int i;

    writeStr(hCom, "\nHELO\n");

    for (i=0; ; i++) {
        read = readLine(hCom, buf, sizeof(buf), timeout);
        if (timeout) {
            fprintf(stdout,"\nWaiting for connection %3d ...",i);
            continue;
        }
        if ((read > 0) && (buf[read-1] == '\n'))
            break;
    } // end for

    // We need the first \n in case the 1200 came up after we started.
    // The 1200 will eat up to the first \n before processing commands
    writeStr(hCom, "\nHELO\n");
    read = readLine(hCom, buf, sizeof(buf), timeout);
    if (timeout) {
        fprintf(stderr,"ERROR: connection timed out\n");
        return false;
    }
    if (i > 0) {
        fprintf(stdout," Connected\n");
    }
    printf("%s",buf);

    writeStr(hCom, "INFO\n");
    do {
        read = readLine(hCom, buf, sizeof(buf), timeout);
        if (timeout) {
            fprintf(stderr,"ERROR: connection timed out\n");
            return false;
        }
    } while (0 != strncmp((char*)buf, "okin", 4));

    finfo->set_cpu_rev(fromHex(buf+5, 8));
    finfo->set_width(fromHex(buf+14, 2));
    finfo->set_size(fromHex(buf+17, 1));
    for (i=0; i<4; i++) {
        finfo->set_type(i,fromHex(3*i + buf + 19, 2));
    }

    return true;
}

bool
sendDataRaw(HANDLE hCom, unsigned char *buff, DWORD size, DWORD unc_size,
            DWORD& sent, DWORD ssize)
{
    int read, write;
    unsigned char buf[80];
    bool timeout;
    DWORD crc, q1,q2,d1;
    unsigned char *c;

    if (ui.m_debug) {
        fprintf(stderr,"DEBUG: DATA 0 %X\n",size);
    }

    writeStr(hCom, "DATA");
    write32(hCom, 0);
    write32(hCom, size);
    writeStr(hCom, "\n");

    read = readLine(hCom, buf, sizeof(buf), timeout);
    if (timeout) {
        fprintf(stderr,"ERROR: connection timed out\n");
        return false;
    }
    if (0 != strncmp((char*)buf, "ok", 2)) {
        fprintf(stderr,"Error in doing data send %X: '%s'\n",
                size, buf);
        return false;
    }

    crc = crc32(0, Z_NULL, 0);
    crc = crc32(crc, buff, size);
    q1 = (sent * 100 / ssize);
#define MUL 512
    d1 = (unc_size * MUL)/size;
    for (c = buff; c < buff+size; ) {
        write = buff+size - c;
        if (write > 128)
            write = 128;
        write = write_n(hCom, c, write);
        c += write;

        q2 = RDIV((sent + RDIV(((c - buff) * d1), MUL)) * 100, ssize);
        if (q2 != q1) {
            q1 = q2;
            fprintf(stdout,"\r Percent complete: %3d ", q2);
        }
    }
    sent += unc_size;
    writeStr(hCom, "\n");

    read = readLine(hCom, buf, sizeof(buf), timeout);
    if (timeout) {
        fprintf(stderr,"ERROR: connection timed out\n");
        return false;
    }
    if (0 != strncmp((char*)buf, "okda", 4)) {
        fprintf(stderr,"Error in doing data send %X: '%s'\n",
                size, buf);
        return false;
    }

    sscanf((char*)buf+4, "%X", &q1);
    if (q1 != crc) {
        fprintf(stderr,"Error in CRC for data send: %X %X\n",q1,crc);
        return false;
    }

    return true;
}

bool
uncompress(HANDLE hCom, DWORD bsize, DWORD start)
{
    int read;
    unsigned char buf[80];
    bool timeout;

    if (ui.m_debug) {
        fprintf(stderr,"DEBUG: INFL 0 %X %X\n",bsize,start);
    }

    writeStr(hCom, "INFL");
    write32(hCom, 0);
    write32(hCom, bsize);
    write32(hCom, start);
    writeStr(hCom, "\n");

    read = readLine(hCom, buf, sizeof(buf), timeout);
    if (timeout) {
        fprintf(stderr,"ERROR: connection timed out\n");
        return false;
    }
    if (0 != strncmp((char*)buf, "okin", 4)) {
        fprintf(stderr,"Error in doing inflate: '%s'\n", buf);
        return false;
    }

    return true;
}

bool
sendData(HANDLE hCom, unsigned char *buf, DWORD start, DWORD end, DWORD& sent,
         DWORD size)
{
    static unsigned char buffer[FILE_SIZE_MAX + (FILE_SIZE_MAX / 8)];
    int status;
    DWORD bsize;

    bsize = sizeof(buffer);
    status = compress(buffer, &bsize,
                      buf+start, end-start);
    if (Z_OK != status) {
        fprintf(stderr,"Error in data compression, aborting: %d\n", status);
        return false;
    }
    if (!sendDataRaw(hCom, buffer, bsize, end-start, sent, size))
        return false;

    if (!uncompress(hCom, bsize, start))
        return false;

    return true;
}

bool
sendZeros(HANDLE hCom, DWORD start, DWORD end)
{
    int read;
    unsigned char buf[80];
    bool timeout;

    if (ui.m_debug) {
        fprintf(stderr,"DEBUG: FILL %X %X 0\n",start,end-start);
    }

    writeStr(hCom, "FILL");
    write32(hCom, start);
    write32(hCom, end-start);
    write32(hCom, 0);
    writeStr(hCom, "\n");

    read = readLine(hCom, buf, sizeof(buf), timeout);
    if (timeout) {
        fprintf(stderr,"ERROR: connection timed out\n");
        return false;
    }
    if (0 != strncmp((char*)buf, "okfi", 4)) {
        fprintf(stderr,"Error in doing zero fill %X %X: '%s'\n",
                start, end, buf);
        return false;
    }
    return true;
}

bool 
updateFlash(HANDLE hCom, DWORD start, DWORD end, DWORD bank, DWORD width32,
	    DWORD& sent, DWORD size, DWORD bankSize)
{
    int read;
    unsigned char buf[80];
    bool timeout;
    DWORD addr, end_addr;

    if (ui.m_debug) {
      fprintf(stderr,"DEBUG: WRIT %X %X %X %X\n",
              start,end-start,start|(bank<<30),width32);
    }

    writeStr(hCom, "WRIT");
    write32(hCom, start);
    write32(hCom, end-start);
    write32(hCom, start | (bank << 30));
    write32(hCom, width32);
    writeStr(hCom, "\n");

    addr = start + bank * bankSize;

    while (1) {
        read = readLine(hCom, buf, sizeof(buf), timeout);
        if (timeout) {
            fprintf(stderr,"ERROR: connection timed out\n");
            return false;
        }

        if (0 == strncmp((char*)buf, "okwr", 4))
            break;
        if (0 == strncmp((char*)buf, "stat", 4)) {
            end_addr = fromHex(buf+5, 8);
            sent += end_addr - addr;
            addr = end_addr;
            printf("\r Percent Complete: %3d ",sent * 100 / size);
        } else {
            fprintf(stderr,"Error in doing Flash Write %X %X: '%s'\n",
                    start, end, buf);
            return false;
        }
    } // end while 1

    return true;
}

bool
downloadData(HANDLE hCom, FLASH_INFO *finfo, user_info *ui)
{
    span *sp = ui->m_span;
    DWORD start, end;
    bool zero;
    void *state;
    DWORD size, sent;
    unsigned char *buf;
    DWORD bankSize;


    // Make the uncompresses remote buffer look like our buffer,
    // using the compressed buffer as needed to move data.

    // 1. Count size of non-zero spans
    size = 0;
    for (state = NULL; sp->get_span(start, end, zero, state); ) {
        if (! zero) {
            size += end - start;
        }
    }

    // 2. Send data to remote system

    printf("Sending data to remote system...\n");
    sent = 0;
    // buf corresponds to flash address zero
    buf = &ui->m_buffer[ui->m_file_offset - ui->m_flash_offset];

    for (state = NULL; sp->get_span(start, end, zero, state); ) {
        if (zero) {
            if (!sendZeros(hCom, start, end))
                return false;
        } else {
            if (!sendData(hCom, buf, start, end, sent, size))
                return false;
        }
    } // end for


    // 3. update flash

    printf("\nUpdating flash...\n");

    // 3.1 Count size of spans
    size = 0;
    for (state = NULL; sp->get_span_joined(start, end, state); ) {
        size += end - start;
    }

    bankSize = finfo->bankSize();
    if ( (ui->m_force != F_16) && (finfo->width() == 16) ) {
        bankSize *= 2;
    } else     if ( (ui->m_force == F_16) && (finfo->width() != 16) ) {
        bankSize /= 2;
    }

    sent = 0;
    for (state = NULL; sp->get_span_joined(start, end, state); ) {
        if (! updateFlash(hCom, start, end, ui->m_bank, (ui->m_force != F_16),
                          sent, size, bankSize))
            return false;
    } // end for
    printf("\ndone\n");

    return true;
}

void
mainLoop(int argc, char* argv[])
{
    HANDLE hCom;
    FLASH_INFO finfo;

    if (! ui.init(argc, argv))
        return;
    
    if (! ui.prompt1())
        return;

    if (ui.m_comm_hpc)
        hCom = openHpc(ui.m_hpc_port);
    else
        hCom = openSer(ui.m_comm_port);
    if (hCom == INVALID_HANDLE_VALUE)
        return;

    if (! sync(hCom, &finfo)) {
        CloseHandle(hCom);
        return;
    }

    if (finfo.validVector() == 0) {
        fprintf(stderr,
               "No recognized flash types found on board: %02X %02X %02X %02X\n",
                finfo.type(0),finfo.type(1),finfo.type(2),finfo.type(3));
        CloseHandle(hCom);
        return;
    }
    
    finfo.printSummary(stdout);

    if (ui.m_force == F_NONE) {
        ui.m_force = (finfo.width() == 16) ? F_16 : F_32;
    }

    if (! ui.prompt2(&finfo)) {
        CloseHandle(hCom);
        return;
    }

    if (! downloadData(hCom, &finfo, &ui)) {
        CloseHandle(hCom);
        return;
    }

    CloseHandle(hCom);
}
