/* span.h
 * Class for dealing with regions of addresses
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: span.h $
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:39p
 * Created in $/dev_1200/BoardSupport/FlashUtil/host
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 * This code defines a class that represents ranges of addresses. The ranges
 * can have holes.
 * ------------------------------------------------------------------------
 */

#ifndef __span_h__
#define __span_h__

class span_item {
private:
    unsigned char *m_buf;
    DWORD          m_bsize;
    DWORD          m_start; // start of span
    DWORD          m_end;   // end+1 of span
    bool           m_zero;  // true if span is all zero
    span_item     *m_next;
    span_item     *m_prev;
    span_item* splitBefore(DWORD ptr);
    span_item* splitAfter(DWORD ptr);
    friend class span;
public:
    span_item(unsigned char *buf, DWORD bsize);
    span_item(span_item *sp, DWORD start, DWORD end, bool zero,
              span_item *next, span_item *prev);
    ~span_item();
    void makeZeros(DWORD threash);
};

class span {
private:
    span_item *m_head;
public:
    span(unsigned char *buf, DWORD bsize);
    ~span();
    void limit(DWORD start, DWORD end);
    void insertHole(DWORD start, DWORD end);
    void makeZeros(DWORD threash = 1024);
    bool get_span(DWORD& start, DWORD& end, bool& zero, void* &state);
    bool get_span_joined(DWORD& start, DWORD& end, void* &state);
    bool overlaps(DWORD start, DWORD end);
};

#endif /* ifdef __span_h__ */
