/* sport.cpp
 * Serial port interface
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: sport.cpp $
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash/host
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 1:38p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add HPC support
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 5/10/00    Time: 4:42p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add BootMgr feature
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:39p
 * Created in $/dev_1200/BoardSupport/FlashUtil/host
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <string.h>

#include "sport.h"

void myerror(char *str);

#define BAUD 38400

OVERLAPPED ol;

HANDLE
openSer(int port)
{
    HANDLE hCom;
    DCB dcb;
    char com[5];
    COMMTIMEOUTS to;

    sprintf(com,"COM%d",port);
    hCom = CreateFile(com,
                      GENERIC_READ | GENERIC_WRITE,
                      0,
                      NULL,
                      OPEN_EXISTING,
                      FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
                      NULL);
    if (hCom == INVALID_HANDLE_VALUE) {
        myerror("Can't open com port");
        return hCom;
    }
    if (! GetCommState(hCom, &dcb)) {
        myerror("in GetCommState");
        CloseHandle(hCom);
        return INVALID_HANDLE_VALUE;
    }
    dcb.BaudRate = BAUD;
    dcb.ByteSize = 8;
    dcb.Parity = NOPARITY;
    dcb.StopBits = ONESTOPBIT;

    dcb.fAbortOnError = FALSE;

    if (! SetCommState(hCom, &dcb)) {
        myerror("in SetCommState");
        CloseHandle(hCom);
        return INVALID_HANDLE_VALUE;
    }

    ol.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

    to.ReadIntervalTimeout         = MAXDWORD;
    to.ReadTotalTimeoutMultiplier  = MAXDWORD;
    to.ReadTotalTimeoutConstant    = 100*1000;
    to.WriteTotalTimeoutMultiplier = 0;
    to.WriteTotalTimeoutConstant   = 0;
    // if there are any character in the input buffer, ReadFile returns
    // them immediate; otherwise if a character arrives within 10 seconds,
    // it is returned; otherwise, the read request times out. 

    if (! SetCommTimeouts(hCom, &to)) {
        myerror("in SetCommTimeouts");
        CloseHandle(hCom);
        return INVALID_HANDLE_VALUE;
    }

    return hCom;
}

HANDLE
openHpc(int port)
{
    HANDLE hCom;
    char com[15];

    sprintf(com,"\\\\.\\IXPHPC%02d",port);
    hCom = CreateFile(com,
                      GENERIC_READ | GENERIC_WRITE,
                      0,
                      NULL,
                      OPEN_EXISTING,
                      FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED,
                      NULL);
    if (hCom == INVALID_HANDLE_VALUE) {
        myerror("Can't open com port");
        return hCom;
    }

    ol.hEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

    return hCom;
}

int
readLineInt(HANDLE hCom, unsigned char* buf, int buf_size, bool &timeout)
{
    int i;
    DWORD num_bytes_read;
    DWORD err;

    timeout = false;

    buf_size--;
    for (i=0; i<buf_size; ) {
        if (! ReadFile(hCom, &buf[i], 1, &num_bytes_read, &ol)) {
            err = GetLastError();
            if (err == ERROR_IO_PENDING) {
                err = WaitForSingleObject(ol.hEvent, 30*1000);
                if (err == WAIT_TIMEOUT) {
                    timeout = true;
                    buf[i] = 0;
                    return i;
                }
                GetOverlappedResult(hCom, &ol, &num_bytes_read, true);
            } else {
                i += num_bytes_read;
                if (err == ERROR_TIMEOUT) {
                    timeout = true;
                    buf[i] = 0;
                    return i;
                } else {
                    myerror("in ReadFile");
                    buf[i] = 0;
                    return i;
                }
            }
        } // if ! Readfile

        i += num_bytes_read;
        if (buf[i-1] == '\n') {
            buf[i] = 0;
            return i;
        }
    } /* for i */
    buf[i] = 0;
    return i;
}

int
readLine(HANDLE hCom, unsigned char* buf, int buf_size, bool &timeout)
{
  int rval;

  while (1) {
    rval = readLineInt(hCom, buf, buf_size, timeout);
    //fprintf(stderr,"\nreadline %d '%s'\n",timeout, buf);
    if (timeout)
	return rval;
    if (buf[0] != '*')
	break;
    fprintf(stderr,"%s",buf);
  }
  return rval;
}

bool
myWriteFile(HANDLE hCom, void* buf, DWORD size, DWORD *psize, void* dummy)
{
  int err;

  if (! WriteFile(hCom, buf, size, psize, &ol)) {
    err = GetLastError();
    if (err = ERROR_IO_PENDING) {
      err = WaitForSingleObject(ol.hEvent, INFINITE);
      GetOverlappedResult(hCom, &ol, psize, true);
    } else {
      myerror("in WriteFile");
      return FALSE;
    }
  }
  return TRUE;
}

void
writeStr(HANDLE hCom, unsigned char *buf)
{
    DWORD size = strlen((char*)buf);

    if (! myWriteFile(hCom, buf, size, &size, NULL)) {
        myerror("in myWriteFile");
    }
}

void 
writeStr(HANDLE hCom,          char *buf)
{
    DWORD size = strlen(buf);

    if (! myWriteFile(hCom, buf, size, &size, NULL)) {
        myerror("in myWriteFile");
    }
}

DWORD
write_n(HANDLE hCom, unsigned char *buf, int n)
{
    DWORD size;

    if (! myWriteFile(hCom, buf, n, &size, NULL)) {
        myerror("in myWriteFile");
    }
    return size;
}
    
void 
write32(HANDLE hCom, unsigned int data)
{
    unsigned char buf[4];
    DWORD size;

    buf[0] = data & 0xFF;
    buf[1] = (data >> 8) & 0xFF;
    buf[2] = (data >> 16) & 0xFF;
    buf[3] = (data >> 24) & 0xFF;
    if (! myWriteFile(hCom, buf, sizeof(buf), &size, NULL)) {
        myerror("in myWriteFile");
    }
}
