/* userinfo.cpp
 * Code to manage information from user
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: userinfo.cpp $
 * 
 * *****************  Version 9  *****************
 * User: Jdguilfo     Date: 11/16/00   Time: 1:20p
 * Updated in $/dev_1200/BoardSupport/BootFlash/host
 * Clean up 16-bit mode
 * 
 * *****************  Version 8  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash/host
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 7  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 1:38p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add HPC support
 * 
 * *****************  Version 5  *****************
 * User: Jdguilfo     Date: 5/11/00    Time: 10:15a
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add BootMgr to region description
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 5/10/00    Time: 5:02p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Fix bug with non-interactive use
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 5/10/00    Time: 4:42p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add BootMgr feature
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 5/05/00    Time: 4:38p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add 16-bit flash support
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:39p
 * Created in $/dev_1200/BoardSupport/FlashUtil/host
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 */

#include <windows.h>
#include <string.h>

#include "userinfo.h"
#include "prompt.h"
#include "file.h"
#include "span.h"
#include "flash.h"

region::region()
{
    m_next = m_prev = this;

    m_start = 0;
    m_end   = 0;
    m_desc[0] = 0;
    m_default_skip = false;
    m_spec_beh = 0;
}

region::~region()
{
    m_prev->m_next = m_next;
    m_next->m_prev = m_prev;
}

region::region(region* root, char* desc, DWORD start, DWORD end, bool def_skip,
               DWORD spec_beh)
{
    // insert selves at end
    m_next = root;
    m_prev = root->m_prev;
    m_prev->m_next = this;
    m_next->m_prev = this;

    m_start = start;
    m_end = end;
    strcpy(m_desc, desc);
    m_default_skip = def_skip;
    m_spec_beh = spec_beh;
}

//////////////////////////////////////////////////////////////

DWORD
atoh(char *str)
{
    DWORD result;

    if (1 != sscanf(str,"%x",&result))
        result = 0;
    return result;
}

//////////////////////////////////////////////////////////////

user_info::user_info()
{
    m_interactive  = true;
    m_filename[0]  = 0;
    m_file_offset  = 0;
    m_flash_offset = 0;
    m_flash_end    = MAXDWORD;
    m_bank         = -1;
    m_span         = NULL;
    m_regions_set  = false;
    m_comm_hpc     = false;
    m_hpc_port     = 2;
    m_comm_port    = 1;
    m_debug        = false;
    m_force        = F_NONE;

    m_buffer = new unsigned char[FILE_SIZE_MAX];
}

user_info::~user_info()
{
    region *r;

    delete m_buffer;

    if (m_span)
        delete m_span;
    while ((r = m_region_root.m_next) != &m_region_root)
        delete m_region_root.m_next;
}

void
user_info::set_default_regions()
{
    new region(&m_region_root,
               "Flash Utility / Boot Mgr", 0x00000000, 0x00008000, true, 1);
    new region(&m_region_root,
               "Boot Parameter",           0x00008000, 0x0000C000, true);
    new region(&m_region_root,
               "Diagnostics",              0x0000C000, 0x00020000, false);
    new region(&m_region_root,
               "Angel/UCOS",               0x00020000, 0x00040000, false);
    new region(&m_region_root,
               "VxWorks",                  0x00040000, 0x00140000, false);
    new region(&m_region_root,
               "Cygmon",                   0x00140000, DEFAULT_FLASH_SIZE, true);
}

void
user_info::usage(FILE *fp)
{
    fprintf(fp,
            "Usage: FUtil [options]\n"
            "  -H           : Use HPC port defined by -c for communicatins\n"
            "  -c port      : Comm port (1,2) used for communications\n"
            "  -n           : Non-interactive -- don't prompt for missing arguments\n"
            "  -f filename  : filename to load\n"
            "  -o offset    : starting offset in flash\n"
            "  -e offset    : ending offset in flash\n"
            "  -O offset    : starting offset in file\n"
//          "  -b bank      : flash bank\n"
            "  -d           : Do not define default regions\n"
            "  -r name start end skip : define a memory region\n"
            "              name  -- name of region\n"
            "              start -- starting offset of region\n"
            "              end   -- ending offset of region\n"
            "              skip  -- 0/1 : default skip value for region\n"
            "  -R start end : define anonymous region with skip\n"
            "  -s name skip : set skip value (0/1) for named region\n"
            "  -a skip      : set skip value (0/1) for all regions\n"
            "  -16          : force 16-bit mode\n"
            "  -32          : force 32-bit mode\n"
            "  -p           : print default regions\n"
            "  -h           : print this help message\n"
            "  -D           : print debug info\n"
            "\n"
            "The -R option would typically not be used without -n.\n"
            "If the -n option is given, then the default region skip values\n"
            "are used, to change them, specify a different set of regions.\n"
            "\n"
            "Note that all numeric values (except for bank) are taken as\n"
            "hexadecimal, without a leading '0x'.\n"
            "For -s, the region modified is the first one whose name begins\n"
            "with the given string.\n"
//          "\n"
//          "If bank is specified as -1, then the lowest bank containing valid\n"
//          "flash will be used.\n"
            "The option -d is implied by -r or -R.\n"
            );
}

#define CHECK_ARGS(count) \
    if ((argc - 1 - i) < (count)) { \
        fprintf(stderr, "Not enough arguments for '%s'\n", argv[i]); \
        usage(stderr); \
        return false; \
    } 

bool
user_info::init(int argc, char* argv[])
{
    bool cont = true;
    bool print_regions = false;
    int i;
    char *desc;
    DWORD start, end;
    bool skip, a_skip, a_set = false;
    region *r;
    int s_array[100];
    int s_ptr = 0;

    for (i = 1; i < argc; i++) {
        if (argv[i][0] != '-') {
            fprintf(stderr,"Unknown option: '%s'\n", argv[i]);
            usage(stderr);
            return false;
        }
        switch (argv[i][1]) {
        case 'H':
            m_comm_hpc = true;
            break;
        case 'c':
            CHECK_ARGS(1);
            m_comm_port = atoh(argv[++i]);
            m_hpc_port = m_comm_port;
            break;
        case 'n':
            m_interactive = false;
            break;
        case 'f':
            CHECK_ARGS(1);
            strcpy(m_filename, argv[++i]);
            break;
        case 'o':
            CHECK_ARGS(1);
            m_flash_offset = atoh(argv[++i]);
            break;
        case 'e':
            CHECK_ARGS(1);
            m_flash_end = atoh(argv[++i]);
            break;
        case 'O':
            CHECK_ARGS(1);
            m_file_offset = atoh(argv[++i]);
            break;
//      case 'b':
//          CHECK_ARGS(1);
//          m_bank = atoi(argv[++i]);
//          break;
        case 'r':
            CHECK_ARGS(4);
            desc = argv[++i];
            start = atoh(argv[++i]);
            end   = atoh(argv[++i]);
            skip  = (atoi(argv[++i]) != 0);
            new region(&m_region_root, desc, start, end, skip);
            m_regions_set = true;
            break;
        case 'R':
            CHECK_ARGS(2);
            start = atoh(argv[++i]);
            end   = atoh(argv[++i]);
            new region(&m_region_root, "", start, end, true);
            m_regions_set = true;
            break;
        case 'd':
            m_regions_set = true;
            break;
        case 'p':
            print_regions = true;
            break;
        case 'D':
            m_debug = true;
            break;
        case 'a':
            CHECK_ARGS(1);
            a_skip  = (atoi(argv[++i]) != 0);
            a_set = true;
            break;
        case 's':
            CHECK_ARGS(2);
            s_array[s_ptr++] = ++i;
            ++i;
            break;
        case 'h':
            usage(stderr);
            return false;
            break;
        default:
            if (0 == strcmp(argv[i], "-16")) {
                m_force = F_16;
            } else if (0 == strcmp(argv[i], "-32")) {
                m_force = F_32;
            } else {
                fprintf(stderr,"Unknown option: '%s'\n", argv[i]);
                usage(stderr);
                return false;
            }
        } // end switch

    } // end for i

    if (! m_regions_set)
        set_default_regions();

    if (a_set)
        set_def_regions(a_skip);
    for (i=0; i<s_ptr; i++) {
        desc = argv[s_array[i]];
        skip  = (atoi(argv[s_array[i]+1]) != 0);
        if (!set_region(desc, skip)) {
            fprintf(stderr,"Error: Can't find region to match '%s'\n", desc);
            cont = false;
        }
    }

    if (!cont)
        return false;

    if (print_regions) {
        for (r = m_region_root.m_next; r != &m_region_root; r = r->m_next) {
            printf("%08X %08X %s Region, default is to%s skip\n",
                   r->m_start, r->m_end, r->m_desc,
                   r->m_default_skip ? "" : " not");
        }
        return false;
    }

    return true;
}

bool
user_info::prompt1()
{
    char buf[STRING_MAX];

    if (! m_interactive)
        return true;

    printf("Numeric parameters (other than the bank and comm port) should be\n"
           "entered as hexadecimal numbers without the leading '0x'.\n"
           "Enter a blank line to accept the default value.\n"
           "Enter a period ('.') to abort the program.\n");

    if (m_comm_hpc) {
        strcpy(buf,"No");
    } else {
        strcpy(buf,"Yes");
    }

    if (! promptString("Use Serial Port", buf))
        return false;

    if ((buf[0] != 'y') && (buf[0] != 'Y')) {
        m_comm_hpc = true;
        if (! promptInt("HPC port", m_hpc_port))
            return false;
    } else {
        if (!promptInt("Comm port", m_comm_port))
            return false;
    }

    return true;
}

bool
user_info::prompt2(FLASH_INFO *finfo)
{
    char buf[STRING_MAX], buf2[STRING_MAX];
    DWORD size;
    FILESTATUS fs;
    region *r;

    if (m_flash_end == MAXDWORD)
        m_flash_end = finfo->bankSize();
    if ((m_force != F_16) && (finfo->width() == 16)) {
        m_flash_end *= 2;
    }

    if ((m_force == F_16) && (m_flash_end > FILE_SIZE_MAX/2))
        m_flash_end = FILE_SIZE_MAX/2;

    if ((m_force == F_16) != (finfo->width() == 16)) {
        set_def_regions(false);
    }

    if (m_interactive) {

        while (1) {
//          if (! promptInt("Starting Bank", m_bank))
//              return false;
            if (m_bank == -1)
                m_bank = finfo->lowestValid();
            if (finfo->isValid(m_bank))
                break;
            fprintf(stderr,"Error bank %d is not valid.\n", m_bank);
            fprintf(stderr,"Valid banks are:");
            for (size=0; size<4; size++) {
                if (finfo->isValid(size))
                    fprintf(stderr," %d", size);
            }
            fprintf(stderr,"\n");
            return false;
        }
        printf("Starting at bank %d\n",m_bank);
        size = finfo->flashSize(m_bank);

        while (1) {
            if (! promptString("Filename", &m_filename[0]))
                return false;
            fs = readFile(m_filename, &m_buffer[0], FILE_SIZE_MAX, m_file_size);
            if (fs == FS_ERROR)
                continue;
            if (m_file_size > size) {
                fprintf(stderr,
                        "Error: file size (%x) is greater than the bank size (%d).\n",
                        m_file_size, size);
                continue;
            }
            break;
        }

        if ((fs == FS_BIN) && (! sanityCheck(&m_buffer[0], m_file_size))) {
            strcpy(buf,"No");
            if (! promptString("This does not appear to be a valid ARM file, "
                               "do you want to continue", buf))
                return false;
            if ((buf[0] != 'y') && (buf[0] != 'Y'))
                return false;
        }

    } else { // end if interactive
        if (m_bank == -1)
            m_bank = finfo->lowestValid();

        if (!finfo->isValid(m_bank)) {
            fprintf(stderr,"Error bank %d is not valid.\n", m_bank);
            fprintf(stderr,"Valid banks are:");
            for (size=0; size<4; size++) {
                if (finfo->isValid(size))
                    fprintf(stderr," %d", size);
            }
            fprintf(stderr,"\n");
            return false;
        }
        size = finfo->flashSize(m_bank);

        fs = readFile(m_filename, &m_buffer[0], FILE_SIZE_MAX, m_file_size);
        if (fs == FS_ERROR)
            return false;
        if (m_file_size > size) {
            fprintf(stderr,
                    "Error: file size (%x) is greater than the bank size (%d).\n",
                    m_file_size, size);
            return false;
        }

    }

    for (r = m_region_root.m_next; r != &m_region_root; r = r->m_next) {
        if ((! finfo->startOfBlock(m_bank, r->m_start)) ||
            (! finfo->startOfBlock(m_bank, r->m_end))) {
            fprintf(stderr,
                    "Error: region definition '%s' %06X - %06X is not a flash block boundary.\n",
                    r->m_desc, r->m_start, r->m_end);
            return false;
        }
    }

    while (1) {
        if (m_interactive) 
            if (! promptHex("Starting offset in flash", m_flash_offset))
                return false;
            
        if (finfo->startOfBlock(m_bank, m_flash_offset))
            break;

        fprintf(stderr,
                "Error: Starting offset %06X is not a flash block boundary.\n",
                m_flash_offset);
        if (!m_interactive)
            return false;
    }

    while (1) {
        if (m_interactive) 
            if (! promptHex("Ending offset in flash", m_flash_end))
                return false;

        if (finfo->startOfBlock(m_bank, m_flash_end))
            break;

        fprintf(stderr,
                "Error: Ending offset %06X is not a flash block boundary.\n",
                m_flash_end);
        if (!m_interactive)
            return false;
    }

    while (1) {
        if (m_interactive)
            if (! promptHex("Starting offset in file", m_file_offset))
                return false;

        if (m_file_offset & 3) {
            fprintf(stderr,
                    "Error: Ending offset %06X is not longword aligned.\n",
                    m_file_offset);
            if (!m_interactive)
                return false;
            continue;
        }
        if (m_file_offset + (m_flash_end - m_flash_offset) > finfo->flashSize(m_bank)){
            fprintf(stderr,
                    "Error: Area specified extends beyond end of flash.\n");
            if (!m_interactive)
                return false;
            continue;
        }
        break;
    }

    m_span = new span(&m_buffer[m_file_offset - m_flash_offset],
                      m_flash_end + m_file_offset);
    m_span->limit(m_flash_offset, m_flash_end);

    for (r = m_region_root.m_next; r != &m_region_root; r = r->m_next) {
        if (m_span->overlaps(r->m_start, r->m_end)) {
            if (m_interactive) {
                if (r->m_default_skip)
                    strcpy(buf,"Yes");
                else
                    strcpy(buf,"No");
                sprintf(buf2,"Do you wish to skip the %s block", r->m_desc);
                if (! promptString(buf2, buf))
                    return false;
                if ((buf[0] == 'y') || (buf[0] == 'Y')) {
                    m_span->insertHole(r->m_start, r->m_end);
                } else if (r->m_spec_beh & 1) {
                    fprintf(stdout,
                            "Updating the %s block is potentially dangerous\n",
                            r->m_desc);
                    if ((m_force == F_16) != (finfo->width() == 16)) {
                        strcpy(buf, "Yes");
                    } else {
                        strcpy(buf, "No");
                    }
                    sprintf(buf2, "Do you want to update the %s block",
                            r->m_desc);
                    if (! promptString(buf2, buf))
                        return false;
                    if ((buf[0] == 'n') || (buf[0] == 'N')) {
                        m_span->insertHole(r->m_start, r->m_end);
                    } else if (*(DWORD*)(m_buffer + BOOTFLASH_COOKIE_ADDR) !=
                               BOOTFLASH_COOKIE) {
                        fprintf(stdout,
                                "This file does not look like it contains a valid %s\n",
                                r->m_desc);
                        strcpy(buf, "No");
                        sprintf(buf2, "Are you sure you want to continue");
                        if (! promptString(buf2, buf))
                            return false;
                        if ((buf[0] == 'n') || (buf[0] == 'N')) {
                            m_span->insertHole(r->m_start, r->m_end);
                        }
                    } // end if cookies do not match
                } // end if spec_beh_1
            } else {
                if (r->m_default_skip)
                    m_span->insertHole(r->m_start, r->m_end);
            }
        }
    }

    m_span->makeZeros();

    return true;
}

void
user_info::set_def_regions(bool skip)
{
    region *r;

    for (r = m_region_root.m_next; r != &m_region_root; r = r->m_next) {
        r->m_default_skip = skip;
    }
}

bool
user_info::set_region(char *name, bool skip)
{
    int i;
    region *r;
    i = strlen(name);

    for (r = m_region_root.m_next; r != &m_region_root; r = r->m_next) {
        if (0 == strnicmp(name, r->m_desc, i)) {
            r->m_default_skip = skip;
            return true;
        }
    }
    return false;
}
