/* userinfo.cpp
 * Code to manage information from user
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: FUtil
 * author: jdg 05/03/00
 * revisions:
 * $History: userinfo.h $
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash/host
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 1:38p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add HPC support
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 5/05/00    Time: 4:38p
 * Updated in $/dev_1200/BoardSupport/FlashUtil/host
 * Add 16-bit flash support
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/03/00    Time: 4:39p
 * Created in $/dev_1200/BoardSupport/FlashUtil/host
 * Add new Flash Utility (FUtil)
 * 
 * ------------------------------------------------------------------------
 */

#ifndef __USERINFO_H__
#define __USERINFO_H__

#include <stdio.h>
#include "consts.h"

class span;
class FLASH_INFO;

class region {
private:
public:
    region *m_next;
    region *m_prev;
    DWORD m_start;
    DWORD m_end;
    char  m_desc[STRING_MAX];
    bool  m_default_skip;
    DWORD m_spec_beh;

    region();
    region(region* root, char* desc, DWORD start, DWORD end, bool def_skip,
           DWORD spec_beh = 0);
    ~region();
};

enum  FORCE  {F_NONE, F_16, F_32};

class user_info {
private:
    void set_default_regions();
public:
    bool    m_interactive;
    bool    m_regions_set;
    region  m_region_root;
    char    m_filename[STRING_MAX];
    DWORD   m_file_offset;
    DWORD   m_flash_offset;
    DWORD   m_flash_end;
    int     m_bank;
    span   *m_span;
    bool    m_comm_hpc;
    int     m_hpc_port;
    int     m_comm_port;
    bool    m_debug;
    FORCE   m_force;
    
    unsigned char *m_buffer; // [FILE_SIZE_MAX]
    DWORD         m_file_size;

    user_info();
    ~user_info();
    bool init(int argc, char* argv[]); // returns false to abort
    void usage(FILE *fp);
    bool prompt1();
    bool prompt2(FLASH_INFO *info);
    void set_def_regions(bool skip);
    bool set_region(char *name, bool skip);
};

#endif /* ifdef __USERINFO_H__ */
