/* hpc.c
 * Routines needed for HPC support
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: bootFlash
 * author: jdg 10/20/00
 * revisions:
 * $History: hpc.c $
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 10/26/00   Time: 3:41p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * clean up typos
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 10/26/00   Time: 3:38p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge serial and HPC versions into one version, see bootMgr.txt for
 * details.
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 2:35p
 * Created in $/dev_1200/BoardSupport/BootFlash
 * Merge bootmgr and flashutility into bootflash
 * ------------------------------------------------------------------------
 */

#define NO_OS

#include "chip.h"
#include "hpc.h"
#include "genericIO.h"

#define UINT unsigned int
#define UINT32 UINT
#define UINT16 unsigned short int
#define BOOL UINT

#define sysWrite16(addr,data) *(volatile UINT16*)(addr) = (data)

void
bcopy(unsigned char *src, unsigned char *dst, int size)
{
    for (; size > 0; size--) {
        *dst++ = *src++;
    }
}

#include "ixp1200hpc.c"

static void 
sysPciInit (void)
{
    volatile int i, j;
    int central_func;

    /* assert PCI reset */

    *(UINT32 *)(IXP1200_SA_CONTROL) =
	    (*(UINT32 *)(IXP1200_SA_CONTROL)) & ~IXP1200_SA_CONTROL_PNR;

    /* wait a few usecs to settle the device and PCI bus (no timers yet) */

    for (i = 0; i < 50 ; i++)
        j = i + 1;


    /* Disable Outbound & Doorbell Ints in Outbound Interrupt Mask Register*/

    *(UINT32 *)(IXP1200_OUTB_INT_MASK) = 0xC;


    /* Disable doorbell interrupts to PCI in Doorbell PCI Mask Register */

    *(UINT32 *)(IXP1200_DBELL_PCI_MASK) = 0;


    /* Disable doorbell interrupts to SA-110 in Doorbell SA-110 Mask Reg */

    *(UINT32 *)(IXP1200_DBELL_SA_MASK) = 0;


    /* Set PCI Address Extension Register to known state */

    *(UINT32 *)(IXP1200_PCI_ADDR_EXT) = 0x00000000;


    /* Set Int pin */
    *(UINT32 *)(IXP1200_PCI_INT_LAT) = 1 << 8;

    /* Negate PCI reset (has no effect if we are not central function) */

    *(UINT32 *)(IXP1200_SA_CONTROL) =
	    (*(UINT32 *)(IXP1200_SA_CONTROL)) | IXP1200_SA_CONTROL_PNR;

    /* wait a few usecs to settle the device and PCI bus (no timers yet) */

    for (i = 0; i < 50 ; i++)
        j = i + 1;


    /* Setup SDRAM Base Address Mask Reg: LOCAL_MEM_SIZE_ACTUAL window (32Meg)
       from PCI into SDRAM */

    *(UINT32 *)(IXP1200_DRAM_BASE_ADDR_MASK) = LOCAL_MEM_SIZE_ACTUAL - 0x00040000; 


    /* Set up CSR Base Address Mask Reg. 
     * Due to a Pass-A HW bug, we need to ask for at least 1 Meg. */

    *(UINT32 *)(IXP1200_CSR_BASE_ADDR_MASK) = 0x000c0000;

    /* Stop IXP1200 from responding to any PCI transactions */

    sysWrite16 ((volatile UINT16 *)(IXP1200_PCI_COMMAND), 0);


    /* Check to see if we are central function, if not then do not
     * set up BARs */

#ifdef SPECTACLE_ISLAND
    central_func = 1;
#else
    central_func =
        IXP1200_SA_CONTROL_PCF & *(UINT32 *)IXP1200_SA_CONTROL;
#endif /* SPECTACLE_ISLAND */

    if (central_func)
        {

        /* Set CSR Memory Base Address Register */

        *(UINT32 *)(IXP1200_PCI_MEM_BAR) = 0x40000000;


        /*
         * Set CSR I/O Base Address Register: set base address of CSRs from PCI,
         * CSR Base Address Mask Register will setup 128 byte window.
         */

        *(UINT32 *)(IXP1200_PCI_IO_BAR) = 0x0000F000;


        /* Setup SDRAM Base Address Register: */

        *(UINT32 *)(IXP1200_PCI_DRAM_BAR) = 0x00000000;


        /*
         * Enable IXP1200 as Bus master and respond to I/O space and memory space
         * transactions as target
         */

        sysWrite16 ((volatile UINT16 *)(IXP1200_PCI_COMMAND),
                    IXP1200_PCI_COMMAND_IOSE |  /* I/O Space Enable */
                    IXP1200_PCI_COMMAND_MSE  |  /* Mem Space Enable */
                    IXP1200_PCI_COMMAND_ME   |  /* Master Enable */
                    IXP1200_PCI_COMMAND_MWI     /* Master Wr & Invalidate En */
            );

        }

    /*
     * Set Initialise Complete in PCI Control Register: allow IXP1200 to
     * respond to PCI configuration cycles.
     */

    *(UINT32 *)(IXP1200_SA_CONTROL) =
	    (*(UINT32 *)(IXP1200_SA_CONTROL)) | 0x00000001;

    return;
}

void
hpcInit(void)
{
    if (! (bootFlags & BOOT_DONT_INIT_HPC)) {
        hpcMemInit();
    }
    if (! (bootFlags & BOOT_DONT_INIT_PCI)) {
        sysPciInit();
    }
    if (! (bootFlags & BOOT_DONT_INIT_HPC)) {
        IXP1200_REG_WRITE(IXP1200_DBELL_PCI_MASK, 0);
        IXP1200_REG_WRITE(IXP1200_DBELL_SA_MASK,  0);
        hpcSysInit(0);
    } else {
        hpcSysInit(1);
    }
}

static void
b0_intlat_fix(void)
{
    int dat;

    IXP1200_REG_READ(IXP1200_PCI_INT_LAT, dat);
    if ((dat & 0x0000FF00) != 0x00000100) {
        dat = (dat & 0x000000FF) | 0x100;
        IXP1200_REG_WRITE(IXP1200_PCI_INT_LAT, dat);
    }
}

void
hpcSendChar(unsigned char c)
{
    unsigned int j;
    for (j=0; j<1; ) {
        if (sysCpuRev == 1)
            b0_intlat_fix();
        j += hpcPolledSend(HPC_CHANNEL, &c, 1);
    }
}

void
hpcSend(unsigned char *buf)
{
    unsigned int i,j;
    for (i=0; buf[i]; i++);
    for (j=0; j<i; ) {
        if (sysCpuRev == 1)
            b0_intlat_fix();
        j += hpcPolledSend(HPC_CHANNEL, buf+j, i-j);
    }
}

void
hpcSendMaybe(unsigned char *buf)
{
    unsigned int i,j;
    for (i=0; buf[i]; i++);
    for (j=0; j<i; ) {
        if (sysCpuRev == 1)
            b0_intlat_fix();
        j += hpcPolledSend(HPC_CHANNEL, buf+j, i-j);
        if (j == 0)
            break;
    }
}

unsigned char
hpcGet(void)
{
    unsigned char buf[1];

    if (sysCpuRev == 1)
        b0_intlat_fix();
    while (0 == hpcPolledRcv(HPC_CHANNEL, buf, 1, 0)) {
        if (sysCpuRev == 1)
            b0_intlat_fix();
        scroll_marquee();
    }

    return buf[0];
}

int
hpcGetMaybe(char *c)
{
    if (sysCpuRev == 1)
        b0_intlat_fix();
    return (0 == hpcPolledRcv(HPC_CHANNEL, c, 1, 0));
}
