/* os.h
 * Definitions of constants related to os memory map
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: BootMgr
 * author: jdg 05/03/00
 * revisions:
 * $History: os.h $
 * 
 * *****************  Version 7  *****************
 * User: Jdguilfo     Date: 11/14/00   Time: 5:21p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Spectacle Island Port, includes fixes bugs 1226 & 1227, more checks
 * before updating flash util block
 * 
 * *****************  Version 6  *****************
 * User: Jdguilfo     Date: 10/26/00   Time: 3:38p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge serial and HPC versions into one version, see bootMgr.txt for
 * details.
 * 
 * *****************  Version 5  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 2:27p
 * Updated in $/dev_1200/BoardSupport/BootFlash
 * Merge Boot Manager & Flash Utility, add HPC
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 8/28/00    Time: 6:05p
 * Updated in $/dev_1200/BoardSupport/BootMgr
 * Add support for multiple OS (cygmon)
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 5/10/00    Time: 4:46p
 * Created in $/dev_1200/BoardSupport/BootMgr
 * Add BootMgr feature
 * 
 * ------------------------------------------------------------------------
 */

#ifndef __OS_H__
#define __OS_H__

#define FUTIL_ADDR  (UINT)(flash_main)
#define DIAGS_ADDR  0x0000C000
#define ANGEL_ADDR  0x00020000
#define VX_ADDR     0x00040000
#define CYGMON_ADDR 0x00140000

#define EXCEPT_ADDR 0x101FFFC0

#define BOOT_FLAGS  (BOOT_DONT_INIT_PCI | BOOT_DONT_INIT_HPC | BOOT_IO_HPC)
#define BOOT_SFLAGS (BOOT_DONT_INIT_PCI | BOOT_DONT_INIT_HPC | BOOT_IO_SERIAL)

/*
 * OS Table fields:
 * string:  OS name
 * int:     Starting address
 * int:     Boot Flags
 * int:     Boot Flags for Serial IO
 * int:     Boot Flags for HPC IO
 */
#define OS_TABLE {                 \
    {"Flash Utility", FUTIL_ADDR,  0, BOOT_IO_SERIAL, BOOT_FLAGS}, \
    {"Diagnostics",   DIAGS_ADDR,  0, 0,              0          }, \
    {"Angel/UCOS",    ANGEL_ADDR,  0, 0,              0          }, \
    {"VxWorks",       VX_ADDR+4,   2, BOOT_SFLAGS,    BOOT_FLAGS}, \
    {"Cygmon",        CYGMON_ADDR, 0, BOOT_IO_SERIAL, BOOT_IO_HPC} }

#endif /* ifndef __OS_H__ */
