;*************************************************************************
; *
; * fbox.s : Source file for uEngine tests and routines.
; * 
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: fbox.s $
; *
;*************************************************************************

;Header file includes...
	INCLUDE	H_MENG.S
	INCLUDE	H_PCI.S
	INCLUDE	DIAG.S
	INCLUDE	MMUMACRO.S
	INCLUDE	COMMACRO.S
	INCLUDE	H_SASYS.S
	INCLUDE	H_CONFIG.S

	EXPORT	uStoreLoad


	AREA	ucode,	CODE

	IF	:DEF:	USTORE_TEST
	
	EXPORT	StartUstoreTest

;*************************************************************************
;
; StartUstoreTest : Routine to test the ustore for all 6 uengines. 
;
;	Passed:		Nothing.
;
;	Uses:		R4, R5, R8-R12
;
;	Assumes:	Nothing.
;
;	Returns:	R0 = 0: test passed, R0 != 0: test failed.
;
;*************************************************************************
StartUstoreTest
	mov	r12, lr

;cycle the F-Boxs in and out of reset...
	ldr     r9, =PCI_RESET		;PCI reset reg addr
	ldr     r8, =0x3f 			;all uEngines to reset
	str     r8, [r9]        	;
	eor     r8, r8, r8			;remove uEngines from reset
	str     r8, [r9]			;now Uengine out of reset 
								; and contexts disabled

; test will load ustore with data = (ustoreidx || uEngCSRaddr || ECS)...

	ldr	r9,  =MENG5_BASE_ADDR	;address of last engine (main loop limit)
	ldr	r10, =MENG0_BASE_ADDR:OR:USTORE_ADDR	;init main loop var with 
												; engine0 ustore reg
3
	mov	r4, #ECS				;init r4 to set ECS bit in addr reg
	orr	r5, r4, #USTORE_SIZE	;set write loop limit = (1K||ECS)
	bl	Test1EngUstore			;test uEng N ustore
	cmp	r0, #0x0				;check for error condition
	bne	ExitUcodeLoad			;non-zero indicates test fail
	add	r10, r10, #0x0800		;else, inc to the next uEng ustore addr
	cmp	r10, r9					;all engines completed?
	ble	%B03					;if not then do it again
								;else we are done
ExitUcodeLoad
	mov	pc, r12					;return to caller


;*************************************************************************
;
; Test1EngUstore : Routine to write and read verify the ustore of one ueng.
;
;	Passed:		R10 is io addres of the ustore index csr for this uengine.
;
;	Uses:		R0, R2-R4
;
;	Assumes:	Nothing.
;
;	Returns:	R0 = 0: test passed, R0 != 0: test failed.
;
;*************************************************************************
Test1EngUstore
1
	add	r3, r4, r10				;((uEngCSRbase + uCodestoreidx + ECS) to r3
	str	r4, [R10]				;(uCodestoreidx + ECS) to Ustoreidxreg
	str	r3, [R10,#+4]			;r3 to uCode data reg
	add	r4, r4, #1				;inc r4 to next ustore idx
	cmp	r4, r5					;more then 1k?
	ble	%B01					;if not then keep filling
								;else move on to do read test
	mov	r4, #ECS				;re-init r4 to ECS bit in addr reg
2
	add	r3, r4, r10				;((uEngCSRbase + uCodestoreidx + ECS) to r3
	str	r4, [R10]				;(uCodestoreidx + ECS) to Ustoreidxreg
	ldr	r2, [R10,#+4]			;r2 = uCode data reg
	cmp	r2, r3					;read data OK? (ie: compare with addr?)
	bne	ExitUcodeError			;if not exit to error!
	add	r4, r4, #1				;else, inc r4
	cmp	r4, r5					;more then 1K?
	ble	%B02					;if not then keep read/verifying

	eor	r0, r0, r0
	b	ExitUcode
ExitUcodeError
	mov	r0, #0x1
ExitUcode
	mov	pc, lr					;return to caller

	ENDIF	;USTORE_TEST


	IF	:DEF:	UENGINE_TEST

	IMPORT	UARTSendString
	IMPORT	PutHexc1
	IMPORT	PutWordc1
	IMPORT	Meng	
	IMPORT	Ctx		
	IMPORT	Cmplt
	IMPORT	Nfg		

	EXPORT	uEngFuncTest

;*************************************************************************
;
; uEngFuncTest : Routine to test the functionality of all 6 uengines by:
;					1) Resetting  the uengines
;					2) Downloading ucode
;					3) Enabling each context
;					4) Waiting for the completion code from the ctx
;					5) Reporting any errors encountered
;
;	Passed:		Nothing.
;
;	Uses:		R0-R10, R12
;
;	Assumes:	Nothing.
;
;*************************************************************************
uEngFuncTest
	mov		r12, lr

;cycle the F-Boxs into reset...
	ldr     r9, =PCI_RESET		;PCI reset reg addr
	ldr     r8, =0x3F 			;all uEngines in reset
	str     r8, [r9]        	;

    LDR	R10, =MENG0_BASE_ADDR 

LoadNextUstore

;Disable Event Wake_up for this Fbox (ctx0-3) 
    MOV	R0, #0 
    STR	R0, [R10,#CTX_0_WAKE_EVENTS]	
    STR	R0, [R10,#CTX_1_WAKE_EVENTS] 	
    STR	R0, [R10,#CTX_2_WAKE_EVENTS] 	
    STR	R0, [R10,#CTX_3_WAKE_EVENTS] 	

;Disable Ctx_enables for Fbox 
    STR	R0, [R10,#CTX_ENABLES] 	

;Download the code to uStore...
	adrl	r0, ucode_start						;start address of ucode
	mov		r1, r10
	add		r1, r1, #USTORE_ADDR				;ustore addres
	adrl	r3, ucode_start					
	adrl	r4, ucode_end					
	sub		r2, r4, r3							;length of ucode block
	mov		r2, r2, LSR #0x2					;div length by 4 for dword
	bl		uStoreLoad							;load the code
	add		r10, r10, #0x800

	ldr		r0, =MENG5_BASE_ADDR
	cmp		r10, r0
	ble		LoadNextUstore

;cycle the F-Box out of reset...
	ldr     r9, =PCI_RESET		;PCI reset reg addr
	ldr     r8, =0x0 			;uEngines out of reset
	str     r8, [r9]			;now Uengine out of reset 
	
    LDR	R10, =MENG0_BASE_ADDR 

ExecNextUstore

;clear bit10 of Ustore_address CSR (use tbox_icl address instead)
    MOV    R2, #0    	; <10> = enable bit, <9:0> address 
    STR    R2, [R10] 

;set CTX_ARB_CNTL 
    STR     R2, [R10,#CTX_ARB_CNTL] 

;set CTX_STATUS 
    STR     R2, [R10,#CTX_0_STATUS] 
    STR     R2, [R10,#CTX_1_STATUS] 
    STR     R2, [R10,#CTX_2_STATUS] 
    STR     R2, [R10,#CTX_3_STATUS] 

;Enable Event Wake_up for this Fbox (ctx0-3) 
    MOV	R0, #1 
    STR	R0, [R10,#CTX_0_WAKE_EVENTS] 
    STR	R0, [R10,#CTX_1_WAKE_EVENTS] 
    STR	R0, [R10,#CTX_2_WAKE_EVENTS] 
    STR	R0, [R10,#CTX_3_WAKE_EVENTS] 

UEngCtxOK		EQU	0xF
UEngCtxNG		EQU	0xbaddbadd
TempStackBase	EQU	DRAMBase+DRAMSize-SZ_16K

; First clear out mailboxes in sram...
	ldr	r6,	=SRAMBase		
	ldr	r7, =0x0
	str	r7, [r6, #0x400]
	str	r7, [r6, #0x800]
	str	r7, [r6, #0xC00]
	add	r6,	r6, #0x1000 
	str	r7, [r6]
	ldr	r8, =UEngCtxOK
	ldr	r5, =UEngCtxNG
	ldr	r6,	=SRAMBase+0x400
	ldr	r13, =TempStackBase
	mov	r9, #0

;Enable Ctx_enables for Fbox 
    LDR	R0, =0xf00 
    STR	R0, [R10,#CTX_ENABLES] 

WaitOnCtx
	ldr r0, [r6]				;data to r0
	cmp	r0, r5					;is the ctx fail code present?
	bne	Ctx_OK					;if not check for ctx passed code
	stmfd	r13!, {r0 - r12}	;else report error for this engine/thread
	mov	r2, #0x1				;r11 to !0 for error
	bl	ReportMengStatus		;send the message to user
	ldmfd	r13!, {r0 - r12}	;restore regs
	b	NextCtx					;on to next thread

Ctx_OK							;arrive here and there was no failure (yet)
	cmp	r0, r8					;check for thread completed ok code
	bne	WaitOnCtx				;if not then keep polling

;Reporting that each thread is OK is a bit verbose on the terminal screen
; so remove for release of the code.  Can be added back at any time.
;
;	stmfd	r13!, {r0 - r12}	;else report thread completed OK
;	mov	r2, #0					;r11 to 0 for thread OK
;	bl	ReportMengStatus		;send the message to user
;	ldmfd	r13!, {r0 - r12}

NextCtx							;setup for next context
	add	r6, r6, #0x400			;next thread's sram mailbox
	add	r9, r9, #1				;inc context by one
	mov	r0, #0x4				
	cmp	r9, r0					;any ctx left to test
	blt	WaitOnCtx				;then do 'em

;Disable Event Wake_up for this Fbox (ctx0-3) 
    MOV	R0, #0 
    STR	R0, [R10,#CTX_0_WAKE_EVENTS]	
    STR	R0, [R10,#CTX_1_WAKE_EVENTS] 	
    STR	R0, [R10,#CTX_2_WAKE_EVENTS] 	
    STR	R0, [R10,#CTX_3_WAKE_EVENTS] 	

;Disable Ctx_enables for Fbox 
    STR	R0, [R10,#CTX_ENABLES] 	

	add		r10, r10, #0x800
	ldr		r0,	=MENG5_BASE_ADDR
	cmp		r10, r0
	ble		ExecNextUstore

	mov		pc,	r12								;done!

	ENDIF	;UENGINE_TEST

;*************************************************************************
;
; uStoreLoad : Routine to load the uStore of one microengine.
;
;	Passed:	R0 = 32bit address of uCode to be loaded
;			R1 = index to USTORE_ADDR reg of microengine to load
;			R2 = size (in DWORDs) of ucode to be loaded
;
;	Uses:	R3, R4 contents destroyed
;
;	Assumes:	Specified microengine is in reset and/or contexts disabled
;
;	Returns:	Nothing.
;
;*************************************************************************

uStoreLoad

	orr	r2, r2, #ECS
	ldr	r4, =ECS			; init for ustore addr
4
	ldr	r3, [r0], #4		; ustore data to r3
	str	r4, [r1]			; write ustore addr
	str	r3, [r1, #4]		; write ustore data
	add	r4, r4, #1			; inc ustore addr
	cmp	r4, r2
	bne	%B04
	
	mov	pc, lr				;return to caller



	IF	:DEF:	USTORE_TEST

;*************************************************************************
;
; uStoreVerify : Routine to verify contents of ustore against data pointed 
;					to by R0.  
;
;	Passed:	R0 = 32bit address of uCode
;			R1 = index to USTORE_ADDR reg of microengine to verify
;			R2 = size (in DWORDs) of ucode to be verified
;
;	Uses:	R3, R4, and r5 contents destroyed
;
;	Assumes:	Specified microengine is NOT in reset and contexts disabled
;
;	Returns:	R0 = 0: test passed, R0 != 0: test failed.
;
;*************************************************************************

uStoreVerify
	orr	r2, r2, #ECS
	ldr	r4, =ECS			; init for ustore addr
5
	ldr	r3, [r0], #4		; ustore data to r3
	str	r4, [r1]			; write ustore addr
	ldr	r5, [r1, #4]		; read ustore data
	cmp	r3, r5				; data equal to that written?
	bne	ExitErrorUSV		; if not then exit w/error

	add	r4, r4, #1			; else, inc ustore addr
	cmp	r4, r2				; done with all locations?
	bne	%B05				; if not then keep goin'
							; elase exit w/no error
ExitOKUSV					
	mov	r0, #0x0
	b	ExitUSV
ExitErrorUSV
	mov	r0, #0x1
ExitUSV	
	mov	pc, lr				;return to caller

	ENDIF	;USTORE_TEST


	IF	:DEF:	UENGINE_TEST

;*************************************************************************
;
; ReportMengStatus : Routine to report test status of one microengine context.
;
;	Passed:	R9  = context # (0-3)
;			R10 = microengine base address
;			R2 = 0:context passed, !=0:context failed
;
;	Uses:		R0, R8, R12
;
;	Assumes:	R13 is a valid stack pointer
;
;	Returns:	R0 = 0: test passed, R0 != 0: test failed.
;
;*************************************************************************
ReportMengStatus
	mov	r12, lr

	adrl	r0, Meng
	printfs

	ldr	r8, =MENG0_BASE_ADDR
	sub	r0, r10, r8
	mov	r0, r0, LSR #11
	PutWordStack

	adrl	r0, Ctx
	printfs

	mov	r0, r9
	PutWordStack

	cmp	r2, #0x0
	beq	UEPassed

	adrl	r0, Nfg
	b	UESendStat

UEPassed
	adrl	r0, Cmplt

UESendStat
	bl	UARTSendString

	mov	pc, r12



;------------------------ 
; Ucode Data for FBOX 
;------------------------ 
ucode_start

	INCLUDE	fbox_proto.s

ucode_end
	ENDIF	;UENGINE_TEST

	END
