;*************************************************************************
; *
; * H_SASYS.S : Equate file for the IXP1200 StrongARM System registers.
; * 
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: h_sasys.s $
; *
;*************************************************************************

	INCLUDE	H_CONFIG.S


;StrongARM core register base and offset definitions...

SA_BASE_ADDR	EQU	0x90000000

OFF_PLL_CTL		EQU	0x0C00	;PLL control register
OFF_INTR_FIQ	EQU	0x1000	;FIQ Interrupt status register
OFF_INTR_IRQ	EQU	0x1400	;IRQ Interrupt status register
OFF_GPIO_PEN	EQU	0x1800	;GPIO enable register
OFF_GPIO_PDR	EQU	0x1C00	;GPIO data register
OFF_RTC_DIV		EQU	0x2000	;RTC divisor register
OFF_RTC_TINT	EQU	0x2400	;RTC trim interval register
OFF_RTC_TVAL	EQU	0x2800	;RTC trim value register
OFF_RTC_TIM		EQU	0x2C00	;RTC time register
OFF_RTC_ALM		EQU	0x3000	;RTC alarm register
OFF_UART_USR	EQU	0x3400	;UART Status Reg IO addr
OFF_UART_UCR	EQU	0x3800	;UART Control Reg IO addr
OFF_UART_UDR	EQU	0x3C00	;UART Data Reg IO addr


;StrongARM core register absolute definitions...

PLL_CTL		EQU	SA_BASE_ADDR+OFF_PLL_CTL	;PLL control register
INTR_FIQ	EQU	SA_BASE_ADDR+OFF_INTR_FIQ	;FIQ Interrupt status register
INTR_IRQ	EQU	SA_BASE_ADDR+OFF_INTR_IRQ	;IRQ Interrupt status register
GPIO_PEN	EQU	SA_BASE_ADDR+OFF_GPIO_PEN	;GPIO enable register
GPIO_PDR	EQU	SA_BASE_ADDR+OFF_GPIO_PDR	;GPIO data register
RTC_DIV		EQU	SA_BASE_ADDR+OFF_RTC_DIV	;RTC divisor register
RTC_TINT	EQU	SA_BASE_ADDR+OFF_RTC_TINT	;RTC trim interval register
RTC_TVAL	EQU	SA_BASE_ADDR+OFF_RTC_TVAL	;RTC trim value register
RTC_TIM		EQU	SA_BASE_ADDR+OFF_RTC_TIM	;RTC time register
RTC_ALM		EQU	SA_BASE_ADDR+OFF_RTC_ALM	;RTC alarm register
UART_USR	EQU	SA_BASE_ADDR+OFF_UART_USR	;UART Status Reg IO addr
UART_UCR	EQU	SA_BASE_ADDR+OFF_UART_UCR	;UART Control Reg IO addr
UART_UDR	EQU	SA_BASE_ADDR+OFF_UART_UDR	;UART Data Reg IO addr


;PLL_CTL Register bit definitions...

;Rev "A" silicon:
Core29Mhz	EQU	0x00
Core37Mhz	EQU	0x01
Core44Mhz	EQU	0x02
Core52Mhz	EQU	0x03
Core59Mhz	EQU	0x04
Core66Mhz	EQU	0x05
Core74Mhz	EQU	0x06
Core81Mhz	EQU	0x07
Core88Mhz	EQU	0x08
Core96Mhz	EQU	0x09
Core103Mhz	EQU	0x0A
Core111Mhz	EQU	0x0B
Core118Mhz	EQU	0x0C
Core133Mhz	EQU	0x0D
Core147Mhz	EQU	0x0E
Core155Mhz	EQU	0x0F
Core162Mhz	EQU	0x10
Core177Mhz	EQU	0x11
Core192Mhz	EQU	0x12

;Rev "B" silicon:
BCore29Mhz	EQU	Core29Mhz	
BCore37Mhz	EQU	Core37Mhz	
BCore44Mhz	EQU	Core44Mhz	
BCore52Mhz	EQU	Core52Mhz	
BCore59Mhz	EQU	Core59Mhz	
BCore66Mhz	EQU	Core66Mhz	
BCore74Mhz	EQU	Core74Mhz	
BCore81Mhz	EQU	Core81Mhz	
BCore88Mhz	EQU	Core88Mhz	
BCore96Mhz	EQU	Core96Mhz	
BCore103Mhz	EQU	Core103Mhz
BCore111Mhz	EQU	Core111Mhz
BCore133Mhz	EQU	0x0C
BCore147Mhz	EQU	0x0D
BCore155Mhz	EQU	0x0E
BCore162Mhz	EQU	0x0F
BCore166Mhz	EQU	0x10
BCore177Mhz	EQU	Core177Mhz
BCore192Mhz	EQU	Core192Mhz
BCore199Mhz	EQU	0x13


;INTR_FIQ Register bit definitions...

SDRAMFIQ	EQU	0x80	;SDRAM
RTCFIQ		EQU	0x40	;RTC (fiq only)
SRAMFIQ		EQU	0x20	;SRAM
FBOXFIQ		EQU	0x10	;FBOX
MACFIQ		EQU	0x08	;MAC
PCIFIQ		EQU	0x04	;PCI


;INTR_IRQ Register bit definitions...

SDRAMIRQ	EQU	SDRAMFIQ
UARTIRQ		EQU	RTCFIQ		;UART (irq only)
SRAMIRQ		EQU	SRAMFIQ	
FBOXIRQ		EQU	FBOXFIQ	
MACIRQ		EQU	MACFIQ	
PCIIRQ		EQU	PCIFIQ	


;RTC_DIV Register bit definitions...

DIV_INTWR	EQU	0x00010000
DIV_INTEN	EQU	0x00020000
DIV_INTRST	EQU	0x00040000


;RTC_TVAL Register bit definitions...

TVAL_TEST0	EQU	0x00010000
TVAL_TEST1	EQU	0x00020000


;UART_USR register bit definitions...

GIO			EQU	0x00000100		;State of the RxD input
TBY			EQU	0x00000080		;Transmit FIFO full
RFF			EQU	0x00000040		;Receive FIFO full
TFE			EQU	0x00000020		;Transmit FIFO empty
RCV			EQU	0x00000010		;Receiver ready
ROR			EQU	0x00000008		;Receive overrun
TXV			EQU	0x00000004		;Transmtter ready
FRE			EQU	0x00000002		;Framing error
PRE			EQU	0x00000001		;Parity error


;UART CONTROL register bit definitions...

BAUD_MASK	EQU	0x03FF0000		;BAUD rate divisor mask
BAUD_9600	EQU	0x00170000		;divisor for 9600 BAUD 3.6864MHz
BAUD_19200	EQU	0x000B0000		;divisor for 9600 BAUD 3.6864MHz
BAUD_38400	EQU	0x00050000		;divisor for 9600 BAUD 3.6864MHz
BAUD_57600	EQU	0x00030000		;divisor for 9600 BAUD 3.6864MHz
BAUD_115200	EQU	0x00010000		;divisor for 9600 BAUD 3.6864MHz
BAUD_230400	EQU	0x00000000		;divisor for 9600 BAUD 3.6864MHz

UIC			EQU	0x00000200		;UART interrupt clear
XIE			EQU	0x00000100		;Transmit interrupt enable
UE			EQU	0x00000080		;UART enable
DSS_MASK	EQU	0x00000060		;Data size select 0=5,...3=8
DSS_8		EQU	0x00000060		;Data size = 8 bits
DSS_7		EQU	0x00000040		;Data size = 7 bits
DSS_6		EQU	0x00000020		;Data size = 6 bits
DSS_5		EQU	0x00000000		;Data size = 5 bits
RIE			EQU	0x00000010		;Receive interrupt enable
SBS_MASK	EQU	0x00000008		;Stop bits select 0=1, 1=2
SBS_2		EQU	0x00000008		;2 Stop bits selected (TxD)
SBS_1		EQU	0x00000000		;1 Stop bits selected (TxD)
OES_MASK	EQU	0x00000004		;Parity select 0=even, 1=odd
OES_ODD		EQU	0x00000004		;Parity odd select
OES_EVN		EQU	0x00000000		;Parity even select
P_ENBL		EQU	0x00000002		;Parity enable
BRK			EQU	0x00000001		;Transmit break


; UART DATA register bit definitions...

UDR_ROR		EQU	0x400			; Receive overrun
UDR_FRE		EQU	0x200			; Framing error
UDR_PRE		EQU	0x100			; Parity error


	END
