;*************************************************************************
; *
; * MACPORT.S : Source file for the validation platform MAC tests.
; * 		   
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: macport.s $
; *
;*************************************************************************

;Header file includes...
	INCLUDE	H_21440.S
	INCLUDE	H_82547.S
	INCLUDE	DIAG.S
	INCLUDE	H_SASYS.S
	INCLUDE	H_CONFIG.S
	INCLUDE	MMUMACRO.S

;Macro file includes...

;External procedures...
	IMPORT	UARTSendString
	IMPORT	PutHexc1
	
;Procedures used externally found in this module...
	
;Diag message strings...
	IMPORT	MAC0DID
	IMPORT	MAC0REV
	IMPORT	GMACDID
	IMPORT	GMACREV
	IMPORT	MAC1DID
	IMPORT	MAC1REV
	IMPORT	CRLF
	IMPORT	Nfg
	IMPORT	Read
	IMPORT	Expected
	IMPORT	MACPort


MacTempStackBase	EQU		DRAMBase+DRAMSize-SZ_16K	;use top of dram-16k in case of 
														;mmu descriptor table 

	AREA	MACTEST, CODE, READONLY

	IF	:DEF:	MAC_TEST

	EXPORT	MACPortTest

;*************************************************************************
;
; MACPortTest : Routine to read the MAC device ID and REV registers and 
;				compare the data with that which is expected. 
;
;	Passed:		Nothing.
;
;	Uses:		R0-3, R12, R13
;
;	Assumes:	Nothing.
;
;	Returns:	Reports test status to console.
;
;*************************************************************************
MACPortTest		; NOTE: 21440 MAC devices are byte-wide on the CPU port!

	mov	r12, lr				;save the link register so we can do calls
	ldr	r13, =MacTempStackBase	;init stack pointer
	ldr	r1, =MAC0_Dev_ID	; address to MAC 0 device ID register
	
	stmfd	r13!, {r0 - r12}
	adrl	r0, MAC0DID
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	ldrb	r0,	[r1], #4	; get the chip ID data

	stmfd	r13!, {r0 - r12}
	bl	PutHexc1
	adrl	r0, CRLF
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	cmp	r0, #Dev_ID_Data	; compare with expected data
	bne	%F01				; if no match then exit with error

	stmfd	r13!, {r0 - r12}
	adrl	r0, MAC0REV
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	ldrb	r0,	[r1]		; else read the device revision ID register

	stmfd	r13!, {r0 - r12}
	bl	PutHexc1
	adrl	r0, CRLF
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	cmp	r0, #Rev_ID_Data	; compare with expected data
	bne	%F01				; if no match then exit with error



	ldr	r1, =GMAC_Dev_ID_RevL	; address to GMAC  device ID register
	
	stmfd	r13!, {r0 - r12}
	adrl	r0, GMACDID
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	ldrb	r0,	[r1]			; get the ID data
	mov		r2, r0				; save into r2

	stmfd	r13!, {r0 - r12}
	bl	PutHexc1				; display id
	adrl	r0, CRLF
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	adrl	r0, GMACREV
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	ldr	r1, =GMAC_Dev_ID_RevH	; address to GMAC  device rev register
	ldrb	r0,	[r1]			; get the rev / ID data

	stmfd	r13!, {r0 - r12}
	bl	PutHexc1				; display rev
	adrl	r0, CRLF
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	cmp	r2, #GMAC_Dev_ID_Data	; compare with expected data
	bne	%F01				; if no match then exit with error


	mov	r0, #0x0			; return with r0=0 if no error
	b	%F02
1	
	mov	r0, #1				; return with r0!=0 if error
2
	mov	pc, r12				; return to calling routine

	ENDIF	;MAC_TEST

	
	IF	:DEF:	MAC_WRTEST

	EXPORT	MacPortUnique
	IMPORT	MACPortTbl

;*************************************************************************
;
; MacPortUnique : Routine to determine all ports are uniquely addressable. 
;
;	Passed:		Nothing.
;
;	Uses:		R0-R7		
;
;	Assumes:	Nothing.
;
;	Returns:	Test failures/success reported to console.
;
;*************************************************************************
MacPortUnique
	ldr	r13, =MacTempStackBase	;init stack pointer
	mov		r12, lr

	mov		r1, #0x0			;init port data var
	adrl	r5, MACPortTbl		;offset of table to r5

MacDevWrLoop
	ldr		r0, [r5], #4		;load mac io address
	cmp		r0, #0xFFFFFFFF		;end of table?
	beq		MacDevWrDone		;if so we are done
								;else
	mov		r4, #0x0			;init mac port count
	ldr		r2, [r5], #4		;and process table entry
	ldr		r7, [r5], #4		;

MacPortWrloop
	strb	r1, [r0]			;store data to mac reg
	add		r1, r1, #0x1		;inc data for next port
	add		r4, r4, #0x1		;inc total mac port count
	add		r0, r0, r7			;inc addr for next port
	cmp		r4, r2				;was this the last port?
	blt		MacPortWrloop		;if not then do next port
	b		MacDevWrLoop

MacDevWrDone
	mov		r4, #0x0			;init port count
	mov		r1, #0x0			;init port data var
	adrl	r5, MACPortTbl		;offset of table to r5

MacDevRdLoop
	ldr		r0, [r5], #4		;
	cmp		r0, #0xFFFFFFFF		;end of table?
	beq		MacDevRdDone		;if so we are done

	mov		r6, #0x0			;init port count
	ldr		r2, [r5], #4		;else process table entry
	ldr		r7, [r5], #4		;

MacPortRdloop
	ldrb	r3, [r0]			;get port reg data
	cmp		r3, r1				;equal to that written?
	beq	MacPortRdOK
	stmfd	r13!, {r0 - r12}	;else report thread completed OK
	bl	ReportMACStatus			;send the message to user
	ldmfd	r13!, {r0 - r12}

MacPortRdOK
	add		r1, r1, #0x1		;prepare data for next port
	add		r4, r4, #0x1		;inc total port count
	add		r6, r6, #0x1		;inc mac port count
	add		r0, r0, r7			;inc addr for next port
	cmp		r6, r2				;was this the last port?
	blt		MacPortRdloop		;if not then do next port
	b		MacDevRdLoop

MacDevRdDone
	mov		r1, #0x0			;init port data var
	adrl	r5, MACPortTbl		;offset of table to r5

MacDevClrLoop
	ldr		r0, [r5], #4		;
	cmp		r0, #0xFFFFFFFF		;end of table?
	beq		MacDevClrDone		;if so we are done

	mov		r4, #0x0			;init port count
	ldr		r2, [r5], #4		;else process table entry
	ldr		r7, [r5], #4		;

MacPortClrloop
	strb	r1, [r0]			;store data to mac reg
	add		r4, r4, #0x1		;inc port count
	add		r0, r0, r7		;inc addr for next port
	cmp		r4, r2				;was this the last port?
	blt		MacPortClrloop		;if not then do next port
	b		MacDevClrLoop

MacDevClrDone
	mov		r0, #0x0
	mov		pc, r12


;*************************************************************************
;
; ReportMACStatus : Routine to report test status of MAC test.
;
;	Passed:		R4= port#
;				R3= data read
;				R1= data expected
;
;	Uses:		R12
;
;	Assumes:	Nothing.
;
;	Returns:	Nothing.
;
;*************************************************************************
ReportMACStatus

	mov	r12, lr

	stmfd	r13!, {r0 - r12}
	adrl	r0, MACPort
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	mov	r0, r4
	bl	PutHexc1 
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	adrl	r0, Nfg
	bl	UARTSendString
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	adrl	r0, Read
	bl	UARTSendString 
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	mov	r0, r3
	bl	PutHexc1 
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	adrl	r0, Expected
	bl	UARTSendString 
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	mov	r0, r1
	bl	PutHexc1 
	ldmfd	r13!, {r0 - r12}

	stmfd	r13!, {r0 - r12}
	adrl	r0, CRLF
	bl	UARTSendString 
	ldmfd	r13!, {r0 - r12}

	mov	pc, r12

	ENDIF	;MAC_WRTEST

	END