;*************************************************************************
; *
; * MEMORY.S : Source file for the IXP1200 Memory Test Procedures.
; * 		   
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: MEMORY.S $
; 
; *****************  Version 13  *****************
; User: Tepulkow     Date: 8/28/00    Time: 5:28p
; Updated in $/dev_1200/BoardSupport/Diagnostics
; Added relocation of the exception vector table to SRAM.
; *
;*************************************************************************

;Header file includes...
	INCLUDE	H_ARM.S
	INCLUDE	H_CONFIG.S

;Macro file includes...
	INCLUDE MMUMACRO.S
	INCLUDE COMMACRO.S

;External procedures...

;Procedures used externally found in this module...
	
;Diag message strings...

;Data tables...


	AREA	MEMTEST, CODE, READONLY

	IF	:DEF:	MEM_TEST
	EXPORT	MemoryTest


	IMPORT	MemProgress
	IMPORT	MemWrite
	IMPORT	MemRead
	IMPORT	MemClear
	IMPORT	CRLF
	IMPORT	Nfg		
	IMPORT	Read	
	IMPORT	Expected
	IMPORT	CRLF	
	IMPORT	MemLoc
	

;A little presumptuous here using scratch memory for stack in the routine 
; that tests memory (including scratch!) but it is probably our best bet
; being on-chip.  Knowing that...

ScratchTempStackBase	EQU		ScratchBase+ScratchSize	;point to top of scratch
SRAMTempStackBase		EQU		SRAMRWBase+SRAMRWSize-JumpTblSize	;point to top of sram

;*************************************************************************
;
; MemoryTest : Routine to test memory apertures described in table whose
;				offset (32 bit address) is passed in R0. 
;
;	Passed:		R0 = pointer to table of memory base addrs and size.
;
;	Uses:		R0-R5, R12, R13
;
;	Assumes:	Nothing.
;
;	Returns:	Test failures/success reported to console.
;
;*************************************************************************
MemoryTest
	mov		r12, lr

TestNextMem
	ldr	r1, [r0], #4			;table entry (mem base addr), then r0=r0+4
	cmp	r1, #0xFFFFFFFF			;check end of table?
	moveq	r0, #0				;indicate success to caller
	beq	MemTestDone				;if so we are done

	ldr	r2, [r0], #4			;else, load reg with mem size, then r0=r0+4

	cmp	r1, #DRAMBase			;if this is DRAM then need to account for page table
	bne	%F01					;no DRAM, no adjustment

	readControl	r3				;if DRAM then see if MMU is enabled 
	ands	r3, r3, #MMUEn		;if MMU is not enabled don't need to adjust
	beq	%F01

	sub	r2, r2, #TTB_SIZE		;else, decrease the DRAM size by the size of xlate tbl

1
	ldr	r13, =ScratchTempStackBase	;assume scratch avail for stack
	ldr	r10, =ScratchBase
	cmp	r1, r10					;is this test for scratch?
	bne	%F09					;if not, leave stack in scratch

	ldr	r13, =SRAMTempStackBase	;else, use SRAM for stack
9

	stmfd	r13!, {r0 - r12}	;save all 
	ldr	r0, [r0]				;Get descriptor string offset
	bl	UARTSendString			;Let the User know what we are testing
	adrl	r0, MemWrite		;
	bl	UARTSendString			;
	ldmfd	r13!, {r0 - r12}	;restore all
	add		r0, r0, #4			;point to next table entry

; to this point r1=membase, r2=memsize, r0=pointer to next memtest table entry
	ldr	r11, =0xFFFF			;address mask for progress report
	mov	r3, r1					;base to r3
	add	r4, r1, r2				;r4 = limit (base+size)
2
	mov		r10, r3
	ands	r10, r10, r11
	bne		%F06
		
	stmfd	r13!, {r0 - r12}	;save all 
	adrl	r0, MemProgress		;Get progress string offset
	bl	UARTSendString			;Let the User know what we are testing
	ldmfd	r13!, {r0 - r12}	;restore all

6
	str	r3, [r3], #4			;mem addr (r3) is data to mem location, then r3=r3+4
	cmp	r3, r4
	blt	%B02

	mov	r3, r1					;base to r3
	stmfd	r13!, {r0 - r12}	;save all 
	adrl	r0, CRLF			;
	bl	UARTSendString			;
	adrl	r0, MemRead			;
	bl	UARTSendString			;
	ldmfd	r13!, {r0 - r12}	;restore all

3
	mov		r10, r3
	ands	r10, r10, r11
	bne		%F07
		
	stmfd	r13!, {r0 - r12}	;save all 
	adrl	r0, MemProgress		;Get progress string offset
	bl	UARTSendString			;Let the User know what we are testing
	ldmfd	r13!, {r0 - r12}	;restore all

7
	ldr	r5, [r3]				;mem addr (r3) is data from mem location

	cmp	r5, r3

	beq	%F05

; here and we have a memory comparison error
; r0= -> memtbl (pased by caller and non-zero)
; r3= culprit memory location that failed
; r5= data read from culprit memory location 	
	stmfd	r13!, {r0 - r12}	;else    														
	bl	ReportMemStatus			;send the message to user
	ldmfd	r13!, {r0 - r12}
5
	add	r3, r3, #4				;else point to next dword
	cmp	r3, r4					;end of mem range to test?
	blt	%B03					;if not contiue

;Clear out data in this aperture..
	ldr	r5, =0x0
	mov	r3, r1				; base to r3
	stmfd	r13!, {r0 - r12}	;save all 
	adrl	r0, CRLF			;
	bl	UARTSendString			;
	adrl	r0, MemClear		;
	bl	UARTSendString			;
	ldmfd	r13!, {r0 - r12}	;restore all

10
	mov		r10, r3
	ands	r10, r10, r11
	bne		%F08
		
	stmfd	r13!, {r0 - r12}	;save all 
	adrl	r0, MemProgress		;Get progress string offset
	bl	UARTSendString			;Let the User know what we are testing
	ldmfd	r13!, {r0 - r12}	;restore all
8
	str	r5, [r3], #4			;mem addr (r3) is data to mem location, then r3=r3+4
	cmp	r3, r4
	blt	%B10

	stmfd	r13!, {r0 - r12}	;save all 
	adrl	r0, CRLF			;add carridge return after each memory aperture 
	bl	UARTSendString			; 
	ldmfd	r13!, {r0 - r12}	;restore all

	b	TestNextMem				;else look for next aperture to test

MemTestDone
	mov		pc, r12




;*************************************************************************
;
; ReportMemStatus : Routine to report test status of Mem test.
;
;	Passed:	r3= location that failed
;			r5= data read 
;
;	Uses:		R0	
;
;	Assumes:	Nothing.
;
;	Returns:	Test failures/success reported to console.
;
;*************************************************************************
ReportMemStatus

	stmfd	r13!, {r0, lr}			;save lr

	adrl	r0, MemLoc
	printfs

	mov	r0, r3
	PutHexStack 

	adrl	r0, Nfg
	printfs

	adrl	r0, Read
	printfs

	mov	r0, r5
	PutHexStack

	adrl	r0, Expected
	printfs

	mov	r0, r3
	PutHexStack

	adrl	r0, CRLF
	printfs

	ldmfd	r13!, {r0, pc}			;return to caller


	ENDIF	;MEM_TEST


	IF	:DEF:	ROM_TEST
;*************************************************************************
;
; BootDevTest : Routine to determine integrity of boot rom sockets. 
;
;	Passed:		Nothing.
;
;	Uses:		R0-R3
;
;	Assumes:	Nothing.
;
;	Returns:	Test failures/success reported to console.
;
;*************************************************************************
;The purpose of this ROM test is to run out of the NETROM device and check
; the PCB and socket connections to the flash rom device.  For this test 
; to pass,the rom installed must contain data = DWORD address of the 
; location:
;
;	Address		Data
;
;	0x0000000	0x00000000
;	0x0000004	0x00000004
;	0x0000008	0x00000008
;		.			.
;		.			.
;		.			.
;					
;	0X01FFFF4	0X01FFFF4
;	0X01FFFF8	0X01FFFF8
;	0X01FFFFC	0X01FFFFC
;
;*************************************************************************
TRBaseAddr	EQU	0x200000		;test rom base address

	EXPORT	BootDevTest

BootDevTest

	mov	r1, #0x4			;offset to r1
	mov	r0, #TRBaseAddr		;flashrom base address when executing from NetRom

BDTNextLocation
	orr	r2, r1, r0			;assemble rom address in r2
	ldr	r3, [r2]			;load data 
	mov	r3, r3, LSL #2		;multiply by 4
	cmp	r3, r1				;compare with expected
	bne	BDTError

	mov	r1, r1, LSL #1		;walk address up
	cmp	r1, #0x200000		;TRBaseAddr		;Are we through?
	blt	BDTNextLocation

	mov	r0, #0x0
	b	BDTExit

BDTError
	mov	r0, #1

BDTExit
	mov	pc, lr

	ENDIF	;ROM_TEST


	END