;*************************************************************************
; *
; * MMUMACRO.S : Macro definition source file for MMU functions.
; * 		   
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: MMUMACRO.S $
; 
; *****************  Version 11  *****************
; User: Tepulkow     Date: 8/28/00    Time: 5:28p
; Updated in $/dev_1200/BoardSupport/Diagnostics
; Added relocation of the exception vector table to SRAM.
; *
;*************************************************************************

; Coprocessor read of ID register 
;
	MACRO 
$label	readChipId	$w1
	mrc     p15, 0, $w1, c0, c0 ,0
	MEND


; Coprocessor read of Control register 
; 
	MACRO 
$label	readControl	$w1
            mrc     p15, 0, $w1, c1, c0 ,0
	MEND


; Coprocessor write of Control register
;
	MACRO 
$label	writeControl	$w1
            mcr     p15, 0, $w1, c1, c0 ,0
            nop
            nop
            nop
	MEND


; Processor Status Register Read
; 
	MACRO 
$label	readPsr		$w1
            mrs     $w1,cpsr
	MEND


; Processor Status Register Write
;
	MACRO 
$label	writePsr	$w1
            msr     cpsr,$w1
	MEND


; Coprocessor read of Translation Table Base reg. 
; 
	MACRO 
$label	readTBase	$w1
            mrc     p15,0,$w1,c2,c0,0
	MEND


; Coprocessor write of Translation Table Base reg. 
; 
	MACRO 
$label	writeTBase	$w1
            mcr     p15,0,$w1 ,c2,c0,0
	MEND


; Coprocessor read of Domain Access Control reg. 
; 
	MACRO 
$label	readDomainControl	$w1
            mrc     p15,0,$w1,c3,c0,0
	MEND


; Coprocessor write of Domain Access Control reg. 
; 
	MACRO 
$label	writeDomainControl	$w1
            mcr     p15,0,$w1,c3,c0,0
	MEND


; Coprocessor read of Fault Status register 
; 
	MACRO 
$label	readFaultStatus	$w1
            mrc     p15,0,$w1,c5,c0,0
	MEND


; Coprocessor write of Fault Status register 
; 
	MACRO 
$label	writeFaultStatus	$w1
            mcr     p15,0,$w1,c5,c0,0
	MEND


; Coprocessor read of Fault Address register 
; 
	MACRO 
$label	readFaultAddress	$w1
            mrc     p15,0,$w1,c6,c0,0
	MEND


; Coprocessor write of Fault Address register 
; 
	MACRO 
$label	writeFaultAddress	$w1
            mcr     p15,0,$w1,c6,c0,0
	MEND


; Coprocessor cache control 
; Flush ICache + DCache
; 
	MACRO 
$label	flushIDcache
            mcr     p15,0,a1,c7,c7,0
	MEND


; Coprocessor cache control 
; Flush ICache
; 
	MACRO 
$label	flushIcache
            mcr     p15,0,a1,c7,c5,0
	MEND


; Coprocessor cache control 
; Flush DCache
; 
	MACRO 
$label	flushDcache
            mcr     p15,0,a1,c7,c6,0
	MEND


; Coprocessor cache control 
; Flush DCache entry
; 
	MACRO 
$label	flushDcacheEntry
            mcr     p15,0,a1,c7,c6,1
	MEND


; Coprocessor cache control 
; Clean DCache entry
; 
	MACRO 
$label	cleanDcacheEntry
            mcr     p15,0,a1,c7,c10,1
	MEND


; Coprocessor cache control 
; Clean + Flush DCache entry
; 
	MACRO 
$label	cleanFlushDcacbe
            mcr     p15,0,a1,c7,c14,1
	MEND


; Coprocessor Drain Write Buffer 
; 
	MACRO 
$label	drainWriteBuffer
            mcr     p15,0,a1,c7,c10,4
	MEND


; Coprocessor TLB control 
; Flush ITB + DTB
; 
	MACRO 
$label	flushIDTlb
            mcr     p15,0,a1,c8,c7,0
	MEND


; Coprocessor TLB control 
; Flush ITB
; 
	MACRO 
$label	flushItb
            mcr     p15,0,a1,c8,c5,0
	MEND


; Coprocessor TLB control 
; Flush DTB
; 
	MACRO 
$label	flushDtb
            mcr     p15,0,a1,c8,c6,0
	MEND


; Coprocessor TLB control 
; Flush DTB entry

	MACRO 
$label	flushDtbEntry
            mcr     p15,0,a1,c8,c6,1
	MEND


; Coprocessor test/clock/idle control 
; Enable Clock Switching
; 
	MACRO 
$label	enableClockSwitch
            mcr     p15,0,a1,c15,c1,2
	MEND


; Coprocessor test/clock/idle control 
; Disable Clock Switching
; 
	MACRO 
$label	disableClockSwitch
            mcr     p15,0,a1,c15,c2,2
	MEND


; Coprocessor test/clock/idle control 
; Disable nMCLK output
; 
	MACRO 
$label	disableMClk
            mcr     p15,0,a1,c15,c4,2
	MEND


; Coprocessor test/clock/idle control 
; Wait for Interrupt
; 
	MACRO 
$label	waitForInterrupt
            mcr     p15,0,a1,c15,c8,2
	MEND


        ; ------------------------------------------------------------------
        ; SETUPMMU
        ; --------
        ; Everything is 1-1.
		;
        MACRO
$label  SETUPMMU        $tmp1, $tmp2, $tmp3, $tmp4, $tmp5, $tabl

        ; Setup the StrongARM MMU with a mapped page table:
        ; set up the page tables - all for domain0, switched address map
        ;       RAM_SIZE / 1Mb sections for DRAM
        ;               - Read/Write all, cachable & bufferable
        ;       ROM_SIZE / 1MB sections for ROM
        ;               - Read all, cachable, nonbufferable
        ;       The rest of address space
        ;               - noncachable, nonbufferable

        ; set the domain register: domain0 CLIENT, domain1-15 NO_ACCESS
        ; write the translation base register
        ; Flush ITB's and DTB's
        ; Flush the Icache


        ; First clear all TT entries as FAULT
        ;
        LDR     $tmp4, =TTB_BASE
        LDR     $tmp3, =0
        LDR     $tmp1, =(TTB_SIZE / 4)
1
        STR     $tmp3, [$tmp4], #4
        SUBS    $tmp1, $tmp1, #1        ; decrement loop count
        BNE     %B01

;        ; --------------------------------------------------
;        ; Map DRAM non-prefetch area to itself
;        ; --------------------------------------------------
;
;        LDR     $tmp2, =TTB_BASE
;        LDR     $tmp1, =DRAMBase                ; physical address
;        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
;        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry
;
;        ; Access permissions
;        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
;        ADD     $tmp5, $tmp1, $tmp3             ; table entry for upper RAM
;
;        LDR     $tmp3, =DRAMSize                ; Size of first bank
;        MOV     $tmp3, $tmp3, LSR #20           ; no. of 1MB segments
;2
;        STR     $tmp5, [$tmp4], #4              ; real entry
;        ADD     $tmp5, $tmp5, #SZ_1M
;        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
;        BNE     %B02
;
        ; --------------------------------------------------
        ; Map DRAM non-prefetch area to itself
        ; --------------------------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMBase                ; physical address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =DRAMBase                ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; table entry for upper RAM

        LDR     $tmp3, =DRAMSize                ; Size of first bank
        MOV     $tmp3, $tmp3, LSR #20           ; no. of 1MB segments
2
        STR     $tmp1, [$tmp4], #4              ; real entry
        ADD     $tmp1, $tmp1, #SZ_1M
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B02

        ; --------------------------------
        ; Map DRAM prefetch area to itself
        ; --------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMPrefetchBase        ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =DRAMPrefetchBase        ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =DRAMPrefetchSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
3
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B03

        ; -----------------------------------
        ; Map DRAM cache flush area to itself
        ; -----------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMFlushBase           ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =CACHED_NOBUFFER_ACCESS  ; Yes cache, no buffer
        LDR     $tmp1, =DRAMFlushBase           ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =DRAMFlushSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
4
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B04

        ; -------------------------------
        ; Map DRAM control area to itself
        ; -------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMControlBase         ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =DRAMControlBase         ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =DRAMControlSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
5
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B05

        ; ------------------------------
        ; Map AMBA Translation Unit area
        ; ------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =AMBABase                ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =AMBABase                ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =AMBASize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
6
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B06

        ; -----------------
        ; Map ARM Core area
        ; -----------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =ARMCOREBase             ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =ARMCOREBase             ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =ARMCORESize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
7
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B07

        ; --------------------
        ; Map PCI Memory Space
        ; --------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIMemBase              ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIMemBase              ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIMemSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
8
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B08

        ; -----------------
        ; Map PCI I/O Space
        ; -----------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIIOBase               ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIIOBase               ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIIOSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
9
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B09

        ; ------------------------------------
        ; Map PCI Type One Configuration Space
        ; ------------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIConfigTypeOneBase    ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIConfigTypeOneBase    ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIConfigTypeOneSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
10
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B10

        ; -------------------------------------
        ; Map PCI Type Zero Configuration Space
        ; -------------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIConfigTypeZeroBase   ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIConfigTypeZeroBase   ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIConfigTypeZeroSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
11
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B11


        ; ---------------------------------
        ; Map PCI Local Configuration Space
        ; ---------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCILocalBase            ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCILocalBase            ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCILocalSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
12
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B12

        ; -----------------------
        ; Map SRAM & Control area
        ; -----------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =SRAMBase                ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
;        LDR     $tmp3, =CACHED_BUFFERED_ACCESS ; No cache, no buffer
        LDR     $tmp1, =SRAMBase                ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =SRAMSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
13
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B13

        ; ------------
        ; Map ROM area
        ; ------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =ROM_BASE                ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry

        ; Access permissions
        LDR     $tmp3, =CACHED_NOBUFFER_ACCESS  ; No cache, no buffer
        LDR     $tmp1, =ROMBase                 ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =ROM_SIZE
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
14
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B14

        ; ----------------
        ; Map SYS I/O area
        ; ----------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =SYSIO_BASE              ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =SYSIOBase               ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =SYSIO_SIZE
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
15
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B15

        MEND





SZ_1K                   EQU     0x00000400
SZ_8K                   EQU     0x00002000
SZ_16K                  EQU     0x00004000
SZ_32K                  EQU     0x00008000
SZ_64K                  EQU     0x00010000
SZ_128K                 EQU     0x00020000
SZ_256K                 EQU     0x00040000
SZ_512K                 EQU     0x00080000

SZ_1M                   EQU     0x00100000
SZ_2M                   EQU     0x00200000
SZ_4M                   EQU     0x00400000
SZ_5M                   EQU     0x00500000
SZ_8M                   EQU     0x00800000
SZ_16M                  EQU     0x01000000
SZ_32M                  EQU     0x02000000
SZ_64M                  EQU     0x04000000
SZ_128M                 EQU     0x08000000
SZ_256M                 EQU     0x10000000
SZ_512M                 EQU     0x20000000

SZ_1G                   EQU     0x40000000


; --------------------------------
; IXP1200 physical memory addresses
; --------------------------------

ScratchBase				EQU		0xB0044000
ScratchSize				EQU		0x00001000

SYSIOBase               EQU     0x38400000
SYSIOSize               EQU     SZ_1M

ROMBase                 EQU     0x00000000
ROMSize                 EQU     SZ_4M		;2M actual flash rom and add 2 meg for CE1

SRAMBase                EQU     0x10000000
SRAMSize                EQU     (SZ_512M + SZ_256M)

SRAMRWBase              EQU     SRAMBase
SRAMRWSize              EQU     SZ_2M

RDLCKBase               EQU     0x12000000
RDLCKSize               EQU     SRAMSize

WRUNLCKBase             EQU     0x14000000
WRUNLCKSize             EQU     SRAMSize

CAMUNLCKBase            EQU     0x16000000
CAMUNLCKSize            EQU     SRAMSize

BITWRCLRBase            EQU     0x18000000
BITWRCLRSize            EQU     SRAMSize

BITWRSETBase            EQU     0x18800000
BITWRSETSize            EQU     SRAMSize

BITTSTCLRBase           EQU     0x19000000
BITTSTCLRSize           EQU     SRAMSize

BITTSTSETBase           EQU     0x19800000
BITTSTSETSize           EQU     SRAMSize

PUSHCMDBase             EQU     0x20000000
PUSHCMDSize             EQU     SZ_64M  

POPCMDBase              EQU     0x24000000
POPCMDSize              EQU     SZ_64M  

SRAMIOBase              EQU     0x38000000
SRAMIOSize              EQU     SZ_1M

PCILocalBase            EQU     0x42000000
PCILocalSize            EQU     SZ_1M

PCIConfigTypeOneBase    EQU     0x52000000
PCIConfigTypeOneSize    EQU     SZ_32M

PCIConfigTypeZeroBase   EQU     0x53000000
PCIConfigTypeZeroSize   EQU     SZ_32M

PCIIOBase               EQU     0x54000000
PCIIOSize               EQU     SZ_1M

PCIMemBase              EQU     0x60000000
PCIMemSize              EQU     SZ_512M

PCIConfigBase           EQU     0x52000000
PCIConfigSize           EQU     SZ_32M

ARMCOREBase             EQU     0x90000000
ARMCORESize             EQU     SZ_1M

AMBABase                EQU     0xB0000000
AMBASize                EQU     SZ_1M

DRAMBase                EQU     0xC0000000
DRAMSize                EQU     SZ_32M		;leave this at 8 meg on A0 sili

DRAMPrefetchBase        EQU     0xD0000000
DRAMPrefetchSize        EQU     DRAMSize

DRAMFlushBase           EQU     0xE0000000
DRAMFlushSize           EQU     DRAMSize

DRAMControlBase         EQU     0xFF000000
DRAMControlSize         EQU     SZ_1M


; ----------------------------
; MMU Virtual IXP1200 Addresses
; ----------------------------


SYSIO_BASE              EQU     SYSIOBase         
SYSIO_SIZE              EQU     SYSIOSize         

ROM_BASE                EQU     ROMBase         
ROM_SIZE                EQU     ROMSize         

SRAM_BASE               EQU     SRAMBase        
SRAM_SIZE               EQU     SRAMSize        

RDLCK_BASE              EQU     RDLCKBase       
RDLCK_SIZE              EQU     RDLCKSize       

WRUNLCK_BASE            EQU     WRUNLCKBase     
WRUNLCK_SIZE            EQU     WRUNLCKSize     

CAMUNLCK_BASE           EQU     CAMUNLCKBase    
CAMUNLCK_SIZE           EQU     CAMUNLCKSize    

BITWRCLR_BASE           EQU     BITWRCLRBase    
BITWRCLR_SIZE           EQU     BITWRCLRSize    

BITWRSET_BASE           EQU     BITWRSETBase    
BITWRSET_SIZE           EQU     BITWRSETSize    

BITTSTCLR_BASE          EQU     BITTSTCLRBase   
BITTSTCLR_SIZE          EQU     BITTSTCLRSize   

BITTSTSET_BASE          EQU     BITTSTSETBase   
BITTSTSET_SIZE          EQU     BITTSTSETSize   

PUSHCMD_BASE            EQU     PUSHCMDBase     
PUSHCMD_SIZE            EQU     PUSHCMDSize     

POPCMD_BASE             EQU     POPCMDBase      
POPCMD_SIZE             EQU     POPCMDSize      

SRAMIO_BASE             EQU     SRAMIOBase      
SRAMIO_SIZE             EQU     SRAMIOSize      

PCILocal_BASE           EQU     PCILocalBase    
PCILocal_SIZE           EQU     PCILocalSize    

PCIConfig_BASE          EQU     PCIConfigBase   
PCIConfig_SIZE          EQU     PCIConfigSize   

PCIIO_BASE              EQU     PCIIOBase       
PCIIO_SIZE              EQU     PCIIOSize       

PCIMem_BASE             EQU     PCIMemBase      
PCIMem_SIZE             EQU     PCIMemSize      

ARMCORE_BASE            EQU     ARMCOREBase     
ARMCORE_SIZE            EQU     ARMCORESize     

AMBA_BASE               EQU     AMBABase        
AMBA_SIZE               EQU     AMBASize        

DRAM_BASE               EQU     DRAMBase        
DRAM_SIZE               EQU     DRAMSize        

DRAMPrefetch_BASE       EQU     DRAMPrefetchBase
DRAMPrefetch_SIZE       EQU     DRAMPrefetchSize

DRAMFlush_BASE          EQU     DRAMFlushBase   
DRAMFlush_SIZE          EQU     DRAMFlushSize   

DRAMControl_BASE        EQU     DRAMControlBase 
DRAMControl_SIZE        EQU     DRAMControlSize 


L1_TABLE_ENTRIES        EQU     0x1000  ; 16kB table (word entries) 

; Allow different sized L2 Page tables, each L2 entry needs 256 words (1KB)
L2_TABLE_ENTRIES        EQU     0x100   ;  1kB table (word entries) 

TTB_SIZE                EQU     SZ_16K

; Translation Table at the end of DRAM, occupy 32 Kbytes
TTB_BASE                EQU     (DRAMBase + DRAMSize - TTB_SIZE)

; Access Permissions
; Depending on the setup, 0 represents one of these two access permissions
PT_NO_ACCESS            EQU     (0 << 10)
PT_SVC_R                EQU     (0 << 10)
PT_SVC_RW               EQU     (1 << 10)
PT_NO_USR_W             EQU     (2 << 10)
PT_ALL_ACCESS           EQU     (3 << 10)

PT_C_BIT                EQU     (1 << 3)
PT_B_BIT                EQU     (1 << 2)
PT_CB_BITS              EQU     (PT_C_BIT + PT_B_BIT)


; Entry type
PT_SECTION              EQU     2
PT_PAGE                 EQU     1
PT_INVALID              EQU     0

PT_SMALL                EQU     2
PT_LARGE                EQU     1


; Cached, buffered access, section translation
; AP=11, domain0, C=1, B=1
CACHED_BUFFERED_ACCESS  EQU     (PT_ALL_ACCESS + PT_CB_BITS + PT_SECTION)

; Cached, non-buffered access, section translation
; AP=11, domain0, C=1, B=0
CACHED_NOBUFFER_ACCESS  EQU     (PT_ALL_ACCESS + PT_C_BIT + PT_SECTION)

; Non-cached, buffered access, section translation
; AP=11, domain0, C=0, B=1
NOCACHE_BUFFERED_ACCESS EQU     (PT_ALL_ACCESS + PT_B_BIT + PT_SECTION)

; Non-cached, non-buffered access, section translation
; AP=11, domain0, C=0, B=0
NOCACHE_NOBUFFER_ACCESS EQU     (PT_ALL_ACCESS + PT_SECTION)

JumpTblSize	EQU	0x40 	;Vector_Init_Block_End-Vector_Init_Block
JumpTblBase	EQU	(SRAMRWBase+SRAMRWSize)-JumpTblSize

	END
