;*************************************************************************
; *
; * PCITIMER.S : Source file for PCI timer testing routines.
; * 
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: pcitimers.s $
; *
;*************************************************************************

;Header file includes...
	INCLUDE	H_SASYS.S
	INCLUDE	H_PCI.S
	INCLUDE	H_ARM.S
	INCLUDE	MMUMACRO.S
	INCLUDE	H_CONFIG.S

;Macro file includes...
	INCLUDE	DIAG.S

;External procedures...
	IMPORT	InitDwordCSRs

;Diag message strings...

;Data tables...
	IMPORT	TimerCSRTbl

;Externally used procedures...


TestCntMin	EQU	0x0
TestCntMax	EQU	0x5

	AREA	PCITIMER, CODE, READONLY

	IF	:DEF:	PCITIMER_TEST

	EXPORT	PCITimerTest

;*************************************************************************
;
; PCITimerTest : Routine to test PCI system timer functionality.  Uses
;					ARM core IRQs and FIQs to inc and dec registers
;					when timer interrupts are received. 
;
;	Passed:		Nothing.
;
;	Uses:		R0, R3-R7, R11-R13
;
;	Assumes:	Nothing.
;
;	Returns:	Nothing.
;
;*************************************************************************
PCITimerTest
	mov	r13, lr				;common bug is NOT to save lr before calls!
	adrl	r0,	TimerCSRTbl	;init the timer csrs for this test
	bl	InitDwordCSRs

;Setup for the IRQ test...
	readPsr r7				;retrieve arm cpu status reg
	bic	r7, r7, #IrqMask	;unmask the irqs
	writePsr	r7			;write new cpu status reg
	mov	r11, #TestCntMax
	mov	r12, #TestCntMin
	ldr	r5, =IRQ_STAT
	b	CommonSetup

;Setup for the FIQ test...
FIQTestCfg
	readPsr r7				;retrieve arm cpu status reg
	orr	r7, r7, #IrqMask	;mask the irqs
	bic	r7, r7, #FiqMask	;unmask the fiqs
	writePsr	r7			;write new cpu status reg
	mov	r11, #TestCntMin	;reverse r11/r12 this time for count down
	mov	r12, #TestCntMax
	ldr	r5, =FIQ_STAT

CommonSetup
	mov	r6, #T1				;r6 is timer to test (also used by isr)
	ldr	r4, =TIMER_1_CTRL	;timer control register

DoNextTimer
	mov	r7, r12				;init r7 to count up on irqs down on fiqs
	str	r6, [r5, #8]		;enable the selected timer's irq
	ldr	r3, [r4]			;read the timer ctrl register
	orr	r3, r3, #TMR_EN		;set enable timer bit
	str	r3, [r4]			;write back to start the timer test

IntNotDone
	cmp	r7, r11				;wait for r7 to be equal to testcnt
	OutData	r7, r3			;display what we have to user
	bne	IntNotDone

	str	r6, [r5, #0xC]		;disable the selected timer's irq
	ldr	r3, [r4]			;read the timer ctrl register
	bic	r3, r3, #TMR_EN		;clr enable timer bit
	str	r3, [r4]			;write back to disable this timer
	mov	r6, r6, LSL #1		;adjust for next timer 
	add	r4, r4, #0x20		;ditto
	ldr	r7, =TIMER_4_CTRL
	cmp	r4, r7				;are we finished?
	ble	DoNextTimer			;if not then keep it up

	cmp	r11, #0				;if r11=0 then we have completed irq and fiq test
	beq irqfiqDone			;so exit
							;else
	b	FIQTestCfg			;

irqfiqDone
	readPsr r7				;retrieve arm cpu status reg
	orr	r7, r7, #FiqMask	;mask the fiqs
	writePsr	r7			;write new cpu status reg

	mov	pc, r13				;lr was saved to r13 due to subsequent 'bl's

	ENDIF	;PCITIMER_TEST

	END