;*************************************************************************
; *
; * UARTPRIM.S : Source file for the IXP1200 UART primitives.
; * 		   
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: uartprim.s $
; *
;*************************************************************************

	INCLUDE	H_SASYS.S
	INCLUDE	DIAG.S

	EXPORT	CfgUartDiag
	EXPORT	UARTSendString
	EXPORT	PutHexc1
	EXPORT	PutWordc1
	EXPORT	GetBytec1
	EXPORT	PutBytec1

	AREA	UART, CODE, READONLY


;*************************************************************************
;
; CfgUartDiag : Configure the uart for diag output and flush rcv buffer
;
;	Passed:		Nothing.		
;
;	Uses:		R1-R3
;
;	Assumes:	Nothing.
;
;	Returns:	Nothing.
;
;*************************************************************************
CfgUartDiag
	ldr	r1, =UART_UCR
	ldr	r2, =BAUD_9600:OR:UIC:OR:UE:OR:DSS_8:OR:SBS_1
;	ldr	r2, =BAUD_115200:OR:UIC:OR:UE:OR:DSS_8:OR:SBS_1
	str	r2, [r1]

	ldr	r3, =UART_UDR
	ldr	r1, =UART_USR
FlushRCV
	ldr	r2, [r1]
	ands	r2, r2, #RCV
	ldrne	r2, [r3]
	bne		FlushRCV
	mov	pc, lr		; return to caller


;*************************************************************************
;
; UARTSendString : 
;
;	Passed:		R0 = pointer to null terminated string to send to serial 
;				port.		
;
;	Uses:		R0, R2-R5		
;
;	Assumes:	Nothing.	
;
;	Returns:	Nothing.
;
;*************************************************************************
UARTSendString
	ldr	r2, =UART_USR		; status reg addr to r2
	ldr	r3, =UART_UDR		; data reg addr to r3

Wait4TXV
	ldr	r4, [r2]			; check status in r4
	ands	r4, r4, #TXV	; leave only txv
	beq	Wait4TXV			; txv = 0 = transmit not rdy
	ldrb	r5, [r0], #1	; next char in string to r5
	cmp	r5, #0				; if char is null we are done
	beq	USSExit
	str	r5, [r3]			; else send char
	b	Wait4TXV			; and process next one
	
USSExit
	ldr	r4, [r2]			; check status in r4
	ands	r4, r4, #TFE	; leave only tfe
	beq	USSExit				; tfe = 1 = transmit empty

	mov	pc, lr				; return to caller


;*************************************************************************
;
; PutBytec1 : This routine will put a byte out on the serial port
;
;	Passed:		R0 = data to send to serial port.		
;
;	Uses:		R1, R2		
;
;	Assumes:	Nothing.	
;
;	Returns:	Nothing.
;
;*************************************************************************
PutBytec1   
            ldr     r1,=UART_USR		;address of uart status register
            
01
            ldr     r2,[r1]				;get uart status
            tst     r2,#TBY				;transmit fifo ready?
            bne     %b01				;if not then keep polling

            strb    r0,[r1,#0x800]		;else write the character
            mov     pc,lr				;and return to caller

;*************************************************************************
;
; PutHexc1 : This routine will prepend ' 0x' to a 4 byte word and send it 
;				out on serial port.
;
;	Passed:		R0 = data to send to serial port.		
;
;	Uses:		R0, R4, R5
;
;	Assumes:	Nothing.	
;
;	Returns:	Nothing.	
;
;*************************************************************************
PutHexc1    
            mov     r4,lr
            mov     r5,r0
            mov     r0,#&20                 ; ' '
            bl      PutBytec1
            mov     r0,#&30                 ; '0'
            bl      PutBytec1
            mov     r0,#&78                 ; 'x'
            bl      PutBytec1
            b       PutWord2

;*************************************************************************
;
; PutWordc1 : This routine will put a word value out on serial port
;
;	Passed:		R0 = data to send to serial port.
;
;	Uses:		R0, R3-R5		
;
;	Assumes:	Nothing.	
;
;	Returns:	Nothing.	
;
;*************************************************************************
PutWordc1   
            mov     r4,lr
            mov     r5,r0
PutWord2    mov     r3,#28
01          mov     r0,r5, LSR r3
            ands    r0,r0, #&f
            bne     %f02                    ; the first non-zero
            subs    r3,r3, #4
            bne     %b01                    ; all but the last digit
            mov     r0,r5                   ; Get the only digit
02          cmp     r0,#10
            addlo   r0,r0,#&30              ; 0 - 9
            addhs   r0,r0,#&61 - 10         ; a - f
            bl      PutBytec1
            subs    r3,r3,#4
            mov     r0,r5,LSR r3
            and     r0,r0,#&f
            bpl     %b02
            mov     pc,r4


;*************************************************************************
;
; GetBytec1 : This routine will get a byte from the serial port
;
;	Passed:		Nothing.		
;
;	Uses:		R0-R2		
;
;	Assumes:	Nothing.	
;
;	Returns:	R0 = Byte read from serial port 	
;
;*************************************************************************
GetBytec1   
            ldr     r1,=UART_USR		;address of uart status register
            		
01
            ldr     r2,[r1]				;get uart status
            tst     r2,#RCV				;receive fifo ready?
            beq     %b01				;if not then keep polling
										
            ldrb    r0,[r1,#0x800]		;else read the character
            mov     pc,lr				;and return to caller

	END
