;*************************************************************************
; *
; * RCVSCHED.UC : Micro code source file for receive scheduler thread used 
; * 				in IX bus test.
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; *
;*************************************************************************
.xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7

;
;  Simple receive scheduler thread for proto debug - only in context 0
;  Use simple mapping:
;	fbox0/ctx0 - handles port 0, rfifo entry0
;	fbox0/ctx1 - handles port 1, rfifo entry1
;	fbox0/ctx2 - handles port 2, rfifo entry2
;	fbox0/ctx3 - handles port 3, rfifo entry3
;	fbox1/ctx0 - handles port 4, rfifo entry4
;	fbox1/ctx1 - handles port 5, rfifo entry5
;         ....
;	fbox3/ctx3 - handles port 15, rfifo entry15
;
;  Thread done communication is via self_destruct.
;
	br=ctx [0, context_0#]
	ctx_arb [inter_thread]

context_0#:
	; init some CSRs (this may be moved to the core)
;EVAL has only one MAC in ocltal mac mode...
	immed_w0 [temp, 0x9cf7]				; tx rdy fetch, autopush
    immed_w1 [temp, 0x1ffb]				; rx rdy fetch, nop
	alu [$xfer3, --, b, temp]			; combine w/2 nops
	csr [write, $xfer3, rdybus_template_prog1]
	csr [write, $xfer3, rdybus_template_prog2]	
	csr [write, $xfer3, rdybus_template_prog3]

	; set the following bits in rdybus_template_ctl:
	;	<11>	- enable readybus
	;	<10> 	- 2 mac mode
	;	<9>	- bidirectional
	;	<8>	- master
	immed [temp, 0xf00]
	alu [$xfer4, --, b, temp]
	csr [write, $xfer4, rdybus_template_ctl], ctx_swap

	; init some values
	immed [thread_ready_mask, 0xFFFF]
	immed [fbi_sig_outstanding, 0]

big_loop#:
	csr [read, $xfer0, rcv_rdy_lo]						; get the receive ready bits
	csr [read, $xfer1, self_destruct], ctx_swap			; get the done message from the threads

	alu [thread_ready_mask, thread_ready_mask, or, $xfer1]	; or in the new thread done messages

	; Look at the ports 1 by 1.  If the thread is ready and the port is ready, issue a rec_req to that port
	; and clear that bit in the thread_ready_mask.
	alu [both_ready, $xfer0, and, thread_ready_mask]	; look for ports with thread ready and port ready
      br=0 [big_loop#]
		
	immed [bit_pos, 1]
	immed [count, 0]

small_loop#:
	alu [temp, bit_pos, and, both_ready]
	br=0 [next_bit_pos#]

	; clear the local thread done bit
	alu [thread_ready_mask, thread_ready_mask, and~, bit_pos]

	; create the rec_req:
	;	<5:0> port number
	;	<10:6> thread number
	;	<21:18> rfifo element
      alu [temp, --, b, count]
	alu_shf [temp, temp, or, count, <<6]			; fill in port number and thread number

	; wait for rec_req_avail and write the csr
	alu [--, --, b, fbi_sig_outstanding]			; is there an outstanding write to rec_req?
	br=0 [poll_fifo_ready#]
	immed [fbi_sig_outstanding, 0]
	ctx_arb [fbi]

poll_fifo_ready#:
	alu_shf [$xfer0, temp, or, count, <<18]			; fill in the rfifo element in rec_req
	br_inp_state[REC_REQ_AVAIL, write_rr#]			; spin waiting for room in the rec_req fifo
	br[poll_fifo_ready#]
	; write the rec_req
write_rr#:
	csr [write, $xfer0, rcv_req], sig_done			; note: sig_done NOT ctx_swap
	immed [fbi_sig_outstanding, 1]					; remember the outstanding sig_done

	; need to report count of rec_req issued (could do it per port)

next_bit_pos#:
	; move to the next bit position
      alu [both_ready, both_ready, and~, bit_pos]
	br=0[clear_sig_and_loop#]						; any more to do?
	alu [count, count, +, 1]						; incr count
	alu [--, count, xor, 16]						; check for completion (16 ports)
	br=0 [clear_sig_and_loop#]
	alu_shf [bit_pos, --, b, bit_pos, <<1]			; shift to the next bit position
	br [small_loop#]

clear_sig_and_loop#:								; clear out any pending sig_done and loop again
	alu [--, --, b, fbi_sig_outstanding]
	br=0 [big_loop#]
	immed [fbi_sig_outstanding, 0]
	ctx_arb [fbi]					
	br [big_loop#]
	nop
	nop
	nop
	nop

	


















