;*************************************************************************
; *
; * RCVSCHEDFP.UC : Micro code source file for receive scheduler thread used 
; * 				in IX bus test fast port version.
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: db
; * revisions:
; *
;*************************************************************************
.xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7

;
;  Simple receive scheduler thread for proto debug - only in context 0.
;  This version only does 2 fast ports - port #8 and 9
;  Use simple mapping:
;	fbox0/ctx0 -  rfifo entry0
;	fbox0/ctx1 -  rfifo entry1
;	fbox0/ctx2 -  rfifo entry2
;	fbox0/ctx3 -  rfifo entry3
;	fbox1/ctx0 -  rfifo entry4
;	fbox1/ctx1 -  rfifo entry5
;         ....
;	fbox3/ctx3 -  rfifo entry15
;
;  Thread done communication is via self_destruct.
;
;  scratch communication with core:
;    0x50 - receive pass base - 16 entries
;    0x60 - receive fail base - 16 entries
;    0x70 - receive cancel base - 16 entries
 
	br=ctx [0, context_0#]
	ctx_arb [inter_thread]

context_0#:
; init some CSRs (this may be moved to the core)

    immed [temp, 0x0000]				; flwctl, flwctl
	alu [$xfer3, --, b, temp]			; 
	csr [write, $xfer3, rdybus_template_prog1]
	csr [write, $xfer3, rdybus_template_prog3]
	immed [temp, 0x00d7]				; tx rdy fetch
	immed [temp, 0xd7db]				; tx rdy fetch, rxrdy fetch
	alu [$xfer3, --, b, temp]			; 
	csr [write, $xfer3, rdybus_template_prog2]	

; set the following bits in rcv_rdy_ctl:
    ;     <14> REC_RDY_CTL_1THREAD
    alu_shf [$xfer5, --, b, 1, <<14]
    csr [write, $xfer5, rcv_rdy_ctl]

	csr [read, $xfer0, self_destruct], ctx_swap			; make sure it is clear

	; set the following bits in rdybus_template_ctl:
	;	<11>	- enable readybus
	;	<10> 	- 2 mac mode
	;	<9>	- =0 unidirectional
	;	<8>	- master
	immed [temp, 0xf00]			;64 bit bi-dir bus
	alu [$xfer4, --, b, temp]
	csr [write, $xfer4, rdybus_template_ctl], ctx_swap

; init some values
	immed [thread_ready_mask, 0xFFFF]
	immed [fbi_sig_outstanding, 0]
	immed [rrcntmask, 0xFF]
	immed [scratchbase, 0x30]

big_loop#:
	csr [read, $xfer0, RCV_RDY_CNT], ctx_swap				; get the receive ready bits
	csr [read, $xfer1, self_destruct], ctx_swap				; get the done message from the threads
	alu [thread_ready_mask, thread_ready_mask, or, $xfer1]	; or in the new thread done messages

	alu_shf [rrcnt0, rrcntmask, and, $xfer0, >>8]			; 
	alu_shf [temp4, 0x3, and, $xfer0, >>24]					; check for both fp ready bits
	alu [--, temp4, -, 0x3]

    br!=0 [big_loop#]




		
	immed [bit_pos, 1]
	immed [count, 0]
    immed [count2, 0]

small_loop#:
	alu [temp, bit_pos, and, thread_ready_mask]
	br=0 [next_bit_pos#]

; clear the local thread done bit
	;alu [thread_ready_mask, thread_ready_mask, and~, bit_pos]

	; create the rec_req:
	;	<5:0> port number
	;	<10:6> thread number
	;   <17:16> seq number
	;	<21:18> rfifo element
    alu [temp2, 0x1, and, count2]
	alu [temp, 0x8, +, temp2]						; alternate between port 8 and port9
    alu [count2, count2, +, 1]
	alu [temp2, temp2, +, 1]
	alu [temp4, temp4, and~, temp2]
	alu [temp, temp, or, temp2, <<16]				; fill in fast port seq number (1 or 2)
	alu_shf [temp, temp, or, count, <<6]			; fill in thread number

; wait for rec_req_avail and write the csr
	alu [--, --, b, fbi_sig_outstanding]			; is there an outstanding write to rec_req?
	br=0 [poll_fifo_ready#]

	immed [fbi_sig_outstanding, 0]
	ctx_arb [fbi]

poll_fifo_ready#:
	alu_shf [$xfer0, temp, or, count, <<18]			; fill in the rfifo element in rec_req
	br_inp_state[REC_REQ_AVAIL, write_rr#]			; spin waiting for room in the rec_req fifo

	br[poll_fifo_ready#]

; write the rec_req
write_rr#:
	csr [write, $xfer0, rcv_req], sig_done			; note: sig_done NOT ctx_swap
	immed [fbi_sig_outstanding, 1]					; remember the outstanding sig_done

next_bit_pos#:
; move to the next bit position	 b
    alu [thread_ready_mask, thread_ready_mask, and~, bit_pos]
	br=0[clear_sig_and_loop#]						; any more to do?

	alu [--, --, b, temp4]
	br!=0 [norefreshfprdy#]

	alu [rrcnt0, rrcnt0, +, 2]
	alu [rrcnt0, rrcnt0, and, rrcntmask]




	alu [$xfer0, --, b, rrcnt0]
	scratch [write, $xfer0, scratchbase, 0, 1], ctx_swap
						 



refreshfprdy#:
	csr [read, $xfer0, RCV_RDY_CNT], ctx_swap		; get the receive ready bits
	alu_shf [rrcnt1, rrcntmask, and, $xfer0, >>8]	; 
	alu [--, rrcnt0, -, rrcnt1]
	br!=0 [refreshfprdy#]

	alu_shf [temp4, 0x3, and, $xfer0, >>24]			; check for both fp ready bits
	alu [--, temp4, -, 0x3]
	br!=0 [refreshfprdy#]
	
norefreshfprdy#:
	alu [count, count, +, 1]						; incr count
	alu [--, count, xor, 16]						; check for completion (16 ports)
	br=0 [clear_sig_and_loop#]

	alu_shf [bit_pos, --, b, bit_pos, <<1]			; shift to the next bit position
	br [small_loop#]

clear_sig_and_loop#:								; clear out any pending sig_done and loop again
	alu [--, --, b, fbi_sig_outstanding]
	br=0 [big_loop#]

	immed [fbi_sig_outstanding, 0]
	ctx_arb [fbi]					
	br [big_loop#]

	nop
	nop
	nop
	nop

	


