;*************************************************************************
; *
; * RECEIVE.UC : Micro code source file for receive threads used 
; * 				in IX bus test.
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; *
;*************************************************************************

#include	"ixtest.h"

.xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7
.xfer_order $$xfer0 $$xfer1

;
;  Simple receive processing thread for proto debug - all 4 contexts do the same function
;  1st longword of rfifo data is the offset into verify data array
;  2nd longword of rfifo data is verify length
;
	immed [FlagWord, IXbusTestFlags]
    immed [sdram_base_addr, SDRAM_PktBase]          ; set pkt ptr
    immed [handshake, FPHandShake]       			;

    immed [scratch_pass_status_base, OPpassStat]         	; pass ptr
    immed [scratch_fail_status_base, OPfailStat]         	; fail ptr
    immed [scratch_cancel_status_base, OPcnclStat]       	; cancel ptr

	scratch [read, $xfer0, FlagWord, 0, 1], ctx_swap
    alu [flags, --, b, $xfer0]
	alu [flags, flags, and, GigMacModeFlg]
	alu [--, flags, -, GigMacModeFlg]
	br!=0 [rec_loop#]

    immed [scratch_pass_status_base, FPpassStat]         	; pass ptr
    immed [scratch_fail_status_base, FPfailStat]         	; fail ptr
    immed [scratch_cancel_status_base, FPcnclStat]       	; cancel ptr

rec_loop#:
    ctx_arb [start_receive]                         ; wait for the signal that a receive is complete

    csr [read, $xfer0, rcv_cntl], ctx_swap          ; get the receive info

    alu_shf [rfifo_elem, 0xf, and, $xfer0, >>10]    ; extract the rfifo element number
    br_bset[$xfer0, 31, cancel#]                    ; check for cancel
    alu_shf [rfifo_addr, --, b, rfifo_elem, <<3]    ; create the rfifo address

    r_fifo_rd [$xfer1, rfifo_addr, 0, 1], ctx_swap, defer[1]
    alu [save_rec_control, --, b, $xfer0]

    alu [offset, --, b, $xfer1]						
    alu [length, --, b, $xfer2]						; packet length in quadwords contained in 2nd dword of pkt
    alu [count, --, b, 0]
    alu [offset_sh, --, b, offset, <<3]				; offset contained in pkt is Nth pkt in library
    alu [data_base, sdram_base_addr, +, offset_sh]	; create quadword offset of this pkt
    alu [data_base, data_base, +, 1]                ; assuming that the offset quadword is stored in
                                            		; sdram as well
    alu [rfifo_addr, rfifo_addr, +, 1]


    ; future: can add check of rcv_cntl: length, eop, sop.  
      ; can add variable length

compare_loop#:
    ; compare the data 1 quadword at a time
    ; (not the fastest, but most flexible for doing different lengths in the future)
    sdram [read, $$xfer0, data_base, count, 1], sig_done
    r_fifo_rd [$xfer0, rfifo_addr, count, 1], ctx_swap
    ctx_arb [sdram]

    alu [temp0, --, b, $$xfer0]
    alu [temp1, --, b, $$xfer1]

compare_data#:
    ; check the 2 longwords of data                             
    alu [--, $xfer0, xor, temp0]
    br!=0 [bad_data#]

    alu [--, $xfer1, xor, temp1]
    br!=0 [bad_data#]
    
    alu [count, count, +, 1]
    alu [--, 7, -, count]
    br!=0 [compare_loop#]


pass#:
    ; need to communicate success back to core
    scratch [incr, --, scratch_pass_status_base, rfifo_elem, 1], ctx_swap

    ; communicate complete to the scheduler thread
    alu_shf [temp, --, b, rfifo_elem, <<5]       	; more the fast_wr data to bit<5>
    alu_shf [temp, temp, or, 1, <<15]               ; or in the override bit
    fast_wr [0, self_destruct], indirect_ref

    br [rec_loop#]                             		; do it all over again...                                       

cancel#:
    ; need to communicate cancel back to core
    scratch [incr, --, scratch_cancel_status_base, rfifo_elem, 1]

    ; communicate complete to the scheduler thread
    alu_shf [temp, --, b, rfifo_elem, <<5]          ; more the fast_wr data to bit<5>
    alu_shf [temp, temp, or, 1, <<15]               ; or in the override bit
    fast_wr [0, self_destruct], indirect_ref


    br [rec_loop#]                             		; do it all over again...                                                                          


    ; data miscompare - report the error via scratch
bad_data#:
    nop
	;immed [const_80, 0x81]
	;alu_shf [status_offset, --, b, rfifo_elem, <<1]
	;r_fifo_rd [$xfer4, status_offset, const_80, 2], ctx_swap
	;alu_shf [temp, --, b, rfifo_elem, <<3]
    ; need to add a communication scheme to core
    scratch [incr, --, scratch_fail_status_base, rfifo_elem, 1]
    
    ; communicate complete to the scheduler thread
    alu_shf [temp, --, b, rfifo_elem, <<5]       	; more the fast_wr data to bit<5>
    alu_shf [temp, temp, or, 1, <<15]               ; or in the override bit
    fast_wr [0, self_destruct], indirect_ref

    br [rec_loop#]                             		; do it all over again...       
    
    ; suicide on error
    ;ctx_arb [voluntary]
    nop
    nop
    nop
    nop
    















