;*************************************************************************
; *
; * START1.S : Source file (entry point) for the IXP1200 validation
; * 
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; * $History: start1.s $
; 
; *****************  Version 31  *****************
; User: Tepulkow     Date: 12/05/00   Time: 3:53p
; Updated in $/dev_1200/BoardSupport/Diagnostics
; Updated to current correct code.  Earlier today I mistakingly checked
; in an obsolete module.
; 
; *****************  Version 29  *****************
; User: Tepulkow     Date: 11/27/00   Time: 1:35p
; Updated in $/dev_1200/BoardSupport/Diagnostics
; Added support to display rev "C" silicon.
; 
; *****************  Version 27  *****************
; User: Tepulkow     Date: 8/29/00    Time: 11:32a
; Updated in $/dev_1200/BoardSupport/Diagnostics
; Fixed conditional assembly around the test of SRAM location at which
; the reloced vector table will reside.
; 
; *****************  Version 26  *****************
; User: Tepulkow     Date: 8/28/00    Time: 5:28p
; Updated in $/dev_1200/BoardSupport/Diagnostics
; Added relocation of the exception vector table to SRAM.
; *
;*************************************************************************

;Header file includes...
	INCLUDE	H_SDRAM.S
	INCLUDE	H_FBI.S
	INCLUDE	H_SASYS.S
	INCLUDE	H_MENG.S
	INCLUDE	H_PCI.S
	INCLUDE	H_SRAM.S
	INCLUDE	H_ARM.S
	INCLUDE	MMUMACRO.S
 	INCLUDE	H_CONFIG.S
	INCLUDE	H_GDIAGS.S

;Macro file includes...
	INCLUDE	DIAG.S

;External procedures...
	IMPORT	UARTSendString
	IMPORT	CfgUartDiag
	IMPORT	FIQHandler
	IMPORT	IRQHandler

;Procedures used externally found in this module...
	EXPORT	InitDwordCSRs
;	EXPORT	InitByteCSRs
	
;Diag message strings...
	IMPORT	SignonBanner1
	IMPORT	SignonBanner2
	IMPORT	EndUartCfg
	IMPORT	EarlyInitMsg
	IMPORT	EndEarlyInitMsg
	IMPORT	BootDiagComplete
;	IMPORT	PollBurnIn		
	IMPORT	AckBurnIn		
	IMPORT	UndefInstMsg
	IMPORT	SWIntMsg
	IMPORT	PrefAbrtMsg
	IMPORT	DataAbrtMsg
	IMPORT	CRLF
	
;Data tables...
	IMPORT	EarlyInitTblA
	IMPORT	EarlyInitTblB
	IMPORT	MemTestTbl
	IMPORT	SRAMEVTestTbl
	IMPORT	RevASpeedTbl
	IMPORT	RevBSpeedTbl

DIAGS_Entry	EQU	0x0000C000
RTOS1_Entry	EQU	0x00020000	; This leaves 128K for diagnostics
RTOS2_Entry	EQU	0x00040000	; 

	AREA	Init, CODE, READONLY

	ENTRY

Vector_Init_Block1
ProcResetVect1	b	ProcResetSvc	;0x00
UndefInstVect1	b	UndefInstSvc	;0x04
SoftInterVect1	b	SftWrIntrSvc	;0x08
PrefAbortVect1	b	PrefAbortSvc	;0x0C
DataAbortVect1	b	DataAbortSvc	;0x10
				nop					;0x14
IRQEventsVect1	b	IRQHandler		;0x18
FIQEventsVect1	b	FIQHandler		;0x1C
Vector_Init_Block1_End				;0x20
									
	% (DIAGS_Entry-(Vector_Init_Block1_End-Vector_Init_Block1))		;pad with 0's to entry point

	LTORG

Vector_Init_Block
	ldr		pc, Reset_Addr
	ldr		pc, Undefined_Addr
	ldr		pc, SWI_Addr
	ldr		pc, Prefetch_Addr
	ldr		pc, DataAbort_Addr
	nop
	ldr		pc, Irq_Addr
	ldr		pc, Fiq_Addr

Reset_Addr		DCD	ProcResetSvc
Undefined_Addr	DCD	UndefInstSvc
SWI_Addr		DCD	SftWrIntrSvc
Prefetch_Addr	DCD	PrefAbortSvc
DataAbort_Addr	DCD	DataAbortSvc
				DCD	0
Irq_Addr		DCD	IRQHandler
Fiq_Addr		DCD	FIQHandler
Vector_Init_Block_End

UndefInstSvc						
	adrl	r0, UndefInstMsg
	B	GoingDownNow

SftWrIntrSvc
	adrl	r0, SWIntMsg
	B	GoingDownNow

PrefAbortSvc
	adrl	r0, PrefAbrtMsg
	B	GoingDownNow

DataAbortSvc
	adrl	r0, DataAbrtMsg
	
GoingDownNow
	bl		UARTSendString

Broken
	mov	r0, #GD_Error		; "E" is for Error!
	OutDiag	r0, r1
DONE	
	B	DONE


;*************************************************************************
;
; ProcResetSvc : ARM reset vector which is entry point to diagnostics code. 
;
;	Passed:		N/A
;
;	Uses:		N/A
;
;	Assumes:	Nothing.
;
;	Returns:	N/A
;
;*************************************************************************
ProcResetSvc

;Get the PLL up to speed asap...

	adrl	r0, RevASpeedTbl	;assume silicon is rev A
	readChipId	r1				;chip id reg contains rev info
	ands	r1, r1,	#0xF		;info in lowest nibble
	beq		WritePllReg			;0 = Rev A Sili 

	adrl	r0, RevBSpeedTbl	;1 = Rev B Sili

WritePllReg
	bl		InitDwordCSRs


;Let the outside world know that we are here!

	CfgGpioDiag	r0, r1

	IF	:DEF:	GPIO_TEST
	mov	r0, #0x0
countup
	OutDiag	r0, r1
	add	r0, r0, #0x1
	cmp	r0, #0x10
	bne	countup

	ENDIF		;GPIO_TEST


	IF	:DEF:	GPIO_TEST2
GpioLoop
	InGpioData	r5, r1, r2, r3
	OutData		r5, r4		;display data
	b	GpioLoop			;loop for ever
	ENDIF		;GPIO_TEST2


	IF	:DEF:	CPUREG_TEST
	mov	r0, #GD_RegTest
	OutDiag	r0, r1

;now for a simple cpu register test...
	bl	CPURegTest

	cmp	r0, #0					;return code should be zero!
RegTestFailed
	bne	RegTestFailed			;non-zero means something went wrong so bail here.
	ENDIF		;CPUREG_TEST


	mov	r0, #GD_UARTCfg
	OutDiag	r0, r1

;Configure the UART device to send diag strings to a console...

	bl	CfgUartDiag

;We should now have a COM port so let's try it...
	adrl	r0, CRLF
	bl		UARTSendString
	adrl	r0, CRLF
	bl		UARTSendString
	adrl	r0, SignonBanner1
	bl		UARTSendString
	adrl	r0, SignonBanner2
	bl		UARTSendString
	bl		CPUSpdRev
	adrl	r0, CRLF
	bl		UARTSendString
	adrl	r0, EndUartCfg
	bl		UARTSendString


	mov	r0, #GD_SDRAMReset		;update the gpio display
	OutDiag	r0, r1

;cycle the SDRAM Ctrlr (and all else) out of reset...
	ldr     r9, =PCI_RESET		;PCI reset reg addr
	ldr		r8, =0x2f			;uEngines left in reset
	str     r8, [r9]			;now out of reset 

;Perform early register init from table (mem ctlr csrs etc.)...
	mov	r0, #GD_CSRInit			;update the gpio display
	OutDiag	r0, r1
	adrl	r0, EarlyInitMsg	
	bl		UARTSendString		;tell the console we are about to init CSRs

	adrl	r0, EarlyInitTblA	;assume silicon is rev A
	readChipId	r1				;chip id reg contains rev info
	ands	r1, r1,	#0xF		;info in lowest nibble
	beq		WriteCSRInit		;0 = Rev A Sili 

	adrl	r0, EarlyInitTblB	;1 = Rev B Sili

WriteCSRInit
	bl		InitDwordCSRs


	enableClockSwitch			;enable clock switching after any PLL adjustments

	adrl	r0, EndEarlyInitMsg	
	bl		UARTSendString


BurnInLoop


;Set up the exception jump table in SRAM...
	IF	:DEF:	MEM_TEST
	IF	:DEF:	TEST_SRAM
	IMPORT	MemVectTest		
	IMPORT	EndMemVectTest
;First test memory...
	adrl	r0, MemVectTest
	bl		UARTSendString		;out to console
	adrl	r0, SRAMEVTestTbl	;
	bl		MemoryTest			;
	adrl	r0, EndMemVectTest
	bl		UARTSendString		;out to console
	ENDIF	;TEST_SRAM
	ENDIF	;MEM_TEST
;Now copy exception table to SRAM...
	adrl	r8, Vector_Init_Block
	ldr		r9, =JumpTblBase
	ldmia	r8!, {r0-r7}
	stmia	r9!, {r0-r7}
	ldmia	r8!, {r0-r7}
	stmia	r9!, {r0-r7}


	IF	:DEF:	CPUCACHE_EN

	IMPORT	MMUInitMsg
	IMPORT	EndMMUInitMsg
	IMPORT	BypassMMUInitMsg
	IMPORT	CacheOn
	IMPORT	CacheOff

;Check here if user selected cache enable
	InGpioData	r0, r1, r2, r3	;get gpio input data

	ands r0, r0, #CacheEnable	;check for cache enable
	beq	ByPassMMUInit			;if not selected then hop over this time

; Call OEM platform specific MMU initialization routine
	mov	r0, #GD_MMUInit
	OutDiag	r0, r1			;update the  display
	adrl	r0, MMUInitMsg
	bl		UARTSendString		;out to console

	bl		CacheOn				;let's speed things up

	adrl	r0, EndMMUInitMsg
	bl		UARTSendString		;out to console
	b		MMUDone

ByPassMMUInit
	mov	r0, #GD_CacheOff
	OutDiag	r0, r1			;update the  display

	bl		CacheOff
	adrl	r0, BypassMMUInitMsg
	bl		UARTSendString		;out to console
MMUDone
	ENDIF	;CPUCACHE_EN


	IF	:DEF:	FINALCSRINIT

	IMPORT	FinalInitTbl
	IMPORT	FinalInitMsg
	IMPORT	EndFinalInitMsg

	mov	r0, #GD_FinalCSR
	OutDiag	r0, r1
	adrl	r0, FinalInitMsg
	bl		UARTSendString

	adrl	r0, FinalInitTbl	; address of init table
	bl		InitDwordCSRs

 	adrl	r0, EndFinalInitMsg
	bl		UARTSendString
	ENDIF	;FINALCSRINIT


;Insert calls to additional tests here (SDRAM,SRAM,MAC REG,uEngine,etc.)


	IF	:DEF:	MEM_TEST

	IMPORT	MemAperTest		
	IMPORT	MemAperFail		
	IMPORT	EndMemAperTest
	IMPORT	MemoryTest

;Test system memory apertures...
	mov	r0, #GD_MemTest
	OutDiag	r0, r1
	adrl	r0, MemAperTest
	bl		UARTSendString

	adrl	r0, MemTestTbl		;offset of the early init table
	bl		MemoryTest			;send it out

	cmp	r0, #0					; return completion code in r0
	beq	MemTestPassed			; 0= all mem areas tested passed

	adrl	r0, MemAperFail		; else let user know we are bailing
	bl		UARTSendString
	b	Broken					; 

MemTestPassed
	adrl	r0, EndMemAperTest	; to here and we are OK
	bl		UARTSendString		; so tell user and continue...
	ENDIF


	IF	:DEF:	ROM_TEST

	IMPORT	RomAddrTest		
	IMPORT	RomAddrTestFail
	IMPORT	EndRomAddrTest	
	IMPORT	PutHexc1
	IMPORT	BootDevTest

	mov	r0, #GD_ROMTest
	OutDiag	r0, r1

	adrl	r0, RomAddrTest
	bl		UARTSendString

	bl		BootDevTest
	cmp		r0, #0x0
	beq		RomTestPassed
	mov		r0, r2				;culprit address
	mov		r13, r3				;save data
	bl		PutHexc1
	mov		r0, r13				;data read
	bl		PutHexc1
	adrl	r0, RomAddrTestFail
	bl		UARTSendString
	b		EndRomTest

RomTestPassed
	adrl	r0, EndRomAddrTest
	bl		UARTSendString

EndRomTest

	ENDIF	;ROM_TEST


	IF	:DEF:	PCITIMER_TEST

	IMPORT	PCITimerIrqFiqTest
	IMPORT	IRQLEDInfo
	IMPORT	FIQLEDInfo
	IMPORT	EndTimerIrqFiqTest
	IMPORT	PCITimerTest

;Test PCI timers and IRQ/FIQ...
;Let user know about the test and give some instructions...
	mov	r0, #GD_PCITimer
	OutDiag	r0, r1
	adrl	r0, PCITimerIrqFiqTest
	bl		UARTSendString
	adrl	r0, IRQLEDInfo
	bl		UARTSendString
	adrl	r0, FIQLEDInfo
	bl		UARTSendString

	bl	PCITimerTest

 	adrl	r0, EndTimerIrqFiqTest	;If we return from this test we pased!
	bl		UARTSendString
	ENDIF	;PCITIMER_TEST


	IF	:DEF:	USTORE_TEST

	IMPORT	UcodeLoadTest
	IMPORT	UcodeLoadTestOK
	IMPORT	UcodeLoadTestFail
	IMPORT	StartUstoreTest

;MicroEngine microstore test...
	mov	r0, #GD_UstoreTest
	OutDiag	r0, r1
	adrl	r0, UcodeLoadTest
	bl		UARTSendString

	bl	StartUstoreTest			; test the 6 microengine's ucode storage
	cmp	r0, #0x0				; completion code returned in r0
	bne	UcodeError				; non-zero means test failed

UcodeOK
	adrl	r0, UcodeLoadTestOK	; else we are OK so let user know
	bl		UARTSendString
	b		UcodeDone

UcodeError
	adrl	r0, UcodeLoadTestFail
	bl		UARTSendString

UcodeDone
	ENDIF	;USTORE_TEST


	IF	:DEF:	MAC_TEST

	IMPORT	MACTest
	IMPORT	MACTestEnd
	IMPORT	MACTestFail
	IMPORT	MACPortTest

;MAC CPU port test...
	mov	r0, #GD_MACPresence
	OutDiag	r0, r1
	adrl	r0, MACTest
	bl		UARTSendString

	bl	MACPortTest
	cmp	r0, #0x0				; completion code returned in r0
	bne	MACPortError			; non-zero means test failed

	adrl	r0, MACTestEnd		; else we are OK so let user know
	bl		UARTSendString
	b		MACPortDone

MACPortError
	adrl	r0, MACTestFail
	bl		UARTSendString

MACPortDone
	ENDIF	;MAC_TEST


	IF	:DEF:	MAC_WRTEST

	IMPORT	MACWRTest
	IMPORT	MACWRTestEnd
	IMPORT	MACWRTestFail
	IMPORT	MacPortUnique
;MAC CPU port test...
	mov	r0, #GD_MACPort
	OutDiag	r0, r1
	adrl	r0, MACWRTest
	bl		UARTSendString

	bl	MacPortUnique
	cmp	r0, #0x0				; completion code returned in r0
	bne	MACPortWRError			; non-zero means test failed

	adrl	r0, MACWRTestEnd		; else we are OK so let user know
	bl		UARTSendString
	b		MACPortWRDone

MACPortWRError
	adrl	r0, MACWRTestFail
	bl		UARTSendString

MACPortWRDone
	ENDIF	;MAC_WRTEST


	IF	:DEF:	UENGINE_TEST

	IMPORT	UengineTest		
	IMPORT	UengineTestEnd
	IMPORT	uEngFuncTest

	mov	r0, #GD_UengFunc
	OutDiag	r0, r1
	adrl	r0, UengineTest
	bl		UARTSendString
	bl	uEngFuncTest				; load uCode to fbox and access sram
	adrl	r0, UengineTestEnd
	bl		UARTSendString
	ENDIF	;UENGINE_TEST


	
	IF	:DEF:	IXBUS_TEST
	IMPORT	ixbus_test
	bl		ixbus_test
	ENDIF	;IXBUS_TEST




;Check here if user selected burn in mode enable
	adrl	r0, BootDiagComplete
	bl		UARTSendString

	adrl	r0, AckBurnIn
	bl		UARTSendString

	adrl	r0, CRLF
	bl		UARTSendString
	adrl	r0, CRLF
	bl		UARTSendString
	b	BurnInLoop				; else, burn in enabled so do it again...



;*************************************************************************
;
; InitDwordCSRs : Routine called to pump a table of csr programming
;					data out to the csr address also contained in the 
;					source table. 
;
;	Passed:	R0 contains the 32 bit address of the table to be programmed
;			into CSRs.
;
;	Uses:	R0, R1, R2 contents destroyed.
;
;	Assumes: Table pointed to by R0 is of the following format (entries are
;			 DWORD (32 bit) length):
;
;		Csr1Addr, 	Csr1Data
;		Csr2Addr, 	Csr2Data
;		Csr3Addr, 	Csr3Data
;			.		 	.
;			.		 	.
;			.		 	.
;		Csr(N-1)Addr, Csr(N-1)Data
;		CsrNAddr, 	  CsrNData
;		0xFFFFFFFF				;DWORD 0xFFFFFFFF is endo of table marker			
;
;	Returns:	Nothing
;
;*************************************************************************
InitDwordCSRs
1
	ldr	r1, [r0], #4		; table entry (reg addr) to r1, then r0=r0+4
	cmp	r1, #0xFFFFFFFF		; check end of table?
	beq	%F02				; if so we are done

	ldr	r2, [r0], #4		; else, load reg with init data, then r0=r0+4
	str	r2, [r1]			; and init chip reg
	b	%B01

2
	mov	pc, lr



	IF	:DEF:	CPUREG_TEST

;*************************************************************************
;
; CPURegTest : Routine to perform a simple integrity test of the CPU
;					general purpose registers
;
;	Passed:		Nothing.
;
;	Uses:		All registers.
;
;	Assumes:	Nothing.
;
;	Returns:	R0=0=TEST PASSED, R0 !=0=TEST FAILED
;
;*************************************************************************
CPURegTest
	ldr	r0, =0xaaaaaaaa
	mov	r1, r0
	mov	r2, r1
	mov	r3, r2
	mov	r4, r3
	mov	r5, r4
	mov	r6, r5
	mov	r7, r6
	mov	r8, r7
	mov	r9, r8
	mov	r10, r9
	mov	r11, r10
	mov	r12, r11
	mov	r13, r12
	cmp	r0, r13
	bne	CPURegTestFail

	ldr	r0, =0x55555555
	mov	r1, r0
	mov	r2, r1
	mov	r3, r2
	mov	r4, r3
	mov	r5, r4
	mov	r6, r5
	mov	r7, r6
	mov	r8, r7
	mov	r9, r8
	mov	r10, r9
	mov	r11, r10
	mov	r12, r11
	mov	r13, r12
	cmp	r0, r13
	bne	CPURegTestFail

	ldr	r0, =0xffffffff
	mov	r1, r0
	mov	r2, r1
	mov	r3, r2
	mov	r4, r3
	mov	r5, r4
	mov	r6, r5
	mov	r7, r6
	mov	r8, r7
	mov	r9, r8
	mov	r10, r9
	mov	r11, r10
	mov	r12, r11
	mov	r13, r12
	cmp	r0, r13
	bne	CPURegTestFail

	ldr	r0, =0x0
	mov	r1, r0
	mov	r2, r1
	mov	r3, r2
	mov	r4, r3
	mov	r5, r4
	mov	r6, r5
	mov	r7, r6
	mov	r8, r7
	mov	r9, r8
	mov	r10, r9
	mov	r11, r10
	mov	r12, r11
	mov	r13, r12
	cmp	r0, r13
	bne	CPURegTestFail

	b	CPURegTestExit	;exit with r0 = 0 = test pased

CPURegTestFail
	mov	r0, #1			;exit with r0 != 0 = test failed
CPURegTestExit
	mov	pc, lr	

	ENDIF	;CPUREG_TEST







	IMPORT	CPUrev	
	IMPORT	Silicon
	IMPORT	RevA	
	IMPORT	RevB	
	IMPORT	RevC	
	IMPORT	CoreSpd
	IMPORT	MHZ	
	IMPORT	Spd29	
	IMPORT	Spd36	
	IMPORT	Spd44	
	IMPORT	Spd51	
	IMPORT	Spd58	
	IMPORT	Spd66	
	IMPORT	Spd73	
	IMPORT	Spd81	
	IMPORT	Spd88	
	IMPORT	Spd95	
	IMPORT	Spd103
	IMPORT	Spd110
	IMPORT	Spd117
	IMPORT	Spd132
	IMPORT	Spd147
	IMPORT	Spd154
	IMPORT	Spd162
	IMPORT	Spd165
	IMPORT	Spd176
	IMPORT	Spd191
	IMPORT	Spd199
	IMPORT	SpdNG
	IMPORT	Unknown

;*************************************************************************
;
; CPUSpdRev : Routine to detect core CPU speed and revision and display 
;				to console
;
;	Passed:		Nothing.
;
;	Uses:		R0-R3, R6, R12		
;
;	Assumes:	Nothing.
;
;	Returns:	Nothing.
;
;*************************************************************************

CPUSpdRev

	mov		r12, lr				;save link register

	adrl	r0, CPUrev			;cpu rev preamble
	bl		UARTSendString

	readChipId	r7				;get chip ID
	and		r7, r7,	#0xF		;info in lowest nibble
	cmp		r7, #3				;is it valid?
	bgt		UnSupportedRev		;if yes continue
	cmp		r7, #2				;is it valid?
	bne		SupportedRev		;if yes continue

UnSupportedRev
	adrl	r0, Unknown			;else say it is unknown
	bl		UARTSendString
	
	mov	pc, r12					;and return to caller

SupportedRev
	adrl	r0, RevA			;assume is rev A sili
	adrl	r6, RevASpdTbl		;ditto
	cmp		r7, #0				;check for rev A
	beq		OutRev				;0 = Rev A Sili 

	adrl	r0, RevB			
	adrl	r6, RevBSpdTbl
	cmp		r7, #1				;check for rev B
	beq		OutRev				;1 = Rev B Sili 

	adrl	r0, RevC			;must be C


OutRev
	bl		UARTSendString		;out rev info

	adrl	r0, Silicon			;postamble to rev data
	bl		UARTSendString

	adrl	r0, CoreSpd			;cpu spd preamble
	bl		UARTSendString
	
	ldr		r2, =PLL_CTL		;pll address
	ldr		r3, [r2]			;get setting

	ldr		r8, =((RevASpdTblend-RevASpdTbl)/4)
	cmp		r3, r8				;range check pll setting
	ble		SpdOK				;continue if OK

	adrl	r0, SpdNG			;else display unknown spd
	b		OutSpd

SpdOK
	mov		r3, r3, LSL #2		;shift to long word address
	add		r3, r3, r6			;add table base
	ldr		r0, [r3]			;load string offset
OutSpd
	bl		UARTSendString		;and write it out

	adrl	r0, MHZ				;cpu spd post amble
	bl		UARTSendString

	mov	pc, r12					;return to caller


RevASpdTbl
	DCD		Spd29			
	DCD		Spd36	
	DCD		Spd44	
	DCD		Spd51	
	DCD		Spd58	
	DCD		Spd66	
	DCD		Spd73	
	DCD		Spd81	
	DCD		Spd88	
	DCD		Spd95	
	DCD		Spd103
	DCD		Spd110
	DCD		Spd117
	DCD		Spd132
	DCD		Spd147
	DCD		Spd154
	DCD		Spd162
	DCD		Spd176
	DCD		Spd191
	DCD		SpdNG		;pad to same length as RevBSpdTbl 
RevASpdTblend


RevBSpdTbl
	DCD		Spd29			
	DCD		Spd36	
	DCD		Spd44	
	DCD		Spd51	
	DCD		Spd58	
	DCD		Spd66	
	DCD		Spd73	
	DCD		Spd81	
	DCD		Spd88	
	DCD		Spd95	
	DCD		Spd103
	DCD		Spd110
	DCD		Spd132
	DCD		Spd147
	DCD		Spd154
	DCD		Spd162
	DCD		Spd165
	DCD		Spd176
	DCD		Spd191
	DCD		Spd199
RevBSpdTblend





	END
