;*************************************************************************
; *
; * TRANSMIT.UC : Micro code source file for transmit thread used in IX bus
; * 				test.
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; *
;*************************************************************************

#include	"ixtest.h"

;Misc defines...
#define ValidPortMaskLo 0x00ff				;Evaluation
#define ValidPortMaskHi 0x0000
#define	TXPacketCount	0x00


; define sram xfer regs...
.xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 

; define sdram xfer regs...
;.xfer_order $$xfer0 $$xfer1 $$xfer2 $$xfer3 $$xfer4 $$xfer5 $$xfer6 $$xfer7 


    br=ctx[ 0, continue_0# ]                ; only using 1 thread in this tbox

    ctx_arb [ inter_thread ] 


continue_0#:
;Move the packet data from SDRAM to tfifo...
    immed [index, SDRAM_PktBase]            ;ptr to sdram pkt data
    immed [temp0, SDRAM_PktLibSz]
    immed [validate_override, 0x0]
    alu[indexend, index, +, temp0]          ;add base and length for end

RestartXmitLoop#:
;Here test for RUN and BURST flags to determine if test is enabled...
    immed [scradr, IXbusTestFlags]					;get test flags
    scratch[read, $xfer0, scradr, 0, 1], ctx_swap
	alu[tempx, --, b, $xfer0]						;to tempx
	alu[tempy, --, b, $xfer0]						;and tempy
	alu[tempz, --, b, $xfer0]						;and tempz
	alu[tempx, tempx, and, RunTestFlag]				;isolate the run flag
	alu[--, tempx, -, RunTestFlag]
	br=0[GoTest#]									;if set then do the test
	alu[tempy, tempy, and, BurstTestFlag]			;isolate the burst flag
	alu[--, tempy, -, BurstTestFlag]				
	br!=0[RestartXmitLoop#]							;if not set then spin on flags
	alu[tempz, tempz, and~, BurstTestFlag] 			;else clear the burst flag
	alu[$xfer0, --, b, tempz]		
    scratch[write, $xfer0, scradr, 0, 1], ctx_swap	;update flags and do one pass

GoTest#:
    immed [tfifo_addr, 0]                   ;ptr to tfifo element
    immed [stop_addr, 128]                  ;tfifo data addr limit

tfifo_loop#: 
    alu_shf[--, validate_override, or, tfifo_addr, <<4]
    sdram[t_fifo_wr, --, index, 0, 8], indirect_ref
    alu[index, index, +, 8]                 ;inc index reg
    alu[--, indexend, -, index]             ;reached end of pkt lib?
    br>0[contfifofill#]                     ;if not then continue
    immed [index, SDRAM_PktBase]            ;else, reset ptr to start of sdram pkt data 
contfifofill#:
    alu [tfifo_addr, tfifo_addr, +, 8]      ;tfifo_addr = address of next tfifo data element
    alu [--, stop_addr, -, tfifo_addr]      ;test for end of tfifo
    br!=0 [tfifo_loop#]             		;if not full then do more


;Write the t-fifo status fields...
;First the prepend field
    immed [$xfer2, 0]                       ;prepend field = 0's
    immed [$xfer3, 0]
    
;Then the control field
    immed_w0 [temp0, 0xFF00]                ;8 qwords/8 bytes in last qword and EOP/SOP
    immed_w1 [temp0, 0x0]				                
    immed [temp1, 0x0]				                
    immed [$xfer1, 0]
    immed [stop_addr, 0160]
    immed [tfifo_addr, 0x80]

tfifo_loop3#:
    alu [$xfer0, --, b, temp0]
    t_fifo_wr[$xfer0, tfifo_addr, 0, 2],ctx_swap
    alu [tfifo_addr, tfifo_addr, +, 2]
    alu [temp0, temp0, +, 1]               ;increment port address for this element
    alu [temp1, temp1, +, 1]
	alu [--, temp1, -, 8]
	br!=0 [contctlfld#]

    immed_w0 [temp0, 0xFF00]                ;reset port num
    immed [temp1, 0x0]				                


contctlfld#:
    alu [--, stop_addr, -, tfifo_addr]
    br!=0 [tfifo_loop3#]


;Determine that all ports are ready to xmit...
test_loop#:
    immed_w0 [port_mask, ValidPortMaskLo]           ;set mask with ports used for this test
    immed_w1 [port_mask, ValidPortMaskHi] 

;    alu_shf[validate_override, --, B, 1, <<15]      ;setup indirect ov bit for write data
;    immed [ptr, 0]                                  ;init validate ptr 

Poll_Ready_Loop1#:
    csr[ read, $xfer0, xmit_rdy_lo],ctx_swap        ;read the rdy bits
    alu[temp0, $xfer0, AND, port_mask]              ;leave only port valid ports
    alu[temp0, temp0, -, port_mask]               
    br!=0 [Poll_Ready_Loop1#]                       ;result = 0 and all ports ready

;Validate all TFIFO entries...
    alu_shf[validate_override, --, B, 1, <<15]      ;setup indirect ov bit for write data
    immed [ptr, 0]                                  ;init validate ptr 
    immed [ptr1, 0]                                 ;init validate ptr 
    immed [ptrmask, 0x3ff]                                 ;init validate ptr 
validate_loop#:
    alu_shf[validate_override, validate_override, and~, ptrmask, <<5]    ;
    alu_shf[--, validate_override, OR, ptr, <<5]    ;setup indirect data from element no.
    fast_wr[0, XMIT_VALIDATE], indirect_ref         ;start xmit
    alu [ptr, ptr, +, 1]                            ;inc ptr
    alu [--, 16, -, ptr]                           ;sub ptr from 16
	br!=0 [validate_loop#]                          ;result = 0 and we are done
                                                    ;else, loop till we are done

;wait for XMIT_PTR to increment by at least two adresses...

;   csr[read, $xfer0, xmit_ptr],ctx_swap
;    alu[temp0, $xfer0, +, 13]
 
WaitForPtr#:
;    csr[read, $xfer0, xmit_ptr],ctx_swap
;    alu[--, temp0, -, $xfer0]
;    br>0[WaitForPtr#]
	csr [read, $xfer0, xmit_rdy_ctl], ctx_swap
	alu_shf [xmit_valid_bits, --, b, $xfer0, >>16]
	br!=0 [WaitForPtr#]


;Determine that all ports are ready to xmit again...
;duped code need to make subroutine
    immed_w0 [port_mask, ValidPortMaskLo]           ;set mask with ports used for this test
    immed_w1 [port_mask, ValidPortMaskHi] 

Poll_Ready_Loop2#:
    csr[ read, $xfer0, xmit_rdy_lo],ctx_swap        ;read the rdy bits
    alu[temp0, $xfer0, AND, port_mask]              ;leave only port valid ports
    alu[temp0, temp0, -, port_mask]             	;
    br!=0 [Poll_Ready_Loop2#]                       ;result = 0 and all ports ready


;Update port status in scratch...
    immed [scr_addr, TXPacketCount]                 ;scratch addr ptr begin @ first port
    scratch[incr, --, scr_addr, 0, 1]


;loop to xmit another burst...
    
    br[RestartXmitLoop#]
;    br [test_loop#]
    
    nop
    nop
    nop



