;*************************************************************************
; *
; * TRANSMITFP.UC : Micro code source file for transmit thread used in IX bus
; * 				test.
; * 
; *
; * ---------------------------------------------------------------------	
; *                                                                      
; *                  I N T E L   P R O P R I E T A R Y                   
; *                                                                      
; *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
; *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
; *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
; *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
; *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
; *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
; *     THE PRIOR WRITTEN PERMISSION OF :                                
; *                                                                      
; *                        INTEL  CORPORATION                            
; *                                                                     
; *                     2200 MISSION COLLEGE BLVD                        
; *                                                                      
; *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
; *                                                                      
; * ---------------------------------------------------------------------
; *
; * system: IXP1200
; * subsystem: Diagnostics
; * author: tep
; * revisions:
; *
;*************************************************************************


#include	"ixtest.h"


;Misc defines...
#define ValidPortMaskLo 0x0300				;Evaluation fastports

#define ValidPortMaskHi 0x0000

#define	PktSubBurst		0x2

#define	TXPacketCount	0x40


; define sram xfer regs...
.xfer_order $xfer0 $xfer1 $xfer2 $xfer3 $xfer4 $xfer5 $xfer6 $xfer7 

; define sdram xfer regs...
;.xfer_order $$xfer0 $$xfer1 $$xfer2 $$xfer3 $$xfer4 $$xfer5 $$xfer6 $$xfer7 


    br=ctx[ 0, continue_0# ]                ; only using 1 thread in this tbox

    ctx_arb [ inter_thread ] 

continue_0#:
    immed [scradr, IXbusTestFlags]					;
	immed [rxpktsum, 0x0]

;Move the packet data from SDRAM to tfifo...
    immed [index, SDRAM_PktBase]            ;ptr to sdram pkt data
    immed [temp0, SDRAM_PktLibSz]
    immed [validate_override, 0x0]
    alu[indexend, index, +, temp0]          ;add base and length for end

RestartXmitLoop#:
    scratch[read, $xfer0, scradr, 0, 1], ctx_swap
	alu[tempx, --, b, $xfer0]						;to tempx
	alu[tempy, --, b, $xfer0]						;and tempy
	alu[tempz, --, b, $xfer0]						;and tempz
	alu[tempx, tempx, and, RunTestFlag]				;isolate the run flag
	alu[--, tempx, -, RunTestFlag]
	br=0[GoTest#]									;if set then do the test

	alu[tempy, tempy, and, BurstTestFlag]			;isolate the burst flag
	alu[--, tempy, -, BurstTestFlag]				
	br!=0[RestartXmitLoop#]							;if not set then spin on flags

	alu[tempz, tempz, and~, BurstTestFlag] 			;else clear the burst flag
	alu[$xfer0, --, b, tempz]		
    scratch[write, $xfer0, scradr, 0, 1], ctx_swap	;update flags and do one pass

GoTest#:
    immed [tfifo_addr, 0]                   ;ptr to tfifo element
    immed [stop_addr, 128]                  ;tfifo data addr limit

tfifo_loop#: 
    alu_shf[--, validate_override, or, tfifo_addr, <<4]
    sdram[t_fifo_wr, --, index, 0, 8], indirect_ref
    alu[index, index, +, 8]                 ;inc index reg
    alu[--, indexend, -, index]             ;reached end of pkt lib?
    br>0[contfifofill#]                     ;if not then continue
    immed [index, SDRAM_PktBase]            ;else, reset ptr to start of sdram pkt data 
contfifofill#:
    alu [tfifo_addr, tfifo_addr, +, 8]      ;tfifo_addr = address of next tfifo data element
    alu [--, stop_addr, -, tfifo_addr]      ;test for end of tfifo
    br!=0 [tfifo_loop#]             		;if not full then do more


;Write the t-fifo status fields...
;First the prepend field
    immed [$xfer2, 0]                       ;prepend field = 0's
    immed [$xfer3, 0]
    
;Then the control field
    immed_w0 [temp0, 0xFF08]                ;8 qwords/8 bytes in last qword and EOP/SOP
    immed_w1 [temp0, 0x0]	
    			                
    immed_w0 [temp1, 0xFF09]                ;temp0 w/max port num
    immed_w1 [temp1, 0x0]	

    immed [$xfer1, 0]
    immed [stop_addr, 0160]
    immed [tfifo_addr, 0x80]

tfifo_loop3#:
    alu [$xfer0, --, b, temp0]
    t_fifo_wr[$xfer0, tfifo_addr, 0, 2],ctx_swap
    alu [tfifo_addr, tfifo_addr, +, 2]
    alu[--, temp0, -, temp1]				;check for port 9
    br!=0 [skip_port_count_reset#]			;if not port 9, just continue
											;else, reset to port 8...
    alu[temp0, temp0, AND~, 0xf]			;clear port field
    alu[temp0, temp0, +, 8]					;set fp base
	br [skip_port_count_inc#]				;no inc of port this time

skip_port_count_reset#:
    alu [temp0, temp0, +, 1]              	;increment port address for this element

skip_port_count_inc#:
    alu [--, stop_addr, -, tfifo_addr]
    br!=0 [tfifo_loop3#]


;Determine that all ports are ready to xmit...
test_loop#:
    immed_w0 [port_mask, ValidPortMaskLo]           ;set mask with ports used for this test
    immed_w1 [port_mask, ValidPortMaskHi] 

    alu_shf[validate_override, --, B, 1, <<15]      ;setup indirect ov bit for write data
    immed [ptr, 0]                                  ;init validate ptr 
    immed [ptrmask, 0x3ff]                          ;init validate ptr 
    immed [countpkts, PktSubBurst]	                ;inner loop var 

Poll_Ready_Loop1#:
    csr[ read, $xfer0, xmit_rdy_lo],ctx_swap        ;read the rdy bits
    alu[temp0, $xfer0, AND, port_mask]              ;leave only port valid ports
    alu[temp0, temp0, -, port_mask]               
    br!=0 [Poll_Ready_Loop1#]                       ;result = 0 and all ports ready

;Validate TFIFO entries...
validate_loop#:
    alu_shf[validate_override, validate_override, AND~, ptrmask, <<5]    ;clear field
    alu_shf[--, validate_override, OR, ptr, <<5]    ;setup indirect data from element no.
    fast_wr[0, XMIT_VALIDATE], indirect_ref         ;start xmit
    alu [ptr, ptr, +, 1]                            ;inc ptr
    alu [--, countpkts, -, ptr]
	br!=0 [validate_loop#]                          ;result = 0 and we are done

    alu [--, countpkts, -, 16]
	br!=0 [Wait4Ptr#]                          		;result = 0 and we are done
    immed [countpkts, 0x0] 
	
Wait4Ptr#:
   	csr[read, $xfer0, xmit_ptr],ctx_swap
    alu [--, $xfer0, -, countpkts]
	br!=0 [Wait4Ptr#]                          		;result = 0 and we are done

WaitForValidFlags#:
	csr [read, $xfer0, xmit_rdy_ctl], ctx_swap
	alu_shf [xmit_valid_bits, --, b, $xfer0, >>16]
	br!=0 [WaitForValidFlags#]


;Here we will sum up the rx threads results until the number of rcvd pkts is equal
; to the number of tfifo elements that were validated above.  We add the # of pkts rcvd 
; and the number of data errors to reduce the possibility of hanging the test.  The test 
; will hang here if any pkts that were sent are somehow dropped.

	alu [rxpktsum, rxpktsum, +, PktSubBurst]
	immed [FPpassStatptr, FPpassStat]
	immed [maxindex, 32]

wait4rx#:
	immed [rxpktsum2, 0]
	immed [FPpassStatidx, 0]

innerwait4rx#:
	scratch [read, $xfer0, FPpassStatptr, FPpassStatidx, 8], ctx_swap
	alu [rxpktsum2, rxpktsum2, +, $xfer0]
	alu [rxpktsum2, rxpktsum2, +, $xfer1]
	alu [rxpktsum2, rxpktsum2, +, $xfer2]
	alu [rxpktsum2, rxpktsum2, +, $xfer3]
	alu [rxpktsum2, rxpktsum2, +, $xfer4]
	alu [rxpktsum2, rxpktsum2, +, $xfer5]
	alu [rxpktsum2, rxpktsum2, +, $xfer6]
	alu [rxpktsum2, rxpktsum2, +, $xfer7]
	
	alu [FPpassStatidx, FPpassStatidx, +, 8]
	alu [--, FPpassStatidx, -, maxindex]
	br!=0 [innerwait4rx#]									   

	alu [--, rxpktsum2, -, rxpktsum]
	br!=0 [wait4rx#]									   

    alu [countpkts, countpkts, +, PktSubBurst]
    alu [--, 16, -, ptr]                            ;sub ptr from 16
	br!=0 [Poll_Ready_Loop1#]                       ;result = 0 and we are done

;Update port status in scratch...
    immed [scr_addr, TXPacketCount]                 ;scratch addr ptr begin @ first port
    scratch[incr, --, scr_addr, 0, 1], ctx_swap

;loop to xmit another burst...
    br[RestartXmitLoop#]							;do more tx
    
    nop
    nop
    nop


























