/* Define open modes */

#define HPCFS_O_CREAT  0x01
#define HPCFS_O_TRUNC  0x02
#define HPCFS_O_APPEND 0x04
#define HPCFS_O_EXCL   0x08
#define HPCFS_O_TEXT   0x10

#define HPCFS_O_RDONLY 0x20
#define HPCFS_O_WRONLY 0x40
#define HPCFS_O_RDWR   0x60


/* Permission bits */

#define HPCFS_S_IFREG 0x4000 /* Regular file */
#define HPCFS_S_IFDIR 0x8000 /* Directory */


/* IOCTL commands */

#define HPCFS_IOCTL_FIOSEEK     1
#define HPCFS_IOCTL_FIOWHERE    2
#define HPCFS_IOCTL_FIONREAD    3
#define HPCFS_IOCTL_FIORENAME   4
#define HPCFS_IOCTL_FIOFSTATGET 5 /* not implemented */
#define HPCFS_IOCTL_FIOREADIR   6 /* not implemented */
#define HPCFS_IOCTL_FIOMKDIR    7 /* not implemented */
#define HPCFS_IOCTL_FIORMDIR    8 /* not implemented */
#define HPCFS_IOCTL_FIOGETFL    9 /* not implemented */

/* RPC COMMANDS */

#define HPCFS_FUNC_BASE  0x0000F200

#define HPCFS_FUNC_OPEN     (HPCFS_FUNC_BASE + 1)
#define HPCFS_FUNC_DEL      (HPCFS_FUNC_BASE + 2)
#define HPCFS_FUNC_CLOSE    (HPCFS_FUNC_BASE + 3)
#define HPCFS_FUNC_READ     (HPCFS_FUNC_BASE + 4)
#define HPCFS_FUNC_WRITE    (HPCFS_FUNC_BASE + 5)
#define HPCFS_FUNC_IOCTL    (HPCFS_FUNC_BASE + 6)

#define RPC_FUNCTION_MASK 0x0000FFFF
#define RPC_CHANNEL_MASK  0xFFFF0000
#define RPC_CHANNEL_OFFSET 16
#define GET_CHANNEL(x) (((unsigned int)(x)) >> RPC_CHANNEL_OFFSET)
#define MAKE_CHANNEL(x) ((x) << RPC_CHANNEL_OFFSET)

/* Error Numbers */

#define HPCFS_ERRNO_ENOTEMPTY        ( 1 << 16)
#define HPCFS_ERRNO_EDEADLK          ( 2 << 16)
#define HPCFS_ERRNO_ENOLCK           ( 3 << 16)
#define HPCFS_ERRNO_EMSGSIZE         ( 4 << 16)
#define HPCFS_ERRNO_EOPNOTSUPP       ( 5 << 16)
#define HPCFS_ERRNO_EADDRNOTAVAIL    ( 6 << 16)
#define HPCFS_ERRNO_ENOTSOCK         ( 7 << 16)
#define HPCFS_ERRNO_ENETRESET        ( 8 << 16)
#define HPCFS_ERRNO_ECONNABORTED     ( 9 << 16)
#define HPCFS_ERRNO_ECONNRESET       (10 << 16)
#define HPCFS_ERRNO_ECONNREFUSED     (11 << 16)
#define HPCFS_ERRNO_ENOBUFS          (12 << 16)
#define HPCFS_ERRNO_ENOTCONN         (13 << 16)
#define HPCFS_ERRNO_ESHUTDOWN        (14 << 16)
#define HPCFS_ERRNO_ETIMEDOUT        (15 << 16)
#define HPCFS_ERRNO_EINPROGRESS      (16 << 16)
#define HPCFS_ERRNO_EWOULDBLOCK      (17 << 16)
#define HPCFS_ERRNO_ENOSR            (18 << 16)
#define HPCFS_ERRNO_ELOOP            (19 << 16)
#define HPCFS_ERRNO_ENAMETOOLONG     (20 << 16)
#define HPCFS_ERRNO_EBADMSG          (21 << 16)


/* Maximum file path length is 256, make buffer size larger than that */
#define RPC_BUFF_SIZE (256+40)
