/* ixp1200HPC.c
 * Host communications driver via PCI (implementation for ixp1200)
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: IXP1200
 *  subsystem: VxWorks
 *  author: jdg, Mar 29, 00  Initial version
 *  revisions:
 *  $History: ixp1200HPC.c $
 * 
 * *****************  Version 8  *****************
 * User: Jdguilfo     Date: 10/20/00   Time: 2:43p
 * Updated in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Merge bootmgr and flashutility into bootflash
 * 
 * *****************  Version 7  *****************
 * User: Jdguilfo     Date: 8/31/00    Time: 7:10p
 * Updated in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Added sysCpuIDGet, ID printout on boot, fix in HPC for PCI csr write HW
 * bug
 * 
 * *****************  Version 6  *****************
 * User: Jdguilfo     Date: 8/11/00    Time: 5:02p
 * Updated in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Fix problem with HPC SIO interface, which would hang on mulitple sends
 * 
 * *****************  Version 5  *****************
 * User: Jdguilfo     Date: 8/04/00    Time: 11:54a
 * Updated in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Increase priority of HPC "DPC", I had the order of priorities backwards
 * before
 * 
 * *****************  Version 4  *****************
 * User: Jdguilfo     Date: 7/07/00    Time: 9:58a
 * Updated in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Fix bug where two threads using common IRP could corrupt it
 * 
 * *****************  Version 3  *****************
 * User: Jdguilfo     Date: 7/06/00    Time: 5:17p
 * Updated in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Allow app to specify incomplete message, overlapped I/O returns pending
 * status
 * 
 * *****************  Version 2  *****************
 * User: Jdguilfo     Date: 6/29/00    Time: 2:18p
 * Updated in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Allow users to cancel read or write transactions separately
 * 
 * *****************  Version 1  *****************
 * User: Jdguilfo     Date: 6/29/00    Time: 11:22a
 * Created in $/dev_1200/BoardSupport/VxWorks/IXP1200EB
 * Add HPC, merge VxWorks into one makefile
 * 
 * ---------------------------------------------------------------------
 */

#include "ixp1200HPC.h"
#ifndef NO_OS
#include "config.h"

#include <string.h>
#include <intLib.h>
#include <dllLib.h>
#include <taskLib.h>
#include <pciIomapLib.h>
#endif /* NO_OS */
#ifdef TEST
#include <stdio.h>
#endif

HPC_CONFIG_STATUS* const configStatus = (HPC_CONFIG_STATUS*)
     (SDRAM_PHYS_BASE + RESERVED_LOW_MEM - sizeof(HPC_CONFIG_STATUS));

HPC_BUFFER* const dataBuffer = (HPC_BUFFER*)
     (SDRAM_PHYS_BASE + RESERVED_LOW_MEM -
      ((NUM_SHARED_DATA_BUFFERS) * sizeof(HPC_BUFFER)) -
      sizeof(HPC_CONFIG_STATUS));

static HPC_GLOBAL_STATE hpcGlobalState;

#define SET(var, mask) ((var) |= (mask))
#define CLEAR(var, mask) ((var) &= ~(mask))

#ifndef NO_OS
/*
 * BEGIN QUEUE FUNCTIONS
 *
 * NOTE: These are not protected against multiple accesses. It is
 * assumed that a higher layer provides protection.
 */

static void
HPC_QUEUE_INIT(HPC_QUEUE *queue)
{
    queue->head = queue->tail = NULL;
}

static void
HPC_QUEUE_PUSH(HPC_QUEUE *queue, HPC_IRP *irp)
{
    if (queue->tail != NULL) {
        queue->tail->next = irp;
        queue->tail = irp;
        irp->next = NULL;
    } else {
        queue->head = queue->tail = irp;
        irp->next = NULL;
    }
}

#define HPC_QUEUE_PEEK(queue) ((queue)->head)
#define HPC_QUEUE_IS_EMPTY(queue) (HPC_QUEUE_PEEK(queue) == NULL)

static HPC_IRP*
HPC_QUEUE_POP(HPC_QUEUE *queue)
{
    HPC_IRP* irp;

    if (queue->head) {
        irp = queue->head;
        queue->head = irp->next;
        if (queue->head == NULL)
            queue->tail = NULL;
        return irp;
    } else {
        return NULL;
    }
}

/*
 * BEGIN main functions
 */

/* return generateInterrupt */
static BOOL
procRcvBuffer(HPC_BUFFER_INFO *binfo, int index)
{
    BOOL generateInterrupt = FALSE;
    HPC_IRP *irp;
    unsigned int size, size2;
    unsigned int sizeValid;
    unsigned int incomplete;

    switch (binfo->interface) {
    case HPC_W32:

        /* while buffer is available and shared buffer not empty */
        while ((NULL != (irp = HPC_QUEUE_PEEK(&binfo->queue))) &&
               (0 != (sizeValid = configStatus->sizeValid[index]))) {
            incomplete = sizeValid & BUFFER_INCOMPLETE_MASK;
            CLEAR(sizeValid, BUFFER_INCOMPLETE_MASK);

            size = sizeValid - binfo->bytesUsed;;
            size2 = irp->bSize - irp->bUsed;
            if (size > size2)
                size = size2;

            bcopy(binfo->buff + binfo->bytesUsed, /* src */
                  irp->buff + irp->bUsed,         /* dst */
                  size);
            irp->bUsed += size;
            binfo->bytesUsed += size;

            /*
             * If message is complete or we are out of user buffer,
             * complete the I/O
             */
            if ((!incomplete) || (irp->bUsed == irp->bSize)) {
                HPC_QUEUE_POP(&binfo->queue);
                irp->returnStatus = HPC_OK;
                if (incomplete || (binfo->bytesUsed < sizeValid)) {
                    switch (binfo->ibMode) {
                    case HPC_IB_RC:
                        irp->returnStatus = HPC_INCOMPLETE;
                        break;
                    case HPC_IB_SIZE:
                        irp->bUsed |= HPC_IB_BIT;
                        break;
                    case HPC_IB_NONE:
                        break;
                    } /* end switch ibMode */
                } /* end if we need to flag this buffer as incomplete */
                semGive(irp->done);
            }

            if (binfo->bytesUsed == sizeValid) {
                /* buffer is empty */
                configStatus->sizeValid[index] = 0;
                binfo->bytesUsed = 0;
                if (configStatus->NTIsWaitingMask & binfo->waitingMask)
                    generateInterrupt = TRUE;
                break;
            }
        } /* end while */
        if (HPC_QUEUE_IS_EMPTY(&binfo->queue)) {
            CLEAR(configStatus->IXP1200IsWaitingMask,
                  binfo->waitingMask);
        }

        break;
    case HPC_SIO:

        if (0 != (sizeValid = configStatus->sizeValid[index])) {
            if (binfo->ibMode != HPC_IB_SIZE)
                sizeValid &= (~BUFFER_INCOMPLETE_MASK);
            (*binfo->putRcvChars)(binfo->buff,
                                  sizeValid,
                                  binfo->putData);
            configStatus->sizeValid[index] = 0;
            if (configStatus->NTIsWaitingMask & binfo->waitingMask)
                generateInterrupt = TRUE;
        }

        break;
    case HPC_NONE:
        break;
    } /* end switch interface */

    return generateInterrupt;
}

static BOOL
procTxBuffer(HPC_BUFFER_INFO *binfo, int index)
{
    BOOL generateInterrupt = FALSE;
    HPC_IRP *irp;
    unsigned int size;
    unsigned char *buff;

    switch (binfo->interface) {
    case HPC_W32:

        if ((0 == configStatus->sizeValid[index]) &&
            (NULL != (irp = HPC_QUEUE_PEEK(&binfo->queue)))) {

            if (irp->bUsed == irp->bSize) {
                /* buffer is empty, so return it to app */
                HPC_QUEUE_POP(&binfo->queue);
                irp->returnStatus = HPC_OK;
                semGive(irp->done);
                irp = HPC_QUEUE_PEEK(&binfo->queue);
                if (irp == NULL) {
                    /* queue is empty */
                    CLEAR(configStatus->IXP1200IsWaitingMask,
                          binfo->waitingMask);
                    break;
                }
            } /* end if irp is used up */
            size = irp->bSize - irp->bUsed;
            if (size > SHARED_DATA_BUFFER_SIZE)
                size = SHARED_DATA_BUFFER_SIZE;
            bcopy(irp->buff + irp->bUsed, /* src */
                  binfo->buff,            /* dst */
                  size);
            irp->bUsed += size;
            if (irp->incomplete || (irp->bUsed < irp->bSize))
                size |= BUFFER_INCOMPLETE_MASK;
            configStatus->sizeValid[index] = size;
            if (configStatus->NTIsWaitingMask & binfo->waitingMask)
                generateInterrupt = TRUE;

        } /* end if ready and processable */
        break;
    case HPC_SIO:

        if ((0 == configStatus->sizeValid[index]) && (binfo->txWaiting)) {

            if (binfo->irp.bSize == binfo->irp.bUsed) {
                /* current buffer is empty, get new buffer */
                /*
                 * We have to handle txWaiting carefully. We set it to false
                 * before calling getTxChars, and then conditionally set it
                 * to true afterwards. This is to handle the case where the
                 * start routine is called between the time when getTxChars
                 * returns 0, and we update the txWaiting flag.
                 */
                binfo->txWaiting = FALSE;
                size = (*binfo->getTxChars)(&buff, binfo->getData);
                if (size == 0) {
                    /* no more data */
                    CLEAR(configStatus->IXP1200IsWaitingMask,
                          binfo->waitingMask);
                    break;
                } else {
                    binfo->txWaiting = TRUE;
                    SET(configStatus->IXP1200IsWaitingMask,
                        binfo->waitingMask);
                }
                if (size & HPC_IB_BIT) {
                    CLEAR(size, HPC_IB_BIT);
                    binfo->irp.incomplete = 1;
                } else {
                    binfo->irp.incomplete = 0;
                }
                binfo->irp.buff = buff;
                binfo->irp.bSize = size;
                binfo->irp.bUsed = 0;
            }
            size = binfo->irp.bSize - binfo->irp.bUsed;
            if (size > SHARED_DATA_BUFFER_SIZE)
                size = SHARED_DATA_BUFFER_SIZE;
            bcopy(binfo->irp.buff + binfo->irp.bUsed, /* src */
                  binfo->buff,                        /* dst */
                  size);
            binfo->irp.bUsed += size;
            if (binfo->irp.incomplete || (binfo->irp.bUsed < binfo->irp.bSize))
                size |= BUFFER_INCOMPLETE_MASK;
            configStatus->sizeValid[index] = size;
            if (configStatus->NTIsWaitingMask & binfo->waitingMask)
                generateInterrupt = TRUE;
                    
        } /* end if ready and IX1200IsWaiting */

        break;
    case HPC_NONE:
        break;
    } /* end switch interface */

    return generateInterrupt;
}


static void
procBuffers(HPC_GLOBAL_STATE *state)
{
    int i;
    BOOL generateInterrupt = FALSE;

    for (i=0; i<NUM_SHARED_DATA_BUFFERS; i+=2) {
        if (procRcvBuffer(&state->bufferInfo[i+HPC_RX_OFFSET],
                            i+HPC_RX_OFFSET)) {
            generateInterrupt = TRUE;
        }
        if (procTxBuffer(&state->bufferInfo[i+HPC_TX_OFFSET],
                           i+HPC_TX_OFFSET)) {
            generateInterrupt = TRUE;
        }
    }


    if (generateInterrupt) {
        IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_SEND_MASK);
        if (sysCpuRev < 2) {
            /* Due to an A3/B0 bug, if a PCI device is writing to a PCI CSR at the right
               time, out write can get lost; so we had better check it. */
            while (1) {
                IXP1200_REG_READ(IXP1200_DOORBELL, i);
                if ((i & DOORBELL_SEND_MASK) != 0)
                    break;
                IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_SEND_MASK);
            } /* end while */
        } /* end if sysCpuRev < 2 */
    }
}

void
procBufferLoop(HPC_GLOBAL_STATE *state)
{
    while (1) {
        semTake(state->DPCSem, WAIT_FOREVER);
        semTake(state->commonSem, WAIT_FOREVER);
        procBuffers(state);
        semGive(state->commonSem);
    } /* end while 1 */
}

#endif /* NO_OS */

int
hpcPolledRcv(int channel, unsigned char *buf, unsigned int bsize,
               unsigned int *incomp)
{
    int index = 2 * channel + HPC_RX_OFFSET;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[index];
    unsigned int size, size2;
    unsigned int sizeValid;
    unsigned int incomplete;
    unsigned int i;

    sizeValid = configStatus->sizeValid[index];
    if (0 == sizeValid) {
        return 0;
    }

    incomplete = sizeValid & BUFFER_INCOMPLETE_MASK;
    CLEAR(sizeValid, BUFFER_INCOMPLETE_MASK);

    size = sizeValid - binfo->bytesUsed;;
    size2 = bsize;
    if (size > size2)
        size = size2;

    bcopy(binfo->buff + binfo->bytesUsed, /* src */
          buf,                            /* dst */
          size);
    binfo->bytesUsed += size;

    if (incomp) {
        if (incomplete || (binfo->bytesUsed < sizeValid))
            *incomp = 1;
        else
            *incomp = 0;
    }

    if (binfo->bytesUsed == sizeValid) {
        /* buffer is empty */
        configStatus->sizeValid[index] = 0;
        binfo->bytesUsed = 0;
        if (configStatus->NTIsWaitingMask & binfo->waitingMask) {
            IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_SEND_MASK);
            if (sysCpuRev < 2) {
                /* Due to an A3/B0 bug, if a PCI device is writing to a PCI CSR at the right
                   time, out write can get lost; so we had better check it. */
                while (1) {
                    IXP1200_REG_READ(IXP1200_DOORBELL, i);
                    if ((i & DOORBELL_SEND_MASK) != 0)
                        break;
                    IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_SEND_MASK);
                } /* end while */
            } /* end if sysCpuRev < 2 */
        }
    }
        
    return size;
}

int
hpcPolledSend(int channel, unsigned char *buf, unsigned int bsize)
{
    int index = 2 * channel + HPC_TX_OFFSET;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[index];
    unsigned int size;
    unsigned int i;

    if (0 != configStatus->sizeValid[index])
        return 0;

    size = bsize;
    if (size > SHARED_DATA_BUFFER_SIZE)
        size = SHARED_DATA_BUFFER_SIZE;
    bcopy(buf,          /* src */
          binfo->buff,  /* dst */
          size);

    if (size < bsize)
        size |= BUFFER_INCOMPLETE_MASK;
    configStatus->sizeValid[index] = size;
    if (configStatus->NTIsWaitingMask & binfo->waitingMask) {
        IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_SEND_MASK);
        if (sysCpuRev < 2) {
            /* Due to an A3/B0 bug, if a PCI device is writing to a PCI CSR at the right
               time, out write can get lost; so we had better check it. */
            while (1) {
                IXP1200_REG_READ(IXP1200_DOORBELL, i);
                if ((i & DOORBELL_SEND_MASK) != 0)
                    break;
                IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_SEND_MASK);
            } /* end while */
        } /* end if sysCpuRev < 2 */
    }

    return size & ~BUFFER_INCOMPLETE_MASK;
}

void
hpcMemInit()
{
    int i;

    for (i=0; i<NUM_SHARED_DATA_BUFFERS; i++)
        configStatus->sizeValid[i] = 0;
    configStatus->NTIsWaitingMask = 0;
    configStatus->IXP1200IsWaitingMask = 0;
    configStatus->Signature = SIGNATURE;
}

void
hpcMemDummyInit()
{
    configStatus->Signature = 0;
}

#ifndef NO_OS

void
hpcIsr(int param)
{
    UINT32 val;

    IXP1200_REG_READ(IXP1200_DOORBELL, val);
    if ((val & DOORBELL_RECV_MASK) == 0) {
        IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_RECV_MASK);
        if (sysCpuRev < 2) {
            /* Due to an A3/B0 bug, if a PCI device is writing to a PCI CSR at the right
               time, out write can get lost; so we had better check it. */
            while (1) {
                IXP1200_REG_READ(IXP1200_DOORBELL, val);
                if ((val & DOORBELL_RECV_MASK) != 0)
                    break;
                IXP1200_REG_WRITE(IXP1200_DOORBELL, DOORBELL_RECV_MASK);
            } /* end while */
        } /* end if sysCpuRev < 2 */
        semGive(hpcGlobalState.DPCSem);
    }
}

#endif /* NO_OS */

/*
 * A "simple" init skips the following steps:
 *    configuration of doorbell registers
 * Normally this should not be skipped (i.e. simple should be 0), but this
 * can be set to 1 in the case that a previous OS has set up the doorbell
 * registers, and the new OS does not want to clobber them.
 */
void
hpcSysInit(int simple)
{
    int i;
    HPC_BUFFER_INFO *binfo;
    UINT32 val;

    for (i=0; i<NUM_SHARED_DATA_BUFFERS; i++) {
        binfo = &hpcGlobalState.bufferInfo[i];
        binfo->buff = &dataBuffer[i][0];
        binfo->interface = HPC_NONE;
        binfo->waitingMask = 1 << i;
#ifndef NO_OS
        HPC_QUEUE_INIT(&binfo->queue);
        binfo->irp.done = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
        binfo->irpSem = semBCreate(SEM_Q_FIFO, SEM_FULL);
#endif /* NO_OS */
    } /* end for i */
#ifndef NO_OS
    hpcGlobalState.DPCSem    = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    hpcGlobalState.commonSem = semBCreate(SEM_Q_FIFO, SEM_FULL);

    pciIntConnect((void*) INT_VEC_DFH, hpcIsr, 0);
#endif /* NO_OS */

    if (! simple) {
        IXP1200_REG_READ(IXP1200_DOORBELL_SETUP, val);
        SET(val, DOORBELL_RECV_MASK);
        CLEAR(val, DOORBELL_SEND_MASK);
        IXP1200_REG_WRITE(IXP1200_DOORBELL_SETUP, val);

        IXP1200_REG_READ(IXP1200_DBELL_PCI_MASK, val);
        val |= DOORBELL_SEND_MASK;
        IXP1200_REG_WRITE(IXP1200_DBELL_PCI_MASK, val);

        IXP1200_REG_READ(IXP1200_DBELL_SA_MASK, val);
        val |= DOORBELL_RECV_MASK;
        IXP1200_REG_WRITE(IXP1200_DBELL_SA_MASK, val);
    }

#ifndef NO_OS
    taskSpawn("tHpcDpc", /* name */
              6,         /* priority: 0..255, 0 is highest */
              0,         /* options */
              4*1024,    /* stack size */
              (FUNCPTR) procBufferLoop, /* entry */
              (int)&hpcGlobalState, 0, 0, 0, 0, 0, 0, 0, 0, 0); /* arguments for entry */
#endif /* NO_OS */
}

#ifndef NO_OS
HPC_ERROR_CODE
hpcOpenW32(int channelNum)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];
    int i;

    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);
    if (binfo->interface != HPC_NONE) {
        semGive(hpcGlobalState.commonSem);
        return HPC_ALREADY_OPENED;
    }
    for (i=0; i<2; i++) {
        if (! HPC_QUEUE_IS_EMPTY(&binfo[i].queue)) {
        semGive(hpcGlobalState.commonSem);
        return HPC_INTERNAL_ERROR;
        }
    }

    for (i=0; i<2; i++) {
        binfo[i].interface = HPC_W32;
        binfo[i].bytesUsed = 0;
        CLEAR(configStatus->IXP1200IsWaitingMask, binfo[i].waitingMask);
        binfo[i].irp.returnStatus = HPC_OK;
        binfo[i].ibMode = HPC_IB_NONE;
    }

    semGive(hpcGlobalState.commonSem);

    return HPC_OK;
}

HPC_ERROR_CODE
hpcOpenSio(int channelNum,
             PUT_RCV_CHARS putRcvChars, void* putData,
             GET_TX_CHARS  getTxChars,  void* getData)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];
    int i;

    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);
    if (binfo->interface != HPC_NONE) {
        semGive(hpcGlobalState.commonSem);
        return HPC_ALREADY_OPENED;
    }

    for (i=0; i<2; i++) {
        binfo[i].interface = HPC_SIO;
        binfo[i].bytesUsed = 0;
        binfo[i].putRcvChars = putRcvChars;
        binfo[i].putData = putData;
        binfo[i].getTxChars = getTxChars;
        binfo[i].getData = getData;
        if (HPC_IS_TX(buffNum+i)) {
            CLEAR(configStatus->IXP1200IsWaitingMask, binfo[i].waitingMask);
            binfo[i].txWaiting = FALSE;
        } else {
            SET(configStatus->IXP1200IsWaitingMask, binfo[i].waitingMask);
        }
        binfo[i].irp.bUsed = 0;
        binfo[i].irp.bSize = 0;
        binfo[i].ibMode = HPC_IB_NONE;
    }

    semGive(hpcGlobalState.commonSem);

    return HPC_OK;
}

static void
hpcDoCancelio(int channelNum, HPC_OP whichOnes)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];
    HPC_IRP *irp;

    if (whichOnes !=  HPC_READ) {
        while ((irp = HPC_QUEUE_POP(&binfo[HPC_TX_OFFSET].queue))) {
            irp->returnStatus = HPC_CANCELED;
            semGive(irp->done);
        } /* end while */
    }

    if (whichOnes !=  HPC_WRITE) {
        while ((irp = HPC_QUEUE_POP(&binfo[HPC_RX_OFFSET].queue))) {
            irp->returnStatus = HPC_CANCELED;
            semGive(irp->done);
        } /* end while */
    }
}

HPC_ERROR_CODE
hpcCancelio(int channelNum, HPC_OP whichOnes)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];

    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);

    if (binfo->interface == HPC_NONE) {
        semGive(hpcGlobalState.commonSem);
        return HPC_NOT_OPENED;
    }
    hpcDoCancelio(channelNum, whichOnes);

    semGive(hpcGlobalState.commonSem);
    return HPC_OK;
}

HPC_ERROR_CODE
hpcClose(int channelNum)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];
    int i;

    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);

    if (binfo->interface == HPC_NONE) {
        semGive(hpcGlobalState.commonSem);
        return HPC_NOT_OPENED;
    }
    
    hpcDoCancelio(channelNum, HPC_BOTH);

    for (i=0; i<2; i++) {
        binfo[i].interface = HPC_NONE;
        CLEAR(configStatus->IXP1200IsWaitingMask, binfo[i].waitingMask);
        configStatus->sizeValid[buffNum+i] = 0;
    } /* end for i */

    semGive(hpcGlobalState.commonSem);
    return HPC_OK;
}

HPC_ERROR_CODE
hpcSetmode(int channelNum, HPC_IB_MODE mode)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];

    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);

    if (binfo->interface == HPC_NONE) {
        semGive(hpcGlobalState.commonSem);
        return HPC_NOT_OPENED;
    }
    binfo[0].ibMode = mode;
    binfo[1].ibMode = mode;

    semGive(hpcGlobalState.commonSem);
    return HPC_OK;
}

HPC_ERROR_CODE
hpcGetmode(int channelNum, HPC_IB_MODE *mode)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];

    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);

    if (binfo->interface == HPC_NONE) {
        semGive(hpcGlobalState.commonSem);
        return HPC_NOT_OPENED;
    }
    *mode = binfo[0].ibMode;

    semGive(hpcGlobalState.commonSem);
    return HPC_OK;
}

HPC_ERROR_CODE
hpcStartio(int channelNum)
{
    /* we don't want to take the common semaphore here, in case we're called
       from ISR priority. */

    int buffNum = channelNum * QUEUES_PER_CHANNEL + HPC_TX_OFFSET;
    HPC_BUFFER_INFO *binfo = &hpcGlobalState.bufferInfo[buffNum];

    if ( binfo->interface != HPC_SIO ) {
        if ( binfo->interface == HPC_NONE )
            return HPC_NOT_OPENED;
        else
            return HPC_WRONG_MODE;
    }

    binfo->txWaiting = TRUE;

    semGive(hpcGlobalState.DPCSem);
    return HPC_OK;
}

HPC_ERROR_CODE
hpcReadwrite(int channelNum, HPC_OP op, unsigned char *buff,
              int buffsize, unsigned int *bytesUsed, HPC_IRP *userIrp)
{
    int buffNum = channelNum * QUEUES_PER_CHANNEL;
    HPC_BUFFER_INFO *binfo;
    HPC_IRP *irp;
    HPC_ERROR_CODE result;

    if (buffsize < 1)
        return HPC_INVALID_SIZE;

    if (op == HPC_WRITE)
        buffNum += HPC_TX_OFFSET;
    else if (op == HPC_READ)
        buffNum += HPC_RX_OFFSET;
    else
        return HPC_INVALID_OP;

    binfo = &hpcGlobalState.bufferInfo[buffNum];

    if (!userIrp) {
        semTake(binfo->irpSem, WAIT_FOREVER);
    }

    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);
    
    if ( binfo->interface != HPC_W32 ) {
        semGive(hpcGlobalState.commonSem);
        if (!userIrp)
            semGive(binfo->irpSem);
        if ( binfo->interface == HPC_NONE ) {
            return HPC_NOT_OPENED;
        } else {
            return HPC_WRONG_MODE;
        }
    }

    if (userIrp) {
        irp = userIrp;
    } else {
        irp = &binfo->irp;
    }
    irp->buff = buff;
    irp->incomplete = 0;
    if (buffsize & HPC_IB_BIT) {
        irp->incomplete = 1;
        CLEAR(buffsize, HPC_IB_BIT);
    }
    irp->bSize = buffsize;
    irp->bUsed = 0;
    semTake(irp->done, NO_WAIT); /* make sure it is taken */
    irp->returnStatus = HPC_PENDING;

    HPC_QUEUE_PUSH(&binfo->queue, irp);
    SET(configStatus->IXP1200IsWaitingMask, binfo->waitingMask);

    semGive(hpcGlobalState.DPCSem);
    semGive(hpcGlobalState.commonSem);

    if (!userIrp) {
        /* block on IRP */
        semTake(irp->done, WAIT_FOREVER);
        *bytesUsed = irp->bUsed;
        result = irp->returnStatus;
        semGive(binfo->irpSem);
        return result;
    }

    return HPC_PENDING;
}

HPC_ERROR_CODE
hpcGetIrpStatus( HPC_IRP *irp, unsigned int *bytesUsed, BOOL wait )
{
    HPC_ERROR_CODE returnStatus;

    if (wait)
        semTake(irp->done, WAIT_FOREVER);
    semTake(hpcGlobalState.commonSem, WAIT_FOREVER);
    *bytesUsed = irp->bUsed;
    returnStatus = irp->returnStatus;
    semGive(hpcGlobalState.commonSem);

    return returnStatus;
}

void
hpcInitIrp( HPC_IRP *irp)
{
    irp->done = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    irp->returnStatus = HPC_OK;
    irp->bUsed = 0;
}

HPC_ERROR_CODE
hpcDeleteIrp( HPC_IRP *irp)
{
    if (irp->returnStatus == HPC_PENDING)
        return HPC_PENDING;
    semDelete(irp->done);
    return HPC_OK;
}

#endif /* NO_OS */

#ifdef TEST

VOID pciInt(int irq);

void
hpcStartup()
{
    /* To be done before PCI starts up */
    hpcMemInit();

    /* To be done after OS starts up */
    IXP1200_REG_WRITE(IXP1200_DBELL_PCI_MASK, 0);
    IXP1200_REG_WRITE(IXP1200_DBELL_SA_MASK,  0);
    
    intConnect((void*)INT_VEC_DFH, pciInt, INT_VEC_DFH);
    intEnable(INT_VEC_DFH);

    hpcSysInit();

}

#define BSIZE 40
#define BSIZE2 1024
#define NBUFFS 3

void
hpcRecPkts(int n)
{
    HPC_ERROR_CODE rc;
    unsigned char buf[BSIZE];
    int i;
    unsigned int j, bytesUsed;

    rc = hpcClose(1);
    if (rc != HPC_NOT_OPENED) {
        printf("Channel 1 was not properly closed earlier: %x\n",rc);
    }
    rc = hpcOpenW32(1);
    if (rc != HPC_OK) {
        printf("Error opening channel 1: %X\n",rc);
        return;
    }
    rc = hpcSetmode(1, HPC_IB_SIZE);
    if (rc != HPC_OK) {
        printf("Error setting mode: %X\n",rc);
        hpcClose(1);
        return;
    }
    for (i=0; i<n; i++) {
        rc = hpcReadwrite(1, HPC_READ, buf, BSIZE, &bytesUsed, NULL);
        if (rc != HPC_OK) {
            printf("Error in HPC_READWRITE: %X\n",rc);
            hpcClose(1);
            return;
        }
        printf("bytesUsed = %X\n",bytesUsed);
        bytesUsed &= ~HPC_IB_BIT;
        for (j=0; j < bytesUsed; j++) {
            printf(" %02X",buf[j]);
        }
        printf("\n");
    }
    hpcClose(1);
}

typedef struct {
    unsigned char buf[BSIZE];
    HPC_IRP       irp;
} IRPB;

void
hpcRecPkts2(int n)
{
    HPC_ERROR_CODE rc;
    IRPB irpb[NBUFFS];
    int i,k;
    unsigned int j, bytesUsed;

    rc = hpcClose(1);
    if (rc != HPC_NOT_OPENED) {
        printf("Channel 1 was not properly closed earlier: %x\n",rc);
    }
    rc = hpcOpenW32(1);
    if (rc != HPC_OK) {
        printf("Error opening channel 1: %X\n",rc);
        return;
    }
    rc = hpcSetmode(1, HPC_IB_SIZE);
    if (rc != HPC_OK) {
        printf("Error setting mode: %X\n",rc);
        hpcClose(1);
        return;
    }

    for (i=0; i<NBUFFS; i++) {
        hpcInitIrp(&irpb[i].irp);
        rc = hpcReadwrite(1, HPC_READ, &irpb[i].buf[0], BSIZE, &bytesUsed, &irpb[i].irp);
        if (rc != HPC_PENDING) {
            printf("hpcReadwrite not pending\n");
            goto myexit;
        }
    }

    k = 0;

    for (i=0; i<n; i++) {
        rc = hpcGetIrpStatus(&irpb[k].irp, &bytesUsed, TRUE);
        if (rc != HPC_OK) {
            printf("Error in HPCGetIrpStatus: %X\n",rc);
            goto myexit;
        }
        printf("bytesUsed = %X\n",bytesUsed);
        bytesUsed &= ~HPC_IB_BIT;
        for (j=0; j < bytesUsed; j++) {
            printf(" %02X",irpb[k].buf[j]);
        }
        printf("\n");
        rc = hpcReadwrite(1, HPC_READ, &irpb[k].buf[0], BSIZE, &bytesUsed, &irpb[k].irp);
        if (rc != HPC_PENDING) {
            printf("hpcReadwrite not pending\n");
            goto myexit;
        }
        k++;
        if (k >= NBUFFS)
            k = 0;
    }
 myexit:
    hpcClose(1);

    for (i=0; i<NBUFFS; i++) {
        hpcDeleteIrp(&irpb[i].irp);
    }
    
}

void
hpcSendPkts(int n, unsigned int size)
{
    HPC_ERROR_CODE rc;
    unsigned char buf[BSIZE2];
    int i;
    unsigned int j, k, bytesUsed;

    if (size > BSIZE2) {
        printf("size exceeds %d\n",BSIZE2);
        return;
    }

    rc = hpcClose(1);
    if (rc != HPC_NOT_OPENED) {
        printf("Channel 1 was not properly closed earlier: %x\n",rc);
    }
    rc = hpcOpenW32(1);
    if (rc != HPC_OK) {
        printf("Error opening channel 1: %X\n",rc);
        return;
    }
    rc = hpcSetmode(1, HPC_IB_SIZE);
    if (rc != HPC_OK) {
        printf("Error setting mode: %X\n",rc);
        hpcClose(1);
        return;
    }
    for (i=0; i<n; i++) {
        k = 0;
        for (j=0; j<size; j++) {
            k += i+1;
            buf[j] = 0x30 + (k & 0x3F);
        }
        rc = hpcReadwrite(1, HPC_WRITE, buf, size, &bytesUsed, NULL);
        if (rc != HPC_OK) {
            printf("Error in HPC_READWRITE: %X\n",rc);
            hpcClose(1);
            return;
        }
        printf("bytesUsed = %X\n",bytesUsed);
    }
    hpcClose(1);
}

void
hpcSendPkts2(int n, unsigned int size)
{
    HPC_ERROR_CODE rc;
    IRPB irpb[NBUFFS];
    int i,p;
    unsigned int j, k, bytesUsed;

    if (size > BSIZE2) {
        printf("size exceeds %d\n",BSIZE2);
        return;
    }

    if (n < NBUFFS) {
        printf("n must be at least %d\n",NBUFFS);
        return;
    }

    rc = hpcClose(1);
    if (rc != HPC_NOT_OPENED) {
        printf("Channel 1 was not properly closed earlier: %x\n",rc);
    }
    rc = hpcOpenW32(1);
    if (rc != HPC_OK) {
        printf("Error opening channel 1: %X\n",rc);
        return;
    }
    rc = hpcSetmode(1, HPC_IB_SIZE);
    if (rc != HPC_OK) {
        printf("Error setting mode: %X\n",rc);
        hpcClose(1);
        return;
    }
    for (i=0; i<NBUFFS; i++) {
        hpcInitIrp(&irpb[i].irp);
    }

    p = 0;
    for (i=0; i<n+NBUFFS; i++) {
        if (i >= NBUFFS) {
            rc = hpcGetIrpStatus(&irpb[p].irp, &bytesUsed, TRUE);
            if (rc != HPC_OK) {
                printf("Error in HPCGetIrpStatus: %X\n",rc);
                goto myexit;
            }
            printf("bytesUsed = %X\n",bytesUsed);
        }
        if (i < n) {
            k = 0;
            for (j=0; j<size; j++) {
                k += i+1;
                irpb[p].buf[j] = 0x30 + (k & 0x3F);
            }
            rc = hpcReadwrite(1, HPC_WRITE, &irpb[p].buf[0], size, &bytesUsed, &irpb[p].irp);
            if (rc != HPC_PENDING) {
                printf("Error in HPC_READWRITE: %X\n",rc);
                goto myexit;
            }
        }
        p++;
        if (p >= NBUFFS)
            p = 0;
    }
 myexit:
    hpcClose(1);

    for (i=0; i<NBUFFS; i++) {
        hpcDeleteIrp(&irpb[i].irp);
    }
}
#endif
