/* ixp1200Sio.h - Intel IXP1200 UART header file */

/* Copyright 1998 Wind River Systems, Inc.; Copyright 1999 Intel Corp. */

/*
modification history
--------------------
01c,07Mar00,jdg  Added support for B0 revision ixp1200
01b,13aug99,jdg  changed name from vbsa1200 to ixp1200eb
01a,07may99,jdg  written from sa1100Sio.h, version 01a
*/

#ifndef __INCixp1200Sioh 
#define __INCixp1200Sioh 

#ifdef __cplusplus
extern "C" {
#endif

/* Register description of IXP1200 UART */

#ifndef _ASMLANGUAGE


/* Register addresses */

#define UART_SR  0x90003400    /* UART Status Register */
#define UART_CR  0x90003800    /* UART Control Register */
#define UART_DR  0x90003C00    /* UART Data Register */

/* bit definitions within UART_SR */

#define UART_TX_FULL  (1 << 7)  /* Transmit fifo full */
#define UART_TX_EMPTY (1 << 5)  /* Transmit fifo empty */
#define UART_TX_RDY   (1 << 2)  /* Transmit fifo ready (not full) */
#define UART_RX_RDY   (1 << 4)  /* Receive fifo ready (not empty) */

/* bit definitions within UART_CR */

#define UART_IRQ_SEL   (1 << 9)  /* IRQ select      (Rev B) */
#define UART_INT_CLR   (1 << 9)  /* Interrupt clear (Rev A) */
#define UART_TX_INT_EN (1 << 8)  /* Transmit interupt enable */
#define UART_EN        (1 << 7)  /* UART Enable */
#define UART_RX_INT_EN (1 << 4)  /* Receive interupt enable */

#define UART_BDR_OFFSET 16
#define UART_BDR_MASK (0x3FF << 16) /* Baud Rate Mask */

#define UART_DSS_5BIT (0 << 5)   /* Data size select */
#define UART_DSS_6BIT (1 << 5)
#define UART_DSS_7BIT (2 << 5)
#define UART_DSS_8BIT (3 << 5)

#define UART_SBS_1BIT (0 << 3)   /* Stop Bit Select */
#define UART_SBS_2BIT (1 << 3)

#define UART_PS_EVEN (0 << 2)    /* Parity Select */
#define UART_PS_ODD  (1 << 2) 

#define UART_PE      (1 << 1)    /* Parity Enable */

/* bit definitions within UART_DR */

#define UART_ROR (1 << 10) /* Receive OverRun Error */
#define UART_FRE (1 <<  9) /* Framing Error */
#define UART_PRE (1 <<  8) /* Parity Error */
#define UART_DATA (0xFF)   /* Data */


typedef struct IXP1200_CHAN
    {
    /* must be first */

    SIO_CHAN	sio;		/* standard SIO_CHAN element */

    /* callbacks */

    STATUS	(*getTxChar) ();  /* installed Tx callback routine */
    STATUS	(*putRcvChar) (); /* installed Rx callback routine */
    void *	getTxArg;	      /* argument to Tx callback routine */
    void *	putRcvArg;	      /* argument to Rx callback routine */

    UINT32	channelMode;	  /* such as INT, POLL modes */
    int		baudRate;	      /* the current baud rate */
    UINT32	xtal;		      /* UART clock frequency */     
    UINT32  uartCR;           /* Current value of Contrl Register */

    } IXP1200_CHAN;


/* function declarations */

IMPORT void ixp1200UartInt (IXP1200_CHAN *pChan);
IMPORT void ixp1200UartDevInit (IXP1200_CHAN *pChan);
IMPORT void ixp1200UartDevInit2 (IXP1200_CHAN *pChan); /* will enable interrupts */
IMPORT void ixp1200UartDevReset (IXP1200_CHAN *pChan);

#endif	/* _ASMLANGUAGE */

#ifdef __cplusplus
}
#endif
 
#endif /* __INCixp1200Sioh */
