/* ixp1200eb.h - Intel's ixp1200eb board header */

/* Copyright 1998 Wind River Systems, Inc.; Copyright 1999 Intel Corp. */

/*
modification history
--------------------
01d,06Oct00,jdg  Added Big-Endian support
01c,07Mar00,jdg  Added support for B0 revision ixp1200
01b,13aug99,jdg  changed name from vbsa1200 to ixp1200eb
01a,08apr99,jdg  created from 01e of brutus
*/

/*
 * This file contains I/O address and related constants for the 
 * Intel ixp1200eb board.
 */

#ifndef	INCixp1200ebh
#define	INCixp1200ebh


/* General Purpose 32-bit register read/write macros */

#define IXP1200_REG_READ(a,val) ((val) = *(volatile UINT32 *)(a))
#define IXP1200_REG_WRITE(a,val) (*(volatile UINT32 *)(a) = (val))


#define TARGET_IXP1200EB

#define BUS		BUS_TYPE_PCI

/* Define macros for fixing 8 and 16 bit addresses for big-endian operation */
#if (_BYTE_ORDER == _BIG_ENDIAN)
#define FIX_ADDR_16(x) ((x) ^ 2)
#define FIX_ADDR_32(x) ((x) ^ 3)
#else
#define FIX_ADDR_16(x) (x)
#define FIX_ADDR_32(x) (x)
#endif

/*
 * Local-to-Bus memory address constants:
 * the local memory address always appears at 0 locally;
 * it is not dual ported.
 */

#define LOCAL_MEM_LOCAL_ADRS    0x00000000      /* fixed */
#define LOCAL_MEM_BUS_ADRS      0x00000000      /* fixed */

/* PLL Config */

/*
 * Values for Core Clock Configuration: all values assume that the PLL
 * is fed by a 3.6864 MHZ crystal oscillator.
 *
 * NOTE: changing the value used for the core clock speed means that
 * the other configuration parameters that specify timings in terms of
 * units of the core clock, e.g. the memory timings, may also need to be
 * changed.
 */

#define CCF_A_29_49   0
#define CCF_A_36_87   1
#define CCF_A_44_24   2
#define CCF_A_51_61   3
#define CCF_A_58_98   4
#define CCF_A_66_36   5
#define CCF_A_73_73   6
#define CCF_A_81_10   7
#define CCF_A_88_48   8
#define CCF_A_95_85   9
#define CCF_A_103_22 10
#define CCF_A_110_59 11
#define CCF_A_117_97 12
#define CCF_A_132_71 13
#define CCF_A_147_46 14
#define CCF_A_154_83 15
#define CCF_A_162_20 16
#define CCF_A_176_95 17
#define CCF_A_191_70 18

#define CCCR_INIT_VALUE_A (CCF_A_176_95) /* see warning about changing above */
#define CLOCK_SPEED_STRING_A "177"

/* add parameters for fast clock rate */

#define CCF_29_49   0
#define CCF_36_87   1
#define CCF_44_24   2
#define CCF_51_61   3
#define CCF_58_98   4
#define CCF_66_36   5
#define CCF_73_73   6
#define CCF_81_10   7
#define CCF_88_48   8
#define CCF_95_85   9
#define CCF_103_22 10
#define CCF_110_59 11
#define CCF_132_71 12
#define CCF_147_46 13
#define CCF_154_83 14
#define CCF_162_20 15
#define CCF_165_89 16
#define CCF_176_95 17
#define CCF_191_69 18
#define CCF_199_07 19

#define CCF_206_44 20
#define CCF_221_18 21
#define CCF_232_24 22

#define CCCR_INIT_VALUE (CCF_199_07) /* see warning about changing above */
#define CLOCK_SPEED_STRING "200"

#define CCCR_INIT_VALUE_C (CCF_232_24) /* see warning about changing above */
#define CLOCK_SPEED_STRING_C "233"

/* Real Time Clock control bits */

#define RTCDIV_IRQS (1 << 19) /* Interrupt sent on IRQ */
#define RTCDIV_IRST (1 << 18) /* Interrupt reset bit */
#define RTCDIV_IEN  (1 << 17) /* Interrupt Enable bit */
#define RTCDIV_WEN  (1 << 16) /* Write enable: 1 -> only write bits 17:19 */

#define RTCDIV_RESET_VAL (RTCDIV_WEN)


/* SRAM CSR bits */

#define SRAM_RLK  1 /* ?? Read Lock Order Enable */
#define SRAM_RDY  0 /* ?? Slowport RDY enable */
#define SRAM_CFT  0 /* Comand FIFO Test Enable */
#define SRAM_FLWT 0 /* Flowthrough SRAM Enable */
#define SRAM_BKW  0 /* Bank Switch Wait Enable */
#define SRAM_8MF  1 /* 8MB Flash */
#define SRAM_4MF  0 /* 4MB Flash */
#define SRAM_2MF  0 /* 2MB Flash */
#define SRAM_8MSR 0 /* 8MB SRAM */
#ifdef SPECTACLE_ISLAND
#define SRAM_4MSR 1 /* 4MB SRAM */
#define SRAM_2MSR 0 /* 2MB SRAM */
#else
#define SRAM_4MSR 0 /* 4MB SRAM */
#define SRAM_2MSR 1 /* 2MB SRAM */
#endif
#define SRAM_1MSR 0 /* 1MB SRAM */
#define SRAM_RLAM 0 /* ASB Read Lock Achieved Mask */
#define SRAM_IRQ  0 /* IRQ Enable */
#define SRAM_FIQ  0 /* FIQ Enable */
#define SRAM_RLRS 0 /* ASB Read Lock Retry Achieved Status */
#define SRAM_RLS  0 /* ASB Read Lock Achieved Status */
#define SRAM_CSR ((SRAM_RLK  << 20) | (SRAM_RDY  << 19) | \
                  (SRAM_CFT  << 18) |                     (SRAM_FLWT << 16) | \
                  (SRAM_BKW  << 15) | (SRAM_8MF  << 14) | (SRAM_4MF  << 13) | \
                  (SRAM_2MF  << 12) | (SRAM_8MSR << 11) | (SRAM_4MSR << 10) | \
                  (SRAM_2MSR <<  9) | (SRAM_1MSR <<  8) | (SRAM_RLAM <<  4) | \
                  (SRAM_IRQ  <<  3) | (SRAM_FIQ  <<  2) | (SRAM_RLRS <<  1) | \
                  (SRAM_RLS))

/* SRAM Cycle times for flash/mac (based on 199 MHz clock) */

#define SRAM_FCT 0x18 /* Flash cycle time */
#define SRAM_MCT 0x59 /* Mac cycle time */
#define SRAM_SLOW_CFG ((SRAM_FCT <<  8) | (SRAM_MCT))
#define SRAM_SLOW_CFG_A 0x00001852
#define SRAM_SLOW_CFG_C SRAM_SLOW_CFG

/* SRAM Flash timing (based on 199 MHz clock) */

#define SRAM_FRWA 0x18 /* Flash R/W Assert */
#define SRAM_FEA  0x18 /* Flash Enable Assert */
#define SRAM_FRWD 0x04 /* Flash R/W Deassert */
#define SRAM_FED  0x00 /* Flash Enable Deassert */
#define SRAM_FLASH_CFG ((SRAM_FRWA << 24) | (SRAM_FRWD <<  8) | \
                        (SRAM_FEA  << 16) | (SRAM_FED))
#define SRAM_FLASH_CFG_A SRAM_FLASH_CFG
#define SRAM_FLASH_CFG_C SRAM_FLASH_CFG
    
/* SRAM Mac timing (based on 199 MHz clock) */

#define SRAM_MRWA 0x4C /* Mac R/W Assert */
#define SRAM_MEA  0x52 /* Mac Enable Assert */
#define SRAM_MRWD 0x0F /* Mac R/W Deassert */
#define SRAM_MED  0x07 /* Mac Enable Deassert */
#define SRAM_MAC_CFG ((SRAM_MRWA << 24) | (SRAM_MRWD << 8) | \
                      (SRAM_MEA  << 16) | (SRAM_MED))
#define SRAM_MAC_CFG_A 0x454B0E07
#define SRAM_MAC_CFG_C SRAM_MAC_CFG

    
/* SDRAM Config */

#define SDRAM_EN_FTST   0 /* Enable FIFO RAM test */
#define SDRAM_EN_INT    0 /* Enable Parity Interrupt */
#define SDRAM_EN_IRQ    0 /* Select IRQ */
#define SDRAM_EN_FIQ    0 /* Select FIQ */
#define SDRAM_EN_CPE    0 /* Enable StrongARM Core parity error */
#define SDRAM_EN_PCIPE  0 /* Enable PCI parity error */
#define SDRAM_EN_UEPE   0 /* Enable Microengine parity error */
#define SDRAM_GENPAR    0 /* Enable parity generation */
#define SDRAM_EVENPAR   0 /* Generate even parity */
#define SDRAM_TF_BENDN  0 /* 1 = big endian mode */
#define SDRAM_INIT      1 /* Initialize SDRAM unit */
#define SDRAM_CSR ((SDRAM_EN_FTST   << 15) | (SDRAM_EN_INT    << 10) | \
                   (SDRAM_EN_IRQ    <<  9) | (SDRAM_EN_FIQ    <<  8) | \
                   (SDRAM_EN_CPE    <<  6) | (SDRAM_EN_PCIPE  <<  5) | \
                   (SDRAM_EN_UEPE   <<  4) | (SDRAM_GENPAR    <<  3) | \
                   (SDRAM_EVENPAR   <<  2) | (SDRAM_TF_BENDN  <<  1) | \
                   (SDRAM_INIT))
    
#define SDRAM_EN_INT_BIT   10

#define SDRAM_RFRSH         0x0612 /* Refresh Count */
#define SDRAM_BURSTL        0x8    /* Burst Length */
#define SDRAM_CASL          0x2    /* CAS Latency */
#ifdef SPECTACLE_ISLAND
#define SDRAM_ROW_ADR_WIDTH 0xD    /* Row Address Width */
#define SDRAM_COL_ADR_WIDTH 0x9    /* Column Address Width */
#else
#define SDRAM_ROW_ADR_WIDTH 0xC    /* Row Address Width */
#define SDRAM_COL_ADR_WIDTH 0x8    /* Column Address Width */
#endif
#define SDRAM_MEMCTL0 ((SDRAM_RFRSH << 16) | (SDRAM_BURSTL << 12) | \
                       (SDRAM_CASL  <<  8) | (SDRAM_ROW_ADR_WIDTH << 4) | \
                       (SDRAM_COL_ADR_WIDTH))

#define SDRAM_MEMCTL0_A  (0x05660000 | (SDRAM_MEMCTL0 & 0x0000FFFF))
#define SDRAM_MEMCTL0_C  (0x07160000 | (SDRAM_MEMCTL0 & 0x0000FFFF))


#define SDRAM_TRWT     0x1 /* Read/Write turnaround time */
#define SDRAM_TDPL     0x2 /* Data In to Precharge time */
#define SDRAM_TDQZ     0x3 /* DQM Data Out Disable latency */
#define SDRAM_TRC      0x6 /* Bank Cycle time */
#define SDRAM_TRRD     0x1 /* Bank to Bank delay time */
#define SDRAM_TRCD     0x1 /* RAS to CAS delay */
#define SDRAM_TRASMIN  0x4 /* Active Command Period */
#define SDRAM_TRP      0x1 /* Precharge Time */
#define SDRAM_MEMCTL1 ((SDRAM_TRWT << 28) | (SDRAM_TDPL << 24) | \
                       (SDRAM_TDQZ << 20) | (SDRAM_TRC  << 16) | \
                       (SDRAM_TRRD << 12) | (SDRAM_TRCD <<  8) | \
                       (SDRAM_TRASMIN << 4) | (SDRAM_TRP))
#define SDRAM_MEMCTL1_A SDRAM_MEMCTL1
#define SDRAM_MEMCTL1_C 0x12382252

#define SDRAM_TRSC       0x1     /* Mode register set delay */
#define SDRAM_INIT_RFRSH 0x2     /* Initialization Refresh Count */
#define SDRAM_INIT_DLY   0x26DF  /* Initialization Delay */
#define SDRAM_MEMINIT ((SDRAM_TRSC << 20) | (SDRAM_INIT_RFRSH << 16) | \
                       (SDRAM_INIT_DLY))
#define SDRAM_MEMINIT_A  0x00122292
#define SDRAM_MEMINIT_C  0x00122D5F

/* Memory space definition */

#define FLASH_PHYS_BASE   0x00000000 /* Physical address of start of FLASH */

#define SRAM_BASE         0x10000000 /* Base of SRAM R/W area */
#define SRAM_RL_BASE      0x12000000 /* Read-lock area */
#define SRAM_WU_BASE      0x14000000 /* Write-unlock area */
#define SRAM_CU_BASE      0x16000000 /* CAM Unlock area */
#define SRAM_BWC_BASE     0x18000000 /* Bit-Write-Clear area */
#define SRAM_BWS_BASE     0x18800000 /* Bit-Write-Set area */
#define SRAM_BTC_BASE     0x19000000 /* Bit Test & Clear area */
#define SRAM_BTS_BASE     0x19800000 /* Bit Test & Set area */
#define SRAM_PUSH_BASE    0x20000000 /* Push command area */
#define SRAM_PUSH_BASE_Q0 0x20000000 /* Push command area: queue 0 */
#define SRAM_PUSH_BASE_Q1 0x20800000 /* Push command area: queue 1 */
#define SRAM_PUSH_BASE_Q2 0x21000000 /* Push command area: queue 2 */
#define SRAM_PUSH_BASE_Q3 0x21800000 /* Push command area: queue 3 */
#define SRAM_PUSH_BASE_Q4 0x22000000 /* Push command area: queue 4 */
#define SRAM_PUSH_BASE_Q5 0x22800000 /* Push command area: queue 5 */
#define SRAM_PUSH_BASE_Q6 0x23000000 /* Push command area: queue 6 */
#define SRAM_PUSH_BASE_Q7 0x23800000 /* Push command area: queue 7 */
#define SRAM_POP_BASE     0x24000000 /* Pop command area */
#define SRAM_POP_BASE_Q0  0x24000000 /* Pop command area: queue 0 */
#define SRAM_POP_BASE_Q1  0x24800000 /* Pop command area: queue 1 */
#define SRAM_POP_BASE_Q2  0x25000000 /* Pop command area: queue 2 */
#define SRAM_POP_BASE_Q3  0x25800000 /* Pop command area: queue 3 */
#define SRAM_POP_BASE_Q4  0x26000000 /* Pop command area: queue 4 */
#define SRAM_POP_BASE_Q5  0x26800000 /* Pop command area: queue 5 */
#define SRAM_POP_BASE_Q6  0x27000000 /* Pop command area: queue 6 */
#define SRAM_POP_BASE_Q7  0x27800000 /* Pop command area: queue 7 */
#define SRAM_CSR_BASE     0x38000000 /* SRAM CSRs */
#define IXP1200_SRAM_CSR          0x38000000 /* SRAM CSR */
#define IXP1200_SRAM_SLOW_CONFIG  0x38000020 /* SRAM Flash/Mac cycle times */
#define IXP1200_SRAM_FLASH_CONFIG 0x38000024 /* Flash timing */
#define IXP1200_SRAM_MAC_CONFIG   0x38000028 /* Mac timing */
#define SRAM_IO_BASE      0x38400000 /* Async I/O (MAC) space */

#define PCI_CSR_BASE      0x42000000 /* Local config / CSR space */
#define PCI_CONFIG1_BASE  0x52000000 /* Config cycle 1 space */
#define PCI_CONFIG0_BASE  0x53000000 /* Config cycle 0 space */
#define PCI_IO_BASE       0x54000000 /* I/O cycle space */
#define PCI_MEM_BASE      0x60000000 /* Mem cycle space */

#define IXP1200_SYS_REG_BASE 0x90000000 /* Base of IXP1200 System registers */
#define IXP1200_PLL_CFG      0x90000C00 /* PLL Configuration Reg */
#define IXP1200_FIQ          0x90001000 /* FIQ interrupt register */
#define IXP1200_IRQ          0x90001400 /* IRQ interrupt register */
#define IXP1200_GPIO_EN      0x90001800 /* Address of GPIO_EN register */
#define IXP1200_GPIO_DATA    0x90001C00 /* Address of GPIO_DATA register */
#define IXP1200_RTC_DIV      0x90002000 /* Real Time Clock Divider & Int enable */
#define IXP1200_UART_CR      0x90003800 /* UART Control Register */
#define IXP1200_SYS_REG_SIZE 0x00003C00

/*
 * Area used to flush caches.
 * HW area that doesn't cause AMBA transactions
 */
#define CACHE_FLUSH_BASE 0xA0000000 /* Cache Flush area */
#define MINICACHE_FLUSH_BASE  (CACHE_FLUSH_BASE + 0x00100000)

#define UENG_CSR_BASE    0xB0000000 /* MicroEngine CSR Base */
#define UENG_CSR_SIZE    0x00003000 /* MicroEngine CSR size */
#define UENG_XFER_BASE   0xB0004000 /* MicroEngine Xfer Reg Base */
#define UENG_XFER_SIZE   0x00003000 /* MicroEngine Xfer Reg size */

#define FBI_CSR_BASE     0xB0040000 /* FBI CSR Base, size == 1 page */
#define IXP1200_IREG      0xB00401E0 /* FBI Interrupt Register: IREG */

#define FBI_SCRATCH_BASE 0xB0044000 /* Scratchpad Base */
#define FBI_SCRATCH_SIZE 0x00001000 /* Scratchpad size */

#define SDRAM_PHYS_BASE  0xC0000000 /* Physical address of start of SDRAM */
#define SDRAM_PF_BASE    0xD0000000 /* SDRAM Pre-Fetch Base */
#define SDRAM_CSR_BASE   0xFF000000 /* SDRAM CSR Base */
#define IXP1200_SDRAM_CSR     0xFF000000 /* SDRAM CSR */
#define IXP1200_SDRAM_MEMCTL0 0xFF000004 /* SDRAM timing info */
#define IXP1200_SDRAM_MEMCTL1 0xFF000008 /* SDRAM timing info */
#define IXP1200_SDRAM_MEMINIT 0xFF00000C /* SDRAM initialization timing */

/*
 * SDRAM_VIRT_OFFSET is the physical address of SDRAM corresponding
 * to virtual address 0
 */

#define SDRAM_VIRT_OFFSET (SDRAM_PHYS_BASE + RESERVED_LOW_MEM)

/*
 * PCI REGISTERS
 */

/* PCI Reset register control bit values*/

#define RESET_MICROENGINE   0x0000003F   /* Reset 6 microengines */
#define RESET_EXTERNAL      0x00008000   /* External interrupt (output) */
#define RESET_FULL          0xFFFFFFFF   /* Full chip reset */

/* Interrupt addresses */

#define IXP1200_IRQSTATUS      0x42000180 /* IRQ Status */
#define IXP1200_IRQRAWSTATUS   0x42000184 /* IRQ Raw Status */
#define IXP1200_IRQENABLE      0x42000188 /* IRQ Enable R/O */
#define IXP1200_IRQENABLE_SET  IXP1200_IRQENABLE /* IRQ Enable set W/O */
#define IXP1200_IRQENABLECLEAR 0x4200018C /* IRQ Enable clear */
#define IXP1200_IRQ_SOFT       0X42000190

#define IXP1200_FIQSTATUS      0x42000280 /* FIQ Status */
#define IXP1200_FIQRAWSTATUS   0x42000284 /* FIQ Raw Status */
#define IXP1200_FIQENABLE      0x42000288 /* FIQ Enable R/O */
#define IXP1200_FIQENABLE_SET  IXP1200_FIQENABLE /* FIQ Enable set W/O */
#define IXP1200_FIQENABLECLEAR 0x4200028C /* FIQ Enable clear */
#define IXP1200_FIQ_SOFT       0X42000290

#define IXP1200_PCI_CSR_BASE 0x42000000 /* Base of PCI Unit CSR's */

#define IXP1200_PCI_VENDOR_ID  FIX_ADDR_16(IXP1200_PCI_CSR_BASE + 0x000)
#define IXP1200_PCI_DEVICE_ID  FIX_ADDR_16(IXP1200_PCI_CSR_BASE + 0x002)
#define IXP1200_PCI_MEM_BAR           (IXP1200_PCI_CSR_BASE + 0x010)
#define IXP1200_PCI_IO_BAR            (IXP1200_PCI_CSR_BASE + 0x014)
#define IXP1200_PCI_DRAM_BAR          (IXP1200_PCI_CSR_BASE + 0x018)
#define IXP1200_OUTB_INT_STATUS       (IXP1200_PCI_CSR_BASE + 0x030)
#define IXP1200_OUTB_INT_MASK         (IXP1200_PCI_CSR_BASE + 0x034)
#define IXP1200_PCI_INT_LAT           (IXP1200_PCI_CSR_BASE + 0x03c)
#define IXP1200_I20_INB_FIFO          (IXP1200_PCI_CSR_BASE + 0x040)
#define IXP1200_I20_OUTB_FIFO         (IXP1200_PCI_CSR_BASE + 0x044)
#define IXP1200_MAILBOX0              (IXP1200_PCI_CSR_BASE + 0x050)
#define IXP1200_MAILBOX1              (IXP1200_PCI_CSR_BASE + 0x054)
#define IXP1200_MAILBOX2              (IXP1200_PCI_CSR_BASE + 0x058)
#define IXP1200_MAILBOX3              (IXP1200_PCI_CSR_BASE + 0x05c)
#define IXP1200_DOORBELL              (IXP1200_PCI_CSR_BASE + 0x060)
#define IXP1200_DOORBELL_SETUP        (IXP1200_PCI_CSR_BASE + 0x064)
#define IXP1200_CAP_PTR_EXT           (IXP1200_PCI_CSR_BASE + 0x070)
#define IXP1200_PWR_MGMT              (IXP1200_PCI_CSR_BASE + 0x074)
#define IXP1200_RESET                 (IXP1200_PCI_CSR_BASE + 0x07C)
#define IXP1200_CHAN_1_BYTE_COUNT     (IXP1200_PCI_CSR_BASE + 0x080)
#define IXP1200_CHAN_1_PCI_BAR        (IXP1200_PCI_CSR_BASE + 0x084)
#define IXP1200_CHAN_1_DRAM_ADDR      (IXP1200_PCI_CSR_BASE + 0x088)
#define IXP1200_CHAN_1_DESC_PTR       (IXP1200_PCI_CSR_BASE + 0x08C)
#define IXP1200_CHAN_1_CONTROL        (IXP1200_PCI_CSR_BASE + 0x090)
#define IXP1200_DMA_INF_MODE          (IXP1200_PCI_CSR_BASE + 0x09C)
#define IXP1200_CHAN_2_BYTE_COUNT     (IXP1200_PCI_CSR_BASE + 0x0A0)
#define IXP1200_CHAN_2_PCI_BAR        (IXP1200_PCI_CSR_BASE + 0x0A4)
#define IXP1200_CHAN_2_DRAM_ADDR      (IXP1200_PCI_CSR_BASE + 0x0A8)
#define IXP1200_CHAN_2_DESC_PTR       (IXP1200_PCI_CSR_BASE + 0x0AC)
#define IXP1200_CHAN_2_CONTROL        (IXP1200_PCI_CSR_BASE + 0x0B0)
#define IXP1200_CSR_BASE_ADDR_MASK    (IXP1200_PCI_CSR_BASE + 0x0F8)
#define IXP1200_DRAM_BASE_ADDR_MASK   (IXP1200_PCI_CSR_BASE + 0x100)
#define IXP1200_I20_INB_FLIST_HPTR    (IXP1200_PCI_CSR_BASE + 0x120)
#define IXP1200_I20_INB_PLIST_TPTR    (IXP1200_PCI_CSR_BASE + 0x124)
#define IXP1200_I20_OUTB_PLIST_HPTR   (IXP1200_PCI_CSR_BASE + 0x128)
#define IXP1200_I20_OUTB_FLIST_TPTR   (IXP1200_PCI_CSR_BASE + 0x12C)
#define IXP1200_I20_INB_FLIST_CNT     (IXP1200_PCI_CSR_BASE + 0x130)
#define IXP1200_I20_OUTB_PLIST_CNT    (IXP1200_PCI_CSR_BASE + 0x134)
#define IXP1200_I20_INB_PLIST_CNT     (IXP1200_PCI_CSR_BASE + 0x138)
#define IXP1200_SA_CONTROL            (IXP1200_PCI_CSR_BASE + 0x13C)
#define IXP1200_PCI_ADDR_EXT          (IXP1200_PCI_CSR_BASE + 0x140)
#define IXP1200_PREFETCH_RANGE        (IXP1200_PCI_CSR_BASE + 0x144)
#define IXP1200_DBELL_PCI_MASK        (IXP1200_PCI_CSR_BASE + 0x150)
#define IXP1200_DBELL_SA_MASK         (IXP1200_PCI_CSR_BASE + 0x154)

#define IXP1200_SA_CONTROL_BEO (1 << 18) /* PCI Big Endian Byte Enable Out */
#define IXP1200_SA_CONTROL_DEO (1 << 17) /* PCI Big Endian Data Enable Out */
#define IXP1200_SA_CONTROL_BEI (1 << 16) /* PCI Big Endian Byte Enable In */
#define IXP1200_SA_CONTROL_DEI (1 << 15) /* PCI Big Endian Data Enable In */

#define IXP1200_SA_CONTROL_PNR (1 << 9)  /* PCI Not Reset bit of SA_CONTROL */
#define IXP1200_SA_CONTROL_PCF (1 << 31) /* PCI Centrl Function bit */

/* 16 bit PCI registers */

#define IXP1200_PCI_COMMAND FIX_ADDR_16(IXP1200_PCI_CSR_BASE + 0x04)

/* Define PCI_COMMAND bits */

#define IXP1200_PCI_COMMAND_IOSE (1 << 0) /* I/O Space Enable */
#define IXP1200_PCI_COMMAND_MSE  (1 << 1) /* Mem Space Enable */
#define IXP1200_PCI_COMMAND_ME   (1 << 2) /* Master Enable */
#define IXP1200_PCI_COMMAND_MWI  (1 << 4) /* Master Write And Invalidate Enable */
#define IXP1200_PCI_COMMAND_PEC  (1 << 6) /* Parity Error Control */
#define IXP1200_PCI_COMMAND_SE   (1 << 8) /* SERR# enable */

/*
 * Interrupt Controller definitions
 */

#define IXP1200_INT_NUM_LEVELS	 1
#define IXP1200_INT_NUM_VECTORS  32

#define INT_VEC_DPE           31	/* Detect Parity Error */
#define INT_VEC_RTA           30	/* Received Target Abort */
#define INT_VEC_RMA           29	/* Received Master Abort */
#define INT_VEC_DPED          28	/* Data Parity Error Detected */

#define INT_VEC_DTE           27	/* Discard Timer Expired */
#define INT_VEC_reserved_26   26
#define INT_VEC_IIP           25	/* I20 inbound post_list */
#define INT_VEC_SDPAR         24	/* SDRAM parity */

#define INT_VEC_RSERR         23	/* Received SERR */
#define INT_VEC_SB            22	/* Start BIST */
#define INT_VEC_DMA2NB        21	/* DMA2 not busy */
#define INT_VEC_DMA1NB        20	/* DMA1 not busy */

#define INT_VEC_reserved_19   19
#define INT_VEC_PIL           18	/* pci_irq_l */
#define INT_VEC_DMA2          17	/* DMA channel 2 */
#define INT_VEC_DMA1          16	/* DMA channel 1 */

#define INT_VEC_DFH           15	/* Doorbell from host */
#define INT_VEC_UART          14	/* UART unit IRQ */
#define INT_VEC_SDRAM         13	/* SDRAM unit IRQ */
#define INT_VEC_RTC           12    /* Real Time Clock */

#define INT_VEC_SRAM          11	/* SRAM unit IRQ */
#define INT_VEC_UENG          10	/* Microengine IRQ */
#define INT_VEC_CINT           9	/* CINT pin IRQ */
#define INT_VEC_reserved_08    8

#define INT_VEC_T4             7	/* Timer 4 */
#define INT_VEC_T3             6	/* Timer 3 */
#define INT_VEC_T2             5	/* Timer 2 */
#define INT_VEC_T1             4	/* Timer 1 */

#define INT_VEC_reserved_03    3
#define INT_VEC_reserved_02    2
#define INT_VEC_SI             1	/* Soft interrupt */
#define INT_VEC_reserved_00    0

#define IXP1200_INT_CTRL_MASK 0xFBF780F2  /* Interrupts controllable from */
                                         /* the PCI registers */

/* definitions for IXP1200 UART */

#define UART_XTAL_FREQ                3686400    /* UART baud rate clock freq */
#define N_UART_CHANNELS               1          /* Total number UART channels */
#define N_SIO_CHANNELS         (N_UART_CHANNELS) /* Total number SIO channels */
#define N_IXP1200_UART_CHANNELS (N_UART_CHANNELS) /* Total number IXP1200 chan */

/*
 * Definitions for the AMBA Timer:
 * There are four timers. Timer four can be used as a watchdog. We will use
 * Timer-1 for the system clock, Timer-2 for the aux clock, and Timer-3 for
 * the timer stamp clock. The clocks are run from the system clock, not
 * divided.
 */

#define AMBA_TIMER_BASE		0x42000300	/* Address of base of timer */

#define AMBA_TIMER_SYS_TC_DISABLE	(TC_DISABLE | TC_PERIODIC)
#define AMBA_TIMER_SYS_TC_ENABLE	(TC_ENABLE  | TC_PERIODIC)
#define AMBA_TIMER_AUX_TC_DISABLE	(TC_DISABLE | TC_PERIODIC)
#define AMBA_TIMER_AUX_TC_ENABLE	(TC_ENABLE  | TC_PERIODIC)

#define SYS_TIMER_CLK_DEFAULT	 	(199070000)	/* Frequency of counter/timer*/
#define AUX_TIMER_CLK_DEFAULT		(199070000) /* Frequency of counter/timer*/
#define SYS_TIMER_CLK_A             (176950000)
#define SYS_TIMER_CLK_C             (232240000)

#define SYS_TIMER_CLK	 	(sys_timer_clk)	/* Frequency of counter/timer*/
#define AUX_TIMER_CLK		(sys_timer_clk) /* Frequency of counter/timer*/

#define AMBA_RELOAD_TICKS	0		/* no overhead reloading timer*/

#define SYS_TIMER_CLEAR(x)	(AMBA_TIMER_T3CLEAR(x)) /* sys Clk is timer 1 */
#define SYS_TIMER_CTRL(x)	(AMBA_TIMER_T3CTRL(x))
#define SYS_TIMER_LOAD(x)	(AMBA_TIMER_T3LOAD(x))
#define SYS_TIMER_VALUE(x)	(AMBA_TIMER_T3VALUE(x))
#define AUX_TIMER_CLEAR(x)	(AMBA_TIMER_T2CLEAR(x)) /* aux Clk is timer 2 */
#define AUX_TIMER_CTRL(x)	(AMBA_TIMER_T2CTRL(x))
#define AUX_TIMER_LOAD(x)	(AMBA_TIMER_T2LOAD(x))
#define AUX_TIMER_VALUE(x)	(AMBA_TIMER_T2VALUE(x))

#define SYS_TIMER_INT_LVL 	(INT_VEC_T3)
#define AUX_TIMER_INT_LVL	(INT_VEC_T2)

/* Definitions for driving LED on board */

#define LED_MEM_MAPPED

#ifdef LED_MEM_MAPPED
#define IXP1200_LED_ADDR   (0x38400000+0x108000)  /* Address of LED display */
#else
#define IXP1200_LED_ADDR   IXP1200_GPIO_DATA
#endif

/*
 * Maximum clock rates depend upon CPU power and work load of application.
 * They will also critically depend on whether caches are enabled.
 * The values below are minimum and maximum allowed by the hardware/software.
 * The counter is 24-bit and we don't accept values that would mean a zero
 * reload value as we don't know what that will do.
 * So:
 * min frequency = roundup(clock_rate/(max_counter_value))
 * max frequency = rounddown(clock_rate/(min_counter_value))
 * i.e. SYS_CLK_RATE_MAX = SYS_TIMER_CLK
 *      AUX_CLK_RATE_MAX = AUX_TIMER_CLK
 *
 * However, we must set maxima that are sustainable on a running
 * system. The values below have been chosen so that there is a
 * reasonable margin and the BSP passes the test suite.
 */

#define SYS_CLK_RATE_MIN ((SYS_TIMER_CLK+0xFFFFFF)/0x1000000)
#define AUX_CLK_RATE_MIN ((AUX_TIMER_CLK+0xFFFFFF)/0x1000000)
#define SYS_CLK_RATE_MAX 60000
#define AUX_CLK_RATE_MAX 60000

/* Flash memory configuration - see config.h for more details */

#ifdef INCLUDE_FLASH
#define FLASH_WIDTH     4		/* 32-bit wide (two 16-bit wide parts)*/
#endif /* INCLUDE_FLASH */

/* GPIO registers control bit values */

#define GPIO_OWN_CCCC    (0<<4)        /* Core owns all GPIO */
#define GPIO_OWN_CCCF    (1<<4)        /* Fbi owns GPIO[0] */
#define GPIO_OWN_FFFC    (2<<4)        /* Fbi owns GPIO[3:1] */
#define GPIO_OWN_FFFF    (3<<4)        /* Fbi owns GPIO[3:0] */
#define GPIO_OUTPUT(mask) (mask)       /* Which GPIO's are outputs */

#ifdef INCLUDE_PCI
/*
 * PCI definitions
 * 
 * First, defines for generic pciIoMapLib.c code
 * 
 * Cache Line Size - in number of 32-bit words
 * It might be thought that we would set this to (_CACHE_ALIGN_SIZE/4), but,
 * in fact, the areas of memory mapped for use either by PCI I/O and
 * PCI Configuration  and the area of (ARM) memory used by the Ethernet
 * controller are marked as non-cacheable, so we should set it to zero.
 */

#define PCI_CLINE_SZ    0

/*
 * Latency Timer value
 * A sensible value for this depends what we are using PCI for.
 * (Rather arbitrarily) leave it at 0 (the reset default value)
 */

#define PCI_LAT_TIMER           0x0


#define SWAP16(x) ((((x) << 8) | ((x) >> 8)) & 0xFFFF)
#define SWAP32(x)               \
  (((x) << 24) |                \
  (((x) & 0x0000FF00) << 8) |   \
  (((x) & 0x00FF0000) >> 8) |   \
  (((unsigned int)(x)) >> 24))

#if (_BYTE_ORDER == _BIG_ENDIAN)
#define SWAP32PCIFUNC(x) (swap32(x))
#define SWAP16PCIFUNC(x) (swap16(x))
#define SWAP32PCI(x)     (SWAP32(x))
#else
#define SWAP32PCIFUNC(x) (x)
#define SWAP16PCIFUNC(x) (x)
#define SWAP32PCI(x)     (x)
#endif

#define PCI_IN_BYTE(x)      *(UINT8 *) (x)
#define PCI_IN_WORD(x)      (SWAP16PCIFUNC(sysRead16((UINT16 *)(x))))
#define PCI_IN_LONG(x)      (SWAP32PCIFUNC(*(UINT32 *)(x)))
#define PCI_OUT_BYTE(x,y)   *(UINT8 *) (x) = (UINT8)  y
#define PCI_OUT_WORD(x,y)   sysWrite16((UINT16 *)(x), SWAP16PCIFUNC((UINT16)(y)))
#define PCI_OUT_LONG(x,y)   *(UINT32 *)(x) = SWAP32PCIFUNC((UINT32) (y))


/*
 * Dummy interrupt number for pciIntConnect(): must be passed a real
 * interrupt vector number. This allows for non-linear mappings
 * between interrupts, slots etc.
 */

#define INT_NUM_IRQ0    0


/* memory map as seen by the CPU on the local bus */
#ifdef INCLUDE_EXTRA_PCI_MAPPING
#define CPU_PCI_MEM_ADRS    PCI_MEM_BASE /* PCI memory space base */
#define CPU_PCI_MEM_SIZE    0x08000000   /* 128 Mbytes */
#endif

#define CPU_PCI_IO_ADRS     PCI_IO_BASE  /* PCI I/O space base */
#define CPU_PCI_IO_SIZE     0x00010000   /* 64 kbytes */

#define CPU_PCI_CNFG_ADRS   PCI_CONFIG0_BASE  /* type 0 PCI config space */
#define CPU_PCI_CNFG_SIZE   0x00C00000        /* 12 Mbytes */

#define CPU_PCI_CNFG_1_ADRS PCI_CONFIG1_BASE  /* type 1 PCI config space */
#define CPU_PCI_CNFG_1_SIZE 0x01000000        /* 16 Mbytes */


/* PCI view of PCI I/O space for PCI devices */

#define PCI_IO_ADRS         0x00000000  /* base of PCI I/O address */

/* This is where physical SDRAM[0] appears on PCI bus */
#define PCI2DRAM_BASE_ADRS  0x00000000  /* memory seen from PCI bus */


/* PCI bus resources */

#define PCI_IO_SIZE     0x80            /* PCI I/O size */

/* PCI device 0: slot I/O 4 */

#define PCI_MEM_ADR0            0x00            /* Memory base for CSR */
#define PCI_IO_ADR0             PCI_IO_ADRS     /* I/O base for CSR 32Bytes */
#define PCI_IO_SIZE0            PCI_IO_SIZE     /* I/O size for CSR */
#define PCI_INT_LVL0            INT_VEC_PIL     /* Interrupt level */
#define PCI_INT_VEC0            IVEC_TO_INUM(PCI_INT_LVL0) /* Interrupt vector */

/* PCI device 1: slot I/O 3 */

#define PCI_MEM_ADR1            0x00
#define PCI_IO_ADR1             PCI_IO_ADRS
#define PCI_IO_SIZE1            PCI_IO_SIZE
#define PCI_INT_LVL1            INT_VEC_PIL
#define PCI_INT_VEC1            IVEC_TO_INUM(PCI_INT_LVL1)

/* PCI device 2: slot I/O 2 */

#define PCI_MEM_ADR2            0x00
#define PCI_IO_ADR2             PCI_IO_ADRS
#define PCI_IO_SIZE2            PCI_IO_SIZE
#define PCI_INT_LVL2            INT_VEC_PIL
#define PCI_INT_VEC2            IVEC_TO_INUM(PCI_INT_LVL2)

/* PCI device 3: slot I/O 1 */

#define PCI_MEM_ADR3            0x00
#define PCI_IO_ADR3             PCI_IO_ADRS
#define PCI_IO_SIZE3            PCI_IO_SIZE
#define PCI_INT_LVL3            INT_VEC_PIL
#define PCI_INT_VEC3            IVEC_TO_INUM(PCI_INT_LVL3)

/*
 * N.B. Only the following values have been tested. In particular, no
 * testing has been performed with more than one PCI bus, i.e. Type 1
 * configuration.
*/

#define PCI_MAX_DEV    12
#define PCI_MAX_BUS     1       /* Only one bus */


/* Allocate Device Space (Offsets) for PCI devices */

#define LAN_DEV_SPACE       0
#define PCI_IO_LN_ADRS      (PCI_IO_ADRS + LAN_DEV_SPACE)


/* Allocated base address of h/w devices as seen from CPU */

#define LAN_BASE_ADRS       (CPU_PCI_IO_ADRS + LAN_DEV_SPACE)


/* Create configuration address */

#define MAKE_CFG_ADDR(bus, device, func, offset) ((bus) ? \
    ((UINT32)((bus & 0xFF) << 16) | (UINT32)((device & 0x1F) << 11) | \
     (UINT32)((func & 0x7) << 8)  | 0x01) : \
    ((UINT32)(1 << ((device & 0x0F) + 11)) | (UINT32)((func & 0x7) << 8) | \
     (UINT32) (offset & 0xFC)))

#endif /* ifdef INCLUDE_PCI */

#endif	/* INCixp1200ebh */
