/* romInit.s - Intel ixp1200eb ROM initialisation module */

/* Copyright 1996-1998 Wind River Systems, Inc.; Copyright 1999 Intel Corp. */
	.data
	.globl	_copyright_wind_river
	.long	_copyright_wind_river

.nolist
	
/*
modification history
--------------------
01c,07Mar00,jdg  Added support for B0 revision ixp1200
01b,13aug99,jdg  Changed name from vbsa1200 to ixp1200eb
01a,08apr99,jdg  created from 01a of brutus
*/

/*
DESCRIPTION
This module contains the entry code for VxWorks images that start
running from ROM, such as 'bootrom' and 'vxWorks_rom'.
The entry point, romInit(), is the first code executed on power-up.
It performs the minimal setup needed to call the
generic C routine romStart() with parameter BOOT_COLD.

The routine romInit() masks interrupts in the processor, initialises
the MMU, sets the initial stack pointer (to STACK_ADRS which is defined
in configAll.h) and initialises system hardware including configuring
the DRAM controller. Other hardware and device initialisation is
performed later in the sysHwInit routine in sysLib.c.  The MMU
initialisation is more complex than for most BSPs, as we need to enable
the MMU, setting up some page tables, in order to map DRAM from zero
upwards, over the exception vectors.

The routine sysToMonitor() jumps to a location after the beginning of
romInit, (defined by ROM_WARM_ADRS) to perform a "warm boot". This
entry point allows a parameter to be passed to romStart().
It first has to disable the MMU, though, as romInit will disable it
during initialization.

The routines in this module don't use the "C" frame pointer %r11@ ! or
establish a stack frame.

SEE ALSO:
.I "ARM Architecture Reference Manual,"
*/

#define	_ASMLANGUAGE
#include "vxWorks.h"
#include "regs.h"
#include "arch/arm/arm.h"
#include "arch/arm/mmuArmLib.h"
#include "sysLib.h"
#include "config.h"

.list
	
/* Exported internal functions */

	.globl	_romInit	 /* start of system code */
	.globl	_sdata		 /* start of data */

/* externals */

	.extern	_romStart	/* system initialisation routine */

_sdata:
	.asciz	"start of data"
	.balign	4


	.text
	.balign 4

/*******************************************************************************
*
* romInit - entry point for VxWorks in ROM
*
*
* romInit
*     (
*     int startType	/@ only used by 2nd entry point @/
*     )

* INTERNAL
* sysToMonitor examines the ROM for the first instruction and the string
* "Copy" in the third word so if this changes, sysToMonitor must be updated.
*/

_romInit:
cold:
	MOV	r0,#BOOT_COLD	/* fall through to warm boot entry */
warm:
	B	start

	/* copyright notice appears at beginning of ROM (in TEXT segment) */

	.ascii	"Copyright 1984-1998 Wind River Systems, Inc."
	.balign 4

start:
	/* disable interrupts in CPU and switch to SVC32 mode */

	MRS	r1, cpsr
	BIC	r1, r1, #MASK_MODE
	ORR	r1, r1, #MODE_SVC32 | I_BIT | F_BIT
	MSR	cpsr, r1

#ifdef USE_LEDS_FOR_DEBUG
	
	/* Initialize LED's */
	
	LDR	r1, L$_IXP1200_LED_ADDR
#ifndef LED_MEM_MAPPED
	
	/* 
	 * Set GPIO's to core ownership, outputs, and output a zero
	 * IXP1200_LED_ADDR is GPIO_DATA 
	 */
	
	MOV	r2, #(GPIO_OWN_CCCC | GPIO_OUTPUT(0xF))
	STR	r2, [r1,#-(IXP1200_LED_ADDR-IXP1200_GPIO_EN)]
#endif /* LED_MEM_MAPPED */
	MOV	r2, #0
	STR	r2, [r1]
#endif /* USE_LEDS_FOR_DEBUG */


	/*
	 * Check for Chip revision: read chip ID into r8
	 * 0:	A rev
	 * 1:	B rev
	 * 3:	C rev
	 */

	MRC	CP_MMU, 0, r8, c0, c0, 0
	AND	r8,r8,#0xF
	

	/*
	 * CPU INTERRUPTS DISABLED
	 *
	 * Set processor and MMU to known state as follows (we may have not
	 * been entered from a reset):
	 *
	 * MMU Control Register layout.
	 *
	 * bit
	 *  0 M 0 MMU disabled
	 *  1 A 0 Address alignment fault disabled, initially
	 *  2 C 0 Data cache disabled
	 *  3 W 0 Write Buffer disabled
	 *  4 P 1 Should Be One (PROG32 on earlier CPUs)
	 *  5 D 1 Should Be One (DATA32 on earlier CPUs)
	 *  6 L 1 Should Be One (Late abort on earlier CPUs)
	 *  7 B 0 Little-endian
	 *  8 S 0 System bit to zero } Modifies MMU protections, not really
	 *  9 R 1 ROM bit to one     } relevant until MMU switched on later.
	 * 10 F 0 Should Be Zero
	 * 11 Z 0 Should Be Zero (Branch prediction control on 810)
	 * 12 I 0 Instruction cache disabled
	 *
	 * Setup MMU Control Register
	 */

	MOV	r1, #MMU_INIT_VALUE
	MCR	CP_MMU, 0, r1, c1, c0, 0	/* Write to MMU CR */
	
	/*
	 * MMU is now off => addresses are physical addresses, no protection.
	 *
	 * If MMU was on before this, then we'd better hope it was set up
	 * for flat translation (at least of this code) or there will
	 * be problems. The next three instructions will still be
	 * fetched "translated".
	 *
	 * Set Process ID Register to zero, this effectively disables
	 * the process ID remapping feature.
	 */

	MOV	r1, #0
	MCR	CP_MMU, 0, r1, c13, c0, 0
	
	MRC	CP_MMU, 0, r7, c0, c0, 0
	AND	r7,r7,#0xF
	

	/*
	 * Set core clock speed in Power Manager PLL Configuration Register.
	 * This must be done before enabling clock switching - there are
	 * strong suspicions that it will have no effect otherwise.
	 */

	LDR	r1, L$_IXP1200_PPL_CFG	/* PLL Configuration Reg*/
	CMP     r8, #1
	MOVLT	r2, #CCCR_INIT_VALUE_A  /* 177 MHz for A3 */
#if 0
	MOVEQ	r2, #CCCR_INIT_VALUE    /* 200 MHz for B0 */
	MOVGT	r2, #CCCR_INIT_VALUE_C  /* 233 MHz for B0 */
#else
	MOVGE	r2, #CCCR_INIT_VALUE    /* 200 MHz for B0 */
#endif
	STR	r2, [r1]

	/*
	 * Put the CPU in its normal (fast) clock switching mode
	 * Note that this may prevent any kernel that a bootrom loads from
	 * changing the core clock.
	 */

	MCR	CP_MMU, 0, r14, c15, c1, 2

	/* Clear Debug Support features: VxWorks makes no use of these */

	MOV	r1, #0
	MCR	CP_MMU, 0, r1, c14, c3, 0	/* Clear DBCR */
	MCR	CP_MMU, 0, r1, c14, c8, 0	/* Clear IBCR */

	/*
	 * We would like to discard the contents of the Write-Buffer
	 * altogether, but there is no facility to do this. Failing that,
	 * we do not want any pending writes to happen at a later stage,
	 * so drain the Write-Buffer, i.e. force any pending writes to
	 * happen now.
	 */

	MCR	CP_MMU, 0, r1, c7, c10, 4	/* Drain write-buffer */

	/* Flush, (i.e. invalidate) all entries in all caches */

	MCR	CP_MMU, 0, r1, c7, c6, 0	/* Flush D-cache & mini-cache */
	MCR	CP_MMU, 0, r1, c7, c5, 0	/* Flush (inval.) all I-cache */

	/*
	 * On StrongARM, the next four instructions could still come
	 * from (what was in) the Instruction cache.
	 *
	 * The BSP makes no explicit use of Read Buffer, but make sure
	 * that nothing already in it affects us.
	 */

	MCR	CP_MMU, 0, r1, c9, c0, 0	/* Flush Read Buffer */


	/*
	 * Delay for a while to let the SDRAM clock PLL stabalize after
	 * the clock was changed above.
	 */
	
	MOV	r1, #0x40000
loop1:	SUBS	r1, r1, #1
	BNE	loop1


	/* 
	 * Program SRAM Controller
	 * Note that we need to change SRAM_FLASH_CONFIG *before* 
	 * SRAM_SLOW_CONFIG, otherwise, the timing parameters become 
	 * inconsistent 
	 */

	CMP	r8, #1
	ADRLT	r1, L$_SRAM_CONFIG_PARAMS_A
#if 0
	ADREQ	r1, L$_SRAM_CONFIG_PARAMS
	ADRGT	r1, L$_SRAM_CONFIG_PARAMS_C
#else
	ADRGE	r1, L$_SRAM_CONFIG_PARAMS
#endif
	LDMIA	r1, {r2-r6}	/* get parameters from table */
	STR	r3, [r2]	/* Write SRAM CSR */
	STR	r5, [r2,#+(IXP1200_SRAM_FLASH_CONFIG - IXP1200_SRAM_CSR)]
	STR	r6, [r2,#+(IXP1200_SRAM_MAC_CONFIG - IXP1200_SRAM_CSR)]
	STR	r4, [r2,#+(IXP1200_SRAM_SLOW_CONFIG - IXP1200_SRAM_CSR)]
	

	/* Bring other functional units out of reset */

	LDR	r1, L$_IXP1200_RESET
	MOV	r2, #RESET_MICROENGINE
	STR	r2, [r1]

	/* Set up the SDRAM 
	 * Note that the SDRAM CSR must be the last one set
	 * r7 contains GPIO's, bit 3 
	 */
	
	CMP     r8, #1
	ADRLT	r1, L$_SDRAM_CONFIG_PARAMS_A
#if 0
	ADREQ	r1, L$_SDRAM_CONFIG_PARAMS
	ADRGT	r1, L$_SDRAM_CONFIG_PARAMS_C
#else
	ADRGE	r1, L$_SDRAM_CONFIG_PARAMS
#endif
	LDMIA	r1, {r2-r6}	/* get parameters from table */
	STMIA	r2, {r4-r6}	/* store MEMCTL0, MEMCTL1, MEMINIT */
	STR	r3, [r2,#-(IXP1200_SDRAM_MEMCTL0 - IXP1200_SDRAM_CSR)]

#ifndef USE_LEDS_FOR_DEBUG
	
	/* Initialize LED's */
	
	LDR	r1, L$_IXP1200_LED_ADDR
#ifndef LED_MEM_MAPPED
	
	/* 
	 * Set GPIO's to core ownership, outputs, and output a zero 
	 * IXP1200_LED_ADDR is GPIO_DATA 
	 */
	MOV	r2, #(GPIO_OWN_CCCC | GPIO_OUTPUT(0xF))
	STR	r2, [r1,#-(IXP1200_LED_ADDR-IXP1200_GPIO_EN)]
#endif /* LED_MEM_MAPPED */
	MOV	r2, #0
	STR	r2, [r1]
#endif /* USE_LEDS_FOR_DEBUG */
#ifdef USE_LEDS_FOR_DEBUG
	LDR	r1, L$_IXP1200_LED_ADDR
	MOV	r2, #1
	STR	r2, [r1]
#endif /* USE_LEDS_FOR_DEBUG */

	/* 
	 * Enable MMU and map sdram to 0. This is tricky because
	 * we also have to remap the addresses we are running out of.
	 * First copy the appropriate instructions from here to SDRAM,
	 * then when the MMU comes on line, there will be the same
	 * code in SDRAM 
	 */

	/* 
	 * Copy two instructions from ROM to SDRAM, starting at
	 * copy_start 
	 */
	
	ADR	r1, copy_start
	LDMIA	r1, {r2-r3}
	
	/*
	 * The following two instructions implements:
	 * ADD	r1, r1, #SDRAM_VIRT_OFFSET
	 * but it must be broken into two instructions because otherwise
	 * the constant is too big
	 */

	ADD	r1, r1, #(SDRAM_VIRT_OFFSET & 0xff000000)
	ADD	r1, r1, #(SDRAM_VIRT_OFFSET & 0x00ff0000)
	
	STMIA	r1, {r2-r3}

	/* Get (physical) address of our Level 1 Page Table in the ROM. */
	
	LDR	r1, L$_romL1PTable
	MOV	r2, #0

	/* 
	 * Read the Stack address and the boot rom address before we
	 * remap the MMU, so that we don't have to copy them into SDRAM 
	 */
	
	LDR	r3, L$_rStrtInRom
	LDR	sp, L$_STACK_ADDR

	/* 
	 * Write to the Translation Table Base Register in the MMU, to
	 * make these the current set of page tables and flush TLBs. 
	*/

	MCR	CP_MMU, 0, r1, c2, c0, 0	/* Write the TTBR in MMU */
	MCR	CP_MMU, 0, r2, c8, c7, 0	/* Flush all I+D TLBs */

	MVN	r2, #0		                /* r0 <= 0xFFFFFFFF */
	MCR	CP_MMU, 0, r2, c3, c0, 0	/* Write the DACR in MMU */

	/* Enable the MMU */

	MRC	CP_MMU, 0, r1, c1, c0, 0	/* Read MMUCR */
	ORR	r1, r1, #MMUCR_M_ENABLE		/* Set MMU Enable bit */
	MCR	CP_MMU, 0, r1, c1, c0, 0	/* Write to MMU CR */
	
	/*
	 * We are now executing at the same virtual address, although
	 * the physical address is now in SDRAM.
	 * The caches are still switched off.
	 *
	 * Initialise the stack pointer to just before where the
	 * uncompress code, copied from ROM to RAM will run.
	 */

copy_start:	
	MOV	fp, #0			/* zero frame pointer */

	/* jump to C entry point in ROM: routine - entry point + ROM base */

	MOV	pc, r3		/* L$_rStrtInRom, r0 = boot type */
	
/******************************************************************************/

	.balign	4
	

	/*
	 * PC-relative-addressable pointers - LDR Rn,=sym was (is?) broken
	 * note "_" after "$" to stop preprocessor performing substitution
	 */

L$_rStrtInRom:
	.long	ROM_TEXT_ADRS + _romStart - _romInit

L$_STACK_ADDR:
	.long	STACK_ADRS

L$_IXP1200_PPL_CFG:	
	.long IXP1200_PLL_CFG

L$_SRAM_CONFIG_PARAMS:	
	.long IXP1200_SRAM_CSR	/* CSR address */
	.long SRAM_CSR
	.long SRAM_SLOW_CFG
	.long SRAM_FLASH_CFG
	.long SRAM_MAC_CFG

L$_SRAM_CONFIG_PARAMS_A:
	.long IXP1200_SRAM_CSR	/* CSR address */
	.long SRAM_CSR
	.long SRAM_SLOW_CFG_A
	.long SRAM_FLASH_CFG_A
	.long SRAM_MAC_CFG_A

L$_SRAM_CONFIG_PARAMS_C:
	.long IXP1200_SRAM_CSR	/* CSR address */
	.long SRAM_CSR
	.long SRAM_SLOW_CFG_C
	.long SRAM_FLASH_CFG_C
	.long SRAM_MAC_CFG_C

L$_SDRAM_CONFIG_PARAMS:
	.long IXP1200_SDRAM_MEMCTL0  /* SDRAM MEMCTL0 address */
	.long SDRAM_CSR
	.long SDRAM_MEMCTL0
	.long SDRAM_MEMCTL1
	.long SDRAM_MEMINIT

L$_SDRAM_CONFIG_PARAMS_A:
	.long IXP1200_SDRAM_MEMCTL0  /* SDRAM MEMCTL0 address */
	.long SDRAM_CSR
	.long SDRAM_MEMCTL0_A
	.long SDRAM_MEMCTL1_A
	.long SDRAM_MEMINIT_A

L$_SDRAM_CONFIG_PARAMS_C:
	.long IXP1200_SDRAM_MEMCTL0  /* SDRAM MEMCTL0 address */
	.long SDRAM_CSR
	.long SDRAM_MEMCTL0_C
	.long SDRAM_MEMCTL1_C
	.long SDRAM_MEMINIT_C
	
	
L$_IXP1200_RESET:	
	.long IXP1200_RESET

	/* IXP1200_LED_ADDR is either in memory or is the same as GPIO_DATA */
	
L$_IXP1200_LED_ADDR:	
	.long IXP1200_LED_ADDR
	
L$_IXP1200_GPIO_EN:
	.long IXP1200_GPIO_EN
	
	/* 
	 * The page table address needs to be a physical address, so
	 * remove the 08000000 offset that we are providing in the
	 * page table itself. 
	 */
	
L$_romL1PTable:
	.long	(ROM_TEXT_ADRS + romL1PTable - _romInit) & ~ROM_VIRT_OFFSET

	/*
	 * Page tables (actually section descriptors) to be used to
	 * remap machine so that RAM is contiguously arranged from 0
	 * upwards. 
	 *
	 * Each entry corresponds to a megabyte of virtual address space
	 * and is a section descriptor, containing the physical address
	 * for the megabyte of virtual address space.
	 *
	 * A Level 1 page table must be 16 kilobyte-aligned WHEN IT IS IN
	 * THE FLASH DEVICE. Note use of ".space" to get around problems 
	 * with ".org".
	 */

	.space	(0x4000 * \
		((. - _romInit + (ROM_TEXT_ADRS - ROM_BASE_ADRS) + 0x3fff) \
		/ 0x4000)) - (ROM_TEXT_ADRS - ROM_BASE_ADRS) - (. - _romInit)

romL1PTable:

	/* 
	 * First Map 128 Meg of SDRAM, starting after reserved area
	 * RAM_SECT_DESC is cacheable, OTHER_SECT_DESC is non-cacheable
	 * Map cache flush area (0xA0000000) as cacheable 
	 */
	
	.set	count, 0
	.rept	(0x080 - 0x000)
	.long	((count << 20)+SDRAM_VIRT_OFFSET) | RAM_SECT_DESC
	.set	count, count+1
	.endr

	/* Map 8 Meg of Flash */
	
	.rept	(0x088 - 0x080)
	.long	((count & 127) << 20) | OTHER_SECT_DESC
	.set	count, count+1
	.endr

	/* Map rest of memory flat */
	
	.rept	(0xa00 - 0x088)
	.long	(count << 20) | OTHER_SECT_DESC
	.set	count, count+1
	.endr

	/* Map cache flush area as cacheable */
	.rept	(0xb00 - 0xa00)
	.long	(count << 20) | RAM_SECT_DESC
	.set	count, count+1
	.endr

	/* Map rest of memory flat */
	.rept	(0xe00 - 0xb00)
	.long	(count << 20) | OTHER_SECT_DESC
	.set	count, count+1
	.endr

	/* Map E0000000 as invalid */
	.rept	(0xf00 - 0xe00)
	.long	(count << 20)
	.set	count, count+1
	.endr

	/* Map rest of memory flat */
	.rept	(0x1000 - 0xf00)
	.long	(count << 20) | OTHER_SECT_DESC
	.set	count, count+1
	.endr
