/* sysNetif.c - system network interface support library */

/* Copyright 1989-1999 Wind River Systems, Inc.  */

#include "copyright_wrs.h"

/*
modification history
--------------------
01q,12mar99,jkf  renamed boardResource and pciResources to dev specific.
01p,09mar99,sbs  moved sysEltIntEnable and sysEltIntDisable to
                 sysElt3c509End.c
                 corrected previous version number.
01o,08feb99,jkf  removed PCI_CFG_* definitions, now set in configAll.h
01n,01feb99,jkf  added support for AMD 7997x PCI card.
01m,26nov98,ms_  add support for end enabled elt3c509
01l,27aug98,dat  fixed conflicting symbols from if_fei.h and fei82557
01k,16apr98,cn   added import of feiIntConnect, feiEndIntConnect and 
		 feiEndIntDisconnect and their initialisation.
01g,31mar98,cn   Added enhanced network driver support, changed BOARD_INFO 
		 in FEI_BOARD_INFO if INCLUDE_FEI_END is defined.
01f,17mar98,sbs  moved PRO100B PCI definitions to if_fei.h.
                 documentation corrections.
01e,03mar98,sbs  removed update of mmu table entries.
                 using sysMmuMapAdd() for adding mmu entries.
                 added extra members to feiResource structure.
                 changed initialization of feiResources arrray. 
01d,03dec96,hdn  changed UINT32 to INT32 for timeout, str[6], *pResults.
		 added sys557PciInit(). added configType to the resource.
01c,20nov96,dat  chg'd name to sysNetif.c, incorporated #defines
		 and struct typedefs from header file. Combines old
		 if_eex32.c and if_i82557.c files.
01b,07nov96,hdn  re-written.
01a,31aug96,dzb  written, based on v01a of src/drv/netif/if_iep.c.
*/

/*
Currently, this module only supports the Intel EtherExpress PRO100B LAN
Adapter.  If support for more 82557-based adapters is desired, this module
must be modified.

SEE ALSO: ifLib,
.I "Intel 82557 User's Manual,"
.I "Intel PRO100B PCI Adapter Driver Technical Reference,"
*/

#include "vxWorks.h"
#include "taskLib.h"
#include "sysLib.h"
#include "config.h"

#ifdef INCLUDE_FEI_END  /* END-style Intel 82557 (Fast EI) Support */
#   include "drv/end/fei82557End.h"
#else /* INCLUDE_FEI_END */
#   include "drv/netif/if_fei.h"
#endif

 
/* imports */

#ifdef INCLUDE_EEV
#include "eeVsysNetif.c"
#endif 

#ifdef INCLUDE_FEI	/* Intel 82557 (Fast EI) Support */
IMPORT FUNCPTR feiIntConnect;
#endif /* INCLUDE_FEI */

#ifdef INCLUDE_FEI_END  /* END-style Intel 82557 (Fast EI) Support */
IMPORT FUNCPTR feiEndIntConnect;
IMPORT FUNCPTR feiEndIntDisconnect;
#endif /* INCLUDE_FEI_END */

#if defined (INCLUDE_FEI) || defined (INCLUDE_FEI_END)

/* defines */

#ifdef I82557_DEBUG
#   undef	LOCAL
#   define	LOCAL
#endif	/* I82557_DEBUG */

/* Intel EtherExpress PRO100B LAN Adapter type */

#define	TYPE_PRO100B_PCI	1	/* Intel EtherExpress PRO-100B PCI */

/* EEPROM control bits */

#define EE_SK		0x01		/* shift clock */
#define EE_CS		0x02		/* chip select */
#define EE_DI		0x04		/* chip data in */
#define EE_DO		0x08		/* chip data out */

/* EEPROM opcode */

#define EE_CMD_WRITE	0x05		/* WRITE opcode, 101 */
#define EE_CMD_READ	0x06		/* READ  opcode, 110 */
#define EE_CMD_ERASE	0x07		/* ERASE opcode, 111 */

/* EEPROM misc. defines */

#define EE_CMD_BITS	3		/* number of opcode bits */
#define EE_ADDR_BITS	6		/* number of address bits */
#define EE_DATA_BITS	16		/* number of data bits */
#define EE_SIZE		0x40		/* 0x40 words */
#define EE_CHECKSUM	0xbaba		/* checksum */

/* unknown values */

#define UNKNOWN 	-1

/* typedefs */

typedef struct feiResource		/* FEI_RESOURCE */
    {
    UINT32	membaseCsr;		/* Base Address Register 0 */
    UINT32	iobaseCsr;		/* Base Address Register 1 */
    UINT32 	membaseFlash;		/* Base Address Register 2 */
    char	irq;			/* Interrupt Request Level */
    UINT32	configType;		/* type of configuration - unused */
    UINT32	boardType;		/* type of LAN board this unit is */
    UINT32	pciBus;			/* PCI Bus number */
    UINT32	pciDevice;		/* PCI Device number */
    UINT32	pciFunc;		/* PCI Function number */
    UINT16	eeprom[0x40];		/* Ethernet Address of this unit */
    INT32	timeout;		/* timeout for the self-test */
    INT32	str[6];			/* storage for the self-test result */
    INT32	*pResults;		/* pointer to the self-test result */
    UINT        memLength;		/* required memory size */
    UINT        initialStateMask; 	/* mask parameter to vmStateSet */
    UINT        initialState;		/* state parameter to vmStateSet */

    } FEI_RESOURCE;


/* locals */

LOCAL UINT32 feiUnits;			/* number of FEIs we found */

LOCAL FEI_RESOURCE feiResources [FEI_MAX_UNITS] =
    {
    {FEI0_MEMBASE0, FEI0_IOBASE0, FEI0_MEMBASE1, FEI0_INT_LVL, UNKNOWN,
     UNKNOWN, UNKNOWN, UNKNOWN, UNKNOWN, {UNKNOWN}, UNKNOWN, {UNKNOWN}, NULL, 
     FEI0_MEMSIZE0, FEI0_INIT_STATE_MASK, FEI0_INIT_STATE},
    {FEI1_MEMBASE0, FEI1_IOBASE0, FEI1_MEMBASE1, FEI1_INT_LVL, UNKNOWN,
     UNKNOWN, UNKNOWN, UNKNOWN, UNKNOWN, {UNKNOWN}, UNKNOWN, {UNKNOWN}, NULL, 
     FEI1_MEMSIZE0, FEI1_INIT_STATE_MASK, FEI1_INIT_STATE},
    {FEI2_MEMBASE0, FEI2_IOBASE0, FEI2_MEMBASE1, FEI2_INT_LVL, UNKNOWN,
     UNKNOWN, UNKNOWN, UNKNOWN, UNKNOWN, {UNKNOWN}, UNKNOWN, {UNKNOWN}, NULL,
     FEI2_MEMSIZE0, FEI2_INIT_STATE_MASK, FEI2_INIT_STATE},
    {FEI3_MEMBASE0, FEI3_IOBASE0, FEI3_MEMBASE1, FEI3_INT_LVL, UNKNOWN,
     UNKNOWN, UNKNOWN, UNKNOWN, UNKNOWN, {UNKNOWN}, UNKNOWN, {UNKNOWN}, NULL,
     FEI3_MEMSIZE0, FEI3_INIT_STATE_MASK, FEI3_INIT_STATE},
     };


LOCAL const char *phys[] = 
    {
     "None", "i82553-A/B", "i82553-C", "i82503",
     "DP83840", "80c240", "80c24", "unknown"
     };

enum phy_chips 
    {
    NonSuchPhy=0, I82553AB, I82553C, I82503,
    DP83840, S80C240, S80C24, UndefinedPhy
    };

LOCAL const char *connectors[] = {" RJ45", " BNC", " AUI", " MII"};


/* forward declarations */

LOCAL UINT16	sys557eepromRead (int unit, int location);
LOCAL UINT32	sys557mdioRead   (int unit, int phyId, int location);
LOCAL UINT32	sys557mdioWrite  (int unit, int phyId, int location, int value);
LOCAL int       sys557IntEnable  (int unit);
LOCAL int       sys557IntDisable (int unit);
LOCAL int       sys557IntAck     (int unit);

/*******************************************************************************
*
* sys557PciInit - prepare LAN adapter for 82557 initialization
*
* This routine find out the PCI device, and map its memory and IO address.
* It must be done prior to initializing the 82557, sys557Init().  Also
* must be done prior to MMU initialization, usrMmuInit().
*
* RETURNS: N/A
*/

void sys557PciInit (void)
    {
    FEI_RESOURCE *pReso;
    int pciBus;
    int pciDevice;
    int pciFunc;
    int unit;
    BOOL duplicate;
    UINT32 membaseCsr;
    UINT32 iobaseCsr;
    UINT32 membaseFlash;
    char irq;
    int ix;

    for (unit = 0; unit < FEI_MAX_UNITS; unit++)
	{
        if (pciFindDevice (PRO100B_PCI_VENDOR_ID, PRO100B_PCI_DEVICE_ID, unit, 
		           &pciBus, &pciDevice, &pciFunc) != OK)
	    break;

	/* check the duplicate */

        pReso     = &feiResources [0];
        duplicate = FALSE;

        for (ix = 0; ix < FEI_MAX_UNITS; ix++, pReso++)
            {
            if ((ix != unit) && (pReso->pciBus == pciBus) &&
                (pReso->pciDevice == pciDevice) && (pReso->pciFunc == pciFunc))
                duplicate = TRUE;
            }

	if (duplicate)
	    continue;

	/* we found the right one */

        pReso = &feiResources [unit];
	pReso->pciBus    = pciBus;
	pReso->pciDevice = pciDevice;
	pReso->pciFunc   = pciFunc;

        if (PCI_CFG_TYPE == PCI_CFG_FORCE) 
            {
            pciConfigOutLong (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
                              PCI_CFG_BASE_ADDRESS_0, pReso->membaseCsr);
            pciConfigOutLong (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
                              PCI_CFG_BASE_ADDRESS_1, pReso->iobaseCsr |
                              PCI_BASE_IO);
            pciConfigOutLong (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
                              PCI_CFG_BASE_ADDRESS_2, pReso->membaseFlash);
            pciConfigOutByte (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
                              PCI_CFG_DEV_INT_LINE, pReso->irq);
            }
	    
        /* get memory base address and IO base address */

	pciConfigInLong (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
	                 PCI_CFG_BASE_ADDRESS_0, &membaseCsr);
	pciConfigInLong (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
	                 PCI_CFG_BASE_ADDRESS_1, &iobaseCsr);
	pciConfigInLong (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
	                 PCI_CFG_BASE_ADDRESS_2, &membaseFlash);
        pciConfigInByte (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
	                 PCI_CFG_DEV_INT_LINE, &irq);
	membaseCsr   &= PCI_MEMBASE_MASK;
	iobaseCsr    &= PCI_IOBASE_MASK;
	membaseFlash &= PCI_MEMBASE_MASK;

	/* add the entry to sysPhysMemDesc[] */

        if (sysMmuMapAdd((void *)membaseCsr, pReso->memLength,
                          pReso->initialStateMask, 
                          pReso->initialState) == ERROR)
            break; 

	/* over write the resource table with read value */

	pReso->membaseCsr   = membaseCsr;
	pReso->iobaseCsr    = iobaseCsr;
	pReso->membaseFlash = membaseFlash;
	pReso->irq          = irq;

	/* enable mapped memory and IO addresses */

	pciConfigOutWord (pReso->pciBus, pReso->pciDevice, pReso->pciFunc,
			  PCI_CFG_COMMAND, PCI_CMD_IO_ENABLE |
			  PCI_CMD_MEM_ENABLE | PCI_CMD_MASTER_ENABLE);
	feiUnits++;
	}

    /* specify the interrupt connect/disconnect routines to be used */
     
#ifdef INCLUDE_FEI	/* Intel 82557 (Fast EI) Support */
    feiIntConnect = (FUNCPTR) pciIntConnect;
#endif /* INCLUDE_FEI */

#ifdef INCLUDE_FEI_END          /* END-style Intel 82557 (Fast EI) Support */
    feiEndIntConnect = (FUNCPTR) pciIntConnect;
    feiEndIntDisconnect = (FUNCPTR) pciIntDisconnect;
#endif /* INCLUDE_FEI_END */

    }

/*******************************************************************************
*
* sys557Init - prepare LAN adapter for 82557 initialization
*
* This routine is expected to perform any adapter-specific or target-specific
* initialization that must be done prior to initializing the 82557.
*
* The 82557 driver calls this routine from the driver attach routine before
* any other routines in this library.
*
* This routine returns the interrupt level the <pIntLvl> parameter.
*
* RETURNS: OK or ERROR if the adapter could not be prepared for initialization.
*/

STATUS sys557Init
    (
    int	unit,			/* unit number */
#ifdef INCLUDE_FEI_END          /* END-style Intel 82557 (Fast EI) Support */
    FEI_BOARD_INFO *pBoard      /* board information for the end driver */
#else
    BOARD_INFO *pBoard          /* board information for the if_fei driver */
#endif /* INCLUDE_FEI_END */
    )
    {
    FEI_RESOURCE *pReso = &feiResources [unit];
    UINT16 sum          = 0;
    int	ix;
    int	iy;
    UINT16 value;

    if (unit >= feiUnits)
	return (ERROR);

    /* locate the 82557 based adapter. PRO100B and XXX */

    if (pReso->boardType == TYPE_PRO100B_PCI)	/* only setup once */
	{
	}
    else 
	{
	/* read the configuration in EEPROM */

	for (ix = 0; ix < EE_SIZE; ix++) 
	    {
	    value = sys557eepromRead (unit, ix);
	    pReso->eeprom[ix] = value;
	    sum += value;
	    }
	if (sum != EE_CHECKSUM)
	    printf ("i82557(%d): Invalid EEPROM checksum %#4.4x\n", unit, sum);

	/* DP83840 specific setup */

	if (((pReso->eeprom[6]>>8) & 0x3f) == DP83840)
	    {
	    int reg23 = sys557mdioRead (unit, pReso->eeprom[6] & 0x1f, 23);
	    sys557mdioWrite (unit, pReso->eeprom[6] & 0x1f, 23, reg23 | 0x0420);
	    }

	/* perform a system self-test. */

        pReso->timeout     = 16000;		/* Timeout for set-test. */
        pReso->pResults    = (int *) ((((int) pReso->str) + 0xf) & ~0xf);
	pReso->pResults[0] = 0;
	pReso->pResults[1] = -1;
	sysOutLong (pReso->iobaseCsr + SCB_PORT, (int)pReso->pResults | 1);
	do {
	    sysDelay ();	/* delay for one IO READ cycle */
	    } while ((pReso->pResults[1] == -1)  &&  (--pReso->timeout >= 0));

        pReso->boardType = TYPE_PRO100B_PCI;
	}

    /* initializes the board information structure */

    pBoard->vector	  = pReso->irq + INT_NUM_IRQ0;
    pBoard->baseAddr	  = pReso->membaseCsr;
    for (ix = 0, iy = 0; ix < 3; ix++)
	{
	pBoard->enetAddr[iy++] = pReso->eeprom[ix] & 0xff;
	pBoard->enetAddr[iy++] = (pReso->eeprom[ix] >> 8) & 0xff;
	}
    pBoard->intEnable	  = sys557IntEnable;
    pBoard->intDisable	  = sys557IntDisable;
    pBoard->intAck	  = sys557IntAck;
    pBoard->sysLocalToBus = NULL; 
    pBoard->sysBusToLocal = NULL; 

#ifdef FEI_10MB 
    pBoard->phySpeed	  = NULL;
    pBoard->phyDpx	  = NULL;
#endif

    sysIntEnablePIC (pReso->irq);

    return (OK);
    }

/*******************************************************************************
*
* sys557IntAck - acknowledge an 82557 interrupt
*
* This routine performs any 82557 interrupt acknowledge that may be
* required.  This typically involves an operation to some interrupt
* control hardware.
*
* This routine gets called from the 82557 driver's interrupt handler.
*
* This routine assumes that the PCI configuration information has already
* been setup.
*
* RETURNS: OK, or ERROR if the interrupt could not be acknowledged.
*/

LOCAL STATUS sys557IntAck
    (
    int	unit		/* unit number */
    )
    {
    FEI_RESOURCE *pReso = &feiResources [unit];

    switch (pReso->boardType)
	{
	case TYPE_PRO100B_PCI:		/* handle PRO100B LAN Adapter */
	    /* no addition work necessary for the PRO100B */
	    break;

	default:
	    return (ERROR);
	}

    return (OK);
    }

/*******************************************************************************
*
* sys557IntEnable - enable 82557 interrupts
*
* This routine enables 82557 interrupts.  This may involve operations on
* interrupt control hardware.
*
* The 82557 driver calls this routine throughout normal operation to terminate
* critical sections of code.
*
* This routine assumes that the PCI configuration information has already
* been setup.
*
* RETURNS: OK, or ERROR if interrupts could not be enabled.
*/

LOCAL STATUS sys557IntEnable
    (
    int	unit		/* unit number */
    )
    {
    FEI_RESOURCE *pReso = &feiResources [unit];

    switch (pReso->boardType)
	{
	case TYPE_PRO100B_PCI:		/* handle PRO100B LAN Adapter */
	    /* no addition work necessary for the PRO100B */
	    break;

	default:
	    return (ERROR);
	}

    return (OK);
    }

/*******************************************************************************
*
* sys557IntDisable - disable 82557 interrupts
*
* This routine disables 82557 interrupts.  This may involve operations on
* interrupt control hardware.
*
* The 82557 driver calls this routine throughout normal operation to enter
* critical sections of code.
*
* This routine assumes that the PCI configuration information has already
* been setup.
*
* RETURNS: OK, or ERROR if interrupts could not be disabled.
*/

LOCAL STATUS sys557IntDisable
    (
    int	unit		/* unit number */
    )
    {
    FEI_RESOURCE *pReso = &feiResources [unit];

    switch (pReso->boardType)
	{
	case TYPE_PRO100B_PCI:		/* handle PRO100B LAN Adapter */
	    /* no addition work necessary for the PRO100B */
	    break;

	default:
	    return (ERROR);
	}

    return (OK);
    }

/*******************************************************************************
*
* sys557eepromRead - read a word from the 82557 EEPROM
*
* RETURNS: the EEPROM data word read in.
*/

LOCAL UINT16 sys557eepromRead
    (
    int	unit,		/* unit number */
    int	location	/* address of word to be read */
    )
    {
    UINT16 iobase = feiResources[unit].iobaseCsr;
    UINT16 retval = 0;
    UINT16 dataval;
    volatile UINT16 dummy;
    int ix;
 
    sysOutWord (iobase + SCB_EEPROM, EE_CS);	/* enable EEPROM */
 
    /* write the READ opcode */

    for (ix = EE_CMD_BITS - 1; ix >= 0; ix--)
	{
        dataval = (EE_CMD_READ & (1 << ix)) ? EE_DI : 0;
        sysOutWord (iobase + SCB_EEPROM, EE_CS | dataval);
        sysDelay ();	/* delay for one IO READ cycle */
        sysOutWord (iobase + SCB_EEPROM, EE_CS | dataval | EE_SK);
        sysDelay ();	/* delay for one IO READ cycle */
        sysOutWord (iobase + SCB_EEPROM, EE_CS | dataval); 
        sysDelay ();	/* delay for one IO READ cycle */
        }

    /* write the location */

    for (ix = EE_ADDR_BITS - 1; ix >= 0; ix--)
	{
        dataval = (location & (1 << ix)) ? EE_DI : 0;
        sysOutWord (iobase + SCB_EEPROM, EE_CS | dataval);
        sysDelay ();	/* delay for one IO READ cycle */
        sysOutWord (iobase + SCB_EEPROM, EE_CS | dataval | EE_SK);
        sysDelay ();	/* delay for one IO READ cycle */
        sysOutWord (iobase + SCB_EEPROM, EE_CS | dataval); 
        sysDelay ();	/* delay for one IO READ cycle */
	dummy = sysInWord (iobase + SCB_EEPROM);
        }

    if ((dummy & EE_DO) == 0)		/* dummy read */
	;
 
    /* read the data */

    for (ix = EE_DATA_BITS - 1; ix >= 0; ix--)
	{
        sysOutWord (iobase + SCB_EEPROM, EE_CS | EE_SK);
        sysDelay ();	/* delay for one IO READ cycle */
        retval = (retval << 1) | 
	         ((sysInWord (iobase + SCB_EEPROM) & EE_DO) ? 1 : 0);
        sysOutWord (iobase + SCB_EEPROM, EE_CS);
        sysDelay ();	/* delay for one IO READ cycle */
        }
 
    sysOutWord (iobase + SCB_EEPROM, 0x00);	/* disable EEPROM */

    return (retval);
    }

/*******************************************************************************
*
* sys557mdioRead - read MDIO
*
* RETURNS: read value
*/

LOCAL UINT32 sys557mdioRead
    (
    int	unit,		/* unit number */
    int phyId,		/* PHY ID */
    int location	/* location to read */
    )
    {
    UINT16 iobase = feiResources[unit].iobaseCsr;
    int timeout   = 64*4;	/* <64 usec. to complete, typ 27 ticks */
    int val;

    sysOutLong (iobase + SCB_MDI, 0x08000000 | (location<<16) | (phyId<<21));
    do {
    	sysDelay ();	/* delay for one IO READ cycle */
    	val = sysInLong (iobase + SCB_MDI);
    	if (--timeout < 0)
    	    printf ("sys557mdioRead() timed out with val = %8.8x.\n", val);
        } while (! (val & 0x10000000));

    return (val & 0xffff);
    }

/*******************************************************************************
*
* sys557mdioWrite - write MDIO
*
* RETURNS: write value
*/

LOCAL UINT32 sys557mdioWrite
    (
    int unit,		/* unit number */
    int phyId,		/* PHY ID */
    int location,	/* location to write */
    int value		/* value to write */
    )
    {
    UINT16 iobase = feiResources[unit].iobaseCsr;
    int timeout   = 64*4;	/* <64 usec. to complete, typ 27 ticks */
    int val;

    sysOutLong (iobase + SCB_MDI,
		0x04000000 | (location<<16) | (phyId<<21) | value);
    do {
    	sysDelay ();	/* delay for one IO READ cycle */
    	val = sysInLong (iobase + SCB_MDI);
    	if (--timeout < 0)
    	    printf ("sys557mdioWrite() timed out with val = %8.8x.\n", val);
        } while (! (val & 0x10000000));

    return (val & 0xffff);
    }

/*******************************************************************************
*
* sys557Show - shows 82557 configuration 
*
* this routine shows (Intel Pro Express 100) configuration 
*
* RETURNS: N/A
*/

void sys557Show
    (
    int	unit		/* unit number */
    )
    {
    FEI_RESOURCE *pReso = &feiResources [unit];
    UINT16 iobase       = pReso->iobaseCsr;
    UCHAR etheraddr[6];
    int ix;
    int iy;

    if (unit >= feiUnits)
	{
	printf ("none\n");
	return;
	}

    for (ix = 0, iy = 0; ix < 3; ix++) 
	{
        etheraddr[iy++] = pReso->eeprom[ix];
        etheraddr[iy++] = pReso->eeprom[ix] >> 8;
        }
    printf ("82557(%d): Intel EtherExpress Pro 10/100 at %#3x ", unit, iobase);
    for (ix = 0; ix < 5; ix++)
    	printf ("%2.2X:", etheraddr[ix]);
    printf ("%2.2X\n", etheraddr[ix]);

    printf ("CSR mem base address = %x, Flash mem base address = %x\n", 
	    pReso->membaseCsr, pReso->membaseFlash);
    printf ("PCI bus no. = %x, device no. = %x, function no. = %x, IRQ = %d\n",
	    pReso->pciBus, pReso->pciDevice, pReso->pciFunc, pReso->irq);

    if (pReso->eeprom[3] & 0x03)
        printf ("Receiver lock-up bug exists -- enabling work-around.\n");

    printf ("Board assembly %4.4x%2.2x-%3.3d, Physical connectors present:",
	pReso->eeprom[8], pReso->eeprom[9]>>8, pReso->eeprom[9] & 0xff);
    for (ix = 0; ix < 4; ix++)
	if (pReso->eeprom[5] & (1 << ix))
	    printf (connectors [ix]);

    printf ("\nPrimary interface chip %s PHY #%d.\n",
	phys[(pReso->eeprom[6]>>8)&7], pReso->eeprom[6] & 0x1f);
    if (pReso->eeprom[7] & 0x0700)
	printf ("Secondary interface chip %s.\n", 
		phys[(pReso->eeprom[7]>>8)&7]);

#if	FALSE  /* we don't show PHY specific info at this time */
    /* ToDo: Read and set PHY registers through MDIO port. */
    for (ix = 0; ix < 2; ix++)
	printf ("MDIO register %d is %4.4x.\n",
		ix, sys557mdioRead (unit, pReso->eeprom[6] & 0x1f, ix));
    for (ix = 5; ix < 7; ix++)
	printf ("MDIO register %d is %4.4x.\n",
		ix, sys557mdioRead (unit, pReso->eeprom[6] & 0x1f, ix));
    printf ("MDIO register %d is %4.4x.\n",
	    25, sys557mdioRead (unit, pReso->eeprom[6] & 0x1f, 25));
#endif	/* FALSE */

    if (pReso->timeout < 0) 
        {		/* Test optimized out. */
	printf ("Self test failed, status %8.8x:\n"
	        " Failure to initialize the 82557.\n"
	        " Verify that the card is a bus-master capable slot.\n",
	        pReso->pResults[1]);
	}
    else 
        {
	printf ("General self-test: %s.\n"
	        " Serial sub-system self-test: %s.\n"
	        " Internal registers self-test: %s.\n"
	        " ROM checksum self-test: %s (%#8.8x).\n",
	        pReso->pResults[1] & 0x1000 ? "failed" : "passed",
	        pReso->pResults[1] & 0x0020 ? "failed" : "passed",
	        pReso->pResults[1] & 0x0008 ? "failed" : "passed",
	        pReso->pResults[1] & 0x0004 ? "failed" : "passed",
	        pReso->pResults[0]);
        }
    }

#endif /* INCLUDE_FEI  || INCLUDE_FEI_END */

#ifdef INCLUDE_EEX32

/* includes */

#include "drv/netif/if_eex32.h"

/* defines */

#undef		EEX32_DEBUG		/* Compiles debug output */
#define 	MAX_UNITS	4	/* maximum units to support */

/* LAN Board Types */

#define		TYPE_UNKNOWN	0	/* initial value */
#define		TYPE_EEX32	1	/* Intel EtherExpress Flash 32 */

/* typedefs */

typedef struct i596Info			/* extra information struct */
    {
    UINT	port;			/* I/O port base address */
    UINT	type;			/* Type of LAN board this unit is */
    int		ilevel;			/* Interrupt level of this unit */
    } I596_INFO;


/* globals */

STATUS 	sysEnetAddrGet	(int unit, char addr[]);
STATUS 	sys596Init	(int unit);
STATUS 	sys596IntAck	(int unit);
STATUS 	sys596IntEnable (int unit);
void   	sys596IntDisable (int unit);
void   	sys596Port	(int unit, int cmd, UINT32 addr);
void   	sys596ChanAtn	(int unit);

/* locals */

LOCAL I596_INFO i596Info [MAX_UNITS];

LOCAL unsigned char eex32IdString [] =	/* Fixed-length, no NUL terminator! */
    {
    EEX32_EISA_ID0,
    EEX32_EISA_ID1,
    EEX32_EISA_ID2,
    EEX32_EISA_ID3
    };

LOCAL unsigned char eex32IntLevel [8] = /* first 4 are PLX, rest are FLEA */
    {
    5, 9, 10, 11, 3, 7, 12, 15
    };

/* forward function declarations */

LOCAL STATUS sysInitNecessary (int unit);
LOCAL STATUS sysFindEisaBoard (I596_INFO *pI596Info, UCHAR *idString);

/*******************************************************************************
*
* sysEnetAddrGet - retrieve net unit's Ethernet address
*
* The driver expects this routine to provide the six byte Ethernet address
* that will be used by this unit.  This routine must copy the six byte
* address to the space provided by pCopy.  This routine is expected to
* return OK on success, or ERROR.
*
* The driver calls this routine, once per unit, from the eiattach() routine.
* In if_ei.c version 03c,15jan93, this routine is the first 82596 support
* routine called.  In the PC environment, it is necessary for this routine
* then to find the hardware corresponding to the given unit number, before
* it can return the Ethernet address!  This is done by the function
* sysInitNecessary() (q.v.).
*
* Here we blithely assume that sysInitNecessary() finds a unit for us,
* as if no one would have attched the unit if it didn't exist!
*
* RETURNS: OK or ERROR.
*
* SEE ALSO:
*/

STATUS sysEnetAddrGet
    (
    int unit,
    char addr[]
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];
    UINT charIndex;

    /* 
     * Find a physical board to go with this unit and set up the
     * I596_INFO structure to refer to that board.
     */

    if (sysInitNecessary (unit) != OK)
	return (ERROR);

    for (charIndex = 0; charIndex < 6; charIndex++)
	addr[charIndex] = sysInByte (pI596Info->port + NET_IA0 + charIndex);

    return (OK);
    }

/*******************************************************************************
*
* sys596Init - prepare a LAN board for Ethernet initialization
*
* This routine performs target-specific initialization that must occur
* before the Intel 82596 Ethernet chip is initialized.  Typically, this
* routine is empty.
*
* The driver calls this routine from the eiattach() routine, once per unit,
* immediately before starting up the Intel 82596.  We here call
* sysInitNecessary() again in case eiattach() hasn't called sysInetAddrGet()
* yet.
*
* RETURNS: OK or ERROR.
*
* SEE ALSO: eiattach(), sysInitNecessary()
*/

STATUS sys596Init
    (
    int unit  /* unit number */
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];
    UCHAR intIndex;

    if (sysInitNecessary (unit) != OK)
	return (ERROR);

    if ( (sysInByte (pI596Info->port + IRQCTL) & IRQ_EXTEND) == 0)
        {
        intIndex = (sysInByte (pI596Info->port + PLX_CONF0) &
                    (IRQ_SEL0 | IRQ_SEL1)) >> 1;
        }
    else
        {
        intIndex = ( (sysInByte (pI596Info->port + IRQCTL) &
                      (IRQ_SEL0 | IRQ_SEL1)) >> 1) + 4;
        }

    pI596Info->ilevel = eex32IntLevel[intIndex];

    sysIntEnablePIC (pI596Info->ilevel);

    return (OK);
    }

/*******************************************************************************
*
* sys596IntAck - acknowledge an Ethernet chip interrupt 
*
* This routine acknowledges any specified non-Intel 82596 Ethernet chip
* interrupt.  Typically, this involves an operation to some interrupt
* control hardware.
*
* The interrupt signal from the 82596 behaves in an "edge-triggered" mode;
* therefore, this routine clears a latch within the control
* circuitry.
*
* NOTE: The driver calls this routine from the interrupt service routine.
*
* RETURNS: OK, or ERROR if the  
*/

STATUS sys596IntAck
    (
    int unit  /* unit number */
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];

    if (pI596Info->type == TYPE_UNKNOWN)
	return (ERROR);

    sysOutByte (pI596Info->port + IRQCTL,
                sysInByte (pI596Info->port + IRQCTL) | IRQ_LATCH);
    return (OK);
    }

/*******************************************************************************
*
* sys596IntEnable - enable an interrupt from an Intel 82596 Ethernet chip
*
* This routine manipulates i386/i486 board hardware to permit an interrupt.
*
* The Ethernet chip driver calls this routine throughout normal operation to
* terminate critical sections of code.
*
* RETURNS: OK, or ERROR if
*
* SEE ALSO: sys596IntDisable
*/

STATUS sys596IntEnable
    (
    int unit  /* unit number */
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];

    if (pI596Info->type == TYPE_UNKNOWN)
	return (ERROR);

    sysOutByte (pI596Info->port + IRQCTL,
		sysInByte (pI596Info->port + IRQCTL) & ~IRQ_FORCE_LOW);

    return (OK);
    }

/*******************************************************************************
*
* sys596IntDisable - disable an interrupt from an Intel 82596 Ethernet chip
*
* This routine manipulates i386/i486 board hardware to prevent interrupts.
*
* The Ethernet chip driver calls this routine throughout normal operation to
* protect critical sections of code from interrupt service routine
* intervention.
*
* RETURNS: N/A
*
* SEE ALSO: sys596IntEnable()
*/

void   sys596IntDisable
    (
    int unit  /* unit number */
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];

    if (pI596Info->type == TYPE_UNKNOWN)
	return;

    sysOutByte (pI596Info->port + IRQCTL,
		sysInByte (pI596Info->port + IRQCTL) | IRQ_FORCE_LOW);
    }

/*******************************************************************************
*
* sys596Port - issue PORT command to 82596
*
* This routine provides access to the special port function of the Intel
* 82596 Ethernet chip.  The driver expects this routine to deliver the
* command and address arguments to the port of the specified unit.
*
* The Ethernet chip driver calls this routine primarily during
* initialization, but may also call it during error recovery procedures.
* Because it is called during initialization, this routine can be called
* before the board for this unit has been found, depending on how eiattach()
* might be modified in the future.
*
* RETURNS: N/A
*/

void   sys596Port
    (
    int unit,
    int cmd,
    UINT32 addr
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];

    if (sysInitNecessary (unit) != OK)	/* Just in case not called yet! */
	return;

    /* PORT command wants to see 16 bits at a time, low-order first */

    sysOutWord (pI596Info->port + PORT, (cmd + addr) & 0xffff);
    sysOutWord (pI596Info->port + PORT + 2, ((cmd + addr) >> 16) & 0xffff);
    }

/*******************************************************************************
*
* sys596ChanAtn - assert channel attention signal to an Intel 82596
*
* This routine provides the channel attention signal to the Intel 82596
* Ethernet chip for a specified unit.
*
* The driver calls this routine frequently throughout all phases of
* operation.
*
* RETURNS: N/A
*/

void   sys596ChanAtn
    (
    int unit  /* unit number */
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];

    if (pI596Info->type == TYPE_UNKNOWN)
	return;

    sysOutByte (pI596Info->port + CA, 0);
    }

/*******************************************************************************
*
* sysInitNecessary - if unit is undefined, locate and set up the next board
*
*/

LOCAL STATUS sysInitNecessary
    (
    int unit  /* unit number */
    )
    {
    I596_INFO *pI596Info = &i596Info[unit];

    if (pI596Info->type == TYPE_UNKNOWN)
	{
	if (sysFindEisaBoard (pI596Info, eex32IdString) == ERROR)
	    return (ERROR);

	pI596Info->type = TYPE_EEX32;
	}

    /* Set up next unit structure in case there's another board */

    if (unit < (MAX_UNITS - 1))
        i596Info[unit + 1].port = i596Info[unit].port;

    return (OK);
    }

/*******************************************************************************
*
* sysFindEisaBoard - scan selected EISA slots for a board with given ID bytes
*
*/

LOCAL STATUS sysFindEisaBoard
    (
    I596_INFO *pI596Info,		/* Start at I/O address in here */
    UCHAR *idString
    )
    {
    UINT byteNumber;
    BOOL failed;

    do
        {
	failed = FALSE;
	for (byteNumber = 0; (byteNumber < 4) && !failed; byteNumber++)
	    {
	    failed = (sysInByte (pI596Info->port + EISA_ID0 + byteNumber)
		     != idString[byteNumber]);
	    }
	if (!failed)
	    return (OK);

	pI596Info->port += 0x1000;
        }
    while (pI596Info->port != 0xf000);

    return (ERROR);
    }

#endif /* INCLUDE_EEX32 */

#ifdef INCLUDE_END

#ifdef INCLUDE_LN_97X_END
 
#include "drv/end/ln97xEnd.h"
 
/* defines */
 
/* Offsets of PCI resources in host space */
 
#ifndef CPU_PCI_IO_ADRS
#define CPU_PCI_IO_ADRS         0               /* base of PCI I/O addr */
#endif /* CPU_PCI_IO_ADRS */
 
#ifndef CPU_PCI_MEM_ADRS
#define CPU_PCI_MEM_ADRS        0               /* base of PCI mem address */
#endif /* CPU_PCI_MEM_ADRS */
 
#ifndef PCI2DRAM_BASE_ADRS
#define PCI2DRAM_BASE_ADRS      0               /* DRAM base addrs from PCI */
#endif /* PCI2DRAM_BASE_ADRS */
 
#ifndef CPU_PCI_ISA_IO_ADRS
#define CPU_PCI_ISA_IO_ADRS     0               /* IO space */
#endif /* CPU_PCI_ISA_IO_ADRS */
 
#define EXT_INTERRUPT_BASE      INT_NUM_IRQ0    /* irq0     */
 
#define PCI_DEV_MMU_MSK         (~(VM_PAGE_SIZE - 1))   /* Mask MMU page */
 
#define PCI_DEV_ADRS_SIZE       VM_PAGE_SIZE    /* one page */
 
/* PCI device configuration type definitions */
 
#ifndef PCI_CFG_TYPE
#define PCI_CFG_TYPE            PCI_CFG_NONE
#endif
 
/* AMD 7997x 10/100Base-TX Board type */
 
#define LN_TYPE_970             1 /* AMD 97c970 PCI Ethernet PCNet */
#define LN_TYPE_971             2 /* AMD 97c970 PCI Ethernet PCNet Fast */
#define LN_TYPE_972             3 /* AMD 97c970 PCI Ethernet PCNet Fast+ */
#define LN_TYPE_7990            4
 
#define LN97X_MAX_UNITS         4 /* number of board types */
 
/* LN970 driver user flags */
 
#define LN_USR_FLAGS_970        0
#define LN_USR_FLAGS_971        0
#define LN_USR_FLAGS_972        0
#define LN_USR_FLAGS_7990       0
 
/* PCI vendor/device Ids and rev mask */
 
#define LN97X_PCI_VENDOR_ID     0x1022  /* AMD */
#define LN97X_PCI_DEVICE_ID     0x2000  /* PCI device ID */
#define LN7990_PCI_DEVICE_ID    0x0000  /* PCI device ID */
#define LN970_PCI_REV_MASK      0x10    /* 79970 pci rev mask */
#define LN971_PCI_REV_MASK      0x20    /* 79971 pci rev mask */
#define LN972_PCI_REV_MASK      0x30    /* 79972 pci rev mask */
#define LN97X_CSR3_VALUE        0       /* csr3 value */


/*
 * default values if PCI_CFG_TYPE defined to be PCI_CFG_FORCE
 * note: memory addresses must be aligned on MMU page boundaries
 */
 
#define LN97X_IO_ADR0           0xf400
#define LN97X_MEM_ADR0          0xfd000000
#define LN97X_INT_LVL0          0x0b
#define LN97X_INT_VEC0          0
 
#define LN97X_IO_ADR1           0xf420
#define LN97X_MEM_ADR1          0xfd200000
#define LN97X_INT_LVL1          0x05
#define LN97X_INT_VEC1          0
 
#define LN97X_IO_ADR2           0xf440
#define LN97X_MEM_ADR2          0xfd300000
#define LN97X_INT_LVL2          0x0c
#define LN97X_INT_VEC2          0
 
#define LN97X_IO_ADR3           0xf460
#define LN97X_MEM_ADR3          0xfd400000
#define LN97X_INT_LVL3          0x9
#define LN97X_INT_VEC3          0
 
#define LN97X_MAX_DEV           4
 
#define NET_END_USER_FLAGS      LN_USR_FLAGS_972
#define NET_ADAPTER_VENDOR_ID   LN97X_PCI_VENDOR_ID
#define NET_ADAPTER_DEVICE_ID   LN97X_PCI_DEVICE_ID
 
#define LN_97X_LOAD_FUNC        ln97xEndLoad
 
/* externs */
/* defined in sysLib.c */
IMPORT STATUS           sysMmuMapAdd (void * address, UINT len,
                                      UINT initialStateMask,
                                      UINT initialState);
IMPORT END_TBL_ENTRY    endDevTbl[];    /* end device table */
 
/* typedefs */
 
typedef struct ln97xPciResource              /* LN_97X_PCI_RESOURCES */
    {
    UINT32      iobaseCsr;              /* Base Address Register 0 */
    UINT32      membaseCsr;             /* Base Address Register 1 */
    char        irq;                    /* Interrupt Request Level */
    UINT32      irqvec;                 /* Interrupt Request vector */
    UINT32      configType;             /* type of configuration */
    UINT32      boardType;              /* type of LAN board for this unit */
    UINT32      pciBus;                 /* PCI Bus number */
    UINT32      pciDevice;              /* PCI Device number */
    UINT32      pciFunc;                /* PCI Function number */
    } LN_97X_PCI_RESOURCES;
 
typedef struct ln97xBoardResource            /* LN_97X_BOARD_RESOURCES */
    {
    UINT32      type;                   /* type of the board */
    UINT32      vendorId;               /* Vendor ID */
    UINT32      deviceId;               /* Device ID */
    UINT32      lnUsrFlags;             /* LN driver user flags */
    FUNCPTR     mediaSelectFunc;        /* media select routine */
    } LN_97X_BOARD_RESOURCES;
 
/* locals */
 
/*
 * This array defines the board-specific PCI resources, the base address
 * register configuration mode and the Ethernet adapter type. It's indexed
 * using the device number returned from pciFindDevice().
 */
 
LOCAL LN_97X_PCI_RESOURCES ln97xPciResources [LN97X_MAX_DEV] =
    {
    {LN97X_IO_ADR0, LN97X_MEM_ADR0, LN97X_INT_LVL0, LN97X_INT_VEC0,
          PCI_CFG_TYPE, LN_TYPE_970, 0, 0, 0},
    {LN97X_IO_ADR1, LN97X_MEM_ADR1, LN97X_INT_LVL1, LN97X_INT_VEC1,
          PCI_CFG_TYPE, LN_TYPE_970, 0, 0, 0},
    {LN97X_IO_ADR2, LN97X_MEM_ADR2, LN97X_INT_LVL2, LN97X_INT_VEC2,
          PCI_CFG_TYPE, LN_TYPE_970, 0, 0, 0},
    {LN97X_IO_ADR3, LN97X_MEM_ADR3, LN97X_INT_LVL3, LN97X_INT_VEC3,
          PCI_CFG_TYPE, LN_TYPE_970, 0, 0, 0},
    };
 
/*
 * This array defines board-specific vendor and device ids, flags to pass to
 * the drive load routine and the function used to select the media.
 */
 
LOCAL LN_97X_BOARD_RESOURCES ln97xBoardResources [LN97X_MAX_UNITS] =
    {
    {LN_TYPE_970, LN97X_PCI_VENDOR_ID, LN97X_PCI_DEVICE_ID, LN_USR_FLAGS_970,
     NULL},
    {LN_TYPE_971, LN97X_PCI_VENDOR_ID, LN97X_PCI_DEVICE_ID, LN_USR_FLAGS_971,
     NULL},
    {LN_TYPE_972, LN97X_PCI_VENDOR_ID, LN97X_PCI_DEVICE_ID, LN_USR_FLAGS_972,
     NULL},
    {LN_TYPE_7990, LN97X_PCI_VENDOR_ID, LN7990_PCI_DEVICE_ID,
     LN_USR_FLAGS_7990, NULL},
    };
 
/* forward declarations */
 
LOCAL void sysLan97xInitStrCook (LN_97X_PCI_RESOURCES *, int unitNum);
 
/*******************************************************************************
*
* sysLan97xPciInit - prepare LAN adapter for LN97X initialization
*
* This routine finds out the PCI device, maps its memory and IO address.
* It must be done prior to initializing the LN970, sysLn97XInit().  Also
* must be done prior to MMU initialization, usrMmuInit().
*
* RETURNS: OK/ERROR
*/
 
STATUS sysLan97xPciInit (void)
    {
    LN_97X_PCI_RESOURCES *     pRsrc;          /* pointer board resources */
    UINT32              membaseCsr;
    UINT32              iobaseCsr;
    char                irq;
    int                 pciBus;
    int                 pciDevice;
    int                 pciFunc;
    int                 unit;
    int                 found = 0;
    int                 ln97XUnits = 0;
 
    /* for all the support lan devices find if some of them exist */
 
    for (unit = 0; unit < LN97X_MAX_UNITS; unit++)
        {
        if (pciFindDevice (ln97xBoardResources [unit].vendorId,
                           ln97xBoardResources [unit].deviceId,
                           unit, &pciBus, &pciDevice, &pciFunc) == OK)
            {
             /* board detected */
             /* for now, we're ignoring the possibility of duplicates */
 
             found = TRUE;
 
             /* load up the PCI device table */
 
             pRsrc = ln97xPciResources + ln97XUnits; /* get the pci entry */
 
             pRsrc->pciBus    = pciBus;
             pRsrc->pciDevice = pciDevice;
             pRsrc->pciFunc   = pciFunc;
 
             ln97XUnits++;      /* number of units found */
             }
        }
 
    if ((found != TRUE) || (pciDevice > PCI_MAX_DEV))
        return (ERROR);
 
    /* Now initialize all the units we found */
 
    for (unit = 0; unit < ln97XUnits; unit++)
        {
        /* Fill in the resource entry */
 
        pRsrc = ln97xPciResources + unit;
 
        if (pRsrc->configType == PCI_CFG_FORCE)
            {
            /* write the iobase, membase, and irq */
 
            pciConfigOutLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                              PCI_CFG_BASE_ADDRESS_0, pRsrc->iobaseCsr |
                             PCI_BASE_IO);
            pciConfigOutLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                              PCI_CFG_BASE_ADDRESS_1, pRsrc->membaseCsr);
            pciConfigOutByte (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                              PCI_CFG_DEV_INT_LINE, pRsrc->irq);
            }
 
        /*
         * get memory base address and IO base address
         * Note: we read it in again, even if we just wrote it out because the
         * device can change what we wrote
         */
 
        pciConfigInLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                         PCI_CFG_BASE_ADDRESS_0, &iobaseCsr);
        pciConfigInLong (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                         PCI_CFG_BASE_ADDRESS_1, &membaseCsr);
        pciConfigInByte (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                         PCI_CFG_DEV_INT_LINE, &irq);
 
        /*
         * mask off registers. IO base needs to be masked off because bit0
         * will always be set to 1
         */
 
        membaseCsr   &= PCI_MEMBASE_MASK;
        iobaseCsr    &= PCI_IOBASE_MASK;
 
#ifdef INCLUDE_MMU_BASIC
        if (sysMmuMapAdd ((void *)(membaseCsr & PCI_DEV_MMU_MSK),
                          PCI_DEV_ADRS_SIZE,
                          VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE |
                         VM_STATE_MASK_CACHEABLE,
                          VM_STATE_VALID      | VM_STATE_WRITABLE |
                          VM_STATE_CACHEABLE_NOT
                          ) == ERROR)
            {
             /* for now, exit, but later break and stop where we're at
                when we're detecting multiple units */
             return (ERROR);
            }
#else
#warning INCLUDE_MMU_BASIC not defined
#endif /* INCLUDE_MMU_BASIC */
 
        /* over write the resource table with values read */
 
        pRsrc->membaseCsr   = membaseCsr;
        pRsrc->iobaseCsr    = iobaseCsr;
        pRsrc->irq          = irq;
        pRsrc->irqvec       = irq + EXT_INTERRUPT_BASE;
 
        /* enable mapped memory and IO addresses */
 
        pciConfigOutWord (pRsrc->pciBus, pRsrc->pciDevice, pRsrc->pciFunc,
                          PCI_CFG_COMMAND, PCI_CMD_IO_ENABLE |
                          PCI_CMD_MEM_ENABLE | PCI_CMD_MASTER_ENABLE);
 
        /* disable sleep mode */
 
        pciConfigOutWord (pciBus, pciDevice, pciFunc, PCI_CFG_MODE,
                          SLEEP_MODE_DIS);
 
        sysLan97xInitStrCook (pRsrc, unit);
        }
 
    return (OK);
    }
 
/*******************************************************************************
*
* sysLan97xIntEnable - enable Lan97x interrupts
*
* This routine enables ln7997x interrupts.  This may involve operations
on
* interrupt control hardware.
*
* RETURNS: OK or ERROR for invalid arguments.
*/
 
STATUS sysLan97xIntEnable
    (
    int level           /* level number */
    )
    {
    return (sysIntEnablePIC(level));
    }
 
/*******************************************************************************
*
* sysLanIntDisable - disable ln970 interrupts
*
* This routine disables ln970 interrupts.  This may involve operations
on
* interrupt control hardware.
*
* RETURNS: OK or ERROR for invalid arguments.
*/
 
STATUS sysLan97xIntDisable
    (
    int level           /* level number */
    )
    {
    return (sysIntDisablePIC(level));
    }
 
/*******************************************************************************
*
* sysLan97xEnetAddrGet - get Ethernet address
*
* This routine provides a target-specific interface for accessing a
* device Ethernet address.
*
* -- Specific to 971 ldt
*
* RETURNS: OK or ERROR if could not be obtained.
*/
 
STATUS sysLan97xEnetAddrGet
    (
    LN_97X_DRV_CTRL *  pDrvCtrl,   /* Driver control */
    char *             enetAdrs
    )
    {
    char        aprom [LN_97X_APROM_SIZE]; /* copy of address prom space */
    char *      ioaddr;
    int         ix;
 
    /* get IO address of unit */
    ioaddr = (char *)(pDrvCtrl->devAdrs);
 
    /* load aprom into an array */
    for (ix=0; ix<32; ix++)
        {
         SYS_IN_BYTE(pDrvCtrl,(ioaddr + ix), aprom [ix]);
        }
 
    /* check for 'w's at end of list */
    if ((aprom [0xe] != 'W') || (aprom [0xf] != 'W'))
        {
        /* should set errno here ? ldt */
        logMsg ("sysLn97XEnetAddrGet W's not stored in aprom\n",
                0, 1, 2, 3, 4, 5);
        return ERROR;
        }
 
    bcopy (aprom, enetAdrs, 6);
 
    return (OK);
    }
 
 
/*******************************************************************************
*
* sysLan97xInitStrCook - make the END load string for the device
*
* The END device load string formed by this routine is in the following
* following format.
* <devMemAddr>:<devIoAddr>:<pciMemBase:<vecnum>:<intLvl>:<memAdrs>
* :<memSize>:<memWidth>:<csr3b>:<offset>:<flags>
*
* .IP <devMemAddr>
* Device register base memory address
* .IP <devIoAddr>
* Device register base IO address
* .IP <pciMemBase>
* Base address of PCI memory space
* .IP <vecNum>
* Interrupt vector number.
* .IP <intLvl>
* Interrupt level.
* .IP <memAdrs>
* Memory pool address or NONE.
* .IP <memSize>
* Memory pool size or zero.
* .IP <memWidth>
* Memory system size, 1, 2, or 4 bytes (optional).
* .IP <CSR3>
* Value of CSR3 (for endian-ness mainly)
* .IP <offset>
* Offset of starting of data in the device buffers.
* .IP <flags>
* Device specific flags.
*
* The endDevTbl entry is updated with the pointer to the load string
* formed by this routine. This function expects all the ln97x entries
* to declared in the endDevTbl to be contiguous.
*
* NOTE
* Note that this routine assumes that the entry for the END device is the
* first entry in the in the END device table (endDevTbl[]).
*
* RETURNS: N/A
*/
 
LOCAL void sysLan97xInitStrCook
    (
    LN_97X_PCI_RESOURCES *     pRsrc,          /* board resources */
    int                 endDevNum       /* lan device # found on PCI bus */
    )
    {
    int                 ix;
    int                 entryIxMax;
    END_TBL_ENTRY *     pDevTbl;
 
    for (pDevTbl = endDevTbl, ix = 0; pDevTbl->endLoadFunc != END_TBL_END;
         pDevTbl++, ix++)
        ;
 
    /* max index of valid entry */
 
    entryIxMax = ix - 1;
 
    for (pDevTbl = endDevTbl, ix = 0; pDevTbl->endLoadFunc != END_TBL_END;
         pDevTbl++, ix++)
        {
        if ((ix + endDevNum) > entryIxMax)
            break;
 
        if ((UINT32)pDevTbl->endLoadFunc == (UINT32)LN_97X_LOAD_FUNC)
            {
            pDevTbl->unit = endDevNum;
            sprintf (endDevTbl [ix + endDevNum].endLoadString,
                     "0x%x:0x%x:0x%x:%d:%d:-1:-1:-1:0x%x:%d:0x%x",
                     /* device memory Io base */
                     (UINT) pRsrc->membaseCsr + CPU_PCI_MEM_ADRS,
                     /* device Io base */
                     (UINT) pRsrc->iobaseCsr  + CPU_PCI_IO_ADRS,
                     PCI2DRAM_BASE_ADRS,        /* pciMemBase */
                     pRsrc->irqvec,             /* interrupt IRQ vector */
                     pRsrc->irq,                /* interrupt irq number */
                     LN97X_CSR3_VALUE,          /* csr3 register value */
                     0,                         /* offset */
                     NET_END_USER_FLAGS         /* flags */
                     );
            break;
            }
        }
    return;
    }
#endif /* INCLUDE_LN_97X_END */






#endif /* INCLUDE_END */
