'\" t
.so wrs.an
.\" ixp1200eb/target.nr - Intel ixp1200eb target specific documentation
.\"
.\" Copyright 1999 Intel Corporation
.\"
.\" modification history
.\" --------------------
.\" 01f,23feb00,jdg  updated for Rev-E board
.\" 01e,28jan00,jdg  updated to match target.txt
.\" 01d,05Jan00,jdg  added support for Intel 82559
.\" 01c,27oct99,jdg  changed from A01 to B01 version
.\" 01b,13aug99,jdg  changed name from vbsa1200 to ixp1200eb
.\" 01a,10aug99,jdg  created, derived from ebsa285
.\"
.TH ixp1200eb T "Intel IXP1200EB" "Rev: 23 FEB 00" "TORNADO REFERENCE: VXWORKS"

.SH "NAME"
.aX "Intel IXP1200EB"

.SH "INTRODUCTION"
This manual entry provides board-specific information necessary to
run VxWorks for the IXP1200 Evaluation Board (IXP1200EB). Before
using a board with VxWorks, verify that the board runs in the
factory configuration using vendor-supplied firmware and jumper
settings and checking the RS-232 and ethernet connections.


.SS "Boot ROMs"
The IXP1200EB board uses two VxWorks boot ROMs (actually two
16-bit wide Intel TE28F800B3B90 Flash memory devices). Install the
upper half-word chip in XE1 (component side of board) with pin-1
nearer to the PCI connector. Install the lower half-word chip in
XE2 (wiring side of board) with pin-1 nearer to the PCI
connector. For further information, see the section "ROM
Considerations" below.

The BSP does not have non-volatile RAM; however it uses part of
the Flash memory to save boot parameters. Thus boot parameters are
preserved whenever the system is powered off.

To load VxWorks and for more information, follow the instructions
in the 
.I "Tornado User's Guide: Getting Started."

    
.SS "Board/Chip revisions"

This document describes the production revision of the evaluation
board. If the board in question does not match the picture at the
bottom of this document, then it is a prototype board. The details of
the prototype board can be found in an appendix.

.SS "Jumpers"

The following jumpers are relevant to VxWorks configuration.

.TS E
expand;
cf3 s s
lf3 lf3 lf3
l l lw(3i) .
.ne 5
IXP1200EB
.sp .25
Jumper	Function	Description
_
J9	PCI Int En 4	 T{
Enables PCI Interrupts for Slot 4 (INTA): U=enabled
T}
J10	PCI Int En 1	 T{
Enables PCI Interrupts for Slot 1 (INTD): U=enabled
T}
J11	PCI Int En 2	 T{
Enables PCI Interrupts for Slot 2 (INTC): U=enabled
T}
J12	GPIO[0]	T{
General Purpose I/O:  D=0, U=1 (1=Select Angel boot image)
T}
J13	CFN[1]	T{
PCI Arbiter Mode:  L=IXP1200, R=backplane
T}
J14	CFN[0]	T{
PCI Central Function, IXP1200 generates PCI reset:  L=Yes, R=No
T}
J15	NetROM CE	T{
Chip Enable for NetROM:  D=CE0, U=CE1
T}
J16	Flash CE	T{
Chip Enable for Flash:   D=CE0, U=CE1
T}
J17	PCI Int En 3	 T{
Enables PCI Interrupts for Slot 3 (INTB): U=enabled
T}
J21	System Reset	T{
Resets system when shorted.
T}
J23	PCI Clk En	T{
Board generates PCI clock:  L=Yes, R=No
T}
J26	GPIO[1]	T{
General Purpose I/O:  D=0, U=1 (1=Select VxWorks boot image)
T}
J27	GPIO[2]	T{
General Purpose I/O:  D=0, U=1 (should be 0)
T}
J28	GPIO[3]	T{
General Purpose I/O:  D=0, U=1 (should be 1) 
T}
J34	GMAC En	T{
GMAC Enable:  D=Disable, U=Enable
T}
J35	MAC En	T{
Octal MAC Enable:  D=Disable, U=Enable
T}
J36	Quad Phy En[0]	T{
Quad Phy Enable[0-3]:  D=Disable, U=Enable
T}
J45	Quad Phy En[1]	T{
Quad Phy Enable[4-7]:  D=Disable, U=Enable
T}
.TE

For details of jumper configuration, see the board diagram at the
end of this entry and in the hardware manual.


.SH "FEATURES"
The IXP1200EB is an evaluation board for Intel's IXP1200
network processor. This provides an interface between the IXP1200,
SDRAM, SRAM, Flash ROM, Octal MAC, GigaMAC, and PCI bus. It also
provides DMA controllers, timers, a serial port (UART), and a PCI
Bus arbiter.

The IXP1200 processor features six independent programming
MicroEngines for network data processing, and an F-Bus Interface.

The board supports 32 megabytes of SDRAM, 2 megabytes of SRAM, and
2 megabytes of Flash ROM.

The IXP1200EB can act in three different modes: as a Host Bridge
(where the IXP1200 is the Host Processor on a PCI backplane), as an
Add-in card (in a PCI backplane which already contains a Host
Bridge), or as a Stand-alone Card (not plugged into any PCI
backplane).


.SH "SUPPORTED FEATURES"
The BSP is configured by default to act as a Host Bridge on a PCI
backplane which has the PCI bus arbitration handled by the
backplane. It assumes that there is an ethernet card installed
which is based on a DEC 21040, 21140, or 21143 PCI Ethernet
Controller chip, or on the Intel 82559 LAN chip.


.SS "RAM size"
The BSP supports the default configuration of 32 Mb SDRAM. Of
this, the upper 8 Mb are remapped to virtual address 0 and are
available for use by VxWorks. The lower 24 Mb are mapped to
0xC0000000 and are uncached. This may be changed by modifying the
BSP. For more details, see the "Cache/MMU considerations" section
below.

The BSP supports 2 Mb of SRAM, although this is not included in
the VxWorks memory pool. Use of this memory must be managed by
application specific code.


.SS "Flash Memory"
A driver is provided supporting access to the Flash memory and the
use of a section of it as NVRAM.

By default (if INCLUDE_FLASH_UTILS is defined), a simple utility
for rewriting the Flash image is included. This utility (invoked
by the command "updateFlash") initiates a dialog with the user
over the serial port.


.SS "Timers"
The BSP uses hardware timers 2 and 3. Timers 1 and 4 are available
to application code.


.SS "NetROM Support"
The BSP supports the use of NetROM in place of Flash
memory. However, in the present revision, the Flash memory parts
must be physically removed when using NetROM.


.SS "Unsupported Features"
No support is provided for the Real Time Clock. No use is made of
the following features, although there should be no reason why
they cannot be used:
    DMA Controllers,
    I20 Message unit,
    timers 1 and 4

The BSP provides no support for use of the MicroEngines or F-Bus
Interface. This support is provided through application libraries.

The BSP will need modifications to make use of the following
features:
    Use of the IXP1200EB as an Add-in Card,
    Use of other than a DEC 21040/21140/21143-based PCI Ethernet card or Intel 82559-based PCI Ethernet card

Jumpers J9, J10, J11, and J17 select which PCI interrupt line is
connected to the chip. The jumpers allow several lines to be shorted
together so that multiple PCI slots can be used.


.SH "HARDWARE DETAILS"
.SS "Devices"
The device drivers included are:

    1xp1200Timer.c - IXP1200 timer driver
    ixp1200Sio.c - IXP1200 UART serial driver
    ixp1200IntrCtl.c - IXP1200 interrupt controller driver
    flashMem.c - Flash memory (including 28F800) driver
    nullNvRam.c - NVRAM-to-flash memory library
    pciIomapLib.c - PCI driver
    pciIomapShow.c - PCI Show routines
    sysEnd.c - Ethernet driver

See the manual entry for ixp1200Sio.c.


.SS "Shared Memory"
This BSP has not been tested with shared memory. There is no
support provided for the use of the PCI mailbox/doorbell features,
and there is no BSP-specific support for test-and-set
primitives. The vxTas() primitive is provided in the
architecture-specific code to allow access to the ARM SWPB
instruction. For further information, see the vxTas() man entry.


.SS "Interrupts"
This BSP only supports non-preemptive interrupt processing. So
effectively there is only 1 interrupt level. 32 Interrupt vectors
are provided:
.TS E
expand;
c c c .
     0	reserved_00   
     1	SI           	Soft interrupt
     2	reserved_02 
     3	reserved_03 
     4	T1           	Timer 1
     5	T2           	Timer 2
     6	T3           	Timer 3
     7	T4           	Timer 4
     8	reserved_08 
     9	CINT         	CINT pin (IRQ)
    10	UENG         	Microengine (IRQ)
    11	SRAM         	SRAM unit (IRQ)
    12	UART         	UART unit (IRQ)
    13	SDRAM        	SDRAM unit (IRQ)
    14	RTC          	Real Time Clock: Currently Unused
    15	DFH          	Doorbell from host
    16	DMA1         	DMA channel 1
    17	DMA2         	DMA channel 2
    18	PIL          	pci_irq_l
    19	reserved_19 
    20	DMA1NB       	DMA1 not busy
    21	DMA2NB       	DMA2 not busy
    22	SB           	Start BIST
    23	RSERR        	Received SERR
    24	SDPAR        	SDRAM parity
    25	IIP          	I20 inbound post_list
    26	reserved_26 
    27	DTE          	Discard Timer Expired
    28	DPED         	Data Parity Error Detected
    29	RMA          	Received Master Abort
    30	RTA          	Received Target Abort
    31	DPE          	Detect Parity Error
.TE

Only interrupt vectors 5, 6, 12, and 18 are used by default in
this BSP.

Interrupt connection, enabling, and disabling are performed using
the standard intArchLib routines. Interrupts 9 through 14
(inclusive) cannot be enabled or disabled using the standard
routines. These must be enabled or disabled by writing to the
appropriate control register directly. The interrupt controller
driver is provided in ixp1200IntrCtl.c.


.SS "Serial Configuration"
There is one serial port on the board. This connector is located
on the right edge of the card (see the board diagram at the end of
this entry). The port is "data leads only", that is, there are no
handshake lines provided at all. The default configuration is 9600
baud, 8 data bits, no parity, 1 stop bit.


.SS "SCSI Configuration"
This BSP does not support SCSI.


.SS "Network Configuration"
A network interface is provided for a PCI Ethernet adapter with a
DEC 21040, 21140, or 21143 Ethernet controller. The interface is
called "dc" and should be specified as the boot device to the boot
ROMs. The BSP automatically detects the type and the PCI slot of
the Ethernet card and initializes it accordingly. Consequently,
PCI Ethernet cards may be plugged into any of the PCI slots, as
long as the INTA signal in that slot connects to the INTA signal
in the system slot. The Ethernet MAC address is read from the PCI
card.

The system also supports PCI Ethernet adapters based on the 
Intel 82559 and 82559er chips.


.SS "VME Access"
The board is a PCI board. VME is not supported.


.SS "PCI Access"
The configuration access method is via accesses to areas in the
memory map that IXP1200 maps onto Type 0 and Type 1
configurations. By default, the SDRAM memory is mapped to PCI
memory address 0, so that virtual address 0 is mapped to PCI
memory address 24 Mb. See section "Cache/MMU considerations" for
more details.


.SS "Boot Devices"
By default, VxWorks is booted on this board by running the bootrom
image from Flash memory, and then loading the VxWorks kernel via
the network driver.


.SS "ROM considerations"
Intel ships the boards with Flash memory parts containing three
different systems that can be booted. At the start of Flash
memory, there is an Initial Boot Loader (IBL) that reads the state
of jumpers J12 and J26 to determine which system to boot. If J12
is set (see Section "Jumpers" above), then the ARM Angel monitor
is booted. If J26 is set, then the VxWorks image is booted. If
neither is set, then some simple diagnostics are booted. For
further details, see 
.I IXP1200 Network Processor: Ethernet Evaluation System User's Guide.

By default, the Angel image starts an an address of 0x20000 and
the VxWorks image at 0x40000. The VxWorks boot parameters are
stored in the block of Flash memory extending from 0x4000 to
0x8000.

Flash parts containing a valid VxWorks boot image can be
reprogrammed in-situ by using a utility provided by the BSP. To
use this utility, boot VxWorks and connect a Tornado
console. Connect the serial port to an appropriate terminal or
terminal emulator. Within the Tornado console window, issue the
command "updateFlash". This will initiate a dialog over the serial
port to specify the file name containing the new image and other
details. To reprogram the entire flash, accept the default values
of all queries other than the filename. For further details, see
.I StrongARM SA-1200 Network Processor: Evaluation System User's Guide.

The image that is programmed into the Flash must be a raw binary
image. These can be made by making the *.bin targets in
Makefile. Additionally, if a complete Flash boot image is to be
made, the VxWorks image must be relocated to address 0x40000 and
an appropriate IBL loaded at 0. This can be done for VxWorks alone
by building the *.ful targets defined in ixp1200eb.mak (which is
included by Makefile). Alternately, a separate utility may be used
to combine the diagnostic, Angel, and VxWorks images into one
Flash image.


.SS "Flash Memory as NVRAM"
Intel has allocated the block of Flash extending from address
0x4000 to 0x8000 as non-volatile storage.


.SH "SPECIAL CONSIDERATIONS"

.SS "Make Targets"
The image loaded into Flash memory must be a raw binary
image. These targets are defined with a ".bin" extension.
Additionally, VxWorks boot images must be loaded at the
appropriate address with some IBL loaded at zero. These targets
are defined with a ".ful" extension. So the corresponding targets
defined by the Makefile are:

.TS E
expand;
cf3 s s
lf3 lf3 lf3
l l l .
.ne 5
Make Targets
.sp .25
Original Target	Binary Target	Full Target
_
bootrom	bootrom.bin	bootrom.ful
bootrom_res	bootrom_res.bin	bootrom_res.ful
bootrom_uncmp	bootrom_uncmp.bin	bootromu.ful
vxWorks_rom	vxWorks_rom.bin	rom.ful
vxWorks.st_rom	vxWorks.st_rom.bin	st_rom.ful
vxWorks.res_rom_nosym	vxWorks.res_rom_nosym.bin	vxWorks_res_rom_nosym.ful
vxWorks
vxWorks.st
.TE

.SS "Programming the Flash"
Since there is only one set of flash parts, if the image stored in
them does not boot, then the only recourse is to either remove the
flash parts and program them externally, or to boot using the NetRom
sockets. If the board is booted to an image using ethernet, then the
function updateFlash() can be used to reprogram the flash. This
initiates a dialog on the serial port which queries the user as to how
the flash should be reprogrammed. Note that if the board is booted
with NetRom (so that the flash is located on CE1), then before the
updateFlash() command is given, one must issue the function call
flashSetBase(1). 

.SS "Delivered Objects"
There are several VxWorks internal routines that needed to be
modified for this BSP. There are provided in binary form in
"wrs/wrs.a". Additionally, drivers for the Digital 21x4x Ethernet
cards and Intel 82559 Ethernet cards are supplied in binary form in
"wrs/dc_DRV.a" and "wrs/eeV_44_DRV.a".

Additionally, several VxWorks include files needed to be modified
to support the 28F800 series of Flash memory parts. These include:
    mem/FlashMem.c
    drv/mem/flash28.h

And the MMU file:
    arch/arm/mmuArmLib.h


.SS "Special routines"
The routine sysLedsReadWrite() is used to control the LEDs on the
board. For further information, see the man entry for this
routine.

The routines sysRead16() and sysWrite16() are provided to allow C
code to use the LRDH and STRH instructions explicitly (16-bit
load/stores). For further information, see the source code in
sysALib.s.

The routine ixpVersion() returns the Intel IXP version number, which
relates this BSP version with other tools in the IXP suite.

.SS "NetROM"
The EBIXP1200 board may be booted via NetROM.

To boot with NetROM, jumper J15 (NetROM CE) should be set to CE0 (down)
and jumper J16 (Flash CE) should be set to CE1 (up). With these
settings, the NetROM will appear at virtual address 0x08000000 and the
flash will appear at virtual address 0x08200000.

.SS "Divide by Zero Exception"
The ARM architecture does not provide for an integer divide by
zero exception. Therefore, no exception is generated when an
integer divide by zero is performed programmatically.


.SS "Bus Error Exception"
The only Data Abort exceptions that the board can generate are
caused by the MMU during accesses to un-mapped memory.


.SS "Cache/MMU considerations"
The extra state VM_STATE_CACHEABLE_MINICACHE is available on the
IXP1200, which is not available on other ARM CPUs (with the
exception of the SA1100). Setting pages to this state using
vmStateSet() will result in those pages being cached in the
mini-cache, and not in the main data cache. Calling
cacheInvalidate(DATA_CACHE, ENTIRE_CACHE) will also invalidate the
mini-cache, but in all other aspects, no support is provided for
the mini-cache, and the user is entirely responsible for ensuring
cache coherency.

No support is provided to the read-buffer, but it is invalidated
during BSP startup.

The IXP1200 cannot support a memory mapping where RAM is present
over the exception vectors without using the MMU. In doing so it
must provide a mapping where physical and virtual addresses are
not the same. In order for the mmuLib/cacheLib code to support
this, the BSP has to provide routines to map between virtual and
physical addresses. See the man entries for the BSP routines
sysVirtToPhys() and sysPhysToVirt() for details.

The default mapping remaps the top part of SDRAM to virtual
address 0, so that the exception vectors lie in RAM. This then
requires the Flash memory to be remapped to a higher address to
make room for the SDRAM. Since a large amount of SDRAM is to be
used by the MicroEngines for packet data, and since this data must
be non-cacheable when accessed by the ARM core, the bottom part of
SDRAM is mapped in a separate virtual memory range that is not
adjacent to the portion of SDRAM used by VxWorks, and this range
is marked as non-cacheable.

The default physical to virtual mapping is defined as:

.TS E
expand;
cf3 s s
lf3 lf3 lf3
l l l .
.ne 5
Physical to Virtual Address Mapping
.sp .25
Physical Addresses	Virtual Addresses	Comments
_
C0000000-C1800000	C0000000-C1800000	SDRAM used by MicroEngines
C1800000-C2000000	00000000-00800000	SDRAM used by VxWorks
00000000-00200000	08000000-08200000	Flash Memory
.TE

The remaining address ranges are either mapped flat or are
invalid.


.SS "DEVSEL considerations"
The DEVSEL pins on the PCI backplane are typically attached to
address/data lines. In searching the PCI space for boards, the
IXP1200 can only utilize address/data lines 16-20 (inclusive). Thus
the backplane must be configured so that the DEVSELs for any used
slots lie in this range.


.SS "PCI Interrupt considerations"
The IXP1200 has only a single PCI interrupt pin, that can be connected
to any of the PCI interrupt lines (or more than one line) via board
jumpers. 

.SS "Validation Test Suite Failures"
In the test "bootline.tcl", the boot commands test fails due to a
problem with the test. This is because of an assumption that the
processor has a particular endian status.

The test "busTas.tcl" is not applicable.

The divide by zero test in "error1.tcl" fails because the ARM hardware
does not support a divde-by-zero exception.

The UDP packet echo test in "network.tcl" fails due to a problem in
the test, not in the BSP.

The tests in "scsi.tcl" are not applicable as the board does not
support SCSI. The loopback tests in "serial.tcl" are not applicable
because there is only one serial port.

In "timestamp.tcl", the disable timestamp test is not supported. The
timestamp timer cannot be stopped.

.SH "BOARD LAYOUT"

The diagram below shows the relevant jumpers on the Rev A01 board
for a VxWorks configuration. The default configuration is
illustrated (a Host Bridge with separate PCI Arbiter).

.bS
+-----------------------------------^^^^^^^^^^^^^^---^^^^^^^^^^^^^----+
|  ___    U J15  --NetROM-J5--                                        |
| |XE1|   D J16  --NetROM-J6--                                        |
| Flash                              U J35                            |
| |*__|   UDUD J28,27,26,12                                           |
|         ::U: J9,10,11,17                                   U J36    |
<                                                                     |
<              U J34       ___                                      S |
<                         |   |                                     e >
<                         |CPU|                                     r >
|      R   L              |___|                                     i >
|      J13 J14  R                                   J21      U J45  a >
|               J23                                                 l |
|                                                                     |
+---------+   _        _  +-------------------------------------------+
          |||| |||||||| |||
.bE
       Key:
       D  three pin vertical jumper, lower jumper installed
       U  three pin vertical jumper, upper jumper installed
       L  three pin horizontal jumper, left jumper installed
       R  three pin horizontal jumper, right jumper installed
       :  vertical jumper absent
       <  Optical interconnect
       ^  100 Base-T interconnect
       >  Serial port



.SS "Socket Descriptions"

The NetROM socket J5 interfaces the low-order word, and the NetROM
socket J6 interfaces the high-order word.

The Flash socket XE1 (component side) contains the high-order
word, and the socket XE2 (wiring side) contains the low-order
word.

    
.SH "SEE ALSO"
.tG "Getting Started,"
.pG "Configuration,"
.pG "Architecture Appendix"


.SH "BIBLIOGRAPHY"

.I "Intel PCI Development Backplane User's Guide,"
.I "IXP1200 Network Processor: Ethernet Evaluation System User's Guide"

.SH "PROTOTYPE BOARD"

.SS "Revision Information"

The board revision can be identified by a sticker on the wiring
side of the board. It contains a number similar to:
"54-30296-01-A01". The last three characters indicate the board
revision. This version of this document describes the "B01"
revision.

.SS "Jumpers"

.TS E
expand;
cf3 s s
lf3 lf3 lf3
l l lw(3i) .
.ne 5
IXP1200EB (Rev B01)
.sp .25
Jumper	Function	Description
_
J10	Test Clk Bypass	T{
Test Clock Bypass:  L=Test Clk, R=Crystal (should be Crystal)
T}
J11	Crystal En	T{
Crystal Output Enable:  D=Disable, U=Enable (should be Enable)
T}
J12	GPIO[0]	T{
General Purpose I/O:  D=1, U=0 (1=Select Angel boot image)
T}
J13	CFN[1]	T{
PCI Arbiter Mode:  L=IXP1200, R=backplane
T}
J14	CFN[0]	T{
PCI Central Function, IXP1200 generates PCI reset:  L=Yes, R=No
T}
J15	NetROM CE	T{
Chip Enable for NetROM:  D=CE0, U=CE1
T}
J16	Flash CE	T{
Chip Enable for Flash:   D=CE0, U=CE1
T}
J22	GMAC Osc En	T{
Enable GMAC Oscilator:  L=Enable, R=Disable
T}
J23	PCI Clk En	T{
Board generates PCI clock:  L=Yes, R=No
T}
J26	GPIO[1]	T{
General Purpose I/O:  D=1, U=0 (1=Select VxWorks boot image)
T}
J27	GPIO[2]	T{
General Purpose I/O:  D=1, U=0
T}
J28	GPIO[3]	T{
General Purpose I/O:  D=1, U=0 (should be 1) 
T}
J34	GMAC En	T{
GMAC Enable:  D=Disable, U=Enable
T}
J35	MAC En	T{
Octal MAC Enable:  L=Enable, R=Disable
T}
J36	Quad Phy En[0]	T{
Quad Phy Enable[0-3]:  D=Disable, U=Enable
T}
J37	Quad Phy En[1]	T{
Quad Phy Enable[4-7]:  D=Disable, U=Enable
T}
.TE


.SH "BOARD LAYOUT"

The diagram below shows the relevant jumpers on the Rev A01 board
for a VxWorks configuration. The default configuration is
illustrated (a Host Bridge with separate PCI Arbiter).

.bS
+-----------------------------------^^^^^^^^^^^^^^---^^^^^^^^^^^^^----+
|  ___    U J15  --NetROM-J5--                                        |
| |XE1|   D J16  --NetROM-J6--                                        |
| Flash                              L J35                            |
| |*__|                                                               |
|                   L J22                                 J36 U U J37 |
<                   U J34                                             |
<                          ___                                      S |
<                         |   |                                     e >
<                         |CPU|                                     r >
|  R   R   L              |___|                J27 J12              i >
|  J10 J13 J14  R                   U J11    D U D U                a >
|               J23                          J28 J26                l |
|                                                                     |
+---------+   _        _  +-------------------------------------------+
          |||| |||||||| |||
.bE
       Key:
       D  three pin vertical jumper, lower jumper installed
       U  three pin vertical jumper, upper jumper installed
       L  three pin horizontal jumper, left jumper installed
       R  three pin horizontal jumper, right jumper installed
       :  vertical jumper absent
       <  Optical interconnect
       ^  100 Base-T interconnect
       >  Serial port
