/* -*-C-*-
 *
 * Copyright (c) 1996 Advanced RISC Machines Limited
 * All Rights Reserved
 *
 * $Revision: 1 $
 *   $Author: Hqianx $
 *     $Date: 8/17/99 4:16p $
 *
 * makelo.c - This file is part of the Angel build world. It is used
 * to construct the assembler manifests source file. This ensures that
 * we only need to maintain a single copy of important definitions,
 * with this program being used to automatically build the target
 * header, if the 'C' source headers change.
 *
 * It is unfortunate that we need to keep this file up-to-date by
 * hand. However, the assembler values will always match the 'C'
 * definitions... and the build will fail if a definition required by
 * assembler code has not been added to this file.
 */

#include <stdio.h>
#include <time.h>

/* The following are the Angel headers from which we take information: */
#include "angel.h"
#include "adp.h"
#include "arm.h"
#include "devices.h"
#include "serial.h"
#include "serlock.h"
#include "devconf.h"
#include "support.h"
#include "logging.h"

#define maxstring       (256)

int main(int argc, char **argv)
{
  time_t     tval = time(NULL);   /* seconds since "January 1st 1970 GMT" */
  struct tm *timeptr;             /* full time structure */
  FILE      *outfile = stdout;    /* in-case we want to add code to write
                                   * to a specified file */
  char       buffer[maxstring];   /* temporary string construction buffer */
  char      *tptr;                /* temporary string pointer */
  int        index;               /* general index counter */

  timeptr = gmtime(&tval);
  if (NULL == timeptr) {
        fprintf(stderr, "\nCoordinated Universal Time not available\n");
  }
  
  /* Check for a command line parameter, and if so, use as the output file */
  if (argc > 1) 
  {
      if ((int)(outfile = fopen(argv[1], "w")) <= (int)0)
          outfile = stdout;
  }

  /*
   * output the file header
   */
  fprintf(outfile, "\tTTL Automatically generated by \"%s\"\n",argv[0]);
  fprintf(outfile, "\t; Do not edit this file directly. Changes made to the 'C' header\n");
  fprintf(outfile, "\t; files will force this file to be re-built by the make process.\n");
  fprintf(outfile, "\tASSERT\t(listopts_s)\t; ensure listopt.s included\n");
  fprintf(outfile, "old_opt\tSETA\t{OPT}\n");
  fprintf(outfile, "\tOPT\t(opt_off)\n\n");

  fprintf(outfile, "\t\tGBLL\tlolevel_s\n");
  fprintf(outfile, "lolevel_s\tSETL\t{TRUE}\n\n");

  /*
   * We provide time and date information, in-case the developer wants
   * to embed identification information into the lo-level assembler
   * code:
   */
  fprintf(outfile, "\t; Time and Date variables\n");
  fprintf(outfile, "\tGBLA\tMakeTime\t; secs since 1st Jan 1970\n");
  fprintf(outfile, "\tGBLS\tMakeDate\t; full ANSI time and date\n");
  fprintf(outfile, "\tGBLS\tMakeDay\t\t; full weekday name\n");
  fprintf(outfile, "\tGBLS\tMakeMDay\t; month day\n");

  fprintf(outfile, "\tGBLS\tMakeMonth\t; full month name\n");
  fprintf(outfile, "\tGBLS\tMakeYear\t; full year number\n");
  fprintf(outfile, "\tGBLS\tMakeClock\t; 24hour clock\n\n");

  fprintf(outfile, "MakeTime\tSETA\t&%08X\n", tval);

  sprintf(buffer, "%s", ctime(&tval));

  for (tptr = buffer; (*tptr && (*tptr != '\n')); tptr++)
    /* do nothing */
    ;

  *tptr = '\0'; /* replace newline with NULL */
  fprintf(outfile, "MakeDate\tSETS\t\"%s\"\n", buffer);

  if (NULL != timeptr) {
          index = strftime(buffer, maxstring, "%A", timeptr);
          buffer[index] = '\0';
          fprintf(outfile, "MakeDay\t\tSETS\t\"%s\"\n", buffer);

          index = strftime(buffer, maxstring, "%d", timeptr);
          buffer[index] = '\0';
          fprintf(outfile, "MakeMDay\tSETS\t\"%s\"\n", buffer);

          index = strftime(buffer, maxstring, "%B", timeptr);
          buffer[index] = '\0';
          fprintf(outfile, "MakeMonth\tSETS\t\"%s\"\n", buffer);

          index = strftime(buffer, maxstring, "%Y", timeptr);
          buffer[index] = '\0';
          fprintf(outfile, "MakeYear\tSETS\t\"%s\"\n", buffer);

          index = strftime(buffer, maxstring, "%H:%M", timeptr);
          buffer[index] = '\0';
          fprintf(outfile, "MakeClock\tSETS\t\"%s\"\n", buffer);
  } else {
        char *ptr1, *bufptr;
                /* We'll fudge ctime instead its of the form 
                  "Tue Aug 26 10:41:20 1997"  */
                  
                bufptr = buffer;  
                sprintf(bufptr, "%s", ctime(&tval));
                ptr1 = bufptr;
                while (*ptr1 != ' ') 
                        ptr1++;
                *ptr1 = '\0';                     
                fprintf(outfile, "MakeDay\t\tSETS\t\"%s\"\n", bufptr);
                
                bufptr = ptr1+1;
                while (*ptr1 != ' ') 
                        ptr1++;
                *ptr1 = '\0';           
                                        
                fprintf(outfile, "MakeMonth\tSETS\t\"%s\"\n", bufptr);
                bufptr = ptr1+1;
                while (*ptr1 != ' ') 
                        ptr1++;
                *ptr1 = '\0';                   
                fprintf(outfile, "MakeMDay\tSETS\t\"%s\"\n", bufptr);
                bufptr = ptr1+1;
                while (*ptr1 != ':') 
                        ptr1++;
                        
                ptr1++;
                while (*ptr1 != ':') 
                        ptr1++;                 
                *ptr1 = '\0';                           
                fprintf(outfile, "MakeClock\tSETS\t\"%s\"\n", bufptr);
  
                while (*ptr1 != ' ') 
                        ptr1++;
                bufptr = ptr1 + 1;
                while (*ptr1 != '\n') 
                        ptr1++;
                *ptr1 = '\0';
                fprintf(outfile, "MakeYear\tSETS\t\"%s\"\n", bufptr);
          
  }
  
  /* Standard ARM definitions */
  fprintf(outfile, "\n\t; Standard ARM definitions\n");
  fprintf(outfile, "reg_a1\tRN\t%d\n", reg_a1);
  fprintf(outfile, "reg_a2\tRN\t%d\n", reg_a2);
  fprintf(outfile, "reg_a3\tRN\t%d\n", reg_a3);
  fprintf(outfile, "reg_a4\tRN\t%d\n", reg_a4);
  fprintf(outfile, "reg_v1\tRN\t%d\n", reg_v1);
  fprintf(outfile, "reg_v2\tRN\t%d\n", reg_v2);
  fprintf(outfile, "reg_v3\tRN\t%d\n", reg_v3);
  fprintf(outfile, "reg_v4\tRN\t%d\n", reg_v4);
  fprintf(outfile, "reg_v5\tRN\t%d\n", reg_v5);
  fprintf(outfile, "reg_v6\tRN\t%d\n", reg_v6);
  fprintf(outfile, "reg_sl\tRN\t%d\n", reg_sl);
  fprintf(outfile, "reg_fp\tRN\t%d\n", reg_fp);
  fprintf(outfile, "reg_ip\tRN\t%d\n", reg_ip);
  fprintf(outfile, "reg_sp\tRN\t%d\n", reg_sp);
  fprintf(outfile, "reg_lr\tRN\t%d\n", reg_lr);
  fprintf(outfile, "reg_pc\tRN\t%d\n", reg_pc);
  fprintf(outfile, "reg_sb\tRN\t%d\n", reg_sb);
  fprintf(outfile, "IRQDisable\tEQU\t0x%08X\n", (unsigned int)IRQDisable);
  fprintf(outfile, "FIQDisable\tEQU\t0x%08X\n", (unsigned int)FIQDisable);
  fprintf(outfile, "InterruptMask\tEQU\t0x%08X\n", (unsigned int)InterruptMask);
  fprintf(outfile, "ModeMask\tEQU\t0x%08X\n", ModeMask);
  fprintf(outfile, "ModeMaskUFIS\tEQU\t0x%08X\n", ModeMaskUFIS);
  fprintf(outfile, "USRmode\t\tEQU\t0x%08X\n", USRmode);
  fprintf(outfile, "SYSmode\t\tEQU\t0x%08X\n", SYSmode);
  fprintf(outfile, "SVCmode\t\tEQU\t0x%08X\n", SVCmode);
  fprintf(outfile, "IRQmode\t\tEQU\t0x%08X\n", IRQmode);
  fprintf(outfile, "FIQmode\t\tEQU\t0x%08X\n", FIQmode);
  fprintf(outfile, "ABTmode\t\tEQU\t0x%08X\n", ABTmode);
  fprintf(outfile, "UNDmode\t\tEQU\t0x%08X\n", UNDmode);
  fprintf(outfile, "USR26mode\tEQU\t0x%08X\n", USR26mode);
  fprintf(outfile, "IRQ26mode\tEQU\t0x%08X\n", IRQ26mode);
  fprintf(outfile, "SVC26mode\tEQU\t0x%08X\n", SVC26mode);
  fprintf(outfile, "cond_EQ\t\tEQU\t0x%08X\n", cond_EQ);
  fprintf(outfile, "cond_mask\tEQU\t0x%08X\n", cond_mask);
  fprintf(outfile, "Tbit\tEQU\t0x%08X\n", (unsigned int)Tbit);

  fprintf(outfile, "vec_reset\t\t\tEQU\t0x%08X\n", vec_reset);
  fprintf(outfile, "vec_undefined_instruction\tEQU\t0x%08X\n",
          vec_undefined_instruction);
  fprintf(outfile, "vec_SWI\t\t\t\tEQU\t0x%08X\n", vec_SWI);
  fprintf(outfile, "vec_prefetch_abort\t\tEQU\t0x%08X\n", vec_prefetch_abort);
  fprintf(outfile, "vec_data_abort\t\t\tEQU\t0x%08X\n", vec_data_abort);
  fprintf(outfile, "vec_unused\t\t\tEQU\t0x%08X\n", vec_unused);
  fprintf(outfile, "vec_IRQ\t\t\t\tEQU\t0x%08X\n", vec_IRQ);
  fprintf(outfile, "vec_FIQ\t\t\t\tEQU\t0x%08X\n", vec_FIQ);

  fprintf(outfile, "angel_BreakPointInstruction_ARM\tEQU\t0x%08X\n",
          angel_BreakPointInstruction_ARM);
  fprintf(outfile, "angel_BreakPointInstruction_THUMB\tEQU\t0x%08X\n",
          angel_BreakPointInstruction_THUMB);
  fprintf(outfile, "angel_SWI_ARM\t\tEQU\t0x%08X\n", angel_SWI_ARM);
  fprintf(outfile, "angel_SWI_THUMB\t\tEQU\t0x%08X\n", angel_SWI_THUMB);
  fprintf(outfile, "angel_SWIreason_CLibBase\t\tEQU\t0x%08X\n",
          angel_SWIreason_CLibBase);
  fprintf(outfile, "angel_SWIreason_CLibLimit\t\tEQU\t0x%08X\n",
          angel_SWIreason_CLibLimit);
  fprintf(outfile, "angel_SWIreasonLimit\t\tEQU\t0x%08X\n",
          angel_SWIreasonLimit);
  fprintf(outfile, "angel_SWIreason_EnterSVC\t\tEQU\t0x%08X\n",
          angel_SWIreason_EnterSVC);
  fprintf(outfile, "angel_SWIreason_ReportException\t\tEQU\t0x%08X\n",
          angel_SWIreason_ReportException);
  fprintf(outfile, "angel_SWIreason_ApplDevice\t\tEQU\t0x%08X\n",
          angel_SWIreason_ApplDevice);
  fprintf(outfile, "angel_SWIreason_LateStartup\t\tEQU\t0x%08X\n",
          angel_SWIreason_LateStartup);

  fprintf(outfile, "angel_SWIreason_SysElapsed\t\tEQU\t0x%08X\n",
          SYS_ELAPSED);
  fprintf(outfile, "angel_SWIreason_SysTickFreq\t\tEQU\t0x%08X\n",
          SYS_TICKFREQ);
  fprintf(outfile, "AL_CONTINUE\t\tEQU\t0x%08X\n", AL_CONTINUE);
  fprintf(outfile, "AL_BLOCK\t\tEQU\t0x%08X\n", AL_BLOCK);

  fprintf(outfile, "BRANCH_TO_HIGH_ROM_ON_RESET\t\tEQU\t0x%08X\n",
          BRANCH_TO_HIGH_ROM_ON_RESET);

  {
    angel_RegBlock *rbp = (angel_RegBlock *)NULL;
    fprintf(outfile, "Angel_RegBlockSize\t\tEQU\t0x%08X\n",  sizeof(angel_RegBlock));
    fprintf(outfile, "RB_NumRegblocks\t\t\tEQU\t0x%08X\n", RB_NumRegblocks);
  }

  fprintf(outfile, "AngelInterruptMask\t\t\tEQU\t0x%08X\n", AngelInterruptMask);
  fprintf(outfile, "NotAngelInterruptMask\t\t\tEQU\t0x%08X\n", NotAngelInterruptMask);
  
  /* regblock array indexes */
  fprintf(outfile, "RB_Interrupted\t\t\tEQU\t0x%08X\n", (int)RB_Interrupted);
  fprintf(outfile, "RB_Desired\t\t\tEQU\t0x%08X\n", (int)RB_Desired);
  fprintf(outfile, "RB_SWI\t\t\t\tEQU\t0x%08X\n", (int)RB_SWI);
  fprintf(outfile, "RB_UNDEF\t\t\tEQU\t0x%08X\n", (int)RB_UNDEF);
  fprintf(outfile, "RB_Yield\t\t\tEQU\t0x%08X\n", (int)RB_Yield);
  fprintf(outfile, "RB_ABORT\t\t\tEQU\t0x%08X\n", (int)RB_ABORT);

  {
      angel_TaskQueueItem* tqi = 0;
      
      fprintf(outfile, "TQI_Type\t\t\tEQU\t%8d\n", (int)(& (tqi->type))  );
      fprintf(outfile, "TQI_Priority\t\t\tEQU\t%8d\n", (int)(& (tqi->priority))  );
      fprintf(outfile, "TQI_Name\t\tEQU\t%8d\n", (int)(& (tqi->name))  );
      fprintf(outfile, "TQI_SignalWaiting\t\tEQU\t%8d\n", (int)(& (tqi->signalWaiting))  );
      fprintf(outfile, "TQI_State\t\tEQU\t%8d\n", (int)(& (tqi->state))  );
      fprintf(outfile, "TQI_RB\t\tEQU\t%8d\n", (int)(& (tqi->rb))  );
  }
  fprintf(outfile, "TS_Undefined\t\t\tEQU\t0x%08X\n", (int)TS_Undefined);
  fprintf(outfile, "TS_Defined\t\t\tEQU\t0x%08X\n", (int)TS_Defined);
  fprintf(outfile, "TS_Runnable\t\t\tEQU\t0x%08X\n", (int)TS_Runnable);
  fprintf(outfile, "TS_Running\t\tEQU\t0x%08X\n", (int)TS_Running);
  fprintf(outfile, "TS_Blocked\t\tEQU\t0x%08X\n", (int)TS_Blocked);

  fprintf(outfile, "LOG_SERLASM\t\t\tEQU\t0x%08X\n", (int)LOG_SERLASM);

  fprintf(outfile, "TP_AngelInit\tEQU\t0x%08X\n", (int) TP_AngelInit);
  fprintf(outfile, "TP_Application\tEQU\t0x%08X\n", (int) TP_Application);
  fprintf(outfile, "TP_ApplCallBack\tEQU\t0x%08X\n", (int) TP_ApplCallBack);
  fprintf(outfile, "TP_AngelCallBack\tEQU\t0x%08X\n", (int) TP_AngelCallBack);
  fprintf(outfile, "TP_AngelWantLock\tEQU\t0x%08X\n", (int) TP_AngelWantLock);

  fprintf(outfile, "arm_padding\t\t\tEQU\t0x%08X\n", arm_padding);
  fprintf(outfile, "MMUCP\t\t\tCP\t%d\n", MMUCP);
  fprintf(outfile, "MMUIDReg\t\tCN\t%d\n", MMUIDReg);
  fprintf(outfile, "MMUControlReg\t\tCN\t%d\n", MMUControlReg);
  fprintf(outfile, "MMUPageTableBaseReg\tCN\t%d\n", MMUPageTableBaseReg);
  fprintf(outfile, "MMUDomainAccessReg\tCN\t%d\n", MMUDomainAccessReg);
  fprintf(outfile, "MMUFlushTLBReg\t\tCN\t%d\n", MMUFlushTLBReg);
  fprintf(outfile, "MMUPurgeTLBReg\t\tCN\t%d\n", MMUPurgeTLBReg);
  fprintf(outfile, "MMUFlushIDCReg\t\tCN\t%d\n", MMUFlushIDCReg);
  fprintf(outfile, "MMUOn\t\t\tEQU\t0x%08X\n", MMUOn);
  fprintf(outfile, "AlignFaultOn\t\tEQU\t0x%08X\n", AlignFaultOn);
  fprintf(outfile, "CacheOn\t\t\tEQU\t0x%08X\n", CacheOn);
  fprintf(outfile, "WriteBufferOn\t\tEQU\t0x%08X\n", WriteBufferOn);
  fprintf(outfile, "Config32\t\tEQU\t0x%08X\n", Config32);
  fprintf(outfile, "ConfigBE\t\tEQU\t0x%08X\n", ConfigBE);
  fprintf(outfile, "DefaultMMUConfig\tEQU\t0x%08X\n", DefaultMMUConfig);

  fprintf(outfile, "\t; ARM identification manifests:\n");
  fprintf(outfile, "ID_unknown\t\tEQU\t0x%08X\n", ID_unknown);
  fprintf(outfile, "ID_ARM1\t\t\tEQU\t0x%08X\n", ID_ARM1);
  fprintf(outfile, "ID_ARM2\t\t\tEQU\t0x%08X\n", ID_ARM2);
  fprintf(outfile, "ID_ARM3\t\t\tEQU\t0x%08X\n", ID_ARM3);
  fprintf(outfile, "ID_ARM6\t\t\tEQU\t0x%08X\n", ID_ARM6);
  fprintf(outfile, "ID_TYPE_MASK\t\tEQU\t0x%08X\n", ID_TYPE_MASK);
  fprintf(outfile, "ID_LE\t\t\tEQU\t0x%08X\n", ID_LE);
  fprintf(outfile, "ID_BE\t\t\tEQU\t0x%08X\n", ID_BE);
  fprintf(outfile, "ID_MUL\t\t\tEQU\t0x%08X\n", ID_MUL);
  fprintf(outfile, "ID_SWP\t\t\tEQU\t0x%08X\n", ID_SWP);
  fprintf(outfile, "ID_CACHE\t\tEQU\t0x%08X\n", ID_CACHE);
  fprintf(outfile, "ID_MMU\t\t\tEQU\t0x%08X\n", ID_MMU);
  fprintf(outfile, "ID_ENDIAN\t\tEQU\t0x%08X\n", ID_ENDIAN);
  fprintf(outfile, "ID_CACHE_ON\t\tEQU\t0x%08X\n", ID_CACHE_ON);
  fprintf(outfile, "ID_WB_ON\t\tEQU\t0x%08X\n", ID_WB_ON);
  fprintf(outfile, "ID_HWCONFIG_MASK\tEQU\t0x%08X\n", ID_HWCONFIG_MASK);

  /* Angel memory information: */
  fprintf(outfile, "\n\t; Angel memory information:\n");
  fprintf(outfile, "APCS_STACKGUARD\t\t\tEQU\t0x%08X\n", APCS_STACKGUARD);

  /* ADP definitions needed by the assembler code: */
  fprintf(outfile, "\n\t; ADP_Stopped reason codes:\n");
  fprintf(outfile, "ADP_Stopped_BranchThroughZero\t\tEQU\t%d\n",
          ADP_Stopped_BranchThroughZero);
  fprintf(outfile, "ADP_Stopped_UndefinedInstr\tEQU\t%d\n",
          ADP_Stopped_UndefinedInstr);
  fprintf(outfile, "ADP_Stopped_SoftwareInterrupt\t\tEQU\t%d\n",
          ADP_Stopped_SoftwareInterrupt);
  fprintf(outfile, "ADP_Stopped_PrefetchAbort\t\tEQU\t%d\n",
          ADP_Stopped_PrefetchAbort);
  fprintf(outfile, "ADP_Stopped_DataAbort\t\t\tEQU\t%d\n",
          ADP_Stopped_DataAbort);
  fprintf(outfile, "ADP_Stopped_AddressException\t\tEQU\t%d\n",
          ADP_Stopped_AddressException);
  fprintf(outfile, "ADP_Stopped_IRQ\t\t\t\tEQU\t%d\n", ADP_Stopped_IRQ);
  fprintf(outfile, "ADP_Stopped_FIQ\t\t\t\tEQU\t%d\n", ADP_Stopped_FIQ);
  fprintf(outfile, "ADP_Stopped_BreakPoint\t\t\tEQU\t%d\n",
          ADP_Stopped_BreakPoint);
  fprintf(outfile, "ADP_Stopped_RunTimeErrorUnknown\t\tEQU\t%d\n",
          ADP_Stopped_RunTimeErrorUnknown);
  fprintf(outfile, "ADP_Stopped_InternalError\t\tEQU\t%d\n",
          ADP_Stopped_InternalError);
  fprintf(outfile, "ADP_Stopped_UserInterruption\t\tEQU\t%d\n",
          ADP_Stopped_UserInterruption);
  fprintf(outfile, "ADP_Stopped_ApplicationExit\t\tEQU\t%d\n",
          ADP_Stopped_ApplicationExit);
  fprintf(outfile, "ADP_Stopped_StackOverflow\t\tEQU\t%d\n",
          ADP_Stopped_StackOverflow);
  fprintf(outfile, "ADP_Stopped_DivisionByZero\t\tEQU\t%d\n",
          ADP_Stopped_DivisionByZero);

  /* Misc ADP things: */
  fprintf(outfile, "\n\t; Misc ADP things:\n");
  fprintf(outfile, "ADP_HandleUnknown\t\tEQU\t0x%08X\n", ADP_HandleUnknown);
  fprintf(outfile, "ADP_CPUmode_Current\t\tEQU\t0x%08X\n",
          ADP_CPUmode_Current);
  fprintf(outfile, "ADP_CPUread_PCmode\t\tEQU\t0x%08X\n", ADP_CPUread_PCmode);
  fprintf(outfile, "ADP_CPUread_CPSR\t\tEQU\t0x%08X\n", ADP_CPUread_CPSR);
  fprintf(outfile, "ADP_CPUread_32bitIRQ\t\tEQU\t0x%08X\n",
          ADP_CPUread_32bitIRQ);
  fprintf(outfile, "ADP_CPUread_32bitUndef\t\tEQU\t0x%08X\n",
          ADP_CPUread_32bitUndef);
  fprintf(outfile, "ADP_CPUread_32bitSVC\t\tEQU\t0x%08X\n",
          ADP_CPUread_32bitSVC);
  fprintf(outfile, "ADP_CPU_LE\t\tEQU\t0x%08X\n", ADP_CPU_LE);
  fprintf(outfile, "ADP_CPU_BE\t\tEQU\t0x%08X\n", ADP_CPU_BE);
  fprintf(outfile, "ADP_CPU_BigEndian\t\tEQU\t0x%08X\n", ADP_CPU_BigEndian);

  /* Target specific definitions: */
  fprintf(outfile, "\n\t; Target specific definitions for Brutus:\n");
#ifdef Angel_ApplHeap
  fprintf(outfile, "Angel_ApplHeap\t\tEQU\t0x%08X\n", Angel_ApplHeap);
#endif
#ifdef Angel_ApplHeapLimit
  fprintf(outfile, "Angel_ApplHeapLimit\t\tEQU\t0x%08X\n", Angel_ApplHeapLimit);
#endif
  fprintf(outfile, "Angel_DefaultTopOfMemory\t\tEQU\t0x%08X\n",
          Angel_DefaultTopOfMemory);
#ifdef Angel_FixedStackBase
  fprintf(outfile, "Angel_FixedStackBase\t\tEQU\t0x%08X\n",
          Angel_FixedStackBase);
#endif
  fprintf(outfile, "Angel_ApplStackOffset\t\tEQU\t0x%08X\n",
          Angel_ApplStackOffset);
  fprintf(outfile, "Angel_ApplStackLimitOffset\tEQU\t0x%08X\n",
          Angel_ApplStackLimitOffset);
  fprintf(outfile, "Angel_SVCStackOffset\t\tEQU\t0x%08X\n",
          Angel_SVCStackOffset);
  fprintf(outfile, "Angel_SVCStackSize\t\tEQU\t0x%08X\n",
          Angel_SVCStackSize);
  fprintf(outfile, "Angel_SVCStackLimitOffset\t\tEQU\t0x%08X\n",
          Angel_SVCStackLimitOffset);

  fprintf(outfile, "Angel_IRQStackSize\t\tEQU\t0x%08X\n",
          Angel_IRQStackSize);
  fprintf(outfile, "Angel_IRQStackOffset\t\tEQU\t0x%08X\n",
          Angel_IRQStackOffset);
  fprintf(outfile, "Angel_IRQStackLimitOffset\t\tEQU\t0x%08X\n",
          Angel_IRQStackLimitOffset);
  fprintf(outfile, "Angel_FIQStackOffset\t\tEQU\t0x%08X\n",
          Angel_FIQStackOffset);
  fprintf(outfile, "Angel_FIQStackLimitOffset\t\tEQU\t0x%08X\n",
          Angel_FIQStackLimitOffset);

  fprintf(outfile, "Angel_ABTStackOffset\t\tEQU\t0x%08X\n",
          Angel_ABTStackOffset);
  fprintf(outfile, "Angel_ABTStackSize\t\tEQU\t0x%08X\n",
          Angel_ABTStackSize);
  fprintf(outfile, "Angel_UNDStackOffset\t\tEQU\t0x%08X\n",
          Angel_UNDStackOffset);
  fprintf(outfile, "Angel_UNDStackLimitOffset\t\tEQU\t0x%08X\n",
          Angel_UNDStackLimitOffset);

  fprintf(outfile, "Angel_AngelStackOffset\t\tEQU\t0x%08X\n",
          Angel_AngelStackOffset);
  fprintf(outfile, "Angel_AngelStackLimitOffset\t\tEQU\t0x%08X\n",
          Angel_AngelStackLimitOffset);

#if 0
  /* where are the page tables */
  fprintf(outfile, "Level1tab\tEQU\t0x%08X\n", Level1tab);
  fprintf(outfile, "Level2tab_ROM\tEQU\t0x%08X\n", Level2tab_ROM);
  fprintf(outfile, "Level2tab_FLASH\tEQU\t0x%08X\n", Level2tab_FLASH);
  fprintf(outfile, "TTB_BASE\tEQU\t0x%08X\n", TTB_BASE);
#endif

  /* Device driver variables, set at compile time */
  fprintf(outfile, "\n\t; Definitions for device drivers.\n");
  fprintf(outfile, "CACHE_SUPPORTED\tEQU\t%d\n", CACHE_SUPPORTED);
  fprintf(outfile, "PROFILE_SUPPORTED\tEQU\t%d\n\n", PROFILE_SUPPORTED);
  fprintf(outfile, "ENABLE_MMU\tEQU\t%d\n\n", ENABLE_MMU);
  fprintf(outfile, "ENABLE_CLOCK_SWITCHING\tEQU\t%d\n\n", ENABLE_CLOCK_SWITCHING);
  fprintf(outfile, "IH_SERIAL\tEQU\t%d\n", IH_SERIAL);
  fprintf(outfile, "DE_NUM_INT_HANDLERS\tEQU\t%d\n", DE_NUM_INT_HANDLERS);
  fprintf(outfile, "DC_SET_LED\tEQU\t%d\n", DC_SET_LED);
  fprintf(outfile, "DI_LED_DEVICE\tEQU\t%d\n", DI_LED_DEVICE);
  fprintf(outfile, "HANDLE_INTERRUPTS_ON_IRQ\tEQU\t%d\n", HANDLE_INTERRUPTS_ON_IRQ);
  fprintf(outfile, "HANDLE_INTERRUPTS_ON_FIQ\tEQU\t%d\n", HANDLE_INTERRUPTS_ON_FIQ);

  fprintf(outfile, "\n\t; Definitions for SYS library handler functions.\n");
  fprintf(outfile, "SYS_OPEN\tEQU\t%d\n", SYS_OPEN);
  fprintf(outfile, "SYS_CLOSE\tEQU\t%d\n", SYS_CLOSE);
  fprintf(outfile, "SYS_WRITE0\tEQU\t%d\n", SYS_WRITE0);
  fprintf(outfile, "SYS_WRITEC\tEQU\t%d\n", SYS_WRITEC);
  fprintf(outfile, "SYS_WRITE\tEQU\t%d\n", SYS_WRITE);
  fprintf(outfile, "SYS_READ\tEQU\t%d\n", SYS_READ);
  fprintf(outfile, "SYS_ISERROR\tEQU\t%d\n", SYS_ISERROR);
  fprintf(outfile, "SYS_ISTTY\tEQU\t%d\n", SYS_ISTTY);
  fprintf(outfile, "SYS_SEEK\tEQU\t%d\n", SYS_SEEK);
  fprintf(outfile, "SYS_ENSURE\tEQU\t%d\n", SYS_ENSURE);
  fprintf(outfile, "SYS_FLEN\tEQU\t%d\n", SYS_FLEN);
  fprintf(outfile, "SYS_TMPNAM\tEQU\t%d\n", SYS_TMPNAM);
  fprintf(outfile, "SYS_REMOVE\tEQU\t%d\n", SYS_REMOVE);
  fprintf(outfile, "SYS_RENAME\tEQU\t%d\n", SYS_RENAME);
  fprintf(outfile, "SYS_CLOCK\tEQU\t%d\n", SYS_CLOCK);
  fprintf(outfile, "SYS_TIME\tEQU\t%d\n", SYS_TIME);
  fprintf(outfile, "SYS_SYSTEM\tEQU\t%d\n", SYS_SYSTEM);
  fprintf(outfile, "SYS_ERRNO\tEQU\t%d\n", SYS_ERRNO);
  fprintf(outfile, "SYS_GET_CMDLINE\tEQU\t%d\n", SYS_GET_CMDLINE);
  fprintf(outfile, "SYS_HEAPINFO\tEQU\t%d\n", SYS_HEAPINFO);

  /* Terminate the file cleanly */
  fprintf(outfile, "\n\tOPT\t(old_opt)\n");
  fprintf(outfile, "\tEND\t; EOF lolevel_s\n");

  return(0);
}

/* EOF makelo.c */
