; mmu_h.s
;
;------------------------------------------------------------
;                                                                      
;                  I N T E L   P R O P R I E T A R Y                   
;                                                                      
;     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;     THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                      
;                        INTEL  CORPORATION                            
;                                                                     
;                     2200 MISSION COLLEGE BLVD                        
;                                                                      
;               SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                      
;------------------------------------------------------------
; system: SA1200
; subsystem: Angel Debug Monitor
; author: Henry Qian 01/29/99
; revisions:
;

        INCLUDE platform.s

L1_TABLE_ENTRIES        EQU     0x1000  ; 16kB table (word entries) 

; Allow different sized L2 Page tables, each L2 entry needs 256 words (1KB)
L2_TABLE_ENTRIES        EQU     0x100   ;  1kB table (word entries) 

TTB_SIZE                EQU     SZ_32K

; Translation Table at the end of DRAM, occupy 32 Kbytes
TTB_BASE                EQU     (DRAMBase + DRAMSize - TTB_SIZE)

; Access Permissions
; Depending on the setup, 0 represents one of these two access permissions
PT_NO_ACCESS            EQU     (0 << 10)
PT_SVC_R                EQU     (0 << 10)
PT_SVC_RW               EQU     (1 << 10)
PT_NO_USR_W             EQU     (2 << 10)
PT_ALL_ACCESS           EQU     (3 << 10)

PT_C_BIT                EQU     (1 << 3)
PT_B_BIT                EQU     (1 << 2)
PT_CB_BITS              EQU     (PT_C_BIT + PT_B_BIT)


; Entry type
PT_SECTION              EQU     2
PT_PAGE                 EQU     1
PT_INVALID              EQU     0

PT_SMALL                EQU     2
PT_LARGE                EQU     1


; Cached, buffered access, section translation
; AP=11, domain0, C=1, B=1
CACHED_BUFFERED_ACCESS  EQU     (PT_ALL_ACCESS + PT_CB_BITS + PT_SECTION)

; Cached, non-buffered access, section translation
; AP=11, domain0, C=1, B=0
CACHED_NOBUFFER_ACCESS  EQU     (PT_ALL_ACCESS + PT_C_BIT + PT_SECTION)

; Non-cached, buffered access, section translation
; AP=11, domain0, C=0, B=1
NOCACHE_BUFFERED_ACCESS EQU     (PT_ALL_ACCESS + PT_B_BIT + PT_SECTION)

; Non-cached, non-buffered access, section translation
; AP=11, domain0, C=0, B=0
NOCACHE_NOBUFFER_ACCESS EQU     (PT_ALL_ACCESS + PT_SECTION)

DOMAIN_CLIENT_ACCESS    EQU     1
DOMAIN_MANAGER_ACCESS   EQU     3

; Definitions used in conditional assembly of Icache, Dcache and Write Buffer
; options

IC_ON           EQU     0x1000
IC_OFF          EQU     0x0

DC_ON           EQU     0x4
DC_OFF          EQU     0x0

WB_ON           EQU     0x8
WB_OFF          EQU     0x0


; Bit definitions for the control register: 

; enables are logically OR'd with the control register
; use bit clears (BIC's) to disable functions 
;       *** all bits cleared on RESET ***

EnableMMU               EQU     0x1
EnableAlignFault        EQU     0x2
EnableDcache            EQU     0x4
EnableWB                EQU     0x8
EnableBigEndian         EQU     0x80
EnableMMU_S             EQU     0x100           ; selects MMU access checks 
EnableMMU_R             EQU     0x200           ; selects MMU access checks 
EnableIcache            EQU     0x1000

        END
