/* udbg_cmd.c
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: console
 * author: Henry Qian 03/04/99
 * revisions:
 *
 */
#include <stdio.h>
#include <string.h>
#include "os.h"
#include "udbg_cmd.h"
#include "debug_1200.h"

ConsoleCommand udbg_cmd[] = {
    {"InitDebugLib",        InitDebugLib,       "Init debug lib"},
    {"DeleteDebugLib",      DeleteDebugLib,     "Delete debug lib"},
    {"ResetUeng",           ResetUeng,          "Reset uEngine"},
    {"StartUeng",           StartUeng,          "Start uEngine from uPC0"},
    {"PauseUeng",           PauseUeng,          "Stop uEngines at next arbitration"},
    {"ResumeUeng",          ResumeUeng,         "Start uEngine from current uPC"},
    {"HopUeng",             HopUeng,            "Cause uEngines to hop"},
    {"GetUengStatus",       GetUengStatus,      "Obtain info on uEngine"},
    {"SetUengPC",           SetUengPC,          "Set uPC of a uEngine's threads"},
    {"GetUengPC",           GetUengPC,          "Get uPC of a uEngine's threads"},
    {"SetUthdPC",           SetUthdPC,          "Set uThread's uPC"},
    {"GetUthdPC",           GetUthdPC,          "Get uThread's uPC"},
    {"SetUword",            SetUword,           "Write a word to uEngine's uStore"},
    {"GetUword",            GetUword,           "Read a word from uEngine's uStore"},
    {"SetNumUword",         SetNumUword,        "Write words to uEninge's uStore"},
    {"GetNumUword",         GetNumUword,        "Read words from uEngine's uStore"},
    {"SetBP",               SetBP,              "Set a breakpoint"},
    {"ShowBP",              ShowBP,             "list current breakpoints"},
    {"ClearBP",             ClearBP,            "Clear a breakpoint"},
    {"ClearAllBP",          ClearAllBP,         "Clear all breakpoints"},
    {"SetGPR",              SetGPR,             "Write a word to rel GPR"},
    {"GetGPR",              GetGPR,             "Read a word from rel GPR"},
    {"SetSRAM_WrXfer",      SetSRAM_WrXfer,     "Write a word to rel SRAM WrXfer reg"},
    {"GetSRAM_WrXfer",      GetSRAM_WrXfer,     "Read a word from rel SRAM WrXfer reg"},
    {"SetSRAM_RdXfer",      SetSRAM_RdXfer,     "Write a word to rel SRAM RdXfer reg"},
    {"GetSRAM_RdXfer",      GetSRAM_RdXfer,     "Read a word from rel SRAM RdXfer reg"},
    {"SetSDRAM_WrXfer",     SetSDRAM_WrXfer,    "Write a word to rel SDRAM WrXfer reg"},
    {"GetSDRAM_WrXfer",     GetSDRAM_WrXfer,    "Read a word from rel SDRAM WrXfer reg"},
    {"SetSDRAM_RdXfer",     SetSDRAM_RdXfer,    "Write a word to rel SDRAM RdXfer reg"},
    {"GetSDRAM_RdXfer",     GetSDRAM_RdXfer,    "Read a word from rel SDRAM RdXfer reg"},
    {"SetFree_uStore",      SetFree_uStore,     "Set free uStore regions"},
    {"GetFree_uStore",      GetFree_uStore,     "Get free uStore regions"},

#if 0 // Not ready yet.
    {"Cntl_RegControl",     Cntl_RegControl,    "Register control callback functions"},
    {"Cntl_RegInitUof",     Cntl_RegInitUof,    "Register Init UOF file callback func"},
    {"Cntl_GiveControl",    Cntl_GiveControl,   "Invoke GiveControl callback function"},
    {"Cntl_TakeControl",    Cntl_TakeControl,   "Invoke TakeControl callback function"},
    {"Cntl_InitUof",        Cntl_InitUof,       "Invoke Init UOF file callback func"},
    {"Cntl_Reset",          Cntl_Reset,         "Reset SA-1200 system"},
    {"Cntl_Start",          Cntl_Start,         "Start SA-1200 system"},
    {"Cntl_Pause",          Cntl_Pause,         "Pause SA-1200 system"},
    {"Cntl_Resume",         Cntl_Resume,        "Resume SA-1200 system"},
    {"Cntl_Hop",            Cntl_Hop,           "Run uEngine to next context switch"},
#endif

    {(char*)0,  0,      0}
};

void callBack(  unsigned short bkptId, unsigned char ueng,
                unsigned char ctx, void *usrData)
{
    printf(" - Break @ ueng%d ctx%d bpID: %d\n", ueng, ctx, bkptId);
}

#define MAX_ENGINE_MASK 0x3F
#define MAX_NUM_WORDS 1024

uDbg_uStoreFree_T uDbg_uStoreFree;

unsigned int uDbg_uStoreFreeInit = 0;    // indicate not initialized yet.

void SetFree_uStore(char* param)
{
    int ii;
    unsigned int numericValue;

    // temp variables to hold and check input parameters
    //
    unsigned short upc[MAX_UENGS];
    unsigned short size[MAX_UENGS];

    for (ii = 0; ii < MAX_UENGS; ii++)
    {
        if (getParam(&param, &numericValue) != 0)
        {
            printf(" - format: SetFree_uStore upc0, upc1, upc2, ");
            printf("upc3, upc4, upc5, sz0, sz1, sz2, sz3, sz4, sz5\n");
            return;
        }

        upc[ii] = numericValue;
    }

    for (ii = 0; ii < MAX_UENGS; ii++)
    {
        if (getParam(&param, &numericValue) != 0)
        {
            printf(" - format: SetFree_uStore uPC0, upc1, upc2, ");
            printf("upc3, upc4, upc5, sz0, sz1, sz2, sz3, sz4, sz5\n");
            return;
        }

        size[ii] = numericValue;
    }

    for (ii = 0; ii < MAX_UENGS; ii++)
    {
        uDbg_uStoreFree.freeUpc[ii] = upc[ii];
        uDbg_uStoreFree.freeSize[ii] = size[ii];
    }

    uDbg_uStoreFreeInit = 1;
}

void GetFree_uStore(char*)
{
    int ii;

    if (uDbg_uStoreFreeInit != 1)
        printf(" - uStore not initialized\n");
    else
    {
        for (ii = 0; ii < MAX_UENGS; ii++)
        {
            printf(" - eng%d: upc: 0x%04x size: 0x%04x\n", ii,
                    uDbg_uStoreFree.freeUpc[ii],
                    uDbg_uStoreFree.freeSize[ii]);
        }
    }
}

uDbg_Handle_T uDbg_Handle = 0;

void InitDebugLib(char*)
{
    /* uDbg_Handle_T handle = uDbg_InitLib(&uDbg_uStoreFree); */
    uDbg_Handle_T handle = uDbg_InitLib(0);

    if (handle == 0)
        printf(" - failed\n");
    else
        uDbg_Handle = handle;
}


void DeleteDebugLib(char*)
{
    if (uDbg_Handle != 0)
    {
        uDbg_DeleLib(uDbg_Handle);
        uDbg_Handle = 0;
    }
    else
        printf(" - uDebugLib not initialized\n");
}

void ResetUeng(char* param)
{
    int retval;
    unsigned int mask;
    int clrReg; 

    if ((getParam(&param, &mask) != 0) ||
        (getParam(&param, &clrReg) != 0))
         printf(" - format: ResetUeng <mask> <clrReg> <\n");
    else
    {
        retval = uDbg_ResetUeng(uDbg_Handle, mask,0);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


void StartUeng(char* param)
{
    int retval;
    unsigned int mask;

    if (getParam(&param, &mask) != 0)
        printf(" - format: StartUeng <mask>\n");
    else
    {
        retval = uDbg_StartUeng(uDbg_Handle, mask);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


void PauseUeng(char* param)
{
    int retval;
    unsigned int delay;
    unsigned int mask;

    if (getParam(&param, &mask) != 0)
        printf(" - format: PauseUeng <mask> <delay>\n");
    else
    {
        if (getParam(&param, &delay) != 0)
            delay = 5;
        retval = uDbg_PauseUeng((char*)uDbg_Handle, mask, delay);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


void ResumeUeng(char* param)
{
    int retval;
    unsigned int mask;

    if (getParam(&param, &mask) != 0)
        printf(" - format: ResumeUeng <mask>\n");
    else
    {
        retval = uDbg_ResumeUeng(uDbg_Handle, mask);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


void GetActiveUeng(char*)
{
    printf(" - active engine(s): 0x%x\n",
        uDbg_GetActiveUengs(uDbg_Handle, MAX_ENGINE_MASK));
}


void SetUengPC(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int upc[4];

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &upc[0]) != 0)    ||
        (getParam(&param, &upc[1]) != 0)    ||
        (getParam(&param, &upc[2]) != 0)    ||
        (getParam(&param, &upc[3]) != 0))
    {
        printf(" - format: SetUengPC <engNum> <upc0> <upc1> <upc2> <upc3>\n");
    }
    else
    {
        retval = uDbg_SetUengPCs((char*)uDbg_Handle, engineNum, upc);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


void GetUengPC(char* param)
{
    int retval;
    int ii;
    unsigned int engineMask;
    unsigned int engineNum;
    unsigned int upc[4];

    if (getParam(&param, &engineMask) != 0)
        engineMask = 0x3f; /* Display all */

    for (engineNum = 0; engineNum < 6; engineNum++)
    {
        if (engineMask & (1 << engineNum))
        {
            printf(" - eng%d: ", engineNum);

            retval = uDbg_GetUengPCs((char*)uDbg_Handle, engineNum, upc);
            if (retval != UDBG_SUCCESS)
                printf(" failed\n");
            else
            {
                for (ii = 0; ii < 4; ii++)
                    printf("upc[%d]:0x%04x ", ii, upc[ii]);
                printf("\n");
            }
        }
    }
}


void SetUthdPC(char* param)
{
    int retval;
    unsigned int engineMask;
    unsigned int threadMask;
    unsigned int upc;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &threadMask) != 0) ||
        (getParam(&param, &upc) != 0))
    {
        printf(" - format: SetUthdPC <engineMask> <threadMask> <upc>\n");
    }
    else
    {
        retval = uDbg_SetUthreadsPC((char*)uDbg_Handle, engineMask, threadMask, upc);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


void GetUthdPC(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int upc;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0))
    {
        printf(" - format: SetUthdPC <engNum> <thdNum>\n");
    }
    else
    {
        retval = uDbg_GetUthreadPC((char*)uDbg_Handle, engineNum, threadNum, &upc);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
            printf(" - upc: 0x%x\n", upc);
    }
}


void SetUword(char* param)
{
    int retval;
    unsigned int engineMask;
    unsigned int uWordAddr;
    unsigned int uWord;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &uWordAddr) != 0) ||
        (getParam(&param, &uWord) != 0))
    {
        printf(" - format: SetUword <engineMask> <uWordAddr> <uWord>\n");
    }
    else
    {
        retval = uDbg_SetUword( uDbg_Handle,
                                engineMask,
                                uWordAddr,
                                uWord);

        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


void GetUword(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int uWordAddr;
    unsigned int uWord;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &uWordAddr) != 0))
    {
        printf(" - format: GetUword <engNum> <uWordAddr>\n");
    }
    else
    {
        retval = uDbg_GetUword( uDbg_Handle,
                                engineNum,
                                uWordAddr,
                                &uWord);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
            printf(" - 0x%08x\n", uWord);
    }
}


void SetNumUword(char* param)
{
    int retval;
    unsigned int engineMask;
    unsigned int uWordAddr;
    unsigned int numOfWords;
    unsigned int uWord;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &uWordAddr) != 0) ||
        (getParam(&param, &numOfWords) != 0) ||
        (getParam(&param, &uWord) != 0))
    {
        printf(" - format: SetNumUword <engineMask> <uWordAddr> <numOfWords> <uWord>\n");
    }
    else
    {
        retval = uDbg_SetNumUword(  uDbg_Handle,
                            	    engineMask,
                            	    uWordAddr,
                            	    numOfWords,
                            	    (unsigned int*)uWord);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}


unsigned int temp[MAX_NUM_WORDS];

void GetNumUword(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int uWordAddr;
    unsigned int numOfWords;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &uWordAddr) != 0) ||
        (getParam(&param, &numOfWords) != 0))
    {
        printf(" - format: GetNumUword <engNum> <uWordAddr> <numOfWords>\n");
    }
    else
    {
        retval = uDbg_GetNumUword(  uDbg_Handle,
                            	    engineNum,
                            	    uWordAddr,
                            	    numOfWords,
                            	    temp);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
        {
            for (int ii = 0; ii < numOfWords; ii++)
            {
                if (ii%4 == 0)
                    printf("\n 0x%04x - ", uWordAddr++);

                printf(" 0x%08x", temp[ii]);
            }

            printf("\n");
        }
    }
}


uDbg_Bkpt_T uDbg_Bkpt;

void SetBP(char* param)
{
    int retval;
    unsigned int uAddr;
    unsigned int engineNum;
    unsigned int contextNum;
    unsigned int engineMask;
    unsigned int breakIfThread;

    if ((getParam(&param, &uAddr) != 0) ||
        (getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &contextNum) != 0) ||
        (getParam(&param, &engineMask) != 0))
    {
        printf(" - format: SetBP <uAddr> <engNum> <ctxNum> <engMask>\n");
    }
    else
    {
        uDbg_Bkpt.callback = callBack;
        uDbg_Bkpt.ctx = 0;
        uDbg_Bkpt.uAddr = uAddr;
        uDbg_Bkpt.uEng = engineNum;
        uDbg_Bkpt.usrData = 0;
        uDbg_Bkpt.stpOnBrkUengMask = engineMask;

        switch (contextNum)
        {
            case 0:
                uDbg_Bkpt.breakIfCtx = UDBG_CTX0;
                break;

            case 1:
                uDbg_Bkpt.breakIfCtx = UDBG_CTX1;
                break;

            case 2:
                uDbg_Bkpt.breakIfCtx = UDBG_CTX2;
                break;

            case 3:
                uDbg_Bkpt.breakIfCtx = UDBG_CTX3;
                break;

            case 0xF:
                uDbg_Bkpt.breakIfCtx = UDBG_CTXANY;
                break;

            default:
                printf(" - error: ctxNum range: 0-3, 0xf\n");
                return;
        }

        retval = uDbg_SetBreakpoint((char*)uDbg_Handle, &uDbg_Bkpt);
        if (retval != UDBG_SUCCESS)
            printf(" - failed %x\n", retval);
        else
            printf(" - Breadpoint id: 0x%x\n", uDbg_Bkpt.id);
    }
}


uDbg_BkptArray_T uDbg_BkptArray;

void ShowBP(char* param)
{
    int retval;
    unsigned int engineMask;

    if (getParam(&param, &engineMask) != 0)
        engineMask = 0x3f;

    retval = uDbg_ShowBreakpoints(uDbg_Handle, engineMask, &uDbg_BkptArray);
    if (retval != UDBG_SUCCESS)
        printf(" - failed\n");
    else
    {
        for (int ii = 0; ii < uDbg_BkptArray.len; ii++)
        {
            printf(" - id 0x%02x ctx %d uAddr 0x%04x uEng %d brkCtx %d engMask 0x%02x\n",
                            uDbg_BkptArray.bkpts[ii].id,
                            uDbg_BkptArray.bkpts[ii].ctx,
                            uDbg_BkptArray.bkpts[ii].uAddr,
                            uDbg_BkptArray.bkpts[ii].uEng,
                            uDbg_BkptArray.bkpts[ii].breakIfCtx,
                            uDbg_BkptArray.bkpts[ii].stpOnBrkUengMask);
        }
    }
}



void ClearBP(char* param)
{
    int retval;
    unsigned int breakpoint_id;

    if (getParam(&param, &breakpoint_id) != 0)
    {
        printf(" - format: ClearBP <breakpoint_id>\n");
    }
    else
    {
        retval = uDbg_ClearBreakpoint(uDbg_Handle, breakpoint_id);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}



void ClearAllBP(char* param)
{
    int retval;
    unsigned int engineMask;

    if (getParam(&param, &engineMask) != 0)
    {
        printf(" - format: ClearAllBP <engineMask>\n");
    }
    else
    {
        retval = uDbg_ClearAllBreakpoints(uDbg_Handle, engineMask);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
    }
}



void SetGPR(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int bank;
    unsigned int regNum;
    unsigned int regData[16];
    unsigned int count;
    unsigned int ii;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &bank) != 0) ||
        (getParam(&param, &regNum) != 0))
    {
        printf(" - format: SetGPR <engNum> <thdNum> <bank> <regNum> <regData> ...\n");
        return;
    }

    if (regNum > 15)
    {
        printf(" - regNum range: [0:15]\n");
        return;
    }

    count = 0;
    for (ii = regNum; ii < 16; ii++)
    {
        if (getParam(&param, &regData[ii]) != 0)
            break;
        count++;
    }

    if (count == 0) // At least we need one register data item.
    {
        printf(" - format: SetGPR <engNum> <thdNum> <bank> <regNum> <regData> ...\n");
        return;
    }

    retval = uDbg_PutRelGPR(    uDbg_Handle,
                                engineNum,
                                threadNum,
                                (uDbg_Bank_T)bank,
                                regNum,
                                (unsigned int*)&regData[regNum],
                                count);
    if (retval != UDBG_SUCCESS)
        printf(" - failed\n");
}


void GetGPR(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int bank;
    unsigned int regNum;
    unsigned int regData[16];
    unsigned int count;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &bank) != 0) ||
        (getParam(&param, &regNum) != 0) ||
        (getParam(&param, &count) != 0))
    {
        printf(" - format: GetGPR <engNum> <thdNum> <bank> <regNum> <cnt>\n");
    }
    else
    {
        retval = uDbg_GetRelGPR(    uDbg_Handle,
                                    engineNum,
                                    threadNum,
                                    (uDbg_Bank_T)bank,
                                    regNum,
                                    (unsigned int*)regData,
                                    count);

        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
        {
            for (int ii = 0; ii < count; ii++)
                printf(" - regNum 0x%x: 0x%08x\n", regNum + ii, regData[ii]);
        }
    }
}


void SetSRAM_WrXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[8];
    unsigned int count;
    unsigned int ii;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0))
    {
        printf(" - format: SetSRAM_WrXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    if (regNum > 7)
    {
        printf(" - regNum range: [0:7]\n");
        return;
    }

    count = 0;
    for (ii = regNum; ii < 8; ii++)
    {
        if (getParam(&param, &regData[ii]) != 0)
            break;
        count++;
    }

    if (count == 0) // At least we need one register data item.
    {
        printf(" - format: SetSRAM_WrXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    retval = uDbg_PutRelSRAM_WrXfer(    uDbg_Handle,
                                        engineNum,
                                        threadNum,
                                        regNum,
                                        (unsigned int*)&regData[regNum],
                                        count);
    if (retval != UDBG_SUCCESS)
        printf(" - failed\n");
}




void GetSRAM_WrXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[16];
    unsigned int count;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0) ||
        (getParam(&param, &count) != 0))
    {
        printf(" - format: GetSRAM_WrXfer <engNum> <thdNum> <regNum> <cnt>\n");
    }
    else
    {
        retval = uDbg_GetRelSRAM_WrXfer(    uDbg_Handle,
                                            engineNum,
                                            threadNum,
                                            regNum,
                                            regData,
                                            count);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
        {
            for (int ii = 0; ii < count; ii++)
                printf(" - regNum 0x%x: 0x%08x\n", regNum + ii, regData[ii]);
        }
    }
}



void SetSRAM_RdXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[8];
    unsigned int count;
    unsigned int ii;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0))
    {
        printf(" - format: SetSRAM_RdXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    if (regNum > 7)
    {
        printf(" - regNum range: [0:7]\n");
        return;
    }

    count = 0;
    for (ii = regNum; ii < 8; ii++)
    {
        if (getParam(&param, &regData[ii]) != 0)
            break;
        count++;
    }

    if (count == 0) // At least we need one register data item.
    {
        printf(" - format: SetSRAM_RdXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    retval = uDbg_PutRelSRAM_RdXfer(    uDbg_Handle,
                                        engineNum,
                                        threadNum,
                                        regNum,
                                        (unsigned int*)&regData[regNum],
                                        count);
    if (retval != UDBG_SUCCESS)
        printf(" - failed\n");
}




void GetSRAM_RdXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[16];
    unsigned int count;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0) ||
        (getParam(&param, &count) != 0))
    {
        printf(" - format: GetSRAM_RdXfer <engNum> <thdNum> <regNum> <cnt>\n");
    }
    else
    {
        retval = uDbg_GetRelSRAM_RdXfer(    uDbg_Handle,
                                            engineNum,
                                            threadNum,
                                            regNum,
                                            regData,
                                            count);

        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
        {
            for (int ii = 0; ii < count; ii++)
                printf(" - regNum 0x%x: 0x%08x\n", regNum + ii, regData[ii]);
        }
    }
}


void SetSDRAM_WrXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[8];
    unsigned int count;
    unsigned int ii;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0))
    {
        printf(" - format: SetSDRAM_WrXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    if (regNum > 7)
    {
        printf(" - regNum range: [0:7]\n");
        return;
    }

    count = 0;
    for (ii = regNum; ii < 8; ii++)
    {
        if (getParam(&param, &regData[ii]) != 0)
            break;
        count++;
    }

    if (count == 0) // At least we need one register data item.
    {
        printf(" - format: SetSDRAM_WrXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    retval = uDbg_PutRelSDRAM_WrXfer(   uDbg_Handle,
                                        engineNum,
                                        threadNum,
                                        regNum,
                                        (unsigned int*)&regData[regNum],
                                        count);
    if (retval != UDBG_SUCCESS)
        printf(" - failed\n");
}




void GetSDRAM_WrXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[16];
    unsigned int count;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0) ||
        (getParam(&param, regData) != 0) ||
        (getParam(&param, &count) != 0))
    {
        printf(" - format: GetSDRAM_WrXfer <engNum> <thdNum> <regNum> <cnt>\n");
    }
    else
    {
        retval = uDbg_GetRelSDRAM_WrXfer(   uDbg_Handle,
                                            engineNum,
                                            threadNum,
                                            regNum,
                                            regData,
                                            count);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
        {
            for (int ii = 0; ii < count; ii++)
                printf(" - regNum 0x%x: 0x%08x\n", regNum + ii, regData[ii]);
        }
    }
}



void SetSDRAM_RdXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[8];
    unsigned int count;
    unsigned int ii;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0))
    {
        printf(" - format: SetSDRAM_RdXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    if (regNum > 7)
    {
        printf(" - regNum range: [0:7]\n");
        return;
    }

    count = 0;
    for (ii = regNum; ii < 8; ii++)
    {
        if (getParam(&param, &regData[ii]) != 0)
            break;
        count++;
    }

    if (count == 0) // At least we need one register data item.
    {
        printf(" - format: SetSDRAM_RdXfer <engNum> <thdNum> <regNum> <regData> ...\n");
        return;
    }

    retval = uDbg_PutRelSDRAM_RdXfer(   uDbg_Handle,
                                        engineNum,
                                        threadNum,
                                        regNum,
                                        (unsigned int*)&regData[regNum],
                                        count);
    if (retval != UDBG_SUCCESS)
        printf(" - failed\n");
}


void GetSDRAM_RdXfer(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int threadNum;
    unsigned int regNum;
    unsigned int regData[16];
    unsigned int count;

    if ((getParam(&param, &engineNum) != 0) ||
        (getParam(&param, &threadNum) != 0) ||
        (getParam(&param, &regNum) != 0) ||
        (getParam(&param, regData) != 0) ||
        (getParam(&param, &count) != 0))
    {
        printf(" - format: GetSDRAM_RdXfer <engNum> <thdNum> <regNum> <cnt>\n");
    }
    else
    {
        retval = uDbg_GetRelSDRAM_RdXfer(   uDbg_Handle,
                                            engineNum,
                                            threadNum,
                                            regNum,
                                            regData,
                                            count);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
        {
            for (int ii = 0; ii < count; ii++)
                printf(" - regNum 0x%x: 0x%08x\n", regNum + ii, regData[ii]);
        }
    }
}




uDbg_uEngStatus_T uEngStatus;

void GetUengStatus(char* param)
{
    int retval;
    unsigned int engineNum;
    unsigned int engineMask;

    if (getParam(&param, &engineMask) != 0)
        engineMask = 0x3f;

    for (engineNum = 0; engineNum < 6; engineNum++)
    {
        if (engineMask & (1 << engineNum))
        {
            retval = uDbg_GetUengStatus(    uDbg_Handle,
                                            engineNum,
                                            &uEngStatus);

            if (retval != UDBG_SUCCESS)
                printf(" - eng%d: failed\n", engineNum);
            else
            {
                printf(" - eng%d: ctx%d ",
                            engineNum,
                            uEngStatus.ctx);

                if (uEngStatus.state == UDBG_UENGACT)
                    printf("running  ");
                else if (uEngStatus.state == UDBG_UENGINACT)
                    printf("stopped  ");
                else if (uEngStatus.state == UDBG_UENGATBREAK)
                    printf("break %02d ", uEngStatus.breakId);
                else
                    printf("state: %x ", uEngStatus.state);

                printf("upc:0x%04x ", uEngStatus.pc);

                printf("enFlags: ctx=0x%01x ", uEngStatus.enables & 0xF);
                printf("cc=%d ", (uEngStatus.enables >> 6) & 3);
                printf("csrWrt=%d ", (uEngStatus.enables >> 5) & 1);
                printf("sramJou=%d", (uEngStatus.enables >> 4) & 1);
                printf("\n");
            }
        }
    }
}


void HopUeng(char* param)
{
    int retval;
    unsigned int delay;
    unsigned int engineMask;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &delay) != 0))
    {
        printf(" - format: HopUeng <engineMask> <delay>\n");
    }
    else
    {
        retval = uDbg_HopUeng(uDbg_Handle, &engineMask, delay);
        if (retval != UDBG_SUCCESS)
            printf(" - failed\n");
        else
            printf(" - Active eng: 0x%x\n", engineMask);
    }
}

#if 0 // Not ready yet.

void Cntl_RegControl(char* param)
{
    int retval;
    unsigned int controlType;
    unsigned int callbackFunc;
    unsigned int userData;

    if ((getParam(&param, &controlType) != 0) ||
        (getParam(&param, &callbackFunc) != 0) ||
        (getParam(&param, &userData) != 0))

    {
        printf(" - format: Cntl_RegControl <cntlType> <callback> <userData>\n");
    }
    else
    {
#if 0
        retval = CNTL_RegisterControl(  (ControlType)controlType,
                                        (FUNC_CONTROL)callbackFunc,
                                        (void*)userData);
#endif
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_RegInitUof(char* param)
{
    int retval;
    unsigned int controlType;
    unsigned int callbackFunc;
    unsigned int userData;

    if ((getParam(&param, &controlType) != 0) ||
        (getParam(&param, &callbackFunc) != 0) ||
        (getParam(&param, &userData) != 0))

    {
        printf(" - format: Cntl_RegInitUof <cntlType> <callback> <userData>\n");
    }
    else
    {
#if 0
        retval = CNTL_RegisterInitUof(  (ControlType)controlType,
                                        (FUNC_INITUOF)callbackFunc,
                                        (void*)userData);
#endif
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_GiveControl(char* param)
{
    int retval;
    unsigned int engineMask;

    if (getParam(&param, &engineMask) != 0)
    {
        printf(" - format: Cntl_GiveControl <engineMask>\n");
    }
    else
    {
#if 0 /* ??? */
        retval = CNTL_GiveControl(engineMask);
#endif
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_TakeControl(char* param)
{
    int retval;
    unsigned int engineMask;

    if (getParam(&param, &engineMask) != 0)
    {
        printf(" - format: Cntl_TakeControl <engineMask>\n");
    }
    else
    {
#if 0 /* ??? */
        retval = CNTL_TakeControl(engineMask);
#endif
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_InitUof(char* param)
{
    int retval;
    unsigned int buf;
    unsigned int buflen;

    if ((getParam(&param, &buf) != 0) ||
        (getParam(&param, &buflen) != 0))
    {
        printf(" - format: Cntl_InitUof <buf> <buflen>\n");
    }
    else
    {
        retval = CNTL_InitUof((char*)buf, (int)buflen);
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_Reset(char*param)
{
    int retval;
    unsigned int engineMask;
    unsigned int portMask;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &portMask) != 0))
    {
        printf(" - format: Cntl_Reset <engineMask> <portMask>\n");
    }
    else
    {
        retval = CNTL_Reset(uDbg_Handle, engineMask, portMask);
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_Start(char*param)
{
    int retval;
    unsigned int engineMask;
    unsigned int portMask;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &portMask) != 0))
    {
        printf(" - format: Cntl_Start <engineMask> <portMask>\n");
    }
    else
    {
        retval = CNTL_Start(uDbg_Handle, engineMask, portMask);
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_Pause(char*param)
{
    int retval;
    unsigned int engineMask;
    unsigned int portMask;
    unsigned int delay;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &portMask) != 0) ||
        (getParam(&param, &delay) != 0))
    {
        printf(" - format: Cntl_Pause <engineMask> <portMask> <delay>\n");
    }
    else
    {
        retval = CNTL_Pause(uDbg_Handle, engineMask, portMask, delay);
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_Resume(char*param)
{
    int retval;
    unsigned int engineMask;
    unsigned int portMask;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &portMask) != 0))
    {
        printf(" - format: Cntl_Resume <engineMask> <portMask>\n");
    }
    else
    {
        retval = CNTL_Resume(uDbg_Handle, engineMask, portMask);
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

void Cntl_Hop(char*param)
{
    int retval;
    unsigned int engineMask;
    unsigned int portMask;
    unsigned int delay;

    if ((getParam(&param, &engineMask) != 0) ||
        (getParam(&param, &portMask) != 0) ||
        (getParam(&param, &delay) != 0))
    {
        printf(" - format: Cntl_Hop <engineMask> <portMask> <delay>\n");
    }
    else
    {
        retval = CNTL_Hop(uDbg_Handle, engineMask, portMask, delay);
        if (retval != CNTL_SUCCESS)
            printf(" - failed\n");
    }
}

#endif // Not ready yet.

// end of file

