/* uload_cmd.c
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: console
 * author: Henry Qian 03/04/99
 * revisions:
 *
 */
#include <stdio.h>
#include <string.h>
#include "os.h"
#include "uload_cmd.h"
#include "uclo.h"

ConsoleCommand uload_cmd[] = {
    {"uLoad",               uLoad,              "Load a micro code file"},
    {"InitUcLoLib",         InitUcLoLib,        "Init uCode Loader Library"},
    {"LoadUofFile",         LoadUofFile,        "Load UOF file to core memory"},
    {"DeleteUofFile",       DeleteUofFile,      "Remove all refs to a uCode obj"},
    {"BindSymbol",          BindSymbol,         "Bind an app value to a uCode symbol"},
    {"GetUFuncID",          GetUFuncID,         "Get uCode function ID"},
    {"WriteUImageAll",      WriteUImageAll,     "Write all uImages to uEngines"},
    {"WriteUImage",         WriteUImage,        "Write a specific uImage to uEngines"},
    {"VerifyUEngine",       VerifyUEngine,      "Compare uEng's contents to its uImage"},
    {"VerifyUImage",        VerifyUImage,       "Compare a uImage to its uEng Content"},

    {(char*)0,  0,      0}
};

void uLoad(char* param)
{
    int retval;
    char fileName[MAXNAME + 1];

    if (getStringParam(&param, fileName) != 0)
        printf(" - format: uLoad <fileName>\n");
    else
    {
        InitUcLoLib("");
        LoadUofFile(fileName);
        WriteUImageAll("");
    }
}

void InitUcLoLib(char*)
{
    UcLo_InitLib();
}


void * UcLo_handle = 0;

void LoadUofFile(char* param)
{
    int retval;
    char fileName[MAXNAME + 1];

    if (getStringParam(&param, fileName) != 0)
        printf(" - format: LoadUofFile <fileName>\n");
    else
    {
        retval = UcLo_LoadObjFile(&UcLo_handle, fileName);

        switch (retval)
        {
            case UCLO_FILEFAIL:
                printf(" - file operation failed\n");
                break;

            case UCLO_MEMFAIL:
                printf(" - memory alocation failed\n");
                break;
          
            case UCLO_BADARG:
                printf(" - invalid argumemt\n");
                break;
          
            case UCLO_BADOBJ:
                printf(" - error in file or checksum\n");
                break;
          
            case UCLO_SUCCESS:
                break;

            default:
                printf(" - failed\n");
                break;
        }
    }
}


void DeleteUofFile(char*)
{
    int retval = UcLo_DeleObj(UcLo_handle);

    if (retval != UCLO_SUCCESS)
        printf(" - failed\n");

    UcLo_handle = 0;
}



void BindSymbol(char* param)
{
    int retval;
    char imageName[MAXNAME + 1];
    char symbolName[MAXNAME + 1];
    unsigned int initValue;

    if ((getStringParam(&param, imageName) != 0) ||
        (getStringParam(&param, symbolName) != 0) ||
        (getParam(&param, &initValue) != 0))
    {
        printf(" - format: BindSymbol <imageName> <symbolName> <initValue>\n");
    }
    else
    {
        retval = UcLo_BindSymbol(UcLo_handle, imageName, symbolName, initValue);

        switch (retval)
        {
            case UCLO_BADARG:
                printf(" - invalid argumemt\n");
                break;
          
            case UCLO_NOOBJ:
                printf(" - no object was either loaded or mapped\n");
                break;

            case UCLO_IMGNOTFND:
                printf(" - image not found\n");
                break;
          
            case UCLO_SYMNOTFND:
                printf(" - symbol not found\n");
                break;
          
            case UCLO_SUCCESS:
                break;

            default:
                printf(" - failed\n");
                break;
        }
    }
}





void GetUFuncID(char* param)
{
    int retval;
    char funcName[MAXNAME + 1];

    if (getStringParam(&param, funcName) != 0)
    {
        printf(" - format: GetUFuncID <funcName>\n");
    }
    else
    {
        retval = UcLo_GetUfuncId(UcLo_handle, funcName);

        if ((retval < 0) || (retval > 255))
            printf(" - failed, retval: %d\n", retval);
        else
            printf(" - uCode function ID: %d\n", retval);
    }
}



void WriteUImageAll(char*)
{
    int retval;

    retval = UcLo_WriteUimageAll(UcLo_handle);

    if (retval != UCLO_SUCCESS)
        printf(" - failed\n");
}



void WriteUImage(char* param)
{
    int retval;
    char imageName[MAXNAME + 1];

    if (getStringParam(&param, imageName) != 0)
    {
        printf(" - format: WriteUImage <imageName>\n");
    }
    else
    {
        retval = UcLo_WriteUimage(UcLo_handle, imageName);

        switch (retval)
        {
            case UCLO_NOOBJ:
                printf(" - no object was either loaded or mapped\n");
                break;

            case UCLO_IMGNOTFND:
                printf(" - image not found\n");
                break;
          
            case UCLO_BADARG:
                printf(" - invalid argumemt\n");
                break;
          
            case UCLO_SUCCESS:
                break;

            default:
                printf(" - failed\n");
                break;
        }
    }
}




void VerifyUEngine(char* param)
{
    int retval;
    unsigned int engineNum;

    if (getParam(&param, &engineNum) != 0)
    {
        printf(" - format: VerifyUEngine <engineNum>\n");
    }
    else
    {
        retval = UcLo_VerifyUengine(UcLo_handle, engineNum);

        switch (retval)
        {
            case UCLO_NOOBJ:
                printf(" - no object was either loaded or mapped\n");
                break;

            case UCLO_IMGNOTFND:
                printf(" - image not found\n");
                break;
          
            case UCLO_BADARG:
                printf(" - invalid argumemt\n");
                break;
          
            case UCLO_SUCCESS:
                break;

            default:
                printf(" - failed\n");
                break;
        }
    }
}



void VerifyUImage(char* param)
{
    int retval;
    char imageName[MAXNAME + 1];

    if (getStringParam(&param, imageName) != 0)
    {
        printf(" - format: VerifyUImage <imageName>\n");
    }
    else
    {
        retval = UcLo_VerifyUimage(UcLo_handle, imageName);

        switch (retval)
        {
            case UCLO_NOOBJ:
                printf(" - no object was either loaded or mapped\n");
                break;

            case UCLO_IMGNOTFND:
                printf(" - image not found\n");
                break;
          
            case UCLO_BADARG:
                printf(" - invalid argumemt\n");
                break;
          
            case UCLO_SUCCESS:
                break;

            default:
                printf(" - failed\n");
                break;
        }
    }
}


#if 0 // Will be ported later, after PCI module.  Henry

void LoadFuncID(char* param)
{
    int retval;
    unsigned int funcID;

    if (getParam(&param, &funcID) != 0)
    {
        printf(" - format: LoadFuncID <funcID>\n");
    }
    else
    {
        retval = UcLoPci_Init(UcLo_handle, funcID);

        switch (retval)
        {
          
            case UCLO_BADARG:
                printf(" - invalid argumemt\n");
                break;
          
            case UCLO_SUCCESS:
                break;

            default:
                printf(" - failed\n");
                break;
        }
    }
}

#endif // Henry

// end of file
