; boot.s
;
;------------------------------------------------------------
;                                                                      
;                  I N T E L   P R O P R I E T A R Y                   
;                                                                      
;     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;     THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                      
;                        INTEL  CORPORATION                            
;                                                                     
;                     2200 MISSION COLLEGE BLVD                        
;                                                                      
;               SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                      
;------------------------------------------------------------
; system: SA1200
; subsystem: uHAL
; author: Henry Qian 04/08/99
; revisions:
;

        INCLUDE platform.s
        INCLUDE except_h.s
        GET target.s    ; Target specific macros

        IMPORT  main    ; Start of user program

        IMPORT  SAir_DebugUndefinedInst ;
        IMPORT  SAir_PlatformInit       ; in platform.c
        IMPORT  SAr_DisableInt          ;
    
        EXPORT  |__main|                ; Program Entry Point

        EXPORT  SAr_TrapSWI             ; SWI exception handler
        EXPORT  SAiv_BaseOfFreeMemory
        EXPORT  SAiv_TopOfFreeMemory

CRASH_INFO_BUF_SIZE EQU 0x1000

        AREA    SelfBoot, CODE, READONLY

        ; Start of executable code - label must follow ENTRY for scripts
        ; to automatically convert between different assemblers.
        ENTRY
|__main| 
        ; Ensure the processor is in SVC32 mode, with IRQs and FIQs
        ; disabled before continuing with the initialisation.

        MRS     r14,CPSR                ; Get the processor status
        BIC     r14,r14,#ModeMask
        ORR     r14,r14,#(SVC32Mode + NoInt)
        MSR     CPSR,r14                ; SVC 32 mode with interrupts disabled

        ; The following macro is from Angel's target.s
        ; It initializes SDRAM, SRAM, caches, and MMU.
        ;
        STARTUPCODE     r0,r1,r4,r5

        ; Vectors are copied to RAM at zero. 
        ; SAip_LastVector should be after _FIQVector _and_ the FIQ interrupt
        ; code which may follow to avoid an unnecessary branch. 

        ; First 8 addresses are interrupt vectors.
        ;
        ;   Addr.   Exception Type          Mode    Priority
        ;   0x00    Reset                   svc     1   (1 = High)
        ;   0x04    Undefined instruction   undef   6   (6 = Low)
        ;   0x08    Software Interrupt SWI  svc     6
        ;   0x0C    Prefetch Abort          abort   5
        ;   0x10    Data Abort              abort   2
        ;   0x14    Reserved                NA      N/A
        ;   0x18    Interrupt IRQ           irq     4
        ;   0x1C    Fast Interrupt FIQ      fiq     3
        ;

        ; NOTE: Can't use simple branches which are relative and would
        ;   be wrong when moved!
SAip_FirstVector
        LDR PC, SAip_ResetStart
        LDR PC, SAip_UndefVector
        LDR PC, SAip_SWIVector
        LDR PC, SAip_PreAbtVector
        LDR PC, SAip_DataAbtVector
        LDR PC, SAip_ReservdVector
        LDR PC, SAip_IRQVector
        LDR PC, SAip_FIQVector  ; Routine for FIQs - normally code is
                                ; placed here for maximum speed.

        ; If this were a real embedded application, all these vectors
        ; would have to be properly defined.

SAip_ResetStart     DCD SAir_ResetGo    ; Branch on Reset & go
SAip_UndefVector    DCD SAir_UndefProcess ; Reset on unknown instruction
SAip_SWIVector      DCD SAr_TrapSWI ; Routine to handle SWI
SAip_PreAbtVector   DCD SAir_ResetStop  ; Not handled in this example..
SAip_DataAbtVector  DCD SAir_ResetStop
SAip_ReservdVector  DCD SAir_ResetStop
SAip_IRQVector      DCD SAir_IRQProcess ; Routine for IRQs
SAip_FIQVector      DCD SAir_ResetStop  ; Routine for FIQs

SAip_LastVector     ; EVERYTHING before this point is copied to RAM

        ; Skip over vector table & FIQ code (if any) to HERE
SAir_ResetStop

        ; If DEBUG is not defined, ResetStop and ResetGo are the same.

 IF :DEF: DEBUG
        ; Post Mortem assumes memory system (uses stacks) is working and
        ; OS_COMPORT has been initialised (see below).
        B   PostMortem
        NOP
        NOP
        NOP
 ENDIF

SAir_ResetGo
        ; Copy vectors (and any in-line FIQ code) to RAM at zero.
        MOV r0, #0
        LDR r1, =SAip_FirstVector   ; Copy from here..
        LDR r2, =SAip_LastVector    ; ..to here
03
        LDR r3, [r1], #4    ; get word and store to RAM  
        STR r3, [r0], #4
        CMP r1, r2          ; Repeat until all done
        BLO %03

        ; The following linker generated symbols are used to control
        ; the ROM-to-RAM copy loop:
        ;
        IMPORT  |Image$$RO$$Base|  ; start of ROM code
        IMPORT  |Image$$RO$$Limit| ; end of ROM code
        IMPORT  |Image$$RW$$Base|  ; destination address of initialised R/W
        IMPORT  |Image$$ZI$$Base|  ; start of the zero initialised (BSS) data.
        IMPORT  |Image$$RW$$Limit| ; used to size initialised R/W section

        LDR     r1, =|Image$$RO$$Limit| ; source address
        LDR     r2, =|Image$$RW$$Base|  ; destination address
        LDR     r3, =|Image$$ZI$$Base|  ; limit

        ; For ROM based applications we must perform the work of
        ; initialising the RW initialised and BSS data here.
        ;
        ; Copy the data to the final RAM address:
01  
        LDR     r0,[r1],#4              ; get from ROM
        STR     r0,[r2],#4              ; store to RAM
        CMP     r2,r3                   ; check for end condition
        BLO     %BT01                   ; if (r2 < r3) we have more to transfer

        ; Now initialise the BSS area.
        ;
        LDR     r3,=|Image$$RW$$Limit|
        MOV     r0,#0
02
        STR     r0,[r2],#4
        CMP     r2,r3
        BLO     %BT02

        ; Start of free memory region up to system stack.
        ;
        ADD r3, r3, #0x10   ; precaution
        ; ADRL r0, SAiv_BaseOfFreeMemory
        ; The above line is replaced by following two lines due to compile err.
        LDR r0, =SAiv_BaseOfFreeMemory
        STR r3, [r0]

        ; Setup the area to keep system infomation in case of crash
        ;
        LDR r0, =(DRAM_SIZE - TTB_SIZE - CRASH_INFO_BUF_SIZE)
        ; ADRL r1, SAiv_BaseOfCrashInfoArea
        ; The above line is replaced by following two lines due to compile err.
        LDR r1, =SAiv_BaseOfCrashInfoArea
        STR r0, [r1]
        SUB r0, r0, #4

        ; Now set up the stack pointers for all the different modes.
        ; r0 points to the top of system stacks.
        ;
        MRS r4, cpsr
        ORR r4, r4, #NoIRQ      ; No IRQs & blank mode
        ORR r4, r4, #NoFIQ      ; Or FIQs
        BIC r4, r4, #ModeMask
        MOV r2, #User32Mode

        ; Abort mode
        ORR r5, r4, #Abort32Mode
        MSR cpsr, r5
        MSR spsr, r2
        BIC r0, r0, #3      ; Must be word-aligned
        MOV sp, r0          ; Initial Abort stack pointer
        SUB r0, r0, #AbortStackSize

        ; Undefined mode
        ORR r5, r4, #Undef32Mode
        MSR cpsr, r5
        MSR spsr, r2
        BIC r0, r0, #3      ; Must be word-aligned
        MOV sp, r0          ; Initial Undefined stack pointer
        SUB r0, r0, #UndefStackSize

        ; FIQ mode
        ORR r5, r4, #FIQ32Mode
        MSR cpsr, r5
        MSR spsr, r2
        BIC r0, r0, #3      ; Must be word-aligned
        MOV sp, r0          ; Initial FIQ stack pointer
        SUB r0, r0, #FIQStackSize

        ; IRQ mode
        ORR r5, r4, #IRQ32Mode
        MSR cpsr, r5
        MSR spsr, r2
        BIC r0, r0, #3      ; Must be word-aligned
        MOV sp, r0          ; Initial IRQ stack pointer
        SUB r0, r0, #IRQStackSize

        ; Supervisor mode
        ORR r5, r4, #SVC32Mode
        MSR cpsr, r5
        MSR spsr, r2
        BIC r0, r0, #3      ; Must be word-aligned
        MOV sp, r0          ; Initial SVC stack pointer
        SUB r0, r0, #SVCStackSize
        MOV r10, r0         ; Bottom of SVC stack, sl

        ; Setup User/System mode stacks.
        ORR r5, r4, #User32Mode
        MSR cpsr, r5
        BIC r0, r0, #3      ; Must be word-aligned
        MOV sp, r0          ; Initial User stack pointer...
        SUB r0, r0, #UserStackSize
        MOV r10, r0         ; Bottom of stacks, sl

        SUB r0, r0, #4
        ; The above line is replaced by following two lines due to compile err.
        ; ADRL r1, SAiv_TopOfFreeMemory
        LDR r1, =SAiv_TopOfFreeMemory
        STR r0, [r1]

        SWI SWI_EnterOS     ; Back to supervisor mode

        MOV r11, #0         ; No previous frame, so fp=0

        ; Do the C based platform specific platform initialization
        BL  SAir_PlatformInit   ; Everything required to init HW
                                        
        ; ARM Procedure Call Standard nomenclature
        MOV r0, #0          ; set argc to 0
        MOV r1, #0          ; and argv to NUL
        BL  main            ; Call main, falling through to
                            ; exit on return.

; SWI handler - currently only handles switch to SVC mode.
SAr_TrapSWI
        ; SWI switches to SVC mode, with no IRQs (ARM Architecture Reference)
        STMFD   sp!, {a1}
        LDR a1, [lr, #-4]       ; Read SWI instruction
        BIC a1, a1, #0xff000000 ; extract the SWI number
        CMP a1, #SWI_EnterOS
        LDMFD   sp!, {a1}       ; restore register
        BEQ SAir_SWIEnterOS     ; return in SVC mode
        MOVS    pc, lr          ; just ignore SWI

SAir_SWIEnterOS
        MOV pc, lr          ; return without restoring mode!


; Dummy IRQ handler - gets overwritten
SAir_IRQProcess
        STMFD   sp!, {v1}
        MRS v1, cpsr        ; Re-enable interrupt
        BIC v1, v1, #NoIRQ
        MSR cpsr, v1
        LDMFD   sp!, {v1}
        SUBS    pc, r14, #4     ; return from IRQ


; Try and capture some sensible data after undefined instruction.
SAir_UndefProcess
    MOV a1, sp
    ; Hacked from vector copy. Should be done properly to a system
    ; defined address.
    MOV r0, #0
    LDR r1, =SAip_FirstVector   ; Copy from here..
    LDR r2, =SAip_LastVector    ; ..to here
    MOV r6, #0x00010000     ; Debug - make a copy of originals
    MOV r5, r14         ; Old return address (if any)
    STR r5, [r6], #4
    MRS r5, cpsr        ; CPSR
    STR r5, [r6], #4
    MRC p15, 0, r5, c5, c0, 0   ; Coproc 5 - 1 byte of status
    STR r5, [r6], #4
    MRC p15, 0, r5, c6, c0, 0   ; Coproc 6 - Fault address
    STR r5, [r6],#4
    MOV r5, sp
    STR r4, [r6],#4     ; Old Stack
    STR r5, [r6],#4     ; New Stack
_undefLoop
    LDR r4, [r5], #-4       ; Look at stack
    STR r4, [r6], #4
    CMP r1,r2           ; Repeat until all done
    BLO _undefLoop

    ; Okay, we've done enough, add further specific initialisation here
;   BL  SAir_DebugUndefinedInst
_Undef4ever
    B   _Undef4ever     ; Try and restart..


 IF :DEF: DEBUG

    IMPORT  sputchar
    EXPORT  PostMortem
    EXPORT  spitword

;===========================================================================
; Ths routine will put a word value via sputchar
;===========================================================================
PutWord
    mov r4, lr
    mov r5, r0
    mov r0, #0x20       ; ' '
    bl  sputchar
PutWord2
    mov r3, #28
01  mov r0, r5, LSR r3
    ands    r0, r0, #0xf
    bne %f02            ; the first non-zero
;   subs    r3, r3, #4
;   bne %b01            ; all but the last digit
;   mov r0, r5          ; Get the only digit
02  cmp r0, #10
    addlo   r0, r0, #0x30       ; 0 - 9
    addhs   r0, r0, #0x61 - 10  ; a - f
    bl  sputchar
    subs    r3, r3, #4
    mov r0, r5, LSR r3
    and r0, r0, #0xf
    bpl %b02
    mov pc, r4

; 'C' callable version of PutWord
spitword
    STMFD   sp!, {r3-r5, lr}
    BL  PutWord
    LDMFD   sp!, {r3-r5, pc}

spitstr
    MOV r4, lr
    MOV r5, r0
3
    LDRB    r0, [r5], #1
    CMP r0, #0
    MOVEQ   pc, r4

    BL  sputchar
    B   %B03

;===========================================================================
; It's dead Jim, dead. Who, what where, when, how?
;
; 'Routine' to print out r0-r12, r14, spsr, fsr & far, ttb, tlb & contents
; Called only when debug system crashes (doesn't return). Uses stack so
; memory must be working and sputchar() - serial-only version of putchar().
; Make sure 'DEBUG' is defined for 'C' too.
;===========================================================================
PostMortem
    STMFD   sp!, {r9-r12, r14}  ; X marks the spot
    MOV r9, r0
    MOV r10, r3
    MOV r11, r4
    MOV r12, r5

    LDR r0, =Mort1
    BL  spitstr
    MOV r0, r9
    BL  PutWord         ; r0
    MOV r0, r1
    BL  PutWord         ; r1
    MOV r0, r2
    BL  PutWord         ; r2
    MOV r0, r10
    BL  PutWord         ; r3
    MOV r0, r11
    BL  PutWord         ; r4
    MOV r0, r12
    BL  PutWord         ; r5
    MOV r0, r6
    BL  PutWord         ; r6
    LDR r0, =Mort2
    BL  spitstr
    MOV r0, r7
    BL  PutWord         ; r7
    MOV r0, r8
    BL  PutWord         ; r8
    LDMFD   sp!, {r9-r12, r14}  ; X marks the spot
    MOV r8, r14
    MOV r0, r9
    BL  PutWord         ; r9
    MOV r0, r10
    BL  PutWord         ; r10
    MOV r0, r11
    BL  PutWord         ; r11
    MOV r0, r12
    BL  PutWord         ; r12
    LDR r0, =Mort3
    BL  spitstr
    MOV r0, r8
    BL  PutWord         ; r14

    ; Mode before exception
    LDR r0, =Mort4
    BL  spitstr
    MRS r0, spsr
    BL  PutWord

    ; Read Fault Status register and Fault Address register
    LDR r0, =Mort5
    BL  spitstr
    RDCP15_FaultStatus  r0
    BL  PutWord
    LDR r0, =Mort6
    BL  spitstr
    RDCP15_FaultAddress r0
    MOV r8, r0, LSR #20     ; Strip irrelevent bits
    BL  PutWord
    ; Read Translation Table Base reg.
    LDR r0, =Mort7
    BL  spitstr
    RDCP15_TTBase    r0
    MOV r9, r0, LSR #14     ; Strip irrelevent bits
    BL  PutWord

    ; Now rebuild address of page table entry
    MOV r9, r9, LSL #14
    ORR r9, r9, r8, LSL #2
    LDR r0, =Mort8
    BL  spitstr
    MOV r0, r9
    BL  PutWord
    LDR r0, =':'
    BL  sputchar
    LDR r0, [r9]
    BL  PutWord
    LDR r0, ='\n'
    BL  sputchar        ; Newline
99
    ; Loop forever..
    B   %B99

Mort1   DCB "\nr0-r6\t", 0
Mort2   DCB "\nr7-r12\t", 0
Mort3   DCB "\nr14\t", 0
Mort4   DCB "\nspsr\t", 0
Mort5   DCB "\tfsr", 0
Mort6   DCB "\tfar", 0
Mort7   DCB "\nttb\t", 0
Mort8   DCB "\ttlb", 0

 ENDIF
    LTORG

    AREA SA_BootRam, DATA


SAiv_BaseOfFreeMemory       % 4 ; Will be managed by malloc() and
SAiv_TopOfFreeMemory        % 4 ; free().
SAiv_BaseOfCrashInfoArea    % 4 ; Area to keep crash situation.

exit

    END

