/* cache.c
 *
 *
 * --------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------
 * system: SA1200
 * subsystem: uHAL 
 * author: Henry Qian 4/9/99
 * revisions:
 *
 */

#include "platform.h"
#include "coprocessor.h"

extern unsigned int SAir_ReadControlReg(void);
extern void SAir_WriteControlReg(unsigned int);

/*
**  void SAr_EnableCache(void)
**
**  DESCRIPTION
**      Enable all caching
**  
**  RETURNS
**      None.
*/
void SAr_EnableCache(void)
{
    unsigned int mode;

    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode | IC_ON | DC_ON | WB_ON) & 0x1FFF);
}

/*
**  void SAir_EnableICache(void)
**
**  DESCRIPTION
**      Enable instruction cache
**  
**  RETURNS
**      None.
*/
void SAir_EnableICache(void)
{
    unsigned int mode;
    
    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode | IC_ON) & 0x1FFF);
}

/*
**  void SAir_EnableDCache(void)
**
**  DESCRIPTION
**      Enable data cache
**  
**  RETURNS
**      None.
*/
void SAir_EnableDCache(void)
{
    unsigned int mode;
    
    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode | DC_ON) & 0x1FFF);
}

/*
**  void SAir_EnableWriteBuffer(void)
**
**  DESCRIPTION
**      Enable write buffer
**  
**  RETURNS
**      None.
*/
void SAir_EnableWriteBuffer(void)
{
    unsigned int mode;
    
    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode | WB_ON) & 0x1FFF);
}


/*
**  void SAr_DisableCache(void)
**
**  DESCRIPTION
**      Disable all caching
**  
**  RETURNS
**      None.
*/
void SAr_DisableCache(void)
{
    unsigned int mode;

    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode & ~(IC_ON | DC_ON | WB_ON)) & 0x1FFF);
}


/*
**  void SAir_DisableICache(void)
**
**  DESCRIPTION
**      Disable instruction cache
**  
**  RETURNS
**      None.
*/
void SAir_DisableICache(void)
{
    unsigned int mode;
    
    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode & ~IC_ON) & 0x1FFF);
}

/*
**  void SAir_DisableDCache(void)
**
**  DESCRIPTION
**      Disable data cache
**  
**  RETURNS
**      None.
*/
void SAir_DisableDCache(void)
{
    unsigned int mode;
    
    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode & ~DC_ON) & 0x1FFF);
}


/*
**  void SAir_DisableWriteBuffer(void)
**
**  DESCRIPTION
**      Disable write buffer
**  
**  RETURNS
**      None.
*/
void SAir_DisableWriteBuffer(void)
{
    unsigned int mode;
    
    mode = SAir_ReadControlReg();
    SAir_WriteControlReg((mode & ~WB_ON) & 0x1FFF);
}


/* end of file */

