; interrupt.s
;
;------------------------------------------------------------
;                                                                      
;                  I N T E L   P R O P R I E T A R Y                   
;                                                                      
;     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;     THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                      
;                        INTEL  CORPORATION                            
;                                                                     
;                     2200 MISSION COLLEGE BLVD                        
;                                                                      
;               SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                      
;------------------------------------------------------------
; system: SA1200
; subsystem: uHAL
; author: Henry Qian 04/09/99
; revisions:
;

    INCLUDE except_h.s

	AREA	|C$$code$$intRoutines|, CODE, READONLY

; -------------------------
;   IRQ - Regular Interrupt
; -------------------------

;	IRQStackInit - Install IRQ stack
;
;   Stack has to be word boundary.
;
	EXPORT	SAr_IRQStackInit
SAr_IRQStackInit
    STMFD   sp!, {v1-v2, lr}
    MRS     v1, cpsr
    BIC     v2, v1, #ModeMask
    ORR     v2, v2, #(NoInt | IRQ32Mode)
    MSR     cpsr, v2

    BIC     a1, a1, #3  ; make it long word aligned
    MOV     sp, a1
    MSR     cpsr, v1
    LDMFD   sp!, {v1-v2, pc}


;	IRQTrapWrap - IRQ trap handler
;
;	Wrapper to service the trap and call a high-level handler.
;
    IMPORT  SA_IRQ_Dispatcher
	EXPORT	SAir_IRQTrapWrap
SAir_IRQTrapWrap
    STMFD   sp!, {r0-r12, lr}   ; save registers
    MRS     v1, SPSR            ; push SPSR to allow nested interrupts
    STMFD   sp!, {v1}

    BL      SA_IRQ_Dispatcher

    LDMFD   sp!, {v1}           ; pop SPSR
    MSR     SPSR, v1
    LDMFD   sp!, {r0-r12, lr}   ; restore registers
    SUBS    pc, lr, #4          ; return from trap
 

; ----------------------
;   FIQ - Fast Interrupt
; ----------------------

;	FIQStackInit - Install FIQ stack
;
;   Stack has to be word boundary.
;
	EXPORT	SAr_FIQStackInit
SAr_FIQStackInit
    STMFD   sp!, {v1-v2, lr}
    MRS     v1, cpsr
    BIC     v2, v1, #ModeMask
    ORR     v2, v2, #(NoInt | FIQ32Mode)
    MSR     cpsr, v2

    BIC     a1, a1, #3  ; make it long word aligned
    MOV     sp, a1
    MSR     cpsr, v1
    LDMFD   sp!, {v1-v2, pc}


;	FIQTrapWrap - FIQ trap handler
;
;	Wrapper to service the trap and call a high-level handler.
;
    IMPORT  SA_FIQ_Dispatcher
	EXPORT	SAir_FIQTrapWrap
SAir_FIQTrapWrap
    STMFD   sp!, {r0-r12, lr}   ; save registers
    MRS     v1, SPSR            ; push SPSR to allow nested interrupts
    STMFD   sp!, {v1}

    BL      SA_FIQ_Dispatcher

    LDMFD   sp!, {v1}           ; pop SPSR
    MSR     SPSR, v1
    LDMFD   sp!, {r0-r12, lr}   ; restore registers
    SUBS    pc, lr, #4          ; return from trap
 


;	void SAr_DisableInt(void)
;	void SAr_EnableInt(unsigned int)
;
;	Disable and enable IRQ and FIQ preserving current CPU mode.
;
	EXPORT	SAr_DisableInt
SAr_DisableInt
    STMFD   sp!, {v1,lr}
    MRS     v1, cpsr
    AND     a1, v1, #NoInt
    ORR     v1, v1, #NoInt
    MSR     cpsr, v1
    LDMFD   sp!, {v1,pc}


	EXPORT	SAr_EnableInt
SAr_EnableInt
    STMFD   sp!, {v1, lr}
    MRS     v1, cpsr
    BIC     v1, v1, #NoInt
    ORR     v1, v1, a1
    MSR     cpsr, v1
    LDMFD   sp!, {v1, pc}


	EXPORT	SAr_GetCPSR
SAr_GetCPSR
    MRS     a1, cpsr
    MOV     pc, lr


	EXPORT	SAr_SetCPSR
SAr_SetCPSR
    MSR     cpsr, a1
    MOV     pc, lr


	END
