/* platform.h
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: Angel Debug Monitor
 * author: Henry Qian 01/29/99
 * revisions:
 * Dan White 
 *	10/18/99 - added FLASH_SIZE
 *
 * 03/31/00 - Dan White - added Rev B0 changes
 *
 *
 */

#ifndef __address_h
#define __address_h                     1

#define BIT0                            0x00000001
#define BIT1                            0x00000002
#define BIT2                            0x00000004
#define BIT3                            0x00000008
#define BIT4                            0x00000010
#define BIT5                            0x00000020
#define BIT6                            0x00000040
#define BIT7                            0x00000080
#define BIT8                            0x00000100
#define BIT9                            0x00000200
#define BIT10                           0x00000400
#define BIT11                           0x00000800
#define BIT12                           0x00001000
#define BIT13                           0x00002000
#define BIT14                           0x00004000
#define BIT15                           0x00008000
#define BIT16                           0x00010000
#define BIT17                           0x00020000
#define BIT18                           0x00040000
#define BIT19                           0x00080000
#define BIT20                           0x00100000
#define BIT21                           0x00200000
#define BIT22                           0x00400000
#define BIT23                           0x00800000
#define BIT24                           0x01000000
#define BIT25                           0x02000000
#define BIT26                           0x04000000
#define BIT27                           0x08000000
#define BIT28                           0x10000000
#define BIT29                           0x20000000
#define BIT30                           0x40000000
#define BIT31                           0x80000000
        
#define SZ_1K                           0x00000400
#define SZ_4K                           0x00001000
#define SZ_8K                           0x00002000
#define SZ_16K                          0x00004000
#define SZ_32K                          0x00008000
#define SZ_64K                          0x00010000
#define SZ_128K                         0x00020000
#define SZ_256K                         0x00040000
#define SZ_512K                         0x00080000

#define SZ_1M                           0x00100000
#define SZ_2M                           0x00200000
#define SZ_4M                           0x00400000
#define SZ_8M                           0x00800000
#define SZ_16M                          0x01000000
#define SZ_32M                          0x02000000
#define SZ_64M                          0x04000000
#define SZ_128M                         0x08000000
#define SZ_256M                         0x10000000
#define SZ_512M                         0x20000000

#define SZ_1G                           0x40000000

#define ALL_BITS                        0xFFFFFFFF
        
/* -------------------------------- */
/* SA1200 physical memory addresses */
/* -------------------------------- */

#define ROMBase                         0x00000000
#define ROMSize                         SZ_1M

#define SRAMBase                        0x10000000
#define SRAMSize                        (SZ_512M + SZ_256M)

#define PCILocalBase                    0x42000000
#define PCILocalSize                    SZ_1M

#define PCIConfigTypeOneBase            0x52000000
#define PCIConfigTypeOneSize            SZ_32M

#define PCIConfigTypeZeroBase           0x53000000
#define PCIConfigTypeZeroSize           SZ_32M

#define PCIIOBase                       0x54000000
#define PCIIOSize                       SZ_1M

#define PCIMemBase                      0x60000000
#define PCIMemSize                      SZ_512M

#define ARMCOREBase                     0x90000000
#define ARMCORESize                     SZ_1M

#define AMBABase                        0xB0000000
#define AMBASize                        SZ_1M

#define DRAMBase                        0xC0000000 /* Leave first 24Mb for ucode to use */
#define DRAMSize                        SZ_8M

#define DRAMPrefetchBase                0xD0000000
#define DRAMPrefetchSize                DRAMSize

#define DRAMFlushBase                   0xE0000000
#define DRAMFlushSize                   DRAMSize

#define DRAMControlBase                 0xFF000000
#define DRAMControlSize                 SZ_1M

/* ---------------------------- */
/* MMU Virtual SA1200 Addresses */
/* ---------------------------- */

#define DRAM_BASE                       0x00000000
#define DRAM_SIZE                       SZ_8M

#define ROM_BASE                        0x08000000
#define ROM_SIZE                        SZ_1M

#define SRAM_BASE                       SRAMBase
#define SRAM_SIZE                       SZ_8M

#define FLASH_SIZE						SZ_16K


/* ================ */
/* SA1200 REGISTERS */
/* ================ */

/* -------------------------- */
/* PLL Configuration Register */
/* -------------------------- */
#define PLL_CFG                         0x90000C00

/* PLL_CFG Constants @ 3.6864 MHz */

#define PLL_A_CLK_29MHz                   0x00
#define PLL_A_CLK_36MHz                   0x01
#define PLL_A_CLK_44MHz                   0x02
#define PLL_A_CLK_51MHz                   0x03
#define PLL_A_CLK_58MHz                   0x04
#define PLL_A_CLK_66MHz                   0x05
#define PLL_A_CLK_73MHz                   0x06
#define PLL_A_CLK_81MHz                   0x07
#define PLL_A_CLK_88MHz                   0x08
#define PLL_A_CLK_95MHz                   0x09
#define PLL_A_CLK_103MHz                  0x0A
#define PLL_A_CLK_110MHz                  0x0B
#define PLL_A_CLK_117MHz                  0x0C
#define PLL_A_CLK_132MHz                  0x0D
#define PLL_A_CLK_147MHz                  0x0E
#define PLL_A_CLK_154MHz                  0x0F
#define PLL_A_CLK_162MHz                  0x10
#define PLL_A_CLK_176MHz                  0x11
#define PLL_A_CLK_191MHz                  0x12


/* add parameters for fast clock rate an B0 */

#define PLL_CLK_29MHz                   0x00
#define PLL_CLK_36MHz                   0x01
#define PLL_CLK_44MHz                   0x02
#define PLL_CLK_51MHz                   0x03
#define PLL_CLK_58MHz                   0x04
#define PLL_CLK_66MHz                   0x05
#define PLL_CLK_73MHz                   0x06
#define PLL_CLK_81MHz                   0x07
#define PLL_CLK_88MHz                   0x08
#define PLL_CLK_95MHz                   0x09
#define PLL_CLK_103MHz                  0x0A
#define PLL_CLK_110MHz                  0x0B
#define PLL_CLK_132MHz                  0x0C
#define PLL_CLK_147MHz                  0x0D
#define PLL_CLK_154MHz                  0x0E
#define PLL_CLK_162MHz                  0x0F
#define PLL_CLK_165MHz                  0x10
#define PLL_CLK_176MHz                  0x11
/* #define PLL_CLK_191MHz                  0x12 */

#define CCCR_INIT_VALUE (PLL_CLK_165MHz)
#define CCCR_INIT_VALUE_A (PLL_A_CLK_162MHz)
/* ------------------------------ */
/* FBI Control & Status Registers */
/* ------------------------------ */
#define IREG                            0xB00401E0

/* IREG Contants */
#define UEFIQ_ENABLE                    BIT31
#define UEIRQ_ENABLE                    BIT30
#define CINTFIQ_ENABLE                  BIT29
#define CINTIRQ_ENABLE                  BIT28
#define CINT_BIT                        BIT27
#define BREAKPOINT                      BIT24

/* -------------------------- */
/* Master Interrupt Registers */
/* -------------------------- */
#define FIQ                             0x90001000
#define IRQ                             0x90001400

/* FIQ Status Constants */
#define F_UART                          BIT8
#define F_SDRAM                         BIT7
#define F_RTC                           BIT6
#define F_SRAM                          BIT5
#define F_UENG                          BIT4
#define F_CINT                          BIT3
#define F_PCI                           BIT2

/* IRQ Status Constants */
#define I_UART                          BIT8
#define I_SDRAM                         BIT7
#define I_RTC                           BIT6
#define I_SRAM                          BIT5
#define I_UENG                          BIT4
#define I_CINT                          BIT3
#define I_PCI                           BIT2

/* ------------------------------ */
/* GPIO Enable and data Registers */
/* ------------------------------ */
#define GPIO_EN                         0x90001800
#define GPIO_DATA                       0x90001C00

/* ------------------------ */
/* Real Time Clock Register */
/* ------------------------ */
#define RTC_DIV                         0x90002000
#define RTC_TINT                        0x90002400
#define RTC_TVAL                        0x90002800
#define RTC_CNTR                        0x90002C00
#define RTC_ALM                         0x90003000

/* RTC_DIV Register Constant */
#define RTC_IRQ_SELECT					BIT19 /* 1 means will gen IRQ interrupt */
#define RTC_INT_RESET                   BIT18
#define RTC_INT_ENABLE                  BIT17 /* 1 means can gen an interrupt */
#define RTC_WRITE_ENABLE                BIT16

/* ---- */
/* UART */
/* ---- */
#define UART_SR                         0x90003400
#define UART_CR                         0x90003800
#define UART_DR                         0x90003C00

/* UART Status Register Constants */
#define UART_TX_FIFO_FULL               BIT7
#define UART_RX_FIFO_FULL               BIT6
#define UART_TX_FIFO_EMPTY              BIT5
#define UART_RX_READY                   BIT4
#define UART_RX_OVERRUN                 BIT3
#define UART_TX_READY                   BIT2
#define UART_FRAMING_ERR                BIT1
#define UART_PARITY_ERR                 BIT0

/* UART Control Register Contants */
#define UART_INT_CLEAR                  BIT9  /* Rev A */
#define UART_INT_SELECT                 BIT9  /* Rev B */
#define UART_TX_INT_ENABLE              BIT8
#define UART_ENABLE_BIT                 BIT7
#define UART_DATA_BITS_8                (BIT6 | BIT5)
#define UART_RX_INT_ENABLE              BIT4
#define UART_STOP_BIT_1                 BIT3
#define UART_PARITY_ENABLE              BIT1
#define UART_BREAK                      BIT0

/* Clock frequency for UART */
#ifdef SA1200_A0
#define UART_CLOCK                      100000000/32
#else
#define UART_CLOCK                      3686400
#endif

#define BAUD_9600                       (((UART_CLOCK/16/9600)-1)   << 16)
#define BAUD_19200                      (((UART_CLOCK/16/19200)-1)  << 16)
#define BAUD_38400                      (((UART_CLOCK/16/38400)-1)  << 16)
#define BAUD_57600                      (((UART_CLOCK/16/57600)-1)  << 16)
#define BAUD_115200                     (((UART_CLOCK/16/115200)-1) << 16)

#define CLEAR_BAUD_RATE_FIELD           (0x3FF << 16)

/* UART Data Register with Error Status */
#define UART_DATA_FRMING_ERR            BIT10
#define UART_DATA_RX_OVERRUN            BIT9
#define UART_DATA_PARITY_ERR            BIT8

/* ---------------- */
/* PCI Control Unit */
/* ---------------- */
#define CSR_BASE                        0x42000000

#define PCI_VENDOR_ID                   0x00
#define PCI_DEVICE_ID                   0x02
#define PCI_COMMAND                     0x04
#define PCI_STATUS                      0x06
#define PCI_REV_ID                      0x08
#define PCI_CLASS_ID                    0x09
#define PCI_LATENCY                     0x0D
#define PCI_MEM_BAR                     0x10
#define PCI_CSR_BAR                     0x10
#define PCI_IO_BAR                      0x14
#define PCI_DRAM_BAR                    0x18
#define PCI_OUT_INT_STATUS              0x30
#define PCI_OUT_INT_MASK                0x34
#define PCI_INT_LINE                    0x3C
#define MAILBOX_0                       0x50
#define MAILBOX_1                       0x54
#define MAILBOX_2                       0x58
#define MAILBOX_3                       0x5C
#define DOORBELL                        0x60
#define DOORBELL_SETUP                  0x64
#define ROM_BYTE_WRITE                  0x68
#define CAP_PTR_EXT                     0x70
#define PWR_MGMT                        0x74
#define PCI_RESET                       0x7C
#define CHAN_1_BYTE_COUNT               0x80
#define CHAN_1_PCI_ADDR                 0x84
#define CHAN_1_DRAM_ADDR                0x88
#define CHAN_1_DESC_PTR                 0x8C
#define CHAN_1_CONTROL                  0x90
#define DMA_INF_MODE                    0x9C
#define CHAN_2_BYTE_COUNT               0xA0
#define CHAN_2_PCI_ADDR                 0xA4
#define CHAN_2_DRAM_ADDR                0xA8
#define CHAN_2_DESC_PTR                 0xAC
#define CHAN_2_CONTROL                  0xB0
#define CSR_BASE_ADDR_MASK              0xF8
#define CSR_BASE_ADDR_OFF               0xFC
#define DRAM_BASE_ADDR_MASK             0x100
#define DRAM_BASE_ADDR_OFF              0x104
#define ROM_BASE_ADDR_MASK              0x108
#define DRAM_TIMING                     0x10C
#define DRAM_ADDR_SIZE_0                0x110
#define DRAM_ADDR_SIZE_1                0x114
#define DRAM_ADDR_SIZE_2                0x118
#define DRAM_ADDR_SIZE_3                0x11C
#define I2O_IFH                         0x120
#define I2O_IPT                         0x124
#define I2O_OPH                         0x128
#define I2O_OFT                         0x12C
#define I2O_IFC                         0x130
#define I2O_OPC                         0x134
#define I2O_IPC                         0x138
#define SA_CONTROL                      0x13C
#define PCI_ADDR_EXT                    0x140
#define PREFETCH_RANGE                  0x144
#define PCI_ABITOR_STATUS               0x148
#define DBELL_PCI_MASK                  0x150
#define DBELL_SA_MASK                   0x154
#define IRQ_STATUS                      0x180 
#define IRQ_RAW_STATUS                  0x184 
#define IRQ_ENABLE                      0x188 
#define IRQ_ENABLE_SET                  0x188 
#define IRQ_ENABLE_CLEAR                0x18C 
#define IRQ_SOFT                        0x190 
#define FIQ_STATUS                      0x280 
#define FIQ_RAW_STATUS                  0x284 
#define FIQ_ENABLE                      0x288 
#define FIQ_ENABLE_SET                  0x288 
#define FIQ_ENABLE_CLEAR                0x28C 
#define FIQ_SOFT                        0x290 
#define TIMER_1_LOAD                    0x300 
#define TIMER_1_VALUE                   0x304 
#define TIMER_1_CONTROL                 0x308 
#define TIMER_1_CLEAR                   0x30C 
#define TIMER_2_LOAD                    0x320 
#define TIMER_2_VALUE                   0x324 
#define TIMER_2_CONTROL                 0x328 
#define TIMER_2_CLEAR                   0x32C 
#define TIMER_3_LOAD                    0x340 
#define TIMER_3_VALUE                   0x344 
#define TIMER_3_CONTROL                 0x348 
#define TIMER_3_CLEAR                   0x34C 
#define TIMER_4_LOAD                    0x360 
#define TIMER_4_VALUE                   0x364 
#define TIMER_4_CONTROL                 0x368 
#define TIMER_4_CLEAR                   0x36C 

/* ---------------------------- */
/* PCI_RESET Register Constants */
/* ---------------------------- */

#define UENG0_RST                       BIT0
#define UENG1_RST                       BIT1
#define UENG2_RST                       BIT2
#define UENG3_RST                       BIT3
#define UENG4_RST                       BIT4
#define UENG5_RST                       BIT5
#define PCIRST_RST                      BIT14
#define EXT_RST                         BIT15
#define FXCMD_RST                       BIT17
#define FBI_RST                         BIT16
#define SDRAM_RST                       BIT18
#define SRAM_RST                        BIT29
#define PCI_RST                         BIT30
#define SACORE_RST                      BIT31

/* ------------------------------ */
/* ARM control register constants */
/* ------------------------------ */
#define INIT_COMPLETE                   BIT0
#define PCI_NRESET                      BIT9
#define PCI_CFN                         BIT31
#define ROM_TIMING                      0x0fff0000
#define ROM_SETUP                       0x04aa0000
#define SA_CTL_COMPLETE                 (INIT_COMPLETE | PCI_NRESET)

/* --------------- */
/* Interrupt Flags */
/* --------------- */
#define SOFT_INT                        BIT1
#define TIMER1_INT                      BIT4
#define TIMER2_INT                      BIT5
#define TIMER3_INT                      BIT6
#define TIMER4_INT                      BIT7
#define DOORBELL_INT                    BIT15
#define DMA1_INT                        BIT16
#define DMA2_INT                        BIT17
#define PCI_IRQ_L_INT                   BIT18
#define DMA1_NOT_BUSY_INT               BIT20
#define DMA2_NOT_BUSY_INT               BIT21
#define START_BIST_INT                  BIT22
#define SERR_INT                        BIT23
#define SDRAM_PARITY_INT                BIT24
#define I2O_INPOST_INT                  BIT25
#define DISCARD_TIMER_INT               BIT27
#define DATA_PARITY_INT                 BIT28
#define PCI_MASTER_ABORT_INT            BIT29
#define PCI_TARGET_ABORT_INT            BIT30
#define PCI_PARITY_INT                  BIT31

/* ------------------------------- */
/* Timer Control Register Constant */
/* ------------------------------- */
#define TIMER_ENABLE                    BIT7
#define TIMER_MODE                      BIT6

/*  timer stuff:
 *  On SA1200 there is an external clock which ticks at 3.68Mhz. This is wired
 *  into the DC21285 chip's timer #3. There are 24 bits of counter that we can
 *  set to divide this external timer down. Each external tick is 3.68*(10**-6)
 *  seconds long, 1mSEC needs counter of 3680 (every 3680 external ticks, we
 *  get one clock interrupt).
 */

#define OS_TIMER                        3

#ifdef SA1200_A0
#define mSEC_1                          100000000/1000
#else
#define mSEC_1                          162200000/1000
#endif

#define mSEC_5                          (mSEC_1 * 5)
#define mSEC_10                         (mSEC_1 * 10)
#define mSEC_100                        (mSEC_1 * 100)
#define SEC_1                           (mSEC_1 * 1000)
#define SEC_10                          (SEC_1  * 10)

/* ---------------------------------- */
/* Translation Lookaside Buffer Table */
/* ---------------------------------- */
#define TTB_SIZE                        SZ_32K

/* -------------- */
/* CoProcessor 15 */
/* -------------- */
#define IC_ON                           BIT12
#define DC_ON                           BIT2
#define WB_ON                           BIT3


/* -------------- */
/* SRAM Registers */
/* -------------- */

#define SRAM_CSR                        0x38000000 /* SRAM CSR */
#define SRAM_SLOW_CONFIG                0x38000020 /* SRAM Flash/Mac cycle times */
#define SRAM_FLASH_CONFIG               0x38000024 /* Flash timing */
#define SRAM_MAC_CONFIG                 0x38000028 /* Mac timing */
#define SRAM_SLOW_CONFIG_V_165          0x00001852
#define SRAM_SLOPORT_CFG_V_165          0x454B0E07

/* ------------------ */
/* SRAM CSR constants */
/* ------------------ */

#define SRAM_RLK                        BIT20   /* Read lock order bit */
#define SRAM_RDY                        BIT19   /* Slow port ready enable */
#define SRAM_CFT                        BIT18   /* Comand FIFO Test Enable */
#define SRAM_MAC                        BIT17   /* MAC Configuration */
#define SRAM_FLWT                       BIT16   /* Flowthrough SRAM Enable */
#define SRAM_BKW                        BIT15   /* Bank Switch Wait Enable */
#define SRAM_8MF                        BIT14   /* 8MB Flash */
#define SRAM_4MF                        BIT13   /* 4MB Flash */
#define SRAM_2MF                        BIT12   /* 2MB Flash */
#define SRAM_8MSR                       BIT11   /* 8MB SRAM */
#define SRAM_4MSR                       BIT10   /* 4MB SRAM */
#define SRAM_2MSR                       BIT9    /* 2MB SRAM */
#define SRAM_1MSR                       BIT8    /* 1MB SRAM */
#define SRAM_RLAM                       BIT4    /* ASB Read Lock Achieved Mask */
#define SRAM_IRQ                        BIT3    /* IRQ Enable */
#define SRAM_FIQ                        BIT2    /* FIQ Enable */
#define SRAM_RLRS                       BIT1    /* ASB Read Lock Retry Achieved Status */
#define SRAM_RLS                        BIT0    /* ASB Read Lock Achieved Status */

/* --------------- */
/* SDRAM Registers */
/* --------------- */

#define SDRAM_CSR                       0xFF000000 /* SDRAM CSR */
#define SDRAM_MEMCTL0                   0xFF000004 /* SDRAM timing info */
#define SDRAM_MEMCTL1                   0xFF000008 /* SDRAM timing info */
#define SDRAM_MEMINIT                   0xFF00000C /* SDRAM initialization timing */

/* ------------------- */
/* SDRAM CSR Constants */
/* ------------------- */

#define SDRAM_EN_FTST                   BIT15   /* Enable FIFO RAM test */
#define SDRAM_EN_INT                    BIT10   /* Enable Parity Interrupt */
#define SDRAM_EN_IRQ                    BIT9    /* Select IRQ */
#define SDRAM_EN_FIQ                    BIT8    /* Select FIQ */
#define SDRAM_EN_CPE                    BIT6    /* Enable StrongARM Core parity error */
#define SDRAM_EN_PCIPE                  BIT5    /* Enable PCI parity error */
#define SDRAM_EN_UEPE                   BIT4    /* Enable Microengine parity error */
#define SDRAM_GENPAR                    BIT3    /* Enable parity generation */
#define SDRAM_EVENPAR                   BIT2    /* Generate even parity */
#define SDRAM_TF_BENDN                  BIT1    /* 1 = big endian mode */
#define SDRAM_INIT                      BIT0    /* Initialize SDRAM unit */
#define SDRAM_CSR_VALUE (SDRAM_INIT)
    

/* These are VxWorks specific.  It comes from VxWorks file ebsa285.h */
/* One VxWorks porting is done.  It should get from there */

/* interrupt levels */
#define INT_LVL_SOFT		1	/* soft interrupt */
#define INT_LVL_AMBARX		2	/* AMBA serial port Rx */
#define INT_LVL_AMBATX		3	/* AMBA serial port Tx */
#define INT_LVL_TIMER_1 	4	/* timer 1 */
#define INT_LVL_TIMER_2 	5	/* timer 2 */
#define INT_LVL_TIMER_3 	6	/* timer 3 */
#define INT_LVL_TIMER_4 	7	/* timer 4 */
#define	INT_LVL_IRQ_IN_0	8	/* external to 21285 */
#define	INT_LVL_IRQ_IN_1	9	/* external to 21285 */
#define	INT_LVL_IRQ_IN_2	10	/* external to 21285 */
#define	INT_LVL_IRQ_IN_3	11	/* external to 21285 */
#define INT_LVL_SPIO		12	/* Super I/O serial ports */
#define INT_LVL_DOORBELL	15	/* PCI doorbell -- Henry Qian 11/04/98 */
#define	INT_LVL_PCI		18	/* PCI external interrupts */

#endif /* __address_h */

/* end of file */

