; target.s
;------------------------------------------------------------
;                                                                      
;                  I N T E L   P R O P R I E T A R Y                   
;                                                                      
;     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;     THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                      
;                        INTEL  CORPORATION                            
;                                                                     
;                     2200 MISSION COLLEGE BLVD                        
;                                                                      
;               SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                      
;------------------------------------------------------------
; This file defines the Brutus manifests and macros required to
; support the Angel world. It is not a complete description of the
; board, only the information necessary for Angel. It is envisaged
; that individual device drivers or application code will hold the
; necessary definitions for the other board features.
;
; NOTE: To help keep the main Angel source as generic as
; possible, heavy use is made of macros. However, it is likely
; that the use of macros will hide some very obvious and
; simple optimisations. If optimised Angel source is required,
; then a fixed target version should be created by the porting
; developer. The aim of Angel is to be as simple and quick to
; port as possible.
; ------------------------------------------------------------------
; system: SA1200
; subsystem: Angel Debug Monitor
; author: Henry Qian 01/29/99
; revisions:
;

 IF :DEF: OPT
        ASSERT  (listopts_s)
old_opt SETA    {OPT}
        OPT     (opt_off)       ; disable listing of include files
 ENDIF

        ; ------------------------------------------------------------------

        IF      :LNOT: :DEF: target_s

                GBLL    target_s
target_s        SETL    {TRUE}

        ; ------------------------------------------------------------------
        ; The "ROMonly" build variable controls whether the system
        ; being built supports a system where only ROM is mapped at
        ; zero, and RAM cannot be mapped there. i.e. where the ARM
        ; vectors are hard-wired in ROM. We create this variable
        ; un-conditionally, to ensure that if an attempt is made to
        ; define it elsewhere then a build error will occur.
        
                GBLL    ROMonly ; depends on target
ROMonly         SETL    {FALSE} ; {TRUE} if no RAM at zero


        INCLUDE platform.s
        GET     mmumacro.s

        ; ------------------------------------------------------------------
        ; -- ANGEL and uHAL Support macros ---------------------------------
        ; ------------------------------------------------------------------

        ; ------------------------------------------------------------------
        ; UNMAPROM
        ; --------
        ; Provide code to deal with mapping the reset ROM away from zero
        ; (if required).

        MACRO
$label  UNMAPROM        $w1,$w2
        MEND

        ; ------------------------------------------------------------------
        ; STARTUPCODE
        ; -----------
        ; Provide code to deal with the main act of mapping the reset
        ; ROM away from zero (if required), and ensuring that the
        ; target I/O world is suitably clean before attempting to
        ; initialise the rest of Angel.
        ;
        ; NOTE:  do not use r4, that's $pos.
        ;
        
        MACRO
$label  STARTUPCODE     $w1, $w2, $pos, $ramsize

        ; leave $pos alone - already set - unless we have good reason
        ; to change it.  Here we don't.

        ; Bring everything out of reset except microengines.
		;
		MOV		$w1, #0x3F
		LDR		$w2, =(CSR_BASE + PCI_RESET)
		STR		$w1, [$w2]

        ldr     $w1, =PLL_CFG           ; get pointer to PLL control Reg
        ldr     $w2, =PLL_CLK_162MHz    ; get value for 88 MHz
        str     $w2, [$w1]              ; set speed
        
        ; Flush Instruction Cache and Data Cache
        WRCP15_FlushITB_DTB     $w1     ; Flush ITB's + DTB's
        WRCP15_FlushIC_DC       $w1     ; Flush the ICache and DCaches
        WRCP15_DrainWriteBuffer $w1
        NOP                             ; make sure that the pipe is empty
        NOP
        NOP

        ; target images can set their own clock rate, if required.
        ; turn on clock switching
        WRCP15_EnableClockSW $w1

        DISABLE_INTS    $w1, $w2

        INIT_RAM        $w1, $w2, $ramsize

        ; For unknow reason, $pos (r4) has the wrong value.
        ; It dosen't keep the value __rom, which it fetched
        ; at very beginning.
        ;
        LDR		$pos, =ROM_BASE

        INIT_GPIOS      $w1, $w2

        MEND

        ; ------------------------------------------------------------------
        ; INIT_RAM
        ; --------
        ; ANGEL and uHAL macro to initialise memory on startup. See the calling
        ; macros for reasons why this particular register set is used.

        MACRO
        INIT_RAM        $w1, $w2, $w3

        ; By checking if the MMU is already active, memory will not be
        ; initialized when running under Angel or from RAM.
        RDCP15_Control  $w2             ; Read the MMU state
        LDR     $w3, =EnableMMU
        TST     $w2, $w3                ; Already active?

        BNE     %F42                    ; Yes, so nothing to do
        ; Falling through, assume MMU disabled
        nop

        ; ------------------------------------------------------------------
        ; SRAM Controller Setup
        ; ---------------------
        ;
        LDR     $w1, =SRAM_CSR
        LDR     $w2, =SRAM_CSR_VALUE
        STR     $w2, [$w1]

        LDR     $w1, =SRAM_BOOT_CONFIG
        LDR     $w2, =SRAM_BOOT_CONFIG_VALUE
        STR     $w2, [$w1]

        LDR     $w1, =SRAM_SLOWPORT_CONFIG
        LDR     $w2, =SRAM_SLOWPORT_CONFIG_V
        STR     $w2, [$w1]

        LDR     $w1, =SRAM_SLOW_CONFIG
        LDR     $w2, =SRAM_SLOW_CONFIG_VALUE
        STR     $w2, [$w1]

        ; ------------------------------------------------------------------
        ; SDRAM Control Setup
        ; -------------------
        ;
        LDR $w1,     =SDRAM_MEMCTL0
        LDR $w2,     =SDRAM_MEMCTL0_VALUE
        STR $w2,     [$w1]

        LDR $w1,     =SDRAM_MEMCTL1
        LDR $w2,     =SDRAM_MEMCTL1_VALUE
        STR $w2,     [$w1]

        LDR $w1,     =SDRAM_MEMINIT
        LDR $w2,     =SDRAM_MEMINIT_VALUE
        STR $w2,     [$w1]

        LDR $w1,     =SDRAM_CSR
        LDR $w2,     =SDRAM_CSR_VALUE
        STR $w2,     [$w1]

        ; Memory autosizing code would go here..

        SA_INITMMU      $w1, $w2, r2, r3, r6, r7

42
        ; set up the return arguments.
        LDR     $w3, =DRAM_SIZE

        MEND

        ; ------------------------------------------------------------------
        ; SA_INITMMU
        ; ----------
        ; uHAL macro, calls the core SETUPMMU macro if the MMU is
        ; enabled and then sets ICache, DCache & WBuffer as required.
        ; On certain system where the target does not have a MMU, then this
        ; macro is almost a NOP, since using the ICache & WBuffer on their
        ; own is still a big gain.
        ;

        MACRO
$label  SA_INITMMU       $tmp1, $tmp2, $tmp3, $tmp4, $tmp5, $tmp6

        ; setup and enable MMU, with caches off
        SETUPMMU        $tmp1, $tmp2, $tmp3, $tmp4, $tmp5, $tmp6

        ; enable caches
        LDR     $tmp1, =(IC_ON + DC_ON + EnableMMU)

 IF {ENDIAN} = "big"
        ORR     $tmp1, $tmp1, #EnableBigEndian
 ENDIF  ; BIG_ENDIAN

        WRCP15_Control  $tmp1   ; Update control register
        ; Flush Instruction Cache and Data Cache
        WRCP15_FlushITB_DTB     $tmp1   ; Flush ITB's + DTB's
        WRCP15_FlushIC_DC       $tmp1   ; Flush the ICache and DCaches
        WRCP15_DrainWriteBuffer $tmp1
        NOP                             ; make sure that the pipe is empty
        NOP
        NOP

        MEND

        ; ------------------------------------------------------------------
        ; SETUPMMU
        ; --------
        ; Setup the memory map world for the target. This macro is called by
        ; INITMMU from ANGEL if required; or by uHAL when running a
        ; standalone program.
        ; Startup is more painful on SA1200 than usual.
        ; Consider a program built to run from ROM at 16MB (with RAM at 0),
        ; but is actually in ROM at 0 (with RAM at high addresses). If we
        ; try to build a new page table with ROM at EPROM_BASE & RAM at 0,
        ; the program will use the low RAM alias for the page table which
        ; doesn't exist yet. As soon as we try and use the new map the
        ; program will have gone (to ROM_BASE), but we have not changed
        ; the pc yet. So we have to build the desired page table in the real
        ; RAM address range but overwrite the entry for the current address
        ; to map back to ROM. Then, we can switch to the mapped version of
        ; the ROM and restore the low memory page table entry (and then
        ; flush the ITBs, DTBs and ICache).
        ;
        ; NOTE: This macro needs to know which RAM Bank the page tables live
        ;       in, since it writes to them absolutely. If TTB_BASE is not
        ;       defined, the tables are assumed to be in the first RAM bank.
        ;
        ; Does your brain hurt yet? OK, picture time:
        ;
        ; Start         Physical Address    Virtual Address     Size
        ; 0x00000000    ROMBase             0x08000000          8Mb
        ; 0xC0000000    DRAMBase            0x00000000          64Mb
        ;
        ; Everything else is 1-1.

        MACRO
$label  SETUPMMU        $tmp1, $tmp2, $tmp3, $tmp4, $tmp5, $tabl

        ; Setup the StrongARM MMU with a mapped page table:
        ; set up the page tables - all for domain0, switched address map
        ;       RAM_SIZE / 1Mb sections for DRAM
        ;               - Read/Write all, cachable & bufferable
        ;       ROM_SIZE / 1MB sections for ROM
        ;               - Read all, cachable, nonbufferable
        ;       The rest of address space
        ;               - noncachable, nonbufferable

        ; set the domain register: domain0 CLIENT, domain1-15 NO_ACCESS
        ; write the translation base register
        ; Flush ITB's and DTB's
        ; Flush the Icache

        ; First clear all TT entries as FAULT
        ;
        LDR     $tmp4, =TTB_BASE
        LDR     $tmp3, =0
        LDR     $tmp1, =(TTB_SIZE / 4)
1
        STR     $tmp3, [$tmp4], #4
        SUBS    $tmp1, $tmp1, #1        ; decrement loop count
        BNE     %B01

        ; --------------------------------------------------
        ; Map DRAM non-prefetch area to address 0 and itself
        ; --------------------------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMBase                ; physical address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        ADD     $tmp5, $tmp1, $tmp3             ; table entry for upper RAM
        LDR     $tmp3, =CACHED_NOBUFFER_ACCESS  ; cachable & bufferable
        ADD     $tmp1, $tmp1, $tmp3             ; table entry for lower RAM

        LDR     $tmp3, =DRAMSize                ; Size of first bank
        MOV     $tmp3, $tmp3, LSR #20           ; no. of 1MB segments
2
        STR     $tmp1, [$tmp2], #4              ; virtual entry
        STR     $tmp5, [$tmp4], #4              ; real entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        ADD     $tmp5, $tmp5, #SZ_1M
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B02

        ; --------------------------------
        ; Map DRAM prefetch area to itself
        ; --------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMPrefetchBase        ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =DRAMPrefetchBase        ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =DRAMPrefetchSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
3
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B03

        ; -----------------------------------
        ; Map DRAM cache flush area to itself
        ; -----------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMFlushBase           ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =CACHED_NOBUFFER_ACCESS  ; Yes cache, no buffer
        LDR     $tmp1, =DRAMFlushBase           ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =DRAMFlushSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
4
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B04

        ; -------------------------------
        ; Map DRAM control area to itself
        ; -------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =DRAMControlBase         ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =DRAMControlBase         ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =DRAMControlSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
5
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B05

        ; ------------------------------
        ; Map AMBA Translation Unit area
        ; ------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =AMBABase                ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =AMBABase                ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =AMBASize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
6
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B06

        ; -----------------
        ; Map ARM Core area
        ; -----------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =ARMCOREBase             ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =ARMCOREBase             ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =ARMCORESize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
7
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B07

        ; --------------------
        ; Map PCI Memory Space
        ; --------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIMemBase              ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIMemBase              ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIMemSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
8
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B08

        ; -----------------
        ; Map PCI I/O Space
        ; -----------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIIOBase               ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIIOBase               ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIIOSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
9
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B09

        ; ------------------------------------
        ; Map PCI Type One Configuration Space
        ; ------------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIConfigTypeOneBase    ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIConfigTypeOneBase    ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIConfigTypeOneSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
10
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B10

        ; -------------------------------------
        ; Map PCI Type Zero Configuration Space
        ; -------------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCIConfigTypeZeroBase   ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCIConfigTypeZeroBase   ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCIConfigTypeZeroSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
11
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B11


        ; ---------------------------------
        ; Map PCI Local Configuration Space
        ; ---------------------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =PCILocalBase            ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =PCILocalBase            ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =PCILocalSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
12
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B12

        ; -----------------------
        ; Map SRAM & Control area
        ; -----------------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =SRAMBase                ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; Index to the table
        ADD     $tmp4, $tmp4, $tmp2             ; address of entry

        ; Access permissions
        LDR     $tmp3, =NOCACHE_NOBUFFER_ACCESS ; No cache, no buffer
        LDR     $tmp1, =SRAMBase                ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =SRAMSize
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
13
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B13

        ; ------------
        ; Map ROM area
        ; ------------

        LDR     $tmp2, =TTB_BASE
        LDR     $tmp1, =ROM_BASE                ; virtual address
        MOV     $tmp4, $tmp1, LSR #(20-2)       ; index to table entry
        ADD     $tmp4, $tmp4, $tmp2             ; address to table entry

        ; Access permissions
        LDR     $tmp3, =CACHED_NOBUFFER_ACCESS  ; No cache, no buffer
        LDR     $tmp1, =ROMBase                 ; physical address
        ADD     $tmp1, $tmp1, $tmp3             ; entry

        LDR     $tmp3, =ROM_SIZE
        MOV     $tmp3, $tmp3, LSR #20           ; number of 1MB segments
14
        STR     $tmp1, [$tmp4], #4              ; virtual entry
        ADD     $tmp1, $tmp1, #SZ_1M            ; add section number field
        SUBS    $tmp3, $tmp3, #1                ; decrement loop count
        BNE     %B14

        ; ------------------------------------------------------------------
        ; No MMU active, and executing from ROM space
        ; ------------------------------------------------------------------
        RDCP15_Control  $tmp1                   ; Read the MMU state
        LDR     $tmp2, =EnableMMU               ; MMU enable bit
        ORR     $tmp1, $tmp1, $tmp2             ; Set the Enable bit

        ; After MMU active current execution locaton (PC value) will be
        ; in DRAM space (0x00000000 - 0x00100000) and the physical
        ; ROM position we are at now will be remapped to
        ; (0x08000000 - 0x08100000) area.
        ; So in order to continue our execution after MMU active, we
        ; need to temporarily map the current section into DRAM area.
        ; After MMU activation, we'll jump to 0x08000000 area and
        ; recover the temporary section in DRAM area.
        ;
        LDR     $tmp5, =TTB_BASE
        MOV     $tmp4, pc, LSR #(20-2)          ; Current execution location
        ADD     $tmp4, $tmp4, $tmp5             ; RAM Page Table entry
        ; Don't overwrite $tmp3 and $tmp4, so we can restore these values
        LDR     $tmp3, [$tmp4]                  ; save current section

        LDR     $tmp2, =ROM_BASE:SHR:(20-2)     ; we'll be there after MMU
        ADD     $tmp2, $tmp2, $tmp5             ; ROM Page Table entry
        LDR     $tmp5, [$tmp2]

        ; Overwrite RAM entry temporarily
        STR     $tmp5, [$tmp4]

        ; ------------------------------------------------------------------
        ; Turn on the MMU and flush everything
        ; ------------------------------------------------------------------
        LDR     $tmp2, =EnableMMU
        LDR     $tmp5, =TTB_BASE   ;TTB address is 2**14 aligned

        WRCP15_DAControl $tmp2          ;Initialise Domain Access Control
        WRCP15_TTBase    $tmp5          ;Initialise Translation Table Base reg.

        WRCP15_Control  $tmp2           ; Turn on the MMU

        ; ------------------------------------------------------------------
        ; Jump from DRAM area (0x00000000 - 0x00100000) to
        ; ROM area (0x08000000 - 0x08100000).
        ; ------------------------------------------------------------------
        MOV     $tmp5, pc
        ADD     $tmp5, $tmp5, #ROM_BASE         ; add offset to mapped ROM
        ADD     $tmp5, $tmp5, #0x8

        MOV     pc, $tmp5
        ; Don't forget return address (if any)
        ADD     lr, lr, #ROM_BASE

        ; ------------------------------------------------------------------
        ; Restore the overwritten page table entry & flush everything
        ; ------------------------------------------------------------------
        STR     $tmp3, [$tmp4]
        WRCP15_FlushITB_DTB     $tmp1   ; Flush ITB's + DTB's
        WRCP15_FlushIC_DC       $tmp1   ; Flush the ICache and DCaches
        NOP                             ; make sure that the pipe is empty
        NOP
        NOP
        NOP

        MEND


        ; ------------------------------------------------------------------
        ; INITMMU
        ; -------
        ; Dummy Angel macro because Brutus MMU has already been setup by now

        MACRO
$label  INITMMU  $tmp1, $tmp2, $tmp3, $tmp4, $tmp5, $tmp6
        MEND

        ; ------------------------------------------------------------------
        ; INITTIMER
        ; ---------
        ; Angel macro is provided purely as a holder for any code that may
        ; be required to initialise a hardware timer as part of the reset
        ; sequence. Angel does *NOT* make use of any timer resources, so by
        ; default this macro should not be required to do anything. However,
        ; under exceptional circumstances special code may be required to
        ; ensure that Angel starts cleanly and that later application
        ; specific code can have full control of the timer.

        MACRO
$label  INITTIMER       $w1, $w2
        MEND

        ; ------------------------------------------------------------------
        ; REVERSE
        ; -------
        ; Quick word byte-order reversal macro. Handy for endian stuff.

        MACRO
$label  REVERSE $w1, $w2                ; w1:    D    C    B    A
        EOR     $w2, $w1, $w1, ROR #16  ; w2:   D^B, C^A, B^D, A^C
        BIC     $w2, $w2, #0xff0000     ; w2:   D^B,  0 , B^D, A^C
        MOV     $w1, $w1, ROR #8        ; w1:    A    D    C    B
        EOR     $w1, $w1, $w2, LSR #8   ; w2>>8: 0   D^B   0   B^D

        MEND                            ; w1:    A    B    C    D

        ; ------------------------------------------------------------------
        ; GETSOURCE
        ; ---------
        ; This Angel macro is used to read the current interrupt source
        ; activity status for the Angel device driver interrupts:

        MACRO
$label  GETSOURCE       $w1, $w2
        ; Find out what caused the interrupt
        MOV     $w1, #IH_SERIAL
        MEND

        ; ------------------------------------------------------------------
        ; CACHE_IBR
        ; ---------
        ; This Angel macro implements an instruction barrier for a range of
        ; addresses (i.e. it makes instruction and data memory coherent for
        ; this range) .
        ; w1 contains the start of the range
        ; w2 the next address after the end
        ; Note that w1 will be corrupted

        MACRO
$label  CACHE_IBR       $w1, $w2, $temp, $temp2

        ASSERT  ( $w1 = r0 )

        ; Protect lr from the SWI
        STMFD   sp!, {lr}
 
        ; Save the old mode
        MRS     $temp, CPSR
        AND     $temp, $temp, #ModeMask
        CMP     $temp, #USRmode
        BNE     NonUSRmodeCase1

        ; If in USR mode then get into SVC mode
        MOV     $temp2, r0
        LDR     r0, = angel_SWIreason_EnterSVC
        DCD     angel_SWI_ARM
        MOV     r0, $temp2
        
NonUSRmodeCase1
        ; Clean the Dcache entries
01      
        WRCP15_CleanDCentry $w1
        ADD     $w1, $w1, #32
        CMP     $w1, $w2
        BLT     %B01

        ; Drain the write buffer
        WRCP15_DrainWriteBuffer $w1     

        ; Flush the Icache
        MOV     $w1, #0
        WRCP15_FlushIC $w1

        ; Clear the pipeline..
        NOP
        NOP
        NOP
        NOP
        
        ; Return to old mode if we changed in the first place
        CMP     $temp, #USRmode
        BNE     NonUSRmodeCase2
        ; If originally in USR mode then leave SVC mode properly
        IMPORT  Angel_ExitToUSR
        BL      Angel_ExitToUSR
        
NonUSRmodeCase2
        ; Restore lr
        LDMFD   sp!, {lr}

        MEND

        ; ------------------------------------------------------------------
        ; Angel and uHAL macro to disable interrupts on powerup
        MACRO
$label  DISABLE_INTS    $w1, $w2
        MOV     $w2, #0
        LDR     $w1, =IREG
        STR     $w2, [$w1]          ; disable micro engine interrupts
        LDR     $w1, =UART_CR
        STR     $w2, [$w1]          ; disable UART interrupts
        LDR     $w1, =(CSR_BASE + IRQ_ENABLE_CLEAR)
        MOV     $w2, #ALL_BITS
        STR     $w2, [$w1]          ; disable rest of IRQ interrupts
        LDR     $w1, =(CSR_BASE + FIQ_ENABLE_CLEAR)
        MOV     $w2, #ALL_BITS
        STR     $w2, [$w1]          ; disable rest of FIQ interrupt
        MEND

        ; ------------------------------------------------------------------
        ; Angel and uHAL macro to setup GPIOs on powerup.
        ; set up the direction of the GPIO pins connnected to the LEDs

        MACRO
$label  INIT_GPIOS      $w1, $w2
        MOV     $w2, #0xF           ; all pins output
        LDR     $w1, =GPIO_EN       ; GPIO enable register
        STR     $w2, [$w1]          ; set the direction of bits
        MOV     $w1, #0x0
        SET_LEDS $w1, $w2
        MEND
        
        MACRO
$label  DELAY   $w1
		ROUT
99
        SUBS    $w1, $w1, #0x1
        BNE     %B99
        MEND

        ; ------------------------------------------------------------------
        ; macro to set the LEDs to the value given in $w1
        ; NOTE: this should be the same as SetLEDs in driver.s

        MACRO
$label  SET_LEDS    $w1, $w2
        LDR     $w2, =GPIO_DATA     ; GPIO data register
        STR     $w1, [$w2]          ; display data
        MEND

        ENDIF    ; target_s

 IF :DEF: OPT
        OPT     (old_opt)       ; restore previous listing options
 ENDIF


        END     ; EOF target.s

