/* timer.c
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: uHAL
 * author: Henry Qian 04/09/99
 * revisions:
 *
 */

#include "platform.h"
#include "interrupts.h"
#include "timer.h"

void timer_ISR(int timer);

/* For simplicity, SAiv_Timer[0] is not used */
SAis_Timer SAiv_Timer[MAX_TIMER+1];

/*
**  void SAr_ResetTimers(void)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
void SAr_ResetTimers(void)
{
    int ii;

    SA_IRQ_Disable(TIMER1);
    SA_IRQ_Disable(TIMER2);
    SA_IRQ_Disable(TIMER3);
    SA_IRQ_Disable(TIMER4);

    for (ii = 0; ii <= MAX_TIMER; ii++)
    {
        SAiv_Timer[ii].type = T_FREE;
        SAiv_Timer[ii].tickUnitTime = 0;
        SAiv_Timer[ii].ticks = 0;
        SAiv_Timer[ii].ticksleft = 0;
        SAiv_Timer[ii].expire = 0;
        SAiv_Timer[ii].handler = 0;
    }
}



/*
**  int SAr_InitTimer(  unsigned int,
**                      unsigned int,
**                      unsigned int,
**                      unsigned int,
**                      PFV)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
int SAr_InitTimer(  unsigned int timer,
                    unsigned int type,
                    unsigned int tickUnitTime,
                    unsigned int ticks,
                    PFV handler)
{
    SAr_StopTimer(timer);

    /* Sanity check of parameters */
    switch (tickUnitTime)
    {
        case mSEC_1:
            break;

        case mSEC_5:
            break;

        case mSEC_10:
            break;
#if 0 /* Temporariely commented out for test clock frequency */
        case mSEC_100:
            break;

        case SEC_1:
            break;

        case SEC_10:
            break;
#endif
        default:
            return(-1); /* wrong tickUnitTime value */
    }

    switch (type)
    {
        case T_ONESHOT:
            break;

        case T_INTERVAL:
            break;

        default:
            return(-1); /* wrong type value */
    }

    switch (timer)
    {
        case TMR1:
            *(unsigned int*)(CSR_BASE + TIMER_1_LOAD) = tickUnitTime;
            *(unsigned int*)(CSR_BASE + TIMER_1_CONTROL) = 0xCC;
            SA_IRQ_Connect(TIMER1, (PFVI)timer_ISR, TMR1);
            break;

        case TMR2:
            *(unsigned int*)(CSR_BASE + TIMER_2_LOAD) = tickUnitTime;
            *(unsigned int*)(CSR_BASE + TIMER_2_CONTROL) = 0xCC;
            SA_IRQ_Connect(TIMER2, (PFVI)timer_ISR, TMR2);
            break;

        case TMR3:
            *(unsigned int*)(CSR_BASE + TIMER_3_LOAD) = tickUnitTime;
            *(unsigned int*)(CSR_BASE + TIMER_3_CONTROL) = 0xCC;
            SA_IRQ_Connect(TIMER3, (PFVI)timer_ISR, TMR3);
            break;

        case TMR4:
            *(unsigned int*)(CSR_BASE + TIMER_4_LOAD) = tickUnitTime;
            *(unsigned int*)(CSR_BASE + TIMER_4_CONTROL) = 0xCC;
            SA_IRQ_Connect(TIMER4, (PFVI)timer_ISR, TMR4);
            break;

        default:
            return(-1); /* wrong timer number */
    }

    /* Setup a timer */
    SAiv_Timer[timer].tickUnitTime = tickUnitTime;
    SAiv_Timer[timer].type = type;
    SAiv_Timer[timer].ticks = ticks;
    SAiv_Timer[timer].ticksleft = ticks;
    SAiv_Timer[timer].handler = handler;

    return(0);
}

/*
**  int SAr_StartTimer(unsigned int timer)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
int SAr_StartTimer(unsigned int timer)
{
    switch (timer)
    {
        case TMR1:
            SA_IRQ_Enable(TIMER1);
            break;

        case TMR2:
            SA_IRQ_Enable(TIMER2);
            break;

        case TMR3:
            SA_IRQ_Enable(TIMER3);
            break;

        case TMR4:
            SA_IRQ_Enable(TIMER4);
            break;

        default:
            return(-1);
    }

    return(0);
}

/*
** int SAr_StopTimer(unsigned int)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
int SAr_StopTimer(unsigned int timer)
{
    switch (timer)
    {
        case TMR1:
            SA_IRQ_Disable(TIMER1);
            break;

        case TMR2:
            SA_IRQ_Disable(TIMER2);
            break;

        case TMR3:
            SA_IRQ_Disable(TIMER3);
            break;

        case TMR4:
            SA_IRQ_Disable(TIMER4);
            break;

        default:
            return(-1); /* fail */
    }

    return(0); /* success */
}


/*
**  int SAr_SetTimerTicks(unsigned int, unsigned int)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
int SAr_SetTimerTicks(unsigned int timer, unsigned int ticks)
{
    if ((timer == 0) || (timer > MAX_TIMER))
        return(-1);

    SAr_StopTimer(timer);

    SAiv_Timer[timer].ticks = ticks;

    return(0);
}


/*
**  int SAr_GetTimerTicksLeft(unsigned int, unsigned int*)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
int SAr_GetTimerTicksLeft(unsigned int timer, unsigned int* ticks)
{
    if ((timer == 0) || (timer > MAX_TIMER))
        return(-1);

    *ticks = SAiv_Timer[timer].ticksleft;

    return(0);
}


/*
**  void timer_ISR(void)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
void timer_ISR(int timer)
{
    /* Clear the interrupt */
    switch (timer)
    {
        case 1:
            *(unsigned int*)(CSR_BASE + TIMER_1_CLEAR) = 0;
            break;

        case 2:
            *(unsigned int*)(CSR_BASE + TIMER_2_CLEAR) = 0;
            break;

        case 3:
            *(unsigned int*)(CSR_BASE + TIMER_3_CLEAR) = 0;
            break;

        case 4:
            *(unsigned int*)(CSR_BASE + TIMER_4_CLEAR) = 0;
            break;

        default:
            return;
    }

    (SAiv_Timer[timer].ticksleft)--;
    if (SAiv_Timer[timer].ticksleft == 0)
    {
        if (SAiv_Timer[timer].handler != 0)
            SAiv_Timer[timer].handler();

        /* reload if it is periodic timer */
        if (SAiv_Timer[timer].type == T_INTERVAL)
            SAiv_Timer[timer].ticksleft = SAiv_Timer[timer].ticks;
        else if (SAiv_Timer[timer].type == T_ONESHOT)
        {
            switch (timer)
            {
                case 1:
                    SA_IRQ_Disable(TIMER1);
                    break;

                case 2:
                    SA_IRQ_Disable(TIMER2);
                    break;

                case 3:
                    SA_IRQ_Disable(TIMER3);
                    break;

                case 4:
                    SA_IRQ_Disable(TIMER4);
                    break;

                default:
                    return;
            }
        }
    }
}

/* end of file */
