/* uart.c
 *
 *
 * --------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------
 * system: SA1200
 * subsystem: uHAL 
 * author: Henry Qian 4/9/99
 * revisions:
 *
 */

#include <stdarg.h>
#include <string.h>
#include "platform.h"
#include "uart.h"

/*
**  void SAr_ResetPort(unsigned int)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
void SAr_ResetPort(unsigned int baud)
{
    /* init UART port */
    *(unsigned int*)UART_CR = 0;
    *(unsigned int*)UART_CR =   baud | \
                                UART_DATA_BITS_8 | \
                                UART_ENABLE_BIT;

	/* interrupts are left disabled */
}


/*
**  void SAr_putchar(unsigned char)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
void SAr_putchar(unsigned char ch)
{
    while((*(unsigned int*)UART_SR & UART_TX_READY) == 0)
        ;   /* Wait until transmitter ready */

    *(unsigned int*)UART_DR = (unsigned int)ch;
}


/*
**  unsigned int SAr_GetChar()
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
unsigned int SAr_GetChar()
{
    return(*(unsigned int*)UART_DR);
}

/*
**  unsigned int SAr_GetStatus()
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
unsigned int SAr_GetStatus()
{
    return(*(unsigned int*)UART_SR);
}


/*
**  void SAr_PutString(char*)
**
**  DESCRIPTION
**  
**  RETURNS
**      None.
*/
void SAr_PutString(char *string)
{
	while (*string != '\0')
    {
	    SAr_putchar(*string);
		if (*string == '\n')
            SAr_putchar('\r');
		string++ ;
	}
}

#define MAXLINE 128
#define DELETE_KEY  0x08

char* gets(char* buf)
{
    unsigned int ch;
    unsigned int status;
    unsigned int buflen = 0;

    while(1)
    {
        status = SAr_GetStatus();
        if (status & UART_RX_READY)
        {
            ch = SAr_GetChar();
#ifdef SA1200_A0
            ch = SAr_GetChar(); /* To fix a hardware problem */
#endif
            /* Delete key, erase previous char on the line */
            if ((ch == DELETE_KEY) && (buflen > 0))
            {
                buflen--;
                SAr_putchar(ch);
                SAr_putchar(' ');
                SAr_putchar(ch);
            }
            else if ((ch != DELETE_KEY) && (buflen < MAXLINE))
            {
                SAr_putchar(ch);
                if (ch == '\r')
                {
                    SAr_putchar('\n');
                    buf[buflen] = 0;
                    break; /* end of the line */
                }
                else
                    buf[buflen++] = ch;
            }
            else if ((ch == DELETE_KEY) && (buflen == 0))
                continue;
            else
                break;
        }
    }

    if (buflen >= MAXLINE)
        return(0);
    else
        return(buf);

}

/* end of file */
