/* mailbox.c
*
*------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*------------------------------------------------------------
* system: SA1200
* subsystem: uCOS
* author: Henry Qian 12/02/98
* revisions:
*
*/
#include <stdio.h>
#include "os.h"  /* uC/OS interface */

#define STACKSIZE 0x400
unsigned int Stack1[STACKSIZE];
unsigned int Stack2[STACKSIZE];
unsigned int Stack3[STACKSIZE];

/* mailbox event control blocks */
OS_EVENT *Mbox1;
OS_EVENT *Mbox2;
OS_EVENT *Mbox3;

char PassMsg[] = "Example 1";

/*
 * Task running at the lowest priority. 
 * Wait for a message in the first mailbox and post
 * a messages to the third mailbox.
 */
void
Task1(void* IGNORE)
{
    char *Msg;
    unsigned int err;
    
    for (;;) {
        OSTimeDly(10);
        
        /* wait for a message from the input mailbox */
        Msg = (char *)OSMboxPend(Mbox1, 0, &err);
        
        /* printf(" -- task 1: %08x\n", OSTime); */
        
        /* post the input message to the output mailbox */
        OSMboxPost(Mbox2, Msg);
    }
}

void
Task2(void* IGNORE)
{
    char *Msg;
    unsigned int err;
    
    for (;;) {
        OSTimeDly(10);
        
        /* wait for a message from the input mailbox */
        Msg = (char *)OSMboxPend(Mbox2, 0, &err);
        
        /* printf(" -- task 2: %08x\n", OSTime); */
        
        /* post the input message to the output mailbox */
        OSMboxPost(Mbox3, Msg);
    }
}


void
Task3(void* IGNORE)
{
    char *Msg;
    unsigned int err;
    
    for (;;) {
        OSTimeDly(10);
        
        /* wait for a message from the input mailbox */
        Msg = (char *)OSMboxPend(Mbox3, 0, &err);
        
        /* printf(" -- task 3: %08x\n", OSTime); */
        
        /* post the input message to the output mailbox */
        OSMboxPost(Mbox1, Msg);
    }
}

void mbox_demo(void)
{
    /* 
     * create the first mailbox in the pipeline with a message 
     * in it to get the first task started.
     */
    Mbox1 = OSMboxCreate(PassMsg); 
    Mbox2 = OSMboxCreate((void *)0);
    Mbox3 = OSMboxCreate((void *)0);
    
    /* 
     * create the tasks in uC/OS and assign increasing
     * priorities to them so that Task3 at the end of
     * the pipeline has the highest priority.
     */
    OSTaskCreate(   Task1,
                    0,
                    (void *)&Stack1[STACKSIZE],
                    0x10,
                    STACKSIZE*sizeof(int));
    
    OSTaskCreate(   Task2,
                    0,
                    (void *)&Stack2[STACKSIZE],
                    0x11,
                    STACKSIZE*sizeof(int));
    
    OSTaskCreate(   Task3,
                    0,
                    (void *)&Stack3[STACKSIZE],
                    0x12,
                    STACKSIZE*sizeof(int));
}

int main(int argc, char** argv)
{
    /* Start uC/OS and run userInit in root task
     * with highest priority.  After userInit()
     * returns, root task commits suicide.
     */
    ucosStart(mbox_demo, USE_CONSOLE);

    /* ucosStart() doesn't return. */
}


/* end of file */