/* hal_sdram.h
 * SA1200 Hardware Absraction Library -- SDRAM
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: dfh, Feb 24, 99  Initial version
 *  revisions:
 *  $History: hal_sdram.h $
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 12/05/00   Time: 6:43p
 * Updated in $/dev_1200/Include
 * Replaced enums with #defines -- VxWorks sets the value to 0 if either
 * bit 31 or 30 if set.
 * 
 * *****************  Version 19  *****************
 * User: Drjohns1     Date: 11/30/00   Time: 8:26p
 * Updated in $/dev_1200/Include
 * Parentisized macro arguments
 * 
 * *****************  Version 18  *****************
 * User: Ewalker      Date: 11/30/00   Time: 1:49p
 * Updated in $/dev_1200/Include
 * Included iostyle.h
 * 
 * *****************  Version 17  *****************
 * User: Ewalker      Date: 11/30/00   Time: 1:45p
 * Updated in $/dev_1200/Include
 * Removed extraneous semicolon from end of macro definition.
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 11/16/00   Time: 4:50p
 * Updated in $/dev_1200/Include
 * 
 * *****************  Version 15  *****************
 * User: Dmwhite      Date: 4/05/00    Time: 4:37p
 * Updated in $/dev_1200/Include
 * took out some unwanted comments
 * 
 * ---------------------------------------------------------------------
 */
#ifndef HAL_SDRAM_H
#define HAL_SDRAM_H

#include "iostyle.h"
#include "core_io.h"

#if (IOSTYLE==XACTIO)

#define SDRAM_WRITE(addr, val) XactFastWrite(SDRAM, (unsigned int)(val), (int)(addr))
#define SDRAM_READ(addr, val) XactFastRead(SDRAM, (int)(addr), (unsigned int *)(val))

#else

#define SDRAM_WRITE(addr, val) WRITE_LWORD(((addr)<<2) + SDRAM_NON_PREFETCH_BEGIN, (val))
#define SDRAM_READ(addr, val) READ_LWORD(((addr)<<2) + SDRAM_NON_PREFETCH_BEGIN, (val))

#endif

#define HAL_SDRAM_SUCCESS 1

/* SdramAddresses */
#define SDRAM_CSR_BASE				0xFF000000
#define CACHE_FLUSH_AREA_BEGIN		0xE0000000
#define CACHE_FLUSH_AREA_END 		0xE0FFFFFF
#define SDRAM_PREFETCH_BEGIN 		0xD0000000
#define SDRAM_PREFETCH_END			0xD1FFFFFF
#define SDRAM_NON_PREFETCH_BEGIN 	0xC0000000
#define SDRAM_NON_PREFETCH_END 		0xC1FFFFFF

/* SdramCsrByteOffsets */
#define SDRAM_CSR 					0x0
#define SDRAM_MEM_CTL0				0x4
#define SDRAM_MEM_CTL1				0x8
#define SDRAM_MEM_INIT				0xC



#ifdef __cplusplus

class SdramUnit{
public:
	SdramUnit(){}
	~SdramUnit(){}

#if ((IOSTYLE==HARDWARE)||(IOSTYLE==AMBAIO))

	inline void setSDRAM_CSR(unsigned int val){WRITE_LWORD(&sdram_csr, val);}
	inline void getSDRAM_CSR(unsigned int *val){READ_LWORD(&sdram_csr, val);}

	inline void setSDRAM_MEM_CTL0(unsigned int val){WRITE_LWORD(&sdram_mem_ctl0, val);}
	inline void getSDRAM_MEM_CTL0(unsigned int *val){READ_LWORD(&sdram_mem_ctl0, val);}

	inline void setSDRAM_MEM_CTL1(unsigned int val){WRITE_LWORD(&sdram_mem_ctl1, val);}
	inline void getSDRAM_MEM_CTL1(unsigned int *val){READ_LWORD(&sdram_mem_ctl1, val);}

	inline void setSDRAM_MEM_INIT(unsigned int val){WRITE_LWORD(&sdram_mem_init, val);}
	inline void getSDRAM_MEM_INIT(unsigned int *val){READ_LWORD(&sdram_mem_init, val);}

#endif		//((IOSTYLE==HARDWARE)||(IOSTYLE==AMBAIO))

	inline void write(unsigned int lwaddr, unsigned int val){SDRAM_WRITE(lwaddr, val);}
	inline void read(unsigned int lwaddr, unsigned int *val){SDRAM_READ(lwaddr, val);}

private:											// word offset: hex byte offset:
	unsigned int sdram_csr;							//		0			0x0
	unsigned int sdram_mem_ctl0;					//		1			0x4
	unsigned int sdram_mem_ctl1;					//		2			0x8
	unsigned int sdram_mem_init;					//		3			0xC
};

extern SdramUnit *SDRAM_Attach();


extern "C"
{
#endif		// __cplusplus


extern void SDRAM_Write(unsigned int addr, unsigned int val);
extern unsigned int SDRAM_Read(unsigned int addr);


#if defined(__cplusplus)
}
#endif			// __cplusplus

#endif			// HAL_SDRAM_H // leave extra blank line here

