/* hdbm.h
 * Hash Database Manager Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: IXP1200
 *  subsystem: Hash Database Manager Library
 *  author: drj Sept 6, 2000  Initial version
 *  revisions:
 *  $History: hdbm.h $
 * 
 * *****************  Version 6  *****************
 * User: Drjohns1     Date: 12/14/00   Time: 2:16p
 * Updated in $/dev_1200/Include
 * Fixed Release build problem
 * 
 * *****************  Version 5  *****************
 * User: Drjohns1     Date: 12/13/00   Time: 4:15p
 * Updated in $/dev_1200/Include
 * Fixed sdram overwrite bug
 * 
 * *****************  Version 4  *****************
 * User: Drjohns1     Date: 12/05/00   Time: 10:05a
 * Updated in $/dev_1200/Include
 * 
 * *****************  Version 3  *****************
 * User: Drjohns1     Date: 10/20/00   Time: 5:10p
 * Updated in $/dev_1200/Include
 * Dev update -- piggy back user data onto database table
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 10/10/00   Time: 3:48p
 * Updated in $/dev_1200/Include
 * Dev update
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 10/05/00   Time: 5:19p
 * Created in $/dev_1200/Include
 * Hash Database Manager
 * 
 * ---------------------------------------------------------------------
 */
#ifndef HDBM_H
#define HDBM_H

#include "sem.h"
#include "hal_sram.h"
#include "ixp_hash.h"

#define EOQ_MARKER			0xffffffff
#define HDBM_MAXLEVEL		33

#ifdef HDBM_EXT
#define HDBM_KEY_QWORDS		1		//key entry quadword size
#else
#define HDBM_KEYDATA_QWORDS	2		//key and data entry quadword size 
#endif


#ifdef CREATE_IND
void logSram(unsigned int addr, unsigned int val);
void logSdram(int full64, unsigned int addr, unsigned int val1, unsigned int val2);
#endif

typedef enum{
	FREELST_SRAM,
	FREELST_SDRAM
}FreeLstMemTyp_T;


enum {
	HDBM_SUCCESS,
	HDBM_ERROR,
	HDBM_NO_ENTRY,
	HDBM_ENTRY_COL,
	HDBM_ENTRY_NO_COL,
	HDBM_LOCK_ERROR,
	HDBM_DUPL_ENTRY,
	HDBM_NO_COL_TABLE,
	HDBM_EMPTY_COL_LST,
	HDBM_EMPTY_KEYDATA_LST,
	HDBM_BAD_LSTADDR,
	HDBM_BAD_LSTFULL,
	HDBM_BAD_LSTEMPTY,
	HDBM_BAD_SDADDR
};

typedef enum {
	HDBM_HWQ_0,
	HDBM_HWQ_1,
	HDBM_HWQ_2,
	HDBM_HWQ_3,
	HDBM_HWQ_4,
	HDBM_HWQ_5,
	HDBM_HWQ_6,
	HDBM_HWQ_7,
	HDBM_SWQ = 255
}HdmQueId_T;

typedef struct HdbmKey{
    unsigned int B3210;				// bytes 3-0
    unsigned int B7654;				// bytes 7-4
}HdbmKey_T;

class Que {
public:
	Que(void){}
	~Que(){};
	inline virtual int push(unsigned int eleAddr) = 0;
	inline virtual unsigned int pop(void) = 0;

#ifdef HDBM_DEBUG
	virtual void dumpQue(void){return;}
	virtual int chkQue(unsigned int hiAddr, unsigned int loAddr){return 0;}
#endif
};


class HwQ : public Que {
public:
	HwQ(unsigned short hwQId){
		pHwQId = hwQId;
		pSram = SRAM_Attach();
	}
	~HwQ(){}
#if (IOSTYLE == XACTIO)
	//	Hardware queue push/pop is not supported in xactio
	inline int push(unsigned int addr){printf("HW que push if not supported in XACTIO\n"); return(0);}
	inline unsigned int pop(void){printf("HW que pop if not supported in XACTIO\n"); return(0);}
#else
	inline int push(unsigned int addr){pSram->push(pHwQId, addr); return(0);}
	inline unsigned int pop(void){unsigned int addr; pSram->pop(pHwQId, &addr); return(addr);}
#endif

private:
	unsigned short pHwQId;
	SramUnit *pSram;
};


class SwQ : public Que {
public:
	SwQ(unsigned int nullVal, FreeLstMemTyp_T memType){pNull = nullVal; pQhead = pNull;	pMemType = memType;}
	~SwQ(){}
	int push(unsigned int eleAddr);
	unsigned int pop(void);

#ifdef HDBM_DEBUG
	void dumpQue(void);
	int chkQue(unsigned int hiAddr, unsigned int loAddr);
#endif

private:
	unsigned int pQhead, pMemType, pNull;
};


class FreeLst {
public:
	FreeLst(unsigned short qId, FreeLstMemTyp_T memType, unsigned int lstBase,
			unsigned int eleWordSize, unsigned int maxEle);
	~FreeLst(){}
	int allocEle(unsigned int *eleAddr);
	int freeEle(unsigned int eleAddr);
	inline unsigned int getBase(void){return(pLstBase);}
	inline int getAvailEle(void){return(pNumEle);}

#ifdef HDBM_DEBUG
	void dumpLst(void){pQ->dumpQue(); return;}
	int chkLst(void){return(pQ->chkQue(pLastAddr, pLstBase));}
#endif

private:
	Que *pQ;
	sem_T sem;
	unsigned int pLstBase, pLastAddr, pEleWordSize;
	unsigned int pMaxEle, pNumEle;
};



typedef struct HdbmPriTableDesc{
	unsigned int baseAddr; 			//SRAM base addr
	unsigned int indxStartBit;		//index bitpos in key	
	unsigned int numIndxBits;		//num of bits in index
}HdbmPriTableDesc_T;

typedef struct HdbmColTableDesc{
	FreeLst		 *freeLst;			//collision table free list object
	unsigned int indxStartBit;		//index bitpos in key	
	unsigned int numIndxBits;		//num of bits in index
	unsigned int colShiftBits;		//collision index stepping
}HdbmColTableDesc_T;


typedef enum{
	HASH_NONE=0,
	HASH48=48,
	HASH64=64
}HdbmHashType_T;


#ifdef HDBM_DEBUG
typedef struct{
	unsigned int collisions;
	unsigned int empty;
	unsigned int occupied;
	unsigned int totSpace;
}HdbmStats_T;
#endif


// Hash Database manager base class
class HashDBM{
public:
	HashDBM(HdbmHashType_T hashType, unsigned int hashMultHi, unsigned int hashMultLo,
		HdbmPriTableDesc *priTableDesc, HdbmColTableDesc *colTableDesc,	FreeLst *keyDataFreeLst);

	HashDBM(HdbmPriTableDesc *priTableDesc, HdbmColTableDesc *colTableDesc, FreeLst *keyDataFreeLst);

	~HashDBM();
	int Add(HdbmKey_T *key, void *userData);
	int Lookup(HdbmKey_T *key, void *userData);
	int Remove(HdbmKey_T *key);
	int Traverse(int (*funcPtr)(HdbmKey_T *key, void *dataEntry, void *userData), void *userData);
	int RemoveAllEntries(void);

protected:

// collision bit <31>, lock bit<30>, table index<29:0>
#define LOCK_MASK			0x40000000
#define COLLISION_MASK		0x80000000
#define TABLE_INDEX_POS		0
#define TABLE_INDEX_MASK	0x3fffffff


	void writeKeyData(unsigned int userDataArg, unsigned int sdAddr, HdbmKey_T *key, unsigned int userData);
	void readKeyData(unsigned int userDataArg, unsigned int sdAddr, HdbmKey_T *key, unsigned int *userData);
	unsigned int getIndexField(HdbmKey_T *key, unsigned int startBit, unsigned int fieldLen);
	int deleEntries(unsigned int tableBase, unsigned int tableSize, unsigned int level);

	inline void readFwdIndex(unsigned int sAddr, unsigned int *fwdIndex){sram->read(sAddr, fwdIndex);}
	inline void writeFwdIndex(unsigned int sAddr, unsigned int fwdIndex){
		sram->write(sAddr, fwdIndex);
#ifdef CREATE_IND
		logSram(sAddr, fwdIndex);
#endif
	}

	inline unsigned int getTableIndex(unsigned int dbv){return((dbv >> (TABLE_INDEX_POS)) & TABLE_INDEX_MASK);}
	inline unsigned int setTableIndex(unsigned int dbv, unsigned int ti){return((dbv & ~TABLE_INDEX_MASK) | ((ti & TABLE_INDEX_MASK) << TABLE_INDEX_POS));}
	inline unsigned int setCollision(unsigned int a){return(a | COLLISION_MASK);}
	inline unsigned int clrCollision(unsigned int a){return(a & (~COLLISION_MASK));}
	inline unsigned int setLock(unsigned int a){(a | LOCK_MASK);}
	inline unsigned int clrLock(unsigned int a){return(a & (~LOCK_MASK));}

	
	HdbmHashType_T pHashType;
	int hashEncode(HdbmKey_T *key);
	int getState(unsigned int base, unsigned int index, unsigned int *ret_ptr);
	unsigned int pPriTableBase, pPriStartBit, pPriIndxBitLen;
	unsigned int pColShiftMask, pColShiftBits, pColStartBit, pColIndxBitLen;
	unsigned int pPriTableSize, pColTableSize;
	FreeLst				*pColTabFreeLst;
	FreeLst				*pKeyDataFreeLst;

private:
	SramUnit			*sram;
	ixpHash48Handle    pHash48;
	ixpHash64Handle    pHash64;

#ifdef HDBM_DEBUG
	HdbmStats_T			pStats[HDBM_MAXLEVEL];
	unsigned int		pTotEntries;
	int					getStats(unsigned int tableBase, unsigned int tableSize, unsigned int level);
	void				printStats();
#endif

	virtual unsigned int getKeyIndex(unsigned int dbv){return(getTableIndex(dbv));}
	virtual unsigned int setKeyIndex(unsigned int dbv, unsigned int ki){return(setTableIndex(dbv, ki));}
	virtual int traverseDb(unsigned int tableBase, unsigned int tableSize,
					       int (*funcPtr)(HdbmKey_T *key, void *dataEntry, void *userData), void *userData);
public:
	int	analyze(void);

};

#ifdef HDBM_EXT
#include "hdbm_ext.h"
#endif		// HDBM_EXT

#endif
