/* ixp_hash.h
 * IXP Hash Function
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *
 *			INTEL CORPORATION PROPRIETARY INFORMATION
 *	This software is supplied under the terms of a license agreement or
 *	nondisclosure agreement with Intel Corporation and may not be copied
 *	or disclosed except in accordance with the terms of that agreement.
 *		Copyright (C) 2000 Intel Corporation. All rights Reserved.
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: IXP1200
 *  subsystem: IXP Hash Library
 *  author: 
 *  revisions:
 *  $History: ixp_hash.h $
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 12/04/00   Time: 10:15a
 * Created in $/dev_1200/Include
 * IXP1200 hash emulation
 * 
 * ---------------------------------------------------------------------
 */
#ifndef	__IXP_HASH_H
#define	__IXP_HASH_H


#include <sys/types.h>


/* there is no consistant "spelling" for integers that
 * are guaranteed to be at least 32, 48, or 64 bits.
 */
#ifdef WIN32
typedef unsigned __int64	ix_48bit;
typedef unsigned __int64	ix_64bit;
#else
typedef unsigned long long  ix_48bit;
typedef unsigned long long  ix_64bit;
#endif



typedef void * ixpHash48Handle;
typedef void * ixpHash64Handle;

/* ixp_hash return values */
enum{
	IXP_HASH_SUCCESS = 		0,		/* successful completion */
	IXP_HASH_INVLDARG =		1,		/* invalid argument */
	IXP_HASH_ALLOCFAIL =	2		/* resource allocation failure */
};


#ifdef	__cplusplus
extern "C" {
#endif

int Ixp_Hash48New(ixpHash48Handle *hashHandle);
int Ixp_Hash48Del(ixpHash48Handle hashHandle);
int Ixp_Hash48SetMult(ixpHash48Handle hashHandle, ix_48bit hashMultplier);
int Ixp_Hash48(ixpHash48Handle hashHandle, ix_48bit *outValue, ix_48bit inValue);

int Ixp_Hash64New(ixpHash64Handle *hashHandle);
int Ixp_Hash64Del(ixpHash64Handle hashHandle);
int Ixp_Hash64SetMult(ixpHash64Handle hashHandle, ix_64bit hashMultplier);
int Ixp_Hash64(ixpHash64Handle hashHandle, ix_64bit *outValue, ix_64bit inValue);

#ifdef	__cplusplus
};
#endif



#endif		/*__IXP_HASH_H*/
