/* ueng.h
 * Ucode Uengine Interface Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: drj, June 16,98  Initial version
 *  revisions:
 *  $History: ueng.h $
 * 
 * *****************  Version 37  *****************
 * User: Ewalker      Date: 11/30/00   Time: 6:41p
 * Updated in $/dev_1200/Include
 * Moved VERIFY_*  macros to debug library when they are actually used.
 * 
 * *****************  Version 36  *****************
 * User: Ewalker      Date: 11/27/00   Time: 2:28p
 * Updated in $/dev_1200/Include
 * 
 * *****************  Version 35  *****************
 * User: Ewalker      Date: 11/27/00   Time: 12:39p
 * Updated in $/dev_1200/Include
 * Renamed uEng_getDeviceType to uEng_getCpuRev
 * 
 * *****************  Version 34  *****************
 * User: Drjohns1     Date: 11/17/00   Time: 5:28p
 * Updated in $/dev_1200/Include
 * uC support
 * 
 * *****************  Version 33  *****************
 * User: Ewalker      Date: 11/16/00   Time: 12:53p
 * Updated in $/dev_1200/Include
 * Changed getDeviceType function name to uEng_getDeviceType. This in
 * conjunction with the new XACT_getDeviceType function in the transactor
 * allows the IOSTYLE switch in core_io.h to define GetDeviceType to pick
 * between the two functions.
 * 
 * *****************  Version 32  *****************
 * User: Drjohns1     Date: 10/27/00   Time: 2:27p
 * Updated in $/dev_1200/Include
 * Updated for C0 fixes
 * 
 * *****************  Version 31  *****************
 * User: Drjohns1     Date: 10/20/00   Time: 5:09p
 * Updated in $/dev_1200/Include
 * Support for C0
 * 
 * *****************  Version 30  *****************
 * User: Drjohns1     Date: 8/31/00    Time: 3:18p
 * Updated in $/dev_1200/Include
 * Updated for C0 changes
 * 
 * ---------------------------------------------------------------------
 */
#ifndef __UENG_H
#define __UENG_H

#include "hal_ueng.h"
#include "sem.h"
#include "ueng_constants.h"
#include "core_io.h"

#define ADDR_EN_BITPOS				10
#define USTORE_2K_EN_BITPOS			11
#define CTX_ENABLES_BITPOS			8
#define CTX_ACTIVE_AB_BITPOS		2
#define CTX_ACTIVE_AB_MASK			0x4
#define CTX_X_USTORE_SEGMENT_BITPOS	14
#define CTX_X_USTORE_SEGMENT_MASK	0x4000

#define IS_CTX_RUNNING(activeCtxSts, ctx) ((activeCtxSts & 0x4) & ((activeCtxSts & 0x3) == ctx))
#define CTX_X_STS_PC(csr) (csr & 0x3ff)

#define USTORE_SEGMENT_BITPOS	10
#define UENG_MAX_USTORE			(1 << USTORE_SEGMENT_BITPOS)	/* max ustore per segment */
#define MAX_EXEC_INST			100
#define MAX_XFER_REG			8
#define MAX_FIFO_QWADDR			160
#define UENG_ALL_CTX			0xf
#define UENG_ALL_UENG			0x3f

#define ACTIVE_CTX_MASK			0x3
#define IS_ACTIVE_MASK			0x4
#define BR_ADDR_LSB				7
#define UADDR_MASK				0x3ff
#define UADDR_2K_MASK			0x7ff
#define LONG_BR_BIT				17
#define BR_CTX_LSB				23
#define BR_CTX_MASK				0x3
#define SET_BR_ADDR(brInst, brAddr) brInst | ((brAddr & UADDR_MASK) << BR_ADDR_LSB)
#define SET_LONG_BR_ADDR(brInst, brAddr) brInst | ((brAddr & UADDR_2K_MASK) << BR_ADDR_LSB)

#define SET_BR_CTX(brInst, ctx) brInst | ((ctx & BR_CTX_MASK) << BR_CTX_LSB)

typedef enum{
   UENG_MSG_INT,
   UENG_DBG_INT,
   UENG_NUM_INTS			/* number of ueng interrupts defined -- must be the last entry */
}Ueng_ints_T;

typedef enum{
	IXP1200_REV_A3=0,
	IXP1200_REV_B0=1,
	IXP1200_REV_C0=3,
	IXP1200_REV_D0=5
}IxpDevRev_T;


#ifdef __cplusplus
class Uengine{
public:
   Uengine(unsigned char uEngNum);
   ~Uengine(){Sem_Destroy(uEngSem);}

   inline void getCtxEnables(unsigned int *ctxEnables){
	   uEngCsr->getCTX_ENABLES(ctxEnables);
	   return;
   }

   inline void getActiveStatus(unsigned int *activeCtxStatus){
	   uEngCsr->getACTIVE_CTX_STS(activeCtxStatus);
	   return;
   }

   inline void getCsrWriteEn(unsigned int *csrWriteEn){
	   uEngCsr->getFBOX_CSR_WRITE_SELF_ENABLE(csrWriteEn);
	   return;
   }
   inline void getSramJouEn(unsigned int *sramJouEn){
	   uEngCsr->getENABLE_SRAM_JOURNALING(sramJouEn);
	   return;
   }
   inline void getCondCodeEn(unsigned int *ccEn){
	   uEngCsr->getCC_ENABLE(ccEn);
	   return;
   }
   inline void setCondCodeEn(unsigned int ccEn){
	   uEngCsr->setCC_ENABLE(ccEn);
	   return;
   }

   inline int getActiveCtx(){
	   unsigned int activeCtxStatus;
	   uEngCsr->getACTIVE_CTX_STS(&activeCtxStatus);
	   if(!(activeCtxStatus & 0x4)) return MAX_CTX;	// invalid context
	   return(activeCtxStatus & 0x3);
   }

   inline unsigned int setBrInstAddr(unsigned int brInst, unsigned int uAddr){
		if(uAddr & UENG_MAX_USTORE) return(SET_LONG_BR_ADDR(brInst, uAddr));
		else return(SET_BR_ADDR(brInst, uAddr));
   }

   int disableCtx(unsigned int ctxMask);
   int enableCtx(unsigned int ctxMask);

   int getPC(unsigned char ctx, unsigned int *upc);
   int setPC(unsigned int ctxMask, unsigned int upc);
   int getCtxStatus(unsigned char ctx, unsigned int *ctxStatus);
   int setCtxStatus(unsigned int ctxMask, unsigned int ctxStatus);

   int getCtxSigEvents(unsigned char ctx, unsigned int *events);
   int setCtxSigEvents(unsigned int ctxMask, unsigned int events);
   int getCtxWakeupEvents(unsigned char ctx, unsigned int *events);
   int setCtxWakeupEvents(unsigned int ctxMask, unsigned int events);

   int isUengActive();
   int getUword(unsigned int uAddr, unsigned int *uWord);
   int setUword(unsigned int uAddr, unsigned int uWord);
   int start(unsigned char startCtx, unsigned int ctxMask);
   inline int resume(void){return(enableCtx(0xf));}			/* enable all contexts */
   int getCtxArb(int unsigned *ctxArbCtl);
   int setCtxArb(int unsigned ctxArbCtl);
   int getNumUword(unsigned int uAddr, unsigned short numWords, unsigned int *uWord);
   int setNumUword(unsigned int uAddr, unsigned short numWords, unsigned int *uWord);


   void *getHandle(void){return((void *)this); }

   inline int readRelGpr(unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int *data){
	   return(readRelReg(type, ctx, regNum, data));}
   inline int writeRelGpr(unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int data){
	   return(writeRelReg(type, ctx, regNum, data));}

   inline int readRelSramRdXfer(unsigned char ctx, unsigned short regNum, unsigned int *data){
	      return(readRelReg(2, ctx, regNum, data));}
   int readRelSramWrXfer(unsigned char ctx, unsigned short regNum, unsigned int *data);
   int writeRelSramRdXfer(unsigned char ctx, unsigned short regNum, unsigned int data);
   int writeRelSramWrXfer(unsigned char ctx, unsigned short regNum, unsigned int data);

   int writeRelSdramWrXfer(unsigned char ctx, unsigned short regNum, unsigned int *data);
   int writeRelSdramRdXfer(unsigned char ctx, unsigned short regNum, unsigned int *data);
   int readRelSdramWrXfer(unsigned char ctx, unsigned short regNum, unsigned int *data);
   int readRelSdramRdXfer(unsigned char ctx, unsigned short regNum, unsigned int *data);

   int writeTxFifo(unsigned short regNum, unsigned int *data, int setValid);
   int readRxFifo(unsigned short regNum, unsigned int *data);

   int execMicroInst(unsigned char ctx, unsigned int *microInst,
						   unsigned int numInst, int condCodeOff,
						   unsigned int waitForMs = 5);

   int setUstoreFreeMem(unsigned int begFreeAddr, unsigned int size);
   int getUstoreFreeMem(unsigned int *begFreeAddr, unsigned int *size);
   int setUstoreFreeMem(unsigned int seg1FreeAddr, unsigned int seg1FreeSize,
						unsigned int seg2FreeAddr, unsigned int seg2FreeSize);
   int getUstoreFreeMem(unsigned int *seg1FreeAddr, unsigned int *seg1FreeSize,
						unsigned int *seg2FreeAddr, unsigned int *seg2FreeSize);
   inline void setRst(void){rst = 1; return;}
   inline void clrRst(void){rst = 0; return;}
   void reset(int clrReg=0);
   void clrReset();

   /* dj: debug */
   void dump(void);

private:
   unsigned int		thisUengNum, rst, uStoreSize;
   unsigned int		freeAddr, freeAddr2, freeSize, freeSize2;
   UengCSRs			*uEngCsr;
   SramXferReg		*sramXferReg[MAX_CTX];
   sem_T			uEngSem;
   int				semLock;

   enum{LOW=0,HIGH};

   int readRelReg(unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int *data);
   int writeRelReg(unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int data);

   inline int lockAccess(int level){
      if((level == HIGH) || (semLock == LOW)){
	     Sem_Wait(uEngSem, SEM_WAITFOREVER);
	     semLock = level;
		 return(1);
	  }
	  return(0);
   }

   inline void unlockAccess(int key){
	   if(key){
		   semLock = LOW;
		   Sem_Post(uEngSem);
	   }
   }
}; 

Uengine **initUengs();
#endif		/* c++ */


#ifdef __cplusplus
extern "C" {
#endif

int uEng_getCpuRev(void);  /* returns chip revision number - A3=0, B0=1, C0=3, D0=5 (see IxpDevRev_T values) */

int uEng_intConnect(int index, unsigned int intMask, ISR_FUNC isr, int param);
void uEng_clrIregBp();
void uEng_clrIregTh(unsigned int thMask);
void uEng_disableUEFIQ();
void uEng_enableUEFIQ();
void uEng_disableUEIRQ();
void uEng_enableUEIRQ();

int uEng_getCtxEnables(unsigned char uEng, unsigned int *ctxEnables);
int uEng_getActiveStatus(unsigned char uEng, unsigned int *activeCtxStatus);
int uEng_getCsrWriteEn(unsigned char uEng, unsigned int *csrWriteEn);
int uEng_getSramJouEn(unsigned char uEng, unsigned int *sramJouEn);
int uEng_getCondCodeEn(unsigned char uEng, unsigned int *ccEn);
int uEng_setCondCodeEn(unsigned char uEng, unsigned int ccEn);

void uEng_initUengs();
int uEng_getCtxStatus(unsigned char uEng, unsigned char ctx, unsigned int *status);
int uEng_setCtxStatus(unsigned char uEng, unsigned int ctxMask, unsigned int status);
int uEng_getCtxSigEvents(unsigned char uEng, unsigned char ctx, unsigned int *events);
int uEng_setCtxSigEvents(unsigned char uEng, unsigned int ctxMask, unsigned int events);
int uEng_getCtxWakeupEvents(unsigned char uEng, unsigned char ctx, unsigned int *events);
int uEng_setCtxWakeupEvents(unsigned char uEng, unsigned int ctxMask, unsigned int events);

int uEng_setUstoreFreeMem(unsigned char uEng, unsigned int begFreeAddr, unsigned int size);
int uEng_getUstoreFreeMem(unsigned char uEng, unsigned int *begFreeAddr, unsigned int *size);

int uEng_setUstoreSegFreeMem(unsigned char uEng,
							 unsigned int seg1FreeAddr, unsigned int seg1FreeSize,
							 unsigned int seg2FreeAddr, unsigned int seg2FreeSize);

int uEng_getUstoreSegFreeMem(unsigned char uEng,
							 unsigned int *seg1FreeAddr, unsigned int *seg1FreeSize,
							 unsigned int *seg2FreeAddr, unsigned int *seg2FreeSize);


int uEng_disableCtx(unsigned char uEng, unsigned int ctxMask);
int uEng_getPC(unsigned char uEng, unsigned char ctx, unsigned int *upc);
int uEng_setPC(unsigned char uEng, unsigned int ctxMask, unsigned int upc);
int uEng_isUengActive(unsigned char uEng);
int uEng_start(unsigned char uEng, unsigned char startCtx, unsigned int ctxEnMask);

int uEng_resume(unsigned char uEng);
int uEng_getNumUword(unsigned char uEng, unsigned int uAddr,
					 unsigned short numWords, unsigned int *uWord);
int uEng_setNumUword(unsigned char uEng, unsigned int uAddr,
					 unsigned short numWords, unsigned int *uWord);
int uEng_getUword(unsigned char uEng, unsigned int uAddr, unsigned int *uWord);
int uEng_setUword(unsigned char uEng, unsigned int uAddr, unsigned int uWord);
int uEng_getCtxArb(unsigned char uEng, int unsigned *ctxArbCtl);
int uEng_setCtxArb(unsigned char uEng, int unsigned ctxArbCtl);
int uEng_readRelGpr(unsigned char uEng, unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int *data);
int uEng_writeRelGpr(unsigned char uEng, unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int data);
int uEng_readRelSramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data);
int uEng_readRelSramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data);
int uEng_writeRelSramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int data);
int uEng_writeRelSramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int data);

int uEng_writeRelSdramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data);
int uEng_writeRelSdramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data);
int uEng_readRelSdramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data);
int uEng_readRelSdramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data);

int uEng_writeFIFO_Tx(unsigned char uEng, unsigned int fifoAddr, unsigned int *data64, int setValid);
int uEng_readFIFO_Rx(unsigned char uEng, unsigned int fifoAddr, unsigned int *data64);

int uEng_execMicroInst(unsigned char uEng, unsigned char ctx, unsigned int *microInst,
						   unsigned int numInst, int condCodeOff, unsigned int waitForMs);

void uEng_resetUengs(unsigned int uEngMask, int clrReg);
void uEng_clrResetUengs(unsigned int uEngMask);

#ifdef __cplusplus
}
#endif

#endif			/* __UENG_H */
