/* uof.h
 * uCode Object File header file
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: SA1200
 *  subsystem: Linker/Loader
 *  author: drj Dec 31, 97  Initial version
 *  revisions:
 *  $History: uof.h $
 * 
 * *****************  Version 23  *****************
 * User: Drjohns1     Date: 11/28/00   Time: 6:55p
 * Updated in $/dev_1200/Include
 * Changed local variable prefix
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 11/21/00   Time: 1:26p
 * Updated in $/dev_1200/Include
 * Added retrieval of variables memory segments values
 * 
 * *****************  Version 21  *****************
 * User: Drjohns1     Date: 11/17/00   Time: 5:28p
 * Updated in $/dev_1200/Include
 * uC support
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 11/16/00   Time: 4:50p
 * Updated in $/dev_1200/Include
 * Removed c++ style comments
 * 
 * *****************  Version 19  *****************
 * User: Drjohns1     Date: 11/15/00   Time: 3:44p
 * Updated in $/dev_1200/Include
 * Added uC support
 * 
 * *****************  Version 18  *****************
 * User: Drjohns1     Date: 9/06/00    Time: 6:41p
 * Updated in $/dev_1200/Include
 * Fixed CPU rev checking.
 * 
 * *****************  Version 17  *****************
 * User: Drjohns1     Date: 9/05/00    Time: 7:08p
 * Updated in $/dev_1200/Include
 * Added CPU rev checking.
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 8/31/00    Time: 3:18p
 * Updated in $/dev_1200/Include
 * Updated for C0 changes
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 5/06/99    Time: 5:57p
 * Updated in $/dev_1200/Include
 * Fixed fill-pattern being counted as valid uwords
 * 
 * the making of DLL/lib
 *
 * ---------------------------------------------------------------------
 */
#ifndef __UOF_H__
#define __UOF_H__

#ifdef __cplusplus
extern "C"{
#endif

#define UOF_MAX_FENGINES    6
#define UOF_MAX_THREADS	    4
#define UOF_MAX_PAGES 	    1
#define UOF_MAX_UWORDS_SEG	1024
#define UOF_MAX_UWORDS	    UOF_MAX_UWORDS_SEG * 2
#define UOF_OBJID_LEN       8
#define UOF_FIELD_POS_SIZE  12

#define UOF_MAXFUNC_ID      255
#define UOF_AUTONOMOUS_TYPE 0
#define UOF_SERVICE_TYPE    1

#define UOF_ONEWAY	    1
#define UOF_TWOWAY	    2

#define UOF_CFID_0	    0xC6C2
#define UOF_CFID_1	    0xC3CC

#define UOF_OBJS	    "UOF_OBJS"

#define UOF_BLNK	    "        "
#define UOF_STRT	    "UOF_STRT"
#define UOF_GTID	    "UOF_GTID"
#define UOF_IMAG	    "UOF_IMAG"
#define UOF_IMEM	    "UOF_IMEM"
#define UOF_MSEG	    "UOF_MSEG"

#define UOF_SUCCESS	    0
#define UOF_FAILURE	    1

#define EXPFUNC_TID_MSK     0x00ff
#define EXPFUNC_TYP_MSK     0xff00
#define EXPFUNC_TID_LSB     0
#define EXPFUNC_TYP_LSB     8

#define ADDR_ATTR_UWORD(addrAttr) ((addrAttr & 0x0000ffff))
#define EXP_FUNC_TID(threadId) ((threadId & 0x0000000f))
#define IMAGE_MICRO_ENGINE(imageAttr) ((imageAttr & 0x000000ff))
#define IMAGE_THREAD_TYPE(imageAttr) ((imageAttr & 0x00000f00) >> 8)
#define IMAGE_FILL_PATTERN(imageAttr) ((imageAttr & 0x01000))

#define SET_EXPFUNC_TID(attr, val) attr &= ~EXPFUNC_TID_MSK; attr |= (val << EXPFUNC_TID_LSB)
#define SET_EXPFUNC_TYP(attr, val) attr &= ~EXPFUNC_TYP_MSK; attr |= (val << EXPFUNC_TYP_LSB)
#define GET_EXPFUNC_TID(attr) ((attr & EXPFUNC_TID_MSK) >> EXPFUNC_TID_LSB)
#define GET_EXPFUNC_TYP(attr) ((attr & EXPFUNC_TYP_MSK) >> EXPFUNC_TYP_LSB)

#define GLOBAL_SCOPE	0
#define LOCAL_SCOPE		1

#define LOCAL_NAME(localName, symName, uengNum) sprintf(localName, "%d!%s", uengNum, symName)

typedef struct uof_fileHdr_S{
   char 		fileId[UOF_OBJID_LEN];	/* chunk file id and endian indicator */
   short 		maxChunks;				/* max chunks in file */
   short 		numChunks;				/* num of actual chunks */
   /* maxChunks of uof_fileChunkHdr_T immediately follows this header */
}uof_fileHdr_T;

typedef struct uof_fileChunkHdr_S{
   char 		chunkId[UOF_OBJID_LEN];	/* chunk identifier */
   unsigned int	checksum;				/* chunk checksum */
   int	 		offset;					/* offset of the chunk in the file */
   int	 		size;					/* size of the chunk */
}uof_fileChunkHdr_T;					/* 20 bytes total */

typedef struct uof_objHdr_S{
   unsigned int cpuType;				/* CPU type */
   short		minCpuVer;				/* starting CPU version */
   short		maxCpuVer;				/* ending CPU version */
   short 		maxChunks;				/* max chunks in chunk obj */
   short 		numChunks;				/* num of actual chunks */
   unsigned int	reserved1;				/* reserved for future use */
   unsigned int	reserved2;				/* reserved for future use */
   /* maxChunks of uof_chunkHdr_T immediately follows this header */
}uof_objHdr_T;

typedef struct uof_chunkHdr_S{
   char 		chunkId[UOF_OBJID_LEN];	/* chunk identifier */
   int	 		offset;					/* offset of the chunk in the file */
   int	 		size;					/* size of the chunk */
}uof_chunkHdr_T;						/* 16 bytes total */


typedef struct uof_strTab_S{
   int			tableLen;				/* length of table */
   char			*strings;				/* NULL terminated strings */
}uof_strTab_T;


typedef struct uof_initMem_S{
   char			region;					/* memory region -- SRAM=0, SDRAM=1, SCRATCH=2 */
   char			reserved1;				/* reserved for future use */
   short		numLwords;				/* number of long-words */
   unsigned int addr;					/* memory address */
   unsigned int valOffset;				/* file offset of values */
}uof_initMem_T;


typedef struct uof_VarMemSeg_S{
   unsigned int sramBase;				/* uC variables SRAM memory segment base address */
   unsigned int sramSize;				/* uC variables SRAM segment size bytes */
   unsigned int sdramBase;				/* uC variables SDRAM memory segment base address */
   unsigned int sdramSize;				/* uC variables SDRAM segment size bytes */
   unsigned int scratchBase;			/* uC variables SCRATCH memory segment base address */
   unsigned int scratchSize;			/* uC variables SCRATCH segment size bytes */
}uof_varMemSeg_T;


typedef struct uof_GTID_S{
   char			toolId[UOF_OBJID_LEN];	/* creator tool ID */
   int			toolVersion;			/* creator tool version */
   unsigned int	reserved1;				/* reserved for future use */
   unsigned int	reserved2;				/* reserved for future use */
}uof_GTID_T;


typedef struct uof_codePage_S{
   short		impVarTabSize;			/* import var table size */
   short		codeAreaSize;			/* code area size */
   short		expFuncTabSize;			/* export func table size */
   short		ucVarTabSize;			/* uC variable table size */
   unsigned int	ucVarTabOffset;			/* offset to uC var table */
   int			impVarTabOffset;		/* offset to import var table */
   int			codeAreaOffset;			/* offset to code area */
   int			expFuncTabOffset;		/* offset to export func table */
}uof_codePage_T;

typedef struct uof_Image_S{
   int			imageName;				/* image name */
   int			fillPattern;			/* 4-byte fill pattern */
   unsigned int	reserved1;				/* reserved for future use */
   unsigned int	reserved2;				/* reserved for future use */
   unsigned int	cpuType;				/* cpu type */
   unsigned short maxVer;				/* max cpu ver on which the image can run */
   unsigned short minVer;				/* min cpu ver on which the image can run */
   short		imageAttrib;			/* image attributes */
   short		entryPage;				/* entry point upage num */
   short		entryAddress;			/* entry point uaddress */
   short		numOfPage;				/* number of pages */
   /* numOfPage of uof_codePage follows this header */
}uof_Image_T;


typedef struct uof_importVar_S{
   int			varName;				/* offset to string table */
   int			uwordAddress;			/* micro word address */
   char			fieldAttrs[UOF_FIELD_POS_SIZE];	/* field pos, size, and right shift value */
}uof_importVar_T;


typedef struct uof_impVarTab_S{
   int			numEntries;				/* number of table entries */
   /* numEntries of uof_importVar_T follows */
}uof_impVarTab_T;


typedef struct uof_ucVar_S{
   int			symName;				/* offset to string table */
   char			scope;					/* global=0, local=1*/
   char			reserved1;				/* reserved for future use */
   short		reserved2;				/* reserved for future use */
   int			uwordAddress;			/* micro word address */
   int			value;					/* microword fixup value */
   char			fieldAttrs[UOF_FIELD_POS_SIZE];	/* field pos, size, and right shift value */
}uof_ucVar_T;


typedef struct uof_ucVarTab_S{
   int			numEntries;				/* number of table entries */
   /* numEntries of uof_ucVar_T follows */
}uof_ucVarTab_T;




typedef struct uof_expFuncTab_S{
   int			numEntries;				/* number of table entries */
   /* numEntries of uof_exportFunc_T follows */
}uof_expFuncTab_T;


typedef struct uof_exportFunc_S{
   int			funcName;		/* func name offset to string table */
   short		funcAddress;	/* func micro word address */
   short		attr;			/* threadID(byte0:<3:0>), funcType(byte1:<1:0>) */
   short		inArgLen;		/* input arg len in long-words */
   short		outArgLen;		/* output arg len in long-words */
}uof_exportFunc_T;


typedef struct uof_codeArea_S{
   int 			numMicroWords;			/* number of micro words in seg1 */
   int 			numMicroWords_2;		/* number of micro words in seg2 */
   /* numMicroWords + numMicroWords_2 of microWords follows */
}uof_codeArea_T;



/* The following structures are used to encapsulate the uof image--they will
   be mapped over the uof image in memory */

typedef struct uof_encapUofObj_S{
   char				*begUof;
   uof_objHdr_T 	*objHdr;
   uof_chunkHdr_T	*chunkHdr;
   uof_varMemSeg_T	*varMemSeg;
}uof_encapUofObj_T;

typedef struct uof_encapPage_S{
   int				numUcVar;			/* num of uC var in array */
   uof_ucVar_T		*ucVar;				/* array of import variables */
   int				numImpVar;			/* num of import var in array */
   uof_importVar_T	*importVar;			/* array of import variables */
   int				numExpFunc;			/* num of export func in array */
   uof_exportFunc_T	*exportFunc;		/* array of export functions */
   int				numMicroWords;		/* microwords in the seg1 */
   int				numMicroWords_2;	/* microwords in the seg2 */
   int				*microWords;		/* numMicroWords + numMicroWords_1 uwords */
}uof_encapPage_T;

typedef struct uof_encapFeng_S{
   uof_Image_T		*images;
   uof_encapPage_T	*pages;				/* array of pages */
}uof_encapFeng_T;

typedef struct uof_initMemTab_S{
	unsigned int numEntries;			/* number of memory init entries */
	uof_initMem_T *initMem;				/* array of init memory values */
}uof_initMemTab_T;

#ifdef __cplusplus
}
#endif

#endif	/* __UOF_H__ */
