//---------------------------------------------------------------------------
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
//---------------------------------------------------------------------------
// stdmac.uc
// standard macros
//-----------------------------------------------------------------------------------
//
// system: SA1200
// subsystem: microcode
// usage: common macros
// author: dfh 1/12/99
// revisions:
//
//-----------------------------------------------------------------------------------

// constant args allow us to pass constants to functions w/o using registers
//
#define CONST_ARG1 0
#define CONST_ARG2 0
#define CONST_ARG3 0
#define CONST_ARG4 0


// immed32[reg, val]
//   load reg with 32 bit value
//   if value fits in 16 bits, expand to a single immed 
//   if value doesn't, expand into a immed_w0 and immed_w1
#macro immed32[reg,val]
#if   (((val & 0xFFFF0000) == 0) || ((val & 0xFFFF0000) == 0xFFFF0000))
    immed[reg,val]
#elif (((val & 0xFF0000FF) == 0) || ((val & 0xFF0000FF) == 0xFF0000FF))
    immed[reg, (val >> 8), <<8]
#elif ((val & 0x0000FFFF) == 0)
    /* make sure that number is not seen as negative */
    immed[reg, (0x0000FFFF & (val >> 16)), <<16]
#elif ((val & 0x0000FFFF) == 0x0000FFFF)
    /* make sure that number is seen as negative */
    immed[reg, (0xFFFF0000 | (val >> 16)), <<16]
#else
    immed_w0[reg, (val & 0xFFFF)]
    immed_w1[reg, ((val >> 16) & 0xFFFF)]
#endif
#endm

// balr[reg, label]
//   branch and link
#macro balr[reg, label]
    br [label], defer[1]
    load_addr[reg, ret_lab#]
ret_lab#:
#endm

// move[dst, src]
//   move the source value into the destination register
//
#macro move[dst, src]
    alu[dst, --, b, src]
#endm


// delay[n]
//	delay approx n cycles
//
#macro delay[n]
.local delay_val
	immed[delay_val, n]
	alu_shf[delay_val, --, B, delay_val, >>2]		; to account for 4 cycles per iteration
	.while (delay_val > 0)
	alu[delay_val, delay_val, -, 1]
	.endw
.endlocal
#endm


// shf_right_ind
//		reg_dest		destination register
//		reg_source		source register
//		reg_shift		register with shift amount
//
#macro shf_right_ind[reg_dest, reg_source, reg_shift]
	alu[--, reg_shift, B, 0]
	alu[reg_dest, --, B, reg_source, >>indirect]
#endm


// shf_left_ind
//		reg_dest		destination register
//		reg_source		source register
//		reg_shift		register with shift amount
//
#macro shf_left_ind[reg_dest, reg_source, reg_shift]
.local temp
	alu[temp, reg_shift, B-A, 1, <<5]
	alu[--, temp, B, 0]
	alu[reg_dest, --, B, reg_source, >>indirect]
.endlocal
#endm


// br_bset_ind
//		reg_test		register under test
//		reg_bit_pos		bit position to test
//		target_label	if bit position set, branch to this label
//
//
#macro br_bset_ind[reg_test, reg_bit_pos, target_label]
	alu[--, reg_bit_pos, B, 0]
	alu[--, 1, AND, reg_test, >>indirect]
	br>0[target_label]
#endm


// br_bclr_ind
//		reg_test		register under test
//		reg_bit_pos		bit position to test
//		target_label	if bit position clear, branch to this label
//
#macro br_bclr_ind[reg_test, reg_bit_pos, target_label]
	alu[--, reg_bit_pos, B, 0]
	alu[--, 1, AND, reg_test, >>indirect]
	br=0[target_label]
#endm


// bits_set_ind[reg, reg_bit_pos, mask]
//		reg				register to modify
//		reg_start_bit	register or constant<32, starting bit position
//		reg_mask		register or constant<32, bit mask							
//
#macro bits_set_ind[reg, reg_start_bit, reg_mask]
.local temp
	alu[temp, reg_start_bit, B-A, 1, <<5]
	alu[--, temp, B, 0]
	alu_shf[reg, reg, OR, reg_mask, <<indirect]
.endlocal
#endm


// bits_clr_ind[reg, reg_bit_pos, mask]
//		reg				register to modify
//		reg_start_bit	register or constant<32, starting bit position
//		reg_mask		register or constant<32, bit mask							
//
#macro bits_clr_ind[reg, reg_start_bit, reg_mask]
.local temp
	alu[temp, reg_start_bit, B-A, 1, <<5]
	alu[--, temp, B, 0]
	alu_shf[reg, reg, AND~, reg_mask, <<indirect]
.endlocal
#endm


#define SYNC 0
#define ASYNC 1
#define NOSYNC 2


// scratch_bset_ind
//	set the bit indicated by position at scratch address const_addr, using transfer reg reg_xfer
//	no context swap or sig_done
//
#macro scratch_bset_ind[reg_xfer, position, const_addr, sig_spec]
.local tempa tempb
	alu[tempa, position, B-A, 1, <<5]							; invert shift amount (32 - output_port) for left indirect
	alu[--, tempa, B, 0]
	alu_shf[reg_xfer, --, B, 1, <<indirect]
	immed[tempb, const_addr]
#if (sig_spec == SYNC)
	scratch [bit_wr, reg_xfer, tempb, 0, set_bits], ctx_swap
#elif (sig_spec == ASYNC)	
	scratch [bit_wr, reg_xfer, tempb, 0, set_bits], sig_done
#else
	scratch [bit_wr, reg_xfer, tempb, 0, set_bits]
#endif

.endlocal
#endm

// scratch_bclr_ind
//	clear the bit indicated by position at scratch address const_addr, using transfer reg reg_xfer
//	no context swap or sig_done
//
#macro scratch_bclr_ind[reg_xfer, position, const_addr, sig_spec]
.local tempa tempb
	alu[tempa, position, B-A, 1, <<5]							; invert shift amount (32 - output_port) for left indirect
	alu[--, tempa, B, 0]
	alu_shf[reg_xfer, --, B, 1, <<indirect]
	immed[tempb, const_addr]
#if (sig_spec == SYNC)
	scratch [bit_wr, reg_xfer, tempb, 0, clear_bits], ctx_swap
#elif (sig_spec == ASYNC)	
	scratch [bit_wr, reg_xfer, tempb, 0, clear_bits], sig_done
#else
	scratch [bit_wr, reg_xfer, tempb, 0, clear_bits]
#endif

.endlocal
#endm



//sdram_r_fifo_rd
//	transfer r_fifo to sdram
//
//	this hides the indirect manipulations
//	example:
//	transfer 8 quadwords to/from address offset 4
//		sdram_r_fifo_rd[sdaddr, raddr, 4, 8, SYNC]
//
#macro sdram_r_fifo_rd[sd_addr, r_fifo_addr, const_offset, const_qw_count, sig_spec]

#if (const_offset == 0)
	alu_shf[--, --, B, r_fifo_addr, <<4]
#else
.local tempa
	alu_shf[tempa, --, B, const_offset, <<4]
	alu_shf[--, tempa, +, r_fifo_addr, <<4]
.endlocal
#endif

#if (sig_spec == SYNC)
	sdram[r_fifo_rd, --, sd_addr, const_offset, const_qw_count], indirect_ref, ctx_swap
#elif (sig_spec == ASYNC)	
	sdram[r_fifo_rd, --, sd_addr, const_offset, const_qw_count], indirect_ref, sig_done
#else
	sdram[r_fifo_rd, --, sd_addr, const_offset, const_qw_count], indirect_ref
#endif

#endm


//------------------------------MESSAGE macros----------------------------
//
// each message is 1 longword, with bit 31 reserved as an invalid bit
// location 0 of the msgq must be initialized to 0x80000000 to make it invalid
// when assembling the message, bit 31=0 indicates the message is valid
//
// 17 locations are used for a messageq 16 deep
// the messageq starts at msg_base_addr, and msg_id index increments by 4
// using +4 nibble add
// this results in circular messageq 0-16, 1-16, 1-16, etc.
//
#define MSGQ_SIZE 16

// msg_send
// send message to mailbox
//
#macro msg_send[xfer_reg, msg_base_addr, msg_id, const_sync]
#ifdef SRAM_MSG

#if (const_sync = SYNC)
	sram[write, xfer_reg, msg_base_addr, msg_id, 1], ctx_swap
#elif (const_sync = ASYNC)
	sram[write, xfer_reg, msg_base_addr, msg_id, 1], sig_done
#else
	sram[write, xfer_reg, msg_base_addr, msg_id, 1]
#endif

#else

#if (const_sync = SYNC)
	scratch[write, xfer_reg, msg_base_addr, msg_id, 1], ctx_swap
#elif (const_sync = ASYNC)
	scratch[write, xfer_reg, msg_base_addr, msg_id, 1], sig_done
#else
	scratch[write, xfer_reg, msg_base_addr, msg_id, 1]
#endif

#endif
#endm

// msgq_send
// send message to next slot on circular message queue
//	this assumes that queue is large enough that receiver takes messages
//	or is blocked so that messages won't overwrite
//  increment circular message queue pointer: msg_id
//
#macro msgq_send[xfer_reg, msg_base_addr, msg_id, const_sync]
	msg_send[xfer_reg, msg_base_addr, msg_id, const_sync]
	alu[msg_id, 1, +4, msg_id]					; 17 addresses then wrap
#endm



// msg_receive
// place message from (offset+msg_base) in xfer_reg
// test_and_set_bits is used to retrieve the message
//
#macro msg_receive[xfer_reg, msg_base_addr, msg_id, const_sync]
#ifdef SRAM_MSG

#if (const_sync = ASYNC)
	sram[bit_wr, xfer_reg, msg_base_addr, msg_id, test_and_set_bits], sig_done
#elif (const_sync = SYNC)
	sram[bit_wr, xfer_reg, msg_base_addr, msg_id, test_and_set_bits], ctx_swap
#else
	sram[bit_wr, xfer_reg, msg_base_addr, msg_id, test_and_set_bits]
#endif

#else

#if (const_sync = ASYNC)
	scratch[bit_wr, xfer_reg, msg_base_addr, msg_id, test_and_set_bits], sig_done
#elif (const_sync = SYNC)
	scratch[bit_wr, xfer_reg, msg_base_addr, msg_id, test_and_set_bits], ctx_swap
#else
	scratch[bit_wr, xfer_reg, msg_base_addr, msg_id, test_and_set_bits]
#endif

#endif
#endm


// msgq_receive
// get message. If invalid. repeat get message
// if valid, increment circular message queue pointer: msg_id
//
#macro msgq_receive[xfer_reg, msg_base_addr, msg_id, const_sync]
do#:
	msg_receive[xfer_reg, msg_base_addr, msg_id, const_sync]
	alu_shf[--, --, B, xfer_reg, >>31]
	br=0[do#]
	alu[msg_id, 1, +4, msg_id]					; 17 addresses then wrap
#endm


// msgq_wait
// for ASYNC messages, wait for completion of the previous send or receive request
//
#macro msgq_wait
#ifdef SRAM_MSG
	ctx_arb[sram]
#else
	ctx_arb[fbi]
#endif
#endm


// msgq_init
//	initialize all mailbox slots in the message queue to invalid
//
#macro msgq_init[write_xfer, msg_base_addr, const_sync]
	immed32[write_xfer, 0x80000000]
.local count
	immed[count, MSGQ_SIZE]
	.while (count > 0)
	msg_send[write_xfer, msg_base_addr, count, SYNC]
	alu[count, count, -, 1]
	.endw
.endlocal
	msg_send[write_xfer, msg_base_addr, count, const_sync]

#endm



//----------------------------FREELIST macros-------------------------

// There are 8 freelists (0-7) which could be used, see mem_map.h defines
//



// freelist_create
//	function call to create hardware supported freelist
//	calls freelist_create_func in stdfunc.uc
//
//		args:
//		arg_link_pc	tempa		return ucode address		
//		const_freelist_id		0-7, identify which of 8 hardware freelists
//		const_base_addr			starting address in sram for the memory region
//		const_stride			number of longwords per entry, including a first longword
//								which hardware uses as the node pointer
//		const_num_entries		number of nodes to be created
//
#macro freelist_create[const_freelist_id, const_base_addr, const_stride, const_num_entries]
.local  buf_addr ii count

	immed32[buf_addr, const_base_addr]					; base address for memory region
	immed[ii, 0]
	immed32[count, const_num_entries]					; num entries

freelist_loop#:

	sram[push, --, buf_addr, ii, const_freelist_id]		; push to freelist
	alu[ii, ii, +, const_stride]						; add stride
	alu[count, count, -, 1]
	br>0[freelist_loop#]

	immed[$xfer7, 0]
	sram[write, $xfer7, buf_addr, 0, 1], ctx_swap		; end of list is set to 0

.endlocal

#endm



// malloc
//		get next available address (mem_ptr) from freelist (freelist_id)
//		using the read transfer reg read_xfer_reg
//
#macro malloc[mem_ptr, freelist_id, read_xfer_reg]
.local tempa tempb
	sram[pop, read_xfer_reg, tempa, 0, freelist_id], ctx_swap

#define_eval FREELIST_IDENT freelist_id
#if (FREELIST/**/FREELIST_IDENT/**/_TYPE == STORAGE_CLASS_SDRAM)
	immed32[tempa, FREELIST/**/FREELIST_IDENT/**/_PTR_BASE]						; get sram ptr base
	alu[tempa, read_xfer_reg, -, tempa]											; calculate pointer offset
	immed32[tempb, FREELIST/**/FREELIST_IDENT/**/_BASE]							; get sdram base
	alu[mem_ptr, tempb, +, tempa, <<FREELIST/**/FREELIST_IDENT/**/_PTR_SHIFT]	; add shifted offset to base
#else
	move[mem_ptr, read_xfer_reg]
#endif
.endlocal
#endm



// free
//		free an address (mem_ptr) by pushing it onto freelist (freelist_id)
//		using the write transfer reg write_xfer_reg
//
#macro free[mem_ptr, freelist_id]
.local tempa tempb
#define_eval FREELIST_IDENT freelist_id
#if (FREELIST/**/FREELIST_IDENT/**/_TYPE == STORAGE_CLASS_SDRAM)
	immed32[tempa, FREELIST/**/FREELIST_IDENT/**/_BASE]							; get sdram base
	alu[tempa, mem_ptr, -, tempa]												; subtract base from sdram memptr
	alu[tempa, --, B, tempa, >>FREELIST/**/FREELIST_IDENT/**/_PTR_SHIFT]		; shift offset
	immed32[tempb, FREELIST/**/FREELIST_IDENT/**/_PTR_BASE]						; get sram ptr base
	sram[push, --, tempa, tempb, freelist_id], ctx_swap							; push ptr base + shifted offset
#else
	sram[push, --, mem_ptr, 0, freelist_id], ctx_swap
#endif

.endlocal
#endm



//------------------------------HASH macros----------------------------
//
//
//		FREELIST*_BASE for each freelist associates that freelist with a block of memory
//			note: freelist 0 is an exception in that it associates with 2 blocks, in sram and sdram
//
// (2 freelists are needed per hash database, one for L2 and one for database entries)
//		

// get base address associated with a freelist
//
#macro get_db_base[db_base_addr, freelist_id]
#define_eval THIS_ID freelist_id

#if (freelist_id == 0)
	immed32[db_base_addr, FREELIST0_BASE]
#elif (freelist_id == 1)
	immed32[db_base_addr, FREELIST1_BASE]
#elif (freelist_id == 2)
	immed32[db_base_addr, FREELIST2_BASE]
#elif (freelist_id == 3)
	immed32[db_base_addr, FREELIST3_BASE]
#elif (freelist_id == 4)
	immed32[db_base_addr, FREELIST4_BASE]
#elif (freelist_id == 5)
	immed32[db_base_addr, FREELIST5_BASE]
#elif (freelist_id == 6)
	immed32[db_base_addr, FREELIST6_BASE]
#elif (freelist_id == 7)
	immed32[db_base_addr, FREELIST7_BASE]
#endif

#endm

// get base address associated with a freelist
//
#macro get_L2_base[L2_base_addr, freelist_id]

#if (freelist_id == 0)
	immed32[L2_base_addr, FREELIST0_BASE]
#elif (freelist_id == 1)
	immed32[L2_base_addr, FREELIST1_BASE]
#elif (freelist_id == 2)
	immed32[L2_base_addr, FREELIST2_BASE]
#elif (freelist_id == 3)
	immed32[L2_base_addr, FREELIST3_BASE]
#elif (freelist_id == 4)
	immed32[L2_base_addr, FREELIST4_BASE]
#elif (freelist_id == 5)
	immed32[L2_base_addr, FREELIST5_BASE]
#elif (freelist_id == 6)
	immed32[L2_base_addr, FREELIST6_BASE]
#elif (freelist_id == 7)
	immed32[L2_base_addr, FREELIST7_BASE]
#endif

#endm

#define L2_SHIFT_AMOUNT 4
#define L2_SHIFT_MASK 0xf
#define_eval INV_L2_SHIFT_AMOUNT (32 - L2_SHIFT_AMOUNT)

// hashdb_add_level2
//
//
#macro	hashdb_add_level2[hash_rem, lookup, L2_freelist, L2_hash_base, db_freelist, xfer0, sd_xfer0, sd_xfer1]

.local tempa tempb table_sel index saved_hash_rem bit_pos

	move[saved_hash_rem, hash_rem]
	immed[bit_pos, 0]

hash_add_loop#:
	alu_shf[index, lookup, +L2_SHIFT_AMOUNT, hash_rem]					; shift table sel and add masked hash_result
   
    sram[read, xfer0, L2_hash_base, index, 1], ctx_swap, optimize_mem	; read L2 table with tableptr+hash_rem
    alu[lookup, 0, b, xfer0]

	br!=0[entry_or_collision#]

// if no entry, add entry
	malloc[lookup, db_freelist,  xfer0]						; get next database index (mem_ptr, freelist, xfer)
	move[xfer0, lookup]
	sram[write, xfer0, L2_hash_base, index, 1], ctx_swap	; write the new entry database index to L2
	br[done#]

// if collision bit set, add the entry to L2
//
entry_or_collision#:
	alu[--, xfer0, AND, 1, <<31]
	br=0[entry_and_no_collision#]

// existing entry and collision
//
	alu[bit_pos, bit_pos, +, L2_SHIFT_AMOUNT]				; increment pointer into hash_rem
	alu[--, bit_pos, -, 1, <<5]								; compare to 32
	br=0[exit#]												; for 64 bit add, exit to caller							

	alu[lookup, --, B, lookup, <<2]
	alu[lookup, --, B, lookup, >>7]							; get just the tableptr
    br[hash_add_loop#], defer[1]
	alu[hash_rem, --, B, hash_rem, >>L2_SHIFT_AMOUNT]

// existing entry and no collision, get existing entry hash_rem and loop
//
entry_and_no_collision#:
	malloc[level2_index, L2_freelist, xfer0]				; get new level2 table index
	alu[level2_index, level2_index, -, L2_hash_base]		; get offset

// insert the original entry in the new L2 table
	get_db_base[db_hash_base, db_freelist]					; pointer to the db table
	alu[tempa, 1, +, lookup]								; read the second quadword to get hash_rem
	sdram[read, sd_xfer0, db_hash_base, tempa, 1], ctx_swap, defer[1]	; read database (will return 2 longwords)
	alu[bit_pos, bit_pos, +, L2_SHIFT_AMOUNT]
	.if (saved_hash_rem == sd_xfer1)
		br[exit#]											; use existing lookup_result
	.endif
	alu[--, bit_pos, B, 0]									; setup indirect right shift amount
	alu_shf[tempa, L2_SHIFT_MASK, AND, sd_xfer1, >>indirect]; shift to get next bits of hash remainder
	alu_shf[tempb, tempa, +, level2_index]
	move[xfer0, lookup]
    sram[write, xfer0, L2_hash_base, tempb, 1], ctx_swap	; write original entry to new L2 table

// write the L2 collision entry
	alu[lookup, INV_L2_SHIFT_AMOUNT, OR, level2_index, <<5]					; insert L2 index and shift amt
	alu[xfer0, lookup, OR, 1, <<31]							; insert collision
	sram[write, xfer0, L2_hash_base, index, 1], ctx_swap	; write the collision info to L2

// iterate on new entry
	alu[lookup, --, B, level2_index]
    br[hash_add_loop#], defer[1]
	alu[hash_rem, --, B, hash_rem, >>L2_SHIFT_AMOUNT]
	

done#:
	move[sd_xfer0, 0]
	move[sd_xfer1, saved_hash_rem]
	sdram[write, sd_xfer0, 1, lookup, 1], ctx_swap			; write hash_rem to database
exit#:
.endlocal
#endm


// hashdb_add48
//		Add new hash entry using hashed 48 bit value, by creating table lookups
//		lookup_result will point to database entry, so parameters can be inserted there
//		See also: hashdb_lookup48
//
//	parameters:
//		hashr_hi_xfer		hashed_result <47:32>
//		hashr_lo_xfer		hashed_result <31:0>
//		L1_base				level 1 table base constant, 32 bits
//		L2_freelist			id of level 2 freelist (1-7)
//		db_freelist			id of database freelist (1-7)
//		lookup_result		index to database
//
#macro hashdb_add48[hashr_hi_xfer, hashr_lo_xfer, L1_base, L2_freelist, db_freelist, lookup_result, sd_xfer0, sd_xfer1]

.local tempc tempd hash_rem level2_index L2_hash_base db_hash_base L1_addr
.operand_synonym xfer0 hashr_hi_xfer					; note: reusing hashr_hi_xfer
.operand_synonym xfer1 hashr_lo_xfer					; note: reusing hashr_lo_xfer

	get_db_base[db_hash_base, db_freelist]				; pointer to the db table

    ld_field_w_clr[hash_rem, 0011, hashr_lo_xfer, >>16]
    ld_field[hash_rem, 1100, hashr_hi_xfer, <<16]		; concatenate hash remainder

	immed32[tempd, L1_base]
	ld_field_w_clr[tempc, 0011, hashr_lo_xfer]			; table index = low 16 bits of hash result
	alu[L1_addr, tempd, +, tempc]
    sram[read, xfer0, L1_addr, 0, 1], ctx_swap, optimize_mem ; lookup SRAM_L1 hash

	alu[--, --, B, xfer0]
	br!=0[entry_or_collision#]

// if lookup_xfer is 0, there is no entry, so enter a new one
//
	malloc[lookup_result, db_freelist, xfer1]			; get next database index
	move[xfer1, lookup_result]
	sram[write, xfer1, tempc, tempd, 1], ctx_swap		; write the database index
	br[L1done#]

// if collision bit set, add the entry to L2
//
entry_or_collision#:
	alu[--, xfer0, AND, 1, <<31]
	br=0[entry_and_no_collision#]

// collision and existing entry 
	ld_field_w_clr[level2_index, 0111, xfer0, >>5]
	get_L2_base[L2_hash_base, L2_freelist]
	br[lookup_L2#]

// no collision and existing entry,
//		get L2 table
//		set collision bit
//		insert pointer to L2
//		shift amount = L2_SHIFT_AMOUNT (4 or 8, level 2 tables are 16 or 256 entries). 
//			insert INV_L2_SHIFT_AMOUNT, 28 or 24 (32-L2_SHIFT_AMOUNT) for faster lookup later
//			add both entries to L2
entry_and_no_collision#:
	get_L2_base[L2_hash_base, L2_freelist]
	malloc[level2_index, L2_freelist, xfer1]				; get next level2 table index
	alu[level2_index, L2_hash_base, B-A, level2_index]		; get offset

// insert original entry into level2
	alu[tempc, 1, +, xfer0]									; read the second quadword to get hash_rem
	sdram[read, sd_xfer0, db_hash_base, tempc, 1], ctx_swap	; read database
	alu_shf[tempc, L2_SHIFT_MASK, AND, sd_xfer1]			; get lo 4 or 8 bits of hash remainder
	.if (hash_rem == sd_xfer1)
		br[end#]											; use existing lookup_result
	.endif
	alu_shf[tempd, tempc, +, level2_index]
	move[xfer0, xfer0]
    sram[write, xfer0, L2_hash_base, tempd, 1], ctx_swap	; write original entry to new L2 table

// write the collision entry
	alu[lookup_result, INV_L2_SHIFT_AMOUNT, OR, level2_index, <<5]	; insert L2 index and shift amt
	alu[xfer0, lookup_result, OR, 1, <<31]					; insert collision
	sram[write, xfer0, L1_addr, 0, 1], ctx_swap				; write the collision info to L1


// iterate on new entry
lookup_L2#:
	move[lookup_result, level2_index]
	hashdb_add_level2[hash_rem, lookup_result, L2_freelist, L2_hash_base, db_freelist, xfer0, sd_xfer0, sd_xfer1]
	br[end#]

L1done#:
	move[sd_xfer0, 0]
	move[sd_xfer1, hash_rem]
	alu[tempc, lookup_result, +, 1]
	sdram[write, sd_xfer0, db_hash_base, tempc, 1], ctx_swap	; write hash_rem to database
end#:
.endlocal	
#endm


#macro hashdb_lookup_L2[hash_rem, lookup_result, L2_hash_base]
.local tempa tempb

// solve_hash_conflict
// inputs:
//     lookup_xfer  - value from hash table lookup
//			<30>	- lock bit		entry not valid (update from core in progress)
//          <31>	- collision bit 
//              =0 in this case, means no entry, bail out
//              =1 collision
//          <29>    MBZ
//          <28:5>  L2_table select - 16 bits
//          <4:0>   L2_shift - hash table index shift left amount - 5 bits.  Format is 32-shift left amount
//            
//
//     hash_rem - output of L1 hash, shifted right 16 (32 bits)
//     sram_L2_addr_hash_base
// 
// outputs:
//     if a collision found and resolved:
//			lookup_xfer - forwarding table index
//		if not resolved, no entry
//			lookup_xfer = 0
//
.local table_sel index

hash_lookup_loop#:
	ld_field_w_clr[table_sel, 0111, lookup_result, >>5]		; table_sel in 23:0

// note: table shift value in <5:0> is stored as left shift indirect value (32-n)
	alu[index, --, ~B, 0]									; generate -1
	alu_shf[index, hash_rem, AND, index, >>INV_L2_SHIFT_AMOUNT]	; mask hash result with -1 shifted by lookup_xfer<4;0>

	alu_shf[index, index, +, table_sel]						; add masked hash_result to table sel
   
    sram[read, $xfer7, L2_hash_base, index, 1], defer[1], ctx_swap, optimize_mem
	alu_shf[hash_rem, --, B, hash_rem, >>L2_SHIFT_AMOUNT]	; get new hash_rem
	move[lookup_result, $xfer7]

    br<0[hash_lookup_loop#]			  						; check for empty (=0) or conflict (<31> = 1)
.endlocal

hash_resolved#:
.endlocal
#endm


// hashdb_resolve
//	function call to resolve level 2 conflict
//	calls hash_resolve48_func in stdfunc.uc
//
#macro hashdb_resolve[hashr_hi_xfer, hashr_lo_xfer, lookup_result, const_L2_base]
    ld_field_w_clr[tempb, 0011, hashr_lo_xfer, >>16]
    alu[tempb, tempb, or, hashr_hi_xfer, <<16]				; concatenate hash remainder
	move[tempc, lookup_result]
	immed32[tempd, const_L2_base]
	balr[tempa, hashdb_resolve_func#]

	move[lookup_result, tempc]
#endm


// hashdb_lookup48
//		Resolve 48 bit hash via table lookups to get an index to database.
//	Prerequisites:
//		1. Load 48 bit hash multiplier. This is chosen so as to get high likelyhood
//			of no conflict on low hash argument bits <15:0>
//		2. Add hash entries to table using Bridge Manager spanning tree or other L1/L2 table management
//			This creates the lookups in L1 and L2 tables.
//			The L1 table is an index lookup into a 64K entry table using hash_result<15:0>
//			If collision occurs, do sequence of L2 lookups (see below)
//			L2 consists of many small tables, e.g. 16 or 256 entry
//			For example, if tables are 16 entry, each L2 lookup would take the next 4 bits
//				of hash remainder.
//			If the lookup returns 0, there is no entry in the table.
//			If the lookup has sign bit<31>, there is a collision and 
//				hash_rem (shift_mt) is added to L2 index<<shift_amt, and another L2 lookup is preformed
//			Otherwise the lookup contains an index to the database.
//		3. Perform hardware hash function to get a 48 bit hash result
//				  <47:32> in hashr_hi_xfer, <31:0> in hashr_lo_xfer
//

#macro hashdb_lookup48[hashr_hi_xfer, hashr_lo_xfer, const_L1_base, const_L2_base, lookup_result]
.local tempa tempb hash_rem reg_L2_base 

    ld_field[hash_rem, 0011, hashr_lo_xfer, >>16]
    alu[hash_rem, hash_rem, OR, hashr_hi_xfer, <<16]	; concatenate hash remainder

	immed32[tempb, const_L1_base]
	ld_field_w_clr[tempa, 0011, hashr_lo_xfer]			; table index = low 16 bits of hash result
    sram[read, hashr_lo_xfer, tempa, tempb, 1], ctx_swap, optimize_mem ; lookup SRAM_L1 hash

    move[lookup_result, hashr_lo_xfer]                   ; check results of DA lookup in hash table 1
	br>0[hash_resolved#]

	immed32[reg_L2_base, const_L2_base]
	hashdb_lookup_L2[hash_rem, lookup_result, reg_L2_base]

hash_resolved#:
.endlocal
#endm


// hashdb_add64_level2
//		add an entry, iterating through hash remainder until there is no collision
//		parameters
//		hi_lo			if 1 iterating on 16 bit hash_rem_hi, if 0, iterating on 48 bit hash_rem_hi and lo
//		hash_rem_hi		bits 47:32 of hash remainder
//		hash_rem_lo		bits 32:0 of hash remainder
//		lookup			previous lookup result, at exit pointer to  database entry
//		L2_freelist		identify of L2 freelist
//		L2_hash_base	base address for L2 tables
//		db_freelist		identify of sdram database freelist
//
#macro	hashdb_add64_level2[hi_lo, hash_rem_hi, hash_rem, lookup, L2_freelist, L2_hash_base, db_freelist, xfer0, sd_xfer0, sd_xfer1]

.local tempa tempb table_sel index saved_hash_rem bit_pos

	.if (hi_lo == 1)
		move[hash_rem, hash_rem_hi]										; working on hi bits
	.endif
	move[saved_hash_rem, hash_rem]
	immed[bit_pos, 0]

hash_add_loop#:
	alu_shf[index, lookup, +L2_SHIFT_AMOUNT, hash_rem]					; shift table sel and add masked hash_result
   
    sram[read, xfer0, L2_hash_base, index, 1], ctx_swap, optimize_mem	; read L2 table with tableptr+hash_rem
    alu[lookup, 0, b, xfer0]

	br!=0[entry_or_collision#]

// if no entry, add entry
	malloc[lookup, db_freelist,  xfer0]						; get next database index (mem_ptr, freelist, xfer)
	move[xfer0, lookup]
	sram[write, xfer0, L2_hash_base, index, 1], ctx_swap	; write the new entry database index to L2
	br[done#]

// if collision bit set, add the entry to L2
//
entry_or_collision#:
	alu[--, xfer0, AND, 1, <<31]
	br=0[entry_and_no_collision#]

// existing entry and collision
//
	alu[bit_pos, bit_pos, +, L2_SHIFT_AMOUNT]				; increment pointer into hash_rem
	alu[--, bit_pos, -, 1, <<5]								; compare to 32
	br=0[exit#]												; for 64 bit add, exit to caller							

	alu[lookup, --, B, lookup, <<2]
	alu[lookup, --, B, lookup, >>7]							; get just the tableptr
    br[hash_add_loop#], defer[1]
	alu[hash_rem, --, B, hash_rem, >>L2_SHIFT_AMOUNT]

// existing entry and no collision, get existing entry hash_rem and loop
//
entry_and_no_collision#:
	malloc[level2_index, L2_freelist, xfer0]				; get new level2 table index
	alu[level2_index, level2_index, -, L2_hash_base]		; get offset

// insert the original entry in the new L2 table
	get_db_base[db_hash_base, db_freelist]					; pointer to the db table
	alu[tempa, 1, +, lookup]								; read the second quadword to get hash_rem
	sdram[read, sd_xfer0, db_hash_base, tempa, 1], ctx_swap, defer[1]	; read database (will return 2 longwords)
	alu[bit_pos, bit_pos, +, L2_SHIFT_AMOUNT]
	.if (hi_lo == 0)
		.if (saved_hash_rem == sd_xfer1)
			.if (hash_rem_hi == sd_xfer0)					; if 48 bit hash remainder is same
				br[exit#]									; use existing lookup_result
			.endif
		.endif
	.else
		.if (saved_hash_rem == sd_xfer0)					; if hi hash remainder is same
			br[exit#]										; use existing lookup_result
		.endif
	.endif
	alu[--, bit_pos, B, 0]									; setup indirect right shift amount
	alu_shf[tempa, L2_SHIFT_MASK, AND, sd_xfer1, >>indirect]; shift to get next bits of hash remainder
	alu_shf[tempb, tempa, +, level2_index]
	move[xfer0, lookup]
    sram[write, xfer0, L2_hash_base, tempb, 1], ctx_swap	; write original entry to new L2 table

// write the L2 collision entry
	alu[lookup, INV_L2_SHIFT_AMOUNT, OR, level2_index, <<5]					; insert L2 index and shift amt
	alu[xfer0, lookup, OR, 1, <<31]							; insert collision
	sram[write, xfer0, L2_hash_base, index, 1], ctx_swap	; write the collision info to L2

// iterate on new entry
	alu[lookup, --, B, level2_index]
    br[hash_add_loop#], defer[1]
	alu[hash_rem, --, B, hash_rem, >>L2_SHIFT_AMOUNT]
	

done#:
	move[sd_xfer0, 0]
	move[sd_xfer1, saved_hash_rem]
	sdram[write, sd_xfer0, 1, lookup, 1], ctx_swap			; write hash_rem to database
exit#:
.endlocal
#endm



// hashdb_add64
//		Add new hash entry using hashed 64 bit value, by creating table lookups
//		lookup_result will point to database entry, so parameters can be inserted there
//		See also: hashdb_lookup64
//
//	parameters:
//		hashr_hi_xfer		hashed_result <47:32>
//		hashr_lo_xfer		hashed_result <31:0>
//		L1_base				level 1 table base constant, 32 bits
//		L2_freelist			id of level 2 freelist (1-7)
//		db_freelist			id of database freelist (1-7)
//		lookup_result		index to database
//
#macro hashdb_add64[hashr_hi_xfer, hashr_lo_xfer, L1_base, L2_freelist, db_freelist, lookup_result, sd_xfer0, sd_xfer1]

.local tempc tempd hash_rem hash_rem_hi level2_index L2_hash_base db_hash_base L1_addr
.operand_synonym use_xfer0 hashr_hi_xfer				; note: reusing hashr_hi_xfer
.operand_synonym use_xfer1 hashr_lo_xfer				; note: reusing hashr_lo_xfer

	get_db_base[db_hash_base, db_freelist]				; pointer to the db table

    ld_field_w_clr[hash_rem, 0011, hashr_lo_xfer, >>16]
    ld_field[hash_rem, 1100, hashr_hi_xfer, <<16]		; concatenate hash remainder
	alu[hash_rem_hi, 0, +16, hashr_hi_xfer, >>16]

	immed32[tempd, L1_base]
	ld_field_w_clr[tempc, 0011, hashr_lo_xfer]			; table index = low 16 bits of hash result
	alu[L1_addr, tempd, +, tempc]
    sram[read, use_xfer0, L1_addr, 0, 1], ctx_swap, optimize_mem ; lookup SRAM_L1 hash

	alu[--, --, B, use_xfer0]
	br!=0[entry_or_collision#]

// if lookup_xfer is 0, there is no entry, so enter a new one
//
	malloc[lookup_result, db_freelist, use_xfer1]			; get next database index
	move[use_xfer1, lookup_result]
	sram[write, use_xfer1, tempc, tempd, 1], ctx_swap		; write the database index
	br[L1done#]

// if collision bit set, add the entry to L2
//
entry_or_collision#:
	alu[--, use_xfer0, AND, 1, <<31]
	br=0[entry_and_no_collision#]

// collision and existing entry 
	ld_field_w_clr[level2_index, 0111, use_xfer0, >>5]
	get_L2_base[L2_hash_base, L2_freelist]
	br[lookup_L2#]

// no collision and existing entry,
//		get L2 table
//		set collision bit
//		insert pointer to L2
//		shift amount = L2_SHIFT_AMOUNT (4 or 8, level 2 tables are 16 or 256 entries). 
//			insert INV_L2_SHIFT_AMOUNT, 28 or 24 (32-L2_SHIFT_AMOUNT) for faster lookup later
//			add both entries to L2
entry_and_no_collision#:
	get_L2_base[L2_hash_base, L2_freelist]
	malloc[level2_index, L2_freelist, use_xfer1]				; get next level2 table index
	alu[level2_index, L2_hash_base, B-A, level2_index]			; get offset

// insert original entry into level2
	alu[tempc, 1, +, use_xfer0]									; read the second quadword to get hash_rem
	sdram[read, sd_xfer0, db_hash_base, tempc, 1], ctx_swap		; read database
	alu_shf[tempc, L2_SHIFT_MASK, AND, sd_xfer1]				; get lo 4 or 8 bits of hash remainder
	.if (hash_rem == sd_xfer1)
		alu[tempc, 0, +16, sd_xfer0]
		.if (hash_rem_hi == tempc)								; if hash remainder is same
			br[end#]											; use existing lookup_result
		.endif
	.endif
	alu_shf[tempd, tempc, +, level2_index]
	move[use_xfer0, use_xfer0]
    sram[write, use_xfer0, L2_hash_base, tempd, 1], ctx_swap	; write original entry to new L2 table

// write the collision entry
	alu[lookup_result, INV_L2_SHIFT_AMOUNT, OR, level2_index, <<5]	; insert L2 index and shift amt
	alu[use_xfer0, lookup_result, OR, 1, <<31]					; insert collision
	sram[write, use_xfer0, L1_addr, 0, 1], ctx_swap				; write the collision info to L1


// iterate on new entry
lookup_L2#:
	move[lookup_result, level2_index]
	hashdb_add64_level2[0, hash_rem_hi, hash_rem, lookup_result, L2_freelist, L2_hash_base, db_freelist, use_xfer0, sd_xfer0, sd_xfer1]
	alu[--, --, B, lookup_result]
	br>=0[end#]
	hashdb_add64_level2[1, hash_rem_hi, hash_rem, lookup_result, L2_freelist, L2_hash_base, db_freelist, use_xfer0, sd_xfer0, sd_xfer1]
	br[end#]

L1done#:
	move[sd_xfer0, 0]
	move[sd_xfer1, hash_rem]
	alu[tempc, lookup_result, +, 1]
	sdram[write, sd_xfer0, db_hash_base, tempc, 1], ctx_swap	; write hash_rem to database
end#:
.endlocal	
#endm


// lookup 64 bit hash result
//
#macro hashdb_lookup64[hashr_hi_xfer, hashr_lo_xfer, const_L1_base, const_L2_base, lookup_result]
	immed32[tempb, const_L1_base]
	ld_field_w_clr[tempa, 0011, hashr_lo_xfer]					; table index = low 16 bits of hash result
    sram[read, $xfer7, tempa, tempb, 1], ctx_swap, optimize_mem ; lookup SRAM_L1 hash

    move[lookup_result, $xfer7]                   		; check results of DA lookup in hash table 1
	br>0[done#]

	hashdb_resolve[hashr_hi_xfer, hashr_lo_xfer, lookup_result, const_L2_base]
	.if (lookup_result < 0)
		hashdb_resolve[0, hashr_hi_xfer, lookup_result, const_L2_base]
	.endif
done#:
#endm
