//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// rec_ipverify.uc
// IPV4 verify the ip header prior to routing decision
//
// Version = 1.0.NoBldNum
//
// system: SA1200
// subsystem: IP route microcode
// usage: example
// author: dfh 11/8/97
//	
// revisions:
//		dfh		1/12/99		bi-endian. lower level macros perform swaps if #define LITTLE_ENDIAN
//
// ---------------------------SA1200 microcode--------------------------	

 

// Prerequisite: 
//	rec_nextpac.uc   has executed
//	first 4 longwords of the packet has been read in from reveive FIFO
//

//	register usage at this point
//
//	$xfer0 					MAC DA<31:0>
//	$xfer1					MAC  DA<48:32> and SA<15:0> 
//	$xfer2					MAC  SA<31:0>  
//	$xfer3					MAC length and ip version, IHL, TOS
//	protocol_len			datalink payload length
//	packet_buf_addr			address of buffer in sdram where packet will be stored
//	buf_handle				address offset of the current descriptor
//	rec_state				fastport flag, packet sequence number, byte enables, eop, sop

// 

ip_verify#:
.local len_version

	move[$$xfer2, $xfer2]						; save to sdram write xfer
	move[$$xfer3, $xfer3]						; save to sdram write xfer  

// move second 4 quadwords of receive fifo element to sdram 
//
	sdram_r_fifo_rd[packet_buf_addr, rfifo_entry, 4, 4, ASYNC]		; (see stdmac.uc)


read_rfifo2#:
// get the next 6 longwords of the packet from the receive FIFO
// note: we are overwriting $xfer2(sa2) and $xfer3(len_version)
//
    r_fifo_rd [$xfer2, rfifo_entry, 2, 3], ctx_swap, defer[1]
	move[len_version, $xfer3]										; (see stdmac.uc)

// now ipheader is in these regs 
// packet must be big-endian
//
// len_version	bytes 0-1 have data link payload len 
//				bytes 2-3 ip ver 4 bits, IHL 4 bits, QOS 8 bits
// $xfer2		ip total len, id
// $xfer3		ip frag, ttl, protocol
// $xfer4		ip cksum 16, sa 16
// $xfer5		ip sa 16, da 16
// $xfer6		da 16, options or payload
// $xfer7		payload


ip_verify_len#:
// check that total ip packet length in $xfer2 bytes 0-1 is at least 20 bytes
//
	br_less_field[$xfer2, 0, 1, 0x14, ip_bad_total_length#] 		; (see endian.uc)


ip_verify_ttl#:
// Check that TTL in byte 2 is greater than 1.  
// A packet with TTL equal to 1 is legal but
// we can not forward it.
// if bad ttl, ask host ICMP to handle
//
    br_less_field[$xfer3, 2, 2, 0x1, ip_bad_ttl#]


ip_verify_checksum#:
// verify ip header checksum
// regs:
// len_version	bytes <3:2> have data link payload len 
//				bytes <1:0> ip ver 4 bits, IHL 4 bits, TOS 8 bits
// $xfer2        ip total len, id
// $xfer3        ip frag, ttl, protocol
// $xfer4        ip cksum 16, sa 16
// $xfer5        ip sa 16, da 16
// $xfer6        da 16, options
//
// (macro defined in refdes_macros.uc)
//
	IP_VerifyChecksum_Hw[len_version, $xfer2, $xfer3, $xfer4, $xfer5, $xfer6, ip_bad_checksum#]
//
// the ip header is good
	ctx_arb[voluntary]

ip_modify_ttl#:
// modify the ip header and put modified words in sdram write xfer
// registers for packet modification
//
// subtract 1 from ttl and move it to sdram write xfer reg
//
	decr_field[$$xfer5, $xfer3, 2, 2]				; (see endian.uc)


ip_modify_checksum#:
// add 1 to checksum from $xfer4 bytes 0-1 to compensate for subtracting 1 from ttl
//
// have to switch bytes first then switch back
    ld_field_w_clr[temp, 0011, $xfer4]
	ld_field[temp, 0100, $xfer4, >>8]
	ld_field[temp, 1000, $xfer4, <<8]
	incr_field[temp1, temp, 0, 1]				; modify checksum and insert checksum/sa to write xfer reg
	ld_field[temp, 0100, temp1, >>8]
	ld_field[temp, 1000, temp1, <<8]
    alu[$$xfer6, --, B, temp]

// packet is going somewhere. move rest of header to sdram write xfer
// freeing read $xfer regs for use in forwarding lookup
//
	alu[sa4732, 0, +16, $xfer1]		; hold on to MAC sa hi 16 
	move[$$xfer4, $xfer2]

	br[route_lookup#], defer[1]
	move[$$xfer7, $xfer5]


ip_bad_total_length#:
    immed[exception, IP_BAD_TOTAL_LENGTH]
    br[packet_discard#]


ip_bad_ttl#:
    immed[exception, IP_BAD_TTL]
    br[packet_discard#]


ip_bad_checksum#:
    immed[exception, IP_BAD_CHECKSUM]
    br[packet_discard#]

.endlocal