//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// rec_lmatch.uc
// IPV4 longest specific address match
//
// Version = 1.0.NoBldNum
//
// system: SA1200
// subsystem: IP router microcode
// usage: example
// author: dfh 9/28/97
//
// revisions:
//  ejj     March, 15, 2000 Changed route lookup to simply use tries
//	dfh		July 29, 1998	FAST PORT ucode supports 1 fast port, 16 100M ports
//	dfh		Aug 11, 1998	move sequence check to enqueue time
//  mff     Sept 10, 1999   ifdef A1_CHIP - workaround optimize_mem hw bug


// ---------------------------SA1200 microcode--------------------------	
//
// Prerequisite Register Usage:
//	symbol
//	 trie_base		  - base address of trie table of 4bit tree lookup
//						sram table is 128K entries, entries are longwords (512KB)
//   route_table_base -	sdram route entries 128K x 8 longwords (1MB)
//						32 byte stride = offset must be shifted left 5
//	 $xfer5/$xfer6    - Must contain the 32 bit IP destination address
//   rec_state        - Must contain valid ingress port of the packet (bits 24 - 29)
//   first_tries      - an array of first (head) trie pointers (one per port)
//
//
// description:
//	Get the route entry whose ip address has the most specific match with ip_da.
//	Also known as longest-prefix match. 
// result:
//   $$route_entry0	- has 8 longwords of (non-exception) route entry info,
//					  including the forwarding interface
// Algorithm:
//   See the comment in IP_Utils.cc on ip_route_lookup

#include "..\src\IP_Utils.uc"

;.xfer_order $$xfer0 $$xfer1 $$xfer2 $$xfer3 $$xfer4 $$xfer5 $$xfer6 $$xfer7
.import_var trie_base route_table_base first_tries
.local ip_da route_ent_offset head_ptr temp_base4 inport first_tries_reg trie_base_reg

.operand_synonym $$route_entry0 $$xfer0
.operand_synonym $$route_entry1 $$xfer1
.operand_synonym $$route_entry2 $$xfer2
.operand_synonym $$route_entry3 $$xfer3
.operand_synonym $$route_entry6 $$xfer6

// Entry point
route_lookup#:

// Copy the shared variables from the core into registers
immed_w0[first_tries_reg, first_tries]
immed_w1[first_tries_reg, first_tries>>16]
immed_w0[trie_base_reg, trie_base]
immed_w1[trie_base_reg, trie_base>>16]

// extract the ip destination address spanning $xfer5 byte 2 to $xfer6 byte 1
dbl_extract_field[ip_da, $xfer5, 2, $xfer6, 1]

// Locate the first trie pointer for this ingress port
alu[inport, 0xf, AND, rec_state, >>24]
sram[read, $xfer0, first_tries_reg, inport, 1], ctx_swap

// If the trie pointer is zero, bail out
alu[head_ptr, $xfer0, -, 0]
br=0[no_route#]

// Perform the actual route lookup
ip_route_lookup[route_ent_offset, ip_da, $xfer0, trie_base_reg, head_ptr]
alu[--, route_ent_offset, -, 0]
br!=0[get_route#]

no_route#:
ip_no_route#:
    immed[exception, IP_NO_ROUTE]
    br[packet_discard#] ; see rec_nextpac.uc
	

get_route#:
    immed_w0[temp_base4, route_table_base]				; load shared address value
    immed_w1[temp_base4, route_table_base>>16]
//
// the transfer from rfifo was done at the top of ipverify, in order to free the rfifo element earlier
// we should have the signal back well before now
//
#ifndef STANDALONE
	ctx_arb[sdram]															; is transfer from rfifo done?
#endif

//	Write thread done as soon as the rfifo element is transferred to sdram
//  By the time it turns around and gets to the Receive Scheduler, this thread will be done
//
#ifdef FAST_PORT_ENABLED
	.if (bit(rec_state, 31) == 1)		; if fastport
		fast_wr[2, THREAD_DONE]			; notify receive scheduler with EOP encode
	.else
#endif
#ifndef RECEIVE16
		.if (bit(rec_state, 9) == 1)	; if EOP
			fast_wr[3, THREAD_DONE]		; notify receive scheduler with EOP encode
		.else
			fast_wr[1, THREAD_DONE]		; notify receive scheduler witn non-EOP encode
		.endif
#endif
#ifdef FAST_PORT_ENABLED
	.endif
#endif

#ifdef LAYER4
// layer 4+
	delay[8]
	sram[read, $xfer2, temp_base3, 2, 1], ctx_swap
	delay[8]
	move[$xfer0, 0]
	move[$xfer1, 1]
	move[$xfer2, 0]
	move[$xfer3, 1]
	hash2_48[$xfer0], ctx_swap						; 2 hash lookups
	delay[8]
	sram[read, $xfer2, temp_base3, 2, 1], ctx_swap
	delay[8]
	sram[read, $xfer2, temp_base3, 2, 1], ctx_swap
#endif
//---------------------------------------------------------------------------------------

	sdram[read, $$route_entry0, temp_base4, route_ent_offset, 4], optimize_mem, ctx_swap	;get route entry

#ifdef STANDALONE
	alu[interface, 0, B, $$route_entry0]				; output physical port interface id
	nop													; let interface be stored
	nop
	nop
found_route#:
;for testing, interface can be verified, ip_da can be deposited now to a new value
	br[route_lookup#]
#endif

// There is a choice of queues the packet can be placed on:
//
//		qtype						count/description			handled by	
//		1=destination queues		8 per port x 32 ports		transmit microcode
//									qselect per port, 
//										0 = hi priority
//										7 = lo priority
//								
//		0=slow path queue			1							core receive application
//		others unused				filter, trap, etc
//

// There are 8 free lists for descriptors. The dtype field picks which one.
//	dtype
//	0 = normal packet data:		sram and sdram allocated, 8 bytes sram, 2k byte sdram chunks
//	1-7	unused

// this is fast path
// dtype		= 0 normal packet
// qtype		= 1 destination queue
// qselect	= 0 hi priority

got_output_port#:
	alu[rec_state, rec_state, OR, $$route_entry0]			; save output port for enqueue

.endlocal

write_network_layer#:	
//
// note ipverify previously put other 6 words of packet  on sdram write xfer regs
// insert MAC DA from route entry  
	move[$$xfer0, $$route_entry1]							; da 48:16
	alu[$$xfer1, sa4732, +, $$route_entry2, <<16]			; da 15: 0 merge with sa 47:32
	sdram[write, $$xfer0, packet_buf_addr, 0, 4]			; write first 32 bytes of modified mpacket

; rec_enqueue.uc follows
