//////////////////////////////////////////////////////////////////////////////
//           INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//      Copyright (c) 1999-2000 Intel Corporation.  All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// $Source: /mnt/bigpine/src/FE/FEAPI/IXP1200/MicroCode/src/Attic/Sv_Dispatch.uc,v $
// $Revision: 1 $
// $Date: 4/18/00 5:06p $
//
// Description:
//    This file contains code that must be included within each service thread.
//    It has the main entry point for a service thread (Sv_WaitForCall).
//
// Entry Points:
//   

// WARNING
//  The thread id has been hardcoded to 22 to avoid register bloat, this should be
//  fixed
.local thread_mb_base from_core_mb func reply format

//////////////////////////////////////////////////////////////////////////////
// Routine: Sv_WaitForCall()
//
// Parameters:
//    N/A
//
// Description:
//    Waits for a signal from the core and then dispatches a call to the appropriate
//    routine in this service thread.  This routine jumps to the appropriate service
//    routine based on the Sv_Dispatch label.
//
// Returns:
//    N/A
//
// Register usage:
//    $xfer0 - $xfer4 depending upon the number of arguments passed and returned from
//                    the service routines
// Comments:
//
Sv_WaitForCall#:
    ctx_arb[inter_thread]                                       ; wait for signal from core
                                                                ; when we wake, there is 
																; a message from core
    immed[thread_mb_base, CORE_TO_UTHREAD_MAILBOXES]			; shared constant
    //alu_shf[from_core_mb, thread_mb_base, +, my_thread_id, <<2]	; mailbox addr
    alu_shf[from_core_mb, thread_mb_base, +, 22, <<2]	; mailbox addr

    ; issue. do we read 1 then depending on format up to 3 more, or 4 up front
    sram[read, $xfer0, from_core_mb, 0, 4], ctx_swap

    ; prepare the ack
    ; swap source and dest, set ack bit
    ; +------+-----+--------+----+-------+--------+--------+--------+
    ; | pend | ack | format |fail|  xid  |  src   |  dest  |  func  |
    ; |  31  | 30  | 29:28  | 27 | 26:24 |  23:16 |  15:8  |  7:0   |
    ; +------+-----+--------+----+-------+--------+--------+--------+

    ; keep sync, func and xid
    alu_shf[reply, $xfer0, OR, 1, <<30]		; set ack
    ld_field_w_clr[func, 0001, $xfer0]
    ld_field[reply, 0100, $xfer0, <<8]
    ld_field[reply, 0010, $xfer0, >>8]
    alu_shf[reply, reply, AND~, 3, <<28]	; format 0 = preamble only
    immed[format,0]							; reinitialize reply format to 0

    ; test if asynchronous
    alu_shf[--, 0, B, reply, >>31]			; bit 31 on means synchronous
    ; if synchronous, call function first, ack later
    br>0[Sv_CallFunction#]					; go call the function
    ; asynchronous, ack now and do function later
    br[Sv_Ack#]


//////////////////////////////////////////////////////////////////////////////
// Routine: Sv_CallFunction()
//
// Parameters:
//    N/A
//
// Description:
//    Invokes the currently requested service routine
//
// Returns:
//    N/A
//
// Register usage:
//
// Comments:
//
Sv_CallFunction#:
    jump[func, Sv_Dispatch#]


//////////////////////////////////////////////////////////////////////////////
// Routine: Sv_Return()
//
// Parameters:
//    N/A
//
// Description:
//    Sends control back to Sv_WaitForSignal, but first, it checks to see if the
//    current service request is synchronous.  If it is, it first ack's the core before
//    returning to Sv_WaitForSignal.  This should be unconditionally branched to at the
//    end of all service routines.
//
// Returns:
//    N/A
//
// Register usage:
//
// Comments:
//
Sv_Return#:
    alu_shf[--, 0, B, reply, >>31]        ; if (reply is synchronous) {
    br>0[Sv_Ack#]                         ;     Sv_Ack();
	br[Sv_WaitForCall#]                   ; } else Sv_WaitForCall();


//////////////////////////////////////////////////////////////////////////////
// Routine: Sv_Ack()
//
// Parameters:
//    N/A
//
// Description:
//    Sends a message to the core acknowledging the last service request.  This is
//    done both for asynchronous and synchronous request, only at different times.
//
// Returns:
//    N/A
//
// Register usage:
//
// Comments:
//
Sv_Ack#:
    immed[thread_mb_base, UTHREAD_TO_CORE_MAILBOXES]			; shared constant
    //alu_shf[to_core_mb, thread_mb_base, +, my_thread_id, <<2]	; mailbox addr
    alu_shf[to_core_mb, thread_mb_base, +, 22, <<2]	; mailbox addr

    ; all mem services functions have preamble only reply
    alu_shf[reply, reply, OR, format, <<28]						; insert format
    alu[$xfer0, --, B, reply]
    alu_shf[indirect_wr_len, format, OR, 0x10]		; setup word count
    alu_shf[indirect_wr_len, --, B, indirect_wr_len, <<16]
    sram[write, $xfer0, to_core_mb, 0, 1], indirect_ref, ctx_swap

    ; write ireg
    fast_wr[1, ireg]

    alu_shf[--, 0, B, reply, >>31]			; bit 31 on means synchronous
    ; if synchronous, function done
    br>0[Sv_WaitForCall#]
    ; asynchronous, call function
    br[Sv_CallFunction#]					; go call the function


;----------------------------------------------------------------------
; Sv_InvalidFunc
;	function id is not recognized
;
Sv_InvalidFunc#:
;TBD reply with fail code
	nop
	ctx_arb[kill]



//////////////////////////////////////////////////////////////////////////////
// Routine: Sv_Dispatch()
//
// Parameters:
//    N/A
//
// Description:
//   
//
// Returns:
//    N/A
//
// Register usage:
//
// Comments:
//
Sv_Dispatch#:
.func_table LockSv#
.export_func twoway 2 LockSv_ReadLock# 1 1
.export_func twoway 2 LockSv_WriteUnlock# 2 0

LockSv#:
LockSv_ReadLock#:
    br[LockSv_ReadLockImp#]                ; 0
LockSv_WriteUnlock#:
    br[LockSv_WriteUnlockImp#]            ; 1


#include "Sv_Lock.uc"

.endlocal
