//////////////////////////////////////////////////////////////////////////////
//           INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//      Copyright (c) 1999-2000 Intel Corporation.  All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
// $Source: /mnt/bigpine/src/FE/FEAPI/IXP1200/MicroCode/src/Attic/Trie.uc,v $
// $Revision: 1 $
// $Date: 4/18/00 5:06p $
//
// Description:
//   This file contains routines (macros) used to manipulate and deal
//   with the trie data structures used in the IP routing routines
// Entry Points:
//   trie_next_block

//////////////////////////////////////////////////////////////////////////////
// Routine: trie_next_block
//
// Parameters:
//    ipaddr       - (IN) The IP address being used in the lookup
//    offset       - (IN) The position in the IP address currently being 
//                   examined
//                   (e.g., 28 for the first lookup, 24 next, etc.)
//    route_ptr    - (OUT) Contains the route ptr found
//    trie_ptr     - (IN/OUT) A pointer to the current trie on entry, and
//                   the next trie on exit
//    xfer_reg     - (TEMP) The SRAM transfer register to read the next trie ptr 
//                   into
//    const_base   - (IN) The base address (offset into SRAM) for all of the trie
//                   pointers
//
// Description:
//    Given a current trie, this routine looks at the appropriate entry
//    in the trie to see if there is either no more tries (in which case
//    there may be a valid route pointer), or if there is another trie,
//    updates a pointer to the next trie.
//
// Returns:
//    The next route and trie pointers
//
// Register usage:
//    Uses only the arguments passed in
//
// Comments:
#macro trie_next_block[ipaddr, offset, route_ptr, trie_ptr, xfer_reg, const_base]
    ; trie_ptr += (ipaddr >> offset) & 0xf
    alu[--, offset, -, 0]
    alu_shf[trie_ptr, trie_ptr, +4, ipaddr, >>indirect]

    ; xfer_reg = *(const_base + trie_ptr)
    sram[read, xfer_reg, const_base, trie_ptr, 1], 
                        ctx_swap, optimize_mem
    ; route_ptr = xfer_reg >> 16
    alu_shf[route_ptr, --, B, xfer_reg, >>16]
    ; trie_ptr = xfer_reg & 0xffff;
    ld_field_w_clr[trie_ptr, 0011, xfer_reg]	
#endm
