//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// rec_enqueue.uc
// receive packet enqueue following bridge or route decision
//
// Version = 1.0.NoBldNum
//
// system: SA1200
// subsystem: receive microcode
// usage: enqueue the packet
// author: dfh 2/20/98
// revisions:
//	dfh		July 28, 1998	FAST PORT ucode supports 1 fast port, 16 100M ports
//	dfh		Aug 5, 1998		implement Non-SOP
//	dfh		Aug 11, 1988	implement enqueue seq check(move from rec_lmatch) and fast_wr[INCR_SOP1]
//  mff     Sept 10, 1999   ifdef A1_CHIP - workaround optimize_mem hw bug
//
// ---------------------------SA1200 microcode--------------------------


//-------------------------------macros--------------------------------------------


// Rx_WriteDescriptor
//	write 2 descriptor words; zero in first word, rec_state and mpacket count in second word
//
//  inputs:
//		buf_handle				relative address of sram buffer and mpacket count
//		rec_state				receive state information
//		descriptor_base			base address for sram buffer descriptors
//	outputs:
//		relative_buf_addr		relative address of new descriptor
//
#macro Rx_WriteDescriptor[relative_buf_addr, buf_handle, rec_state, descriptor_base]
.local data_field holder
	immed[$xfer6, 0]
	alu_shf[relative_buf_addr, --, B, buf_handle, >>16]						; extract relative buffer address
	ld_field_w_clr[holder, 0010, rec_state, <<8]	; move byte cnt, eop, sop from 7:0 to 15:8
	alu[data_field, 1, AND, rec_state, >>29]		; peel off packet_freelist
	alu[holder, holder, or, data_field, <<16]		; insert packet_freelist at position 16
	alu[data_field, 0xf, AND, rec_state, >>20]		; peel off SOP seq #
	alu[holder, holder, or, data_field, <<20]		; insert SOP seq # in at 23:20
	alu[data_field, 1, AND, rec_state, >>30]		; peal off fast_port source
	alu[holder, holder, or, data_field, <<31]		; insert fast_port source at 31
	ld_field[buf_handle, 1110, holder]				; insert rec_state 31:8
	alu_shf[$xfer7, --, B, buf_handle]										; move to transfer reg
	sram [write, $xfer6, descriptor_base, relative_buf_addr, 2], ctx_swap	; write the packet_link
.endlocal
#endm

// write fast port descriptor
//
#macro Rx_F_WriteDescriptor[index, buf_handle, rec_state, descriptor_base]
.local data_field holder
	//format the data for the transmit side (- transmit should be made compatable with rx format)
	alu_shf[$xfer6, --, B, buf_handle, >>16]		; extract relative buffer address
	ld_field_w_clr[holder, 0010, rec_state, <<8]	; move byte cnt, eop, sop from 7:0 to 15:8
	alu[data_field, 1, AND, rec_state, >>29]		; peel off packet_freelist
	alu[holder, holder, or, data_field, <<16]		; insert packet_freelist at position 16
	alu[data_field, 0xf, AND, rec_state, >>20]		; peel off SOP seq #
	alu[holder, holder, or, data_field, <<20]		; insert SOP seq # in at 23:20
	alu[data_field, 1, AND, rec_state, >>30]		; peal off fast_port source
	alu[holder, holder, or, data_field, <<31]		; insert fast_port source at 31
	ld_field[buf_handle, 1110, holder]				; insert rec_state 31:8
	alu_shf[$xfer7, --, B, buf_handle]				; move to transfer reg
	sram [write, $xfer6, descriptor_base, index, 2], ctx_swap	; write the packet_link - need ctx_swap
.endlocal
#endm



// update number of elements stored for fast output port
// 
#macro FastPort_UpdateCount[gig_count_addr]
	immed[tempb, gig_count_addr]				; elements queued to fast port
	scratch [incr, --, tempb, 0, 1]				; read the current count
#endm



// for testing under ZERO_PROTOCOL
// insert the receive control and status<31:0> to packet address + 4 (bytes 32-39 of packet)
#macro tag_packet
.local tempa tempb

	alu_shf[tempa, 1, +, rfifo_entry, >>2]				; rfifo_entry is ele<<3)	; ele no * 2 + 1
	immed[tempb, 0x80]
	r_fifo_rd[$xfer0, tempa, tempb, 1], ctx_swap

	immed32[tempa, SDRAM_PKT_BUFFER_BASE]
	alu_shf[tempa, tempa, +, buf_handle, >>8]			; 2k byte chunks would be <<11, *8 makes it <<8
	alu[$$xfer1, --, B, $xfer7]							; receive control
#if (ZERO_PROTOCOL == 1)
	immed32[tempb, 0x32333435]							; known key
	alu[$$xfer0, --, B, tempb]
#elif (ZERO_PROTOCOL == 2)
	alu[$$xfer0, --, B, $xfer0]							; status <31:0>
#endif
	sdram[write, $$xfer0, tempa, 4, 1], ctx_swap
.endlocal
#endm

//----------------------------end macros-------------------------------------------




.local queue_descriptor_addr descriptor_base queue_packet_count next_ptr head_ptr tail_ptr



// ---------------------------------------------------------------------
//	register usage at this point
//
//	buf_handle				address offset of the SOP sram descriptor, mpacket count
//	rec_state				contains byte count, eop, sop, fast port info, dest port
//	packet_buf_addr			address of buffer in sdram





// we will get to done# in about 175 cycles

 
// now update the packet queue / packet link descriptors
//
// format the new packet link descriptor
//
// packet_link format:
//										$packet_link0
//   		 +---------------------------------------------------------------------------+
//			 |						  NPL (next packet link)						     |
//			 |							     31:0									     |
//			 +---------------------------------------------------------------------------+
//
//										$packet_link1
// from		 +-----+-------+------+------+--------+-------------+---+-----+-------+-------+
// rec_state |FPORT|qselect|inport|F seq |freelist|qw/byte count|eop|qtype|discard|outport|
//			 |     |       |      |      |        |             |   |     |       |  ele# |
//			 | 31  | 30:28 |27:24 |23:20 |  19:16 |     15:10   | 9 |  8  |   7   |  6:0  |
//			 +-----+-------+------+------+--------+-------------+---+-----+-------+-------+

// 
//	FPORT			1 if source port is fast port
//	qselect			selected q
//	inport			slow input port number
//	F seq		    fast port packet sequence number if SOP, mpacket sequence number if not SOP		
//	freelist		buffer freelist
//	qw byte count	3 bits quadword count 3 bits byte count of last element
//	qtype			destination or core queue (note: up until qselect, sop is held here)
//  discard			if =1, packet is not to be enqueued
//	outport/ele#	up until enqueue, output port.	number of elements in packet

// There is a choice of queues the packet can be placed on:
//
//		qtype						count/description			handled by	
//		1=destination queues		8 per port x 32 ports		transmit microcode
//									qselect per port, 
//										0 = hi priority
//										7 = lo priority
//								
//		0=slow path queue			1							core receive application
//		others unused				filter, trap, etc
//
									
// There are 8 free lists for descriptors. The dtype field picks which one.
//	dtype
//	0 = normal packet data:		sram and sdram allocated, 8 bytes sram, 2k byte sdram chunks
//	1-7	unused

// this is fast path
// dtype		= 0 normal packet
// qtype		= 1 destination queue
// qselect	= 0 hi priority

header_stored#:


	.if (bit(rec_state, 1) == 0)							; if not EOP don't enqueue yet
sop_save#:
		.if (bit(rec_state, 30) ==1)						; if fast port
			alu[tempb, 0xf, AND, rec_state, >>20]			; get SOP seq number
			state_save[$xfer5, output_port, RX_FPORT1_OUTPORT, tempb]	; save output_port for EOP thread to pick up
			br[next_packet#]
		.endif

slow_save1#:
		Rx_SaveSlowState[buf_handle, rec_state, output_port]
		br[next_packet#]
	.endif



rec_enqueue#:


// queue_descriptor array has space for up to 8 queues per port
// 2 words per queue descriptor = 16 words per port
// FAST PORT ref design has 17 ports
//
rec_queue_addr#: 
	immed[queue_descriptor_addr, SRAM_QUEUE_DESCRIPTOR_BASE]

// test for multicast - not supported
//	alu_shf[tempb, 0x7, AND, rec_state, >>16]				; get freelist choice
//	alu[--, 0x3, -, tempb]
//	br=0[rec_enqueue_multicast#]						; dtype == multicast


rec_enqueue_unicast#:

.operand_synonym $new_tail $xfer4
.operand_synonym $queue_descriptor0	$xfer2
.operand_synonym $queue_descriptor1	$xfer3



// if fast input port, enqueue in order
//
	.if (bit(rec_state, 30) == 1)
fast_seq_check#:												; FAST input PORT
		alu_shf[tempa, 0xf, AND, rec_state, >>20]				; extract seq number
		#ifdef FAST_PORT1
			FastPort_SeqCheck[ENQUEUE_SEQ1, tempa]					; wait for my seq no. to come up
			fast_wr[3, INCR_ENQ_NUM1]								; increment fast port 1 seq number
		#else
		#ifdef FAST_PORT2
			FastPort_SeqCheck[ENQUEUE_SEQ2, tempa]					; wait for my seq no. to come up
			fast_wr[3, INCR_ENQ_NUM2]								; increment fast port 1 seq number
		#endif ; FAST_PORT2
		#endif ; FAST_PORT1
	.endif

// test fast or slow output port
	alu[--, output_port, -, FAST_PORT1]
	br!=0[not_fast_output_port#], defer[2]

	alu[tempb, --, b, 0]											; queue fixed to queue 0
	alu[queue_descriptor_addr, queue_descriptor_addr, +4, tempb]				; add qselect
	alu[queue_descriptor_addr, queue_descriptor_addr, +, 1, <<8]		; @single fast output port

fast_enqueue#:
	immed[descriptor_base, XMIT_FPORT_DESCRIPTOR_BASE]

// for fast port, $queue_descriptor0 will contain index to packet link array
//	the head is stored in transmit fill internal registers

// note: fast port queue descriptors are initialized to point to initial tails XMIT_FPORT_TAILS
// transmit fill will read these to get started

		sram [read_lock, $queue_descriptor0, queue_descriptor_addr, 0, 1],	
					ctx_swap								; read current packet link index from queue

		alu[tempa, 2, +, $queue_descriptor0]				; increment to next packet link index
		alu[$new_tail, 0, +8, tempa]						; truncate so wrap is = transmit wrap
		sram[write_unlock, $new_tail, queue_descriptor_addr, 0, 1], priority		; increment index

		Rx_F_WriteDescriptor[$queue_descriptor0, buf_handle, rec_state, descriptor_base] ; write packet link

fport_update_count#:
	#ifdef FAST_PORT1
		FastPort_UpdateCount[XMIT_FPORT1_ELE_COUNT]						; update queued element count
	#else
	#ifdef FAST_PORT2
		FastPort_UpdateCount[XMIT_FPORT2_ELE_COUNT]						; update queued element count
	#endif ;FAST_PORT2
	#endif ;FAST_PORT1


#ifdef PROFILE
	immed[tempa, TOTAL_RECEIVES]							; total enqueues
	scratch[incr, --, tempa, 0, 1]
#endif
	br [next_packet#]
//end fast output port enqueue


//*******************************************************************************
//					slow output port enqueue
//*******************************************************************************
not_fast_output_port#:

	alu[queue_descriptor_addr, queue_descriptor_addr, +, output_port, <<4]		; add output port

slow_enqueue#:
// order of sram accesses for queue update for slow output port
//	1. read the queue_descriptor and lock the linked list(old head/tail pointer = queue descriptor word 0)
//	2. get the current tail
//	3. write new queue tail
//	4. write the new queue_descriptor and unlock the linked list


write_buf_descriptor#:
	immed[descriptor_base, SRAM_BUFF_DESCRIPTOR_BASE]
	Rx_WriteDescriptor[relative_buf_addr, buf_handle, rec_state, descriptor_base]


// the queue descriptor format for slow ports is
//		word 0		<31:16>		head packet link pointer
//		word 0		<15:0>		tail packet link pointer
//		word 1					packet count
//


	sram [read_lock, $queue_descriptor0, queue_descriptor_addr, 0, 2], 
				optimize_mem, ctx_swap								; read the queue descriptor 2 words
	alu_shf[queue_packet_count, 1, +16, $queue_descriptor1]			; update queue element count
	.if (queue_packet_count == 1)									; if queue was empty before
		scratch_bset_ind[$xfer5, output_port, XMIT_PWP_VECTOR, NOSYNC]	; set packet queued bit for output port
		alu_shf [next_ptr, 0, b, relative_buf_addr, <<16]	
		alu [$queue_descriptor0, next_ptr, or, relative_buf_addr]	; set head pointer and tail pointer to new buffer descr
	.else															; if queue had one or more packet
		ld_field_w_clr[tail_ptr, 0011, $queue_descriptor0]
		ld_field_w_clr[head_ptr, 1100, $queue_descriptor0]
		alu [$queue_descriptor0, head_ptr, or, relative_buf_addr]	; OR in the new tail pointer for queue descr
		alu [$new_tail, --, B, relative_buf_addr]					; relative_buf_addr is the new tail
		sram [write, $new_tail, descriptor_base, tail_ptr, 1], ctx_swap	; rewrite the current (now old) tail
	.endif
	ld_field_w_clr[$queue_descriptor1, 0111, queue_packet_count]
	sram [write_unlock, $queue_descriptor0, queue_descriptor_addr, 0, 2], priority, ctx_swap ; write the updated queue descriptor and release the lock - ctx_swap required

#ifdef REC_STATE_SAVE
	Rx_EOPUnBlockSlowPort[rec_state]					; unblock the port enabling next SOP for this port
#endif

#ifdef PROFILE
	immed[tempa, TOTAL_RECEIVES]							; total enqueues
	scratch[incr, --, tempa, 0, 1]
#endif
	br [next_packet#]
//
//*******************************************************************************
//					end slow output port enqueue
//*******************************************************************************







// not sop case from rec_nextpac, fast port only
//
fast_not_sop#:
	alu[--, $xfer7, AND, 1, <<1]										; test EOP
	br=0[fast_wait_for_pop#], defer[1]									; if not EOP, state will be saved later
	alu[tempb, 0x1e, AND, $xfer7, >>13]									; get mseq num
	Rx_ValidateNextFastState[tempb]										; validate the next slot

fast_wait_for_pop#:

#ifdef RECEIVE16
	#if (VERSION_MIN == 0)									; will not work on A3 chip
		#error 3 "16 threads writing receive request register is not supported on chip version 0"
	#endif
	sem_flip[@req_inflight]								; clear to allow other context to send rec req
#endif

	alu[--, --, b, skip_pop]			; if skip_pop = 1, then we already did the ctx_arb for the pop
	br!=0[fast_rest#]
	ctx_arb[sram]														; wait for pop to complete

fast_rest#:
	Rx_RestoreFastState[buf_handle, rec_state]							; get state from previous mpacket

not_using_buffer#:
	alu[skip_pop, --, b, 1]												; not using buffer, so save for next packet
	ld_field[rec_state, 0001, $xfer7]								; merge byte count, eop, sop from rcv_cntl 7:0 to rec_state 7:0

// calculate packet buffer address, update mpacket count in buf_handle
//
	Rx_SetPktBufAddr[packet_buf_addr, buf_handle, rec_state]						; defined in rec_nextpac



store_data#:


// move the packet data from receive fifo element to sdram (macro defined in rec_nextpac.uc)
//
	.if (bit(rec_state, 1) == 1)										; if EOP
		.if (bit(rec_state, 19) == 1)									; if discard
			br[next_packet#]											; if discard, don't enqueue
		.endif

	    sdram_r_fifo_rd[packet_buf_addr, rfifo_entry, 0, 8, SYNC]		; move mpacket to sdram, ctx_swap


#ifdef ZERO_PROTOCOL
#if (ZERO_PROTOCOL > 0)
tag_the_packet#:
		tag_packet							; insert receive control and status in bytes 32-39
#endif
#endif

last_mpacket_stored#:
			alu[tempb, 0xf, AND, rec_state, >>20]						; get SOP seq number
			state_restore[$xfer5, output_port, RX_FPORT1_OUTPORT, tempb]
			br[rec_enqueue#]											; write descriptor and enqueue 

	.else  ; Not EOP

		.if (bit(rec_state, 19) == 0)									; if no discard
			sdram_r_fifo_rd[packet_buf_addr, rfifo_entry, 0, 8, ASYNC]	; move mpacket to sdram, sig_done
		.endif
mid_mpacket_stored#:													; not SOP not EOP
			alu[tempb, 0x1e, AND, $xfer7, >>13]							; get mseq num
fast_save2#:
#ifdef ZERO_PROTOCOL
			.if (bit(rec_state, 0) == 0)								; if not SOP and ZERO_PROTOCOL
#endif
				Rx_SaveFastState[buf_handle, rec_state, tempb]			; save state to next mpacket
#ifdef ZERO_PROTOCOL
			.endif
#endif
			alu_shf[--, rec_state, AND, 1, <<19]
			br>0[no_sdram_pending1#]									; if discard there will be no sdram sig
			ctx_arb[sdram]
no_sdram_pending1#:

			br[next_packet#]											; get next slow packet or fast mpacket

	.endif



rec_enqueue_multicast#:
; *tbd*
	br[rec_enqueue_multicast#]

.endlocal

