;------------------------------------------------------------------------------------
;                                                                      
;                   I N T E L   P R O P R I E T A R Y                   
;                                                                       
;      COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;      THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                       
;                         INTEL  CORPORATION                            
;                                                                      
;                      2200 MISSION COLLEGE BLVD                        
;                                                                       
;                SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                       
;------------------------------------------------------------------------------------
// crc32_8.uc
// crc 32, using an 8 bit table lookup
//
// Version = 1.1.NoBldNum
// system: SA1200
// subsystem: crc microcode
// usage: example
// author: dfh 5/4/98
// revisions:
// status: assembles and runs from crc32_8.ind
// ---------------------------------------------------------------------

// est performance:		ucode time= 4.5 cycles/byte,  sram time= 32/8= 4.0 cycles per byte
//						8.5 cycles per byte per context 

// inputs:
// packet_addr	is set from .ind at initialization
// packet_len	is set fro model .ind at initialization
// $packet data0, $packet_data1	is set from model .ind each iteration

// the four contexts in the microengine will rotate calculations and memory accesses.
// each performing crc calculations and setting up table reads,
// then context swapping to let the next context run. 
// The net effect is the microengine stays busy.

// note: in the real thing, packet_addr would most likely come from a freelist 
// using the pop instruction, packet_len would come form the header, and $packet_data
// would come from the receive fifo.
// crc32_8_table would be setup by a core "C" application. it would be a shared
// address between core application and the microcode.

// c code to generate the table. extract this and run separately.

/*
	int j;
	unsigned i;
	unsigned generator;
	unsigned crc32_8_table[256];
 */
/* 32 bit generator is:
           x^32 + x^26 + x^23 + x^22 + x^16 + x^12 + x^11 + x^10 + x^8
         + x^7 + x^5 + x^4 + x^2 + x^1 + x^0
          0000 0100 1100 0001 0001 1101 1011 0111 
          0x04c11db7
 */
 /*
	generator = 0x04c11db7;
	// Now loop through entries 0 through 0xff.  This forms the 8 bit table.
	for (i = 0; i <= 0xff; i++)
	{
		crc32_8_table[i] = i;

		// Shift table number to line up with ms bit of generator.
		crc32_8_table[i] = crc32_8_table[i] << 24;

		// Now j the 8 bit quantity 8 times.
		for (j = 0; j < 8; j++)
		{
			if ((crc32_8_table[i] & 0x80000000) != 0)
			{
				crc32_8_table[i] = (crc32_8_table[i] << 1);
				crc32_8_table[i] = crc32_8_table[i] ^ generator;
			}
			else
			{
				crc32_8_table[i] = (crc32_8_table[i] << 1);
			}
		}
 */


// this brings in standard macros, notably immed32
#include "stdmac.uc"

	immed32[crc, 0xffffffff]						; preset the starting crc

// macro to perform crc32 lookup on 1 byte
#macro crc32_byte[result, source_data, shift_amount]
	ld_field_w_clr[data, 0001, source_data, shift_amount]
	alu_shf[addr, data, XOR, crc, >>24]
	sram[read, result, crc32_8_table, addr, 1]
#endm


.xfer_order $$packet_data0 $$packet_data1
.operand_synonym $table_entry6 $packet_data0
.operand_synonym $table_entry7 $packet_data1


iterate#:
// perform lookup on 8 bytes

	crc32_byte[$table_entry0, $packet_data0, >>24]
	crc32_byte[$table_entry1, $packet_data0, >>16]
	crc32_byte[$table_entry2, $packet_data0, >>8]
	crc32_byte[$table_entry3, $packet_data0, >>0]
	crc32_byte[$table_entry4, $packet_data1, >>24]
	crc32_byte[$table_entry5, $packet_data1, >>16]
	crc32_byte[$table_entry6, $packet_data1, >>8]

	ld_field_w_clr[data, 0001, $packet_data1]
	alu_shf[addr, data, XOR, crc, >>24]
	sram[read,$table_entry7 , crc32_8_table, addr, 1], sig_done		; last one asks for a signal

// save packet data in sdram

	alu[$$packet_data0, --, B, $packet_data0]				; move the data to write transfer reg
	alu[$$packet_data1, --, B, $packet_data1]				; move the data to write transfer reg
	sdram[write, $$packet_data0, packet_addr, 0, 1]			; write sdram 64 bytes 1 quadword
	ctx_arb[sram]											; come back when table reads are complete

// with ctx_arb, the next context can now run and process its packet

// we are back. calculate the crc for each byte using the table

	alu[crc, $table_entry0, XOR, crc, <<8]
	alu[crc, $table_entry1, XOR, crc, <<8]
	alu[crc, $table_entry2, XOR, crc, <<8]
	alu[crc, $table_entry3, XOR, crc, <<8]
	alu[crc, $table_entry4, XOR, crc, <<8]
	alu[crc, $table_entry5, XOR, crc, <<8]
	alu[crc, $table_entry6, XOR, crc, <<8]
	alu[crc, $table_entry7, XOR, crc, <<8]

// note: for byte count that is not multiple of 8, we would need to 
// process the last quadword separately with a test after each byte calc

	alu[packet_bytes, packet_bytes, -, 8]	
	br>0[iterate#], defer[1]
	alu[packet_addr, packet_addr, +, 8]

finish#:
	alu[crc, --, ~B, crc]
done#:
	ctx_arb[voluntary]
	br[done#]
	
	

	
