;------------------------------------------------------------------------------------
;                                                                      
;                   I N T E L   P R O P R I E T A R Y                   
;                                                                       
;      COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;      THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                       
;                         INTEL  CORPORATION                            
;                                                                      
;                      2200 MISSION COLLEGE BLVD                        
;                                                                       
;                SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                       
;------------------------------------------------------------------------------------
; mpy8x24.uc
; Version = 1.2.76
; 8x24 multiply using add if sign
;
;
; system: SA1200
; subsystem: utility microcode
; usage: example
; author: dfh 2/6/98
; revised: mgf 5/15/00
;
; ---------------------------SA1200 microcode--------------------------	


; Mpy_8x24
;	accumulator<31:0> = multiplier<7:0> x multiplicand<23:0>
;
; Prerequisite Register Usage:
;	symbol
;   multiplicand    - 24 bit multiplicand (ends up left-shifted by 7 at end of multiplication)
;   multiplier		- 8 bit multiplier
;
;	Shift the multiplier to cause a carry out at the test bit.
;	Then add the multiplicand if there was a carry out.
;	Note: the shift and add loop is unrolled to avoid a branch step per iteration.
;
; See Also:
;	mpy_8x24.ind	Tests this code. Sets iterate to 1 at end of multiply and repeats
;					for various argument values.
;	
Mpy_8x24#:
; initialize the iterate
	immed[iterate, 0]


	alu_shf [multiplicand, --, b, multiplicand, <<7]

	alu_shf[--, --, b, multiplier, <<31]
	alu_shf[--, --, b, multiplier, <<30]

    alu [accumulator, multiplicand, +ifsign, 0]
    alu [accumulator, multiplicand, +ifsign, accumulator, >>1]

	alu_shf[--, --, b, multiplier, <<29]
	alu_shf[--, --, b, multiplier, <<28]

    alu [accumulator, multiplicand, +ifsign, accumulator, >>1]
    alu [accumulator, multiplicand, +ifsign, accumulator, >>1]

	alu_shf[--, --, b, multiplier, <<27]
	alu_shf[--, --, b, multiplier, <<26]

    alu [accumulator, multiplicand, +ifsign, accumulator, >>1]
    alu [accumulator, multiplicand, +ifsign, accumulator, >>1]

	alu_shf[--, --, b, multiplier, <<25]
	alu_shf[--, --, b, multiplier, <<24]

    alu [accumulator, multiplicand, +ifsign, accumulator, >>1]
    alu [accumulator, multiplicand, +ifsign, accumulator, >>1]

Mpy_8x24Done#:
	nop
	nop
	nop
	alu[--, 0, B, iterate]		; interactively set this to >0 to iterate
	br>0[Mpy_8x24#]
; else 
	ctx_arb[voluntary]
	br[Mpy_8x24Done#]
