//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// critsect.uc
// critical section macros 
//
//
// system: IXP1200
// subsystem: OS emulation microcode
// usage: library macros
//
// revisions:
// 		v1.0, May 1, 2000,  jdg, dfh	initial version	


#ifndef CRITSECT_UC
#define CRITSECT_UC


// API:
//	critsect_init[abs_reg]
//	critsect_enter[abs_reg]
//	critsect_exit[abs_reg]


// critsect_init
//		description: Initialize a microengine's critical section absolute register to -1
//		inputs: 
//			abs_reg		absolute register containing the semaphore
//
#macro critsect_init(abs_reg)
	immed[abs_reg, 0xFFFFFFFF]
#endm


// critsect_enter
//		description: Enter a microengine's critical section.
//		inputs: 
//			abs_reg		absolute register containing the semaphore
//
#macro critsect_enter(abs_reg)
begin#:
	alu[--, --, B, abs_reg]
	br<0[end#], guess_branch
	ctx_arb[voluntary]
	br[begin#]
end#:
	immed[abs_reg, 0]
#endm


// critsect_exit
//		description: Exit a microengine's  critical section.
//		inputs: 
//			abs_reg		absolute register containing the semaphore
//
#macro critsect_exit(abs_reg)
	immed[abs_reg, 0xFFFFFFFF]
#endm	


#endif  //  CRITSECT_UC