//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// sdram.uc
// macros to access sdram
//
//
// system: IXP1200
// subsystem: utility microcode
// usage: library macros
// author: dfh	Aug 22, 2000. Originally from sdram.uc, change to generic name, add masked write, tfifo writes
//
// revisions:


#ifndef DRAM_UC
#define DRAM_UC

// API:
//	dram_read(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
//	dram_write(in_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
//	dram_mask_write(in_xfer, in_base_qw_addr, in_qw_offset, in_byte_mask, optional_token)
//	dram_rfifo_read[in_dram_addr, in_dram_offset, in_rfifo_addr, in_rfifo_offset, in_qw_count, optional_token]
//	dram_tfifo_write[in_dram_addr, in_dram_offset, in_tfifo_addr, in_tfifo_offset, in_qw_count, optional_token]
//	dram_align_tfifo_write[in_dram_addr, in_dram_offset, in_tfifo_addr, in_tfifo_offset, in_byte_align, in_qw_count, optional_token]
//

#include "stdmac.uc"
#include "xbuf.uc"


// dram_read
//		description: Read from dram.
//		outputs:
//			out_xfer			first dram xfer reg of sequence to read to, array notation
//								must be in xbuf array notation, index range  0-7
//		inputs:
//			in_base_qw_addr		register or constant base longword address
//			in_qw_offset		register or constant longword address offset
//			in_qw_count			register or constant longword count
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//								optimize_mem	optimize queue selection and sig_done
//								priority		priority queue and sig_done
//		temp register usage:	0-2 	uses regs if constant addr args > MAX_IMMEDIATE, or register length
//		example:
//			dram_read($packet[2], addr, 0, LWCOUNT3, ctx_swap);
//
//
#macro	dram_read(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
.local _temp_addr

#if ((isnum(in_base_qw_addr))&&(!isnum(in_qw_offset)))
	#if ((in_base_qw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_qw_addr]
		_dram_read_addr_asis(out_xfer, _temp_addr, in_qw_offset, in_qw_count, optional_token)
	#else
		_dram_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
	#endif  // ((in_base_qw_addr > MAX_IMMEDIATE))
 	// ((isnum(in_base_qw_addr))&&(!isnum(in_qw_offset)))

#elif ((isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))
	#if ((in_qw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_qw_offset]
		_dram_read_addr_asis(out_xfer, _temp_addr, in_base_qw_addr, in_qw_count, optional_token)
	#else
		_dram_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
	#endif  // ((in_qw_offset > MAX_IMMEDIATE))
 	// ((isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))

#elif ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))
	#define_eval _addr_sum (in_qw_offset + in_base_qw_addr)
	immed32[_temp_addr, _addr_sum]
	_dram_read_addr_asis(out_xfer, _temp_addr, 0, in_qw_count, optional_token)

// else both addresses are GPRs
#else //((!isnum(in_qw_offset))&&(!in_base_qw_addr))) 
	_dram_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)

#endif // ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))
.endlocal	// _temp_addr
#endm


// _dram_read_addr_asis
// 		description: dram read where addresses are acceptable as is to Assembler
// 					 support macro dram_read
#macro _dram_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
xbuf_find[out_xfer]		// if found, #define _xbuf_found, _cur_d_xbuf_name, _cur_d_xbuf_size
#if (isnum (in_qw_count))
	#ifdef _xbuf_found
		#define _perform_xbuf_op
	#endif
#endif
#ifdef _perform_xbuf_op
	_dram_read_to_xbufs[in_base_qw_addr, in_qw_offset, in_qw_count, optional_token]
	#undef _perform_xbuf_op
#elif (isnum (in_qw_count))
     _dram_wtok[read, out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token]
#else
	.local _temp_ind
	immed32 [_temp_ind, 0x100000]							; set bit 20 indirect refcount override bit
	alu_shf_left[--, _temp_ind, OR, in_qw_count, 16]		; insert indirect refcount
	_dram_ind_wtok[read, out_xfer, in_base_qw_addr, in_qw_offset, 1, optional_token]
	.endlocal //  _temp_ind
#endif	// (isnum (in_qw_count))

#endm


// _dram_read_to_xbufs
//		description: read to one or more(linked) xbufs.  
//					 support macro for dram_read
//		
#macro _dram_read_to_xbufs(in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
.local new_base_addr

#define_eval _rem_buf (_cur_d_xbuf_size - _cur_d_xbuf_index)
#define_eval _cur_lw_count (in_qw_count *2)
#define_eval _cur_qw_count (in_qw_count)
#define_eval _cur_qw_offset (in_qw_offset)

#if (_cur_lw_count > _rem_buf)
	#define _use_new_base
#endif

#if (!isnum(in_qw_offset))
	#ifndef _use_new_base
		#define _use_new_base
	#endif
#endif

#ifdef _use_new_base
	alu[new_base_addr, in_base_qw_addr, +, in_qw_offset]
	#define_eval _cur_qw_offset (0)
#endif

#while (_cur_lw_count > _rem_buf)
	#define_eval _cur_qw_count (_rem_buf / 2)
	sdram[read, _cur_d_xbuf_name/**/_cur_d_xbuf_index, new_base_addr, _cur_qw_offset, _cur_qw_count]
	#define_eval _cur_qw_offset (_cur_qw_offset + _cur_qw_count)
	#define_eval _cur_lw_count (_cur_lw_count - _rem_buf)
	#define_eval _cur_qw_count (_cur_lw_count / 2)

	#ifdef _cur_d_xbuf_next
		 xbuf_param_set[_cur_d_xbuf_next]
		#define_eval _cur_xbuf_index 0
		#define_eval _rem_buf _cur_d_xbuf_size
	#else
		#error	"_dram_read_to_xbufs. out of xbuf range"
	#endif	//  _cur_d_xbuf_next
#endloop

// perform last operation
#ifdef _use_new_base
	_dram_wtok[read, _cur_d_xbuf_name/**/_cur_d_xbuf_index, new_base_addr, _cur_qw_offset, _cur_qw_count, optional_token]
	#undef _use_new_base
#else
	_dram_wtok[read, _cur_d_xbuf_name/**/_cur_d_xbuf_index, in_base_qw_addr, _cur_qw_offset, _cur_qw_count, optional_token]
#endif
.endlocal
#endm


// dram_write
//		description: Write to dram.  Please note: in_qw_count should be qw_count - 1 
//		when a register is used to store in_qw_count.
//
//		inputs:
//			in_xfer				first dram xfer reg of sequence to write from, array notation
//								must be in xbuf array notation, index range  0-7
//			in_base_qw_addr		register or constant base longword address
//			in_qw_offset		register or constant longword address offset
//			in_qw_count			register or constant longword count.
//							    if register is input -> in_qw_count = quadword count
//								if constant is input -> in_qw_count = quadword count - 1
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//								optimize_mem	optimize queue selection and sig_done
//								priority		priority queue and sig_done
//		temp register usage:	0-2 	uses regs if constant addr args > MAX_IMMEDIATE, or register length
//		example:
//			dram_write($packet[2], addr, 0, LWCOUNT3, priority);
//
#macro	dram_write(in_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
.local _temp_addr

#if ((isnum(in_base_qw_addr))&&(!isnum(in_qw_offset)))
	#if ((in_base_qw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_qw_addr]
		_dram_write_addr_asis(in_xfer, _temp_addr, in_qw_offset, in_qw_count, optional_token)
	#else
		_dram_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
	#endif  // ((in_base_qw_addr > MAX_IMMEDIATE))
 	// ((isnum(in_base_qw_addr))&&(!isnum(in_qw_offset)))

#elif ((isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))
	#if ((in_qw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_qw_offset]
		_dram_write_addr_asis(in_xfer, _temp_addr, in_base_qw_addr, in_qw_count, optional_token)
	#else
		_dram_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
	#endif  // ((in_qw_offset > MAX_IMMEDIATE))
 	// ((isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))

#elif ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))
	#define_eval _addr_sum (in_qw_offset + in_base_qw_addr)
	immed32[_temp_addr, _addr_sum]
	_dram_write_addr_asis(in_xfer, _temp_addr, 0, in_qw_count, optional_token)

// else both addresses are GPRs
#else // ((!isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))
	_dram_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)

#endif // ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))
.endlocal	// _temp_addr

#endm


// _dram_write_addr_asis
// 		description: dram write where addresses are acceptable as is to Assembler
// 					 support macro dram_write
//
#macro _dram_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
xbuf_find[out_xfer]		// if found, #define _xbuf_found, _cur_d_xbuf_name, _cur_d_xbuf_size
#if (isnum (in_qw_count))
	#ifdef _xbuf_found
		#define _perform_xbuf_op
	#endif
#endif
#ifdef _perform_xbuf_op
	_dram_write_from_xbufs[in_base_qw_addr, in_qw_offset, in_qw_count, optional_token]
	#undef _perform_xbuf_op
#elif (isnum (in_qw_count))
     _dram_wtok[write, in_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token]
#else
	.local _temp_ind
	immed32 [_temp_ind, 0x100000]							; set bit 20 indirect refcount override bit
	alu_shf_left[--, in_qw_count, OR, _temp_ind, 16]		; insert indirect refcount
     _dram_ind_wtok[write, in_xfer, in_base_qw_addr, in_qw_offset,  2, optional_token]
	.endlocal //  _temp_ind
#endif	// (isnum (in_qw_count))

#endm


// _dram_write_from_xbufs
//		description: write from multiple xbufs. 
//					 support macro dram_write
//		
#macro _dram_write_from_xbufs(in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
.local new_base_addr

#define_eval _rem_buf (_cur_d_xbuf_size - _cur_d_xbuf_index)
#define_eval _cur_lw_count (in_qw_count * 2)
#define_eval _cur_qw_count (in_qw_count)
#define_eval _cur_qw_offset (in_qw_offset)

#if (_cur_lw_count > _rem_buf)
	#define _use_new_base
#endif

#if (!isnum(in_qw_offset))
	#ifndef _use_new_base
		#define _use_new_base
	#endif
#endif

#ifdef _use_new_base
	alu[new_base_addr, in_base_qw_addr, +, in_qw_offset]
	#define_eval _cur_qw_offset (0)
#endif

#while (_cur_lw_count > _rem_buf)
	#define_eval _cur_qw_count (_rem_buf / 2)
	sdram[write, _cur_d_xbuf_name/**/_cur_d_xbuf_index, new_base_addr, _cur_qw_offset, _cur_qw_count]
	#define_eval _cur_qw_offset (_cur_qw_offset + _cur_qw_count)
	#define_eval _cur_lw_count (_cur_lw_count - _rem_buf)
	#define_eval _cur_qw_count (_cur_lw_count / 2)

	#ifdef _cur_d_xbuf_next
		xbuf_param_set[_cur_d_xbuf_next]
		#define_eval _cur_xbuf_index 0
		#define_eval _rem_buf _cur_d_xbuf_size
	#else
		#error	"_dram_write_from_xbufs. out of xbuf range"
	#endif	//  _cur_d_xbuf_next
#endloop
// perform last operation
#ifdef _use_new_base
	_dram_wtok[write, _cur_d_xbuf_name/**/_cur_d_xbuf_index, new_base_addr, _cur_qw_offset, _cur_qw_count, optional_token]
	#undef _use_new_base
#else
	_dram_wtok[write, _cur_d_xbuf_name/**/_cur_d_xbuf_index, in_base_qw_addr, _cur_qw_offset, _cur_qw_count, optional_token]
#endif
.endlocal
#endm


// dram_mask_write
//		description: Write selected bytes of a 64 bit quadword to dram.
//		inputs:
//			in_xfer				first dram xfer reg of sequence to write from, array notation
//								must be in xbuf array notation, index range  0-7
//			in_base_qw_addr		register or constant base longword address
//			in_qw_offset		register or constant longword address offset
//			in_byte_mask		register or constant 8 bit mask indicating which bytes to write
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//								optimize_mem	optimize queue selection and sig_done
//								priority		priority queue and sig_done
//		temp register usage:	0-2 	uses regs if constant addr args > MAX_IMMEDIATE, or register mask
//		example:
//			dram_write($packet[2], addr, 0, LWCOUNT3, priority);
//
#macro	dram_mask_write(in_xfer, in_base_qw_addr, in_qw_offset, in_byte_mask, optional_token)
.local _temp_addr

#if ((isnum(in_base_qw_addr))&&(!isnum(in_qw_offset)))
	#if ((in_base_qw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_qw_addr]
		_masked_write_addr_asis(in_xfer, _temp_addr, in_qw_offset, in_byte_mask, optional_token)
	#else
		_masked_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_byte_mask, optional_token)
	#endif  // ((in_base_qw_addr > MAX_IMMEDIATE))
 	// ((isnum(in_base_qw_addr))&&(!isnum(in_qw_offset)))

#elif ((isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))
	#if ((in_qw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_qw_offset]
		_masked_write_addr_asis(in_xfer, _temp_addr, in_base_qw_addr, in_byte_mask, optional_token)
	#else
		_masked_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_byte_mask, optional_token)
	#endif  // ((in_qw_offset > MAX_IMMEDIATE))
 	// ((isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))

#elif ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))
	#define_eval _addr_sum (in_qw_offset + in_base_qw_addr)
	immed32[_temp_addr, _addr_sum]
	_masked_write_addr_asis(in_xfer, _temp_addr, 0, in_byte_mask, optional_token)

// else both addresses are GPRs
#else // ((!isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))
	_masked_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_byte_mask, optional_token)

#endif // ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))
.endlocal	// _temp_addr

#endm


// _masked_write_addr_asis
// 		description: dram write where addresses are acceptable as is to Assembler
// 					 support macro dram_mask_write
//
#macro _masked_write_addr_asis(in_xfer, in_base_qw_addr, in_qw_offset, in_byte_mask, optional_token)
.local _temp_mask _temp_ind

#ifdef _mask_is_temp_gpr
	#undef _mask_is_temp_gpr
#endif

#if (isnum (in_byte_mask))
	#if (in_byte_mask > MAX_IMMEDIATE)
		immed32[_temp_mask, in_byte_mask]
		#define _mask_is_temp_gpr
	#endif
#endif
		
immed32 [_temp_ind, 0x8000]								; set bit 15 mask override bit
#ifdef _mask_is_temp_gpr
	alu_shf_left[--, _temp_ind, OR, _temp_mask, 7]		; insert indirect mask
#else
	alu_shf_left[--, _temp_ind, OR, in_byte_mask, 7]		; insert indirect mask
#endif
//#define_eval _const_qw_count _temp_ind
_dram_ind_wtok[write, in_xfer, in_base_qw_addr, in_qw_offset, 1, optional_token]

.endlocal // _temp_mask  _temp_ind
#endm


// dram_rfifo_read
// 		description: Copy 1-16 quadwords from rfifo(base+offset) to sdram.
//		outputs:
//		inputs:
//			in_dram_addr		register or constant. 64 bit(quadword) base address to be added to form dram address
//			in_dram_offset		register or constant. 64 bit(quadword) offset to be added to form dram address 
//			in_rfifo_addr		register or constant. 64 bit(quadword) base address to be added to form rfifo address
//			in_rfifo_offset		register or constant. 64 bit(quadword) offset to be added to form rfifo address
//									since this is an offset into rfifo element, max value is 31
//			in_qw_count			register or constant. number of quadwords to write
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 1-5 instr
//		example usage:
//
#macro dram_rfifo_read[in_dram_addr, in_dram_offset, in_rfifo_addr, in_rfifo_offset, in_qw_count, optional_token]
.local _temp_ind _temp_dram_addr

#if ((isnum(in_dram_addr)) && (!isnum(in_dram_offset)))
	#if (in_dram_addr > MAX_IMMEDIATE)
		immed32[_temp_dram_addr, in_dram_addr]
		#define _temp_addr_encode 2
	#else
		#define _temp_addr_encode 0
	#endif

#elif ((!isnum(in_dram_addr)) && (isnum(in_dram_offset)))
	#if (in_dram_offset > MAX_IMMEDIATE)
		immed32[_temp_dram_addr, in_dram_offset]
		#define _temp_addr_encode 1
	#else
		#define _temp_addr_encode 0
	#endif

#elif ((isnum(in_dram_addr)) && (isnum(in_dram_offset)))
	#define_eval _dram_addr_sum (in_dram_addr + in_dram_offset)
	immed32[_temp_dram_addr, _dram_addr_sum]
	#define _temp_addr_encode 3
#else
	#define _temp_addr_encode 0
#endif

#if (isnum(in_qw_count))
	#if (isnum (in_rfifo_offset))
		#if (in_rfifo_offset == 0)
			alu_shf[--, --, B, in_rfifo_addr, <<4]
		#else
			alu_shf[_temp_ind, --, B, in_rfifo_offset, <<4]
			alu_shf[--, _temp_ind, +, in_rfifo_addr, <<4]
		#endif
	#else
		alu_shf[_temp_ind, --, B, in_rfifo_offset, <<4]
		alu_shf[--, _temp_ind, +, in_rfifo_addr, <<4]
	#endif
#else	// in_qw_count is a register
// use indirect count (bit 20 overide, 16-19 count)
	alu[_temp_ind, --, B, 1, <<20]
	alu[_temp_ind, _temp_ind, +, in_qw_count, <<16]
	#if (in_rfifo_offset == 0)
		alu_shf[--, _temp_ind, OR, in_rfifo_addr, <<4]
	#else
		alu_shf[_temp_ind, _temp_ind, OR, in_rfifo_offset, <<4]
		alu_shf[--, _temp_ind, +, in_rfifo_addr, <<4]
	#endif
#endif	// in_qw_count is a register

#if (isnum(in_qw_count))
	#define_eval _const_qw_count in_qw_count
#else
	#define_eval _const_qw_count 1
#endif

#if (_temp_addr_encode == 0)
	_dram_ind_wtok[r_fifo_rd, --, in_dram_addr, in_dram_offset, _const_qw_count, optional_token]
#elif (_temp_addr_encode == 1)
	_dram_ind_wtok[r_fifo_rd, --, in_dram_addr, _temp_dram_addr, _const_qw_count, optional_token]
#elif (_temp_addr_encode == 2)
	_dram_ind_wtok[r_fifo_rd, --, _temp_dram_addr, in_dram_offset, _const_qw_count, optional_token]
#else
	_dram_ind_wtok[r_fifo_rd, --, _temp_dram_addr, 0, _const_qw_count, optional_token]
#endif

#undef _temp_addr_encode
.endlocal // _temp_ind  _temp_dram_addr
#endm


// dram_tfifo_write
// 		description: Copy 1-16 quadwords from dram to tfifo(base+offset) 
//		outputs:
//		inputs:
//			in_dram_addr		register or constant. 64 bit(quadword) base address to be added to form dram address
//			in_dram_offset		register or constant. 64 bit(quadword) offset to be added to form dram address 
//			in_tfifo_addr		register or constant. 64 bit(quadword) base address to be added to form tfifo address
//			in_tfifo_offset		register or constant. 64 bit(quadword) offset to be added to form tfifo address
//									since this is an offset into tfifo element, max value is 31
//			in_qw_count			register or constant. number of quadwords to write
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 1-5 instr
//		example usage:
//
#macro dram_tfifo_write[in_dram_addr, in_dram_offset, in_tfifo_addr, in_tfifo_offset, in_qw_count, optional_token]
.local _temp_ind _temp_dram_addr

#if ((isnum(in_dram_addr)) && (!isnum(in_dram_offset)))
	#if (in_dram_addr > MAX_IMMEDIATE)
		immed32[_temp_dram_addr, in_dram_addr]
		#define _temp_addr_encode 2
	#else
		#define _temp_addr_encode 0
	#endif

#elif ((!isnum(in_dram_addr)) && (isnum(in_dram_offset)))
	#if (in_dram_offset > MAX_IMMEDIATE)
		immed32[_temp_dram_addr, in_dram_offset]
		#define _temp_addr_encode 1
	#else
		#define _temp_addr_encode 0
	#endif

#elif ((isnum(in_dram_addr)) && (isnum(in_dram_offset)))
	#define_eval _dram_addr_sum (in_dram_addr + in_dram_offset)
	immed32[_temp_dram_addr, _dram_addr_sum]
	#define _temp_addr_encode 3
#else
	#define _temp_addr_encode 0
#endif

#if (isnum(in_qw_count))
	#if (isnum (in_tfifo_offset))
		#if (in_tfifo_offset == 0)
			alu_shf[--, --, B, in_tfifo_addr, <<4]
		#else
			alu_shf[_temp_ind, --, B, in_tfifo_offset, <<4]
			alu_shf[--, _temp_ind, +, in_tfifo_addr, <<4]
		#endif
	#else
		alu_shf[_temp_ind, --, B, in_tfifo_offset, <<4]
		alu_shf[--, _temp_ind, +, in_tfifo_addr, <<4]
	#endif
#else	// in_qw_count is a register
// use indirect count (bit 20 overide, 16-19 count)
	alu[_temp_ind, --, B, 1, <<20]
	alu[_temp_ind, _temp_ind, +, in_qw_count, <<16]
	#if (in_tfifo_offset == 0)
		alu_shf[--, _temp_ind, OR, in_tfifo_addr, <<4]
	#else
		alu_shf[_temp_ind, _temp_ind, OR, in_tfifo_offset, <<4]
		alu_shf[--, _temp_ind, +, in_tfifo_addr, <<4]
	#endif
#endif	// in_qw_count is a register

#if (isnum(in_qw_count))
	#define_eval _const_qw_count in_qw_count
#else
	#define_eval _const_qw_count 1
#endif

#if (_temp_addr_encode == 0)
	_dram_ind_wtok[t_fifo_wr, --, in_dram_addr, in_dram_offset, _const_qw_count, optional_token]
#elif (_temp_addr_encode == 1)
	_dram_ind_wtok[t_fifo_wr, --, in_dram_addr, _temp_dram_addr, _const_qw_count, optional_token]
#elif (_temp_addr_encode == 2)
	_dram_ind_wtok[t_fifo_wr, --, _temp_dram_addr, in_dram_offset, _const_qw_count, optional_token]
#else
	_dram_ind_wtok[t_fifo_wr, --, _temp_dram_addr, 0, _const_qw_count, optional_token]
#endif

#undef _temp_addr_encode
.endlocal // _temp_ind  _temp_dram_addr
#endm


// dram_align_tfifo_write
// 		description: Copy 1-16 quadwords from dram to tfifo(in_tfifo_addr+) 
//		outputs:
//		inputs:
//			in_dram_addr		register or constant. 64 bit(quadword) base address to be added to form dram address
//			in_dram_offset		register or constant. 64 bit(quadword) offset to be added to form dram address 
//			in_tfifo_addr		register or constant. 64 bit(quadword) base address to be added to form tfifo address
//			in_tfifo_offset		register or constant. 64 bit(quadword) offset to be added to form tfifo address
//									since this is an offset into tfifo element, max value is 31
//			in_byte_align		register or constant. byte alignment (0-7) for start of data at tfifo
//			in_qw_count			register or constant. number of quadwords to write
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 1-5 instr
//		example usage:
//
#macro dram_align_tfifo_write[in_dram_addr, in_dram_offset, in_tfifo_addr, in_tfifo_offset, in_byte_align, in_qw_count, optional_token]
.local _temp_ind _temp_dram_addr

#if ((isnum(in_dram_addr)) && (!isnum(in_dram_offset)))
	#if (in_dram_addr > MAX_IMMEDIATE)
		immed32[_temp_dram_addr, in_dram_addr]
		#define _temp_addr_encode 2
	#else
		#define _temp_addr_encode 0
	#endif

#elif ((!isnum(in_dram_addr)) && (isnum(in_dram_offset)))
	#if (in_dram_offset > MAX_IMMEDIATE)
		immed32[_temp_dram_addr, in_dram_offset]
		#define _temp_addr_encode 1
	#else
		#define _temp_addr_encode 0
	#endif

#elif ((isnum(in_dram_addr)) && (isnum(in_dram_offset)))
	#define_eval _dram_addr_sum (in_dram_addr + in_dram_offset)
	immed32[_temp_dram_addr, _dram_addr_sum]
	#define _temp_addr_encode 3
#else
	#define _temp_addr_encode 0
#endif

#if (isnum(in_qw_count))
	#if (isnum (in_tfifo_offset))
		#if (in_tfifo_offset == 0)
			alu_shf[_temp_ind, --, B, in_tfifo_addr, <<4]
		#else
			alu_shf[_temp_ind, --, B, in_tfifo_offset, <<4]
			alu_shf[_temp_ind, _temp_ind, +, in_tfifo_addr, <<4]
		#endif
	#else
		alu_shf[_temp_ind, --, B, in_tfifo_offset, <<4]
		alu_shf[_temp_ind, _temp_ind, +, in_tfifo_addr, <<4]
	#endif
#else	// in_qw_count is a register
// use indirect count (bit 20 overide, 16-19 count)
	alu[_temp_ind, --, B, 1, <<20]
	alu[_temp_ind, _temp_ind, +, in_qw_count, <<16]
	#if (in_tfifo_offset == 0)
		alu_shf[_temp_ind, _temp_ind, OR, in_tfifo_addr, <<4]
	#else
		alu_shf[_temp_ind, _temp_ind, OR, in_tfifo_offset, <<4]
		alu_shf[_temp_ind, _temp_ind, +, in_tfifo_addr, <<4]
	#endif
#endif	// in_qw_count is a register
alu_shf[--, _temp_ind, OR, in_byte_align, <<12]


#if (isnum(in_qw_count))
	#define_eval _const_qw_count in_qw_count
#else
	#define_eval _const_qw_count 1
#endif

#if (_temp_addr_encode == 0)
	_dram_ind_wtok[t_fifo_wr, --, in_dram_addr, in_dram_offset, _const_qw_count, optional_token]
#elif (_temp_addr_encode == 1)
	_dram_ind_wtok[t_fifo_wr, --, in_dram_addr, _temp_dram_addr, _const_qw_count, optional_token]
#elif (_temp_addr_encode == 2)
	_dram_ind_wtok[t_fifo_wr, --, _temp_dram_addr, in_dram_offset, _const_qw_count, optional_token]
#else
	_dram_ind_wtok[t_fifo_wr, --, _temp_dram_addr, 0, _const_qw_count, optional_token]
#endif

#undef _temp_addr_encode
.endlocal // _temp_ind  _temp_dram_addr
#endm


// support for optional_token
//	for optimize_mem and priority, force sig_done also
//
#macro _dram_wtok(op_code, io_xfer, in_base_lw_addr, in_lw_offset, op_count, optional_token)
	#if (streq(optional_token, optimize_mem))
		#define _combined_token	optimize_mem, sig_done
	#elif (streq(optional_token, priority))
		#define _combined_token priority, sig_done
	#else
		#define_eval _combined_token (optional_token)
	#endif

	sdram[op_code, io_xfer, in_base_lw_addr, in_lw_offset, op_count], _combined_token 

	#undef _combined_token
#endm

// support for optional_token
//	for optimize_mem and priority, force sig_done also
//
#macro _dram_ind_wtok(op_code, io_xfer, in_base_lw_addr, in_lw_offset, op_count, optional_token)
	#if (streq(optional_token, optimize_mem))
		#define _combined_token	optimize_mem, sig_done
	#elif (streq(optional_token, priority))
		#define _combined_token priority, sig_done
	#else
		#define_eval _combined_token (optional_token)
	#endif
	sdram[op_code, io_xfer, in_base_lw_addr, in_lw_offset, op_count], indirect_ref, _combined_token 

	#undef _combined_token

#endm

#endif // DRAM_UC