//------------------------------------------------------------------------------------
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  1999-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
//-----------------------------------------------------------------------------------
// field.uc
// IXPblock big/little-endian field extract/compare/branch macros
//
//
// system: IXP1200
// subsystem: IO microcode
// usage: library macros
// author: dfh Apr 3, 2000
//
// revisions:


#ifndef FIELD_UC
#define FIELD_UC
#include "endian.uc"

// API:
// field_br_eq(in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label)
// field_br_gtr(in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label)
// field_br_less(in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label)
// field_br_gtreq(in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label)
// field_br_lesseq(in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label)
// field_comp(in_src, in_start_byte, in_end_byte, COMPARE_VAL)
// field_dbl_extract(out_dest,  in_a, in_start_byte, in_b, in_end_byte)
// field_decr(out_dest, in_src, in_start_byte, in_end_byte)
// field_extract(out_dest, in_src, in_start_byte, in_end_byte)
// field_incr(out_dest, in_src, in_start_byte, in_end_byte)
// field_insert(io_a, in_byte_mask, in_b)
// field_shf_left_insert(io_a, in_byte_mask, in_b, in_shift_amt)
// field_shf_right_insert(io_a, in_byte_mask, in_b, in_shift_amt)
// field_clr_insert(out_a, in_byte_mask, in_b)
// field_clr_shf_left_insert(out_a, in_byte_mask, in_b, in_shift_amt)
// field_clr_shf_right_insert(out_a, in_byte_mask, in_b, in_shift_amt)


// default is BIG-ENDIAN
// by defining LITTLE_ENDIAN, the appropriate underlying swaps will be inserted
// this will allow you to write code that runs big- or little-endian
// for compatibility of new naming conventions with legacy macro names
//



// field_br_eq
//		description: compare byte field to a constant and branch if equal
//		inputs:
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//			COMPARE_VAL		constant value to compare field with
//			target_label	label to branch to
//		example:
//			field_br_eq(test_reg, 0, 2, 0xff, exception_handler#);
//
#macro field_br_eq[in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label]
	comp_field[in_src, in_start_byte, in_end_byte, COMPARE_VAL]
	br=0[target_label]
#endm


// field_br_gtr
//		description: compare byte field to a constant and branch if field is greater
//		inputs:
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//			COMPARE_VAL		constant value to compare field with
//			target_label	label to branch to
//		example:
//			field_br_gtr(test_reg, 0, 2, 0xff, exception_handler#);
//
#macro field_br_gtr[in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label]
	comp_field[in_src, in_start_byte, in_end_byte, COMPARE_VAL]
	br>0[target_label]
#endm


// field_br_gtreq
//		description: compare byte field to a constant and branch if field is greator or equal
//		inputs:
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//			COMPARE_VAL		constant value to compare field with
//			target_label	label to branch to
//		example:
//			field_br_gtreq(test_reg, 0, 2, 0xff, exception_handler#);
//
#macro field_br_gtreq[in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label]
	comp_field[in_src, in_start_byte, in_end_byte, COMPARE_VAL]
	br>=0[target_label]
#endm


// field_br_less
//		description: compare byte field to a constant and branch if field is less
//		inputs:
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//			COMPARE_VAL		constant value to compare field with
//			target_label	label to branch to
//		example:
//			field_br_less(test_reg, 0, 2, 0xff, exception_handler#);
//
#macro field_br_less[in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label]
	comp_field[in_src, in_start_byte, in_end_byte, COMPARE_VAL]
	br<0[target_label]
#endm


// field_br_lesseq
//		description: compare byte field to a constant and branch if field is less or equal
//		inputs:
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//			COMPARE_VAL		constant value to compare field with
//			target_label	label to branch to
//		example:
//			field_br_lesseq(test_reg, 0, 2, 0xff, exception_handler#);
//
#macro field_br_lesseq[in_src, in_start_byte, in_end_byte, COMPARE_VAL, target_label]
	comp_field[in_src, in_start_byte, in_end_byte, COMPARE_VAL]
	br<=0[target_label]
#endm


// field_comp
// 		description: extract byte field based on endian mode
// 					 compare byte field to COMPARE_VAL
//		cond code:			set as result of compare
//		inputs: 
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//			COMPARE_VAL		constant value to compare field with
//		example:
//			field_comp(test_reg, 0, 2, 0xff);
//
#macro field_comp[in_src, in_start_byte, in_end_byte, COMPARE_VAL]
.local comp_val1 comp_val2
#define_eval SBYTE in_start_byte
#define_eval EBYTE in_end_byte

#if (in_start_byte == in_end_byte)
	extract/**/SBYTE[comp_val1, in_src, 0]
#else
	extract/**/SBYTE/**/EBYTE[comp_val1, in_src, 0]
#endif
#if (COMPARE_VAL < 32)
	alu[--, comp_val1, -, COMPARE_VAL]
#else
	immed[comp_val2, COMPARE_VAL]
	alu[--, comp_val1, -, comp_val2]
#endif
.endlocal
#endm


// field_decr
// 		description: increment byte field from in_src and place result in out_dest
//		outputs:
//			out_dest		GPR or write transfer register
//		inputs: 
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//		example:
//			field_decr(reg, $xfer[0], 1, 3);  // decrement 3 byte field
//
#macro field_decr[out_dest, in_src, in_start_byte, in_end_byte]
#define_eval SBYTE in_start_byte
#define_eval EBYTE in_end_byte

#if (in_start_byte == in_end_byte)
	decr/**/SBYTE[out_dest, in_src]
#else
	decr/**/SBYTE/**/EBYTE[out_dest, in_src]
#endif
#endm


// field_incr
// 		description: increment byte field from in_src and place result in out_dest
//		outputs:
//			out_dest		GPR or write transfer register
//		inputs: 
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//		example:
//			field_incr(reg, $xfer[0], 1, 3);  // increment 3 byte field
//
#macro field_incr[out_dest, in_src, in_start_byte, in_end_byte]
#define_eval SBYTE in_start_byte
#define_eval EBYTE in_end_byte

#if (in_start_byte == in_end_byte)
	incr/**/SBYTE[out_dest, in_src]
#else
	incr/**/SBYTE/**/EBYTE[out_dest, in_src]
#endif
#endm


// field_extract
// 		description: extract byte field from in_src and place in out_dest
//		outputs:
//			out_dest		GPR
//		inputs: 
//			in_src			GPR or read transfer reg containing byte field
//			in_start_byte	starting byte position 0-3 based on endian mode
//			in_end_byte		ending byte position 0-3 based on endian mode
//						 	must be greater than in_start byte
//		example:
//			field_extract(reg, $xfer[0], 1, 3);  // extract 3 byte field
//
//
#macro field_extract(out_dest, in_src, in_start_byte, in_end_byte)
#define_eval SBYTE in_start_byte
#define_eval EBYTE in_end_byte

#if (SBYTE == EBYTE)
	extract/**/SBYTE[out_dest, in_src, 0]
#else
	extract/**/SBYTE/**/EBYTE[out_dest, in_src, 0]
#endif
#endm


// field_dbl_extract
//		description: get field that spans 2 in_src regs
//		outputs:
//			out_dest		GPR
//		inputs:
//			in_a			GPR or read transfer register, left source longword
//			in_start_byte	starting byte position 0-3 of in _a based on endian mode
//			in_b			GPR or read transfer register, right source longword
//			in_end_byte		ending byte position 0-3 of in_b based on endian mode
//						 	must be less than in_start byte
//		example:
//			field_dbl_extract(reg, $xfer[0], 3, $xfer[1], 1);  // extract 3 byte field
//
#macro field_dbl_extract(out_dest,  in_a, in_start_byte, in_b, in_end_byte)
#define_eval SBYTE in_start_byte
#define_eval EBYTE in_end_byte
#define_eval SHIFT (8+in_end_byte*8)

#if (SBYTE == 3)
	extract/**/SBYTE[out_dest, in_a, SHIFT]
#else
	extract/**/SBYTE/**/3[out_dest, in_a, SHIFT]
#endif
#if (EBYTE == 0)
	merge_extract/**/EBYTE[out_dest, in_b]
#else
	merge_extract0/**/EBYTE[out_dest, in_b]
#endif

#endm


// field_insert
//		description: inserts bytes of in_b specified by in_byte_mask in io_a
//					 source and dest registers must be same endian flavor
//		inputs:
//			io_a			GPR
//			in_byte_mask	xxxx, where x = 0 or 1. if 1 insert byte
//			in_b			GPR or read transfer register
//		example
//			field_insert(x, 0110, y);		// insert  y bytes 1,2 into x bytes 1.2
//
#macro field_insert(io_a, in_byte_mask, in_b)
	ld_field[io_a, in_byte_mask, in_b]
#endm


// field_shf_left_insert
//		description: inserts bytes of in_b specified by in_byte_mask in io_a
//					 source and dest registers must be same endian flavor
//		inputs:
//			io_a			GPR
//			in_byte_mask	xxxx, where x = 0 or 1. if 1 insert byte
//			in_b			GPR or read transfer register
//			in_shift_amt	shift amount 0-31
//		example
//			field_shf_left_insert(x, 0110, y, 8);	// insert  y bytes 2,3 into x bytes 1,2
//
#macro field_shf_left_insert(io_a, in_byte_mask, in_b, in_shift_amt)
#define_eval _SHF_AMT in_shift_amt
	ld_field[io_a, in_byte_mask, in_b, <</**/_SHF_AMT]
#endm


// field_shf_right_insert
//		description: inserts bytes of in_b specified by in_byte_mask in io_a
//					 after shifting in_b left by in_shift_amt
//					 source and dest registers must be same endian flavor
//		inputs:
//			io_a			GPR
//			in_byte_mask	xxxx, where x = 0 or 1. if 1 insert byte
//			in_b			GPR or read transfer register
//			in_shift_amt	shift amount 0-31
//		example
//			field_shf_right_insert(x, 0110, y, 8);		// insert  y bytes 0,1 into y bytes 1,2
//
#macro field_shf_right_insert(io_a, in_byte_mask, in_b, in_shift_amt)
#define_eval _SHF_AMT in_shift_amt
	ld_field[io_a, in_byte_mask, in_b, >>/**/_SHF_AMT]
#endm


// field_clr_insert
//		description: inserts bytes of in_b specified by in_byte_mask in out_a
//					 clear out_a prior to insert
//					 source and dest registers must be same endian flavor
//		inputs:
//			out_a			GPR or write transfer register
//			in_byte_mask	xxxx, where x = 0 or 1. if 1 insert byte
//			in_b			GPR or read transfer register
//		example
//			field_insert(x, 0110, y);		// insert  y bytes 1,2 into x bytes 1.2
//
#macro field_clr_insert(out_a, in_byte_mask, in_b)
	ld_field_w_clr[out_a, in_byte_mask, in_b]
#endm


// field_clr_shf_left_insert
//		description: inserts bytes of in_b specified by in_byte_mask in out_a
//					 after shifting in_b left by in_shift_amt
//					 clear out_a prior to insert
//					 source and dest registers must be same endian flavor
//		inputs:
//			out_a			GPR or write transfer register
//			in_byte_mask	xxxx, where x = 0 or 1. if 1 insert byte
//			in_b			GPR or read transfer register
//			in_shift_amt	shift amount 0-31
//		example
//			field_shf_left_insert(x, 0110, y, 8);	// insert  y bytes 2,3 into x bytes 1,2
//
#macro field_clr_shf_left_insert(out_a, in_byte_mask, in_b, in_shift_amt)
#define_eval _SHF_AMT in_shift_amt
	ld_field_w_clr[out_a, in_byte_mask, in_b, <</**/_SHF_AMT]
#endm


// field_clr_shf_right_insert
//		description: inserts bytes of in_b specified by in_byte_mask in out_a
//					 after shifting in_b right by in_shift_amt
//					 clear out_a prior to insert
//					 source and dest registers must be same endian flavor
//		inputs:
//			out_a			GPR or write transfer register
//			in_byte_mask	xxxx, where x = 0 or 1. if 1 insert byte
//			in_b			GPR or read transfer register
//			in_shift_amt	shift amount 0-31
//		example
//			field_shf_right_insert(x, 0110, y, 8);		// insert  y bytes 0,1 into y bytes 1,2
//
#macro field_clr_shf_right_insert(out_a, in_byte_mask, in_b, in_shift_amt)
#define_eval _SHF_AMT in_shift_amt
	ld_field_w_clr[out_a, in_byte_mask, in_b, >>/**/_SHF_AMT]
#endm





// legacy:
#define br_eq_field field_br_eq
#define br_gtr_field field_br_gtr 
#define br_gtreq_field field_br_gtreq 
#define br_less_field field_br_less
#define br_lesseq_field field_br_lesseq
#define comp_field field_comp
#define dbl_extract_field field_dbl_extract
#define decr_field field_decr
#define extract_field field_extract
#define incr_field field_incr


#endif // FIELD_UC