//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// port.uc
// macros to access ports (IX bus) 
//
//
// system: IXP1200
// subsystem: Bus IO microcode
// usage: library macros
// author: dfh	May 1, 2000
//
// revisions:
//	dfh	June 22, 2000. GPR arg names begin with in_, io_ or out_
//  mff July 31, 2000. Added tx macros.

#ifndef PORT_UC
#define PORT_UC
#include "critsect.uc"

// API:
//	port_rx_init(out_rec_req, out_rfifo_addr, UENGINE_ID)
//	port_rxrdy_chk(io_inflight_sem, in_rec_req)
//	port_rx_request(in_rec_req)
//	port_rx_receive(out_exception, io_rec_state, PORT_TYPE)
//	port_rx_fail(io_rec_state, in_rec_req, in_exception)
//	port_rx_error(io_rec_state, in_rec_req, in_exception)
//	port_rx_discard(io_rec_state, in_rec_req, in_exception)
//	port_rx_bytecount_extract(out_bytecount, in_rec_state)
//  port_txrdy_chk(out_abs_mpkt_tx_ok_cnt, in_port_rdy_bit)
//  port_txrdy_wait(in_port_rdy_bit)




// port types
#define SLOW_PORT  0x0100
#define ETHER_10M  0x0101
#define ETHER_100M 0x0102
#define ATM_OC3	   0x0103
#define FAST_PORT  0x0201
#define ETHER_GIG  0x0202
#define ATM_OC12   0x0203
#define ATM_OC48   0x0204


// port exceptions, range is 0-0x7
#define    PORT_RXERROR					0x01
#define	   PORT_RXFAIL					0x02
#define	   PORT_RXCANCEL				0x04
#define    PORT_SHDBE_SOP				0x05
#define    PORT_SHDBE_NOT_SOP			0x06

// define starting bit of fields for receive state
//
// rec_state is receive control rotated left 8, leaving the resulting 7:0 blank
#define REC_STATE_ELE_COUNT 0			// element or cell count since SOP or first cell
#define REC_STATE_SOP_BIT 8				// SOP indication from bus interface
#define REC_STATE_EOP_BIT 9				// EOP indication from bus interface 
#define REC_STATE_BYTE_COUNT 10			// 10-15 byte count from bus interface
#define REC_STATE_FREELIST_ID 24		// 24-25 freelist id
#define REC_STATE_DISCARD_BIT 26		// discard flag from verification routine
#define REC_STATE_SOPSEQ 27				// 27-30 fast port SOP sequence number from bus interface





// port_rx_init
// 		description: Initialize registers with receive request format and rfifo element address 
//		outputs:
//			out_rec_req			receive request with element, thread and port. See PRM
//			out_rfifo_addr		quadword addrsss of receive fifo element	
//		inputs:
//			UENGINE_ID			microengine id 0-5
//			CTX0PORT			port number for context 0
//			CTX1PORT			port number for context 1
//			CTX2PORT			port number for context 2
//			CTX3PORT			port number for context 3
//		size: 11 instr
//		latency: 4 cycles
//		see also: stdmac.uc
//		example usage:
//			#define UENGINE_ID 0
//			#deifne PORT_NUMS 0,1,2,3
//			port_rx_init(rec_req, rfifo_addr, UENGINE_ID, PORT_NUMS)
//
#macro port_rx_init(out_rec_req, out_rfifo_addr, UENGINE_ID, CTX0PORT, CTX1PORT, CTX2PORT, CTX3PORT)
br=ctx[1, context1#]
br=ctx[2, context2#]
br=ctx[3, context3#]

context0#:
#define_eval MY_TID0 (UENGINE_ID<<2)

#if ((CTX0PORT >= FAST_PORT_LO) & (CTX0PORT <= FAST_PORT_HI))
#define_eval FAST_PORT_SEL0 ((CTX0PORT & 1) +1)
#else
#define_eval FAST_PORT_SEL0 (0)
#endif
#define_eval RREQUEST0 ((CTX0PORT) | (MY_TID0<<6) |(MY_TID0<<18) | (FAST_PORT_SEL0<<16))
#define_eval RFIFO_QW_ADDR0 (MY_TID0<<3)

	immed32[out_rec_req, RREQUEST0]
	immed[out_rfifo_addr, RFIFO_QW_ADDR0]
	br[end#]

context1#:
#define_eval MY_TID1 ((UENGINE_ID<<2)+1)

#if ((CTX1PORT >= FAST_PORT_LO) & (CTX1PORT <= FAST_PORT_HI))
#define_eval FAST_PORT_SEL1 ((CTX1PORT & 1) +1)
#else
#define_eval FAST_PORT_SEL1 (0)
#endif

#define_eval RREQUEST1 (CTX1PORT | (MY_TID1<<6) |(MY_TID1<<18) | (FAST_PORT_SEL1<<16))
#define_eval RFIFO_QW_ADDR1 (MY_TID1<<3)

	immed32[out_rec_req, RREQUEST1]
	immed[out_rfifo_addr, RFIFO_QW_ADDR1]
br[end#]

context2#:
#define_eval MY_TID2 ((UENGINE_ID<<2)+2)

#if ((CTX2PORT >= FAST_PORT_LO) & (CTX2PORT <= FAST_PORT_HI))
#define_eval FAST_PORT_SEL2 ((CTX2PORT & 1) +1)
#else
#define_eval FAST_PORT_SEL2 (0)
#endif

#define_eval RREQUEST2 (CTX2PORT | (MY_TID2<<6) |(MY_TID2<<18) | (FAST_PORT_SEL2<<16))
#define_eval RFIFO_QW_ADDR2 (MY_TID2<<3)

	immed32[out_rec_req, RREQUEST2]
	immed[out_rfifo_addr, RFIFO_QW_ADDR2]
br[end#]

context3#:
#define_eval MY_TID3 ((UENGINE_ID<<2)+3)

#if ((CTX3PORT >= FAST_PORT_LO) & (CTX3PORT <= FAST_PORT_HI))
#define_eval FAST_PORT_SEL3 ((CTX3PORT & 1) +1)
#else
#define_eval FAST_PORT_SEL3 (0)
#endif

#define_eval RREQUEST3 (CTX3PORT | (MY_TID3<<6) |(MY_TID3<<18) | (FAST_PORT_SEL3<<16))
#define_eval RFIFO_QW_ADDR3 (MY_TID3<<3)

	immed32[out_rec_req, RREQUEST3]
	immed[out_rfifo_addr, RFIFO_QW_ADDR3]
end#:
#endm

// port_rxrdy_chk
//		description: Check if receive port is ready
// 					check to see if read rcv ready already in flight from this uengine
// 					if there is, then wait for it to finish before this read.  We can not have every
// 					slow port receive thread doing this csr at the same time, or the threads that
// 					are processing packets will not get to use the fbi
//
//		inputs:
//			in_rec_req			GPR. assembled receive request
//			in_inflight_sem		Absolute GPR inflight semaphore initialized by critsect_init
//		size: 12 instr
//		example usage:
//			crit_sect_init(@rdready_inflight);
//			port_rx_check_ready(@rdready_inflight, rec_req);	
//

#macro port_rxrdy_chk(io_inflight_sem, in_rec_req)
.local $rec_rdy

check_port#:
	critsect_enter[io_inflight_sem]				; block other contexts from reading rec ready
	csr[read, $rec_rdy, RCV_RDY_LO], ctx_swap	; get port ready
	critsect_exit[io_inflight_sem]				; allow other contexts to read rec ready
	alu[--, in_rec_req, B, 0]					; shift right by lo 5 bits (port number)
	alu[--, 1, AND,  $rec_rdy, >>indirect]
	br!=0[port_rdy1#]							; if our port has data then go process it
	ctx_arb[voluntary]							; if our port doesn't have data, stay in this loop waiting for it
	br[check_port#]
port_rdy1#:

.endlocal
#endm

// port_rx_request
//		description: Issue receive request
//		inputs:
//			in_rec_req			GPR. assembled receive request
//
#macro port_rx_request(in_rec_req)
.local $req_csr
	move[$req_csr, in_rec_req]
	csr[write, $req_csr, rcv_req]						// send req to IX Bus Interface
.endlocal
#endm

// port_rx_receive
// 		description: Wait for start_receive signal, then read receive control and load rec_state
//					 If PORT_TYPE is a fast port, issue without checking port ready.
//					 If PORT_TYPE is a slow port, check port ready then issue.
//		outputs:
//			out_exception		0 if no exception,
//								otherwise PORT_RXERROR, PORT_RXFAIL, or PORT_RXCANCEL
//		output/input:
//			io_rec_state		captured receive control info into receive state	
//		size: 12 instr
//		example usage:
//			port_rx_receive(exception, rec_state, ETHER_100M);	
//
#macro port_rx_receive(out_exception, io_rec_state, PORT_TYPE)
#if ((PORT_TYPE & 0xFF00) == SLOW_PORT)
	_port_rx_slow_receive[out_exception, io_rec_state]
#else
	_port_rx_fast_receive[out_exception, io_rec_state]
#endif
#endm

// _port_rx_slow_receive
//		description: internal macro to support port_rx_receive with slow PORT_TYPE
//
#macro _port_rx_slow_receive[out_exception, io_rec_state]
.local temp $rec_csr

	ctx_arb[start_receive]								; wait for fbi signal when receive control can be read
 
    csr[read, $rec_csr, RCV_CNTL], ctx_swap				; read the rcv_ctrl register


// format of RCV_CTL is
// +-----+-----+-----+------+----+-----+-----+----+----+-----------+---+---+
// | msg |port |seq# |rxfail|err |elem2|elem1|1or2|seq | validbytes|EOP|SOP|
// |31:30|29:24|23:20|  19  | 18 |17:14|13:10| 9  | 8  |    7:2    | 1 | 0 |
// +-----+-----+-----+------+----+-----+-----+----+----+-----------+---+---+
//
	alu[--, $rec_csr, AND, 1]
	br>0[sop#], defer[1]
	alu[out_exception, 0x3, AND, $rec_csr, >>18]			; error or fail
		ld_field[io_rec_state, 0010, $rec_csr, <<8]				; byte count, eop, sop from rcv_cntl 7:0 to 15:8
	br[done#]
	sop#:
		// initialize rec_staet if SOP
		ld_field_w_clr[io_rec_state, 0010, $rec_csr, <<8]	; byte count, eop, sop from rcv_cntl 7:0 to 15:8
	done#:
.endlocal
#endm


// _port_rx_fast_receive
//		description: internal macro to support port_rx_receive with fast PORT_TYPE
//
#macro _port_rx_fast_receive[out_exception, io_rec_state]
.local $rec_csr temp_sopseq

	ctx_arb[start_receive]								; wait for fbi signal when receive control can be read
 
    csr[read, $rec_csr, RCV_CNTL], ctx_swap, defer[1]	; read the rcv_ctrl register
	immed[exception, 0]


// format of RCV_CTL is
// +-----+-----+-----+------+----+-----+-----+----+----+-----------+---+---+
// | msg |port |seq# |rxfail|err |elem2|elem1|1or2|seq | validbytes|EOP|SOP|
// |31:30|29:24|23:20|  19  | 18 |17:14|13:10| 9  | 8  |    7:2    | 1 | 0 |
// +-----+-----+-----+------+----+-----+-----+----+----+-----------+---+---+
//
	alu[--, 0x3, -, $rec_csr, >>30]						; test cancel
	br!=0[not_cancel#], defer[2]
	ld_field_w_clr[io_rec_state, 0010, $rec_csr, <<8]			; byte count, eop, sop from rcv_cntl 7:0 to 15:8
	alu[out_exception, 0x3, AND, $rec_csr, >>18]		; error or fail (but not both)
	move[out_exception, PORT_RXCANCEL]
	br[end#]
not_cancel#:
	alu[--, 1, AND, $rec_csr]							; check sop
	br=0[end#]											; if not sop only load 7:0
	alu[temp_sopseq, 0xf, AND, $rec_csr, >>20]

	// note: this is rotated to move seq# to left byte for faster merge at fast thread mailbox
	ld_field[io_rec_state, 1000, temp_sopseq, <<REC_STATE_SOPSEQ]		; if SOP also load sop seq

end#:
.endlocal
#endm


// port_rx_fail
// 		description: Increment a mib counter and otherwise behave like EOP, except
//					 that the packet is not queued. This is an indication that the
//					 port is terminating this packet and any packet data received
//					 is probably junk.
//		note: io_rec_state is presently not used, but we might want to set an indication there
//
#macro port_rx_fail(io_rec_state, in_rec_req, out_exception)
.local tempa
	immed[tempa, EXCEPTION_COUNTERS]					; 11 contexts, 16 locations each
	alu[tempa, tempa, +8,in_rec_req, <<4]					; add context id * 16
	scratch[incr, --, tempa, out_exception, 1]

	immed[tempa, TOTAL_DISCARDS]
	scratch[incr, --, tempa, 0, 1]
.endlocal
#endm


// port_rx_error
//		description: Increment exception counter
//      note: this could be expanded to read status word and decide based on the type of error whether to set discard
//
#macro port_rx_error(io_rec_state, in_rec_req, out_exception)
.local tempa
	immed[tempa, EXCEPTION_COUNTERS]					; 11 contexts, 16 locations each
	alu[tempa, tempa, +8,in_rec_req, <<4]					; add context id * 16
	scratch[incr, --, tempa, out_exception, 1]
.endlocal
#endm


// port_rx_discard
// 		description: Increment exception counter, total discards, set discard flag
//
#macro port_rx_discard(io_rec_state, in_rec_req, out_exception)
.local tempa
	immed[tempa, EXCEPTION_COUNTERS]					; 11 contexts, 16 locations each
	alu[tempa, tempa, +8,in_rec_req, <<4]					; add context id * 16
	scratch[incr, --, tempa, out_exception, 1]

	immed[tempa, TOTAL_DISCARDS]
	scratch[incr, --, tempa, 0, 1]

	alu[io_rec_state, io_rec_state, OR, 1, <</**/REC_STATE_DISCARD_BIT/**/]	; set discard
.endlocal
#endm


// port_rx_bytecount_extract
//		description: Extract last mpacket's bytecount from rec_state.
//
#macro port_rx_bytecount_extract(out_bytecount, io_rec_state)
	alu[out_bytecount, 0, +8, io_rec_state, >>REC_STATE_BYTE_COUNT]
	alu[out_bytecount, out_bytecount, AND~, 0x3, <<6]
#endm


// port_txrdy_chk
//
// Description: This macro checks for the target port to be ready by
//				testing the corresponding bit of the XMIT_RDY_LO hw register.
//				If the port is ready, then the output mpkt_tx_ok_cnt register
//				is reset to 5.  mpkt_tx_ok_cnt is a counter that is used by
//				the tfifo_validate macros to validate tfifo elements without having
//				to check xmit_rdy again.  The target port's xmit_rdy threshold
//				MUST be set to at least 5 * our max transmit burst size.  The
//				max burst size is typically 64 bytes.
//
// arguments:
//		out_abs_mpkt_tx_ok_cnt - Set to 5 if the target port is ready.  Not modified
//								 when the target port is not ready.
//		in_port_rdy_bit		   - register with bit set corresponding to the target port.
//
// size: 5 instructions
// side effects: out_abs_mpkt_tx_ok_cnt can be a gpr or an absolute register.
// see also: N/A
// example usage: 	port_txtrdy_chk[@mpkt_tx_ok_cnt, port_rdy_bit]		; check output port and reset cnt
//
#macro port_txrdy_chk(out_abs_mpkt_tx_ok_cnt, in_port_rdy_bit)
.local $_temp_xfr_reg _temp
check_port_rdy#:
	csr[read, $_temp_xfr_reg, XMIT_RDY_LO], ctx_swap
	alu[--, $_temp_xfr_reg, and, in_port_rdy_bit]		; port_rdy_bit set at startup to 1 << port_num
	br=0[endm#], defer[1]						; if port not rdy then don't reset counter
	alu[_temp, --, b, 5]						; if port is ready, then reset the 
	alu[out_abs_mpkt_tx_ok_cnt, --, b, _temp]	;  counter to 5
endm#:
.endlocal
#endm


// port_txrdy_wait
//
// Description: This macro checks for the target port to be ready by
//				testing the corresponding bit of the XMIT_RDY_LO hw register.
//				If the target port is not ready for transmit, then this macro 
//				will continue checking for port ready until it is.
//
// arguments:
//		in_port_rdy_bit - Register that has a bit set for the coresponding
//						  port that is to be waited for.  Bit 0 -> port 0,
//						  Bit 1 -> port 1, and so on.
// 
// size: 3 instructions
// side effects: If the transmit port never becomes ready, then this macro
//				 will never finish.
// see also: N/A
// example usage: port_txrdy_wait[port_rdy_bit]
//
#macro port_txrdy_wait(in_port_rdy_bit)
.local $_xmit_rdy_lo

check_port_rdy_loop#:
	csr[read, $_xmit_rdy_lo, XMIT_RDY_LO], ctx_swap
	alu[--, $_xmit_rdy_lo, and, in_port_rdy_bit]	; port_rdy_bit set at startup to 1 << port_num
	br=0[check_port_rdy_loop#]

.endlocal
#endm




#endif 	// PORT_UC