//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// rfifo.uc
// macros to access rfifo 
//
//
// system: IXP1200
// subsystem: microcode
// usage: library macros
// author: dfh	May 1, 2000
//
// revisions:
//		dhf	Aug 22, 2000	accept register or constant args, traverse xbufs


#ifndef RFIFO_UC
#define RFIFO_UC

// API:
//	rfifo_read(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)

#include "stdmac.uc"
#include "xbuf.uc"


// rfifo_read
//		description: Read from rfifo.
//		outputs:
//			out_xfer			first xfer reg of sequence to read to, array notation
//		inputs:
//			in_base_qw_addr		register or constant base 64 bit (quadword) address
//			in_qw_offset		register or constant 64 bit (quadword) address offset
//			in_qw_count			register or constant 64 bit (quadword) count
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 1-3 instr
//		example usage:
//			rfifo_read($xfer[1], base_addr, base_offset, ref_count, ctx_swap]
//
#macro	rfifo_read(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
.local _temp_addr

#if ((isnum(in_base_qw_addr))&&(!isnum(in_qw_offset)))
	#if ((in_base_qw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_qw_addr]
		_rfifo_read_addr_asis(out_xfer, _temp_addr, in_qw_offset, in_qw_count, optional_token)
	#else
		_rfifo_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
	#endif  // ((in_base_qw_addr > MAX_IMMEDIATE))

#elif ((isnum(in_qw_offset))&&(!isnum(in_base_qw_addr)))
	#if ((in_qw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_qw_offset]
		_rfifo_read_addr_asis(out_xfer, _temp_addr, in_base_qw_addr, in_qw_count, optional_token)
	#else
		_rfifo_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
	#endif  // ((in_qw_offset > MAX_IMMEDIATE))

#elif ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))
	#define_eval _addr_sum (in_qw_offset + in_base_qw_addr)
	immed32[_temp_addr, _addr_sum]
	_rfifo_read_addr_asis(out_xfer, _temp_addr, 0, in_qw_count, optional_token)

// else both addresses are GPRs
#else
	_rfifo_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)

#endif // ((isnum(in_qw_offset))&&(isnum(in_base_qw_addr)))

.endlocal	// _temp_addr
#endm


// _rfifo_read_addr_asis
// 		description: rfifo read where addresses are acceptable as is to Assembler
// 					 support macro rfifo_read
//
#macro _rfifo_read_addr_asis(out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
xbuf_find(out_xfer)		// if found, #define _xbuf_found, _cur_d_xbuf_name, _cur_d_xbuf_size
#if (isnum (in_qw_count))
	#ifdef _xbuf_found
		#define _perform_xbuf_op
	#endif
#endif
#ifdef _perform_xbuf_op
	_rfifo_read_to_xbufs[in_base_qw_addr, in_qw_offset, in_qw_count, optional_token]
	#undef _perform_xbuf_op
#elif (isnum (in_qw_count))
	r_fifo_rd[out_xfer, in_base_qw_addr, in_qw_offset, in_qw_count], optional_token
#else
	.local _temp_ind
	immed32 [_temp_ind, 0x100000]							; set bit 20 indirect refcount override bit
	alu_shf_left[--, _temp_ind, OR, in_qw_count, 16]		; insert indirect refcount
	r_fifo_rd[out_xfer, in_base_qw_addr, in_qw_offset, 1], optional_token, indirect_ref
	.endlocal //  _temp_ind
#endif	// (isnum (in_qw_count))

#endm


// _rfifo_read_to_xbufs
//		description: read to one or more(linked) xbufs. 
//					 support macro rfifo_read
//
#macro _rfifo_read_to_xbufs(in_base_qw_addr, in_qw_offset, in_qw_count, optional_token)
.local new_base_addr

#define_eval _rem_buf (_cur_xbuf_size - _cur_xbuf_index)
#define_eval _cur_lw_count (in_qw_count * 2)
#define_eval _cur_qw_count (in_qw_count)
#define_eval _cur_qw_offset (in_qw_offset)

#if (_cur_lw_count > _rem_buf)
	#define _use_new_base
#endif

#if (!isnum(in_qw_offset))
	#ifndef _use_new_base
		#define _use_new_base
	#endif
#endif

#ifdef _use_new_base
	alu[new_base_addr, in_base_qw_addr, +, in_qw_offset]
	#define_eval _cur_qw_offset (0)
#endif

#while (_cur_lw_count > _rem_buf)
	#define_eval _cur_qw_count (_rem_buf / 2)
	r_fifo_rd[_cur_xbuf_name/**/_cur_xbuf_index, new_base_addr, _cur_qw_offset, _cur_qw_count]
	#define_eval _cur_qw_offset (_cur_qw_offset + _cur_qw_count)
	#define_eval _cur_lw_count (_cur_lw_count - _rem_buf)
	#define_eval _cur_qw_count (_cur_lw_count / 2)
	
	#ifdef _cur_xbuf_next
		xbuf_param_set[_cur_xbuf_next]
		#define_eval _cur_xbuf_index 0
		#define_eval _rem_buf _cur_xbuf_size
	#else
		#error	"_rfifo_read_to_xbufs. out of xbuf range"
	#endif	//  _cur_xbuf_next
#endloop

// perform last operation
#ifdef _use_new_base
	r_fifo_rd[_cur_xbuf_name/**/_cur_xbuf_index, new_base_addr, _cur_qw_offset, _cur_qw_count], optional_token
	#undef _use_new_base
#else
	r_fifo_rd[_cur_xbuf_name/**/_cur_xbuf_index, in_base_qw_addr, _cur_qw_offset, _cur_qw_count], optional_token
#endif
.endlocal
#endm


#endif // RFIFO_UC