//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// scratch.uc
// macros to access scratch pad 
//
//
// system: IXP1200
// subsystem: utility microcode
// usage: library macros
// author: dfh	May 1, 2000
//
// revisions:
//		dfh		Aug 22, 2000	accept constant or register args, traverse xbufs


#ifndef SCRATCH_UC
#define SCRATCH_UC


// API:
//	scratch_read(out_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
//	scratch_write(in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
//	scratch_incr(in_base_lw_addr, in_lw_offset)
//	scratch_bits_set(in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
//  scratch_bits_test_and_set(out_xfer, in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
//	scratch_bits_clr(in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
//  scratch_bits_test_and_clr(out_xfer, in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)

#include "stdmac.uc"
#include "xbuf.uc"


// scratch_read
//		description: Read from scratch.
//		outputs:
//			out_xfer			first scratch xfer reg of sequence to read to, array notation
//		inputs:
//			in_base_lw_addr		register or constant base longword address
//			in_lw_offset		register or constant longword address offset
//			in_lw_count			register or constant longword count
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 1-3 instr
//		example usage:
//			scratch_read($xfer[1], base_addr, base_offset, ref_count, ctx_swap)
//
#macro	scratch_read(out_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
.local _temp_addr

#if ((isnum(in_base_lw_addr))&&(!isnum(in_lw_offset)))
	#if ((in_base_lw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_lw_addr]
		_scratch_read_addr_asis(out_xfer, _temp_addr, in_lw_offset, in_lw_count, optional_token)
	#else
		_scratch_read_addr_asis(out_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
	#endif  // ((in_base_lw_addr > MAX_IMMEDIATE))
	// ((isnum(in_base_lw_addr))&&(!isnum(in_lw_offset)))

#elif ((isnum(in_lw_offset))&&(!isnum(in_base_lw_addr)))
	#if ((in_lw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_lw_offset]
		_scratch_read_addr_asis(out_xfer, _temp_addr, in_base_lw_addr, in_lw_count, optional_token)
	#else
		_scratch_read_addr_asis(out_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
	#endif  // ((in_lw_offset > MAX_IMMEDIATE))
 	// ((isnum(in_lw_offset))&&(!isnum(in_base_lw_addr)))

#elif ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))
	#define_eval _addr_sum (in_lw_offset + in_base_lw_addr)
	immed32[_temp_addr, _addr_sum]
	_scratch_read_addr_asis(out_xfer, _temp_addr, 0, in_lw_count, optional_token)

// else both addresses are registers
#else
	_scratch_read_addr_asis(out_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)

#endif // ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))

.endlocal	// _temp_addr
#endm


// _scratch_read_addr_asis
// 		description: scratch read where addresses are acceptable as is to Assembler
// 					 support macro scratch_read
//
#macro _scratch_read_addr_asis(out_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
xbuf_find[out_xfer]		// if found, #define _xbuf_found, _cur_d_xbuf_name, _cur_d_xbuf_size
#if (isnum (in_lw_count))
	#ifdef _xbuf_found
		#define _perform_xbuf_op
	#endif
#endif
#ifdef _perform_xbuf_op
	_scratch_read_to_xbufs[in_base_lw_addr, in_lw_offset, in_lw_count, optional_token]
	#undef _perform_xbuf_op
#elif (isnum (in_lw_count))
	scratch[read, in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count], optional_token
#else
	.local _temp_ind
	immed32 [_temp_ind, 0x100000]							; set bit 20 indirect refcount override bit
	alu_shf_left[--, _temp_ind, OR, in_lw_count, 16]		; insert indirect refcount
	scratch[read, out_xfer, in_base_lw_addr, in_lw_offset, 1], optional_token, indirect_ref
	.endlocal //  _temp_ind
#endif	// (isnum (in_lw_count))

#endm


// _scratch_read_to_xbufs
//		description: read to one or more(linked) xbufs. 
//					 support macro scratch_read
//
#macro _scratch_read_to_xbufs(in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
.local new_base_addr

#define_eval _rem_buf (_cur_xbuf_size - _cur_xbuf_index)
#define_eval _cur_lw_offset (in_lw_offset)
#define_eval _cur_lw_count (in_lw_count)

#if (_cur_lw_count > _rem_buf)
	#define _use_new_base
#endif

#if (!isnum(in_lw_offset))
	#ifndef _use_new_base
		#define _use_new_base
	#endif
#endif

#ifdef _use_new_base
	alu_op[new_base_addr, in_base_lw_addr, +, in_lw_offset]
	#define_eval _cur_lw_offset (0)
#endif

#while (_cur_lw_count > _rem_buf)
	scratch[read, _cur_xbuf_name/**/_cur_xbuf_index, new_base_addr, _cur_lw_offset, _rem_buf]
    #define_eval _cur_lw_offset (_cur_lw_offset + _rem_buf)
    #define_eval _cur_lw_count (_cur_lw_count - _rem_buf)

	#ifdef _cur_xbuf_next
		xbuf_param_set[_cur_xbuf_next]
		#define_eval _cur_xbuf_index 0
		#define_eval _rem_buf _cur_xbuf_size
	#else
		#error	"_scratch_read_to_xbufs. out of xbuf range"
	#endif	//  _cur_xbuf_next
#endloop

// perform last operation
#ifdef _use_new_base
	scratch[read, _cur_xbuf_name/**/_cur_xbuf_index, new_base_addr, _cur_lw_offset, _cur_lw_count], optional_token
	#undef _use_new_base
#else
	scratch[read, _cur_xbuf_name/**/_cur_xbuf_index, in_base_lw_addr, _cur_lw_offset, _cur_lw_count], optional_token
#endif
.endlocal
#endm


// scratch_write
//		description: Write to scratch.
//		inputs:
//			in_xfer			first scratch xfer reg of sequence to write from, array notation
//			in_base_lw_addr		register or constant base longword address
//			in_lw_offset		register or constant longword address offset
//			in_lw_count			register or constant longword count
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 1-3 instr
//		example usage:
//			scratch_write($xfer[1], base_addr, base_offset, ref_count, ctx_swap)
//
#macro	scratch_write(in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
.local _temp_addr

#if ((isnum(in_base_lw_addr))&&(!isnum(in_lw_offset)))
	#if ((in_base_lw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_lw_addr]
		_scratch_write_addr_asis(in_xfer, _temp_addr, in_lw_offset, in_lw_count, optional_token)
	#else
		_scratch_write_addr_asis(in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
	#endif  // ((in_base_lw_addr > MAX_IMMEDIATE))
 	// ((isnum(in_base_lw_addr))&&(!isnum(in_lw_offset)))

#elif ((isnum(in_lw_offset))&&(!isnum(in_base_lw_addr)))
	#if ((in_lw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_lw_offset]
		_scratch_write_addr_asis(in_xfer, _temp_addr, in_base_lw_addr, in_lw_count, optional_token)
	#else
		_scratch_write_addr_asis(in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
	#endif  // ((in_lw_offset > MAX_IMMEDIATE))
 	// ((isnum(in_lw_offset))&&(!isnum(in_base_lw_addr)))

#elif ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))
	#define_eval _addr_sum (in_lw_offset + in_base_lw_addr)
	immed32[_temp_addr, _addr_sum]
	_scratch_write_addr_asis(in_xfer, _temp_addr, 0, in_lw_count, optional_token)

// else both addresses are registers
#else
	_scratch_write_addr_asis(in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)

#endif // ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))

.endlocal	// _temp_addr
#endm


// _scratch_write_addr_asis
// 		description: scratch write where addresses are acceptable as is to Assembler
// 					 support macro scratch_write
#macro _scratch_write_addr_asis(in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
xbuf_find[in_xfer]		// if found, #define _xbuf_found, _cur_d_xbuf_name, _cur_d_xbuf_size
#if (isnum (in_lw_count))
	#ifdef _xbuf_found
		#define _perform_xbuf_op
	#endif
#endif
#ifdef _perform_xbuf_op
	_scratch_write_from_xbufs[in_base_lw_addr, in_lw_offset, in_lw_count, optional_token]
	#undef _perform_xbuf_op
#elif (isnum (in_lw_count))
	scratch[write, in_xfer, in_base_lw_addr, in_lw_offset, in_lw_count], optional_token
#else
	.local _temp_ind
	immed32 [_temp_ind, 0x100000]							; set bit 20 indirect refcount override bit
	alu_shf_left[--, _temp_ind, OR, in_lw_count, 16]		; insert indirect refcount
	scratch[write, in_xfer, in_base_lw_addr, in_lw_offset, 1], optional_token, indirect_ref
	.endlocal //  _temp_ind
#endif	// (isnum (in_lw_count))

#endm


// _scratch_write_from_xbufs
//		description: write multiple xbufs. 
//					 support macro scratch_write
//		
#macro _scratch_write_from_xbufs(in_base_lw_addr, in_lw_offset, in_lw_count, optional_token)
.local new_base_addr

#define_eval _rem_buf (_cur_xbuf_size - _cur_xbuf_index)
#define_eval _cur_lw_offset (in_lw_offset)
#define_eval _cur_lw_count (in_lw_count)

#if (_cur_lw_count > _rem_buf)
	#define _use_new_base
#endif

#if (!isnum(in_lw_offset))
	#ifndef _use_new_base
		#define _use_new_base
	#endif
#endif

#ifdef _use_new_base
	alu_op[new_base_addr, in_base_lw_addr, +, in_lw_offset]
	#define_eval _cur_lw_offset (0)
#endif

#while (_cur_lw_count > _rem_buf)
	scratch[write, _cur_xbuf_name/**/_cur_xbuf_index, new_base_addr, _cur_lw_offset, _rem_buf]
    #define_eval _cur_lw_offset (_cur_lw_offset + _rem_buf)
    #define_eval _cur_lw_count (_cur_lw_count - _rem_buf)

	#ifdef _cur_xbuf_next
		xbuf_param_set[_cur_xbuf_next]
		#define_eval _cur_xbuf_index 0
		#define_eval _rem_buf _cur_xbuf_size
	#else
		#error	"_scratch_write_to_xbufs. out of xbuf range"
	#endif	//  _cur_xbuf_next
#endloop

// perform last operation
#ifdef _use_new_base
	scratch[write, _cur_xbuf_name/**/_cur_xbuf_index, new_base_addr, _cur_lw_offset, _cur_lw_count], optional_token
	#undef _use_new_base
#else
	scratch[write, _cur_xbuf_name/**/_cur_xbuf_index, in_base_lw_addr, in_lw_offset, _cur_lw_count], optional_token
#endif
.endlocal
#endm


// scratch_incr
//		description: Write to scratch.
//		inputs:
//			in_base_lw_addr		base longword address
//			in_lw_offset		longword offset
//		size: 1-2 instr
//		example usage:
//			scratch_incr(base_addr, base_offset)
//
#macro	scratch_incr[in_base_lw_addr, in_lw_offset]
.local _temp_addr
#if ((isnum(in_base_lw_addr))&&(!isnum(in_lw_offset)))
	#if ((in_base_lw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_lw_addr]
		scratch[incr, --, _temp_addr, in_lw_offset, 1]
	#else
		scratch[incr, --, in_base_lw_addr, in_lw_offset, 1]
	#endif  // ((in_base_lw_addr > MAX_IMMEDIATE))
 	// ((isnum(in_base_lw_addr))&&(!isnum(in_lw_offset)))

#elif ((isnum(in_lw_offset))&&(!isnum(in_base_lw_addr)))
	#if ((in_lw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_lw_offset]
		scratch[incr, --, _temp_addr, in_base_lw_addr, 1]
	#else
		scratch[incr, --, in_base_lw_addr, in_lw_offset, 1]
	#endif  // ((in_lw_offset > MAX_IMMEDIATE))
 	// ((isnum(in_lw_offset))&&(!isnum(in_base_lw_addr)))

#elif ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))
	#define_eval _addr_sum (in_lw_offset + in_base_lw_addr)
	immed32[_temp_addr, _addr_sum]
	scratch[incr, --, _temp_addr, 0, 1]

// else both addresses fit
#else
	scratch[incr, --, in_base_lw_addr, in_lw_offset, 1]

#endif // ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))

.endlocal // _temp_addr
#endm


// scratch_bits_set
//		description: Set in_vector bits shifted left by in_start_bit_pos at scratch.
//		inputs:
//			in_base_lw_addr		register or constant base longword address
//			in_lw_offset		register or constant longword address offset
//			in_start_bit_pos	register or constant<32, starting bit position
//								in_vector is shifted left by this amount.
//			in_vector			register or constant, mask of bits to set							
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 2-6 instr
//		xfer regs used:
//			1
//		example usage:
//			scratch_bits_set(0x111, 4, 0x12, 0x16, ctx_swap)
//
#macro scratch_bits_set(in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
xbuf_alloc($bset_vector, 1);
_scratch_bits_op(set_bits, $bset_vector[0], in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
xbuf_free($bset_vector);
#endm


// scratch_bits_test_and_set
//		description: Set in_vector bits shifted left by in_start_bit_pos at scratch.
//					 Read contents of scratch address prior to the write.
// 		outs:
//			out_xfer			read xfer register result
//		inputs:
//			in_base_lw_addr		register or constant base longword address
//			in_lw_offset		register or constant longword address offset
//			in_start_bit_pos	register or constant<32, starting bit position
//								in_vector is shifted left by this amount.
//			in_vector			register or constant, mask of bits to set							
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 2-6 instr
//		xfer regs used:
//			1
//		example usage:
//			scratch_bits_test_and_set(0x111, 0, bit_position, ctx_swap);
//
//
#macro scratch_bits_test_and_set(out_xfer, in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
_scratch_bits_op(test_and_set_bits, out_xfer, in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
#endm


// scratch_bits_clr
//		description: Clear in_vector bits shifted left by in_start_bit_pos at scratch.
//		inputs:
//			in_base_lw_addr		register or constant base longword address
//			in_lw_offset		register or constant longword address offset
//			in_start_bit_pos	register or constant<32, starting bit position
//								in_vector is shifted left by this amount.
//			in_vector			register or constant, mask of bits to set							
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 2-6 instr
//		xfer regs used:
//			1
//		example usage:
//			scratch_bits_clr(0x111, 0, bit_position,0x12, ctx_swap);
//
//
#macro scratch_bits_clr(in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
xbuf_alloc($bset_vector, 1);
_scratch_bits_op(clear_bits, $bset_vector[0], in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
xbuf_free($bset_vector);
#endm


// scratch_bits_test_and_clr
//		description: Clear in_vector bits shifted left by in_start_bit_pos at scratch.
//					 Read contents of scratch address prior to the write.
// 		outs:
//			out_xfer			read xfer register result
//		inputs:
//			in_base_lw_addr		register or constant base longword address
//			in_lw_offset		register or constant longword address offset
//			in_start_bit_pos	register or constant<32, starting bit position
//								in_vector is shifted left by this amount.
//			in_vector			register or constant, mask of bits to set							
//			optional_token		ctx_swap:  waiting on completion signal, context swap
//								sig_done:  requesting signal when done, continuing execution
//								no_signal or no_option: not requesting signal, continuing
//		size: 2-6 instr
//		xfer regs used:
//			1
//		example usage:
//			scratch_bits_test_and_clr(0x111, 0, bit_position, 0x12, ctx_swap);
//
//
#macro scratch_bits_test_and_clr(out_xfer, in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
_scratch_bits_op(test_and_clear_bits, out_xfer, in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)
#endm


// _scratch_bits_op
//		description: support macros scratch_bits_set, scratch_bits_clr
//
#macro _scratch_bits_op(operation, io_xfer, in_base_lw_addr, in_lw_offset, in_start_bit_pos, in_vector, optional_token)

.local _temp_shift _temp_addr
#if (!isnum(in_start_bit_pos))
	alu[_temp_shift, in_start_bit_pos, B-A, 1, <<5]	; invert shift amount (32 - output_port) for left indirect
	alu[--, _temp_shift, B, 0]					; feed to A operand for indirect shift
	alu_shf[io_xfer, --, B, in_vector, <<indirect]
#else
	#define_eval _SHIFT_AMT (in_start_bit_pos)
	shf_left[io_xfer, in_vector, _SHIFT_AMT]
#endif

#if ((isnum(in_base_lw_addr))&&(!isnum(in_lw_offset)))
	#if ((in_base_lw_addr > MAX_IMMEDIATE))
		immed32[_temp_addr, in_base_lw_addr]
		scratch [bit_wr, io_xfer, _temp_addr, in_lw_offset, operation], optional_token
	#else
		scratch [bit_wr, io_xfer, in_base_lw_addr, in_lw_offset, operation], optional_token
	#endif  // ((in_base_lw_addr > MAX_IMMEDIATE))

#elif ((isnum(in_lw_offset))&&(!isnum(in_base_lw_addr)))
	#if ((in_lw_offset > MAX_IMMEDIATE))
		immed32[_temp_addr, in_lw_offset]
		scratch [bit_wr, io_xfer, _temp_addr, in_base_lw_addr, operation], optional_token
	#else
		scratch [bit_wr, io_xfer, in_base_lw_addr, in_lw_offset, operation], optional_token
	#endif  // ((in_lw_offset > MAX_IMMEDIATE))

#elif ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))
	#define_eval _addr_sum (in_lw_offset + in_base_lw_addr)
	immed32[_temp_addr, _addr_sum]
	scratch [bit_wr, io_xfer, _temp_addr, 0, operation], optional_token

// else both addresses are GPRs
#else
	scratch [bit_wr, io_xfer, in_base_lw_addr, in_lw_offset, operation], optional_token

#endif // ((isnum(in_lw_offset))&&(isnum(in_base_lw_addr)))

.endlocal
#endm



#endif // SCRATCH_UC