//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// sig.uc
// interthread signal macros 
//
//
// system: IXP1200
// subsystem: utility microcode
// usage: library macros
// author: dfh	May 1, 2000
//
// revisions:


#ifndef SIG_UC
#define SIG_UC


// API:

//	sig_post(in_sig_type, in_sig_id)
//	sig_ifctx_post(ctx, in_sig_type, in_sig_id)
//	sig_block(in_sig_type, in_sig_id)
//	sig_ifctx_block(ctx, in_sig_type, in_sig_id)
//	sig_wait(in_sig_type, in_sig_id)
//	sig_ifctx_wait(ctx, in_sig_type, in_sig_id)



// sig_post
//		description: signal another microengine thread.
//		inputs:
//			in_sig_handle:
//				in_sig_type		IXP1200 forced to be inter_thread	
//				in_sig_id		constant or register inter_thread signal id
//
#macro sig_post(in_sig_type, in_sig_id)
#if(isnum(in_sig_id))
	fast_wr[in_sig_id, inter_thd_sig]
#else
.local sig_post_temp
	shf_left[sig_post_temp, in_sig_id, 5]
	alu_shf_left[sig_post_temp, sig_post_temp, OR, 1, 15]
	fast_wr[1, inter_thd_sig], indirect_ref
.endlocal	
#endif
#endm


// sig_ifctx_post
//		description: perform sig_post if executing context is IN_CTX
//		inputs:
//			IN_CTX				constant context id, 0, 1, 2, or 3
//			in_sig_handle:
//				in_sig_type		must be inter_thread	
//				in_sig_id		constant or register inter_thread signal id
//
#macro sig_ifctx_post(IN_CTX, in_sig_type, in_sig_id)
br!=ctx[IN_CTX, end#]
	sig_post[in_sig_type, in_sig_id]
end#:
#endm


// sig_wait
// 		description: swap out. swap back in when one or more signal received.
//		inputs:
//			in_sig_handle:
//				in_sig_type		token inter_thread, sram, sdram, fbi
//				in_sig_op		operation performed on selected signals
//								if evaluates to 1, thread can wake up
//								wake thread if
//								or: if ((in_sig_vector AND signals) > 0), wake thread
//								and: if ((in_sig_vector AND signals) == in_sig_vector)
//								ignored on IXP1200
//				in_sig_vector	constant or register vector of signals  to select
//								ignored on IXP1200
//
//		example:
//				sig_wait(inter_thread, OR, 0xff);
//
#macro sig_wait(in_sig_type, in_sig_op, in_sig_vector)
	ctx_arb[in_sig_type]
#endm


// sig_ifctx_wait
// 		description: perform sig_wait if executing context is IN_CTX
//		inputs:
//			IN_CTX				constant context id, 0, 1, 2, or 3
//			in_sig_handle:
//				in_sig_type		inter_thread, sram, sdram, fbi	
//				in_sig_op		operation performed on selected signals
//								if evaluates to 1, thread can wake up
//								wake thread if
//								or: if ((in_sig_vector AND signals) > 0), wake thread
//								and: if ((in_sig_vector AND signals) == in_sig_vector)
//								ignored on IXP1200
//				in_sig_vector	constant or register vector of signals  to select
//								ignored on IXP1200
//
//		example:
//				sig_ifctx_wait(0, inter_thread, OR, 0xff);
//
#macro sig_ifctx_wait(IN_CTX, in_sig_type, in_sig_op, in_sig_vector)
br!=ctx[IN_CTX, end#]
	sig_wait[in_sig_type, in_sig_op, in_sig_vector]
end#:
#endm


// sig_block
// 		description: block this microengine's further execution until signal is received.
//		inputs:
//			in_sig_handle:
//				in_sig_type		token inter_thread, sram, sdram, fbi	
//				in_sig_op		operation performed on selected signals
//								if evaluates to 1, thread can wake up
//								wake thread if
//								or: if ((in_sig_vector AND signals) > 0), wake thread
//								and: if ((in_sig_vector AND signals) == in_sig_vector)
//								ignored on IXP1200
//				in_sig_vector	constant or register vector of signals  to select
//								ignored on IXP1200
//
//		example:
//				sig_block(inter_thread, OR, 0xff);
//
#macro sig_block(in_sig_type, in_sig_op, in_sig_vector)
block#:
	br_!signal[in_sig_type, block#]  		; loop until signal is received
#endm


// sig_ifctx_block
// 		description: perform sig_block if executing context is IN_CTX
//		inputs:
//			IN_CTX				constant context id, 0, 1, 2, or 3
//			in_sig_handle:
//				in_sig_type		inter_thread, sram, sdram, fbi	
//				in_sig_op		operation performed on selected signals
//								if evaluates to 1, thread can wake up
//								wake thread if
//								or: if ((in_sig_vector AND signals) > 0), wake thread
//								and: if ((in_sig_vector AND signals) == in_sig_vector)
//								ignored on IXP1200
//				in_sig_vector	constant or register vector of signals  to select
//								ignored on IXP1200
//
//		example:
//				sig_ifctx_block(0, inter_thread, OR, 0xff);
//
//				in_sig_id		signal id. for IXP1200 ignored
//
#macro sig_ifctx_block(IN_CTX, in_sig_type, in_sig_op, in_sig_vector)
br!=ctx[IN_CTX, end#]
	sig_block[in_sig_type, in_sig_op, in_sig_vector]
end#:
#endm


#endif // SIG_UC