//---------------------------------------------------------------------------
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
//---------------------------------------------------------------------------
// stdmac.uc
// standard macros
//
//
// system: IXP1200
// subsystem: common low-level microcode
// usage: library macros
// author: dfh Jan 12, 1999
// revisions:
//		dfh April 28, 2000 add alu macros, move other macros to individual files
//      bkh July 31, 2000 add, add_c, sub, alu_op, alu_shf_right and alu_shf_left
//						 have been modified to support negative constant .
//						 bits_set_ind and bits_clr_ind have been modified to support constant
//					     

#ifndef STDMAC_UC
#define STDMAC_UC

#include "constants.uc"

// API:
//		immed32(out_dst, VAL)
// 		balr(in_link_pc, target_label)
//		move(out_dst, in_src)
//		add(out_dst, in_src_a, in_src_b)
//		add_c(out_dst, in_src_a, in_src_b)
//		sub(out_dst, in_src_a, in_src_b)
//		alu_op(out_dst, in_src_a, op_spec, in_src_b)
//		shf_right(out_dst, in_src, in_shift_amt)
//		shf_left(out_dst, in_src, in_shift_amt)		
//		rot_right(out_dst, in_src, in_shift_amt)
//		rot_left(out_dst, in_src, in_shift_amt)		
//		alu_shf_right(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
//		add_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
//		sub_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
//		and_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
//		or_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
//		alu_shf_left(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
//		add_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
//		sub_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
//		and_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
//		or_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
//		alu_rot_right(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
//		alu_rot_left(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
//		bits_set(out_reg, in_start_bit_pos, in_vector)
//		bits_clr(out_reg, in_start_bit_pos, in_vector)
//		thread_id(out_tid, UENG_ID)
//		multiply(output, in_A, in_B)
//		divide(output, divisor, DIVIDEND)
//		addr_from_index(out_address, in_index, ELEMENT_SIZE, BASE_ADDRESS)
//		index_from_addr(out_index, in_address, ELEMENT_SIZE, BASE_ADDRESS)


// immed32
//		description:load out_dst with 32 bit constant
//					if constant fits in 16 bits, expand to a single immed 
//					if constant doesn't, expand into a immed_w0 and immed_w1
//					if it is a name (not a number), assume that it is being IMPORTED
//		outputs:
//			out_dst	GPR. destination register. 
//		inputs:
//			VAL		constant
//		size: 1-2 instructions
//		example: immed32(output, 0x12345678);
//
#macro immed32(out_dst, VAL)
.local immed32_temp
#if (!isnum(VAL))
	#if (strstr(out_dst, $))	; check for output xfer reg
		immed_w0[immed32_temp, VAL]
		immed_w1[immed32_temp, VAL>>16]
		alu[out_dst, --, b, immed32_temp]
	#else 	; !strstr(out_dst, $)
		immed_w0[out_dst, VAL]
		immed_w1[out_dst, VAL>>16]
	#endif	; strstr(out_dst, $)
#elif (((VAL & 0xFFFF0000) == 0) || ((VAL & 0xFFFF0000) == 0xFFFF0000))
    immed[out_dst,VAL]
#elif (((VAL & 0xFF0000FF) == 0) || ((VAL & 0xFF0000FF) == 0xFF0000FF))
    immed[out_dst, (VAL >> 8), <<8]
#elif ((VAL & 0x0000FFFF) == 0)
    /* make sure that number is not seen as negative */
    immed[out_dst, (0x0000FFFF & (VAL >> 16)), <<16]
#elif ((VAL & 0x0000FFFF) == 0x0000FFFF)
    /* make sure that number is seen as negative */
    immed[out_dst, (0xFFFF0000 | (VAL >> 16)), <<16]
#else	; isnum(VAL)
	#if (strstr(out_dst, $))	; check for output xfer reg
		immed_w0[immed32_temp, (VAL & 0xFFFF)]
		immed_w1[immed32_temp, ((VAL >> 16) & 0xFFFF)]
		alu[out_dst, --, b, immed32_temp]
	#else	; !(strstr(out_dst, $))
		immed_w0[out_dst, (VAL & 0xFFFF)]
		immed_w1[out_dst, ((VAL >> 16) & 0xFFFF)]
	#endif	; (strstr(out_dst, $))
#endif	; (!isnum(VAL))
.endlocal
#endm


// balr
//	 	description:  Save the current program counter in gpr in_link_pc and branch to
//					  target_label.  Program control can be returned to
//					  the current PC the current location using the rtn(in_link_pc) 
//					  instruction.
//		inputs:
//			in_link_pc		GPR. to hold next pc for use by rtn instruction
//			target_label	subroutine start
//		size: 2 instructions
//		example: balr(next, target);
//
#macro balr(in_link_pc, target_label)
    br [target_label], defer[1]
    load_addr[in_link_pc, ret_lab#]
ret_lab#:
#endm


// move
//		description: move the source value into the destination register
//		outputs:
//			out_dst	destination GPR or write xfer
//		inputs:
//			in_src	source GPR, read xfer or constant
//		size: 1-3 instructions
//		example: move(output, input);
//
#macro move(out_dst, in_src)
.local move_temp

#if (isnum(in_src)) //&& ((in_src > MAX_IMMEDIATE) || (in_src < 0)))
	#if ((in_src > MAX_IMMEDIATE) || (in_src < 0))
		immed32[out_dst, in_src]
	#else
     	alu[out_dst, --, b, in_src]		; in_src > 31
	#endif	; in_src > 31
#else	;  ! isnum(in_src) && ((in_src > MAX_IMMEDIATE) || (in_src < 0)))
    alu[out_dst, --, b, in_src]		; in_src > 31
#endif	; (isnum(in_src) && ((in_src > MAX_IMMEDIATE) || (in_src < 0)))

.endlocal
#endm


// alu_op
//	 	description: perform alu operation in_src_a op_spec in_src_b
//	 	outputs:
//			out_dst		GPR
//		inputs:
//			in_src_a	register or constant	
//			op_spec		ALU operator
//						Legal operators are:
//							B 
//							~B 
//							+ 
//							+carry 
//							+4 
//							+8 
//							+16
//							- 
//							B-A 
//							AND 
//							~AND 
//							AND~ 
//							OR 
//							XOR 
//							+IFsign
//			in_src_b	register or constant	
//		size: 1-5 instructions
//		example: alu_op(output, 0x12345678, +,  10);
//
#macro alu_op(out_dst, in_src_a, op_spec, in_src_b)
.local _tempa _tempb
#if ((isnum(in_src_b))&&(isnum(in_src_a))) //in_src_b and in_src_a are constant
#if ((in_src_b > MAX_IMMEDIATE)||(in_src_b<0))
	immed32[_tempb, in_src_b]
#if ((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
	immed32[_tempa,in_src_a]
	alu[out_dst, _tempa, op_spec, _tempb]
#else 				// in_src_a < 32	
	alu[out_dst, in_src_a, op_spec, _tempb]
#endif					
#else 				// in_src_b < 32
	immed32[_tempa,in_src_a]
	alu[out_dst, _tempa, op_spec, in_src_b]
#endif					
#elif (isnum(in_src_b))		//in_src_b is constant
#if ((in_src_b > MAX_IMMEDIATE)||(in_src_b<0))
	immed32[_tempb, in_src_b]
	alu[out_dst, in_src_a, op_spec, _tempb]
#else
	alu[out_dst, in_src_a, op_spec, in_src_b]
#endif
#elif (isnum(in_src_a)) //in_src_a is constant
#if((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
	immed32[_tempa, in_src_a]
	alu[out_dst, _tempa, op_spec, in_src_b]
#else
	alu[out_dst, in_src_a, op_spec, in_src_b]
#endif
#else
	alu[out_dst, in_src_a, op_spec, in_src_b]
#endif
.endlocal
#endm

// add
//		description: 32 bit add in_src_a + in_src_b 
//	 	outputs:
//			out_dst		GPR
//		inputs:
//			in_src_a	register or constant	
//			in_src_b	register or constant	
//		size: 1-5 instructions
//		example: add(output, 0x1234, 0x12345678);
//
#macro add(out_dst, in_src_a, in_src_b)
alu_op[out_dst, in_src_a, +, in_src_b]
#endm


// add_c
// 		description: 32 bit add in_src_a + in_src_b + previous carry
//	 	outputs:
//			out_dst		GPR
//		inputs:
//			in_src_a	register or constant	
//			in_src_b	register or constant	
//		size: 1-5 instructions
//		example: add_c(output, source_a, 0x12345678);
//
#macro add_c(out_dst, in_src_a, in_src_b)
alu_op[out_dst, in_src_a, +carry, in_src_b]
#endm


// sub
//	 	description: subtract in_src_b from in_src_a
//	 	outputs:
//			out_dst		GPR
//		inputs:
//			in_src_a	register or constant	
//			in_src_b	register or constant	
//		size: 1-3 instructions
//		example: sub(output, 0x12345678, source);
//
#macro sub(out_dst, in_src_a, in_src_b)
alu_op[out_dst, in_src_a, -, in_src_b]
#endm


// shf_right
//		description: shift right in_src by in_shift_amt bit positions
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src			GPR	
//			in_shift_amt	register or constant	(0-31)
//		size: 1-2 instructions
//		example: shf_right(output, input, 3);
//
#macro shf_right(out_dst, in_src, in_shift_amt)
.local shf_right_temp_src shf_right_temp_shf
#if (isnum(in_src))
	#if ((in_src > MAX_IMMEDIATE)||(in_src<0))
		#define _use_temp_src
		immed32[shf_right_temp_src, in_src]
	#endif
#endif
#if (isnum(in_shift_amt))
	#define_eval SHF_AMT in_shift_amt
	#ifdef _use_temp_src
		alu[out_dst, --, B, shf_right_temp_src, >>/**/SHF_AMT/**/]
		#undef _use_temp_src
	#else
		alu[out_dst, --, B, in_src, >>/**/SHF_AMT/**/]
	#endif
#else
	alu[--, in_shift_amt, B, 0]
	#ifdef _use_temp_src
		alu[out_dst, --, B, shf_right_temp_src, >>indirect]
		#undef _use_temp_src
	#else
		alu[out_dst, --, B, in_src, >>indirect]
	#endif
#endif
.endlocal
#endm


// shf_left
//		description: shift left in_src by in_shift_amt bit positions
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src			GPR	
//			in_shift_amt	register or constant	(0-31)
//		size: 1-4 instructions
//		example: shf_left(output, input, reg);
//
#macro shf_left(out_dst, in_src, in_shift_amt)
.local shf_left_temp_src shf_left_temp_shf
#if (isnum(in_src))
	#if ((in_src > MAX_IMMEDIATE)||(in_src<0))
		#define _use_temp_src
		immed32[shf_left_temp_src, in_src]
	#endif
#endif
#if (isnum(in_shift_amt))
	#define_eval SHF_AMT in_shift_amt
	#ifdef _use_temp_src
		alu[out_dst, --, B, shf_left_temp_src, <</**/SHF_AMT/**/]
		#undef _use_temp_src
	#else
		alu[out_dst, --, B, in_src, <</**/SHF_AMT/**/]
	#endif
#else
	alu[shf_left_temp_shf, in_shift_amt, B-A, 1, <<5]
	alu[--, shf_left_temp_shf, B, 0]
	#ifdef _use_temp_src
		alu[out_dst, --, B, shf_left_temp_src, <<indirect]
		#undef _use_temp_src
	#else
		alu[out_dst, --, B, in_src, <<indirect]
	#endif
#endif
.endlocal
#endm


// rot_right
//		description: shift right in_src by in_shift_amt bit positions
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src			GPR	
//			in_shift_amt	constant	(0-31)
//		size: 1 instructions
//		example: rot_right(output, input, 3);
//
#macro rot_right(out_dst, in_src, in_shift_amt)
#define_eval SHF_AMT in_shift_amt
	alu[out_dst, --, B, in_src, >>rot /**/SHF_AMT/**/] 
#endm


// rot_left
//		description: shift left in_src by in_shift_amt bit positions
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src			GPR	
//			in_shift_amt	constant	(0-31)
//		size: 1 instruction
//		example: rot_left(output, input, 3);
//
#macro rot_left(out_dst, in_src, in_shift_amt)
#define_eval SHF_AMT in_shift_amt
	alu[out_dst, --, B, in_src, <<rot /**/SHF_AMT/**/] 
#endm


// alu_shf_right
//		description: shift right in_src_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src_a		register or constant	
//			op_spec			alu operation
//			in_src_b		GPR	
//			in_shift_amt	register or constant	(0-31)
//		size: 2-4 instructions
//		example: alu_shf_right(output, 0xABCD1234, +, input, 3);
//

#macro alu_shf_right(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
.local _tempa
	#if (isnum(in_shift_amt))
		#define_eval SHF_AMT in_shift_amt
		#if (isnum(in_src_a))
			#if ((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
				immed32[_tempa, in_src_a]
				alu[out_dst, _tempa, op_spec, in_src_b, >>/**/SHF_AMT/**/]
			#else
				alu[out_dst, in_src_a, op_spec, in_src_b, >>/**/SHF_AMT/**/]
			#endif
		#else
			alu[out_dst, in_src_a, op_spec, in_src_b, >>/**/SHF_AMT/**/]
		#endif 
	#else		// not isnum in_shift_amt
		alu[--, in_shift_amt, B, 0]
		#if (isnum(in_src_a)) 
			#if ((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
				immed32[_tempa, in_src_a]
				alu[--, in_shift_amt, B, 0]
				alu[out_dst, _tempa, op_spec, in_src_b, >>indirect]
			#else
				alu[out_dst, in_src_a, op_spec, in_src_b, >>indirect]
			#endif
		#else
			alu[out_dst, in_src_a, op_spec, in_src_b, >>indirect]
		#endif
	#endif
.endlocal	
#endm


// commonly used alu ops
#macro add_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_right[out_dst, in_src_a, +, in_src_b, in_shift_amt]
#endm
#macro sub_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_right[out_dst, in_src_a, -, in_src_b, in_shift_amt]
#endm
#macro and_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_right[out_dst, in_src_a, AND, in_src_b, in_shift_amt]
#endm
#macro or_shf_right(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_right[out_dst, in_src_a, OR, in_src_b, in_shift_amt]
#endm




// alu_shf_left
//		description: shift left in_src_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src_a		register or constant	
//			op_spec			alu operation
//			in_src_b		GPR	
//			in_shift_amt	register or constant	(0-31)
//		size: 2-4 instructions
//		example: alu_shf_left(output, 0xABCD1234, +, input, 3);
//
#macro alu_shf_left(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
.local _tempa _temp
	#if (isnum(in_shift_amt))
		#define_eval SHF_AMT in_shift_amt
		#if (isnum(in_src_a))
			#if ((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
				immed32[_tempa, in_src_a]
				alu[out_dst, _tempa, op_spec, in_src_b, <</**/SHF_AMT/**/]
			#else		// not (in_src_a > 31)
				alu[out_dst, in_src_a, op_spec, in_src_b, <</**/SHF_AMT/**/]
			#endif		// not (in_src_a > 31)
		#else		// not (isnum(in_src_a))
			alu[out_dst, in_src_a, op_spec, in_src_b, <</**/SHF_AMT/**/]
		#endif 		// not (isnum(in_src_a))
	#else		// not (isnum(in_shift_amt))
		#if (isnum(in_src_a))
			#if ((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
				immed32[_tempa, in_src_a]
				alu[_temp, in_shift_amt, B-A, 1, <<5]
				alu[--, _temp, B, 0]
				alu[out_dst, _tempa, op_spec, in_src_b, <<indirect]
			#else
				alu[_temp, in_shift_amt, B-A, 1, <<5]
				alu[--, _temp, B, 0]
				alu[out_dst, in_src_a, op_spec, in_src_b, <<indirect]
			#endif
		#else
			alu[_temp, in_shift_amt, B-A, 1, <<5]
			alu[--, _temp, B, 0]
			alu[out_dst, in_src_a, op_spec, in_src_b, <<indirect]
		#endif
	#endif
.endlocal
#endm


// commonly used alu ops
#macro add_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_left[out_dst, in_src_a, +, in_src_b, in_shift_amt]
#endm
#macro sub_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_left[out_dst, in_src_a, -, in_src_b, in_shift_amt]
#endm
#macro and_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_left[out_dst, in_src_a, AND, in_src_b, in_shift_amt]
#endm
#macro or_shf_left(out_dst, in_src_a, in_src_b, in_shift_amt)
	alu_shf_left[out_dst, in_src_a, OR, in_src_b, in_shift_amt]
#endm


// alu_rot_right
//		description: rotate right in_src_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src_a		register or constant	
//			op_spec			alu operation
//			in_src_b		GPR	
//			in_shift_amt	constant	(0-31)
//		size: 1-3 instructions
//		example: alu_rot_right(output, input2, +, input, 4);
//						when: input2=5, input=0x12345678
//						results in output=0x8123456c
//
#macro alu_rot_right(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
#define_eval SHF_AMT in_shift_amt
.local _tempa
	#if (isnum(in_src_a))
		#if ((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
			immed32[_tempa, in_src_a]
			alu[out_dst, _tempa, op_spec, in_src_b, >>rot /**/SHF_AMT/**/]
		#else
			alu[out_dst, in_src_a, op_spec, in_src_b, >>rot /**/SHF_AMT/**/]
		#endif
	#else
		alu[out_dst, in_src_a, op_spec, in_src_b, >>rot /**/SHF_AMT/**/]
	#endif 
.endlocal		// _temp
#endm


// alu_rot_left
//		description: rotate left in_src_b by in_shift_amt bit positions, then perform
//					 operation op_spec against_src_a and shift result
//	 	outputs:
//			out_dst			GPR
//		inputs:
//			in_src_a		register or constant	
//			op_spec			alu operation
//			in_src_b		GPR	
//			in_shift_amt	constant	(0-31)
//		size: 1-3 instructions
//		example: alu_rot_left(output, input2, +, input, 3);
//
#macro alu_rot_left(out_dst, in_src_a, op_spec, in_src_b, in_shift_amt)
.local _tempa
#define_eval SHF_AMT in_shift_amt
	#if (isnum(in_src_a))
		#if ((in_src_a > MAX_IMMEDIATE)||(in_src_a<0))
			immed32[_tempa, in_src_a]
			alu[out_dst, _tempa, op_spec, in_src_b, <<rot /**/SHF_AMT/**/]
		#else
			alu[out_dst, in_src_a, op_spec, in_src_b, <<rot /**/SHF_AMT/**/]
		#endif
	#else
		alu[out_dst, in_src_a, op_spec, in_src_b, <<rot /**/SHF_AMT/**/]
	#endif 
.endlocal		// _temp
#endm


// bits_set
//		description:		set bits indicated by mask at starting position in_start_bit_pos
//		output/input:
//			io_reg				register to modify
//		inputs:
//			in_start_bit_pos	register or constant<32, starting bit position. 
//								in_vector is shifted left by this amount.
//			in_vector			register or constant, mask of bits to clear							
//	 	size: 3-5 instructions
//		example: bits_set(reg2,5,0xff);
//
#macro bits_set(io_reg, in_start_bit_pos, in_vector)
.local _temp
#if (isnum(in_start_bit_pos))
	immed32[_temp,in_start_bit_pos]
	alu[_temp, _temp, B-A, 1, <<5]
#else
	alu[_temp, in_start_bit_pos, B-A, 1, <<5]
#endif
	alu[--, _temp, B, 0]
	alu_shf[io_reg, io_reg, OR, in_vector, <<indirect]
.endlocal
#endm


// bits_clr
//		description:		clear bits indicated by mask at starting position in_start_bit_pos
//		output/input:
//			io_reg				register to modify
//		inputs:
//			in_start_bit_pos	register or constant<32, starting bit position
//								in_vector is shifted left by this amount.
//			in_vector			register or constant, mask of bits to clear							
//		size: 3-5 instructions
//		example: bits_clr(reg2, bitpos, 0x3);
//
#macro bits_clr(io_reg, in_start_bit_pos, in_vector)
.local _temp
#if (isnum(in_start_bit_pos))
	immed32[_temp,in_start_bit_pos]
	alu[_temp, _temp, B-A, 1, <<5]
#else
	alu[_temp, in_start_bit_pos, B-A, 1, <<5]
#endif
	alu[--, _temp, B, 0]
	alu_shf[io_reg, io_reg, AND~, in_vector, <<indirect]
.endlocal
#endm


// thread_id
//		description:		given UENGINE_ID, get thread id of a context
//		output:
//		out_tid			GPR thread id
//		input:
//		UENG_ID			constant microengine identifier
//		size: 2-5 instructions
//		example:#define UENG_ID 5 
//				thread_id(ctx_id, UENG_ID);
//
#macro thread_id(out_tid, UENG_ID)
br=ctx[1, context1#]
br=ctx[2, context2#]
br=ctx[3, context3#]
context0#:
#define_eval MY_TID0 (UENG_ID<<2)
immed32[out_tid, MY_TID0]
br[end#]

context1#:
#define_eval MY_TID1 ((UENG_ID<<2)+1)
immed32[out_tid, MY_TID1]
br[end#]

context2#:	
#define_eval MY_TID2 ((UENG_ID<<2)+2)
immed32[out_tid, MY_TID2]
br[end#]

context3#:
#define_eval MY_TID3 ((UENG_ID<<2)+3)
immed32[out_tid, MY_TID3]
end#:
#endm

// multiply
//
//		Description:
//					output = mpy_in * MPY_CONST
//
//					MPY_CONST is known at compile time, so the
//					pre-processor uses its bit value to construct
//					an optimal multiply via alu/shift operations.
//
//					A simple implementation would take every
//					1-bit position n in MPY_CONST, and shift&add mpy_in
//					by n to accumulate the output.  So if there were
//					B 1-bits in MPY_CONST, this would take B instructions --
//					up to 32.
//
//					This implementation also uses subtraction, reducing the
//					maximum number of instructions to 16.
//
//					Eg. given MPY_CONST = 123 = 1111011b
//					123x = 64x + 32x + 16x + 8x + 2x + 1x
//					but also:
//					123x = 128x - 4x - 1x
//
//					
//		Outputs:
//					output		output = mpy_in * MPY_CONST
//		Inputs:
//					mpy_in		input GPR
//					MPY_CONST	any 32-bit input constant 
//
//		Performance:	n cycles: 1 to 16
//
//					Worst case performance is the number of 1 bits
//					in MPY_CONST.  0xxAAAAAAAA and 0x55555555 take
//					the maximum 16 cycles.
//
//					Actual performance depends on MPY_CONST like so:
//					1) Spans of two or more 1's cost 2 cycles.
//					1a) The exception is if there is only 1 span of 1's
//						and it is adjacent to bit 31 -- this costs 1 cycle.
//					2) isolated 0's or 1's cost 1 cycle each.
//
//	Resources:
//					memory -- no accesses, none consumed
//					registers -- no additional registers, just
//								output & input parameters.
//					instructions -- n: (n <= 16), as described above
//
//	Condition codes: data dependent -- assume they are clobbered.
//
//	Limitations:
//					mpy_in and output typically can't be the same GPR.
//					This is because add_shf_left and sub_shf_left
//					will operate on both output and mpy_in in the same cycle
//					and would result in a compile-time error.
//
//					However, mpy_in and output can be the same GPR in cases
//					where the macro expands to just a single instruction.
//					Eg. If MPY_CONST is zero or a positive or negative
//					even power of two.
//
//	Algorithm:
//			Start scanning the MPY_CONST from one end looking for the first
//			1-bit. If it is a single 1-bit, emit an add_shift and
//			continue. Otherwise, start counting 1-bits until you find two
//			or more 0-bits in a row. Then add the first of the two zero bits,
//			subtract the first 1-bit, and subtract any single 0-bits along
//			the way. 
//
#macro multiply(output, in_A, in_B)

#if (isnum(in_A) && isnum(in_B))
	// Both in_A and in_B are constant
	immed32(output, ((in_A) * (in_B)))	; pre-processor multiply

#elif (!(isnum(in_A) || isnum(in_B)))
	// Neither in_A nor in_B are constant
	#error 4 "multiply() requires 1 or 2 constant parameters."

#else
	// One of A or B is constant
	// arrange parameters for rest of macro
	#if isnum(in_A)
		// in_A is constant
		#define_eval MPY_CONST in_A
		#define_eval mpy_in in_B
	#elif isnum(in_B)
		// in_B is constant
		#define_eval MPY_CONST in_B
		#define_eval mpy_in in_A
	#endif

	#if (MPY_CONST == 0)	// multiply by 0
		immed32(output, 0) ; output = 0
	#endif

	#define MPY_FIRST_OUTPUT 1				// track first write to output word
	#define MPY_BASE_BIT 0
	#while (MPY_BASE_BIT < 32)

		// not currently inside a "span" of 1's

		// If this bit set...
		#if (MPY_CONST & (1 << MPY_BASE_BIT))

			// If this 1-bit is in position 31, or...
			// If next bit is clear, then "lone-1" -- not a span
			#define_eval MPY_NEXT_BIT (MPY_BASE_BIT + 1)
			#if ((MPY_BASE_BIT == 31) || (!(MPY_CONST & (1 << MPY_NEXT_BIT))))
				#if (MPY_FIRST_OUTPUT)
					shf_left(output, mpy_in, MPY_BASE_BIT) ; output = (mpy_in * 2^MPY_BASE_BIT)
					#define_eval MPY_FIRST_OUTPUT 0
				#else
					add_shf_left(output, output, mpy_in, MPY_BASE_BIT) ; output = output + (mpy_in * 2^MPY_BASE_BIT)
				#endif

			#else
				// MPY_BASE_BIT has a 1-bit
				// MPY_NEXT_BIT has a 1-bit
				// MPY_BASE_BIT < 31, so MPY_NEXT_BIT < 32

				// handle a span of 1's
				// span can be punctuated by single 0's
				// span is termin_Ated by two 0's or end of word

				// handle bottom of span
				#if (MPY_FIRST_OUTPUT)
					sub_shf_left(output, 0, mpy_in, MPY_BASE_BIT); output = 0 - (mpy_in * 2^MPY_BASE_BIT)
					#define_eval MPY_FIRST_OUTPUT 0
				#else
					sub_shf_left(output, output, mpy_in, MPY_BASE_BIT); output = output - (mpy_in * 2^MPY_BASE_BIT)
				#endif

				// bits at BASE and NEXT are 1's, look ahead till end of span
				#define_eval MPY_NEXT_BIT (MPY_NEXT_BIT + 1)

				#while (((MPY_CONST & (1 << MPY_NEXT_BIT)) && (MPY_NEXT_BIT < 32)) || ((MPY_CONST & (1 << MPY_NEXT_BIT + 1)) && ((MPY_NEXT_BIT + 1) < 32)))

					// if skipping a lone 0, subtract it here
					#if (!(MPY_CONST & (1 << MPY_NEXT_BIT)))
						sub_shf_left(output, output, mpy_in, MPY_NEXT_BIT); output = output - (mpy_in * 2^MPY_BASE_BIT)
					#endif
					#define_eval MPY_NEXT_BIT (MPY_NEXT_BIT + 1)
				#endloop // #while in span

				// now MPY_NEXT_BIT indicates the next 0 past the span

				#if (MPY_NEXT_BIT == 32) // special case
					// if span extends through bit 31
					// we don't need to add in (mpy_in * 2^32),
					// because the previous subtraction did a borrow.
				#else
					add_shf_left(output, output, mpy_in, MPY_NEXT_BIT); output = output + (mpy_in * 2^SPAN_TOP_BIT)
				#endif
					
				#define_eval MPY_BASE_BIT (MPY_NEXT_BIT)	// skip past span
			#endif // we had a span of at least 2 1's in a row
			#undef MPY_NEXT_BIT						// clean up name space

		#endif // MPY_CONST has MPY_BASE_BIT set

		#define_eval MPY_BASE_BIT (MPY_BASE_BIT + 1)
	#endloop // #while (MPY_BASE_BIT < 32)

	#undef MPY_BASE_BIT					// clean up name space
	#undef MPY_FIRST_OUTPUT				// clean up name space
	#undef mpy_in						// clean up name space
	#undef MPY_CONST					// clean up name space

#endif // A or B is constant
#endm // multiply



// divide
//
//		Description:
//					output = divisor / DIVIDEND
//		Outputs:
//					output		output = mpy_in * MPY_CONST
//		Inputs:
//					divisor		variable or CONSTANT
//					DIVIDEND	for variable divisor
//									DIVIDEND must be power of 2
//								for CONSTANT divisor
//									DIVIDEND can be any CONSTANT
//
//		Performance:	variable/CONSTANT: 1 cycle
//						CONSTANT/CONSTANT: 1 - 2 cycles
//
//
//		Resources:
//					memory -- no accesses, none consumed
//					registers -- no additional registers, just
//								output & input parameters.
//					instructions -- 1 or 2
//
//		Condition codes: data dependent -- assume they are clobbered.
//
//		Limitations:
//					power of 2 constant DIVIDEND
//
//		Algorithm:
//					Determine what DIV_SHIFT yeilds (2^DIV_SHIFT == DIVIDEND)
//					Right shift divisor by DIV_SHIFT.
//
#macro divide(output, divisor, DIVIDEND)

	#if (!isnum(DIVIDEND))
		#error 4 "divide(out, divisor, DIVIDEND) requires CONSTANT power of 2 DIVIDEND."
	#elif (DIVIDEND == 0)
		#error 4 "divide(out, divisor, DIVIDEND): divide by 0"
	#elif (isnum(divisor))
		// Both divisor and DIVIDEND are constant -- go for it
		immed32(output, ((divisor) / (DIVIDEND)))	; pre-processor divide
	#else
		// divisor is variable and DIVIDEND is constant

		// Find what DIV_SHIFT gives (2^DIV_SHIFT == DIVIDEND)
		#define_eval DIV_SHIFT 0
		#define_eval DIV_TWO2SHIFT (1 << DIV_SHIFT)
		#while((DIV_SHIFT < 32) && (DIVIDEND != DIV_TWO2SHIFT))
			#define_eval DIV_SHIFT (DIV_SHIFT + 1)
			#define_eval DIV_TWO2SHIFT (1 << DIV_SHIFT)
		#endloop

		#if (DIV_TWO2SHIFT == DIVIDEND)
			shf_right(output, divisor, DIV_SHIFT)
		#else
			#error 1 "divide(out, divisor, DIVIDEND): DIVIDEND must be power of 2."
		#endif

		#undef DIV_SHIFT			// clean up name space
		#undef DIV_TWO2SHIFT		// clean up name space
	#endif // divisor is variable and DIVIDEND is constant

#endm // divide

//	index_from_addr
//
//	Description:
//		Convert an address into an array index. Ie.
//		out_index = (in_address - BASE_ADDRESS) / ELEMENT_SIZE
//
//	Outputs:
//		out_index: index of the array element
//
//	Inputs: 
//		in_address		Array element's address
//		ELEMENT_SIZE	Size of Array Element in address units
// 		BASE_ADDRESS	Memory Address of 1st element in the array.
//
//	Implementation:
//		out_index = (in_address / ELEMENT_SIZE) - (BASE_ADDRESS / ELEMENT_SIZE)
//
//	Size: Depends on ELEMENT_SIZE and BASE_ADDRESS
//			typical: 3 = 2 immeds + 1 ALU
//
//	Limitations:
//		Attempts to compile for non power of 2 ELEMENT_SIZE will fail
//		because divide doesn't currently support non power of 2 divisors.

#macro index_from_addr(out_index, in_address, BASE_ADDRESS, ELEMENT_SIZE)

	// Find what IFA_SHIFT gives (2^IFA_SHIFT == ELEMENT_SIZE)
	#define_eval IFA_SHIFT 0
	#define_eval IFA_TWO2SHIFT (1 << IFA_SHIFT)
	#while((IFA_SHIFT < 32) && (ELEMENT_SIZE != IFA_TWO2SHIFT))
		#define_eval IFA_SHIFT (IFA_SHIFT + 1)
		#define_eval IFA_TWO2SHIFT (1 << IFA_SHIFT)
	#endloop

	// if ELEMENT_SIZE is a power of two
	#if (IFA_TWO2SHIFT == ELEMENT_SIZE)
		alu_shf_right(out_index, ((BASE_ADDRESS) / (ELEMENT_SIZE)), b-a, in_address, IFA_SHIFT)

	#else	// ELEMENT_SIZE is NOT power of two

		// This is what we want to do, but
		// divide will not compile for non power of 2.
		sub[out_index, in_address, BASE_ADDRESS]
		divide[out_index, out_index, ELEMENT_SIZE]
	#endif

	#undef IFA_SHIFT			// clean up name space
	#undef IFA_TWO2SHIFT		// clean up name space
#endm

//	addr_from_index
//
//	Description:
//		Convert an array element address to an array index. Ie.
//		out_address = (in_index * ELEMENT_SIZE) + BASE_ADDRESS
//
//	Outputs:
//		out_address		Address of the array element
//
//	Inputs: 
//		in_index		Index of the array element
//		ELEMENT_SIZE	Size of Array Element in address units
// 		BASE_ADDRESS	Memory Address of 1st element in the array.
//
//	Implementation:
//		if (ELEMENT_SIZE is power of 2)
//			out_address = (in_index << BUF_DAFI_SHIFT) + BASE_ADDRESS
//		else
//			out_address = (in_index * ELEMENT_SIZE) + BASE_ADDRESS
//
//	Size: Depends on ELEMENT_SIZE and BASE_ADDRESS
//			best: 1 ALU
//			typical: 3 = 2 immeds + 1 ALU
//			worst: 21 = 4 immeds + 17 ALU
//
#macro addr_from_index(out_address, in_index, BASE_ADDRESS, ELEMENT_SIZE)

	// Find what AFI_SHIFT gives (2^AFI_SHIFT == ELEMENT_SIZE)
	#define_eval AFI_SHIFT 0
	#define_eval AFI_TWO2SHIFT (1 << AFI_SHIFT)
	#while((AFI_SHIFT < 32) && (ELEMENT_SIZE != AFI_TWO2SHIFT))
		#define_eval AFI_SHIFT (AFI_SHIFT + 1)
		#define_eval AFI_TWO2SHIFT (1 << AFI_SHIFT)
	#endloop

	// if ELEMENT_SIZE is a power of two
	#if (AFI_TWO2SHIFT == ELEMENT_SIZE)

		add_shf_left(out_address, BASE_ADDRESS, in_index, AFI_SHIFT)

	#else	// else ELEMENT_SIZE NOT a power of two
		multiply(out_address, in_index, ELEMENT_SIZE)
		add[out_address, out_address, BASE_ADDRESS]
	#endif

	#undef AFI_SHIFT			// clean up name space
	#undef AFI_TWO2SHIFT		// clean up name space
#endm

#endif // STDMAC_UC