//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// xbuf.uc
// macros for ethernet protocol header 
//
// system: IXP1200
// subsystem: IP microcode
// usage: library macros
// author: dfh	July 10, 2000
//
// revisions:


#ifndef XBUF_UC
#define XBUF_UC

#include "field.uc"

// API:
// sram transfer register buffer macros
//		xbuf_alloc[str_xbuf_name, NUMBER_OF_REGS]
//		xbuf_free[str_xbuf_name]
//		xbuf_link[str_xbuf_name, str_nextxbuf_name]
//		xbuf_param_set[str_xbuf_name]
//		xbuf_xfer_set[str_xbuf_name, BYTE_OFFSET]
//		xbuf_extract[out_byte_field, str_xbuf_name, WINDOW_START, FIELD_START, NUMBER_OF_BYTES]
//		xbuf_type_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE, DATATYPE]



// xbuf_alloc[str_xbuf_name, NUMBER_OF_REGS]
//      description:    locally declares contiguous 'NUMBER_OF_REGS' sram xfer registers
//                      and sets an appropriate .xfer_order
//                      registers may be referenced as an array
//                      allows transfer regs to be divided into to 4 buffers max
//      inputs:
//            str_xbuf_name       string uniquely identifying the transfer reg buffer
//            NUMBER_OF_REGS    number of transfer registers to allocate
// 
//      example usage:
//            xbuf_alloc($x, 8);                               // setup xfer regs $x0 thru $x7
//            sram_read($x0, addr1, addr2, LWCOUNT2, pend);    // load the first 2 xfer regs
//            xbuf_free($x);
//
#macro xbuf_alloc[str_xbuf_name, NUMBER_OF_REGS]
#if (strstr(str_xbuf_name, $$))
	_xbuf_d_alloc[str_xbuf_name, NUMBER_OF_REGS]
#else
	#ifndef _xbuf1_name
	#define_eval _xbuf1_name (str_xbuf_name)
	#define_eval _xbuf1_size (NUMBER_OF_REGS)
	#else
		#ifndef _xbuf2_name
		#define_eval _xbuf2_name (str_xbuf_name)
		#define_eval _xbuf2_size (NUMBER_OF_REGS)
		#else
			#ifndef _xbuf3_name
			#define_eval _xbuf3_name (str_xbuf_name)
			#define_eval _xbuf3_size (NUMBER_OF_REGS)
			#else
				#ifndef _xbuf4_name
				#define_eval _xbuf4_name (str_xbuf_name)
				#define_eval _xbuf4_size (NUMBER_OF_REGS)
				#else
					#error 2 "No more xfer bufs available."
				#endif
			#endif
		#endif
	#endif
	#if (NUMBER_OF_REGS == 1)
	    .local str_xbuf_name/**/0
	    .xfer_order str_xbuf_name/**/0
	#elif (NUMBER_OF_REGS == 2)
	    .local str_xbuf_name/**/0, str_xbuf_name/**/1
	    .xfer_order str_xbuf_name/**/0, str_xbuf_name/**/1
	#elif (NUMBER_OF_REGS == 3)
	    .local str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2
	    .xfer_order str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2
	#elif (NUMBER_OF_REGS == 4)
	    .local str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3
	    .xfer_order str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3
	#elif (NUMBER_OF_REGS == 5)
	    .local str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4
	    .xfer_order str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4
	#elif (NUMBER_OF_REGS == 6)
	    .local str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4, str_xbuf_name/**/5
	    .xfer_order str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4, str_xbuf_name/**/5
	#elif (NUMBER_OF_REGS == 7)
	    .local str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4, str_xbuf_name/**/5, str_xbuf_name/**/6
	    .xfer_order str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4, str_xbuf_name/**/5, str_xbuf_name/**/6
	#elif (NUMBER_OF_REGS == 8)
	    .local str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4, str_xbuf_name/**/5, str_xbuf_name/**/6, str_xbuf_name/**/7
	    .xfer_order str_xbuf_name/**/0, str_xbuf_name/**/1, str_xbuf_name/**/2, str_xbuf_name/**/3, str_xbuf_name/**/4, str_xbuf_name/**/5, str_xbuf_name/**/6, str_xbuf_name/**/7
	#else
	    #error 2 "xbuf_alloc, Illegal number of SRAM Transfer Registers supplied."
	#endif

#endif // strstr
#endm


// xbuf_free
//      description:    alias for .endlocal, used to deallocate xfer regs.
//      inputs:
//            str_xbuf_name        name of transfer reg buffer
// 
//        example usage:
//            xbuf_alloc($x, 8);                               // setup xfer regs $x0 thru $x7
//            sram_read($x0, addr1, addr2, LWCOUNT2, pend);    // load the first 2 xfer regs
//            xbuf_free($x);
//
#macro xbuf_free[str_xbuf_name]
#if (strstr(str_xbuf_name, $$))
	_xbuf_d_free[str_xbuf_name]
#else
	#define_eval _this_xbuf_name (str_xbuf_name)
	#if (streq(_this_xbuf_name, _xbuf1_name))
		#undef _xbuf1_name
		#undef _xbuf1_size
	#ifdef _xbuf1_next
		#undef _xbuf1_next
	#endif

	#elif (streq(_this_xbuf_name, _xbuf2_name))
		#undef _xbuf2_name
		#undef _xbuf2_size
		#ifdef _xbuf2_next
			#undef _xbuf2_next
		#endif
	#elif (streq(_this_xbuf_name, _xbuf3_name))
		#undef _xbuf3_name
		#undef _xbuf3_size
		#ifdef _xbuf3_next
			#undef _xbuf3_next
		#endif
	#elif (streq(_this_xbuf_name, _xbuf4_name))
		#undef _xbuf4_name
		#undef _xbuf4_size
		#ifdef _xbuf4_next
			#undef _xbuf4_next
		#endif
	#else
	    #error 2 "xbuf_free. Invalid str_xbuf_name."
	#endif
	.endlocal
#endif //strstr
#endm


// xbuf_link
//      description:    link one transfer reg buffer to another
//                      the purpose is to allow processing streams of bytes
//                      by continually loading buffers and providing macros
//                      that can traverse the buffers.
//      inputs:
//          str_xbuf_name          a transfer buffer allocated by xbuf_alloc
//          str_nextxbuf_name      another transfer buffer allocated by xbuf_alloc
//                               the next buffer continues the stream of bytes
//
//      example usage:
//          xbuf_alloc($pkt, 4);                                // setup xfer regs $pkt0 thru $pkt3
//          // load the xfer buf $pkt, use it
//          xbuf_alloc($morepkt, 4);                            // setup xfer regs $x0 thru $x7
//          xbuf_link($pkt, $morepkt);                            // logical link pkt to morepkt
//          // load the second xfer buf $morepkt
//          ip_verify(exception, $pkt, ip_header_byte_position); // verify ip header that spans the buffers
//          xbuf_free($pkt);
//          xbuf_free($morepkt);
//    
#macro xbuf_link[str_xbuf_name, str_nextxbuf_name]
#if (strstr(str_xbuf_name, $$))
	_xbuf_d_link[str_xbuf_name, str_nextxbuf_name]
#else	
	#define_eval _this_xbuf_name (str_xbuf_name)

	#if (streq(_this_xbuf_name, _xbuf1_name))
		#define_eval _xbuf1_next  (str_nextxbuf_name)
	#elif (streq(_this_xbuf_name, _xbuf2_name))
		#define_eval _xbuf2_next  (str_nextxbuf_name)
	#elif (streq(_this_xbuf_name, _xbuf3_name))
		#define_eval _xbuf3_next  (str_nextxbuf_name)
	#elif (streq(_this_xbuf_name, _xbuf4_name))
		#define_eval _xbuf4_next   (str_nextxbuf_name)
	#else
	    #error 2 "xbuf_link. Invalid str_xbuf_name."
	#endif

#endif // strstr
#endm


#macro xbuf_find[xfer_name]
#ifdef _xbuf_found
#undef _xbuf_found
#endif
// the xfer reg name could be $name[0] - $name[7] or $$name[0] - $$name[7] 
#define_eval _xfer_len (strlen(xfer_name))
#define_eval _this_xbuf_name (strleft(xfer_name, (_xfer_len - 3)))
#define_eval _cur_xbuf_index_left (strleft(xfer_name, (_xfer_len - 1)))
#define_eval _cur_xbuf_index (strright (_cur_xbuf_index_left, 1))
#if (streq(_this_xbuf_name, _xbuf1_name))
	#define  _xbuf_found
	xbuf_param_set[_this_xbuf_name]
#elif (streq(_this_xbuf_name, _xbuf2_name))
	#define  _xbuf_found
	xbuf_param_set[_this_xbuf_name]
#elif (streq(_this_xbuf_name, _xbuf3_name))
	#define  _xbuf_found
	xbuf_param_set[_this_xbuf_name]
#elif (streq(_this_xbuf_name, _xbuf4_name))
	#define  _xbuf_found
	xbuf_param_set[_this_xbuf_name]
#elif (streq(_this_xbuf_name, _xbuf1_d_name))
	#define  _xbuf_found
	xbuf_d_param_set[_this_xbuf_name]
#elif (streq(_this_xbuf_name, _xbuf2_d_name))
	#define  _xbuf_found
	xbuf_d_param_set[_this_xbuf_name]
#elif (streq(_this_xbuf_name, _xbuf3_d_name))
	#define  _xbuf_found
	xbuf_d_param_set[_this_xbuf_name]
#elif (streq(_this_xbuf_name, _xbuf4_d_name))
	#define  _xbuf_found
	xbuf_d_param_set[_this_xbuf_name]
else
	// or the xfer reg name could be $name0 - $name7 or $$name0 - $$name7 
	#define_eval _this_xbuf_name (strleft(xfer_name, (_xfer_len - 1)))
	#define_eval _cur_xbuf_index (strright (xfer_name, 1))
	#if (streq(_this_xbuf_name, _xbuf1_name))
		#define  _xbuf_found
		xbuf_param_set[_this_xbuf_name]
	#elif (streq(_this_xbuf_name, _xbuf2_name))
		#define  _xbuf_found
		xbuf_param_set[_this_xbuf_name]
	#elif (streq(_this_xbuf_name, _xbuf3_name))
		#define  _xbuf_found
		xbuf_param_set[_this_xbuf_name]
	#elif (streq(_this_xbuf_name, _xbuf4_name))
		#define  _xbuf_found
		xbuf_param_set[_this_xbuf_name]
	#elif (streq(_this_xbuf_name, _xbuf1_d_name))
		#define  _xbuf_found
		xbuf_d_param_set[_this_xbuf_name]
	#elif (streq(_this_xbuf_name, _xbuf2_d_name))
		#define  _xbuf_found
		xbuf_d_param_set[_this_xbuf_name]
	#elif (streq(_this_xbuf_name, _xbuf3_d_name))
		#define  _xbuf_found
		xbuf_d_param_set[_this_xbuf_name]
	#elif (streq(_this_xbuf_name, _xbuf4_d_name))
		#define  _xbuf_found
		xbuf_d_param_set[_this_xbuf_name]
	#endif
#endif
#endm


// xbuf_param_set
//      description: set global internal params _cur_xbuf_name,  _cur_xbuf_size and _cur_xbuf_next.
//					 It is used for folowing a chain of xbufs.
// 		inputs:
//			str_xbuf_name	name of sram transfer buffer
//
//		side-effects: 
//			Modify _cur_xbuf_name,  _cur_xbuf_size and _cur_xbuf_next.
//
#macro xbuf_param_set[str_xbuf_name]
#if (strstr(str_xbuf_name, $$))
	_xbuf_d_param_set[str_xbuf_name]
#else
	#ifdef _cur_xbuf_name
		#undef _cur_xbuf_name
	#endif
	#ifdef _cur_xbuf_size
		#undef _cur_xbuf_size
	#endif
	#ifdef _cur_xbuf_next
		#undef _cur_xbuf_next
	#endif
	#define_eval _cur_xbuf_name (str_xbuf_name)

	#if streq(_cur_xbuf_name, _xbuf1_name)
		#define _cur_xbuf_size	_xbuf1_size
		#ifdef _xbuf1_next
			#define _cur_xbuf_next _xbuf1_next
		#endif
	#elif streq(_cur_xbuf_name, _xbuf2_name)
		#define _cur_xbuf_size	_xbuf2_size
		#ifdef _xbuf2_next
			#define _cur_xbuf_next _xbuf2_next
		#endif
	#elif streq(_cur_xbuf_name, _xbuf3_name)
		#define _cur_xbuf_size	_xbuf3_size
		#ifdef _xbuf3_next
			#define _cur_xbuf_next _xbuf3_next
		#endif
	#elif streq(_cur_xbuf_name, _xbuf4_name)
		#define _cur_xbuf_size	_xbuf4_size
		#ifdef _xbuf4_next
			#define _cur_xbuf_next _xbuf4_next
		#endif
	#else
		#error 4 "xbuf_param_set. no matching str_xbuf_name."
	#endif

#endif // strstr
#endm


// xbuf_xfer_set
//		description: Define tokens _XFER0-7 from the xbufs, starting at longword indicated by BYTE_OFFSET.
//					 These tokens can then be used as individual identification of transfer regs.
//					 The search for xfers can span multiple linked xbufs.
//		inputs:
//			str_xbuf_name		name of first sram transfer buf
//			BYTE_OFFSET			byte offset where _XFER0 is
//		side-effects: 
//			Modify global accessible tokens _XFER0, _XFER1, ..., _XFER7
//
#macro xbuf_xfer_set[str_xbuf_name, BYTE_OFFSET]
#if (strstr(str_xbuf_name, $$))
	_xbuf_d_xfer_set[str_xbuf_name, BYTE_OFFSET]
#else
	#define_eval _LWORD_OFFSET (BYTE_OFFSET /4)
	xbuf_param_set[str_xbuf_name]

	#define_eval _LWORD_IN_BUF (0)
	#define_eval _LWORDS_CHECKED (0)

	// find first xfer reg
	#while (_LWORDS_CHECKED < _LWORD_OFFSET)
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
		#define_eval _LWORDS_CHECKED (_LWORDS_CHECKED + 1)
		#if (_LWORD_IN_BUF == _cur_xbuf_size)
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF 0
		#endif
	#endloop

	#ifdef _XFER0
		#undef _XFER0
	#endif

	#define_eval _XFER0 _cur_xbuf_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)


	#ifdef _XFER1
		#undef _XFER1
	#endif
	#if(_LWORD_IN_BUF < _cur_xbuf_size)
		#define_eval _XFER1 _cur_xbuf_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
	#else
		#ifdef _cur_xbuf_next
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF (0)
			#define_eval _XFER1 _cur_xbuf_name/**/_LWORD_IN_BUF
			#define_eval _LWORD_IN_BUF (1)
		#endif
	#endif

	#ifdef _XFER2
		#undef _XFER2
	#endif
	#if(_LWORD_IN_BUF < _cur_xbuf_size)
		#define_eval _XFER2 _cur_xbuf_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
	#else
		#ifdef _cur_xbuf_next
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF (0)
			#define_eval _XFER2 _cur_xbuf_name/**/_LWORD_IN_BUF
			#define_eval _LWORD_IN_BUF (1)
		#endif
	#endif

	#ifdef _XFER3
		#undef _XFER3
	#endif
	#if(_LWORD_IN_BUF < _cur_xbuf_size)
		#define_eval _XFER3 _cur_xbuf_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
	#else
		#ifdef _cur_xbuf_next
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF (0)
			#define_eval _XFER3 _cur_xbuf_name/**/_LWORD_IN_BUF
			#define_eval _LWORD_IN_BUF (1)
		#endif
	#endif

	#ifdef _XFER4
		#undef _XFER4
	#endif
	#if(_LWORD_IN_BUF < _cur_xbuf_size)
		#define_eval _XFER4 _cur_xbuf_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
	#else
		#ifdef _cur_xbuf_next
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF (0)
			#define_eval _XFER4 _cur_xbuf_name/**/_LWORD_IN_BUF
			#define_eval _LWORD_IN_BUF (1)
		#endif
	#endif

	#ifdef _XFER5
		#undef _XFER5
	#endif
	#if(_LWORD_IN_BUF < _cur_xbuf_size)
		#define_eval _XFER5 _cur_xbuf_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
	#else
		#ifdef _cur_xbuf_next
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF (0)
			#define_eval _XFER5 _cur_xbuf_name/**/_LWORD_IN_BUF
			#define_eval _LWORD_IN_BUF (1)
		#endif
	#endif

	#ifdef _XFER6
		#undef _XFER6
	#endif
	#if(_LWORD_IN_BUF < _cur_xbuf_size)
		#define_eval _XFER6 _cur_xbuf_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
	#else
		#ifdef _cur_xbuf_next
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF (0)
			#define_eval _XFER6 _cur_xbuf_name/**/_LWORD_IN_BUF
			#define_eval _LWORD_IN_BUF (1)
		#endif
	#endif

	#ifdef _XFER7
		#undef _XFER7
	#endif
	#if(_LWORD_IN_BUF < _cur_xbuf_size)
		#define_eval _XFER7 _cur_xbuf_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
	#else
		#ifdef _cur_xbuf_next
			xbuf_param_set[_cur_xbuf_next]
			#define_eval _LWORD_IN_BUF (0)
			#define_eval _XFER7 _cur_xbuf_name/**/_LWORD_IN_BUF
			#define_eval _LWORD_IN_BUF (1)
		#endif
	#endif

#endif // strstr
#endm


// xbuf_extract
//      description: Extract a numeric byte field from transfer register buffer.
//				     Perform endian swap if global #define LITTLE_ENDIAN.
//		output:
//			out_byte_field	GPR output field, right justified
//		inputs:
//			str_xbuf_name		name of sram read transfer register
//			WINDOW_START		start byte position of window or header
//			FIELD_START			start byte offset of field from window start
//			NUMBER_OF_BYTES 	number of bytes in field	
//
#macro xbuf_extract[out_byte_field, str_xbuf_name, WINDOW_START, FIELD_START, NUMBER_OF_BYTES]
#if (strstr(str_xbuf_name, $$))
	_xbuf_d_extract[out_byte_field, str_xbuf_name, WINDOW_START, FIELD_START, NUMBER_OF_BYTES]
#else
	// set _cur_xbuf_size and _cur_xbuf_next
	xbuf_param_set[str_xbuf_name]
	#define_eval _START_BYTE (WINDOW_START + FIELD_START)
	#define_eval _END_BYTE (_START_BYTE + (NUMBER_OF_BYTES - 1))

	// setup word0 defs
	#define_eval _WORD0 (_START_BYTE / 4)
	#define_eval _WORD1 (_END_BYTE / 4)

	#if ((_WORD1 - _WORD0) > 1)
		#error 2   "xbuf_num_extract: extract of byte field greater than 4 bytes. Not supported."
	#else

	#ifdef _str_word0_xfer
		#undef _str_word0_xfer
	#endif
	#ifdef _str_word1_xfer
		#undef _str_word1_xfer
	#endif

	// first of 4 possible xbufs
	#define_eval _RELATIVE_SBYTE _START_BYTE
	#if (_WORD0 < _cur_xbuf_size)
		#define_eval _str_word0_xfer _cur_xbuf_name/**/_WORD0/**/

	#else

		#define_eval _WORD0 (_WORD0 - _cur_xbuf_size)
		#define_eval _RELATIVE_SBYTE (_RELATIVE_SBYTE - (_cur_xbuf_size * 4))
		xbuf_param_set[_cur_xbuf_next]

		// second of 4 possible xbufs
		#if (_WORD0 < _cur_xbuf_size)
			#define_eval _str_word0_xfer _cur_xbuf_name/**/_WORD0/**/

		#else
			#define_eval _WORD0 (_WORD0 - _cur_xbuf_size)
			#define_eval _RELATIVE_SBYTE (_RELATIVE_SBYTE - (_cur_xbuf_size * 4))
			xbuf_param_set[_cur_xbuf_next]

			// third of 4 possible xbufs
			#if (_WORD0 < _cur_xbuf_size)
				#define_eval _str_word0_xfer _cur_xbuf_name/**/_WORD0/**/

			#else
				#define_eval _WORD0 (_WORD0 - _cur_xbuf_size)
				#define_eval _RELATIVE_SBYTE (_RELATIVE_SBYTE - (_cur_xbuf_size * 4))
				xbuf_param_set[_cur_xbuf_next]

				// fourth of 4 possible xbufs
				#if (_WORD0 < _cur_xbuf_size)
					#define_eval _str_word0_xfer _cur_xbuf_name/**/_WORD0/**/
				#else
					#error 2   "xbuf_num_extract: start byte is out of range of transfer buffers."
				#endif
			#endif
		#endif
	#endif

	// setup word1 defs
	xbuf_param_set[str_xbuf_name]

	#define_eval _WORD1 (_END_BYTE / 4)
	#define_eval _RELATIVE_EBYTE _END_BYTE

	// first of 4 possible xbufs
	#if (_WORD1 < _cur_xbuf_size)
	 #define_eval _str_word1_xfer _cur_xbuf_name/**/_WORD1/**/

	#else
		#define_eval _WORD1 (_WORD1 - _cur_xbuf_size)
		#define_eval _RELATIVE_EBYTE (_RELATIVE_EBYTE - (_cur_xbuf_size * 4))
		xbuf_param_set[_cur_xbuf_next]

		// second of 4 possible xbufs
		#if (_WORD1 < _cur_xbuf_size)
			#define_eval _str_word1_xfer _cur_xbuf_name/**/_WORD1/**/

		#else
			#define_eval _WORD1 (_WORD1 - _cur_xbuf_size)
			#define_eval _RELATIVE_EBYTE (_RELATIVE_EBYTE - (_cur_xbuf_size * 4))
			xbuf_param_set[_cur_xbuf_next]

			// third of 4 possible xbufs
			#if (_WORD1 < _cur_xbuf_size)
				#define_eval _str_word1_xfer _cur_xbuf_name/**/_WORD1/**/

			#else
				#define_eval _WORD1 (_WORD1 - _cur_xbuf_size)
				#define_eval _RELATIVE_EBYTE (_RELATIVE_EBYTE - (_cur_xbuf_size * 4))
				xbuf_param_set[_cur_xbuf_next]

				// fourth of 4 possible xbufs
				#if (_WORD1 < _cur_xbuf_size)
					#define_eval _str_word1_xfer _cur_xbuf_name/**/_WORD1/**/
				#else
					#error 2   "xbuf_num_extract: start byte is out of range of transfer buffers."
				#endif
			#endif
		#endif
	#endif

	#define_eval _SBYTE_IN_WD (_RELATIVE_SBYTE & 0x3)
	#define_eval _EBYTE_IN_WD (_RELATIVE_EBYTE & 0x3)

	#if (_WORD0 == _WORD1)
	    field_extract[out_byte_field, _str_word0_xfer, _SBYTE_IN_WD, _EBYTE_IN_WD]
	#else
	    field_dbl_extract[out_byte_field, _str_word0_xfer, _SBYTE_IN_WD, _str_word1_xfer, _EBYTE_IN_WD]
	#endif

#endif // ((_WORD2 - _WORD1) > 1)

#endif // strstr
#endm


// xbuf_type_extract 
//		description: extract from xbuf based on datatype
//		output:
//			out_byte_field		GPR containing field, right justified	
//		inputs:
//			str_xbuf_name		name of sram transfer register buffer
//			WINDOW_START		start position(in datatype increments) of window or header
//			FIELD_START			start offset(in datatype increments) of field from window start
//			SIZE				size of field 
//			DATATYPE			datatype of the field
//				types supported:		
#define BIG_ENDIAN_BYTES 0		// data is big-endian in memory
#define LITTLE_ENDIAN_BYTES 1	// data is little-endian in memory, perform swap
//
//
#macro xbuf_type_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE, DATATYPE]
#if (strstr(str_xbuf_name, $$))
	_xbuf_d_type_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE, DATATYPE]
#else
	#if (DATATYPE == BIG_ENDIAN_BYTES)
		#ifdef LITTLE_ENDIAN
			#undef LITTLE_ENDIAN
			xbuf_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
			#define LITTLE_ENDIAN
		#else
			xbuf_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
		#endif

	#elif (DATATYPE == LITTLE_ENDIAN_BYTES)
		#ifndef LITTLE_ENDIAN
			#define LITTLE_ENDIAN
			xbuf_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
			#undef LITTLE_ENDIAN
		#else
			xbuf_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
		#endif

	#else
		#error 2 "xbuf_type_extract. Datatype not supported."
	#endif

#endif // strstr
#endm


// _xbuf_d_alloc[str_xbuf_d_name, NUMBER_OF_REGS]
//      description:    locally declares contiguous 'NUMBER_OF_REGS' dram xfer registers
//                      and sets an appropriate .xfer_order
//                      registers may be referenced as an array
//                      allows transfer regs to be divided into to 4 buffers max
//      inputs:
//            str_xbuf_d_name       string uniquely identifying the transfer reg buffer
//            NUMBER_OF_REGS    number of transfer registers to allocate
// 
//      example usage:
//            _xbuf_d_alloc($$dx, 8);                               // setup xfer regs $$dx0 - $$dx7
//            dram_read($$dx0, addr1, addr2, QWCOUNT4, pend);    // load the xfer regs
//            _xbuf_d_free($$dx);
//
#macro _xbuf_d_alloc[str_xbuf_d_name, NUMBER_OF_REGS]
#ifndef _xbuf_d1_name
	#define_eval _xbuf_d1_name (str_xbuf_d_name)
	#define_eval _xbuf_d1_size (NUMBER_OF_REGS)
#else
	#ifndef _xbuf_d2_name
		#define_eval _xbuf_d2_name (str_xbuf_d_name)
		#define_eval _xbuf_d2_size (NUMBER_OF_REGS)
	#else
		#ifndef _xbuf_d3_name
			#define_eval _xbuf_d3_name (str_xbuf_d_name)
			#define_eval _xbuf_d3_size (NUMBER_OF_REGS)
		#else
			#ifndef _xbuf_d4_name
				#define_eval _xbuf_d4_name (str_xbuf_d_name)
				#define_eval _xbuf_d4_size (NUMBER_OF_REGS)
			#else
				#error 2 "No more dram xfer bufs available."
			#endif
		#endif
	#endif
#endif

#if (NUMBER_OF_REGS == 2)
    .local str_xbuf_d_name/**/0, str_xbuf_d_name/**/1
    .xfer_order str_xbuf_d_name/**/0, str_xbuf_d_name/**/1
#elif (NUMBER_OF_REGS == 3)
    .local str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2
    .xfer_order str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2
#elif (NUMBER_OF_REGS == 4)
    .local str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3
    .xfer_order str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3
#elif (NUMBER_OF_REGS == 5)
    .local str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4
    .xfer_order str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4
#elif (NUMBER_OF_REGS == 6)
    .local str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4, str_xbuf_d_name/**/5
    .xfer_order str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4, str_xbuf_d_name/**/5
#elif (NUMBER_OF_REGS == 7)
    .local str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4, str_xbuf_d_name/**/5, str_xbuf_d_name/**/6
    .xfer_order str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4, str_xbuf_d_name/**/5, str_xbuf_d_name/**/6
#elif (NUMBER_OF_REGS == 8)
    .local str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4, str_xbuf_d_name/**/5, str_xbuf_d_name/**/6, str_xbuf_d_name/**/7
    .xfer_order str_xbuf_d_name/**/0, str_xbuf_d_name/**/1, str_xbuf_d_name/**/2, str_xbuf_d_name/**/3, str_xbuf_d_name/**/4, str_xbuf_d_name/**/5, str_xbuf_d_name/**/6, str_xbuf_d_name/**/7
#else
    #error 2 "_xbuf_d_alloc, Illegal number of SRAM Transfer Registers supplied."
#endif
#endm

// _xbuf_d_free
//      description:    alias for .endlocal, used to deallocate xfer regs.
//      inputs:
//            str_xbuf_d_name        name of transfer reg buffer
// 
//        example usage:
//            $$xbuf_d_alloc($$dx, 8);                               // setup xfer regs $dx0 - $dx7
//            dram_read($$dx0, addr1, addr2, QWCOUNT4, pend);    // load the xfer regs
//            _xbuf_d_free($$dx);
//
#macro _xbuf_d_free[str_xbuf_d_name]
#define_eval _this_xbuf_d_name (str_xbuf_d_name)
#if (streq(_this_xbuf_d_name, _xbuf_d1_name))
	#undef _xbuf_d1_name
	#undef _xbuf_d1_size
	#ifdef _xbuf_d1_next
		#undef _xbuf_d1_next
	#endif

#elif (streq(_this_xbuf_d_name, _xbuf_d2_name))
	#undef _xbuf_d2_name
	#undef _xbuf_d2_size
	#ifdef _xbuf_d2_next
		#undef _xbuf_d2_next
	#endif
#elif (streq(_this_xbuf_d_name, _xbuf_d3_name))
	#undef _xbuf_d3_name
	#undef _xbuf_d3_size
	#ifdef _xbuf_d3_next
		#undef _xbuf_d3_next
	#endif
#elif (streq(_this_xbuf_d_name, _xbuf_d4_name))
	#undef _xbuf_d4_name
	#undef _xbuf_d4_size
	#ifdef _xbuf_d4_next
		#undef _xbuf_d4_next
	#endif
#else
    #error 2 "_xbuf_d_free. Invalid str_xbuf_d_name."
#endif
.endlocal
#endm

// _xbuf_d_link
//      description:    link one dram transfer reg buffer to another
//                      the purpose is to allow processing streams of bytes
//                      by continually loading buffers and providing macros
//                      that can traverse the buffers.
//      inputs:
//          str_xbuf_d_name          a transfer buffer allocated by $$xbuf_d_alloc
//          str_nextxbuf_name      another transfer buffer allocated by $$xbuf_d_alloc
//                               the next buffer continues the stream of bytes
//
//      example usage:
//          _xbuf_d_alloc($$pkt, 4);                               // setup xbuf using 4 xfer regs
//          // load the xfer buf $$pkt, use it
//          _xbuf_d_alloc($$morepkt, 4);                           // setup xbuf using 4 xfer regs
//          _xbuf_d_link($$pkt, $$morepkt);                        // logical link pkt to morepkt
//          // load the second xfer buf morepkt
//          ip_verify(exception, $$pkt, ip_header_byte_position); // verify ip header that spans the buffers
//          _xbuf_d_free($$pkt);
//          _xbuf_d_free($$morepkt);
//    
#macro _xbuf_d_link[str_xbuf_d_name, str_nextxbuf_name]

#define_eval _this_xbuf_d_name (str_xbuf_d_name)

#if (streq(_this_xbuf_d_name, _xbuf_d1_name))
	#define_eval _xbuf_d1_next  (str_nextxbuf_name)
#elif (streq(_this_xbuf_d_name, _xbuf_d2_name))
	#define_eval _xbuf_d2_next  (str_nextxbuf_name)
#elif (streq(_this_xbuf_d_name, _xbuf_d3_name))
	#define_eval _xbuf_d3_next  (str_nextxbuf_name)
#elif (streq(_this_xbuf_d_name, _xbuf_d4_name))
	#define_eval _xbuf_d4_next   (str_nextxbuf_name)
#else
    #error 2 "_xbuf_d_link. Invalid str_xbuf_d_name."
#endif
#endm


// _xbuf_d_param_set
//      description: set global internal params _cur_xbuf_d_name,  _cur_xbuf_d_size and _cur_xbuf_d_next.
//					 It is used for folowing a chain of xbufs.
// 		inputs:
//			str_xbuf_d_name	name of dram transfer buffer
//
//		side-effects: 
//			Modify _cur_xbuf_name,  _cur_xbuf_d_size and _cur_xbuf_d_next.
//
#macro _xbuf_d_param_set[str_xbuf_d_name]
#ifdef _cur_xbuf_d_name
	#undef _cur_xbuf_d_name
#endif
#ifdef _cur_xbuf_d_size
	#undef _cur_xbuf_d_size
#endif
#ifdef _cur_xbuf_d_next
	#undef _cur_xbuf_d_next
#endif
#define_eval _cur_xbuf_d_name (str_xbuf_d_name)

#if streq(_cur_xbuf_d_name, _xbuf_d1_name)
	#define _cur_xbuf_d_size	_xbuf_d1_size
	#ifdef _xbuf_d1_next
		#define _cur_xbuf_d_next _xbuf_d1_next
	#endif
#elif streq(_cur_xbuf_d_name, _xbuf_d2_name)
	#define _cur_xbuf_d_size	_xbuf_d2_size
	#ifdef _xbuf_d2_next
		#define _cur_xbuf_d_next _xbuf_d2_next
	#endif
#elif streq(_cur_xbuf_d_name, _xbuf_d3_name)
	#define _cur_xbuf_d_size	_xbuf_d3_size
	#ifdef _xbuf_d3_next
		#define _cur_xbuf_d_next _xbuf_d3_next
	#endif
#elif streq(_cur_xbuf_d_name, _xbuf_d4_name)
	#define _cur_xbuf_d_size	_xbuf_d4_size
	#ifdef _xbuf_d4_next
		#define _cur_xbuf_d_next _xbuf_d4_next
	#endif
#else
	#error 4 "_xbuf_d_param_set. no matching str_xbuf_d_name."
#endif
 
#endm


// _xbuf_d_xfer_set
//		description: Define tokens _D_XFER0-7 from the xbufs, starting at longword indicated by BYTE_OFFSET.
//					 These tokens can then be used as individual identification of transfer regs.
//					 The search for xfers can span multiple linked xbufs.
//		inputs:
//			str_xbuf_d_name		name of first transfer buf
//			BYTE_OFFSET			byte offset where _D_XFER0 is
//		side-effects: 
//			Modify global accessible tokens _D_XFER0, _D_XFER1, ..., _D_XFER7
//
#macro _xbuf_d_xfer_set[str_xbuf_d_name, BYTE_OFFSET]
#define_eval _LWORD_OFFSET (BYTE_OFFSET /4)
_xbuf_d_param_set[str_xbuf_d_name]

#define_eval _LWORD_IN_BUF (0)
#define_eval _LWORDS_CHECKED (0)

// find first xfer reg
#while (_LWORDS_CHECKED < _LWORD_OFFSET)
	#if (_LWORD_IN_BUF < _cur_xbuf_d_size)
		#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
		#define_eval _LWORDS_CHECKED (_LWORDS_CHECKED + 1)
	#else
		#ifdef _cur_xbuf_d_next
			_xbuf_d_param_set[_cur_xbuf_d_next]
			#define_eval _LWORD_IN_BUF 0
			#define_eval _LWORDS_CHECKED (_LWORDS_CHECKED + 1)
		#else
			#error 2 "_xbuf_d_xfer_set. BYTE_OFFSET beyond buffer range"
		#endif
	#endif
#endloop

// check current _LWORD__IN_BUF for overflow
#if (_LWORD_IN_BUF >= _cur_xbuf_d_size)
	#ifdef _cur_xbuf_d_next
		xbuf_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF 0
	#else
		#error 2 "xbuf_d_xfer_set. BYTE_OFFSET beyond buffer range"
	#endif
#endif

#ifdef _D_XFER0
	#undef _D_XFER0
#endif

#define_eval _D_XFER0 _cur_xbuf_d_name/**/_LWORD_IN_BUF
#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)


#ifdef _D_XFER1
	#undef _D_XFER1
#endif
#if(_LWORD_IN_BUF < _cur_xbuf_d_size)
	#define_eval _D_XFER1 _cur_xbuf_d_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
#else
	#ifdef _cur_xbuf_d_next
		_xbuf_d_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF (0)
		#define_eval _D_XFER1 _cur_xbuf_d_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (1)
	#endif
#endif

#ifdef _D_XFER2
	#undef _D_XFER2
#endif
#if(_LWORD_IN_BUF < _cur_xbuf_d_size)
	#define_eval _D_XFER2 _cur_xbuf_d_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
#else
	#ifdef _cur_xbuf_d_next
		_xbuf_d_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF (0)
		#define_eval _D_XFER2 _cur_xbuf_d_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (1)
	#endif
#endif

#ifdef _D_XFER3
	#undef _D_XFER3
#endif
#if(_LWORD_IN_BUF < _cur_xbuf_d_size)
	#define_eval _D_XFER3 _cur_xbuf_d_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
#else
	#ifdef _cur_xbuf_d_next
		_xbuf_d_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF (0)
		#define_eval _D_XFER3 _cur_xbuf_d_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (1)
	#endif
#endif

#ifdef _D_XFER4
	#undef _D_XFER4
#endif
#if(_LWORD_IN_BUF < _cur_xbuf_d_size)
	#define_eval _D_XFER4 _cur_xbuf_d_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
#else
	#ifdef _cur_xbuf_d_next
		_xbuf_d_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF (0)
		#define_eval _D_XFER4 _cur_xbuf_d_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (1)
	#endif
#endif

#ifdef _D_XFER5
	#undef _D_XFER5
#endif
#if(_LWORD_IN_BUF < _cur_xbuf_d_size)
	#define_eval _D_XFER5 _cur_xbuf_d_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
#else
	#ifdef _cur_xbuf_d_next
		_xbuf_d_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF (0)
		#define_eval _D_XFER5 _cur_xbuf_d_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (1)
	#endif
#endif

#ifdef _D_XFER6
	#undef _D_XFER6
#endif
#if(_LWORD_IN_BUF < _cur_xbuf_d_size)
	#define_eval _D_XFER6 _cur_xbuf_d_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
#else
	#ifdef _cur_xbuf_d_next
		_xbuf_d_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF (0)
		#define_eval _D_XFER6 _cur_xbuf_d_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (1)
	#endif
#endif

#ifdef _D_XFER7
	#undef _D_XFER7
#endif
#if(_LWORD_IN_BUF < _cur_xbuf_d_size)
	#define_eval _D_XFER7 _cur_xbuf_d_name/**/_LWORD_IN_BUF
	#define_eval _LWORD_IN_BUF (_LWORD_IN_BUF + 1)
#else
	#ifdef _cur_xbuf_d_next
		_xbuf_d_param_set[_cur_xbuf_d_next]
		#define_eval _LWORD_IN_BUF (0)
		#define_eval _D_XFER7 _cur_xbuf_d_name/**/_LWORD_IN_BUF
		#define_eval _LWORD_IN_BUF (1)
	#endif
#endif

#endm


// _xbuf_d_extract
//      description: Extract a numeric byte field from dram transfer register buffer.
//				     Perform endian swap if global #define LITTLE_ENDIAN.
//		output:
//			out_byte_field	GPR output field, right justified
//		inputs:
//			str_xbuf_d_name		name of dram read transfer register
//			WINDOW_START		start byte position of window or header
//			FIELD_START			start byte offset of field from window start
//			NUMBER_OF_BYTES 	number of bytes in field	
//
#macro _xbuf_d_extract[out_byte_field, str_xbuf_d_name, WINDOW_START, FIELD_START, NUMBER_OF_BYTES]
// set _cur_xbuf_d_size and _cur_xbuf_d_next
_xbuf_d_param_set[str_xbuf_d_name]
#define_eval _START_BYTE (WINDOW_START + FIELD_START)
#define_eval _END_BYTE (_START_BYTE + (NUMBER_OF_BYTES - 1))

// setup word0 defs
#define_eval _WORD0 (_START_BYTE / 4)
#define_eval _WORD1 (_END_BYTE / 4)

#if ((_WORD1 - _WORD0) > 1)
	#error 2   "xbuf_d_num_extract: extract of byte field greater than 4 bytes. Not supported."
#else

	#ifdef _str_word0_xfer
		#undef _str_word0_xfer
	#endif
	#ifdef _str_word1_xfer
		#undef _str_word1_xfer
	#endif

	// first of 4 possible xbufs
	#define_eval _RELATIVE_SBYTE _START_BYTE
	#if (_WORD0 < _cur_xbuf_d_size)
		#define_eval _str_word0_xfer _cur_xbuf_d_name/**/_WORD0/**/

	#else

		#define_eval _WORD0 (_WORD0 - _cur_xbuf_d_size)
		#define_eval _RELATIVE_SBYTE (_RELATIVE_SBYTE - (_cur_xbuf_d_size * 4))
		_xbuf_d_param_set[_cur_xbuf_d_next]

		// second of 4 possible xbufs
		#if (_WORD0 < _cur_xbuf_d_size)
			#define_eval _str_word0_xfer _cur_xbuf_d_name/**/_WORD0/**/

		#else
			#define_eval _WORD0 (_WORD0 - _cur_xbuf_d_size)
			#define_eval _RELATIVE_SBYTE (_RELATIVE_SBYTE - (_cur_xbuf_d_size * 4))
			_xbuf_d_param_set[_cur_xbuf_d_next]

			// third of 4 possible xbufs
			#if (_WORD0 < _cur_xbuf_d_size)
				#define_eval _str_word0_xfer _cur_xbuf_d_name/**/_WORD0/**/

			#else
				#define_eval _WORD0 (_WORD0 - _cur_xbuf_d_size)
				#define_eval _RELATIVE_SBYTE (_RELATIVE_SBYTE - (_cur_xbuf_d_size * 4))
				_xbuf_d_param_set[_cur_xbuf_d_next]

				// fourth of 4 possible xbufs
				#if (_WORD0 < _cur_xbuf_d_size)
					#define_eval _str_word0_xfer _cur_xbuf_d_name/**/_WORD0/**/
				#else
					#error 2   "xbuf_d_num_extract: start byte is out of range of transfer buffers."
				#endif
			#endif
		#endif
	#endif

	// setup word1 defs
	_xbuf_d_param_set[str_xbuf_d_name]

	#define_eval _WORD1 (_END_BYTE / 4)
	#define_eval _RELATIVE_EBYTE _END_BYTE

	// first of 4 possible xbufs
	#if (_WORD1 < _cur_xbuf_d_size)
		#define_eval _str_word1_xfer _cur_xbuf_d_name/**/_WORD1/**/

	#else
		#define_eval _WORD1 (_WORD1 - _cur_xbuf_d_size)
		#define_eval _RELATIVE_EBYTE (_RELATIVE_EBYTE - (_cur_xbuf_d_size * 4))
		_xbuf_d_param_set[_cur_xbuf_d_next]

		// second of 4 possible xbufs
		#if (_WORD1 < _cur_xbuf_d_size)
			#define_eval _str_word1_xfer _cur_xbuf_d_name/**/_WORD1/**/

		#else
			#define_eval _WORD1 (_WORD1 - _cur_xbuf_d_size)
			#define_eval _RELATIVE_EBYTE (_RELATIVE_EBYTE - (_cur_xbuf_d_size * 4))
			_xbuf_d_param_set[_cur_xbuf_d_next]

			// third of 4 possible xbufs
			#if (_WORD1 < _cur_xbuf_d_size)
				#define_eval _str_word1_xfer _cur_xbuf_d_name/**/_WORD1/**/

			#else
				#define_eval _WORD1 (_WORD1 - _cur_xbuf_d_size)
				#define_eval _RELATIVE_EBYTE (_RELATIVE_EBYTE - (_cur_xbuf_d_size * 4))
				_xbuf_d_param_set[_cur_xbuf_d_next]

				// fourth of 4 possible xbufs
				#if (_WORD1 < _cur_xbuf_d_size)
					#define_eval _str_word1_xfer _cur_xbuf_d_name/**/_WORD1/**/
				#else
					#error 2   "xbuf_d_num_extract: start byte is out of range of transfer buffers."
				#endif
			#endif
		#endif
	#endif

	#define_eval _SBYTE_IN_WD (_RELATIVE_SBYTE & 0x3)
	#define_eval _EBYTE_IN_WD (_RELATIVE_EBYTE & 0x3)

	#if (_WORD0 == _WORD1)
	    field_extract[out_byte_field, _str_word0_xfer, _SBYTE_IN_WD, _EBYTE_IN_WD]
	#else
	    field_dbl_extract[out_byte_field, _str_word0_xfer, _SBYTE_IN_WD, _str_word1_xfer, _EBYTE_IN_WD]
	#endif

#endif		// ((_WORD2 - _WORD1) > 1)
#endm


// _xbuf_d_type_extract 
//		description: extract from dram xbuf based on datatype
//		output:
//			out_byte_field		GPR containing field, right justified	
//		inputs:
//			str_xbuf_name		name of dram transfer register buffer
//			WINDOW_START		start position(in datatype increments) of window or header
//			FIELD_START			start offset(in datatype increments) of field from window start
//			SIZE				size of field 
//			DATATYPE			datatype of the field
//				types supported:		
//#define BIG_ENDIAN_BYTES 0	// data is big-endian in memory
//#define LITTLE_ENDIAN_BYTES 1	// data is little-endian in memory, perform swap
//
//
#macro _xbuf_d_type_extract [out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE, DATATYPE]
#if (DATATYPE == BIG_ENDIAN_BYTES)

	#ifdef LITTLE_ENDIAN
		#undef LITTLE_ENDIAN
		_xbuf_d_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
		#define LITTLE_ENDIAN
	#else
		_xbuf_d_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
	#endif

#elif (DATATYPE == LITTLE_ENDIAN_BYTES)
	#ifndef LITTLE_ENDIAN
		#define LITTLE_ENDIAN
		_xbuf_d_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
		#undef LITTLE_ENDIAN
	#else
		_xbuf_d_extract[out_field, str_xbuf_name, WINDOW_START, FIELD_START, SIZE]
#endif

#else
	#error 2 "_xbuf_d_type_extract. Datatype not supported."
#endif

#endm



#endif // XBUF_UC