//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// rx_pin.uc
// test receive pin side
// 
// run receive in fbox 1
//
#include "stdmac.uc"

.xfer_order $xfer0  $xfer1 $xfer2 $xfer3 $xfer4 $xfer5



	immed32[num_elements, 100]
	immed[const255, 0xFF]
	immed[const256, 0x100]
	immed[const258, 0x102]
	immed[not_first_time, 0]							; first pass through
	alu_shf[bit15on, --, B, 1, <<15]					; setup indirect ov bit to save a cycle
	immed[rx_prev_count, 0]

// set up start, current and end port per context
//
br=ctx[1, context_1#]
br=ctx[2, context_2#]
br=ctx[3, context_3#]

	immed32[data_start, 0x100]
	immed32[status_start, 0x100]
	immed[current_port, 1]
	immed[next_tid, 5]									; for interthread signal
	immed[my_context, 4]
	fast_wr[4, inter_thd_sig]							; signal context 0 to go at first rr avail check
	br[begin#]

context_1#:
	immed32[data_start, 0x10000]
	immed32[status_start, 0x1000]
	immed[current_port, 2]
	immed[next_tid, 6]									; for interthread signal
	immed[my_context, 5]
	br[begin#]

context_2#:
	immed32[data_start, 0x20000]
	immed32[status_start, 0x2000]
	immed[current_port, 3]
	immed[next_tid, 7]									; for interthread signal
	immed[my_context, 6]
	br[begin#]

context_3#:
	immed32[data_start, 0x30000]
	immed32[status_start, 0x3000]
	immed[current_port, 4]
	immed[next_tid, 4]									; for interthread signal
	immed[my_context, 7]
	br[begin#]



begin#:
	alu[status_end, num_elements, +, status_start]
	move[status_addr, status_start]
	move[data_addr, data_start]
	alu[next_tid, bit15on, OR, next_tid, <<5]

loop#:
	alu[--, status_end, -, status_addr]
	br=0[rec_done#]
	
// start read of receive port ready
//
get_rx_ready#:
wait_on_rdy#:
	csr[read, $xfer0, RCV_RDY_CNT]
	csr[read, $xfer1, RCV_RDY_LO], ctx_swap
	alu[rx_count, const255, AND, $xfer0]
	alu[rx_rdy, --, B, $xfer1]

has_count_advanced#:
	alu[rx_count, const256, +, rx_count]				; add 256 to new count to get positive diff if wrap
	alu[diff, rx_count, -, rx_prev_count]				; counter advancing twice means prev xfer is done
	alu[--, diff, -, const258]
	br<0[wait_on_rdy#]

	alu[--, current_port, B, 0]
	alu_shf[--, 1, AND, rx_rdy, >>indirect]				; is current port ready
	br=0[wait_on_rdy#]

	alu[rx_prev_count, const255, AND, $xfer0]					; save the count we just used
	
port_is_ready#:
    alu_shf[this_rr, --, B, my_context, <<6]			; insert thread num into receive request

	ctx_arb[inter_thread]								; wait until prev thread write to rec req is complete
														; because we can't trust rec_req_avail otherwise
rec_req_avail_check#:
    br_inp_state[rec_req_avail, _got_rr_avail#]
    br[rec_req_avail_check#]

_got_rr_avail#:

// present format of RCV_REQ (3/13/98 transactor) is
// +--------+-----+----+-----+-----+-----+----+-----+------+-------+------+
// | fetch 9| msg |stat| elem| elem|seq# |1or2|misc | sig  | thread| port |
// |        | pkt |    | #2  | #1  |     |    |     | sched|       |      |
// |     29 |28:27| 26 |25:22|21:18|17:16| 15 |14:12|  11  | 10:6  | 5:0  |
// +--------+-----+----+-----+-----+-----+----+-----+------+-------+------+
// this will cause FBI to set RCV_CNTL

; write receive request register
    alu[this_rr, this_rr, OR, current_port]				; insert port num
    alu_shf[$xfer0, this_rr, OR, current_port, <<18]	; insert element num, same as port

    csr[write, $xfer0, rcv_req], ctx_swap				; send req to FBI


; wakeup on receive control from fbi
;
	alu[--, --, B, next_tid]
	fast_wr[0, inter_thd_sig], indirect_ref				; signal the next thread to go at rr avail check

	ctx_arb[start_receive]								; wait for fbi signal
; 

; read the rcv_ctrl register 
    csr[read, $xfer7, RCV_CNTL], ctx_swap
	alu[rcv_cmd, --, B, $xfer7]

// format of RCV_CNTL (3/13/98 transactor) is
// +-----+-----+-----+------+----+-----+-----+----+----+-----------+---+---+
// | msg |port |seq# |rxfail|err |elem2|elem1|1or2|seq | validbytes|EOP|SOP|
// |31:30|29:24|23:20|  19  | 18 |17:14|13:10| 9  | 8  |    7:2    | 1 | 0 |
// +-----+-----+-----+------+----+-----+-----+----+----+-----------+---+---+
//
read_rfifo#:
    alu_shf [rfifo_entry, 0xF, AND, rcv_cmd, >>10]		; isolate the RFIFO element number 
    alu_shf [rfifo_entry, 0, b, rfifo_entry, <<3]		; address the rfifo (entry_number * 8) 

// read 3 quadwords of the packet as if to process the header
    r_fifo_rd [$xfer0, rfifo_entry, 0, 3]				; do this just to keep fbi busy

// transfer data to sdram
//
	alu[--, --, B, not_first_time]
	br=0[write_sdram#], defer[1]
	immed[not_first_time, 1]
	ctx_arb[sdram]
write_sdram#:
	alu_shf[--, --, B, rfifo_entry, <<4]				; address the rfifo based on element no.
    sdram[r_fifo_rd, --, data_addr, 0, 8], indirect_ref, sig_done		; write all 8 qw

	alu[data_addr, 8, +, data_addr]						; sdram addr for next element

// format of status word is
// +-----+-------+---------+---+---+----+----+
// |valid| num qw|num bytes|EOP|SOP|skip|port|
// | 31  | 15:13 |  12:10  | 9 | 8 | 7  |6:0 |
// +-----+-------+---------+---+---+----+----+
//
	ld_field_w_clr[status, 0010, $xfer7, <<8]			; byte count, eop, sop from rcv_cntl 7:0 to 15:8
	alu_shf[status, status, OR, 1, <<31]				; make it valid for transmit ucode
	alu_shf[$status, status, +4, $xfer7, >>24]			; OR in the port number, (output port = input port)  
	
	sram[write, $status, status_addr, 0, 1]				; write it to sram
	alu[status_addr, 1, +, status_addr]					; update status addr for next element
status_queued#:

	br[loop#]											;normal loop iterate


// you go here after receiving num_elements elements	(see .ind)
rec_done#:
	br[rec_done#]
