;------------------------------------------------------------------------------------
;                                                                      
;                   I N T E L   P R O P R I E T A R Y                   
;                                                                       
;      COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;      THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                       
;                         INTEL  CORPORATION                            
;                                                                      
;                      2200 MISSION COLLEGE BLVD                        
;                                                                       
;                SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                       
;------------------------------------------------------------------------------------
; transmit pin side test
; dfh 3/25/98
;

; registers
;	data_addr		sdram address of packet data 64 byte chunk
;	status_addr		sram address of status corresponding to 64 byte data chunk
;						note: data is in this chunk, but can be any number of bytes 1-64
;
#include "stdmac.uc"

.xfer_order $xfer0 $xfer1 $xfer2 $xfer3
.operand_synonym $status_word0 $xfer2

	immed32[num_elements, 100]
	immed[const_128, 128]
	immed[$xfer3, 0]								; to setup second status word
	alu_shf[validate_override, --, B, 1, <<15]		; setup indirect ov bit to save a cycle
	alu_shf[refcount_override, --, B, 1, <<20]		; indirect refcount override bit
	br=ctx[1, context1#]
	br=ctx[2, context2#]
	br=ctx[3, context3#]

	immed32[data_start, 0x100]
	immed32[status_start, 0x100]
	immed[@current_inptr, 0]
	br[begin#]
context1#:
	immed32[data_start, 0x10000]
	immed32[status_start, 0x1000]
	br[begin#]
context2#:
	immed32[data_start, 0x20000]
	immed32[status_start, 0x2000]
	br[begin#]
context3#:
	immed32[data_start, 0x30000]
	immed32[status_start, 0x3000]

begin#:
	alu[status_end, num_elements, +, status_start]
	move[status_addr, status_start]
	move[data_addr, data_start]




; procedure:
;	read xmit ready and xmit_valid_outptr
; loop
;	while local_outptr not greater than xmit_valid_outptr
;		read status for next packet
;		transfer the chunk to tfifo
;		write the status (ctx_arb)
;		write xmit validate

; transfer
;
send_loop#:


	alu[--, status_addr, -, status_end]
	br>0[done#]

// get the status word for an element
// repeat if not valid (not yet written by receive)
get_status#:
	alu_shf[$status_word, --, B, 1, <<31]
	sram[bit_wr, $status_word, status_addr, 0, test_and_clear_bits], ctx_swap	; retrieve status
	alu_shf[--, 1, AND, $status_word, >>31]
	br=0[get_status#]

	alu[status_addr, status_addr, +, 1]

; get inptr and update it for next thread
incr_inptr#:
	alu[current_inptr, --, B, @current_inptr]
	alu[tempa, 1, +,current_inptr]					; increment ptr
	br!=byte[tempa, 0, 16, incr_inptr_done#]
	immed[tempa, 0]							; if 16, wrap around to 0
incr_inptr_done#:
	alu[@current_inptr, --, B, tempa]				; put it back for next thread

; transfer the element to the tfifo
transfer_data#:

	alu_shf[qw, 0x7, AND, $status_word, >>13]							; number of quadwords -1

; <<4 (field) plus <<3 (x8)
	alu_shf[temp, refcount_override, OR, current_inptr, <<7]			; put element no. in indirect operand
	alu_shf[--, temp, OR, qw, <<16]										; insert number of quadwords	

//    sdram[t_fifo_wr, --, data_addr, 0, 8], indirect_ref, ctx_swap		; transfer to tfifo
    sdram[t_fifo_wr, --, data_addr, 0, 8], indirect_ref, sig_done		; transfer to tfifo
// note: gain 2.5% performance by removing ctx_swap and doing ctx_arb[sdram] later
	alu[data_addr, data_addr, +, 8]										; 64 byte element increment



got_tx_ready#:
	alu[port, 0x1f, AND, $status_word]
	alu[temp, --, B, @tx_rdy]
	alu[--, port, B, 0]
	alu_shf[--, 1, AND, temp, >>indirect]					; is the port ready
	br=0[get_tx_ready#]

	alu[--, --, B, @not_first_time]
	br=0[write_status#], defer[1]
	alu[temp, --, B, @tx_outptr]

	alu[--, temp, -, current_inptr]							; is element available
	br=0[get_tx_ready#], defer[1]
	immed[@not_first_time, 1]

; write the status word for an element
write_status#:
	alu[$status_word0, --, B, $status_word]
	alu_shf[status_tfifo_addr, const_128, +, current_inptr, <<1]	; 128 + current_inptr <<1
    t_fifo_wr [$status_word0, status_tfifo_addr, 0, 1],  ctx_swap
	ctx_arb[sdram]
status_sent#:
	alu_shf[--, validate_override, OR, current_inptr, <<5]			; setup indirect data from element no.
	fast_wr[0, XMIT_VALIDATE], indirect_ref							; tell fbi to run with that element no.
	br[send_loop#]




get_tx_ready#:
	csr[read, $xfer0, XMIT_PTR]
	csr[read, $xfer1, XMIT_RDY_LO], ctx_swap
	alu[@tx_outptr, --, B, $xfer0]
	br[got_tx_ready#], defer[1]
	alu[@tx_rdy, --, B, $xfer1]

done#:
	ctx_arb[voluntary]
	br[done#]
