;------------------------------------------------------------------------------------
;                                                                      
;                   I N T E L   P R O P R I E T A R Y                   
;                                                                       
;      COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
;      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
;      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
;      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
;      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
;      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
;      THE PRIOR WRITTEN PERMISSION OF :                                
;                                                                       
;                         INTEL  CORPORATION                            
;                                                                      
;                      2200 MISSION COLLEGE BLVD                        
;                                                                       
;                SANTA  CLARA,  CALIFORNIA  95052-8119                  
;                                                                       
;------------------------------------------------------------------------------------
; tx_pinx8
; transmit pin side test, 8 ports
; dfh 3/25/98


// uca usage:
//
// for fbox 0
// uca -D TX0 -o tx_pin0.list tx_pinx8
// for fbox 1
// uca -D TX1 -o tx_pin1.list tx_pinx8

; registers
;	data_addr		sdram address of packet data 64 byte chunk
;	status_addr		sram address of status corresponding to 64 byte data chunk
;						note: data is in this chunk, but can be any number of bytes 1-64
;

.xfer_order $xfer0 $xfer1 $xfer2 $xfer3
.operand_synonym $status_word0 $xfer2
//	immed[$xfer0, 0x200]
//	csr [write, $xfer0, RDYBUS_TEMPLATE_CTL]

#ifdef TX0
	immed[my_thread_base, 0]
	immed[hibox, 0]
#endif

#ifdef TX1
	immed[my_thread_base, 4]
	immed[hibox, 1]
#endif

// when receive threads are in fbox 0-2, use these
#ifdef TX3
	immed[my_thread_base, 12]
	immed[hibox, 0]
#endif

#ifdef TX4
	immed[my_thread_base, 16]
	immed[hibox, 1]
#endif

	immed[const_128, 128]
	immed[const8, 8]
	immed[@hiptr, 0]
	immed[$xfer3, 0]								; to setup second status word
	alu_shf[validate_override, --, B, 1, <<15]		; setup indirect ov bit to save a cycle
	alu_shf[bit15on, --, B, 1, <<15]				; setup indirect ov bit to save a cycle
	alu_shf[refcount_override, --, B, 1, <<20]		; indirect refcount override bit
	alu[data_addr, --, B, data_start]
	alu[status_addr, --, B, status_start]

// 
// ind will set the values for 
//		data_start, status_start and status_end, 
//
	br=ctx[0, context_0#]
	br=ctx[1, context_1#]
	br=ctx[2, context_2#]
	br=ctx[3, context_3#]


context_0#:
	alu[my_tid, --, B, my_thread_base]
	alu[next_tid, 1, +, my_thread_base]
	alu[next_tid, bit15on, OR, next_tid, <<5]
	br[send_loop#]
context_1#:
	alu[my_tid, 1, +, my_thread_base]
	alu[next_tid, 2, +, my_thread_base]
	alu[next_tid, bit15on, OR, next_tid, <<5]
	br[send_loop#]
context_2#:
	alu[my_tid, 2, +, my_thread_base]
	alu[next_tid, 3, +, my_thread_base]
	alu[next_tid, bit15on, OR, next_tid, <<5]
	br[send_loop#]
context_3#:
	alu[my_tid, 3, +, my_thread_base]

#ifdef TX0
	immed[next_tid, 4]
	alu[next_tid, bit15on, OR, next_tid, <<5]
#endif

#ifdef TX1
	immed[next_tid, 0]
	alu[next_tid, bit15on, OR, next_tid, <<5]
#endif

// when receive threads are in fbox 0-2, use these
#ifdef TX3
	alu[next_tid, 4, +, my_thread_base]
#endif

#ifdef TX4
	immed[next_tid, 12]
#endif


; procedure:
;	read xmit ready and xmit_valid_outptr
; loop
;	while local_outptr not greater than xmit_valid_outptr
;		read status for next packet
;		transfer the chunk to tfifo
;		write the status (ctx_arb)
;		write xmit validate

; transfer
;
send_loop#:
; current inptr
	alu[--, hibox, OR, @not_first_timeptr]
	br=0[got_signal#], defer[1]
	immed[@not_first_timeptr, 1]
wait_for_sig#:
	ctx_arb[inter_thread]							; wait for prevous thread to signal
got_signal#:
	alu[current_inptr, @hiptr, OR, my_tid]
	alu[--, --, B, next_tid]
	fast_wr[0, inter_thd_sig], indirect_ref			; signal the next thread to go
	br!=ctx[3, got_inptr#]
	alu[@hiptr, @hiptr, XOR, const8]				; flip the hiptr if ctx 3
got_inptr#:

; get the status word for an element
get_status#:
	alu[--, status_addr, -, status_end]
	br>0[done#]

	sram[read, $status_word, status_addr, 0, 1], ctx_swap, defer[1]
	alu[status_addr, status_addr, +, 1]


; transfer the element to the tfifo
transfer_data#:

	alu_shf[qw, 0x7, AND, $status_word, >>13]							; number of quadwords -1

; <<4 (field) plus <<3 (x8)
	alu_shf[temp, refcount_override, OR, current_inptr, <<7]			; put element no. in indirect operand
	alu_shf[--, temp, OR, qw, <<16]										; insert number of quadwords	

//    sdram[t_fifo_wr, --, data_addr, 0, 8], indirect_ref, ctx_swap		; transfer to tfifo
    sdram[t_fifo_wr, --, data_addr, 0, 8], indirect_ref, sig_done		; transfer to tfifo
// note: gain 2.5% performance by removing ctx_swap and doing ctx_arb[sdram] later
	alu[data_addr, data_addr, +, 8]										; 64 byte element increment



got_tx_ready#:
	alu[port, 0x1f, AND, $status_word]
	alu[temp, --, B, @tx_rdy]
	alu[--, port, B, 0]
	alu_shf[--, 1, AND, temp, >>indirect]					; is the port ready
	br=0[get_tx_ready#]

	alu[--, --, B, @not_first_time]
	br=0[write_status#], defer[1]
	alu[temp, --, B, @tx_outptr]

	alu[--, temp, -, current_inptr]							; is element available
	br=0[get_tx_ready#], defer[1]
	immed[@not_first_time, 1]

; write the status word for an element
write_status#:
	alu[$status_word0, --, B, $status_word]
	alu_shf[status_tfifo_addr, const_128, +, current_inptr, <<1]	; 128 + current_inptr <<1
    t_fifo_wr [$status_word0, status_tfifo_addr, 0, 1],  ctx_swap
	ctx_arb[sdram]
status_sent#:
	alu_shf[--, validate_override, OR, current_inptr, <<5]		; setup indirect data from element no.
	fast_wr[0, XMIT_VALIDATE], indirect_ref								; tell fbi to run with that element no.
	br[send_loop#]




get_tx_ready#:
	csr[read, $xfer0, XMIT_PTR]
	csr[read, $xfer1, XMIT_RDY_LO], ctx_swap
	alu[@tx_outptr, --, B, $xfer0]
	br[got_tx_ready#], defer[1]
	alu[@tx_rdy, --, B, $xfer1]

done#:

	alu_shf[$status_word0, --, B, 1, <<7]							; skip bit
	alu_shf[status_tfifo_addr, const_128, +, current_inptr, <<1]	; 128 + current_inptr <<1
    t_fifo_wr [$status_word0, status_tfifo_addr, 0, 1],  ctx_swap

	alu_shf[--, validate_override, OR, current_inptr, <<5]		; setup indirect data from element no.
	fast_wr[0, XMIT_VALIDATE], indirect_ref								; tell fbi to run with that element no.
	br[send_loop#]



//	br_!signal[inter_thread_sig, done#]
//pass_token#:
//	alu[--, --, B, next_tid]
//	fast_wr[0, inter_thd_sig], indirect_ref			; signal the next thread to go
//	br[done#]

	
