//rx_ether100m.uc

// Debugging in Hardware:
// 		Remove VALIDATION_CARD in build settings for microengines 2 and 3
//		This allows only 8 receive threads to run, for ports 0-7.

// Debugging in Simulation (present settings)
// 		Define VALIDATION_CARD in build settings for microengines 2 and 3.
//		This allows 16 receive threads to run, for ports 0-15.
	


#include "ports_validcard.h"
#include "mem_map.h"
#include "ixplib.uc"


// packet buffers
#define DESC_SIZE 4						// size of descriptor
#define PKBUF_SIZE 256					// quadword size of packet buffer

// even receive microengines use freelist 0 and low sdram bank
#if ((UENGINE_ID & 1) == 0)
	#define FREELIST_ID 0
	#define FREELIST_HANDLE FREELIST_ID, SDRAM_PKT_BUFFER_BASE, PKBUF_SIZE, SRAM_BUFF_DESCRIPTOR_BASE, DESC_SIZE
#else
	#ifdef ALT_BANKS
// odd receive microengines use freelist 1 and high sdram bank
		#define FREELIST_ID 1
		#define FREELIST_HANDLE FREELIST_ID, SDRAM_PKT_BUFFER_HIGH_BASE, PKBUF_SIZE, SRAM_BUFF_DESCRIPTOR_BASE, DESC_SIZE
	#else
		#define FREELIST_ID 0
		#define FREELIST_HANDLE FREELIST_ID, SDRAM_PKT_BUFFER_BASE, PKBUF_SIZE, SRAM_BUFF_DESCRIPTOR_BASE, DESC_SIZE
	#endif
#endif

#define Q_HANDLE Q_LINKED_LIST_8, XMIT_PWP_VECTOR, SRAM_QUEUE_DESCRIPTOR_BASE, SRAM_BUFF_DESCRIPTOR_BASE

// port to context assignments
#if (UENGINE_ID == 0)
#define PORT_NUMS 0,1,2,3
#elif (UENGINE_ID == 1)
#define PORT_NUMS 4,5,6,7
#elif (UENGINE_ID == 2)
#define PORT_NUMS 8,9,10,11
#elif (UENGINE_ID == 3)
#define PORT_NUMS 12,13,14,15
#endif

#define CORE_STACK_INTF1 18						// output interface for queueing to core


// initializations
sig_ifctx_block(0, inter_thread, OR, 1);		// block until signal global initialization complete

port_rx_init(rec_req, rfifo_addr, UENGINE_ID, PORT_NUMS);	// create a canned receive request
critsect_init(@rdready_inflight);			// init semaphore to limit 1 rec ready read per engine
critsect_init(@req_inflight);				// init semaphore to limit 1 rec req per engine

immed32(route_base, SDRAM_ROUTE_FWD_BASE);					// init table addresses	
move(rec_state, 0);											// init rec_state
immed32(packet_buf_addr, UNALLOCATED);						// address to pop from freelist
xbuf_alloc($$dxfer, 8);					// allocate sdram xfers as a block of 8


// A validation card would have 16 100M ports. For simulation, define VALIDATION_CARD to enable
//	receive microengines 0-3, ports 0-15.
//
// the eval card only has 8 100M ports. For eval card hardware do not define VALIDATION_CARD.
//	This will enable receive icroengines 0-1, ports 0-7.
//
#ifndef VALIDATION_CARD		
#if (UENGINE_ID > 1)
	ctx_arb[kill]
	nop
	nop
	nop
#endif
#endif

// main loop   
.while (1)
	xbuf_alloc($pop_xfer, 1);
    .if (packet_buf_addr == UNALLOCATED)
    	buf_pop($pop_xfer[0], FREELIST_HANDLE, sig_done);							// if no buffer, get ptr
    .endif

	port_rxrdy_chk(@rdready_inflight, rec_req);
	critsect_enter[@req_inflight]						; block other contexts from sending rec_req
	port_rx_request(rec_req);							// get mpacket
    port_rx_receive(exception, rec_state, ETHER_100M);	// get mpacket status

	mpacket_received#:
																				// wait for mem_pop to complete
    .if (packet_buf_addr == UNALLOCATED)
		buf_wait();
		#if (FREELIST_ID == 0)
			#define BASE_ADDR  SRAM_BUFF_DESCRIPTOR_BASE
		#else 
			#define_eval HALF_BUFFER_COUNT (BUFFER_COUNT / 2)
			#define_eval BASE_ADDR (SRAM_BUFF_DESCRIPTOR_BASE + (HALF_BUFFER_COUNT * 4))
		#endif
	    .while ($pop_xfer[0] == BASE_ADDR)
			buf_pop($pop_xfer[0], FREELIST_HANDLE, ctx_swap);					// if no buffer available, retry
		.endw
		buf_dram_addr_from_sram_addr(packet_buf_addr, $pop_xfer[0], FREELIST_HANDLE);		// calc packet buffer address
		move(descriptor_addr, $pop_xfer[0]);
	.endif
	xbuf_free($pop_xfer);

	.if (exception == PORT_RXFAIL)
		port_rx_fail(rec_state, rec_req, exception);							// fail is like EOP w/o queueing
		buf_dram_addr_from_sram_addr(packet_buf_addr, descriptor_addr, FREELIST_HANDLE);		// calc packet buffer address
		critsect_exit[@req_inflight]						; allow other contexts to send rec_req
		.continue
	.elif (exception == PORT_RXERROR)
		port_rx_error(rec_state, rec_req, exception);							// bump mib counter
		buf_dram_addr_from_sram_addr(packet_buf_addr, descriptor_addr, FREELIST_HANDLE);		// calc packet buffer address
		critsect_exit[@req_inflight]						; allow other contexts to send rec_req
		.continue
	.endif

	critsect_exit[@req_inflight]						; allow other contexts to send rec_req

	.if (!(1 & (rec_state >> REC_STATE_SOP_BIT)))
		port_rx_bytecount_extract(current_bytecount, rec_state);				// update packet byte count
		shf_right(current_qwcount, current_bytecount, 3);
		add(packet_buf_addr, packet_buf_addr, QWCOUNT8);
		dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, current_qwcount, ctx_swap);// write mpacket
	.elif (bit(rec_state, REC_STATE_SOP_BIT))		// if SOP
		xbuf_alloc($pkt_buf, 4);													// allocate xfer buffer 4 lwords
		nop

		rfifo_read($pkt_buf[0], rfifo_addr, QWOFFSET0, QWCOUNT2, ctx_swap);				// read link layer, 1st 2 quadwords
		xbuf_extract(protocol_len, $pkt_buf, BYTEOFFSET0, ETHER_PROTOCOL_LENGTH); // get protocol length field
#ifdef LITTLE_ENDIAN
		shf_right(sa01, $pkt_buf[1], 16);
#else
		alu_op(sa01, 0, +16, $pkt_buf[1]);										// save sa bytes 0-1 for later merge
#endif
		.if(protocol_len > ETHER_MTU)				// if ethernet				// see RFC 1122, 2.3.3
			xbuf_extract(ip_verslen, $pkt_buf, BYTEOFFSET14, IP_VERSION_LENGTH); // get version/length field
			.if (ip_verslen == IPV4_NO_OPTIONS)		// if ipv4 no options		// see RFC 791

				xbuf_alloc($pkt_buf_ip, 4);										// allocate xfer buffer 4 lwords
				xbuf_link($pkt_buf, $pkt_buf_ip);								// linke the xfer buffers
				xbuf_link($pkt_buf_ip, $pkt_buf);								// create circular buffer

				dram_rfifo_read(packet_buf_addr, QWOFFSET4, rfifo_addr, QWOFFSET4, QWCOUNT4, sig_done);// write 2nd 32B

    			rfifo_read($pkt_buf_ip[0], rfifo_addr, QWOFFSET2, QWCOUNT3, ctx_swap);	// read next 3 quadwords
				ip_verify(exception, $pkt_buf, BYTEOFFSET14);					// check ip header
				ctx_arb[voluntary]
				ip_modify($$dxfer, BYTEOFFSET14, $pkt_buf, BYTEOFFSET14);			// modifiy ip packet
				xbuf_extract(ip_da, $pkt_buf, BYTEOFFSET14, IP_DESTINATION_ADDRESS); // get ip dest field
				xbuf_free($pkt_buf_ip);

				.if (exception)
					packet_late_discard#:
					port_rx_discard(rec_state, rec_req, exception);				// set discard, inc counter
					sig_wait(sdram, OR, 1);
				.else								// not ip exception, do lookup
					ip_route_lookup(rt_ptr, ip_da, SRAM_ROUTE_LOOKUP_BASE, 5);	// longest prefix match, trie depth 5
					sig_wait(sdram, OR, 1);
					dram_read($$dxfer[0], route_base, rt_ptr, QWCOUNT3, ctx_swap);	// get output_intf, MAC DA

					// $$dxfer0 = output port, $$dxfer1 = MAC DA bytes 0-3, $$dxfer2 = MAC DA bytes 4-5
					shf_left(output_intf, $$dxfer[0], 3);								// save for enqueue
					move($$dxfer[0], $$dxfer[1]);									// new DA bytes 0-3
#ifdef LITTLE_ENDIAN	
					add_shf_left($$dxfer[1], $$dxfer[2], sa01, 16);				// merge new DA 4-5 with SA 0-1
#else
					add_shf_left($$dxfer[1], sa01, $$dxfer[2], 16);				// merge new DA 4-5 with SA 0-1
#endif
					move($$dxfer[2], $pkt_buf[2]);									// previous SA bytes 2-5
					dram_write($$dxfer[0], packet_buf_addr, QWOFFSET0, QWCOUNT4, no_signal);	// write modified packet
				.endif   							// no ip exception do lookup
			.else									// IP with options or frag
				dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, QWCOUNT8, ctx_swap);
				shf_left(output_intf, CORE_STACK_INTF1, 3);							// if options or frag, q to core
			.endif									// IP with options or frag
		.else										// not ethernet
			dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, QWCOUNT8, ctx_swap);
			shf_left(output_intf, CORE_STACK_INTF1, 3);								// if not ethernet, q to core
		.endif									// not ethernet
	.endif											// not SOP
		xbuf_free($pkt_buf);


	.if (bit(rec_state, REC_STATE_EOP_BIT))			// if EOP
		.if (!bit(rec_state, REC_STATE_DISCARD_BIT))		// if not discard
			xbuf_alloc($desc_buf, 2);
			move($desc_buf[0], 0);														// descriptor to xfer
			.local com_descriptor
				shf_left(com_descriptor, FREELIST_ID, DESC1_FREELIST);					// insert freelist id
				#if (DESC1_PKT_END_BYTE <= REC_STATE_BYTE_COUNT);
					#define_eval _SHIFT_AMT  (REC_STATE_BYTE_COUNT - DESC1_PKT_END_BYTE)
					field_shf_right_insert(com_descriptor, 0010, rec_state, _SHIFT_AMT);// insert last byte count
				#else
					#define_eval _SHIFT_AMT  (DESC1_PKT_END_BYTE - REC_STATE_BYTE_COUNT)
					field_shf_left_insert(com_descriptor, 0010, rec_state, _SHIFT_AMT);
				#endif
				alu_op(packet_buf_addr, QWCOUNT8, +8, packet_buf_addr);					// incr ele countex
				alu_shf_right($desc_buf[1], com_descriptor, +8, packet_buf_addr, 3); 	// insert element count
			.endlocal
			sram_write($desc_buf[0], descriptor_addr, LWOFFSET0, LWCOUNT2, ctx_swap);	// write descriptor
enqueue#:
			packetq_send(descriptor_addr, output_intf, Q_HANDLE); 						// at eop, q to xmit
			immed32(packet_buf_addr, UNALLOCATED);
			xbuf_free($desc_buf);
		.else											// Discard
			// set packet_buf_addr to begining of the packet - Reuse the packet
			buf_dram_addr_from_sram_addr(packet_buf_addr, descriptor_addr, FREELIST_HANDLE);
		.endif												// not discard
		move(rec_state, 0);															// init rec_state
	.endif											// EOP

header_stored#:
mid_mpacket_stored#:
last_mpacket_stored#:
.endw
xbuf_free($$dxfer);
nop


