//rx_ether1gig.uc

#include "ports_evalcard1.h"
#include "mem_map.h"
#include "ixplib.uc"
#include "project_config.h"

// define macros to be used

// if this was not SOP thread. send output interface to EOP thread
//
#macro Rx_SaveOutputIntf(in_rec_state, in_output_port)
.local sopseq temp_state
	alu_shf_left(temp_state, in_rec_state, AND, 1, REC_STATE_DISCARD_BIT); 	// get discard bit
	alu_op(temp_state, temp_state, OR, in_output_port);						// or in discard bit
	alu_shf_right(sopseq, 0xF, AND, in_rec_state, REC_STATE_SOPSEQ);
	#if (UENGINE_ID < 2)
		mailbox_send(temp_state, sram, RX_FPORT1_OUTPORT, sopseq, ctx_swap);	// send output interface to eop thread
	#else
		mailbox_send(temp_state, sram, RX_FPORT2_OUTPORT, sopseq, ctx_swap);	// send output interface to eop thread
	#endif
.endlocal
#endm


// if this was not SOP thread. get output interface from SOP thread
//
#macro Rx_RestoreOutputIntf(out_output_intf, in_rec_state)
xbuf_alloc($classify_state, 1);
.local sopseq
	alu_shf_right(sopseq, 0xF, AND, in_rec_state, REC_STATE_SOPSEQ);
	#if (UENGINE_ID < 2)
		mailbox_receive($classify_state[0], sram, RX_FPORT1_OUTPORT, sopseq, retry);
	#else
		mailbox_receive($classify_state[0], sram, RX_FPORT2_OUTPORT, sopseq, retry);
	#endif
	move(out_output_intf, $classify_state[0]);								// queue spacing
.endlocal
xbuf_free($classify_state);
#endm



br!=ctx[0, set_next_tid1#]
immed32(next_tid, NEXT_CTX0_TID);

set_next_tid1#:
br!=ctx[1, set_next_tid2#]
immed32(next_tid, NEXT_CTX1_TID)
 
set_next_tid2#:
br!=ctx[2, set_next_tid3#]
immed32(next_tid, NEXT_CTX2_TID)

set_next_tid3#:
br!=ctx[3, set_next_tid_done#]
immed32(next_tid, NEXT_CTX3_TID)

set_next_tid_done#:




// initializations
sig_ifctx_block(0, inter_thread, OR, 1);	// block until signal global initialization complete

port_rx_init(rec_req, rfifo_addr, UENGINE_ID, PORT_NUMS);						// create a canned receive request

// interthread mailbox setup
// rec_req csr 10:6 has thread id which is used for mailbox offset of 16 mailboxes
//
alu_shf_right(my_mpacket_mb, 0x7, AND, rec_req, 6);

#if (UENGINE_ID < 2)								// first fast port
	// get mailbox offsets 0-7 from thread if low 3 bits
	alu_op(my_mpacket_mb, my_mpacket_mb, +, RX_FPORT1_MPACKET_STATE);
#else												// second fast port
	// get mailbox offsets 0-7 from thread if low 3 bits
	alu_op(my_mpacket_mb, my_mpacket_mb, +, RX_FPORT2_MPACKET_STATE);
#endif

#if ((UENGINE_ID == 0) || (UENGINE_ID == 2))
alu_op(next_thd_mpacket_mb, my_mpacket_mb, +, 4);						// 0-3 next = 4-7
#elif ((UENGINE_ID == 1) || (UENGINE_ID == 3))
alu_op(next_thd_mpacket_mb, my_mpacket_mb, -, 3);						// 4-7 next = 1-4
alu_op(next_thd_mpacket_mb, next_thd_mpacket_mb, and, 0xFF3);			// 4-7 next = 1,2,3,0
#endif


immed32(route_base, SDRAM_ROUTE_FWD_BASE);										// init table addresses	
immed32(rec_state, 0);
immed32(packet_buf_addr, UNALLOCATED);											// address to pop from freelist
xbuf_alloc($$dxfer, 8);					// allocate sdram xfers as a block of 8

// initialize the output port mailboxes to invalid
// this should be done by  Core and by initialization script
xbuf_alloc($init_val, 1)
.local count
immed32(count, 0);
immed32($init_val[0], 0);
.while (count < 32)
	sram_write($init_val[0], RX_FPORT1_OUTPORT, count, 1, ctx_swap);
	add(count, count, 1);
.endw
.endlocal
sram_write($init_val[0], my_mpacket_mb, 0, 1, ctx_swap);		// init my mpacket mb to invalid
xbuf_free($init_val)


#if (UENGINE_ID == 0)	// context 0 for fast port 1 will start the round robin sequence
	sig_ifctx_post(0, inter_thread, 0);			// if thread 0 or 8, signal self
	br!=ctx[0, fast_port_init_done#]
	fast_wr[3, INCR_ENQ_NUM1]					// increment fast port 1 seq number
    mailbox_send(0x80000000, sram, my_mpacket_mb, 0, ctx_swap);	// validate my 1st mb
#endif
#if (UENGINE_ID == 2)	// context 0 for fast port 2 will start the round robin sequence
	sig_ifctx_post(0, inter_thread, 8);			// if thread 0 or 8, signal self
	br!=ctx[0, fast_port_init_done#]
	fast_wr[3, INCR_ENQ_NUM2]					// increment fast port 1 seq number
    mailbox_send(0x80000000, sram, my_mpacket_mb, 0, ctx_swap);	// validate my 1st mb
#endif
fast_port_init_done#:

critsect_init(@req_inflight);					// init semaphore to limit 1 rec req per engine

// main loop   
.while (1)
	xbuf_alloc($pop_xfer, 1);
    .if (next_buf_index == UNALLOCATED)
    	buf_pop($pop_xfer[0], FREELIST_HANDLE, sig_done);				// if no buffer, get ptr
    .endif

	sig_wait(inter_thread, OR, 1);										// wait on previous rec thread
	critsect_enter[@req_inflight]						// block other contexts from sending rec_req
	port_rx_request(rec_req);							// get mpacket
	sig_post(inter_thread, next_tid);
    port_rx_receive(exception, rec_state, FAST_PORT);	// get mpacket status


mpacket_received#:
																		// wait for mem_pop to complete
    .if (next_buf_index == UNALLOCATED)
		buf_wait();
		#if (FREELIST_ID == 0)
			#define BASE_ADDR  SRAM_BUFF_DESCRIPTOR_BASE
		#else 
			#define_eval HALF_BUFFER_COUNT (BUFFER_COUNT / 2)
			#define_eval BASE_ADDR (SRAM_BUFF_DESCRIPTOR_BASE + (HALF_BUFFER_COUNT * 4))
		#endif
	    .while ($pop_xfer[0] == BASE_ADDR)
			buf_pop($pop_xfer[0], FREELIST_HANDLE, ctx_swap);			// if no buffer available, retry
		.endw
		buf_index_from_sram_addr(next_buf_index, $pop_xfer[0], FREELIST_HANDLE);		// calc packet buffer address
	.endif
	xbuf_free($pop_xfer);


	xbuf_alloc($mpacket_mb, 1);

	.local mailbox_data
	.if (exception == PORT_RXCANCEL)
		// pass sopseq and address to next thread
		mailbox_receive($mpacket_mb[0], sram, my_mpacket_mb, 0, retry);			// read and invalidate my mb
		mailbox_send($mpacket_mb[0], sram, next_thd_mpacket_mb, 0, ctx_swap);	// write and validate next mb
		critsect_exit[@req_inflight]						// allow other contexts to send rec_req
		.continue
	.elif (exception == PORT_RXFAIL)
		port_rx_fail(rec_state, rec_req, exception);							// fail is like EOP w/o queueing
		mailbox_receive($mpacket_mb[0], sram, my_mpacket_mb, 0, retry);			// read and invalidate my mb
		mailbox_send(0, sram, next_thd_mpacket_mb, 0, ctx_swap);				// validate next mb
		// push buffer back if fail
		buf_index_from_dram_addr(mailbox_data, packet_buf_addr, FREELIST_HANDLE);
		buf_push(mailbox_data, 0, FREELIST_HANDLE);
		critsect_exit[@req_inflight]						// allow other contexts to send rec_req
		.continue
	.elif (exception == PORT_RXERROR)
		port_rx_error(rec_state, rec_req, exception);							// bump mib counter
		// pass sopseq and address to next thread
		mailbox_receive($mpacket_mb[0], sram, my_mpacket_mb, 0, retry);			// read and invalidate my mb
		alu_shf_left(mailbox_data, $mpacket_mb[0], OR, 1, REC_STATE_DISCARD_BIT);	// set discard
		mailbox_send(mailbox_data, sram, next_thd_mpacket_mb, 0, ctx_swap);		// validate next mb
		critsect_exit[@req_inflight]						// allow other contexts to send rec_req
		.continue
	.endif

	critsect_exit[@req_inflight]						// allow other contexts to send rec_req


	.if (bit(rec_state, REC_STATE_EOP_BIT))									// if EOP and SOP
		mailbox_send(0, sram, next_thd_mpacket_mb, 0, no_signal);			// validate next mb
	.endif

	// write to next thread's mailbox freelist/sopseq and element count (bits 7:0)
	.if (bit(rec_state, REC_STATE_SOP_BIT))
		buf_dram_addr_from_index(packet_buf_addr, next_buf_index, FREELIST_HANDLE);
		buf_sram_addr_from_index(descriptor_addr, next_buf_index, FREELIST_HANDLE);
		immed32(next_buf_index, UNALLOCATED);

		alu_shf_left(rec_state, rec_state, OR, FREELIST_ID, REC_STATE_FREELIST_ID);		// insert freelist_id
		.if (bit(rec_state, REC_STATE_EOP_BIT))									// if EOP and SOP
			mailbox_receive($mpacket_mb[0], sram, my_mpacket_mb, 0, retry);		// read and invalidate my mb
		.else																	// if just SOP
			move($mpacket_mb[0], 0);
			alu_shf_right(mailbox_data, 1, +, packet_buf_addr, 3)				// increment element count
			field_insert(mailbox_data, 1000, rec_state);						// merge in sop seq
			mailbox_receive($mpacket_mb[0], sram, my_mpacket_mb, 0, retry);		// read and invalidate my mb
			mailbox_send(mailbox_data, sram, next_thd_mpacket_mb, 0, ctx_swap);	// write and validate next mb
		.endif
	.elif (bit(rec_state, REC_STATE_EOP_BIT))
		mailbox_receive($mpacket_mb[0], sram, my_mpacket_mb, 0, retry);			// read and invalidate my mb
		field_insert(rec_state, 1000, $mpacket_mb[0]);							// merge in original sop seq
		field_clr_shf_left_insert(packet_buf_addr, 0111,$mpacket_mb[0], 3);		// convert 64B to 8B addr
	.else																		// not SOP not EOP
		mailbox_receive($mpacket_mb[0], sram, my_mpacket_mb, 0, retry);			// read and invalidate my mb
		alu_op(mailbox_data, $mpacket_mb[0], +, 1)								// increment element count
		mailbox_send(mailbox_data, sram, next_thd_mpacket_mb, 0, ctx_swap);		// write and validate next mb
		field_insert(rec_state, 1000, mailbox_data);							// merge in original sop seq
		field_clr_shf_left_insert(packet_buf_addr, 0111,$mpacket_mb[0], 3);		// convert 64B to 8B addr
	.endif
	.endlocal	// mailbox_data
	xbuf_free($mpacket_mb);

	.if (bit(rec_state, REC_STATE_SOP_BIT))		// if SOP
process_header#:

		xbuf_alloc($pkt_buf, 4);
		nop													// allocate xfer buffer 4 lwords
		rfifo_read($pkt_buf[0], rfifo_addr, QWOFFSET0, QWCOUNT2, ctx_swap);				// read link layer, 1st 2 quadwords
		xbuf_extract(protocol_len, $pkt_buf, BYTEOFFSET0, ETHER_PROTOCOL_LENGTH); // get protocol length field
#ifdef LITTLE_ENDIAN
		shf_right(sa01, $pkt_buf[1], 16);
#else
		alu_op(sa01, 0, +16, $pkt_buf[1]);										// save sa bytes 0-1 for later merge
#endif
		.if(protocol_len > ETHER_MTU)				// if ethernet				// see RFC 1122, 2.3.3
			xbuf_extract(ip_verslen, $pkt_buf, BYTEOFFSET14, IP_VERSION_LENGTH); // get version/length field
			.if (ip_verslen == IPV4_NO_OPTIONS)		// if ipv4 no options		// see RFC 791

				xbuf_alloc($pkt_buf_ip, 4);										// allocate xfer buffer 4 lwords
				xbuf_link($pkt_buf, $pkt_buf_ip);								// linke the xfer buffers
				xbuf_link($pkt_buf_ip, $pkt_buf);								// create circular buffer

				dram_rfifo_read(packet_buf_addr, QWOFFSET4, rfifo_addr, QWOFFSET4, QWCOUNT4, sig_done);// write 2nd 32B

    			rfifo_read($pkt_buf_ip[0], rfifo_addr, QWOFFSET2, QWCOUNT3, ctx_swap);	// read next 3 quadwords
				ip_verify(exception, $pkt_buf, BYTEOFFSET14);					// check ip header
				ctx_arb[voluntary]
				ip_modify($$dxfer, BYTEOFFSET14, $pkt_buf, BYTEOFFSET14);			// modifiy ip packet
				xbuf_extract(ip_da, $pkt_buf, BYTEOFFSET14, IP_DESTINATION_ADDRESS); // get ip dest field
				xbuf_free($pkt_buf_ip);

				.if (exception)
					packet_late_discard#:
					port_rx_discard(rec_state, rec_req, exception);				// set discard, inc counter
					sig_wait(sdram, OR, 1);
				.else								// not ip exception, do lookup
					ip_route_lookup(rt_ptr, ip_da, SRAM_ROUTE_LOOKUP_BASE, 5);		// longest prefix match, trie5
					sig_wait(sdram, OR, 1);
					dram_read($$dxfer[0], route_base, rt_ptr, QWCOUNT3, ctx_swap);	// get output_intf, MAC DA

					// $$dxfer0 = output port, $$dxfer1 = MAC DA bytes 0-3, $$dxfer2 = MAC DA bytes 4-5
					shf_left(output_intf, $$dxfer0, 3);							// shift for queue spacing
					.if (!bit(rec_state, REC_STATE_EOP_BIT))					// if not EOP
						Rx_SaveOutputIntf(rec_state, output_intf);
					.endif
					move($$dxfer[0], $$dxfer[1]);								// new DA bytes 0-3
#ifdef LITTLE_ENDIAN	
					add_shf_left($$dxfer[1], $$dxfer[2], sa01, 16);				// merge new DA 4-5 with SA 0-1
#else
					add_shf_left($$dxfer[1], sa01, $$dxfer[2], 16);				// merge new DA 4-5 with SA 0-1
#endif
					move($$dxfer[2], $pkt_buf[2]);									// previous SA bytes 2-5
					dram_write($$dxfer[0], packet_buf_addr, QWOFFSET0, QWCOUNT4, no_signal);	// write modified packet
header_stored#:
				.endif   							// no ip exception do lookup
			.else									// IP with options or frag
				dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, QWCOUNT8, ctx_swap);
				shf_left(output_intf, CORE_STACK_INTF1, 3);							// shift for queue spacing
				.if (!bit(rec_state, REC_STATE_EOP_BIT))			// if not EOP
					Rx_SaveOutputIntf(rec_state, output_intf);
				.endif
			.endif
		.else										// not ethernet
			dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, QWCOUNT8, ctx_swap);
			shf_left(output_intf, CORE_STACK_INTF1, 3);							// shift for queue spacing
			.if (!bit(rec_state, REC_STATE_EOP_BIT))			// if not EOP
				Rx_SaveOutputIntf(rec_state, output_intf);
			.endif
		.endif									// not ethernet
		xbuf_free($pkt_buf);
	.else											// not SOP

		port_rx_bytecount_extract(current_bytecount, rec_state);				// update packet byte count
		shf_right(current_qwcount, current_bytecount, 3);
		dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, current_qwcount, ctx_swap);// write mpacket
		.if (!bit(rec_state, REC_STATE_EOP_BIT))	
			mid_mpacket_stored#:
			immed32(packet_buf_addr, UNALLOCATED);
		.endif
	.endif											// not SOP

	.if (bit(rec_state, REC_STATE_EOP_BIT))			// if EOP
		.if (!bit(rec_state, REC_STATE_SOP_BIT))			// if not SOP
			last_mpacket_stored#:
			Rx_RestoreOutputIntf(output_intf, rec_state);
			.local temp_state
				alu_shf_left(temp_state, output_intf, AND, 1, REC_STATE_DISCARD_BIT);	// isolate discard
				alu_op(rec_state, temp_state, OR, rec_state);							// or in late discard
			.endlocal
			alu_op(output_intf, 0, +8, output_intf);		// just the output intf
			// recover the descriptor address
			buf_index_from_dram_addr(descriptor_addr, packet_buf_addr, FREELIST_HANDLE);
			buf_sram_addr_from_index(descriptor_addr, descriptor_addr, FREELIST_HANDLE);
		.endif


		.if (!bit(rec_state, REC_STATE_DISCARD_BIT))		// if not discard
			enqueue#:
			// get the output interface from thread that processed the header


			xbuf_alloc($desc_buf, 3);


			#if (UENGINE_ID < 2)
				packetq_seq_wait(rec_state, FAST_PORT_LO);
			#else
				packetq_seq_wait(rec_state, FAST_PORT_HI);
			#endif
			.local queue_index
				// these next 2 lines cause port 8&9 to be at 16&17 in the queue array, to be compatible with transmit
				alu_shf_left(queue_index, output_intf, +, 8, 3);
				packetq_index($desc_buf[2], queue_index, SRAM_QUEUE_DESCRIPTOR_BASE, 2, 128);	// get next index
			.endlocal
			.local com_descriptor ele_count
				sub($desc_buf[0], descriptor_addr, SRAM_BUFF_DESCRIPTOR_BASE);		// relative buf addr to xfer

				// build the common descriptor
				alu_shf_right(com_descriptor, 0x3, AND, rec_state, REC_STATE_FREELIST_ID);	// get freelist id
				shf_left(com_descriptor, com_descriptor, DESC1_FREELIST);					// insert freelist id
				#if (DESC1_PKT_END_BYTE <= REC_STATE_BYTE_COUNT);
					#define_eval _SHIFT_AMT  (REC_STATE_BYTE_COUNT - DESC1_PKT_END_BYTE)
					field_shf_right_insert(com_descriptor, 0010, rec_state, _SHIFT_AMT);// insert last byte count
				#else
					#define_eval _SHIFT_AMT  (DESC1_PKT_END_BYTE - REC_STATE_BYTE_COUNT)
					field_shf_left_insert(com_descriptor, 0010, rec_state, _SHIFT_AMT);
				#endif
				alu_op(ele_count, QWCOUNT8, +8, packet_buf_addr);					// incr ele count
				alu_shf_right($desc_buf[1], com_descriptor, +8, ele_count, 3); 		// insert element count
			.endlocal
			#define_eval FAST_PORT_LO_INTF (FAST_PORT_LO <<3)
			.if (output_intf == FAST_PORT_LO_INTF)
				sram_write($desc_buf[0],XMIT_FPORT_DESCRIPTOR_BASE , $desc_buf[2], LWCOUNT2, ctx_swap);
				packetq_send(descriptor_addr, output_intf, Q_HANDLE_LO); 		// at eop, q to xmit
			.else
				sram_write($desc_buf[0],XMIT_FPORT_DESCRIPTOR_BASE_2 , $desc_buf[2], LWCOUNT2, ctx_swap);
				packetq_send(descriptor_addr, output_intf, Q_HANDLE_HI); 		// at eop, q to xmit
			.endif
			
			xbuf_free($desc_buf);
		.else												// discard
			#if (UENGINE_ID < 2)
				packetq_seq_wait(rec_state, FAST_PORT_LO);
			#else
				packetq_seq_wait(rec_state, FAST_PORT_HI);
			#endif
			// push buffer back if discard
			buf_index_from_dram_addr(mailbox_data, packet_buf_addr, FREELIST_HANDLE);
			buf_push(mailbox_data, 0, FREELIST_HANDLE);
		.endif												// not discard
	.endif											// EOP

.endw
xbuf_free($$dxfer);
nop


