// xmit_sched.uc
//		1 transmit scheduler for use with either xmit_8x100.uc or xmit_fast_port.uc
//		used on 2 microengines
//		1 microengine takes even tfifo elements, the other takes odd tfifo elements
//		each element is statically bound to a port. don't need binding to block a port

//		each microengine: define personality  ODD or not ODD
//
// PORT_INCR AND PORT_MASK must be preprocessor definitions.
//

#define_eval SKIP_BIT_SET 0x100
#define SKIP_BIT 8

#ifdef VALIDATION_CARD
	#include "ports_validcard.h"
	#define PORT_MASK 0xf				; 16 slow ports
	#define PORT_TO_TFIFO_MULT 1		; map of port number to TFIFO element
	#define PORT_INCR 2					; increment between ports serviced by each FBOX
#else									; eval card
	#include "ports_evalcard1.h"
	#define PORT_MASK 0x7				; 8 slow ports 
	#define PORT_TO_TFIFO_MULT 2		; map of slow port number to TFIFO element
	#define PORT_INCR 1					; increment between slow ports serviced 
#endif


#ifndef INITIAL_PORT
	#define INITIAL_PORT FID
#endif

#ifndef HW_INIT_UENGINE
	#define HW_INIT_UENGINE 5
#endif


#include "stdmac.uc"
#include "mem_map.h"
#include "sem.uc"
#include "tx.uc"
#include "port.uc"
#include "tfifo.uc"


	// startup
	br=ctx[1,tx_fill#]				; fill thread
	br=ctx[2,tx_fill#]				; fill thread
	br=ctx[3,tx_fill#]				; fill thread
// *************************** START TX_ETHER100M scheduler microcode **************************************



.local target_port pwp_addr aggregate_pwp $_pwp

	tx_ether100m_init[target_port, pwp_addr, INITIAL_PORT, XMIT_PWP_VECTOR]

	hw_init[FID, HW_INIT_UENGINE, SDRAM_VALIDATE, RDYBUS_PROG1, 
		RDYBUS_PROG2, RDYBUS_PROG3, ALT_BANKS, BUFFER_COUNT]
	
	#if (FID == HW_INIT_UENGINE)		; this is needed to run in the transactor.  
		freelist_created#:
	#endif

	// Begin main scheduler loop
	.while (1)
			scratch[read, $_pwp, pwp_addr, 0, 1], ctx_swap			;  poll ports with packets
			alu[aggregate_pwp, $_pwp, or, @local_pwp]
			tx_100m_assign[target_port, @assign1, aggregate_pwp, SKIP_BIT, PORT_INCR, PORT_MASK]
			tx_100m_assign[target_port, @assign2, aggregate_pwp, SKIP_BIT, PORT_INCR, PORT_MASK]
			tx_100m_assign[target_port, @assign3, aggregate_pwp, SKIP_BIT, PORT_INCR, PORT_MASK]
	.endw

.endlocal

// three fill threads
//
#include "tx_ether100m_fill.uc"
		
