// tx_ether1gig.uc
//
//	Required Preprocessor Definitions:  
//		FAST_PORT_NUM:	Fast port number, NOT system port number.  Legal values are 1(st) and 2(nd)
//		FID:			FBox ID number.  Legal values are 0..5 with restriction that Min(all tx FBoxs) is even. 
//
#define SDRAM_VALIDATE_FLAG 0

#define ASSIGN_SKIP 		0x100
#define NO_ASSIGNMENT	 	0x80000000
#define_eval CTX1_TID 		(FID*4+1)
#define_eval CTX2_TID 		(FID*4+2)
#define_eval CTX3_TID 		(FID*4+3)

#ifndef INITIAL_FIFO_ELEM
	#define INITIAL_FIFO_ELEM FID  ; if INITIAL_FIFO_ELEM is not defined, then set it based on FID
#endif

#ifndef HW_INIT_UENGINE
	#define HW_INIT_UENGINE 5
#endif


#include "ports_evalcard1.h"		; include port info and rdy_bus program
#include "stdmac.uc"
#include "mem_map.h"				; need XMIT_FPORT1_ELE_COUNT and XMIT_FPORT2_ELE_COUNT
#include "sem.uc"
#include "tx.uc"
#include "constants.uc"


	// startup
	br=ctx[1,tx_fill#]				; fill thread
	br=ctx[2,tx_fill#]				; fill thread
	br=ctx[3,tx_fill#]				; fill thread


// *************************** START TX_ETHER1GIG scheduler microcode **************************************


	hw_init[FID, HW_INIT_UENGINE, SDRAM_VALIDATE_FLAG, RDYBUS_PROG1, 
		RDYBUS_PROG2, RDYBUS_PROG3, ALT_BANKS, BUFFER_COUNT]

.local gig_ele_count_addr skip_assign const_0 const_1 enqueued_packets $_gig_ele_count

	tx_ether1gig_init[@assign1, @assign2, @assign3, @port_info_lock, @assigned_mpkt_cnt, 
						@fifo_elem, gig_ele_count_addr, skip_assign, const_0, const_1,
						INITIAL_FIFO_ELEM, FAST_PORT_QUEUE, 
						ASSIGN_SKIP, XMIT_FPORT1_ELE_COUNT, XMIT_FPORT2_ELE_COUNT]   ; initialize variables

	#if (FID == HW_INIT_UENGINE)		; this is needed to run in the transactor.  
		freelist_created#:
	#endif

	//signal fill threads indicating initialization is complete
	fast_wr[CTX1_TID, inter_thd_sig]
	fast_wr[CTX2_TID, inter_thd_sig]
	fast_wr[CTX3_TID, inter_thd_sig]


	.while (1)

		scratch[read, $_gig_ele_count, gig_ele_count_addr, 0, 1], ctx_swap	
		alu[enqueued_packets, 0, +16, $_gig_ele_count]   					; limit count to 16 bits

		tx_gig_assign[@assigned_mpkt_cnt, @assign1, enqueued_packets, const_0, const_1, skip_assign]
		tx_gig_assign[@assigned_mpkt_cnt, @assign2, enqueued_packets, const_0, const_1, skip_assign]
		tx_gig_assign[@assigned_mpkt_cnt, @assign3, enqueued_packets, const_0, const_1, skip_assign]

	.endw

.endlocal

// *************************** END Tx_Ether1Gig scheduler microcode **************************************

// three fill threads
//
#include "tx_ether1gig_fill.uc"
		

