// tx_ether1gig_fill.uc
//
// Required Preprocessor definitions:
//	PORT_NUM - Output port number
#include "tfifo.uc"
		
tx_fill#:
.local temp tfifo_entry queue bank_mask bank queue_descriptor_base pkt_buffer_base next_thread_ir buf_descriptor_base port_rdy_bit const_x38 const_0 const_1 const_2 port const_x3f bit15on bit20on port_info ele_remaining buf_offset skip_flag
.xfer_order $xfer0 $xfer1 $xfer2 $xfer3
.operand_synonym $packet_link0 $xfer0				; packet links descriptors from queue
.operand_synonym $packet_link1 $xfer1
.operand_synonym $tfifo_ctl_wd0 $xfer2				; status to tfifo
.operand_synonym $tfifo_ctl_wd1 $xfer3


#define EOP_AND_SOP 3
#define EIGHT_QUADWORDS 8
#define TOP_BUF_BYTE_OFFSET 0

	tx_gig_fill_init[const_0, const_1, const_2, const_x38, const_x3f, skip_bit, 
						bit15on, bit20on, pkt_buffer_base, buf_descriptor_base, 
						$tfifo_ctl_wd1, bank_mask, bit20_15on, bank, next_thread_ir, 
						queue_descriptor_base, port_rdy_bit, @port_info_lock,
						FAST_PORT_QUEUE, XMIT_FPORT_DESCRIPTOR_BASE, 
						XMIT_FPORT_DESCRIPTOR_BASE_2, ASSIGN_SKIP, PORT_NUM, FID]


main_loop#:

.while(1)
	// read the next assignment for this thread
	br=ctx[1, read1#]
	br=ctx[2, read2#]
	br=ctx[3, read3#]
read1#:								
	tx_gigassignment_read[tfifo_entry, queue, skip_flag, 
			@assign1, @fifo_elem, skip_bit, const_0, const_2]	
	br[process_assignment#]
read2#:
	tx_gigassignment_read[tfifo_entry, queue, skip_flag, 
			@assign2, @fifo_elem, skip_bit, const_0, const_2]
	br[process_assignment#]
read3#:
	tx_gigassignment_read[tfifo_entry, queue, skip_flag, 
			@assign3, @fifo_elem, skip_bit, const_0, const_2]				

process_assignment#:

	.if (skip_flag != ASSIGN_SKIP)

		// Wait for the previous thread to save output port status before reading port info
		sem_wait[@port_info_lock]	
		tx_gig_portinfo_restore[port_info, ele_remaining, bank, buf_offset, 
							 last_mpkt_byte_cnt, @fast_port_info, bank_mask, const_x3f]	

		.if (ele_remaining == 0)								; if SOP

			// If we are starting a new packet then we need to go out and read packet
			// info from sram 
			sem_clr[@port_info_lock]	; lock while we read packet link
			tx_packetlink_read[buf_offset, $packet_link0, 
								$packet_link1, ele_remaining, bank, last_mpkt_byte_cnt, 	
								bank_mask, const_x3f, queue, const_x38, 
								queue_descriptor_base, const_1]	

			.if (ele_remaining == 1)							; if at eop and sop

				// If EOP then send data and free the buffer that we were using
				sem_set[@port_info_lock] 			; unlock now because we have eop
				tx_last_mpkt_xfr[bank, buf_offset, last_mpkt_byte_cnt, TOP_BUF_BYTE_OFFSET, 
					tfifo_entry, bit20on, bit20_15on, pkt_buffer_base, 
					SDRAM_VALIDATE_FLAG]
				tx_buf_free[buf_offset, bank, bit20on, buf_descriptor_base]
				// byte count is in last_mpkt_byte_cnt, eop = true, sop = true
				tx_status_set[$tfifo_ctl_wd0, last_mpkt_byte_cnt, EOP_AND_SOP, PORT_NUM]


			.else												; sop, but not_eop
				// Decrement assigned packet count by num of mpacket in this
				// packet so that all mpackets of this packet are scheduled for xmit
				sub(@assigned_mpkt_cnt, @assigned_mpkt_cnt, ele_remaining)
				add(@assigned_mpkt_cnt, @assigned_mpkt_cnt, const_1)

				// If not EOP then send data, save port info, and increment 
				// mpackets cnt for next mpacket
				tx_mpkt_xfr[TOP_BUF_BYTE_OFFSET, bank, buf_offset, tfifo_entry, 
					bit20_15on, bit20on, pkt_buffer_base, EIGHT_QUADWORDS]
				tx_gig_portinfo_save[@buf_base_offset, @fast_port_info, ele_remaining, 
								buf_offset, last_mpkt_byte_cnt, bank] 
				sem_set[@port_info_lock] 			; unlock after tx_gig_portinfo_save
				// byte count is 64, eop = false, sop = true
				tx_status_set[$tfifo_ctl_wd0, const_0, 0xfd, PORT_NUM] 

			.endif

			#ifdef IXF1002_RevD	; if we have a rev D MAC then we don't have to worry about 2 packet limit mac bug
				tfifo_validate[@mpkt_tx_ok_cnt, $tfifo_ctl_wd0, tfifo_entry,
						port_rdy_bit, bit15on, const_1, next_thread_ir, SDRAM_VALIDATE]
			#else
				tfifo_validate_sop[@mpkt_tx_ok_cnt, $tfifo_ctl_wd0, tfifo_entry, 
						port_rdy_bit, bit15on, next_thread_ir, SDRAM_VALIDATE]
			#endif

		.else													; not_sop

			.if(ele_remaining == 1)								; if eop

				// If EOP then send data, free the buffer that we were using, and 
				// update port status
				tx_last_mpkt_xfr[bank, buf_offset, last_mpkt_byte_cnt, TOP_BUF_BYTE_OFFSET, 
					tfifo_entry, bit20on, bit20_15on, pkt_buffer_base,
					SDRAM_VALIDATE_FLAG]
				tx_gig_portinfo_update[port_info, const_1, @fast_port_info]	
				tx_buf_free[@buf_base_offset, bank, bit20on, buf_descriptor_base]
				// byte count is in last_mpkt_byte_cnt, eop = true, sop = false
				tx_status_set[$tfifo_ctl_wd0, last_mpkt_byte_cnt, 0x2, PORT_NUM] 

			.else												; tx_not_eop_not_sop#:

				// If not EOP then send data, save port info, and increment 
				// mpackets cnt for next mpacket
				tx_mpkt_xfr[TOP_BUF_BYTE_OFFSET, bank, buf_offset, tfifo_entry, 
					bit20_15on, bit20on, pkt_buffer_base, EIGHT_QUADWORDS]
				tx_gig_portinfo_update[port_info, const_1, @fast_port_info]	
				// byte count = 64, eop = false, sop = false
				tx_status_set[$tfifo_ctl_wd0, const_0, 0xfc, PORT_NUM]

			.endif

			tfifo_validate[@mpkt_tx_ok_cnt, $tfifo_ctl_wd0, tfifo_entry,
						port_rdy_bit, bit15on, const_1, next_thread_ir, SDRAM_VALIDATE]

		.endif													; ele_remaining == 0
transmit_done#:

	.else	; we were given a "skip tfifo element" assignment

		tfifo_element_skip[@mpkt_tx_ok_cnt, tfifo_entry, pkt_buffer_base, port_rdy_bit, 
					   next_thread_ir, bit15on, SDRAM_VALIDATE_FLAG, FID]

	.endif		; skip_flag != ASSIGN_SKIP
.endw
nop				; bug fix.  last instruction can not be .endw
nop
nop
.endlocal
