//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
// ether_filter.uc
// macros for ethernet filter
//
// system: IXP1200
// subsystem: IP microcode
// usage: library macros
// author: ssk	Aug 31, 2000
//
// revisions:


#ifndef ETHER_FILTER_UC
#define ETHER_FILTER_UC 
		
#include "br_share.h"

#define L2_SHIFT_AMOUNT 4
#define_eval INV_L2_SHIFT_AMOUNT (32 - L2_SHIFT_AMOUNT)

// ether_filter
//		description: Ethernet filter based on input/output port configuration
//					 
//
//		input:
//			Port #				Port Number
//		output:
//			MAC address			Device MAC address (same for all ports)
//			Port configuration	Port configuration containing, port state, filter type, and ethertype
//		size:	XXX instr
//		example usage:
///

#macro ether_filter[out_da_port_num, out_pkt_action, out_dst_port_entry, in_src_port_entry, in_packet_ethertype, in_pkt_buf, in_sdram_xfers]
.local sa_lookup_result da_lookup_result _ethfilt_tempa _ethfilt_tempb
.local hash0 hash1 hash2 hash3

	xbuf_alloc($hash_buf, 4);													// allocate xfer buffer 4 lwords

	//in_port_filter_type - L2 filtering options
	//	00 Ethertype based filtering
	//	01 Explict rule, action specified in SDRAM fwd entry
	//	10 Positive filtering, action implied by presense/absense of filter rule
	//	11 Negative filtering, action implied by presense/absense of filter rule
	// setup for da hash

	xbuf_extract(_ethfilt_tempa, in_pkt_buf, BYTEOFFSET0, 0, 2);  load DA bytes 0-1 to write xfer reg for hash
	move [$hash_buf[0],_ethfilt_tempa]
	xbuf_extract(_ethfilt_tempa, in_pkt_buf, BYTEOFFSET0, 2, 4);  load DA bytes 2-5 to write xfer reg for hash
	move [$hash_buf[1],_ethfilt_tempa]
	xbuf_extract(_ethfilt_tempa, in_pkt_buf, BYTEOFFSET0, 6, 2);  load SA bytes 0-1 to write xfer reg for hash
	move [$hash_buf[2],_ethfilt_tempa]
	xbuf_extract(_ethfilt_tempa, in_pkt_buf, BYTEOFFSET0, 8, 4);  load SA bytes 2-5 to write xfer reg for hash
	move [$hash_buf[3],_ethfilt_tempa]

	hash2_48[$hash_buf[0]], ctx_swap						; Hash the DA and SA

	// Wake up here - the 2 hash ops are complete.   Register usage:
	//
	// $hash_buf 0,1 - hash result DA
	// $hash_buf 2,3 - hash result SA
	// hash lookup DA and SA
	//
	// Freeup transfer registers for other use
	move[hash0, $hash_buf[0]]
	move[hash1, $hash_buf[1]]
	move[hash2, $hash_buf[2]]
	move[hash3, $hash_buf[3]]

	// L1 lookup
	immed32[_ethfilt_tempb, SRAM_L1_ADDR_HASH_BASE]
	ld_field_w_clr[_ethfilt_tempa, 0011, hash1]					; table index = low 16 bits of hash result
    sram[read, $hash_buf[0], _ethfilt_tempa, _ethfilt_tempb, 1], optimize_mem					; lookup DA, SRAM_L1 hash

	ld_field_w_clr[_ethfilt_tempa, 0011, hash3]					; table index = low 16 bits of hash result
    sram[read, $hash_buf[1], _ethfilt_tempa, _ethfilt_tempb, 1], ctx_swap, optimize_mem 		; lookup SA, SRAM_L1 hash
    move[da_lookup_result, $hash_buf[0]]						; check results of DA lookup in hash table 1
    move[sa_lookup_result, $hash_buf[1]]						; check results of DA lookup in hash table 1

	xbuf_free($hash_buf)

	// L2 lookup
	alu_shf_right(_ethfilt_tempb, 0x1, AND, da_lookup_result, 31)			; Check for collision bit
	.if (_ethfilt_tempb == 1)
		hash_resolve[hash0, hash1, da_lookup_result, SRAM_L2_ADDR_HASH_BASE]
	.endif

	alu_shf_right(_ethfilt_tempb, 0x1, AND, sa_lookup_result, 31)			; Check for collision bit
	.if (_ethfilt_tempb == 1)
		hash_resolve[hash2, hash3, sa_lookup_result, SRAM_L2_ADDR_HASH_BASE]
	.endif
	
.endlocal //hash0 hash1 hash2 hash3

	.if (da_lookup_result == 0)	 // Indicates mac entry for DA does not exist
		immed[out_da_port_num, DEST_PORT_NO_MATCH]	
	.else
		alu_op(_ethfilt_tempb, 0xfffffff, AND, da_lookup_result)			; SDRAM index = low 16 bits of lookup result
		move(da_lookup_result, _ethfilt_tempb);
	    sdram[read, in_sdram_xfers[0], 0, da_lookup_result, 2], ctx_swap	; read the forwarding table for DA (4 longwords)
		// Wakeup here - have forwarding table entry
		// Register usage:
		// in_sdram_xfers[3:0]  - DA forwarding table entry
		//
		immed[_ethfilt_tempa, 0];
    	alu_shf_right(_ethfilt_tempa, BR_FORWARD_ENTRY_MASK, AND, in_sdram_xfers[0], BR_FORWARD_ENTRY_SHIFT_SIZE)
		.if (_ethfilt_tempa)	// If forwarding information is associated with this entry (aka Bridging)
	    	alu[out_da_port_num, in_sdram_xfers[0], AND, 0x1f]			; isolate DA port number from forwarding table
			// Get port information of the destination port
			ether_port_info(_ethfilt_tempa, _ethfilt_tempa, out_dst_port_entry, out_da_port_num)
		.endif
	.endif

.local dst_port_filtertype src_port_filtertype dst_port_ethertype src_port_ethertype
	ld_field_w_clr[src_port_ethertype, 0011, in_src_port_entry, >>8]		; get ethertype from port entry
	ld_field_w_clr[dst_port_ethertype, 0011, out_dst_port_entry, >>8]		; get ethertype from port entry
	alu_shf_right(src_port_filtertype, 0x03, AND, in_src_port_entry, 4)	; get filter_type from port entry
	alu_shf_right(dst_port_filtertype, 0x03, AND, out_dst_port_entry, 4)	; get filter_type from port entry

	immed[out_pkt_action, PKT_PERMIT]		; Default action

	.if (((src_port_filtertype == BR_PORT_FILT_TYPE_ETHERTYPE) || (dst_port_filtertype == BR_PORT_FILT_TYPE_ETHERTYPE)) && (out_da_port_num != DEST_PORT_NO_MATCH))
		alu_op(_ethfilt_tempa, in_packet_ethertype, AND, src_port_ethertype)
		.if (_ethfilt_tempa != dst_port_ethertype)
			immed[out_pkt_action, PKT_DENY]		
			br[filter_return#]
		.endif
	.endif

	.if ((src_port_filtertype == BR_PORT_FILT_TYPE_EXPLICT) && sa_lookup_result)
		// SA filter
		alu_op(_ethfilt_tempb, 0xfffffff, AND, sa_lookup_result)			; SDRAM index = low 16 bits of lookup result
		move(sa_lookup_result, _ethfilt_tempb);
	    sdram[read, in_sdram_xfers[1], 0, sa_lookup_result, 2], ctx_swap	; read the forwarding table for DA (4 longwords)
		// Wakeup here - have forwarding table entry
		// Register usage:
		// in_sdram_xfers[3:0]  - SA Filter table entry
		//
    	alu_shf_right(out_pkt_action, BR_FILTER_ACTION_MASK, AND, in_sdram_xfers[1], BR_FILTER_ACTION_SA_SHIFT_SIZE)
		.if (out_pkt_action == PKT_DENY)
			br[filter_return#]
		.endif
	.endif
	
	.if ((dst_port_filtertype == BR_PORT_FILT_TYPE_EXPLICT) && da_lookup_result)
		// DA filter
    	alu_shf_right(out_pkt_action, BR_FILTER_ACTION_MASK, AND, in_sdram_xfers[0], BR_FILTER_ACTION_DA_SHIFT_SIZE)
		.if (out_pkt_action == PKT_DENY)
			br[filter_return#]
		.endif
	.endif

	.if ((src_port_filtertype == BR_PORT_FILT_TYPE_POSITIVE) || (dst_port_filtertype == BR_PORT_FILT_TYPE_POSITIVE))
		// Positive filtering - default action is to permit. SA/DA that have an entry in the table will be 
		//  dropped. 
		
		//DA filter
		.if (da_lookup_result)
			immed[out_pkt_action, PKT_DENY]		
			br[filter_return#]
		.endif
		// SA filter
		.if (sa_lookup_result)
			immed[out_pkt_action, PKT_DENY]		
			br[filter_return#]
		.endif
	.endif

	.if ((src_port_filtertype == BR_PORT_FILT_TYPE_NEGATIVE) || (dst_port_filtertype == BR_PORT_FILT_TYPE_NEGATIVE))
		// Negative filtering - default action is to deny. SA/DA that have an entry in the table will be allowed.

		// DA filter
		.if (!da_lookup_result)
			immed[out_pkt_action, PKT_DENY]		
			br[filter_return#]
		.endif
		// SA filter
		.if (!sa_lookup_result)
			immed[out_pkt_action, PKT_DENY]		
			br[filter_return#]
		.endif
	.endif

filter_return#:
.endlocal //dst_port_filtertype src_port_filtertype dst_port_ethertype src_port_ethertype  
.endlocal //sa_lookup_result da_lookup_result _ethfilt_tempa _ethfilt_tempb
#endm 

#macro hash_lookup_L2[hash_rem, lookup_result, L2_hash_base]
.local tempa tempb

// solve_hash_conflict
// inputs:
//     lookup_xfer  - value from hash table lookup
//			<30>	- lock bit		entry not valid (update from core in progress)
//          <31>	- collision bit 
//              =0 in this case, means no entry, bail out
//              =1 collision
//          <29>    MBZ
//          <28:5>  L2_table select - 16 bits
//          <4:0>   L2_shift - hash table index shift left amount - 5 bits.  Format is 32-shift left amount
//            
//
//     hash_rem - output of L1 hash, shifted right 16 (32 bits)
//     sram_L2_addr_hash_base
// 
// outputs:
//     if a collision found and resolved:
//			lookup_xfer - forwarding table index
//		if not resolved, no entry
//			lookup_xfer = 0
//
.local table_sel index

hash_loop#:
	ld_field_w_clr[table_sel, 0111, lookup_result, >>5]		; table_sel in 23:0

// note: table shift value in <5:0> is stored as left shift indirect value (32-n)
	alu[index, --, ~B, 0]									; generate -1
	alu_shf[index, hash_rem, AND, index, >>INV_L2_SHIFT_AMOUNT]	; mask hash result with -1 shifted by lookup_xfer<4;0>

	alu_shf[index, index, +, table_sel]						; add masked hash_result to table sel
   
.local $local_xfer7
    sram[read, $local_xfer7, L2_hash_base, index, 1], defer[1], ctx_swap, optimize_mem
	alu_shf[hash_rem, --, B, hash_rem, >>L2_SHIFT_AMOUNT]	; get new hash_rem
	move[lookup_result, $local_xfer7]
.endlocal // $xfer7
    br<0[hash_loop#]			  						; check for empty (=0) or conflict (<31> = 1)
.endlocal

;hash_resolved#:
.endlocal
#endm


// hash_resolve
//	function call to resolve level 2 conflict
//	calls hash_resolve48_func in stdfunc.uc
//
#macro hash_resolve[hashr_hi_xfer, hashr_lo_xfer, lookup_result, const_L2_base]
    ld_field_w_clr[tempb, 0011, hashr_lo_xfer, >>16]
    alu[tempb, tempb, or, hashr_hi_xfer, <<16]				; concatenate hash remainder
	move[tempc, lookup_result]
	immed32[tempd, const_L2_base]
	balr[tempa, hash_resolve_func#]

	move[lookup_result, tempc]
#endm

#endif // ETHER_FILTER_UC
