//rec_ether100m.uc

// Debugging in Hardware:
//		The evaluation card has 8 100M ports. To have this project to run on 8 ports,
// 		remove VALIDATION_CARD definition in "PreProcessor Definitions" found in menu Build->Settings for 
//		microengines 2 and 3. This will enable receive microengines 0-1 (8 threads) for ports 0-7 and
//		disable microengnies 2 and 3 for ports 8-15.

// Debugging in Simulation (present settings)
// 		A validation card would have 16 100M ports. To have this project to run on 16 ports,
//		define VALIDATION_CARD definition in "PreProcessor Definitions" found in menu 
//		Build->Settings for microengines 2 and 3. This will enable receive microengines 0-3 (16 threads) 
//		to serve 0-15 ports.


//Global Include
#include "ports_validcard.h"
#include "mem_map.h"
#include "ixplib.uc"

//Local Include
#include "ether_filter.uc"

// packet buffers
#define DESC_SIZE 4						// size of descriptor
#define PKBUF_SIZE 256					// quadword size of packet buffer

// even receive microengines use freelist 0 and low sdram bank
#if ((UENGINE_ID & 1) == 0)
	#define FREELIST_ID 0
	#define FREELIST_HANDLE FREELIST_ID, SDRAM_PKT_BUFFER_BASE, PKBUF_SIZE, SRAM_BUFF_DESCRIPTOR_BASE, DESC_SIZE
#else
	#ifdef ALT_BANKS
// odd receive microengines use freelist 1 and high sdram bank
		#define FREELIST_ID 1
		#define FREELIST_HANDLE FREELIST_ID, SDRAM_PKT_BUFFER_HIGH_BASE, PKBUF_SIZE, SRAM_BUFF_DESCRIPTOR_BASE, DESC_SIZE
	#else
		#define FREELIST_ID 0
		#define FREELIST_HANDLE FREELIST_ID, SDRAM_PKT_BUFFER_BASE, PKBUF_SIZE, SRAM_BUFF_DESCRIPTOR_BASE, DESC_SIZE
	#endif
#endif

#define Q_HANDLE Q_LINKED_LIST_8, XMIT_PWP_VECTOR, SRAM_QUEUE_DESCRIPTOR_BASE, SRAM_BUFF_DESCRIPTOR_BASE

#define PKT_INFO_SHIFT_PKTACTION_PKTFORME		31
#define PKT_INFO_SHIFT_PKTACTION_PERMIT_DENY	30
#define PKT_INFO_SHIFT_PKTACTION_QTOCORE		29
#define PKT_INFO_SHIFT_PKTLINKTYPE_ETHERNET		28
#define PKT_INFO_SHIFT_PKTLINKTYPE_LLC			27
#define PKT_INFO_SHIFT_PKTLINKTYPE_LLCSNAP		26
#define DEST_PORT_NO_MATCH						0xffff

// port to context assignments
#if (UENGINE_ID == 0)
#define PORT_NUMS 0,1,2,3
#elif (UENGINE_ID == 1)
#define PORT_NUMS 4,5,6,7
#elif (UENGINE_ID == 2)
#define PORT_NUMS 8,9,10,11
#elif (UENGINE_ID == 3)
#define PORT_NUMS 12,13,14,15
#endif

#define CORE_STACK_INTF1 18						// output interface for queueing to core

// initializations
sig_ifctx_block(0, inter_thread, OR, 1);			// block until signal global initialization complete

#if (UENGINE_ID == 0)
// initialize Hash multiplier in FBI same as what is used in core
br!=ctx[0, skip_hash_init#]
ether_init_hash(BR_HASH_LOAD_HI, BR_HASH_LOAD_LO);	
skip_hash_init#:
#endif // UENGINE_ID = 0

port_rx_init(rec_req, rfifo_addr, UENGINE_ID, PORT_NUMS);						// create a canned receive request
critsect_init(@rdready_inflight);			// init semaphore to limit 1 rec ready read per engine
critsect_init(@req_inflight);				// init semaphore to limit 1 rec req per engine

immed32(route_base, SDRAM_ROUTE_FWD_BASE);										// init table addresses	
immed32(rec_state, 0);
immed32(packet_buf_addr, UNALLOCATED);											// address to pop from freelist
xbuf_alloc($$dxfer, 8);					// allocate sdram xfers as a block of 8
xbuf_link($$dxfer, $$dxfer);								// linke the xfer buffers

////////////////////////////////////////////////////////////////////////////////////////
//	Please see note at the begining of this file for more information on VALIDATION_CARD
////////////////////////////////////////////////////////////////////////////////////////
#ifndef VALIDATION_CARD		
#if (UENGINE_ID > 1)
	ctx_arb[kill]
	nop
	nop
	nop
#endif
#endif

// main loop   
.while (1)
	xbuf_alloc($pop_xfer, 1);
    .if (packet_buf_addr == UNALLOCATED)
    	buf_pop($pop_xfer[0], FREELIST_HANDLE, sig_done);							// if no buffer, get ptr
    .endif

	port_rxrdy_chk(@rdready_inflight, rec_req);
	critsect_enter[@req_inflight]						; block other contexts from sending rec_req
	port_rx_request(rec_req);							// get mpacket
    port_rx_receive(exception, rec_state, ETHER_100M);	// get mpacket status
	mpacket_received#:																				// wait for mem_pop to complete

    .if (packet_buf_addr == UNALLOCATED)
		buf_wait();
		#if (FREELIST_ID == 0)
			#define BASE_ADDR  SRAM_BUFF_DESCRIPTOR_BASE
		#else 
			#define BASE_ADDR SDRAM_PKT_BUFFER_HIGH_BASE
		#endif
	    .while ($pop_xfer[0] == BASE_ADDR)
			buf_pop($pop_xfer[0], FREELIST_HANDLE, ctx_swap);					// if no buffer available, retry
		.endw
		buf_dram_addr_from_sram_addr(packet_buf_addr, $pop_xfer[0], FREELIST_HANDLE);		// calc packet buffer address
		move(descriptor_addr, $pop_xfer[0]);
	.endif
	xbuf_free($pop_xfer);

	.if (exception == PORT_RXFAIL)
		port_rx_fail(rec_state, rec_req, exception);							// fail is like EOP w/o queueing
		buf_dram_addr_from_sram_addr(packet_buf_addr, descriptor_addr, FREELIST_HANDLE);
		critsect_exit[@req_inflight]						; allow other contexts to send rec_req
		.continue
	.elif (exception == PORT_RXERROR)
		port_rx_error(rec_state, rec_req, exception);							// bump mib counter
		buf_dram_addr_from_sram_addr(packet_buf_addr, descriptor_addr, FREELIST_HANDLE);		// calc packet buffer address
		critsect_exit[@req_inflight]						; allow other contexts to send rec_req
		.continue
	.endif

	critsect_exit[@req_inflight]						; allow other contexts to send rec_req

	.if (bit(rec_state, REC_STATE_SOP_BIT))		// if SOP
.local sa01 output_intf																		
.local dst_port_entry src_port_entry packet_state da_port_num packet_ethertype;				
		// Initializations
		immed[packet_state, 0]
		immed[packet_ethertype, 0]

.operand_synonym tempa rec_port_num
.operand_synonym tempb port_mac_addr32
.operand_synonym tempc port_mac_addr16
		alu_op(rec_port_num, rec_req, AND, 0x1f);	
		// Get packet processing command and other port attributes for packets 
		// arriving on this port:
		
		ether_port_info(port_mac_addr32, port_mac_addr16, src_port_entry, rec_port_num); 
		// Now go get packet
		xbuf_alloc($pkt_buf, 4);										// allocate xfer buffer 4 lwords
		xbuf_alloc($pkt_buf_eth, 2);									// allocate xfer buffer 4 lwords
		xbuf_link($pkt_buf, $pkt_buf_eth);								// linke the xfer buffers

		rfifo_read($pkt_buf[0], rfifo_addr, QWOFFSET0, QWCOUNT3, ctx_swap);				// read link layer, 1st 2 quadwords

#ifdef LITTLE_ENDIAN
		shf_right(sa01, $pkt_buf[1], 16);
#else
		alu_op(sa01, 0, +16, $pkt_buf[1]);										// save sa bytes 0-1 for later merge
#endif
.local proto_len
.local da_mac_addr32 da_mac_addr16 ; 		
		xbuf_extract(proto_len, $pkt_buf, BYTEOFFSET0, ETHER_PROTOCOL_LENGTH); // get protocol length field
		xbuf_extract(da_mac_addr32, $pkt_buf, BYTEOFFSET0, 0, 4);	// 5 MAC addres starts from 2nd 32bit word
		xbuf_extract(da_mac_addr16, $pkt_buf, BYTEOFFSET0, 4, 2);
		.if ((port_mac_addr32 == da_mac_addr32) && (port_mac_addr16 == da_mac_addr16))
	    	alu_shf[packet_state, packet_state, OR, TRUE, <<PKT_INFO_SHIFT_PKTACTION_PKTFORME]
		.endif
.endlocal // da_mac_addr32 da_mac_addr16
		.if(proto_len < ETHER_MTU)				// if ethernet				// see RFC 1122, 2.3.3
			// Need to parse 802.3 header here to determine where IP header begins
			ether_parse_802_3[packet_ethertype, $pkt_buf]
			.if (packet_ethertype > 0)
		    	alu_shf[packet_state, packet_state, OR, TRUE, <<PKT_INFO_SHIFT_PKTLINKTYPE_LLCSNAP]
			.else
		    	alu_shf[packet_state, packet_state, OR, TRUE, <<PKT_INFO_SHIFT_PKTLINKTYPE_LLC]
			.endif
		.else
	    	alu_shf[packet_state, packet_state, OR, TRUE, <<PKT_INFO_SHIFT_PKTLINKTYPE_ETHERNET]
			move[packet_ethertype, proto_len]	; 
		.endif
.endlocal
		xbuf_free($pkt_buf_eth);			

.operand_synonym tempd packet_action
		immed[packet_action, 0]
		ether_filter(da_port_num, packet_action, dst_port_entry, src_port_entry, packet_ethertype, $pkt_buf, $$dxfer);
		ctx_arb[voluntary]
		.if (packet_action == PKT_PERMIT)
	    	alu_shf[packet_state, packet_state, OR, TRUE, <<PKT_INFO_SHIFT_PKTACTION_PERMIT_DENY]
		.elif (packet_action == PKT_DENY)
	    	alu_shf[packet_state, packet_state, OR, FALSE, <<PKT_INFO_SHIFT_PKTACTION_PERMIT_DENY]
			br[packet_late_discard#]
		.elif (packet_action == PKT_QUEUE_TO_CORE)
	    	alu_shf[packet_state, packet_state, OR, TRUE, <<PKT_INFO_SHIFT_PKTACTION_QTOCORE]
		.endif

.local src_port_state dst_port_state	; 	
    	alu_op(src_port_state, 0xf, AND, src_port_entry); get port state from port entry
    	alu_op(dst_port_state, 0xf, AND, dst_port_entry); get port state from port entry

		.if (dst_port_state || src_port_state || (bit(packet_state, PKT_INFO_SHIFT_PKTACTION_QTOCORE) == TRUE))
			dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, QWCOUNT8, ctx_swap);
			shf_left(output_intf, CORE_STACK_INTF1, 3);			// if not ethernet, q to core
			br[enqueue#]
		.else
.endlocal  // src_port_state dst_port_state

			.if (bit(packet_state, PKT_INFO_SHIFT_PKTACTION_PKTFORME) == FALSE) 	; L2 MAC/L2 Protocol filtering and/or Bridging

				alu_op(rec_port_num, rec_req, AND, 0x1f);
			    .if ((da_port_num == rec_port_num) || (bit(packet_state, PKT_INFO_SHIFT_PKTACTION_PERMIT_DENY) == FALSE))     ; check if the dest port = recv port
					br[packet_late_discard#]
				.endif

				//capture the output port
				.if (da_port_num == DEST_PORT_NO_MATCH)
					br[packet_late_discard#]
				.endif

				shf_left(output_intf, da_port_num, 3);
				dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, QWCOUNT8, ctx_swap);// write 2nd 32B

			.else ;  IP forwarding
.local ip_dest
				xbuf_alloc($pkt_buf_ip, 4);										// allocate xfer buffer 4 lwords
				xbuf_link($pkt_buf, $pkt_buf_ip);								// linke the xfer buffers
				xbuf_link($pkt_buf_ip, $pkt_buf);								// create circular buffer
	   			rfifo_read($pkt_buf_ip[0], rfifo_addr, QWOFFSET2, QWCOUNT3, ctx_swap);	// read next 3 quadwords

.local  ip_verslen;	
				.if (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_ETHERNET) == TRUE)			; Ethernet
					xbuf_extract(ip_verslen, $pkt_buf, BYTEOFFSET14, IP_VERSION_LENGTH); // get version/length field
				.elif (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_LLC) == TRUE)		; LLC
					xbuf_extract(ip_verslen, $pkt_buf, BYTEOFFSET17, IP_VERSION_LENGTH); // get version/length field
				.elif (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_LLCSNAP) == TRUE)		; LLC SNAP frame
					xbuf_extract(ip_verslen, $pkt_buf, BYTEOFFSET22, IP_VERSION_LENGTH); // get version/length field
					// Save SA 2-5 as the packet wraps and overwrites pkt_buf0-1
					xbuf_extract(tempa, $pkt_buf, BYTEOFFSET0, 8, 4);  Save SA bytes 2-5 as the following rfifo_read overwrites
					xbuf_extract(tempb, $pkt_buf, BYTEOFFSET0, 12, 4);  Save len/ssap/dsap as the following rfifo_read overwrites
					xbuf_extract(tempc, $pkt_buf, BYTEOFFSET0, 16, 4);  Save CTL/OUI as the following rfifo_read overwrites
		   			rfifo_read($pkt_buf[2], rfifo_addr, QWOFFSET5, QWCOUNT1, ctx_swap);	// read next 1 quadwords
				.endif
				.if (ip_verslen == IPV4_NO_OPTIONS)		// if ipv4 no options		// see RFC 791
.endlocal // ip_verslen						
					.if (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_ETHERNET) == TRUE)			; Ethernet
						dram_rfifo_read(packet_buf_addr, QWOFFSET4, rfifo_addr, QWOFFSET4, QWCOUNT4, sig_done);// write 2nd 32B
						ip_verify(exception, $pkt_buf, BYTEOFFSET14);					// check ip header
						ctx_arb[voluntary]
						ip_modify($$dxfer, BYTEOFFSET14, $pkt_buf, BYTEOFFSET14);			// modifiy ip packet
						xbuf_extract(ip_dest, $pkt_buf, BYTEOFFSET14, IP_DESTINATION_ADDRESS); // get ip dest field
					.elif (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_LLC) == TRUE)			; LLC
						dram_rfifo_read(packet_buf_addr, QWOFFSET4, rfifo_addr, QWOFFSET4, QWCOUNT4, sig_done);// write 2nd 32B
						ip_verify(exception, $pkt_buf, BYTEOFFSET17);					// check ip header
						ctx_arb[voluntary]
						ip_modify($$dxfer, BYTEOFFSET17, $pkt_buf, BYTEOFFSET17);			// modifiy ip packet
						move($$dxfer[3], $pkt_buf[3]);
						xbuf_extract(ip_dest, $pkt_buf, BYTEOFFSET17, IP_DESTINATION_ADDRESS); // get ip dest field
					.elif (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_LLCSNAP) == TRUE)			; LLC SNAP
						dram_rfifo_read(packet_buf_addr, QWOFFSET5, rfifo_addr, QWOFFSET5, QWCOUNT3, sig_done);// write 2nd 32B
						ip_verify(exception, $pkt_buf, BYTEOFFSET22);					// check ip header
						ctx_arb[voluntary]
						ip_modify($$dxfer, BYTEOFFSET22, $pkt_buf, BYTEOFFSET22);			// modifiy ip packet
						xbuf_extract(ip_dest, $pkt_buf, BYTEOFFSET22, IP_DESTINATION_ADDRESS); // get ip dest field
					.endif
					xbuf_free($pkt_buf_ip);	

.local rt_ptr;								
					.if (exception)
						sig_wait(sdram, OR, 1);
packet_late_discard#:
						port_rx_discard(rec_state, rec_req, exception);				// set discard, inc counter
					.else								// not ip exception, do lookup
						ip_trie5_lookup(rt_ptr, ip_dest, SRAM_ROUTE_LOOKUP_BASE);		// longest prefix match
						sig_wait(sdram, OR, 1);
						dram_read($$dxfer[0], route_base, rt_ptr, QWCOUNT3, ctx_swap);	// get output_intf, MAC DA
.endlocal // rt_ptr
						.if (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_LLCSNAP) == TRUE)			; LLC SNAP
							dram_write($$dxfer[0], packet_buf_addr, QWOFFSET4, QWCOUNT1, no_signal);	// write modified packet
						.endif

						// $$dxfer0 = output port, $$dxfer1 = MAC DA bytes 0-3, $$dxfer2 = MAC DA bytes 4-5
						shf_left(output_intf, $$dxfer[0], 3);								// save for enqueue
						move($$dxfer[0], $$dxfer[1]);									// new DA bytes 0-3
#ifdef LITTLE_ENDIAN	
						add_shf_left($$dxfer[1], $$dxfer[2], sa01, 16);				// merge new DA 4-5 with SA 0-1
#else
						add_shf_left($$dxfer[1], sa01, $$dxfer[2], 16);				// merge new DA 4-5 with SA 0-1
#endif
						.if ((bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_ETHERNET) == TRUE) || (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_LLC) == TRUE))
							move($$dxfer[2], $pkt_buf[2]);									// previous SA bytes 2-5
						.elif (bit(packet_state, PKT_INFO_SHIFT_PKTLINKTYPE_LLCSNAP) == TRUE)			; LLC SNAP
							move($$dxfer[2], tempa);				// previous SA bytes 2-5
							move($$dxfer[3], tempb);				// len/ssap/dsap 
							move($$dxfer[4], tempc);				// CTL/OUI 
						.endif
.endlocal //ip_dest
						dram_write($$dxfer[0], packet_buf_addr, QWOFFSET0, QWCOUNT4, ctx_swap);	// write modified packet
					.endif   							// no ip exception do lookup
				.else									// IP with options or frag
					dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, QWCOUNT8, ctx_swap);
					shf_left(output_intf, CORE_STACK_INTF1, 3);							// if options or frag, q to core
				.endif									// IP with options or frag
			.endif									// not ethernet
		.endif
	.else											// not SOP
		port_rx_bytecount_extract(current_bytecount, rec_state);				// update packet byte count
		shf_right(current_qwcount, current_bytecount, 3);
		add(packet_buf_addr, packet_buf_addr, QWCOUNT8);
		dram_rfifo_read(packet_buf_addr, QWOFFSET0, rfifo_addr, QWOFFSET0, current_qwcount, ctx_swap);// write mpacket
.endlocal // dst_port_entry src_port_entry packet_state da_port_num packet_ethertype
	.endif											// not SOP
	xbuf_free($pkt_buf);

	.if (bit(rec_state, REC_STATE_EOP_BIT))			// if EOP
		.if (!bit(rec_state, REC_STATE_DISCARD_BIT))		// if not discard
			xbuf_alloc($desc_buf, 2);
			move($desc_buf[0], 0);														// descriptor to xfer
			.local com_descriptor
				shf_left(com_descriptor, FREELIST_ID, DESC1_FREELIST);					// insert freelist id
				#if (DESC1_PKT_END_BYTE <= REC_STATE_BYTE_COUNT);
					#define_eval _SHIFT_AMT  (REC_STATE_BYTE_COUNT - DESC1_PKT_END_BYTE)
					field_shf_right_insert(com_descriptor, 0010, rec_state, _SHIFT_AMT);// insert last byte count
				#else
					#define_eval _SHIFT_AMT  (DESC1_PKT_END_BYTE - REC_STATE_BYTE_COUNT)
					field_shf_left_insert(com_descriptor, 0010, rec_state, _SHIFT_AMT);
				#endif
				alu_op(packet_buf_addr, QWCOUNT8, +8, packet_buf_addr);					// incr ele countex
				alu_shf_right($desc_buf[1], com_descriptor, +8, packet_buf_addr, 3); 	// insert element count
			.endlocal
			sram_write($desc_buf[0], descriptor_addr, LWOFFSET0, LWCOUNT2, ctx_swap);	// write descriptor
enqueue#:
			packetq_send(output_intf, descriptor_addr, Q_HANDLE); 						// at eop, q to xmit
			move(packet_buf_addr, UNALLOCATED);
			xbuf_free($desc_buf);
		.else											// Discard
			// set packet_buf_addr to begining of the packet - Reuse the packet
			buf_dram_addr_from_sram_addr(packet_buf_addr, descriptor_addr, FREELIST_HANDLE);
		.endif
		move(rec_state, 0);															// init rec_state
	.endif											// EOP
.endlocal // sa01 output_intf

header_stored#:
mid_mpacket_stored#:
last_mpacket_stored#:
.endw
xbuf_free($$dxfer);
#include "hash_resolve.uc"


