/* GigGmiiAnRg.h
 * 
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                 
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: GigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: GigGmiiAnRg.h $
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 7/16/99    Time: 2:45p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:49p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 
 * ---------------------------------------------------------------------
 */

//    GMII Auto Negotiation:
//  1.  GMII Control register 
//  2.  GMII Status register 
//  3.  AN advertisement register 
//  4.  AN link partner ability base page register 
//  5.  AN expansion register 
//  6.  AN next page transmit register
//  7.  AN link partner receive next page register 
//  8.  Extended status register 
//  9.  GPCS status register


#ifndef GIG_MAC_GMII_AN_REGS
#define GIG_MAC_GMII_AN_REGS

class GigMacGmiiAnRegs{
public:

	GigMacGmiiAnRegs(GigMacControlRegs & regs):_ControlRegs(regs){
		//cout << "GigMacGmiiAnRegs CTOR" << endl;
	};

	~GigMacGmiiAnRegs(){};



/*
unsigned short int GMIIControl; 					//GMII_CTL		50H-51H	2 Bits 9, 12, 14, 15 R/W, bits 6, 7, 8, 10, 11, 13 R.	
unsigned short int GMIIStatus; 						//GMII_STT		52H-53H	2 R	
unsigned short int ANAdvertisement; 				//AN_ADV		58H-59H	2 Bits 7, 8, 12, 13, 15 R/W. Bits 6:0, 9, 10, 11, 14, R.	
unsigned short int ANLinkPartnerAbilityBasePage; 	//AN_PRT_ABL	5AH-5BH	2 R	
unsigned short int ANExpansion; 					//AN_EXP		5CH-5DH	2 R	
unsigned short int ANNextPageTransmit;	
*/


/*
Advance Datasheet GMII Management Register Set
The following sections describe the individual GMII management register set as defined in the
IEEE 802.3z standard.
Any bit in these registers that has a fixed value is not writable.
Note: These registers are accessible only in GPCS mode.



GMII Control Register
Mnemonic: GMII_CTL
Address: 50H - 51H

The control register provides the mechanism to enable or disable Auto-Negotiation, restart
Auto-Negotiation, and allow for manual configuration when Auto-Negotiation is disabled.
Bit Name	Bit #	Bit	Description
RST			15		Reset.
					When set, causes reset to the GPCS logic and to the GMII management registers. When
					reset, indicates normal operation. This bit is self clearing. The value of this bit remains at 1,
					until the reset process is completed.
LPBK		14		External loopback.
					When set, enables external loopback mode, and causes assertion of ewrap_{i} signal.
					When reset, disables external loopback mode, and causes deassertion of ewrap_{i} signal.
SPEED_2		13		Speed selection.
					This bit is fixed at the value of 0 and the value of bit 6 is fixed to 1, determining a speed
					selection of 1000 Mb/s.
ANENBL		12		Auto-Negotiation enable.
					When set, Auto-Negotiation process is enabled. When reset, Auto-Negotiation process is
					disabled.
PWRD		11		Power down.
					This bit is fixed at the value of 0. The 82547EB GPCS does not support the power down
					function.
ISLT		10		Isolate.
					This bit is fixed at the value of 0. The 82547EB GPCS does not support the isolate
					function.
RESAN		9		Restart Auto-Negotiation.
					When set, restarts Auto-Negotiation process. When reset, indicates normal operation.
					This bit is self clearing. The value of this bit will change back to 0 when the
					Auto-Negotiation process is initiated.
					If Auto-Negotiation is disabled (ANENBL=0), this bit will have a value of 0.
DUPLEX		8		Duplex mode.
					This bit is fixed at the value of 1. The 82547EB is operating only in the full-duplex mode.
COLLTST		7		Collision test.
					This bit is fixed at the value of 0. The 82547EB GPCS does not support the collision signal
					test.
SPEED_1		6		Speed selection.
					This bit is fixed at the value of 1 and the value of bit 13 is fixed at 0, determining a speed
					selection of 1000 Mb/s
			5:0		RESERVED.
		
Access Rules Register
access Bits 9, 12, 14, 15 R/W, bits 6, 7, 8, 10, 11, 13 R.
Value after reset 1140H
*/


public:

	inline unsigned short int GetGMIIControl()const { return GMIIControlF();};

	inline unsigned int GetGMIIControlRST()const { return ( GMIIControlF() >> RST_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlLPBK()const { return ( GMIIControlF() >> LPBK_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlSPEED_2()const { return ( GMIIControlF() >> SPEED_2_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlANENBL()const { return ( GMIIControlF() >> ANENBL_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlPWRD()const { return ( GMIIControlF() >> PWRD_SH) & GMII_CTRL_MSK;};	
	inline unsigned int GetGMIIControlISLT()const { return ( GMIIControlF() >> ISLT_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlRESAN()const { return ( GMIIControlF() >> RESAN_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlDUPLEX()const { return ( GMIIControlF() >> DUPLEX_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlCOLLTST()const { return ( GMIIControlF() >> COLLTST_SH) & GMII_CTRL_MSK;};
	inline unsigned int GetGMIIControlSPEED_1()const { return ( GMIIControlF() >> SPEED_1_SH) & GMII_CTRL_MSK;};
	
	inline void SetGMIIControl(unsigned short int val){ GMIIControlF(val);};

	inline void SetGMIIControlRST(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << RST_SH));};
	inline void SetGMIIControlLPBK(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << LPBK_SH));};
	//RO inline void SetGMIIControlSPEED_2(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << SPEED_2_SH));};
	inline void SetGMIIControlANENBL(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << ANENBL_SH));};
	//RO inline void SetGMIIControlPWRD(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << PWRD_SH));};
	//RO inline void SetGMIIControlISLT(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << ISLT_SH));};
	inline void SetGMIIControlRESAN(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << RESAN_SH));};
	//RO inline void SetGMIIControlDUPLEX(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << DUPLEX_SH));};
	//RO inline void SetGMIIControlCOLLTST(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << COLLTST_SH));};
	//RO inline void SetGMIIControlSPEED_1(){   GMIIControlF (GMIIControlF() |(GMII_CTRL_MSK << SPEED_1_SH));};


private:

	inline unsigned int GMIIControlF()const {return _ControlRegs.GetGMIIControl();};
	inline void GMIIControlF(unsigned short int val){_ControlRegs.SetGMIIControl(val);};
    
enum GMIIManagementAccessShifts{
	RST_SH=	    15,		
	LPBK_SH=    14,		
	SPEED_2_SH=	13,
	ANENBL_SH=	12,	
	PWRD_SH=	11,	
	ISLT_SH=	10,
	RESAN_SH=   9,
	DUPLEX_SH=  8,
	COLLTST_SH= 7,
	SPEED_1_SH= 6
};

enum GMIIManagementAccessMasks{
	GMII_CTRL_MSK =0x1
	};

	
/*
----------------------------

GMII Status Register
Mnemonic: GMII_STT
Address: 52H - 53H

The GMII status register includes information about all modes of operations supported by the local
device and the status of Auto-Negotiation.


Bit Name	Bit #	Bit Description
100T4		15		100BASE-T4 ability.
					This bit is fixed at the value of 0.
100XFD		14		100BASE-X full-duplex ability.
					This bit is fixed at the value of 0.
100XHD		13		100BASE-X half-duplex ability.
					This bit is fixed at the value of 0.
10FD		12		10 Mb/s full-duplex ability.
					This bit is fixed at the value of 0.
10HD		11		10 Mb/s half-duplex ability.
					This bit is fixed at the value of 0.
100T2FD		10		100BASE-T2 full-duplex ability.
					This bit is fixed at the value of 0.
100T2HD		9		100BASE-T2 half-duplex ability.
					This bit is fixed at the value of 0.
EXTSTT		8		Extended status.
					This bit is fixed at the value of 1, indicating additional status information in the extended
					status register (GMII_EXT_STT).
			7		RESERVED
MFPRE		6		MF preamble suppression.
					This bit is fixed at the value of 0. The 82547EB registers cannot be accessed through the
					mdio pin.
ANCMPL		5		Auto-Negotiation complete.
					A value of 1 indicates that Auto-Negotiation process is completed. A value of 0 indicates
					that Auto-Negotiation process is not completed.
RMTFLT		4		Remote fault.
					A value of 1 indicates that a remote fault condition was detected in the link partners base
					page. A value of 0 indicates that a remote fault condition was not detected.
ANABLT		3		Auto-negotiation ability.
					This bit is fixed at the value of 1, indicating that the GPCS is able to perform
					Auto-Negotiation.
LNKSTT		2		Link status.
					A value of 1 indicates that the link is up. A value of 0 indicates that the link is down. Link up
					is defined as a status in which the synchronization process and the Auto-Negotiation
					process were completed and the device is ready to transmit or receive data.
JBRDTC		1		Jabber detect.
					This bit is fixed at the value of 0. The 82547EB GPCS does not support jabber detection.
EXTCAP		0		Extended capability.
					This bit is fixed at the value of 0, indicating basic register set capabilities only.
Access Rules Register access R
Value after reset 0108H
*/

public:

	inline unsigned short int GetGMIIStatus()const { return GMIIStatusF();};

	inline unsigned int GMIIStatusS_100T4()const { return ( GMIIControlF() >> S_100T4_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusS_100XFD()const { return ( GMIIControlF() >> S_100XFD_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusS_100XHD()const { return ( GMIIControlF() >> S_100XHD_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusS_10FD()const { return ( GMIIControlF() >> S_10FD_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusS_10HD()const { return ( GMIIControlF() >> S_10HD_SH) & GMII_STAT_MSK;};	
	inline unsigned int GMIIStatusS_100T2FD()const { return ( GMIIControlF() >> S_100T2FD_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusS_100T2HD()const { return ( GMIIControlF() >> S_100T2HD_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusEXTSTT()const { return ( GMIIControlF() >> EXTSTT_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusMFPRE()const { return ( GMIIControlF() >> MFPRE_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusANCMPL()const { return ( GMIIControlF() >> ANCMPL_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusRMTFLT()const { return ( GMIIControlF() >> RMTFLT_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusANABLT()const { return ( GMIIControlF() >> ANABLT_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusLNKSTT()const { return ( GMIIControlF() >> LNKSTT_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusJBRDTC()const { return ( GMIIControlF() >> JBRDTC_SH) & GMII_STAT_MSK;};
	inline unsigned int GMIIStatusEXTCAP()const { return ( GMIIControlF() >> EXTCAP_SH) & GMII_STAT_MSK;};
	

private:

	inline unsigned int GMIIStatusF()const {return _ControlRegs.GetGMIIStatus();};
    
enum GMIIStatusShifts{
	S_100T4_SH	=	15,
	S_100XFD_SH =	14,
	S_100XHD_SH =	13,
	S_10FD_SH	=	12,
	S_10HD_SH	=	11,
	S_100T2FD_SH =	10,
	S_100T2HD_SH =	9,
	EXTSTT_SH   = 	8,
	MFPRE_SH    =	6,
	ANCMPL_SH	=	5,
	RMTFLT_SH	=	4,
	ANABLT_SH	=	3,
	LNKSTT_SH	=	2,
	JBRDTC_SH	=	1,
	EXTCAP_SH	=	0
};

enum GMIIStatusMasks{
	GMII_STAT_MSK =0x1
	};

/*
-----------------------------------------


AN Advertisement Register
Mnemonic: AN_ADV
Address: 58H - 59H

The AN advertisement register contains the advertised ability of the local device. If another value
than the default value is written into this register, then Auto-Negotiation should be restarted by
setting bit GMII_CTL<RESAN>. In GPCS mode when Auto-Negotiation is not disabled, the
ASM_DIR and PAUSE bits contain the desired flow-control mode of operation. The actual
flow-control mode will be figured during the Auto-Negotiation process and will be reported in the
GPCS status register GPCS_STT.

Bit Name	Bit #	Bit Description
NXTPG		15		Next page.
					When set, indicates a request for next page exchange.
			14		RESERVED
RF			13:12	Remote fault.
					The remote fault bits are used to notify the link partner about a fault that had occurred.
					These bits are cleared after successful page exchanging. The table below specifies the
					remote fault encoding.
					<12>	<13>	Description
					0		0		No error, link OK
					0		1		Off line
					1		0		Link failure
					1		1		Auto-Negotiation error
			11:9	RESERVED
ASM_DIR		8		Asymmetric PAUSE connection is desired.
					When set, results in independent enabling/disabling of the flow-control receive and transmit.
					When reset, results in symmetric enabling/disabling of the flow-control receive and transmit.
PAUSE		7		PAUSE function.
					When set, indicates that the local device is capable and intends to stop upon reception of
					flow-control packets as defined in IEEE 802.3x.
					When reset, indicates that the local device is not capable or does not intend to stop upon
					reception of a flow-control packet.
HD			6		half-duplex ability.
					This bit is fixed at a value of 0.
FD			5		full-duplex ability.
					This bit is fixed at a value of 1.
			4:0		RESERVED

Access Rules Register access Bits 7, 8, 12, 13, 15 R/W. Bits 6:0, 9, 10, 11, 14, R.
Value after reset 01A0H
*/


public:

	inline unsigned short int GetANAdvertisement()const { return ANAdvertisementF();};

	inline unsigned int GetANAdvertisementNXTPG()const { return ( ANAdvertisementF() >> NXTPG_SH) & AN_ADV_MSK;};
	inline unsigned int GetANAdvertisementRF()const { return ( ANAdvertisementF() >> RF_SH) & AN_ADV_RF_MSK;};
	inline unsigned int GetANAdvertisementASM_DIR()const { return ( ANAdvertisementF() >> ASM_DIR_SH) & AN_ADV_MSK;};
	inline unsigned int GetANAdvertisementPAUSE()const { return ( ANAdvertisementF() >> PAUSE_SH) & AN_ADV_MSK;};
	inline unsigned int GetANAdvertisementHD()const { return ( ANAdvertisementF() >> HD_SH) & AN_ADV_MSK;};	
	inline unsigned int GetANAdvertisementFD()const { return ( ANAdvertisementF() >> FD_SH) & AN_ADV_MSK;};



	inline void SetANAdvertisement(unsigned short int val){ ANAdvertisementF(val);};

	inline void SetANAdvertisementRF(){   ANAdvertisementF (ANAdvertisementF() |(AN_ADV_RF_MSK << RF_SH));};
	inline void SetANAdvertisementASM_DIR(){   ANAdvertisementF (ANAdvertisementF() |(AN_ADV_MSK << ASM_DIR_SH));};
	inline void SetANAdvertisementPAUSE(){   ANAdvertisementF (ANAdvertisementF() |(AN_ADV_MSK << PAUSE_SH));};


private:

	inline unsigned int ANAdvertisementF()const {return _ControlRegs.GetANAdvertisement();};
	inline void ANAdvertisementF(unsigned short int val){_ControlRegs.SetANAdvertisement(val);};
    
enum ANAdvertisementShifts{
	NXTPG_SH=	15,		
	RF_SH=		12,	
	ASM_DIR_SH=	8,	
	PAUSE_SH=	7,
	HD_SH=   6,
	FD_SH=  5
};

enum ANAdvertisementMasks{
	AN_ADV_RF_MSK =0x3,
	AN_ADV_MSK =0x1
};

	
/*
-------------------------------

AN Link Partner Ability Base Page Register
Mnemonic: AN_PRT_ABL
Address: 5AH - 5BH

The AN link partner ability base page register contains the advertised ability of the partners device.
This register is updated once every Auto-Negotiation process, at the end of the page exchange state.

Bit Name	Bit		# Bit Description
NXTPG		15		Next page.
					When set, indicates a request for next page exchange was made by the link partner.
ACK			14      Acknowledge.
					When set, indicates that the link partner has successfully received the base page of the
					local device.
RF			13:12	Remote fault.
					The remote fault bits are used to notify the local device about a fault that had occurred at
					the link partner device. The table below specifies the remote fault encoding.
					<12>	<13>	Description
					0		0		No error, link OK
					0		1		Off line
					1		0		Link failure
					1		1		Auto-Negotiation error
			11:9	RESERVED
ASM_DIR		8		Asymmetric PAUSE connection is desired.
					When set, results in independent enabling of the flow-control receive and transmit.
					When reset, results in symmetric enabling of the flow-control receive and transmit.
PAUSE		7		Capable of configurating the PAUSE function. When set, indicates that the link
					partners device is capable and intends to stop upon reception of flow-control packets, as
					defined in IEEE 802.3x.
HD			6		Half duplex.
					When set, indicates link partners half-duplex capability.
FD			5		Full duplex.
					When set, indicates link partners full-duplex capability.

			4:0		RESERVED
Access Rules Register access R
Value afterreset 0000H
*/


public:

	inline unsigned short int GetANLinkPartnerAbilityBasePage()const 
								{ return ANLinkPartnerAbilityBasePageF();};

	inline unsigned int GetANLinkPartnerAbilityBasePageNXTPG()const 
								{ return ( ANLinkPartnerAbilityBasePageF() >> NXTPG_PAR_SH) & AN_ADV_PAR_MSK;};
	inline unsigned int GetANLinkPartnerAbilityBasePageRF()const 
								{ return ( ANLinkPartnerAbilityBasePageF() >> RF_PAR_SH) & AN_ADV_RF_PAR_MSK;};
	inline unsigned int GetANLinkPartnerAbilityBasePageACK()const 
								{ return ( ANLinkPartnerAbilityBasePageF() >> ACK_PAR_SH) & AN_ADV_PAR_MSK;};
	inline unsigned int GetANLinkPartnerAbilityBasePageASM_DIR()const 
								{ return ( ANLinkPartnerAbilityBasePageF() >> ASM_DIR_PAR_SH) & AN_ADV_PAR_MSK;};
	inline unsigned int GetANLinkPartnerAbilityBasePagePAUSE()const 
								{ return ( ANLinkPartnerAbilityBasePageF() >> PAUSE_PAR_SH) & AN_ADV_PAR_MSK;};
	inline unsigned int GetANLinkPartnerAbilityBasePageHD()const 
								{ return ( ANLinkPartnerAbilityBasePageF() >> HD_PAR_SH) & AN_ADV_PAR_MSK;};	
	inline unsigned int GGetANLinkPartnerAbilityBasePageFD()const 
								{ return ( ANLinkPartnerAbilityBasePageF() >> FD_PAR_SH) & AN_ADV_PAR_MSK;};


private:

	inline unsigned int ANLinkPartnerAbilityBasePageF()const 
								{return _ControlRegs.GetANLinkPartnerAbilityBasePage();};

    
enum ANLinkPartnerAbilityBasePageShifts{
	NXTPG_PAR_SH = 15,
	ACK_PAR_SH	 = 14,
	RF_PAR_SH    = 12,	
	ASM_DIR_PAR_SH=	8,	
	PAUSE_PAR_SH=	7,
	HD_PAR_SH=   6,
	FD_PAR_SH=  5
};

enum ANLinkPartnerAbilityBasePageMasks{
	AN_ADV_RF_PAR_MSK =0x3,
	AN_ADV_PAR_MSK =0x1
};

	
/*
-------------------------

AN Expansion Register
Mnemonic: AN_EXP
Address: 5CH - 5DH

The AN expansion register includes information about page reception and ability of next page
function.

Bit Name	Bit #	Bit Description
			15:8	RESERVED
NPRCV		7		Next page received.
					A value of 1 in this bit, indicates that the link partners current received page is a next page.
					A value of 0 in this bit indicates that the link partners current received page is the base
					page.
			6:3		RESERVED
NPABL		2		Next page able.
					This bit is fixed at the value of 1. The 82547EB has next page ability.
PGRCV		1		Page received.
					A value of 1 in this bit indicates that the link partners base or next page has been received.
					Bit #7 in this register indicates whether the received page is a base or next page.
					This bit is reset by read.
			0		RESERVED
			
Access Rules Register access R
Value after reset 0004H

*/


public:

	inline unsigned short int GetANExpansion()const { return ANExpansionF();};

	inline unsigned int GetANExpansionNPRCV()const { return ( ANExpansionF() >> NPRCV_SH) & AN_EXP_MSK;};
	inline unsigned int GetANExpansionNPABL()const { return ( ANExpansionF() >> NPABL_SH) & AN_EXP_MSK;};
	inline unsigned int GetANExpansionPGRCV()const { return ( ANExpansionF() >> PGRCV_SH) & AN_EXP_MSK;};


private:

	inline unsigned int ANExpansionF()const {return _ControlRegs.GetANExpansion();};

    
enum ANExpansionShifts{
	NPRCV_SH	= 7,
	NPABL_SH	= 2,
	PGRCV_SH    = 1
};

enum ANExpansionMasks{
	AN_EXP_MSK =0x1
};

/*
--------------------------------


AN Next Page Transmi Register
Mnemonic: AN_NP_TR
Address: 5EH - 5FH

The AN t register contains the data that will be transmitted during next page
exchange.
Note: When CPU bus is in 8 bit mode, write access to this register must be done to both addresses,
starting at the low address.

Bit Name	Bit #	Bit Description
NXTPG		15		Next page.
					When set, indicates a request for an additional next page exchange was made by the link
					partner.
			14		RESERVED
MP			13		Message page.
					This bit indicates whether this page is a message page or an unformatted page.
ACK2		12		Acknowledge 2.
					This bit is used to indicate whether the device has the ability to comply with the message.
			11		RESERVED
ENCOD		10:0	Next page encoding.

Access Rules Register access Bits 15, 13:0 R/W. Bit 14, 11 R.
Value after reset 0000H

*/

public:

	inline unsigned short int GetANNextPageTransmit()const { return ANNextPageTransmitF();};

	inline unsigned int GetANNextPageTransmitNXTPG()const { return ( ANNextPageTransmitF() >> NXTPG_TX_SH) & AN_NPAGE_TX_MSK;};
	inline unsigned int GetANNextPageTransmitMP()const { return ( ANNextPageTransmitF() >> MP_TX_SH) & AN_NPAGE_TX_MSK;};
	inline unsigned int GetANNextPageTransmitACK2()const { return ( ANNextPageTransmitF() >> ACK2_TX_SH) & AN_NPAGE_TX_MSK;};
	inline unsigned int GetANNextPageTransmitENCODE()const { return ( ANNextPageTransmitF() >> ENCODE_SH) & AN_NPAGE_TX_ENCOD_MSK;};


	inline void SetANNextPageTransmit(unsigned short int val){ ANNextPageTransmitF(val);};

	inline void SetANNextPageTransmitNXTPG(){ ANNextPageTransmitF (ANNextPageTransmitF() |(AN_NPAGE_TX_MSK << NXTPG_TX_SH));};
	inline void SetANNextPageTransmitMP(){    ANNextPageTransmitF (ANNextPageTransmitF() |(AN_NPAGE_TX_MSK << MP_TX_SH));};
	inline void SetANNextPageTransmitACK2(){  ANNextPageTransmitF (ANNextPageTransmitF() |(AN_NPAGE_TX_MSK << ACK2_TX_SH));};
	inline void SetANNextPageTransmitENCODE(){ANNextPageTransmitF (ANNextPageTransmitF() |(AN_NPAGE_TX_ENCOD_MSK << ENCODE_SH));};


private:

	inline unsigned int ANNextPageTransmitF()const {return _ControlRegs.GetANNextPageTransmit();};
	inline void ANNextPageTransmitF(unsigned short int val){_ControlRegs.SetANNextPageTransmit(val);};
    
enum ANNextPageTransmitShifts{
	NXTPG_TX_SH=	15,		
	MP_TX_SH=		12,	
	ACK2_TX_SH=	8,	
	ENCODE_SH=	7
};

enum ANNextPageTransmitMasks{
	AN_NPAGE_TX_MSK =0x1,
	AN_NPAGE_TX_ENCOD_MSK =0x3ff
};



/*
----------------------------------------------
AN Link Partner Receive Next Page Register
Mnemonic: AN_PRT_NP
Address: 60H - 61H

The AN link partner receive next page register contains the data of the link parameter next page.

Bit Name	Bit #	Bit Description
NXTPG		15		Next page.
					When set, indicates a request for an additional next page exchange was made by the link
					partner.
			14		RESERVED
MP			13		Message page.
					This bit indicates whether this page is a message page or an unformatted page.
ACK2		12		Acknowledge 2.
					This bit is used to indicate whether the link partners device has the ability to comply with
					the message.
TGL			11		Toggle.
					This bit should get the opposite value of the toggle bit in the previously exchanged page.
ENCOD		10:0	Next page encoding.

Access Rules Register access R
Value after reset 0000H
*/

public:

	inline unsigned short int ANLinkPartnerReceiveNextPage()const { return ANLinkPartnerReceiveNextPageF();};

	inline unsigned int ANLinkPartnerReceiveNextPageNXTPG()
					const { return ( ANLinkPartnerReceiveNextPageF() >> NXTPG_PRX_SH) & LK_PR_RX_MSK;};
	inline unsigned int ANLinkPartnerReceiveNextPageMP()
					const { return ( ANLinkPartnerReceiveNextPageF() >> MP_SH) & LK_PR_RX_MSK;};
	inline unsigned int ANLinkPartnerReceiveNextPageACK2()
					const { return ( ANLinkPartnerReceiveNextPageF() >> ACK2_SH) & LK_PR_RX_MSK;};
	inline unsigned int ANLinkPartnerReceiveNextPageTGL()
					const { return ( ANLinkPartnerReceiveNextPageF() >> TGL_SH) & LK_PR_RX_MSK;};
	inline unsigned int ANLinkPartnerReceiveNextPageENCOD()
					const { return ( ANLinkPartnerReceiveNextPageF() >> ENCOD_SH) & LK_PR_RX_MSK;};

private:

	inline unsigned int ANLinkPartnerReceiveNextPageF()const {return _ControlRegs.GetANLinkPartnerReceiveNextPage();};

    
enum ANLinkPartnerReceiveNextPageShifts{
	NXTPG_PRX_SH  = 15,
	MP_SH	  = 13,
	ACK2_SH   = 12,
	TGL_SH    = 11,
	ENCOD_SH  = 10
};

enum ANLinkPartnerReceiveNextPageMasks{
	LK_PR_RX_MSK =0x1
};





/*
------------------------------------------------
Advance Datasheet Extended Status Register
Mnemonic: GMII_EXT_STT
Address: 6EH - 6FH

The extended status register includes information about Giga bit modes of operations supported by
the local device.

Bit Name	Bit #	Bit Description
1000XFD		15		1000BASE-X full-duplex ability.
					This bit is fixed at the value of 1.
1000XHD		14		1000BASE-X half-duplex ability.
					This bit is fixed at the value of 0.
1000TFD		13		1000BASE-T full-duplex ability.
					This bit is fixed at the value of 0.
1000THD		12		1000BASE-T half-duplex ability.
					This bit is fixed at the value of 0.
			11: 0	RESERVED
			
Access Rules Register access R
Value after reset 8000H
*/


public:

	inline unsigned short int ExtendedStatus()const { return ExtendedStatusF();};

	inline unsigned int ExtendedStatus1000XFD()const { return ( ExtendedStatusF() >> M_1000XFD_SH) & EXT_ST_MSK;};
	inline unsigned int ExtendedStatus1000XHD()const { return ( ExtendedStatusF() >> M_1000XHD_SH) & EXT_ST_MSK;};
	inline unsigned int ExtendedStatus1000TFD()const { return ( ExtendedStatusF() >> M_1000TFD_SH) & EXT_ST_MSK;};
	inline unsigned int ExtendedStatus1000THD()const { return ( ExtendedStatusF() >> M_1000THD_SH) & EXT_ST_MSK;};

private:

	inline unsigned int ExtendedStatusF()const {return _ControlRegs.GetExtendedStatus();};

    
enum ExtendedStatusShifts{
	M_1000XFD_SH = 15,
	M_1000XHD_SH = 14,
	M_1000TFD_SH = 13,
	M_1000THD_SH = 12
};

enum ExtendedStatusMasks{
	EXT_ST_MSK =0x1
};



/*
--------------------------------------------
GPCS Status Register
Mnemonic: GPCS_STT
Address: 70H - 71H

The GPCS status register reports the parameters of the Auto-Negotiation and synchronization
process. This register is used only in GPCS mode.

Bit Name	Bit #	Bit Description
			15:6	RESERVED
SYNST		5		Synchronization status.
					A value of '0' indicates out of synchronization status. A value of '1' indicates that
					synchronization was acquired.
			4:2		RESERVED
LPPS		1		Link partner flow-control status.
					A value of '0' indicates that link partners transmission will not be stopped upon reception
					of flow-control packets.
					A value of '1' indicates that link partners transmission will be stopped upon reception of
					flow-control packets.
RFC			0		Receive flow-control status.
					A value of '0' indicates that transmission will not be stopped upon reception of flow-control
					packets.
					A value of '1' indicates that transmission will be stopped upon reception of flow-control
					packets.
Access Rules Register access R
Value after reset 0000H

*/

public:

	inline unsigned short int GPCSstatus()const { return GPCSstatusF();};

	inline unsigned int GPCSstatusSYNST()const { return ( GPCSstatusF() >> SYNST_SH) & GPCS_ST_MSK;};
	inline unsigned int GPCSstatusLPPS ()const { return ( GPCSstatusF() >> LPPS_SH) & GPCS_ST_MSK;};
	inline unsigned int GPCSstatusRFC  ()const { return ( GPCSstatusF() >> RFC_SH) & GPCS_ST_MSK;};


private:

	inline unsigned int GPCSstatusF()const {return _ControlRegs.GetGPCSstatus();};

    
enum GPCSstatusShifts{
	SYNST_SH = 5,
	LPPS_SH	 = 1,
	RFC_SH   = 0
};

enum GPCSstatusMasks{
	GPCS_ST_MSK =0x1
};

//

GigMacControlRegs & _ControlRegs;
};

#endif //GIG_MAC_GMII_AN_REGS 


