/* GigGmiiRg.h
 * 
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                 
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: GigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: GigGmiiRg.h $
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 7/16/99    Time: 2:45p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:49p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 
 * ---------------------------------------------------------------------
 */

//  GMII:
//  1. GMII managment accesss
//  2. GMII Managment data


#ifndef GIG_MAC_GMII_REGS
#define GIG_MAC_GMII_REGS
//These registers are valid only in port 0


class GigMacGmiiRegs{
public:

	GigMacGmiiRegs(GigMacControlRegs & regs):_ControlRegs(regs){
		//cout << "GigMacGmiiRegs CTOR" << endl;
	};

	~GigMacGmiiRegs(){};

/*
GMII Management Access Registers
---------------------------------
This set of registers is intended to simplify access to the GMII management registers in the PHY,
while working in GMII mode.
In order to access a GMII management register in the PHY, an access command should be written
in the GMII management access register (GMII_MNG_ACC). The 82547EB carries out the
command and accesses the PHY through the mdc and mdio pins.
In GPCS mode, the set of GMII management registers is provided on the 82547EB in addresses
50H - 71GH, and these addresses should be accessed directly.


Note: !!!!!!!!!!!  These registers are valid only in port 0.


GMII Management Access Register
Mnemonic: GMII_MNG_ACC
Address: 4AH  4BH (only in port 0)

The GMII Management Access register controls the mdc and mdio I/O pins. Two modes of access
to the PHY are available
Direct access mode: In this mode each of the mdc and mdio I/O pins are set to be input or
output, and can be read or written through this register.
Automatic access mode: This mode is used to access the GMII management register set in the
PHY. The address of the desired PHY and register, and the mode of access (R/W), are defined
in this register. In this mode the 82547EB will transmit and receive frames to/from the PHY
through the mdc and mdio pins, as defined in the 802.3u standard.
The automatic access will begin after writing a value of 1 to the AMAM bit, if the CPU is in the
16-bit mode, or after a write access to the high byte of this register if the value of the AMAM bit is
1 and the CPU is in 8-bit mode.




Bit Name	Bit #	Bit Description
MDIO		15		GMII data IO
					In direct access mode (AMAM=0):
					During mdio write access (RWGMI=1), the value written to this bit is driven onto the
					mdio signal.
					During mdio read access (RWGMI=0), the mdio signal value is latched into this bit.
					In automatic access mode (AMAM=1):
					This bit is not valid.
MDC			14		MDC clock value
					In direct access mode (AMAM=0):
					During mdc write access (MDCEN=1), the value written to this bit is driven onto the
					mdc signal. When access to the PHY is done using this bit, each new value must
					remain stable for at least 200 ns.
					During mdc read access (MDCEN=0), the mdc signal value is latched into this bit.
					In automatic access mode (AMAM=1):
					This bit is not valid
MDCEN		13		Enable MDC output
					In direct access mode (AMAM=0):
					When set enables mdc output.
					In automatic access mode (AMAM=1):
					This bit is not valid.
RWGMI		12		Read/write GMII management registers.
					In direct access mode (AMAM=0):
					When set enables mdio output.
					In automatic access mode (AMAM=1):
					Set this bit for writing the data from the GMII_MNG_DAT register into the GMII
					management desired register.
					Reset this bit for reading the GMII management desired register into the
					GMII_MNG_DAT register.
PHY_ADD		11:7	GMII PHY address.
					The address of the desired PHY. Bit 11 is the most significant bit and is the first PHY
					address bit to be transmitted and received.
REG_ADD		6:2		GMII register address.
					The address of the desired register. Bit 6 is the most significant bit and is the first register
					address bit to be transmitted and received.
GMIRY		1		GMII management access registers are ready.
					This bit indicates whether the access to the GMII management registers had been
					completed. During the access process, the value of this bit will be 0.
					The bit will be set at the end of the process.
					Writing to the GMII management access registers while this bit is cleared will be ignored,
					although the crdy_l signal will be asserted.
AMAM		0		Automatic mdio access mode
					Setting this bit will begin the Automatic access.
					Note: If the cpu bus is in 8-bit mode the automatic access will occur after a write access
					to the high byte of the register.


Register access During GMII automatic access (GMIRY = 0) R, otherwise R/W.
Value after reset 0003H
*/
public:

	inline unsigned short int GetGMIIManagementAccess()const { return GMIIManagementAccessF();};

	inline unsigned int GetGMIIManagementAccessMDIO()const { return ( GMIIManagementAccessF() >> MDIO_SH) & GMII_MNGM_ACC_MSK;};
	inline unsigned int GetGMIIManagementAccessMDC()const { return ( GMIIManagementAccessF() >> MDC_SH) & GMII_MNGM_ACC_MSK;};
	inline unsigned int GetGMIIManagementAccessMDCEN()const { return ( GMIIManagementAccessF() >> MDCEN_SH) & GMII_MNGM_ACC_MSK;};
	inline unsigned int GetGMIIManagementAccessRWGMI()const { return ( GMIIManagementAccessF() >> RWGMI_SH) & GMII_MNGM_ACC_MSK;};
	inline unsigned int GetGMIIManagementAccessPHY_ADD()const { return ( GMIIManagementAccessF() >> PHY_ADD_SH) & PHY_ADD_MSK;};	
	inline unsigned int GetGMIIManagementAccessREG_ADD()const { return ( GMIIManagementAccessF() >> REG_ADD_SH) & REG_ADD_MSK;};
	inline unsigned int GetGMIIManagementAccessGMIRY()const { return ( GMIIManagementAccessF() >> GMIRY_SH) & GMII_MNGM_ACC_MSK;};
	inline unsigned int GetGMIIManagementAccessAMAM()const { return ( GMIIManagementAccessF() >> AMAM_SH) & GMII_MNGM_ACC_MSK;};
	
	inline void SetGMIIManagementAccess(unsigned short int val){ GMIIManagementAccessF(val);};

	inline void SetGMIIManagementAccessMDIO()  {   GMIIManagementAccessF (GMIIManagementAccessF() |(GMII_MNGM_ACC_MSK << MDIO_SH));};
	inline void SetGMIIManagementAccessMDC()   {   GMIIManagementAccessF (GMIIManagementAccessF() |(GMII_MNGM_ACC_MSK << MDC_SH));};
	inline void SetGMIIManagementAccessMDCEN() {   GMIIManagementAccessF (GMIIManagementAccessF() |(GMII_MNGM_ACC_MSK << MDCEN_SH));};
	inline void SetGMIIManagementAccessRWGMI() {   GMIIManagementAccessF (GMIIManagementAccessF() |(GMII_MNGM_ACC_MSK << RWGMI_SH));};
	inline void SetGMIIManagementAccessPHY_ADD(){   GMIIManagementAccessF (GMIIManagementAccessF() |(PHY_ADD_MSK << PHY_ADD_SH));};
	inline void SetGMIIManagementAccessREG_ADD(){   GMIIManagementAccessF (GMIIManagementAccessF() |(REG_ADD_MSK << REG_ADD_SH));};
	inline void SetGMIIManagementAccessGMIRY()  {   GMIIManagementAccessF (GMIIManagementAccessF() |(GMII_MNGM_ACC_MSK << GMIRY_SH));};
	inline void SetGMIIManagementAccessAMAM()   {   GMIIManagementAccessF (GMIIManagementAccessF() |(GMII_MNGM_ACC_MSK << AMAM_SH));};


private:

	inline unsigned int GMIIManagementAccessF()const {return _ControlRegs.GetGMIIManagementAccess();};
	inline void GMIIManagementAccessF(unsigned short int val){_ControlRegs.SetGMIIManagementAccess(val);};
    
enum GMIIManagementAccessShifts{
	MDIO_SH=	15,		
	MDC_SH=		14,		
	MDCEN_SH=	13,
	RWGMI_SH=	12,	
	PHY_ADD_SH=	7,	//7-11
	REG_ADD_SH=	2,  //6-2
	GMIRY_SH=	1,
	AMAM_SH=   0
};

enum GMIIManagementAccessMasks{
	PHY_ADD_MSK =0x1f,
	REG_ADD_MSK =0x1f,
	GMII_MNGM_ACC_MSK =0x1
	};

//--------------------
/*
GMII Management Data Register
Mnemonic: GMII_MNG_DAT
Address: 4CH - 4DH (only in port 0)

In read mode, this register will get the value of the desired GMII management register. In write
mode, the desired management register will get the value of this register.
Note: When CPU bus is in 8 bit mode, write access to this register must be done to both addresses,
starting at the low address.

Access Rules Register access During GMII automatic access (GMIRY = 0) R, otherwise R/W.
Value after reset 0003H
Bit Name	Bit #	Bit Description
GMII_DT		15:0	GMII data.
					Bit 15 is the most significant bit, corresponding to bit 15 of the accessed register. Bit 0 is
					the least significant bit.
Access Rules Register
access During GMII automatic access (GMII_MNG_ACC<GMIRY> = 0) R, otherwise R/W
Value after reset 0000H

*/

public:

	inline unsigned short int GetGMIIManagementData()const { return GMIIManagementDataF();};
	inline void SetGMIIManagementData(unsigned short int val){ GMIIManagementDataF(val);};


private:

	inline unsigned int GMIIManagementDataF()const {return _ControlRegs.GetGMIIManagementData();};
	inline void GMIIManagementDataF(unsigned short int val){_ControlRegs.SetGMIIManagementData(val);};
    

GigMacControlRegs & _ControlRegs;
};

#endif //GIG_MAC_GMII_REGS 


