/* GigNetStat.h
 * 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                 
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: GigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: GigNetStat.h $
 * 
 * *****************  Version 8  *****************
 * User: Skornfel     Date: 10/22/99   Time: 5:47p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Larger output for byte counters and Netstat access fix
 * 
 * 
 * *****************  Version 7  *****************
 * User: Skornfel     Date: 9/22/99    Time: 7:51p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Byte Counters aligned correctly
 * 
 * *****************  Version 6  *****************
 * User: Tclancyx     Date: 8/20/99    Time: 7:02p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Change reference to iostream::operator<<(void *) to (unsigned int)
 * since vxWorks doesn't seem to be able to print a pointer to void
 * 
 * *****************  Version 5  *****************
 * User: Skornfel     Date: 8/04/99    Time: 4:53p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Corrected the types for the fuctions to work with the regs
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:49p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 * 
 * ---------------------------------------------------------------------
 */

#ifndef GIG_MAC_NET_STATS
#define GIG_MAC_NET_STATS

/*
The network statistic counters access rules are as follows:

- The counters can be accessed with a base address equal to 100H or 200H. Accessing the
  counters with the 100H base address causes them to reset. Accessing them with the 200H will
  not reset the counter.
- Any of the counter bytes can be read without reading the other bytes. Only the accessed bytes
  are reset (if accessed with the 100H base address).
- When the CPU data bus is in 16 bit mode, access to network statistic counters should be done
  only with even numbered addresses.
- The counters must be read from the lower to the upper bytes, in consecutive accesses.
- The statistic counters are not writable.

Note: When a counter reaches its highest possible value, it will continue to count from zero,
      and the corresponding counter overflow status bit will be set.
Note: Receive statistic counters are updated even if the receive FIFO overflows.
Note: All the byte counters take CRC bytes into account, but exclude the framing bits in the 
      packets.


*/
class GigMacNetStats{
public:
//friend
//static void * operator new(size_t size, void * ptr , int dma){return ptr; }
static void * operator new(size_t size, void * ptr){return ptr; }

//friend
static void operator delete( void * ptr){ return; }




	GigMacNetStats(){
		//cout << "GigMacNetStats CTOR" << endl;
	};

	~GigMacNetStats(){};


/*

	inline unsigned GetFramesTransmittedOK()	{return FramesTransmittedOK;};
	inline unsigned GetSingleCollisionFrames()	{return SingleCollisionFrames;};
	inline unsigned GetMultipleCollisionFrames(){return MultipleCollisionFrames;};
	inline unsigned GetFramesReceivedOK()		{return FramesReceivedOK;};
	inline unsigned GetFramesCheckSequenceErrors(){return FramesCheckSequenceErrors;};
	inline unsigned GetAlignmentErrors()		{return AlignmentErrors;};
	inline unsigned GetOctetsTransmittedOK()	{return OctetsTransmittedOK;};
	inline unsigned GetFramesWithDeferredXmissions(){return FramesWithDeferredXmissions;};
	inline unsigned GetLateCollisions()			{return LateCollisions;};
	inline unsigned GetFramesAbortedDueToXSColls(){return FramesAbortedDueToXSColls;};
	inline unsigned GetOctetsReceivedOK()		{return OctetsReceivedOK;};
	inline unsigned GetFrameTooLongErrors()		{return FrameTooLongErrors;};
	inline unsigned GetMulticastFramesXmittedOK(){return MulticastFramesXmittedOK;};
	inline unsigned GetBroadcastFramesReceivedOK(){return BroadcastFramesReceivedOK;};
	inline unsigned GetMulticastFramesReceivedOK(){return MulticastFramesReceivedOK;};
	inline unsigned GetBroadcastFramesXmittedOK(){return BroadcastFramesXmittedOK;};
	inline unsigned GetPauseFramesTransmitted()	{return PauseFramesTransmitted;};
	inline unsigned GetPauseFramesReceived()	{return PauseFramesReceived;};

*/

	
	//this function for GigMacNetStats on the offse 0x100 only plese be very carful!!!
	inline void CopyGigMacNetStatsAndClear(GigMacNetStats &_CopyOfNetSt){
	// this is not the best way to code it you can run loop to do .. - i just like to see ..._CopyOfNetSt.

	_CopyOfNetSt.TxUniOkCnt=TxUniOkCnt; 
	_CopyOfNetSt.TxMltOkCnt=TxMltOkCnt;	
	_CopyOfNetSt.TxBrdOkCnt=TxBrdOkCnt;	
	_CopyOfNetSt.TxPkt64Cnt=TxPkt64Cnt;	
	_CopyOfNetSt.TxPkt65Cnt=TxPkt65Cnt;	
	_CopyOfNetSt.TxPkt128Cnt=TxPkt128Cnt;	
	_CopyOfNetSt.TxPkt256Cnt=TxPkt256Cnt;	
	_CopyOfNetSt.TxPkt512Cnt=TxPkt512Cnt;	
	_CopyOfNetSt.TxPkt1024Cnt=TxPkt1024Cnt;	
	_CopyOfNetSt.TxPkt1519Cnt=TxPkt1519Cnt;	
	_CopyOfNetSt.TxPauseCnt=TxPauseCnt;	
	_CopyOfNetSt.TxErrCnt=TxErrCnt;	
	//------
	_CopyOfNetSt.TxUniOkCnt1=TxUniOkCnt1; 
	_CopyOfNetSt.TxMltOkCnt1=TxMltOkCnt1;	
	_CopyOfNetSt.TxBrdOkCnt1=TxBrdOkCnt1;	
	_CopyOfNetSt.TxPkt64Cnt1=TxPkt64Cnt1;	
	_CopyOfNetSt.TxPkt65Cnt1=TxPkt65Cnt1;	
	_CopyOfNetSt.TxPkt128Cnt1=TxPkt128Cnt1;	
	_CopyOfNetSt.TxPkt256Cnt1=TxPkt256Cnt1;	
	_CopyOfNetSt.TxPkt512Cnt1=TxPkt512Cnt1;	
	_CopyOfNetSt.TxPkt1024Cnt1=TxPkt1024Cnt1;	
	_CopyOfNetSt.TxPkt1519Cnt1=TxPkt1519Cnt1;	
	_CopyOfNetSt.TxPauseCnt1=TxPauseCnt1;	
	_CopyOfNetSt.TxErrCnt1=TxErrCnt1;	
	//-------
	_CopyOfNetSt.TxUniOkCnt2=TxUniOkCnt2; 
	_CopyOfNetSt.TxMltOkCnt2=TxMltOkCnt2;	
	_CopyOfNetSt.TxBrdOkCnt2=TxBrdOkCnt2;	
	_CopyOfNetSt.TxPkt64Cnt2=TxPkt64Cnt2;	
	_CopyOfNetSt.TxPkt65Cnt2=TxPkt65Cnt2;	
	_CopyOfNetSt.TxPkt128Cnt2=TxPkt128Cnt2;	
	_CopyOfNetSt.TxPkt256Cnt2=TxPkt256Cnt2;	
	_CopyOfNetSt.TxPkt512Cnt2=TxPkt512Cnt2;	
	_CopyOfNetSt.TxPkt1024Cnt2=TxPkt1024Cnt2;	
	_CopyOfNetSt.TxPkt1519Cnt2=TxPkt1519Cnt2;	
	_CopyOfNetSt.TxPauseCnt2=TxPauseCnt2;	
	_CopyOfNetSt.TxErrCnt2=TxErrCnt2;	
	//-------
	_CopyOfNetSt.TxUniOkCnt3=TxUniOkCnt3; 
	_CopyOfNetSt.TxMltOkCnt3=TxMltOkCnt3;	
	_CopyOfNetSt.TxBrdOkCnt3=TxBrdOkCnt3;	
	_CopyOfNetSt.TxPkt64Cnt3=TxPkt64Cnt3;	
	_CopyOfNetSt.TxPkt65Cnt3=TxPkt65Cnt3;	
	_CopyOfNetSt.TxPkt128Cnt3=TxPkt128Cnt3;	
	_CopyOfNetSt.TxPkt256Cnt3=TxPkt256Cnt3;	
	_CopyOfNetSt.TxPkt512Cnt3=TxPkt512Cnt3;	
	_CopyOfNetSt.TxPkt1024Cnt3=TxPkt1024Cnt3;	
	_CopyOfNetSt.TxPkt1519Cnt3=TxPkt1519Cnt3;	
	_CopyOfNetSt.TxPauseCnt3=TxPauseCnt3;	
	_CopyOfNetSt.TxErrCnt3=TxErrCnt3;	

		
	_CopyOfNetSt.TxOctOkCnt=TxOctOkCnt;	
	_CopyOfNetSt.TxOctBadCnt=TxOctBadCnt;
	_CopyOfNetSt.RxOctOkCnt=RxOctOkCnt;	
	_CopyOfNetSt.RxOctBadCnt=RxOctBadCnt;	
	//-------
	_CopyOfNetSt.TxOctOkCnt1=TxOctOkCnt1;	
	_CopyOfNetSt.TxOctBadCnt1=TxOctBadCnt1;
	_CopyOfNetSt.RxOctOkCnt1=RxOctOkCnt1;	
	_CopyOfNetSt.RxOctBadCnt1=RxOctBadCnt1;	
	//-------
	_CopyOfNetSt.TxOctOkCnt2=TxOctOkCnt2;	
	_CopyOfNetSt.TxOctBadCnt2=TxOctBadCnt2;
	_CopyOfNetSt.RxOctOkCnt2=RxOctOkCnt2;	
	_CopyOfNetSt.RxOctBadCnt2=RxOctBadCnt2;	
	//-------
	_CopyOfNetSt.TxOctOkCnt3=TxOctOkCnt3;	
	_CopyOfNetSt.TxOctBadCnt3=TxOctBadCnt3;
	_CopyOfNetSt.RxOctOkCnt3=RxOctOkCnt3;	
	_CopyOfNetSt.RxOctBadCnt3=RxOctBadCnt3;	


	_CopyOfNetSt.RxOvfCnt=RxOvfCnt;		
	_CopyOfNetSt.RxShortOkCnt=RxShortOkCnt;	
	_CopyOfNetSt.RxShortCrcCnt=RxShortCrcCnt;	
	_CopyOfNetSt.RxUniOkCnt=RxUniOkCnt;	
	_CopyOfNetSt.RxMltOkCnt=RxMltOkCnt;	
	_CopyOfNetSt.RxBrdOkCnt=RxBrdOkCnt;	
	_CopyOfNetSt.RxNormCrcCnt=RxNormCrcCnt;	
	_CopyOfNetSt.RxLongOkCnt=RxLongOkCnt;	
	_CopyOfNetSt.RxLongCrcCnt=RxLongCrcCnt;	
	_CopyOfNetSt.RxPkt64Cnt=RxPkt64Cnt;	
	_CopyOfNetSt.RxPkt65Cnt=RxPkt65Cnt;	
	_CopyOfNetSt.RxPkt128Cnt=RxPkt128Cnt;	
	_CopyOfNetSt.RxPkt256Cnt=RxPkt256Cnt;	
	_CopyOfNetSt.RxPkt512Cnt=RxPkt512Cnt;	
	_CopyOfNetSt.RxPkt1024Cnt=RxPkt1024Cnt;	
	_CopyOfNetSt.RxPkt1519Cnt=RxPkt1519Cnt;	
	_CopyOfNetSt.RxPauseCnt=RxPauseCnt;	
	_CopyOfNetSt.RxFalsCrsCnt=RxFalsCrsCnt;	
	_CopyOfNetSt.RxSymErrCnt=RxSymErrCnt;
	//--------
	_CopyOfNetSt.RxOvfCnt1=RxOvfCnt1;		
	_CopyOfNetSt.RxShortOkCnt1=RxShortOkCnt1;	
	_CopyOfNetSt.RxShortCrcCnt1=RxShortCrcCnt1;	
	_CopyOfNetSt.RxUniOkCnt1=RxUniOkCnt1;	
	_CopyOfNetSt.RxMltOkCnt1=RxMltOkCnt1;	
	_CopyOfNetSt.RxBrdOkCnt1=RxBrdOkCnt1;	
	_CopyOfNetSt.RxNormCrcCnt1=RxNormCrcCnt1;	
	_CopyOfNetSt.RxLongOkCnt1=RxLongOkCnt1;	
	_CopyOfNetSt.RxLongCrcCnt1=RxLongCrcCnt1;	
	_CopyOfNetSt.RxPkt64Cnt1=RxPkt64Cnt1;	
	_CopyOfNetSt.RxPkt65Cnt1=RxPkt65Cnt1;	
	_CopyOfNetSt.RxPkt128Cnt1=RxPkt128Cnt1;	
	_CopyOfNetSt.RxPkt256Cnt1=RxPkt256Cnt1;	
	_CopyOfNetSt.RxPkt512Cnt1=RxPkt512Cnt1;	
	_CopyOfNetSt.RxPkt1024Cnt1=RxPkt1024Cnt1;	
	_CopyOfNetSt.RxPkt1519Cnt1=RxPkt1519Cnt1;	
	_CopyOfNetSt.RxPauseCnt1=RxPauseCnt1;	
	_CopyOfNetSt.RxFalsCrsCnt1=RxFalsCrsCnt1;	
	_CopyOfNetSt.RxSymErrCnt1=RxSymErrCnt1;
	//--------
	_CopyOfNetSt.RxOvfCnt2=RxOvfCnt2;		
	_CopyOfNetSt.RxShortOkCnt2=RxShortOkCnt2;	
	_CopyOfNetSt.RxShortCrcCnt2=RxShortCrcCnt2;	
	_CopyOfNetSt.RxUniOkCnt2=RxUniOkCnt2;	
	_CopyOfNetSt.RxMltOkCnt2=RxMltOkCnt2;	
	_CopyOfNetSt.RxBrdOkCnt2=RxBrdOkCnt2;	
	_CopyOfNetSt.RxNormCrcCnt2=RxNormCrcCnt2;	
	_CopyOfNetSt.RxLongOkCnt2=RxLongOkCnt2;	
	_CopyOfNetSt.RxLongCrcCnt2=RxLongCrcCnt2;	
	_CopyOfNetSt.RxPkt64Cnt2=RxPkt64Cnt2;	
	_CopyOfNetSt.RxPkt65Cnt2=RxPkt65Cnt2;	
	_CopyOfNetSt.RxPkt128Cnt2=RxPkt128Cnt2;	
	_CopyOfNetSt.RxPkt256Cnt2=RxPkt256Cnt2;	
	_CopyOfNetSt.RxPkt512Cnt2=RxPkt512Cnt2;	
	_CopyOfNetSt.RxPkt1024Cnt2=RxPkt1024Cnt2;	
	_CopyOfNetSt.RxPkt1519Cnt2=RxPkt1519Cnt2;	
	_CopyOfNetSt.RxPauseCnt2=RxPauseCnt2;	
	_CopyOfNetSt.RxFalsCrsCnt2=RxFalsCrsCnt2;	
	_CopyOfNetSt.RxSymErrCnt2=RxSymErrCnt2;
	//--------
	_CopyOfNetSt.RxOvfCnt3=RxOvfCnt3;		
	_CopyOfNetSt.RxShortOkCnt3=RxShortOkCnt3;	
	_CopyOfNetSt.RxShortCrcCnt3=RxShortCrcCnt3;	
	_CopyOfNetSt.RxUniOkCnt3=RxUniOkCnt3;	
	_CopyOfNetSt.RxMltOkCnt3=RxMltOkCnt3;	
	_CopyOfNetSt.RxBrdOkCnt3=RxBrdOkCnt3;	
	_CopyOfNetSt.RxNormCrcCnt3=RxNormCrcCnt3;	
	_CopyOfNetSt.RxLongOkCnt3=RxLongOkCnt3;	
	_CopyOfNetSt.RxLongCrcCnt3=RxLongCrcCnt3;	
	_CopyOfNetSt.RxPkt64Cnt3=RxPkt64Cnt3;	
	_CopyOfNetSt.RxPkt65Cnt3=RxPkt65Cnt3;	
	_CopyOfNetSt.RxPkt128Cnt3=RxPkt128Cnt3;	
	_CopyOfNetSt.RxPkt256Cnt3=RxPkt256Cnt3;	
	_CopyOfNetSt.RxPkt512Cnt3=RxPkt512Cnt3;	
	_CopyOfNetSt.RxPkt1024Cnt3=RxPkt1024Cnt3;	
	_CopyOfNetSt.RxPkt1519Cnt3=RxPkt1519Cnt3;	
	_CopyOfNetSt.RxPauseCnt3=RxPauseCnt3;	
	_CopyOfNetSt.RxFalsCrsCnt3=RxFalsCrsCnt3;	
	_CopyOfNetSt.RxSymErrCnt3=RxSymErrCnt3;
//--------sorry too long :)
	};



//	void SetTxUniOkCnt(unsigned char val)	{TxUniOkCnt=val;};	//00H  03H
//Transmit Statistic Counters
inline	unsigned int GetTxUniOkCnt()	{int temp[4],ret; 
					 temp[0] = TxUniOkCnt;
					 temp[1] = TxUniOkCnt1; temp[1] = temp[1]  <<8;
                                         temp[2] = TxUniOkCnt2; temp[2] = temp[2] << 16;
                                         temp[3] = TxUniOkCnt3; temp[3] = temp[3] << 24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};	//00H  03H

inline	unsigned int GetTxMltOkCnt()	{int temp[4],ret;
					 temp[0] = TxMltOkCnt;
					 temp[1] = TxMltOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxMltOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxMltOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};	

inline	unsigned int GetTxBrdOkCnt()	{int temp[4],ret;
					 temp[0] = TxBrdOkCnt;
					 temp[1] = TxBrdOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxBrdOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxBrdOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPkt64Cnt()	{int temp[4],ret;
					 temp[0] = TxPkt64Cnt;
					 temp[1] = TxPkt64Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPkt64Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPkt64Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPkt65Cnt()	{int temp[4],ret;
					 temp[0] = TxPkt65Cnt;
					 temp[1] = TxPkt65Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPkt65Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPkt65Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPkt128Cnt()	{int temp[4],ret;
					 temp[0] = TxPkt128Cnt;
					 temp[1] = TxPkt128Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPkt128Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPkt128Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPkt256Cnt()	{int temp[4],ret;
					 temp[0] = TxPkt256Cnt;
					 temp[1] = TxPkt256Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPkt256Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPkt256Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPkt512Cnt()	{int temp[4],ret;
					 temp[0] = TxPkt512Cnt;
					 temp[1] = TxPkt512Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPkt512Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPkt512Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPkt1024Cnt()	{int temp[4],ret;
					 temp[0] = TxPkt1024Cnt;
					 temp[1] = TxPkt1024Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPkt1024Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPkt1024Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPkt1519Cnt()	{int temp[4],ret;
					 temp[0] = TxPkt1519Cnt;
					 temp[1] = TxPkt1519Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPkt1519Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPkt1519Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxPauseCnt()	{int temp[4],ret;
					 temp[0] = TxPauseCnt;
					 temp[1] = TxPauseCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxPauseCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxPauseCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetTxErrCnt()	{int temp[4],ret;
					 temp[0] = TxErrCnt;
					 temp[1] = TxErrCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxErrCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxErrCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

//////
inline	unsigned int GetTxOctOkCnt()	{int temp[4],ret;
					 temp[0] = TxOctOkCnt;
					 temp[1] = TxOctOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxOctOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxOctOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetTxOctOkCntHi()	{int temp[2],ret;
					 temp[0] = TxOctOkCntHi;
					 temp[1] = TxOctOkCntHi1; temp[1] = temp[1]<<8;
					 ret = temp[0] + temp[1];
					 return ret;};

inline	unsigned int GetTxOctBadCnt()	{int temp[4],ret;
					 temp[0] = TxOctBadCnt;
					 temp[1] = TxOctBadCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = TxOctBadCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = TxOctBadCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetTxOctBadCntHi()	{int temp[2],ret;
					 temp[0] = TxOctBadCntHi;
					 temp[1] = TxOctBadCntHi1; temp[1] = temp[1]<<8;
					 ret = temp[0] + temp[1];
					 return ret;};
//////

inline	unsigned int GetRxOctOkCnt()	{int temp[4],ret;
					 temp[0] = RxOctOkCnt;
					 temp[1] = RxOctOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxOctOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxOctOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxOctOkCntHi()	{int temp[2],ret;
					 temp[0] = RxOctOkCntHi;
					 temp[1] = RxOctOkCntHi1; temp[1] = temp[1]<<8;
					 ret = temp[0] + temp[1];
					 return ret;};

inline	unsigned int GetRxOctBadCnt()	{int temp[4],ret;
					 temp[0] = RxOctBadCnt;
					 temp[1] = RxOctBadCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxOctBadCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxOctBadCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxOctBadCntHi()	{int temp[2],ret;
					 temp[0] = RxOctBadCntHi;
					 temp[1] = RxOctBadCntHi1; temp[1] = temp[1]<<8;
 					 ret = temp[0] + temp[1];
					 return ret;};



///////

inline	unsigned int GetRxOvfCnt()	{int temp[4],ret;
					 temp[0] = RxOvfCnt;
					 temp[1] = RxOvfCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxOvfCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxOvfCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxShortOkCnt()	{int temp[4],ret;
					 temp[0] = RxShortOkCnt;
					 temp[1] = RxShortOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxShortOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxShortOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxShortCrcCnt()	{int temp[4],ret;
					 temp[0] = RxShortCrcCnt;
					 temp[1] = RxShortCrcCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxShortCrcCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxShortCrcCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxUniOkCnt()	{int temp[4],ret;
					 temp[0] = RxUniOkCnt;
					 temp[1] = RxUniOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxUniOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxUniOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxMltOkCnt()	{int temp[4],ret;
					 temp[0] = RxMltOkCnt;
					 temp[1] = RxMltOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxMltOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxMltOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxBrdOkCnt()	{int temp[4],ret;
					 temp[0] = RxBrdOkCnt;
					 temp[1] = RxBrdOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxBrdOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxBrdOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxNormCrcCnt()	{int temp[4],ret;
					 temp[0] = RxNormCrcCnt;
					 temp[1] = RxNormCrcCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxNormCrcCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxNormCrcCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxLongOkCnt()	{int temp[4],ret;
					 temp[0] = RxLongOkCnt;
					 temp[1] = RxLongOkCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxLongOkCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxLongOkCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};
inline	unsigned int GetRxLongCrcCnt()	{int temp[4],ret;
					 temp[0] = RxLongCrcCnt;
					 temp[1] = RxLongCrcCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxLongCrcCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxLongCrcCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

//..
inline	unsigned int GetRxPkt64Cnt()	{int temp[4],ret;
					 temp[0] = RxPkt64Cnt;
					 temp[1] = RxPkt64Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPkt64Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPkt64Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxPkt65Cnt()	{int temp[4],ret;
					 temp[0] = RxPkt65Cnt;
					 temp[1] = RxPkt65Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPkt65Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPkt65Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxPkt128Cnt()	{int temp[4],ret;
					 temp[0] = RxPkt128Cnt;
					 temp[1] = RxPkt128Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPkt128Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPkt128Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxPkt256Cnt()	{int temp[4],ret;
					 temp[0] = RxPkt256Cnt;
					 temp[1] = RxPkt256Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPkt256Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPkt256Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxPkt512Cnt()	{int temp[4],ret;
					 temp[0] = RxPkt512Cnt;
					 temp[1] = RxPkt512Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPkt512Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPkt512Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxPkt1024Cnt()	{int temp[4],ret;
					 temp[0] = RxPkt1024Cnt;
					 temp[1] = RxPkt1024Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPkt1024Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPkt1024Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxPkt1519Cnt()	{int temp[4],ret;
					 temp[0] = RxPkt1519Cnt;
					 temp[1] = RxPkt1519Cnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPkt1519Cnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPkt1519Cnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxPauseCnt()	{int temp[4],ret;
					 temp[0] = RxPauseCnt;
					 temp[1] = RxPauseCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxPauseCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxPauseCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxFalsCrsCnt()	{int temp[4],ret;
					 temp[0] = RxFalsCrsCnt;
					 temp[1] = RxFalsCrsCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxFalsCrsCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxFalsCrsCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

inline	unsigned int GetRxSymErrCnt()	{int temp[4],ret;
					 temp[0] = RxSymErrCnt;
					 temp[1] = RxSymErrCnt1; temp[1] = temp[1]<<8;
                                         temp[2] = RxSymErrCnt2; temp[2] = temp[2]<<16;
                                         temp[3] = RxSymErrCnt3; temp[3] = temp[3]<<24; 
					 ret = temp[0] + temp[1] + temp[2] + temp[3];
					 return ret;};

//inline	unsigned int GetTxUniOkCnt()	{return TxUniOkCnt+(TxUniOkCnt1<<8)+(TxUniOkCnt2<<16)+(TxUniOkCnt3<<24);};	//00H  03H
//inline	unsigned int GetTxMltOkCnt()	{int temp; temp = ((((TxMltOkCnt)+(TxMltOkCnt1<<8))+(TxMltOkCnt2<<16))+(TxMltOkCnt3<<24)); return temp;};	//04H  07H
//inline	unsigned int GetTxBrdOkCnt()	{int temp; temp = ((((TxBrdOkCnt)+(TxBrdOkCnt1<<8))+(TxBrdOkCnt2<<16))+(TxBrdOkCnt3<<24)); return temp;};	//08H  0BH
//inline	unsigned int GetTxPkt64Cnt()	{return TxPkt64Cnt+(TxPkt64Cnt1<<8)+(TxPkt64Cnt2<<16)+(TxPkt64Cnt3<<24);};	//24H  27H
//inline	unsigned int GetTxPkt65Cnt()	{return TxPkt65Cnt+(TxPkt65Cnt1<<8)+(TxPkt65Cnt2<<16)+(TxPkt65Cnt3<<24);};	//28H  2BH
//inline	unsigned int GetTxPkt128Cnt()	{return TxPkt128Cnt+(TxPkt128Cnt1<<8)+(TxPkt128Cnt2<<16)+(TxPkt128Cnt3<<24);};	//2CH  2FH
//inline	unsigned int GetTxPkt256Cnt()	{return TxPkt256Cnt+(TxPkt256Cnt1<<8)+(TxPkt256Cnt2<<16)+(TxPkt256Cnt3<<24);};	//30H  33H
//inline	unsigned int GetTxPkt512Cnt()	{return TxPkt512Cnt+(TxPkt512Cnt1<<8)+(TxPkt512Cnt2<<16)+(TxPkt512Cnt3<<24);};	//34H  37H
//inline	unsigned int GetTxPkt1024Cnt()	{return TxPkt1024Cnt+(TxPkt1024Cnt1<<8)+(TxPkt1024Cnt2<<16)+(TxPkt1024Cnt3<<24);};	//38H  3BH
//inline	unsigned int GetTxPkt1519Cnt()	{return TxPkt1519Cnt+(TxPkt1519Cnt1<<8)+(TxPkt1519Cnt2<<16)+(TxPkt1519Cnt3<<24);};	//3CH  3FH
//inline	unsigned int GetTxPauseCnt()	{return TxPauseCnt+(TxPauseCnt1<<8)+(TxPauseCnt2<<16)+(TxPauseCnt3<<24);};	//40H  43H
//inline	unsigned int GetTxErrCnt()		{return TxErrCnt+(TxErrCnt1<<8)+(TxErrCnt2<<16)+(TxErrCnt3<<24);};		//44H  47H

//Transmit Byte Counters
		
//inline	unsigned int GetTxOctOkCnt()	{return TxOctOkCnt+(TxOctOkCnt1<<8)+(TxOctOkCnt2<<16)+(TxOctOkCnt3<<24);};	//60H  64H
//inline	unsigned int GetTxOctBadCnt()	{return TxOctBadCnt+(TxOctBadCnt1<<8)+(TxOctBadCnt2<<16)+(TxOctBadCnt3<<24);};	//68H  6CH
////inline	unsigned int GetTxOctOkCntHi()	{return TxOctOkCntHi+(TxOctOkCntHi1<<8);};
////inline	unsigned int GetTxOctBadCntHi()	{return TxOctBadCntHi+(TxOctBadCntHi1<<8);};

//Receive Byte Counters	

//inline	unsigned int GetRxOctOkCnt()	{return RxOctOkCnt+(RxOctOkCnt1<<8)+(RxOctOkCnt2<<16)+(RxOctOkCnt3<<24);};				//70H  74H
//inline	unsigned int GetRxOctBadCnt()	{return RxOctBadCnt+(RxOctBadCnt1<<8)+(RxOctBadCnt2<<16)+(RxOctBadCnt3<<24);};			//78H  7CH
//inline	unsigned int GetRxOctOkCntHi()	{return RxOctOkCntHi+(RxOctOkCntHi1<<8);};				
//inline	unsigned int GetRxOctBadCntHi()	{return RxOctBadCntHi+(RxOctBadCntHi1<<8);};			

//Receive Statistic Counters


//inline	unsigned int GetRxOvfCnt()	{return RxOvfCnt+(RxOvfCnt1<<8)+(RxOvfCnt2<<16)+(RxOvfCnt3<<24);};						//84H  87H
//inline	unsigned int GetRxShortOkCnt()	{return RxShortOkCnt+(RxShortOkCnt1<<8)+(RxShortOkCnt2<<16)+(RxShortOkCnt3<<24);};		//88H  8BH
//inline	unsigned int GetRxShortCrcCnt()	{return RxShortCrcCnt+(RxShortCrcCnt1<<8)+(RxShortCrcCnt2<<16)+(RxShortCrcCnt3<<24);};	//8CH  8FH
//inline	unsigned int GetRxUniOkCnt()	{return RxUniOkCnt+(RxUniOkCnt1<<8)+(RxUniOkCnt2<<16)+(RxUniOkCnt3<<24);};				//90H  93H
//inline	unsigned int GetRxMltOkCnt()	{return RxMltOkCnt+(RxMltOkCnt1<<8)+(RxMltOkCnt2<<16)+(RxMltOkCnt3<<24);};				//94H  97H
//inline	unsigned int GetRxBrdOkCnt()	{return RxBrdOkCnt+(RxBrdOkCnt1<<8)+(RxBrdOkCnt2<<16)+(RxBrdOkCnt3<<24);};				//98H  9BH
//inline	unsigned int GetRxNormCrcCnt()	{return RxNormCrcCnt+(RxNormCrcCnt1<<8)+(RxNormCrcCnt2<<16)+(RxNormCrcCnt3<<24);};		//9CH  9FH
//inline	unsigned int GetRxLongOkCnt()	{return RxLongOkCnt+(RxLongOkCnt1<<8)+(RxLongOkCnt2<<16)+(RxLongOkCnt3<<24);};			//A4H  A7H
//inline	unsigned int GetRxLongCrcCnt()	{return RxLongCrcCnt+(RxLongCrcCnt1<<8)+(RxLongCrcCnt2<<16)+(RxLongCrcCnt3<<24);};		//A8H  ABH
//inline	unsigned int GetRxPkt64Cnt()	{return RxPkt64Cnt+(RxPkt64Cnt1<<8)+(RxPkt64Cnt2<<16)+(RxPkt64Cnt3<<24);};				//ACH  AFH
//inline	unsigned int GetRxPkt65Cnt()	{return RxPkt65Cnt+(RxPkt65Cnt1<<8)+(RxPkt65Cnt2<<16)+(RxPkt65Cnt3<<24);};				//B0H  B3H
//inline	unsigned int GetRxPkt128Cnt()	{return RxPkt128Cnt+(RxPkt128Cnt1<<8)+(RxPkt128Cnt2<<16)+(RxPkt128Cnt3<<24);};			//B4H  B7H
//inline	unsigned int GetRxPkt256Cnt()	{return RxPkt256Cnt+(RxPkt256Cnt1<<8)+(RxPkt256Cnt2<<16)+(RxPkt256Cnt3<<24);};			//B8H  BBH
//inline	unsigned int GetRxPkt512Cnt()	{return RxPkt512Cnt+(RxPkt512Cnt1<<8)+(RxPkt512Cnt2<<16)+(RxPkt512Cnt3<<24);};			//BCH  BFH
//inline	unsigned int GetRxPkt1024Cnt()	{return RxPkt1024Cnt+(RxPkt1024Cnt1<<8)+(RxPkt1024Cnt2<<16)+(RxPkt1024Cnt3<<24);};		//C0H  C3H
//inline	unsigned int GetRxPkt1519Cnt()	{return RxPkt1519Cnt+(RxPkt1519Cnt1<<8)+(RxPkt1519Cnt2<<16)+(RxPkt1519Cnt3<<24);};		//C4H  C7H
//inline	unsigned int GetRxPauseCnt()	{return RxPauseCnt+(RxPauseCnt1<<8)+(RxPauseCnt2<<16)+(RxPauseCnt3<<24);};				//C8H  CBH
//inline	unsigned int GetRxFalsCrsCnt()	{return RxFalsCrsCnt+(RxFalsCrsCnt1<<8)+(RxFalsCrsCnt2<<16)+(RxFalsCrsCnt3<<24);};		//CCH  CFH
//inline	unsigned int GetRxSymErrCnt()	{return RxSymErrCnt+(RxSymErrCnt1<<8)+(RxSymErrCnt2<<16)+(RxSymErrCnt3<<24);};			//D0H  D3H




	inline void Show(){

cout
<< "!!!##################" << (unsigned int)(&TxUniOkCnt) << "####################" << endl
<< "TxUniOkCnt       " << GetTxUniOkCnt()   << endl
<< "TxMltOkCnt       " << GetTxMltOkCnt()   << "            RxOvfCnt      " << GetRxOvfCnt()<< endl		
<< "TxBrdOkCnt       " << GetTxBrdOkCnt()   << "            RxShortOkCnt  " << GetRxShortOkCnt()<< endl
<< "===                                 RxShortCrcCnt " << GetRxShortCrcCnt()<< endl	
<< "===                                 RxUniOkCnt    " << GetRxUniOkCnt()<< endl	
<< "===                                 RxMltOkCnt    " << GetRxMltOkCnt()<< endl			
<< "===                                 RxBrdOkCnt    " << GetRxBrdOkCnt()<< endl		
<< "===                                 RxNormCrcCnt  " << GetRxNormCrcCnt()<< endl	
<< "TxPkt64Cnt       " << GetTxPkt64Cnt()   << "			RxLongOkCnt   " << GetRxLongOkCnt()<< endl	
<< "TxPkt65Cnt       " << GetTxPkt65Cnt()   << "			RxLongCrcCnt  " << GetRxLongCrcCnt()<< endl
<< "TxPkt128Cnt      " << GetTxPkt128Cnt()  << "			RxPkt64Cnt    " << GetRxPkt64Cnt() << endl	
<< "TxPkt256Cnt      " << GetTxPkt256Cnt()  << "			RxPkt65Cnt    " << GetRxPkt65Cnt()<< endl//" @" << (unsigned int)(&RxPkt65Cnt)<<endl	
<< "TxPkt512Cnt      " << GetTxPkt512Cnt()  << "			RxPkt128Cnt   " << GetRxPkt128Cnt()<< endl		
<< "TxPkt1024Cnt     " << GetTxPkt1024Cnt() << "			RxPkt256Cnt   " << GetRxPkt256Cnt()<< endl
<< "TxPkt1519Cnt     " << GetTxPkt1519Cnt() << "			RxPkt512Cnt   " << GetRxPkt512Cnt()<< endl 	
<< "TxPauseCnt       " << GetTxPauseCnt()   << "			RxPkt1024Cnt  " << GetRxPkt1024Cnt()<< endl		
<< "TxErrCnt         " << GetTxErrCnt()     << "			RxPkt1519Cnt  " << GetRxPkt1519Cnt()<< endl		
<< "TxOctOkCnt(byte) " << GetTxOctOkCntHi()<<GetTxOctOkCnt()   << "			RxPauseCnt    " << GetRxPauseCnt()<< endl   
<< "TxOctBadCnt(byte)" << GetTxOctBadCntHi()<<GetTxOctBadCnt() << "			RxFalsCrsCnt  " << GetRxFalsCrsCnt()<< endl    	
<< "RxOctOkCnt(byte) " << GetRxOctOkCntHi()<<GetRxOctOkCnt()   << "			RxSymErrCnt   " << GetRxSymErrCnt() << endl
<< "RxOctBadCnt(byte)" << GetRxOctBadCntHi()<<GetRxOctBadCnt()  << endl;
                             

};

private:

/*
Table 6 lists each network statistic 82547EB register name, mnemonic, and offset.
I/O Base address: 100H or 200H.

--------------Transmit Statistic Counters
	
The number of unicast packets transmitted			TX_UNI_OK_CNT	00H  03H
without any errors.

The number of multicast packets that are not		TX_MLT_OK_CNT	04H  07H
broadcast, transmitted without any errors.

The number of broadcast packets transmitted			TX_BRD_OK_CNT	08H  0BH
without any errors.

The number of transmitted packets, 64 bytes in		TX_PKT_64_CNT	24H  27H
length, including bad packets.

The number of transmitted packets, 65 to 127		TX_PKT_65_CNT	28H  2BH
bytes in length, including bad packets.

The number of transmitted packets, 128 to 255		TX_PKT_128_CNT	2CH  2FH
bytes in length, including bad packets.

The number of transmitted packets, 256 to 511		TX_PKT_256_CNT	30H  33H
bytes in length, including bad packets.

The number of transmitted packets, 512 to 1023		TX_PKT_512_CNT	34H  37H
bytes in length, including bad packets.

The number of transmitted packets, 1024 to			TX_PKT_1024_CNT 38H  3BH

1518 bytes in length, including bad packets.		TX_PKT_1519_CNT 3CH  3FH
The number of transmitted packets with lengths
between 1519 bytes and the maximum packet
size, including bad packets.

The number of correct transmitted flow-control		TX_PAUSE_CNT	40H  43H
packets.	

The number of packets transmitted with an error		TX_ERR_CNT		44H  47H
due to transmit FIFO underflow or txerr signal
assertion.


-----------------Transmit Byte Counters   
		
The number of bytes transmitted in good				TX_OCT_OK_CNT	60H  64H
packets.

The number of bytes transmitted in packets with		TX_OCT_BAD_CNT	68H  6CH
errors.

  
-----------------Receive Byte Counters	

The number of bytes received in good packets.		RX_OCT_OK_CNT	70H  74H
The number of bytes received in packets with		RX_OCT_BAD_CNT	78H  7CH
errors.

----------------Receive Statistic Counters

The number of receive packets not fully				RX_OVF_CNT		84H  87H
accepted due to receive FIFO overflow.


The number of packets, less than 64 bytes in		RX_SHORT_OK_CNT 88H  8BH
length, received without any error.

The number of packets less than 64 bytes in			RX_SHORT_CRC_CNT 8CH  8FH
length, received with CRC error.

The number of unicast packets with lengths			RX_UNI_OK_CNT	90H  93H
between 64 bytes and the maximum packet size,
received without any errors.

The number of multicast packets with lengths		RX_MLT_OK_CN	94H  97H
between 64 bytes and the maximum packet size,
received without any errors.

The number of broadcast packets with lengths		RX_BRD_OK_CNT	98H  9BH
between 64 bytes and the maximum packet size,
received without any errors.

The number of packets with lengths between 64		RX_NORM_CRC_CNT	9CH  9FH
bytes and the maximum packet size, received
with an integral number of bytes and a CRC
error.

The number of packets, larger than the				RX_LONG_OK_CNT	A4H  A7H
maximum packet size, received without any
error.

The number of packets, larger than the				RX_LONG_CRC_CNT	A8H  ABH
maximum packet size, received with a CRC
error.

The number of received packets, 64 bytes in			RX_PKT_64_CNT	ACH  AFH
length, including bad packets.

The number of received packets, 65 to 127 bytes		RX_PKT_65_CNT	B0H  B3H
in length, including bad packets.

The number of received packets, 128 to 255			RX_PKT_128_CNT	B4H  B7H
bytes in length, including bad packets.

The number of received packets, 256 to 511			RX_PKT_256_CNT	B8H  BBH
bytes in length, including bad packets.

The number of received packets, 512 to 1023			RX_PKT_512_CNT	BCH  BFH
bytes in length, including bad packets.

The number of received packets, 1024 to 1518		RX_PKT_1024_CNT C0H  C3H
bytes in length, including bad packets.

The number of received packets, with lengths		RX_PKT_1519_CNT	C4H  C7H
between 1519 bytes and the maximum packet
size (programmable value), including bad
packets.

The number of correct received flow-control			RX_PAUSE_CNT	C8H  CBH
packets.

The number of false carrier events detected.		RX_FALS_CRS_CNT	CCH  CFH

The number of received packets during which			RX_SYM_ERR_CNT	D0H  D3H
PHY symbol errors were detected.
*/


//Transmit Statistic Counters
unsigned int TxUniOkCnt,	TxUniOkCnt1,	TxUniOkCnt2,	TxUniOkCnt3;	//00H  03H
unsigned int TxMltOkCnt,	TxMltOkCnt1,	TxMltOkCnt2,	TxMltOkCnt3;	//04H  07H
unsigned int TxBrdOkCnt,	TxBrdOkCnt1,	TxBrdOkCnt2,	TxBrdOkCnt3;	//08H  0BH
unsigned int pad1[4] ;		//0CH  0FH
unsigned int pad2[4] ;		//10H  13H
unsigned int pad3[4] ;		//14H  17H
unsigned int pad4[4] ;		//18H  1BH
unsigned int pad5[4] ;		//1CH  1FH
unsigned int pad6[4] ;		//20H  23H
unsigned int TxPkt64Cnt,	TxPkt64Cnt1,	TxPkt64Cnt2,	TxPkt64Cnt3;	//24H  27H
unsigned int TxPkt65Cnt,	TxPkt65Cnt1,	TxPkt65Cnt2,	TxPkt65Cnt3;	//28H  2BH
unsigned int TxPkt128Cnt,	TxPkt128Cnt1,	TxPkt128Cnt2,	TxPkt128Cnt3;	//2CH  2FH
unsigned int TxPkt256Cnt,	TxPkt256Cnt1,	TxPkt256Cnt2,	TxPkt256Cnt3;	//30H  33H
unsigned int TxPkt512Cnt,	TxPkt512Cnt1,	TxPkt512Cnt2,	TxPkt512Cnt3;	//34H  37H
unsigned int TxPkt1024Cnt,	TxPkt1024Cnt1,	TxPkt1024Cnt2,	TxPkt1024Cnt3;	//38H  3BH
unsigned int TxPkt1519Cnt,	TxPkt1519Cnt1,	TxPkt1519Cnt2,	TxPkt1519Cnt3;	//3CH  3FH
unsigned int TxPauseCnt,	TxPauseCnt1,	TxPauseCnt2,	TxPauseCnt3;	//40H  43H
unsigned int TxErrCnt,		TxErrCnt1,		TxErrCnt2,		TxErrCnt3;		//44H  47H
//
unsigned int PadIt1[4];
unsigned int PadIt2[4];
unsigned int PadIt3[4];
unsigned int PadIt4[4];
unsigned int PadIt5[4];
unsigned int PadIt6[4];
//unsigned int PadIt7[4];
//unsigned int PadIt8[4];
//unsigned int PadIt9[4];
//unsigned int PadIt10[4];

//Transmit Byte Counters
		
unsigned int TxOctOkCnt,	TxOctOkCnt1,	TxOctOkCnt2,	TxOctOkCnt3;	//60H  64H  (or 65)
unsigned int TxOctOkCntHi,	TxOctOkCntHi1;
unsigned int TempPadIt1[2];
unsigned int TxOctBadCnt,	TxOctBadCnt1,	TxOctBadCnt2,	TxOctBadCnt3;	//68H  6CH   (or 6D)
unsigned int TxOctBadCntHi,	TxOctBadCntHi1;
unsigned int TempPadIt2[2];
//Receive Byte Counters	

unsigned int RxOctOkCnt,	RxOctOkCnt1,	RxOctOkCnt2,	RxOctOkCnt3;	//70H  74H  (or 75)
unsigned int RxOctOkCntHi,	RxOctOkCntHi1;
unsigned int TempPadIt3[2];
unsigned int RxOctBadCnt,	RxOctBadCnt1,	RxOctBadCnt2,	RxOctBadCnt3;	//78H  7CH   (or 7D)
unsigned int RxOctBadCntHi,	RxOctBadCntHi1;
unsigned int TempPadIt4[2];
//Receive Statistic Counters

unsigned int pad7[4] ;		//80H  83H
unsigned int RxOvfCnt,		RxOvfCnt1,		RxOvfCnt2,		RxOvfCnt3;		//84H  87H
unsigned int RxShortOkCnt,	RxShortOkCnt1,	RxShortOkCnt2,	RxShortOkCnt3;	//88H  8BH
unsigned int RxShortCrcCnt,	RxShortCrcCnt1,	RxShortCrcCnt2,	RxShortCrcCnt3;	//8CH  8FH
unsigned int RxUniOkCnt,	RxUniOkCnt1,	RxUniOkCnt2,	RxUniOkCnt3;	//90H  93H
unsigned int RxMltOkCnt,	RxMltOkCnt1,	RxMltOkCnt2,	RxMltOkCnt3;	//94H  97H
unsigned int RxBrdOkCnt,	RxBrdOkCnt1,	RxBrdOkCnt2,	RxBrdOkCnt3;	//98H  9BH
unsigned int RxNormCrcCnt,	RxNormCrcCnt1,	RxNormCrcCnt2,	RxNormCrcCnt3;	//9CH  9FH
unsigned int pad8[4] ;		//A0H  A3H
unsigned int RxLongOkCnt,	RxLongOkCnt1,	RxLongOkCnt2,	RxLongOkCnt3;	//A4H  A7H
unsigned int RxLongCrcCnt,	RxLongCrcCnt1,	RxLongCrcCnt2,	RxLongCrcCnt3;	//A8H  ABH
unsigned int RxPkt64Cnt,	RxPkt64Cnt1,	RxPkt64Cnt2,	RxPkt64Cnt3;	//ACH  AFH
unsigned int RxPkt65Cnt,	RxPkt65Cnt1,	RxPkt65Cnt2,	RxPkt65Cnt3;	//B0H  B3H
unsigned int RxPkt128Cnt,	RxPkt128Cnt1,	RxPkt128Cnt2,	RxPkt128Cnt3;	//B4H  B7H
unsigned int RxPkt256Cnt,	RxPkt256Cnt1,	RxPkt256Cnt2,	RxPkt256Cnt3;	//B8H  BBH
unsigned int RxPkt512Cnt,	RxPkt512Cnt1,	RxPkt512Cnt2,	RxPkt512Cnt3;	//BCH  BFH
unsigned int RxPkt1024Cnt,	RxPkt1024Cnt1,	RxPkt1024Cnt2,	RxPkt1024Cnt3;	//C0H  C3H
unsigned int RxPkt1519Cnt,	RxPkt1519Cnt1,	RxPkt1519Cnt2,	RxPkt1519Cnt3;	//C4H  C7H
unsigned int RxPauseCnt,	RxPauseCnt1,	RxPauseCnt2,	RxPauseCnt3;	//C8H  CBH
unsigned int RxFalsCrsCnt,	RxFalsCrsCnt1,	RxFalsCrsCnt2,	RxFalsCrsCnt3;	//CCH  CFH
unsigned int RxSymErrCnt,	RxSymErrCnt1,	RxSymErrCnt2,	RxSymErrCnt3;	//D0H  D3H

};


#endif //GIG_MAC_NET_STATS 
