/* GigMacPort.h
 * 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                 
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: GigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: GigPort.h $
 * 
 * *****************  Version 17  *****************
 * User: Dmwhite      Date: 12/14/00   Time: 3:53p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * new gig mac multipacket feature
 * 
 * *****************  Version 16  *****************
 * User: Dmwhite      Date: 12/13/00   Time: 2:19p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * fix mltpkt feature
 * 
 * *****************  Version 15  *****************
 * User: Dmwhite      Date: 12/13/00   Time: 11:03a
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * added multipacket feature
 * 
 * *****************  Version 14  *****************
 * User: Dmwhite      Date: 12/11/00   Time: 2:18p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * add multipacket feature
 * 
 * *****************  Version 12  *****************
 * User: Mffallon     Date: 5/30/00    Time: 5:56p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Set MAC's tx threshold to 0xbe.  I am lowering threshold to the size of
 * the largets allowable packet.  Tx_Rdy threshold was just increased, and
 * the size of the transmit threshold plus the rx rdy threshold can not be
 * bigger than the FIFO size.
 * 
 * MFF
 * 
 * *****************  Version 9  *****************
 * User: Mffallon     Date: 1/07/00    Time: 6:51p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Set default fifo thresholds.
 * 
 * *****************  Version 8  *****************
 * User: Skornfel     Date: 9/07/99    Time: 2:14p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Changed the default to LITTLE ending
 * 
 * *****************  Version 7  *****************
 * User: Skornfel     Date: 8/04/99    Time: 4:53p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Corrected the types for the fuctions to work with the regs
 * 
 * *****************  Version 6  *****************
 * User: Dhooper      Date: 8/04/99    Time: 2:11p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * 
 * *****************  Version 5  *****************
 * User: Skornfel     Date: 7/30/99    Time: 3:10p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * controls 21640 on SA1200 eval board
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 7/16/99    Time: 2:45p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:49p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 * 
 * 
 * ---------------------------------------------------------------------
 */

#ifndef BIG_MAC_PORT
#define BIG_MAC_PORT


/*
some design ideas : 30 regs


  interrupt :
  1. Interrupt status register 
  2. Interrupt enable register
  
  port:
  1. Port control register
  2. MAC address register
  3. Port working mode register
  4. VLAN tag length/type register


  chip:
  1. Identification and revision number register --- port 0 only



  Rx Tx  

   1. Transmit and receive status register 
   2. Transmit counter overflow status register 
   3. Receive counter overflow status register

   4. Transmit and receive error mode register 
   5. FIFO threshold register            RX and TX
   6. Transmit and receive parameter register 
   7. Transmit threshold register 
   8. Transmit flow-control pause time register 
   9. Maximum packet size register       RX 
   10.Inter packet gap value             TX
   11.Transmit counter overflow mask register 
   12.Receive counter overflow mask register



  GMII:
  1. GMII managment accesss
  2. GMII Managment data

  GMII Auto Negotiation:

  1.  GMII Control register 
  2.  GMII Status register 
  3.  AN advertisement register 
  4.  AN link partner ability base page register 
  5.  AN expansion register 
  6.  AN next page transmit register
  7.  AN link partner receive next page register 
  8.  Extended status register 
  9.  GPCS status register

	



*/
#include "GigNetStat.h"

#include "rmonhtst.h"
#include "rmonetst.h"
#include "snmpst.h"
#include "GigCtrlRg.h"

#include "GigIntRg.h"
#include "GigChipRg.h"
#include "GigGmiiAnRg.h"
#include "GigGmiiRg.h"
#include "GigRxTxRg.h"
//#include "GigTxRg.h"
//#include "GigRxRg.h"
#include "GigPortRg.h"


//Extern NeworkControl STABS
extern void PortStop(int port);
extern void CounterOverflow(int port, SnmpStats &SnmpBeforeClear, 
							          RmonHostStats	&RmonHostBeforeClear, 
									  RmonEtherStats &RmonEtherBeforeClear);
extern void LinkChange(int port);
extern void TransmitOk(int port);
extern void PacketEvent(int port, unsigned short tx_rx_stat_reg);
extern void ReceiveOk(int port);
extern void AutoNegotiationErr(int port);
extern void AutoNegotiationPageRx(int port);


enum NumPorts{
	NUM_OF_PORTS =2
};
//moved to API
/*
enum PotrtRunDirection{
	RX_AND_TX = 0,
	RX = 1,
	TX = 2
};
enum BusWidth{
	BUS_WIDTH_32 = 0,
	BUS_WIDTH_64 = 1,
	BUS_SPLIT    = 2,
	BUS_RESERVED = 3
};
*/
//enum FIFO_Threshold{
//	THRESHOLD_1   = 0, 
//	THRESHOLD_32  = 0x1,
//	THRESHOLD_48  = 0x2,
//	THRESHOLD_64  = 0x3,
//	THRESHOLD_80  = 0x4,
//	THRESHOLD_96  = 0x5,
//	THRESHOLD_112 = 0x6,
//	THRESHOLD_128 = 0x7,
//	THRESHOLD_144 = 0x8,
//	THRESHOLD_160 = 0x9,
//	THRESHOLD_176 = 0xa,
//	THRESHOLD_192 = 0xb,
//	THRESHOLD_208 = 0xc,
//	THRESHOLD_224 = 0xd,
//	THRESHOLD_240 = 0xe,
//	THRESHOLD_256 = 0xf,

//	THRESHOLD_512 = 0x1f,
//	THRESHOLD_768 = 0x2f,
//	THRESHOLD_1024 = 0x3f,
//	THRESHOLD_1280 = 0x4f,
//	THRESHOLD_1536 = 0x5f,
//	THRESHOLD_1776 = 0x6f,
	//THRESHOLD_384 = 0x7f
//};

class GigMacPort{

public:


	GigMacPort(int port, void * ctrl_regs, void * net_regs, void * net_regs_clear){
		//cout << "GigMacPort " << port << " CTOR" << endl;
		Port = port;
    	Bus_width    =BUS_WIDTH_64;	/*BUS_WIDTH_32*/
        Rx_threshold =7;			/*THRESHOLD_128*/ 
        Tx_threshold =3;			/*THRESHOLD_64*/
		Order = LITTLE;				// LITTLE;

//SK need for 2 separetly maped copies of net_regs 100 and 200 maped
//or maybe only one pointer to the port's memory including 0x0 offset for ctrl
//0x100 and 0x200 for net_stats
		PortMacNetStats			= new(net_regs) GigMacNetStats;
		PortMacNetStatsClear	= new(net_regs_clear) GigMacNetStats;
		PortMacNetStatsShadow   = new (&PtrPortMacNetStatsShadowInstance) GigMacNetStats;
		PortMacControlRegs      = new(ctrl_regs) GigMacControlRegs;
		
		PortSnmpCounters		= new SnmpStats(*PortMacNetStats);
		PortRmonHostCounters	= new RmonHostStats(*PortMacNetStats);
		PortRmonEtherCounters	= new RmonEtherStats(*PortMacNetStats);

		PortSnmpCountersBeforeClear		= new SnmpStats(*PortMacNetStatsShadow);
		PortRmonHostCountersBeforeClear	= new RmonHostStats(*PortMacNetStatsShadow);
		PortRmonEtherCountersBeforeClear= new RmonEtherStats(*PortMacNetStatsShadow);

		ChipSpecReg				= new GigMacChipRegs(*PortMacControlRegs);
		PortControl				= new GigMacPortRegs(*PortMacControlRegs);
		PortTransmitReceive     = new GigMacRxTxRegs (*PortMacControlRegs);
///		PortReceive				= new GigMacReceive(*PortMacControlRegs);
///		PortTransmit			= new GigMacTransmit(*PortMacControlRegs);
		PortInterrupt			= new GigMacInterruptRegs(*PortMacControlRegs);
		PortGmiiAn				= new GigMacGmiiAnRegs(*PortMacControlRegs);
		PortGmii				= new GigMacGmiiRegs(*PortMacControlRegs);

		//PortMacControlRegs->Show();
		

	};

	~GigMacPort(){

		delete PortMacControlRegs;
		delete PortMacNetStats;		
		delete PortMacNetStatsClear;
		delete PortMacNetStatsShadow;   
		delete PortSnmpCounters;
		delete PortRmonHostCounters;
		delete PortRmonEtherCounters;

		delete PortSnmpCountersBeforeClear;
		delete PortRmonHostCountersBeforeClear;
		delete PortRmonEtherCountersBeforeClear;
	

		delete ChipSpecReg;
		delete PortControl;
		delete PortTransmitReceive;
///		delete PortReceive;
///		delete PortTransmit;
		delete PortInterrupt;
		delete PortGmiiAn;
		delete PortGmii;
	};
/**

Initialization Sequence
-----------------------
Each 82547EB port must be initialized according to the following sequence:
1. Disable port and reset it by writing a value of 07h to the port control register (PORT_CTR).
	PTRST		2		Port reset.
	CTRST		1		Control reset.
	CNRST		0		Counter reset.
2. Initialize the port by writing to the relevant configuration registers.
3. Enable port operation by writing a value of 18h to the port control register (PORT_CTR).
	TXEN		4		Transmit enable.
	RXEN		3		Receive enable.

  
Mode Change Sequence
--------------------
In order to change the 82547EB working mode without impacting packet transfer, the following
sequence must be used:
1. Disable the port by writing a value of 00h to the port control register (PORT_CTR).
2. Wait until the port enters the stop state. The stop state entry may generate an interrupt if
enabled, and is reported by the stop bit in the interrupt status register (INT_STT<STOP>).
3. Change the configuration register values.
4. Enable port operation by writing a value of 18h to the port control register (PORT_CTR).

Note: The port mode register (PORT_MODE) can be updated only by using the initialization sequence.
**/ 
	
	void Configure(void);

	inline  GigMacControlRegs &
	GetPortControlRegs(){return *PortMacControlRegs;};

    inline  GigMacPortRegs &
	GetPortRegs(){return *PortControl;};


	inline void
	Start ( int tx_rx = RX_AND_TX ){ // direction to start (bitwise OR of TX / RX)
	    		cout << "GigMacStart" << endl; 	

		//configure the port
		Configure();
	
		//enable RX and TX
		if (tx_rx == RX_AND_TX)
		PortControl->SetPortControl(
			//(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXSTT_SH) | 
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXEN_SH)  | 
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::RXEN_SH)  );

		if (tx_rx == RX)
		PortControl->SetPortControl(
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::RXEN_SH)  );


		if (tx_rx == TX)
		PortControl->SetPortControl(
			//(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXSTT_SH) | 
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXEN_SH)  ); 

	}
 
	inline void 
	Stop ( int tx_rx = RX_AND_TX ) { // direction to stop (bitwise OR of TX / RX)

    //Stop the port 


		if (tx_rx == RX_AND_TX){
			PortControl->ClearPortControlTXSTT(); 
			PortControl->ClearPortControlTXEN();  
			PortControl->ClearPortControlRXEN(); 
		}
	

		if (tx_rx == RX){
			PortControl->ClearPortControlRXEN(); 
		}


		if (tx_rx == TX){
			PortControl->ClearPortControlTXSTT(); 
			PortControl->ClearPortControlTXEN();  
		}

	//PortControl->ClearPortControlPTRST(); 
	//PortControl->ClearPortControlCTRST(); 
	//PortControl->ClearPortControlCNRST();  
	//PortControl->ClearPortControl();

	}
        
	inline void //the same at this point as start
	Restart ( int tx_rx = RX_AND_TX ) {

	//reconfigure the port
	Configure();

			//enable RX and TX
		if (tx_rx == RX_AND_TX)
		PortControl->SetPortControl(
			//(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXSTT_SH) | 
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXEN_SH)  | 
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::RXEN_SH)  );

		if (tx_rx == RX)
		PortControl->SetPortControl(
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::RXEN_SH)  );


		if (tx_rx == TX)
		PortControl->SetPortControl(
			//(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXSTT_SH) | 
			(GigMacPortRegs::PORT_CTRL_MSK << GigMacPortRegs::TXEN_SH)  ); 
	}
        
	inline void 
	Reset (void) {

    //Reset the port 
	//PortControl->ClearPortControlPTRST(); 
	//PortControl->ClearPortControlCTRST(); 
	//PortControl->ClearPortControlCNRST();  
	PortControl->ClearPortControl();
	PortControl->SetPortControlPTRST(); 
	PortControl->SetPortControlCTRST();
	PortControl->SetPortControlCNRST();

	}

	inline void 
	PassConfiguration( BusWidth bus_width=BUS_WIDTH_64,          /*BUS_WIDTH_32*/
					   int /*FIFO_Threshold*/ rx_threshold= 0x7,/*THRESHOLD_128,THRESHOLD_64*/ 
					   int /*FIFO_Threshold*/ tx_threshold= 0x7/*THRESHOLD_64  THRESHOLD_128*/
					 ) {

		Bus_width    = bus_width;       
		Rx_threshold = rx_threshold;    
		Tx_threshold = tx_threshold;    
	};
	
	inline void
	PassEndian(	 Endian order=LITTLE ){
		Order = order; 
	};  

	
	inline void
	CopyStatsAndClear(){
		PortMacNetStatsClear->CopyGigMacNetStatsAndClear(PtrPortMacNetStatsShadowInstance);
	};
     

	inline void
	HandleInterrupt(){
/*
STOP		14		Port stop.	Indicates that transmit or receive path is in the stop state,
OVFC		13		Counter overflow. at least one network statistic counter has reached its highest possible value.
					This bit is a logical OR of all bits in the counter overflow status registers 
LNCH		12		Link change.
TXOK		10		Transmit OK. Indicates successful packet transmission.
PKEV		9		Packet event. logical OR of all the bits in the transmit and receive status register (TX_RX_STT),
					excluding the packet count (PKC) bits. Upon receive events, this bit is set only after the
					packet is fully received by the chip.
RXOK		8		Receive OK. Indicates that a packet has been received without error. 
ANER		7		Auto-Negotiation error.
					Indicates that the link partner advertises that it does not support full-duplex operation, and
					that Auto-Negotiation was restarted.
PGRC		6		Page received.Indicates that a base or next page has been received during
                    Auto-Negotiation process.

*/
//dbg get real regs later 
unsigned short tx_rx_err_stat_reg;


//#ifdef INT
extern volatile int port_int_stat[8]; 
//#endif

//STOP
//#ifndef LAB
//		if ( PortInterrupt->GetInterruptStatusSTOP() )
//#endif
//#ifdef INT
		if ( PortInterrupt->GetInterruptStatusSTOP(port_int_stat[Port]) )
//#endif
		{
			PortStop(Port);	
		};
//OVFC
//#ifndef LAB
//		if ( PortInterrupt->GetInterruptStatusOVFC() )
//#endif
//#ifdef INT
		if ( PortInterrupt->GetInterruptStatusOVFC(port_int_stat[Port]) )
//#endif
		{
		cout << "GetInterruptStatusOVFC - counter overflow " << endl << endl;

		
		// read from address 100 all couters - rest the conter
		// inform that conter overflow happen possibly pass the last updated structure to MIB ?

		//PortMacNetStatsClear->CopyBigMacNetStatsAndClear(PtrPortMacNetStatsShadowInstance);
		//moved to 
		//CopyStatsAndClear ();
		//called from ISR
		PortSnmpCountersBeforeClear->UpdateSnmpCounters();
		PortRmonHostCountersBeforeClear->UpdateHostCounters();
		PortRmonEtherCountersBeforeClear->UpdateEtherCounters();

			CounterOverflow(Port, 
							*PortSnmpCountersBeforeClear, 
							*PortRmonHostCountersBeforeClear, 
							*PortRmonEtherCountersBeforeClear);
		};
//LNCH
//#ifndef LAB
//		if ( PortInterrupt->GetInterruptStatusLNCH() )
//#endif
//#ifdef INT
		if ( PortInterrupt->GetInterruptStatusLNCH(port_int_stat[Port]) )
//#endif
		{
			LinkChange(Port );	
		};
//TXOK
//#ifndef LAB
//		if ( PortInterrupt->GetInterruptStatusTXOK() )
//#endif
//#ifdef INT
		if ( PortInterrupt->GetInterruptStatusTXOK(port_int_stat[Port]) )
//#endif
		{
			TransmitOk(Port);	
		};


//PKEV
//#ifndef LAB
//		if ( PortInterrupt->GetInterruptStatusPKEV() )
//#endif
//#ifdef INT
		if ( PortInterrupt->GetInterruptStatusPKEV(port_int_stat[Port]) )
//#endif
		{
			PacketEvent(Port , tx_rx_err_stat_reg);	
		};

//RXOK
//#ifndef LAB
//		if ( PortInterrupt->GetInterruptStatusRXOK() )
//#endif
//#ifdef INT 
		if ( PortInterrupt->GetInterruptStatusRXOK(port_int_stat[Port]) )
//#endif
		{
			ReceiveOk(Port);
		};	
	
	};

	inline RmonHostStats *
	GetRmonHostStatistics(){
		PortRmonHostCounters->UpdateHostCounters();
		return PortRmonHostCounters->GetHostCounters();
	};

	inline RmonEtherStats *
	GetRmonEtherStatistics(){
		PortRmonEtherCounters->UpdateEtherCounters();
		return PortRmonEtherCounters->GetEtherCounters();
	};

	inline SnmpStats *
	GetSnmpStatistics(){
		PortSnmpCounters->UpdateSnmpCounters();
		return PortSnmpCounters->GetSnmpCounters();
	};


private:

GigMacNetStats		*PortMacNetStats;
GigMacNetStats      *PortMacNetStatsClear;
GigMacNetStats		 PtrPortMacNetStatsShadowInstance;
GigMacNetStats      * PortMacNetStatsShadow;
GigMacControlRegs	*PortMacControlRegs;
SnmpStats			*PortSnmpCounters;
SnmpStats		    *PortSnmpCountersBeforeClear;
RmonHostStats		*PortRmonHostCounters;
RmonEtherStats		*PortRmonEtherCounters;
RmonHostStats		*PortRmonHostCountersBeforeClear;
RmonEtherStats		*PortRmonEtherCountersBeforeClear;
GigMacChipRegs      *ChipSpecReg;
GigMacPortRegs		*PortControl;
GigMacRxTxRegs      *PortTransmitReceive;
///GigMacReceive		*PortReceive;
///GigMacTransmit		*PortTransmit;
GigMacInterruptRegs	*PortInterrupt;
GigMacGmiiAnRegs	*PortGmiiAn;			
GigMacGmiiRegs		*PortGmii;				
//for Bus_width, Rx_threshold, Tx_threshold configuration
BusWidth Bus_width;       
int /*FIFO_Threshold*/ Rx_threshold;    
int /*FIFO_Threshold*/ Tx_threshold;
Endian Order;       
int Port;

};

void GigMacPort::Configure(void){


// Configure the port driver will use settings for the 
// 
//
//	InterruptEnableReg
//
	PortInterrupt->SetInterruptEnable(0x0); //or val

	//PortInterrupt->SetInterruptEnableSTOPE();
	///PortInterrupt->SetInterruptEnableOVFCE();
	///PortInterrupt->SetInterruptEnableLNCHE();
	//PortInterrupt->SetInterruptEnableTXOKE(); 
	///PortInterrupt->SetInterruptEnablePKEVE();
	///PortInterrupt->SetInterruptEnableRXOKE();
	//PortInterrupt->SetInterruptEnableANERE();
	//PortInterrupt->SetInterruptEnablePGRCE();




//	PortControl set by Stop Start Restart Reset methods
 
//	PortWorkingMode
	//PortControl->SetPortWorkingMode(0x0);//or val
    // We want to set MLTPKT on all GIG boards later than EC
    PortControl->SetPortWorkingModeFIFMD(Bus_width);
	//PortControl->SetPortWorkingModeHRYD();
	//PortControl->SetPortWorkingModePS_D();
    if (Order)/*if bigendian*/PortControl->SetPortWorkingModeBEND();
	//PortControl->SetPortWorkingModeLCKE();
	//PortControl->SetPortWorkingModeFXECD();
	//PortControl->SetPortWorkingModeILPKT();
	//PortControl->SetPortWorkingModeCPUBW();
	//PortControl->SetPortWorkingModeSDDIS();
	PortControl->SetPortWorkingModeGPCS(); 


//TransmitReceiveErrorMode
	//PortTransmitReceive->SetTransmitReceiveErrorMode(0x0);//or val

	//PortTransmitReceive->SetTransmitReceiveErrorModePFLC();
	//PortTransmitReceive->SetTransmitReceiveErrorModePMER();
	//PortTransmitReceive->SetTransmitReceiveErrorModePRTL();
	//PortTransmitReceive->SetTransmitReceiveErrorModePSRT();
	//PortTransmitReceive->SetTransmitReceiveErrorModePCRC();
	//PortTransmitReceive->SetTransmitReceiveErrorModePOVF();
	//PortTransmitReceive->SetTransmitReceiveErrorModeUNFS();


//FIFOThreshold

	PortTransmitReceive->SetFIFOThreshold(0x0);//(0x307);//or val
    PortTransmitReceive->SetFIFOThresholdTTH(Tx_threshold);
	PortTransmitReceive->SetFIFOThresholdRTH(Rx_threshold);

//TransmitReceiveParameter
	//PortTransmitReceive->SetTransmitReceiveParameter(0x0);//or val

	PortTransmitReceive->SetTransmitReceiveParameterCRCR();
	//PortTransmitReceive->SetTransmitReceiveParameterHRPL();
	//PortTransmitReceive->SetTransmitReceiveParameterHDRS();
	//PortTransmitReceive->SetTransmitReceiveParameterADFC();
	//PortTransmitReceive->SetTransmitReceiveParameterSPM();
	//PortTransmitReceive->SetTransmitReceiveParameterXOND();
	//PortTransmitReceive->SetTransmitReceiveParameterRFLCE();
	//PortTransmitReceive->SetTransmitReceiveParameterCRCD();
	//PortTransmitReceive->SetTransmitReceiveParameterPADD();

//TransmitThreshold
	PortTransmitReceive->SetTransmitThreshold(0xbe); //or val.  0xbe = just larger than max sized packet

	//PortTransmitReceive->SetTransmitThresholdTSD_SH(0xff);


//TransmitFlowControlPauseTime
	//PortTransmitReceive->SetTransmitFlowControlPauseTime(0x0); //or val

//MaximumPacketSize
	//PortTransmitReceive->SetMaximumPacketSize(0x0);//or val

//InterPacketGapValue
	//PortTransmitReceive->SetInterPacketGapValue(0x0);//or val


//TransmitCounterOverflowMask
	//PortTransmitReceive->SetTransmitCounterOverflowMask(0x0);

	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_BD();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_OK();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_ER();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_PS();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_1519();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_1024();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_512();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_256();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_128();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_65();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_64();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_BRD();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_MLT();
	//PortTransmitReceive->SetTransmitCounterOverflowMaskTX_UNI();


//ReceiveCounterOverflowMask
	//PortTransmitReceive->SetReceiveCounterOverflowMask(0x0);

	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_ER();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_FAL();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_PS();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_1519();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_1024();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_512();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_256();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_128();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_65();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_64();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_L_CR();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_L_OK();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_N_CR();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_BRD();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_MLT();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_UNI();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_S_CR();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_S_OK();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_OV();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_BD();
	//PortTransmitReceive->SetReceiveCounterOverflowMaskRX_OK();

	
//PortGmiiAn				
//PortGmii
//PortGmiiAn->SetGMIIControl(140); //DBG

};





//  Potr Ptrs
//
typedef struct _PortPtrs {
	GigMacPort * ptr_RxMsgDptr[NUM_OF_PORTS];
	}PortPtrs;


#endif //BIG_MAC_PORT 

