/* GigPortRg.h
 * 
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                  
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: GigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: GigPortRg.h $
 * 
 * *****************  Version 11  *****************
 * User: Dmwhite      Date: 12/14/00   Time: 3:53p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * new gig mac multipacket feature
 * 
 * *****************  Version 10  *****************
 * User: Dmwhite      Date: 12/11/00   Time: 2:18p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * add multipacket feature
 * 
 * *****************  Version 6  *****************
 * User: Skornfel     Date: 8/04/99    Time: 4:53p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Corrected the types for the fuctions to work with the regs
 * 
 * *****************  Version 5  *****************
 * User: Skornfel     Date: 7/30/99    Time: 3:10p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * controls 21640 on SA1200 eval board
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 7/16/99    Time: 2:45p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:49p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 
 * ---------------------------------------------------------------------
 */

//  port:
//  1. Port control register
//  2. MAC address register
//  3. Port working mode register
//  4. VLAN tag length/type register

#ifndef GIG_MAC_PORT_REGS
#define GIG_MAC_PORT_REGS

class GigMacPortRegs{
public:

	GigMacPortRegs(GigMacControlRegs & regs):_ControlRegs(regs){
		//cout << "GigMacPortRegs CTOR" << endl;
	};

	~GigMacPortRegs(){};


	inline unsigned short GetPortControl()const { return PortControlF();};

	inline unsigned int GetPortControlFCT()const { return ( PortControlF() >> FCT_SH) & PORT_CTRL_MSK;};
	inline unsigned int GetPortControlTXSTT()const { return ( PortControlF() >> TXSTT_SH) & PORT_CTRL_MSK;};
	inline unsigned int GetPortControlTXEN()const  { return ( PortControlF() >> TXEN_SH) & PORT_CTRL_MSK;};
	inline unsigned int GetPortControlRXEN()const  { return ( PortControlF() >> RXEN_SH) & PORT_CTRL_MSK;};
	inline unsigned int GetPortControlPTRST()const { return ( PortControlF() >> PTRST_SH) & PORT_CTRL_MSK;};
	inline unsigned int GetPortControlCTRST()const { return ( PortControlF() >> CTRST_SH) & PORT_CTRL_MSK;};
	inline unsigned int GetPortControlCNRST()const { return ( PortControlF() >> CNRST_SH) & PORT_CTRL_MSK;};
	
	inline void SetPortControl(unsigned short val){ PortControlF( val );};

	inline void SetPortControlFCT() {   PortControlF (PortControlF() | (PORT_CTRL_MSK << FCT_SH));};
	inline void SetPortControlTXSTT() {   PortControlF (PortControlF() | (PORT_CTRL_MSK << TXSTT_SH));};
	inline void SetPortControlTXEN()  {   PortControlF (PortControlF() | (PORT_CTRL_MSK << TXEN_SH));};
	inline void SetPortControlRXEN()  {   PortControlF (PortControlF() | (PORT_CTRL_MSK << RXEN_SH));};
	inline void SetPortControlPTRST() {   PortControlF (PortControlF() | (PORT_CTRL_MSK << PTRST_SH));};
	inline void SetPortControlCTRST() {   PortControlF (PortControlF() | (PORT_CTRL_MSK << CTRST_SH));};
	inline void SetPortControlCNRST() {   PortControlF (PortControlF() | (PORT_CTRL_MSK << CNRST_SH));};

	inline void ClearPortControl(){ PortControlF( 0x0 );};

	inline void ClearPortControlFCT() {   PortControlF (PortControlF() & ~(PORT_CTRL_MSK << FCT_SH));};
	inline void ClearPortControlTXSTT() {   PortControlF (PortControlF() & ~(PORT_CTRL_MSK << TXSTT_SH));};
	inline void ClearPortControlTXEN()  {   PortControlF (PortControlF() & ~(PORT_CTRL_MSK << TXEN_SH));};
	inline void ClearPortControlRXEN()  {   PortControlF (PortControlF() & ~(PORT_CTRL_MSK << RXEN_SH));};
	inline void ClearPortControlPTRST() {   PortControlF (PortControlF() & ~(PORT_CTRL_MSK << PTRST_SH));};
	inline void ClearPortControlCTRST() {   PortControlF (PortControlF() & ~(PORT_CTRL_MSK << CTRST_SH));};
	inline void ClearPortControlCNRST() {   PortControlF (PortControlF() & ~(PORT_CTRL_MSK << CNRST_SH));};


//moved to do logical or on the bits
enum PortControlShifts{
	FCT_SH  =   8,
	TXSTT_SH=	5,		
	TXEN_SH =	4,
	RXEN_SH =	3,	
	PTRST_SH=	2,	
	CTRST_SH=	1,	
	CNRST_SH=	0
};

enum PortControlMasks{
	PORT_CTRL_MSK =0x1
	};


	inline unsigned short GetPortWorkingMode()const { return PortWorkingModeF();};

	inline unsigned int GetPortWorkingModeFIFMD()const { return ( PortWorkingModeF() >> FIFMD_SH) & FIFMD_MASK;};
	inline unsigned int GetPortWorkingModeHRYD()const { return ( PortWorkingModeF() >> HRYD_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeMLTPKT()const { return ( PortWorkingModeF() >> MLTPKT_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModePS_D()const  { return ( PortWorkingModeF() >> PS_D_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeBEND()const  { return ( PortWorkingModeF() >> BEND_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeLCKE()const { return ( PortWorkingModeF() >> LCKE_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeFXECD()const { return ( PortWorkingModeF() >> FXECD_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeILPKT()const { return ( PortWorkingModeF() >> ILPKT_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeCPUBW()const { return ( PortWorkingModeF() >> CPUBW_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeSDDIS()const { return ( PortWorkingModeF() >> SDDIS_SH) & PORT_WORK_MODE_MSK;};
	inline unsigned int GetPortWorkingModeGPCS()const { return ( PortWorkingModeF() >> GPCS_SH) & PORT_WORK_MODE_MSK;};


	inline void SetPortWorkingMode(unsigned short val){ PortWorkingModeF( val );};
	//inline void SetPortWorkingModeFIFMD() {   PortWorkingModeF (PortWorkingModeF() | (FIFMD_MASK << FIFMD_SH));};
    // setmodeF with getmodeF & ~(0xf << 11)
    // SetmodeF with getmodeF | ((val&f)<<11)
    inline void SetPortWorkingModeFIFMD(unsigned short val) {
					PortWorkingModeF (PortWorkingModeF() & ~(FIFMD_MASK<<FIFMD_SH)); 
					PortWorkingModeF (PortWorkingModeF() | ((val&FIFMD_MASK)<<FIFMD_SH));
	};
    inline void SetPortWorkingModeMLTPKT() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << MLTPKT_SH));};
	inline void SetPortWorkingModeHRYD() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << HRYD_SH));};
	inline void SetPortWorkingModePS_D()  {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << PS_D_SH));};
	inline void SetPortWorkingModeBEND()  {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << BEND_SH));};
	inline void SetPortWorkingModeLCKE() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << LCKE_SH));};
	inline void SetPortWorkingModeFXECD() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << FXECD_SH));};
	inline void SetPortWorkingModeILPKT() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << ILPKT_SH));};
	inline void SetPortWorkingModeCPUBW() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << CPUBW_SH));};
	inline void SetPortWorkingModeSDDIS() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << SDDIS_SH));};
	inline void SetPortWorkingModeGPCS() {   PortWorkingModeF (PortWorkingModeF() | (PORT_WORK_MODE_MSK << GPCS_SH));};


	inline void ClearPortWorkingMode(){ PortControlF( 0x0 );};

	inline void ClearPortWorkingModeFIFMD() {   PortWorkingModeF (PortWorkingModeF() & ~(FIFMD_MASK << FIFMD_SH));};
	inline void ClearPortWorkingModeHRYD() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << HRYD_SH));};
	inline void ClearPortWorkingModeMLTPKT() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << MLTPKT_SH));};
	inline void ClearPortWorkingModePS_D()  {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << PS_D_SH));};
	inline void ClearPortWorkingModeBEND()  {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << BEND_SH));};
	inline void ClearPortWorkingModeLCKE() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << LCKE_SH));};
	inline void ClearPortWorkingModeFXECD() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << FXECD_SH));};



	inline void ClearPortWorkingModeILPKT() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << ILPKT_SH));};
	inline void ClearPortWorkingModeCPUBW() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << CPUBW_SH));};
	inline void ClearPortWorkingModeSDDIS() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << SDDIS_SH));};
	inline void ClearPortWorkingModeGPCS() {   PortWorkingModeF (PortWorkingModeF() & ~(PORT_CTRL_MSK << GPCS_SH));};



	inline unsigned int GetMACAddress()const { return MACAddressF();}; //fix the length later
	inline void SetMACAddress(unsigned int val){ MACAddressF( val );}; //fix the length later

	inline unsigned int GetPortVLANTagLengthType()const { return PortVLANTagLengthTypeF();};
	inline void SetPortVLANTagLengthType(unsigned int val){ PortVLANTagLengthTypeF( val );};


private:

	inline unsigned int PortControlF()const {return _ControlRegs.GetPortControl();};
	inline void PortControlF(unsigned int val){_ControlRegs.SetPortControl(val);};

	inline unsigned int PortWorkingModeF()const {return _ControlRegs.GetPortWorkingMode();};
	inline void PortWorkingModeF(unsigned int val){_ControlRegs.SetPortWorkingMode(val);};

	inline unsigned int MACAddressF()const {return _ControlRegs.GetMACAddress();};
	inline void MACAddressF(unsigned int val){_ControlRegs.SetMACAddress(val);};

	inline unsigned int PortVLANTagLengthTypeF()const {return _ControlRegs.GetVLANTagLengthType();};
	inline void PortVLANTagLengthTypeF(unsigned int val){_ControlRegs.SetVLANTagLengthType(val);};
/*
Port Control Register
Mnemonic: PORT_CTR
Address: 04H - 05H

The port control register handles all the control bits of the port.
Bit Name	Bit #	Bit Description
			15:9	RESERVED
FCT			8		Flow-control packet trigger.
					When set, initiates a flow-control packet transmission to the remote node, as defined in the
					802.3x IEEE Standard. The pause time field in the transmitted packet will be copied from
					the PAUSE_TIME register (see Section 3.2.3.6).
			7:6		RESERVED
TXSTT		5		Transmit restart.
					When set, the chip restarts the transmission process after stopping due to a transmit error.
					The TXSTT is a trigger bit (does not require reset before setting).
TXEN		4		Transmit enable.
					When this bit is set, the port enters the transmit working mode. When reset, the port
					completes transmission of the packet currently processed, and then stops. Stop state is
					reported in the STOP bit located in the interrupt status register (INT_STT).
RXEN		3		Receive enable.
					When this bit is set, the port enters the receive working mode. When reset, the port
					completes reception of the packet currently processed, and then stops. Stop state is
					reported in the STOP bit located in the interrupt status register (INT_STT).
PTRST		2		Port reset.
					When set, this bit causes the transmit and receive logic to reset. setting this bit will not
					reset the control registers.
CTRST		1		Control reset.
					When set, this bit resets all the registers, except PORT_CTR, to their default values.
					Transmit and receive logic are also reset. The setting of this bit causes the mgmt to
					generate a reset pulse. This bit is not self clearing, it requires a reset before setting
CNRST		0		Counter reset.
					When set, this bit resets all the network statistic counters. The CNRST is a trigger bit (does
					not require resetting before setting.)

Access Rules Register access R/W
Value after reset 0000H

*/



/*------------------------

Port Working Mode Register
Mnemonic: PORT_MODE
Address: 24H - 25H

The port mode register controls the CPU bus, FIFO bus, and serial interface modes of work.
!!!!!!  Note: Bits 12, 11, 2, and 0 must have the same value in both ports.

Bit Name	Bit #	Bit Description
			15:14	RESERVED
            13      Multi-Packet Mode
                    When set the device operates in Multipacket mode
FIFMD		12:11	FIFO bus mode:
					This field sets the FIFO bus mode. Both ports of the 82547EB must be operating in the
					same FIFO bus mode. The table below shows the FIFO bus mode according to bits 12
					and 11.
					Bit 12	Bit 11	Mode
					0		0		Narrow-32 bit mode
					0		1		Full-64 bit mode
					1		0		Split mode
					1		1		Reserved
					In the full-64 bit mode, the 64 bits fdat<63:0> are used for either transmitting 
					or receiving data.
					In the split mode, the 32 lower bits fdat<31:0> are used for receiving packets from the
					receive FIFO and the 32 higher bits fdat<63:32> are used for transmitting packets to the
					transmit FIFO.
					In the narrow mode, the lower 32 bits fdat<31:1> are used for either transmitting
					or receiving packets.
HRYD		10		Header ready disable.
					When set, the rxrdy signal will not be asserted when a packet header is in FIFO, but only
					according to FIFO threshold values.
PS_D		9		Packet status disable.
					When set, the packet status will not be appended to received packets that are transferred
					onto the FIFO bus. When reset, The packet status is appended to any packet completely
					transferred onto the FIFO bus, and is driven onto the FIFO bus in the access following the
					last byte transfer (see 4.3.1.1).
BEND		8		Big or little endian mode.
					Defines the byte ordering mode on the FIFO bus. When set, the port uses the big endian
					mode. When reset, the little endian mode is used.
LCKE		7		Enable activation of lock to reference signal. When set, the lckref_{i} signal will be
					asserted to the PHY when there is no activity (no signal detected) on the line. Used only in
					GPCS mode.

FXECD		6		Fix enable comma detect signal.
					When set, the encdet_{i} signal to the PHY will be continuously asserted. When cleared,
					the encdet_{i} signal will be asserted only during loss of synchronization. Used only in
					GPCS mode.
ILPK		5		Internal loopback mode.
					When set, the port is disconnected from the line and all the transmitted packets are sent
					back to the receive side. Packets are not transmitted onto the line and packets received
					from the line are rejected.
			4:3		RESERVED
CPUBW		2		CPU data bus width.
					When asserted, bus width is 16, all cdat<15:0> bits are in use.
					When deasserted, bus width is 8, indicating that the 8 low bits cdat<7:0> are in use.
SDDIS		1		Signal detect disable.
					Setting this bit will cause 82547EB to consider the value of the sd{i} input signal as high,
					regardless to the actual value on the pin. Used only in GPCS mode.
GPCS		0		GPCS port mode.
					This bit defines the GMII/GPCS port mode. If a GPCS PHY device is connected, this bit
					must be set. If a GMII PHY device is connected, this bit must be reset.

Access Rules Register access R/W
Value after reset 0000H

*/
enum BusMode{
	NARROW_32 = 0,
	FULL_64   = 1,
	SPLIT     = 2,
	RESERVED  = 3
};

enum PortWorkingModeShifts{
	MLTPKT_SH  = 13,
	FIFMD_SH  = 11,
	HRYD_SH=	10,		
	PS_D_SH =	9,
	BEND_SH =	8,	
	LCKE_SH=	7,	
	FXECD_SH=	6,	
	ILPKT_SH=	5,
	CPUBW_SH=	2,
	SDDIS_SH=	1,
	GPCS_SH=	0
};

enum PortWorkingModeMasks{
	FIFMD_MASK = 0x3,
	PORT_WORK_MODE_MSK =0x1
	};

/*

---------------------------


MAC Address Registers
Mnemonic: MAC_ADD
Address: 30H - 35H

The MAC address register contains the Ethernet physical address of the port. This address is
inserted into transmitted flow-control packets in the source address field.

Bit Name	Bit		# Bit Description
EAD			47:0	Port Ethernet address.
Bit			47		is the most significant bit, and bit 0 is the least significant bit and is the first bit of the
					MAC serial bit stream.
					
Access Rules Register access R/W
Value after reset 0000H

*/


/*-------------------------


VLAN Tag Length/Type Register
Mnemonic: VLAN_TAG
Address: 36H - 37H

The VLAN length/type register is used to define the VLAN tag length/type field.
Bit Name	Bit #	Bit Description
VLAN_T YP	15:0	VLAN type
					This field defines length/type field of the VLAN tag when inserted into transmitted
					frames.

Access Rules Register access R/W
Value after reset 8100H


*/

GigMacControlRegs & _ControlRegs;
};

#endif //GIG_MAC_PORT_REGS 
