/* GigRxTxRg.h
 * 
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: GigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: GigRxTxRg.h $
 * 
 * *****************  Version 5  *****************
 * User: Mffallon     Date: 1/07/00    Time: 6:51p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Set default fifo thresholds.
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 8/04/99    Time: 4:53p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * Corrected the types for the fuctions to work with the regs
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 7/30/99    Time: 3:10p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * controls 21640 on SA1200 eval board
 * 
 * *****************  Version 2  *****************
 * User: Skornfel     Date: 7/16/99    Time: 2:45p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * 
 * *****************  Version 1  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:50p
 * Created in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 
 * ---------------------------------------------------------------------
 */

//    Rx [Tx]  

//   1. Transmit and receive status register    RX-TX
//   2. ....Transmit counter overflow status register TX
//   3. .......................................Receive counter overflow status register
//   4. Transmit and receive error mode registerRX-TX 
//   5. FIFO threshold register                 RX-TX
//   6. Transmit and receive parameter register RX-TX
//   7. ....Transmit threshold register                TX
//   8. ....Transmit flow-control pause time register  TX
//   9. ........................................Maximum packet size register          RX 
//   10.....Inter packet gap value                     TX
//   11.....Transmit counter overflow mask register    TX 
//   12........................................Receive counter overflow mask register RX



#ifndef GIG_MAC_RX_REGS
#define GIG_MAC_RX_REGS

class GigMacRxTxRegs{
public:

	GigMacRxTxRegs(GigMacControlRegs & regs):_ControlRegs(regs){
		//cout << "GigMacRxTxRegs CTOR" << endl;
	};

	~GigMacRxTxRegs(){};


/*
Transmit and Receive Status Register
Mnemonic: TX_RX_STT
Address: 08H - 09H

This register reports events that have occurred during packet reception and transmission. All bits,
except PKC bit, are reset upon reading this register. The UNF bit is set only if packet transmission
is programmed to be stopped following FIFO underflow in the transmit and receive error mode
register (TX_RX_ERR<UNFS>).
The receive error bits (TX_RX_STT<15:8>) in this register, are set only if the receive logic is
programmed to pass packets with the corresponding event in the transmit and receive error mode
register (TX_RX_ERR<15:8>). If a receive packet with multiple errors is to be passed and not all
the corresponding bits in the transmit and receive error mode register (TX_RX_ERR) are set, none
of the bits in this register are set. The receive status is also appended to the end of the packet in the
receive FIFO in packet status append mode (see Section 4.3.1.1).

Bit Name	Bit #	Bit Description
FLC			15		Flow-control packet.
					Indicates that a flow-control packet was received.
			14		RESERVED
MER			13		GMII error.
					Indicates either that a symbol error was detected in the GPCS mode or that an GMII error
					signal was asserted in the GMII mode during packet reception.
RTL			12		Too long packet.
					Indicates that a packet longer than the maximum allowable packet size specified in the
					PKT_MAX_SIZE register, has been received.
SRT			11		Short packet.
					Indicates that a packet shorter than 64 bytes has been received.
			10		RESERVED
CRC			9		CRC error.
					Indicates that a packet was received with a CRC error.
OVF			8		FIFO overflow.
					Indicates that there was not enough space available in the receive FIFO during packet
					reception.
PKC			7:6		Packet count.
					Indicates the number of packets currently stored in the transmit FIFO. The packet count is
					incremented while loading the first byte of a packet and decremented following
					transmission of a packet.
			5:1		RESERVED
UNF			0		FIFO underflow.
					Indicates that data was not available in the transmit FIFO during packet transmission.

Access Rules Register access R
Value after reset 0000H
*/

	inline unsigned short GetTransmitReceiveStatus()const { return TransmitReceiveStatusF();};

	inline unsigned int GetTransmitReceiveStatusFLC()const { return ( TransmitReceiveStatusF() >> FLC_SH) & TX_RX_STAT_MSK;};
	inline unsigned int GetTransmitReceiveStatusMER()const { return ( TransmitReceiveStatusF() >> MER_SH) & TX_RX_STAT_MSK;};
	inline unsigned int GetTransmitReceiveStatusRTL()const { return ( TransmitReceiveStatusF() >> RTL_SH) & TX_RX_STAT_MSK;};
	inline unsigned int GetTransmitReceiveStatusSRT()const { return ( TransmitReceiveStatusF() >> SRT_SH) & TX_RX_STAT_MSK;};
	inline unsigned int GetTransmitReceiveStatusCRC()const { return ( TransmitReceiveStatusF() >> CRC_SH) & TX_RX_STAT_MSK;};
	inline unsigned int GetTransmitReceiveStatusOVF()const { return ( TransmitReceiveStatusF() >> OVF_SH) & TX_RX_STAT_MSK;};
	inline unsigned int GetTransmitReceiveStatusPKC()const { return ( TransmitReceiveStatusF() >> PKC_SH) & PKC_MSK;};
	inline unsigned int GetTransmitReceiveStatusUNF()const { return ( TransmitReceiveStatusF() >> UNF_SH) & TX_RX_STAT_MSK;};




	inline unsigned int TransmitReceiveStatusF()const {return _ControlRegs.GetTransmitReceiveStatus();};

enum TransmitReceiveStatusShifts{
	FLC_SH		=15,		
	MER_SH		=13,		
	RTL_SH		=12,		
	SRT_SH		=11,		
	CRC_SH		=9,		
	OVF_SH		=8,		
	PKC_SH		=6,		
	UNF_SH		=0		
};

enum TransmitReceiveStatusMasks{
	TX_RX_STAT_MSK =0x1,
	PKC_MSK =0x3
	};
/*	
Transmit Counter Overflow Status Register
Mnemonic: TX_OV_STT
Address: 0AH - 0DH

The transmit counter overflow status register reports overflow events of each transmit counter.
When a counter reaches its highest possible value, it will continue to count from zero, and the
corresponding status bit will be set. Reading this register will reset all the bits, and will clear the
interrupt if it was activated.

Bit Name	Bit #	Bit Description
  
..			31:20	RESERVED
TX_BD		19		TX_OCT_BAD_CNT counter overflow.
TX_OK		18		TX_OCT_OK_CNT counter overflow.
TX_ER		17		TX_ERR_CNT counter overflow.
TX_PS		16		TX_PAUSE_CNT counter overflow.
TX_1519		15		TX_PKT_1519_CNT counter overflow.
TX_1024		14		TX_PKT_1024_CNT counter overflow.
TX_512		13		TX_PKT_512_CNT counter overflow.
TX_256		12		TX_PKT_256_CNT counter overflow.
TX_128		11		TX_PKT_128_CNT counter overflow.
TX_65		10		TX_PKT_65_CNT counter overflow.
TX_64		9		TX_PKT_64_CNT counter overflow.
			8:3		RESERVED
TX_BRD		2		TX_BRD_OK_CNT counter overflow.
TX_MLT		1		TX_MLT_OK_CNT counter overflow.
TX_UNI		0		TX_UNI_OK_CNT counter overflow.

Access Rules Register access R
Value after reset 0000H


*/

	inline unsigned int GetTransmitCounterOverflowStatus()const { return TransmitCounterOverflowStatusF();};

	inline unsigned int GetTransmitCounterOverflowStatusTX_BD()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_BD_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_OK()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_OK_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_ER()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_ER_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_PS()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_PS_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_1519()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_1519_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_1024()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_1024_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_512()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_512_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_256()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_256_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_128()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_128_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_65()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_65_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_64()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_64_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_BRD()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_BRD_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_MLT()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_MLT_SH) & TX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetTransmitCounterOverflowStatusTX_UNI()const 
						{ return ( TransmitCounterOverflowStatusF() >> TX_UNI_SH) & TX_CNT_OVERSTAT_MSK;};


	inline unsigned int TransmitCounterOverflowStatusF()const 
						{return _ControlRegs.GetTransmitCounterOverflowStatus();};
  
enum TransmitCounterOverflowStatusShifts{
	TX_BD_SH		=19,		
	TX_OK_SH		=18,		
	TX_ER_SH		=17,		
	TX_PS_SH		=16,		
	TX_1519_SH		=15,		
	TX_1024_SH		=14,		
	TX_512_SH		=13,		
	TX_256_SH		=12,		
	TX_128_SH		=11,		
	TX_65_SH		=10,		
	TX_64_SH		=9,	
	TX_BRD_SH		=2,		
	TX_MLT_SH		=1,		
	TX_UNI_SH		=0		
};

enum TransmitCounterOverflowStatusMasks{
	TX_CNT_OVERSTAT_MSK =0x1
	};
	
	  
/*


Receive Counter Overflow Status Register
Mnemonic: RX_OV_STT
Address: 0EH - 11H

The receive counter overflow status register reports overflow events of each receive counter. When
a counter reaches its highest possible value, it will continue to count from zero, and the
corresponding status bit will be set. Reading this register will reset all the bits, and will clear the
interrupt if it was activated.


Bit Name	Bit #	Bit Description
			31:23	RESERVED
RX_ER		22		RX_GPCS_ERR_CNT counter overflow.
RX_FAL		21		RX_FALS_CRS_CNT counter overflow.
RX_PS		20		RX_PAUSE_CNT counter overflow.
RX_1519		19		RX_PKT_1519_CNT counter overflow.
RX_1024		18		RX_PKT_1024_CNT counter overflow.
RX_512		17		RX_PKT_512_CNT counter overflow.
RX_256		16		RX_PKT_256_CNT counter overflow.
RX_128		15		RX_PKT_128_CNT counter overflow.
RX_65		14		RX_PKT_65_CNT counter overflow.
RX_64		13		RX_PKT_64_CNT counter overflow.
RX_L_CR		12		RX_LONG_CRC_CNT counter overflow.
			11		RESERVED
RX_L_OK		10		RX_LONG_OK_CNT counter overflow.
RX_N_CR		9		RX_NORM_CRC_CNT counter overflow.
RX_BRD		8		RX_BRD_OK_CNT counter overflow.
RX_MLT		7		RX_MLT_OK_CN counter overflow.
RX_UNI		6		RX_UNI_OK_CNT counter overflow.
RX_S_CR		5		RX_SHORT_CRC_CNT counter overflow.
RX_S_OK		4		RX_SHORT_OK_CNT counter overflow.
RX_OV		3		RX_OVF_CNT counter overflow.
			2		RESERVED
RX_BD		1		RX_OCT_BAD_CNT counter overflow.
RX_OK		0		RX_OCT_OK_CNT counter overflow.

Access Rules Register access R
Value after reset 0000H

*/

	inline unsigned int GetReceiveCounterOverflowStatus()const { return ReceiveCounterOverflowStatusF();};

	inline unsigned int GetReceiveCounterOverflowStatusRX_ER()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_ER_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_FAL()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_FAL_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_PS()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_PS_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_1519()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_1519_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_RX_1024()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_1024_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_RX_512()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_512_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_RX_256()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_256_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_128()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_128_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_65()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_65_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_64()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_64_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_L_CR()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_L_CR_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_L_OK()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_L_OK_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_N_CR()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_N_CR_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_BRD()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_BRD_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_MLT()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_MLT_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_UNI()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_UNI_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_S_CR()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_S_CR_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_S_OK()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_S_OK_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_OV()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_OV_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_BD()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_BD_SH) & RX_CNT_OVERSTAT_MSK;};
	inline unsigned int GetReceiveCounterOverflowStatusRX_OK()const 
						{ return ( ReceiveCounterOverflowStatusF() >> RX_OK_SH) & RX_CNT_OVERSTAT_MSK;};
	

	inline unsigned int ReceiveCounterOverflowStatusF()const 
						{return _ControlRegs.GetReceiveCounterOverflowStatus();};



enum ReceiveCounterOverflowStatusShifts{
	RX_ER_SH		=22,		
	RX_FAL_SH		=21,	
	RX_PS_SH		=20,	
	RX_1519_SH		=19,		
	RX_1024_SH		=18,		
	RX_512_SH		=17,		
	RX_256_SH		=16,		
	RX_128_SH		=15,		
	RX_65_SH		=14,		
	RX_64_SH		=13,		
	RX_L_CR_SH		=12,		
	RX_L_OK_SH		=10,		
	RX_N_CR_SH		=9,		
	RX_BRD_SH		=8,		
	RX_MLT_SH		=7,		
	RX_UNI_SH		=6,		
	RX_S_CR_SH		=5,		
	RX_S_OK_SH		=4,		
	RX_OV_SH		=3,		
	RX_BD_SH		=1,		
	RX_OK_SH		=0		
};

enum ReceiveCounterOverflowStatusMasks{
	RX_CNT_OVERSTAT_MSK =0x1
	};
/*

Transmit and Receive Error Mode Register
Mnemonic: TX_RX_ERR
Address: 20H - 21H

This register defines the actions to be performed following a transmit or receive error.
When a receive packet with a specific event is programmed to be passed, the corresponding
packets are not discarded and are transferred as regular packets. The transmit and receive events
are reported in the transmit and receive status register (TX_RX_STT) and can generate an interrupt
according to the interrupt enable register (INT_EN).
If a specific event is not programmed to enable reception of a packet, the corresponding packets are
discarded from the receive FIFO. Failure will be reported onto the rxfail signal if the packet has
already started to be transferred onto the FIFO bus. If a packet with multiple errors is to be passed,
all corresponding bits must be set.

Bit Name	Bit #	Bit Description
PFLC		15		Pass flow-control packets.
			14		RESERVED
PMER		13		Pass packets with GMII error.
PRTL		12		Pass too long packets.
PSRT		11		Pass short packets.
			10		RESERVED
PCRC		9		Pass CRC error packets.
POVF		8		Pass FIFO overflow.
			7:1		RESERVED
UNFS		0		Stop transmission after FIFO underflow.
					When set, the transmit FIFO is flushed and transmission stops if an underflow occurs.
					When transmission is stopped after FIFO underflow, flow-control packets can still be
					transmitted. A value of 1 in this bit will be followed by activation of PKEV interrupt if it is
					enabled (see Section 3.2.2.1), and not by the port stop interrupt.
					When reset, if an underflow occurs the packet will be flushed from the transmit FIFO, and
					the following packet will be transmitted.
					
Access Rules Register access R/W
Value after reset 0000H

*/

	inline unsigned short GetTransmitReceiveErrorMode()const { return TransmitReceiveErrorModeF();};

	inline unsigned int GetTransmitReceiveErrorModePFLC()const { return ( TransmitReceiveErrorModeF() >> PFLC_SH) & TX_RX_ERR_MSK;};
	inline unsigned int GetTransmitReceiveErrorModePMER()const { return ( TransmitReceiveErrorModeF() >> PMER_SH) & TX_RX_ERR_MSK;};
	inline unsigned int GetTransmitReceiveErrorModePRTL()const { return ( TransmitReceiveErrorModeF() >> PRTL_SH) & TX_RX_ERR_MSK;};
	inline unsigned int GetTransmitReceiveErrorModePSRT()const { return ( TransmitReceiveErrorModeF() >> PSRT_SH) & TX_RX_ERR_MSK;};
	inline unsigned int GetTransmitReceiveErrorModePCRC()const { return ( TransmitReceiveErrorModeF() >> PCRC_SH) & TX_RX_ERR_MSK;};
	inline unsigned int GetTransmitReceiveErrorModePOVF()const { return ( TransmitReceiveErrorModeF() >> POVF_SH) & TX_RX_ERR_MSK;};
	inline unsigned int GetTransmitReceiveErrorModeUNFS()const { return ( TransmitReceiveErrorModeF() >> UNFS_SH) & TX_RX_ERR_MSK;};
	
	inline void SetTransmitReceiveErrorMode(unsigned short val){ TransmitReceiveErrorModeF( val );};

	inline void SetTransmitReceiveErrorModePFLC(){ TransmitReceiveErrorModeF (TransmitReceiveErrorModeF() | (TX_RX_ERR_MSK << PFLC_SH));};
	inline void SetTransmitReceiveErrorModePMER(){ TransmitReceiveErrorModeF (TransmitReceiveErrorModeF() | (TX_RX_ERR_MSK << PMER_SH));};
	inline void SetTransmitReceiveErrorModePRTL(){ TransmitReceiveErrorModeF (TransmitReceiveErrorModeF() | (TX_RX_ERR_MSK << PRTL_SH));};
	inline void SetTransmitReceiveErrorModePSRT(){ TransmitReceiveErrorModeF (TransmitReceiveErrorModeF() | (TX_RX_ERR_MSK << PSRT_SH));};
	inline void SetTransmitReceiveErrorModePCRC(){ TransmitReceiveErrorModeF (TransmitReceiveErrorModeF() | (TX_RX_ERR_MSK << PCRC_SH));};
	inline void SetTransmitReceiveErrorModePOVF(){ TransmitReceiveErrorModeF (TransmitReceiveErrorModeF() | (TX_RX_ERR_MSK << POVF_SH));};
	inline void SetTransmitReceiveErrorModeUNFS(){ TransmitReceiveErrorModeF (TransmitReceiveErrorModeF() | (TX_RX_ERR_MSK << UNFS_SH));};

	inline unsigned int TransmitReceiveErrorModeF()const {return _ControlRegs.GetTransmitReceiveErrorMode();};
	inline void TransmitReceiveErrorModeF(unsigned short val){_ControlRegs.SetTransmitReceiveErrorMode(val);};

enum TransmitReceiveErrorModeShifts{
	PFLC_SH	=	15,		
	PMER_SH	=	13,		
	PRTL_SH	=	12,		
	PSRT_SH	=	11,		
	PCRC_SH	=	9,		
	POVF_SH	=	8,		
	UNFS_SH	=	0	
};

enum TransmitReceiveErrorModeMasks{
	TX_RX_ERR_MSK =0x1
	};
	
/*

FIFO Threshold Register
Mnemonic: FFO_TSHD
Address: 22H - 23H
The FIFO register handles the transmit and receive FIFO threshold.

Bit Name	Bit #	Bit Description
			15:14	RESERVED
TTH			13:8	This field defines the minimum amount of free space required in the transmit FIFO in order
					to assert the txrdy signal. The effective threshold in bytes is equal to 16  (TTH + 1).
			7		RESERVED
RTH			6:0		This field defines the minimum amount of data required in the receive FIFO in order to
					assert the rxrdy signal. The effective threshold in bytes is equal to 16  (RTH + 1).
					
Access Rules Register access R/W
Value after reset 0303H

*/
	inline unsigned short GetFIFOThreshold()const { return FIFOThresholdF();};

	inline unsigned int GetFIFOThresholdTTH()const { return ( FIFOThresholdF() >> TTH_SH) & TTH_MSK;};
	inline unsigned int GetFIFOThresholdRTH()const { return ( FIFOThresholdF() >> RTH_SH) & RTH_MSK;};

	inline void SetFIFOThreshold(unsigned short val){ FIFOThresholdF( val );};


	inline void SetFIFOThresholdTTH (unsigned short val) {
		//FIFOThreshold &= ~(TTH_MSK<<TTH_SH); FIFOThreshold |= ((val&0x3f)<<TTH_SH);
		FIFOThresholdF (FIFOThresholdF() & ~(TTH_MSK<<TTH_SH)); 
		FIFOThresholdF (FIFOThresholdF() |  ((val&TTH_MSK)<<TTH_SH));//(3<<TTH_SH));//
	};	
	inline void SetFIFOThresholdRTH (unsigned short val) {
		//FIFOThreshold &= ~(RTH_MSK<<RTH_SH); FIFOThreshold |= ((val&0xff)<<RTH_SH);
		FIFOThresholdF (FIFOThresholdF() & ~(RTH_MSK<<RTH_SH)); 
		FIFOThresholdF (FIFOThresholdF() | ((val&RTH_MSK)<<RTH_SH));
	};


	inline unsigned int FIFOThresholdF()const {return _ControlRegs.GetFIFOThreshold();};
	inline void FIFOThresholdF(unsigned short val){_ControlRegs.SetFIFOThreshold(val);};


enum FIFOThresholdShifts{
	TTH_SH		=8,	
	RTH_SH		=0
};

enum FIFOThresholdMasks{
	TTH_MSK =0x3f,
	RTH_MSK =0x3f
	};
	
/*

Transmit and Receive Parameters Register
Mnemonic: TX_RX_PARAM
Address: 26H - 27H

The transmit and receive parameters register handles the control of the serial interface.

Bit Name	Bit #	Bit Description
CRCR		15		CRC remove.
					When set, the last four bytes of the received packet will not be transferred onto the FIFO
					bus. Packets shorter than 4 bytes will be discarded.
HRPL		14		Header replay.
					When set, packet header is transferred twice onto the FIFO bus.
HDRS		13:8	Header size.
					Header size is used for the header replay function and for rxrdy signal assertion (even if the
					header replay function is disabled). The header size is calculated in bytes as 4  HDRS
					(HDRS > 4).
			7:6		RESERVED

ADFC		5		Additional flow control.
					When this bit is asserted, the 82547EB will send an additional flow-control packet if a
					flow-control packet was sent upon assertion of the flct_{i} signal, and while this signal was
					asserted, the link partners pause time period was about to end. This will cause the link
					partner to receive the additional flow-control packet before its pause time counting ends.

					NOTE: This feature is operative only if XOND bit is equal to 0.

SPM			4		Single packet mode.
					When set, a packet is loaded in the transmit FIFO only after the previous packet was
					transmitted onto the serial line.
XOND		3		XON disable.
					When set, a flow-control packet with pause time equal to zero, will not be sent upon flct_{i}
					signal deassertion.
RFLCE		2		Receive flow-control mode enable.
					When set, transmission is paused upon receiving flow-control packets.

					NOTE: This bit is used only in GMII mode and in GPCS mode when Auto-Negotiation is
					disabled (GMII_CTL<ANENBL=0>). In GPCS mode when Auto-Negotiation is
					enabled, the desired mode should be written in the AN_ADV<PAUSE> bit, and the
					final mode after the Auto-Negotiation will be reported in the GPCS_STT<RFC> bit.
CRCD		1		CRC appending disable.
					When set, packets are transmitted without padding or CRC appending to the end of the
					packet. This field is ignored if the txasis signal is asserted during the start of packet loading.

					NOTE: In case of VLAN tag append, strip or replace, the frame check sequence (FCS)
					field will be calculated by the 82547EB (see 4.2.2.1).
PADD		0		Padding appending disable.
					When set, short packets are transmitted without the addition of bytes complementing their
					sizes to 64 bytes. When the CRCD bit is set, this bit is ignored. This field is ignored if the
					txasis signal is asserted during the start of packet loading.

Access Rules Register access R/W
Value after reset 1004H

*/
	inline unsigned short GetTransmitReceiveParameter()const { return TransmitReceiveParameterF();};

	inline unsigned int GetTransmitReceiveParameterCRCR()const { return ( TransmitReceiveParameterF() >> CRCR_SH) & TX_RX_PARM_MSK;};
	inline unsigned int GetTransmitReceiveParameterHRPL()const { return ( TransmitReceiveParameterF() >> HRPL_SH) & TX_RX_PARM_MSK;};
	inline unsigned int GetTransmitReceiveParameterHDRS()const { return ( TransmitReceiveParameterF() >> HDRS_SH) & HDRS_MSK;};
	inline unsigned int GetTransmitReceiveParameterADFC()const { return ( TransmitReceiveParameterF() >> ADFC_SH) & TX_RX_PARM_MSK;};
	inline unsigned int GetTransmitReceiveParameterSPM()const { return ( TransmitReceiveParameterF() >> SPM_SH) & TX_RX_PARM_MSK;};
	inline unsigned int GetTransmitReceiveParameterXOND()const { return ( TransmitReceiveParameterF() >> XOND_SH) & TX_RX_PARM_MSK;};
	inline unsigned int GetTransmitReceiveParameterRFLCE()const { return ( TransmitReceiveParameterF() >> RFLCE_SH) & TX_RX_PARM_MSK;};
	inline unsigned int GetTransmitReceiveParameterCRCD()const { return ( TransmitReceiveParameterF() >> CRCD_SH) & TX_RX_PARM_MSK;};
	inline unsigned int GetTransmitReceiveParameterPADD()const { return ( TransmitReceiveParameterF() >> PADD_SH) & TX_RX_PARM_MSK;};
	
	inline void SetTransmitReceiveParameter(unsigned short val){ TransmitReceiveParameterF( val );};

	inline void SetTransmitReceiveParameterCRCR(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << CRCR_SH));};
	inline void SetTransmitReceiveParameterHRPL(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << HRPL_SH));};
	inline void SetTransmitReceiveParameterHDRS(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (HDRS_MSK << HDRS_SH));};
	inline void SetTransmitReceiveParameterADFC(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << ADFC_SH));};
	inline void SetTransmitReceiveParameterSPM(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << SPM_SH));};
	inline void SetTransmitReceiveParameterXOND(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << XOND_SH));};
	inline void SetTransmitReceiveParameterRFLCE(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << RFLCE_SH));};
	inline void SetTransmitReceiveParameterCRCD(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << CRCD_SH));};
	inline void SetTransmitReceiveParameterPADD(){ TransmitReceiveParameterF (TransmitReceiveParameterF() | (TX_RX_PARM_MSK << PADD_SH));};


	inline unsigned int TransmitReceiveParameterF()const {return _ControlRegs.GetTransmitReceiveParameter();};
	inline void TransmitReceiveParameterF(unsigned short val){_ControlRegs.SetTransmitReceiveParameter(val);};

enum TransmitReceiveParameterShifts{
	CRCR_SH		=15,	
	HRPL_SH		=14,		
	HDRS_SH		=8,   //13:8	
	ADFC_SH		=5,
	SPM_SH		=4,
	XOND_SH		=3,	
	RFLCE_SH	=2,
	CRCD_SH		=1,
	PADD_SH		=0		
};

enum TransmitReceiveParameterMasks{
	HDRS_MSK =0xf,
	TX_RX_PARM_MSK =0x1
	};
	
/*

Transmit Threshold Register
Mnemonic: TX_TSHD
Address: 28H -29H

The transmission threshold register handles the packet transmission threshold.

Bit Name	Bit #	Bit Description
			15:8	RESERVED
TSD			7:0		Packet transmission threshold.
					Packet transmission starts when the amount of data in the transmit FIFO is larger than or
					equal to the threshold, or the entire packet enters onto the FIFO. The effective threshold in
					bytes is equal to 8  TSD + 4. If the sum of the effective serial threshold and the effective
					parallel threshold bring the system to a deadlock, the transmission can start before
					threshold is reached to prevent deadlock. (TSD > 1)

Access Rules Register access R/W
Value after reset 0003H

*/
	inline unsigned short GetTransmitThreshold()const { return TransmitThresholdF();};

	inline unsigned int GetTransmitThresholdTSD_SH()const { return ( TransmitThresholdF() >> TSD_SH) & TSD_MSK;};

	inline void SetTransmitThreshold(unsigned short val){ TransmitThresholdF( val );};

	inline void SetTransmitThresholdTSD_SH(){ TransmitThresholdF (TransmitThresholdF() | (TSD_MSK << TSD_SH));};



	inline unsigned int TransmitThresholdF()const {return _ControlRegs.GetTransmitThreshold();};
	inline void TransmitThresholdF(unsigned short val){_ControlRegs.SetTransmitThreshold(val);};

  
enum TransmitThresholdShifts{
	TSD_SH			=7		
};

enum TransmitThresholdMasks{
	TSD_MSK =0xff
	};
	
	  
/*
Transmit Flow-Control Pause Time Register
Mnemonic: PAUSE_TIME
Address: 2AH - 2BH

The transmit flow-control pause time register handles the time field used in the transmitted
flow-control packets.

Bit Name	Bit #	Bit Description
PSTM		15:0	Indicates the number of units of 512 bit time (512 ns), during which the remote mode
					cannot send packets. This field is inserted into the transmitted flow-control packets.

Access Rules Register access R/W
Value after reset 0000H

*/
	inline unsigned short GetTransmitFlowControlPauseTime()const { return TransmitFlowControlPauseTimeF();};

	inline void SetTransmitFlowControlPauseTime(unsigned short val){ TransmitFlowControlPauseTimeF( val );};


	inline unsigned int TransmitFlowControlPauseTimeF()const 
						{return _ControlRegs.GetTransmitFlowControlPauseTime();};
	inline void TransmitFlowControlPauseTimeF(unsigned short val)
						{_ControlRegs.SetTransmitFlowControlPauseTime(val);};
  
	  
/*

Maximum Packet Size Register
Mnemonic: PKT_MAX_SIZE
Address: 2CH - 2DH

This register controls the maximum allowed received packet size.

Bit Name	Bit #	Bit Description
MPS			15:0	Packets received with a longer size are treated as too long packets.

Access Rules Register access R/W
Value after reset 05EEH (1518 decimal)

*/

	inline unsigned short GetMaximumPacketSize()const { return MaximumPacketSizeF();};

	inline void SetMaximumPacketSize(unsigned short val){ MaximumPacketSizeF( val );};



	inline unsigned int MaximumPacketSizeF()const {return _ControlRegs.GetMaximumPacketSize();};
	inline void MaximumPacketSizeF(unsigned short val){_ControlRegs.SetMaximumPacketSize(val);};
  

	  
/*

Inter Packet Gap Value Register
Mnemonic: IPG_VAL
Address: 2EH - 2FH

This register controls the inter packet gap value between transmitted packets.

Bit Name	Bit		# Bit Description
1			5:5		RESERVED
IPG			4:0		Interpacket gap value in units of 8 bit time. This field must get a minimum value of 6H
				(48-bit time).
Access Rules Register access R/W
Value after reset 000CH (96-bit time)

*/

	inline unsigned short GetInterPacketGapValue()const { return InterPacketGapValueF();};

	//inline unsigned int GetInterPacketGapValueIPG()const { return ( InterPacketGapValueF() >> IPG_SH) & INT_PKT_MASK;};

	inline void SetInterPacketGapValue(unsigned short val){ InterPacketGapValueF( val );};

	//inline void SetInterPacketGapValueIPG(){ InterPacketGapValueF (InterPacketGapValueF() | (INT_PKT_MASK << IPG_SH));};



	inline unsigned int InterPacketGapValueF()const {return _ControlRegs.GetInterPacketGapValue();};
	inline void InterPacketGapValueF(unsigned short val){_ControlRegs.SetInterPacketGapValue(val);};
  

  
enum InterPacketGapValueShifts{
	IPG_SH			=4		
};

enum InterPacketGapValueMaskMasks{
	INT_PKT_MASK =0x1f
	};
	
	  
/*

Transmit Counter Overflow Mask Register
Mnemonic: TX_OV_MSK
Address: 38H - 3BH

The transmit counter overflow mask register enables masking of specific transmit counters
overflow interrupt. Masking a counter does not influence the setting of the counter overflow status
bit in TX_OV_STT.

Bit Name	Bit #	Bit Description
			31:20	RESERVED
TX_BD		19		Mask TX_OCT_BAD_CNT counter overflow.
TX_OK		18		Mask TX_OCT_OK_CNT counter overflow.
TX_ER		17		Mask TX_ERR_CNT counter overflow.
TX_PS		16		Mask TX_PAUSE_CNT counter overflow.
TX_1519		15		Mask TX_PKT_1519_CNT counter overflow.
TX_1024		14		Mask TX_PKT_1024_CNT counter overflow.
TX_512		13		Mask TX_PKT_512_CNT counter overflow.
TX_256		12		Mask TX_PKT_256_CNT counter overflow.
TX_128		11		Mask TX_PKT_128_CNT counter overflow.
TX_65		10		Mask TX_PKT_65_CNT counter overflow.
TX_64		9		Mask TX_PKT_64_CNT counter overflow.
			8:3		RESERVED
TX_BRD		2		Mask TX_BRD_OK_CNT counter overflow.
TX_MLT		1		Mask TX_MLT_OK_CNT counter overflow.
TX_UNI		0		Mask TX_UNI_OK_CNT counter overflow.

Access Rules Register access R/W
Value after reset 0000H

*/
	inline unsigned int GetTransmitCounterOverflowMask()const { return TransmitCounterOverflowMaskF();};

	inline unsigned int GetTransmitCounterOverflowMaskTX_BD()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_BD_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_OK()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_OK_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_ER()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_ER_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_PS()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_PS_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_1519()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_1519_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_1024()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_1024_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_512()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_512_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_256()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_256_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_128()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_128_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_65()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_65_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_64()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_64_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_BRD()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_BRD_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_MLT()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_MLT_M_SH) & TX_CNT_OVERMASK_MSK;};
	inline unsigned int GetTransmitCounterOverflowMaskTX_UNI()const 
						{ return ( TransmitCounterOverflowMaskF() >> TX_UNI_M_SH) & TX_CNT_OVERMASK_MSK;};


	inline void SetTransmitCounterOverflowMask(unsigned int val){ TransmitCounterOverflowMaskF( val );};

	inline void SetTransmitCounterOverflowMaskTX_BD()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_BD_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_OK()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_OK_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_ER()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_ER_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_PS()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_PS_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_1519()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_1519_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_1024()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_1024_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_512()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_512_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_256()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_256_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_128()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_128_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_65()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_65_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_64()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_64_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_BRD()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_BRD_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_MLT()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_MLT_M_SH));};
	inline void SetTransmitCounterOverflowMaskTX_UNI()
		{ TransmitCounterOverflowMaskF (TransmitCounterOverflowMaskF() | (TX_CNT_OVERMASK_MSK << TX_UNI_M_SH));};


	inline unsigned int TransmitCounterOverflowMaskF()const {return _ControlRegs.GetTransmitCounterOverflowMask();};
	inline void TransmitCounterOverflowMaskF(unsigned int val){_ControlRegs.SetTransmitCounterOverflowMask(val);};
    
enum TransmitCounterOverflowMaskShifts{
	TX_BD_M_SH		=19,		
	TX_OK_M_SH		=18,		
	TX_ER_M_SH		=17,		
	TX_PS_M_SH		=16,		
	TX_1519_M_SH	=15,		
	TX_1024_M_SH	=14,		
	TX_512_M_SH		=13,		
	TX_256_M_SH		=12,		
	TX_128_M_SH		=11,		
	TX_65_M_SH		=10,		
	TX_64_M_SH		=9,	
	TX_BRD_M_SH		=2,		
	TX_MLT_M_SH		=1,		
	TX_UNI_M_SH		=0		
};

enum TransmitCounterOverflowMaskMasks{
	TX_CNT_OVERMASK_MSK =0x1
	};
	
	  
/*


Receive Counter Overflow Mask Register
Mnemonic: RX_OV_MSK
Address: 3CH - 3FH

The receive counter overflow mask register enables masking of specific receive counters overflow
interrupt. Masking a counter does not influence the setting of the counter overflow status bit in
RX_OV_STT

Bit Name	Bit #	Bit Description
			31:23	RESERVED
RX_ER		22		Mask RX_GPCS_ERR_CNT counter overflow.
RX_FAL		21		Mask RX_FALS_CRS_CNT counter overflow.
RX_PS		20		Mask RX_PAUSE_CNT counter overflow.
RX_1519		19		Mask RX_PKT_1519_CNT counter overflow.
RX_1024		18		Mask RX_PKT_1024_CNT counter overflow.
RX_512		17		Mask RX_PKT_512_CNT counter overflow.
RX_256		16		Mask RX_PKT_256_CNT counter overflow.
RX_128		15		Mask RX_PKT_128_CNT counter overflow.
RX_65		14		Mask RX_PKT_65_CNT counter overflow.
RX_64		13		Mask RX_PKT_64_CNT counter overflow.
RX_L_CR		12		Mask RX_LONG_CRC_CNT counter overflow.
			11		RESERVED
RX_L_OK		10		Mask RX_LONG_OK_CNT counter overflow.
RX_N_CR		9		Mask RX_NORM_CRC_CNT counter overflow.
RX_BRD		8		Mask RX_BRD_OK_CNT counter overflow.
RX_MLT		7		Mask RX_MLT_OK_CN counter overflow.
RX_UNI		6		Mask RX_UNI_OK_CNT counter overflow.
RX_S_CR		5		Mask RX_SHORT_CRC_CNT counter overflow.
RX_S_OK		4		Mask RX_SHORT_OK_CNT counter overflow.
RX_OV		3		Mask RX_OVF_CNT counter overflow.
			2		RESERVED
RX_BD		1		Mask RX_OCT_BAD_CNT counter overflow.
RX_OK		0		Mask RX_OCT_OK_CNT counter overflow.


Access Rules Register access R/W
Value after reset 0000H



*/

	inline unsigned int GetReceiveCounterOverflowMask()const { return ReceiveCounterOverflowMaskF();};

	inline unsigned int GetReceiveCounterOverflowMaskRX_ER()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_ER_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_FAL()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_FAL_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_PS()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_PS_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_1519()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_1519_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_RX_1024()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_1024_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_RX_512()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_512_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_RX_256()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_256_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_128()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_128_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_65()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_65_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_64()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_64_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_L_CR()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_L_CR_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_L_OK()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_L_OK_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_N_CR()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_N_CR_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_BRD()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_BRD_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_MLT()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_MLT_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_UNI()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_UNI_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_S_CR()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_S_CR_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_S_OK()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_S_OK_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_OV()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_OV_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_BD()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_BD_M_SH) & RX_CNT_OVERMASK_MSK;};
	inline unsigned int GetReceiveCounterOverflowMaskRX_OK()const 
						{ return ( ReceiveCounterOverflowMaskF() >> RX_OK_M_SH) & RX_CNT_OVERMASK_MSK;};
	
	inline void SetReceiveCounterOverflowMask(unsigned int val){ ReceiveCounterOverflowMaskF( val );};

	inline void SetReceiveCounterOverflowMaskRX_ER()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_ER_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_FAL()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_FAL_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_PS()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_PS_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_1519()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_1519_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_1024()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_1024_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_512()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_512_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_256()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_256_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_128()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_128_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_65()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_65_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_64()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_64_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_L_CR()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_L_CR_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_L_OK()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_L_OK_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_N_CR()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_N_CR_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_BRD()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_BRD_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_MLT()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_MLT_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_UNI()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_UNI_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_S_CR()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_S_CR_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_S_OK()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_S_OK_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_OV()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_OV_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_BD()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_BD_M_SH));};
	inline void SetReceiveCounterOverflowMaskRX_OK()
		{ ReceiveCounterOverflowMaskF (ReceiveCounterOverflowMaskF() | (RX_CNT_OVERMASK_MSK << RX_OK_M_SH));};

	inline unsigned int ReceiveCounterOverflowMaskF()const {return _ControlRegs.GetReceiveCounterOverflowMask();};
	inline void ReceiveCounterOverflowMaskF(unsigned int val){_ControlRegs.SetReceiveCounterOverflowMask(val);};
    
enum ReceiveCounterOverflowMaskShifts{
	RX_ER_M_SH		=22,		
	RX_FAL_M_SH		=21,	
	RX_PS_M_SH		=20,	
	RX_1519_M_SH	=19,		
	RX_1024_M_SH	=18,		
	RX_512_M_SH		=17,		
	RX_256_M_SH		=16,		
	RX_128_M_SH		=15,		
	RX_65_M_SH		=14,		
	RX_64_M_SH		=13,		
	RX_L_CR_M_SH	=12,		
	RX_L_OK_M_SH	=10,		
	RX_N_CR_M_SH	=9,		
	RX_BRD_M_SH		=8,		
	RX_MLT_M_SH		=7,		
	RX_UNI_M_SH		=6,		
	RX_S_CR_M_SH	=5,		
	RX_S_OK_M_SH	=4,		
	RX_OV_M_SH		=3,		
	RX_BD_M_SH		=1,		
	RX_OK_M_SH		=0		
};

enum ReceiveCounterOverflowMaskMasks{
	RX_CNT_OVERMASK_MSK =0x1
	};
GigMacControlRegs & _ControlRegs;
};



#endif //GIG_MAC_RX_REGS 
