/* RmonEtherStats.h
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                  
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: BigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: RMONETST.H $
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:49p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 * 
 * 
 * ---------------------------------------------------------------------
 */

#ifndef GIG_MAC_RMON_ETHER_STATS
#define GIG_MAC_RMON_ETHER_STATS

#include "GigNetStat.h"

class RmonEtherStats{


public:
	RmonEtherStats(GigMacNetStats & stats):_MacNetStats(stats){
		//cout << "RmonEtherStats CTOR" << endl;
	};

	~RmonEtherStats(){};

	inline RmonEtherStats * GetEtherCounters()			{return this;};

	inline void UpdateEtherCounters() {
			
		CalculateEtherStatsDropEvents();
		CalculateEtherStatsOctets();
		CalculateEtherStatsPkts();
		CalculateEtherStatsBroadcastPkts();	
		CalculateEtherStatsMulticastPkts();	
		CalculateEtherStatsCRCAlignErrors();	
		CalculateEtherStatsUndersizePkts();	
		CalculateEtherStatsOversizePkts();	
		CalculateEtherStatsFragments();		
		CalculateEtherStatsJabber();			
		CalculateEtherStatsPkts64Octets();		
		CalculateEtherStatsPkts65to127Octets();	
		CalculateEtherStatsPkts128to255Octets();	
		CalculateEtherStatsPkts256to511Octets();	
		CalculateEtherStatsPkts512to1023Octets();
		CalculateEtherStatsPkts1024to1518Octets();
	};

	inline unsigned int GetEtherStatsDropEvents()		{return etherStatsDropEvents;};
	inline unsigned int GetEtherStatsOctets()			{return etherStatsOctets;};
	inline unsigned int GetEtherStatsPkts()				{return etherStatsPkts;};

	inline unsigned int GetEtherStatsBroadcastPkts()	{return etherStatsBroadcastPkts;};
	inline unsigned int GetEtherStatsMulticastPkts()	{return etherStatsMulticastPkts;};
	inline unsigned int GetEtherStatsCRCAlignErrors()	{return etherStatsCRCAlignErrors;};
	inline unsigned int GetEtherStatsUndersizePkts()	{return etherStatsUndersizePkts;};
	inline unsigned int GetEtherStatsOversizePkts()		{return etherStatsOversizePkts;};
	inline unsigned int GetEtherStatsFragments()		{return etherStatsFragments;};
	inline unsigned int GetEtherStatsJabber()			{return etherStatsJabber;};

	inline unsigned int GetEtherStatsPkts64Octets()			{return etherStatsPkts64Octets;};
	inline unsigned int GetEtherStatsPkts65to127Octets()	{return etherStatsPkts65to127Octets;};
	inline unsigned int GetEtherStatsPkts128to255Octets()	{return etherStatsPkts128to255Octets;};
	inline unsigned int GetEtherStatsPkts256to511Octets()	{return etherStatsPkts256to511Octets;};
	inline unsigned int GetEtherStatsPkts512to1023Octets()	{return etherStatsPkts512to1023Octets;};
	inline unsigned int GetEtherStatsPkts1024to1518Octets()	{return etherStatsPkts1024to1518Octets;};

	
	inline void CalculateEtherStatsDropEvents()	{ etherStatsDropEvents = _MacNetStats.GetRxOvfCnt();};
	inline void CalculateEtherStatsOctets()		{ etherStatsOctets = _MacNetStats.GetTxOctOkCnt() +
																	 _MacNetStats.GetTxOctBadCnt() +
																	 _MacNetStats.GetRxOctOkCnt() +
																	 _MacNetStats.GetRxOctBadCnt();};
	inline void CalculateEtherStatsPkts(){
								CalculateEtherStatsBroadcastPkts();
								CalculateEtherStatsMulticastPkts();
								CalculateEtherStatsCRCAlignErrors();
								CalculateEtherStatsUndersizePkts();
								CalculateEtherStatsFragments();
								CalculateEtherStatsOversizePkts();
								CalculateEtherStatsJabber();
								etherStatsPkts = _MacNetStats.GetTxUniOkCnt() +
											     _MacNetStats.GetRxUniOkCnt() +
												 etherStatsBroadcastPkts +
												 etherStatsMulticastPkts +
												 etherStatsCRCAlignErrors +
												 etherStatsUndersizePkts +
												 etherStatsFragments +
												 etherStatsOversizePkts +
												 etherStatsJabber
								      ;};

	inline void CalculateEtherStatsBroadcastPkts()	{ etherStatsBroadcastPkts = _MacNetStats.GetTxBrdOkCnt() + _MacNetStats.GetRxBrdOkCnt();};
	inline void CalculateEtherStatsMulticastPkts()	{ etherStatsMulticastPkts = _MacNetStats.GetTxMltOkCnt() + _MacNetStats.GetRxMltOkCnt();};
	inline void CalculateEtherStatsCRCAlignErrors()	{ etherStatsCRCAlignErrors = 
																				 _MacNetStats.GetRxNormCrcCnt() +
																				 _MacNetStats.GetTxErrCnt();};
	inline void CalculateEtherStatsUndersizePkts()	{ etherStatsUndersizePkts = _MacNetStats.GetRxShortOkCnt();};
	inline void CalculateEtherStatsOversizePkts()	{ etherStatsOversizePkts = _MacNetStats.GetRxLongOkCnt();};
	inline void CalculateEtherStatsFragments()		{ etherStatsFragments = _MacNetStats.GetRxShortCrcCnt() ;};
	inline void CalculateEtherStatsJabber()			{ etherStatsJabber = _MacNetStats.GetRxNormCrcCnt();};


	inline void CalculateEtherStatsPkts64Octets()		{ etherStatsPkts64Octets = _MacNetStats.GetTxPkt64Cnt() + _MacNetStats.GetRxPkt64Cnt();};
	inline void CalculateEtherStatsPkts65to127Octets()	{ etherStatsPkts65to127Octets = _MacNetStats.GetTxPkt65Cnt() + _MacNetStats.GetRxPkt65Cnt();};
	inline void CalculateEtherStatsPkts128to255Octets()	{ etherStatsPkts128to255Octets = _MacNetStats.GetTxPkt128Cnt() + _MacNetStats.GetRxPkt128Cnt();};
	inline void CalculateEtherStatsPkts256to511Octets()	{ etherStatsPkts256to511Octets = _MacNetStats.GetTxPkt256Cnt() + _MacNetStats.GetRxPkt256Cnt();};
	inline void CalculateEtherStatsPkts512to1023Octets(){ etherStatsPkts512to1023Octets = _MacNetStats.GetTxPkt512Cnt() + _MacNetStats.GetRxPkt512Cnt();};
	inline void CalculateEtherStatsPkts1024to1518Octets(){ etherStatsPkts1024to1518Octets = _MacNetStats.GetTxPkt1024Cnt() + _MacNetStats.GetRxPkt1024Cnt();};




	inline void Show(){
cout 
		<< "EtherStatsDropEvents           " << etherStatsDropEvents << endl
		<< "EtherStatsOctets               " << etherStatsOctets << endl
		<< "EtherStatsPkts                 " << etherStatsPkts << endl
		<< "EtherStatsBroadcastPkts        " << etherStatsBroadcastPkts << endl	
		<< "EtherStatsMulticastPkts        " << etherStatsMulticastPkts << endl	
		<< "EtherStatsCRCAlignErrors       " << etherStatsCRCAlignErrors << endl	
		<< "EtherStatsUndersizePkts        " << etherStatsUndersizePkts << endl	
		<< "EtherStatsOversizePkts         " << etherStatsOversizePkts << endl	
		<< "EtherStatsFragments            " << etherStatsFragments << endl		
		<< "EtherStatsJabber               " << etherStatsJabber << endl			
		<< "EtherStatsPkts64Octets         " << etherStatsPkts64Octets << endl		
		<< "EtherStatsPkts65to127Octets    " << etherStatsPkts65to127Octets << endl	
		<< "EtherStatsPkts128to255Octets   " << etherStatsPkts128to255Octets << endl	
		<< "EtherStatsPkts256to511Octets   " << etherStatsPkts256to511Octets << endl	
		<< "EtherStatsPkts512to1023Octets  " << etherStatsPkts512to1023Octets << endl
		<< "EtherStatsPkts1024to1518Octets " << etherStatsPkts1024to1518Octets << endl;

	};


public:

//lets see later if int is not long enought

//RMON Statistic Counters 21440 Statistic Counters
unsigned int etherStatsDropEvents;			//RX_OVF_CNT
unsigned int etherStatsOctets;				//TX_OCT_OK_CNT + TX_OCT_BAD_CNT + RX_OCT_OK_CNT + RX_OCT_BAD_CNT
unsigned int etherStatsPkts;				//TX_UNI_OK_CNT + TX_LCOL_CNT + RX_UNI_OK_CNT + etherStatsBroadcastPkts +
											//etherStatsMulticastPkts + etherStatsCRCAlignErrors + etherStatsUndersizePkts 
											//+ etherStatsFragments + etherStatsOversizePkts + etherStatsJabber

unsigned int etherStatsBroadcastPkts;		//TX_BRD_OK_CNT + RX_BRD_OK_CNT		
unsigned int etherStatsMulticastPkts;		//TX_MLT_OK_CNT + RX_MLT_OK_CNT
unsigned int etherStatsCRCAlignErrors;		//RX_NORM_ALI_CNT + RX_NORM_CRC_CNT + TX_ERR_CNT
unsigned int etherStatsUndersizePkts;		//RX_SHORT_OK_CNT
unsigned int etherStatsOversizePkts;		//RX_LONG_OK_CNT
unsigned int etherStatsFragments;			//RX_SHORT_CRC_CNT + TX_COL_CNT
unsigned int etherStatsJabber;				//RX_LONG_CRC_CNT

unsigned int etherStatsPkts64Octets;		//TX_PKT_64_CNT + RX_PKT_64_CNT
unsigned int etherStatsPkts65to127Octets;	//TX_PKT_65_CNT + RX_PKT_65_CNT
unsigned int etherStatsPkts128to255Octets;	//TX_PKT_128_CNT + RX_PKT_128_CNT
unsigned int etherStatsPkts256to511Octets;	//TX_PKT_256_CNT + RX_PKT_256_CNT
unsigned int etherStatsPkts512to1023Octets;	//TX_PKT_512_CNT + RX_PKT_512_CNT
unsigned int etherStatsPkts1024to1518Octets;//TX_PKT_1024_CNT + RX_PKT_1024_CNT
//etherStatsDropEvents			RX_OVF_CNT
//etherStatsOctets				TX_OCT_OK_CNT + TX_OCT_BAD_CNT + RX_OCT_OK_CNT + RX_OCT_BAD_CNT

private:
GigMacNetStats & _MacNetStats;

};




#endif //GIG_MAC_RMON_ETHER_STATS 


/*

RFC 1757             Remote Network Monitoring MIB         February 1995



          -- The Ethernet Statistics Group
          --
          -- Implementation of the Ethernet Statistics group is
          -- optional.
          --
          -- The ethernet statistics group contains statistics
          -- measured by the probe for each monitored interface on
          -- this device.  These statistics take the form of free
          -- running counters that start from zero when a valid entry
          -- is created.
          --
          -- This group currently has statistics defined only for
          -- Ethernet interfaces.  Each etherStatsEntry contains
          -- statistics for one Ethernet interface.  The probe must
          -- create one etherStats entry for each monitored Ethernet
          -- interface on the device.

          etherStatsTable OBJECT-TYPE
              SYNTAX SEQUENCE OF EtherStatsEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A list of Ethernet statistics entries."
              ::= { statistics 1 }


          etherStatsEntry OBJECT-TYPE
              SYNTAX EtherStatsEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A collection of statistics kept for a particular
                  Ethernet interface.  As an example, an instance of the
                  etherStatsPkts object might be named etherStatsPkts.1"
              INDEX { etherStatsIndex }
              ::= { etherStatsTable 1 }

          EtherStatsEntry ::= SEQUENCE {
              etherStatsIndex                    INTEGER (1..65535),
              etherStatsDataSource               OBJECT IDENTIFIER,
              etherStatsDropEvents               Counter,
              etherStatsOctets                   Counter,
              etherStatsPkts                     Counter,
              etherStatsBroadcastPkts            Counter,
              etherStatsMulticastPkts            Counter,
              etherStatsCRCAlignErrors           Counter,
              etherStatsUndersizePkts            Counter,
              etherStatsOversizePkts             Counter,
              etherStatsFragments                Counter,
              etherStatsJabbers                  Counter,
              etherStatsCollisions               Counter,
              etherStatsPkts64Octets             Counter,
              etherStatsPkts65to127Octets        Counter,
              etherStatsPkts128to255Octets       Counter,
              etherStatsPkts256to511Octets       Counter,
              etherStatsPkts512to1023Octets      Counter,
              etherStatsPkts1024to1518Octets     Counter,
              etherStatsOwner                    OwnerString,
              etherStatsStatus                   EntryStatus
          }

          etherStatsIndex OBJECT-TYPE
              SYNTAX INTEGER (1..65535)
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The value of this object uniquely identifies this
                  etherStats entry."
              ::= { etherStatsEntry 1 }

          etherStatsDataSource OBJECT-TYPE
              SYNTAX OBJECT IDENTIFIER
              ACCESS read-write
              STATUS mandatory


              DESCRIPTION
                  "This object identifies the source of the data that
                  this etherStats entry is configured to analyze.  This
                  source can be any ethernet interface on this device.
                  In order to identify a particular interface, this
                  object shall identify the instance of the ifIndex
                  object, defined in RFC 1213 and RFC 1573 [4,6], for
                  the desired interface.  For example, if an entry
                  were to receive data from interface #1, this object
                  would be set to ifIndex.1.

                  The statistics in this group reflect all packets
                  on the local network segment attached to the
                  identified interface.

                  An agent may or may not be able to tell if
                  fundamental changes to the media of the interface
                  have occurred and necessitate an invalidation of
                  this entry.  For example, a hot-pluggable ethernet
                  card could be pulled out and replaced by a
                  token-ring card.  In such a case, if the agent has
                  such knowledge of the change, it is recommended that
                  it invalidate this entry.

                  This object may not be modified if the associated
                  etherStatsStatus object is equal to valid(1)."
              ::= { etherStatsEntry 2 }

          etherStatsDropEvents OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of events in which packets
                  were dropped by the probe due to lack of resources.
                  Note that this number is not necessarily the number of
                  packets dropped; it is just the number of times this
                  condition has been detected."
              ::= { etherStatsEntry 3 }

          etherStatsOctets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of octets of data (including
                  those in bad packets) received on the
                  network (excluding framing bits but including
                  FCS octets).

                  This object can be used as a reasonable estimate of
                  ethernet utilization.  If greater precision is
                  desired, the etherStatsPkts and etherStatsOctets
                  objects should be sampled before and after a common
                  interval.  The differences in the sampled values are
                  Pkts and Octets, respectively, and the number of
                  seconds in the interval is Interval.  These values
                  are used to calculate the Utilization as follows:

                                   Pkts * (9.6 + 6.4) + (Octets * .8)
                   Utilization = -------------------------------------
                                           Interval * 10,000

                  The result of this equation is the value Utilization
                  which is the percent utilization of the ethernet
                  segment on a scale of 0 to 100 percent."
              ::= { etherStatsEntry 4 }

          etherStatsPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets (including bad packets,
                  broadcast packets, and multicast packets) received."
              ::= { etherStatsEntry 5 }

          etherStatsBroadcastPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of good packets received that were
                  directed to the broadcast address.  Note that this
                  does not include multicast packets."
              ::= { etherStatsEntry 6 }

          etherStatsMulticastPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of good packets received that were
                  directed to a multicast address.  Note that this
                  number does not include packets directed to the
                  broadcast address."
              ::= { etherStatsEntry 7 }

          etherStatsCRCAlignErrors OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets received that
                  had a length (excluding framing bits, but
                  including FCS octets) of between 64 and 1518
                  octets, inclusive, but but had either a bad
                  Frame Check Sequence (FCS) with an integral
                  number of octets (FCS Error) or a bad FCS with
                  a non-integral number of octets (Alignment Error)."
              ::= { etherStatsEntry 8 }

          etherStatsUndersizePkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets received that were
                  less than 64 octets long (excluding framing bits,
                  but including FCS octets) and were otherwise well
                  formed."
              ::= { etherStatsEntry 9 }

          etherStatsOversizePkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets received that were
                  longer than 1518 octets (excluding framing bits,
                  but including FCS octets) and were otherwise
                  well formed."
              ::= { etherStatsEntry 10 }

          etherStatsFragments OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets received that were less
                  than 64 octets in length (excluding framing bits but
                  including FCS octets) and had either a bad Frame
                  Check Sequence (FCS) with an integral number of
                  octets (FCS Error) or a bad FCS with a non-integral
                  number of octets (Alignment Error).
                  Note that it is entirely normal for
                  etherStatsFragments to increment.  This is because
                  it counts both runts (which are normal occurrences
                  due to collisions) and noise hits."
              ::= { etherStatsEntry 11 }

          etherStatsJabbers OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets received that were
                  longer than 1518 octets (excluding framing bits,
                  but including FCS octets), and had either a bad
                  Frame Check Sequence (FCS) with an integral number
                  of octets (FCS Error) or a bad FCS with a
                  non-integral number of octets (Alignment Error).

                  Note that this definition of jabber is different
                  than the definition in IEEE-802.3 section 8.2.1.5
                  (10BASE5) and section 10.3.1.4 (10BASE2).  These
                  documents define jabber as the condition where any
                  packet exceeds 20 ms.  The allowed range to detect
                  jabber is between 20 ms and 150 ms."
              ::= { etherStatsEntry 12 }

          etherStatsCollisions OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The best estimate of the total number of collisions
                  on this Ethernet segment.

                  The value returned will depend on the location of
                  the RMON probe. Section 8.2.1.3 (10BASE-5) and
                  section 10.3.1.3 (10BASE-2) of IEEE standard 802.3
                  states that a station must detect a collision, in
                  the receive mode, if three or more stations are
                  transmitting simultaneously.  A repeater port must
                  detect a collision when two or more stations are
                  transmitting simultaneously.  Thus a probe placed on
                  a repeater port could record more collisions than a
                  probe connected to a station on the same segment
                  would.
                  Probe location plays a much smaller role when
                  considering 10BASE-T.  14.2.1.4 (10BASE-T) of IEEE
                  standard 802.3 defines a collision as the
                  simultaneous presence of signals on the DO and RD
                  circuits (transmitting and receiving at the same
                  time).  A 10BASE-T station can only detect
                  collisions when it is transmitting.  Thus probes
                  placed on a station and a repeater, should report
                  the same number of collisions.

                  Note also that an RMON probe inside a repeater
                  should ideally report collisions between the
                  repeater and one or more other hosts (transmit
                  collisions as defined by IEEE 802.3k) plus receiver
                  collisions observed on any coax segments to which
                  the repeater is connected."
              ::= { etherStatsEntry 13 }

          etherStatsPkts64Octets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets (including bad
                  packets) received that were 64 octets in length
                  (excluding framing bits but including FCS octets)."
              ::= { etherStatsEntry 14 }

          etherStatsPkts65to127Octets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets (including bad
                  packets) received that were between
                  65 and 127 octets in length inclusive
                  (excluding framing bits but including FCS octets)."
              ::= { etherStatsEntry 15 }

          etherStatsPkts128to255Octets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets (including bad
                  packets) received that were between
                  128 and 255 octets in length inclusive
                  (excluding framing bits but including FCS octets)."
              ::= { etherStatsEntry 16 }

          etherStatsPkts256to511Octets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets (including bad
                  packets) received that were between
                  256 and 511 octets in length inclusive
                  (excluding framing bits but including FCS octets)."
              ::= { etherStatsEntry 17 }

          etherStatsPkts512to1023Octets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets (including bad
                  packets) received that were between
                  512 and 1023 octets in length inclusive
                  (excluding framing bits but including FCS octets)."
              ::= { etherStatsEntry 18 }

          etherStatsPkts1024to1518Octets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The total number of packets (including bad
                  packets) received that were between
                  1024 and 1518 octets in length inclusive
                  (excluding framing bits but including FCS octets)."
              ::= { etherStatsEntry 19 }

          etherStatsOwner OBJECT-TYPE
              SYNTAX OwnerString
              ACCESS read-write
              STATUS mandatory
              DESCRIPTION
                  "The entity that configured this entry and is
                  therefore using the resources assigned to it."
              ::= { etherStatsEntry 20 }

          etherStatsStatus OBJECT-TYPE
              SYNTAX EntryStatus
              ACCESS read-write
              STATUS mandatory
              DESCRIPTION
                  "The status of this etherStats entry."
              ::= { etherStatsEntry 21 }



*/
