/* RmonHostStats.h
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS                  
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: BigMacDriver
 * author: sk, Jan 99 Initial version
 * $History: RMONHTST.H $
 * 
 * *****************  Version 3  *****************
 * User: Skornfel     Date: 4/28/99    Time: 3:49p
 * Updated in $/dev_1200/SA1_CoreLibs/GigaMAC_GC82547EB
 * FIRST VERSION OF THE DRIVER 
 * lightly tested on NT 
 * 
 * ---------------------------------------------------------------------
 */

#ifndef GIG_MAC_RMON_HOST_STATS
#define GIG_MAC_RMON_HOST_STATS

#include "GigNetStat.h"

class RmonHostStats{


public:
	RmonHostStats(GigMacNetStats & stats):_MacNetStats(stats){
		//cout << "RmonHostStats CTOR" << endl;
	};

	~RmonHostStats(){};

	inline RmonHostStats * GetHostCounters()			{return this;};

	inline void UpdateHostCounters() {
		CalculateHostInPkts();
		CalculateHostOutPkts(); 
		CalculateHostInOctets(); 
		CalculateHostOutOctets();
		CalculateHostOutErrors(); 
		CalculateHostOutBroadcastPkts(); 
		CalculateHostOutMulticastPkts(); 
	};

	inline unsigned int GetHostInPkts()				{return hostInPkts;};
	inline unsigned int GetHostOutPkts()			{return hostOutPkts;};
	inline unsigned int GetHostInOctets()			{return hostInOctets;};
	inline unsigned int GetHostOutOctets()			{return hostOutOctets;};
	inline unsigned int GetHostOutErrors()			{return hostOutErrors;};
	inline unsigned int GetHostOutBroadcastPkts()	{return hostOutBroadcastPkts;};
	inline unsigned int GetHostOutMulticastPkts()	{return hostOutMulticastPkts;};


	inline void CalculateHostInPkts(){
					//TX_UNI_OK_CNT + TX_MLT_OK_CNT + TX_BRD_OK_CNT
					hostInPkts = _MacNetStats.GetTxUniOkCnt() +
								 _MacNetStats.GetTxMltOkCnt() +
								 _MacNetStats.GetTxBrdOkCnt();
	};

	inline void CalculateHostOutPkts(){ 
					//RX_UNI_OK_CNT + RX_MLT_OK_CNT + RX_BRD_OK_CNT+ RX_NORM_ALI_CNT +
					//RX_NORM_CRC_CNT + RX_SHORT_OK_CNT +
					//RX_SHORT_CRC_CNT + RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT
					hostOutPkts = _MacNetStats.GetRxUniOkCnt() +
								  _MacNetStats.GetRxMltOkCnt() +
								  _MacNetStats.GetRxBrdOkCnt() +
								  _MacNetStats.GetRxNormCrcCnt() +
								  _MacNetStats.GetRxShortOkCnt() +
								  _MacNetStats.GetRxShortCrcCnt() +
								  _MacNetStats.GetRxLongOkCnt() +
								  _MacNetStats.GetRxLongCrcCnt(); 
	};
	
	inline void CalculateHostInOctets(){ 
					//TX_OCT_OK_CNT
					hostInOctets =_MacNetStats.GetTxOctOkCnt();
	};
	
	inline void CalculateHostOutOctets(){
					//RX_OCT_OK_CNT + RX_OCT_BAD_CNT
					hostOutOctets = _MacNetStats.GetRxOctOkCnt() +
									_MacNetStats.GetRxOctBadCnt();
	};
	
	inline void CalculateHostOutErrors()		{ 
					//RX_NORM_ALI_CNT + RX_NORM_CRC_CNT + RX_SHORT_OK_CNT + RX_SHORT_CRC_CNT + 
					//RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT 
					hostOutErrors = 
									_MacNetStats.GetRxNormCrcCnt() + 
									_MacNetStats.GetRxShortOkCnt() +
									_MacNetStats.GetRxShortCrcCnt() +
									_MacNetStats.GetRxLongOkCnt() +
									_MacNetStats.GetRxLongCrcCnt(); 

	
	};
	
	inline void CalculateHostOutBroadcastPkts()	{ 
					//RX_BRD_OK_CNT
					hostOutBroadcastPkts = _MacNetStats.GetRxBrdOkCnt();
	};
	
	inline void CalculateHostOutMulticastPkts()	{ 
					//RX_MLT_OK_CNT
					hostOutMulticastPkts = _MacNetStats.GetRxMltOkCnt();
	};

	inline void Show(){

cout 

		<< "hostInPkts            " << hostInPkts   << endl				
		<< "hostOutPkts           " << hostOutPkts  << endl					
		<< "hostInOctets          " << hostInOctets   << endl					
		<< "hostOutOctets         " << hostOutOctets   << endl					
		<< "hostOutErrors         " << hostOutErrors  << endl				
		<< "hostOutBroadcastPkts  " << hostOutBroadcastPkts   << endl			
		<< "hostOutMulticastPkts  " << hostOutMulticastPkts   << endl;	

	};

public:

//lets see later if int is not long enought

//------------------------------------------------------------------------------------------------------------------
//RMON Host Counters 21440 Statistic Counters

//RMON Host Group Support
//Describes the 21440 support of the RMON host group when a single node is
//connected to a port. If packets are loaded into the 21440 to be transmitted as bad
//packets, they must be counted in the error counters, too.

unsigned int hostInPkts;				//TX_UNI_OK_CNT + TX_MLT_OK_CNT + TX_BRD_OK_CNT
unsigned int hostOutPkts;				//RX_UNI_OK_CNT + RX_MLT_OK_CNT + RX_BRD_OK_CNT+ RX_NORM_ALI_CNT +
										//RX_NORM_CRC_CNT + RX_SHORT_OK_CNT +
										//RX_SHORT_CRC_CNT + RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT
unsigned int hostInOctets;				//TX_OCT_OK_CNT
unsigned int hostOutOctets;				//RX_OCT_OK_CNT + RX_OCT_BAD_CNT
unsigned int hostOutErrors;				//RX_NORM_ALI_CNT + RX_NORM_CRC_CNT + RX_SHORT_OK_CNT + RX_SHORT_CRC_CNT + 
										//RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT 
unsigned int hostOutBroadcastPkts;		//RX_BRD_OK_CNT
unsigned int hostOutMulticastPkts;		//RX_MLT_OK_CNT


private:
GigMacNetStats & _MacNetStats;



};




#endif //GIG_MAC_RMON_HOST_STATS 





/*         HostEntry ::= SEQUENCE {
              hostAddress             OCTET STRING,
              hostCreationOrder       INTEGER (1..65535),
              hostIndex               INTEGER (1..65535),
              hostInPkts              Counter,
              hostOutPkts             Counter,
              hostInOctets            Counter,
              hostOutOctets           Counter,
              hostOutErrors           Counter,
              hostOutBroadcastPkts    Counter,
              hostOutMulticastPkts    Counter
//----------------------------------------------------------------
//---------------------------------------------------------------
hostEntry OBJECT-TYPE
              SYNTAX HostEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A collection of statistics for a particular host
                  that has been discovered on an interface of this
                  device."
              INDEX { hostIndex, hostAddress }
              ::= { hostTable 1 }

          


HostEntry ::= SEQUENCE {
              hostAddress             OCTET STRING,
              hostCreationOrder       INTEGER (1..65535),
              hostIndex               INTEGER (1..65535),
              hostInPkts              Counter,
              hostOutPkts             Counter,
              hostInOctets            Counter,
              hostOutOctets           Counter,
              hostOutErrors           Counter,
              hostOutBroadcastPkts    Counter,
              hostOutMulticastPkts    Counter
          }

          


hostAddress OBJECT-TYPE
              SYNTAX OCTET STRING
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The physical address of this host."
              ::= { hostEntry 1 }

          


hostCreationOrder OBJECT-TYPE
              SYNTAX INTEGER (1..65535)
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "An index that defines the relative ordering of
                  the creation time of hosts captured for a
                  particular hostControlEntry.  This index shall
                  be between 1 and N, where N is the value of
                  the associated hostControlTableSize.  The ordering
                  of the indexes is based on the order of each entry's
                  insertion into the table, in which entries added
                  earlier have a lower index value than entries added
                  later.

                  It is important to note that the order for a
                  particular entry may change as an (earlier) entry
                  is deleted from the table.  Because this order may
                  change, management stations should make use of the
                  hostControlLastDeleteTime variable in the
                  hostControlEntry associated with the relevant
                  portion of the hostTable.  By observing
                  this variable, the management station may detect
                  the circumstances where a previous association
                  between a value of hostCreationOrder
                  and a hostEntry may no longer hold."
              ::= { hostEntry 2 }

          


hostIndex OBJECT-TYPE
              SYNTAX INTEGER (1..65535)
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The set of collected host statistics of which
                  this entry is a part.  The set of hosts
                  identified by a particular value of this
                  index is associated with the hostControlEntry
                  as identified by the same value of hostControlIndex."
              ::= { hostEntry 3 }

          


hostInPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of packets without errors transmitted to
                  this address since it was added to the hostTable."
              ::= { hostEntry 4 }

          


hostOutPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of packets including errors transmitted
                  by this address since it was added to the hostTable."
              ::= { hostEntry 5 }

          


hostInOctets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of octets transmitted to this address
                  since it was added to the hostTable (excluding
                  framing bits but including FCS octets), except for
                  those octets in packets that contained errors."
              ::= { hostEntry 6 }

          


hostOutOctets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of octets transmitted by this address
                  since it was added to the hostTable (excluding
                  framing bits but including FCS octets), including
                  those octets in packets that contained errors."
              ::= { hostEntry 7 }

          


hostOutErrors OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of error packets transmitted by this
                  address since this host was added to the hostTable."
              ::= { hostEntry 8 }

          


hostOutBroadcastPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of good packets transmitted by this
                  address that were directed to the broadcast address
                  since this host was added to the hostTable."
              ::= { hostEntry 9 }

          


hostOutMulticastPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of good packets transmitted by this
                  address that were directed to a multicast address
                  since this host was added to the hostTable.
                  Note that this number does not include packets
                  directed to the broadcast address."
              ::= { hostEntry 10 }


          -- host Time Table

          


hostTimeTable OBJECT-TYPE
              SYNTAX SEQUENCE OF HostTimeEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A list of time-ordered host table entries."
              ::= { hosts 3 }

          


hostTimeEntry OBJECT-TYPE
              SYNTAX HostTimeEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A collection of statistics for a particular host
                  that has been discovered on an interface of this
                  device.  This collection includes the relative
                  ordering of the creation time of this object."
              INDEX { hostTimeIndex, hostTimeCreationOrder }
              ::= { hostTimeTable 1 }

          

*/
