/* BigMacControlRegs.h
 * BigMacDriver SNMP Statistics file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: BigMacDriver
 * author: sk, Jun 98 Initial version
 * $History: BMCTRLRG.H $
 * 
 * *****************  Version 16  *****************
 * User: Dmwhite      Date: 4/05/00    Time: 2:55p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * no more  U_COS macro, changed to (OS==UCOS)
 * 
 * *****************  Version 13  *****************
 * User: Skornfel     Date: 9/21/99    Time: 3:51p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * MAC ports in to FULL duplex MODE
 * 
 * *****************  Version 12  *****************
 * User: Qabuild      Date: 8/20/99    Time: 5:00p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * Change reference to iostream::operator<<(void *) to (unsigned int)
 * since vxWorks doesn't seem to be able to print a pointer to void
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 7/13/99    Time: 12:50p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * Change UCOS to U_COS to avoid the conflict with UCOS=2 which is defined
 * in ostype.h
 * 
 * *****************  Version 9  *****************
 * User: Skornfel     Date: 6/24/99    Time: 3:53p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * 21440 ISV board support removed 
 * 
 * *****************  Version 8  *****************
 * User: Skornfel     Date: 6/09/99    Time: 8:03p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * runs on1200 eval board
 * supports two 21440 macs with 16 ports
 * in both non interrupt and interrupt modes   
 * 
 * *****************  Version 7  *****************
 * User: Skornfel     Date: 4/07/99    Time: 3:47p
 * Updated in $/development/BigMAC_21440
 * 1 adopted to ucos
 * 2 SemDispachEvent = new Semaphore(); insted of creating on the stack
 * 3 ISR algoritm changed to "OR" 
 * 
 * *****************  Version 5  *****************
 * User: Skornfel     Date: 2/12/99    Time: 3:44p
 * Updated in $/development/BigMAC_21440
 * BL5 implemented 
 * code still have QNX calls and   #ifdefs , but not tested on QNX
 * Tested on NT/VxWorks
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 1/27/99    Time: 11:40a
 * Updated in $/development/BigMAC_21440
 * just to move working directory to my new computer
 * 1.Some BL5 staff implemented
 * 2. VxWorks port
 * 3...
 * 
 * *****************  Version 2  *****************
 * User: Skornfel     Date: 12/02/98   Time: 5:07p
 * Updated in $/BL3_Release/BigMAC_21440
 * Base Level 4 driver - runs on QNX(21440) and NT(simulated)
 * 
 * *****************  Version 1  *****************
 * User: Skornfel     Date: 11/25/98   Time: 2:58p
 * Created in $/BL3_Release/BigMAC_21440
 * BigMAC files tested on 21440 under QNX
 * 
 * *****************  Version 1  *****************
 * User: Skornfel     Date: 8/10/98    Time: 10:06a
 * Created in $/base_level3/BigMAC
 * Initial version
 * 

 * ---------------------------------------------------------------------
 */

#ifndef BIG_MAC_CONTROL_REGS
#define BIG_MAC_CONTROL_REGS

#include "ostype.h"

class BigMacControlRegs{

public:
//friend
static void * operator new(size_t size, void * ptr){ return ptr; }
//friend
static void operator delete( void * ptr){ return; }


	BigMacControlRegs(){
		//cout << "BigMacControlRegs CTOR" << endl;
	};

	~BigMacControlRegs(){};



//--------------------------------------------------------------------------------------------------
inline unsigned char GetChipInterruptSummary()const {return ChipInterruptSummary;};					//R					
//Base Registers 
inline unsigned char GetInterruptStatus()/*const*/ {return (InterruptStatus & 0xff);};				//R					
inline unsigned char GetInterruptEnable()const {return (InterruptEnable & 0xff);};					//R W					
inline unsigned char GetTransmitStatus()const {return (TransmitStatus & 0xff);};					//R					
inline unsigned char GetReceiveStatus()const {return (ReceiveStatus & 0xff);};						//R					
inline unsigned char GetPortControl()const {return (PortControl & 0xff);};							//R W	
inline unsigned char GetDeviceIdentificationNumber()const {return (DeviceIdentificationNumber & 0xff);};//R			
inline unsigned char GetRevisionIdentificationNumber()const {return (RevisionIdentificationNumber & 0xff);};//R				
inline unsigned char GetSerialCommand()const {return (SerialCommand & 0xff);};						//R W				
//Configuration Registers
inline unsigned char GetFIFOThreshold()const {return (FIFOThreshold & 0xff);};						//R W					
inline unsigned char GetFIFOBusMode()const {return (FIFOBusMode & 0xff);};							//R W					
inline unsigned char GetTransmitParameters()const {return (TransmitParameters & 0xff);};			//R W					
inline unsigned char GetTransmitErrorMode()const {return (TransmitErrorMode & 0xff);};				//R W					
inline unsigned char GetTransmitPacketSendingThresholdAndBackoffLimit()const 
						{return (TransmitPacketSendingThresholdAndBackoffLimit & 0xff);};			//R W	
inline unsigned char GetReceiveParameters()const {return (ReceiveParameters & 0xff);};				//R W
inline unsigned char GetReceiveFilteringMode()const {return (ReceiveFilteringMode & 0xff);};		//R W
inline unsigned short int GetTransmitFlowControlPauseTime()const 
						{return ((TransmitFlowControlPauseTime1<<8) +TransmitFlowControlPauseTime);};	                            //R W
inline unsigned short int GetMaximumPacketSize()const 
						{return ((MaximumPacketSize1<< 8)+(MaximumPacketSize /*& 0xff*/));};						                    //R W
//Serial Registers 
inline unsigned char GetSerialMode()const {return (SerialMode & 0xff);};							//R W
inline unsigned char GetLinkStatus()const {return (LinkStatus & 0xff);};							//R
inline unsigned char GetPhysicalAddress()const {return (PhysicalAddress & 0xff);};					//R W

//-----------------------------------------------------------------------------------------------



//SK DEBUG/DEVELOPmnt
inline void SetChipInterruptSummary(unsigned char val){ ChipInterruptSummary = val;};	
inline void SetInterruptStatus(unsigned char val){ InterruptStatus = val;};		
//SK


		
//Base Registers 
				
inline void SetInterruptEnable(unsigned char val){ InterruptEnable = val;};							//R W					
inline void SetPortControl(unsigned char val){ PortControl = val;};									//R W	
inline void SetSerialCommand(unsigned char val){ SerialCommand = val;};								//R W				
//Configuration Registers
inline void SetFIFOThreshold(unsigned char val){ FIFOThreshold = val;};								//R W					
inline void SetFIFOBusMode(unsigned char val){ FIFOBusMode = val;};									//R W					
inline void SetTransmitParameters(unsigned char val){ TransmitParameters = val;};					//R W					
inline void SetTransmitErrorMode(unsigned char val){ TransmitErrorMode = val;};						//R W					
inline void SetTransmitPacketSendingThresholdAndBackoffLimit(unsigned char val)
				{TransmitPacketSendingThresholdAndBackoffLimit = val;};								//R W	
inline void SetReceiveParameters(unsigned char val){ ReceiveParameters = val;};						//R W
inline void SetReceiveFilteringMode(unsigned char val){ ReceiveFilteringMode = val;};				//R W
inline void SetTransmitFlowControlPauseTime(unsigned short int val){
		TransmitFlowControlPauseTime = val & 0xff;
		TransmitFlowControlPauseTime1 = (val & 0xff00) >> 8;
};																									//R W
inline void SetMaximumPacketSize(unsigned short int val){
		MaximumPacketSize = val & 0xff;
		MaximumPacketSize1 = (val & 0xff00)>>8 ;
};																									//R W
//Serial Registers 
inline void SetSerialMode(unsigned char val){ SerialMode = val;};									//R W
inline void SetPhysicalAddress(unsigned char val){ PhysicalAddress = val;};							//R W

//
void RegPrint(){
cout
#if (OS!=UCOS)
	 << hex << "----------------->" << endl 
#endif //UCOS
     << "----------------->" << endl 
//## <<" Base Registers  " << endl
//<<" InterruptStatus    " << (int)GetInterruptStatus() << endl
<<" InterruptEnable      " << (int)GetInterruptEnable() << endl													
//<<" TransmitStatus     " << (int)GetTransmitStatus() << endl												
//<<" ReceiveStatus      " << (int)GetReceiveStatus() << endl												
<<" PortControl          " << (int)GetPortControl() << endl 
<<" ReceiveFilteringMode " << (int)GetReceiveFilteringMode() << endl		
;

}


///

void Show(){
cout 
<< "##################" << (unsigned int)(&ChipInterruptSummary) << "####################" << endl

<<" ChipInterruptSummary         " << (int)(GetChipInterruptSummary()) << endl
//<<" Base Registers " << endl
<<" InterruptStatus              " << (int)GetInterruptStatus() << endl
<<" InterruptEnable              " << (int)GetInterruptEnable() << endl													
<<" TransmitStatus               " << (int)GetTransmitStatus() << endl												
<<" ReceiveStatus                " << (int)GetReceiveStatus() << endl												
<<" PortControl                  " << (int)GetPortControl() << endl										
<<" DeviceIdentNumber            " << (int)GetDeviceIdentificationNumber() << endl 									
<<" RevisionIdentNumber          " << (int)GetRevisionIdentificationNumber() << endl									
<<" SerialCommand                " << (int)GetSerialCommand() << endl													
//<<" Configuration Registers"  << endl
<<" FIFOThreshold                " << (int)GetFIFOThreshold() << endl													
<<" FIFOBusMode                  " << (int)GetFIFOBusMode() << endl															
<<" TransmitParameters           " << (int)GetTransmitParameters() << endl												
<<" TransmitErrorMode            " << (int)GetTransmitErrorMode() << endl													
<<" TransmitPacketSendingThresholdAndBackoffLimit   " 
				<< (int)GetTransmitPacketSendingThresholdAndBackoffLimit() << endl		
<<" ReceiveParameters            " << (int)GetReceiveParameters() << endl								
<<" ReceiveFilteringMode         " << (int)GetReceiveFilteringMode() << endl								
<<" TransmitFlowControlPauseTime " << (int)GetTransmitFlowControlPauseTime() << endl
<<" MaximumPacketSize            " << (int)GetMaximumPacketSize() << endl					
//<<" Serial Registers " << endl 
<<" SerialMode                   " << (int)GetSerialMode() << endl										
<<" LinkStatus                   " << (int)GetLinkStatus() << endl										
<<" PhysicalAddress              " << (int)GetPhysicalAddress() << endl;

};



//

/////SK private:
public:

/* 
Register Mapping lists each CSR register name, 
mnemonic, address offset, and the section that describes the register.

Register Description					Mnemonic		I/O		Section	
														Address
														Offset 
														
Base Registers ---
Chip interrupt summary register			CHIP_INT		00H		3.2.2.1
Interrupt status register				INT_STT			01H		3.2.2.2
Interrupt enable register				INT_EN			02H		3.2.2.3
Transmit status register				TXMT_STT		03H		3.2.2.4
Receive status register					RECV_STT		04H		3.2.2.5
Port control register					PORT_CTR		05H		3.2.2.6
Device identification number register	DEV_ID			06H		3.2.2.7
Revision identification number register REV_ID			07H		3.2.2.8
Serial command register					SER_COM			08H		3.2.2.9

Configuration Registers ---
FIFO threshold register					FFO_TSHD		11H		3.2.3.1
FIFO bus mode register					FFO_BUS			12H		3.2.3.2
Transmit parameters register			TX_PARAM		13H		3.2.3.3
Transmit error mode register			TX_ERR_MOD		14H		3.2.3.4
Transmit packet sending threshold and
backoff limit register					TX_TSHD_BOFF	15H		3.2.3.5
Receive parameters register				RX_PARAM		16H		3.2.3.6
Receive filtering mode register			RX_FILT_MOD		17H		3.2.3.7
Transmit flow-control pause time
register								PAUSE_TIME		18H-19H 3.2.3.8
Maximum packet size register			PKT_MAX_SIZE	1AH-1BH 3.2.3.9

Serial Registers ---
Serial mode register					SER_MOD			21H		3.2.4.1
Link status register					LNK_STT			22H		3.2.4.2
Physical address register				PHY_ADD			28H-2DH 3.2.4.3

*/

//Not related to the port, used by chip, valid only on port 0
unsigned int ChipInterruptSummary;			//R					
//Base Registers 
unsigned int InterruptStatus;				//R					
unsigned int InterruptEnable;				//R W					
unsigned int TransmitStatus;				//R					
unsigned int ReceiveStatus;					//R					
unsigned int PortControl;					//R W	
unsigned int DeviceIdentificationNumber; 	//R			
unsigned int RevisionIdentificationNumber;	//R				
unsigned int SerialCommand;                 //R W	
	unsigned int regpad_9;
	unsigned int regpad_a;
	unsigned int regpad_b;
	unsigned int regpad_c;
	unsigned int regpad_d;
	unsigned int regpad_e;
	unsigned int regpad_f;
	unsigned int regpad_10;
										
//Configuration Registers
unsigned int FIFOThreshold;					                //R W					
unsigned int FIFOBusMode;					                //R W					
unsigned int TransmitParameters;			                //R W					
unsigned int TransmitErrorMode;				                //R W					
unsigned int TransmitPacketSendingThresholdAndBackoffLimit;	//R W	
unsigned int ReceiveParameters;				                //R W
unsigned int ReceiveFilteringMode;			                //R W
unsigned int TransmitFlowControlPauseTime;					//R W
unsigned int TransmitFlowControlPauseTime1;					//R W
unsigned int MaximumPacketSize;								//R W
unsigned int MaximumPacketSize1;							//R W

    unsigned int regpad_1a;
	unsigned int regpad_1b;
	unsigned int regpad_1c;
	unsigned int regpad_1d;
	unsigned int regpad_1e;

//Serial Registers 
unsigned int SerialMode;									//R W

unsigned int LinkStatus;									//R
	unsigned int regpad_23;
	unsigned int regpad_24;
	unsigned int regpad_25;
	unsigned int regpad_26;
	unsigned int regpad_27;
unsigned int PhysicalAddress;								//R W

};


#endif //BIG_MAC_CONTROL_REGS 

