/* BigMacDriver.h
 * BigMacDriver file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: BigMacDriver
 * author: sk, Jun 98 Initial version
 * $History: BMDRV.H $
 * 
 * *****************  Version 26  *****************
 * User: Dmwhite      Date: 12/08/00   Time: 3:08p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * added ixf440 
 * 
 * *****************  Version 25  *****************
 * User: Dmwhite      Date: 11/15/00   Time: 10:08a
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * add ixf440
 * 
 * *****************  Version 23  *****************
 * User: Dmwhite      Date: 4/06/00    Time: 11:21a
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * fix 'os==vxworks ' bug
 * 
 * *****************  Version 22  *****************
 * User: Dmwhite      Date: 4/05/00    Time: 2:55p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * no more  U_COS macro, changed to (OS==UCOS)
 * 
 * *****************  Version 19  *****************
 * User: Dhooper      Date: 7/23/99    Time: 4:46p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * 
 * *****************  Version 18  *****************
 * User: Skornfel     Date: 7/23/99    Time: 4:19p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * Added OctalMACCofigureEndian(int mac, Endian order); //order=LITTLE
 * interface 
 * 
 * *****************  Version 17  *****************
 * User: Lhui         Date: 7/13/99    Time: 12:51p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * Change UCOS to U_COS to avoid the conflict with UCOS=2 which is defined
 * in ostype.h
 * 
 * *****************  Version 16  *****************
 * User: Skornfel     Date: 7/07/99    Time: 6:40p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * adopted to UCOS
 * 
 * *****************  Version 15  *****************
 * User: Skornfel     Date: 6/24/99    Time: 3:53p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * 21440 ISV board support removed 
 * 
 * *****************  Version 14  *****************
 * User: Skornfel     Date: 6/09/99    Time: 8:03p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * runs on1200 eval board
 * supports two 21440 macs with 16 ports
 * in both non interrupt and interrupt modes   
 * 
 * *****************  Version 13  *****************
 * User: Skornfel     Date: 5/27/99    Time: 10:42a
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * QNX support  removed
 * Dual mac support added 
 * ISR for dual mac needs some work
 * Extern "C" api changed
 * 
 * *****************  Version 12  *****************
 * User: Skornfel     Date: 5/24/99    Time: 4:22p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * 
 * *****************  Version 11  *****************
 * User: Skornfel     Date: 4/12/99    Time: 3:06p
 * Updated in $/development/BigMAC_21440
 * changed SA1200 to SA_1200 
 * 
 * *****************  Version 10  *****************
 * User: Hqianx       Date: 4/12/99    Time: 11:47a
 * Updated in $/development/BigMAC_21440
 * Uses interrupts.h for uC/OS.
 * 
 * *****************  Version 9  *****************
 * User: Skornfel     Date: 4/09/99    Time: 3:52p
 * Updated in $/development/BigMAC_21440
 * *
 * 
 * *****************  Version 8  *****************
 * User: Skornfel     Date: 4/09/99    Time: 3:41p
 * Updated in $/development/BigMAC_21440
 * added SA1200 define to support 1200 eval board
 * one 21440 configiration
 * 
 * *****************  Version 7  *****************
 * User: Skornfel     Date: 4/07/99    Time: 3:47p
 * Updated in $/development/BigMAC_21440
 * 1 adopted to ucos
 * 2 SemDispachEvent = new Semaphore(); insted of creating on the stack
 * 3 ISR algoritm changed to "OR" 
 * 
 * *****************  Version 5  *****************
 * User: Skornfel     Date: 2/12/99    Time: 3:44p
 * Updated in $/development/BigMAC_21440
 * BL5 implemented 
 * code still have QNX calls and   #ifdefs , but not tested on QNX
 * Tested on NT/VxWorks
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 1/27/99    Time: 11:40a
 * Updated in $/development/BigMAC_21440
 * just to move working directory to my new computer
 * 1.Some BL5 staff implemented
 * 2. VxWorks port
 * 3...
 * 
 * *****************  Version 2  *****************
 * User: Skornfel     Date: 12/02/98   Time: 5:07p
 * Updated in $/BL3_Release/BigMAC_21440
 * Base Level 4 driver - runs on QNX(21440) and NT(simulated)
 * 
 * *****************  Version 2  *****************
 * User: Skornfel     Date: 8/10/98    Time: 10:08a
 * Updated in $/base_level3/BigMAC
 * General Driver Structure complete, Not testted on HW or Transactor
 * 
 * *****************  Version 1  *****************
 * User: Skornfel     Date: 6/24/98    Time: 6:34p
 * Created in $/base_level3/BigMAC
 * Just to save My typing it is not MAC Driver yet
 * 
 * ---------------------------------------------------------------------
 */

#ifndef BIG_MAC_DRIVER
#define BIG_MAC_DRIVER

enum{
    NON_SI_BOARD = 0x80860001,
    SI_BOARD = 0x8086FFFF
};
enum{
    MAC_21440_DEV_ID = 0x01,
    MAC_IXF440_DEV_ID = 0x01
};
enum{
    MAC_21440_REV_ID = 0x11,
    MAC_IXF440_REV_ID = 0x11
};
//HH#include "BigMacPort.h"
//HH#include "BigMacEventDispatcher.h"
//HH#include "FDriverComm.h"
//HH#include "BigMacInterrupt.h"
//HH#include "BigMacChipSCR.h"

#include "BigMacApi.h"
#include "bmport.h"
#include "bmevdsp.h"
#include "fdrvcom.h"
#include "bmint.h"
#include "bmchpscr.h"
#include "TS.h"  

#if (OS==UCOS)
#include "interrupts.h"
#endif //UCOS

//GHS
#ifdef GHS
#include "c:/tornado/target/h/intLib.h"
#endif  //GHS


//SK BigMac Driver connected to the hardware 
#ifdef LAB
//Global "back door PTRS - netstats and control regs 
BigMacControlRegs * GLOB_CONTROL_REGS[MAX_MACS][8];
BigMacNetStats    * GLOB_NETSTAS_REGS[MAX_MACS][8];
BigMacNetStats    * GLOB_NETSTAS_REGS_CL[MAX_MACS][8];
#endif 

//Global test area SK - netstats and control regs "NT emulator"
#ifndef LAB
BigMacControlRegs GLOB_CONTROL_REGS[MAX_MACS][8];
BigMacNetStats    GLOB_NETSTAS_REGS[MAX_MACS][8];
BigMacNetStats    GLOB_NETSTAS_REGS_CL[MAX_MACS][8];
#endif

//static PortPtrs  P;   //Array of driver Port pointers
static PortPtrs  PA[MAX_MACS];   //Array of driver Port pointers
//--------------------------------------------------------------------

//volatile int int_storage=0xaa;
// This storage provides one time buffer(per port) for the 
// values of the registers on the time of the interrupt.
// For the later, sufficient buffering or some other queueing
// mechanism needed 
//class IntRegestersShadow{  //may be extended to the circular Queue later 
//public:
//int chip_int_source;
//int port_int_stat[8];
//int port_rx_stat[8];
//int port_tx_stat[8];
////Semaphore  * SemDispachEvent; later
//};
IntRegestersShadow IntRegShadow[MAX_MACS]; 

//something for debuging
volatile int int_count/*[2]*/=0;
volatile int drv_count/*[2]*/=0;


Semaphore  * SemDispachEvent[MAX_MACS]; 

//---------------------------------------------------------------------------------
#define OCTAL_MACS_ON_THE_BOARD MAX_MACS
inline void 
ServTheIsr(int octal_mac){ 
		IntRegShadow[octal_mac].chip_int_source |= GLOB_CONTROL_REGS[octal_mac][0] _CALL_ ChipInterruptSummary;
			for(int port=0; port<8; port++){
				if ( IntRegShadow[octal_mac].chip_int_source & (1 << port)){
					//volatile int	
					IntRegShadow[octal_mac].port_int_stat[port] |= GLOB_CONTROL_REGS[octal_mac][port] _CALL_ InterruptStatus;		
					if ( (IntRegShadow[octal_mac].port_int_stat[port] & 0x2) == 0x2)/*RXEV*/ 
						//volatile int	
						IntRegShadow[octal_mac].port_rx_stat[port] |= GLOB_CONTROL_REGS[octal_mac][port] _CALL_ ReceiveStatus;	
					if ( (IntRegShadow[octal_mac].port_int_stat[port] & 0x8) == 0x8)/*TXER*/ 
						//volatile int	
						IntRegShadow[octal_mac].port_tx_stat[port] |= GLOB_CONTROL_REGS[octal_mac][port] _CALL_ TransmitStatus;	
				       //
					if ( (IntRegShadow[octal_mac].port_int_stat[port] & 0x20) == 0x20)/*OVFC*/ 
						//	
						PA[octal_mac].ptr_RxMsgDptr[port]->CopyStatsAndClear();	
				}
			}
}

//----------------------------------------------------------------------------------
#ifdef LAB
extern "C" void IrqExternPinEnable();		//{* (unsigned int *)(0xb00401e0) |=  0x10000000;};
extern "C" void IrqExternPinDisable();		//{* (unsigned int *)(0xb00401e0) &= ~0x10000000;};
extern "C" void IrqExternPinClear();		//{* (unsigned int *)(0xb00401e0) |=  0x08000000;};
#endif //LAB

int MacsToSev;
#if (OS==UCOS)
int
#endif
//--
ISR (void){
//MacsToSev = 0;
//ServTheIsr();
//ServTheIsr();//make sure nothing left..

#ifdef LAB
IrqExternPinDisable();   //*(unsigned int *)(0xb00401e0) &= ~0x10000000;
IrqExternPinClear();     //*(unsigned int *)(0xb00401e0) |=  0x08000000;
#endif //LAB

	for(int octal_mac=0; octal_mac<OCTAL_MACS_ON_THE_BOARD; octal_mac++){ 
		IntRegShadow[octal_mac].chip_int_source |= GLOB_CONTROL_REGS[octal_mac][0] _CALL_ ChipInterruptSummary;
			for(int port=0; port<8; port++){
				if ( IntRegShadow[octal_mac].chip_int_source & (1 << port)){
					//volatile int	
					IntRegShadow[octal_mac].port_int_stat[port] |= GLOB_CONTROL_REGS[octal_mac][port] _CALL_ InterruptStatus;		
					if ( (IntRegShadow[octal_mac].port_int_stat[port] & 0x2) == 0x2)/*RXEV*/ 
						//volatile int	
						IntRegShadow[octal_mac].port_rx_stat[port] |= GLOB_CONTROL_REGS[octal_mac][port] _CALL_ ReceiveStatus;	
					if ( (IntRegShadow[octal_mac].port_int_stat[port] & 0x8) == 0x8)/*TXER*/ 
						//volatile int	
						IntRegShadow[octal_mac].port_tx_stat[port] |= GLOB_CONTROL_REGS[octal_mac][port] _CALL_ TransmitStatus;	
				        //
					if ( (IntRegShadow[octal_mac].port_int_stat[port] & 0x20) == 0x20)/*OVFC*/ 
						//	
						PA[octal_mac].ptr_RxMsgDptr[port]->CopyStatsAndClear();	
				}
			}
	//int read = GLOB_CONTROL_REGS[1]->GetInterruptStatus();//NOT WORKING on QNX	 
	int_count++;
		if (IntRegShadow[octal_mac].chip_int_source) {
			SemDispachEvent[octal_mac]->Give(); //ucos DO it for VXWORKS or NT
//			MacsToSev++;
			ServTheIsr(octal_mac); //one more time
		};
	};

#ifdef LAB
// try later in driver     
IrqExternPinEnable();   //*(unsigned int *)(0xb00401e0) |= 0x10000000;
#endif //LAB

	return (0); 
};


//-----------------------------------------------------------------------------


class BigMacDriver{

private:
	unsigned long int *mac_base;	//MAC base address pointetr

/*
IntHandler SA_IRQ_Connect(  IntSource intSource,
                            PFVI intServiceRoutine,
                            int userParam)
*/
#ifdef INT
	inline void ConnectInterrupt(){
#if (OS==UCOS)
		//U_SOC
		SA_IRQ_Disable(EXTERNAL_C_PIN);
		SA_IRQ_Connect(EXTERNAL_C_PIN, (PFVI)ISR, 0);
		SA_IRQ_Enable(EXTERNAL_C_PIN);
#else //endif //UCOS	
#if ((OS==UCOS) || (OS==VXWORKS))
		//VxWorks
		int status =
		intConnect(
			    (VOIDFUNCPTR *)9,	/* INT_VEC_CINT in vebsa1200 CINT pin IRQ */
									//11,//interrupt vector to attach to    
				(VOIDFUNCPTR )ISR,  //VOIDFUNCPTR routine to be called  
				0				    //parameter to be passed to routine 
		);cout <<"STATUS OF intConnect = " << status << endl;
		//not on 1200 status=intEnable(11);	cout <<"STAUS OF ENABLE = " << status << endl;
#endif //VXWORKS
#endif //UCOS
	};
#endif //INT



public:
	BigMacDriver( int mac_number=0, unsigned int sa1200_mac_base=0x38400000) //qq
    {
#if (OS!=UCOS)
		cout  <<  hex << endl;
#endif
		Sa1200_mac_base =sa1200_mac_base;
		OctalMacNumber = mac_number;
		This[OctalMacNumber] = this;
		SemDispachEvent[mac_number] = new Semaphore();    

#if (OS==UCOS) // fix this thread problem later
		if (mac_number==0)EventDispatcherThread = new Thread(HandleInterrupt,9);
		if (mac_number==1)EventDispatcherThread = new Thread(HandleInterrupt1,10);
#else //UCOS
		if (mac_number==0)EventDispatcherThread = new Thread(HandleInterrupt);
	    if (mac_number==1)EventDispatcherThread = new Thread(HandleInterrupt1);
#endif //UCOS

#ifdef LAB	
		GLOB_CONTROL_REGS[mac_number][0]  = new((void *)Sa1200_mac_base       ) BigMacControlRegs; 
		GLOB_CONTROL_REGS[mac_number][1]  = new((void *)((unsigned long *)Sa1200_mac_base+ 0x400)) BigMacControlRegs; 
		GLOB_CONTROL_REGS[mac_number][2]  = new((void *)((unsigned long *)Sa1200_mac_base+ 0x800)) BigMacControlRegs; 
		GLOB_CONTROL_REGS[mac_number][3]  = new((void *)((unsigned long *)Sa1200_mac_base+ 0xc00)) BigMacControlRegs; 
		GLOB_CONTROL_REGS[mac_number][4]  = new((void *)((unsigned long *)Sa1200_mac_base+ 0x1000)) BigMacControlRegs; 
		GLOB_CONTROL_REGS[mac_number][5]  = new((void *)((unsigned long *)Sa1200_mac_base+ 0x1400)) BigMacControlRegs; 
		GLOB_CONTROL_REGS[mac_number][6]  = new((void *)((unsigned long *)Sa1200_mac_base+ 0x1800)) BigMacControlRegs;
		GLOB_CONTROL_REGS[mac_number][7]  = new((void *)((unsigned long *)Sa1200_mac_base+ 0x1c00)) BigMacControlRegs; 

		GLOB_NETSTAS_REGS[mac_number][0]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x200)) BigMacNetStats;
		GLOB_NETSTAS_REGS[mac_number][1]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x600)) BigMacNetStats;
		GLOB_NETSTAS_REGS[mac_number][2]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0xa00)) BigMacNetStats;
		GLOB_NETSTAS_REGS[mac_number][3]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0xe00)) BigMacNetStats;
		GLOB_NETSTAS_REGS[mac_number][4]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1200)) BigMacNetStats;
		GLOB_NETSTAS_REGS[mac_number][5]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1600)) BigMacNetStats;
		GLOB_NETSTAS_REGS[mac_number][6]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1a00)) BigMacNetStats;
		GLOB_NETSTAS_REGS[mac_number][7]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1e00)) BigMacNetStats;

		GLOB_NETSTAS_REGS_CL[mac_number][0]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x100)) BigMacNetStats;
		GLOB_NETSTAS_REGS_CL[mac_number][1]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x500)) BigMacNetStats;
		GLOB_NETSTAS_REGS_CL[mac_number][2]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x900)) BigMacNetStats;
		GLOB_NETSTAS_REGS_CL[mac_number][3]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0xd00)) BigMacNetStats;
		GLOB_NETSTAS_REGS_CL[mac_number][4]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1100)) BigMacNetStats;
		GLOB_NETSTAS_REGS_CL[mac_number][5]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1500)) BigMacNetStats;
		GLOB_NETSTAS_REGS_CL[mac_number][6]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1900)) BigMacNetStats;
		GLOB_NETSTAS_REGS_CL[mac_number][7]  = new ((void *)((unsigned long *)Sa1200_mac_base+ 0x1d00)) BigMacNetStats;
#ifdef INT
		ConnectInterrupt(); 
#endif //INT
#endif  //LAB



		for (int port =0; port<NUM_OF_PORTS; port++)
#ifndef LAB
			PA[OctalMacNumber].ptr_RxMsgDptr[port] = new BigMacPort(
			                                             IntRegShadow[mac_number],
			                                             port,
			                                             &GLOB_CONTROL_REGS[mac_number][port],
														 &GLOB_NETSTAS_REGS[mac_number][port],
														 &GLOB_NETSTAS_REGS_CL[mac_number][port]);
#else   // NT emulate case
			PA[OctalMacNumber].ptr_RxMsgDptr[port] = new BigMacPort(
				                                         IntRegShadow[mac_number],
				                                         port,
				                                         GLOB_CONTROL_REGS[mac_number][port],
														 GLOB_NETSTAS_REGS[mac_number][port],
														 GLOB_NETSTAS_REGS_CL[mac_number][port]);
#endif  //LAB

 		ChipSCRs			= new BigMacChipSCR((PA[OctalMacNumber].ptr_RxMsgDptr[0])->GetPortControlRegs());
		EventDispatcher		= new BigMacEventDispatcher(IntRegShadow[mac_number],*ChipSCRs, PA[OctalMacNumber]);
		cout << "BigMacDriver CTOR" <<" OctalMacNumber = " << OctalMacNumber  << endl;
	};

	~BigMacDriver(){
		
		for (int port =0; port<NUM_OF_PORTS; port++){ delete PA[OctalMacNumber].ptr_RxMsgDptr[port]; }
		delete ChipSCRs;
		delete EventDispatcher;
	
	};

//	
	inline RmonHostStats *
	GetPortRmonHostStatistics(int port){
		return PA[OctalMacNumber].ptr_RxMsgDptr[port]->GetRmonHostStatistics();
	}

	inline RmonEtherStats *
	GetPortRmonEtherStatistics(int port){
		return PA[OctalMacNumber].ptr_RxMsgDptr[port]->GetRmonEtherStatistics();
	}

	inline SnmpStats *
	GetPortSnmpStatistics(int port){
		return PA[OctalMacNumber].ptr_RxMsgDptr[port]->GetSnmpStatistics();
	}


//
	inline void
	StartPort (int port, int tx_rx = RX_AND_TX )/*direction to start TX / RX) later */ {
		PA[OctalMacNumber].ptr_RxMsgDptr[port]->Start(tx_rx);
	}
 
	inline void 
	StopPort (int port,int tx_rx = RX_AND_TX) {
		PA[OctalMacNumber].ptr_RxMsgDptr[port]->Stop(tx_rx);
	}
        
	inline void 
	RestartPort (int port,int tx_rx = RX_AND_TX ) {
		PA[OctalMacNumber].ptr_RxMsgDptr[port]->Restart(tx_rx);
	}
        
	inline void 
	ResetPort (int port) {
		PA[OctalMacNumber].ptr_RxMsgDptr[port]->Reset();
	} 

	inline void 
	ConfigurePort (int port) {
		PA[OctalMacNumber].ptr_RxMsgDptr[port]->Configure();
	}
	
inline void 
	PassConfigurationToPort(int port,
                       BusWidth bus_width=BUS_WIDTH_64,          /*BUS_WIDTH_32*/
                       FIFO_Threshold rx_threshold=THRESHOLD_128,/*THRESHOLD_64*/ 
					   FIFO_Threshold tx_threshold=THRESHOLD_64  /*THRESHOLD_128*/
						) {
		PA[OctalMacNumber].ptr_RxMsgDptr[port]->PassConfiguration(bus_width,
                                                                  rx_threshold,
                                                                  tx_threshold
                                                                  );
	}; 
	inline void
	PassEndianToMAC( Endian order=LITTLE ){
		for (int port =0; port<8; port++)PA[OctalMacNumber].ptr_RxMsgDptr[port]->PassEndian( order );
	};  

	inline void
	StartAll (void){
		for(int port=0; port<NUM_OF_PORTS; port++)StartPort(port); 
	}
 
	inline void 
	StopAll (void) {
		for(int port=0; port<NUM_OF_PORTS; port++)StopPort(port); 
	}
        
	inline void 
	RestartAll (void) {
		for(int port=0; port<NUM_OF_PORTS; port++)RestartPort(port); 
	}
        
	inline void 
	ResetAll (void) {
		for(int port=0; port<NUM_OF_PORTS; port++)ResetPort(port); 
	} 

	inline void 
	ConfigureAll (void) {
		for(int port=0; port<NUM_OF_PORTS; port++)ConfigurePort(port); 
	} 

	//two functions to make it work at this point
    static
	inline void 
	HandleInterrupt (void) {
	  for(;;){
		SemDispachEvent[0]->Take();
// try	ServTheIsr(0);
		drv_count ++;
		(This[0])->EventDispatcher->Dispatch();
		//cout << "Tread 0" << endl; //if(MacsToSev == 1)*(unsigned int *)(0xb00401e0) = 0x10000000;
// try	ServTheIsr(0);
// try			*(unsigned int *)(0xb00401e0) = 0x10000000;
	  }
	} 

    static
	inline void 
	HandleInterrupt1 (void) {
	  for(;;){
		SemDispachEvent[1]->Take();
		drv_count ++;
		(This[1])-> EventDispatcher->Dispatch();
		//cout << "Tread 1" << endl; //if(MacsToSev == 2)*(unsigned int *)(0xb00401e0) = 0x10000000;
	  }
	} 

BigMacChipSCR	*ChipSCRs;
private:

//BigMacChipSCR	*ChipSCRs; Moved up to public dmw
BigMacEventDispatcher *EventDispatcher;
static BigMacDriver  * This[MAX_MACS];
int OctalMacNumber;
unsigned int Sa1200_mac_base;
Thread * EventDispatcherThread;

};

static BigMacDriver * OctalMAC[MAX_MACS];

#endif //BIG_MAC_DRIVER 
