
/*
ͻ
    Copyright (c) 1996                                                     
    by Digital Equipment Corporation, Maynard, Mass.                       
    All Rights Reserved.                                                   
                                                                           
    The information in this software is subject to change  without  notice 
    and  should  not  be  construed  as  a commitment by Digital Equipment 
    Corporation.                                                            
ͼ 
*/

#include <stdio.h>
#include <sys/osinfo.h>
#include <sys/pci.h>
#include <sys/mman.h>
#include <fcntl.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <sys/irqinfo.h>
#include "pci_op.h"
/*#include "general.h"*/


PCI_reg_desc PCIregs[0x40] = {
		{4,  "Device ID, Vendor ID" },  // 0
		{0, 0}, {0, 0}, {0, 0},
		{4, "Status, Command"},       // 4
	    {0, 0}, {0, 0},  {0, 0},
		{4, "Revision ID, Class Code"},	   
	    {0, 0}, {0, 0},  {0, 0},
		{4, "Cache Line Size,Latency Timer,Header type,BIST"}, // 12
	    {0, 0}, {0, 0},  {0, 0},
		{4, "PCI Base Address 0 for Memory Mapped Configuration Registers"}, // 16
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "PCI Base Address 1 for I/O Mapped Configuration Registers"}, // 20
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "PCI Base Address 2 for Local Address Space 0"}, // 24
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "PCI Base Address 3 for Local Address Space 1"},  // 28
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "PCI Base Address 4 for Local Address Space 2"},  // 32
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "PCI Base Address 5 for Local Address Space 3"},  // 36
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "Cardbus CIS Pointer"}, // 40
		{0, 0}, {0, 0}, {0, 0}, 
		{4,  "Subsystem Vedor ID, Subsystem ID"}, // 44
		{0, 0}, {0, 0}, {0, 0},
		{4, "PCI Base Address for Local Expansion ROM"}, // 48
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "Reserved"}, // 52
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "Reserved"}, // 56
		{0, 0}, {0, 0}, {0, 0}, 
		{4, "Interrupt Line, Interrupt Pin, Min Gnt, Max Lat"}, // 60
		
};

PCI_reg_desc LocalRegs[0x64] = {
	{4, "Local Address Space 0 Range"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 1 Range"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 2 Range"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 3 Range"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Expansion ROM Range"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 0 Local Base Address (Re-map)"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 1 Local Base Address (Re-map)"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 2 Local Base Address (Re-map)"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 3 Local Base Address (Re-map)"},
	{0,0}, {0,0}, {0,0},
	{4, "Expansion ROM Local Base Address (re-map"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 0 Bus Region Descriptors"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 1 Bus Region Descriptors"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 2 Bus Region Descriptors"},
	{0,0}, {0,0}, {0,0},
	{4, "Local Address Space 3 Bus Region Descriptors"},
	{0,0}, {0,0}, {0,0},
	{4, "Expansion ROM Bus Region Descriptors"},
	{0,0}, {0,0}, {0,0},
	{4, "Chip Select 0 Base Address"},
	{0,0}, {0,0}, {0,0},
	{4, "Chip Select 1 Base Address"},
	{0,0}, {0,0}, {0,0},
	{4, "Chip Select 2 Base Address"},
	{0,0}, {0,0}, {0,0},
	{4, "Chip Select 3 Base Address"},
	{0,0}, {0,0}, {0,0},
	{4, "Interrupt Control/Status"},
	{0,0}, {0,0}, {0,0},
	{4, "EEPROM Control, PCI Slave Response, User I/O Control, Init Control"},
	{0,0}, {0,0}, {0,0}
};	

static unsigned long *local_addr_base = NULL;
static unsigned char *base_addr[3] = {NULL, NULL, NULL};
static unsigned int busnum, devfuncnum;
unsigned intnum;

static int showdevice(unsigned busnum, unsigned devfuncnum, struct _pci_config_regs *creg);

int PCIread_reg (int offset, unsigned long *value) {
	char address[4];
	int len;

    len = PCIregs[offset].len;

	if ((offset <0) || (offset > 0x40) || (len == 0)) {
		fprintf (stderr, "invalid offset 0x%x\n", offset);
		return (FAIL);
	}
    if (_CA_PCI_Read_Config_Byte(busnum, devfuncnum, 
         offset, len, address) != PCI_SUCCESS)  
    {
        fprintf(stderr,"Error reading configuration space:%s\n",
                strerror(errno));
        return (FAIL);
    }                

	// build "value"
	*value = 0;
	switch (len) {
		case 4:
			*value |= (unsigned long)address[3] << 24;
		case 3:
			*value |= (unsigned long)address[2] << 16;
		case 2:
			*value |= (unsigned long)address[1] << 8;
		case 1:
			*value |= (unsigned long)address[0];
	}    
			
    return (PASS);
}

int PCIwrite_reg (int offset, unsigned long value) {

	char address[4];
	int len;

	len = PCIregs[offset].len;

	if (len == 0) {
		fprintf (stderr, "invalid offset 0x%x\n", offset);
		return (FAIL);
	}
	switch (len) {
		case 4:
			address[3] = (unsigned char)(value >> 24);
		case 3:
			address[2] = (unsigned char)(value >> 16);
		case 2:
			address[1] = (unsigned char)(value >> 8);
		case 1:
			address[0] = (unsigned char)(value & 0xff);
	}    
	
	if (_CA_PCI_Write_Config_Byte(busnum, devfuncnum, 
    	offset, len, address) != PCI_SUCCESS)  
    {
    	fprintf(stderr,"Error writing configuration space:%s\n",
                strerror(errno));
        return (FAIL);
    }                
    return (PASS);
}

 
int PCIread_local_reg (int offset, unsigned long *value) {

	if ((offset <0) || (offset > 0x50) || (offset %4)) 	{
    	fprintf (stderr, "invalid offset 0x%x\n", offset);
        return (FAIL);
    }

    if (local_addr_base == NULL) {
    	// find local configuration registers memory base 
		if (PCIget_local_conf (&local_addr_base) == FAIL) {
			fprintf (stderr, "Error getting local configuration memory base\n");
			return (FAIL);
		}

	}    
	*value = local_addr_base[offset/4];
	return (PASS);

}

int PCIwrite_local_reg (int offset, unsigned long value) {

    if ((offset <0) || (offset > 0x50) || (offset%4)) {
    	fprintf (stderr, "invalid offset 0x%x\n", offset);
        return (FAIL);      
	}
    if (local_addr_base == NULL) {
        // find local configuration registers' memory base
		if (PCIget_local_conf (&local_addr_base) == FAIL) {
			fprintf (stderr, "Error getting local configuration memory base\n");
			return (FAIL);
		}
	}  

    local_addr_base[offset/4] = value;
    return (PASS);

}

int PCIinit (int device_id, int vendor_id) {
//fprintf(stderr, "... into PCIINIT\n");
	int err;

	if ((err = _CA_PCI_Find_Device(device_id, vendor_id, 0, &busnum, &devfuncnum))!=
            PCI_SUCCESS)
    {
        fprintf(stderr,"_CA_PCI_Find_Device failed: %s\n", 
			err == PCI_DEVICE_NOT_FOUND ? "The device was not found" : "an invalid Vendor ID was given");
        return (FAIL);
    } 

	return (PASS);
}


int PCIread_mem (unsigned long addr, unsigned long *contents) {

	if (base_addr[0] == NULL) {
		return (FAIL);		
	}
	
//	printf ("address : %0.8lx   ", base_addr[0] + addr);
	*contents = *(unsigned long *)(base_addr[0] + addr);
	return (PASS);
}

int PCIwrite_mem (unsigned long addr, unsigned long contents) {

	if (base_addr[0] == NULL) {
		return (FAIL);		
	}

	*(unsigned long *)(base_addr[0]+addr) = contents;
	return (PASS);
}

int PCIget_local_conf (unsigned long **addr) {
                                            
    int fd;
    struct _pci_config_regs address;
    unsigned long memory_base_phys, memory_offset;
    char  *memory_base;
	
	showdevice (busnum, devfuncnum, &address);
    memory_base_phys = PCI_MEM_ADDR(address.Base_Address_Regs[0]);

	// workaround for PLX part
	// if bit 7 in the local configuration space address is 1, the local
	// configuration registers cannot be read
	if (memory_base_phys & 0x80) { 
		memory_base_phys &= ~0x80;
		PCIwrite_reg (0x10, memory_base_phys);
	}

	memory_offset = memory_base_phys & 0x0FFF;

	// Mapping of phys -> virt
	fd = shm_open("Physical", O_RDWR, 0777);
	if (fd == -1) {
		fprintf(stderr, "shm_open failed:%s\n", strerror(errno));
		return(FAIL);
	}
	errno = 0;
	memory_base = (char *)mmap(0, 0x54, PROT_READ|PROT_WRITE, MAP_SHARED, fd,
			(memory_base_phys & 0xFFFFF000));
	
	if (errno != 0) {
		fprintf(stderr, "mmap failed:%s\n", strerror(errno));
		fprintf(stderr, "errno : %d\n", errno);
		return(FAIL);
	}
//	printf ("memory base : %lx\n", memory_base);
    *addr = (unsigned long *)(memory_base + memory_offset);
	return (PASS);
}

int PCIget_base_addr (int range, unsigned long mem_offset, unsigned long size, unsigned char **addr) {
                                            
    int fd;
    struct _pci_config_regs address;
    unsigned long memory_base_phys, memory_offset;
    char  *memory_base;

	printf("....into PCIget_base_addr\n");

	showdevice (busnum, devfuncnum, &address);
    memory_base_phys = PCI_MEM_ADDR(address.Base_Address_Regs[range]);	
	memory_base_phys += mem_offset;	
    memory_offset = memory_base_phys & 0x0FFF;

	printf ("allocating from physical address %x\n", memory_base_phys);
	// Mapping of phys -> virt
	fd = shm_open("Physical", O_RDWR , 0777);
	if (fd == -1) {
		fprintf(stderr, "shm_open failed:%s\n", strerror(errno));
		return(-1);
	}

	errno=0;
	memory_base = (char *)mmap(0, size , PROT_READ|PROT_WRITE, MAP_SHARED, fd,
			(memory_base_phys & 0xFFFFF000));
	if (errno != 0) {
		fprintf(stderr, "mmap failed:%s\n", strerror(errno));
       	fprintf (stderr, "errno : %d\n", errno);
		return(-1);
	}
   	close (fd);	
	*addr = (unsigned char *)(memory_base + memory_offset);
	
	return (PASS);
}



int showdevice(unsigned busnum, unsigned devfuncnum, struct _pci_config_regs *creg)
{
	int						i;

	_CA_PCI_Read_Config_DWord(busnum, devfuncnum, 0, 16, (char *)creg);

	for(i = 0; i < 6; i++)
	{
		if(creg->Base_Address_Regs[i]==0)
			break;
#if 0
		if(PCI_IS_IO(creg->Base_Address_Regs[i]) & 1)
			printf("IO  Base address: %xh\n", PCI_IO_ADDR(creg->Base_Address_Regs[i]));
		else
			printf("MEM Base address: %xh\n", PCI_MEM_ADDR(creg->Base_Address_Regs[i]));
#endif
	}
	return 0;

}

int PCIhint_attach (pid_t (far *handler)(void), unsigned ds) {

	unsigned long reg_val;
	int int_id;

	PCIread_reg (0x3c, &reg_val);
	intnum = reg_val & 0xff;

	// debug
	//intnum = -1;

	// disable the interrupt before attaching the handler
	qnx_hint_mask (intnum, 2);

	// attach the interrupt

	if ((int_id = qnx_hint_attach (intnum, handler,  ds)) == -1) {
		fprintf (stderr, "qnx_hint_attach failed : %s\n", strerror(errno));
		return (int_id);
	}

	// enable the interrupt

	qnx_hint_mask (intnum, 1);
	fprintf (stderr, "Interrupt %d is %s\n", intnum, (qnx_hint_mask(intnum, 0)) ? "disabled" : "enabled");

	return (int_id);
}
	


int PCIhint_enable () {
	return (qnx_hint_mask (intnum, 1));
}

int PCIhint_disable () {
	return (qnx_hint_mask (intnum, 2));
}



