/* RmonHostStats.h
 * BigMacDriver RMON Statistics file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: BigMacDriver
 * author: sk, Jun 98 Initial version
 * $History: RMONHTST.H $
 * 
 * *****************  Version 8  *****************
 * User: Skornfel     Date: 6/24/99    Time: 3:53p
 * Updated in $/dev_1200/SA1_CoreLibs/OctalMAC_21440
 * 21440 ISV board support removed 
 * 
 * *****************  Version 7  *****************
 * User: Skornfel     Date: 4/07/99    Time: 3:47p
 * Updated in $/development/BigMAC_21440
 * 1 adopted to ucos
 * 2 SemDispachEvent = new Semaphore(); insted of creating on the stack
 * 3 ISR algoritm changed to "OR" 
 * 
 * *****************  Version 5  *****************
 * User: Skornfel     Date: 2/12/99    Time: 3:44p
 * Updated in $/development/BigMAC_21440
 * BL5 implemented 
 * code still have QNX calls and   #ifdefs , but not tested on QNX
 * Tested on NT/VxWorks
 * 
 * *****************  Version 4  *****************
 * User: Skornfel     Date: 1/27/99    Time: 11:40a
 * Updated in $/development/BigMAC_21440
 * just to move working directory to my new computer
 * 1.Some BL5 staff implemented
 * 2. VxWorks port
 * 3...
 * 
 * *****************  Version 2  *****************
 * User: Skornfel     Date: 12/02/98   Time: 5:07p
 * Updated in $/BL3_Release/BigMAC_21440
 * Base Level 4 driver - runs on QNX(21440) and NT(simulated)
 * 
 * *****************  Version 2  *****************
 * User: Skornfel     Date: 8/10/98    Time: 10:08a
 * Updated in $/base_level3/BigMAC
 * General Driver Structure complete, Not testted on HW or Transactor
 * 
 * *****************  Version 1  *****************
 * User: Skornfel     Date: 6/24/98    Time: 6:34p
 * Created in $/base_level3/BigMAC
 * Just to save My typing it is not MAC Driver yet
 * 
 * ---------------------------------------------------------------------
 */

#ifndef BIG_MAC_RMON_HOST_STATS
#define BIG_MAC_RMON_HOST_STATS

//HH#include "BigMacNetStats.h"
#include "bmnetst.h"

class RmonHostStats{


public:
	RmonHostStats(BigMacNetStats & stats):_MacNetStats(stats){
		//cout << "RmonHostStats CTOR" << endl;
	};

	~RmonHostStats(){};

	inline RmonHostStats * GetHostCounters()			{return this;};

	inline void UpdateHostCounters() {
		CalculateHostInPkts();
		CalculateHostOutPkts(); 
		CalculateHostInOctets(); 
		CalculateHostOutOctets();
		CalculateHostOutErrors(); 
		CalculateHostOutBroadcastPkts(); 
		CalculateHostOutMulticastPkts(); 
	};

	inline unsigned int GetHostInPkts()				{return hostInPkts;};
	inline unsigned int GetHostOutPkts()			{return hostOutPkts;};
	inline unsigned int GetHostInOctets()			{return hostInOctets;};
	inline unsigned int GetHostOutOctets()			{return hostOutOctets;};
	inline unsigned int GetHostOutErrors()			{return hostOutErrors;};
	inline unsigned int GetHostOutBroadcastPkts()	{return hostOutBroadcastPkts;};
	inline unsigned int GetHostOutMulticastPkts()	{return hostOutMulticastPkts;};

	//TX_UNI_OK_CNT + TX_MLT_OK_CNT + TX_BRD_OK_CNT
	inline void CalculateHostInPkts(){
					hostInPkts = _MacNetStats.GetTxUniOkCnt() +
								 _MacNetStats.GetTxMltOkCnt() +
								 _MacNetStats.GetTxBrdOkCnt();
	};

	//RX_UNI_OK_CNT + RX_MLT_OK_CNT + RX_BRD_OK_CNT+ RX_NORM_ALI_CNT +
	//RX_NORM_CRC_CNT + RX_SHORT_OK_CNT +
	//RX_SHORT_CRC_CNT + RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT
	inline void CalculateHostOutPkts(){ 
					hostOutPkts = _MacNetStats.GetRxUniOkCnt() +
								  _MacNetStats.GetRxMltOkCnt() +
								  _MacNetStats.GetRxBrdOkCnt() +
								  _MacNetStats.GetRxNormAliCnt() +
								  _MacNetStats.GetRxNormCrcCnt() +
								  _MacNetStats.GetRxShortOkCnt() +
								  _MacNetStats.GetRxShortCrcCnt() +
								  _MacNetStats.GetRxLongOkCnt() +
								  _MacNetStats.GetRxLongCrcCnt() +
								  _MacNetStats.GetTxColCnt() +
								  _MacNetStats.GetTxLcolCnt();
	};

	//TX_OCT_OK_CNT
	inline void CalculateHostInOctets(){ 
						hostInOctets =_MacNetStats.GetTxOctOkCnt();
	};

	//RX_OCT_OK_CNT + RX_OCT_BAD_CNT	
	inline void CalculateHostOutOctets(){
					hostOutOctets = _MacNetStats.GetRxOctOkCnt() +
									_MacNetStats.GetRxOctBadCnt();
	};

	//RX_NORM_ALI_CNT + RX_NORM_CRC_CNT + RX_SHORT_OK_CNT + RX_SHORT_CRC_CNT + 
	//RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT 	
	inline void CalculateHostOutErrors()		{ 
					hostOutErrors = _MacNetStats.GetRxNormAliCnt() +
									_MacNetStats.GetRxNormCrcCnt() + 
									_MacNetStats.GetRxShortOkCnt() +
									_MacNetStats.GetRxShortCrcCnt() +
									_MacNetStats.GetRxLongOkCnt() +
									_MacNetStats.GetRxLongCrcCnt() + 
									_MacNetStats.GetTxColCnt() +
									_MacNetStats.GetTxLcolCnt();
	
	};

	//RX_BRD_OK_CNT
	inline void CalculateHostOutBroadcastPkts()	{ 
					hostOutBroadcastPkts = _MacNetStats.GetRxBrdOkCnt();
	};

	//RX_MLT_OK_CNT
	inline void CalculateHostOutMulticastPkts()	{ 
					hostOutMulticastPkts = _MacNetStats.GetRxMltOkCnt();
	};

	inline void Show(){

cout 

		<< "hostInPkts            " << hostInPkts   << endl				
		<< "hostOutPkts           " << hostOutPkts  << endl					
		<< "hostInOctets          " << hostInOctets   << endl					
		<< "hostOutOctets         " << hostOutOctets   << endl					
		<< "hostOutErrors         " << hostOutErrors  << endl				
		<< "hostOutBroadcastPkts  " << hostOutBroadcastPkts   << endl			
		<< "hostOutMulticastPkts  " << hostOutMulticastPkts   << endl;	

	};

public:

//lets see later if int is not long enought

//------------------------------------------------------------------------------------------------------------------
//RMON Host Counters 21440 Statistic Counters

//RMON Host Group Support
//Describes the 21440 support of the RMON host group when a single node is
//connected to a port. If packets are loaded into the 21440 to be transmitted as bad
//packets, they must be counted in the error counters, too.

unsigned int hostInPkts;			//TX_UNI_OK_CNT + TX_MLT_OK_CNT + TX_BRD_OK_CNT
unsigned int hostOutPkts;			//RX_UNI_OK_CNT + RX_MLT_OK_CNT + RX_BRD_OK_CNT+ RX_NORM_ALI_CNT +
									//RX_NORM_CRC_CNT + RX_SHORT_OK_CNT +
									//RX_SHORT_CRC_CNT + RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT
unsigned int hostInOctets;			//TX_OCT_OK_CNT
unsigned int hostOutOctets;			//RX_OCT_OK_CNT + RX_OCT_BAD_CNT
unsigned int hostOutErrors;			//RX_NORM_ALI_CNT + RX_NORM_CRC_CNT + RX_SHORT_OK_CNT + RX_SHORT_CRC_CNT + 
									//RX_LONG_OK_CNT + RX_LONG_CRC_CNT + TX_COL_CNT + TX_LCOL_CNT 
unsigned int hostOutBroadcastPkts;	//RX_BRD_OK_CNT
unsigned int hostOutMulticastPkts;	//RX_MLT_OK_CNT


private:
BigMacNetStats & _MacNetStats;



};




#endif //BIG_MAC_RMON_HOST_STATS 





/*         HostEntry ::= SEQUENCE {
              hostAddress             OCTET STRING,
              hostCreationOrder       INTEGER (1..65535),
              hostIndex               INTEGER (1..65535),
              hostInPkts              Counter,
              hostOutPkts             Counter,
              hostInOctets            Counter,
              hostOutOctets           Counter,
              hostOutErrors           Counter,
              hostOutBroadcastPkts    Counter,
              hostOutMulticastPkts    Counter
//----------------------------------------------------------------
//---------------------------------------------------------------
hostEntry OBJECT-TYPE
              SYNTAX HostEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A collection of statistics for a particular host
                  that has been discovered on an interface of this
                  device."
              INDEX { hostIndex, hostAddress }
              ::= { hostTable 1 }

          


HostEntry ::= SEQUENCE {
              hostAddress             OCTET STRING,
              hostCreationOrder       INTEGER (1..65535),
              hostIndex               INTEGER (1..65535),
              hostInPkts              Counter,
              hostOutPkts             Counter,
              hostInOctets            Counter,
              hostOutOctets           Counter,
              hostOutErrors           Counter,
              hostOutBroadcastPkts    Counter,
              hostOutMulticastPkts    Counter
          }

          


hostAddress OBJECT-TYPE
              SYNTAX OCTET STRING
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The physical address of this host."
              ::= { hostEntry 1 }

          


hostCreationOrder OBJECT-TYPE
              SYNTAX INTEGER (1..65535)
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "An index that defines the relative ordering of
                  the creation time of hosts captured for a
                  particular hostControlEntry.  This index shall
                  be between 1 and N, where N is the value of
                  the associated hostControlTableSize.  The ordering
                  of the indexes is based on the order of each entry's
                  insertion into the table, in which entries added
                  earlier have a lower index value than entries added
                  later.

                  It is important to note that the order for a
                  particular entry may change as an (earlier) entry
                  is deleted from the table.  Because this order may
                  change, management stations should make use of the
                  hostControlLastDeleteTime variable in the
                  hostControlEntry associated with the relevant
                  portion of the hostTable.  By observing
                  this variable, the management station may detect
                  the circumstances where a previous association
                  between a value of hostCreationOrder
                  and a hostEntry may no longer hold."
              ::= { hostEntry 2 }

          


hostIndex OBJECT-TYPE
              SYNTAX INTEGER (1..65535)
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The set of collected host statistics of which
                  this entry is a part.  The set of hosts
                  identified by a particular value of this
                  index is associated with the hostControlEntry
                  as identified by the same value of hostControlIndex."
              ::= { hostEntry 3 }

          


hostInPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of packets without errors transmitted to
                  this address since it was added to the hostTable."
              ::= { hostEntry 4 }

          


hostOutPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of packets including errors transmitted
                  by this address since it was added to the hostTable."
              ::= { hostEntry 5 }

          


hostInOctets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of octets transmitted to this address
                  since it was added to the hostTable (excluding
                  framing bits but including FCS octets), except for
                  those octets in packets that contained errors."
              ::= { hostEntry 6 }

          


hostOutOctets OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of octets transmitted by this address
                  since it was added to the hostTable (excluding
                  framing bits but including FCS octets), including
                  those octets in packets that contained errors."
              ::= { hostEntry 7 }

          


hostOutErrors OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of error packets transmitted by this
                  address since this host was added to the hostTable."
              ::= { hostEntry 8 }

          


hostOutBroadcastPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of good packets transmitted by this
                  address that were directed to the broadcast address
                  since this host was added to the hostTable."
              ::= { hostEntry 9 }

          


hostOutMulticastPkts OBJECT-TYPE
              SYNTAX Counter
              ACCESS read-only
              STATUS mandatory
              DESCRIPTION
                  "The number of good packets transmitted by this
                  address that were directed to a multicast address
                  since this host was added to the hostTable.
                  Note that this number does not include packets
                  directed to the broadcast address."
              ::= { hostEntry 10 }


          -- host Time Table

          


hostTimeTable OBJECT-TYPE
              SYNTAX SEQUENCE OF HostTimeEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A list of time-ordered host table entries."
              ::= { hosts 3 }

          


hostTimeEntry OBJECT-TYPE
              SYNTAX HostTimeEntry
              ACCESS not-accessible
              STATUS mandatory
              DESCRIPTION
                  "A collection of statistics for a particular host
                  that has been discovered on an interface of this
                  device.  This collection includes the relative
                  ordering of the creation time of this object."
              INDEX { hostTimeIndex, hostTimeCreationOrder }
              ::= { hostTimeTable 1 }

          

*/
