/* pci.c
*
*
* --------------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*---------------------------------------------------------------------
* system: SA1200
* subsystem: uHAL 
* author: Henry Qian 04/14/99
* revisions:
*
*/

#include "platform.h"
#include "pci.h"

extern unsigned int Pci_DeviceID;

/*
**  unsigned int SAir_CheckCfgParams(   unsigned int devHandle,
**                                      unsigned int offset)
**
**  DESCRIPTION
**      This routine checks that all the parameters fit into the
**      PCI config space.
**
**      handle = handle of device that we are assigning resources to
**      offset = offset into PCI config space for current BAR
**  
**  RETURNS
**      SUCCESS or ERROR_STATUS.
*/
unsigned int SAir_CheckCfgParams(   unsigned int devHandle,
                                    unsigned int offset)
{
    unsigned int bus, device, func;
    
    /* extract the bus, device, and function number from the handle */
    bus = (unsigned int)GET_BUS(devHandle);
    device = (unsigned int)GET_DEVICE(devHandle);
    func = (unsigned int)GET_FUNCTION(devHandle);
    
    /* check that all parameters are valid */
    if (bus >= 256)
        return(PCI_ERROR_BAD_BUS); 
    if (device >= 12)
        return(PCI_ERROR_BAD_DEVICE); 
    if (func >= 8)
        return(PCI_ERROR_BAD_FUNC); 
    if (offset >= 256)
        return(PCI_ERROR_BAD_OFFSET); 
    
    return(PCI_SUCCESS);
}

/*
**  SAr_PciCfgReadByte( unsigned int handle,
**                      unsigned int offset,
**                      unsigned int* pStatus)
**
**  SAr_PciCfgReadShort(unsigned int handle,
**                      unsigned int offset,
**                      unsigned int* pStatus)
**
**  SAr_PciCfgReadLong( unsigned int handle,
**                      unsigned int offset,
**                      unsigned int* pStatus)
**
**  DESCRIPTION
**      These routines read a byte, short or long value from the given
**      PCI config register.
**
**      handle  = handle of device that we are assigning resources to
**      offset  = offset into PCI config space for current BAR
**      data    = value to be written
**      pStatus = SUCCESS/FAIL flag
**  
**  RETURNS
**      SUCCESS or ERROR_STATUS.
*/
unsigned char SAr_PciCfgReadByte(   unsigned int devHandle,
                                    unsigned int offset,
                                    unsigned int* pStatus)
{
    unsigned char* pAddress;
    unsigned char  data;
    
    /* check device params */
    *pStatus = (int)SAir_CheckCfgParams(devHandle, offset);
    if (*pStatus != PCI_SUCCESS)
        return(0);
    
    /* generate the address of correct configuration space */
    pAddress = (unsigned char*)(SAir_PciMakeConfigAddress(devHandle, offset)); 
    
    /* now that we have valid params, go read the config space data */
    data = *pAddress;
    *pStatus = PCI_SUCCESS;
    
    return(data);
}

unsigned short SAr_PciCfgReadShort( unsigned int devHandle,
                                    unsigned int offset,
                                    unsigned int* pStatus)
{
    unsigned short*    pAddress;
    unsigned short data;
    
    /* check device params */
    *pStatus = (int)SAir_CheckCfgParams(devHandle, offset);
    if (*pStatus != PCI_SUCCESS)
        return(0);
    
    /* generate the address of correct configuration space */
    pAddress = (unsigned short*)(SAir_PciMakeConfigAddress(devHandle, offset)); 
    
    /* now that we have valid params, go read the config space data */
    data = *pAddress;
    *pStatus = PCI_SUCCESS;
    
    return(data);
}

unsigned int SAr_PciCfgReadLong(    unsigned int devHandle,
                                    unsigned int offset,
                                    unsigned int* pStatus)
{
    unsigned int*    pAddress;
    unsigned int data;
    
    /* check device params */
    *pStatus = (int)SAir_CheckCfgParams(devHandle, offset);
    if (*pStatus != PCI_SUCCESS)
        return(0);
    
    /* generate the address of correct configuration space */
    pAddress = (unsigned int*)(SAir_PciMakeConfigAddress(devHandle, offset)); 
    
    /* now that we have valid params, go read the config space data */
    data = *pAddress;
    *pStatus = PCI_SUCCESS;
    
    return(data);
}

/*
**  SAr_PciCfgWriteByte(    unsigned int handle,
**                          unsigned int offset,
**                          unsigned int data,
**                          unsigned int* pStatus)
**
**  SAr_PciCfgWriteShort(   unsigned int handle,
**                          unsigned int offset,
**                          unsigned int data,
**                          unsigned int* pStatus)
**
**  SAr_PciCfgWriteLong(    unsigned int handle,
**                          unsigned int offset,
**                          unsigned int data,
**                          unsigned int* pStatus)
**
**  DESCRIPTION
**      These routines write a byte, short or long value to the given
**      PCI config register.
**
**      handle  = handle of device that we are assigning resources to
**      offset  = offset into PCI config space for current BAR
**      data    = value to be written
**      pStatus = SUCCESS/FAIL flag
**  
**  RETURNS
**      None.
*/
void SAr_PciCfgWriteByte(   unsigned int devHandle,
                            unsigned int offset,
                            unsigned int data,
                            unsigned int* pStatus)
{
    unsigned char* pAddress;
    
    /* check device params */
    *pStatus = (int)SAir_CheckCfgParams(devHandle, offset);
    if (*pStatus != PCI_SUCCESS)
        return;
    
    /* generate the address of correct configuration space */
    pAddress = (unsigned char*)(SAir_PciMakeConfigAddress(devHandle, offset)); 
    
    /* now that we have valid params, go write the config space data */
    *pAddress = (unsigned char)data;
    *pStatus = PCI_SUCCESS;
    
    return;
}

void SAr_PciCfgWriteShort(  unsigned int devHandle,
                            unsigned int offset,
                            unsigned int data,
                            unsigned int* pStatus)
{
    unsigned short*    pAddress;
    
    /* check device params */
    *pStatus = (int)SAir_CheckCfgParams(devHandle, offset);
    if (*pStatus != PCI_SUCCESS)
        return;
    
    /* generate the address of correct configuration space */
    pAddress = (unsigned short*)(SAir_PciMakeConfigAddress(devHandle, offset)); 
    
    /* now that we have valid params, go write the config space data */
    *pAddress = (unsigned short)data;
    *pStatus = PCI_SUCCESS;
    
    return;
}

void SAr_PciCfgWriteLong(   unsigned int devHandle,
                            unsigned int offset,
                            unsigned int data,
                            unsigned int* pStatus)
{
    unsigned int*    pAddress;
    
    /* check device params */
    *pStatus = (int)SAir_CheckCfgParams(devHandle,offset);
    if (*pStatus != PCI_SUCCESS)
        return;
    
    /* generate the address of correct configuration space */
    pAddress = (unsigned int*)(SAir_PciMakeConfigAddress(devHandle, offset)); 
    
    /* now that we have valid params, go write the config space data */
    *pAddress = (unsigned int)data;
    *pStatus = PCI_SUCCESS;
    
    return;
}


/*
**  unsigned int SAir_AlignAddress( unsigned int address,
**                                  unsigned int boundary)
**
**  DESCRIPTION
**      This routine will attempt to align the address up to 
**      the boundary specified.
**
**      address     = start address of the block to align
**      boundary    = alignment boundary (ie 4K = 4K alignment)
**  
**  RETURNS
**      Aligned address.
*/
unsigned int SAir_AlignAddress (    unsigned int address,
                                    unsigned int boundary)
{
    if ((address % boundary) == 0)
        return address;
    
    return (unsigned int)(address + (boundary - (address % boundary)));
}

unsigned int  SAiv_PciMemAvailableAddress = SZ_1M;
unsigned int  SAiv_PciIoAvailableAddress = 0x400;


/*
**  unsigned int SAir_GetNextPciAddr(   unsigned int *base,
**                                      unsigned int size)
**
**  DESCRIPTION
**      This routine will return the next available aligned memory 
**      address block. This address block is added to PCI_MEM to
**      obtain the memory location as seen by the StrongARM.
**
**      *base - pointer to next available PCI address
**      size  - amount of memory to be assigned
**  
**  RETURNS
**      Available aligned memory address.
*/
unsigned int SAir_GetNextPciAddr(   unsigned int *base,
                                    unsigned int size)
{
    unsigned int address;
    
    /* align the address to size (ie 4 meg size requires 4 meg alignment */
    address = SAir_AlignAddress(*base, size);
    
    /* generate next available address for next call */
    *base = address + size;
    
    return(address);
}


/*
**  unsigned int SAir_AssignMemAddr(unsigned int size)
**
**  DESCRIPTION
**      This routine will return the next available aligned memory 
**      address block. This address block is added to PCI_MEM to
**      obtain the memory location as seen by the StrongARM.
**
**      size - converted value read from BAR after writing PCI_INVALID
**  
**  RETURNS
**      Available aligned memory address.
*/
unsigned int SAir_AssignMemAddr(unsigned int size)
{
    return(SAir_GetNextPciAddr(&SAiv_PciMemAvailableAddress, size));
}


/*
**  unsigned int SAir_AssignIoAddr(unsigned int size)
**
**  DESCRIPTION
**      This routine will return the next available aligned IO 
**      address block (must be < 1M, according to PCI spec.). This
**      address is added to PCI_IO to obtain the memory location
*       as seen by the StrongARM.
**
**      size - converted value read from BAR after writing PCI_INVALID
**  
**  RETURNS
**      Available aligned IO address
*/
unsigned int SAir_AssignIoAddr(unsigned int size)
{
    return(SAir_GetNextPciAddr(&SAiv_PciIoAvailableAddress, size));
}


/*
**  unsigned int SAir_ReadBARSize(  unsigned int handle,
**                                  unsigned int *sizePtr,
**                                  unsigned int offset)
**
**  DESCRIPTION
**      This routine will attempt to glean resource requirements
**      from the device pointed to by handle
**
**      handle = handle of device that we are assigning resources to
**      offset = offset into PCI config space for current BAR
**  
**  RETURNS
**      data - value read from BAR
*/
unsigned int SAir_ReadBARSize(  unsigned int handle,
                                unsigned int *sizePtr,
                                unsigned int offset)
{
    unsigned int data;
    unsigned int status;
    
    /* write PCI_INVALID to BAR to get size */
    SAr_PciCfgWriteLong(handle, offset, PCI_INVALID, &status);

    /* check if this is an implemented BAR (size != 0) */
    data = SAr_PciCfgReadLong(handle, offset, &status);
    if (data != 0)
        *sizePtr = (~(data & 0xFFFFFFF0)) + 1;
    else
        *sizePtr = 0;
    
    return (data);
}


/*
**  int SAir_WriteBARSize(  unsigned int handle,
**                          unsigned int data,
**                          unsigned int size,
**                          unsigned int offset)
**
**  DESCRIPTION
**      This routine writes the base address for the requested size
**      to the device pointed to by handle.
**
**      handle = handle of device that we are assigning resources to
**      data   = raw data read from BAR
**      size   = size of PCI space required
**      offset = offset into PCI config space for current BAR
**  
**  RETURNS
**      BAR enable bit.
*/
int SAir_WriteBARSize(  unsigned int handle,
                        unsigned int data,
                        unsigned int size,
                        unsigned int offset)
{
    unsigned int status;
    unsigned int barFlag = 0;
    
    /* Do not assign PCI space to ROM Expansion BAR to IO space */
    if ((offset != PCI_ROM_BAR) && (data & PCI_IO_ENABLE))
    {
        /* IO space BAR */
        SAr_PciCfgWriteLong(    handle,
                                offset,
                                SAir_AssignIoAddr(size),
                                &status);
        barFlag |= PCI_IO_ENABLE;
    }
    else
    {
        /* memory space BAR */
        SAr_PciCfgWriteLong(    handle,
                                offset,
                                SAir_AssignMemAddr(size),
                                &status);
        barFlag |= PCI_MEM_ENABLE;
    }
    
    return barFlag;
}


/*
**  void SAir_AssignResources(  unsigned int handle,
**                              unsigned int *sizePtr)
**
**  DESCRIPTION
**      This routine will attempt to glean resource requirements
**      from the device pointed to by handle.
**
**      handle   = handle of device that we are assigning resources to
**      *sizePtr = pointer to an array which can hold the BAR sizes.
**  
**  RETURNS
**      None.
*/
void SAir_AssignResources(  unsigned int handle,
                            unsigned int *sizePtr)
{
    unsigned int data;
    unsigned int offset = PCI_MEM_BAR;
    int i;
    unsigned int status;
    unsigned int barFlag = 0;
    
    /* Disable PCI IO and memory accesses */
    SAr_PciCfgWriteShort(handle, PCI_COMMAND, 0, &status);
    
    /* Scan each of the BARs for this device */
    for (i = 0; i < PCI_MAX_BAR; i++)
    {
        data = SAir_ReadBARSize(handle, sizePtr, offset);
        if (data != 0)
        {
            /* Assign some PCI memory space to this BAR */
            barFlag |= SAir_WriteBARSize(   handle,
                                            data,
                                            *sizePtr,
                                            offset);
        }
        /* increment the BAR offset value */
        offset += 4;
        sizePtr++;
    }
    
    /* Check out the Expansion ROM, but don't give it any PCI space */
    sizePtr++;
    offset = PCI_ROM_BAR;
    SAir_ReadBARSize(handle, sizePtr, offset);
    
    /* Enable PCI IO and memory accesses as found by scan, with MASTER */
    barFlag |= PCI_MASTER_ENABLE;
    SAr_PciCfgWriteShort(handle, PCI_COMMAND, barFlag, &status);
    SAr_PciCfgWriteByte(handle, PCI_LATENCY, 0xFF, &status);
}


/*
**  void SAr_ConfigurePciDevices(void)
**
**  DESCRIPTION
**      This routine will scan the devNode tree looking for
**      valid PCI devices and assign resources to each one.
**
**  RETURNS
**      None.
*/
void SAr_ConfigurePciDevices(void)
{
    devNodeType *current;
    
    for (current = devNodes; current != 0; current = current->next)
    {
        /* check if device is PCI, if so it needs system resources */
        if ((current->handle) & PCI_TYPE)
        {
            /* call the resource manager to assign resources */
            SAir_AssignResources(current->handle, current->memSize);
        }
    }
}



/*
**  unsigned int SAr_GetNextMemBAR( unsigned int handle,
**                                  unsigned int *bar)
**
**  DESCRIPTION
**      This routine will scan the remaining Base Address Registers
**      for the specified device. When a BAR is found which points to
**      PCI memory space, the address in the BAR is returned.
**
**      handle - handle of device being accessed
**      *bar - offset into Base Address Registers to start from
**  
**  RETURNS
**      contents of BAR (0 if none found)
**      *bar is modified to next possible BAR offset
*/
unsigned int SAr_GetNextMemBAR( unsigned int handle,
                                unsigned int *bar)
{
    unsigned int i, data;
    unsigned int status;
    
    /* Scan remaining BARS for this device */
    for (i = *bar; i < PCI_MAX_BAR; i++)
    {
        /* Read back address assigned for this BAR */
        data = SAr_GetPciBAR(handle, i, &status);
        
        /* If not I/O, then it is memory */
        if ((data & PCI_IO_ENABLE) == 0)
        {
            /* Got one, set next available BAR & return address */
            *bar = i + 1;
            return (data & ~0x0F);
        }
    }
    
    /* Nothing found */
    *bar = 0;
    return 0;
}


/*
**  unsigned int SAr_GetNextIOBAR(  unsigned int handle,
**                                  unsigned int *bar)
**
**  DESCRIPTION
**      This routine will scan the remaining Base Address Registers
**      for the specified device. When a BAR is found which points to
**      PCI I/O space, the address in the BAR is returned.
**
**      address     = start address of the block to align
**      boundary    = alignment boundary (ie 4K = 4K alignment)
**  
**  RETURNS
**      contents of BAR (0 if none found)
**      *bar is modified to next possible BAR offset
*/
unsigned int SAr_GetNextIOBAR(  unsigned int handle,
                                unsigned int *bar)
{
    unsigned int i, data;
    unsigned int status;
    
    /* Scan remaining BARS for this device */
    for (i = *bar; i < PCI_MAX_BAR; i++)
    {
        /* Read back address assigned for this BAR */
        data = SAr_GetPciBAR(handle, i, &status);
        
        /* Check if it is I/O */
        if (data & PCI_IO_ENABLE)
        {
            /* Got one, set next available BAR & return address */
            *bar = i + 1;
            return (data & ~0x0F);
        }
    }
    
    /* Nothing found */
    *bar = 0;
    return 0;
}


/*
**  unsigned int SAr_FindPCIDevClass(   unsigned int devClass,
**                                      unsigned int *inst)
**
**  DESCRIPTION
**      This routine will scan the devNode tree looking for devices
*       which match the class code required. When a device matches
*       the class code, the instance is checked - this enables
*       software to uniquely address a device, even when more than
*       one may be fitted.
**
**      devClass - device class to scan for
**      *inst  - pointer to instance of card to find.
**  
**  RETURNS
**      Aligned address.
*/
unsigned int SAr_FindPCIDevClass(   unsigned int devClass,
                                    unsigned int *inst)
{
    devNodeType *current;
    unsigned int newClass;
    unsigned int count = 0;
    unsigned int status;
    
    for (current = devNodes; current != 0; current = current->next)
    {
        /* check if device is PCI, if so check the device class */
        if ((current->handle) & PCI_TYPE)
        {
            newClass = SAr_GetPciClassID(current->handle, &status);
            
            if (newClass == devClass)
            {
                if (count++ >= *inst)
                {
                    *inst = count;
                    return (current->handle);
                }
            }
        }
    }
    
    /* Nothing found */
    *inst = 0;
    return 0;
}


/*
**  unsigned int SAr_FindPCIDevice( unsigned short vendor,
                                    unsigned short device,
                                    unsigned int *inst)
**
**  DESCRIPTION
**      This routine will scan the devNode tree looking for devices
**      which match the vendor & device code required. When a device
**      matches both vendor and device codes, the instance is checked
**      - this enables software to uniquely address a device, even
**      when more than one may be fitted. If only a specific vendor
**      or device code is required (not both), the other code can
**      be ignored by specifying PCI_NOTFITTED.
**
**      vendor - vendor code to scan for
**      device - device code to scan for
**      *inst  - pointer to instance of card to find.
**  
**  RETURNS
**      handle to required device (0 if not found)
**      *inst is modified to next possible instance.
*/
unsigned int SAr_FindPCIDevice( unsigned short vendor,
                                unsigned short device,
                                unsigned int *inst)
{
    devNodeType *current;
    unsigned int value;
    unsigned int count = 0;
    unsigned int status;
    
    for (current = devNodes; current != 0; current = current->next)
    {
        /* check if device is PCI, if so check the device class */
        if ((current->handle) & PCI_TYPE)
        {
            value = SAr_GetPciVendorID(current->handle, &status);
            
            /* Allow don't care vendor code or match */
            if ((vendor == PCI_NOTFITTED) || (value == vendor))
            {
                value = SAr_GetPciDeviceID(current->handle, &status);
                
                /* Allow don't care device code or match */
                if ((device == PCI_NOTFITTED) || (value == device))
                {
                    if (count++ >= *inst)
                    {
                        *inst = count;
                        return (current->handle);
                    }
                }
            }
        }
    }
    
    /* Nothing found */
    *inst = 0;
    return 0;
}


/*
**  unsigned int SAr_PciHost(void)
**
**  DESCRIPTION
**      This routine determines if the target is PCI Host.
**
**  RETURNS
**      PCI_TRUE or PCI_FALSE.
*/
unsigned int SAr_PciHost(void)
{
    unsigned char *controlReg = (unsigned char *)CSR_BASE + SA_CONTROL;
    
    if (*(unsigned int *)controlReg & PCI_CFN)
        return (PCI_TRUE);
    else
        return (PCI_FALSE);
}


unsigned int SAir_PciMakeConfigAddress( unsigned int devHandle,
                                        unsigned int offset)
{
    unsigned int bus, device, function;
    unsigned int address;
    
    /* extract the bus, device and function from the handle */
    bus = (unsigned int)GET_BUS(devHandle);
    device = (unsigned int)GET_DEVICE(devHandle);
    function = (unsigned int)GET_FUNCTION(devHandle);

    /* SA1200 uses different zones for type 0 and 1 configuration cycles */
    if (bus == 0) {
        if (device == Pci_DeviceID) {	/* use local CSR register */
            address = (unsigned int)CSR_BASE;
        }
        else {
            address = (unsigned int)PCIConfigTypeZeroBase;
            address |= (unsigned int)(1 << (device + 11));
        }
    } else {
        address = (unsigned int)PCIConfigTypeOneBase;
        address |= (device << 11);
        address |= (bus << 16);
    }
    
    /* Note this is NOT shifted left by 2 as per the above comment */
    address |= offset & 0x7f;
    address |= ((function & 0x03) << 8);
    
    return address;
    
}


/* end of file */
