/* pci.h
*
*------------------------------------------------------------
*                                                                      
*                  I N T E L   P R O P R I E T A R Y                   
*                                                                      
*     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
*     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
*     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
*     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
*     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
*     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
*     THE PRIOR WRITTEN PERMISSION OF :                                
*                                                                      
*                        INTEL  CORPORATION                            
*                                                                     
*                     2200 MISSION COLLEGE BLVD                        
*                                                                      
*               SANTA  CLARA,  CALIFORNIA  95052-8119                  
*                                                                      
*------------------------------------------------------------
* system: SA1200
* subsystem: uHAL
* author: Henry Qian 04/14/99
* revisions:
*
*/

#ifndef PCI_H
#define PCI_H

/* General ERROR/SUCCESSFUL returns */
#define PCI_FAILURE                -1
#define PCI_SUCCESS                 0

#define PCI_TRUE                    1
#define PCI_FALSE                   0

/* PCI errors */
#define PCI_ERROR_BAD_BUS        0x8580 
#define PCI_ERROR_BAD_DEVICE     0x8581 
#define PCI_ERROR_BAD_FUNC       0x8582
#define PCI_ERROR_BAD_OFFSET     0x8583

#define PCI_CFG_TYPE_0  0xFB000000                                                                        
#define PCI_CFG_TYPE_1  0xFA000000

/* Offsets used to convert bus, device & function to config (and back) */
#define PCI_BUSOFFSET   16
#define PCI_DEVOFFSET   11
#define PCI_FUNCOFFSET   8

/* Bit masks used to convert bus, device & function to config */
#define PCI_BUSMASK 0xFF
#define PCI_DEVMASK 0x1F
#define PCI_FUNCMASK    0x7

#define GET_BUS(x)      ((x >> PCI_BUSOFFSET) & PCI_BUSMASK) 
#define GET_DEVICE(x)   ((x >> PCI_DEVOFFSET) & PCI_DEVMASK) 
#define GET_FUNCTION(x) ((x >> PCI_FUNCOFFSET) & PCI_FUNCMASK) 

#define MAKE_HANDLE(b,x,y,z)    (unsigned int)((unsigned int)b | \
    (unsigned int)((x & PCI_BUSMASK) << PCI_BUSOFFSET) | \
    (unsigned int)((y & PCI_DEVMASK) << PCI_DEVOFFSET) | \
(unsigned int)((z & PCI_FUNCMASK) << PCI_FUNCOFFSET))

#define MAX_PCI_BUS         1
#define MAX_PCI_SLOT        11


#define PCI_VENDOR_ID       0x00
#define PCI_DEVICE_ID       0x02
#define PCI_COMMAND         0x04
#define PCI_STATUS          0x06
#define PCI_REV_ID          0x08
#define PCI_CLASS_ID        0x09
#define PCI_MEM_BAR         0x10
#define PCI_ROM_BAR         0x30
#define PCI_LATENCY         0x0D
#define PCI_MAX_BAR         6

/*  PCI Command Register
*/
#define PCI_IO_ENABLE       0x0001
#define PCI_MEM_ENABLE      0x0002
#define PCI_MASTER_ENABLE   0x0004
#define PCI_MEM_WRINVALID   0x0010
#define PCI_FAST_B2B        0x0200

/* Another workaround the Assembler limitation. PCI_CFN_INIT should be:
* PCI_CFN_INIT EQU (PCI_IO_ENABLE | PCI_MEM_ENABLE | PCI_MASTER_ENABLE | PCI_MEM_WRINVALID)
*/
#define PCI_CFN_ENABLE      (PCI_IO_ENABLE | PCI_MEM_ENABLE)
#define PCI_MAS_ENABLE      (PCI_MEM_WRINVALID | PCI_MASTER_ENABLE)
#define PCI_CFN_INIT        (PCI_CFN_ENABLE | PCI_MAS_ENABLE)

/*  Initial PCI Interrupt ID is 1.
 */
#define PCI_INT_ID          0x100

/*  PCI Outbound Interrupt Status/Mask
*/
#define PCI_OUT_DOORBELL    0x04
#define PCI_OUT_POSTLIST    0x08
#define NO_OUTBOUND_INTS    (PCI_OUT_DOORBELL | PCI_OUT_POSTLIST)

#define PCI_INVALID         0xFFFFFFFF
#define PCI_NOTFITTED       0xFFFF

/* PCI Class Codes */
#define PCI_HISTORY_CLASS   0x00
#define PCI_STORAGE_CLASS   0x01
#define PCI_NETWORK_CLASS   0x02
#define PCI_DISPLAY_CLASS   0x03
#define PCI_MMEDIA_CLASS    0x04
#define PCI_MEMORY_CLASS    0x05
#define PCI_BRIDGE_CLASS    0x06
#define PCI_COMMS_CLASS     0x07
#define PCI_PERIPH_CLASS    0x08
#define PCI_INPUT_CLASS     0x09
#define PCI_DOCKING_CLASS   0x0A
#define PCI_PROCESS_CLASS   0x0B
#define PCI_SERIAL_CLASS    0x0C
#define PCI_MISC_CLASS      0xFF

/* PCI Sub-class codes */
#define PCI_OLDPCI_SUB      0x00
#define PCI_OLDVGA_SUB      0x01

#define PCI_SCSI_SUB        0x00
#define PCI_IDE_SUB         0x01
#define PCI_FLOPPY_SUB      0x02
#define PCI_IPI_SUB         0x03
#define PCI_RAID_SUB        0x04

#define PCI_ETHER_SUB       0x00
#define PCI_TOKEN_SUB       0x01
#define PCI_FDDI_SUB        0x02
#define PCI_ATM_SUB         0x03

#define PCI_VGA_SUB         0x00    /* v Different interface byte */
#define PCI_8541_SUB        0x00    /* ^ Different interface byte */
#define PCI_XGA_SUB         0x01

#define PCI_VIDEO_SUB       0x00
#define PCI_AUDIO_SUB       0x01

#define PCI_RAM_SUB         0x00
#define PCI_FLASH_SUB       0x01

#define PCI_HOST_SUB        0x00
#define PCI_2ISA_SUB        0x01
#define PCI_2EISA_SUB       0x02
#define PCI_2MCA_SUB        0x03
#define PCI_2PCI_SUB        0x04
#define PCI_2PCMCIA_SUB     0x05
#define PCI_2NUBUS_SUB      0x06
#define PCI_2CARDBUS_SUB    0x07

#define PCI_XTSER_SUB       0x00
#define PCI_16450_SUB       0x01    /* Interface is 0x01 */
#define PCI_16550_SUB       0x01    /* Interface is 0x02 */
#define PCI_PARALLEL_SUB    0x02    /* Interface is 0x00 */
#define PCI_LPT_SUB         0x02    /* Interface is 0x01 */
#define PCI_ECP_SUB         0x02    /* Interface is 0x02 */

#define PCI_8259_SUB        0x00    /* Interface is 0x00 */
#define PCI_PICISA_SUB      0x00    /* Interface is 0x01 */
#define PCI_PICEISA_SUB     0x00    /* Interface is 0x02 */
#define PCI_8237_SUB        0x01    /* Interface is 0x00 */
#define PCI_DMAISA_SUB      0x01    /* Interface is 0x01 */
#define PCI_DMAEISA_SUB     0x01    /* Interface is 0x02 */
#define PCI_8254_SUB        0x02    /* Interface is 0x00 */
#define PCI_TIMERISA_SUB    0x02    /* Interface is 0x01 */
#define PCI_TIMEREISA_SUB   0x02    /* Interface is 0x02 */
#define PCI_RTC_SUB         0x03    /* Interface is 0x00 */
#define PCI_RTCISA_SUB      0x03    /* Interface is 0x01 */

#define PCI_KEYBD_SUB       0x00
#define PCI_PEN_SUB         0x01
#define PCI_MOUSE_SUB       0x02

#define PCI_DOCK_SUB        0x00

#define PCI_386_SUB         0x00
#define PCI_486_SUB         0x01
#define PCI_PENTIUM_SUB     0x02
#define PCI_ALPHA_SUB       0x10
#define PCI_POWERPC_SUB     0x20
#define PCI_COPROC_SUB      0x40

#define PCI_1394_SUB        0x00
#define PCI_ACCESS_SUB      0x01
#define PCI_SSA_SUB         0x02
#define PCI_USB_SUB         0x03
#define PCI_FIBRE_SUB           0x04

/* A few 24-bit device classes.. */
#define PCI_ETHERNET_CLASS  (((PCI_STORAGE_CLASS<<8) + PCI_ETHER_SUB)<<8)
#define PCI_VGA_CLASS       (((PCI_DISPLAY_CLASS<<8) + PCI_VGA_SUB)<<8)
#define PCI_OLDVGA_CLASS    (((PCI_HISTORY_CLASS<<8) + PCI_OLDVGA_SUB)<<8)
#define PCI_SCSI_CLASS      (((PCI_STORAGE_CLASS<<8) + PCI_SCSI_SUB)<<8)
#define PCI_2PCI_CLASS      (((PCI_BRIDGE_CLASS<<8) + PCI_2PCI_SUB)<<8)
#define PCI_2ISA_CLASS      (((PCI_BRIDGE_CLASS<<8) + PCI_2ISA_SUB)<<8)


/*
 * A device node.  All devices in the system are represented by a devnode.
 */
#define MAX_DEVNODE 20

/* Store the size requested by each PCI BAR, because cannot read this
 * easily after initialisation.
 * NOTE: Keep the romSize straight after the memSize array.
 */
typedef struct devNodeStruct {
	struct devNodeStruct	*next ;
	unsigned int id;
	unsigned int handle;
	unsigned int intDma;
	unsigned int memSize[PCI_MAX_BAR + 1];
	unsigned int romSize ;
} devNodeType;

#define FIXED_TYPE      0x00000000
#define PCI_TYPE        0x10000000
#define XBUS_TYPE       0x20000000


/* defined in pci.c */
#ifdef __cplusplus
extern "C" {
#endif
    
    unsigned int SAir_CheckCfgParams(   unsigned int devHandle,
                                        unsigned int offset);
    unsigned char SAr_PciCfgReadByte(   unsigned int devHandle,
                                        unsigned int offset,
                                        unsigned int* pStatus);
    unsigned short SAr_PciCfgReadShort( unsigned int devHandle,
                                        unsigned int offset,
                                        unsigned int* pStatus);
    unsigned int SAr_PciCfgReadLong(    unsigned int devHandle,
                                        unsigned int offset,
                                        unsigned int* pStatus);
    void SAr_PciCfgWriteByte(   unsigned int devHandle,
                                unsigned int offset,
                                unsigned int data,
                                unsigned int* pStatus);
    void SAr_PciCfgWriteShort(  unsigned int devHandle,
                                unsigned int offset,
                                unsigned int data,
                                unsigned int* pStatus);
    void SAr_PciCfgWriteLong(   unsigned int devHandle,
                                unsigned int offset,
                                unsigned int data,
                                unsigned int* pStatus);
    void SAr_ConfigurePciDevices(void);
    
    unsigned int SAr_GetNextMemBAR( unsigned int handle,
                                    unsigned int *bar);
    unsigned int SAr_GetNextIOBAR(  unsigned int handle,
                                    unsigned int *bar);
    unsigned int SAr_FindPCIDevClass(   unsigned int devClass,
                                        unsigned int *inst);
    unsigned int SAr_FindPCIDevice( unsigned short vendor,
                                    unsigned short device,
                                    unsigned int *inst);
    unsigned int SAir_AlignAddress( unsigned int address,
                                    unsigned int boundary);
    unsigned int SAir_GetNextPciAddr(   unsigned int *base,
                                        unsigned int size);
    unsigned int SAir_AssignMemAddr(unsigned int size);
    unsigned int SAir_AssignIoAddr( unsigned int size);
    unsigned int SAir_ReadBARSize(  unsigned int handle,
                                    unsigned int *sizePtr,
                                    unsigned int offset);
    int SAir_WriteBARSize(  unsigned int handle,
                            unsigned int data,
                            unsigned int size,
                            unsigned int offset);
    void SAir_AssignResources(  unsigned int handle,
                                unsigned int *sizePtr);
    void SAr_ConfigurePciDevices(void);
    unsigned int SAr_GetNextMemBAR( unsigned int handle,
                                    unsigned int *bar);
    unsigned int SAr_GetNextIOBAR(  unsigned int handle,
                                    unsigned int *bar);
    unsigned int SAr_FindPCIDevClass(   unsigned int devClass,
                                        unsigned int *inst);
    unsigned int SAr_FindPCIDevice( unsigned short vendor,
                                    unsigned short device,
                                    unsigned int *inst);
    
    
    unsigned int SAr_PciHost(void);
    unsigned int SAir_PciMakeConfigAddress( unsigned int devHandle,
                                            unsigned int offset);

    void initDevNodes(void);
    devNodeType* SAir_AllocateDevNode(void);
    void SAr_BuildDevNodeTree(void);
    void SAr_PrintDevNodeTree(void);
    void SAr_AddDevNode(    unsigned int type,
                            unsigned int id,
                            unsigned int handle,
                            unsigned int intDma);
    unsigned int SAr_GetDevNodeHandle(  unsigned int id,
                                        unsigned int index);
    devNodeType *SAr_FindDevNode(unsigned int handle);

    char *getDeviceClass(   unsigned int id,
                            unsigned int device,
                            unsigned int devClass);
    void mapResources(  unsigned int handle,
                        unsigned int *sizePtr);


#ifdef __cplusplus
}
#endif

extern devNodeType* devNodes;

/* Aliases for direct PCI access routines */
#define SAr_GetPciVendorID(h, s) SAr_PciCfgReadShort(h, PCI_VENDOR_ID, s)
#define SAr_GetPciDeviceID(h, s) SAr_PciCfgReadShort(h, PCI_DEVICE_ID, s)

/* Class ID is top 24 bits, Revision ID is bottom 8 bits */
#define SAr_GetPciClassID(h, s)  (SAr_PciCfgReadLong(h, PCI_REV_ID, s) >> 8)
#define SAr_GetPciRevID(h, s)    SAr_PciCfgReadByte(h, PCI_REV_ID, s)
#define SAr_GetPciBAR(h, o, s)   SAr_PciCfgReadLong(h, PCI_MEM_BAR + (o * 4), s)
#define SAr_GetPciCommand(h, s)  SAr_PciCfgReadShort(h, PCI_COMMAND, s)
#define SAr_GetPciStatus(h, s)   SAr_PciCfgReadShort(h, PCI_STATUS, s)

#endif      /* PCI_H */

/* end of file */
