/* pciAccess.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: PCI Device Communication
 * author: Henry Qian 07/30/98
 * revisions:
 *
 */
#include <stdio.h>
#include "pciComm.h"
#include "assert.h"

/*
**  int Pci_ReadMemByte()
**
**  DESCRIPTION
**      Read an 8-bit data from the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_ReadMemByte(    unsigned int deviceNo,  // IN: destination device number
                        char* address,  // IN: location of device's memory to be read
                        char* value     // OUT: content of the memory location
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert((unsigned int)address < deviceSCT.memorySize);
    char* addr = PCIMemBase + deviceSCT.memoryBase + address;

    *value = *addr;

    return(PCI_SUCCESS);
};

/*
**  int Pci_WriteMemByte()
**
**  DESCRIPTION
**      Write an 8-bit data to the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_WriteMemByte(   unsigned int deviceNo, // IN: destination device number
                        char* address,  // IN: location of device's memory to be written
                        char value      // IN: value to be written
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert((unsigned int)address < deviceSCT.memorySize);
    char* addr = PCIMemBase + deviceSCT.memoryBase + address;

    *addr = value;

    return(PCI_SUCCESS);
};

/*
**  int Pci_ReadMemWord()
**
**  DESCRIPTION
**      Read a 16-bit data from the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_ReadMemWord(    unsigned int deviceNo,  // IN: destination device number
                        char* address,  // IN: location of device's memory to be read
                        short* value    // OUT: content of the memory location
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert((unsigned int)address < deviceSCT.memorySize);
    char* addr = PCIMemBase + deviceSCT.memoryBase + address;

    *value = *(short*)addr;

    return(PCI_SUCCESS);
};

/*
**  int Pci_WriteMemWord()
**
**  DESCRIPTION
**      Write a 16-bit data to the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_WriteMemWord(   unsigned int deviceNo, // IN: destination device number
                        char* address,  // IN: location of device's memory to be written
                        short value     // IN: value to be written
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert((unsigned int)address < deviceSCT.memorySize);
    char* addr = PCIMemBase + deviceSCT.memoryBase + address;

    *(short*)addr = value;

    return(PCI_SUCCESS);
};

/*
**  int Pci_ReadMemLong()
**
**  DESCRIPTION
**      Read a 32-bit data from the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_ReadMemLong(    unsigned int deviceNo,  // IN: destination device number
                        char* address,  // IN: location of device's memory to be read
                        int* value      // OUT: content of the memory location
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert((unsigned int)address < deviceSCT.memorySize);
    char* addr = PCIMemBase + deviceSCT.memoryBase + address;

    *value = *(short*)addr;

    return(PCI_SUCCESS);
};

/*
**  int Pci_WriteMemLong()
**
**  DESCRIPTION
**      Write a 32-bit data to the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_WriteMemLong(   unsigned int deviceNo, // IN: destination device number
                        char* address,  // IN: location of device's memory to be written
                        int value       // IN: value to be written
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert((unsigned int)address < deviceSCT.memorySize);
    char* addr = PCIMemBase + deviceSCT.memoryBase + address;

    *(int*)addr = value;

    return(PCI_SUCCESS);
};

/*
**  int Pci_ReadMemBlock()
**
**  DESCRIPTION
**      Read a block of 8-bit data from the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_ReadMemBlock(   unsigned int deviceNo,  // IN: destination device number
                        char* fromPciAddress,   // IN: location of device's memory to be read
                        char* toLocalAddress,   // OUT: local memory address to be written
                        unsigned int length     // IN: block length in bytes
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert(((unsigned int)fromPciAddress + length) < deviceSCT.memorySize);

    char* addr = PCIMemBase + deviceSCT.memoryBase + fromPciAddress;

    for (unsigned int ii = 0; ii < length; ii++)
        *(toLocalAddress + ii) = *(addr + ii);

    return(PCI_SUCCESS);
};

/*
**  int Pci_WriteMemBlock()
**
**  DESCRIPTION
**      Write a block of 8-bit data to the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_WriteMemBlock(  unsigned int deviceNo,  // IN: destination device number
                        char* toPciAddress,     // IN: location of device's memory to be written
                        char* fromLocalAddress, // IN: source memory address to be read
                        unsigned int length     // IN: block length in bytes
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);

    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    Assert(((unsigned int)toPciAddress + length) < deviceSCT.memorySize);

    char* addr = PCIMemBase + deviceSCT.memoryBase + toPciAddress;

    for (unsigned int ii = 0; ii < length; ii++)
        *(addr + ii) = *(fromLocalAddress + ii);

    return(PCI_SUCCESS);
};

/*
**  int Pci_ReadMailBox()
**
**  DESCRIPTION
**      Read a 32-bit data from the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_ReadMailBox(    unsigned int deviceNo,  // IN: destination device number
                        int mailboxNo,  // IN: mailbox number: {0, 1, 2, 3}
                        int* value      // OUT: content of the mailbox
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);
    
    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    unsigned int addr;
    switch (mailboxNo)
    {
        case 0:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_0;
            break;

        case 1:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_1;
            break;

        case 2:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_2;
            break;

        case 3:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_3;
            break;

        default:
            Assert(0); // error
    }

    *value = *(int*)addr;

    return(PCI_SUCCESS);
};

/*
**  int Pci_WriteMailBox()
**
**  DESCRIPTION
**      Write a 32-bit data to the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_WriteMailBox(   unsigned int deviceNo, // IN: destination device number
                        int mailboxNo,  // IN: mailbox number: {0, 1, 2, 3}
                        int value       // IN: value to be written
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);
    
    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];
    unsigned int addr;
    switch (mailboxNo)
    {
        case 0:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_0;
            break;

        case 1:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_1;
            break;

        case 2:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_2;
            break;

        case 3:
            addr = PCIMemBase + deviceSCT.CSR_Base + MAILBOX_3;
            break;

        default:
            Assert(0); // error
    }

    *(int*)addr = value;

    return(PCI_SUCCESS);
};

/*
**  int Pci_ReadDoorBellReg()
**
**  DESCRIPTION
**      Read a 32-bit data from the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**      The initial value of the doorbell register is 0xFFFFFFF.
**      0 bit is assert bit and 1 is de-assert bit.**  RETURNS
**
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_ReadDoorBellReg(    unsigned int deviceNo,  // IN: destination device number
                            int* value      // OUT: content of the doorbell register
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);
    
    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];

    *value = *(int*)(PCIMemBase + deviceSCT.CSR_Base + DOORBELL);

    return(PCI_SUCCESS);
};

/*
**  int Pci_WriteDoorBellReg()
**
**  DESCRIPTION
**      Write a 32-bit data to the specified device.
**      The address will be mapped to device's memory address
**      in PCI memory space.  The System Configuration Table
**      is used for the translation.
** 
**      The initial value of the doorbell register is 0xFFFFFFFF.
**      Writing to this register from PCI to SA1200 is 1-to-clear
**      (assert), and from SA1200 is 1-to-set (de-assert).
**
**      Writing 0 bits to the register has no effect.
**      Any 1's pattern value writing from PCI to this register
**      (assert) will cause SA1200 interrupt.  If PCI interrupt
**      is enabled and PCI ISR installed, PCI ISR will be invoked.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_WriteDoorBellReg(   unsigned int deviceNo, // IN: destination device number
                            int value   // IN: value to be written
)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    Assert(Pci_SystemConfigTableDone);
    
    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];

    *(int*)(PCIMemBase + deviceSCT.CSR_Base + DOORBELL) = value;

    return(PCI_SUCCESS);
};


// end of file
