/* pciComm.h
 *
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: PCI Device Communication
 * author: Henry Qian 07/30/98
 * revisions:
 *
 */

#ifndef PCI_COMM_H
#define PCI_COMM_H

#include <semLib.h>
#include "platform.h"
#include "pciCommAPI.h"

#define PCI_DEBUG

const unsigned int PCI_HOST_VENDOR_ID = 0x8086;
const unsigned int PCI_HOST_DEVICE_ID = 0x1200;

const unsigned int PCI_TARGET_VENDOR_ID = 0x8086;
const unsigned int PCI_TARGET_DEVICE_ID = 0x1200;

const unsigned int PCI_MEMORY_BASE = PCIMemBase;
const unsigned int PCI_CSR_BASE = PCIIOBase;
const unsigned int PCI_MAX_DEVICES = 4; // in PCI system, including host
const unsigned int PCI_MAX_Q_MSG = 4;   // number of messages in a queue

/* we're using only the lower 30 bits of the door-bell register */
#define PCI_DBELL_MASK		0x7FFFFFFF


// Handshake States between Host and Devices
//
const unsigned int PCI_CFG_STEP_0_DONE = 0;
const unsigned int PCI_CFG_STEP_1_DONE = 1;
const unsigned int PCI_CFG_STEP_2_DONE = 2;
const unsigned int PCI_CFG_STEP_3_DONE = 3;
const unsigned int PCI_CFG_STEP_4_DONE = 4;
const unsigned int PCI_CFG_STEP_5_DONE = 5;
const unsigned int PCI_CFG_STEP_6_DONE = 6;
const unsigned int PCI_CFG_STEP_7_DONE = 7;
const unsigned int PCI_CFG_STEP_8_DONE = 8;
const unsigned int PCI_CFG_STEP_9_DONE = 9;

// 
//
const unsigned int PCI_INCOMING_MSG_SIGNAL = 1;
const unsigned int PCI_MSGQ_AVAILABLE_SIGNAL = 2;

// PCI System Configuration Table Entry
//
struct Device_SCT_Entry {
    unsigned int handle;
    unsigned int memoryBase;
    unsigned int memorySize;
    unsigned int CSR_Base;
    unsigned int CSR_Size;
    QueueControlBlock msgQCB[PCI_MAX_DEVICES];
};

struct InMsgQ {
    unsigned int head;
    unsigned int tail;
    unsigned int statusZero;
    unsigned int statusOne;
    ShrimpMessage queue[PCI_MAX_Q_MSG];
};

extern SEM_ID inQueueEmptySem;
extern SEM_ID outQueueFullSem[];
extern SEM_ID getAccessSem;
extern SEM_ID putAccessSem[];


// ----------------
// global variables
// ----------------

// PCI System Configuration Table
//
extern Device_SCT_Entry* Pci_SystemConfigTable;

extern unsigned int Pci_SystemConfigTableSize;

extern unsigned int Pci_SystemConfigTableChecksum;

// Incoming Message Queue Array
//
extern InMsgQ* Pci_InMsgQ;

extern QueueControlBlock Pci_QueueCB[PCI_MAX_DEVICES];

extern unsigned int Pci_QueueCBChecksum;

// Configuration Status
//
extern unsigned int Pci_SystemConfigTableDone;

extern unsigned int Pci_InitDone;

#endif PCI_COMM_H


// end of file
