/* pciInit.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: IXP1200
 * subsystem: PCI Device Communication
 * author: Henry Qian 07/30/98
 * revisions:
 *
 */
#include <stdio.h>
#include <taskLib.h>
#include <cacheLib.h>
#include <intLib.h>
#include <dllLib.h>
#include <pciIomapLib.h>
#include "ixp1200Pci.h"
#include "pciComm.h"
#include "pci.h"
#include "assert.h"
#include "platform.h"



// Global Variables for PCI Communication
//
Device_SCT_Entry* Pci_SystemConfigTable;
unsigned int Pci_SystemConfigTableSize;
QueueControlBlock Pci_QueueCB[PCI_MAX_DEVICES];
unsigned int Pci_QueueCBChecksum;
unsigned int Pci_SystemConfigTableChecksum;
InMsgQ* Pci_InMsgQ;

SEM_ID inQueueEmptySem;
SEM_ID outQueueFullSem[PCI_MAX_DEVICES];
SEM_ID getAccessSem;
SEM_ID putAccessSem[PCI_MAX_DEVICES];

unsigned int Pci_SystemConfigTableDone;
unsigned int Pci_NumOfDevices;
unsigned int Pci_DeviceID;
unsigned int Pci_InitDone;

/*
**  int Pci_TestQueue(void);
**
**  DESCRIPTION
**      Test all message queues in system.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_TestQueue(void)
{
    int retval;
    unsigned int counter = 0;
    unsigned int msgIndex;
    unsigned int devIndex;
    unsigned int queueIndex;

    ShrimpMessage msg;
    QueueControlBlock queueControlBlock;

    // Make sure all queues are empty
    //
    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
        for (queueIndex = 0; queueIndex < Pci_NumOfDevices; queueIndex++){
            retval = Pci_GetQueueCB(devIndex, queueIndex, &queueControlBlock);
            Assert(retval == PCI_SUCCESS);
#ifdef PCI_DEBUG
            retval = Pci_QueueShow(devIndex, queueIndex);
            Assert(retval == PCI_SUCCESS);
#endif
            retval = Pci_QueueEmpty(&queueControlBlock);
            Assert(retval == PCI_MSGQ_EMPTY);
        }
    }

    // Put maximum number messages into each queue.
    //
    for (msgIndex = 0; msgIndex < PCI_MAX_Q_MSG - 1; msgIndex++){
        for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
            for (queueIndex = 0; queueIndex < Pci_NumOfDevices; queueIndex++){
                msg.word_0 = counter++;
                msg.word_1 = counter++;
                msg.word_2 = counter++;
                msg.word_3 = counter++;

                retval = Pci_QPutMsg(devIndex, queueIndex, &msg);
                Assert(retval == PCI_SUCCESS);
            }
        }
    }

    // Add one more message to each queue to test overflow handling.
    //
    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
        for (queueIndex = 0; queueIndex < Pci_NumOfDevices; queueIndex++){
            msg.word_0 = counter++;
            msg.word_1 = counter++;
            msg.word_2 = counter++;
            msg.word_3 = counter++;

            retval = Pci_QPutMsg(devIndex, queueIndex, &msg);
            Assert(retval == PCI_MSGQ_FULL);
        }
    }

#ifdef PCI_DEBUG
    retval = Pci_AllQueueShow();
    Assert(retval == PCI_SUCCESS);
#endif

    // Get all messages out of each queue.
    //
    counter = 0;
    for (msgIndex = 0; msgIndex < PCI_MAX_Q_MSG - 1; msgIndex++){
        for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
            for (queueIndex = 0; queueIndex < Pci_NumOfDevices; queueIndex++){
                retval = Pci_QGetMsg(devIndex, queueIndex, &msg);
                Assert(retval == PCI_SUCCESS);

                Assert(msg.word_0 == counter++);
                Assert(msg.word_1 == counter++);
                Assert(msg.word_2 == counter++);
                Assert(msg.word_3 == counter++);
            }
        }
    }

    // Try to get one more message from each queue
    // to test overflow handling.
    //
    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
        for (queueIndex = 0; queueIndex < Pci_NumOfDevices; queueIndex++){
            msg.word_0 = counter++;
            msg.word_1 = counter++;
            msg.word_2 = counter++;
            msg.word_3 = counter++;

            retval = Pci_QGetMsg(devIndex, queueIndex, &msg);
            Assert(retval == PCI_MSGQ_EMPTY);
        }
    }

    // Make sure all queues are empty
    //
    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
        for (queueIndex = 0; queueIndex < Pci_NumOfDevices; queueIndex++){
            retval = Pci_GetQueueCB(devIndex, queueIndex, &queueControlBlock);
            Assert(retval == PCI_SUCCESS);

            retval = Pci_QueueEmpty(&queueControlBlock);
            Assert(retval == PCI_MSGQ_EMPTY);
        }
    }


    return(PCI_SUCCESS);
}


/*
**  int Pci_Init()
**
**  DESCRIPTION
**      Depend on current device is a host or target,
**      call Pci_DeviceConfig() or Pci_HostConfig()
**      respectively.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_Init(void)
{
    int retval;

	if(Pci_InitDone == PCI_TRUE) return(PCI_SUCCESS);

    Pci_InitDone = PCI_FALSE;

    // Allocate and clear PCI System Configuration Table
    Pci_SystemConfigTableSize = PCI_MAX_DEVICES * sizeof(Device_SCT_Entry);
    Pci_SystemConfigTable = (Device_SCT_Entry*)malloc(Pci_SystemConfigTableSize);
    Assert(Pci_SystemConfigTable != 0);


    for (int ii = 0; ii < Pci_SystemConfigTableSize; ii++)
        ((unsigned char*)Pci_SystemConfigTable)[ii] = 0;


    cacheFlush(DATA_CACHE, Pci_SystemConfigTable, Pci_SystemConfigTableSize - 1);


	if (SAr_PciHost() == PCI_TRUE){
       retval = Pci_HostConfig();
	}
	else{
        retval = Pci_DeviceConfig();
	}

    Pci_InitDone = PCI_TRUE;
    return(retval);
}



/*
**  int Pci_DeviceConfig ( );
**
**  DESCRIPTION
**  This function configures device to be PCI connection ready.  It does:
**
**  1)  Make its memory area available to be seen by other
**      PCI devices on the bus.
**  2)  Make its Control Status Register (CSR) area available to be seen
**      by other PCI devices on the bus.
**  3)  Determine the locations and sizes of its incoming message queues.
**  4)  Create semaphores for each message queue.  The semaphores are used
**      to indicate message availability in the queues.
**      RTOS thread can wait on the semaphores for the incoming messages.
**  5)  Install a PCI interrupt Service Routine (ISR).
**  6)  Set Initialize Complete bit in the Control Register.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/

int Pci_DeviceConfig(void)
{
    int retval;

    Pci_SystemConfigTableDone = 0;


    /*
    ***********************************************
    ** Set PCI Control and Status Registers
    ***********************************************
    */

    // Don't respond to any commands
    //
    *(unsigned int*)(CSR_BASE + PCI_COMMAND)  = 0;

    // Disable PCI Outbound interrupts
    //
    *(unsigned int*)(CSR_BASE + PCI_OUT_INT_MASK) = NO_OUTBOUND_INTS;

    // Clear mailboxes
    //
	*(unsigned int*)(CSR_BASE + MAILBOX_0) = 0;
	*(unsigned int*)(CSR_BASE + MAILBOX_1) = 0;
	*(unsigned int*)(CSR_BASE + MAILBOX_2) = 0;
	*(unsigned int*)(CSR_BASE + MAILBOX_3) = 0;



    // All bits are used for PCI-to-SA notifications
    //
	*(unsigned int*)(CSR_BASE + DOORBELL_SETUP) = PCI_DBELL_MASK;


    /*
    ***********************************************
    ** Set IXP1200 Control and Status Registers
    ***********************************************
    */

   // Remove PCI_reset
    //
    *(unsigned int*)(CSR_BASE + SA_CONTROL) |= PCI_NRESET;


    // Doorbell register is not used for output interrupt to PCI.
    // It is used as an internal signal to interrupt IXP1200
    //
	*(unsigned int*)(CSR_BASE + DBELL_PCI_MASK) = 0;
	*(unsigned int*)(CSR_BASE + DBELL_SA_MASK)  = PCI_DBELL_MASK;


    /*
    ***********************************************
    ** Set PCI Configuration Registers
    ***********************************************
    */

    // Set Interrupt ID to 1 - fixes problems on systems which cannot
    // handle interrupt 0.
    //
    *(unsigned int*)(CSR_BASE + PCI_INT_LINE) = PCI_INT_ID;


    // Tell host that device has done its own register initialization
    // At same time, let host know where device stores it SCT table.
    // Later the host will copy the complete SCT over.
    //
    // OS_DRAM_OFFSET: IXP1200 board DRAM starts at 0xC0000000,
    // is not entirely mapped to address 0.  It maps the physical
    // address 0xC1800000 (for example) to virtural address 0.
    // It leaves DRAM block 0xC0000000:0xC1800000 to exclusive use
    // by reference design.
    // So when we pass a virtural address to another device,
    // the other device would access the physical address through
    // PCI space by 'PCIMemBase + OS_DRAM_OFFSET + the address'.
    //
    Pci_DeviceSendMail(PCI_CFG_STEP_1_DONE, (unsigned int)Pci_SystemConfigTable, OS_DRAM_OFFSET);


    // Wait for host tell us how many devices (include host) in the
    // system.  We build a number of message queues based on this number.
    //
    Pci_DeviceWaitMail(PCI_CFG_STEP_2_DONE, &Pci_NumOfDevices, &Pci_DeviceID);

    // There should be at least 2 devices (host and me)
    //
    Assert((Pci_NumOfDevices >= 2) && (Pci_NumOfDevices < PCI_MAX_DEVICES));

    Assert((Pci_DeviceID >= 1) && (Pci_DeviceID < PCI_MAX_DEVICES));

#ifdef PCI_DEBUG
    printf("\n");
    printf(" -- Pci_NumOfDevices: %d\n", Pci_NumOfDevices);
    printf(" -- Pci_DeviceID: %d\n", Pci_DeviceID);
#endif

    // Semaphore 'inQueueEmptySem' is used to wait for
    // any incoming message on all queues.
    inQueueEmptySem = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    Assert(inQueueEmptySem != 0);

    getAccessSem = semBCreate(SEM_Q_FIFO, SEM_FULL);
    Assert(getAccessSem != 0);

    // Create semaphores for outgoing messages.
    // The semaphores are used for BLOCK call mode.
    // When the destination message queue is full and not available,
    // the calling functions can sit and wait on these semaphores.
    //
    unsigned int devIndex;
    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
        // These are for other devices on the bus.
        // Not for our own device.
        if (devIndex != Pci_DeviceID){
            outQueueFullSem[devIndex] = semBCreate(SEM_Q_FIFO, SEM_FULL);
            Assert(outQueueFullSem[devIndex] != 0);

            putAccessSem[devIndex] = semBCreate(SEM_Q_FIFO, SEM_FULL);
            Assert(putAccessSem[devIndex] != 0);
        }
        else {
            // These are not used. Zeroed out for deterministic.
            outQueueFullSem[devIndex] = 0;
            putAccessSem[devIndex] = 0;
        }
    }

    retval = Pci_MsgQInit(Pci_NumOfDevices, Pci_QueueCB);
    Assert(retval == PCI_SUCCESS);

    // Generate checksum of Pci_QueueCB
    //
    Pci_QueueCBChecksum = Pci_CheckSum((char*)Pci_QueueCB, sizeof(Pci_QueueCB));

#ifdef PCI_DEBUG
    printf(" -- Local Queue --\n");
    Pci_ShowQueueCB(Pci_QueueCB);
#endif

    // Write out data from cache to memory
    cacheFlush(DATA_CACHE, Pci_QueueCB, sizeof(Pci_QueueCB) - 1);

    // Send Pci_QueueCB to host with its checksum
    //
    Pci_DeviceSendMail(PCI_CFG_STEP_3_DONE, (unsigned int)Pci_QueueCB, Pci_QueueCBChecksum);
#ifdef PCI_DEBUG
    printf(" -- Pci_QueueCB: %08X\n", Pci_QueueCB);
    printf(" -- Pci_QueueCBChecksum: %08X\n", Pci_QueueCBChecksum);
#endif

    // Wait for host to copy the System Config Table over
    // and send us the checksum.
    //
    unsigned int deviceID;
    Pci_DeviceWaitMail( PCI_CFG_STEP_4_DONE, &deviceID, &Pci_SystemConfigTableChecksum);
    Assert(deviceID == Pci_DeviceID); // sanity check

#ifdef PCI_DEBUG
    printf(" -- Pci_SystemConfigTable: %08X\n", Pci_SystemConfigTable);
    printf(" -- Pci_SystemConfigTableChecksum: %08X\n", Pci_SystemConfigTableChecksum);
#endif

    // Make sure to read from memory
    cacheDInvalidate(Pci_SystemConfigTable);
    cacheDInvalidate(Pci_SystemConfigTable + Pci_SystemConfigTableSize - 1);

    unsigned int checksum = Pci_CheckSum((char*)Pci_SystemConfigTable, Pci_SystemConfigTableSize);
    Assert(checksum == Pci_SystemConfigTableChecksum);

    // The table is completely constructed
    //
    Pci_SystemConfigTableDone = 1;

    // Ack: received a good copy of SCT (System Control Table).
    // Device configuration done.
    //
    Pci_DeviceSendMail(PCI_CFG_STEP_5_DONE, 0, Pci_DeviceID);

#ifdef PCI_DEBUG
    printf(" <<< Complete System Config Table >>>\n");
    Pci_ShowSystemConfigTable();
#endif

    /*
    ***********************************************
    ** Complete message queue test (without interrupt)
    ***********************************************
    */
    Pci_DeviceWaitMail( PCI_CFG_STEP_6_DONE, &deviceID, 0);
    Assert(deviceID == Pci_DeviceID); // sanity check
    retval = Pci_TestQueue();
    Assert(retval == PCI_SUCCESS);
    Pci_DeviceSendMail( PCI_CFG_STEP_7_DONE, 0, Pci_DeviceID);

    /*
    ***********************************************
    ** PCI Configuration Step 7
    ***********************************************
    */
#ifdef PCI_DEBUG
        printf(" -- All Test Done --\n");
#endif

    // Install PCI doorbell interrupt handler with OS
    //
    intDisable(INT_LVL_DOORBELL);
//    retval = intConnect((void(**)(...))INT_LVL_DOORBELL,(void(*)(...))Pci_DoorBell_ISR, 0);
    retval = pciIntConnect((void(**)(...))INT_LVL_DOORBELL,(void(*)(...))Pci_DoorBell_ISR, 0);
	Assert(retval == OK);
    intEnable(INT_LVL_DOORBELL);

    return(Pci_DeviceID);
}



/*
**  int Pci_HostConfig()
**
**  DESCRIPTION
**	    This function does the PCI software configuration.  It does:
**
**      1)  Configure all the devices (setup various devices' CSR registers)
**          on the bus.
**      2)  Create a PCI memory space (see Figure 2).
**      3)  Create the System Configuration Table (see Figure 4) and copy
**          it to all the devices on the bus.
**      4)  Do a brief PCI system integrity test, which involves PCI
**          memory map, System Configuration Table, device's message
**          queues and PCI interrupts.
**      5)  When host configuration is finished the message queues and
**          interrupts are ready to use.
**
**  RETURNS
**      PCI_SUCCESS     the function succeeded.
**      PCI_ERROR       the function failed.
*/
int Pci_HostConfig(void)
{
    int retval;
    unsigned int jj;
    unsigned int devIndex;
    unsigned int devHandle;
    unsigned int instance;
    unsigned int CSR_Base;
    unsigned int value;
    unsigned int status;
    Device_SCT_Entry* deviceSCT_Address[PCI_MAX_DEVICES];
    unsigned int deviceSCT_Size;
	PciDevice *pciDevice;

    // Host has a device ID zero
    //
    Pci_DeviceID = 0;


    /*
    ***********************************************
    ** Set PCI Control and Status Registers
    ***********************************************
    */
    // Disable PCI Outbound interrupts
    //
    *(unsigned int*)(CSR_BASE + PCI_OUT_INT_MASK) = NO_OUTBOUND_INTS;

    // All bits are used for PCI-to-SA notifications
    //
	*(unsigned int*)(CSR_BASE + DOORBELL_SETUP) = PCI_DBELL_MASK;

    // Doorbell register is not used for output interrupt to PCI.
    // It is used as an internal signal to interrupt IXP1200
    //
	*(unsigned int*)(CSR_BASE + DBELL_PCI_MASK) = 0;
	*(unsigned int*)(CSR_BASE + DBELL_SA_MASK)  = PCI_DBELL_MASK;

    // clear mailboxes
    //
    *(unsigned int *)(CSR_BASE + MAILBOX_0) = 0;
    *(unsigned int *)(CSR_BASE + MAILBOX_1) = 0;
    *(unsigned int *)(CSR_BASE + MAILBOX_2) = 0;
    *(unsigned int *)(CSR_BASE + MAILBOX_3) = 0;

    
	// The table is not constructed yet.
    //
    Pci_SystemConfigTableDone = 0;

    /* Build PCI bus device tree */
    SAr_BuildDevNodeTree();

#ifdef PCI_DEBUG
    SAr_PrintDevNodeTree();
#endif

    // To find host node on bus (this board),
    // which must be the first node of the tree.
    //
    instance = Pci_DeviceID;
    devHandle = SAr_FindPCIDevice(PCI_HOST_VENDOR_ID, PCI_HOST_DEVICE_ID, &instance);
    Assert((devHandle != 0) && (GET_BUS(devHandle) == 0) &&
		   (GET_DEVICE(devHandle) == 0) && (GET_FUNCTION(devHandle) == 0));

    Pci_SystemConfigTable[Pci_DeviceID].handle = devHandle;

    // Get the Host configuration
	pciDevice = pciInfoGet(Pci_DeviceID);
    Pci_SystemConfigTable[Pci_DeviceID].memoryBase = OS_DRAM_OFFSET + pciDevice->bar[SD_BAR].address;
    Pci_SystemConfigTable[Pci_DeviceID].memorySize = pciDevice->bar[SD_BAR].size;

    Pci_SystemConfigTable[Pci_DeviceID].CSR_Base = pciDevice->bar[CSR_BAR].address;
    Pci_SystemConfigTable[Pci_DeviceID].CSR_Size = pciDevice->bar[CSR_BAR].size;


    // Get the number of devices in the system and their handles
    //
    for (devIndex = 1; devIndex < PCI_MAX_DEVICES; devIndex++){
        // Find the target boards -- all IXP1200 boards.
        devHandle = SAr_FindPCIDevice(PCI_TARGET_VENDOR_ID, PCI_TARGET_DEVICE_ID, &instance);
        if (devHandle == 0) break;	// end search

        Pci_SystemConfigTable[devIndex].handle = devHandle;
    }
    Pci_NumOfDevices = devIndex;


    // Create Host's incoming message queues
    //
    retval = Pci_MsgQInit(Pci_NumOfDevices, Pci_QueueCB);
    Assert(retval == PCI_SUCCESS);

#ifdef PCI_DEBUG
    printf(" -- Local queue --\n");
    Pci_ShowQueueCB(Pci_QueueCB);
#endif

    // Fill in System Configuration Table host's entry
    //
    for (jj = 0; jj < Pci_NumOfDevices; jj++)
        Pci_SystemConfigTable[Pci_DeviceID].msgQCB[jj] = Pci_QueueCB[jj];

    unsigned int OsDRamOffset;
    for (devIndex = 1; devIndex < Pci_NumOfDevices; devIndex++){
		// Wait for the target to complete PCI_CFG_STEP_1_DONE -- configure
		// themselve and send their SDRAM offset
        Pci_HostWaitMail(devIndex, PCI_CFG_STEP_1_DONE,
                         (unsigned int*)&deviceSCT_Address[devIndex], &OsDRamOffset);
        Pci_SystemConfigTable[devIndex].memoryBase = OsDRamOffset;


		// Inform target that host has completed PCI_CFG_STEP_2_DONE -- retreived their
		// SDRAM offset.  Also, inform them on the number of 'targets', includng the host,
		// and assign their device ID.
        Pci_HostSendMail(devIndex, PCI_CFG_STEP_2_DONE, Pci_NumOfDevices, devIndex);
    }


    // Get the target boards configuration from the OS
    //
	devIndex = 1;
    for (int i = 1; i < PCI_MAX_DEVICES; i++){

		if((pciDevice = pciInfoGet(i)) == NULL) break;

		if((pciDevice->error) || (pciDevice->vendor_id != PCI_TARGET_VENDOR_ID) ||
           (pciDevice->device_id != PCI_TARGET_DEVICE_ID)) continue;

        devHandle = Pci_SystemConfigTable[devIndex].handle;


        // Setup device entry in SystemConfigTable
        //
		Pci_SystemConfigTable[devIndex].memoryBase += pciDevice->bar[SD_BAR].address;
		Pci_SystemConfigTable[devIndex].memorySize = pciDevice->bar[SD_BAR].size;

		Pci_SystemConfigTable[devIndex].CSR_Base = pciDevice->bar[CSR_BAR].address;
		Pci_SystemConfigTable[devIndex].CSR_Size = pciDevice->bar[CSR_BAR].size;

        // enable PCI memory access
        //
        value = SAr_PciCfgReadLong(devHandle, PCI_COMMAND, &status);
        Assert(status == PCI_SUCCESS);

        SAr_PciCfgWriteLong(devHandle, PCI_COMMAND, value | PCI_MEM_ENABLE |
							PCI_MASTER_ENABLE | PCI_MEM_WRINVALID, &status);
        Assert(status == PCI_SUCCESS);
		devIndex++;
    }

    // The Table half constructed.  BARs done.
    //
    Pci_SystemConfigTableDone = 1;

#ifdef PCI_DEBUG
    // The Table is half constructed.  BARs done.
    //
    printf(" <<< Partially constructed System Config Table >>>\n");
    Pci_ShowSystemConfigTable();
#endif

    // Make sure all devices have PCI_CFG_STEP_3_DONE
    //
    unsigned int queueCBPtr;
    for (devIndex = 1; devIndex < Pci_NumOfDevices; devIndex++){
        // Wait and receive each device's message queue
        // control block
        //
        Pci_HostWaitMail(devIndex, PCI_CFG_STEP_3_DONE, &queueCBPtr, &Pci_QueueCBChecksum);
#ifdef PCI_DEBUG
        printf(" -- Received qcb(%08X) from dev %d chksum: %08X\n",
                     queueCBPtr, devIndex, Pci_QueueCBChecksum);
        printf(" -- OsDRamOffset: %08X\n", OsDRamOffset);
#endif

        // Copy the device's queue control block over PCI bus
        //
        retval = Pci_ReadMemBlock(devIndex, (char*)queueCBPtr, (char*)Pci_QueueCB,
								  sizeof(Pci_QueueCB));
        Assert(retval == PCI_SUCCESS);

        unsigned int checksum = Pci_CheckSum((char*)Pci_QueueCB, sizeof(Pci_QueueCB));
        Assert(Pci_QueueCBChecksum == checksum);

#ifdef PCI_DEBUG
        printf(" -- Copied qcb over and display the queue\n");
        Pci_ShowQueueCB(Pci_QueueCB);
#endif
		// store the target's qcb into the sys config table
        for (jj = 0; jj < Pci_NumOfDevices; jj++)
            Pci_SystemConfigTable[devIndex].msgQCB[jj] = Pci_QueueCB[jj];
    }

    // Semaphore 'inQueueEmptySem' is used to wait for
    // any incoming message on all queues.
    inQueueEmptySem = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    Assert(inQueueEmptySem != 0);

    getAccessSem = semBCreate(SEM_Q_FIFO, SEM_FULL);
    Assert(getAccessSem != 0);

    // Create semaphores for outgoing messages.
    // The semaphores are used for BLOCK call mode.
    // When the destination message queue is full and not available,
    // the calling functions can sit and wait on these semaphores.
    //
    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
        if (devIndex != Pci_DeviceID){
            outQueueFullSem[devIndex] = semBCreate(SEM_Q_FIFO, SEM_FULL);
            Assert(outQueueFullSem[devIndex] != 0);

            putAccessSem[devIndex] = semBCreate(SEM_Q_FIFO, SEM_FULL);
            Assert(putAccessSem[devIndex] != 0);
        }
    }


#ifdef PCI_DEBUG
    // The table is completely constructed
    //
    printf(" <<< Complete System Config Table >>>\n");
    Pci_ShowSystemConfigTable();
#endif

    // sanity check
#ifdef PCI_DEBUG
    printf(" -- Check all queues empty --\n");
#endif
    Assert(Pci_AllQueueEmpty() == PCI_SUCCESS);



    // Copy system config table over PCI to device
    // Thus each device has a local copy of system config table
    //
    Pci_SystemConfigTableChecksum = Pci_CheckSum((char*)Pci_SystemConfigTable, Pci_SystemConfigTableSize);

    for (devIndex = 1; devIndex < Pci_NumOfDevices; devIndex++){
        retval = Pci_WriteMemBlock(devIndex, (char*)deviceSCT_Address[devIndex],
                                   (char*)Pci_SystemConfigTable, Pci_SystemConfigTableSize);

        Pci_HostSendMail(devIndex, PCI_CFG_STEP_4_DONE, devIndex, Pci_SystemConfigTableChecksum);

#ifdef PCI_DEBUG
        Pci_ShowConfigStatus(devIndex, PCI_CFG_STEP_4_DONE);
#endif
    }

    unsigned int deviceNo;
    for (devIndex = 1; devIndex < Pci_NumOfDevices; devIndex++){
        Pci_HostWaitMail(devIndex, PCI_CFG_STEP_5_DONE, 0, &deviceNo);
        Assert(deviceNo == devIndex);

#ifdef PCI_DEBUG
        Pci_ShowConfigStatus(devIndex, PCI_CFG_STEP_5_DONE);
#endif
    }

#ifdef PCI_DEBUG
    printf(" -- All queues reset --\n");
#endif

    retval = Pci_AllQueueReset();
    Assert(retval == PCI_SUCCESS);

    /*
    ***********************************************
    ** Complete message queue test (without interrupt)
    ***********************************************
    */
    
    // Host tests its message queues
    //
#ifdef PCI_DEBUG
    printf(" -- All queues test --\n");
#endif

    retval = Pci_TestQueue();
    Assert(retval == PCI_SUCCESS);

    // Let each device test its own message queues
    //
    for (devIndex = 1; devIndex < Pci_NumOfDevices; devIndex++){
        Pci_HostSendMail(devIndex, PCI_CFG_STEP_6_DONE, devIndex, 0);
		Pci_HostWaitMail(devIndex, PCI_CFG_STEP_7_DONE, 0, &deviceNo);
        Assert(deviceNo == devIndex); // sanity check
    }

#ifdef PCI_DEBUG
        printf(" -- All Test Done --\n");
#endif

    // Install PCI doorbell interrupt handler with OS
    //
    intDisable(INT_LVL_DOORBELL);

	//dj: Note:- PCI interrupt should be connected via pciIntConnect whenever the BSP is changed
	// to chain interrupt on the host and add-in card.
//    retval = pciIntConnect((void(**)(...))INT_LVL_DOORBELL,(void(*)(...))Pci_DoorBell_ISR, 0);
    retval = intConnect((void(**)(...))INT_LVL_DOORBELL,(void(*)(...))Pci_DoorBell_ISR, 0);
	Assert(retval == OK);
		
    intEnable(INT_LVL_DOORBELL);

    return(PCI_SUCCESS);
}

