/* pciMail.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: PCI Device Communication
 * author: Henry Qian 07/30/98
 * revisions:
 *
 */
#include <stdio.h>
#include <taskLib.h>
#include "pciComm.h"
#include "assert.h"
#include "pci.h"

/*
**  void Pci_HostSendMail()
**  void Pci_HostWaitMail()
**  void Pci_DeviceSendMail()
**  void Pci_DeviceWaitMail()
**
**  DESCRIPTION
**      Used for initial handshake between PCI host and PCI devices
**      during PCI host configuration.
**
**  RETURNS
**      PCI_SUCCESS
**      PCI_ERROR
*/

int Pci_HostSendMail(   unsigned int deviceID,
                        unsigned int command,
                        unsigned int arg2,
                        unsigned int arg3
                        )
{
    unsigned int status;

    Assert((deviceID >= 1) && (deviceID < Pci_NumOfDevices));
    unsigned int devHandle = Pci_SystemConfigTable[deviceID].handle;
    Assert(devHandle > 0);

    SAr_PciCfgWriteLong(devHandle, MAILBOX_2, arg2, &status);
    Assert(status == PCI_SUCCESS);

    SAr_PciCfgWriteLong(devHandle, MAILBOX_3, arg3, &status);
    Assert(status == PCI_SUCCESS);

    SAr_PciCfgWriteLong(devHandle, MAILBOX_1, command, &status);
    Assert(status == PCI_SUCCESS);

    return(PCI_SUCCESS);
}

int Pci_HostWaitMail(   unsigned int deviceID,
                        unsigned int command,
                        unsigned int* arg2,
                        unsigned int* arg3
                        )
{
    unsigned int status;
    int retval;
    unsigned int value;
    unsigned int onlyOnce = 0;

    Assert((deviceID >= 1) && (deviceID < Pci_NumOfDevices));
    unsigned int devHandle = Pci_SystemConfigTable[deviceID].handle;
    Assert(devHandle > 0);

    while (1)
    {
        value = SAr_PciCfgReadLong(devHandle, MAILBOX_0, &status);
        Assert(status == PCI_SUCCESS);

        if (value == command)
            break;

        if (onlyOnce == 0)
        {
            printf(" -- wait for %x ... got %x\n", command, value);
            onlyOnce = 1;
        }

        /* wait for a while, try again */
        retval = taskDelay(5);
        Assert(retval == OK);
    }

    if (arg2 != 0)
    {
        *arg2 = SAr_PciCfgReadLong(devHandle, MAILBOX_2, &status);
        Assert(status == PCI_SUCCESS);
    }

    if (arg3 != 0)
    {
        *arg3 = SAr_PciCfgReadLong(devHandle, MAILBOX_3, &status);
        Assert(status == PCI_SUCCESS);
    }

    return(PCI_SUCCESS);
}

void Pci_DeviceSendMail(    unsigned int command,
                            unsigned int arg2,
                            unsigned int arg3
                            )
{
	*(unsigned int*)(CSR_BASE + MAILBOX_2) = arg2;
	*(unsigned int*)(CSR_BASE + MAILBOX_3) = arg3;

	*(unsigned int*)(CSR_BASE + MAILBOX_0) = command;
}

void Pci_DeviceWaitMail(    unsigned int command,
                            unsigned int* arg2,
                            unsigned int* arg3
                            )
{
    while (*(unsigned int*)(CSR_BASE + MAILBOX_1) != command)
    {
        /* wait for a while, try again */
        taskDelay(5);
    }

    if (arg2 != 0)
	    *arg2 = *(unsigned int*)(CSR_BASE + MAILBOX_2);

    if (arg3 != 0)
	    *arg3 = *(unsigned int*)(CSR_BASE + MAILBOX_3);
}


// end of file