/* pciShow.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: PCI Device Communication
 * author: Henry Qian 07/30/98
 * revisions:
 *
 */
#include <stdio.h>
#include "pciComm.h"

/*
**  void Pci_ShowConfigStatus();
**
**  DESCRIPTION
**      Display configuration stages.  The function is used
**      for test purpose.
**
**  RETURNS
**      None.
*/
void Pci_ShowConfigStatus(  unsigned int deviceNo,
                            unsigned int status)
{
    printf(" ** PCI Config Step %d done device %d **\n",
        status, deviceNo);
}


/*
**  void Pci_ShowSystemConfigTable();
**
**  DESCRIPTION
**      Display System Configuration Table and its
**      checksum.
**
**  RETURNS
**      None.
*/
void Pci_ShowSystemConfigTable(void)
{
    unsigned int devIndex;

    printf(" ==================================\n");
    printf(" --  System Configuration Table  --\n");
    printf(" ==================================\n");

    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++)
    {
        printf(" -- Device  : %8X  Handle  : %08X\n",
                    devIndex,
                    Pci_SystemConfigTable[devIndex].handle);
        printf("    MemBase : %08X  MemSize : %8X\n",
                    Pci_SystemConfigTable[devIndex].memoryBase,
                    Pci_SystemConfigTable[devIndex].memorySize);
        printf("    CSR Base: %08X  CSR Size: %8X\n",
                    Pci_SystemConfigTable[devIndex].CSR_Base,
                    Pci_SystemConfigTable[devIndex].CSR_Size);

        Pci_ShowQueueCB(Pci_SystemConfigTable[devIndex].msgQCB);
        printf(" ==================================\n");
    }

    unsigned int checksum =
        Pci_CheckSum(   (char*)Pci_SystemConfigTable,
                        Pci_SystemConfigTableSize);

    printf(" -- Total %2d device(s) calculated: %08X\n",
        Pci_NumOfDevices, checksum);
    printf(" -- Pci_SystemConfigTable: %08X\n",
                Pci_SystemConfigTable);
    printf(" -- Pci_SystemConfigTableChecksum: %08X\n",
                Pci_SystemConfigTableChecksum);
    printf(" ==================================\n");
}

/*
**  void Pci_ShowQueueCB();
**
**  DESCRIPTION
**      Display Queue Control Block and its
**      checksum.
**
**  RETURNS
**      None.
*/
void Pci_ShowQueueCB(QueueControlBlock* msgQueueCB)
{
    unsigned int devIndex;

    printf("    ~~~~~~~~~~~~~~~~~~~~~~~~~\n");
    printf("    ~~ Queue Control Block ~~\n");
    printf("    ~~~~~~~~~~~~~~~~~~~~~~~~~\n");

    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++)
    {
        printf("    ~~ Q%d pH:%08X pT:%08X pS0:%08X pS1:%08X Base:%08X\n",
            devIndex,
            msgQueueCB[devIndex].headPtr,
            msgQueueCB[devIndex].tailPtr,
            msgQueueCB[devIndex].statusZeroPtr,
            msgQueueCB[devIndex].statusOnePtr,
            msgQueueCB[devIndex].base);
    }

    unsigned int checksum = Pci_CheckSum((char*)msgQueueCB,
            sizeof(QueueControlBlock) * PCI_MAX_DEVICES);

    printf("    ~~ CheckSum: %08X\n", checksum);
}

// end of file
