/* pciSignal.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: PCI Device Communication
 * author: Henry Qian 07/30/98
 * revisions:
 *
 */
#include <stdio.h>
#include "assert.h"
#include "pciComm.h"

/*
**  int Pci_Signal();
**
**  DESCRIPTION
**      Signal other device by writing specific bit
**      into that device's doorbell reigster.  The
**      device's doorbell register is located in PCI
**      memory space.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_Signal(unsigned int deviceNo, unsigned int signal)
{
    Assert((deviceNo >= 0) && (deviceNo < Pci_NumOfDevices));
    // do not signal myself
    Assert(deviceNo != Pci_DeviceID);
    Assert(Pci_SystemConfigTableDone);
    Device_SCT_Entry deviceSCT = Pci_SystemConfigTable[deviceNo];

    // Only these signals are used
    Assert((signal == PCI_INCOMING_MSG_SIGNAL) || (signal == PCI_MSGQ_AVAILABLE_SIGNAL));

	unsigned int confAddr = (unsigned int)SAir_PciMakeConfigAddress(deviceSCT.handle, DOORBELL); 
    // Ring the doorbell and indicate that it's this device that's calling 
    *(unsigned int*)confAddr = signal << (Pci_DeviceID * 2);

    return(PCI_SUCCESS);
}


/*
**  void Pci_DoorBell_ISR (void);
**
**  DESCRIPTION:
**      This routine is registered by function Pci_DeviceConfig() or Pci_HostConfig().
**      It is a doorbell interrupt service routine.  It will be
**      invoked each time doorbell register value changes.  The
**      routine reads the doorbell register and resets it to prevent
**      further interrupt.  Depend on the value from doorbell register
**      determine where the signal came from and what action to take.
**
**  RETURNS: None.
*/

#ifdef PCI_DEBUG
unsigned int intHistory[0x1000];
unsigned int intNumber = 0;

void intHist(void)
{
	printf("IntNum = %d\n", intNumber);
	for(int i = 0; i < intNumber; i++){
		if((i % 8) == 0) printf("\n");
		printf("int %d = 0x%x,", intHistory[i]); fflush(stdout);
	}
	return;
}
#endif // PCI_DEBUG


void Pci_DoorBell_ISR(void)
{
    unsigned int doorBellReg;

	//dj: Note:-  host device owns all the dbell bits because it's
	// attached via intConnect and not pciIntConnect. This should
	// be changed when the DSP attaches dbell int on the host.
	unsigned int dbell_mask = PCI_DBELL_MASK;
	if(Pci_DeviceID == 0) dbell_mask = 0xffffffff;

	// read doorbell register and determine if it's one of our interrupts
    doorBellReg = dbell_mask & (~(*(int*)(CSR_BASE + DOORBELL)));
	if(doorBellReg == 0) return;	// it's not our interrupt

    // reset doorbell register and clear the interrupt -- write 1's to clear
    *(int*)(CSR_BASE + DOORBELL) = doorBellReg;

#ifdef PCI_DEBUG
    intHistory[(intNumber++)%0x1000] = doorBellReg;
#endif // PCI_DEBUG

    // Semaphore 'inQueueEmptySem' will be only released for
    // once for each incoming message interrupt,
    // even if there are multiple incoming messages.
    int inQueueEmptySem_is_released = PCI_FALSE;

    // determine where the signal comes from and
    // what for.
    //
    unsigned int devIndex;
    for (devIndex = 0; devIndex < Pci_NumOfDevices; devIndex++){
        if (doorBellReg & PCI_INCOMING_MSG_SIGNAL){
            // signal indicates there is an incoming message
            if (inQueueEmptySem_is_released != PCI_TRUE){
                semGive(inQueueEmptySem);
                inQueueEmptySem_is_released = PCI_TRUE;
            }
        }

        if (doorBellReg & PCI_MSGQ_AVAILABLE_SIGNAL){
            // signal indicates that device ii's
            // message queue now is available.
            semGive(outQueueFullSem[devIndex]);
        }

        // Next device.
        doorBellReg >>= 2;
    }
}


// end of file

