/* pciTest.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: PCI Device Communication
 * author: Henry Qian 07/30/98
 * revisions:
 *
 */
#include <stdio.h>
#include <vxWorks.h>
#include <taskLib.h>
#include "pciComm.h"
#include "assert.h"


// Global variables used for test
ShrimpMessage sendMsg;
ShrimpMessage recvMsg;
unsigned int serialNumber;
const int MAX_LOOPS = 100;
const int TICKS_TO_WAIT = 20;


/*
**  void showRecvMsg(void);
**  void showSendMsg(void);
**
**  DESCRIPTION
**      These routines are used at VxWorks shell to display
**      test results.
**
**  RETURNS
**      None.
*/
void showRecvMsg(void)
{
    printf(" recvMsg.word_0 = %08x\n", recvMsg.word_0);
    printf(" recvMsg.word_1 = %08x\n", recvMsg.word_1);
    printf(" recvMsg.word_2 = %08x\n", recvMsg.word_2);
    printf(" recvMsg.word_3 = %08x\n", recvMsg.word_3);
    printf(" --\n");
    fflush(stdout);
}

void showSendMsg(void)
{
    printf(" sendMsg.word_0 = %08x\n", sendMsg.word_0);
    printf(" sendMsg.word_1 = %08x\n", sendMsg.word_1);
    printf(" sendMsg.word_2 = %08x\n", sendMsg.word_2);
    printf(" sendMsg.word_3 = %08x\n", sendMsg.word_3);
    printf(" --\n");
    fflush(stdout);
}

#include <iolib.h>
void console(void)
{
    int vf0=open("/vio/0",2,0);
    ioGlobalStdSet(0,vf0);
    ioGlobalStdSet(1,vf0);
    ioGlobalStdSet(2,vf0);
}

void __gccmain()
{
}


/*
**  int Pci_TestSendFast();
**
**  DESCRIPTION
**      Send messages to another device as fast as possible.
**      Eventually it will fill up the inbound message queue
**      on the other device.  Then it will be blocked and
**      wait on a semaphore.  When the other device on the
**      other end of the PCI bus gets the message (makes room
**      available in the queue), it will inform us by doorbell
**      interrupt.  Our doorbell ISR is invoked and it releases
**      the semaphore and this function will continue until it
**      fills up the queue again.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_TestSendFast(unsigned int deviceNo)
{
    int retval;

    sendMsg.word_0 = Pci_DeviceID;
    sendMsg.word_1 = deviceNo;
    sendMsg.word_2 = 0;
    sendMsg.word_3 = 0x1000;

    while(1){
        retval = Pci_PutMsg(deviceNo, &sendMsg, PCI_CALL_BLOCK);
        Assert(retval == PCI_SUCCESS);

        (sendMsg.word_3)++;
    }

    return(PCI_SUCCESS);
}


/*
**  int Pci_TestRecvSlow();
**
**  DESCRIPTION
**      This call works with Pci_TestSendFast() function on the
**      other end of the PCI bus.  It retreaves the message from
**      the inbound queue slowly, so the other end would have a
**      chance to be blocked and we would have a chance to send
**      an interrupt to it.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_TestRecvSlow(unsigned int deviceNo)
{
    int retval;

    while(1){
        retval = Pci_GetMsg(&recvMsg, PCI_CALL_BLOCK);
        Assert(retval == PCI_SUCCESS);

        showRecvMsg();
        taskDelay(91);
    }

    return(PCI_SUCCESS);
}


/*
**  int Pci_TestSendSlow();
**
**  DESCRIPTION
**      It works with Pci_TestRecvFast() on the other end.
**      It sends message over PCI bus to the other device slowly,
**      so it gives a chance for the other device to be blocked and
**      wait on semaphore for incoming message.  And when we do send
**      message we would inform by interrupt and wake the other
**      device.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_TestSendSlow(unsigned int deviceNo)
{
    int retval;

    sendMsg.word_0 = Pci_DeviceID;
    sendMsg.word_1 = deviceNo;
    sendMsg.word_2 = 0;
    sendMsg.word_3 = 0x100;

    while(1){
        retval = Pci_PutMsg(deviceNo, &sendMsg, PCI_CALL_BLOCK);
        Assert(retval == PCI_SUCCESS);

        (sendMsg.word_3)++;
        taskDelay(49);
    }

    return(PCI_SUCCESS);
}


/*
**  int Pci_TestRecvFast();
**
**  DESCRIPTION
**      It retreaves message as fast as possible.  Finally it emptys the
**      inbound message queue.  Then it is blocked and wait on a semaphore.
**      When the new message comes, we would be informed by a doorbell
**      interrupt and in the doorbell ISR, the semaphore is released
**      and we got to continue, until the queue is empty again.
**
**  RETURNS
**      PCI_SUCCESS		the function succeeded.
**      PCI_ERROR		the function failed.
*/
int Pci_TestRecvFast(unsigned int deviceNo)
{
    int retval;

    while(1){
        retval = Pci_GetMsg(&recvMsg, PCI_CALL_BLOCK);
        Assert(retval == PCI_SUCCESS);
        showRecvMsg();
    }

    return(PCI_SUCCESS);
}


int Pci_TestHost(void)
{
    int retval;

    retval = taskSpawn( "SendFast",
                        0x50,   // priority
                        0,      // options
                        0x1000, // stack size
                        (FUNCPTR)Pci_TestSendFast,
                        1, 0, 0, 0, 0, 0, 0, 0, 0, 0
                        );
    Assert(retval != ERROR);

    retval = taskSpawn( "RecvFast",
                        0x50,   // priority
                        0,      // options
                        0x1000, // stack size
                        (FUNCPTR)Pci_TestRecvFast,
                        1, 0, 0, 0, 0, 0, 0, 0, 0, 0
                        );
    Assert(retval != ERROR);

    return(retval);
}


int Pci_TestTarget(void)
{
    int retval;

    retval = taskSpawn( "SendSlow",
                        0x50,   // priority
                        0,      // options
                        0x1000, // stack size
                        (FUNCPTR)Pci_TestSendSlow,
                        0, 0, 0, 0, 0, 0, 0, 0, 0, 0
                        );
    Assert(retval != ERROR);

    retval = taskSpawn( "RecvSlow",
                        0x50,   // priority
                        0,      // options
                        0x1000, // stack size
                        (FUNCPTR)Pci_TestRecvSlow,
                        0, 0, 0, 0, 0, 0, 0, 0, 0, 0
                        );
    Assert(retval != ERROR);

    return(retval);
}


void Pci_Test(void)
{
    Pci_Init();
	if(Pci_DeviceID == 0) Pci_TestHost();
	else Pci_TestTarget();
	return;
}


int putNmsg(int count)
{
    int retval, i;
	unsigned int deviceNo = 0;

	if(deviceNo == Pci_DeviceID) deviceNo = 1;

    sendMsg.word_0 = Pci_DeviceID;
    sendMsg.word_1 = deviceNo;
    sendMsg.word_2 = 0;
    sendMsg.word_3 = 0x100;

    for(i = 0; i < count; i++){
        retval = Pci_PutMsg(deviceNo, &sendMsg, PCI_CALL_UNBLOCK);
        switch(retval){
			case PCI_SUCCESS : (sendMsg.word_3)++; break;
			case PCI_MSGQ_BUSY:  printf("putit: Pci_putMsg busy\n"); break;
			case PCI_MSGQ_FULL:  printf("putit: Pci_putMsg full\n"); break;
			default: printf("Pci_PutMsg failed\n");
		}
        taskDelay(49);
    }

    return(PCI_SUCCESS);
}

int getNmsg(int count)
{
    int retval, i;
	unsigned int deviceNo = 0;

	if(deviceNo == Pci_DeviceID) deviceNo = 1;

    for(i = 0; i < count; i++){
        retval = Pci_GetMsg(&recvMsg, PCI_CALL_BLOCK);
        switch(retval){
			case PCI_SUCCESS : showRecvMsg(); break;
			case PCI_MSGQ_BUSY:  printf("getit: Pci_GetMsg busy\n"); break;
			case PCI_MSGQ_EMPTY:  printf("getit: Pci_GetMsg empty\n"); break;
			default: printf("Pci_GetMsg failed\n");
		}
    }

    return(PCI_SUCCESS);
}

void dump(void)
{
	unsigned int devHandle, confAddr, memAddr, offset=0;

    for (int i = 0; i < Pci_NumOfDevices; i++){
        devHandle = Pci_SystemConfigTable[i].handle;
        confAddr = (unsigned int)SAir_PciMakeConfigAddress(devHandle, offset); 
        memAddr = (unsigned int)(PCIMemBase + Pci_SystemConfigTable[i].memoryBase);
		printf("Device = %d, csrBase=0x%x, memBase=0x%x, ConfAddr=0x%x, MemAddr=0x%x\n",
			i, Pci_SystemConfigTable[i].CSR_Base, Pci_SystemConfigTable[i].memoryBase, confAddr, memAddr);
	}

	return;
}


// end of file
