/* Uclo_Test.cpp
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: Uclo testing
 * author: Lai-Wah Hui 2/28/98
 * revisions:
 * $History: Uclo_Test.cpp $
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 5/20/98    Time: 6:29p
 * Updated in $/base_level2/qa-test/uclo_tests
 * Replace Digital copyright with Intel copyright statement
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 3/13/98    Time: 5:14p
 * Updated in $/base_level1/qa-test/uclo_tests
 * Add test cases
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 3/11/98    Time: 10:12a
 * Updated in $/base_level1/qa-test/uclo_tests
 * Add test cases
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 3/02/98    Time: 3:02p
 * Created in $/base_level1/qa-test/uclo_tests
 * Init. check-in
 */
#include <stdio.h>
#include <stdlib.h>
#include "uclo.h"
#include "uclo_test.h"

#define MaxObjs 20
static NewObjIndex = 0;
static void *hTestObj[MaxObjs];

int UcLoTest_getFileSize(FILE *fp)
{
   fpos_t	endPos;
   long		curFpos;

   curFpos = ftell(fp);
   fseek(fp, 0L, SEEK_END);
   endPos = ftell(fp);
   fseek(fp, curFpos, SEEK_SET);
   return((int)endPos);
}

int UcLoTest_OpenUOFFile(char *fileName,
                 unsigned *fileSize, void ** fBuf)
{
 
    FILE                 *fp;

    if(!fileName) return(UCLO_FILEFAIL);
    if(!(fp = fopen(fileName, "rb"))) return(UCLO_FILEFAIL);

    if(!(*fileSize = UcLoTest_getFileSize(fp))) return(UCLO_FILEFAIL);
    if(!(*fBuf = (char *)malloc(*fileSize))) return(UCLO_MEMFAIL);

    if(fread(*fBuf, 1, *fileSize, fp) != *fileSize){
      fclose(fp);
      free(*fBuf);
      return(UCLO_FILEFAIL);
    }
    fclose(fp);
    return (UCLO_SUCCESS);
}

void test_error( char *fmt, ... )
	{

	char str1[300];

	va_list ap;

	va_start( ap, fmt );

	vsprintf( str1, fmt, ap );

	va_end( ap );

    perror(str1);

	}
  
 /*-----------------------------------------------------------------
	UcLo_TestInIt
	returns: UCLO_SUCCESS 
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									
UcLo_TestInit()    
{
    memset(hTestObj, 0, MaxObjs);
    UcLo_InitLib();
    return UCLO_SUCCESS;
    
}
/*-----------------------------------------------------------------
	UcLo_TestEnd
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int								
UcLo_TestEnd(unsigned int index)
     
{
   int  sts = UCLO_SUCCESS;

    sts = UcLo_DeleObj(hTestObj[index]);
    
    if(sts == UCLO_SUCCESS)
        hTestObj[index]=0;
    
    return sts;

}

 /*-----------------------------------------------------------------
	UcLo_GetObjIndex
	returns: NewObjIndex; 
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									
UcLo_GetObjIndex()    
{
    return NewObjIndex;
    
}

/*-----------------------------------------------------------------
	UcLo_LoadObjFileTest
    Test Uclo_LoadObjFile
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									// UCLO_SUCCESS or UCLO_FAIL
UcLo_LoadObjFileTest(
             char *path, 			// IN : UOF File name to be loaded
             unsigned int DeleteObj)         // IN : 
{             

    int         sts = UCLO_SUCCESS;
    int         index;
    void        *objHandle;

	/* Load the uCode image from file */
   if((sts = UcLo_LoadObjFile(&objHandle,path) )!= UCLO_SUCCESS){
        test_error("UcLo_LoadObjFileTest() : Error in loading %s file, uclo_status = %d", path, sts);
        return(sts);
   }
   if(DeleteObj == TRUE)
      return UcLo_DeleObj(objHandle);
   else{
       for(index=0; index <= MaxObjs; index++){
           if(hTestObj[index] == 0){
                hTestObj[index]=objHandle;
                NewObjIndex = index;
                break;
       }
       if(index == MaxObjs){
           test_error("Uclo_LoadObjFileTest() : ObjHandle array full\n");
           UcLo_DeleObj(objHandle);
           sts = UCLO_FAILURE;
       }
       }
   }

   return(sts);
}

/*-----------------------------------------------------------------
	UcLo_WriteUImageAllTest
    Test UcLo_WriteUImageAllTest
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									// UCLO_SUCCESS or UCLO_FAIL
UcLo_WriteUImageAllTest(unsigned int index)	
{             

    int         sts = UCLO_SUCCESS;
 
    if(sts = UcLo_WriteUimageAll(hTestObj[index]) != UCLO_SUCCESS){
        test_error("UcLo_WriteUImageAllTest() uclo_status = %d", sts);
        return sts;
   
    }

    return(sts);
}




/*-----------------------------------------------------------------
	UcLo_LoadObjFileTest
    Test Uclo_LoadObjFile
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									// UCLO_SUCCESS or UCLO_FAIL
UcLo_MapObjFileTest(char *path, unsigned int DeleteObj)	// IN : UOF File name to be loaded
{             

    int         sts = UCLO_SUCCESS;
    void        *fBuf;
    unsigned int         fileSize;
    int         index;
    void        *objHandle;

    if((sts = UcLoTest_OpenUOFFile(path, &fileSize, &fBuf)) != UCLO_SUCCESS){
        test_error("UcLo_MapObjFileTest() : Error in loading %s file, uclo_status = %d", path, sts);
        return sts;
    }
	/* Load the uCode image from file */
 
    if((sts = UcLo_MapObjAddr(&objHandle, fBuf, fileSize, 0)) != UCLO_SUCCESS){
        test_error("UcLo_MapObjFileTest() : Error in mapping %s file, uclo_status = %d", path, sts);
        return(sts);
   }
    if(DeleteObj == TRUE)
        return UcLo_DeleObj(objHandle);
   else{
       for(index=0; index <= MaxObjs; index++){
           if(hTestObj[index] == 0){
                hTestObj[index]=objHandle;
                NewObjIndex = index;
                break;
       }
       if(index == MaxObjs){
           test_error("Uclo_MapObjFileTest() : ObjHandle array full\n");
           UcLo_DeleObj(objHandle);
           sts = UCLO_FAILURE;
       }
       }
   }

    return(sts);
}


/*-----------------------------------------------------------------
	UcLo_BindSymbolTest
    Test Uclo_BindSymbolTest
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_BindSymbolTest(char *ImageName, 
                        char *SymbolName,
                        unsigned int SymbolVal,
                        unsigned int index)

{

    int         sts = UCLO_SUCCESS;
 

	/* BindSymbol test */
   if((sts = UcLo_BindSymbol(hTestObj[index],ImageName,SymbolName,SymbolVal)) != UCLO_SUCCESS ){
        test_error("UcLo_BindSymbol uclo_status = %d", sts);
        return sts;
   }	

   return sts;

   
}


/*-----------------------------------------------------------------
	UcLo_GetUfuncIdTest
    Test Uclo_GetUfuncId
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_GetUFuncIdTest(char *FunctionName, unsigned int index)

{

    int         sts = UCLO_SUCCESS;
 
    int         ret;
	/* GetFuncId test */
    ret = UcLo_GetUfuncId(hTestObj[index],FunctionName);
    
    if ((ret < 0 ) || (ret > 255)){
        test_error("UcLo_GetUfuncId uclo_status = %d", ret);
        return ret;
   }	

   return sts;

   
}


/*-----------------------------------------------------------------
	UcLo_WriteUImagePageTest
    Test Uclo_BindSymbolTest
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_WriteUImagePageTest(char *ImageName, 
                             unsigned int PageNumber,
                             unsigned int index)

{

 
    int         sts = UCLO_SUCCESS;
 
    if((sts = UcLo_WriteUimagePage(hTestObj[index], ImageName, PageNumber)) != UCLO_SUCCESS){
        test_error("UcLo_WriteUImagePageTest() uclo_status = %d", sts);
        return sts;
   
    }

    return(sts);
}

/*-----------------------------------------------------------------
	UcLo_WriteUImageTest
    Test WriteUImage
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_WriteUImageTest(char *ImageName, unsigned int index) 
{

 
    int         sts = UCLO_SUCCESS;
 
    if((sts = UcLo_WriteUimage(hTestObj[index], ImageName)) != UCLO_SUCCESS){
        test_error("UcLo_WriteUImageTest() uclo_status = %d", sts);
        return sts;
   
    }

    return(sts);
}


/*-----------------------------------------------------------------
	UcLo_VerifyUImageTest
    Test VerifyUImage
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_VerifyUImageTest(char *ImageName, unsigned int index) 
{

 
    int         sts = UCLO_SUCCESS;
 
    if((sts = UcLo_VerifyUimage(hTestObj[index], ImageName)) != UCLO_SUCCESS){
        test_error("UcLo_WriteUImageTest() uclo_status = %d", sts);
        return sts;
   
    }

    return(sts);
}
/*-----------------------------------------------------------------
	UcLo_VerifyUengineTest
    Test WriteUImage
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_VerifyUengineTest(const int uEngNum, unsigned int index) 
{

     int         sts = UCLO_SUCCESS;
 
    if((sts = UcLo_VerifyUengine(hTestObj[index], uEngNum)) != UCLO_SUCCESS){
        test_error("UcLo_VerifyUengineTest() uclo_status = %d", sts);
        return sts;
   
    }

    return(sts);
}

#if 0
int BindSymbol_Test1()
{
    unsigned int baseaddr, key;
    char			*fnam = "uclo_test1.uof";
	char *image_name = "uclo_test1";
    void		*objHandle;
	
	/* Load the uCode image from file */
   if(UcLo_LoadObjFile(&objHandle,fnam)){
      printf("Error; loading file %s\n", "uclo_test1");
      return (0);
   }

   /* BindSymbol test */
   baseaddr = 0x00ff;
   if(UcLo_BindSymbol(objHandle,image_name, "hi256_base",baseaddr )) 
           printf("Error; binding symbol 'hi256_base' in Apps 'Test'\n");
   
   baseaddr = 0x00fe;
   if(UcLo_BindSymbol(objHandle,image_name, "hi56k_base",baseaddr)) 
           printf("Error; binding symbol hi56k_base in Apps 'Test'\n");

    
   baseaddr = 0x00fc;

   if(UcLo_BindSymbol(objHandle,image_name, "rt_base",baseaddr )) 
           printf("Error; binding symbol rt_base in Apps 'Test'\n");
  
   key = 0x00fa;
   if(UcLo_BindSymbol(objHandle, image_name, "ip_da",key)!=0) {
       printf("Error; binding symbol ip_da in Apps 'Test'\n");
   }

   if(UcLo_WriteUimageAll(objHandle)) 
      printf("Error; failed to write Apps to uStore\n");

   /* trash the object */
  UcLo_DeleObj(objHandle);

 return(1);

}

#endif





