/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* hal_console.cpp
 * register console functions so they can be called from cmd line
 *
 *
 * system: SA1200
 * subsystem: core build
 * author: Don Hooper 5/17/98
 * revisions:
 * 
 * 
 */ 
#include <string.h>
// #if (IOSTYLE != HARDWARE)
#include "xact.h"
// #endif
#include "hal_fbi.h"
#include "hal_sdram.h"
#include "hal_sram.h"
#include "hal_console.h"
#include "ueng.h"
#include "hal_ueng.h"


// #ifdef __hardware
//	#define sleep(DELAY_VALUE); delayMs(DELAY_VALUE);
// #else
//	#define sleep(DELAY_VALUE);  
// #endif


unsigned int csr_set(char *unit, char *reg, unsigned int val)
{
	FbiUnit *fbi = FBI_Attach();
	SdramUnit *sdram = SDRAM_Attach();
	SramUnit *sram = SRAM_Attach();
	UengCSRs *ueng = new UengCSRs;

	// doesn't work - shouldn't have to use UEng library
	// Uengine *ueng = new Uengine(0);
	
	// set functions in order as found in hal_fbi.h

	if (!strcmp(unit,"FBI")){
		if (!strcmp(reg, "IREG")){
			fbi->setIREG(val); 
			return(1);

		} else if (!strcmp(reg, "INTER_THD_SIG")){
			fbi->setINTER_THD_SIG(val); 
			return(1);

		} else if (!strcmp(reg, "THREAD_DONE_REG0")){
			fbi->setTHREAD_DONE_REG0(val); 
			return(1);

		} else if (!strcmp(reg, "RCV_RDY_CTL")){
			fbi->setRCV_RDY_CTL(val); 
			return(1);

		} else if (!strcmp(reg, "RCV_REQ")){
			fbi->setRCV_REQ(val); 
			return(1);

		} else if (!strcmp(reg, "FP_READY_WAIT")){
			fbi->setFP_READY_WAIT(val); 
			return(1);

		} else if (!strcmp(reg, "RFIFO_ADDR")){
			fbi->setRFIFO_ADDR(val); 
			return(1);

		} else if (!strcmp(reg, "FLOWCTL_MASK")){
			fbi->setFLOWCTL_MASK(val); 
			return(1);

		} else if (!strcmp(reg, "RDYBUS_TEMPLATE_CTL")){
			fbi->setRDYBUS_TEMPLATE_CTL(val); 
			return(1);

		} else if (!strcmp(reg, "RDYBUS_TEMPLATE_PROG_1")){
			fbi->setRDYBUS_TEMPLATE_PROG_1(val); 
			return(1);

		} else if (!strcmp(reg, "RDYBUS_TEMPLATE_PROG_2")){
			fbi->setRDYBUS_TEMPLATE_PROG_2(val); 
			return(1);

		} else if (!strcmp(reg, "RDYBUS_TEMPLATE_PROG_3")){
			fbi->setRDYBUS_TEMPLATE_PROG_3(val); 
			return(1);

		} else if (!strcmp(reg, "RDYBUS_SYNCH_COUNT_DEFAULT")){
			fbi->setRDYBUS_SYNCH_COUNT_DEFAULT(val); 
			return(1);

		} else if (!strcmp(reg, "HASH_MULTIPLIER_64_HI")){
			fbi->setHASH_MULTIPLIER_64_HI(val); 
			return(1);

		} else if (!strcmp(reg, "HASH_MULTIPLIER_64_LO")){
			fbi->setHASH_MULTIPLIER_64_LO(val); 
			return(1);

		} else if (!strcmp(reg, "HASH_MULTIPLIER_48_HI")){
			fbi->setHASH_MULTIPLIER_48_HI(val); 
			return(1);

		} else if (!strcmp(reg, "HASH_MULTIPLIER_48_LO")){
			fbi->setHASH_MULTIPLIER_48_LO(val); 
			return(1);

		} else if (!strcmp(reg, "SEND_CMD")){
			fbi->setSEND_CMD(val); 
			return(1);

		} else if (!strcmp(reg, "XMIT_RDY_CTL")){
			fbi->setXMIT_RDY_CTL(val); 
			return(1);
		} else {
			printf("Unknown FBI CSR specified, Set command.\n");
			return(-1);
		}

	} else if (!strcmp(unit, "SRAM")){

		if (!strcmp(reg, "SRAM_CSR")){
			sram->setSRAM_CSR(val); 
			return(1);

		} else if (!strcmp(reg, "SRAM_AUTO_BASE")){
			sram->setSRAM_AUTO_BASE(val); 
			return(1);

		} else if (!strcmp(reg, "SRAM_AUTO_PTR")){
			sram->setSRAM_AUTO_PTR(val); 
			return(1);

		} else if (!strcmp(reg, "SRAM_AUTO_END")){
			sram->setSRAM_AUTO_END(val); 
			return(1);

		} else if (!strcmp(reg, "SRAM_TEST_MOD")){
			sram->setSRAM_TEST_MOD(val); 
			return(1);

		} else if (!strcmp(reg, "SRAM_SLOW_CONFIG")){
			sram->setSRAM_SLOW_CONFIG(val); 
			return(1);

		} else if (!strcmp(reg, "SRAM_BOOT_CONFIG")){
			sram->setSRAM_BOOT_CONFIG(val); 
			return(1);

		} else if (!strcmp(reg, "SRAM_SLOWPORT_CONFIG")){
			sram->setSRAM_SLOWPORT_CONFIG(val); 
			return(1);

		} else {
			printf("Unknown SRAM CSR specified, Set command.\n");
			return(-1);
		}

	} else if (!strcmp(unit, "SDRAM")){

		if (!strcmp(reg, "SDRAM_CSR")){
			sdram->setSDRAM_CSR(val); 
			return(1);

		} else if (!strcmp(reg, "SDRAM_MEM_CTL0")){
			sdram->setSDRAM_MEM_CTL0(val); 
			return(1);

		} else if (!strcmp(reg, "SDRAM_MEM_CTL1")){
			sdram->setSDRAM_MEM_CTL1(val); 
			return(1);

		} else if (!strcmp(reg, "SDRAM_MEM_INIT")){
			sdram->setSDRAM_MEM_INIT(val); 
			return(1);

		} else {

			printf("Unknown SDRAM CSR specified, Set command.\n");
			return(-1);
		}


	} else if (!strcmp(unit, "UENG")){

		if (!strcmp(reg, "USTORE_ADDRESS")){
			ueng->setUSTORE_ADDRESS(val); 
			return(1);

		} else if (!strcmp(reg, "USTORE_DATA")){
			ueng->setUSTORE_DATA(val); 
			return(1);

		} else if (!strcmp(reg, "ACTIVE_CTX_STS")){
			ueng->setACTIVE_CTX_STS(val); 
			return(1);

		} else if (!strcmp(reg, "FBOX_CSR_WRITE_SELF_ENABLE")){
			ueng->setFBOX_CSR_WRITE_SELF_ENABLE(val); 
			return(1);

		} else if (!strcmp(reg, "ENABLE_SRAM_JOURNALING")){
			ueng->setENABLE_SRAM_JOURNALING(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_ARB_CNTL")){
			ueng->setCTX_ARB_CNTL(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_ENABLES")){
			ueng->setCTX_ENABLES(val); 
			return(1);

		} else if (!strcmp(reg, "CC_ENABLE")){
			ueng->setCC_ENABLE(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_0_STS")){
			ueng->setCTX_0_STS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_1_STS")){
			ueng->setCTX_1_STS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_2_STS")){
			ueng->setCTX_2_STS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_3_STS")){
			ueng->setCTX_3_STS(val); 
			return(1);
		
		} else if (!strcmp(reg, "CTX_0_SIG_EVENTS")){
			ueng->setCTX_0_SIG_EVENTS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_1_SIG_EVENTS")){
			ueng->setCTX_1_SIG_EVENTS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_2_SIG_EVENTS")){
			ueng->setCTX_2_SIG_EVENTS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_3_SIG_EVENTS")){
			ueng->setCTX_3_SIG_EVENTS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_0_WAKEUP_EVENTS")){
			ueng->setCTX_0_WAKEUP_EVENTS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_1_WAKEUP_EVENTS")){
			ueng->setCTX_1_WAKEUP_EVENTS(val); 
			return(1);
		
		} else if (!strcmp(reg, "CTX_2_WAKEUP_EVENTS")){
			ueng->setCTX_2_WAKEUP_EVENTS(val); 
			return(1);

		} else if (!strcmp(reg, "CTX_3_WAKEUP_EVENTS")){
			ueng->setCTX_3_WAKEUP_EVENTS(val); 
			return(1);		

		} else {

			printf("Unknown UENG CSR specified, Set command.\n");
			return(-1);
		}


	} else{

		printf("Unknown Unit\n");
		return(-1);
	}

	return(1);
}



unsigned int csr_get(char *unit, char *reg)
{
	unsigned int register_val;
	FbiUnit *fbi = FBI_Attach();
	SdramUnit *sdram = SDRAM_Attach();
	SramUnit *sram = SRAM_Attach();
	UengCSRs *ueng = new UengCSRs;

	if (!strcmp(unit, "FBI")){
		if (!strcmp(reg, "IREG")){
			fbi->getIREG(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "SOP_SEQ1")){
			fbi->getSOP_SEQ1(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "SOP_SEQ2")){
			fbi->getSOP_SEQ2(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "ENQUEUE_SEQ1")){
			fbi->getENQUEUE_SEQ1(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "ENQUEUE_SEQ2")){
			fbi->getENQUEUE_SEQ2(&register_val);
			return(register_val);
		}

/*dj: INTER_THD_SIG is a write-only reg.
		else if (!strcmp(reg, "INTER_THD_SIG")){
			fbi->getINTER_THD_SIG(&register_val);
			return(register_val);
		}
*/

		else if (!strcmp(reg, "THREAD_DONE_REG1")){
			fbi->getTHREAD_DONE_REG1(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "RCV_RDY_CNT")){
			fbi->getRCV_RDY_CNT(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "RCV_RDY_HI")){
			fbi->getRCV_RDY_HI(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "RCV_RDY_LO")){
			fbi->getRCV_RDY_LO(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "RCV_RDY_CTL")){
			fbi->getRCV_RDY_CTL(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "RCV_CNTL")){
			fbi->getRCV_CNTL(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "REC_FASTPORT_CTL")){
			fbi->getREC_FASTPORT_CTL(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "FLOWCTL_MASK")){
			fbi->getFLOWCTL_MASK(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "SELF_DESTRUCT")){
			fbi->getSELF_DESTRUCT(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "HASH_MULTIPLIER_64_HI")){
			fbi->getHASH_MULTIPLIER_64_HI(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "HASH_MULTIPLIER_64_LO")){
			fbi->getHASH_MULTIPLIER_64_LO(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "HASH_MULTIPLIER_48_HI")){
			fbi->getHASH_MULTIPLIER_48_HI(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "HASH_MULTIPLIER_48_LO")){
			fbi->getHASH_MULTIPLIER_48_LO(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "GET_CMD")){
			fbi->getGET_CMD(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "XMIT_RDY_LO")){
			fbi->getXMIT_RDY_LO(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "XMIT_RDY_HI")){
			fbi->getXMIT_RDY_HI(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "XMIT_RDY_CTL")){
			fbi->getXMIT_RDY_CTL(&register_val);
			return(register_val);
		}

		else if (!strcmp(reg, "XMIT_PTR")){
			fbi->getXMIT_PTR(&register_val);
			return(register_val);
		}

		else {
			printf("Unknown FBI CSR specified, Get command.\n");
			return(-1);
		}

	} else if (!strcmp(unit,"SRAM")) {

		if (!strcmp(reg, "SRAM_CSR")){
			sram->getSRAM_CSR(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SRAM_AUTO_BASE")){
			sram->getSRAM_AUTO_BASE(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SRAM_AUTO_PTR")){
			sram->getSRAM_AUTO_PTR(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SRAM_AUTO_END")){
			sram->getSRAM_AUTO_END(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SRAM_TEST_MOD")){
			sram->getSRAM_TEST_MOD(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SRAM_SLOW_CONFIG")){
			sram->getSRAM_SLOW_CONFIG(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SRAM_BOOT_CONFIG")){
			sram->getSRAM_BOOT_CONFIG(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SRAM_SLOWPORT_CONFIG")){
			sram->getSRAM_SLOWPORT_CONFIG(&register_val); 
			return(register_val);

		} else {
			printf("Unknown SRAM CSR specified, Get command.\n");
			return(-1);
		}
				
	} else if (!strcmp(unit, "SDRAM")){

		if (!strcmp(reg, "SDRAM_CSR")){
			sdram->getSDRAM_CSR(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SDRAM_MEM_CTL0")){
			sdram->getSDRAM_MEM_CTL0(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SDRAM_MEM_CTL1")){
			sdram->getSDRAM_MEM_CTL1(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "SDRAM_MEM_INIT")){
			sdram->getSDRAM_MEM_INIT(&register_val); 
			return(register_val);

		} else {

			printf("Unknown SDRAM CSR specified, Get command.\n");
			return(-1);
		}

	
	} else if (!strcmp(unit, "UENG")){

		if (!strcmp(reg, "USTORE_ADDRESS")){
			ueng->getUSTORE_ADDRESS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "USTORE_DATA")){
			ueng->getUSTORE_DATA(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "ALU_OUTPUT")){
			ueng->getALU_OUTPUT(&register_val); 
			return(register_val);
		
		} else if (!strcmp(reg, "ACTIVE_CTX_STS")){
			ueng->getACTIVE_CTX_STS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "FBOX_CSR_WRITE_SELF_ENABLE")){
			ueng->getFBOX_CSR_WRITE_SELF_ENABLE(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "ENABLE_SRAM_JOURNALING")){
			ueng->getENABLE_SRAM_JOURNALING(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_ARB_CNTL")){
			ueng->getCTX_ARB_CNTL(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_ENABLES")){
			ueng->getCTX_ENABLES(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CC_ENABLE")){
			ueng->getCC_ENABLE(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_0_STS")){
			ueng->getCTX_0_STS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_1_STS")){
			ueng->getCTX_1_STS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_2_STS")){
			ueng->getCTX_2_STS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_3_STS")){
			ueng->getCTX_3_STS(&register_val); 
			return(register_val);
		
		} else if (!strcmp(reg, "CTX_0_SIG_EVENTS")){
			ueng->getCTX_0_SIG_EVENTS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_1_SIG_EVENTS")){
			ueng->getCTX_1_SIG_EVENTS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_2_SIG_EVENTS")){
			ueng->getCTX_2_SIG_EVENTS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_3_SIG_EVENTS")){
			ueng->getCTX_3_SIG_EVENTS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_0_WAKEUP_EVENTS")){
			ueng->getCTX_0_WAKEUP_EVENTS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_1_WAKEUP_EVENTS")){
			ueng->getCTX_1_WAKEUP_EVENTS(&register_val); 
			return(register_val);
		
		} else if (!strcmp(reg, "CTX_2_WAKEUP_EVENTS")){
			ueng->getCTX_2_WAKEUP_EVENTS(&register_val); 
			return(register_val);

		} else if (!strcmp(reg, "CTX_3_WAKEUP_EVENTS")){
			ueng->getCTX_3_WAKEUP_EVENTS(&register_val); 
			return(register_val);			

		} else {

			printf("Unknown UENG CSR specified, Get command.\n");
			return(-1);
		}

		
	} else {
		printf("Unknown Unit\n");
		return(-1);
	}

	return(register_val);
}


unsigned int sdram_write(unsigned int lwaddr, unsigned int val)
{ 
	SdramUnit *sdram = SDRAM_Attach();

	SDRAM_Write(lwaddr,val); 
	return(1);
}


unsigned int sdram_read(unsigned int lwaddr)
{
	unsigned int sdram_value;
	SdramUnit *sdram = SDRAM_Attach();

	sdram_value = SDRAM_Read(lwaddr);
	
	return sdram_value;
}


unsigned int sram_write(unsigned int lwaddr, unsigned int val)
{ 
	SramUnit *sram = SRAM_Attach();

	// SRAM_Write(lwaddr, val);
	sram->write(lwaddr,val); 
	return(1);
}


unsigned int sram_read(unsigned int lwaddr)
{
	unsigned int sram_value;
	SramUnit *sram = SRAM_Attach();

	sram_value = SRAM_Read(lwaddr);
	
	return sram_value;
}


void sram_push(unsigned short queue, unsigned int addr)
{
	
	SramUnit *sram = SRAM_Attach();
	sram->push(queue, addr);

}


void sram_pop(unsigned short queue, unsigned int addr)
{

	SramUnit *sram = SRAM_Attach();
	sram->pop(queue, &addr);
}


void sram_set_bits(unsigned int lwaddr, unsigned int value)
{
	SramUnit *sram = SRAM_Attach();
	sram->set_bits(lwaddr, value);
}


void sram_clear_bits(unsigned int lwaddr, unsigned int value)
{
	SramUnit *sram = SRAM_Attach();
	sram->clear_bits(lwaddr, value);
}



#if ((IOSTYLE==HARDWARE)||(IOSTYLE==AMBAIO))


unsigned int sram_test_and_set_bits(unsigned int lwaddr, unsigned int setval)
{

	unsigned int *pGetVal=0;
	SramUnit *sram = SRAM_Attach();
	sram->test_and_set_bits(lwaddr, setval, pGetVal);
	return *pGetVal;

}


unsigned int sram_test_and_clear_bits(unsigned int lwaddr, unsigned int setval)
{
	unsigned int getval;
	SramUnit *sram = SRAM_Attach();
	sram->test_and_clear_bits(lwaddr, setval, &getval);
	return getval;

}

#endif


void sram_write_unlock(unsigned int lwaddr, unsigned int value)
{
	SramUnit *sram = SRAM_Attach();
	sram->write_unlock(lwaddr, value);
}


void sram_read_lock(unsigned int lwaddr, unsigned int *value)
{
	SramUnit *sram = SRAM_Attach();
	sram->read_lock(lwaddr, value);
}


bool Locked(void) 
{ 
	// test bit 0 of SRAM_CSR
	unsigned int result;
	SramUnit *sram = SRAM_Attach();

	unsigned int *pVal = 0;
	sram->getSRAM_CSR(pVal);

	result = *pVal & (1<<0);
	if (result == 1) return true;
	return false;
}


bool LockRetry(void) 
{ 
	// test bit 1 of SRAM_CSR
	unsigned int result;
	SramUnit *sram = SRAM_Attach();

	unsigned int *pVal = 0;
	sram->getSRAM_CSR(pVal);

	result = *pVal & (1<<1);
	if (result == 1) return true;
	return false;
}



int hal_model_initialize()
{
#if (IOSTYLE != HARDWARE)
	if (!XACT_register_console_function("csr_get", csr_get, 2, 0) ){
            fprintf(stderr,"XACT_register_console_function() csr_get failed\n");
			return(0);
	}
	if (!XACT_register_console_function("csr_set", csr_set, 2, 1) ){
            fprintf(stderr,"XACT_register_console_function() csr_set failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sdram_read", sdram_read, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() sdram_read failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sdram_write", sdram_write, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() sdram_write failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_read", sram_read, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() sram_read failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_write", sram_write, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() sram_write failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_push", sram_push, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() sram_push failed\n");
			return(0);
	}
			
	if (!XACT_register_console_function("sram_pop", sram_pop, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() sram_pop failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_set_bits", sram_set_bits, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() sram_set_bits failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_clear_bits", sram_clear_bits, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() sram_clear_bits failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_test_and_set_bits", sram_test_and_set_bits, 0, 2)){
            fprintf(stderr,"XACT_register_console_function() sram_test_and_set_bits failed.\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_test_and_clear_bits", sram_test_and_clear_bits, 0, 2)){
            fprintf(stderr,"XACT_register_console_function() sram_test_and_clear_bits failed\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_write_unlock", sram_write_unlock, 0, 2)){
            fprintf(stderr,"XACT_register_console_function() sram_write_unlock failed.\n");
			return(0);
	}

	if (!XACT_register_console_function("sram_read_lock", sram_read_lock, 0, 2)){
            fprintf(stderr,"XACT_register_console_function() sram_read_lock failed.\n");
			return(0);
	}

#endif
	return(1);

}