/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* hal_console_test.cpp
 * test hal set and get functions
 *
 * system: SA1200
 * subsystem: core build
 * author: Tom Clancy 06/27/99
 * revisions:
 * 
 * 
 */ 

// Notes: 
//
// This files created from hal_console_test.ind. Converted to C to link
// into the main network test app under VxWorks and UCOS. This file could be
// converted back to IND format to run under DevWorkBench.
//
// The transactor complains when you try stuffing any values in the SRAM
// and SDRAM config registers. Used the OEMCONFIG file (Tom P.), from the
// reference design to get valid values for some of the registers.
// 
// Many test do a get only of certain registers. This is because we don't
// want the tests to overwrite values written by the VxWorks or UCOC BSP. 
// Look at vbsa1200.h, config.h for values used in these registers. Also note
// that two sets of register definitions are in the sofware stack, the BSP
// registers in vbsa1200.h and the software HAL in platform.h.


#include <stdio.h>
#include <string.h>
#include "core_io.h"
#include "hal_console.h"
#include "hal_sram.h"
#include "hal_sdram.h"

#define HAL_SUCCESS 0 
#define HAL_FAIL 1

// #define __transactor		// for software model, IND file
#define __hardware			// running on hardware

#define DELAY_VALUE	25		// Number of milliseconds to delay between Set() and Get()
							// Need this on Hardware, set to zero for xact

#ifdef IND_FILE

	@start
	dep/s f0.art.enable = 0
	dep/s f1.art.enable = 0
	dep/s f2.art.enable = 0
	dep/s f3.art.enable = 0
	dep/s f4.art.enable = 0
	dep/s f5.art.enable = 0

	// reset the amba interface so FIQ interrupt will work
	k.c_reset = 1;
	go 2
	k.c_reset = 0;
	systemInit_Thread(3);

#endif

// wrap all test functions, so one call can be made from console

int hal_testall() {

	int fail = 0;
	int act_result, exp_result;

	printf("HAL Set() and Get() SDRAM, SRAM, FBI and UENG register tests.\n");
	printf("NOTE: Restart after these tests.\n");


	printf("\n\nFBI register tests.\n");

	// set and get
	printf("Set and Get IREG...\n");
	csr_set("FBI", "IREG", 0xff000000);
	act_result = csr_get("FBI", "IREG");
	exp_result = 0xf0000000;				// note, for IREG, bits 27:0 are write 1 to clear
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, IREG mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get IREG...\n");
	}
	delayMs(DELAY_VALUE);
	

	// set only
	printf("Set INTER_THD_SIG...\n");
	csr_set("FBI", "INTER_THD_SIG", 0xCAFECAFE);
	delayMs(DELAY_VALUE);
	

	// set and get ?? REG0 and REG1 (should have REG1)
	// Bug #615
	printf("Set and Get THREAD_DONE_REG0...\n");
	csr_set("FBI", "THREAD_DONE_REG0", 0x11111111);
	act_result = csr_get("FBI", "THREAD_DONE_REG0");
	exp_result = 0xCAFECAFE;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, THREAD_DONE_REG0 mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get THREAD_DONE_REG0.\n");
	}
	delayMs(DELAY_VALUE);
	

	// set and get
	printf("Set and Get RCV_RDY_CTL...\n");
	csr_set("FBI", "RCV_RDY_CTL", 0x4AFE);
	act_result = csr_get("FBI", "RCV_RDY_CTL");
	exp_result = 0x4AFE;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, RCV_RDY_CTL mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get RCV_RDY_CTL.\n");
	}
	delayMs(DELAY_VALUE);
	

	// set only 
	// printf("Set RCV_REQ...\n");
	// csr_set("FBI", "RCV_REQ", 0x0);
	
	// set only
	printf("Set FP_READY_WAIT...\n");
	csr_set("FBI", "FP_READY_WAIT", 0xFF);
	delayMs(DELAY_VALUE);
	
	
	printf("Set and Get FLOWCTL_MASK...\n");
	csr_set("FBI", "FLOWCTL_MASK", 0xff);
	act_result = csr_get("FBI", "FLOWCTL_MASK");
	exp_result = 0xff;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, FLOWCTL_MASK mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get FLOWCTL_MASK.\n");
	}
	delayMs(DELAY_VALUE);
	

	// set only 
	printf("Set RDYBUS_TEMPLATE_CTL...\n");
	csr_set("FBI", "RDYBUS_TEMPLATE_CTL", 0x15);
	delayMs(DELAY_VALUE);
	
	
	// set only 
	printf("Set RDYBUS_TEMPLATE_PROG_1...\n");
	csr_set("FBI", "RDYBUS_TEMPLATE_PROG_1", 0x9F9F9F9F);
	delayMs(DELAY_VALUE);
	
	
	// set only 
	printf("Set RDYBUS_TEMPLATE_PROG_2...\n");
	csr_set("FBI", "RDYBUS_TEMPLATE_PROG_2", 0xCAFECAFE);
	delayMs(DELAY_VALUE);
	
	
	// set only 
	printf("Set RDYBUS_TEMPLATE_PROG_2...\n");
	csr_set("FBI", "RDYBUS_TEMPLATE_PROG_3", 0xCAFECAFE);
	delayMs(DELAY_VALUE);
	
	
	// set only
	printf("Set RDYBUS_SYNCH_COUNT_DEFAULT...\n");
	csr_set("FBI", "RDYBUS_SYNCH_COUNT_DEFAULT", 0xCAFECAFE);
	delayMs(DELAY_VALUE);
	
	
	// set and get
	printf("Set and Get HASH_MULTIPLIER_64_HI...\n");
	csr_set("FBI", "HASH_MULTIPLIER_64_HI", 0xCAFECAFE);
	act_result = csr_get("FBI", "HASH_MULTIPLIER_64_HI");
	exp_result = 0xCAFECAFE;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, HASH_MULTIPLIER_64_HI mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get HASH_MULTIPLIER_64_HI.\n");
	}
	delayMs(DELAY_VALUE);
	

	// set and get
	printf("Set and Get HASH_MULTIPLIER_64_LO...\n");
	csr_set("FBI", "HASH_MULTIPLIER_64_LO", 0xCAFECAFE);
	act_result = csr_get("FBI", "HASH_MULTIPLIER_64_LO");
	exp_result = 0xCAFECAFE;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, HASH_MULTIPLIER_64_LO mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get HASH_MULTIPLIER_64_LO.\n");
	}
	delayMs(DELAY_VALUE);
	

	// set and get
	printf("Set and Get HASH_MULTIPLIER_48_HI...\n");
	csr_set("FBI", "HASH_MULTIPLIER_48_HI", 0xCAFECAFE);
	act_result = csr_get("FBI", "HASH_MULTIPLIER_48_HI");
	exp_result = 0xCAFECAFE;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, HASH_MULTIPLIER_48_HI mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get HASH_MULTIPLIER_48_HI.\n");
	}
	delayMs(DELAY_VALUE);
	


	// set and get
	printf("Set and Get HASH_MULTIPLIER_48_LO...\n");
	csr_set("FBI", "HASH_MULTIPLIER_48_LO", 0xCAFECAFE);
	act_result = csr_get("FBI", "HASH_MULTIPLIER_48_LO");
	exp_result = 0xCAFECAFE;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, HASH_MULTIPLIER_48_LO mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get HASH_MULTIPLIER_48_LO.\n");
	}
	delayMs(DELAY_VALUE);
	

	// set only
	printf("Set SEND_CMD...\n");
	csr_set("FBI", "SEND_CMD", 0xCAFECAFE);
	delayMs(DELAY_VALUE);
	
	
	// set and get
	csr_set("FBI", "XMIT_RDY_CTL", 0x2fe);
	act_result = csr_get("FBI", "XMIT_RDY_CTL");
	exp_result = 0x2fe;				
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, XMIT_RDY_CTL mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get XMIT_RDY_CTL.\n");
	}
	delayMs(DELAY_VALUE);
	



	/////////////////////////////////////////////////////////////
	// Test FBI registers
	// These registers are GET only 
	// TODO: make sure return value falls within bounds
	/////////////////////////////////////////////////////////////

	printf("Get SOP_SEQ1...\n");
	act_result = csr_get("FBI", "SOP_SEQ1");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, SOP_SEQ1, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get SOP_SEQ2...\n");
	act_result = csr_get("FBI", "SOP_SEQ2");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, SOP_SEQ2, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get ENQUEUE_SEQ1...\n");
	act_result = csr_get("FBI", "ENQUEUE_SEQ1");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, ENQUEUE_SEQ1, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get ENQUEUE_SEQ2...\n");
	act_result = csr_get("FBI", "ENQUEUE_SEQ2");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, ENQUEUE_SEQ2 , actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get THREAD_DONE_REG1...\n");
	act_result = csr_get("FBI", "THREAD_DONE_REG1");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, THREAD_DONE_REG1 , actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get RCV_RDY_CNT...\n");
	act_result = csr_get("FBI", "RCV_RDY_CNT");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, RCV_RDY_CNT , actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get RCV_RDY_HI...\n");
	act_result = csr_get("FBI", "RCV_RDY_HI");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, RCV_RDY_HI , actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get RCV_RDY_LO...\n");
	act_result = csr_get("FBI", "RCV_RDY_LO");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, RCV_RDY_LO, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get RCV_CNTL...\n");
	act_result = csr_get("FBI", "RCV_CNTL");
	if (act_result == 0xcafecafe ){
		printf("ERROR");
		printf("ERROR: hal_console_test, , actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	printf("Get REC_FASTPORT_CTL...\n");
	act_result = csr_get("FBI", "REC_FASTPORT_CTL");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, REC_FASTPORT_CTL , actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	

	
	printf("Get GET_CMD...\n");
	act_result = csr_get("FBI", "GET_CMD");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, GET_CMD, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	


	printf("Get XMIT_RDY_LO...\n");
	act_result = csr_get("FBI", "XMIT_RDY_LO");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, XMIT_RDY_LO, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	


	printf("Get XMIT_RDY_HI...\n");
	act_result = csr_get("FBI", "XMIT_RDY_HI");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, XMIT_RDY_HI, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	


	printf("Get XMIT_PTR...\n");
	act_result = csr_get("FBI", "XMIT_PTR");
	if (act_result < 0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, XMIT_PTR, actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	


	printf("Get SELF_DESTRUCT...\n");
	act_result = csr_get("FBI", "SELF_DESTRUCT");
	if (act_result < 0x0 ){
		printf("ERROR");
		printf("ERROR: hal_console_test, SELF_DESTRUCT , actual = %x\n", act_result);
		fail = 1;
	}
	delayMs(DELAY_VALUE);
	


	////////////////////////////////////////
	// UENG REGISTERS read/write test
	////////////////////////////////////////
	printf("\n\nUENG register tests.\n");

	// Note - bug entered for all UENG HAL tests, can't instantiate a 
	// HAL_UENG object. 
	
	// printf("Set and Get USTORE_ADDRESS...\n");
	// csr_set("UENG", "USTORE_ADDRESS", 0x44);
	// act_result = csr_get("UENG", "USTORE_ADDRESS");
	// exp_result = 0x44;
	// if (act_result != exp_result){
	//	printf("ERROR");
	//  printf("ERROR: hal_console_test, USTORE_ADDRESS mismatch, expected = %x, actual = %x\n", exp_result, act_result);
	// 	fail = 1;
	// } else {
	//	printf("PASS: Set and Get USTORE_ADDRESS.\n");
	// }
	// delayMs(DELAY_VALUE);
	

	// printf("Set and Get USTORE_DATA...\n");
	// csr_set("UENG", "USTORE_DATA", 0xD0000000);
	// act_result = csr_get("UENG", "USTORE_DATA");
	// exp_result = 0xD0000000;		
	// if (act_result != exp_result){
	//	printf("ERROR");
	//	printf("ERROR: hal_console_test, USTORE_DATA mismatch, expected = %x, actual = %x\n", exp_result, act_result);
	//	fail = 1;
	// } else {
	//	printf("PASS: Set and Get USTORE_DATA.\n");
	// }
	//	delayMs(DELAY_VALUE);
	
	// get only
	// printf("Get ALU_OUTPUT...\n");
	// act_result = csr_get("UENG", "ALU_OUTPUT");
	// if (act_result < 0){
	//	printf("ERROR");
	//	printf("ERROR: hal_console_test, ALU_OUTPUT test, actual = %x\n", act_result);
	//	fail = 1;
	// } else {
	//	printf("PASS: Set and Get ALU_OUTPUT.\n");
	// }
	//	delayMs(DELAY_VALUE);
	

	// printf("Set and Get ACTIVE_CTX_STS...\n");
	// csr_set("UENG", "ACTIVE_CTX_STS", 0xD0000000);
	// act_result = csr_get("UENG", "ACTIVE_CTX_STS");
	// exp_result = 0xD0000000;		
	// if (act_result != exp_result){
	//	printf("ERROR");
	//	printf("ERROR: hal_console_test, ACTIVE_CTX_STS mismatch, expected = %x, actual = %x\n", exp_result, act_result);
	//	fail = 1;
	// } else {
	//	printf("PASS: Set and Get ACTIVE_CTX_STS.\n");
	// }
	//	delayMs(DELAY_VALUE);
	

	////////////////////////////////////////
	// SRAM REGISTERS read/write test
	////////////////////////////////////////
	printf("\n\nSRAM register tests.\n");

	// get, set, get
	printf("Get SRAM_CSR...\n");
	act_result = csr_get("SRAM", "SRAM_CSR");
	csr_set("SRAM", "SRAM_CSR", act_result);
	exp_result = csr_get("SRAM", "SRAM_CSR");
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_CSR mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Get SRAM_CSR.\n");
	}
	delayMs(DELAY_VALUE);
	


	printf("Get SRAM_AUTO_BASE...\n");
	act_result = csr_get("SRAM", "SRAM_AUTO_BASE");
	csr_set("SRAM", "SRAM_AUTO_BASE", act_result);
	exp_result = csr_get("SRAM", "SRAM_AUTO_BASE");
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_AUTO_BASE mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Get SRAM_AUTO_BASE.\n");
	}
	delayMs(DELAY_VALUE);
	
	

	printf("Get SRAM_AUTO_PTR...\n");
	act_result = csr_get("SRAM", "SRAM_AUTO_PTR");
	csr_set("SRAM", "SRAM_AUTO_PTR", act_result);
	exp_result = csr_get("SRAM", "SRAM_AUTO_PTR");
	// auto_ptr will be different
	if (act_result == exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_AUTO_PTR mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Get SRAM_AUTO_PTR.\n");
	}
	delayMs(DELAY_VALUE);
	

	
	printf("Set and Get SRAM_AUTO_END...\n");
	act_result = csr_get("SRAM", "SRAM_AUTO_END");
	csr_set("SRAM", "SRAM_AUTO_END", act_result);
	exp_result = csr_get("SRAM", "SRAM_AUTO_END");
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_AUTO_END mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get SRAM_AUTO_END.\n");
	}
	delayMs(DELAY_VALUE);
	

	printf("Get SRAM_TEST_MOD...\n");
	act_result = csr_get("SRAM", "SRAM_TEST_MOD");
	csr_set("SRAM", "SRAM_TEST_MOD", act_result);
	exp_result = csr_get("SRAM", "SRAM_TEST_MOD");
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_TEST_MOD get, actual = %x\n", act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get SRAM_TEST_MOD.\n");
	}
	delayMs(DELAY_VALUE);
	


	printf("Set and Get SRAM_SLOW_CONFIG...\n");
	act_result = csr_get("SRAM", "SRAM_SLOW_CONFIG");
	csr_set("SRAM", "SRAM_SLOW_CONFIG", act_result);
	exp_result = csr_get("SRAM", "SRAM_SLOW_CONFIG");
	 if (act_result != exp_result){
	 	printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_SLOW_CONFIG mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	 } else {
		printf("PASS: Set and Get SRAM_SLOW_CONFIG.\n");
	 }
	delayMs(DELAY_VALUE);
	 

	printf("Set and Get SRAM_BOOT_CONFIG...\n");
	act_result = csr_get("SRAM", "SRAM_BOOT_CONFIG");
	csr_set("SRAM", "SRAM_BOOT_CONFIG", act_result);
	exp_result = csr_get("SRAM", "SRAM_BOOT_CONFIG");
	 if (act_result == exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_BOOT_CONFIG mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	 } else {
		printf("PASS: Set and Get SRAM_BOOT_CONFIG.\n");
	}
	delayMs(DELAY_VALUE);
	

	printf("Set and Get SRAM_SLOWPORT_CONFIG...\n");
	act_result = csr_get("SRAM", "SRAM_SLOWPORT_CONFIG");
	csr_set("SRAM", "SRAM_SLOWPORT_CONFIG", act_result);
	exp_result = csr_get("SRAM", "SRAM_SLOWPORT_CONFIG");
	// always different - use core clock cycles
	if (act_result == exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM_SLOWPORT_CONFIG mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get SRAM_SLOWPORT_CONFIG.\n");
	}
	delayMs(DELAY_VALUE);
	


	////////////////////////////////////////
	// SDRAM REGISTERS read/write test
	////////////////////////////////////////
	printf("\n\nSDRAM register tests.\n");

	printf("Set and Get SDRAM_CSR...\n");
	act_result = csr_get("SDRAM", "SDRAM_CSR");
	csr_set("SDRAM", "SDRAM_CSR", act_result);
	exp_result = csr_get("SDRAM", "SDRAM_CSR");
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SDRAM_CSR mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get SDRAM_CSR.\n");
	}
	delayMs(DELAY_VALUE);
	


	printf("Set and Get SDRAM_MEM_CTL0...\n");
	act_result = csr_get("SDRAM", "SDRAM_MEM_CTL0");
	csr_set("SDRAM", "SDRAM_MEM_CTL0", act_result);
	exp_result = csr_get("SDRAM", "SDRAM_MEM_CTL0");
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SDRAM_MEM_CTL0 mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get SDRAM_MEM_CTL0.\n");
	}
	delayMs(DELAY_VALUE);
	


	printf("Set and Get SDRAM_MEM_CTL1...\n");
	csr_set("SDRAM", "SDRAM_MEM_CTL1", 0x15151515);
	act_result = csr_get("SDRAM", "SDRAM_MEM_CTL1");
	exp_result = 0x15151515;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SDRAM_MEM_CTL1 mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get SDRAM_MEM_CTL1.\n");
	}
	delayMs(DELAY_VALUE);
	

	printf("Set and Get SDRAM_MEM_INIT...\n");
	csr_set("SDRAM", "SDRAM_MEM_INIT", 0x2cafe);
	act_result = csr_get("SDRAM", "SDRAM_MEM_INIT");
	exp_result = 0x2cafe;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SDRAM_MEM_INIT mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: Set and Get SDRAM_MEM_INIT.\n");
	}
	delayMs(DELAY_VALUE);
		

	/////////////////////////////////////////////////
	// SRAM read/write tests - several locations
	/////////////////////////////////////////////////

	// TODO: loop and fill entire sram space. use
	// VxWorks BSP memsize API's

	printf("\n\nStart SRAM memory R/W tests...\n");

	printf("sram_write 0x0950\n");
	sram_write(0x0950, 0x15151515);
	printf("sram_read  0x0950\n");
	act_result = sram_read(0x0950);
	exp_result = 0x15151515;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM compare passed.\n");
	}
	delayMs(DELAY_VALUE);
	
	printf("sram_write 0x0954\n");
	sram_write(0x0954, 0xCAFECAFE);
	printf("sram_read  0x0954\n");
	act_result = sram_read(0x0954);
	exp_result = 0xCAFECAFE;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM compare passed.\n");
	}
	delayMs(DELAY_VALUE);
	

	printf("sram_write 0x0958\n");
	sram_write(0x0958, 0x12345678);
	printf("sram_read  0x0958\n");
	act_result = sram_read(0x0958);
	exp_result = 0x12345678;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM compare passed.\n");
	}
	delayMs(DELAY_VALUE);

	
	printf("sram_write 0x095C\n");
	sram_write(0x0958, 0x87654321);
	printf("sram_read  0x095C\n");
	act_result = sram_read(0x0958);
	exp_result = 0x87654321;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM compare passed.\n");
	}
	delayMs(DELAY_VALUE);

	printf("End SRAM memory R/W tests...\n\n");



	//////////////////////////////
	// SRAM push / pop 
	//////////////////////////////
	// test all queue

	// TODO: Push multiple items on the same queue
	// Pop till end of list reached

	printf("\n\nStart SRAM push/pop tests....\n");

	printf("filling SRAM...\n");
	sram_write(0x0004, 0x15151515);
	sram_write(0x0008, 0x15151515);
	sram_write(0x000C, 0x15151515);
	sram_write(0x0010, 0x15151515);
	sram_write(0x0014, 0x15151515);
	sram_write(0x0018, 0x15151515);
	sram_write(0x001C, 0x15151515);
	sram_write(0x0020, 0x15151515);
	sram_write(0x0024, 0x15151515);
	sram_write(0x0028, 0x15151515);
	sram_write(0x002C, 0x15151515);
	sram_write(0x0030, 0x15151515);
	
	printf("SRAM push: queue=0, addr = 0x0004\n");
	sram_push(0, 0x0004);
	delayMs(DELAY_VALUE);

	printf("SRAM push: queue=1, addr = 0x0008\n");
	sram_push(1, 0x0008);
	delayMs(DELAY_VALUE);

	printf("SRAM push: queue=2, addr = 0x000C\n");
	sram_push(2, 0x000C);
	delayMs(DELAY_VALUE);

	printf("SRAM push: queue=3, addr = 0x0010\n");
	sram_push(3, 0x0010);
	delayMs(DELAY_VALUE);

	printf("SRAM push: queue=4, addr = 0x0014\n");
	sram_push(4, 0x0014);
	delayMs(DELAY_VALUE);

	printf("SRAM push: queue=5, addr = 0x0018\n");
	sram_push(5, 0x0018);
	delayMs(DELAY_VALUE);

	printf("SRAM push: queue=6, addr = 0x001C\n");
	sram_push(6, 0x001C);
	delayMs(DELAY_VALUE);

	printf("SRAM push: queue=7, addr = 0x0020\n");
	sram_push(7, 0x0020);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=0, addr = 0x0004\n");
	sram_pop(0, 0x0004);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=1, addr = 0x0008\n");
	sram_pop(1, 0x0008);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=2, addr = 0x000C\n");
	sram_pop(2, 0x000C);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=3, addr = 0x0010\n");
	sram_pop(3, 0x0010);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=4, addr = 0x0014\n");
	sram_pop(4, 0x0014);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=5, addr = 0x0018\n");
	sram_pop(5, 0x0018);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=6, addr = 0x001C\n");
	sram_pop(6, 0x001C);
	delayMs(DELAY_VALUE);

	printf("SRAM pop:  queue=7, addr = 0x0020\n");
	sram_pop(7, 0x0020);
	delayMs(DELAY_VALUE);

	printf("End SRAM push/pop tests....\n\n");
	

	//////////////////////////////
	// SRAM Set bits
	//////////////////////////////
	printf("Start SRAM Set bit(s) test...\n");
	
	WRITE_LWORD((SRAM_SET_BITS + 0x0000),0x0);
	WRITE_LWORD((SRAM_SET_BITS + 0x0004),0x0); 
	WRITE_LWORD((SRAM_SET_BITS + 0x0008),0x0); 
	WRITE_LWORD((SRAM_SET_BITS + 0x000C),0x0); 
	
	// test 1
 	sram_set_bits(0x0000, 0xFFFFFFFF);
	
	unsigned int *pVal = 0;
	READ_LWORD((SRAM_SET_BITS + 0x0000), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);

	exp_result = 0xFFFFFFFF;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SetBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM set bit test\n");
	}
	delayMs(DELAY_VALUE);


	// test 2
	sram_set_bits(0x0004, 0x7FFFFFFF);
	
	pVal = 0;
	READ_LWORD((SRAM_SET_BITS + 0x0004), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);

	exp_result = 0x7FFFFFFF;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SetBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM set bit test\n");
	}
	delayMs(DELAY_VALUE);


	// test 3
	sram_set_bits(0x0008, 0x3FFFFFFF);

	pVal = 0;
	READ_LWORD((SRAM_SET_BITS + 0x0008), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);


	exp_result = 0x3FFFFFFF;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SetBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM set bit test\n");
	}
	delayMs(DELAY_VALUE);	


	// test 4
	// this test doesn't work
	WRITE_LWORD((SRAM_READ_WRITE_BEGIN + 0x000C),0x0);
	sram_set_bits(0x000C, 0xF);

	pVal = 0;
	READ_LWORD((SRAM_SET_BITS + 0x000C ), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);	

	exp_result = 0xF;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: (4) hal_console_test, SetBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM set bits test\n");
	}
	delayMs(DELAY_VALUE);
	printf("End SRAM Set bit(s) test...\n");


	////////////////////
	// clear bits
	////////////////////
	printf("Start SRAM Clear bit(s) test...\n");

	WRITE_LWORD((SRAM_CLEAR_BITS + 0x0400),0xffffffff); 
	WRITE_LWORD((SRAM_CLEAR_BITS + 0x0404),0xffffffff); 
	WRITE_LWORD((SRAM_CLEAR_BITS + 0x0408),0xffffffff); 
	WRITE_LWORD((SRAM_CLEAR_BITS+ 0x040C), 0xffffffff);

	// test 1
	sram_clear_bits(0x0400, 0xFFFFFFFF);

	pVal = 0;
	READ_LWORD((SRAM_CLEAR_BITS + 0x0400), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);	
	
	exp_result = 0x00000000;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, ClearBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM clear bits test\n");
	}
	delayMs(DELAY_VALUE);

	// test 2
	sram_clear_bits(0x0404, 0xFFFFFFFF);

	pVal = 0;
	READ_LWORD((SRAM_CLEAR_BITS + 0x0404), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);	
	

	exp_result = 0x00000000;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, ClearBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM clear bits\n");
	}
	delayMs(DELAY_VALUE);


	// test 3
	sram_clear_bits(0x0408, 0xFFFFFFFF);

	pVal = 0;
	READ_LWORD((SRAM_CLEAR_BITS + 0x0408), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);	
	
	exp_result = 0x00000000;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, ClearBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM clear bits\n");
	}
	delayMs(DELAY_VALUE);


	// test 4
	sram_clear_bits(0x040C, 0xFFFFFFFF);

	pVal = 0;
	READ_LWORD((SRAM_CLEAR_BITS + 0x040C), pVal);
	act_result = *pVal;
	printf("actual result = 0x%x\n", act_result);	

	exp_result = 0x00000000;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, ClearBits mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SRAM clear bits\n");
	}
	delayMs(DELAY_VALUE);

	printf("End SRAM Clear bit(s) test...\n\n");


	//////////////////////////////
	// SRAM test and set bits
	//////////////////////////////
	printf("Start SRAM test and set bit...\n");
	
	WRITE_LWORD((SRAM_READ_WRITE_BEGIN + 0x8700), 0xffffffff); 
	WRITE_LWORD((SRAM_READ_WRITE_BEGIN + 0x8708), 0xffffffff); 
	WRITE_LWORD((SRAM_READ_WRITE_BEGIN + 0x870C), 0xffffffff);
	// WRITE_LWORD((SRAM_TEST_AND_SET_BITS + 0x8700), 0x12121212); 
	// WRITE_LWORD((SRAM_TEST_AND_SET_BITS + 0x8708), 0x12121212); 
	// WRITE_LWORD((SRAM_TEST_AND_SET_BITS + 0x870C), 0x12121212);
			
	// test 1
	act_result = sram_test_and_set_bits(0x8700, 0xffffffff);
	printf("actual result = 0x%x\n", act_result);	

	exp_result = 0x12121212;		
	delayMs(DELAY_VALUE);

	// READ memory loc to make sure set worked
	// READ_LWORD((SRAM_TEST_AND_SET_BITS + 0x8700))

	
	// test 2
	act_result = sram_test_and_set_bits(0x8708, 0xffffffff);
	printf("actual result = 0x%x\n", act_result);	

	exp_result = 0x12121212;		
	delayMs(DELAY_VALUE);
	

	// test 3
	act_result = sram_test_and_set_bits(0x8708, 0xffffffff);
	printf("actual result = 0x%x\n", act_result);	

	exp_result = 0x12121212;		
	delayMs(DELAY_VALUE);


	// act_result = sram_test_and_clear_bits(0x0700, 0x12121212);
	
	printf("End SRAM test and set bit...\n\n");



	//////////////////////////////
	// SRAM lock and unlock
	//////////////////////////////
	printf("Start SRAM Lock/Unlock tests...\n");
	unsigned int value;

	sram_write_unlock(0x0800, 0x15151515);
	sram_read_lock(0x0800, &value);
	// test SRAM_CSR Read Lock Status
	if (Locked() == false) {
		printf("ERROR");
		printf("ERROR: (1) hal_console_test, SRAM Read Lock, SRAM_CSR Read Lock Status bit not set.\n");
		fail = 1;
	} else {
		printf("PASS: ReadLock bit is set\n");
	}
	act_result = value;
	exp_result = 0x15151515;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: (1) hal_console_test, SRAM Read Lock R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: expected result\n");
	}
	delayMs(DELAY_VALUE);

	sram_write_unlock(0x0804, 0xCAFECAFE);
	sram_read_lock(0x0804, &value);
	// test SRAM_CSR Read Lock Status
	if (Locked() == false) {
		printf("ERROR");
		printf("ERROR: (2) hal_console_test, SRAM Read Lock, SRAM_CSR Read Lock Status bit not set.\n");
		fail = 1;
	} else {
		printf("PASS: ReadLock bit is set\n");
	}
	act_result = value;
	exp_result = 0xCAFECAFE;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: (2) hal_console_test, SRAM Read Lock R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: expected result\n");
	}
	delayMs(DELAY_VALUE);

	// This test fails - RETRY QUEUE BIT ISN'T SET after two sram_read_lock's from the core.
	value = 0;
	// lock the same address !!!!!
	sram_write_unlock(0x0000, 0xCAFECAFE);

	sram_read_lock(0x0000, &value);
	// check that we got the read lock - RLS = 1, got status

	sram_read_lock(0x0000, &value); // retry queue
	// check that you didn't get the read lock - RLS = 0, failed

	// two read locks, make sure SRAM_CSR is RLRS is set (Read Lock Retry Status bit)
	// if (LockRetry() == false) {
	//	printf("ERROR");
	//	printf("ERROR: hal_console_test, SRAM Read Lock, SRAM_CSR Read Lock Status Retry bit not set.\n");
	//	fail = 1;
	//}

	sram_write_unlock(0x0000, 0xCAFECAFE);
	// sram read above address compare - make sure CAFECAFE

	// two read locks, make sure SRAM_CSR is RLRS is set (Read Lock Retry Status bit)
	//if (LockRetry() == false) {
	//	printf("ERROR");
	//	printf("ERROR: hal_console_test, SRAM Read Lock, SRAM_CSR Read Lock Status Retry bit not set.\n");
	//	fail = 1;
	//}

	act_result = value;
	exp_result = 0xCAFECAFE;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM Read Lock R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	}

	// last unlock
	sram_write_unlock(0x0000, 0xCAFECAFE);
	


	// delayMs(DELAY_VALUE);



	sram_write_unlock(0x0008, 0x15151515);
	sram_read_lock(0x0008, &value);
	act_result = value;
	exp_result = 0x15151515;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SRAM Read Lock R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: expected result\n");
	}
	delayMs(DELAY_VALUE);
	printf("End SRAM Lock/Unlock tests...\n\n");


	//////////////////////////////
	// SDRAM read/write tests
	//////////////////////////////
	printf("Start SDRAM memory R/W tests...\n");	

	// test 1
	sdram_write(0xC0000000, 0x15151515);
	act_result = sdram_read(0xC0000000);
	exp_result = 0x15151515;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SDRAM R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SDRAM R/W test.\n");
	}
	delayMs(DELAY_VALUE);
	

	// test 2
	sdram_write(0xC0000002, 0xDEADBEEF);
	act_result = sdram_read(0xC0000002);
	exp_result = 0xDEADBEEF;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SDRAM R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SDRAM R/W test.\n");
	}
	delayMs(DELAY_VALUE);
	
	// test 3
	sdram_write(0xC0000004, 0xCAFECAFE);
	act_result = sdram_read(0xC0000004);
	exp_result = 0xCAFECAFE;		
	if (act_result != exp_result){
		printf("ERROR");
		printf("ERROR: hal_console_test, SDRAM R/W mismatch, expected = %x, actual = %x\n", exp_result, act_result);
		fail = 1;
	} else {
		printf("PASS: SDRAM R/W test.\n");
	}
	delayMs(DELAY_VALUE);
	printf("End SDRAM memory R/W tests...\n\n");

  
	/////////////////////////////////////////////////
	// End all Test Cases, Print All Results
	/////////////////////////////////////////////////
	printf("HAL tests complete.\n");

	if (fail == 1){
		printf("FAIL");
		printf("FAIL: HAL console tests.\n");
		return HAL_FAIL;
	} else {
		printf("SUCCESS");
		printf("SUCCESS: HAL console tests.\n");
		return HAL_SUCCESS;
	}

} // end hal_testall()