/* rs_udbg_TestServer.cpp: Remote uDebug Server Test Application 
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Debug library
 * author: Lai-Wah Hui 12/17/98

 * revisions:
 * $History: rs_udbg_TestServer.cpp $
 * 
 * *****************  Version 25  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:14
 * Updated in $/dev_1200/SA1_CoreLibs/SA1200core_Build
 * Cleanup
 * 
 * *****************  Version 23  *****************
 * User: Lhui         Date: 4/06/00    Time: 12:19p
 * Updated in $/dev_1200/SA1_CoreLibs/SA1200core_Build
 * Fix build error
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 6/29/99    Time: 7:19p
 * Updated in $/dev_1200/SA1_CoreLibs/SA1200core_Build
 * Fix build errors
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 6/25/99    Time: 7:40p
 * Updated in $/dev_1200/SA1_CoreLibs/SA1200core_Build
 * Fix bugs
 * 
 * *****************  Version 17  *****************
 * User: Lhui         Date: 6/23/99    Time: 5:13p
 * Updated in $/dev_1200/SA1_CoreLibs/SA1200core_Build
 * Add remote service 
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:10p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases for uDbg_InitLib and uDbg_GetUengPC
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 4/02/99    Time: 2:50p
 * Updated in $/development/qa-test/rsdbg_tests
 * Fix bugs
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 3/29/99    Time: 6:07p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 10  *****************
 * User: Qabuild      Date: 3/22/99    Time: 11:57a
 * Updated in $/development/qa-test/rsdbg_tests
 * 
 * *****************  Version 9  *****************
 * User: Qabuild      Date: 3/22/99    Time: 11:19a
 * Updated in $/development/qa-test/rsdbg_tests
 * Updated debug.h to debug_1200.h
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 3/19/99    Time: 1:21p
 * Updated in $/development/qa-test/rsdbg_tests
 * Update for the new debug SA1200 path
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 3/18/99    Time: 5:52p
 * Updated in $/development/qa-test/rsdbg_tests
 * change "udbg_lib.h" to "debug.h"
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 3/15/99    Time: 12:07p
 * Updated in $/development/qa-test/rsdbg_tests
 * Link with rs_udbg_client_dll
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 3/09/99    Time: 6:25p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 2/25/99    Time: 8:54p
 * Updated in $/development/qa-test/rsdbg_tests
 * Updated for the latest version of udebug related changes
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 2/22/99    Time: 7:54p
 * Updated in $/development/qa-test/rsdbg_tests
 * Updated for udebug related changes
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 1/28/99    Time: 6:30p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 12/18/98   Time: 5:26p
 * Created in $/development/qa-test/rsdbg_tests
 * Add test cases
 */ 
#include <stdio.h>
#include <iostream.h>
#include <assert.h>

#include "ts.h"
#include "sem.h"
#include "xact.h"
#include "ostype.h"

extern "C" 
{
extern int UseSerialDebugConnection;
void Rs_udbg_server();
void Rs_SC_server();
}

static Thread* rs_udbgServer = NULL;

int Rs_udbg_TestServer_init()   // start remote debug server using RPC over Ethernet
{
    UseSerialDebugConnection = 0;
#if (OS != UCOS)
	rs_udbgServer = new Thread(Rs_udbg_server);
	if(rs_udbgServer == NULL) return 0;
#endif
    return 1;  // success
}

int Rs_SC_Server_init()         // start debug server using Serial default device
{                               // which is PCI for VxWorks and COM2 for UCOS and WIN32
#if 0
    UseSerialDebugConnection = 1;
#if (OS == UCOS)
	rs_udbgServer = new Thread(Rs_SC_server, 6);
#else
	rs_udbgServer = new Thread(Rs_SC_server);
#endif
    if (rs_udbgServer == NULL) return 0;
#endif
    return 1;  // success
}

void Rs_udbg_TestServer_exit()
{
	if(rs_udbgServer != NULL)
		delete (rs_udbgServer);
    rs_udbgServer = NULL;
}



