/* sa1200CoreStart.c
 *
 * -----------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem:
 * author: Henry Qian
 */
#include <stdio.h>
#include "safexact.h"
#include "ambaio.h"
#include "ts.h"
#include "uclo.h"

extern void svlibTest(char *uof_file, int testCase);
extern void ambaIoInitialize(char *ChipName);
extern "C" void  FDriver_Init(void);
extern void testCase_1(void);

extern "C" {
    int XactIoInit(void);
    int Pci_Init();
}

void SA1200_Transactor(void);
void simpleSystemTest(void);
void xactInit(void);
void loadMicroCode(char* microCodeFile);
void enableMicroEngines(void);
void threadWait(int n);

extern void *objHandle;
extern PFV intFboxHandler;

int xactInitDone;

/*
**  void main(void) for Windowns NT
**  void sa1200CoreStart(void) for VxWorks
**
**  DESCRIPTION
**      The main start point of SA1200 core software.
**
**  RETURNS
**      None.
*/
#ifdef WIN32
void main( int argc, char *argv[] )
#endif
#ifdef VX_WORKS
void sa1200CoreStart(void)
#endif
{
    // PCI Device Driver
#ifdef INCLUDE_MODULE_PCI
#ifdef VX_WORKS
    Pci_Init();
#endif // VX_WORKS
#endif // INCLUDE_MODULE_PCI

    // simulation of the SA1200 chip
    SA1200_Transactor();

    // Micro Engine Device Driver
    FDriver_Init();
 
    // Micro Code Loader
    UcLo_InitLib();

    // load the ucode
    loadMicroCode("sv.list");

    enableMicroEngines();

    // System test
	simpleSystemTest();

    // Check out test results with Transactor
    //
    XACT_ExecuteCommandStr("goto sv_waitforcall#,10000");
    XACT_ExecuteCommandStr("go 50");
    printf(" -- core to uThread --\n");
    XACT_ExecuteCommandStr("ex sram[100:103]");
    printf(" -- signal done register --\n");
    XACT_ExecuteCommandStr("ex sram[201]");
    printf(" -- uThread to Core --\n");
    XACT_ExecuteCommandStr("ex sram[1:4]");
    printf(" -- results --\n");
    XACT_ExecuteCommandStr("ex sram[0x400]");

    transactor_exit();
}

void simpleSystemTest(void)
{
    // load the object
    if(UcLo_LoadObjFile(&objHandle, "svlibApp.uof"))
        printf("ERROR: UcLo_LoadObjFile failed\n");

    Thread* TestCaseTh = new  Thread(testCase_1);

    threadWait(100);
}


/*
**  void xactInit(void)
**
**  DESCRIPTION
**      Initialize the Transactor.  Anything related to the
**      chip simulation will be done here, which includes
**      XactIo API and AMBA I/O initializations.  Finally this
**      thread will be kept alive to prevent Transactor quit.
**
**  RETURNS
**      None.
*/
void xactInit(void)
{
    xactInitDone = 0;

    transactor_init();

    intFboxHandler = 0;

    XACT_ExecuteCommandStr("log simpleSystemTest.log");
    XACT_ExecuteCommandStr("chip");
    XACT_ExecuteCommandStr("mem_init sdram 1m");
    XACT_ExecuteCommandStr("mem_init sram 1m");
    XACT_ExecuteCommandStr("init");

    // Transactor has been initialized
    //
    xactInitDone = 1;

    // keep the thread alive
    while (1) {
        threadWait(100000);
    }
}

/*
**  void SA1200_Transactor(void)
**
**  DESCRIPTION
**      Launch a thread to initialize and run Transactor.
**      In order to simulate the micro engines well, the
**      Transactor should be running in one thread, and
**      AMBA I/O API routines will be used in another threads.
**      They run parallelly and not block each other.
**
**      This routine doesn't return until the initialization
**      is completely done.
**
**  RETURNS
**      None.
*/
void SA1200_Transactor(void)
{
    xactInitDone = 0;
    Thread* thread = new Thread((void(*)(void))xactInit);

    // wait for Transactor initializes itself and runs
    // at a separate thread.
    while (xactInitDone == 0)
        threadWait(20);
}

void loadMicroCode(char* microCodeFile)
{
    char buffer[64];

    // simulation reset
    XACT_ExecuteCommandStr("sim_reset");

    sprintf(buffer, "load_uc f0.ctl.ustore %s *", microCodeFile);
    XACT_ExecuteCommandStr(buffer);
    printf(" -- load micro code done");
}

void enableMicroEngines(void)
{
    // exit on error
    XACT_ExecuteCommandStr("dep/s sim.error_handle_mode = 3");

    XACT_ExecuteCommandStr("int standalone = 0;");
    XACT_ExecuteCommandStr("int ii;");
    XACT_ExecuteCommandStr("int TRUE = 1;");
    XACT_ExecuteCommandStr("int FALSE = 0;");
    XACT_ExecuteCommandStr("int result;");
    XACT_ExecuteCommandStr("int any_fail = FALSE;");

    // enable debug state for fbox 0 only
    XACT_ExecuteCommandStr("dep/s f0.art.enable = 1");
    XACT_ExecuteCommandStr("dep/s f1.art.enable = 0");
    XACT_ExecuteCommandStr("dep/s f2.art.enable = 0");
    XACT_ExecuteCommandStr("dep/s f3.art.enable = 0");
    XACT_ExecuteCommandStr("dep/s f4.art.enable = 0");
    XACT_ExecuteCommandStr("dep/s f5.art.enable = 0");
    XACT_ExecuteCommandStr("dep/s f0.art.debug_p0 = 1");
    XACT_ExecuteCommandStr("dep/s f0.art.debug_p1 = 1");
    XACT_ExecuteCommandStr("dep/s f0.art.debug_p2 = 1");
    XACT_ExecuteCommandStr("dep/s f0.art.debug_p3 = 1");
    XACT_ExecuteCommandStr("dep/s f0.art.debug_p4 = 1");

    // set abort ops in the last 4 stages of the pipe
    // in order to avoid processing garbage uwords (these
    // will be latched in the last 4 stages at the start of
    // the clock edge
    XACT_ExecuteCommandStr("dep/s f0.ctl.p0_abort = 1");
    XACT_ExecuteCommandStr("dep/s f0.ctl.p1_abort = 1");
    XACT_ExecuteCommandStr("dep/s f0.ctl.p2_abort = 1");
    XACT_ExecuteCommandStr("dep/s f0.ctl.p3_abort = 1");

    // enable goto label
    XACT_ExecuteCommandStr("dep/s art.fbox_mask = 1");   // enable fbox 0
    XACT_ExecuteCommandStr("dep/s art.ctx_mask = 1");    // enable context 0
    XACT_ExecuteCommandStr("dep/s f0.art.ctx_mask = 1"); // enable fbox 0 context 0

    // reset sram ctrl
    XACT_ExecuteCommandStr("c.c_reset = 1;");
    XACT_ExecuteCommandStr("go 2");
    XACT_ExecuteCommandStr("c.c_reset = 0;");

    XACT_ExecuteCommandStr("goto sv_waitforcall#");

    printf(" -- ambaIoInit() ..");
    ambaIoInitialize("");
    while (ambaIoInitDone() != 1) {
	    printf(".");
        XACT_ExecuteCommandStr("go 10");
    }

    printf("\n -- AMBAIO_Init() done.\n");
}

void threadWait(int n)
{
#ifdef WIN32
        Sleep(n);
#elif VX_WORKS
        taskDelay(n);
#else
        sleep(n);
#endif
}

// end of file
