/* svlib_tests.cpp
 *
 ** ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Xact_IO
 * author: Don Hooper 1/29/98
 * revisions:
 * $History
 * 
 * 
 */ 
/*=================================================================*/


#include <stdio.h>
#include "ueng.h"
#include "xact.h"
#include "ambaio.h"
#include "Xact_API.h"
#include "libd.h"
#include "uclo.h"
#include "memsv.h"
#include "hashsv.h"
#include "TS.h"

void *objHandle = 0;
static int libdInitialized = 0;

#ifdef WIN32
HANDLE ThreadHandle;
DWORD dwThreadId;
DWORD dwThreadParm;
#endif


void assignUdestId(void)
{
	static int once = 0;
	if(once) return;
	once++;

	// assign destination IDs 0 thru 23 to the Uengines
	if(LibD_AssignUThreadId(0, 0, 24)){
		printf("LibD_AssignUThreadId failed\n");
		return;
	}
   return;
}


//
// 2-way sync memory service library test
//
extern void testCase_1(void);
extern void testCase_2(void);
extern void testCase_3(void);
extern void testCase_4(void);
extern void testCase_5(void);

//
// 2-way sync hash service library test
//
extern void testCase_11(void);
extern void testCase_12(void);
extern void testCase_13(void);
extern void testCase_14(void);

//
// 2-way async memory service library test
//
extern void testCase_21(void);
extern void testCase_22(void);
extern void testCase_23(void);
extern void testCase_24(void);
extern void testCase_25(void);

//
// 2-way async hash service library test
//
extern void testCase_26(void);
extern void testCase_27(void);
extern void testCase_28(void);
extern void testCase_29(void);

extern void testCase_33(void);

extern void testCase_43(void);

//
// 1-way sync memory service library test
//
extern void testCase_101(void);
extern void testCase_102(void);
extern void testCase_103(void);
extern void testCase_104(void);
extern void testCase_105(void);

//
// 1-way sync hash service library test
//
extern void testCase_111(void);
extern void testCase_112(void);
extern void testCase_113(void);
extern void testCase_114(void);

//
// 1-way async memory service library test
//
extern void testCase_121(void);
extern void testCase_122(void);
extern void testCase_123(void);
extern void testCase_124(void);
extern void testCase_125(void);

//
// 1-way async hash service library test
//
extern void testCase_126(void);
extern void testCase_127(void);
extern void testCase_128(void);
extern void testCase_129(void);

extern void testCase_133(void);

//
// registered with Transactor
// This function is intended to be launched at Transactor
//
void svlibTest(char *uof_file, int testCase)
{
    Thread* TestCaseTh;
	unsigned char uEng = 0, startCtx = 0;
	unsigned int ctxEnMask = 0xf;

	//
    // load micro code only once
    //
    if (libdInitialized == 0)
    {
        if (objHandle)
            UcLo_DeleObj(objHandle);

	    UcLo_InitLib();

	    // load the object
	    if(UcLo_LoadObjFile(&objHandle, uof_file))
		    printf("ERROR: UcLo_LoadObjFile failed\n");

		// enable ueng0 contexts
		if(uEng_start(uEng, startCtx, ctxEnMask))
			printf("ERROR: uEng_enableCtx failed\n");

        libdInitialized++;
    }


    switch (testCase)
    {
        case 1:
            TestCaseTh = new  Thread(testCase_1);

            break;

        case 2:
            TestCaseTh = new  Thread(testCase_2);
            break;

        case 3:
            TestCaseTh = new  Thread(testCase_3);
            break;

        case 4:
            TestCaseTh = new  Thread(testCase_4);
            break;
 
        case 5:
            TestCaseTh = new  Thread(testCase_5);
            break;

        case 11:
            TestCaseTh = new  Thread(testCase_11);
            break;

        case 12:
            TestCaseTh = new  Thread(testCase_12);
            break;

        case 13:
            TestCaseTh = new  Thread(testCase_13);
            break;

        case 14:
            TestCaseTh = new  Thread(testCase_14);
            break;

        case 21:
            TestCaseTh = new  Thread(testCase_21);
            break;

        case 22:
            TestCaseTh = new  Thread(testCase_22);
            break;

        case 23:
            TestCaseTh = new  Thread(testCase_23);
            break;

        case 24:
            TestCaseTh = new  Thread(testCase_24);
            break;
 
        case 25:
            TestCaseTh = new  Thread(testCase_25);
            break;

        case 26:
            TestCaseTh = new  Thread(testCase_26);
            break;

        case 27:
            TestCaseTh = new  Thread(testCase_27);
            break;

        case 28:
            TestCaseTh = new  Thread(testCase_28);
            break;

        case 29:
            TestCaseTh = new  Thread(testCase_29);
            break;

        case 33:
            TestCaseTh = new  Thread(testCase_33);
            break;

        case 34:
            TestCaseTh = new  Thread(testCase_1);
            TestCaseTh = new  Thread(testCase_2);
            break;

        case 35:
            TestCaseTh = new  Thread(testCase_1);
            TestCaseTh = new  Thread(testCase_2);
            TestCaseTh = new  Thread(testCase_3);
            TestCaseTh = new  Thread(testCase_4);
            TestCaseTh = new  Thread(testCase_5);
            TestCaseTh = new  Thread(testCase_11);
            TestCaseTh = new  Thread(testCase_12);
            TestCaseTh = new  Thread(testCase_13);
            TestCaseTh = new  Thread(testCase_14);
            break;

        case 43:
            TestCaseTh = new  Thread(testCase_43);
            break;

        case 44:
            TestCaseTh = new  Thread(testCase_21);
            TestCaseTh = new  Thread(testCase_22);
            break;

        case 45:
            TestCaseTh = new  Thread(testCase_21);
            TestCaseTh = new  Thread(testCase_22);
            TestCaseTh = new  Thread(testCase_23);
            TestCaseTh = new  Thread(testCase_24);
            TestCaseTh = new  Thread(testCase_25);
            TestCaseTh = new  Thread(testCase_26);
            TestCaseTh = new  Thread(testCase_27);
            TestCaseTh = new  Thread(testCase_28);
            TestCaseTh = new  Thread(testCase_29);
            break;

        case 101:
            TestCaseTh = new  Thread(testCase_101);
            break;

        case 102:
            TestCaseTh = new  Thread(testCase_102);
            break;

        case 103:
            TestCaseTh = new  Thread(testCase_103);
            break;

        case 104:
            TestCaseTh = new  Thread(testCase_104);
            break;
 
        case 105:
            TestCaseTh = new  Thread(testCase_105);
            break;

        case 111:
            TestCaseTh = new  Thread(testCase_111);
            break;

        case 112:
            TestCaseTh = new  Thread(testCase_112);
            break;

        case 113:
            TestCaseTh = new  Thread(testCase_113);
            break;

        case 114:
            TestCaseTh = new  Thread(testCase_114);
            break;

        case 133:
            TestCaseTh = new  Thread(testCase_133);
            break;

        case 134:
            TestCaseTh = new  Thread(testCase_101);
            TestCaseTh = new  Thread(testCase_102);
            break;

        case 135:
            TestCaseTh = new  Thread(testCase_101);
            TestCaseTh = new  Thread(testCase_102);
            TestCaseTh = new  Thread(testCase_103);
            TestCaseTh = new  Thread(testCase_104);
            TestCaseTh = new  Thread(testCase_105);
            TestCaseTh = new  Thread(testCase_111);
            TestCaseTh = new  Thread(testCase_112);
            TestCaseTh = new  Thread(testCase_113);
            TestCaseTh = new  Thread(testCase_114);
            break;

        default:
            printf("ERROR: bad test case.\n");
            break;
    }
#ifdef WIN32
    Sleep(2000);
#endif
#ifdef VX_WORKS
	taskDelay(1000);	
#endif

}


/*------------------------------------------------------------------------
    testCase_1
        2-way synch MemSv_TestAndSet() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_1(void)
{
    printf(" -- MemSv_TestAndSet() (testCase_1) ----------------------\n");

    XactIoWrite(SRAM, 0, 0xFFFF0000, 0x400);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x51);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }

	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    int outArg;
    msvTASIn msvTASInArg = {0xDEADBEEF, (void*)0x400};

    retval = MemSv_TestAndSet(libHandle, &msvTASInArg, &outArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_TestAndSet() return: %x\n", outArg);
    else
        printf("ERROR: MemSv_TestAndSet() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_2
        2-way synch MemSv_BZero() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_2(void)
{
    printf(" -- MemSv_BZero() (testCase_2) ---------------------------\n");

    for (int ii = 0; ii < 0x10; ii++)
        XactIoWrite(SRAM, 0, 0xA5A5A5A5, 0x410 + ii);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x52);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    msvBuf msvBufInArg = {0x10<<2, (void*)(0x410<<2)};

    retval = MemSv_BZero(libHandle, &msvBufInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_BZero()\n");
    else
        printf("ERROR: MemSv_BZero() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_3
        2-way synch MemSv_BCopy() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_3(void)
{
    printf(" -- MemSv_BCopy() (testCase_3) ---------------------------\n");

    for (int ii = 0; ii < 0x10; ii++)
    {
        XactIoWrite(SRAM, 0, 0x33333333, 0x430 + ii);
        XactIoWrite(SRAM, 0, 0x44444444, 0x440 + ii);
    }

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x53);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    msvBCopyIn msvBCopyInArg = {0x10<<2, (void*)(0x430<<2), (void*)(0x440<<2)};

    retval = MemSv_BCopy(libHandle, &msvBCopyInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_BCopy()\n");
    else
        printf("ERROR: MemSv_BCopy() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_4
        2-way synch MemSv_Gather() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_4(void)
{
    printf(" -- MemSv_Gather() (testCase_4) --------------------------\n");

    int ii;
    for (ii = 0; ii < 0x10; ii++)
    {
        // source buffers
        XactIoWrite(SRAM, 0, 0x55555555, 0x450 + ii);
        XactIoWrite(SRAM, 0, 0x66666666, 0x460 + ii);
    }

    // source buffer description
    XactIoWrite(SRAM, 0, 0x10<<2, 0x470 + 0);
    XactIoWrite(SRAM, 0, 0x450<<2, 0x470 + 1);
    XactIoWrite(SRAM, 0, 0x10<<2, 0x470 + 2);
    XactIoWrite(SRAM, 0, 0x460<<2, 0x470 + 3);

    // destination buffer
    for (ii = 0; ii < 0x20; ii++)
        XactIoWrite(SRAM, 0, 0, 0x480 + ii);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x54);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    msvGatherIn msvGatherInArg = {2, (void*)0x470, (void*)0x480};

    retval = MemSv_Gather(libHandle, &msvGatherInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_Gather()\n");
    else
        printf("ERROR: MemSv_Gather() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_5
        2-way synch MemSv_Scatter() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_5(void)
{
    printf(" -- MemSv_Scatter() (testCase_5) -------------------------\n");

    int ii;
    for (ii = 0; ii < 0x10; ii++)
    {
        // source buffer
        XactIoWrite(SRAM, 0, 0x77777777, 0x500 + ii);
        XactIoWrite(SRAM, 0, 0x88888888, 0x510 + ii);
        // destination buffer
        XactIoWrite(SRAM, 0, 0x00000000, 0x530 + ii);
        XactIoWrite(SRAM, 0, 0x00000000, 0x540 + ii);
    }

    // destination buffer description
    XactIoWrite(SRAM, 0, 0x10<<2, 0x520 + 0);
    XactIoWrite(SRAM, 0, 0x530<<2, 0x520 + 1);
    XactIoWrite(SRAM, 0, 0x10<<2, 0x520 + 2);
    XactIoWrite(SRAM, 0, 0x540<<2, 0x520 + 3);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x55);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    msvScatterIn msvScatterInArg = {2, (void*)0x500, (void*)0x520};

    retval = MemSv_scatter(libHandle, &msvScatterInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_Scatter()\n");
    else
        printf("ERROR: MemSv_Scatter() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_11
        2-way synch HashSv_LdMult48() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_11(void)
{
    printf(" -- HashSv_LdMult48() (testCase_11) ----------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x61);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    hsv_Data hsv_DataInArg = {0x1234, 0x56789ABC};

    retval = HashSv_LdMult48(libHandle, &hsv_DataInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: HashSv_LdMult48()\n");
    else
        printf("ERROR: HashSv_LdMult48() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_12
        2-way synch HashSv_LdMult64() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_12(void)
{
    printf(" -- HashSv_LdMult64() (testCase_12) ----------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x62);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    hsv_Data hsv_DataInArg = {0x12345678, 0x9ABCDEF0};

    retval = HashSv_LdMult64(libHandle, &hsv_DataInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: HashSv_LdMult64()\n");
    else
        printf("ERROR: HashSv_LdMult64() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_13
        2-way synch HashSv_Exec48() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_13(void)
{
    printf(" -- HashSv_Exec48() (testCase_13) ------------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x63);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    hsv_Data hsv_DataInArg = {0x12345678, 0x9ABCDEF0};
    hsv_Data hsv_DataOutArg = {0, 0};

    retval = HashSv_Exec48(libHandle, &hsv_DataInArg, &hsv_DataOutArg);
    if ((retval >= 0) && (retval <= 7))
    {
        printf("SUCCESSSUCCESS: HashSv_Exec48()\n");
        printf("return: %08X  %08X\n", hsv_DataOutArg.hi32, hsv_DataOutArg.lo32);
    }
    else
        printf("ERROR: HashSv_Exec48() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_14
        2-way synch HashSv_Exec64() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_14(void)
{
    printf(" -- HashSv_Exec64() (testCase_14) ------------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x64);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    hsv_Data hsv_DataInArg = {0x12345678, 0x9ABCDEF0};
    hsv_Data hsv_DataOutArg = {0, 0};

    retval = HashSv_Exec64(libHandle, &hsv_DataInArg, &hsv_DataOutArg);
    if ((retval >= 0) && (retval <= 7))
    {
        printf("SUCCESSSUCCESS: HashSv_Exec64()\n");
        printf("return: %08X  %08X\n", hsv_DataOutArg.hi32, hsv_DataOutArg.lo32);
    }
    else
        printf("ERROR: HashSv_Exec64() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_21
        2-way asynch MemSv_TestAndSet() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_21(void)
{
    printf(" -- MemSv_TestAndSet() (testCase_21) ---------------------\n");

    XactIoWrite(SRAM, 0, 0xFFFF0000, 0x400);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x71);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    int outArg;
    msvTASIn msvTASInArg = {0xDEADBEEF, (void*)0x400};

    retval = MemSv_TestAndSet(libHandle, &msvTASInArg, &outArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_TestAndSet() return: %x\n", outArg);
    else
        printf("ERROR: MemSv_TestAndSet() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_22
        2-way asynch MemSv_BZero() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_22(void)
{
    printf(" -- MemSv_BZero() (testCase_22) ---------------------------\n");

    for (int ii = 0; ii < 0x10; ii++)
        XactIoWrite(SRAM, 0, 0xA5A5A5A5, 0x410 + ii);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x72);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    msvBuf msvBufInArg = {0x10<<2, (void*)(0x410<<2)};

    retval = MemSv_BZero(libHandle, &msvBufInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_BZero()\n");
    else
        printf("ERROR: MemSv_BZero() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_23
        2-way asynch MemSv_BCopy() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_23(void)
{
    printf(" -- MemSv_BCopy() (testCase_23) ---------------------------\n");

    for (int ii = 0; ii < 0x10; ii++)
    {
        XactIoWrite(SRAM, 0, 0x33333333, 0x430 + ii);
        XactIoWrite(SRAM, 0, 0x44444444, 0x440 + ii);
    }

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x73);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    msvBCopyIn msvBCopyInArg = {0x10<<2, (void*)(0x430<<2), (void*)(0x440<<2)};

    retval = MemSv_BCopy(libHandle, &msvBCopyInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_BCopy()\n");
    else
        printf("ERROR: MemSv_BCopy() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_24
        2-way asynch MemSv_Gather() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_24(void)
{
    printf(" -- MemSv_Gather() (testCase_24) --------------------------\n");

    int ii;
    for (ii = 0; ii < 0x10; ii++)
    {
        // source buffers
        XactIoWrite(SRAM, 0, 0x55555555, 0x450 + ii);
        XactIoWrite(SRAM, 0, 0x66666666, 0x460 + ii);
    }

    // source buffer description
    XactIoWrite(SRAM, 0, 0x10<<2, 0x470 + 0);
    XactIoWrite(SRAM, 0, 0x450<<2, 0x470 + 1);
    XactIoWrite(SRAM, 0, 0x10<<2, 0x470 + 2);
    XactIoWrite(SRAM, 0, 0x460<<2, 0x470 + 3);

    // destination buffer
    for (ii = 0; ii < 0x20; ii++)
        XactIoWrite(SRAM, 0, 0, 0x480 + ii);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x74);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    msvGatherIn msvGatherInArg = {2, (void*)0x470, (void*)0x480};

    retval = MemSv_Gather(libHandle, &msvGatherInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_Gather()\n");
    else
        printf("ERROR: MemSv_Gather() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_25
        2-way asynch MemSv_Scatter() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_25(void)
{
    printf(" -- MemSv_Scatter() (testCase_25) -------------------------\n");

    int ii;
    for (ii = 0; ii < 0x10; ii++)
    {
        // source buffer
        XactIoWrite(SRAM, 0, 0x77777777, 0x500 + ii);
        XactIoWrite(SRAM, 0, 0x88888888, 0x510 + ii);
        // destination buffer
        XactIoWrite(SRAM, 0, 0x00000000, 0x530 + ii);
        XactIoWrite(SRAM, 0, 0x00000000, 0x540 + ii);
    }

    // destination buffer description
    XactIoWrite(SRAM, 0, 0x10<<2, 0x520 + 0);
    XactIoWrite(SRAM, 0, 0x530<<2, 0x520 + 1);
    XactIoWrite(SRAM, 0, 0x10<<2, 0x520 + 2);
    XactIoWrite(SRAM, 0, 0x540<<2, 0x520 + 3);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x75);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    msvScatterIn msvScatterInArg = {2, (void*)0x500, (void*)0x520};

    retval = MemSv_scatter(libHandle, &msvScatterInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_Scatter()\n");
    else
        printf("ERROR: MemSv_Scatter() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_26
        2-way asynch HashSv_LdMult48() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_26(void)
{
    printf(" -- HashSv_LdMult48() (testCase_26) ----------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x81);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    hsv_Data hsv_DataInArg = {0x1234, 0x56789ABC};

    retval = HashSv_LdMult48(libHandle, &hsv_DataInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: HashSv_LdMult48()\n");
    else
        printf("ERROR: HashSv_LdMult48() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_27
        2-way asynch HashSv_LdMult64() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_27(void)
{
    printf(" -- HashSv_LdMult64() (testCase_27) ----------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x82);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    hsv_Data hsv_DataInArg = {0x12345678, 0x9ABCDEF0};

    retval = HashSv_LdMult64(libHandle, &hsv_DataInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: HashSv_LdMult64()\n");
    else
        printf("ERROR: HashSv_LdMult64() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_28
        2-way asynch HashSv_Exec48() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_28(void)
{
    printf(" -- HashSv_Exec48() (testCase_28) ------------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x83);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    hsv_Data hsv_DataInArg = {0x1234, 0x56789ABC};
    hsv_Data hsv_DataOutArg = {0, 0};

    retval = HashSv_Exec48(libHandle, &hsv_DataInArg, &hsv_DataOutArg);
    if ((retval >= 0) && (retval <= 7))
    {
        printf("SUCCESSSUCCESS: HashSv_Exec48()\n");
        printf("return: %08X  %08X\n", hsv_DataOutArg.hi32, hsv_DataOutArg.lo32);
    }
    else
        printf("ERROR: HashSv_Exec48() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_29
        2-way asynch HashSv_Exec64() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_29(void)
{
    printf(" -- HashSv_Exec64() (testCase_29) ------------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x84);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    hsv_Data hsv_DataInArg = {0x12345678, 0x9ABCDEF0};
    hsv_Data hsv_DataOutArg = {0, 0};

    retval = HashSv_Exec64(libHandle, &hsv_DataInArg, &hsv_DataOutArg);
    if ((retval >= 0) && (retval <= 7))
    {
        printf("SUCCESSSUCCESS: HashSv_Exec64()\n");
        printf("return: %08X  %08X\n", hsv_DataOutArg.hi32, hsv_DataOutArg.lo32);
    }
    else
        printf("ERROR: HashSv_Exec64() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_31
        only has .ind test file
        called two different micro code functions sequentially
    return:
  ------------------------------------------------------------------------*/

/*------------------------------------------------------------------------
    testCase_32
        only has .ind test file
        called two same micro code functions sequentially
    return:
  ------------------------------------------------------------------------*/

/*------------------------------------------------------------------------
    testCase_33
        2-way synch MemSv_TestAndSet() call (10 times)
    return:
  ------------------------------------------------------------------------*/

void testCase_33(void)
{
    printf(" -- MemSv_TestAndSet() (testCase_33) ---------------------\n");


    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x85);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_PEND_CALL;

    int outArg;

    // initialize the location to keep test result
    int numOfTestPassed = 0;
    XactIoWrite(SRAM, 0, numOfTestPassed, 0x401);

    for (int ii = 0; ii < 10; ii++)
    {
        printf(" -- call function %dth time\n", ii+1);
        XactIoWrite(SRAM, 0, 0xFFFF0000, 0x400);
        msvTASIn msvTASInArg = {0xDEADA5A5, (void*)0x400};

        retval = MemSv_TestAndSet(libHandle, &msvTASInArg, &outArg);
        if ((retval >= 0) && (retval <= 7))
        {
            numOfTestPassed++;
            XactIoWrite(SRAM, 0, numOfTestPassed, 0x401);
            printf("SUCCESSSUCCESS: MemSv_TestAndSet() return: %x\n", outArg);
        }
        else
            printf("ERROR: MemSv_TestAndSet() failed\n");
    }

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_43
        2-way asynch MemSv_TestAndSet() call (10 times)
    return:
  ------------------------------------------------------------------------*/
void testCase_43(void)
{
    printf(" -- MemSv_TestAndSet() (testCase_43) ---------------------\n");

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x86);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_SYNC_CALL;

    int outArg;

    // initialize the location to keep test result
    int numOfTestPassed = 0;
    XactIoWrite(SRAM, 0, numOfTestPassed, 0x401);

    for (int ii = 0; ii < 10; ii++)
    {
        printf(" -- call function %dth time\n", ii+1);
        XactIoWrite(SRAM, 0, 0xFFFF0000, 0x400);
        msvTASIn msvTASInArg = {0xDEADA5A5, (void*)0x400};

        retval = MemSv_TestAndSet(libHandle, &msvTASInArg, &outArg);
        if ((retval >= 0) && (retval <= 7))
        {
            numOfTestPassed++;
            XactIoWrite(SRAM, 0, numOfTestPassed, 0x401);
            printf("SUCCESSSUCCESS: MemSv_TestAndSet() return: %x\n", outArg);
        }
        else
            printf("ERROR: MemSv_TestAndSet() failed\n");
    }

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_101
        1-way synch MemSv_TestAndSet() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_101(void)
{
    printf(" -- MemSv_TestAndSet() (testCase_101) ----------------------\n");

    XactIoWrite(SRAM, 0, 0xFFFF0000, 0x400);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x51);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    int outArg;
    msvTASIn msvTASInArg = {0xDEADBEEF, (void*)0x400};

    retval = MemSv_TestAndSet(libHandle, &msvTASInArg, &outArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_TestAndSet() return: %x\n", outArg);
    else
        printf("ERROR: MemSv_TestAndSet() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_102
        1-way synch MemSv_BZero() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_102(void)
{
    printf(" -- MemSv_BZero() (testCase_102) ---------------------------\n");

    for (int ii = 0; ii < 0x10; ii++)
        XactIoWrite(SRAM, 0, 0xA5A5A5A5, 0x410 + ii);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x52);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    msvBuf msvBufInArg = {0x10<<2, (void*)(0x410<<2)};

    retval = MemSv_BZero(libHandle, &msvBufInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_BZero()\n");
    else
        printf("ERROR: MemSv_BZero() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_103
        1-way synch MemSv_BCopy() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_103(void)
{
    printf(" -- MemSv_BCopy() (testCase_103) ---------------------------\n");

    for (int ii = 0; ii < 0x10; ii++)
    {
        XactIoWrite(SRAM, 0, 0x33333333, 0x430 + ii);
        XactIoWrite(SRAM, 0, 0x44444444, 0x440 + ii);
    }

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x53);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    msvBCopyIn msvBCopyInArg = {0x10<<2, (void*)(0x430<<2), (void*)(0x440<<2)};

    retval = MemSv_BCopy(libHandle, &msvBCopyInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_BCopy()\n");
    else
        printf("ERROR: MemSv_BCopy() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_104
        1-way synch MemSv_Gather() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_104(void)
{
    printf(" -- MemSv_Gather() (testCase_104) --------------------------\n");

    int ii;
    for (ii = 0; ii < 0x10; ii++)
    {
        // source buffers
        XactIoWrite(SRAM, 0, 0x55555555, 0x450 + ii);
        XactIoWrite(SRAM, 0, 0x66666666, 0x460 + ii);
    }

    // source buffer description
    XactIoWrite(SRAM, 0, 0x10<<2, 0x470 + 0);
    XactIoWrite(SRAM, 0, 0x450<<2, 0x470 + 1);
    XactIoWrite(SRAM, 0, 0x10<<2, 0x470 + 2);
    XactIoWrite(SRAM, 0, 0x460<<2, 0x470 + 3);

    // destination buffer
    for (ii = 0; ii < 0x20; ii++)
        XactIoWrite(SRAM, 0, 0, 0x480 + ii);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x54);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    msvGatherIn msvGatherInArg = {2, (void*)0x470, (void*)0x480};

    retval = MemSv_Gather(libHandle, &msvGatherInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_Gather()\n");
    else
        printf("ERROR: MemSv_Gather() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_105
        1-way synch MemSv_Scatter() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_105(void)
{
    printf(" -- MemSv_Scatter() (testCase_105) -------------------------\n");

    int ii;
    for (ii = 0; ii < 0x10; ii++)
    {
        // source buffer
        XactIoWrite(SRAM, 0, 0x77777777, 0x500 + ii);
        XactIoWrite(SRAM, 0, 0x88888888, 0x510 + ii);
        // destination buffer
        XactIoWrite(SRAM, 0, 0x00000000, 0x530 + ii);
        XactIoWrite(SRAM, 0, 0x00000000, 0x540 + ii);
    }

    // destination buffer description
    XactIoWrite(SRAM, 0, 0x10<<2, 0x520 + 0);
    XactIoWrite(SRAM, 0, 0x530<<2, 0x520 + 1);
    XactIoWrite(SRAM, 0, 0x10<<2, 0x520 + 2);
    XactIoWrite(SRAM, 0, 0x540<<2, 0x520 + 3);

    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x55);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    msvScatterIn msvScatterInArg = {2, (void*)0x500, (void*)0x520};

    retval = MemSv_scatter(libHandle, &msvScatterInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: MemSv_Scatter()\n");
    else
        printf("ERROR: MemSv_Scatter() failed\n");

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_111
        1-way synch HashSv_LdMult48() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_111(void)
{
    printf(" -- HashSv_LdMult48() (testCase_111) ----------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x61);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    hsv_Data hsv_DataInArg = {0x1234, 0x56789ABC};

    retval = HashSv_LdMult48(libHandle, &hsv_DataInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: HashSv_LdMult48()\n");
    else
        printf("ERROR: HashSv_LdMult48() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_112
        1-way synch HashSv_LdMult64() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_112(void)
{
    printf(" -- HashSv_LdMult64() (testCase_112) ----------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x62);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    hsv_Data hsv_DataInArg = {0x12345678, 0x9ABCDEF0};

    retval = HashSv_LdMult64(libHandle, &hsv_DataInArg);
    if ((retval >= 0) && (retval <= 7))
        printf("SUCCESSSUCCESS: HashSv_LdMult64()\n");
    else
        printf("ERROR: HashSv_LdMult64() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_113
        1-way synch HashSv_Exec48() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_113(void)
{
    printf(" -- HashSv_Exec48() (testCase_113) ------------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x63);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    hsv_Data hsv_DataInArg = {0x1234, 0x56789ABC};
    hsv_Data hsv_DataOutArg = {0, 0};

    retval = HashSv_Exec48(libHandle, &hsv_DataInArg, &hsv_DataOutArg);
    if ((retval >= 0) && (retval <= 7))
    {
        printf("SUCCESSSUCCESS: HashSv_Exec48()\n");
        printf("return: %08X  %08X\n", hsv_DataOutArg.hi32, hsv_DataOutArg.lo32);
    }
    else
        printf("ERROR: HashSv_Exec48() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}

/*------------------------------------------------------------------------
    testCase_114
        1-way synch HashSv_Exec64() call.
    return:
  ------------------------------------------------------------------------*/
void testCase_114(void)
{
    printf(" -- HashSv_Exec64() (testCase_114) ------------------------\n");

    int retval;
    LibHandle* libHandle = HashSv_Init(objHandle, 0x64);
    if (libHandle == 0)
    {
		printf("ERROR: HashSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    hsv_Data hsv_DataInArg = {0x12345678, 0x9ABCDEF0};
    hsv_Data hsv_DataOutArg = {0, 0};

    retval = HashSv_Exec64(libHandle, &hsv_DataInArg, &hsv_DataOutArg);
    if ((retval >= 0) && (retval <= 7))
    {
        printf("SUCCESSSUCCESS: HashSv_Exec64()\n");
        printf("return: %08X  %08X\n", hsv_DataOutArg.hi32, hsv_DataOutArg.lo32);
    }
    else
        printf("ERROR: HashSv_Exec64() failed\n");

    HashSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


/*------------------------------------------------------------------------
    testCase_133
        1-way synch MemSv_TestAndSet() call (10 times)
    return:
  ------------------------------------------------------------------------*/

void testCase_133(void)
{
    printf(" -- MemSv_TestAndSet() (testCase_133) ---------------------\n");


    int retval;
    LibHandle* libHandle = MemSv_Init(objHandle, 0x85);
    if (libHandle == 0)
    {
		printf("ERROR: MemSv_Init() failed\n");
        return;
    }
	assignUdestId();

    libHandle->tid = 0;
    libHandle->flag = SVLIB_ASYNC_CALL;

    int outArg;

    // initialize the location to keep test result
    int numOfTestPassed = 0;
    XactIoWrite(SRAM, 0, numOfTestPassed, 0x401);

    for (int ii = 0; ii < 10; ii++)
    {
        printf(" -- call function %dth time\n", ii+1);
        XactIoWrite(SRAM, 0, 0xFFFF0000, 0x400);
        msvTASIn msvTASInArg = {0xDEADA5A5, (void*)0x400};

        retval = MemSv_TestAndSet(libHandle, &msvTASInArg, &outArg);
        if ((retval >= 0) && (retval <= 7))
        {
            numOfTestPassed++;
            XactIoWrite(SRAM, 0, numOfTestPassed, 0x401);
            printf("SUCCESSSUCCESS: MemSv_TestAndSet() return: %x\n", outArg);
        }
        else
            printf("ERROR: MemSv_TestAndSet() failed\n");
    }

    MemSv_Close(libHandle);

#ifdef WIN32
    ExitThread(0x0);
#endif
}


// end of file

