/* Uclo_Test.cpp
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 */
#include <stdio.h>
#include "uclo.h"
#include "uclo_test.h"
static void *objHandle;

int UcLoTest_getFileSize(FILE *fp)
{
   fpos_t	endPos;
   long		curFpos;

   curFpos = ftell(fp);
   fseek(fp, 0L, SEEK_END);
   endPos = ftell(fp);
   fseek(fp, curFpos, SEEK_SET);
   return((int)endPos);
}

int UcLoTest_OpenUOFFile(char *fileName,
                 unsigned *fileSize, void ** fBuf)
{
 
    FILE                 *fp;

    if(!fileName) return(UCLO_FILEFAIL);
    if(!(fp = fopen(fileName, "rb"))) return(UCLO_FILEFAIL);

    if(!(*fileSize = UcLoTest_getFileSize(fp))) return(UCLO_FILEFAIL);
    if(!(*fBuf = (char *)malloc(*fileSize))) return(UCLO_MEMFAIL);

    if(fread(*fBuf, 1, *fileSize, fp) != *fileSize){
      fclose(fp);
      free(*fBuf);
      return(UCLO_FILEFAIL);
    }
    fclose(fp);
    return (UCLO_SUCCESS);
}

void test_error( char *fmt, ... )
	{

	char str1[300];

	va_list ap;

	va_start( ap, fmt );

	vsprintf( str1, fmt, ap );

	va_end( ap );

    perror(str1);

	}
  
 /*-----------------------------------------------------------------
	UcLo_TestInIt
	returns: UCLO_SUCCESS 
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									
UcLo_TestInit()    
{
    UcLo_InitLib();
    return UCLO_SUCCESS;
    
}
/*-----------------------------------------------------------------
	UcLo_TestEnd
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int								
UcLo_TestEnd()
        
{
    return UcLo_DeleObj(objHandle);
    
}

/*-----------------------------------------------------------------
	UcLo_LoadObjFileTest
    Test Uclo_LoadObjFile
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									// UCLO_SUCCESS or UCLO_FAIL
UcLo_LoadObjFileTest(
             char *path, 			// IN : UOF File name to be loaded
             unsigned int DeleteObj)         // IN : 
{             

    int         sts = UCLO_SUCCESS;

	/* Load the uCode image from file */
   if((sts = UcLo_LoadObjFile(&objHandle,path) )!= UCLO_SUCCESS){
        test_error("UcLo_LoadObjFileTest() : Error in loading %s file, uclo_status = %d", path, sts);
        return(sts);
   }
   if(DeleteObj == TRUE)
      return UcLo_DeleObj(objHandle);

   return(sts);
}

/*-----------------------------------------------------------------
	UcLo_WriteUImageAllTest
    Test UcLo_WriteUImageAllTest
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									// UCLO_SUCCESS or UCLO_FAIL
UcLo_WriteUImageAllTest()	
{             

    int         sts = UCLO_SUCCESS;
 
    if(sts = UcLo_WriteUimageAll(objHandle) != UCLO_SUCCESS){
        test_error("UcLo_WriteUImageAllTest() uclo_status = %d", sts);
        return sts;
   
    }

    return(sts);
}




/*-----------------------------------------------------------------
	UcLo_LoadObjFileTest
    Test Uclo_LoadObjFile
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int									// UCLO_SUCCESS or UCLO_FAIL
UcLo_MapObjFileTest(char *path, unsigned int DeleteObj)	// IN : UOF File name to be loaded
{             

    int         sts = UCLO_SUCCESS;
    void        *fBuf;
    unsigned int         fileSize;

    if((sts = UcLoTest_OpenUOFFile(path, &fileSize, &fBuf)) != UCLO_SUCCESS){
        test_error("UcLo_MapObjFileTest() : Error in loading %s file, uclo_status = %d", path, sts);
        return sts;
    }
	/* Load the uCode image from file */
 
    if((sts = UcLo_MapObjAddr(&objHandle, fBuf, fileSize, 0)) != UCLO_SUCCESS){
        test_error("UcLo_MapObjFileTest() : Error in mapping %s file, uclo_status = %d", path, sts);
        return(sts);
   }
    if(DeleteObj == TRUE)
        return UcLo_DeleObj(objHandle);

    return(sts);
}


/*-----------------------------------------------------------------
	UcLo_BindSymbolTest
    Test Uclo_BindSymbolTest
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_BindSymbolTest(char *ImageName, 
                        char *SymbolName,
                        unsigned int SymbolVal)

{

    int         sts = UCLO_SUCCESS;
 

	/* BindSymbol test */
   if((sts = UcLo_BindSymbol(objHandle,ImageName,SymbolName,SymbolVal)) != UCLO_SUCCESS ){
        test_error("UcLo_BindSymbol uclo_status = %d", sts);
        return sts;
   }	

   return sts;

   
}

/*-----------------------------------------------------------------
	UcLo_WriteUImagePageTest
    Test Uclo_BindSymbolTest
	returns: UCLO status
	uses:
	modifies: 
  -----------------------------------------------------------------*/
int UcLo_WriteUImagePageTest(char *ImageName, 
                             unsigned int PageNumber)

{

 
    int         sts = UCLO_SUCCESS;
 
    if((sts = UcLo_WriteUimagePage(objHandle, ImageName, PageNumber)) != UCLO_SUCCESS){
        test_error("UcLo_WriteUImagePageTest() uclo_status = %d", sts);
        return sts;
   
    }

    return(sts);
}

#if 0
int BindSymbol_Test1()
{
    unsigned int baseaddr, key;
    char			*fnam = "uclo_test1.uof";
	char *image_name = "uclo_test1";
    void		*objHandle;
	
	/* Load the uCode image from file */
   if(UcLo_LoadObjFile(&objHandle,fnam)){
      printf("Error; loading file %s\n", "uclo_test1");
      return (0);
   }

   /* BindSymbol test */
   baseaddr = 0x00ff;
   if(UcLo_BindSymbol(objHandle,image_name, "hi256_base",baseaddr )) 
           printf("Error; binding symbol 'hi256_base' in Apps 'Test'\n");
   
   baseaddr = 0x00fe;
   if(UcLo_BindSymbol(objHandle,image_name, "hi56k_base",baseaddr)) 
           printf("Error; binding symbol hi56k_base in Apps 'Test'\n");

    
   baseaddr = 0x00fc;

   if(UcLo_BindSymbol(objHandle,image_name, "rt_base",baseaddr )) 
           printf("Error; binding symbol rt_base in Apps 'Test'\n");
  
   key = 0x00fa;
   if(UcLo_BindSymbol(objHandle, image_name, "ip_da",key)!=0) {
       printf("Error; binding symbol ip_da in Apps 'Test'\n");
   }

   if(UcLo_WriteUimageAll(objHandle)) 
      printf("Error; failed to write Apps to uStore\n");

   /* trash the object */
  UcLo_DeleObj(objHandle);

 return(1);

}

#endif





