/* xact_registration.cpp
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: transactor registration 
 * author: Lai-Wah Hui 3/19/98
 * revisions:
 * $History
 * 
 * 
 */ 

/*=================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#if defined(BUILD_DLL)
#include "xact_dll.h"
#else
#include "xact.h"
#endif

#ifdef PKGEN
#include "packet_gen.h"
#endif

#ifdef HAL_CONSOLE
#include "hal_console.h"
#endif

/* instructions
	1. define ext_model_port as a compiler switch when compiling safe_xact
 */

/* external foreigh model functions */
#if defined(__cplusplus)
extern "C"
{
#endif

extern int testLib_consoleCmd();
extern void TempMain();
extern int rtm_model_initialize();
extern int br_model_initialize();
extern int UcLo_Test_initialize();
extern int Utilities_Register();
extern int XactIoInit();
extern int simTicks(unsigned int ticks);
extern void ambaIoInit();
extern void autoAmbaIoInit();
extern void multiChip_ambaIoInit();
extern int AMBAIO_ResetAllChips();
extern unsigned int ambaIoInitDone();
extern void ambaIoTest();
extern int	uDbg_Test_LibInit();
extern void systemInit_Thread(int flag);

#if defined(__cplusplus)
}
#endif

extern "C" void  FDriver_Init(void);
extern void svlibTest(char *uof_file, int testCase);
extern int Rs_udbg_TestServer_init();
extern void Rs_udbg_TestServer_exit();
extern int Rs_SC_Server_init();
extern void Rs_SC_Server_exit();
extern int netapp_model_initialize();


/* this routine will be called to initialize the foreign model after */
/* the transactor "init" command has successfully executed.  Returning 0 */
/* will result in a transactor error. */
int foreign_model_initialize()
{
    if (!XACT_register_console_function("systemInit_Thread", systemInit_Thread, 0, 1))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

#ifdef UCLO_TEST    
    // register UcLo test console functions
    if (!UcLo_Test_initialize()){
            fprintf(stderr,"UcLo_Test_initialize() failed\n");
			return(0);
	}
#endif
    
#ifdef LIBD_TEST
    // register Dispatch test console functions
   	if (!XACT_register_console_function("TempMain", TempMain, 0, 0) ){
 		fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}
    if (!testLib_consoleCmd()){
            fprintf(stderr,"testLib_consoleCmd() failed\n");
			return(0);
	}
#endif

//dj: #ifdef SVLIB_TEST

	if (!XACT_register_console_function("ambaIoInitDone", ambaIoInitDone, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}


	if (!XACT_register_console_function("ambaIoTest", ambaIoTest, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

	if (!XACT_register_console_function("ambaIoInit", ambaIoInit, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

	if (!XACT_register_console_function("simTicks", simTicks, 0, 1))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

	if (!XACT_register_console_function("autoAmbaIoInit", autoAmbaIoInit, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

	if (!XACT_register_console_function("multiChip_ambaIoInit", multiChip_ambaIoInit, 1, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

	if (!XACT_register_console_function("FDriver_Init", FDriver_Init, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

#ifdef MULTI_CHIP
	if (!XACT_register_console_function("XactIoInit", XactIoInit, 1, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

#else
	if (!XACT_register_console_function("XactIoInit", XactIoInit, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

	if (!XACT_register_console_function("svlibTest", svlibTest, 1, 1))
    {
            fprintf(stderr,"XACT_register_console_function() failed\n");
			return(0);
	}

#endif

//dj: #endif // SVLIB_TEST

#ifdef UDEBUG
	uDbg_Test_LibInit();
#endif


#ifdef RS_UDEBUG

	if (!XACT_register_console_function("Rs_udbg_TestServer_init", Rs_udbg_TestServer_init, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function(Rs_udbg_TestServer_ini) failed\n");
			return(0);
	}
	if (!XACT_register_console_function("Rs_SC_Server_init", Rs_SC_Server_init, 0, 0))
    {
            fprintf(stderr,"XACT_register_console_function(Rs_SC_Server_init) failed\n");
			return(0);
	}

#endif

#ifdef NET_APP
    if (!netapp_model_initialize()){
            fprintf(stderr,"netapp_model_initialize() failed\n");
			return(0);
	}

#endif

#ifdef RTM
     // register Route Manager console functions
    if (!rtm_model_initialize()){
            fprintf(stderr,"rtm_model_initialize() failed\n");
			return(0);
	}
#endif

#ifdef BRM
     // register Bridge Manager console functions
    if (!br_model_initialize()){
            fprintf(stderr,"br_model_initialize() failed\n");
			return(0);
	}
#endif  

#ifdef PKGEN
	pkgen_model_initialize();
	fmon_model_initialize();
	fdrive_model_initialize();
#endif

#ifdef HAL_CONSOLE
	hal_model_initialize();
#endif

    return 1;
}


/* this routine will be called prior to each transactor simulation event */
/* It can be used to deposit state values into the transactor model prior */
/* to simulating the next event.  Returning 0 results in an error. */
extern int foreign_model_pre_sim()
{
	return(1);
}


/* this routine will be called subsequent to each transactor simulation event. */
/* It can be used to query transactor simulation state, in order to copy it */
/* into the foreign model simulator. */
extern int foreign_model_post_sim()
{
	return(1);
}

/* this routine will be called just prior to exiting the simulator. */
/* The routine allows the foreign model to clean up, close files, etc */
/* before shutting down the program. */
extern int foreign_model_exit()
{
#ifdef RS_UDEBUG
	Rs_udbg_TestServer_exit();
#endif
    return(1);
}


/* this routine will be called just after reseting the simulator. */
/* The routine allows the foreign model to reset itself to stay */
/* in sync with the simulator. */
int foreign_model_reset()
{
    AMBAIO_ResetAllChips();
	return(1);
}


#ifdef BUILD_DLL
/*-----------------------------------------------------------------
    GetForeignModelFunctions

    This function is exported as the sole entry point into the DLL version
    of this package.  The Developer Workbench calls it in order to get the
    foreign model entry points for the transactor.  The Workbench thens
    registers these entry points with the transactor.
    
     returns: 
        uses: 
    modifies: 
*/
#if defined(__cplusplus)
extern "C"
#endif

__declspec(dllexport) void __cdecl
GetForeignModelFunctions(
    int (**ForeignModelInitialize)(),
    int (**ForeignModelPreSim)(),
    int (**ForeignModelPostSim)(),
    int (**ForeignModelExit)(),
    int (**ForeignModelReset)())
{
    *ForeignModelInitialize = foreign_model_initialize;
    *ForeignModelPreSim = foreign_model_pre_sim;
    *ForeignModelPostSim = foreign_model_post_sim;
    *ForeignModelExit = foreign_model_exit;
    *ForeignModelReset = foreign_model_reset;
}
#endif      // #if defined(BUILD_DLL)
