	;; Macro for initializing the fboxes and bringing them out of reset
	;; used in conjuction with all.ind
	;; Unused: R4, R10+

	MACRO
	INIT_FBOXES	$num_of_fboxes, $test_flags

	;; If num_of_fboxes is specified, then that many fboxes will
	;; be initialized. If it is blank, then the IND file must
	;; pre-load R1 with the number of fboxes to be initialized.
	;;
	;; if test_flags is not specified, then the IND file must
	;; pre-load R2 with the test flags. The bits are:
	;;  0:	Big-endian = 1, little-endian = 0
	;; 15:	bank_switch_delay
	;; 16:	flow_thru enable

FBOX0_csrbaseADDR 	 EQU 	0xb0000000
SDRAMbaseADDR 		 EQU 	0xc0000000
SDRAM_CSR 		 EQU 	0xFF000000
SDRAM_MEMCTL0 		 EQU 	0xFF000004
SDRAM_MEMCTL1 		 EQU 	0xFF000008
SDRAM_MEMINIT 		 EQU 	0xFF00000c
SRAMbaseADDR  		 EQU 	0x10000000
 

; normal read/write area
FLASHaddress 		 EQU 	0x00000000
RD_LOCKaddress 		 EQU 	0x12000000
WR_UNLOCKaddress 	 EQU 	0x14000000
CAM_UNLOCKaddress 	 EQU 	0x16000000
BIT_WR_CLEARaddress 	 EQU 	0x18000000
BIT_WR_SETaddress 	 EQU 	0x18800000
PUSHaddress 		 EQU 	0x20000000
POPaddress 		 EQU 	0x24000000
SRAMcsr 		 EQU 	0x38000000
SRAM_REGaddress 	 EQU 	0x3c000000
MaskASB_rd_lk_ack 	 EQU 	 0x001

; R5 -- used to contain   current address
; R7 -- used for scratch 


NUM_FBOXES	RN	R1
TEST_FLAGS	RN	R2

	;; R6 is used to signal the transactor that ucode should be loaded
	;; This is an artifact caused by us not wanting to load the ucode
	;; from the core, because that would make the run much too long
__start
	MOV	R6, #0

	;; R1 is used to allow the IND file to tell the code how many
	;; fboxes should be initialized. It is assumed that the IND
	;; file will pre-load R1 with the correct value.
	IF "$num_of_fboxes" /= ""
	MOV	NUM_FBOXES, #$num_of_fboxes
	ENDIF

	;; R2 is used to allow the IND file to tell the code how the
	;; sdram controller should be programmed. 
	;; See the comments at the top of the file for the values for R2.
	IF "$test_flags" /= ""
	MOV	TEST_FLAGS, #$test_flags
	ENDIF

	; Perform RESET ...now...
		
	;       SRAM initialization sequence
	;
	; *******************************************
	
	; *******************************************
	; Write the following to SRAMcsr
	
	LDR     R9, =SRAMcsr		
	AND	R8, TEST_FLAGS, #0x00018000
	ORR	R8, R8, #0x00004800
	STR     R8, [R9]		;  initialize SRAM


	;; set flash assert/deassert times for rd/wr pins and ce
;	LDR	R8, =0x16180804
;	LDR	R8, =0x07080201	; temp value for faster runs
	LDR	R8, =0x090A0501	; temp value for faster runs
	STR	R8, [R9,#+0x24]

	;; set mac assert/deassert times for rd/wr pins and ce
;	LDR	R8, =0x313F0402
	LDR	R8, =0x0B0C0703
	STR	R8, [R9,#+0x28]

	;;  set cycle times for sram/flash
	;;  Note: this needs to be set after the writes to 24, 28
;	LDR	R8, =0x00001F81
;	LDR	R8, =0x00000981	; temp value for faster runs
;	LDR	R8, =0x00000B81	; temp value for faster runs
	LDR	R8, =0x00000B0D	; temp value for faster runs
	STR	R8, [R9,#+0x20]

	;       Release other sub-units  
	;
	; *******************************************
	
	; *******************************************
	; Write the following to PCI_reset register 
	
	LDR     R9, =0x4200007C	
	LDR     R8, =0x00F0003F	        ; Don't bring fboxes out of reset
	STR     R8, [R9]		;; bring functional sub-units out of reset.
		
	;; Note that we shouldn't turn on the cache before this point,
	;; because then the PCI reset register write is immediately
	;; before the SDRAM csr write, and the reset line in the sdram
	;; controller doesn't go low soon enough

	;; Start up MMU for icache & dcache
MMUCP                   CP      15

MMUIDReg                CN      0
MMUControlReg           CN      1
MMUPageTableBaseReg     CN      2
MMUDomainAccessReg      CN      3
MMUFlushTLBReg          CN      5
MMUPurgeTLBReg          CN      6
MMUCacheOps             CN      7
MMUTlbOps               CN      8

table_addr	EQU	0x00004000
domains		EQU	0xFFFFFFFF
		
	;; load translation table pointer to 16K
	LDR	R0, =table_addr
	MCR	MMUCP, 0, R0, MMUPageTableBaseReg, c0
	LDR	R0, =domains
	MCR	MMUCP, 0, R0, MMUDomainAccessReg, c0

	LDR   r0, =0x107D	;  turn on the DCACHE & ICACHE
;	LDR   r0, =0x007D	;  turn on the DCACHE
;	LDR   r0, =0x1079	;  turn on the ICACHE
	MCR   MMUCP, 0, r0, MMUControlReg, c0

	;; end start cache code

	;       SDRAM initialization sequence
	;
	; *******************************************
	
	; *******************************************
	;
	; Write the following to SDRAM_MEMCTL0
	;
	;  REFRCNT           = 12'h500
	;  BURSTL            = 4'b1000
	;  CASL              = 4'b0010
	;  row_addr_width    = 4'b1011
	;  column_addr_width = 4'b1001
	;
	; SDRAM_MEMCTL0 <--  24'h50082b9;
	;
	; *******************************************
	LDR     R5, = SDRAM_MEMCTL0
	LDR     R7, = 0x50082b9
	STR     R7, [R5]
	
	; *******************************************
	;
	; Write the following to SDRAM_MEMCTL1
	;
	;  tRWT    = 4'b0010
	;  tDPL    = 4'b0001
	;  tDQZ    = 4'b0010
	;  tRC     = 4'b1000
	;  tRRD    = 4'b0010
	;  tRCD    = 4'b0001
	;  tRASmin = 4'b0100
	;  tRP     = 4'b0001
	;
	; SDRAM_MEMCTL1 <--  32'h2128_2141
	;
	; *******************************************
	LDR     R5, = SDRAM_MEMCTL1
;	LDR     R7, = 0x21282141
	LDR     R7, = 0x21282252
	STR     R7, [R5]
	
	; *******************************************
	;
	; Write the following to SDRAM_MEMINIT
	;
	;  tRSC        = 4'b0010
	;  INIT_REFRSH = 4'b0010
	;  INIT_DLY    = 16'h0020
	;
	; SDRAM_MEMINIT <--  24'h22_0020;
	;
	; *******************************************
	LDR     R5, = SDRAM_MEMINIT
;	LDR     R7, = 0x220020
	LDR     R7, = 0x000000 ; for faster simulation runs
	STR     R7, [R5]
	
	; *******************************************
	;
	; Programming SDRAM Controller complete!!
	;
	; Write the SDRAM_INIT bit in SDRAM_CSR to
	; begin SDRAM initialization]
	;
	; Also set the "gen_even_parity" bit, to
	; test out parity generation
	;
	;  en_fifo_ram_test    = 1'b0
	;  reserved bits       = 11'b000_0000_0000
	;  en_big_endian_align = 1'b0
	;  en_parity_gen       = 1'b0
	;  gen_even_parity     = 1'b1
	;  start_sdram_init    = 1'b1
	
	;
	; SDRAM_CSR <--  16'h0003;
	;
	; *******************************************
	AND	R5, TEST_FLAGS, #1
	LDR     R7, = 0x0005
	ORR	R7, R7, R5, LSL #1
	LDR     R5, = SDRAM_CSR
	STR     R7, [R5]
	; *******************************************
	B CONTINUE
	LTORG

CONTINUE

	;; For speed, assume tbox control stores already set up
	;; we just need to setup the CSR's
fbox0_csr	EQU	0xb0000000
	
;;	;; step 1: place fboxes in reset
;;	LDR	R9, =0x4200007C
;;	LDR	R8, =0x00F0003F	; fbox5:0 are bits 5:0
;;	STR	R8, [R9]

	;; step 2: clear ctx_enables

	LDR	R7, =fbox0_csr
	MOV	R3, NUM_FBOXES
	
	MOV	R0, #0x0000
disable_sloop
	STR	R0, [R7,#+40]

	ADD	R7, R7, #0x800
	SUBS	R3, R3, #1
	BGT	disable_sloop

	;; step 3: UCode already loaded
	MOV	R6, #1		; signal transactor to load ucode

	;; step 4: set ctx_arb_cntl

	LDR	R7, =fbox0_csr
	MOV	R3, NUM_FBOXES
	
	MOV	R0, #0
enable_sloop
	STR	R0, [R7,#+36]

	;; step 5: set ctx_?_sts (starting addr)
	STR	R0, [R7,#+52]
	STR	R0, [R7,#+56]
	STR	R0, [R7,#+60]
	STR	R0, [R7,#+64]

	ADD	R7, R7, #0x800
	SUBS	R3, R3, #1
	BGT	enable_sloop

	;; step 6: take the fboxes out of reset
;	;; for now, only take fbox 0 out of reset
;	LDR	R8, =0x00F0003E	; fbox5:0 are bits 5:0
	MOV	R8, #0x3F
	MOV	R8, R8, LSL NUM_FBOXES
	AND	R8, R8, #0x3F
	ORR	R8, R8, #0x00F00000
	;; note that R9 still has the value of the reset register
	STR	R8, [R9]

	;; step 7 set wakeup_events (needs to be out of reset)
	
	LDR	R7, =fbox0_csr
	MOV	R3, NUM_FBOXES
	
	MOV	R0, #1
wakeup_sloop
	STR	R0, [R7,#+84]
	STR	R0, [R7,#+88]
	STR	R0, [R7,#+92]
	STR	R0, [R7,#+96]
	
	ADD	R7, R7, #0x800
	SUBS	R3, R3, #1
	BGT	wakeup_sloop

	;; step 8: set ctx_enables

	LDR	R7, =fbox0_csr
	MOV	R3, NUM_FBOXES

	MOV	R0, #0x0F00
tenable_sloop
	STR	R0, [R7,#+40]

	ADD	R7, R7, #0x800
	SUBS	R3, R3, #1
	BGT	tenable_sloop

	MOV	R0, #123

	MEND

    MACRO
    MAKE_FLAT_PAGE_TABLE $first_addr
    ;; Make Flat page table at address 4096 = 0x1000
    ;; $first_addr is symbol of first instruction (at address 0)

	LTORG
	GBLA c
ll
c SETA 0x4000-(ll-$first_addr)
	WHILE c > 0
	DCD 0
c SETA c-4
	WEND

ENTRY * 0xC02			; non-cacheable, non-bufferable

	
c SETA 0
	;; rest enties flat
	WHILE c < 4096
	DCD (((c):SHL:20):OR:ENTRY)
c SETA c+1
	WEND

	MEND

	END
