/* Xact_API.cpp
 *
 * This module provides a preliminary transactor API solution for the first
 * SA1200 software integration testing on VxWorks platform.  
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Xact_IO
 * author: Lai-Wah Hui 1/15/98
 * revisions:
 * $History: Xact_API.cpp $
 * 
 * *****************  Version 49  *****************
 * User: Ewalker      Date: 12/01/00   Time: 4:36p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Created XactIoGetChipName and XactIoSetChipName functions to get/set
 * the chip name to act against when calling XactIo or AMBAIO functions
 * though the core_io.h macros. The default remains the NULL ("") chip for
 * backwards compatibility. Core programs that need to do transactor I/O
 * against a non-NULL chip should call XactIoSetChipName prior to using
 * any of the core_io.h macros and should use XactIoGetChipName in place
 * of hardcoded "" when calling in the AMBAIO and XactIo (multichip)
 * functions directly.
 * 
 * *****************  Version 48  *****************
 * User: Ewalker      Date: 12/01/00   Time: 1:01p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Added ChipName external definition used to direct all Amba I/O and Xact
 * I/O functions against a particular chip.
 * 
 * *****************  Version 47  *****************
 * User: Drjohns1     Date: 11/15/00   Time: 3:46p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Added scratchPad read/write
 * 
 * *****************  Version 42  *****************
 * User: Lhui         Date: 6/18/99    Time: 1:46p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Add support for CSR status
 * 
 * *****************  Version 41  *****************
 * User: Lhui         Date: 6/15/99    Time: 11:37a
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Add support for XACTIO
 * 
 * *****************  Version 40  *****************
 * User: Drjohns1     Date: 6/10/99    Time: 4:49p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Updated for new iostyle -- incomplete
 * 
 * *****************  Version 39  *****************
 * User: Hqianx       Date: 4/27/99    Time: 3:39p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Removed UENGINE_DRIVER.
 * 
 * *****************  Version 37  *****************
 * User: Drjohns1     Date: 4/22/99    Time: 1:50p
 * Updated in $/development/tool/Xact_io
 * Disabled state-transition-callback on writing IREG
 * 
 * *****************  Version 36  *****************
 * User: Drjohns1     Date: 4/16/99    Time: 1:39p
 * Updated in $/development/tool/Xact_io
 * Fixed XactIoReadIREG and created XactIoWriteIREG
 * 
 * *****************  Version 35  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 4:18p
 * Updated in $/development/tool/Xact_io
 * update COPYRIGHT
 * 
 * *****************  Version 34  *****************
 * User: Dhooper      Date: 3/08/99    Time: 4:29p
 * Updated in $/development/tool/Xact_io
 * typo in XactFastWrite change sdam to sdram
 * 
 * *****************  Version 33  *****************
 * User: Dhooper      Date: 3/08/99    Time: 11:17a
 * Updated in $/development/tool/Xact_io
 * add ZeroBuffer and SDZeroBuffer
 * 
 * *****************  Version 32  *****************
 * User: Lhui         Date: 1/12/99    Time: 7:00p
 * Updated in $/development/tool/Xact_io
 * 
 * *****************  Version 31  *****************
 * User: Lhui         Date: 1/04/99    Time: 2:36p
 * Updated in $/development/tool/Xact_io
 * Fix bug in FastRead and FastWrite functions
 * 
 * *****************  Version 30  *****************
 * User: Lhui         Date: 11/24/98   Time: 5:49p
 * Updated in $/development/tool/Xact_io
 * Provide support for the multiple chip feature supported in the
 * Transactor.
 * 
 * *****************  Version 29  *****************
 * User: Hqianx       Date: 10/01/98   Time: 3:14p
 * Updated in $/base_level3/tool/Xact_io
 * Modified XactGetHandle() and XactIoSingalDone() functions
 * to signal all the threads.
 * 
 * *****************  Version 28  *****************
 * User: Lhui         Date: 9/14/98    Time: 1:13p
 * Updated in $/base_level3/tool/Xact_io
 * Update for XACT_Define_Cancel_Callback_State_Transition function call
 * 
 * *****************  Version 27  *****************
 * User: Lhui         Date: 9/04/98    Time: 3:47p
 * Updated in $/base_level3/tool/Xact_io
 * add sram initialization
 * 
 * *****************  Version 26  *****************
 * User: Drjohns1     Date: 8/06/98    Time: 11:39a
 * Updated in $/base_level3/tool/Xact_io
 * Added XactIo_ReadBuffer
 * 
 * *****************  Version 25  *****************
 * User: Dhooper      Date: 7/15/98    Time: 5:29p
 * Updated in $/base_level3/tool/Xact_io
 * add XactIo_WriteBuffer and XactIo_InitSDBuffer
 * 
 * *****************  Version 24  *****************
 * User: Lhui         Date: 6/30/98    Time: 5:52p
 * Updated in $/base_level3/tool/Xact_io
 * Fix name conflict for hDRAM   
 * 
 * *****************  Version 23  *****************
 * User: Rdmurato     Date: 6/09/98    Time: 4:20p
 * Updated in $/base_level3/tool/Xact_io
 * Changes to build a library that links against the DLL version of the
 * transactor.
 * 
 * *****************  Version 22  *****************
 * User: Lhui         Date: 6/04/98    Time: 2:38p
 * Updated in $/base_level3/tool/Xact_io
 * modify XactIoWrite and XactIoRead to support VSIG
 * 
 * *****************  Version 21  *****************
 * User: Lhui         Date: 6/01/98    Time: 10:15a
 * Updated in $/base_level3/tool/Xact_io
 * Remove safexact.h
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 5/20/98    Time: 6:30p
 * Updated in $/base_level2/tool/Xact_io
 * Replace Digital copyright with Intel copyright statement
 * 
 * *****************  Version 19  *****************
 * User: Lhui         Date: 4/24/98    Time: 6:59p
 * Updated in $/base_level2/tool/Xact_io
 * clean up included files
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 3/09/98    Time: 12:01p
 * Updated in $/base_level1/tool/Xact_io
 * Added versioning
 * 
 * *****************  Version 17  *****************
 * User: C-hqian      Date: 3/04/98    Time: 2:30p
 * Updated in $/base_level1/tool/Xact_io
 * Added function: int XactIoReadIREG(unsigned int* value);
 * 
 * *****************  Version 16  *****************
 * User: Lhui         Date: 2/20/98    Time: 1:43p
 * Updated in $/base_level1/tool/Xact_io
 * Fix  bug with semaphore's pointer
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 2/12/98    Time: 10:02a
 * Updated in $/base_level1/tool/Xact_io
 * Add compile switch #ifdef UENGINE_DRIVER for DRIVER code
 * 
 * *****************  Version 14  *****************
 * User: C-hqian      Date: 2/11/98    Time: 5:13p
 * Updated in $/base_level1/tool/Xact_io
 * Made semaphore and thread object global and are instantiated at
 * run-time.
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 2/10/98    Time: 5:37p
 * Updated in $/base_level1/tool/Xact_io
 * Add comments
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 2/09/98    Time: 6:59p
 * Updated in $/base_level1/tool/Xact_io
 * fix the problem with multiple USTORE write
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 2/09/98    Time: 2:03p
 * Updated in $/base_level1/tool/Xact_io
 * Remove path dependencies
 * 
 * *****************  Version 10  *****************
 * User: C-hqian      Date: 2/06/98    Time: 4:55p
 * Updated in $/base_level1/tool/Xact_io
 * Delected Chip, memory initializations from XactIoInit().   It will be
 * done in .ind files.
 * Modified include file's path.
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 2/05/98    Time: 3:06p
 * Updated in $/base_level1/tool/Xact_io
 * Add support for array read/write
 * 
 * *****************  Version 8  *****************
 * User: C-hqian      Date: 2/04/98    Time: 10:30a
 * Updated in $/base_level1/tool/Xact_io
 * Moved #define DRIVER_THREADS to compiler command line.
 * 
 * *****************  Version 7  *****************
 * User: C-hqian      Date: 2/03/98    Time: 11:20a
 * Updated in $/base_level1/tool/Xact_io
 * This is the multi-thread version of the signaling API, which could turn
 * on/off by #define DRIVE_THREADS.
 * 
 * *****************  Version 6  *****************
 * User: C-hqian      Date: 1/26/98    Time: 2:35p
 * Updated in $/base_level1/tool/Xact_io
 * Added functions:
 *     int XactIoSignalDone(void);
 *     int XactIoDefineCallBackFunction(PFI ptrCallBackFunction);
 *     void XactIoDump(int firstMailBox, int numberOfMailBox);
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 1/20/98    Time: 3:33p
 * Updated in $/base_level1/tool/Xact_io
 * Fixed bug with SDRAM 64 bit read/write
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 1/16/98    Time: 8:38p
 * Updated in $/base_level1/tool/VxWork-XactAPI
 * Updated prototype
 *
 *---------------------------------------------------------------------
 */
#ifdef DRIVER_THREADS
#include "TS.h"
Semaphore* driverSemaphore;
Thread* thread1;	
#endif // DRIVER_THREADS

#if defined(BUILD_DLL)
#include "xact_dll.h"
#else
#include "xact.h"
#endif      // #if defined(BUILD_DLL)

#include "Xact_API.h"
#include "msg_map.h"
#include "XactIoLib_Version.h"

XACT_HANDLE hXactSRAM = INVALID_XACT_HANDLE;
XACT_HANDLE hXactSDRAM= INVALID_XACT_HANDLE;
XACT_HANDLE hXactSCRATCH= INVALID_XACT_HANDLE;

static char *ScratchPad = "fbi.scratch_pad";

static void waitOnDriverSignal_Thread(void);
static int signalDone(char *pChipName);

#define NULLChip ""

static char ChipName[125] = NULLChip;  /* current chip name that non-Multi-chip functions act upon */

/* Set the current chip name */
void XactIoSetChipName(char *chipname)
{
    strncpy(ChipName, chipname, sizeof(ChipName) - 1);
    ChipName[sizeof(ChipName) - 1] = '\0';
    return;
}

/* Get the current chip name */
char *XactIoGetChipName(void)
{
    return(&ChipName[0]);
}

/*-------------------------------------------------------------------------
    XactGetHandle
    Get handle of the state
    return: handle of the state
   	uses:       
	modifies:   

  ------------------------------------------------------------------------*/

XACT_HANDLE XactGetHandle(char *pChipName, /* IN: chip name */
						  int storage,    /* IN: either SRAM, SDRAM or USTORE */
			   char MEngineMask,          /* IN: set if storage = USTORE, 0 otherwise */ 
			   int array_index)           /* IN: from memory location within storage */
{
	char CtlBuf[30];
	char StateName[80];
	int i;
	memset(StateName,'\0', sizeof(StateName));
	if(strcmp(pChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", pChipName);

	switch (storage){
		case SRAM:  strcat(StateName,"sram");	
			return XACT_get_handle(StateName, array_index);

		case SDRAM:  strcat(StateName,"sdram");
			return XACT_get_handle(StateName, array_index/2);

		case USTORE:
			for(i=0; i<MAX_MENGINE; i++){
				if(0x01 == MEngineMask)
					break;
				else
					MEngineMask = MEngineMask >> 1;
			}
				sprintf(CtlBuf,"f%d.ctl.ustore",i);
				strcat(StateName, CtlBuf);
		   return XACT_get_handle(StateName, array_index);
		
		case U_SIGNAL:
            // MEngineMask(4:2) uEngine number
            // MEngineMask(1:0) uThread number
			sprintf(CtlBuf,"f%d.ctl.enabled_signals_%d",
                    (MEngineMask & 0x1C) >> 2,
                    MEngineMask & 0x03);
		    strcat(StateName, CtlBuf);
			return XACT_get_handle(StateName, array_index);

		default: return INVALID_XACT_HANDLE;
        }

	return INVALID_XACT_HANDLE;
}


/*-------------------------------------------------------------------------
    XactIORead
    Read data from SRAM or SDRAM storage to buffer location  
    For the USTORE read (read Microcode), it is necessary to 
    specify which Micro Engine the write command applies to.  
    This is done by setting the MEngineMask.  The low six bits 
    indicate the corresponding Micro Engine for that chip 
    (e.g., bit 0 maps to f0, bit 1 maps to f1, etc.)
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:       
	modifies:   

  ------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
int XactIoRead(char	*ChipName,		 /* IN: chip name */
			   int storage,		     /* IN: either SRAM, SDRAM, VSIG or USTORE */ 
               char MEngineMask,     /* IN: set if storage = USTORE, 0 otherwise */
               int from,		     /* IN: from memory location within SRAM or SDRAM */
               unsigned int *element)/* OUT: to CORE memory */		   
#else
/* backward compatibilty */
int XactIoRead(int storage,		     /* IN: either SRAM, SDRAM, VSIG or USTORE */ 
               char MEngineMask,     /* IN: set if storage = USTORE, 0 otherwise */
               int from,		     /* IN: from memory location within SRAM or SDRAM */
               unsigned int *element) /* OUT: to CORE memory */			   
#endif
{
    XACT_HANDLE hXact_Handle;
	int i;

	hXact_Handle = XactGetHandle(ChipName, storage, MEngineMask, from);	
	if(storage == SDRAM){  
            unsigned int SdramElement[2];  
            XACT_get_state_value(hXact_Handle, SdramElement);
            i = from % 2;      
            *element = SdramElement[i];            
            return XACT_SUCCESS;
         }                		
            
	return XACT_get_state_value(hXact_Handle,element);

}
/*------------------------------------------------------------------------
    XactIoWrite
    Write data to SRAM, SDRAM or USTORE storage. For the USTORE write 
    (loading Microcode), it is necessary to specify which Micro Engine the 
    write command applies to.  This is done by setting the MEngineMask.  
    The low six bits indicate the corresponding Micro Engine for that chip 
    (e.g., bit 0 maps to f0, bit 1 maps to f1, etc.)
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:
	modifies:   SDRAM, SRAM, or microengine microcode storage 
 
  ------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
int XactIoWrite(char *ChipName,			/* IN: Chip name */
				int  storage,	        /* IN: SRAM, SDRAM, VSIG or USTORE */
                char MEngineMask,       /* IN: set if storage = USTORE, 0 otherwise */
                unsigned int element,	/* IN: data value */
                int  to)                /* IN: to buffer within SRAM or SDRAM or USTORE */
#else
int XactIoWrite(int  storage,	        /* IN: SRAM, SDRAM, VSIG or USTORE */
                char MEngineMask,       /* IN: set if storage = USTORE, 0 otherwise */
                unsigned int element,	/* IN: data value */
                int  to)                /* IN: to buffer within SRAM or SDRAM or USTORE */
#endif
{

    	XACT_HANDLE hXact_Handle;
        unsigned int SdramElement[2];
		char UstoreMask;
        int i;	
		switch (storage){
			case	SRAM:  
 		    case	U_SIGNAL:
					hXact_Handle = XactGetHandle(ChipName, storage, MEngineMask, to);
				    return XACT_set_state_value(hXact_Handle,&element);
					break;

			case	SDRAM:
					hXact_Handle = XactGetHandle(ChipName, storage, MEngineMask, to);
	                XACT_get_state_value(hXact_Handle, SdramElement);
                    i = to % 2;
                    SdramElement[i] = element;      
                    return XACT_set_state_value(hXact_Handle,SdramElement);
					break;

			case	USTORE:
					UstoreMask = 0x01;

					for(i = 0; i < MAX_MENGINE; i++){
						if((UstoreMask & MEngineMask) != 0){
							hXact_Handle = XactGetHandle(ChipName, storage, (UstoreMask & MEngineMask), to);
							XACT_set_state_value(hXact_Handle, &element);
						}
						UstoreMask = UstoreMask << 1;
					}
					break;
			default: return XACT_ERROR;
         }

         return XACT_SUCCESS;



}

#define BufSize 1000
void TempMain ()
{

	
	// init. chip
#if 0

    unsigned int element;
   	int index, sdramindex;
	unsigned int buf;
	unsigned int DoubleBuf[2];
	unsigned int TestBuf[BufSize];
	unsigned int *pT = &TestBuf[0];

	XACT_ExecuteCommandStr("chip A1");
	XACT_ExecuteCommandStr("chip B1");
	XACT_ExecuteCommandStr("mem_init A1.sdram 1m");
	XACT_ExecuteCommandStr("mem_init B1.sram 1m");
	XACT_ExecuteCommandStr("init");


	XactIoWrite(U_SIGNAL, 0, 0x0A,-1);
    XactIoWrite(U_SIGNAL, 1, 0x1A,-1);
    XactIoWrite(U_SIGNAL, 2, 0x2A,-1);
    XactIoWrite(U_SIGNAL, 3, 0x3A,-1);
    XactIoWrite(U_SIGNAL, 4, 0x4A,-1);
    XactIoWrite(U_SIGNAL, 5, 0x5A,-1);



	XactIoRead(U_SIGNAL, 0, -1, &element);
    XactIoRead(U_SIGNAL, 1, -1, &element);
    XactIoRead(U_SIGNAL, 2, -1, &element);
    XactIoRead(U_SIGNAL, 3, -1, &element);
    XactIoRead(U_SIGNAL, 4, -1, &element);
    XactIoRead(U_SIGNAL, 5, -1, &element);
	for (index=0, sdramindex=0; index < 1000; index++){

      XactFastWrite("A1", SRAM, index, index);
      XactFastWrite("A1", SDRAM, index, index);
      XactFastWrite("B1", SRAM, index, index);
      XactFastWrite("B1", SDRAM, index, index);

	}
	for (index=0; index < 1000; index++){
		XactFastRead("A1", SRAM,index, &buf);
        XactFastRead("A1",SDRAM, index, (unsigned int *)&DoubleBuf);  
		printf("A1: sram[%d]=%d, sdram[%d]=%d:%d\n", index, buf, index, DoubleBuf[0], DoubleBuf[1]);
		XactFastRead("B1",SRAM,index, &buf);
        XactFastRead("B1",SDRAM, index, (unsigned int *)&DoubleBuf);  
		printf("B1: sram[%d]=%d, sdram[%d]=%d:%d\n", index, buf, index, DoubleBuf[0], DoubleBuf[1]);

	}

	for(index=0; index < BufSize; index++)
		TestBuf[index]=index;
	XactIo_WriteBuffer("A1", SRAM, pT,BufSize, 8000);

	for(index=0; index < BufSize; index++)
		TestBuf[index]=index+1000;

 
	XactIo_WriteBuffer("A1", SDRAM, pT,BufSize, 8000);
	for(index=0; index < BufSize; index++)
		TestBuf[index]=index+2000;
 
	
	XactIo_WriteBuffer("B1", SRAM, pT,BufSize, 8000);
	for(index=0; index < BufSize; index++)
		TestBuf[index]=index+3000;

	XactIo_WriteBuffer("B1", SDRAM, pT,BufSize, 8000);
    memset(TestBuf, 0x00, sizeof(TestBuf));

	XactIo_ReadBuffer("A1", SRAM, pT,BufSize, 8000);
	printf("A1-SRAM [0] = %d [500] = %d, [999]= %d\n", TestBuf[0], TestBuf[500], TestBuf[999]);

	XactIo_ReadBuffer("A1", SDRAM, pT,BufSize, 8000);
 	printf("A1-SDRAM [0] = %d [500] = %d, [999]= %d\n", TestBuf[0], TestBuf[500], TestBuf[999]);


	XactIo_ReadBuffer("B1", SRAM, pT,BufSize, 8000);
 	printf("B1-SRAM [0] = %d [500] = %d, [999]= %d\n", TestBuf[0], TestBuf[500], TestBuf[999]);

	XactIo_ReadBuffer("B1", SDRAM, pT,BufSize, 8000);
 	printf("B1-SDRAM [0] = %d [500] = %d, [999]= %d\n", TestBuf[0], TestBuf[500], TestBuf[999]);
#endif


    
#ifdef MULTI_CHIP
    XactIoInit(NULLChip);
#else
    XactIoInit();
#endif
}
/*------------------------------------------------------------------------
    XactIoInit
    Transactor initialization.  This function must be invoked prior
    to performing any driver operations.
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:
  ------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
int XactIoInit(char *ChipName)
#else
int XactIoInit()
#endif
{

    //
    // initialize the beginning section of SRAM, which
    // will be used for simulation of hardware registers
    // and mail boxes (defined in msg_map.h).
    //
    int retval;
    for (int ii = 1; ii < 512; ii++)
    {
#ifdef MULTI_CHIP
        retval = XactIoWrite(ChipName, SRAM,	0,	0,	ii);
#else
        retval = XactIoWrite(SRAM,	0,	0,	ii);
#endif
        if (retval != XACT_SUCCESS)
            return(XACT_ERROR);
    }


#ifdef DRIVER_THREADS
    driverSemaphore = new Semaphore;
    thread1 = new  Thread(waitOnDriverSignal_Thread);
#endif // DRIVER_THREADS

	return XACT_SUCCESS;
}


/*------------------------------------------------------------------------
    XactFastWrite
    Read data from SRAM or SDRAM storage to buffer location   
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:
	modifies:   SDRAM or SRAM
 
  ------------------------------------------------------------------------*/

#ifdef MULTI_CHIP
int XactFastRead(char *ChipName,			 /* IN: Chip name  */
				 int storage,		     /* IN: either SRAM, SDRAM */ 
                 int from,		         /* IN: from memory location within SRAM or SDRAM */
                 unsigned int *element)  /* OUT: to CORE memory */
#else
int XactFastRead(int storage,		     /* IN: either SRAM, SDRAM */ 
                 int from,		         /* IN: from memory location within SRAM or SDRAM */
                 unsigned int *element)  /* OUT: to CORE memory */
			   
#endif
{


    unsigned int SdramElement[2];  
	char StateName[80];
	int i;
 
	memset(StateName,'\0', sizeof(StateName));

	if(strcmp(ChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", ChipName);
#ifdef MULTI_CHIP
	switch (storage){
	
		case	SRAM:
			strcat(StateName, "sram");
			hXactSRAM = XACT_get_handle(StateName, -1);
			return XACT_get_array_state_value(hXactSRAM, from, element ); 
			break;

		case	SDRAM:
			strcat(StateName, "sdram");
			hXactSDRAM= XACT_get_handle(StateName, -1);
			XACT_get_array_state_value(hXactSDRAM, from/2, SdramElement);
		    i = from % 2;      
			*element = SdramElement[i];            
			return XACT_SUCCESS;
		case	SCRATCH:
			strcat(StateName, ScratchPad);
			hXactSCRATCH= XACT_get_handle(StateName, -1);
			return XACT_get_array_state_value(hXactSCRATCH, from, element);
		default: break;
	}                		
#else
	switch (storage){
	
		case	SRAM:
			if(hXactSRAM == INVALID_XACT_HANDLE) {
				strcat(StateName, "sram");
				hXactSRAM = XACT_get_handle(StateName, -1);
			}
			return XACT_get_array_state_value(hXactSRAM, from, element ); 
			break;

		case	SDRAM:
			if(hXactSDRAM== INVALID_XACT_HANDLE) {
					strcat(StateName, "sdram");
					hXactSDRAM= XACT_get_handle(StateName, -1);
			}
			XACT_get_array_state_value(hXactSDRAM, from/2, SdramElement);
            i = from % 2;      
            *element = SdramElement[i];            
            return XACT_SUCCESS;

		case	SCRATCH:
			if(hXactSCRATCH == INVALID_XACT_HANDLE) {
				strcat(StateName, ScratchPad);
				hXactSCRATCH = XACT_get_handle(StateName, -1);
			}
			return XACT_get_array_state_value(hXactSCRATCH, from, element);
		default: break;
	}                		
            
#endif
	return XACT_ERROR;

}

/*------------------------------------------------------------------------
    XactFastWrite
    Write data to SRAM, SDRAM or USTORE storage. 
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:
	modifies:   SDRAM or SRAM. 
 
  ------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
int XactFastWrite(char *ChipName,		/* IN: chip name */
				  int  storage,	        /* IN: SRAM, SDRAM  */
                  unsigned int element,	/* IN: data value */
	              int  to)		        /* IN: to buffer within SRAM or SDRAM or USTORE */

#else
int XactFastWrite(int  storage,	        /* IN: SRAM, SDRAM  */
                  unsigned int element,	/* IN: data value */
	              int  to)		        /* IN: to buffer within SRAM or SDRAM or USTORE */
#endif
{

    unsigned int SdramElement[2];  
	int i,sdram_index;
	char StateName[80];

	memset(StateName,'\0', sizeof(StateName));

	if(strcmp(ChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", ChipName);
#ifdef MULTI_CHIP
	switch (storage){
		
		case	SRAM:
			strcat(StateName, "sram");
			hXactSRAM = XACT_get_handle(StateName, -1);
			return XACT_set_array_state_value(hXactSRAM, to, &element ); 
			break;

		case	SDRAM:
			sdram_index = to/2;
			strcat(StateName,"sdram");
			hXactSDRAM= XACT_get_handle(StateName, -1);
			XACT_get_array_state_value(hXactSDRAM, sdram_index, SdramElement);
			i = to % 2;
			SdramElement[i] = element;      
			return XACT_set_array_state_value(hXactSDRAM, sdram_index, SdramElement ); 

		case	SCRATCH:
			strcat(StateName, ScratchPad);
			hXactSCRATCH = XACT_get_handle(StateName, -1);
			return XACT_set_array_state_value(hXactSCRATCH, to, &element ); 
			break;
		default: break;

	}                		
#else
	switch (storage){
		
		case	SRAM:
			if(hXactSRAM == INVALID_XACT_HANDLE){
				strcat(StateName, "sram");
				hXactSRAM = XACT_get_handle(StateName, -1);
			}
			return XACT_set_array_state_value(hXactSRAM, to, &element ); 
			break;

		case	SDRAM:
			sdram_index = to/2;
			if(hXactSDRAM== INVALID_XACT_HANDLE) {
				strcat(StateName,"sdram");
				hXactSDRAM= XACT_get_handle(StateName, -1);
			}
			XACT_get_array_state_value(hXactSDRAM, sdram_index, SdramElement);
	        i = to % 2;
            SdramElement[i] = element;      
			return XACT_set_array_state_value(hXactSDRAM, sdram_index, SdramElement ); 

		case	SCRATCH:
			if(hXactSCRATCH== INVALID_XACT_HANDLE) {
				strcat(StateName, ScratchPad);
				hXactSCRATCH = XACT_get_handle(StateName, -1);
			}
			return XACT_set_array_state_value(hXactSCRATCH, to, &element ); 
			break;
		default: break;

	}                		
	
#endif

	return XACT_ERROR;

}

/*-----------------------------------------------------------------
	XactIo_ReadBuffer
	Read a buffer of data from transactor SRAM or SDRAM
	returns: XACT_SUCCESS or XACT_ERROR
	uses: XactAPI.cpp
	modifies:
  -----------------------------------------------------------------*/
#ifdef MULTI_CHIP
int										// XACT_SUCCESS or XACT_ERROR
XactIo_ReadBuffer(
				  char *ChipName,		// chip name
               int storage,				// SRAM or USTORE or SDRAM
               unsigned int *inptr,		// location in program memory
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )
#else
int										// XACT_SUCCESS or XACT_ERROR
XactIo_ReadBuffer(
               int storage,				// SRAM or USTORE or SDRAM
               unsigned int *inptr,		// location in program memory
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )

#endif
{
    unsigned int i;

    for (i=0; i < size ; i++) {
#ifdef MULTI_CHIP
        if (!XactFastRead(ChipName, storage,  baseaddr++, inptr++) ){
#else
        if (!XactFastRead(storage,  baseaddr++, inptr++) ){
#endif
            fprintf(stderr,"XactFastRead Failed\n");
            return(XACT_ERROR);
        }
    }
    return(XACT_SUCCESS);
}


/*-----------------------------------------------------------------
	XactIo_WriteBuffer
	Write a buffer of data to transactor SRAM or SDRAM
	returns: XACT_SUCCESS or XACT_ERROR
	uses: XactAPI.cpp
	modifies: transactor sram, ustore or sdram state
  -----------------------------------------------------------------*/
#ifdef MULTI_CHIP
int										// XACT_SUCCESS or XACT_ERROR
XactIo_WriteBuffer(
			   char *ChipName,			// chip name
               int storage,				// SRAM or USTORE or SDRAM (if read modify write)
               unsigned int *inptr,		// location in program memory
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )
#else
XactIo_WriteBuffer(
               int storage,				// SRAM or USTORE or SDRAM (if read modify write)
               unsigned int *inptr,		// location in program memory
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )

#endif
{
    unsigned int i;

    for (i=0; i < size ; i++) {
#ifdef MULTI_CHIP
        if (!XactFastWrite(ChipName, storage, *inptr++, baseaddr++) ){
#else
        if (!XactFastWrite(storage, *inptr++, baseaddr++) ){
#endif
            fprintf(stderr,"XactFastWrite Failed\n");
            return(XACT_ERROR);
        }
    }
    return(XACT_SUCCESS);
}


/*-----------------------------------------------------------------
	XactIo_ZeroBuffer
	Write a buffer of 0s to transactor SRAM or SDRAM
	returns: XACT_SUCCESS or XACT_ERROR
	uses: XactAPI.cpp
	modifies: transactor sram, ustore or sdram state
  -----------------------------------------------------------------*/
#ifdef MULTI_CHIP
int										// XACT_SUCCESS or XACT_ERROR
XactIo_ZeroBuffer(
			   char *ChipName,			// chip name
               int storage,				// SRAM or USTORE or SDRAM (if read modify write)
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )
#else
int
XactIo_ZeroBuffer(
               int storage,				// SRAM or USTORE or SDRAM (if read modify write)
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )

#endif
{
    unsigned int i;

    for (i=0; i < size ; i++) {
#ifdef MULTI_CHIP
        if (!XactFastWrite(ChipName, storage, 0, baseaddr++) ){
#else
        if (!XactFastWrite(storage, 0, baseaddr++) ){
#endif
            fprintf(stderr,"XactFastWrite Failed\n");
            return(XACT_ERROR);
        }
    }
    return(XACT_SUCCESS);
}


/*-----------------------------------------------------------------
	XactIo_InitSDBuffer
	Write an initial buffer of data to transactor SDRAM, writing 2 longwords
	at a time.

	returns: XACT_SUCCESS or XACT_ERROR
	uses: xact_io.c
	modifies: transactor sram, sdram state
  -----------------------------------------------------------------*/
#ifdef MULTI_CHIP
XactIo_InitSDBuffer(char *ChipName,		// chip name
               unsigned int *inptr,		// location in program memory
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )
#else
int										// XACT_SUCCESS or XACT_ERROR
XactIo_InitSDBuffer(
               unsigned int *inptr,		// location in program memory
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )
#endif
{
    unsigned int i; 
	unsigned int SdramElement[2]; 
	unsigned int sdram_index = baseaddr; 
	char StateName[80];

	memset(StateName,'\0', sizeof(StateName));

	if(strcmp(ChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", ChipName);

	strcat(StateName, "sdram");
	hXactSDRAM= XACT_get_handle(StateName, -1);

	for (i=0; i < size ; i+=2) {
	
        SdramElement[0] = *inptr++;      
        SdramElement[1] = *inptr++;      
		if (!XACT_set_array_state_value(hXactSDRAM, sdram_index, SdramElement) ){ 
            fprintf(stderr,"XactIo_WriteSDBuffer Failed\n");
            return(XACT_ERROR);
        }
		sdram_index++;
    }
    return(XACT_SUCCESS);
}

/*-----------------------------------------------------------------
	XactIo_ZeroSDBuffer
	Write an initial buffer of 0s to transactor SDRAM, writing 2 longwords
	at a time.

	returns: XACT_SUCCESS or XACT_ERROR
	uses: xact_io.c
	modifies: transactor sram, sdram state
  -----------------------------------------------------------------*/
#ifdef MULTI_CHIP
int
XactIo_ZeroSDBuffer(char *ChipName,		// chip name
                unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )
#else
int										// XACT_SUCCESS or XACT_ERROR
XactIo_ZeroSDBuffer(
               unsigned int  size,		// size in longwords
               unsigned int baseaddr	// location in transactor memory
               )
#endif
{
    unsigned int i; 
	unsigned int SdramElement[2]; 
	unsigned int sdram_index = baseaddr; 
	char StateName[80];

	memset(StateName,'\0', sizeof(StateName));

	if(strcmp(ChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", ChipName);

	strcat(StateName, "sdram");
	hXactSDRAM= XACT_get_handle(StateName, -1);

	for (i=0; i < size ; i+=2) {
	
        SdramElement[0] = 0;      
        SdramElement[1] = 0;      
		if (!XACT_set_array_state_value(hXactSDRAM, sdram_index, SdramElement) ){ 
            fprintf(stderr,"XactIo_WriteSDBuffer Failed\n");
            return(XACT_ERROR);
        }
		sdram_index++;
    }
    return(XACT_SUCCESS);
}


/*------------------------------------------------------------------------
    XactIoDefineCallBackFunction
    Core uses this function to define a call back function on IREG register,
    which is simulated on SRAM.  Anytime when IREG register changes its
    value, the function gets called.
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:
	modifies:  
 
  ------------------------------------------------------------------------*/

static PFI IREG_CallBackFunction = 0;
#ifdef MULTI_CHIP
int XactIoDefineCallBackFunction(char *ChipName, PFI ptrCallBackFunction )
#else
int XactIoDefineCallBackFunction(PFI ptrCallBackFunction)
#endif
{
	char StateName[80];
	memset(StateName,'\0', sizeof(StateName));

	if(strcmp(ChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", ChipName);

    IREG_CallBackFunction = ptrCallBackFunction;
	strcat(StateName, "sram");

	XACT_HANDLE handle = XACT_get_handle(StateName, INTERRUPT_REASON_REG);
    if (handle == INVALID_XACT_HANDLE)
        return(XACT_ERROR);

    int retval = XACT_Define_Cancel_Callback_State_Transition(
        handle,
        IREG_CallBackFunction);
    if (retval == FALSE)
        return(XACT_ERROR);

	retval = XACT_Define_Callback_State_Transition(
        handle,0,
	    IREG_CallBackFunction,
	    "");
    if (retval == FALSE)
        return(XACT_ERROR);

    return(XACT_SUCCESS);
}


/*------------------------------------------------------------------------
    XactIoReadIREG
        Core drivers use this routine to read IREG.
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:
	modifies:  
 
  ------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
int XactIoReadIREG(char *ChipName, unsigned int* value)
#else
int XactIoReadIREG(unsigned int* value)
#endif
{
	char StateName[80];

	memset(StateName,'\0', sizeof(StateName));

	if(strcmp(ChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", ChipName);

	strcat(StateName, "fbi.ireg");
	XACT_HANDLE handle = XACT_get_handle(StateName, -1);
    if (handle == INVALID_XACT_HANDLE)
        return(XACT_ERROR);
	int retval = XACT_get_state_value(handle,value);
    if (retval != XACT_SUCCESS)
        return(XACT_ERROR);

    return(XACT_SUCCESS);
}


/*------------------------------------------------------------------------
    XactIoWriteIREG
        Core drivers use this routine to write IREG.
    return:     XACT_SUCCESS or XACT_ERROR
   	uses:
	modifies:  
 
  ------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
int XactIoWriteIREG(char *ChipName, unsigned int value)
#else
int XactIoWriteIREG(unsigned int value)
#endif
{
	char StateName[80];
	int retval;

	memset(StateName,'\0', sizeof(StateName));

	if(strcmp(ChipName, NULLChip) != 0)
		sprintf(StateName,"%s.", ChipName);

	strcat(StateName, "fbi.ireg");
	XACT_HANDLE handle = XACT_get_handle(StateName, -1);
    if (handle == INVALID_XACT_HANDLE)
        return(XACT_ERROR);

	if(IREG_CallBackFunction){
	   // disable callback so a write doesn't generate an interrupt
       if((retval = XACT_Cancel_State_Transition_Callback(handle)) == FALSE)
          return(XACT_ERROR);

	   if((retval = XACT_set_state_value(handle, &value)) != XACT_SUCCESS)
          return(XACT_ERROR);

       if((retval = XACT_Define_Cancel_Callback_State_Transition(handle,
									IREG_CallBackFunction)) == FALSE)
          return(XACT_ERROR);


	   if((retval = XACT_Define_Callback_State_Transition(handle,0,
								IREG_CallBackFunction, "")) == FALSE)
          return(XACT_ERROR);	
	}
    else
	   if((retval = XACT_set_state_value(handle, &value)) != XACT_SUCCESS)
          return(XACT_ERROR);

    return(XACT_SUCCESS);
}



/*------------------------------------------------------------------------
    XactIoSignalDone
    Core application uses this function to signal micro engine thread.
    return:     XACT_SUCCESS/XACT_ERROR
   	uses:
	modifies:  
 
  ------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
int XactIoSignalDone(char *ChipName)
#else
int XactIoSignalDone(void)
#endif
{
#ifdef DRIVER_THREADS
    //
    // In multi-thread situation, a thread is waiting on
    // driverSemaphore.  So we only need to release the semaphore.
    // That thread will be unblocked and call signalDone().
    // 
	driverSemaphore->Give();
    return(XACT_SUCCESS);
#else
    //
    // In single thread situation, core driver directly
    // call this routine.
    //

	int retval = signalDone(ChipName);
    return(retval);
#endif // DRIVER_THREADS

}

static int signalDone(char *pChipName)
{
    int retval;
    unsigned int signalDoneRegValue;
    const int coreSignalDoneBit = (1 << 4);
	unsigned int signalValue;
    //
    // read Signal Done Register, which is simulated on SRAM
    // with offset CORE_SIGNAL_DONE (defined in Xact_API.h).
    //
#ifdef MULTI_CHIP
    retval = XactIoRead(pChipName, SRAM, 0, CORE_SIGNAL_DONE, &signalDoneRegValue);
#else
    retval = XactIoRead(SRAM, 0, CORE_SIGNAL_DONE, &signalDoneRegValue);
#endif
    if (retval != XACT_SUCCESS)
        return(XACT_ERROR);

    //
    // Assume we have only one chip and its name is null.
    // The following line needs to be modified if we have
    // multiple chips or chip has its name.
    //
#ifdef MULTI_CHIP
	XactIoRead(pChipName, U_SIGNAL, signalDoneRegValue, -1, &signalValue);
#else
	XactIoRead(U_SIGNAL, signalDoneRegValue, -1, &signalValue);
#endif

    signalValue |= coreSignalDoneBit;
#ifdef MULTI_CHIP
	XactIoWrite(pChipName, U_SIGNAL,signalDoneRegValue, signalValue,-1);
#else
	XactIoWrite(U_SIGNAL,signalDoneRegValue, signalValue,-1);
#endif
    return(XACT_SUCCESS);
}

/*------------------------------------------------------------------------------
    waitOnDriverSignal_Thread
    Transactor spawns this thread, which is waiting on a semaphore first.
    When core driver has a signal to send to us, it releases the semaphore.
    return:      None.
   	uses:
	modifies:  
 
  -----------------------------------------------------------------------------*/


#ifdef DRIVER_THREADS
static void waitOnDriverSignal_Thread(void)
{
	while(1)
	{
		//
		// wait on semaphore for Core drivers to signal us.
		//
		driverSemaphore->Take();

		//
		// received signal from core driver.
		// signalDone() will read Signal Done register and
		// update Transactor accordingly.
		//
		signalDone();
	}
}
#endif // DRIVER_THREADS

/*-----------------------------------------------------------------------------
    XactIoDump
    Display interrupt reason register, signal done register
    and mail boxes, which are used between micro threads and core applications.
    return:      None.
   	uses:
	modifies:  
 
  ----------------------------------------------------------------------------*/
#ifdef MULTI_CHIP
void XactIoDump(char *ChipName,		 // chip name
				int firstMailBox,    // the first mail box to be displayed
                int numberOfMailBox) // the number of mail box to be displayed
#else
void XactIoDump(int firstMailBox,    // the first mail box to be displayed
                int numberOfMailBox) // the number of mail box to be displayed

#endif
{
    int retval;
    int wordIndex;
    unsigned int value;

    //
    // Display two signaling registers, which are used
    // between micro threads and core apps.
    //
#ifdef MULTI_CHIP
	retval = XactIoRead(ChipName, SRAM, 0, INTERRUPT_REASON_REG, &value);
#else
	retval = XactIoRead(SRAM, 0, INTERRUPT_REASON_REG, &value);
#endif
    if (retval == XACT_SUCCESS)
        printf("Interrupt Reason Register: 0x%08X\n", value);
#ifdef MULTI_CHIP
	retval = XactIoRead(ChipName, SRAM, 0, CORE_SIGNAL_DONE, &value);
#else
	retval = XactIoRead(SRAM, 0, CORE_SIGNAL_DONE, &value);
#endif
    if (retval == XACT_SUCCESS)
        printf("Core Signal Done Register: 0x%08X\n", value);

    //
    // Display mail boxes, which are used
    // between micro threads and core apps.
    //
    for (   int mailBoxIndex = firstMailBox;
            mailBoxIndex < firstMailBox + numberOfMailBox;
            mailBoxIndex++)
    {
        printf("Mail box %d:\n", mailBoxIndex);

        printf("  ");
        for (wordIndex = 0; wordIndex < 4; wordIndex++)
        {
#ifdef MULTI_CHIP
	        retval = XactIoRead(ChipName,   SRAM,
                                    0,
                                    UTHREAD_TO_CORE_MAILBOXES + 
                                    (mailBoxIndex * 4) + 
                                    wordIndex,
                                    &value);
#else
	        retval = XactIoRead(    SRAM,
                                    0,
                                    UTHREAD_TO_CORE_MAILBOXES + 
                                    (mailBoxIndex * 4) + 
                                    wordIndex,
                                    &value);

#endif

            if (retval == XACT_SUCCESS)
                printf("%08X ", value);
        }

        printf(" ");
        for (wordIndex = 0; wordIndex < 4; wordIndex++)
        {
#ifdef MULTI_CHIP
	        retval = XactIoRead(ChipName,   SRAM,
                                    0,
                                    CORE_TO_UTHREAD_MAILBOXES + 
                                    (mailBoxIndex * 4) + 
                                    wordIndex,
                                    &value);
#else
	        retval = XactIoRead(    SRAM,
                                    0,
                                    CORE_TO_UTHREAD_MAILBOXES + 
                                    (mailBoxIndex * 4) + 
                                    wordIndex,
                                    &value);

#endif

            if (retval == XACT_SUCCESS)
                printf("%08X ", value);
        }
        printf("\n");
    }
}

static const unsigned int SRAM_LO = 0x00000000, SRAM_HI = 0x40000000;
static const unsigned int SDRAM_LO = 0xc0000000, SDRAM_HI = 0xffffffff;
static const unsigned int SCRATCH_LO = 0xb0044000, SCRATCH_HI = 0xb0044fff;
static const unsigned int UE_CSR_OFFSET_LO = 0x0, UE_CSR_OFFSET_HI = 0x60;
static const unsigned int UE0_CSR_BASE = 0xb0000000, UE1_CSR_BASE = 0xb0000800, UE2_CSR_BASE = 0xb0001000;
static const unsigned int UE3_CSR_BASE = 0xb0001800, UE4_CSR_BASE = 0xb0002000, UE5_CSR_BASE = 0xb0002800;
static const unsigned int UE_CSR_LO = 0xb0000000, UE_CSR_HI = 0xb0002800 + UE_CSR_OFFSET_HI;

#define USTOREADDR 0xefffffff
#define ACTIVE_STS 0xcfffffff
#define isUSTORE_ADDR(addr) !(addr & 0x000000ff)
#define isUSTORE_DATA(addr) ((addr & 0x0000000ff) == 0x00000004)
#define isACTIVE_CTX_STS(addr) ((addr & 0x000000ff) == 0x00000010)

#define isSDRAM(addr) ((addr >= SDRAM_LO) && (addr <= SDRAM_HI))
#define isSRAM(addr) ((addr >= SRAM_LO) && (addr <= SRAM_HI))
#define isSCRATCH(addr) ((addr >= SCRATCH_LO) && (addr <= SCRATCH_HI))
#define isUE_CSR(addr) ((addr >= UE_CSR_LO) && (addr <= UE_CSR_HI))

static unsigned ustoreCsrData[6];

/*-------------------------------------------------------------------------
    XACTIO_GetAddrHandle
    Get handle of the state based on an address value
    return: handle of the state
   	uses:       
	modifies:   

  ------------------------------------------------------------------------*/
XACT_HANDLE XACTIO_GetAddrHandle(char *pChipName, unsigned int pAddr, unsigned *val)
{
	char CtlBuf[30];
	char StateName[80];
	unsigned int baseaddr;
	StateName[0] = '\0';
	if(strcmp(pChipName, NULLChip) != 0) sprintf(StateName,"%s.", pChipName);
#ifdef _DEBUG
	printf("enter XACTIO_GetAddrHandle: pAddr=%x\n", pAddr);
#endif
	// determine the address range
    if(isSRAM(pAddr)){
#ifdef _DEBUG
		printf("isSRAM = %x\n", pAddr);
#endif
		strcat(StateName,"sram");
		return XACT_get_handle(StateName, pAddr);
	}

    if(isSDRAM(pAddr)){
#ifdef _DEBUG
		printf("isSDRAM = %x\n", pAddr);
#endif
		strcat(StateName,"sdram");
		return XACT_get_handle(StateName, pAddr/2);
	}

	
    if(isSCRATCH(pAddr)){
#ifdef _DEBUG
		printf("isSCRATCH = %x\n", pAddr);
#endif
		strcat(StateName, ScratchPad);
		return XACT_get_handle(StateName, pAddr);
	}


    if(isUE_CSR(pAddr)){
		// assume ustore -- only providing ustore CSR access for now
		
		int ueng;
		switch(pAddr & 0x0000ff00){

			case 0x00:  ueng = 0;
						baseaddr = UE0_CSR_BASE;
						break;
			case 0x0800:  ueng = 1;
						  baseaddr = UE1_CSR_BASE;
						  break;
			case 0x1000: ueng = 2;
		   			     baseaddr = UE2_CSR_BASE;
						 break;

			case 0x1800: ueng = 3; 
					     baseaddr = UE3_CSR_BASE;
					     break;
			case 0x2000: ueng = 4;
					     baseaddr = UE4_CSR_BASE;
	   				     break;

			case 0x2800: ueng = 5;
					     baseaddr = UE5_CSR_BASE;
						 break;

			default:   return INVALID_XACT_HANDLE;
		}
#if 0
		if(isUSTORE_ADDR(pAddr)){
			printf("in isUSTORE_ADDR: pAddr=%x\n", pAddr);
			ustoreCsrData[ueng]=*val;
			return USTOREADDR;
		}
		if(isUSTORE_DATA(pAddr)){
			printf("in isUSTORE_DATA: pAddr=%x\n", pAddr);
			sprintf(CtlBuf,"f%d.ctl.ustore",ueng);
			strcat(StateName, CtlBuf);
			return XACT_get_handle(StateName, ustoreCsrData[ueng]);
		}
#endif
#ifdef _DEBUG

		printf("pAddr=%x\n", pAddr);
#endif


		if(isACTIVE_CTX_STS(pAddr)){

		    XACT_HANDLE hXact_Handle;
			unsigned int p1_uaddr, thrd_active, p1_ctx;
			p1_uaddr=thrd_active=p1_ctx = 0;
             /* get p1_uaddr state */
#ifdef _DEBUG
			printf("enter isACTIVE_CTX_STS %x\n", pAddr);
#endif
			sprintf(CtlBuf,"f%d.art.p1_uaddr",ueng);
			strcat(StateName, CtlBuf);
			hXact_Handle = XACT_get_handle(StateName, -1);
            XACT_get_state_value(hXact_Handle, &p1_uaddr);
			XACT_delete_handle(hXact_Handle );

			/* get p1_no_thrd_active state */

			memset(StateName, '\0', sizeof(StateName));
			if(strcmp(pChipName, NULLChip) != 0) sprintf(StateName,"%s.", pChipName);
			sprintf(CtlBuf,"f%d.art.p1_no_thrd_active",ueng);
			strcat(StateName, CtlBuf);
			hXact_Handle = XACT_get_handle(StateName, -1);
            XACT_get_state_value(hXact_Handle, &thrd_active);
			XACT_delete_handle(hXact_Handle );

			/* get ctl.p1_ctx state */

			memset(StateName, '\0', sizeof(StateName));
			if(strcmp(pChipName, NULLChip) != 0) sprintf(StateName,"%s.", pChipName);
			sprintf(CtlBuf,"f%d.ctl.p1_ctx",ueng);
			strcat(StateName, CtlBuf);
			hXact_Handle = XACT_get_handle(StateName, -1);
            XACT_get_state_value(hXact_Handle, &p1_ctx);
			XACT_delete_handle(hXact_Handle);

			*val = (p1_uaddr << 3) | (!thrd_active << 2) | p1_ctx;
#ifdef _DEBUG
			printf("active_ctx_sts = %x, p1_uaddr=%x, thrd_active=%x, p1_ctx=%x\n", *val, p1_uaddr, thrd_active, p1_ctx);
#endif
			return ACTIVE_STS;
		}
		sprintf(CtlBuf,"f%d.ctl.ustore",ueng);
		strcat(StateName, CtlBuf);
		return XACT_get_handle(StateName, pAddr-baseaddr);


	}
	return INVALID_XACT_HANDLE;
}


int XACTIO_Write32(char *chipname,unsigned intaddr, unsigned intval)
{
    XACT_HANDLE hXact_Handle;
#ifdef _DEBUG
	printf("enter XACTIO_Write32\n");
#endif 

	if((hXact_Handle = XACTIO_GetAddrHandle(chipname, intaddr, &intval)) == INVALID_XACT_HANDLE){
#ifdef _DEBUG
	printf("invalid handle\n");
#endif 
		return(XACT_ERROR);
	}

	if((hXact_Handle == USTOREADDR) || (hXact_Handle == ACTIVE_STS))
		return XACT_SUCCESS;

#ifdef _DEBUG
	printf("Call XACT_set_state_value: Handle=%x\n", hXact_Handle);
#endif 
    
    if (!XACT_set_state_value(hXact_Handle,&intval))
		return(XACT_ERROR);


	return(XACT_SUCCESS);
}

int XACTIO_Read32(char *chipname, unsigned intaddr, unsigned int *val)
{


    XACT_HANDLE hXact_Handle;
#ifdef _DEBUG
	printf("enter XACTIO_Read32\n");
#endif 
	if((hXact_Handle = XACTIO_GetAddrHandle(chipname, intaddr, val)) == INVALID_XACT_HANDLE)
		return(XACT_ERROR);
	
	if((hXact_Handle == USTOREADDR) || (hXact_Handle == ACTIVE_STS))
		return XACT_SUCCESS;
#ifdef _DEBUG
	printf("Call XACT_get_state_value: Handle=%x\n", hXact_Handle);
#endif 

    if (!XACT_get_state_value(hXact_Handle,val))
		return(XACT_ERROR);

	return(XACT_SUCCESS);
}


static ISR_FUNC IREG_CkFunction = 0;
static int usrData;
static int XactCallBackFunction(int hXact, void *usrint)
{

	IREG_CkFunction(usrData);
    return(0);
}

int XACTIO_IRQ_Connect(char *chipname, SA1200_INTS vector, ISR_FUNC isr, int param)
{

	char StateName[80];
	memset(StateName,'\0', sizeof(StateName));
	sprintf(StateName,"%s.", chipname);
	if(vector != SAINT_UENG)
		return (XACT_ERROR);

    IREG_CkFunction = isr;
    usrData=param;
	strcat(StateName, "fbi.ireg");

	XACT_HANDLE handle = XACT_get_handle(StateName, NULL);
    if (handle == INVALID_XACT_HANDLE)
        return(XACT_ERROR);

    int retval = XACT_Define_Cancel_Callback_State_Transition(
        handle,
        XactCallBackFunction);
    if (retval == FALSE)
        return(XACT_ERROR);

 	retval = XACT_Define_Callback_State_Transition(
        handle,0,
	    XactCallBackFunction,"");
    if (retval == FALSE)
        return(XACT_ERROR);

    return(XACT_SUCCESS);
}


int XACTIO_IRQ_Enable(char *chipname, SA1200_INTS vector)
{
	return(0);
}

int XACTIO_IRQ_Disable(char *chipname, SA1200_INTS vector)
{
	return(0);
}

int XACTIO_intLock(char *chipname)
{
	return(0);
}

int XACTIO_intUnlock(char *chipname, int prev_val)
{
	return(0);
}


// end of file
