/* ambaIO.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: amba_IO
 * author: Henry Qian 09/08/98
 * revisions:
 * $History: ambaio.cpp $
 * 
 * *****************  Version 46  *****************
 * User: Ewalker      Date: 12/01/00   Time: 4:36p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Created XactIoGetChipName and XactIoSetChipName functions to get/set
 * the chip name to act against when calling XactIo or AMBAIO functions
 * though the core_io.h macros. The default remains the NULL ("") chip for
 * backwards compatibility. Core programs that need to do transactor I/O
 * against a non-NULL chip should call XactIoSetChipName prior to using
 * any of the core_io.h macros and should use XactIoGetChipName in place
 * of hardcoded "" when calling in the AMBAIO and XactIo (multichip)
 * functions directly.
 * 
 * *****************  Version 45  *****************
 * User: Ewalker      Date: 12/01/00   Time: 3:36p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * 
 * *****************  Version 44  *****************
 * User: Ewalker      Date: 12/01/00   Time: 12:59p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Removed unused extern declarations.
 * 
 * *****************  Version 43  *****************
 * User: Ewalker      Date: 12/01/00   Time: 10:04a
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Changed AmbaIoCmd chipName class member to hold the c string rather
 * than just a copy of the c string pointer specified in the constructor.
 * 
 * *****************  Version 39  *****************
 * User: Drjohns1     Date: 12/17/99   Time: 12:08p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Fixed potential problem
 * 
 * *****************  Version 38  *****************
 * User: Drjohns1     Date: 12/08/99   Time: 3:43p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * 
 * *****************  Version 37  *****************
 * User: Drjohns1     Date: 10/29/99   Time: 10:03a
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Adjusted timeout/delay parmeter to system clock resolution
 * 
 * *****************  Version 36  *****************
 * User: Rdmurato     Date: 10/19/99   Time: 12:28p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Add AMBAIO_ResetAllChips to fix sim_reset problem
 * 
 * *****************  Version 35  *****************
 * User: Drjohns1     Date: 9/22/99    Time: 4:34p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * 
 * *****************  Version 34  *****************
 * User: Qabuild      Date: 9/09/99    Time: 2:52p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * .h filename changed....
 * 
 * *****************  Version 32  *****************
 * User: Qabuild      Date: 7/16/99    Time: 4:01p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Raised ambioCmdThread priority to above normal;
 * Added delayMs(0) in simTilDone() waiting loop.
 * 
 * *****************  Version 31  *****************
 * User: Rparen1x     Date: 6/30/99    Time: 3:25p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * 
 * *****************  Version 30  *****************
 * User: Rparen1x     Date: 6/30/99    Time: 11:29a
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * 
 * *****************  Version 29  *****************
 * User: Drjohns1     Date: 6/02/99    Time: 1:28p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Modification per code-review
 * 
 * *****************  Version 28  *****************
 * User: Hqianx       Date: 4/27/99    Time: 3:39p
 * Updated in $/dev_1200/SA1_CoreLibs/Xact_io
 * Removed UENGINE_DRIVER.
 * 
 * *****************  Version 26  *****************
 * User: Tclancyx     Date: 4/08/99    Time: 3:15p
 * Updated in $/development/tool/Xact_io
 * Added OctalMACCreateDriver() here temporarily.
 * 
 * *****************  Version 25  *****************
 * User: Qabuild      Date: 4/06/99    Time: 11:32a
 * Updated in $/development/tool/Xact_io
 * Introduced flag OctalMACStarted.
 * 
 * *****************  Version 24  *****************
 * User: Tclancyx     Date: 4/05/99    Time: 2:52p
 * Updated in $/development/tool/Xact_io
 * Add OctalMACCreateDriver().
 * 
 * *****************  Version 23  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 4:18p
 * Updated in $/development/tool/Xact_io
 * update COPYRIGHT
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 3/05/99    Time: 8:54p
 * Updated in $/development/tool/Xact_io
 * Updated for generic interrupt functions
 * 
 * *****************  Version 21  *****************
 * User: Drjohns1     Date: 3/04/99    Time: 11:40a
 * Updated in $/development/tool/Xact_io
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 3/01/99    Time: 10:54a
 * Updated in $/development/tool/Xact_io
 * 
 * *****************  Version 19  *****************
 * User: Drjohns1     Date: 3/01/99    Time: 9:56a
 * Updated in $/development/tool/Xact_io
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 2/26/99    Time: 9:35a
 * Updated in $/development/tool/Xact_io
 * Check-in for Desmond
 * 
 * *****************  Version 17  *****************
 * User: Drjohns1     Date: 2/24/99    Time: 6:46p
 * Updated in $/development/tool/Xact_io
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 2/22/99    Time: 2:23p
 * Updated in $/development/tool/Xact_io
 * Added enabling/disabling of fbox and breakpoint interupts
 * 
 * *****************  Version 15  *****************
 * User: Drjohns1     Date: 2/12/99    Time: 5:52p
 * Updated in $/development/tool/Xact_io
 * Updated for multi-chip/auto_ambio related changes
 * 
 * *****************  Version 14  *****************
 * User: Drjohns1     Date: 2/10/99    Time: 2:11p
 * Updated in $/development/tool/Xact_io
 * Fixed MULTI_CHIP compile errors
 * 
 * *****************  Version 13  *****************
 * User: Drjohns1     Date: 2/09/99    Time: 7:17p
 * Updated in $/development/tool/Xact_io
 * Changed sim count for amba cycles
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 2/09/99    Time: 2:30p
 * Updated in $/development/tool/Xact_io
 * Updated to make ambaio read/write block until completion
 * 
 * *****************  Version 11  *****************
 * User: Drjohns1     Date: 2/05/99    Time: 7:03p
 * Updated in $/development/tool/Xact_io
 * Added support for breakpoint interupt
 * 
 * *****************  Version 10  *****************
 * User: Hqianx       Date: 12/16/98   Time: 11:31a
 * Updated in $/development/tool/Xact_io
 * Fixed a compile error.
 * 
 * *****************  Version 9  *****************
 * User: Hqianx       Date: 12/15/98   Time: 5:26p
 * Updated in $/development/tool/Xact_io
 * Simplified ambaIoTest().
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 11/24/98   Time: 5:49p
 * Updated in $/development/tool/Xact_io
 * Provide support for the multiple chip feature supported in the
 * Transactor.
 * 
 * *****************  Version 7  *****************
 * User: Hqianx       Date: 11/13/98   Time: 7:12p
 * Updated in $/development/tool/Xact_io
 * Added AMBAIO_Initialize(), ambaIoTestThread().
 * 
 * *****************  Version 6  *****************
 * User: Hqianx       Date: 10/02/98   Time: 5:27p
 * Updated in $/base_level3/tool/Xact_io
 * Added discriptions for functions.
 * 
 * *****************  Version 5  *****************
 * User: Hqianx       Date: 9/30/98    Time: 10:56a
 * Updated in $/base_level3/tool/Xact_io
 * Added function unsigned int ambaIoInitDone(void).
 * 
 * *****************  Version 3  *****************
 * User: Hqianx       Date: 9/29/98    Time: 3:42p
 * Updated in $/base_level3/tool/Xact_io
 * Moved extern "C" to ambaio.h.
 * 
 * *****************  Version 2  *****************
 * User: Hqianx       Date: 9/22/98    Time: 2:46p
 * Updated in $/base_level3/tool/Xact_io
 * Modified the line #include "ts.h" to #include "TS.h".
 * line #include "ambaIO.h" to #include "ambaio.h".
 * 
 * *****************  Version 1  *****************
 * User: Hqianx       Date: 9/22/98    Time: 2:44p
 * Created in $/base_level3/tool/Xact_io
 * Changed the file name all low case.
 * 
 * *****************  Version 1  *****************
 * User: Hqianx       Date: 9/17/98    Time: 5:14p
 * Created in $/base_level3/tool/Xact_io
 * AMBA I/O API.
 */
 
#include "xact_gui.h"
#include "ambaio.h"
#include "delay.h"

Semaphore	AMBAIO_AccessSemphore;
Semaphore	ChipSem;
Semaphore	SimSem;

XactChip	*XactChips[MAXCHIP];

extern "C" int simTicks(unsigned int ticks);

void ambioCmdThread(void *inArg);
int x_AMBAIO_Write32(char *ChipName, unsigned int address, unsigned int data);
int x_AMBAIO_Read32(char *ChipName, unsigned int address, unsigned int* data);

void interuptThread(void *ctx);
unsigned int AMBAIO_InitDone = 0;

int simTicks(unsigned int ticks)
{
   while(ticks--){
      SimSem.Take();
      XACT_ExecuteCommandStr("go 1");
      SimSem.Give();
      delayMs(0);
   }
   return(ticks);
}


AmbaIoCmd::AmbaIoCmd(char *ChipName)
{
   sem = Sem_Init(0);
   busy = Sem_Init(0);
   cmd = IDLE;
   isBusy = 0;
   strcpy(chipName, ChipName); 
   threadId = Thread_Create(THREAD_PRIORITY_ABOVE_NORMAL, ambioCmdThread, (threadArg_T *)this);

   // wait for interupt thread to startup
	while(!ambaIoCmdRdy) delayMs(10);	// wait for thread to start

   return;
}

AmbaIoCmd::~AmbaIoCmd(){
   stop();	// stop the command thread
   Thread_Cancel(threadId);
   Sem_Destroy(busy);
   Sem_Destroy(sem);
   return;
}

inline void AmbaIoCmd::autoRead32(unsigned int addr){
	waitBusy();
	address = addr;
	data = 0xbababa;
	cmd = AMBIO_READ;
	postCmd();
}

inline void AmbaIoCmd::autoWrite32(unsigned int addr, unsigned int val){
	waitBusy();
	address = addr;
	data = val;
	cmd = AMBIO_WRITE;
	postCmd();
}


inline int AmbaIoCmd::simTilDone(unsigned int *val){
	// wait of the cmd thread to set the busy
	while(!isBusy) delayMs(DELAYMS_RES);
	while(isBusy){
		simTicks(1);
	}
    *val = data;
	cmd = IDLE;
	int stat = status;
	postBusy();
	return(stat);
}

inline void AmbaIoCmd::stop(void){
	waitBusy();
	cmd = STOP;
	postCmd();
}


inline int AmbaIoCmd::read32(unsigned int addr, unsigned int *val)
{
	return(x_AMBAIO_Read32((char *)chipName, addr, val));
}


inline int AmbaIoCmd::write32(unsigned int addr, unsigned int val)
{
	return(x_AMBAIO_Write32((char *)chipName, addr, val));
}

static void initDriver(void *arg)
{
	XactChip *chip = (XactChip *)arg;

    AMBAIO_Write32(chip->m_ChipName, 0xb00401e0, 0xc0000000);	// enable fiq & irq

	Thread_Create(MID_PRIORITY, interuptThread, (threadArg_T *)chip);
    XACT_Define_Callback_amba_new_interrupt((char *)chip->m_ChipName, interruptDispatcher, chip);

	AMBAIO_InitDone = TRUE;
	return;
}


XactChip::XactChip(char *ChipName, ISR_FUNC FboxHandler) : autoAmbio(ChipName)
{
	strcpy(m_ChipName, ChipName); 

	m_initThreadDone = FALSE;
	m_intFboxHandler = FboxHandler; 
	if(m_intFboxHandler) m_fboxInt = TRUE;
	else m_fboxInt = FALSE;
	m_intAutoRead = FALSE;
	Thread_Create(MID_PRIORITY, initDriver, (threadArg_T *)this);

	m_Valid = 1;

	return;
}


int XactChip::FIQ_Connect(SA1200_INTS intSrc, ISR_FUNC handler, int param){
	switch(intSrc){
		case SAINT_UENG: m_intFboxHandler; m_intFboxHandler = handler; m_param = param; break;
		default: return(-1);
	}
	return(0);
}

int XactChip::IRQ_Connect(SA1200_INTS intSrc, ISR_FUNC handler, int param){
	switch(intSrc){
		case SAINT_UENG: 	m_intFboxHandler = handler; m_param = param; break;
		default: return(-1);
	}
	return(0);
}

int XactChip::FIQ_Enable(SA1200_INTS intSrc){
	switch(intSrc){
		case SAINT_UENG: 	m_fboxInt = TRUE; break;
		default: return(-1);
	}
	return(0);
}

int XactChip::IRQ_Enable(SA1200_INTS intSrc){
	switch(intSrc){
		case SAINT_UENG: 	m_fboxInt = TRUE; break;
		default: return(-1);
	}
	return(0);
}


int XactChip::FIQ_Disable(SA1200_INTS intSrc){
	switch(intSrc){
		case SAINT_UENG: 	m_fboxInt = FALSE; break;
		default: return(-1);
	}
	return(0);
}

int XactChip::IRQ_Disable(SA1200_INTS intSrc){
	switch(intSrc){
		case SAINT_UENG: 	m_fboxInt = FALSE; break;
		default: return(-1);
	}
	return(0);
}

static XactChip *findChip(char *ChipName)
{
	ChipSem.Take();
	for(int i=0; i < MAXCHIP; i++)
		if((XactChips[i] != NULL) && (XactChips[i]->ChipNameMatch(ChipName) == TRUE)){
			ChipSem.Give();
			return(XactChips[i]);
		}
	ChipSem.Give();
	return(NULL);
}

static int chipInit(char *ChipName, ISR_FUNC handler)
{
	if(findChip(ChipName)) return(-1);		/// chip already exist

	ChipSem.Take();
	// find an empty slot
	for(int i=0; i < MAXCHIP; i++)
		if((XactChips[i] == NULL)){
			XactChips[i]= new XactChip(ChipName, handler);
			ChipSem.Give();
			return(i);
		}
	ChipSem.Give();
	return(-1);
}


/*
**  void AMBAIO_InitChip
**
**  DESCRIPTION
**      This function let use install their fbox interrupt
**      service routine.  The interrupt source status will
**      be at IREG.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_InitChip(char *ChipName, ISR_FUNC handler)
{
	int i;
	static int isrq_init = FALSE;

	// determine if chip alread init
	for(i=0; i < MAXCHIP; i++)
		if(XactChips[i]) return(0);

	if(isrq_init == FALSE){
        for(i=0; i < MAXCHIP; i++) XactChips[i] = NULL;
		ChipSem.Give();
		SimSem.Give();
        AMBAIO_AccessSemphore.Give(); // initialization
        isrq_init = TRUE;
    }

	if((i = chipInit(ChipName, handler)) == -1) return(0);
	if(XactChips[i]->IsValid() == TRUE) return 1;

	return 0;
}

int AMBAIO_ResetAllChips()
{
	for(int i=0; i < MAXCHIP; i++)
        if(XactChips[i] != NULL) {
		   delete(XactChips[i]);
		   XactChips[i] = NULL;
		}

	return 0;
}

int AMBAIO_ResetChip(char *ChipName)
{
    char cname[126];
	strcpy(cname, ChipName);	// to avoid VC++ 6.0 global const str prob.
	for(int i=0; i < MAXCHIP; i++)
		if((XactChips[i] != NULL) && (XactChips[i]->ChipNameMatch(cname) == TRUE)){
		   delete(XactChips[i]);
		   XactChips[i] = NULL;
		   break;
		}

	return 0;
}


void AMBAIO_Reset(void)
{
	for(int i=0; i < MAXCHIP; i++)
		if(XactChips[i]){
		   delete(XactChips[i]);
		   XactChips[i] = NULL;
		}

	return;
}



int AMBAIO_FIQ_Connect(char *ChipName, SA1200_INTS InteruptKind, ISR_FUNC Handler, int param)
{
	XactChip *chip;
	if((chip = findChip(ChipName))) return(chip->FIQ_Connect(InteruptKind, Handler, param));
	return(-1);
}

int AMBAIO_IRQ_Connect(char *ChipName, SA1200_INTS InteruptKind, ISR_FUNC Handler, int param)
{
	XactChip *chip;
	if((chip = findChip(ChipName))) return(chip->IRQ_Connect(InteruptKind, Handler, param));
	return(-1);
}

int AMBAIO_FIQ_Enable(char *ChipName, SA1200_INTS InteruptKind)
{
	XactChip *chip;
	if((chip = findChip(ChipName))) return(chip->FIQ_Enable(InteruptKind));
	return(-1);
}

int AMBAIO_IRQ_Enable(char *ChipName, SA1200_INTS InteruptKind)
{
	XactChip *chip;
	if((chip = findChip(ChipName)))	return(chip->IRQ_Enable(InteruptKind));
	return(-1);
}

int AMBAIO_FIQ_Disable(char *ChipName, SA1200_INTS InteruptKind)
{
	XactChip *chip;
	if((chip = findChip(ChipName))) return(chip->FIQ_Disable(InteruptKind));
	return(-1);
}

int AMBAIO_IRQ_Disable(char *ChipName, SA1200_INTS InteruptKind)
{
	XactChip *chip;
	if((chip = findChip(ChipName)))	return(chip->IRQ_Disable(InteruptKind));
	return(-1);
}


/*
**  int AMBAIO_intLock
**
**  DESCRIPTION
**		  Locks out interrupts, returns previous status.
**
**  RETURNS
*/
int AMBAIO_intLock(char *ChipName)
{
	XactChip *chip;
   // interrupt lock not implemented
	if((chip = findChip(ChipName))) return(chip->intLock());
   return(0);
}


/*
**  int AMBAIO_intUnlock
**
**  DESCRIPTION
**        Sets interrupt lock status to previous value.
**
**  RETURNS
*/
void AMBAIO_intUnlock(char *ChipName, int prev_val)
{
	XactChip *chip;
	if((chip = findChip(ChipName))) chip->intUnlock(prev_val);
   return;
}


/*
**  int AMBAIO_AutoRead32
**
**  DESCRIPTION
**      Read a 32-bit 'data' from memory location 'address' and wait for completion.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_AutoRead32(char *ChipName, unsigned int memAddr, unsigned int *value)
{
	XactChip *xactChip;

	if((xactChip = findChip(ChipName))){
		xactChip->autoAmbio.autoRead32(memAddr);
		
		return(xactChip->autoAmbio.simTilDone(value));
	}
	return(0);
}

/*
**  int AMBAIO_AutoWrite32
**
**  DESCRIPTION
**      Write a 32-bit 'data' to location 'address' and wait for completion.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_AutoWrite32(char *ChipName, unsigned int memAddr, unsigned int value)
{
	XactChip *xactChip;
	if((xactChip = findChip(ChipName))){
		xactChip->autoAmbio.autoWrite32(memAddr, value);
		
		return(xactChip->autoAmbio.simTilDone(&value));
	}
	return(0);
}


/*
**  int AMBAIO_Read32
**
**  DESCRIPTION
**      Read a 32-bit 'data' from memory location 'address' -- requires sim cycles to complete
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_Read32(char *ChipName, unsigned int memAddr, unsigned int *value)
{
	XactChip *xactChip;
	if((xactChip = findChip(ChipName)))
		return(xactChip->autoAmbio.read32(memAddr, value));
	return(0);
}

/*
**  int AMBAIO_Write32
**
**  DESCRIPTION
**      Write a 32-bit 'data' to location 'address' -- requires sim cycles to complete.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_Write32(char *ChipName, unsigned int memAddr, unsigned int value)
{
	XactChip *xactChip;
	if((xactChip = findChip(ChipName)))
		return(xactChip->autoAmbio.write32(memAddr, value));
	return(0);
}


/*
**  int x_AMBAIO_Write32( unsigned int address, unsigned int data)
**
**  DESCRIPTION
**      Write a 32-bit 'data' to location 'address' in memory map.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int x_AMBAIO_Write32(char *ChipName, unsigned int address, unsigned int data)
{
    int retval = 0;
    AMBAIO_AccessSemphore.Take();

    retval = XACT_amba_add_req( ChipName,
                                AMBAIO_WRITE,
                                address, // byte addr
                                4,  // size (bytes)
                                1,  // burst size
                                0, // clocks
                                &data,
                                AMBAIO_CALL_BLOCK);

    AMBAIO_AccessSemphore.Give();
    return(retval);
}


/*
**  int AMBAIO_Read32( unsigned int address,
**                      unsigned int* data)
**
**  DESCRIPTION
**      Read a 32-bit 'data' from location 'address' in memory map.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int x_AMBAIO_Read32(char *ChipName, unsigned int address, unsigned int* data)
{
    int retval = 0;
    AMBAIO_AccessSemphore.Take();

    retval = XACT_amba_add_req( ChipName,
                                AMBAIO_READ,
                                address, // byte addr
                                4,  // size (bytes)
                                1,  // burst size
                                0, // clocks
                                data,
                                AMBAIO_CALL_BLOCK);
    if (retval == 1)
        retval = XACT_amba_get_read_result(ChipName, data);

    AMBAIO_AccessSemphore.Give();
    return(retval);
}


/*
**  int AMBAIO_ReadIREG(unsigned int* data)
**
**  DESCRIPTION
**      Read interrupt reasoning register.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_ReadIREG(char *ChipName, unsigned int* data)
{
    int retval = 0;
    retval = AMBAIO_Read32( ChipName, AMBAIO_FBI_BASE + AMBAIO_IREG, data);
    return(retval);
}

/*
**  int AMBAIO_WriteIREG(unsigned int data)
**
**  DESCRIPTION
**      Write interrupt reasoning register.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_WriteIREG(char *ChipName, unsigned int data)
{
    int retval = 0;
    retval = AMBAIO_Write32(ChipName, AMBAIO_FBI_BASE + AMBAIO_IREG,
                            data);
    return(retval);
}


/*
**  void interuptThread
**
**  DESCRIPTION
**      Thread to service the breakpoint and uEngine interupts.
**
**  RETURNS
**      None.
*/
void interuptThread(void *ctx)
{
	XactChip *pctx = (XactChip *)ctx;

	while(1){
		pctx->Interupt.Take();

		ISR_FUNC isr;
		if(pctx->isFboxIntEn() && ((isr = pctx->GetIntFboxHandler())))
			(*isr)(pctx->GetIntParam());
 	}
	return;
}



/*
**  void interruptDispatcher(   void* ctx,
**                              unsigned int fiqStatus,
**                              unsigned int irqStatus)
**
**  DESCRIPTION
**      Generic interrupt dispatcher for fbox and breakpoint.
**
**  RETURNS
**      None.
*/
void interruptDispatcher(void* ctx, unsigned int fiqStatus, unsigned int irqStatus)
{
    // fbox interrupts
	XactChip *pctx = (XactChip *)ctx;
    if ((fiqStatus & 0x1) || (irqStatus & 0x1))
		pctx->Interupt.Give();
}

/*
**  void AMBAIO_EnableIntAutoRead
**
**  DESCRIPTION
**      Enable autoAmbIoRead in the ISR thread.
**
**  RETURNS
**      None.
*/
void AMBAIO_EnableIntAutoRead(char *pChipName)
{
	XactChip *xactChip;
	if((xactChip = findChip(pChipName))) xactChip->enIntAutoRead();
}



/*
**  unsigned int ambaIoInitDone(void)
**
**  DESCRIPTION
**      It is called at transactor command line, to make sure
**      ambaio API init is done.
**
**  RETURNS
**      0: error
**      1: sucess
*/
unsigned int ambaIoInitDone(void)
{
	delayMs(10);
    return(AMBAIO_InitDone);
}



void ambaIoInit(void)
{
    /* initialize the default chip or current chip */
    /* as set using XactIoSetChipName() */

    if(!AMBAIO_InitChip(XactIoGetChipName(), NULL)) return;
    AMBAIO_FIQ_Enable(XactIoGetChipName(), SAINT_UENG);
}

void autoAmbaIoInit(void)
{
    /* initialize the default chip or current chip */
    /* as set using XactIoSetChipName() */

    if(!AMBAIO_InitChip(XactIoGetChipName(), NULL)) return;
	AMBAIO_EnableIntAutoRead(XactIoGetChipName());

	while(AMBAIO_InitDone == FALSE) simTicks(1);
	AMBAIO_FIQ_Enable(XactIoGetChipName(), SAINT_UENG);
}


void multiChip_ambaIoInit(char *ChipName)
{
	AMBAIO_InitChip(ChipName, NULL);
}



/*
**  int AMBAIO_Initialize(void)
**
**  DESCRIPTION
**      Initialize ambaio API only.
**
**  RETURNS
**      0: error
**      1: sucess
*/
int AMBAIO_Initialize(char *ChipName)
{
	AMBAIO_InitDone = 0;
	AMBAIO_InitChip(ChipName, NULL);
    AMBAIO_InitDone = 1;
    return(0);
}

/*
**  void ambaIoInitialize(char *ChipName)
**
**  DESCRIPTION
**      This function is called from sa1200CoreStart.cpp.
**      In order to init ambaio API, it must be done in another thread
**      other than the transcator thread.
**
**  RETURNS
**      0: error
**      1: sucess
*/
void ambaIoInitialize(char *ChipName)
{
    Thread* TestCaseTh = new Thread((PFV)AMBAIO_Initialize, ChipName);
}


void ambaIoTestThread(void)
{
    unsigned int data;
    int retval;

    data = 0x12345678;

    if((retval = AMBAIO_Write32("Alpha", AMBAIO_SRAM_BASE + 0x10, data)) != 1)
        printf(" -- AMBA I/O write error.\n");
    else
        printf(" -- AMBA I/O write 0x%x at 0x%x.\n",
                data, AMBAIO_SRAM_BASE + 0x10);
        
    if((retval = AMBAIO_Read32("Alpha", AMBAIO_SRAM_BASE + 0x10, &data)) != 1)
        printf(" -- AMBA I/O read error.\n");
    else
        printf(" -- AMBA I/O read 0x%x at 0x%x.\n",
                data, AMBAIO_SRAM_BASE + 0x10);
}


// extern "C" void ambaIoTest();
void ambaIoTest(void)
{
	Thread* thread = new Thread((PFV)ambaIoTestThread);
}


/*
**  void ambioCmdThread
**
**  DESCRIPTION
**      This function is called from ambaio_init as a thread that is independent
**		of the xactor thread.  It will issue amba read and write operations and
**		allows the xactor thread to issue "sim/goto" comand until the operation
**		is completed.
**
**  RETURNS
**      0: error
**      1: sucess
*/
void ambioCmdThread(void *inArg)
{
	int times = 0, c, i = 0;
   AmbaIoCmd *obj = (AmbaIoCmd *)inArg;

   obj->ambaIoCmdRdy = 1;
   obj->postBusy();

   while(1){
	   obj->waitCmd();
	   c = obj->cmd;
	   obj->isBusy = 1;

	   // exec cmd
	   switch(c){
		case AMBIO_READ:
			obj->status = x_AMBAIO_Read32((char *)obj->chipName, obj->address, &obj->data);
			break;

		case AMBIO_WRITE:
			obj->status = x_AMBAIO_Write32((char *)obj->chipName, obj->address, obj->data);
			break;
		case IDLE: delayMs(10); break;
		case STOP: return;
	   }
	   obj->isBusy = 0;
   }
   return;
}


// end of file
