/* config_1200.cpp
 * SA1200 Core config
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: core config
 *  author: dfh, May 13, 99  Initial version
 *  revisions:
 *  $History: config_1200.cpp $
 * 
 * *****************  Version 53  *****************
 * User: Ewalker      Date: 12/08/00   Time: 11:07a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Corrected function name in printf for file open error.
 * 
 * *****************  Version 52  *****************
 * User: Mffallon     Date: 11/29/00   Time: 9:54p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Leave MAC port's 7 and 8's rx ready threashold at 64.  Brining it down
 * causes bad rx_requests when processing variable sized packets.
 * Mike
 * 
 * *****************  Version 50  *****************
 * User: Mffallon     Date: 5/30/00    Time: 4:27p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * typo fix.
 * 
 * *****************  Version 49  *****************
 * User: Qabuild      Date: 5/30/00    Time: 3:58p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * RDY bus program executes flow control instructions in place of NOPs
 * because the flow control instructions execute quicker.
 * 
 * Also, set Gig MAC rx_rdy threshold to 8 mpackets to make compatable
 * with reference design which assumes this to be the case.
 * 
 * MFF
 * 
 * *****************  Version 48  *****************
 * User: Dhooper      Date: 4/11/00    Time: 4:25p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * if  BindSymbol image name does not exist, allow ucode wirte to complete
 * 
 * *****************  Version 47  *****************
 * User: Qabuild      Date: 4/05/00    Time: 5:14p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Fixed "break" in switch statements when certain compile directives are
 * not defined.
 * 
 * *****************  Version 46  *****************
 * User: Mffallon     Date: 1/31/00    Time: 3:56p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Set rx_ready threshold for ports 6 and 7 to 32 bytes (from 64).  This
 * will result in the ready flag going true sooner, and thus avoid input
 * fifo overflows on the MAC.  This change was tested for underflows with
 * only 1 and only 2 ports running (ports 6 and/or 7), and there were no
 * underflows.  This case is the minimum responce time for the reference
 * design to the ready flag, and thus the most likely time to get a fifo
 * underflow.  Everything checked out O.K.
 * 
 * *****************  Version 44  *****************
 * User: Mffallon     Date: 1/07/00    Time: 6:53p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Set default fifo thresholds.
 * 
 * *****************  Version 43  *****************
 * User: Drjohns1     Date: 1/07/00    Time: 5:47p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Added uof init callback installation
 * 
 * *****************  Version 42  *****************
 * User: Drjohns1     Date: 1/04/00    Time: 2:40p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Revert to making sysConfig local to the module
 * 
 * *****************  Version 40  *****************
 * User: Drjohns1     Date: 12/17/99   Time: 12:04p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Incorporated rtm and brm into netApp
 * 
 * *****************  Version 36  *****************
 * User: Lhui         Date: 10/19/99   Time: 12:36p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Fix build errors
 * 
 * *****************  Version 35  *****************
 * User: Dhooper      Date: 10/18/99   Time: 5:25p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Default to big endian.
 * 
 * *****************  Version 28  *****************
 * User: Lhui         Date: 9/25/99    Time: 12:12p
 * Updated in $/Beta_2/SA1_CoreLibs/app_1200
 * Update from Dev_1200
 * 
 * *****************  Version 30  *****************
 * User: Drjohns1     Date: 9/24/99    Time: 9:48p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Added inControl to SysConf to keep track of whether we are in control
 * 
 * *****************  Version 29  *****************
 * User: Mffallon     Date: 9/23/99    Time: 10:22a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * zero thread_done
 * 
 * *****************  Version 28  *****************
 * User: Dhooper      Date: 9/22/99    Time: 10:42a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * write GPIO_EN to configure flow control at FBI configure time
 * 
 * *****************  Version 27  *****************
 * User: Tclancyx     Date: 9/20/99    Time: 6:30p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * initialize sram[0] to 0
 * 
 * *****************  Version 26  *****************
 * User: Tclancyx     Date: 9/20/99    Time: 2:41p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * write 0 to receive rdy control as part of fbi configure
 * 
 * *****************  Version 25  *****************
 * User: Dhooper      Date: 9/16/99    Time: 7:24p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * reset ports then fbi
 * 
 * *****************  Version 24  *****************
 * User: Dhooper      Date: 8/17/99    Time: 5:50p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * configure gigabit default little endian
 * 
 * *****************  Version 23  *****************
 * User: Qabuild      Date: 8/11/99    Time: 4:18p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Use enum type cast to quiet compiler error.
 * 
 * *****************  Version 22  *****************
 * User: Dhooper      Date: 8/09/99    Time: 5:36p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * thresholds for gigabit port
 * 
 * *****************  Version 21  *****************
 * User: Drjohns1     Date: 8/09/99    Time: 12:45p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Added timeout param to Sem_Wait
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 8/05/99    Time: 7:51p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * Add a compile option for printf to avoid the UCOS serial port problem 
 * 
 * *****************  Version 19  *****************
 * User: Dhooper      Date: 8/03/99    Time: 9:40a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * gigamac support
 * 
 * *****************  Version 18  *****************
 * User: Dhooper      Date: 7/30/99    Time: 3:30p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * default fbi to little endian
 * 
 * *****************  Version 17  *****************
 * User: Dhooper      Date: 7/30/99    Time: 10:29a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * double the rate of tx autopush
 * 
 * *****************  Version 15  *****************
 * User: Dhooper      Date: 7/20/99    Time: 10:09a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * on reset, clear pcs
 * Stop, stop ports then microengines
 * 
 * *****************  Version 14  *****************
 * User: Dhooper      Date: 7/19/99    Time: 9:57a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * clean up warnings about unused local variables
 * 
 * *****************  Version 13  *****************
 * User: Dhooper      Date: 7/16/99    Time: 1:05p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * do fbi reset in reset
 * 
 * *****************  Version 12  *****************
 * User: Skornfel     Date: 7/15/99    Time: 2:44p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * move PC=0 to start function
 * 
 * *****************  Version 11  *****************
 * User: Skornfel     Date: 6/16/99    Time: 5:38p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * use latest uEng_reset
 * 
 * *****************  Version 10  *****************
 * User: Skornfel     Date: 6/14/99    Time: 1:54p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * BibdSymbols loop in UCLOadBuffer
 * 
 * *****************  Version 9  *****************
 * User: Skornfel     Date: 6/11/99    Time: 9:21a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * change rdybus template to enable seq prog
 * 
 * *****************  Version 8  *****************
 * User: Jdguilfo     Date: 6/09/99    Time: 7:00p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * #ifdef UCOMM_DRIVER, Octal MAC enums, UCLoadBuffer
 * 
 * *****************  Version 7  *****************
 * User: Dhooper      Date: 6/08/99    Time: 10:41a
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * separate NetApp_UCLoadFile
 * 
 * *****************  Version 6  *****************
 * User: Dhooper      Date: 6/03/99    Time: 9:24p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * extern sysConf
 * 
 * *****************  Version 5  *****************
 * User: Dhooper      Date: 6/03/99    Time: 1:27p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * add portMask for Start and  Stop, add thread registry
 * 
 * *****************  Version 4  *****************
 * User: Dhooper      Date: 6/02/99    Time: 2:50p
 * Updated in $/dev_1200/SA1_CoreLibs/app_1200
 * move netapp functions to sysconf
 * 
 * *****************  Version 1  *****************
 * User: Dhooper      Date: 5/19/99    Time: 9:59a
 * Created in $/BL5_Plus/SA1_CoreLibs/app_1200
 * app_1200 pre-review
 * 
 * 
 * ---------------------------------------------------------------------
 */

#include "config_1200.h"
#include "sem.h"
#include "ueng.h"


static SysConf *sysConf = NULL;
sem_T SystemControlSem;



/*-----------------------------------------------------------------
	SysConf::SysConf
	Construct SA1200 system configuration
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
SysConf::SysConf()
{

	sram = SRAM_Attach();		// get a pointer to sram CSR mapping
	sdram = SDRAM_Attach();		// get a pointer to sdram CSR mapping
	fbi = FBI_Attach();			// get a pointer to fbi CSR mapping

	SystemControlSem = Sem_Init(1);
#if (FBUS_DEVICE_COUNT > 0)
	fdev[0].type = (DeviceType)FBUS_DEV0_TYPE;
#endif
#if (FBUS_DEVICE_COUNT > 1)
	fdev[1].type = (DeviceType)FBUS_DEV1_TYPE;
#endif
#if (FBUS_DEVICE_COUNT > 2)
	fdev[2].type = (DeviceType)FBUS_DEV2_TYPE;
#endif
#if (FBUS_DEVICE_COUNT > 3)
	fdev[3].type = (DeviceType)FBUS_DEV3_TYPE;
#endif
#if (FBUS_DEVICE_COUNT > 4)
#error More than 4 MACS not supported in sysConf
#endif


	byte_order = LITTLE_E;
	rx_autopush = DEFAULT_RX_AUTOPUSH;
	tx_autopush = DEFAULT_TX_AUTOPUSH;
	rdy_synch_count = DEFAULT_RDY_SYNCH_COUNT;
	nextSymbol = 0;
	inControl = 0;
	nextThread = START_THREAD;
	strcpy(uof_file,"");
	pUofCb = NULL;
	pUofCbArg = NULL;
}


/*-----------------------------------------------------------------
	SysConf::Init
	Initialize core libraries and drivers.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::Init()
{
	int i;

	// initialize microcode loader (and indirectly the microengine driver instances)
	UcLo_InitLib();

#ifdef UCOMM_DRIVER
	// initialize microengines communication driver
	FDriver_Init();
#endif

	// initialize FBus devices
	for (i=0;i<FBUS_DEVICE_COUNT; i++){
		switch(fdev[i].type){
		case OCTAL_MAC_21440:
#ifdef MAC_21440
			fdev[i].ports_present = 8;
			fdev[i].rx_rdy_threshold = THRESHOLD_64;
			fdev[i].tx_rdy_threshold = THRESHOLD_64;  
			OctalMACCreateDriver(i);
#endif
			break;
		case GIGA_MAC_GC82547EB:
#ifdef MAC_GC82547EB
			fdev[i].ports_present = 2;
			fdev[i].rx_rdy_threshold = 3;	// min data rec byte threshold is (n+1)*16
			fdev[i].tx_rdy_threshold = 31;	// min free space byte threshold (n+1)*16 = threshold = 8 mpackets 
			GigMACCreateDriver();
#endif
			break;

		default: 
#ifdef _DBG_PRINT
			printf("SysConf::Reset, unknown device type\n");
#endif
#if (OS == VXWORKS)
			return(SYSCONF_UNKNOWN_DEVICE);
#elif (OS == UCOS)
			return(SYSCONF_SUCCESS);
#endif
			break;
		}
	}

	SRAM_Write(0,0);				// initalize so pop of 0 will get 0

	return(SYSCONF_SUCCESS);
}

/*-----------------------------------------------------------------
	SysConf::ResetPorts
	Reset fbus device ports
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::ResetPorts(unsigned int ports_mask)
{
	int i, k=0;

	for (i=0;i<FBUS_DEVICE_COUNT; i++){
		switch(fdev[i].type){
		case OCTAL_MAC_21440:
#ifdef MAC_21440
		{
			int j;
			for (j=0; j<fdev[i].ports_present; j++, k++){
				if ((ports_mask>>k)&1) OctalMACResetPort(j, i);
			}
		}
#endif
			break;

		case GIGA_MAC_GC82547EB:
#ifdef MAC_GC82547EB
			int j;
			for (j=0; j<fdev[i].ports_present; j++, k++){
				if ((ports_mask>>k)&1) GigMACResetPort(j);
			}
#endif
			break;
		default: 
#ifdef _DBG_PRINT
			printf("SysConf::ResetPorts, unknown device type\n");
#endif
#if (OS == VXWORKS)
			return(SYSCONF_UNKNOWN_DEVICE);
#elif (OS == UCOS)
			return(SYSCONF_SUCCESS);
#endif
			break;
		}
	}
	return(SYSCONF_SUCCESS);
}


/*-----------------------------------------------------------------
	SysConf::Reset
	Reset drivers and microengines.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::Reset(unsigned int engine_mask, unsigned int port_mask)
{
	int i, retval;

	if (engine_mask > 0){
		// reset microengines
		uEng_resetUengs(engine_mask,0);
		uEng_clrResetUengs(engine_mask);

		// set PCs to 0, wakeup to voluntary
		for (i=0;i<6;i++){
			if ((engine_mask>>i) & 1){
				uEng_setPC(i, 0xf, 0);
				uEng_setCtxWakeupEvents(i, 0xf, 1);
			}
		}
	}


	// place F Bus devices in Reset
	retval = ResetPorts(port_mask);
	// assert reset, then deassert reset to FBI
	FBI_Reset();

	return(retval);
}


/*-----------------------------------------------------------------
	SysConf::Load
	Load system parameters from a file.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::Load(char *filename)
{
	FILE *fptr;
	char line[LINE_LENGTH_MAX];
	SysParam param_type = NO_PARAM;
	int param_int = 0;
	char param_string[PARAM_STR_LENGTH_MAX];

    if ( (fptr = fopen(filename, "r" ) ) == NULL ) {
#ifdef _DBG_PRINT
        printf("file not found %s\n", filename);
#endif
        return(SYSCONF_FILE_NOT_FOUND);
    }
    while(1) {
        // get one line
        if ( fgets(line, LINE_LENGTH_MAX, fptr) == NULL) {
            break;
        }
        sscanf(line,"%d %d %s", param_type, param_string, param_int);

		switch(param_type){
			case UOF_FILENAME:
				strcpy(uof_file, param_string);
				break;
#ifdef _DBG_PRINT
			default:
		printf("unknown param type %s\n", line);
#endif
		}
		
	}
	
 	return(SYSCONF_SUCCESS);
}


/*-----------------------------------------------------------------
	SysConf::ConfigureFBusInterface
	Configure the FBUS interface based on parameters in config_1200.h.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
void SysConf::ConfigureFBusInterface()
{
	unsigned int temp = 0x900;

	fbi->setRCV_RDY_CTL(0);								// prevent unsolicited autopush
	fbi->setXMIT_RDY_CTL(0);
	fbi->setFLOWCTL_MASK(0);
	fbi->setHASH_MULTIPLIER_64_HI(0);				// initialize multiplier to prime number
	fbi->setHASH_MULTIPLIER_64_LO(17);
	fbi->setHASH_MULTIPLIER_48_HI(0);
	fbi->setHASH_MULTIPLIER_48_LO(17);
	fbi->setTHREAD_DONE_REG0(0xffffffff);			// prevent power up thread dones
	fbi->setTHREAD_DONE_REG1(0xffffffff);

#if ((FBUS_WIDTH != BUS_WIDTH_64) && (FBUS_WIDTH != BUS_WIDTH_32))
#error "FBUS_WIDTH" contains an invalid value: FBUS_WIDTH
#endif

/*
	write GPIO_EN register, 
	if 64 bit FBUS, set bits 5:4 = 1, FBUS interface owns GPIO<0>
	if 32 bit FBUS, set bits 5:4 = 3, FBUS interface owns all GPIO
 */
#if (FBUS_WIDTH == 64)
	WRITE_LWORD(0x90001800, 0x10);
#else
	WRITE_LWORD(0x90001800, 0x30);
#endif

	if (FBUS_WIDTH == BUS_WIDTH_64) temp |= 0x200;

#if (FBUS_DEVICE_COUNT > 7)
#error "FBUS_DEVICE_COUNT" contains an invalid mac count: FBUS_DEVICE_COUNT
#endif
	
	if (FBUS_DEVICE_COUNT < 3)	temp |=	0x400;

#if ((FBUS_NO_STATUS != 0) && (FBUS_NO_STATUS != 1))
#error "FBUS_NO_STATUS" contains an invalid value: FBUS_NO_STATUS
#endif
	
	if (FBUS_NO_STATUS == 1) temp |= 0x040;

	if (byte_order == LITTLE_E) temp |= 0x20;

	fbi->setRDYBUS_TEMPLATE_CTL(temp);

	switch(FBUS_DEVICE_COUNT){
	case 1: 
		temp =	RDY_SEQ_1MACS;
		break;
	case 2:
		temp = RDY_SEQ_2MACS;
		break;
	case 3:
		temp = RDY_SEQ_3MACS;
		break;
	case 4:
		temp = RDY_SEQ_4MACS;
		break;
	case 5:
		temp = RDY_SEQ_5MACS;
		break;
	case 6:
		temp = RDY_SEQ_6MACS;
		break;
	case 7:
		temp = RDY_SEQ_7MACS;
		break;
	}

	if (sysConf->rx_autopush){
		temp |= RDY_SEQ_RX_AUTOPUSH;
	} else temp |= RDY_SEQ_NO_RX_AUTOPUSH;

	if (sysConf->tx_autopush){
		temp |= RDY_SEQ_TX_AUTOPUSH;
	} else temp |= RDY_SEQ_NO_TX_AUTOPUSH;

	fbi->setRDYBUS_TEMPLATE_PROG_1(temp);
	fbi->setRDYBUS_TEMPLATE_PROG_2(RDY_SEQ_FLW_CTLS);			// Flow controls execute faster than nops
	if (sysConf->tx_autopush){
		fbi->setRDYBUS_TEMPLATE_PROG_3(RDY_SEQ_PROG_3_TXAP);
	} else fbi->setRDYBUS_TEMPLATE_PROG_3(RDY_SEQ_FLW_CTLS);	// Flow controls execute faster than nops

	fbi->setRDYBUS_SYNCH_COUNT_DEFAULT(sysConf->rdy_synch_count);

	return;
}


/*-----------------------------------------------------------------
	SysConf::BindSymbol
	Load system parameters from a file.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::BindSymbol(char *image_name, char *symbol_name, int val)
{
	if (nextSymbol < BOUND_SYMBOLS_MAX){
		boundSymbols[nextSymbol].image_name = image_name;
		boundSymbols[nextSymbol].symbol_name = symbol_name;
		boundSymbols[nextSymbol].val = val;
		nextSymbol++;
		return(SYSCONF_SUCCESS);
	} else return (SYSCONF_BOUND_SYMBOLS_LIMIT);
}


/*-----------------------------------------------------------------
	SysConf::UCLoadBuffer
	Load microcode from a buffer.
	returns: SYSCONF_SUCCESS if no errors, else fail code
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::UCLoadBuffer(unsigned int* buf, unsigned int buflen)
{
	int retval, i;

	if (retval = UcLo_MapObjAddr(&uclo_handle, buf, buflen, 1)){
		return(retval);							// return uclo error
	}

	if(pUofCb) if((retval = (pUofCb)(pUofCbArg))) return(retval);

	for (i=0; i<nextSymbol; i++){
		// bind symbol only if matching image_name and symbol_name are in the microcode 
		UcLo_BindSymbol(uclo_handle, boundSymbols[i].image_name, boundSymbols[i].symbol_name, boundSymbols[i].val);
	}
	return(UcLo_WriteUimageAll(uclo_handle));
}


/*-----------------------------------------------------------------
	SysConf::UCLoadFile
	Load microcode from a file.
	returns: SYSCONF_SUCCESS if no errors, else fail code
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::UCLoadFile(char * filename)
{
	int retval, i;

	strcpy(uof_file, filename);


	if (retval = UcLo_LoadObjFile(&uclo_handle, filename)){
		return(retval);							// return uclo error
	}

	if(pUofCb) if((retval = (pUofCb)(pUofCbArg))) return(retval);

	for (i=0; i<nextSymbol; i++){
		// bind symbol only if matching image_name and symbol_name are in the microcode 
		UcLo_BindSymbol(uclo_handle, boundSymbols[i].image_name, boundSymbols[i].symbol_name, boundSymbols[i].val);
	}
	return(UcLo_WriteUimageAll(uclo_handle));
}


/*-----------------------------------------------------------------
	SysConf::Configure
	Load parameters to F Bus devices, F bus interface.
	returns: SYSCONF_SUCCESS if no errors, else fail code
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::Configure()
{
	int i;

	BusWidth fb_width = FBUS_WIDTH;
	
	// configure FBI
	ConfigureFBusInterface();

	// configure MACs
	for (i=0;i<FBUS_DEVICE_COUNT; i++){

		switch(fdev[i].type){
		case OCTAL_MAC_21440:
#ifdef MAC_21440
		{
			int j;
			for (j=0; j<fdev[i].ports_present; j++){
				OctalMACCofigureFifoAndBusWidthOnPort(j, i, fb_width,
                                        (FIFO_Threshold)(fdev[i].rx_rdy_threshold),
                                        (FIFO_Threshold)(fdev[i].tx_rdy_threshold));
			}
			break;
		}
#endif
		case GIGA_MAC_GC82547EB:
#ifdef MAC_GC82547EB
		{
			int j;
			for (j=0; j<fdev[i].ports_present; j++){
				GigMACCofigureFifoAndBusWidthOnPort(j, fb_width, fdev[i].rx_rdy_threshold, fdev[i].tx_rdy_threshold);
				GigMACCofigureEndian(BIG);		// default for now is big
			}
		}
			break;
#endif		default: 
#ifdef _DBG_PRINT
			printf("SysConf::Configure, unknown device type\n");
#endif
#if (OS == VXWORKS)
			return(SYSCONF_UNKNOWN_DEVICE);
#elif (OS == UCOS)
			return(SYSCONF_SUCCESS);
#endif
			break;
		}
	}
	return(SYSCONF_SUCCESS);
}

/*-----------------------------------------------------------------
	SysConf::StartPorts
	Start fbus device ports
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::StartPorts(unsigned int ports_mask, PortRunDirection direction)
{
	int i, k=0;

	for (i=0;i<FBUS_DEVICE_COUNT; i++){
		switch(fdev[i].type){
		case OCTAL_MAC_21440:
#ifdef MAC_21440
		{
			int j;
			for (j=0; j<fdev[i].ports_present; j++, k++){
				if ((ports_mask>>k)&1) OctalMACStartPort(j, i, direction);
			}
			break;
		}
#endif
		case GIGA_MAC_GC82547EB:
#ifdef MAC_GC82547EB
		{
			int j;
			for (j=0; j<fdev[i].ports_present; j++, k++){
				if ((ports_mask>>k)&1) GigMACStartPort(j, direction);
			}
		}
#endif
			break;
		default: 
#ifdef _DBG_PRINT
			printf("SysConf::Start, unknown device type\n");
#endif
#if (OS == VXWORKS)
			return(SYSCONF_UNKNOWN_DEVICE);
#elif (OS == UCOS)
			return(SYSCONF_SUCCESS);
#endif
			break;
		}
	}
	return(SYSCONF_SUCCESS);
}

/*-----------------------------------------------------------------
	SysConf::StopPorts
	Stop fbus device ports
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::StopPorts(unsigned int ports_mask, PortRunDirection direction)
{
	int i, k=0;

	for (i=0;i<FBUS_DEVICE_COUNT; i++){
		switch(fdev[i].type){
		case OCTAL_MAC_21440:
#ifdef MAC_21440
		{
			int j;
			for (j=0; j<fdev[i].ports_present; j++, k++){
				if ((ports_mask>>k)&1) OctalMACStopPort(j, i, direction);
			}
			break;
		}
#endif
		case GIGA_MAC_GC82547EB:
#ifdef MAC_GC82547EB
		{
			int j;
			for (j=0; j<fdev[i].ports_present; j++, k++){
				if ((ports_mask>>k)&1) GigMACStopPort(j, direction);
			}
		}
			break;
#endif
		default: 
#ifdef _DBG_PRINT
			printf("SysConf::Start, unknown device type\n");
#endif
#if (OS == VXWORKS)
			return(SYSCONF_UNKNOWN_DEVICE);
#elif (OS == UCOS)
			return(SYSCONF_SUCCESS);
#endif
			break;
		}
	}
	return(SYSCONF_SUCCESS);
}


/*-----------------------------------------------------------------
	SysConf::Start
	Start devices and microengines
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::Start(unsigned int engine_mask, unsigned int port_mask)
{
	int retval, i;


	// set PCs to 0, wakeup to voluntary
	for (i=0;i<6;i++){
		if ((engine_mask>>i) & 1){
			uEng_setPC(i, 0xf, 0);
			uEng_setCtxWakeupEvents(i, 0xf, 1);
		}
	}

	// start microengines
	for (i=0; i<MICROENGINES_MAX; i++){
		if ((engine_mask>>i)&1){
			uEng_start(i, CONTEXT_0_ID, ALL_CONTEXTS_MASK);
		}
	}
	// wait for microcode to rev up			tbd use signal method
	delayMs(10);

	// start fbus devices
	retval = StartPorts(port_mask, RX_AND_TX);

	return(retval);
}

/*-----------------------------------------------------------------
	SysConf::Stop
	Stop and reset devices. Reset microengines.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::Stop(unsigned int engine_mask, unsigned int port_mask)
{
	int retval;

	// stop and reset fbus devices
	if (retval = StopPorts(port_mask, RX)) return(retval);

	// wait for ports to stop			tbd use signal method
	delayMs(10);

	// reset microengines
	uEng_resetUengs(engine_mask,0);
	uEng_clrResetUengs(engine_mask);

	return(SYSCONF_SUCCESS);
}


/*-----------------------------------------------------------------
	SysConf::TakeControl
	
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::TakeControl()
{
	if(inControl) return(0);
    inControl++;
	return(Sem_Wait(SystemControlSem, SEM_WAITFOREVER));
}


/*-----------------------------------------------------------------
	SysConf::GiveControl
	
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::GiveControl()
{
	if(!inControl) return(0);
    inControl--;
	return(Sem_Post(SystemControlSem));
}



/*-----------------------------------------------------------------
	SysConf::RegisterThread
	Register a thread name with the system
	returns: thread id or negative fail code
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::RegisterThread(char *thread_name)
{
	if (nextThread < COMM_THREADS_MAX){
		strcpy(commThreads[nextThread], thread_name);
		return(nextThread++);
	} else return(SYSCONF_THREAD_LIMIT);
}


/*-----------------------------------------------------------------
	SysConf::ResolveThread
	Get a id for a thread name.
	returns: thread id  or negative fail code
	uses:
	modifies:
  -----------------------------------------------------------------*/
int SysConf::ResolveThread(char *thread_name)
{
	int i;

	for(i=0; (i<COMM_THREADS_MAX && i<nextThread); i++){
		if (strcmp(commThreads[i], thread_name) ==0) return(i);
	}
	return(SYSCONF_THREAD_NOT_FOUND);
}


/*-----------------------------------------------------------------
	SysConf_Attach
	Return a pointer to the SysConf instance
	returns: SysConf*
	uses:
	modifies:
  -----------------------------------------------------------------*/
SysConf *SysConf_Attach()
{
	if(!sysConf) sysConf = new SysConf();
	return(sysConf);
}


