/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* rtm_console.c
 * register console functions so they can be called at the transactor cmd line
 *
 *
 * system: SA1200
 * subsystem: NetApp
 * author: Don Hooper 1/29/98
 * revisions:
 * $History netApp_console.cpp $
 * 
 * 
 */ 

/*=================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "config_1200.h"
#include "net_app.h"

#if (IOSTYLE!=HARDWARE)
#include "ambaio.h"
#if defined(BUILD_DLL)
#include "xact_dll.h"
#else
#include "xact.h"
#endif

extern unsigned int AMBAIO_InitDone, driver_InitDone;
#endif

static SysConf *sysConf = NULL;


void ucommInit_thread(void *)
{
    FDriver_Init();
    driver_InitDone = 1;
	return;
}


int netapp_init()
{
	// creat the sysConfig object;
	sysConf = SysConf_Attach();

// note: because ueng driver must run AMBAIO, we init the AMBAIO here
#if (IOSTYLE != HARDWARE)
// initialize ambaio

	thread_T	threadId;
    driver_InitDone = 0;
    AMBAIO_InitDone = 0;

	ambaIoInit();
	char *spin="|/-|\\";
	int i =0;
	while(AMBAIO_InitDone == 0){
		if(i > 4) i = 0;
        printf("		Init AmbaIO:	%c\r", spin[i++]);
        XACT_ExecuteCommandStr("go 1");
    }

#ifdef BRIDGE
	threadId = Thread_Create(MID_PRIORITY, ucommInit_thread, (void *)NULL);
	while(driver_InitDone == 0)
    {
        printf(".");
        XACT_ExecuteCommandStr("go 1");
	}
#endif
#endif

#ifdef BRIDGE
	Br_Init();
#endif

#ifdef ROUTE
	RTM_Init();
#endif

	return(1);
}


int netapp_ucLoadFile(char *filename)
{
	if(!sysConf) return(0);
	sysConf->UCLoadFile(filename);
	return(1);
}


int netapp_initSv(void)
{
	if(!sysConf) return(0);
	Br_SvInit();
	return(1);
}


/* FOREIGN MODEL INITIALIZE

  instructions
	1. define ext_model_port as a compiler switch when compiling safe_xact
	2. "													   " rtm_registration
 */

/* functions called by transactor to foreign model */

/* this routine will be called to initialize the foreign model after */
/* the transactor "init" command has successfully executed.  Returning 0 */
/* will result in a transactor error. */
#if (IOSTYLE != HARDWARE)
#ifndef STANDALONE
int netapp_model_initialize()
#else
int foreign_model_initialize()
#endif
{
	if (!XACT_register_console_function("netapp_init", netapp_init, 0, 0) ){
		fprintf(stderr,"XACT_register_console_function() netapp_init failed\n");
		return(0);
	}
	if (!XACT_register_console_function("netapp_ucLoadFile", netapp_ucLoadFile, 1, 0) ){
		fprintf(stderr,"XACT_register_console_function() netapp_ucLoadFile failed\n");
		return(0);
	}
	if (!XACT_register_console_function("netapp_initSv", netapp_initSv, 0, 0) ){
		fprintf(stderr,"XACT_register_console_function() netapp_initSv failed\n");
		return(0);
	}

	return(1);
}


/* this routine will be called prior to each transactor simulation event */
/* It can be used to deposit state values into the transactor model prior */
/* to simulating the next event.  Returning 0 results in an error. */
#ifdef STANDALONE 
int foreign_model_pre_sim()
{
	return(1);
}

/* this routine will be called subsequent to each transactor simulation event. */
/* It can be used to query transactor simulation state, in order to copy it */
/* into the foreign model simulator. */
int foreign_model_post_sim()
{
	return(1);
}

/* this routine will be called just prior to exiting the simulator. */
/* The routine allows the foreign model to clean up, close files, etc */
/* before shutting down the program. */
int foreign_model_exit()
{
    return(1);
}

/* this routine will be called just after reseting the simulator. */
/* The routine allows the foreign model to reset itself to stay */
/* in sync with the simulator. */
int foreign_model_reset()
{
    AMBAIO_ResetAllChips();
	return(1);
}


#if defined(BUILD_DLL)
/*-----------------------------------------------------------------
    GetForeignModelFunctions

    This function is exported as the sole entry point into the DLL version
    of this package.  The Developer Workbench calls it in order to get the
    foreign model entry points for the transactor.  The Workbench thens
    registers these entry points with the transactor.
    
     returns: 
        uses: 
    modifies: 
*/
#if defined(__cplusplus)
extern "C"
#endif
__declspec(dllexport) void __cdecl
GetForeignModelFunctions(
    int (**ForeignModelInitialize)(),
    int (**ForeignModelPreSim)(),
    int (**ForeignModelPostSim)(),
    int (**ForeignModelExit)(),
    int (**ForeignModelReset)())
{
    *ForeignModelInitialize = foreign_model_initialize;
    *ForeignModelPreSim = foreign_model_pre_sim;
    *ForeignModelPostSim = foreign_model_post_sim;
    *ForeignModelExit = foreign_model_exit;
    *ForeignModelReset = foreign_model_reset;
}
#endif      // #if defined(BUILD_DLL)
#endif		// STANDALONE
#endif		// IOSTYLE != HARDWARE
