#include <stdio.h>
#include "ueng.h"
#include "debug_1200.h"
#include "debug_1200.hxx"
#include "net_app.h"
#include "hal_sram.h"
#include "sram_man.h"
#include "config_1200.h"
#include "mem_map.h"

#ifdef ROUTE
#ifdef RTM_MVR
#include "rtm_mvr.h"
#else
#include "rtm.h"
#endif
#endif

uDbg_uStoreFree_T FreeU;
uDbg_BkptArray_T BkptArray;
uDbg_Bkpt_T Bkpt;
Debug *DbgObj = NULL;


void bkPtCallback(unsigned short breakpointId, unsigned char ueng,
				  unsigned char ctx, void *myData)
{
	printf("\n\n @@@@@@ BREAKPOINT: BreakpointId=%d, Micro-engine=%d, Context %d stopped! @@@@@@\n",
		breakpointId, ueng, ctx); fflush(stdout);
	fflush(stdout);
	return;
}


void lstBreak()
{
   uDbg_Bkpt_T *bp;
   for(int i = 0; i < BkptArray.len; i++){
      bp = &BkptArray.bkpts[i];
	  printf("Breakpoint: id=%d, uEng=%d, activeCtxMask=0x%x, uAddr=0x%x\n",
	          bp->id, bp->uEng, (bp->ctx & 0xf), bp->uAddr);
   }
   return;
}

int TST_DbgInit(){
	DbgObj = new Debug(NULL);
    Bkpt.callback = bkPtCallback;
    Bkpt.usrData = NULL;
    BkptArray.len = 0;
   return 0;
}



int TST_GetPC(unsigned char uEng, unsigned char ctx)
{
   unsigned int pc;
   if(uEng_getPC(uEng, ctx, &pc) != UENG_SUCCESS) return(-1);
   return(pc);
}

int TST_ShowPCs()
{
	int i, j;
	printf("ShowPCs: "); fflush(stdout);
	for (i=0;i<6;i++){
		for (j=0;j<4;j++){
			printf("F%d/%d=%d, ", i, j, TST_GetPC(i,j)); fflush(stdout);
		}
	}
	printf("\n");fflush(stdout);

	return(1);
}



int TST_GetUword(unsigned char uEng, unsigned int uAddr)
{
   unsigned int uWord;
   if(uEng_getNumUword(uEng, uAddr, 1, &uWord) != UENG_SUCCESS) return(-1);
   return(uWord);
}

// type 0 is A Bank, type 1 is B Bank, type 2 is sram xfer, type 3 is sdram xfer
int TST_GetGPR(unsigned char box, unsigned int type, unsigned char ctx, unsigned short regnum)
{
	unsigned int gpreg;
	if(uEng_readRelGpr(box, type, ctx, regnum, &gpreg) != UENG_SUCCESS) return(-1);;
	return(gpreg);
}


int TST_ShowReadXfers(unsigned char box, unsigned char ctx)
{
	int i;
	printf("ShowReadXfers: "); fflush(stdout);
	for (i=0;i<8;i++){
		printf("$xfer%d=%.8x, ", i, TST_GetGPR(box,2,ctx,i)); fflush(stdout);
	}
	printf("\n"); fflush(stdout);

	return(1);
}


// Microcode writes 100 locations of sram
// ucld.c is created by sram_write0_100.bat
//
#ifdef UC_BUFFER
int TST_UcWriteSram(void)
#else
int TST_UcWriteSram(char *UofFn)
#endif
{
	int status, i;
	if((status = NetApp_Init())) return(status);
#ifdef UC_BUFFER
	if((status = NetApp_UCLoadBuffer())) return(status);
#else
	if((status = NetApp_UCLoadFile(UofFn))) return(status);
#endif
	if((status = NetApp_Run(1, 0))) return(status);
	for (i=0;i<100;i++){
		printf("SRAM(%d) = 0x%x\n", i, SRAM_Read(i)); fflush(stdout);
	}
	return(0);
}

// receive packets on ports 1-4 using microengine 1
// transmit packets on same ports using microengine 0
// ucld.c is created by rx_tx_pin.bat
//
#ifdef UC_BUFFER
int TST_RxTxPin(void)
#else
int TST_RxTxPin(char *UofFn)
#endif
{
	int status;
	if((status = NetApp_Init())) return(status);
#ifdef UC_BUFFER
	if((status = NetApp_UCLoadBuffer())) return(status);
#else
	if((status = NetApp_UCLoadFile(UofFn))) return(status);
#endif
	// zero (invalidate) transmit status locations
	SRAM_BZero(0x100, 100);
	SRAM_BZero(0x1000, 100);
	SRAM_BZero(0x2000, 100);
	SRAM_BZero(0x3000, 100);
	// run microengines 0 and 1, ports 1-4
	if((status = NetApp_Run(0x3, 0x1e))) return(status);
	return(0);
}


// set a breakpoint
//
int TST_SetBreak(int ueng, int activeCtxMask, int breakOnCtx, int upc)
{
   uDbg_CTX_T breakIf;
   int stpUengMask = 0x3f;			// stop all uengines

   switch(breakOnCtx){
      case 0: breakIf = UDBG_CTX0; break;
      case 1: breakIf = UDBG_CTX1; break;
      case 2: breakIf = UDBG_CTX2; break;
      case 3: breakIf = UDBG_CTX3; break;
      default: breakIf = UDBG_CTXANY; break;
   }

   Bkpt.uEng = (unsigned char)ueng;
   Bkpt.ctx = (unsigned short)(activeCtxMask & 0xf);
   Bkpt.uAddr = (unsigned short)upc;
   Bkpt.breakIfCtx = breakIf;
   Bkpt.stpOnBrkUengMask = stpUengMask;
    return DbgObj->SetBreakpoint(&Bkpt);
}


// show all breakpoints
//
int TST_ShowBreak(int uengMask)
{
   int status;
   status = DbgObj->ShowBreakpoints(uengMask, &BkptArray);
   lstBreak();
   return status;
}

// clear all breakpoints
//
int TST_clrAllBreak(int uengMask)
{
  return DbgObj->ClearAllBreakpoints(uengMask);
}

// clear one breakpoint
//
int TST_ClrBreak()
{
   return DbgObj->ClearBreakpoint(Bkpt.id);
}

// resume after pause or breakpoint
//
int TST_Resume(unsigned int engineMask)
{
    return DbgObj->ResumeUeng(engineMask);

}

// resume after pause or breakpoint
//
int TST_Pause(unsigned int engineMask)
{
    return DbgObj->PauseUeng(engineMask, 10);

}

// run all microengines to next context arb
//
int TST_Hop()
{
	unsigned int uemask = 0x3f;
	return DbgObj->HopUeng(&uemask, 5);
}


// dump microengine state
//
void TST_dump(unsigned int uEngMask)
{
	if(DbgObj) DbgObj->regDump(uEngMask);
	return;
}
