/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* br_config.h
 *
 *
 *
 * system: SA1200
 * subsystem: Bridge
 * author: Rajendra Shukla 02/26/99
 * revisions:
 * $History: br_config.h $
 * 
 * *****************  Version 18  *****************
 * User: Kalkunte     Date: 9/19/00    Time: 10:21a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * - Moved code to improve code stability when entries are overwritten. 
 * - Added comments to code.
 * - Changed error codes to be negative, not to be mistaken with other
 * valid values.
 * 
 * *****************  Version 17  *****************
 * User: Kalkunte     Date: 9/05/00    Time: 1:33p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Adding 
 * - L2 filter and port configuration
 * - Software hash routines to avoid using hardware hash unit
 * 
 * *****************  Version 1  *****************
 * User: Kalkunte     Date: 8/31/00    Time: 5:32p
 * Created in $/users/kalkunte/bridge
 * Bridge with L2 filter
 * 
 * *****************  Version 15  *****************
 * User: Drjohns1     Date: 1/20/00    Time: 5:04p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Renamed DEFAULT_PORT1 & DEFAULT_PORT2
 * 
 * *****************  Version 12  *****************
 * User: Dhooper      Date: 5/14/99    Time: 9:44a
 * Updated in $/BL5_Plus/SA1_CoreLibs/bridge
 * move stuff from bridge.h to br_config.h
 * 
 * *****************  Version 11  *****************
 * User: Dhooper      Date: 5/07/99    Time: 11:14a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * use latest iostyle convention
 * 
 * *****************  Version 10  *****************
 * User: Rshukla1     Date: 4/13/99    Time: 9:35p
 * Updated in $/development/bridge
 * 
 * *****************  Version 9  *****************
 * User: Rshukla1     Date: 4/12/99    Time: 5:27p
 * Updated in $/development/bridge
 * 
 * *****************  Version 8  *****************
 * User: Rshukla1     Date: 4/01/99    Time: 1:15p
 * Updated in $/development/bridge
 * 
 * *****************  Version 7  *****************
 * User: Qabuild      Date: 3/24/99    Time: 9:04a
 * Updated in $/development/bridge
 * Updated bridge project again.  Wrong path in links area for debug and
 * release of bridge.exe
 * 
 * *****************  Version 6  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:11p
 * Updated in $/development/bridge
 * update COPYRIGHT
 * 
 * *****************  Version 1  *****************
 * User: Rshukla1     Date: 4/01/99    Time: 10:21a
 * Created in $/users/Shukla/brigde/brm
 * 
 * *****************  Version 3  *****************
 * User: Rshukla1     Date: 3/05/99    Time: 3:01p
 * Updated in $/development/bridge
 * 
 * *****************  Version 2  *****************
 * User: Rshukla1     Date: 3/03/99    Time: 6:43p
 * Updated in $/development/bridge
 * yet not stable. Do not try to run
 * 
 * *****************  Version 1  *****************
 * User: Rshukla1     Date: 3/03/99    Time: 9:44a
 * Created in $/development/bridge
 * initial bridge managment files
 * 
 * 
 */ 
#ifndef BR_CONFIG_H
#define BR_CONFIG_H

#ifdef STANDALONE
#include "iostyle.h"			// core to rest of chip interface style
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include "br_share.h"

#define DEFAULT_PORTA 0 // default port id for forwarding packets
#define DEFAULT_PORTB 1 // default port id for forwarding packets
#define BR_FRWD_ENTRY_SIZE  4 // four long words.
#define BR_NUM_DEFAULT_ENTRY 2 // number of default entries

#define BR_THREAD_ID 18   //Thread Id on which service function runs

#define NUM_OF_L2  4    // if using static map of sram then it 
                        // reflects the size of sram allocation 
                        // in the mem_map.h

#define NUM_OF_FRW_ENTRY    65536  // max 64 entry
#define BR_NUM_PORTS        16

#define SIZE_OF_L1 65536    // 2** 16  fixed shouldn't be changed
#define SIZE_OF_L2 2048      // 2** 11 fixed shouldn't be changed

// size of different table
#define BR_SRAM_L1_SIZE     (SIZE_OF_L1)
#define BR_SRAM_L2_SIZE     (SIZE_OF_L2*NUM_OF_L2)
#define BR_SRAM_PORT_SIZE   (BR_NUM_PORTS * 4 * 4)	// 4 32bit words in each entry.
#define BR_SRAM_PROTO_SIZE  (0)
#define BR_SDRAM_FRWD_SIZE  (BR_FRWD_ENTRY_SIZE* NUM_OF_FRW_ENTRY)


// bridge utility macros

#ifndef WIN32
#define CLEAR "clear"
#else
#define CLEAR "cls"
#endif

#ifndef TRUE
#define TRUE	1
#endif

#ifndef FALSE
#define FALSE	0
#endif

#define BR_FALSE		0
#define BR_TRUE			1
#define BR_ERROR		-2
#define BR_SUCCESS		3
#define BR_DUPLICATE	-4
// bridge could have three types of entry
#define  BR_NO_ENTRY	-1 
// there is no entry

#define  BR_ENTRY_NO_COL 0 
// there is entry but no collision

#define  BR_ENTRY_COL 1 
// there is entry and a collision
// there could be lock error so it can not be compiled

#define BR_LOCK_ERROR -2 
// an entry have a lock bit set

#define BR_L2_INC (32 - L2_SHIFT) 
// sparse table shift value

// 4  bit lookup for l2 tables, dependent on L2_SHIFT
#define L2_SHIFT 4   
#define L2_ENTRY_SIZE 16

#define BR_SHIFT_MASK_SIZE 5	// last five bits are used as shift mask

#define BR_L2_ENTRY_SIZE (0x1 << L2_SHIFT)

// MEMORY MANAGEMENT // 8k 
#define BR_L2_CHUNK_SIZE 0x2000 


//---------------------------------------------
// MASKS
//--------------------------------------------
#define L1_LookupMask 0xFFFF  

// 16 bit lookup for L1 tables
#define L2_LookupMask 0xF  

// each next shift have to four bit dependent on L2_LookupMask
// <28:5> bits
#define L2_INDEX_MASK  0x1FFFFFE0 

#define BR_PORT_MASK  0x001F  // last five bits

#define BR_MASK_L2 0xF  // L2 mask next four bit from the hash index

//-----------------------------------------------

//-----------------------------------------------
//MACROS
//-----------------------------------------------

//set the collition (31st) bit of long word <31:0>
// set 31st bit to 1
#define BR_SET_C_BIT(a)         ((a)|0x80000000)  

//unset the collition (31st) bit of long word <31:0>
// remove 31st bit
#define BR_UNSET_C_BIT(a)       ((a)&0x7FFFFFFF)  

//set the lock (30) bit of long word <31:0>
#define BR_SET_LOCK_BIT(a)      ((a)|0x40000000)

//unset the lock (30st) bit of long word <31:0>
// remove 30st bit
#define BR_UNSET_LOCK_BIT(a)    ((a)&0xBFFFFFFF)  

//set the filter (29th) bit of long word <31:0>
#define BR_SET_F_BIT(a)         ((a)|0x20000000)  

//unset the filter (29th) bit of long word <31:0>
#define BR_UNSET_F_BIT(a)       ((a)&0xDFFFFFFF)  

//test the filter (29th) bit of long word <31:0>
#define BR_TEST_F_BIT(a)         ((a) & 0x20000000)  

// extract forwarding index <28:5> when there is collision
// shift left then get last 24 bits
#define BR_EXTRACT_FWR_INDEX(a) ( (a) >> (BR_SHIFT_MASK_SIZE) & 0xFFFFFF )

// get port MASK
#define BR_GET_PORT(a) ((a) & (BR_PORT_MASK))
//delete the last port and add a new port
#define BR_CHANGE_PORT(a,b) ( ( (a) & (~(BR_PORT_MASK)) ) | ((BR_PORT_MASK) & (b) ) )

// some debug MACROS
#ifdef BR_DEBUG

#define D_PRINT0(a)      (fprintf(stderr,"DEBUG %s:%d:"a,__FILE__,__LINE__))
#define D_PRINT1(a,b)   (fprintf(stderr,"DEBUG %s:%d:"a,__FILE__,__LINE__,b))
#define D_PRINT         printf

#else

#define D_PRINT0(a) //
#define D_PRINT1(a,b)  //
#define D_PRINT     //

#endif // BR_DEBUG


#endif /* BR_CONFIG_H */
