/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* br_console.c
 * register console functions so they can be called at the transactor cmd line
 *
 *
 * system: SA1200
 * subsystem: Bridge 
 * author: Rajendra Shukla 02/26/98
 * revisions:
 * $History br_console.c $
 * 
 * 
 */ 

/*=================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "br_config.h"
#include "brm.h"
#include "br_loader.h"
#include "hal_sram.h"
#include "hal_sdram.h"
#include "packet_gen.h"
#include "thread.h"
#include "uclo.h"


#if (IOSTYLE != HARDWARE)
#include "ambaio.h"
#if defined(BUILD_DLL)
#include "xact_dll.h"
#else
#include "xact.h"
#endif
extern unsigned int AMBAIO_InitDone;
#endif


extern void *Br_objHandle; // micro code object handle
extern "C" void FDriver_Init();
unsigned int driver_InitDone;

inline void br_init(){ Br_Init(); }			// for foreign model interface


void br_sv_init_thread(void *)
{
    FDriver_Init();

    driver_InitDone = 1;

	Br_SvInit();

	return;
}



/*-----------------------------------------------------------------
    br_fwd_add:
    Adds a route to the Bridge Hash Lookup Tables. It calls Br_AddEntry () routine 
	which adds the forwarding entry. 
    input:
    -Source Address (48 bit MAC address)
    -port number
    Output:
    - port number/ forwarding index
  -----------------------------------------------------------------*/
int br_fwd_add(
             unsigned int da2, 
             unsigned int da1, 
             int port_num)
{
    int ret;

    D_PRINT("--------------Entering Function:br_fwd_add---------------\n");
    ret =Br_AddEntry(da2, (unsigned short)da1, FALSE, FALSE, FALSE, port_num);
    D_PRINT ( "RETURN VAL<SUCCESS =1, FAILURE =0>= %d\n",ret);
    return(ret);
}

/*-----------------------------------------------------------------
    br_fwd_lookup:
    Performs a lookup on the forwarding entry associated with the destination address
	Bridge Hash Lookup 
    input:
    -Destination Address (48 bit MAC address)
    - receive port number
    Output: 
    - destination port number/ forwarding index
  -----------------------------------------------------------------*/
int br_fwd_lookup(
                unsigned int  da2,    // upper 32 bits of destination address
                unsigned short da1,   // next 16 bits of destination address
                int rcv_port // port on which this packet was recived
              )
{
    int ret;
    D_PRINT("--------------Entering Function:br_fwd_lookup---------------\n");
    ret = Br_LookupEntry(da2,(unsigned short)da1,rcv_port, NULL, NULL, NULL);
    D_PRINT (" port returned = %d = 0x%0x", ret, ret);
    return(ret);
}

/*-----------------------------------------------------------------
    br_filt_add:
    Adds a filter entry to the Bridge Hash Table.
    input:
    -Destination Address (48 bit MAC address)
    -port number
	-Filter for - Source or destination address
	-Filter action - permit/deny
    Output: 
    -Success/Failure
  -----------------------------------------------------------------*/
int br_filt_add(
             unsigned int da2, 
             unsigned int da1, 
			 bool sada,			// SA == TRUE/DA == FALSE
			 bool filt_action	// PERMIT == TRUE/DENY == FALSE
			 )
{
    int ret;
    D_PRINT("--------------Entering Function:br_filt_add---------------\n");
    ret =Br_AddEntry(da2, (unsigned short)da1, TRUE, sada, filt_action, 0);
    D_PRINT ( "RETURN VAL<SUCCESS =1, FAILURE =0>= %d\n",ret);
    return(ret);
}

/*-----------------------------------------------------------------
    br_filt_sa_lookup:
    Looks up the source address filter action using the Bridge Hash Table.
    input:
    -Source Address (48 bit MAC address)
    Output: 
    -Permit/deny
  -----------------------------------------------------------------*/
int br_filt_sa_lookup(
                unsigned int  sa2,    // upper 32 bits of destination address
                unsigned short sa1   // next 16 bits of destination address
              )
{
    int ret;
	unsigned int sram_entry, sdram_entry;

    D_PRINT("--------------Entering Function:br_filt_sa_lookup---------------\n");
    ret = Br_LookupEntry(sa2,(unsigned short)sa1,0, NULL, &sram_entry, &sdram_entry);

	if (BR_TEST_F_BIT(sram_entry)) {	// Filter associated with this entry
		ret = BR_TEST_FILTER_ACTION_SA (sdram_entry);
	}
	else
		ret = BR_ERROR;

    D_PRINT (" port returned = %d = 0x%0x", ret, ret);
    return(ret);
}

/*-----------------------------------------------------------------
    br_filt_da_lookup:
    Looks up the destination address filter action using the Bridge Hash Table.
    input:
    -Destination Address (48 bit MAC address)
    Output: 
    -Permit/deny
  -----------------------------------------------------------------*/
int br_filt_da_lookup(
                unsigned int  da2,    // upper 32 bits of destination address
                unsigned short da1   // next 16 bits of destination address
              )
{
    int ret;
	unsigned int sram_entry, sdram_entry;

    D_PRINT("--------------Entering Function:br_filt_da_lookup---------------\n");
    ret = Br_LookupEntry(da2,(unsigned short)da1,0, NULL, &sram_entry, &sdram_entry);

	if (BR_TEST_F_BIT(sram_entry)) {	// Filter associated with this entry
		ret = BR_TEST_FILTER_ACTION_DA (sdram_entry);
	}
	else
		ret = BR_ERROR;

    D_PRINT (" port returned = %d = 0x%0x", ret, ret);
    return(ret);
}

/*-----------------------------------------------------------------
    br_port_ethertype_add:
    Sets the ethertype on a port in the port table to enable L2 protocol filtering.
    input:
    -Port #
	-Ethertype filter on each port
    Output: 
  -----------------------------------------------------------------*/
void br_port_ethertype_add(
                unsigned long ether_type,	// Ethertype set on this port
                int port_num				// port on which this packet was recived
              )
{
    D_PRINT("--------------Entering Function:br_port_ethertype_add---------------\n");
    Br_SetPortProtoFilter(ether_type, port_num);
    D_PRINT (" port returned = %d = 0x%0x", ether_type, port_num);
}

/*-----------------------------------------------------------------
    br_port_ethertype_lookup:
    Look up the ethertype on a port in the port table.
    input:
    -Port #
    Output: 
	-Ethertype 
  -----------------------------------------------------------------*/
int br_port_ethertype_lookup(
                int rcv_port // port on which this packet was recived
              )
{
    int ret;
    D_PRINT("--------------Entering Function:br_port_ethertype_lookup---------------\n");
    ret = (int) Br_GetPortProtoFilter(rcv_port);
    D_PRINT (" port returned = %d = 0x%0x", ret, ret);
    return(ret);
}

/*-----------------------------------------------------------------
    br_port_filt_add:
    Sets the filtering method to be employed with a port in the port table.
    input:
    -Port #
	-Filter type on each port
	  BR_PORT_FILT_TYPE_ETHERTYPE			0x0	
	  BR_PORT_FILT_TYPE_EXPLICT				S0x1	
	  BR_PORT_FILT_TYPE_POSITIVE			0x2	
	  BR_PORT_FILT_TYPE_NEGATIVE			0x3	
    Output: 
  -----------------------------------------------------------------*/
void br_port_filt_add(
                unsigned short filt_type, // port on which this packet was recived
                int port_num // port on which this packet was recived
              )
{
    D_PRINT("--------------Entering Function:br_port_filt_add---------------\n");
    Br_SetPortFilterType(filt_type, port_num);
    D_PRINT (" port returned = %d = 0x%0x", filt_type, port_num);
}

/*-----------------------------------------------------------------
    br_port_filt_lookup:
    Lookup the filtering method to be employed with a port in the port table.
    input:
    -Port #
    Output: 
	-Filter type on each port
	  BR_PORT_FILT_TYPE_ETHERTYPE			0x0	
	  BR_PORT_FILT_TYPE_EXPLICT				S0x1	
	  BR_PORT_FILT_TYPE_POSITIVE			0x2	
	  BR_PORT_FILT_TYPE_NEGATIVE			0x3	
  -----------------------------------------------------------------*/
int br_port_filt_lookup(
                int rcv_port // port on which this packet was recived
              )
{
    int ret;
    D_PRINT("--------------Entering Function:br_port_filt_lookup---------------\n");
    ret = (int) Br_GetPortFilterType(rcv_port);
    D_PRINT (" port returned = %d = 0x%0x", ret, ret);
    return(ret);
}

/*-----------------------------------------------------------------
    br_port_filt_lookup:
    Sets the MAC address for the device in the port table.
    input:
    -MAC address
    Output: 
  -----------------------------------------------------------------*/
void br_port_MAC_add(
                unsigned int  da2,    // upper 32 bits of destination address
                unsigned short da1   // next 16 bits of destination address
              )
{
    D_PRINT("--------------Entering Function:br_port_MAC_add---------------\n");
    Br_SetPortMACAddr(da2, da1);
    D_PRINT (" port returned = %d = 0x%0x", da2, da1);
}

/*-----------------------------------------------------------------
    br_port_filt_lookup:
    Sets the MAC address for the device in the port table.
    input:
    Output: 
    -MAC address
  -----------------------------------------------------------------*/
void br_port_MAC_lookup(
                unsigned int  da2,    // upper 32 bits of destination address
                unsigned short da1   // next 16 bits of destination address
              )
{
    D_PRINT("--------------Entering Function:br_port_MAC_lookup---------------\n");
    Br_CheckPortMACAddr(da2, da1);
    D_PRINT (" port returned = %d = 0x%0x", da2, da1);
}

#if (IOSTYLE != HARDWARE)
/*-----------------------------------------------------------------
    br_sv_init
	Initialize ambaio interface, microcode communications driver, service library

  -----------------------------------------------------------------*/

int br_sv_init(char* fname)
{
	thread_T	threadId;

    driver_InitDone = 0;

    AMBAIO_InitDone = 0;

	ambaIoInit();
	while(AMBAIO_InitDone == 0)
    {
        printf(",");
        XACT_ExecuteCommandStr("go 1");
    }

    D_PRINT("--------------Entering Function:br_load_uof_file---------------\n");
	/* Load the uCode image from file */
    if((UcLo_LoadObjFile(&Br_objHandle, fname)) != UCLO_SUCCESS) {
		printf("Error; loading microcode object file %s\n", fname);
		return(BR_ERROR);
	}

	threadId = Thread_Create(MID_PRIORITY, br_sv_init_thread, (void *)NULL);
	while(driver_InitDone == 0)
    {
        printf(".");
        XACT_ExecuteCommandStr("go 1");
	}

	return(1);
}

/* instructions
	1. define ext_model_port as a compiler switch when compiling safe_xact
	2. "													   " br_registration
 */


/* functions called by transactor to foreign model */


/* this routine will be called to initialize the foreign model after */
/* the transactor "init" command has successfully executed.  Returning 0 */
/* will result in a transactor error. */
#ifndef STANDALONE
int br_model_initialize()
#else
int foreign_model_initialize()
#endif
{
	if (!XACT_register_console_function("br_init", br_init, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() br_init failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_sv_init", br_sv_init, 1, 0) ){
            fprintf(stderr,"XACT_register_console_function() br_sv_init failed\n");
			return(0);
	}
   	if (!XACT_register_console_function("br_fwd_add", br_fwd_add, 0, 3) ){
            fprintf(stderr,"XACT_register_console_function() br_fwd_add failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_fwd_lookup", br_fwd_lookup, 0, 3) ){
            fprintf(stderr,"XACT_register_console_function() br_fwd_lookup failed\n");
			return(0);
	}
   	if (!XACT_register_console_function("br_filt_add", br_filt_add, 0, 4) ){
            fprintf(stderr,"XACT_register_console_function() br_filt_add failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_filt_sa_lookup", br_filt_sa_lookup, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() br_filt_sa_lookup failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_filt_da_lookup", br_filt_da_lookup, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() br_filt_da_lookup failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_port_ethertype_add", br_port_ethertype_add, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() br_port_ethertype_add failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_port_ethertype_lookup", br_port_ethertype_lookup, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() br_port_ethertype_lookup failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_port_filt_add", br_port_filt_add, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() br_port_filt_add failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_port_filt_lookup", br_port_filt_lookup, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() br_port_filt_lookup failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_port_MAC_add", br_port_MAC_add, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() br_port_MAC_add failed\n");
			return(0);
	}
	if (!XACT_register_console_function("br_port_MAC_lookup", br_port_MAC_lookup, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() br_port_MAC_lookup failed\n");
			return(0);
	}

#ifdef STANDALONE

	// from hal_1200/hal_sram
	if (!XACT_register_console_function("SRAM_Write", SRAM_Write, 0, 2) ){
		fprintf(stderr,"XACT_register_console_function() SRAM_Write failed\n");
		return(0);
	}
	if (!XACT_register_console_function("SRAM_Read", SRAM_Read, 0, 1) ){
		fprintf(stderr,"XACT_register_console_function() SRAM_Read failed\n");
		return(0);
	}
	// from hal_1200/hal_sdram
	if (!XACT_register_console_function("SDRAM_Write", SDRAM_Write, 0, 2) ){
		fprintf(stderr,"XACT_register_console_function() SDRAM_Write failed\n");
		return(0);
	}
	if (!XACT_register_console_function("SDRAM_Read", SDRAM_Read, 0, 1) ){
		fprintf(stderr,"XACT_register_console_function() SDRAM_Read failed\n");
		return(0);
	}
	pkgen_model_initialize();
	fdrive_model_initialize();
	fmon_model_initialize();

#endif	// STANDALONE

	return(1);
}


/* this routine will be called prior to each transactor simulation event */
/* It can be used to deposit state values into the transactor model prior */
/* to simulating the next event.  Returning 0 results in an error. */
#ifdef STANDALONE 
int foreign_model_pre_sim()
{
	return(1);
}



/* this routine will be called subsequent to each transactor simulation event. */
/* It can be used to query transactor simulation state, in order to copy it */
/* into the foreign model simulator. */
int foreign_model_post_sim()
{
	return(1);
}

/* this routine will be called just prior to exiting the simulator. */
/* The routine allows the foreign model to clean up, close files, etc */
/* before shutting down the program. */
int foreign_model_exit()
{
    return(1);
}

/* this routine will be called just after reseting the simulator. */
/* The routine allows the foreign model to reset itself to stay */
/* in sync with the simulator. */
int foreign_model_reset()
{
	return(1);
}


#if defined(BUILD_DLL)
/*-----------------------------------------------------------------
    GetForeignModelFunctions

    This function is exported as the sole entry point into the DLL version
    of this package.  The Developer Workbench calls it in order to get the
    foreign model entry points for the transactor.  The Workbench thens
    registers these entry points with the transactor.
    
     returns: 
        uses: 
    modifies: 
*/
__declspec(dllexport) void __cdecl
GetForeignModelFunctions(
    int (**ForeignModelInitialize)(),
    int (**ForeignModelPreSim)(),
    int (**ForeignModelPostSim)(),
    int (**ForeignModelExit)(),
    int (**ForeignModelReset)())
{
    *ForeignModelInitialize = foreign_model_initialize;
    *ForeignModelPreSim = foreign_model_pre_sim;
    *ForeignModelPostSim = foreign_model_post_sim;
    *ForeignModelExit = foreign_model_exit;
    *ForeignModelReset = foreign_model_reset;
}
#endif      // #if defined(BUILD_DLL)

#endif  //STANDALONE
#endif		// IOSTYLE != HARDWARE

