/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/
/* br_share.h
 *
 * Shared between the bridge application and microcode (eg. L2L3fwd projects)
 *
 * system: SA1200
 * subsystem: Bridge
 * author: Suresh Kalkunte 02/26/99
 * $History: br_share.h $
 * 
 * *****************  Version 1  *****************
 * User: Kalkunte     Date: 9/05/00    Time: 1:38p
 * Created in $/dev_1200/SA1_CoreLibs/bridge
 * Adding 
 * - L2 filter and port configuration
 * - Software hash routines to avoid core access hardware hash
 */

#ifndef BR_FILTER_H
#define BR_FILTER_H

// hash service loader function
#define BR_HASH_LOAD_HI 0x0
#define BR_HASH_LOAD_LO 0x1

//////////////////////////////////////////////
//////////////////////////////////////////////
// Filter Action (in Forwarding entry)////////
#define BR_FILTER_ACTION_MASK				0x1	// one bit
#define BR_FILTER_ACTION_SA_SHIFT_SIZE		6		// Source Address Filter
#define BR_FILTER_ACTION_DA_SHIFT_SIZE		7		// Destination Address Filter

//////////////////////////////////////////////
// Filter Actions
#define BR_FILTER_ACTION_DENY		0x0
#define BR_FILTER_ACTION_PERMIT		0x1

//////////////////////////////////////////////
// Filter Action utility macros
//set SA Deny
#define BR_SET_FILTER_SA_DENY(a)		((a) | (BR_FILTER_ACTION_DENY << BR_FILTER_ACTION_SA_SHIFT_SIZE))

//set SA Permit
#define BR_SET_FILTER_SA_PERMIT(a)		((a) | (BR_FILTER_ACTION_PERMIT << BR_FILTER_ACTION_SA_SHIFT_SIZE))  

//unset SA filter action
#define BR_UNSET_FILTER_SA(a)			((a) & 0xffffffbf)  

//set SA Deny
#define BR_SET_FILTER_DA_DENY(a)		((a) | (BR_FILTER_ACTION_DENY << BR_FILTER_ACTION_DA_SHIFT_SIZE))

//set SA Permit
#define BR_SET_FILTER_DA_PERMIT(a)		((a) | (BR_FILTER_ACTION_PERMIT << BR_FILTER_ACTION_DA_SHIFT_SIZE))  

//unset DA filter action
#define BR_UNSET_FILTER_DA(a)			((a) & 0xffffff7f)  

//test Filter actions
#define BR_TEST_FILTER_ACTION_SA(a)		(((a) >> BR_FILTER_ACTION_SA_SHIFT_SIZE) & BR_FILTER_ACTION_MASK )  
#define BR_TEST_FILTER_ACTION_DA(a)		(((a) >> BR_FILTER_ACTION_DA_SHIFT_SIZE) & BR_FILTER_ACTION_MASK )  


//////////////////////////////////////////////
//////////////////////////////////////////////
// Port Entry (in port table)/////////////////
#define BR_PORT_ENTRY_MULTIPLER			4	// (2**4) Corresponds to 16 bytes == 4 32bit words.per entry

//////////////////////////////////////////////
// Port Filter Type Masks
#define BR_PORT_FILT_TYPE_MASK			0x03  // two bits
#define BR_PORT_FILT_TYPE_SHIFT_SIZE	4

//////////////////////////////////////////////
// Port Filter Types
#define BR_PORT_FILT_TYPE_ETHERTYPE			0x0		//Filter on ethertype set on port
#define BR_PORT_FILT_TYPE_EXPLICT			0x1		//L2 Filter using filter action specified in SRAM/SDRAM filter entry
#define BR_PORT_FILT_TYPE_POSITIVE			0x2		//Positive L2 Filter using filter entry specified in SRAM
#define BR_PORT_FILT_TYPE_NEGATIVE			0x3		//Negative L2 Filter using filter entry specified in SRAM

//////////////////////////////////////////////
// Port Filter Type utility macros

//set Explicit rule L2 Filter
#define BR_SET_PORT_FILT_TYPE_EXPLICIT(a)		((a) | (BR_PORT_FILT_TYPE_EXPLICT << BR_PORT_FILT_TYPE_SHIFT_SIZE))

//set Positive rule L2 Filter
#define BR_SET_PORT_FILT_TYPE_POSITIVE(a)		((a) | (BR_PORT_FILT_TYPE_POSITIVE << BR_PORT_FILT_TYPE_SHIFT_SIZE))

//set Negative rule L2 Filter
#define BR_SET_PORT_FILT_TYPE_NEGATIVE(a)		((a) | (BR_PORT_FILT_TYPE_NEGATIVE << BR_PORT_FILT_TYPE_SHIFT_SIZE))

//unset Filter rule
#define BR_SET_PORT_FILT_TYPE_ETHERTYPE(a)		((a) & 0xFFFFFFCF)

//get Filter type
#define BR_GET_PORT_FILTERTYPE(a)				(((a) >> BR_PORT_FILT_TYPE_SHIFT_SIZE) & BR_PORT_FILT_TYPE_MASK )  

//////////////////////////////////////////////
//////////////////////////////////////////////
// Port EtherType (in port table)/////////////
#define BR_PORT_ETHERTYPE_MASK			0xFFFF  // 16 bits
#define BR_PORT_ETHERTYPE_SHIFT_SIZE	8

//////////////////////////////////////////////
// Port EtherType utility macros

//set L2 Ethertype
#define BR_SET_PORT_ETHERTYPE(a, b)	((a) | (b << BR_PORT_ETHERTYPE_SHIFT_SIZE))

//unset L2 Ethertype
#define BR_UNSET_PORT_ETHERTYPE(a)	((a) & 0x3F)

//get L2 Ethertype
#define BR_GET_PORT_ETHERTYPE(a)	(((a) >> BR_PORT_ETHERTYPE_SHIFT_SIZE) & BR_PORT_ETHERTYPE_MASK )  

/////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////
// Filter/Forwarding Association (in forwarding table)///
#define BR_FORWARD_ENTRY_MASK			0x1	// one bit
#define BR_FORWARD_ENTRY_SHIFT_SIZE		5		// after 5 bits of port address

//////////////////////////////////////////////
//  Filter/Forward Associations
#define BR_FORWARD_ENTRY_PRESENT	0x1

//////////////////////////////////////////////
// Filter/Forward utility macros

//set Forwarding entry absent
#define BR_SET_FORWARD_ENTRY_ABSENT(a)		((a) & 0xffffffdf)

//set Forwarding entry present
#define BR_SET_FORWARD_ENTRY_PRESENT(a)		((a) | (BR_FORWARD_ENTRY_PRESENT << BR_FORWARD_ENTRY_SHIFT_SIZE))

#define BR_TEST_FORWARD_ENTRY(a)		(((a) >> BR_FORWARD_ENTRY_SHIFT_SIZE) & BR_FORWARD_ENTRY_MASK )  

#endif /* BR_FILTER_H */