/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/
/* br_tables.c
 *
 *
 * system: SA1200
 * subsystem: bridge
 * author: Rajendra Shukla 03/02/99
 * revisions:
 * $History: br_tables.cpp $
 * 
 * *****************  Version 42  *****************
 * User: Kalkunte     Date: 9/19/00    Time: 10:22a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * - Moved code to improve code stability when entries are overwritten. 
 * - Added comments to code.
 * - Changed error codes to be negative, not to be mistaken with other
 * valid values.
 * 
 * *****************  Version 41  *****************
 * User: Kalkunte     Date: 9/18/00    Time: 8:03p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Fixed bug that would not create a sequence of L2 tables for indexes
 * varying on the high order bits.
 * 
 * *****************  Version 40  *****************
 * User: Kalkunte     Date: 9/15/00    Time: 10:18p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * fixed bug causing filter info not updated in fwd table entry
 * 
 * *****************  Version 39  *****************
 * User: Kalkunte     Date: 9/05/00    Time: 1:33p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Adding 
 * - L2 filter and port configuration
 * - Software hash routines to avoid using hardware hash unit
 * 
 * *****************  Version 1  *****************
 * User: Kalkunte     Date: 8/31/00    Time: 5:32p
 * Created in $/users/kalkunte/bridge
 * Bridge with L2 filter
 * 
 * *****************  Version 39  *****************
 * User: kalkunte     Date: 8/24/00    Time: 9:30a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Added filter support
 * 
 * *****************  Version 38  *****************
 * User: Drjohns1     Date: 3/02/00    Time: 9:30a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Fixed runnning from console problem
 * 
 * *****************  Version 36  *****************
 * User: Drjohns1     Date: 2/11/00    Time: 1:49p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * 
 * *****************  Version 35  *****************
 * User: Drjohns1     Date: 2/08/00    Time: 10:31a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * 
 * *****************  Version 34  *****************
 * User: Drjohns1     Date: 1/20/00    Time: 5:04p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Renamed DEFAULT_PORT1 & DEFAULT_PORT2
 * 
 * *****************  Version 33  *****************
 * User: Drjohns1     Date: 1/19/00    Time: 6:26p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * 
 * *****************  Version 32  *****************
 * User: Drjohns1     Date: 1/14/00    Time: 1:57p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Assigned destId's to uthreads
 * 
 * *****************  Version 30  *****************
 * User: Drjohns1     Date: 1/04/00    Time: 2:43p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Prepared for execution on hardware
 * 
 * *****************  Version 29  *****************
 * User: Drjohns1     Date: 12/17/99   Time: 12:05p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * Incorporated rtm and brm into netApp
 * 
 * *****************  Version 27  *****************
 * User: Dhooper      Date: 6/18/99    Time: 10:57a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * use hash execute if IOSTYLE==HARDWARE, workaround current AMBAIO hang
 * 
 * *****************  Version 26  *****************
 * User: Dhooper      Date: 6/16/99    Time: 12:34p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * hsv_Data48 is now hsv_Data
 * 
 * *****************  Version 25  *****************
 * User: Dhooper      Date: 5/21/99    Time: 11:23a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * SDRAM_BRIDGE_FWD_BASE
 * 
 * *****************  Version 26  *****************
 * User: Dhooper      Date: 5/21/99    Time: 10:56a
 * Updated in $/BL5_Plus/SA1_CoreLibs/bridge
 * SDRAM_FWD_BASE  is now SDRAM_BRIDGE_FWD_BASE
 * 
 * *****************  Version 24  *****************
 * User: Dhooper      Date: 5/14/99    Time: 9:44a
 * Updated in $/BL5_Plus/SA1_CoreLibs/bridge
 * move stuff from bridge.h to br_config.h
 * 
 * *****************  Version 23  *****************
 * User: Dhooper      Date: 5/14/99    Time: 9:24a
 * Updated in $/BL5_Plus/SA1_CoreLibs/bridge
 * use SRAM, SDRAM_Attach, inline Br_Init
 * 
 * *****************  Version 22  *****************
 * User: Dhooper      Date: 5/10/99    Time: 12:35p
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * sram, sdram init function call
 * 
 * *****************  Version 21  *****************
 * User: Dhooper      Date: 5/07/99    Time: 11:34a
 * Updated in $/dev_1200/SA1_CoreLibs/bridge
 * use latest IOSTYLE convention
 * 
 * *****************  Version 20  *****************
 * User: Dhooper      Date: 4/15/99    Time: 5:14p
 * Updated in $/development/bridge
 * sawap args to load hash multiplier
 * 
 * *****************  Version 19  *****************
 * User: Dhooper      Date: 4/15/99    Time: 3:32p
 * Updated in $/development/bridge
 * br_sv_init function call from console
 * 
 * *****************  Version 18  *****************
 * User: Rshukla1     Date: 4/13/99    Time: 9:35p
 * Updated in $/development/bridge
 * 
 * *****************  Version 17  *****************
 * User: Dhooper      Date: 4/13/99    Time: 3:01p
 * Updated in $/development/bridge
 * fix some bugs
 * 1 adding fwd base to tmp to get sdram address
 * swapped da1 and da2 args to lookup
 * while ret value
 * L2 table used for l2index in add was 0, use next table
 * precedence of + over &
 * 
 * *****************  Version 16  *****************
 * User: Rshukla1     Date: 4/12/99    Time: 5:27p
 * Updated in $/development/bridge
 * 
 * *****************  Version 15  *****************
 * User: Dhooper      Date: 4/09/99    Time: 11:25a
 * Updated in $/development/bridge
 * L1 sdram address*2
 * 
 * *****************  Version 14  *****************
 * User: Rshukla1     Date: 4/09/99    Time: 9:49a
 * Updated in $/development/bridge
 * 
 * *****************  Version 12  *****************
 * User: Rshukla1     Date: 4/08/99    Time: 9:45a
 * Updated in $/development/bridge
 * 
 * *****************  Version 11  *****************
 * User: Rshukla1     Date: 4/06/99    Time: 9:06a
 * Updated in $/development/bridge
 * 
 * *****************  Version 10  *****************
 * User: Rshukla1     Date: 4/02/99    Time: 10:06a
 * Updated in $/development/bridge
 * Library added
 * 
 * *****************  Version 1  *****************
 * User: Rshukla1     Date: 3/03/99    Time: 6:43p
 * Updated in $/development/bridge
 * yet not stable. Do not try to run
 */
#include "br_config.h"
#include "brm.h"
#ifdef WIN32
#include <winsock.h>
#else
#include <sys/socket.h>
#endif // WIN32
#include "config_1200.h"
#include "xact.h"
#include "Xact_API.h"
#include "uclo.h"
#include "mem_map.h"
#include "hal_sdram.h"
#include "hal_sram.h"
#include "version.h"
#include "ambaio.h"
#include "hashsv.h"
#include "libd.h"

// pointers to sram and sdram
static SramUnit* sram;
static SdramUnit* sdram;


struct Br_Table_S Br_Table;
struct LibHandle* Br_libHandle = NULL; // global libd handle
struct Br_L2_Info_S Br_L2_Info;
void *Br_objHandle = NULL;		// micro code object handle

#define BR_L2_NO_ALLOC 0
#define BR_L2_ALLOC 1
#define BR_L2_LAST_ALLOC 2

//////////////////////////////////////////////
// Local function prototypes
unsigned int Br_RtAddL2(unsigned int hash_remain, unsigned int port_num, unsigned int l2_table,               
						unsigned int *l2_lookup_ret_ptr, bool filter, 
						unsigned int *hash_remain_prev, bool sa_filter, bool filter_action);
int Br_LookupL1(unsigned int index_val, unsigned int *ret_ptr, unsigned long *ret_address);
int Br_LookupL2(unsigned int index_val, unsigned int *ret_ptr, unsigned long *ret_address);

#ifdef WIN32
#include <winsock.h>
#else
#include <sys/types.h>
#endif

#include "internal-hash.h"

ix_hash48              *h48;

/*-----------------------------------------------------------------
	br_init:
	Initialize data structures for Bridge. Call initialize routine 
    for other routines like spanning tree and other funtions.
	returns: BR_SUCCESS or BR_FAIL
  -----------------------------------------------------------------*/
int Br_Init() 
{
#if (IOSTYLE==HARDWARE)						// if running on real SA1200 hardware
    int i;
#endif
    unsigned int baseaddr;
    int libdInitFlag =0 ;
    unsigned int val[1];
	ix_48bit	hash_mult;

	// get sram and sdram pointers
	sram = SRAM_Attach();
	sdram = SDRAM_Attach();


    D_PRINT("--------------Entering Function:br_init---------------\n");

    // these address hash defined are from include file mem_map.h
    Br_Table.L1 = baseaddr = SRAM_L1_ADDR_HASH_BASE; //64K
    D_PRINT1("SRAM_L1_ADDR_HASH_BASE =%d\n",SRAM_L1_ADDR_HASH_BASE);
    D_PRINT1("Br_Table.L1 set=%d\n",Br_Table.L1);

#if (IOSTYLE!=HARDWARE)
    // initalize the XactIo
    XactIoInit();

	// if no hardware, zero this buffer space quickly through XactIo
    if (XactIo_ZeroBuffer(SRAM, BR_SRAM_L1_SIZE, baseaddr) != XACT_SUCCESS) {
	    fprintf(stderr,"fatal error, unable to initialize L1 Hash Table table in SRAM\n");
	    return(BR_ERROR);
	}
#else
	for (i=0; i < BR_SRAM_L1_SIZE ; i++) {
		sram->write(baseaddr+i, 0);
	}
#endif      // (IOSTYLE!=HARDWARE)

    Br_Table.L2 = baseaddr = SRAM_L2_ADDR_HASH_BASE;
    Br_L2_Info.next_free = 0;
    Br_L2_Info.chunk_remain = BR_SRAM_L2_SIZE -L2_ENTRY_SIZE;
    D_PRINT1("Br_Table.L2 set=%d\n",Br_Table.L2);

#if (IOSTYLE!=HARDWARE)
	// if no hardware, zero this buffer space quickly through XactIo
    if (XactIo_ZeroBuffer(SRAM, BR_SRAM_L2_SIZE, baseaddr) != XACT_SUCCESS) {
	    fprintf(stderr,"fatal error, unable to initialize L2 Hash Table table in SRAM\n");
	    return(BR_ERROR);
	}
#else
	for (i=0; i < BR_SRAM_L2_SIZE ; i++) {
		sram->write(baseaddr+i, 0);
	}
#endif      // (IOSTYLE!=HARDWARE)
    
    Br_Table.port = baseaddr = SRAM_PORT_STATE_BASE; 
    D_PRINT1("Br_Table.port set=%d\n",Br_Table.port);

#if (IOSTYLE!=HARDWARE)
	// if no hardware, zero this buffer space quickly through XactIo
    if (XactIo_ZeroBuffer(SRAM, BR_SRAM_PORT_SIZE, baseaddr) != XACT_SUCCESS) {
	    fprintf(stderr,"fatal error, unable to initialize L2 Hash Table table in SRAM\n");
	    return(BR_ERROR);
	}
#else
	for (i=0; i < BR_SRAM_PORT_SIZE ; i++) {
		sram->write(baseaddr+i, 0);
	}
#endif      // (IOSTYLE!=HARDWARE)

    Br_Table.frwd = baseaddr = SDRAM_BRIDGE_FWD_BASE;
    Br_Table.next_frwd = SDRAM_BRIDGE_FWD_BASE;

    D_PRINT1("Br_Table.frwd set=%d\n",Br_Table.frwd);
#if (IOSTYLE!=HARDWARE)
	// if no hardware, zero this buffer space quickly through XactIo
    if (XactIo_ZeroBuffer(SDRAM, BR_SDRAM_FRWD_SIZE, baseaddr) != XACT_SUCCESS) {
	    fprintf(stderr,"fatal error, unable to initialize L2 Hash Table table in SRAM\n");
	    return(BR_ERROR);
	}
#else
	for (i=0; i < BR_SDRAM_FRWD_SIZE ; i++) {
		sdram->write((baseaddr+i)*2, 0);
	}
#endif      // (IOSTYLE!=HARDWARE)

    // setup default rcv_port 0 and default rcv_port 1
    // first two entries are default ports -- rcv_port0
	// defaults through DEFAULT_PORTA, all other rcv_ports
	// default through DEFAULT_PORTB
	//
	if(DEFAULT_PORTA == DEFAULT_PORTB){
	    fprintf(stderr,"fatal error, can't set defaults ports -- they are identical\n");
		return(BR_ERROR);
	}
    sdram->read(Br_Table.next_frwd*2, val);
    *val = BR_CHANGE_PORT(*val, DEFAULT_PORTA);
    sdram->write(Br_Table.next_frwd*2, *val);
#ifdef BR_DEBUG
    sdram->read(Br_Table.next_frwd*2, val);
    D_PRINT1("DEFAULT PORT1=%d\n", *val);
#endif // BR_DEBUG
    Br_Table.next_frwd = Br_Table.next_frwd+BR_FRWD_ENTRY_SIZE;

    sdram->read(Br_Table.next_frwd*2, val);
    *val = BR_CHANGE_PORT(*val, DEFAULT_PORTB);
    sdram->write(Br_Table.next_frwd*2, *val);
#ifdef BR_DEBUG
    sdram->read(Br_Table.next_frwd*2, val);
    D_PRINT1("DEFAULT PORT2=%d\n", *val);
#endif // BR_DEBUG
    Br_Table.next_frwd = Br_Table.next_frwd+BR_FRWD_ENTRY_SIZE;

	// Hash multiplier initialization
    (void) ix_hash48_new(&h48);

	hash_mult = BR_HASH_LOAD_HI;
	hash_mult = hash_mult << 32 | BR_HASH_LOAD_LO;

	ix_hash48_set_mult(h48, hash_mult);

    return(BR_SUCCESS);
}


/*-----------------------------------------------------------------
	br_load_uof_file:
	Initialize drivers, load uof file containing service routine call info,
	and loads hash multiplier.
   -----------------------------------------------------------------*/
int Br_SvInit()
{

    // Bind all the symbols from microcode if there is any need

    //dj: don't need to do this -- done by HashSv_Init
	//LibD_Init(Br_libHandle,0x34);

	// get handle to Uof...the UOF should have been loaded by now
	if(!Br_objHandle){
		// UOF may have been loaded by the workbench
		SysConf *sysConf = SysConf_Attach();
		if(!sysConf || (!(Br_objHandle = sysConf->getUcloHandle()))){
			printf("ERROR: HashSv_Init() failed.. UOF not loaded\n");
			return(BR_ERROR);
		}
	}

    // initialize the hash service routine
	if((Br_libHandle = HashSv_Init(Br_objHandle, 0x34)) == 0){
		printf("ERROR: HashSv_Init() failed\n");
        return(BR_ERROR);
    }
    else {
        D_PRINT0("HashSv_Init() Success\n");
    }
    
    Br_libHandle->tid = BR_THREAD_ID;
    Br_libHandle->flag = SVLIB_PEND_CALL; // two way pended

	/* dj: hash multiplier is init'ed via CSR
     int retval;
   // Function loaded to hash multiplier
    hsv_Data Br_hsv_DataInArg = {BR_HASH_LOAD2,BR_HASH_LOAD1};
    retval = HashSv_LdMult48(Br_libHandle, &Br_hsv_DataInArg);
    
    if ((retval >= 0) && (retval <= 7))
        D_PRINT0("SUCCESSSUCCESS: HashSv_LdMult48()\n");
    else
        D_PRINT0("ERROR: HashSv_LdMult48() failed\n");
	*/

    return(BR_SUCCESS);
}

/*-----------------------------------------------------------------
    Br_AddEntry
	Adds a forwarding or filter entry in to the Bridge hash table
    input:
    -Destination Address (48 bit MAC address)
	-Filter/Forwarding entry qualifier
	-If filter, SA or DA filter entry qualifier
	-If filter, action - permit deny
    -port number (set to zero when a filter table entry needs to be added)
    Output: port number for that perticulr entry
    Channel number/ forwarding index
  -----------------------------------------------------------------*/
int Br_AddEntry(unsigned int da2,
             unsigned short da1,
			 bool filter,
			 bool sa_filter,
			 bool filter_action,
             int port_num)
{
    struct Br_MAC_Addr mac_da;
    unsigned int l1_lookup_ret_ptr, 
        l2_add_ret,
        hash_remain =0,
        hash_remain_curr = 0,
        index=0,
        tmp,
        port_num_curr = 0;
    int ret=0xFFFF;
    struct Br_L2_Info_S   *tmp1;
	struct Br_Table_S *tmp2;
    tmp1 = &Br_L2_Info;
	tmp2 = &Br_Table; 
	ix_48bit	to_hash = 0l;
	ix_48bit	from_hash = 0l;


    D_PRINT("--------------Entering Function:Br_RtAdd---------------\n");

	// Store filter configuration to be stored in SDRAM
	if (filter) {
		if (filter_action == BR_FILTER_ACTION_DENY) {
			if (sa_filter == TRUE) {	
				port_num = BR_SET_FILTER_SA_DENY(port_num);
			}
			else {
				port_num = BR_SET_FILTER_DA_DENY(port_num);
			}
		}
		if (filter_action == BR_FILTER_ACTION_PERMIT) {
			if (sa_filter == TRUE) {	
				port_num = BR_SET_FILTER_SA_PERMIT(port_num);
			}
			else {
				port_num = BR_SET_FILTER_DA_PERMIT(port_num);
			}
		}
	}

	to_hash = da2 >> 16;
	to_hash = ((da2 << 16) | da1) | (to_hash << 32);

	// Hash compute
	(void) ix_hash48_run_wide(h48, &from_hash, to_hash);

	mac_da.B0123 = from_hash >> 16;
	mac_da.B45 = from_hash & 0xffff;

	index = mac_da.B45;
	hash_remain = mac_da.B0123;

	ret=0xFFFF;
    //Using first 16 bits <15:0> lookup in L1 table
	ret = Br_LookupL1(index, &l1_lookup_ret_ptr, NULL);
    D_PRINT1("ret value from L1 lookup = %d\n",ret);
    switch(ret){
        case BR_NO_ENTRY : //there is no L1 table entry or Lock bit is set
            if (l1_lookup_ret_ptr && (l1_lookup_ret_ptr & 0x40000000)){ //lock bit is set 
                //return failure with lock bit set and let the upper layer recover it
                D_PRINT0("LOCK_SET\n");
                return(BR_LOCK_ERROR);
            }
            else { // add an entry in table
                // add forwarding entry and hash remainder in forwarding Table
                D_PRINT0("Adding new entry to the table\n");

				unsigned int sram_entry = 0;

				if (filter == TRUE) {
					sram_entry = BR_SET_F_BIT(Br_Table.next_frwd);
					port_num = BR_SET_FORWARD_ENTRY_ABSENT(port_num);
				}
				else {
					port_num = BR_SET_FORWARD_ENTRY_PRESENT(port_num);
					sram_entry = Br_Table.next_frwd;
				}
				sdram->write((Br_Table.next_frwd & 0xDFFFFFFF)*2, port_num);

                D_PRINT1("Writing Port and Hash info to SDRAM, location %d",Br_Table.next_frwd);
                // add hash remainder to the forwarding table
                sdram->write(((Br_Table.next_frwd& 0xDFFFFFFF)+2)*2, hash_remain); //first 32 bits

                // an entry in L1 table with C (collision bit <31>) = 0
                sram->write((Br_Table.L1 + index), sram_entry);
                D_PRINT("Br_Table.next_frwd=0x%0x\nBR_SHIFT_MASK=0x%0x\n(Br_Table.next_frwd << BR_SHIFT_MASK_SIZE)=0x%0x\nBR_L2_INC=0x%0x\n",Br_Table.next_frwd,BR_SHIFT_MASK_SIZE,(Br_Table.next_frwd << BR_SHIFT_MASK_SIZE),BR_L2_INC);
                D_PRINT("BR_UNSET_C_BIT( ((Br_Table.next_frwd) << BR_SHIFT_MASK_SIZE) |BR_L2_INC)= 0x%0x\nBr_Table_Ptr.L2 + index =0x%0x\n ",BR_UNSET_C_BIT( ((Br_Table.next_frwd) << BR_SHIFT_MASK_SIZE) | BR_L2_INC),Br_Table.L2 + index);
                // increment the free pointer
                Br_Table.next_frwd = Br_Table.next_frwd+BR_FRWD_ENTRY_SIZE;
                return(BR_SUCCESS);
            }
            break;

        case BR_ENTRY_COL: // got the entry but collision bit is set so go 
            // and check the next Entry and proceed with entry in L2 table
		{
            //overwite the entry or add the entry in L2 table
            D_PRINT0("Colision bit is set so add the entry to l2 table\n");
            sram->read(Br_Table.L1+index, &tmp); // tmp hold the read value of index
            if(Br_RtAddL2(hash_remain, port_num, BR_EXTRACT_FWR_INDEX(tmp),&l2_add_ret, filter, 
				NULL, sa_filter, filter_action) == BR_ERROR) {
                D_PRINT0("ERROR in adding entry in L2 Table\n");
                return(BR_ERROR);
            }
            else {
                return(BR_SUCCESS);
            }
            break;
		}
        case BR_ENTRY_NO_COL: // Entry found and collision bit is not set
		{
            // it means this entry contains a forwarding address for another
            // hash table. We have to lock this entry and go further for adding 
            // both entries
            // first set lock bit 
            // set collision bit to 1

            D_PRINT0("Found entry but collision bit is not set\nTwo entries need to be added\n");
            if(Br_L2_Info.chunk_remain < L2_ENTRY_SIZE ) {
                fprintf(stderr,"BRM: Fatal Error: L2 table size overflow, allocate more memory\n");
                fprintf(stderr,"BRM: New Entry can not be added\n");
                return(BR_ERROR);
            }

			// Check if a duplicate entry is requested to be added to the table.
            sram->read(Br_Table.L1+index, &tmp);
            sdram->read(((tmp & 0xDFFFFFFF)+2)*2,&hash_remain_curr);

			// Check if this is an entry that already exists
			if (hash_remain_curr == hash_remain) {
				unsigned int frwd_entry = 0;
				bool action_type = TRUE;

				// Update existing entry with filter/forwarding information
		        sdram->read((tmp & 0xDFFFFFFF)*2,&frwd_entry);
				if (filter == TRUE) {
					if (!BR_TEST_F_BIT(tmp)) {	// Assuming SRAM entry exists
						// If this entry has filter information
						tmp = BR_SET_F_BIT(tmp);
						sram->write(Br_Table.L1+index, tmp);
					}
					// Read SDRAM entry to check if the filter action corresponds to what is
					// is being configured. If same, return BR_DUPLICATE
					if (sa_filter == TRUE) {
						action_type = BR_TEST_FILTER_ACTION_SA(frwd_entry);
						if (action_type == filter_action )
							return BR_DUPLICATE;
						if (filter_action == BR_FILTER_ACTION_DENY) {
							frwd_entry = BR_SET_FILTER_SA_DENY(frwd_entry);
						}
						else {
							frwd_entry = BR_SET_FILTER_SA_PERMIT(frwd_entry);
						}
					}
					else {
						action_type = BR_TEST_FILTER_ACTION_DA(frwd_entry);			
						if (action_type == filter_action )
							return BR_DUPLICATE;
						if (filter_action == BR_FILTER_ACTION_DENY) {
							frwd_entry = BR_SET_FILTER_DA_DENY(frwd_entry);
						}
						else {
							frwd_entry = BR_SET_FILTER_DA_PERMIT(frwd_entry);
						}
					}
					//Write updated SDRAM entry
					sdram->write((tmp & 0xDFFFFFFF)*2, frwd_entry);	// 0xDFFFFFFF to mask F bit
					return BR_SUCCESS;
				}
				else {	// Updating with fowarding entry
					if (BR_TEST_F_BIT(tmp) && !BR_TEST_FORWARD_ENTRY(frwd_entry)) {	
						// This entry does not have forwarding information associated with it
						// i.e the entry has primarily been a filter only
		
						frwd_entry |= port_num;
						frwd_entry = BR_SET_FORWARD_ENTRY_PRESENT(frwd_entry);
		
						//Write updated SDRAM entry
						sdram->write((tmp & 0xDFFFFFFF)*2, frwd_entry);	// 0xDFFFFFFF to mask F bit
						return BR_SUCCESS;
					}
					else
						return BR_DUPLICATE;
				}
			}

            // New entry to be added to l2 table(s)
			int tmp_next_l2_entry = Br_L2_Info.next_free;
            if (Br_RtAddL2(hash_remain, port_num,tmp_next_l2_entry, &l2_add_ret, filter, 
				&hash_remain_curr, sa_filter, filter_action) != BR_ERROR) {
                // L2 table  is added but now we have to remove the forwarding entry and 
                // add the index to L2 table
                sram->read(Br_Table.L1+index, &tmp);
                
                tmp = BR_UNSET_C_BIT(BR_UNSET_C_BIT(tmp));
                // change the hash remain in that entry
                // last four bits need to be removed
                sdram->write(((tmp & 0xDFFFFFFF)+2)*2, (hash_remain_curr >> L2_SHIFT));

                // forwarding entry transfer from L1 to L2
                // L2 table start + next free index + last four bit from hash remain
                sram->write(Br_Table.L2+ l2_add_ret + (hash_remain_curr & BR_MASK_L2),tmp);
                
                // Now add new L2 index in L1
                tmp = BR_SET_C_BIT(BR_UNSET_LOCK_BIT((tmp_next_l2_entry<<BR_SHIFT_MASK_SIZE ) | BR_L2_INC));
                sram->write(Br_Table.L1+index,tmp );

	            // increment the free pointer
		        Br_Table.next_frwd = Br_Table.next_frwd+BR_FRWD_ENTRY_SIZE;
                return (BR_SUCCESS);
            }
            else {
                D_PRINT0("ERROR in adding entry in L2 Table\n");
                return(BR_ERROR);
            }
            break;
		}
        default:
            D_PRINT0("ERROR in adding entry in L2 Table\n");
            return(BR_ERROR);
            break;
    }
}


/*-----------------------------------------------------------------
    Br_RtAddL2
	Add an entry into a L2 table.
    input:
    -Remaining index (after resolving L1 table)
    -port number (set to zero when a filter table entry needs to be added)
	-L2 table
	-Forwarding entry
	-Filter/Forwarding entry qualifier
    Output: port number for that perticulr entry
	-Add status
  -----------------------------------------------------------------*/

unsigned int Br_RtAddL2(
               unsigned int hash_remain, 
               unsigned int port_num,
               unsigned int l2_table,
               unsigned int *l2_lookup_ret_ptr,
			   bool filter,
			   unsigned int *hash_remain_prev,
		       bool sa_filter,
			   bool filter_action
               )
{
    unsigned int ret_val=0,
        index = 0,
        l2_add_ret,
        hash_remain_curr = 0,
        tmp,
        port_num_curr = 0;
    int ret=0xFFFF;
    struct Br_L2_Info_S   *tmp1;
	struct Br_Table_S *tmp2;
    tmp1 = &Br_L2_Info;
	tmp2 = &Br_Table; 
	unsigned int index_prev = 0;
	int tmp_hash_remain = 0;
   
   D_PRINT("--------------Entering Function:Br_RtAddL2---------------\n");

    // index last four bits and L2 pointer
    index = (hash_remain & L2_LookupMask) + l2_table; 
	if (hash_remain_prev) {
	    index_prev = (*hash_remain_prev & L2_LookupMask) + l2_table; 
		tmp_hash_remain = *hash_remain_prev;
		*hash_remain_prev = *hash_remain_prev >> L2_SHIFT;
	}

    hash_remain = hash_remain >> L2_SHIFT;

    ret = Br_LookupL2(index, l2_lookup_ret_ptr, NULL);
    
    switch(ret){
        case BR_NO_ENTRY : //there is no L2 table entry or Lock bit is set
            if (*l2_lookup_ret_ptr & 0x4000000){ //lock bit is set 
                //return failure with lock bit set and let the upper layer recover it
                //return(BR_LOCK_ERROR);
                return(BR_ERROR); // return ERROR
            }
            else { // add an entry in table

				// If lower order bits are similar, create more L2 tables until indexes are unique
				if (hash_remain_prev && index == index_prev) {
		            Br_L2_Info.chunk_remain = Br_L2_Info.chunk_remain-L2_ENTRY_SIZE;
			        Br_L2_Info.next_free = Br_L2_Info.next_free + L2_ENTRY_SIZE;
					int tmp_next_l2_entry = Br_L2_Info.next_free;
					// entry have to added to the next l2 table
				    if (Br_RtAddL2(hash_remain, port_num,tmp_next_l2_entry, l2_lookup_ret_ptr, filter, 
						hash_remain_prev, sa_filter, filter_action) != BR_ERROR) {
		                // Now add new L2 index in L1
				        tmp = BR_SET_C_BIT(BR_UNSET_LOCK_BIT((tmp_next_l2_entry<<BR_SHIFT_MASK_SIZE ) | BR_L2_INC));
						sram->write(Br_Table.L2+index,tmp );
					}
					else
						return BR_ERROR;
				}
				else {	// Previous entry and current entry are now unique.
					int sram_entry;
		            // add forwarding entry and hash remainder in forwarding Table
			        D_PRINT0("Adding new entry to the table\n");
					if (filter == TRUE) {
						sram_entry = BR_SET_F_BIT(Br_Table.next_frwd);
						port_num = BR_SET_FORWARD_ENTRY_ABSENT(port_num);
					}
					else {
						port_num = BR_SET_FORWARD_ENTRY_PRESENT(port_num);
						sram_entry = Br_Table.next_frwd;
					}

	                sdram->write((sram_entry & 0xDFFFFFFF)*2, port_num);
	                D_PRINT1("Writing Port and Hash info to SDRAM, location %d",Br_Table.next_frwd);
		            // add hash remainder to the forwarding table
			        sdram->write(((sram_entry & 0xDFFFFFFF)+2)*2, hash_remain); //first 32 bits

				    // a forwaring entry in L2 table with C (collision bit <31>) = 0
					sram->write((Br_Table.L2 + index), BR_UNSET_C_BIT(sram_entry));
	                D_PRINT ("BR_UNSET_C_BIT( ((sram_entry) << BR_SHIFT_MASK_SIZE) & BR_L2_INC)= %d\nBr_Table_Ptr.L2 + index =%d ",BR_UNSET_C_BIT( ((Br_Table.next_frwd) << BR_SHIFT_MASK_SIZE) | BR_L2_INC),Br_Table.L2 + index);

	                // increment L2 free pointer
		            Br_L2_Info.chunk_remain = Br_L2_Info.chunk_remain-L2_ENTRY_SIZE;
			        Br_L2_Info.next_free = Br_L2_Info.next_free + L2_ENTRY_SIZE;
					*l2_lookup_ret_ptr = l2_table;
					if (hash_remain_prev)
						*hash_remain_prev = tmp_hash_remain;
		            // increment the free pointer
			        Br_Table.next_frwd = Br_Table.next_frwd+BR_FRWD_ENTRY_SIZE;
				}
                return(BR_SUCCESS);
            }
            break;

        case BR_ENTRY_COL: // got the entry but collision bit is set so go 
            // and check the next Entry and proceed with entry in L2 table
            // get the index and pass it back
        {
            //overwite the entry or add the entry in L2 table
            D_PRINT0("Colision bit is set so add the entry to l2 table\n");
            sram->read(Br_Table.L2+index, &tmp); // tmp hold the read value of index

            if(Br_RtAddL2(hash_remain, port_num, BR_EXTRACT_FWR_INDEX(tmp),&l2_add_ret, filter, NULL, sa_filter, filter_action) == BR_ERROR) {
                D_PRINT0("ERROR in adding entry in L2 Table\n");
                return(BR_ERROR);
            }
            else {
				*l2_lookup_ret_ptr = l2_add_ret;			
                return (BR_SUCCESS);
            }
            break;
		}
        case BR_ENTRY_NO_COL: // Entry found and collision bit is not set
            // it means this entry contains a forwarding address for another
            // hash table. We have to lock this entry and go further for adding 
            // both entries
            // first set lock bit 
            // set collision bit to 1
            D_PRINT0("Found entry but collision bit is not set\nTwo entries need to be added\n");
            if(Br_L2_Info.chunk_remain < L2_ENTRY_SIZE ) {
                fprintf(stderr,"BRM: Fatal Error: L2 table size overflow, allocate more memory\n");
                fprintf(stderr,"BRM: New Entry can not be added\n");
                return(BR_ERROR);
            }
            else {
				unsigned int sram_entry;

                // remove the forwarding index entry and add index to L2 table 

                // get the present hash remainder
                // get next free frw table entry and change the modified remainder there
                // copy the present L2 entry to new L2 entry
                // add the next frwd table entry and its index in L2

                // L2 table  is added but now we have to remove the forwarding entry and 
                // add the index to L2 table

                // get the current frwd index
                sram->read(Br_Table.L2+index, &sram_entry);

                sram_entry = BR_UNSET_C_BIT(BR_UNSET_C_BIT(sram_entry));
                // frw table start + index from L2 table + 2
                sdram->read(((sram_entry & 0xDFFFFFFF)+2)*2,&hash_remain_curr);

				// Check if this is an entry that already exists
				if (hash_remain_curr == hash_remain) {
					bool action_type = TRUE;
					unsigned int frwd_entry = 0;
	
			        sdram->read((sram_entry & 0xDFFFFFFF)*2,&frwd_entry);
					if (filter == TRUE) {
						if (!BR_TEST_F_BIT(sram_entry)) {	// Assuming SRAM entry exists
							// If this entry has filter information
							sram_entry = BR_SET_F_BIT(sram_entry);
							sram->write(Br_Table.L2+index, sram_entry);
						}
						// Read SDRAM entry to check if the filter action corresponds to what is
						// is being configured. If same, return BR_DUPLICATE
						if (sa_filter == TRUE) {
							action_type = BR_TEST_FILTER_ACTION_SA(frwd_entry);
							if (action_type == filter_action )
								return BR_DUPLICATE;
							if (filter_action == BR_FILTER_ACTION_DENY) {
								frwd_entry = BR_SET_FILTER_SA_DENY(frwd_entry);
							}
							else {
								frwd_entry = BR_SET_FILTER_SA_PERMIT(frwd_entry);
							}
						}
						else {
							action_type = BR_TEST_FILTER_ACTION_DA(frwd_entry);			
							if (action_type == filter_action )
								return BR_DUPLICATE;
							if (filter_action == BR_FILTER_ACTION_DENY) {
								frwd_entry = BR_SET_FILTER_DA_DENY(frwd_entry);
							}
							else {
								frwd_entry = BR_SET_FILTER_DA_PERMIT(frwd_entry);
							}
						}
						sdram->write((sram_entry & 0xDFFFFFFF)*2, frwd_entry);	// 0xDFFFFFFF to mask F bit
						return BR_SUCCESS;
					}
					else {	// Updating existing entry with fowarding information
						if (BR_TEST_F_BIT(sram_entry) && !BR_TEST_FORWARD_ENTRY(frwd_entry)) {	
							// This entry does not have forwarding information associated with it
							// i.e the entry has primarily been a filter only
			
							frwd_entry |= port_num;
							frwd_entry = BR_SET_FORWARD_ENTRY_PRESENT(frwd_entry);
			
							//Write updated SDRAM entry
							sdram->write((sram_entry & 0xDFFFFFFF)*2, frwd_entry);	// 0xDFFFFFFF to mask F bit
							return BR_SUCCESS;
						}
						else
							return BR_DUPLICATE;
					}
				}
                // change the hash remain in that entry
                // last four bits need to be removed
                sdram->write(((sram_entry & 0xDFFFFFFF)+2)*2, (hash_remain_curr >> L2_SHIFT) );

                // forwarding entry transfer from L2-n to L2-n+1
                sram->write(Br_Table.L2+Br_L2_Info.next_free+(hash_remain_curr & BR_MASK_L2),sram_entry);

                // add next entry
                sdram->write((Br_Table.next_frwd & 0xDFFFFFFF)*2, port_num);
                D_PRINT1("Writing Port and Hash info to SDRAM, location %d",Br_Table.next_frwd);
                // add hash remainder to the forwarding table
                sdram->write(((Br_Table.next_frwd & 0xDFFFFFFF)+2)*2, hash_remain); //first 32 bits

                sram->write(Br_Table.L2+Br_L2_Info.next_free+(hash_remain & BR_MASK_L2),sram_entry);

                // increment the free pointer
                Br_Table.next_frwd = Br_Table.next_frwd+BR_FRWD_ENTRY_SIZE;

                // add the index in L2
                sram_entry = BR_SET_C_BIT(BR_UNSET_LOCK_BIT((Br_L2_Info.next_free<<BR_SHIFT_MASK_SIZE ) | BR_L2_INC));
                sram->write(Br_Table.L2+index, sram_entry);

                Br_L2_Info.chunk_remain = Br_L2_Info.chunk_remain-L2_ENTRY_SIZE;
                Br_L2_Info.next_free = Br_L2_Info.next_free + L2_ENTRY_SIZE;
                return (BR_SUCCESS);
            }
            break;

        default:
            D_PRINT0("ERROR in adding entry in L2 Table\n");
            return(BR_ERROR);
            break;
    }

}


/*-----------------------------------------------------------------
    Br_LookupEntry
	Lookup an entry in Hash table.
    input:
	-Destination MAC address
    -Receive port number
    Output: port number for that perticulr entry
	-Port #
	-sram address/entry and sram entry associated with the lookup
  -----------------------------------------------------------------*/
int Br_LookupEntry(
                unsigned int  da2,    // upper 32 bits of destination address
                unsigned short da1,   // next 16 bits of destination address
                int rcv_port, // port on which this packet was recived
				unsigned int *sram_address,
				unsigned int *sram_entry,
				unsigned int *sdram_entry
                ) 
{
    struct Br_MAC_Addr mac_da;
    unsigned int index;
    unsigned int val;
    int ret ;
    int i =0;
    
    struct Br_L2_Info_S   *tmp1;
	struct Br_Table_S *tmp2;
    tmp1 = &Br_L2_Info;
	tmp2 = &Br_Table; 
	unsigned long ret_address;
	ix_48bit	to_hash = 0l;
	ix_48bit	from_hash = 0l;


    D_PRINT("--------------Entering Function:Br_RtLookup---------------\n");
//    mac_da.B45 = (da1);
//    mac_da.B0123 = (da2);

	to_hash = da2 >> 16;
	to_hash = ((da2 << 16) | da1) | (to_hash << 32);

    // Hash encode the MAC address first
//    if(Br_HashEncode(&mac_da) != BR_SUCCESS) return(BR_ERROR);

	(void) ix_hash48_run_wide(h48, &from_hash, to_hash);

	mac_da.B0123 = from_hash >> 16;
	mac_da.B45 = from_hash & 0xffff;

    // Hash encode the MAC address first
//    Br_HashEncode(&mac_da);
/*
 *  Select 16 bits of low order of Hashed Argument 
 *  and use them as Hash index into Hash Table L1. 
 */
    ret = Br_LookupL1(mac_da.B45, &index, &ret_address);

	if (sram_entry)
		*sram_entry = 0;
	if (sram_address)
		*sram_address = 0;
	if (sdram_entry)
		*sdram_entry = 0;

    switch(ret){
    case BR_ENTRY_NO_COL: //got the forwarding entry
        // goto forwarding table in SDRAM
        // get the port number
        // return it
		if (sram_entry)
			*sram_entry = index;
		if (sram_address)
			*sram_address = ret_address;

		index = index & 0xDFFFFFFF;	// loose the filter 29th bit 

		sdram->read(index*2, &val);
		if (sdram_entry) {
			*sdram_entry = val;
		}

       return(BR_GET_PORT(val));
    case BR_ENTRY_COL: // got to go to L2 table
        // use four bits of hash index to go to next level of L2 table
        i=0;
        while(index){
			ret = Br_LookupL2(((mac_da.B0123 >> i) & BR_MASK_L2) + 
				BR_EXTRACT_FWR_INDEX(index), &index, &ret_address );

            i = i+ L2_SHIFT; // increment shift index
            switch(ret){
            case BR_ENTRY_NO_COL:
                // got the forwarding entry 
                // goto forwarding table in SDRAM
                // get the port number
                // return it
				if (sram_entry)
					*sram_entry = index;
				if (sram_address)
					*sram_address = ret_address;

                sdram->read((index & 0xDFFFFFFF)*2, &val); // 0xDFFFFFFF to loose the filter (29th) bit
				
				if (sdram_entry) {
					*sdram_entry = val;
				}
                return(BR_GET_PORT(val)); 

            case BR_ENTRY_COL: // have to go to another level of lookup in L2
                continue;
				break;
            case BR_NO_ENTRY: // no entry in table
            default:
                return(BR_NO_ENTRY);            
            }
        }
    case BR_NO_ENTRY: // no entry in table
    default:
		return(BR_NO_ENTRY);            
	} 
}


int Br_HashEncode(struct Br_MAC_Addr *mac_addr)
{
    D_PRINT("--------------Entering Function:Br_HashEncode---------------\n");

	// BUG IOSTYLE AMBAIO, HashSv_Exec48 hangs
#if (IOSTYLE == HARDWARE)
	hsv_Data inArg;
	hsv_Data outArg;


	inArg.lo32 = mac_addr->B0123 >>16;
	inArg.hi32 = (mac_addr->B0123 <<16) | mac_addr->B45;

	if(HashSv_Exec48(Br_libHandle, &inArg, &outArg) != HASHSV_SUCCESS) return(BR_ERROR);;

    mac_addr->B0123 = (outArg.lo32 <<16) | (outArg.hi32 >>16);
	mac_addr->B45 = (unsigned short)outArg.hi32 & 0xffff;
#endif
    return(BR_SUCCESS);
}

int Br_LookupL1(unsigned int index_val, 
                unsigned int *ret_ptr,
				unsigned long *ret_address
                )
{
    struct Br_L2_Info_S   *tmp1;
	struct Br_Table_S *tmp2;
    tmp1 = &Br_L2_Info;
	tmp2 = &Br_Table; 

	if (ret_address) *ret_address = 0;
    D_PRINT("--------------Entering Function:Br_LookupL1---------------\n");
    sram->read(Br_Table.L1+index_val, ret_ptr);
     // no entry or 30 bit is set (entry is getting added)
    if((*ret_ptr == 0) || (*ret_ptr & 0x4000000))
        return(BR_NO_ENTRY); // return default entry
    else if (*ret_ptr & 0x80000000)
        return(BR_ENTRY_COL); // goto next lookup collision bit is set
    else {
		if (ret_address) *ret_address = Br_Table.L1+index_val;
        return(BR_ENTRY_NO_COL); // got a forwarding address
	}
}

int Br_LookupL2( 
                unsigned int index_val, 
                unsigned int *ret_ptr,
				unsigned long *ret_address
                )
{
    struct Br_L2_Info_S   *tmp1;
	struct Br_Table_S *tmp2;
    tmp1 = &Br_L2_Info;
	tmp2 = &Br_Table; 

	if (ret_address) *ret_address = 0;
    D_PRINT("--------------Entering Function:Br_LookupL2---------------\n");
    sram->read(Br_Table.L2+index_val, ret_ptr);
     // no enntry or 30 bit is set means this entry is getting added
    if((*ret_ptr == 0) || (*ret_ptr & 0x4000000))
        return(BR_NO_ENTRY); // return default entry
    else if (*ret_ptr & 0x80000000)
        return(BR_ENTRY_COL); // goto next lookup collision bit is set
    else {
		if (ret_address) *ret_address = Br_Table.L2+index_val;
        return(BR_ENTRY_NO_COL); // got a forwarding address
	}
}

int Br_Cleanup(){
    // clean up
    D_PRINT("--------------Entering Function:Br_Cleanup---------------\n");
     HashSv_Close(Br_libHandle);
     UcLo_DeleObj(Br_objHandle);
     return(BR_SUCCESS);
}


int Br_RtDel() 
{
    D_PRINT("--------------Entering Function:Br_RtDel---------------\n");
    return(BR_SUCCESS);
}

int BR_Allocate_New_L2_Chunk()
{
    D_PRINT("--------------Entering Function:BR_Allocate_New_L2_Chunk---------------\n");
    //allocates new chunk of memory from a free list and return
    // BR_SUCCESS if able to allocate memory required
    return(BR_ERROR);
}


/*-----------------------------------------------------------------
    Br_SetPortProtoFilter
	Adds a L2 protocol Ethertype filter on each port
    input:
	-Ethertype
    -port number
    Output:
  -----------------------------------------------------------------*/
void Br_SetPortProtoFilter(unsigned long etherType, int port_num)
{
	unsigned int port_entry =0;

	sram->read(Br_Table.port+(port_num << BR_PORT_ENTRY_MULTIPLER), &port_entry); // tmp hold the read value of index

	port_entry = BR_SET_PORT_ETHERTYPE(port_entry, etherType);

	sram->write(Br_Table.port+(port_num << BR_PORT_ENTRY_MULTIPLER), port_entry);

}

/*-----------------------------------------------------------------
    Br_GetPortProtoFilter
	Adds a L2 protocol Ethertype filter on each port
    input:
    -port number
    Output:
	-Ethertype
  -----------------------------------------------------------------*/
unsigned long Br_GetPortProtoFilter(int port_num)
{
	unsigned int port_entry =0;
	unsigned int ret_ethertype = 0;

	sram->read(Br_Table.port+(port_num << BR_PORT_ENTRY_MULTIPLER), &port_entry); // tmp hold the read value of index

	ret_ethertype = BR_GET_PORT_ETHERTYPE(port_entry);

	return ret_ethertype;
}

/*-----------------------------------------------------------------
    Br_SetPortFilterType
	Set L2 filter type - none, explicit rule, postive or negative on each port
    input:
	-filter type
		Ethertype
		Explict rule
		Positive
		Negative
    -port number
    Output:
  -----------------------------------------------------------------*/
void Br_SetPortFilterType(unsigned short filterType, int port_num)
{
	unsigned int port_entry =0;

	sram->read(Br_Table.port+(port_num << BR_PORT_ENTRY_MULTIPLER), &port_entry); // tmp hold the read value of index

	switch (filterType) {
	case BR_PORT_FILT_TYPE_ETHERTYPE:
		port_entry = BR_SET_PORT_FILT_TYPE_ETHERTYPE(port_entry);
		break;
	case BR_PORT_FILT_TYPE_EXPLICT:
		port_entry = BR_SET_PORT_FILT_TYPE_EXPLICIT(port_entry);
		break;
	case BR_PORT_FILT_TYPE_POSITIVE:
		port_entry = BR_SET_PORT_FILT_TYPE_POSITIVE(port_entry);
		break;
	case BR_PORT_FILT_TYPE_NEGATIVE:
		port_entry = BR_SET_PORT_FILT_TYPE_NEGATIVE(port_entry);
		break;
	};

	sram->write(Br_Table.port+(port_num << BR_PORT_ENTRY_MULTIPLER), port_entry);

}

/*-----------------------------------------------------------------
    Br_GetPortFilterType
	Set L2 filter type - none, explicit rule, postive or negative on each port
    input:
    -port number
    Output:
	-filter type
		None
		Explict rule
		Positive
		Negative
  -----------------------------------------------------------------*/
unsigned short Br_GetPortFilterType(int port_num)
{
	unsigned int port_entry =0;
	unsigned int ret_filtertype = 0;

	sram->read(Br_Table.port+(port_num << BR_PORT_ENTRY_MULTIPLER), &port_entry); // tmp hold the read value of index

	ret_filtertype = BR_GET_PORT_FILTERTYPE(port_entry);

	return ret_filtertype;

}

//			  +-------+-------+------+------+-------+
// port entry |unused | ether |unused|filter| port  |
//			  |       |  type |      | type | state |
//			  | 31:24 | 23:8  | 7:6  |  5:4 |  3:0  |
//			  +-------+-------+------+------+-------+
//            |			  MAC	Address				|
//			  +-------+-------+------+------+-------+
//									 | MAC Address  |
//			  						 +--------------+
//
// Each entry in the port table has the Device MAC address stored even though it is the same
// value. The reasoning for this decision are as follows:
//		- If MAC address was not stored in each entry, two separate SRAM requests would have
//			to be issued by the micro engines.
//		- If MAC address was stored in the GPRs, using two GPRs permanently is a valuable 
//			resource.
/*-----------------------------------------------------------------
    Br_SetPortMACAddr
	Set MAC address for the given port
    input:
	-MAC address
    Output:
  -----------------------------------------------------------------*/
void Br_SetPortMACAddr(unsigned int da2, unsigned short da1)
{
	unsigned int port_entry1 = 0;
	unsigned int port_entry2 = 0;
	int port_count;

	for (port_count = 0; port_count < BR_NUM_PORTS; port_count++)
	{
		sram->read(Br_Table.port+(port_count << BR_PORT_ENTRY_MULTIPLER)+1, &port_entry1); // tmp hold the read value of index
		sram->read(Br_Table.port+(port_count << BR_PORT_ENTRY_MULTIPLER)+2, &port_entry2);

		port_entry1 = da2;	//0123 bytes
		port_entry2 = da1;	//45 bytes (LSB)

		sram->write(Br_Table.port+(port_count << BR_PORT_ENTRY_MULTIPLER)+1, port_entry1);
		sram->write(Br_Table.port+(port_count << BR_PORT_ENTRY_MULTIPLER)+2, port_entry2);
	}
}

/*-----------------------------------------------------------------
    Br_GetPortMACAddr
	Set MAC address for the given port
    input:
    -port number
    Output:
	-MAC address
  -----------------------------------------------------------------*/
int Br_CheckPortMACAddr(unsigned int da2, unsigned short da1)
{
	unsigned int port_entry1 = 0;
	unsigned int port_entry2 = 0;

	// Read the first entry. All other entries have the same MAC address
	sram->read(Br_Table.port+1, &port_entry1);
	sram->read(Br_Table.port+2, &port_entry2);

	if (da2 == port_entry1 && da1 == port_entry2)
		return 1;
	else 
		return -1;

}
