/*--------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *
 *--------------------------------------------------------------------------*/
/*
 * Library for handling interrupts for drivers in an OS-independent way
 */
/*
 * Edit History
 *
 * $Log: /dev_1200/SA1_CoreLibs/core_utils/intlib.h $
 * 
 * 1     5/24/99 3:53p Jdguilfo
 * Initial version, untested
 * 
*/


#ifndef INTLIB_H
#define INTLIB_H

#include "ostype.h"

typedef enum {
    SAINT_reserved_00   = 00,
    SAINT_SI            = 01,   /* Soft interrupt */
    SAINT_reserved_02   = 02,
    SAINT_reserved_03   = 03,

    SAINT_T1            = 04,   /* Timer 1 */
    SAINT_T2            = 05,   /* Timer 2 */
    SAINT_T3            = 06,   /* Timer 3 */
    SAINT_T4            = 07,   /* Timer 4 */

    SAINT_reserved_08   = 08,
    SAINT_CINT          = 09,   /* CINT pin IRQ */
    SAINT_UENG          = 10,   /* Microengine IRQ */
    SAINT_SRAM          = 11,   /* SRAM unit IRQ */

    SAINT_UART          = 12,   /* UART unit IRQ */
    SAINT_SDRAM         = 13,   /* SDRAM unit IRQ */
    SAINT_RTC           = 14,   /* Real Time Clock */
    SAINT_DFH           = 15,   /* Doorbell from host */

    SAINT_DMA1          = 16,   /* DMA channel 1 */
    SAINT_DMA2          = 17,   /* DMA channel 2 */
    SAINT_PIL           = 18,   /* pci_irq_l */
    SAINT_reserved_19   = 19,

    SAINT_DMA1NB        = 20,   /* DMA1 not busy */
    SAINT_DMA2NB        = 21,   /* DMA2 not busy */
    SAINT_SB            = 22,   /* Start BIST */
    SAINT_RSERR         = 23,   /* Received SERR */

    SAINT_SDPAR         = 24,   /* SDRAM parity */
    SAINT_IIP           = 25,   /* I20 inbound post_list */
    SAINT_reserved_26   = 26,
    SAINT_DTE           = 27,   /* Discard Timer Expired */

    SAINT_DPED          = 28,   /* Data Parity Error Detected */
    SAINT_RMA           = 29,   /* Received Master Abort */
    SAINT_RTA           = 30,   /* Received Target Abort */
    SAINT_DPE           = 31    /* Detect Parity Error */
} SA1200_INTS;

typedef void (*ISR_FUNC)(int);

/*
int int_connect(SA1200_INTS vector, ISR_FUNC isr, int param);
  return 0 on success, !=0 on error

int int_enable(SA1200_INTS vector);
  return 0 on success, !=0 on error

int int_disable(SA1200_INTS vector);
  return 0 on success, !=0 on error

int int_lock();
  Locks out interrupts, returns previous status

void int_unlock(int prev_val);
  Sets interrupt lock status to previous value

 */

#if (OS == VXWORKS)
#include "config.h"

#define int_connect(vector, isr, param) \
    intConnect(INUM_TO_IVEC(vector), (isr), (param))

int  int_lock(void);
void int_unlock(int prev_val);

#define int_enable(vector)  intEnable(vector)
#define int_disable(vector) intDisable(vector)

#elif (OS == UCOS)
#include "interrupts.h"

#define int_connect(vector, isr, param) \
    (SA_IRQ_Connect((vector), (isr), (param)).func != (PFVI)-1)

#define int_enable(vector)  SA_IRQ_Enable(vector)
#define int_disable(vector) SA_IRQ_Disable(vector)

#define int_lock()           SAr_DisableInt()
#define int_unlock(prev_val) SAr_EnableInt(prev_val)

#endif /* OS type */

#endif /* INTLIB_H */

