/* control.cpp
 *
 *------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: Debug Library
 * author: Henry Qian 03/16/99
 * revisions:
 *
 */
#include "cntl.h"
#include "uclo.h"
#include "debug_1200.h"
#include "config_1200.h"



#ifdef INC_FDRIVER
#include "FDriverAPI.h"
#endif // INC_FDRIVER

static SysConf *systemConfig;



// Flags to indicate which uEngine driver(s) have been started
//
unsigned int uEngPause = 0;


// load microcode from a buffer
int CNTL_InitUof(char* buf, int buflen)
{
#ifdef _DBG_PRINT
	printf("cntl_inituof\n");
#endif

	systemConfig = SysConf_Attach();

    return(systemConfig->UCLoadBuffer((unsigned int *)buf, (unsigned int)buflen));
}


// reset fbus devices and microengines
// tbd add portMask to this API
int CNTL_Reset(uDbg_Handle_T uDbg_Handle, unsigned int engineMask,unsigned int portMask, int clrReg)
{
	int retval;	
#ifdef _DBG_PRINT
	printf("CNTL_Reset\n");
#endif

	systemConfig = SysConf_Attach();

	if (retval = systemConfig->Reset(engineMask, ALL_PORTS_MASK)){
#ifdef _DBG_PRINT
		printf("Error: systemConfig->Reset, error code: %d\n", retval); 
#endif
		systemConfig->GiveControl();
		return(retval);
	}	
	systemConfig->GiveControl();
	return (uDbg_ResetUeng(uDbg_Handle, engineMask, clrReg));

}


// reset fbus devices and microengines
// tbd add portMask to this API
int CNTL_Start(uDbg_Handle_T uDbg_Handle, unsigned int engineMask,unsigned int portMask)
{
	int retval;
	systemConfig = SysConf_Attach();
#ifdef _DBG_PRINT
	printf("CNTL_Start\n");
#endif

	if (retval = systemConfig->Configure()){
#ifdef _DBG_PRINT
		printf("Error: systemConfig->Configure, error code: %d\n", retval); 
#endif
		systemConfig->GiveControl();
		return(retval);
	}
	retval = systemConfig->Start(engineMask, ALL_PORTS_MASK);
#ifdef _DBG_PRINT
	printf("CNTL_Start: giving control\n");
#endif
	systemConfig->GiveControl();

	return(retval);

}
    


int CNTL_Pause(uDbg_Handle_T uDbg_Handle, unsigned int engineMask, unsigned int portMask,unsigned int delay)
{
    int retval;
	systemConfig = SysConf_Attach();
#ifdef _DBG_PRINT
	printf("CNTL_Pause: before takeControl\n");
#endif
	systemConfig->TakeControl();
#ifdef _DBG_PRINT
	printf("CNTL_Pause: have control\n");
#endif
	systemConfig->StopPorts(portMask, RX);

#ifdef INC_FDRIVER
    for (int engineNum = 0; engineNum <= 5; engineNum++)
    {
		// Make sure the engine is not already paused
		//
        if (((1 << engineNum) & engineMask) &&
			((uEngPause & (1 << engineNum)) == 0))
		{
            FDriver_HoldTxToEngine(engineNum);
			uEngPause |= (1 << engineNum);
		}
    }
#endif // INC_FDRIVER

	if (engineMask != 0){
		retval = uDbg_PauseUeng(uDbg_Handle, engineMask, delay);
		if (retval != UDBG_SUCCESS){
#ifdef _DBG_PRINT
			printf("Error, uDbg_PauseUeng, error code: %d\n", retval);
#endif
			return(retval);
		}
	}
#ifdef _DBG_PRINT
			printf("return from CNTL_PAUSE\n");
#endif


    return(CNTL_SUCCESS);
}



int CNTL_Resume(uDbg_Handle_T uDbg_Handle, unsigned int engineMask,unsigned int portMask)
{
    int retval;
	systemConfig = SysConf_Attach();

	systemConfig->Start(0, portMask);
#ifdef _DBG_PRINT
	printf("CNTL_Resume\n");
#endif


#ifdef INC_FDRIVER
    for (int engineNum = 0; engineNum <= 5; engineNum++)
    {
		// Make sure the engine is not already paused
		//
        if (((1 << engineNum) & engineMask) &&
			((uEngPause & (1 << engineNum)) != 0))
		{
            FDriver_ResumeTxToEngine(engineNum);
			uEngPause &= ~(1 << engineNum);
		}
    }
#endif // INC_FDRIVER

    retval = uDbg_ResumeUeng(uDbg_Handle, engineMask);
    if (retval != UDBG_SUCCESS){
#ifdef _DBG_PRINT
		printf("Error, uDbg_ResumeUeng, error code: %d\n", retval);
#endif
		systemConfig->GiveControl();
		return(retval);
	}
#ifdef _DBG_PRINT
printf("CNTL_Resume: giving control\n");
#endif
	systemConfig->GiveControl();

    return(CNTL_SUCCESS);
}



int CNTL_Hop(uDbg_Handle_T uDbg_Handle, unsigned int *engineMask, unsigned int portMask,unsigned int delay)
{
    int retval;
	systemConfig = SysConf_Attach();

	systemConfig->Start(0, portMask);

#ifdef _DBG_PRINT
	printf("CNTL_Reset\n");
#endif

#ifdef INC_FDRIVER
    for (int engineNum = 0; engineNum <= 5; engineNum++)
    {
		// Make sure the engine is not already paused
		//
        if (((1 << engineNum) & *engineMask) &&
			((uEngPause & (1 << engineNum)) != 0))
		{
            FDriver_ResumeTxToEngine(engineNum);
			uEngPause &= ~(1 << engineNum);
		}
    }
#endif // INC_FDRIVER

	systemConfig->GiveControl();

    retval = uDbg_HopUeng(uDbg_Handle, engineMask, delay);
    if (retval != UDBG_SUCCESS){
#ifdef _DBG_PRINT
		printf("uDbg_HopUeng failed, failing uengine mask: %x, error code: %d\n", *engineMask, retval);
#endif
		return(retval);
	}

	systemConfig->TakeControl();

	systemConfig->StopPorts(portMask, RX);


#ifdef INC_FDRIVER
    for (engineNum = 0; engineNum <= 5; engineNum++)
    {
		// Make sure the engine is not already paused
		//
        if (((1 << engineNum) & *engineMask) &&
			((uEngPause & (1 << engineNum)) == 0))
		{
            FDriver_HoldTxToEngine(engineNum);
			uEngPause |= (1 << engineNum);
		}
    }
#endif // INC_FDRIVER


    return(CNTL_SUCCESS);
}




// end of file
