/* debug_1200.hxx
 * Ucode Debug Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: drj, June 16,98  Initial version
 *  revisions:
 *  $History: debug_1200.hxx $
 * 
 * ---------------------------------------------------------------------
 */
#ifndef __DEBUG_1200_HXX
#define __DEBUG_1200_HXX

#include "udbg_ver.h"
#include "utils.h"
#include "udbg_umm.h"
#include "debug_1200.h"

class Debug{
public:
   static ValidObject(Debug *obj){return(obj != NULL && obj == TheDebugObj);}
   static Debug *GetObject(void){return(TheDebugObj);}
   Debug(uDbg_uStoreFree_T *uStoreFree);
   ~Debug(void);
   int DefineFreeUstore(unsigned int uEngMask, unsigned int freeUaddr, unsigned int freeSize);
   void UpdateFreeUstore(unsigned int uEngMask);

   int ResetUeng(unsigned int uEngMask, int clrReg=0);
   int ClrResetUeng(unsigned int uEngMask);
   int StartUeng(unsigned int uEngMask);
   inline int PauseUeng(unsigned int uEngMask, unsigned int delay){return(pauseUthreads(uEngMask, UDBG_CTXALL, delay));}
   inline int ResumeUeng(unsigned int uEngMask){return(resumeUthreads(uEngMask, UDBG_CTXALL));}

   int GetUthreadPC(unsigned char uEng, unsigned char ctx, unsigned int *upc);
   int SetUthreadsPC(unsigned int uEngMask, unsigned int ctxMask, unsigned int upc);
   int SetUengPCs(unsigned char uEng, unsigned int *upc);
   int GetUengPCs(unsigned char uEng, unsigned int *upc);

   int IsUengActive(unsigned int uEngMask);
   unsigned int GetActiveUengs(unsigned int uEngMask);
   int GetUword(unsigned char uEng, unsigned int uWordAddr, unsigned int *uWord);
   int SetUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned int uWord);
   int GetNumUword(unsigned char uEng, unsigned int uWordAddr, unsigned short numWords,unsigned int *uWord);
   int SetNumUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned short numWords, unsigned int *uWord);
   void GetCSR(unsigned int csrAddr, unsigned int *csrData){READ_LWORD(csrAddr, csrData);}
   void SetCSR(unsigned int csrAddr, unsigned int csrData){WRITE_LWORD(csrAddr, csrData);}
   int SetBreakpoint(uDbg_Bkpt_T *breakpoint);
   int ShowBreakpoints(unsigned int uEngMask, uDbg_BkptArray_T *breakpoints);
   int ClearAllBreakpoints(unsigned int uEngMask);
   int ClearBreakpoint(short breakPointId);
   inline void lockDbg(){Sem_Wait(debugSem, SEM_WAITFOREVER);}
   inline void unlockDbg(){Sem_Post(debugSem);}
   inline int numOfBkPts(){return(numBkPts);}
   int WriteMem32(unsigned int addr, unsigned int *data, unsigned int count);
   int ReadMem32(unsigned int addr, unsigned int *data, unsigned int count);
   int WriteMem64(unsigned int addr, unsigned int *data, unsigned int count);
   int ReadMem64(unsigned int addr, unsigned int *data,unsigned int count);
   int CheckBreak();

   int PutRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned int count);

   int GetRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned int count);

   int GetRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,
				  unsigned int count);
   int PutRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,
				  unsigned int count);

   int GetRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,
				  unsigned int count);
   int PutRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,
				  unsigned int count);

   int GetRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,
				  unsigned int count);
   int PutRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,
				  unsigned int count);

   int PutRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,
				  unsigned int count);
   int GetRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			      unsigned int regNum, unsigned int *regData,
				  unsigned int count);

   int WriteFIFO_Tx(unsigned char uEng, unsigned int fifoAddr,
				  unsigned int *regData, unsigned int count, int setValid);
   int ReadFIFO_Rx(unsigned char uEng, unsigned int fifoAddr,
				  unsigned int *regData, unsigned int count);

   int GetUengStatus(unsigned char uEng, uDbg_uEngStatus_T *uEngStatus);
   int HopUeng(unsigned int *uEngMask, unsigned int delay);
   int GetSysMemSize(unsigned int* sys, unsigned int* sys_all, unsigned int* sdram,
				     unsigned int* sram,unsigned int* flash);

   void regDump(unsigned int uEngMask);
   void ClrBkPtThreadId(void){bkPtThreadId = 0;}

private:
   static Debug *TheDebugObj;  /* only a single object is allowed at a time*/
   int valid;
   unsigned int numBkPts;
   Breakpoint_T bkPts[UDBG_MAX_BREAKPOINTS];
   thread_T bkPtThreadId;
   sem_T	debugSem;

   Uengine **uEngs;
   uStoreMM	*uStore[MAX_UENGS];
   int hopSegs[MAX_UENGS];
   unsigned int ctx0Pc[MAX_UENGS], ctx1Pc[MAX_UENGS], ctx2Pc[MAX_UENGS], ctx3Pc[MAX_UENGS];

   Breakpoint_T *getBkPt(uDbg_Bkpt_T *bkpoint);
   Breakpoint_T *findBkPtId(short breakPointId);
   Breakpoint_T *findBkPtUaddr(unsigned char ueng, unsigned int uAddr);
   Breakpoint_T *findBkPtUeng(unsigned char uEng);
   Breakpoint_T *findBkPtRtnAddr(unsigned char ueng, unsigned int uAddr);

   int setBreakpoint(Breakpoint_T *bkPoint);
   int isAtBreakpoint(Breakpoint_T *bkPoint);
   int removeBreak(Breakpoint_T *bkpt);
   int isUengBreak(unsigned char uEng);

   int pauseUthreads(unsigned int uEngMask, unsigned int ctxMask, unsigned int delay);
   int resumeUthreads(unsigned int uEngMask, unsigned int ctxMask);
   int disableUthreads(unsigned int uEngMask, unsigned int ctxMask);
   int enableUthreads(unsigned int uEngMask, unsigned int ctxMask);
};


#endif			/* __DEBUG_1200_HXX leave next blank line */
