/* udbg_clib.cpp
 * Ucode Debug Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: drj, Jan 11,99  Initial version
 *  revisions:
 *  $History: udbg_clib.cpp $
 * 
 * *****************  Version 27  *****************
 * User: Ewalker      Date: 12/04/00   Time: 2:12p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Enforce a single Debug object in the rs_udbg server.
 * 
 * *****************  Version 26  *****************
 * User: Ewalker      Date: 11/30/00   Time: 6:39p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Better protection against invalid object handle passed from workbench.
 * Also properly return errors for many get/put register functions when
 * microengines are active.
 * 
 * *****************  Version 25  *****************
 * User: Ewalker      Date: 11/28/00   Time: 10:42a
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed DefineFreeUstore to return BADARG when appropriate
 * 
 * *****************  Version 23  *****************
 * User: Ewalker      Date: 8/04/00    Time: 16:31
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Cleanup of comments, numBkPts, and function prototypes
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 1/07/00    Time: 4:38p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Add debug statments
 * 
 * *****************  Version 19  *****************
 * User: Drjohns1     Date: 10/18/99   Time: 11:04a
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Added function which returns the virtual memory regions.
 * 
 * *****************  Version 18  *****************
 * User: Drjohns1     Date: 9/22/99    Time: 12:54p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 1/19/99    Time: 3:59p
 * Created in $/development/udebug
 * Debug library "C" interface
 * 
 * 
 * ---------------------------------------------------------------------
 */
#include <stdio.h>
#include "debug_1200.hxx"


static const char __copyright[] =
"Copyright Intel Corp, 1998. All rights reserved";

/*-----------------------------------------------------------------------------
   Function: 	uDbg_InitLib
   Description: Creates and initialise a Debug library object.

   Returns:		Debug library object handle or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uDbg_Handle_T uDbg_InitLib(uDbg_uStoreFree_T *uStoreFree)
{
    Debug *dbgObj;

#ifdef _DBG_PRINT
	printf("uDbg_InitLib\n");
#endif

    dbgObj = new Debug(uStoreFree);

	return((uDbg_Handle_T) dbgObj);
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_UpdateFreeUstore
   Description: Update the micro-store free region with values from the 
				uengine-driver.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uDbg_UpdateFreeUstore(uDbg_Handle_T dbgH, unsigned int uEngMask)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_UpdateFreeUstore\n");
#endif
	if(Debug::ValidObject(dbgObj)) dbgObj->UpdateFreeUstore(uEngMask);
	return;
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_DefineFreeUstore
   Description: Define the micro-store free region for the specified micro-engines.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_DefineFreeUstore(uDbg_Handle_T dbgH, unsigned int uEngMask, 
						   unsigned int freeUaddr, unsigned int freeSize)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_DefineFreeUstore\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
    return(dbgObj->DefineFreeUstore(uEngMask, freeUaddr, freeSize));
}



/*-----------------------------------------------------------------------------
   Function: 	uDbg_DeleLib
   Description: Deletes a Debug library object.

   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uDbg_DeleLib(uDbg_Handle_T dbgH)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_DeleLib\n");
#endif
    if(Debug::ValidObject(dbgObj)) delete(dbgObj);
	return;
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_ResetUeng
   Description: Resets the micro-engine specified in uengMask.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ResetUeng(uDbg_Handle_T dbgH, unsigned int uEngMask, int clrReg)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ResetUeng\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ResetUeng(uEngMask, clrReg));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_ClrResetUeng
   Description: Resets the micro-engine specified in uengMask.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ClrResetUeng(uDbg_Handle_T dbgH, unsigned int uEngMask)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ClrResetUeng\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ClrResetUeng(uEngMask));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_StartUeng
   Description: Starts all threads of the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_StartUeng(uDbg_Handle_T dbgH, unsigned int uEngMask)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_StartUeng\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->StartUeng(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_PauseUeng
   Description: Pause the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_PauseUeng(uDbg_Handle_T dbgH, unsigned int uEngMask, unsigned int delay)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_PauseUeng\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->PauseUeng(uEngMask, delay));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_ResumeUeng
   Description: Resume the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ResumeUeng(uDbg_Handle_T dbgH, unsigned int uEngMask)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ResumeUeng\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ResumeUeng(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_IsUengActive
   Description: Determine if any of the selected micro-engines are active.

   Returns:		UDBG_UENGACT, UDBG_UENGINACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_IsUengActive(uDbg_Handle_T dbgH, unsigned int uEngMask)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_IsUengActive\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->IsUengActive(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetActiveUengs
   Description: Determine which of the specified micro-engines are active.

   Returns:		A mask of the active micro-engines
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
unsigned int uDbg_GetActiveUengs(uDbg_Handle_T dbgH, unsigned int uEngMask)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetActiveUengs\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetActiveUengs(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_SetUthreadsPC
   Description: Set the program-counter of the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_SetUthreadsPC(uDbg_Handle_T dbgH, unsigned int uEngMask, unsigned int ctxMask, 
					   unsigned int upc)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_SetUthreadsPC\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->SetUthreadsPC(uEngMask, ctxMask, upc));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetUthreadPC
   Description: Get the program-counters of the specified micro-engine.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetUthreadPC(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
					  unsigned int *upc)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetUthreadPC\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetUthreadPC(uEng, ctx, upc));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetUengPCs
   Description: Get the four program-counters of the specified micro-engine.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetUengPCs(uDbg_Handle_T dbgH, unsigned char uEng, unsigned int *upc)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetUengPCs\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetUengPCs(uEng, upc));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_SetUengPCs
   Description: Set the four program-counters of the specified micro-engine.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_SetUengPCs(uDbg_Handle_T dbgH, unsigned char uEng, unsigned int *upc)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_SetUengPCs\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->SetUengPCs(uEng, upc));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_SetUword
   Description: Write uWord to a micro-store address.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_SetUword(uDbg_Handle_T dbgH, unsigned int uEngMask, unsigned int uWordAddr, unsigned int uWord)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_SetUword\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->SetUword(uEngMask, uWordAddr, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetUword
   Description: Read uWord from a micro-store location.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetUword(uDbg_Handle_T dbgH, unsigned char uEng, unsigned int uWordAddr, unsigned int *uWord)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetUword\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetUword(uEng, uWordAddr, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_SetNumUword
   Description: Write a number of uWords to micro-store.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_SetNumUword(uDbg_Handle_T dbgH, unsigned int uEngMask, unsigned int uWordAddr,
					 unsigned short numWords, unsigned int *uWord)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_SetNumUword\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->SetNumUword(uEngMask, uWordAddr, numWords, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetNumUword
   Description: Read a number of uWords from micro-store.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetNumUword(uDbg_Handle_T dbgH, unsigned char uEng, unsigned int uWordAddr,
					 unsigned short numWords, unsigned int *uWord)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetNumUword\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetNumUword(uEng, uWordAddr, numWords, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_WriteMem32
   Description: Write a long-word to a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_WriteMem32(uDbg_Handle_T dbgH, unsigned int addr, unsigned int *data,
					unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_WriteMem32\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->WriteMem32(addr, data, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_ReadMem32
   Description: Read a long-word from a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ReadMem32(uDbg_Handle_T dbgH, unsigned int addr, unsigned int *data,
				   unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ReadMem32\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ReadMem32(addr, data, count));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_WriteMem64
   Description: Write a quad-word to a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_WriteMem64(uDbg_Handle_T dbgH, unsigned int addr, unsigned int *data,
					unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_WriteMem64\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->WriteMem64(addr, data, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_ReadMem64
   Description: Read a quad-word from a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ReadMem64(uDbg_Handle_T dbgH, unsigned int addr, unsigned int *data,
				   unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ReadMem64\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ReadMem64(addr, data, count));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_SetBreakpoint
   Description: Set breakpoint in the specified micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_NOUSTORE,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_SetBreakpoint(uDbg_Handle_T dbgH, uDbg_Bkpt_T *breakpoint)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_SetBreakpoint\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->SetBreakpoint(breakpoint));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_ShowBreakpoints
   Description: Show all breakpoints.

   Returns:		UDBG_SUCCESS,UDBG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ShowBreakpoints(uDbg_Handle_T dbgH, unsigned int uEngMask, uDbg_BkptArray_T *breakpoints)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ShowBreakpoints\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ShowBreakpoints(uEngMask, breakpoints));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_ClearBreakpoint
   Description: Remove the breakpoint associated with breakpointId.

   Returns:		UDBG_SUCCESS,UDBG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ClearBreakpoint(uDbg_Handle_T dbgH, short breakpointId)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ClearBreakpoint\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ClearBreakpoint(breakpointId));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_ClearAllBreakpoints
   Description: Remove all breakpoints in the specified micro-engines.

   Returns:		UDBG_SUCCESS,UDBG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ClearAllBreakpoints(uDbg_Handle_T dbgH, unsigned int uEngMask)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ClearAllBreakpoints\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ClearAllBreakpoints(uEngMask));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_SetCSR
   Description: Write a long-word to a CSR.

   Returns:		UDBG_SUCCESS,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_SetCSR(uDbg_Handle_T dbgH, unsigned int csr, unsigned int data)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_SetCSR\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	dbgObj->SetCSR(csr, data);
	return(UDBG_SUCCESS);

}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetCSR
   Description: Read a long-word from a CSR.

   Returns:		UDBG_SUCCESS,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetCSR(uDbg_Handle_T dbgH, unsigned int csr, unsigned int *data)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetCSR\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	dbgObj->GetCSR(csr, data);
	return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_PutRelGPR
   Description: Write a long-word to a relative GPR.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_PutRelGPR(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
				   uDbg_Bank_T bankType, unsigned int regNum,
				   unsigned int *regData, unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_PutRelGPR\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->PutRelGPR(uEng, ctx, bankType, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetRelGPR
   Description: Read a long-word from a relative GPR.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetRelGPR(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
				   uDbg_Bank_T bankType, unsigned int regNum,
				   unsigned int *regData, unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetRelGPR\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetRelGPR(uEng, ctx, bankType, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_PutRelSRAM_WrXfer
   Description: Write a long-word to a relative SRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_PutRelSRAM_WrXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
						   unsigned int regNum, unsigned int *regData,
						   unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_PutRelSRAM_WrXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->PutRelSRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetRelSRAM_WrXfer
   Description: Read a long-word from a relative SRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetRelSRAM_WrXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
						   unsigned int regNum, unsigned int *regData,
						   unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetRelSRAM_WrXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetRelSRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_PutRelSRAM_RdXfer
   Description: Write a long-word to a relative SRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_PutRelSRAM_RdXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
						   unsigned int regNum, unsigned int *regData,
						   unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_PutRelSRAM_RdXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->PutRelSRAM_RdXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetRelSRAM_RdXfer
   Description: Read a long-word from a relative SRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetRelSRAM_RdXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
						   unsigned int regNum, unsigned int *regData,
						   unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetRelSRAM_RdXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetRelSRAM_RdXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_PutRelSDRAM_WrXfer
   Description: Write long-word(s) to a relative SDRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_PutRelSDRAM_WrXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
							unsigned int regNum, unsigned int *regData,
							unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_PutRelSDRAM_WrXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->PutRelSDRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetRelSDRAM_WrXfer
   Description: Read long-word(s) from a relative SDRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetRelSDRAM_WrXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
							unsigned int regNum, unsigned int *regData,
							unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetRelSDRAM_WrXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetRelSDRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_PutRelSDRAM_RdXfer
   Description: Write long-word(s) to a relative SDRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_PutRelSDRAM_RdXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
							unsigned int regNum, unsigned int *regData,
							unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_PutRelSDRAM_RdXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->PutRelSDRAM_RdXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetRelSDRAM_RdXfer
   Description: Read long-word(s) from a relative SDRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetRelSDRAM_RdXfer(uDbg_Handle_T dbgH, unsigned char uEng, unsigned char ctx,
							unsigned int regNum, unsigned int *regData,
							unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetRelSDRAM_RdXfer\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetRelSDRAM_RdXfer(uEng, ctx, regNum, regData, count));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_WriteFIFO_Tx
   Description: Write long-word(s) to transmit-FIFO.
   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_WriteFIFO_Tx(uDbg_Handle_T dbgH, unsigned char uEng, unsigned int fifoAddr,
					unsigned int *data64, unsigned int count, int setValid)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_WriteFIFO_Tx\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->WriteFIFO_Tx(uEng, fifoAddr, data64, count, setValid));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_ReadFIFO_Rx
   Description: Read long-word(s) from receive-FIFO.
   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_ReadFIFO_Rx(uDbg_Handle_T dbgH, unsigned char uEng, unsigned int fifoAddr,
					unsigned int *data64, unsigned int count)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_ReadFIFO_Rx\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->ReadFIFO_Rx(uEng, fifoAddr, data64, count));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetUengStatus
   Description: Get the micro-engine status.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetUengStatus(uDbg_Handle_T dbgH, unsigned char uEng, uDbg_uEngStatus_T *uEngStatus)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetUengStatus\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->GetUengStatus(uEng, uEngStatus));
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetSysMemSize
   Description: This routine returns sizes of virtual memory regions in the following
				pointers:
					sys     -- Size of memory mapped at zero for OS
					sys_all -- Size of memory mapped at zero and usable by the OS
					sdram   -- Size of memory mapped at C0000000 and usable by HW
					sram    -- Size of sram
					flash   -- Size of Flash

   Returns:		UDBG_SUCCESS,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_GetSysMemSize(uDbg_Handle_T dbgH, unsigned int* sys, unsigned int* sys_all, unsigned int* sdram,
				     unsigned int* sram,unsigned int* flash)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_GetSysMemSize\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	dbgObj->GetSysMemSize(sys, sys_all, sdram, sram, flash);
	return(UDBG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	uDbg_HopUeng
   Description: This function allows the micro-engine to continue and then
				pause at the next context swap.  This function is similar to
				stepping, except the number of statements that gets executed
				is generally indeterminate - determine by the logical flow of
				the program and the time when the function is called. The value
				UDBG_TIMEOUT will be returned if the micro-engine fails to stop
				within 5 milli-seconds. The micro-engine must be inactive prior
				to calling this function -- UDBG_UENGACT will be returned if
				the micro-engine is active.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_UENGACT, 
				UDBG_NOUSTORE, UDBG_TIMEOUT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uDbg_HopUeng(uDbg_Handle_T dbgH, unsigned int *uEngMask, unsigned int delay)
{
	Debug *dbgObj = (Debug *)dbgH;
#ifdef _DBG_PRINT
	printf("uDbg_HopUeng\n");
#endif
	if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);
	return(dbgObj->HopUeng(uEngMask, delay));
}
