/* udbg_lib.cpp
 * Ucode Debug Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: drj, June 16,98  Initial version
 *  revisions:
 *  $History: udbg_lib.cpp $
 * 
 * *****************  Version 95  *****************
 * User: Ewalker      Date: 12/11/00   Time: 4:30p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed breakpoint thread cleanup
 * 
 * *****************  Version 94  *****************
 * User: Ewalker      Date: 12/11/00   Time: 10:49a
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed bug where only microengine zero would pause.
 * 
 * *****************  Version 93  *****************
 * User: Ewalker      Date: 12/04/00   Time: 2:12p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Enforce a single Debug object in the rs_udbg server.
 * 
 * *****************  Version 92  *****************
 * User: Ewalker      Date: 11/30/00   Time: 6:39p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Better protection against invalid object handle passed from workbench.
 * Also properly return errors for many get/put register functions when
 * microengines are active.
 * 
 * *****************  Version 91  *****************
 * User: Ewalker      Date: 11/28/00   Time: 10:43a
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed DefineFreeUstore to return BADARG when appropriate
 * 
 * *****************  Version 90  *****************
 * User: Ewalker      Date: 11/17/00   Time: 3:07p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Hardened free ustore address and size checks to better reject invalid
 * values passed by caller.
 * 
 * *****************  Version 89  *****************
 * User: Ewalker      Date: 11/14/00   Time: 4:20p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed bug where clearing the breakpoint that the thread is currently
 * paused at changes the PC of some other thread to the paused thread's
 * PC. The ueng->setPC required a context mask instead of a context
 * number.
 * 
 * *****************  Version 88  *****************
 * User: Ewalker      Date: 11/14/00   Time: 12:35p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Added some DBG_PRINT blocks to help breakpoint/hop debugging
 * 
 * *****************  Version 87  *****************
 * User: Ewalker      Date: 11/09/00   Time: 4:39p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed problem with GetUengStatus where PC for inactive or at-break
 * contexts being generated incorrectly
 * 
 * *****************  Version 86  *****************
 * User: Ewalker      Date: 11/09/00   Time: 10:52a
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed SCR/bug #1242 - uengine::start was not shifting the starting
 * context into bit positions 2:3 when writing the CTX_ARB_CNTL register,
 * so context 0 was always starting instead of the desired context.
 * 
 * *****************  Version 85  *****************
 * User: Drjohns1     Date: 11/08/00   Time: 5:31p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Retain breakpoints on restart
 * 
 * *****************  Version 84  *****************
 * User: Drjohns1     Date: 11/08/00   Time: 11:29a
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed breaking on two contexts
 * 
 * *****************  Version 83  *****************
 * User: Drjohns1     Date: 11/01/00   Time: 4:37p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed hop/breakpoint issues
 * 
 * *****************  Version 82  *****************
 * User: Ewalker      Date: 10/31/00   Time: 10:30a
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Changed DefineFreeUstore to support C0 by setting free ustore in either
 * segment 1 or segment 2 depending on the passed uaddr.
 * 
 * *****************  Version 81  *****************
 * User: Drjohns1     Date: 10/27/00   Time: 2:30p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Updated for C0 fixes
 * 
 * *****************  Version 80  *****************
 * User: Drjohns1     Date: 10/20/00   Time: 5:05p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Support for C0
 * 
 * *****************  Version 78  *****************
 * User: Ewalker      Date: 8/10/00    Time: 15:18
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed SetUthreadsPC to not ignore uEngMask
 * 
 * *****************  Version 77  *****************
 * User: Ewalker      Date: 8/04/00    Time: 16:26
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Cleanup of comments, numBkPts, and function prototypes
 * 
 * *****************  Version 76  *****************
 * User: Drjohns1     Date: 4/14/00    Time: 8:39p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed reading of RFIFO and writing of TFIFO bug
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 7/01/98    Time: 2:01p
 * Created in $/users/Johnson/SA1200/udebug
 * 
 * ---------------------------------------------------------------------
 */
#include <iostream.h>
#include "ostype.h"
#include "core_io.h"
#include "debug_1200.hxx"


static const char __copyright[] =
"Copyright Intel Corp, 1998. All rights reserved";

#define BITS_IN_WORD 32
static const unsigned int clrMasks[BITS_IN_WORD] ={
	0xfffffffe, 0xfffffffd, 0xfffffffb, 0xfffffff7,
	0xffffffef, 0xffffffdf, 0xffffffbf, 0xffffff7f,
	0xfffffeff, 0xfffffdff, 0xfffffbff, 0xfffff7ff,
	0xffffefff, 0xffffdfff, 0xffffbfff, 0xffff7fff,
	0xfffeffff, 0xfffdffff, 0xfffbffff, 0xfff7ffff,
	0xffefffff, 0xffdfffff, 0xffbfffff, 0xff7fffff,
	0xfeffffff, 0xfdffffff, 0xfbffffff, 0xf7ffffff,
	0xefffffff, 0xdfffffff, 0xbfffffff, 0x7fffffff};

static const unsigned int setMasks[BITS_IN_WORD] ={
	0x00000001, 0x00000002, 0x00000004, 0x00000008,
	0x00000010, 0x00000020, 0x00000040, 0x00000080,
	0x00000100, 0x00000200, 0x00000400, 0x00000800,
	0x00001000, 0x00002000, 0x00004000, 0x00008000,
	0x00010000, 0x00020000, 0x00040000, 0x00080000,
	0x00100000, 0x00200000, 0x00400000, 0x00800000,
	0x01000000, 0x02000000, 0x04000000, 0x08000000,
	0x10000000, 0x20000000, 0x40000000, 0x80000000};

#define SETBIT(value, bit) (setMasks[bit] | value)
#define CLRBIT(value, bit) (clrMasks[bit] & value)

#define VERIFY_LIB() if(!valid) return(UDBG_BADLIB);
#define VERIFY_UENGMASK(uengMask) if(uengMask == 0 || uengMask >= (1 << MAX_UENGS)) return(UDBG_BADARG)
#define VERIFY_UENG(ueng) if(ueng < 0 || ueng >= MAX_UENGS) return(UDBG_BADARG)
#define VERIFY_CTXMASK(ctxMask) if(ctxMask == 0 || ctxMask >= (1 << MAX_CTX)) return(UDBG_BADARG)
#define VERIFY_CTX(ctx) if(ctx < 0 || ctx >= MAX_CTX) return(UDBG_BADARG)

enum {UDBG_BKPT_CLR = 0, UDBG_BKPT_SET, UDBG_BKPT_MET};
static const unsigned int NUM_BKPT_INST = 14;
static const unsigned int BR_CTX0 = 0xF0000007, BR_CTX1 = 0xF0800007;
static const unsigned int BR_CTX2 = 0xF1000007, BR_CTX3 = 0xF1800007, BR_NOT_CTX = 0xF2000007;
static const unsigned int DISABLE_CTX = 0xE0100508, CTX_ARB = 0xE00C0104;
static const unsigned int NOP = 0xDE000000, FAST_WR = 0xE1003C02, BR = 0xF8000007, CTX_ARB_KILL = 0xE00C0004;
static const unsigned int Udbg_intMask = 0x1000000;
static const unsigned char CTX0 = 0, CTX1 = 1, CTX2 = 2, CTX3 = 3;
static const int NOUSTORE = -1, ANY_SEG = 0;

const unsigned int BKPT_COND0 = 0, BKPT_COND1 = 1, BKPT_INST_ORIG = 10, BKPT_INST_BR = 13;
const unsigned int BKPT_UPC_DETECT = 8;

static sem_T BkptSem = 0;

Debug* Debug::TheDebugObj = NULL;  // enforces only a single Debug object instance

#if (OS == UCOS)
#define BKPT_THRD_PRIORITY 61
#else
#define BKPT_THRD_PRIORITY LOW_PRIORITY
#endif

/*-----------------------------------------------------------------------------
   Function: 	bkPtHandler
   Description: Check is breakpoints has been reached.
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void bkPtHandler(int param)
{
   // clear bkpt interrupt first, then check after to avoid sync prob
   uEng_clrIregBp();

   // post bpkt semaphore
   if (BkptSem != 0) Sem_Post(BkptSem);

   return;
}

/*-----------------------------------------------------------------------------
   Function: 	bkPtThread
   Description: Check if breakpoints has been reached.
				Note:- This function deals with only one instance of the debug object
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void bkPtThread(void *param)
{
   Debug *dbgObjPtr = (Debug *)param;

   while(1){
       if (BkptSem == 0) break;
	   Sem_Wait(BkptSem, SEM_WAITFOREVER);
       if (!Debug::ValidObject(dbgObjPtr)) break;
       dbgObjPtr->CheckBreak();
   }

   if (Debug::ValidObject(dbgObjPtr)) dbgObjPtr->ClrBkPtThreadId();

   return;  // should never get here
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::Debug
   Description: Debug class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Debug::Debug(uDbg_uStoreFree_T *uStoreFree)
{
    unsigned int i;
    const int usrData = 0;

    /* delete the previous debug object if user forgot since only one is allowed at a time */
    if (TheDebugObj != NULL) delete TheDebugObj;

    TheDebugObj = this;

    valid = 0;  // mark debug object invalid

    // init new uengs or get ref to old uengs
    uEngs = initUengs();
    if (uEngs == NULL) return;  // error

    // initialize the free ustore regions
    if(uStoreFree){
        for(i = 0; i < MAX_UENGS; i++){
            uStore[i] = new uStoreMM(uStoreFree->freeUpc[i], uStoreFree->freeSize[i],
                                     uStoreFree->seg2FreeUpc[i], uStoreFree->seg2FreeSize[i],
                                     NUM_BKPT_INST);
            hopSegs[i] = NOUSTORE;
        }
    }
    
    else{        
        unsigned int addr, size, seg2_addr, seg2_size;
        for(i = 0; i < MAX_UENGS; i++){
            uEngs[i]->getUstoreFreeMem(&addr, &size, &seg2_addr, &seg2_size);
            uStore[i] = new uStoreMM(addr, size, seg2_addr, seg2_size, NUM_BKPT_INST);
            hopSegs[i] = NOUSTORE;
        }
    }
    
    debugSem = Sem_Init(1);	 // create semaphore
    numBkPts = 0;
        
    for(i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        bkPts[i].id = -1;
        bkPts[i].state = UDBG_BKPT_CLR;
    }

    if (BkptSem != 0){
        sem_T tmpSem = BkptSem;
        BkptSem = 0;
        Sem_Destroy(tmpSem);
    }

    BkptSem = Sem_Init(1);	             // create breakpoint semaphore
    Sem_Wait(BkptSem, SEM_WAITFOREVER);
    
    bkPtThreadId = Thread_Create(BKPT_THRD_PRIORITY, bkPtThread, (void *)this);
    
    // connect breakpoint interrupt handler
    uEng_intConnect(UENG_DBG_INT, Udbg_intMask, bkPtHandler, usrData);
    
    valid = 1;  // mark debug object valid

    return;
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::~Debug
   Description: Debug class destructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Debug::~Debug(void)
{
    if (valid){
        for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++) delete(uStore[ueng]);

        Sem_Destroy(debugSem);

        if (bkPtThreadId != 0){  // can be zero if thread killed itself
            Thread_Cancel(bkPtThreadId);
            bkPtThreadId = 0;
        }
    }

    TheDebugObj = NULL;

    return;
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::UpdateFreeUstore
   Description: Update the micro-store memory manager with values from the uengine
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void Debug::UpdateFreeUstore(unsigned int uEngMask)
{
    unsigned int freeUaddr, freeSize, freeUaddrSeg2, freeSizeSeg2;

    if (!valid) return;

    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if((uEngMask & setMasks[ueng])){
            
            // get the new values from the micro-engine
            uEngs[ueng]->getUstoreFreeMem(&freeUaddr, &freeSize, &freeUaddrSeg2, &freeSizeSeg2);
            
            // update the memory manager with the new region
            uStore[ueng]->defineFreeRegion(freeUaddr, freeSize, NUM_BKPT_INST);
            uStore[ueng]->defineFreeRegion(freeUaddrSeg2, freeSizeSeg2, NUM_BKPT_INST);
        }
    }
    
    return;
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::DefineFreeUstore
   Description: Define the micro-store free region for the specified micro-engines.
                Set either the segment 1 or the segment 2 region depending on the
                uaddr passed.
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::DefineFreeUstore(unsigned int uEngMask, unsigned int new_freeUaddr,
							unsigned int new_freeSize)
{
    unsigned int freeUaddr, freeSize, freeUaddrSeg2, freeSizeSeg2;
    
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    
    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if((uEngMask & setMasks[ueng])){
            
            // get the old values from the micro-engine
            uEngs[ueng]->getUstoreFreeMem(&freeUaddr, &freeSize, &freeUaddrSeg2, &freeSizeSeg2);
            
            // update the micro-engine with the new values
            // retain the old values for the segment not being changed
            if(new_freeUaddr <= MAX_USTORE_PER_SEG && 
                (new_freeUaddr + new_freeSize) <= MAX_USTORE_PER_SEG){
                // new values are valid for segment 1
                uEngs[ueng]->setUstoreFreeMem(new_freeUaddr, new_freeSize, freeUaddrSeg2, freeSizeSeg2);
                
                // update the memory manager with the new segment 1 region
                uStore[ueng]->defineFreeRegion(new_freeUaddr, new_freeSize, NUM_BKPT_INST);
            }
            
            else if(new_freeUaddr >= MAX_USTORE_PER_SEG && new_freeUaddr <= MAX_C0_USTORE &&
                (new_freeUaddr + new_freeSize) <= MAX_C0_USTORE){
                // new values are valid for segment 2
                uEngs[ueng]->setUstoreFreeMem(freeUaddr, freeSize, new_freeUaddr, new_freeSize);
                
                // update the memory manager with the new segment 2 region
                uStore[ueng]->defineFreeRegion(new_freeUaddr, new_freeSize, NUM_BKPT_INST);
            }
            
            else return(UDBG_BADARG);  // address or size invalid
        }
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::ResetUeng
   Description: Reset the specified uengine(s).
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ResetUeng(unsigned int uEngMask, int clrReg)
{
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    uEng_resetUengs(uEngMask, clrReg);
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::ClrResetUeng
   Description: Takes the specified micro-engines out of reset.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ClrResetUeng(unsigned int uEngMask)
{
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    uEng_clrResetUengs(uEngMask);
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::StartUeng
   Description: Start the specified uengine(s).
   Returns:		UDBG_SUCCESS,UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::StartUeng(unsigned int uEngMask)
{
    const unsigned char ctxZero = 0;
    
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    
    if(GetActiveUengs(uEngMask)) return(UDBG_UENGACT);  // an engine is already active
    
    lockDbg();
    
    // clear breaks that has been met
    
    for(int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if((bkPts[i].state == UDBG_BKPT_MET) && (uEngMask & setMasks[bkPts[i].ueng])){
            bkPts[i].state = UDBG_BKPT_SET;
        }
    }
    
    // start all threads of specified uengines, starting in thread zero
    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            uEngs[ueng]->setPC(UDBG_CTXALL, 0);			// set all threads PC to 0
            uEngs[ueng]->start(ctxZero, UDBG_CTXALL);	// start all the threads
        }
    }
    
    unlockDbg();
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::IsUengActive
   Description: Check if all the specified micro-engines are active.
   Returns:		UDBG_UENGINACT -- One or all uengines are not active,
				UDBG_RESET -- One or all uengines are in reset
				UDBG_UENGACT -- All uengines are active
				UDBG_BADLIB -- Debug library/class uninitialized
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::IsUengActive(unsigned int uEngMask)
{
    int status;

    VERIFY_LIB();

    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if((uEngMask & setMasks[ueng]) && 
           ((status = uEngs[ueng]->isUengActive()) != UENG_ACTIVE)) 
           return(status);
    }
    
    return(UDBG_UENGACT);	// all specified engines are active
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetActiveUengs
   Description: Determine which of the selected micro-engines are active.
   Returns:		A mask of active microengines,
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
unsigned int Debug::GetActiveUengs(unsigned int uEngMask)
{
    unsigned int activeUengs = 0;

    if (!valid) return(0);
    
    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if((uEngMask & setMasks[ueng]) && 
           (uEngs[ueng]->isUengActive() == UENG_ACTIVE))
            activeUengs |= setMasks[ueng];
    }
    
    return(activeUengs);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::pauseUthreads
   Description: Stop the specified uengines contexts and save states that
				will enable them to be resumed.
   Returns:		UDBG_SUCCESS,UDBG_BADARG, UDBG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::pauseUthreads(unsigned int uEngMask, unsigned int ctxMask, unsigned int dly)
{
    unsigned int ueng;
    
    VERIFY_UENGMASK(uEngMask);
    VERIFY_CTXMASK(ctxMask);
    
    lockDbg();
    
    // stop the uengines
    for(ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]) uEngs[ueng]->disableCtx(ctxMask);	// stop the contexts
    }
    
    // adjust delay value to sys clock resolution
    dly = dly/DELAYMS_RES;
    
    // verify that they are stopped
    while(uEngMask){
        for(ueng = 0; ueng < MAX_UENGS; ueng++){
            if((uEngMask & setMasks[ueng]) && (uEngs[ueng]->isUengActive() != UENG_ACTIVE)){
                // clear bit in ueng mask to indicate it's not active
                uEngMask = CLRBIT(uEngMask, ueng);
            }
        }
        
        if(dly--) delayMs(DELAYMS_RES);		// delay 1 system clock-tick
        else break;
    }
    
    unlockDbg();
    
    if(uEngMask) return(UDBG_FAIL);
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::resumeUthreads
   Description: Resume the specified micro-engines contexts.  This function will
				not resume a context that has been disabled via ctx_arb[kill].
   Returns:		UDBG_SUCCESS,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::resumeUthreads(unsigned int uEngMask, unsigned int ctxMask)
{
    Breakpoint_T *bkpt;
    
    VERIFY_UENGMASK(uEngMask);
    VERIFY_CTXMASK(ctxMask);
    
    lockDbg();

    // resume the uengines
    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            if((bkpt = findBkPtUeng(ueng))) bkpt->state = UDBG_BKPT_SET;
            uEngs[ueng]->enableCtx(ctxMask);		// all context should be enabled
        }
    }

    unlockDbg();
   
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::disableUthreads
   Description: Disable the specified uengine(s) thread(s).
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::disableUthreads(unsigned int uEngMask, unsigned int ctxMask)
{
    VERIFY_UENGMASK(uEngMask);
    VERIFY_CTXMASK(ctxMask);
    
    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]) uEngs[ueng]->disableCtx(ctxMask);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::enableUthreads
   Description: Enable the specified uengine(s) thread(s).
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::enableUthreads(unsigned int uEngMask, unsigned int ctxMask)
{
    VERIFY_UENGMASK(uEngMask);
    VERIFY_CTXMASK(ctxMask);
    
    for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]) uEngs[ueng]->enableCtx(ctxMask);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetUengPCs
   Description: Get the micro-engine program-counters.  If the uengine is at
				a break, then return the breakpoint PC.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetUengPCs(unsigned char uEng, unsigned int *upc)
{
    VERIFY_LIB();
    VERIFY_UENG(uEng);

    for(unsigned char ctx = 0; ctx < MAX_CTX; ctx++){
        GetUthreadPC(uEng, ctx, &upc[ctx]);
    }

    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::SetUengPCs
   Description: Set the micro-engine program-counters.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::SetUengPCs(unsigned char uEng, unsigned int *upc)
{
    int status;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);

    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(unsigned char ctx = 0; ctx < MAX_CTX; ctx++){ 
        uEngs[uEng]->setPC(setMasks[ctx], upc[ctx]);
    }

    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetUthreadPC
   Description: Get the micro-engine program-counters. If the uengine is at
				a break, then return the breakpoint PC.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetUthreadPC(unsigned char uEng, unsigned char ctx, unsigned int *upc)
{
    Breakpoint_T *bkpt;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    uEngs[uEng]->getPC(ctx, upc);
    
#ifdef _DBG_PRINT
    printf("GetUthreadPC: ueng=%d, ctx=%d, pc=%d\n", uEng, ctx, *upc);
#endif
    
    // check if pc is within a breakpoint routine
    if((bkpt = findBkPtRtnAddr(uEng, *upc))){
        if(bkpt->ctxBreak == ctx){
            // check if it's before/at the break statement
            if(*upc <= (unsigned int)(bkpt->brUpc + BKPT_INST_ORIG)){
                *upc = bkpt->upc;					// at breakpoint
            }
            else *upc = bkpt->upc + 1;			// beyond breakpoint
            
#ifdef _DBG_PRINT
            printf("GetUthreadPC: ueng %d ctx %d found within the breakpoint routine - adjusting pc to %d\n", uEng, ctx, *upc);
#endif
        }
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::SetUthreadsPC
   Description: Set the program-counter of the specified threads.
   Returns:		UDBG_SUCCESS, UDBG_FAIL, UDBG_BADARG, UENG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::SetUthreadsPC(unsigned int uEngMask, unsigned int ctxMask, unsigned int upc)
{
    int status;
    unsigned int ueng;
    
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    VERIFY_CTXMASK(ctxMask);
    
    // return if an engine is active or in reset
    for(ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            if((status = uEngs[ueng]->isUengActive()) != UENG_INACTIVE)
                return(status);
        }
    }

    for(ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            if((status = uEngs[ueng]->setPC(ctxMask, upc)) != UENG_SUCCESS) 
                return(status);
        }
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetUword
   Description: Read micro-engine control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UENG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetUword(unsigned char uEng, unsigned int uWordAddr, unsigned int *uWord)
{
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    return(uEngs[uEng]->getUword(uWordAddr, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::SetUword
   Description: Write to micro-engines control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::SetUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned int uWord)
{
    int status;
    unsigned int ueng;
    
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    
    // return if an engine is active or in reset
    for(ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            if((status = uEngs[ueng]->isUengActive()) != UENG_INACTIVE)
                return(status);
        }
    }
    
    for(ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            if((status = uEngs[ueng]->setUword(uWordAddr, uWord)) != UENG_SUCCESS) 
                return(status);
        }
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetNumUword
   Description: Read micro-engine control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetNumUword(unsigned char uEng, unsigned int uWordAddr,
					   unsigned short numWords, unsigned int *uWord)
{
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    return(uEngs[uEng]->getNumUword(uWordAddr, numWords, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::SetNumUword
   Description: Write to micro-engines control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::SetNumUword(unsigned int uEngMask, unsigned int uWordAddr,
					   unsigned short numWords, unsigned int *uWord)
{
    int status;
    unsigned int ueng;
    
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    
    // return if an engine is active or in reset
    for(ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            if((status = uEngs[ueng]->isUengActive()) != UENG_INACTIVE)
                return(status);
        }
    }
    
    for(ueng = 0; ueng < MAX_UENGS; ueng++){
        if(uEngMask & setMasks[ueng]){
            if((status = uEngs[ueng]->setNumUword(uWordAddr, numWords, uWord)) != UENG_SUCCESS) 
                return(status);
        }
    }

    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::getBkPt
   Description: Initialize and return a pointer to a breakpoint structure.
   Returns:		BreakPoint_T *, or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Breakpoint_T *Debug::getBkPt(uDbg_Bkpt_T *bkpoint)
{
    for(unsigned int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if(bkPts[i].id == -1){
            bkPts[i].callback = bkpoint->callback;
            bkPts[i].usrData = bkpoint->usrData;
            bkPts[i].id = i;
            bkPts[i].ueng = bkpoint->uEng;
            bkPts[i].ctxBreak = 0;
            bkPts[i].upc = bkpoint->uAddr;
            bkPts[i].breakIfCtx = bkpoint->breakIfCtx;
            bkPts[i].stpUengMask = bkpoint->stpOnBrkUengMask;
            return(&bkPts[i]);
        }
    }

    return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	Debug::findBkPtId
   Description: Located and return an pointer to a breakpoint structure.
   Returns:		Breakpoint_T *, or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Breakpoint_T *Debug::findBkPtId(short breakpointId)
{
    if((breakpointId < 0) || (breakpointId >= UDBG_MAX_BREAKPOINTS)) return(NULL);

    if((bkPts[breakpointId].id == breakpointId) && (bkPts[breakpointId].state != UDBG_BKPT_CLR))
        return(&bkPts[breakpointId]);

    return(NULL);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::findBkPtUaddr
   Description: Located and return an pointer to a breakpoint structure.
   Returns:		Breakpoint_T *, or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Breakpoint_T *Debug::findBkPtUaddr(unsigned char ueng, unsigned int uAddr)
{
    for(unsigned int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if((bkPts[i].ueng == ueng) && (bkPts[i].upc == uAddr) &&
            (bkPts[i].state != UDBG_BKPT_CLR)){
            return(&bkPts[i]);
        }
    }
    return(NULL);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::findBkPtRtnAddr
   Description: Located and return an pointer to a breakpoint structure the
				uAddr is within a breakpoint routine.
   Returns:		Breakpoint_T *, or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Breakpoint_T *Debug::findBkPtRtnAddr(unsigned char ueng, unsigned int uAddr)
{
    for(unsigned int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if((bkPts[i].ueng == ueng) && (bkPts[i].state != UDBG_BKPT_CLR) &&
            (uAddr >= (bkPts[i].brUpc + BKPT_UPC_DETECT)) &&
            (uAddr <= (bkPts[i].brUpc + NUM_BKPT_INST))){
            return(&bkPts[i]);
        }
    }
    
    return(NULL);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::findBkPtUeng
   Description: Located and return an pointer to a breakpoint structure if the
				uengine is at a breakpoint.
   Returns:		Breakpoint_T *, or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Breakpoint_T *Debug::findBkPtUeng(unsigned char uEng)
{
    for(unsigned int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if((bkPts[i].ueng == uEng) && (bkPts[i].state == UDBG_BKPT_MET)){
            return(&bkPts[i]);
        }
    }
    
    return(NULL);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::setBreakpoint
   Description: Set a micro-engine breakpoint.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_DUPBKP
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::setBreakpoint(Breakpoint_T *bkPoint)
{
    unsigned int bkPtUword, relSegAddr;
    Uengine *uengine;
    unsigned int breakInst[NUM_BKPT_INST] = {
        NOP,				//0: may be replaced by: BR!=CTX, or BR=CTX
        NOP,				//1: may be replaced by: BR!=CTX, or BR=CTX
        DISABLE_CTX,		//2: local_csr_wr[CTX_ENABLES, 0] -- disable context
        NOP,				//3: Four instructions are needed bet ctx_disable and ctx_arb
        NOP,				//4:
        NOP,				//5:
        FAST_WR,			//6: fast_wr[0x80,ireg] -- breakpoint interupt
        CTX_ARB,			//7: ctx_arb[voluntary] -- force context switch
        NOP,				//8: two instructions to move breakpoint statement out of
        NOP,				//9: the abort-shadow -- allows brk on ctx_arb and br statements
        0x9A0007B0,			//10: breakpoint statement placeholder -- to be altered
        NOP,				//11: two instructions to breakpoint br instruction out of a possible
        NOP,				//12: abort-shadow -- allows brk on ctx_arb and br statements
        BR };				//13: br[pc] -- to be altered to breakpoint + 1
        
    if(!bkPoint) return(UDBG_BADARG);
    VERIFY_UENG(bkPoint->ueng);

    uengine = uEngs[bkPoint->ueng];
        
    // check if a breakpoint is already set at this location
    if(findBkPtUaddr(bkPoint->ueng, bkPoint->upc)) return(UDBG_DUPBKP);
        
    // get the pc relative to the begining of the breakpoint routine
    relSegAddr = (bkPoint->brUpc + BKPT_INST_ORIG) & UADDR_MASK;
        
#ifdef _DBG_PRINT
    printf("setBreakpoint: breakId=%d, bkpt=%d, bkptRtn=%d\n", bkPoint->id, bkPoint->upc, bkPoint->brUpc);
#endif
        
    // setup breakpoint routine to break on the specified contexts
    switch(numBitsSet(bkPoint->breakIfCtx)){
        int i;

    case 4:	// break on any ctx -- default
        break;

    case 3:	// break on any three of the specified contexts
        // find the ctx that is not set
        for(i=0; i < 4; i++) if(!IS_BIT_SET(bkPoint->breakIfCtx, i)) break;

        if(i == CTX0) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX0, relSegAddr);
        else if(i == CTX1) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX1, relSegAddr);
        else if(i == CTX2) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX2, relSegAddr);
        else if(i == CTX3) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX3, relSegAddr);
        break;

    case 2:	// break on two of the specified contexts
        // find the ctx that is not set
        for(i=0; i < 4; i++) if(!IS_BIT_SET(bkPoint->breakIfCtx, i)) break;
                
        if(i == CTX0) breakInst[BKPT_COND0] = uengine->setBrInstAddr(BR_CTX0, relSegAddr);
        else if(i == CTX1) breakInst[BKPT_COND0] = uengine->setBrInstAddr(BR_CTX1, relSegAddr);
        else if(i == CTX2) breakInst[BKPT_COND0] = uengine->setBrInstAddr(BR_CTX2, relSegAddr);
        else if(i == CTX3) breakInst[BKPT_COND0] = uengine->setBrInstAddr(BR_CTX3, relSegAddr);
                
        // find the other ctx that is not set
        for(i++; i < 4; i++) if(!IS_BIT_SET(bkPoint->breakIfCtx, i)) break;
                    
        if(i == CTX0) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX0, relSegAddr);
        else if(i == CTX1) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX1, relSegAddr);
        else if(i == CTX2) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX2, relSegAddr);
        else if(i == CTX3) breakInst[BKPT_COND1] = uengine->setBrInstAddr(BR_CTX3, relSegAddr);
        break;

    case 1:	// break on one specific ctx
        // find the ctx that is set
        for(i=0; i < 4; i++) if(IS_BIT_SET(bkPoint->breakIfCtx, i)) break;
        breakInst[BKPT_COND1] = uengine->setBrInstAddr(SET_BR_CTX(BR_NOT_CTX, i), relSegAddr);
        break;
 
    default:	// none
        return(UDBG_BADARG);
        break;
    }
        
//#ifdef _DBG_PRINT
//    printf("setBreakpoint: bkptCond 0x%x|0x%x\n", breakInst[BKPT_COND0], breakInst[BKPT_COND1]);
//#endif
        
    // save the uword at the breakpoint
    uengine->getUword(bkPoint->upc, &bkPoint->uword); 
        
    // set up branch instruction at the breakpoint location
    bkPtUword = uengine->setBrInstAddr(BR, bkPoint->brUpc);	// set uncond branch to brkpt-routine
    uengine->setUword(bkPoint->upc, bkPtUword);				// write instr to ustore
        
    // install the orginal instr at breakpoint in brkpt-routine
    breakInst[BKPT_INST_ORIG] = bkPoint->uword;
        
    // set uncond branch in brkpt-routine to brkpt + 1
    breakInst[BKPT_INST_BR] = uengine->setBrInstAddr(breakInst[BKPT_INST_BR], bkPoint->upc + 1);
        
    // write the brkpt-routine to ustore
    uengine->setNumUword(bkPoint->brUpc, NUM_BKPT_INST, breakInst);
        
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::isAtBreakpoint
   Description: Determine if a micro-engine is at a the specified breakpoint.
   Returns:		0=no, 1=yes
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::isAtBreakpoint(Breakpoint_T *bkPoint)
{
    unsigned int ctxPc;
    
    if(!bkPoint) return(0);
    if(bkPoint->ueng < 0 || bkPoint->ueng >= MAX_UENGS) return(0);

    if(uEngs[bkPoint->ueng]->isUengActive() == UENG_ACTIVE) return(0);
    
    // determine which context reached the breakpoint
    for(unsigned int ctx = 0; ctx < MAX_CTX; ctx++){
        uEngs[bkPoint->ueng]->getPC(ctx, &ctxPc);
        
        //check if the PC is within the range of the breakpoint detect instructions;
        //if should stop at the NOP following the ctx_arb... but just being flexable
        if((ctxPc >= (bkPoint->brUpc + BKPT_UPC_DETECT)) &&
            (ctxPc <= (bkPoint->brUpc + BKPT_UPC_DETECT + 2))){
            bkPoint->state = UDBG_BKPT_MET;
            bkPoint->ctxBreak = ctx;
            
            // set the ctx as the next to run -- run in the same ctx prior to break
            uEngs[bkPoint->ueng]->setCtxArb((ctx << 2) & 0xC);
            
#ifdef _DBG_PRINT
            printf("isAtBreakpoint: ueng=%d, ctx=%d, breakId=%d, bkpt=%d, bkptRtn=%d\n", 
                bkPoint->ueng, ctx, bkPoint->id, bkPoint->upc, bkPoint->brUpc);
#endif
            return(1);
        }
    }

    return(0);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::removeBreak
   Description: Remove breakpoint.
   Returns:		UDBG_SUCCESS, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::removeBreak(Breakpoint_T *bkpt)
{
#ifdef _DBG_PRINT
    printf("removeBreak: breakId=%d, bkpt=%d, bkptRtn=%d\n", bkpt->id, bkpt->upc, bkpt->brUpc);
#endif
    
    // restore orig word
    SetUword(setMasks[bkpt->ueng], bkpt->upc, bkpt->uword);
    
    // if at break, then set pc to orig word
    if(uEngs[bkpt->ueng] && bkpt->state == UDBG_BKPT_MET){
        uEngs[bkpt->ueng]->setPC(setMasks[bkpt->ctxBreak], bkpt->upc);
#ifdef _DBG_PRINT
        printf("removeBreak: setting ctx %d PC back to %d\n", bkpt->ctxBreak, bkpt->upc);
#endif
    }
    
    bkpt->state = UDBG_BKPT_CLR;
    bkpt->id = -1;

    if(uEngs[bkpt->ueng])
        uStore[bkpt->ueng]->uStoreFreeBlock(bkpt->brUpc);
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::SetBreakpoint
   Description: Set a breakpoint and return the breakpoint identification.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT, UDBG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::SetBreakpoint(uDbg_Bkpt_T *bp)
{
    int status;
    Breakpoint_T *bkPoint;
    
    VERIFY_LIB();
    if(!bp || !bp->callback) return(UDBG_BADARG);
    VERIFY_UENG(bp->uEng);
    if(bp->stpOnBrkUengMask) VERIFY_UENGMASK(bp->stpOnBrkUengMask);
    
    // return if active, or in reset
    if((status = uEngs[bp->uEng]->isUengActive()) != UENG_INACTIVE) return(status);
    
    lockDbg();
    
    if((bkPoint = getBkPt(bp))){
        int uPC;
        
        if((uPC = uStore[bp->uEng]->uStoreAllocBlock(ANY_SEG)) >= 0){
            bkPoint->brUpc = uPC;
            if((status = setBreakpoint(bkPoint)) == UDBG_SUCCESS){
                bp->id = bkPoint->id;
                bkPoint->state = UDBG_BKPT_SET;
                numBkPts++;
            }
            else{
                bkPoint->id = -1;
                uStore[bp->uEng]->uStoreFreeBlock(bkPoint->brUpc);
            }
        }else{
            bkPoint->id = -1;
            status = UDBG_NOUSTORE;
        }
    }
    else status = UDBG_FAIL;

    unlockDbg();
    
    return(status);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::CheckBreak
   Description: Check if breakpoints have been reached.
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::CheckBreak()
{
#ifdef _DBG_PRINT
    printf("CheckBreak called\n");
#endif

    lockDbg();
    
    if(numBkPts > 0){
        // check if any breakpoint has been reached
        for(int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
            if(bkPts[i].state == UDBG_BKPT_CLR) continue;
            
            // check if breakpoint is met; if so, stop other engines and exec callback
            if(isAtBreakpoint(&bkPts[i])){
                if(bkPts[i].stpUengMask) disableUthreads(bkPts[i].stpUengMask, UDBG_CTXALL);
                
                unlockDbg();	 // freeup the debug resource -- incase the callback takes a long time
                
                (bkPts[i].callback)(bkPts[i].id, bkPts[i].ueng, (unsigned char)bkPts[i].ctxBreak,
                    bkPts[i].usrData);		// execute callback

                lockDbg();
            }
        }
    }
    
    unlockDbg();

#ifdef _DBG_PRINT
    printf("CheckBreak done\n");
#endif

    return(0);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::isUengBreak
   Description: Determine if the micro-engine is at a breakpoint.
   Returns:		-1 not at breakpoint, 0 >= the breakpoint ID
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::isUengBreak(unsigned char uEng)
{
    lockDbg();
    
    // check if uengine breakpoint has been reached
    for(int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if((bkPts[i].state == UDBG_BKPT_MET) && (bkPts[i].ueng == uEng)){
            unlockDbg();
            return(bkPts[i].id);
        }
    }
    
    unlockDbg();
    
    return(-1);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::ShowBreakpoints
   Description: Return the defined breakpoints.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ShowBreakpoints(unsigned int uEngMask, uDbg_BkptArray_T *breakpoints)
{
    unsigned short bp = 0;
    
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    if(!breakpoints) return(UDBG_BADARG);
    
    lockDbg();
    
    for(int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if((bkPts[i].state != UDBG_BKPT_CLR) && (uEngMask & setMasks[bkPts[i].ueng])){
            breakpoints->bkpts[bp].id = bkPts[i].id;
            breakpoints->bkpts[bp].uEng = bkPts[i].ueng;
            breakpoints->bkpts[bp].uAddr = bkPts[i].upc;
            breakpoints->bkpts[bp].ctx = bkPts[i].ctxBreak;
            breakpoints->bkpts[bp].breakIfCtx = bkPts[i].breakIfCtx;
            bp++;
        }
    }
    
    unlockDbg();
    
    breakpoints->len = bp;
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::ClearAllBreakpoints
   Description: Remove all breakpoints associated with the specified micro-engines.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ClearAllBreakpoints(unsigned int uEngMask)
{
    VERIFY_LIB();
    VERIFY_UENGMASK(uEngMask);
    
    if(GetActiveUengs(uEngMask)) return(UDBG_UENGACT); 
    
    lockDbg();
    
    for(int i = 0; i < UDBG_MAX_BREAKPOINTS; i++){
        if((bkPts[i].state != UDBG_BKPT_CLR) && (uEngMask & setMasks[bkPts[i].ueng])){
            removeBreak(&bkPts[i]);
            numBkPts--;
        }
    }
    
    unlockDbg();
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::ClearBreakpoint
   Description: Remove a specific breakpoint.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT, UDBG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ClearBreakpoint(short breakpointId)
{
    Breakpoint_T *bkpoint;
    int status;
    
    VERIFY_LIB();

    lockDbg();
    
    if(!(bkpoint = findBkPtId(breakpointId))){
        unlockDbg();
        return(UDBG_FAIL);
    }
    
    if((status = uEngs[bkpoint->ueng]->isUengActive()) != UENG_INACTIVE){
        unlockDbg();
        return(status);
    }
    
    if(bkpoint->state != UDBG_BKPT_CLR){
        removeBreak(bkpoint);
        numBkPts--;
    }
    
    unlockDbg();
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::WriteMem32
   Description: Write a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::WriteMem32(unsigned int addr, unsigned int *data, unsigned int count)
{
    VERIFY_LIB();
    if(!data) return(UDBG_BADARG);
    for(unsigned int i = 0; i < count; i++)
        WRITE_LWORD((((unsigned int *)addr) + i), data[i]);
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::ReadMem32
   Description: Read a long-word from memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ReadMem32(unsigned int addr, unsigned int *data, unsigned int count)
{
    VERIFY_LIB();
    if(!data) return(UDBG_BADARG);
    for(unsigned int i = 0; i < count; i++)
        READ_LWORD(((unsigned int *)addr + i), &data[i]);
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::WriteMem64
   Description: Write a quad-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::WriteMem64(unsigned int addr, unsigned int *data, unsigned int count)
{
    VERIFY_LIB();
    if(!addr) return(UDBG_BADARG);
    if(addr % 8) return(UDBG_BADADDR);	// must be aligned on quad-word boundary
    count = count << 1;
    for(unsigned int i = 0; i < count; i+=2){
        WRITE_LWORD(((unsigned int *)addr + i), data[i]);
        WRITE_LWORD(((unsigned int *)addr + (i+1)), data[i+1]);
    }
    return(UDBG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Debug::ReadMem64
   Description: Read a quad-word from memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ReadMem64(unsigned int addr, unsigned int *data, unsigned int count)
{
    VERIFY_LIB();
    if(!addr) return(UDBG_BADARG);
    if(addr % 8) return(UDBG_BADADDR);	// must be aligned on quad-word boundary
    count = count << 1;
    for(unsigned int i = 0; i < count; i+=2){
        READ_LWORD(((unsigned int *)addr + i), &data[i]);
        READ_LWORD(((unsigned int *)addr + (i+1)), &data[i+1]);
    }
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::PutRelGPR
   Description: Write a number long-words to a micro-engine GPR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::PutRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int type;
    int status;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(bankType == UDBG_GPR_A_BANK) type = 0;
    else if(bankType == UDBG_GPR_B_BANK) type = 1;
    else return(UDBG_BADARG);

    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(unsigned int i = 0; i < count; i++){
        if((status = uEngs[uEng]->writeRelGpr(type, ctx, regNum + i, regData[i])) != UDBG_SUCCESS)
            return(status);
    }    

    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetRelGPR
   Description: Read a number long-word from a micro-engine GPR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int type;
    int status;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData) return(UDBG_BADARG);
    
    if(bankType == UDBG_GPR_A_BANK) type = 0;
    else if(bankType == UDBG_GPR_B_BANK) type = 1;
    else return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(unsigned int i = 0; i < count; i++){
        if((status = uEngs[uEng]->readRelGpr(type, ctx, regNum + i, &regData[i])) != UDBG_SUCCESS)
            return(status);
    }   

    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetRelSRAM_WrXfer
   Description: Read a long-word from a micro-engine SRAM write-xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int status;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(unsigned int i = 0; i < count; i++){
        if((status = uEngs[uEng]->readRelSramWrXfer(ctx, regNum + i, &regData[i])) != UDBG_SUCCESS)
            return(status);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::PutRelSRAM_WrXfer
   Description: Write regData to a micro-engine write-transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::PutRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData, unsigned int count)
{
    int status;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(unsigned int i = 0; i < count; i++){
        if((status = uEngs[uEng]->writeRelSramWrXfer(ctx, regNum + i, regData[i])) != UDBG_SUCCESS)
            return(status);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetRelSRAM_RdXfer
   Description: Read a long-word from a micro-engine SRAM read-xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int status;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(unsigned int i = 0; i < count; i++){
        if((status = uEngs[uEng]->readRelSramRdXfer(ctx, regNum + i, &regData[i])) != UDBG_SUCCESS)
            return(status);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::PutRelSRAM_RdXfer
   Description: Write a long-word from a micro-engine SRAM read-xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::PutRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int status;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(unsigned int i = 0; i < count; i++){
        if((status = uEngs[uEng]->writeRelSramRdXfer(ctx, regNum + i, regData[i])) != UDBG_SUCCESS)
            return(status);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::PutRelSDRAM_WrXfer
   Description: Write a long-words to a micro-engine SDRAM write-xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::PutRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int status;
    unsigned int data64[2], i = 0;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    // writing only one reg -- read reg pair and only update only the target reg.
    if(count == 1){
        // adjust if the target the last register
        if(regNum == MAX_XFER_REG-1){
            regNum--;
            i = 1;
        }
        
        uEngs[uEng]->readRelSdramWrXfer(ctx, regNum, data64);
        data64[i] = regData[0];
        return(uEngs[uEng]->writeRelSdramWrXfer(ctx, regNum, data64));
    }
    
    // forward writing of reg pair -- write the current and next register
    for(i = 0; i < count; i += 2){
        
        // if only one left, then rewrite the prev reg with the prev value
        if((count - i) == 1){
            regNum--;
            data64[0] = data64[1];
            data64[1] = regData[i];
        }
        
        else{
            data64[0] = regData[i];
            data64[1] = regData[i+1];
        }
        
        if((status = uEngs[uEng]->writeRelSdramWrXfer(ctx, regNum, data64)) != UDBG_SUCCESS)
            return(status);
        regNum += 2;
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetRelSDRAM_WrXfer
   Description: Read a long-words from a micro-engine SDRAM write-xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int status;
    unsigned int data64[2], j = 0;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    // make sure we don't access beyond the last reg
    if(regNum && ((regNum + count) == MAX_XFER_REG)){
        j=1;
        regNum--;
    }
    
    // read quad-words
    for(unsigned int i = 0; i < count;){
        if((status = uEngs[uEng]->readRelSdramWrXfer(ctx, regNum + i, data64)) != UDBG_SUCCESS)
            return(status);
        regData[i++] = data64[j++%2];
        if(i < count) regData[i++] = data64[j++%2]; 
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::PutRelSDRAM_RdXfer
   Description: Write a long-words to a micro-engine SDRAM read-xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::PutRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int status;
    unsigned int data64[2], i = 0;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    // writing only one reg -- read reg pair and only update only the target reg.
    if(count == 1){
        // adjust if the target the last register
        if(regNum == MAX_XFER_REG-1){
            regNum--;
            i = 1;
        }
        
        uEngs[uEng]->readRelSdramRdXfer(ctx, regNum, data64);
        data64[i] = regData[0];
        return(uEngs[uEng]->writeRelSdramRdXfer(ctx, regNum, data64));
    }
    
    // forward writing of reg pair -- write the current and next register
    for(i = 0; i < count; i += 2){
        
        // if only one left, then rewrite the prev reg with the prev value
        if((count - i) == 1){
            regNum--;
            data64[0] = data64[1];
            data64[1] = regData[i];
        }
        
        else{
            data64[0] = regData[i];
            data64[1] = regData[i+1];
        }
        
        if((status = uEngs[uEng]->writeRelSdramRdXfer(ctx, regNum, data64)) != UDBG_SUCCESS)
            return(status);
        regNum += 2;
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetRelSDRAM_RdXfer
   Description: Read a long-words from a micro-engine SDRAM read-xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    int status;
    unsigned int data64[2], j = 0;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    VERIFY_CTX(ctx);
    
    if(!regData || (regNum > MAX_XFER_REG-1) || ((regNum + count) > MAX_XFER_REG))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    // make sure we don't access beyond the last reg
    if(regNum && ((regNum + count) == MAX_XFER_REG)){
        j=1;
        regNum--;
    }
    
    // read quad-words
    for(unsigned int i = 0; i < count;){
        if((status = uEngs[uEng]->readRelSdramRdXfer(ctx, regNum + i, data64)) != UDBG_SUCCESS)
            return(status);
        regData[i++] = data64[j++%2];
        if(i < count) regData[i++] = data64[j++%2]; 
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::WriteFIFO_Tx
   Description: Write a number of quad-words to micro-engine transfer-fifo.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::WriteFIFO_Tx(unsigned char uEng, unsigned int fifoAddr,
						unsigned int *data64, unsigned int count, int setValid)
{
    int status;
    unsigned int ii=0, i;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    
    if(!data64 || (fifoAddr > MAX_FIFO_QWADDR-1) || ((fifoAddr + count) > MAX_FIFO_QWADDR))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(i = 0; i < count; i++, ii += 2){
        if((status = uEngs[uEng]->writeTxFifo(fifoAddr+i, &data64[ii], setValid)) != UDBG_SUCCESS)
            return(status);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::ReadFIFO_Rx
   Description: Read a number of quad-words from micro-engine receive-fifo.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::ReadFIFO_Rx(unsigned char uEng, unsigned int fifoAddr,
						unsigned int *data64, unsigned int count)
{
    int status;
    unsigned int ii=0, i;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);

    if(!data64 || (fifoAddr > MAX_FIFO_QWADDR-1) || ((fifoAddr + count) > MAX_FIFO_QWADDR))
        return(UDBG_BADARG);
    
    // return if active or in reset
    if((status = uEngs[uEng]->isUengActive()) != UENG_INACTIVE) return(status);

    for(i = 0; i < count; i++, ii += 2){
        if((status = uEngs[uEng]->readRxFifo(fifoAddr+i, &data64[ii])) != UDBG_SUCCESS)
            return(status);
    }
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetUengStatus
   Description: Retreive the status of the specified micro-engine.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetUengStatus(unsigned char uEng, uDbg_uEngStatus_T *uEngStatus)
{
    int status;
    unsigned int activeCtxStat, ctxEn, ccEn;
    unsigned int csrWrite, sramJou, ctxArbCtl, uPc;
    
    VERIFY_LIB();
    VERIFY_UENG(uEng);
    if(!uEngStatus) return(UDBG_BADARG);
    
    uEngs[uEng]->getActiveStatus(&activeCtxStat);
    uEngs[uEng]->getCtxEnables(&ctxEn);
    uEngs[uEng]->getCondCodeEn(&ccEn);
    uEngs[uEng]->getCsrWriteEn(&csrWrite);
    uEngs[uEng]->getSramJouEn(&sramJou);
    
    // initialize return values
    uEngStatus->state = 0;
    uEngStatus->ctx = 0;
    uEngStatus->breakId = 0;
    uEngStatus->pc = 0;
    uEngStatus->enables = 0;
    
    // determine state
    if(activeCtxStat & IS_ACTIVE_MASK){
        uEngStatus->state = UDBG_UENG_ACTIVE;
        uEngStatus->ctx = activeCtxStat & ACTIVE_CTX_MASK;		// active context
        uEngStatus->pc = (activeCtxStat >> 3) & UADDR_MASK;		// activePC
        if(CTX_X_USTORE_SEGMENT_MASK & activeCtxStat)
            uEngStatus->pc |= UENG_MAX_USTORE;
    }
    
    else{
        Breakpoint_T *bkpt;
        
        if((bkpt = findBkPtUeng(uEng))){
            uEngStatus->state = UDBG_UENG_AT_BREAK;
            uEngStatus->ctx = (char)bkpt->ctxBreak;
            uEngStatus->breakId = (char)bkpt->id;
        }
        
        else{
            uEngStatus->state = UDBG_UENG_INACTIVE;
            if((status = uEngs[uEng]->getCtxArb(&ctxArbCtl)) != UENG_SUCCESS) return(status);
            uEngStatus->ctx = ((ctxArbCtl & 0xC) >> 2);			// next ctx to run
        }
        
        // this uses the uEngStatus->ctx determined above
        if((status = GetUthreadPC(uEng, uEngStatus->ctx, &uPc)) != UDBG_SUCCESS) return(status);
        uEngStatus->pc = uPc;
    }
    
    uEngStatus->enables = (ctxEn >> 8) & 0xf;				// context enable
    uEngStatus->enables |= (sramJou & 0x1) << 4;				// sram journaling enable enable
    uEngStatus->enables |= (csrWrite & 0x1) << 5;			// csr write-self enable
    uEngStatus->enables |= (ccEn & 0x3) << 6;				// cond-code(cur<7>,<prev<6>) enable
    
    return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::HopUeng
   Description: This function allows the micro-engine to continue and then
				pause at the next context swap.  This function is similar to
				stepping, except the number of statements that gets executed
				is generally indeterminate - determine by the logical flow of
				the program and the time when the function is called. The value
				UDBG_TIMEOUT will be returned if the micro-engine fails to stop
				within the specified number of milli-seconds. The micro-engine
				must be inactive prior to calling this function -- UDBG_UENGACT
				will be returned if	the micro-engine is active.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_UENGACT, 
				UDBG_NOUSTORE, UDBG_TIMEOUT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::HopUeng(unsigned int *uEngMask, unsigned int delay)
{
    unsigned int uengMask, relSegAddr, hopUaddr, uPc, activeCtx;
    Uengine *uengine;
    int uEng, uSeg = 0;
    unsigned int  hopInst[NUM_BKPT_INST] = {
		DISABLE_CTX,		//0: local_csr_wr[CTX_ENABLES, 0] -- disable context
		NOP,				//1: nop:
		NOP,				//2:
		NOP,				//3:
		NOP,				//4:
	    BR_CTX0,			//5: BR=CTX[0, hopInst+10]
	    BR_CTX1,			//6: BR=CTX[1, hopInst+11]
	    BR_CTX2,			//7: BR=CTX[2, hopInst+12]
		NOP,				//8:
		BR,					//9:  BR[ctx3_restart]
		BR,					//10: BR[ctx0_restart]
		BR,					//11: BR[ctx1_restart]
		BR};				//12: BR[ctx2_restart]
    unsigned int const brCtxRestart = 9, brCtx = 5;
    int status = UDBG_SUCCESS;

    VERIFY_LIB();
    if(!uEngMask) return(UDBG_BADARG);
    uengMask = *uEngMask;
    VERIFY_UENGMASK(uengMask);
    
#ifdef _DBG_PRINT
    printf("HopUeng: uengMask=0x%x\n", uengMask); fflush(stdout);
#endif
    
    // return if any engine is active
    if(GetActiveUengs(uengMask)) return(UDBG_UENGACT);
    
    lockDbg();

    for(uEng = 0; uEng < MAX_UENGS; uEng++){
        if(!(uengMask & setMasks[uEng])) continue;
        uengine = uEngs[uEng];
        
        // check if storage was already allocated for hoping
        if(hopSegs[uEng] == NOUSTORE){
            if((hopSegs[uEng] = uStore[uEng]->uStoreAllocBlock(uSeg)) == NOUSTORE){
                unlockDbg();
                return(UDBG_NOUSTORE);
            }
        }
        
        hopUaddr = (unsigned int)hopSegs[uEng];
        
        // save the currect PCs
        uengine->getPC(CTX0, &ctx0Pc[uEng]);
        uengine->getPC(CTX1, &ctx1Pc[uEng]);
        uengine->getPC(CTX2, &ctx2Pc[uEng]);
        uengine->getPC(CTX3, &ctx3Pc[uEng]);
        
#ifdef _DBG_PRINT
        printf("HopUeng: saved ueng %d PCs: %d %d %d %d\n", 
            uEng, ctx0Pc[uEng], ctx1Pc[uEng], ctx2Pc[uEng], ctx3Pc[uEng]); fflush(stdout);
#endif
        
        // modify conditional branch instructions to branch to the
        // appropriate context return statement
        relSegAddr = hopUaddr & UADDR_MASK;
        hopInst[brCtx] = uengine->setBrInstAddr(BR_CTX0, relSegAddr + brCtxRestart + 1);		//ctx0
        hopInst[brCtx+1] = uengine->setBrInstAddr(BR_CTX1, relSegAddr + brCtxRestart + 2);	//ctx1
        hopInst[brCtx+2] = uengine->setBrInstAddr(BR_CTX2, relSegAddr + brCtxRestart + 3);	//ctx2
        
        //modify branch instructions to return to the current PC 
        hopInst[brCtxRestart] = uengine->setBrInstAddr(BR, ctx3Pc[uEng]);
        hopInst[brCtxRestart+1] = uengine->setBrInstAddr(BR, ctx0Pc[uEng]);
        hopInst[brCtxRestart+2] = uengine->setBrInstAddr(BR, ctx1Pc[uEng]);
        hopInst[brCtxRestart+3] = uengine->setBrInstAddr(BR, ctx2Pc[uEng]);
        
        // modify the PCs to branch to hop-segment
        uengine->setPC(UENG_ALL_CTX, hopUaddr);		// only the PC bits are writable
        
        // write the hop instructions to ustore
        uengine->setNumUword(hopUaddr, NUM_BKPT_INST-1, hopInst);
        
        // check if uengine is at breakpoint
        Breakpoint_T *bkpt;
        if((bkpt = findBkPtUeng(uEng))) bkpt->state = UDBG_BKPT_SET;
    }
    
    // this code block is not included in the above loop because we want
    // to enable the uengines as close as possible together 
    for(uEng = 0; uEng < MAX_UENGS; uEng++){
        if(!(uengMask & setMasks[uEng])) continue;
        // enable the contexts
        uEngs[uEng]->enableCtx(UDBG_CTXALL);
    }
    
    // poll until the unegines quiet or exceed the time limit
    delay = delay/DELAYMS_RES;
    while((*uEngMask = GetActiveUengs(uengMask))){
        if(delay--) delayMs(DELAYMS_RES);		// delay 1 system clock-tick
        else{
            if(*uEngMask) status = UDBG_TIMEOUT;
            break;
        }
    }
    
    for(uEng = 0; uEng < MAX_UENGS; uEng++){
        if(!(uengMask & setMasks[uEng])) continue;
        uengine = uEngs[uEng];
        
        // get the context that's running...if any
        activeCtx = uengine->getActiveCtx();
        hopUaddr = (unsigned int)hopSegs[uEng];
        
#ifdef _DBG_PRINT
        printf("HopUeng: ueng=%d, activeCtx=0x%x, hopUaddr=%d\n", uEng, activeCtx, hopUaddr); fflush(stdout);
#endif
        
        // restore the PC of the contexts that haven't ran during the hop
        uengine->getPC(CTX0, &uPc);
        if((activeCtx != CTX0) && (uPc == hopUaddr)){
            uengine->setPC(setMasks[CTX0], ctx0Pc[uEng]);
#ifdef _DBG_PRINT
            printf("HopUeng: ueng %d ctx0 was restored to PC=%d\n", uEng, ctx0Pc[uEng]); fflush(stdout); 
#endif
        }
#ifdef _DBG_PRINT
        else{
            printf("HopUeng: ueng %d ctx0 was left at PC=%d\n", uEng, uPc); fflush(stdout); 
        }
#endif
        
        uengine->getPC(CTX1, &uPc);
        if((activeCtx != CTX1) && (uPc == hopUaddr)){
            uengine->setPC(setMasks[CTX1], ctx1Pc[uEng]);
#ifdef _DBG_PRINT
            printf("HopUeng: ueng %d ctx1 was restored to PC=%d\n", uEng, ctx1Pc[uEng]); fflush(stdout); 
#endif
        }
#ifdef _DBG_PRINT
        else{
            printf("HopUeng: ueng %d ctx1 was left at PC=%d\n", uEng, uPc); fflush(stdout); 
        }
#endif
        
        uengine->getPC(CTX2, &uPc);
        if((activeCtx != CTX2) && (uPc == hopUaddr)){
            uengine->setPC(setMasks[CTX2], ctx2Pc[uEng]);
#ifdef _DBG_PRINT
            printf("HopUeng: ueng %d ctx2 was restored to PC=%d\n", uEng, ctx2Pc[uEng]); fflush(stdout); 
#endif
        }
#ifdef _DBG_PRINT
        else{
            printf("HopUeng: ueng %d ctx2 was left at PC=%d\n", uEng, uPc); fflush(stdout); 
        }
#endif
        
        uengine->getPC(CTX3, &uPc);
        if((activeCtx != CTX3) && (uPc == hopUaddr)){
            uengine->setPC(setMasks[CTX3], ctx3Pc[uEng]);
#ifdef _DBG_PRINT
            printf("HopUeng: ueng %d ctx3 was restored to PC=%d\n", uEng, ctx3Pc[uEng]); fflush(stdout); 
#endif
        }
#ifdef _DBG_PRINT
        else{
            printf("HopUeng: ueng %d ctx3 was left at PC=%d\n", uEng, uPc); fflush(stdout); 
        }
#endif
    }

    unlockDbg();
    
    return(status);
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetSysMemSize
   Description: This routine returns sizes of virtual memory regions in the following
				pointers:
					sys     -- Size of memory mapped at zero for OS
					sys_all -- Size of memory mapped at zero and usable by the OS
					sdram   -- Size of memory mapped at C0000000 and usable by HW
					sram    -- Size of sram
					flash   -- Size of Flash
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Debug::GetSysMemSize(unsigned int* sys, unsigned int* sys_all, unsigned int* sdram,
				         unsigned int* sram,unsigned int* flash)
{
    VERIFY_LIB();
	if(!sys || !sys_all, !sdram || !sram || !flash) return(UDBG_BADARG);
#if (IOSTYLE == HARDWARE)
	sysMemVirtSize(sys, sys_all, sdram, sram, flash);
#else
	// for now, ignore this call in simulation mode
#endif
	return(UDBG_SUCCESS);
}


void Debug::regDump(unsigned int uEngMask)
{
	for(unsigned int uEng = 0; uEng < MAX_UENGS; uEng++){
		if(!(uEngMask & setMasks[uEng])) continue;
		uEngs[uEng]->dump();
	}

    return;
}
