/* udbg_umm.cpp
 * Ucode Debug Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: drj, June 16,98  Initial version
 *  revisions:
 *  $History: udbg_umm.cpp $
 * 
 * *****************  Version 17  *****************
 * User: Ewalker      Date: 11/17/00   Time: 3:06p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Hardened address and size checks when changing free ustore regions.
 * Reject invalid values pass by caller.
 * 
 * *****************  Version 16  *****************
 * User: Ewalker      Date: 11/14/00   Time: 4:12p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * 
 * *****************  Version 15  *****************
 * User: Ewalker      Date: 11/14/00   Time: 12:36p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Added some DBG_PRINT blocks to aid debugging
 * 
 * *****************  Version 14  *****************
 * User: Ewalker      Date: 11/09/00   Time: 1:58p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed bug where breakpoints could be set even if there was no free
 * ustore after successfully setting a breakpoint that consumed the last
 * of the free ustore.
 * 
 * *****************  Version 13  *****************
 * User: Drjohns1     Date: 11/03/00   Time: 12:35p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Fixed free-ustore problems
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 10/27/00   Time: 2:30p
 * Updated in $/dev_1200/SA1_CoreLibs/debug_1200
 * Updated for C0 fixes
 * 
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 11/06/98   Time: 11:22a
 * Created in $/users/Johnson/SA1200/udebug
 * 
 * 
 * ---------------------------------------------------------------------
 */
#include <string.h>
#include "ueng.h"
#include "udbg_umm.h"


static const char __copyright[] =
"Copyright Intel Corp, 1998. All rights reserved";


/*-----------------------------------------------------------------------------
   Function: 	uStoreMM::uStoreMM
   Description: Class constructor.
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uStoreMM::uStoreMM(unsigned int freeUaddr, unsigned int freeSize, unsigned int blockSize)
{
   uStoreMM(freeUaddr, freeSize, MAX_C0_USTORE, MAX_USTORE_PER_SEG, blockSize);
}


/*-----------------------------------------------------------------------------
   Function: 	uStoreMM::uStoreMM
   Description: Class constructor.
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uStoreMM::uStoreMM(unsigned int freeUaddr, unsigned int freeSize,
				   unsigned int seg2FreeUaddr, unsigned int seg2FreeSize,
				   unsigned int blockSize)
{
   defineFreeRegion(freeUaddr, freeSize, blockSize);
   defineFreeRegion(seg2FreeUaddr, seg2FreeSize, blockSize);
}


/*-----------------------------------------------------------------------------
   Function: 	uStoreMM::defineFreeRegion
   Description: Redefine the micro-store free region
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uStoreMM::defineFreeRegion(unsigned int freeUaddr, unsigned int freeSize, unsigned int blockSize)
{
   const char zero = 0;

	if(blockSize < MIN_UBLOCK_SIZE) blockSize = MIN_UBLOCK_SIZE;

    if(freeUaddr <= MAX_USTORE_PER_SEG &&
       (freeUaddr + freeSize) <= MAX_USTORE_PER_SEG){
		// define segment 1 free area
		startUaddr = freeUaddr;
		lastBlk = 0;
		blkSize = blockSize;

		if((numBlks = (freeSize / blkSize))) memset(blocks, zero, numBlks);

#ifdef _DBG_PRINT
        printf("defineFreeRegion: %d blocks of size %d starting at %d in segment 1\n",
               numBlks, blkSize, startUaddr);
#endif
    }
	else if (freeUaddr >= MAX_USTORE_PER_SEG && freeUaddr <= MAX_C0_USTORE &&
             (freeUaddr + freeSize) <= MAX_C0_USTORE){
		// define segment 2 free area
		seg2StartUaddr = freeUaddr;
		seg2LastBlk = 0;
		seg2BlkSize = blockSize;

		if((seg2NumBlks = (freeSize / seg2BlkSize))) memset(seg2Blocks, zero, seg2NumBlks);

#ifdef _DBG_PRINT
        printf("defineFreeRegion: %d blocks of size %d starting at %d in segment 2\n",
               seg2NumBlks, seg2BlkSize, seg2StartUaddr);
#endif
	}

    else
    {
#ifdef _DBG_PRINT
        printf("defineFreeRegion: freeUaddr=%u or freeSize=%u is invalid\n", freeUaddr, freeSize); 
#endif
    }

    return;
}

/*-----------------------------------------------------------------------------
   Function: 	uStoreMM::allocSeg1Block
   Description: Allocate a block of micro-store.
   Returns:		The allocated micro-store address
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uStoreMM::allocSeg1Block(void)
{
    int uaddr;
    for(unsigned int i = 0; i < numBlks; i++){
		if(blocks[lastBlk] == 0){
		  blocks[lastBlk] = 1;
          uaddr = (lastBlk * blkSize) + startUaddr;
#ifdef _DBG_PRINT
          printf("allocSeg1Block: allocated block %d at %d of size %d\n", lastBlk, uaddr, blkSize);
#endif
          return(uaddr);
		}
		if(lastBlk >= (numBlks - 1)) lastBlk = 0;
		else lastBlk++;
	}
#ifdef _DBG_PRINT
    printf("allocSeg1Block: unable to allocate block\n");
#endif
	return(-1);
}


/*-----------------------------------------------------------------------------
   Function: 	uStoreMM::allocSeg2Block
   Description: Allocate a block of micro-store.
   Returns:		The allocated micro-store address
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uStoreMM::allocSeg2Block(void)
{
    int uaddr;
	for(unsigned int i = 0; i < seg2NumBlks; i++){
		if(seg2Blocks[seg2LastBlk] == 0){
		  seg2Blocks[seg2LastBlk] = 1;
          uaddr = (seg2LastBlk * seg2BlkSize) + seg2StartUaddr;
#ifdef _DBG_PRINT
          printf("allocSeg2Block: allocated block %d at %d of size %d\n", seg2LastBlk, uaddr, seg2BlkSize);
#endif
		  return(uaddr);
		}
		if(seg2LastBlk >= (seg2NumBlks - 1)) seg2LastBlk = 0;
		else seg2LastBlk++;
	}
#ifdef _DBG_PRINT
    printf("allocSeg2Block: unable to allocate block\n");
#endif
	return(-1);
}

/*-----------------------------------------------------------------------------
   Function: 	uStoreMM::uStoreAllocBlock
   Description: Allocate a block of micro-store from either segment 1 or 2.
   Returns:		The allocated micro-store address
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uStoreMM::uStoreAllocBlock(int uSeg)
{
	switch(uSeg){
		case 0:				// allocate from any free segment
			int blk;
			if((blk = allocSeg1Block()) >= 0) return(blk);
			return(allocSeg2Block());
			break;
		case 1:				// allocate from segment 1
			return(allocSeg1Block());
			break;
		case 2:				// allocate from segment 2
			return(allocSeg2Block());
			break;
	}

	return(-1);
}

/*-----------------------------------------------------------------------------
   Function: 	uStoreMM::uStoreFreeBlock
   Description: Free a block of micro-store that was allocated by uStoreAllocBlock.
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uStoreMM::uStoreFreeBlock(int uAddr)
{
   unsigned int i;

#ifdef _DBG_PRINT
   printf("uStoreFreeBlock: address=%d\n", uAddr);
#endif

	if(uAddr < MAX_USTORE_PER_SEG){
		if((unsigned int)uAddr < startUaddr) return;
		i = (uAddr - startUaddr) / blkSize;
		blocks[i] = 0;
#ifdef _DBG_PRINT
        printf("uStoreFreeBlock: freed segment 1 block %d\n", i);
#endif
	}
	else if (uAddr < MAX_C0_USTORE){
		if((unsigned int)uAddr < seg2StartUaddr) return;
		i = (uAddr - seg2StartUaddr) / seg2BlkSize;
		seg2Blocks[i] = 0;
#ifdef _DBG_PRINT
        printf("uStoreFreeBlock: freed segment 2 block %d\n", i);
#endif
	}

   return;
}

