/* DriverLocks.h
 * Fdriver Engine Lock Management file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: FDriver
 * author: sk, Mar 99 Initial version
 * $History: DriverLocks.h $
 * 
 * *****************  Version 7  *****************
 * User: Skornfel     Date: 6/28/99    Time: 11:43a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Runs on 1200 if SA_1200 defined, use multithreaded model.
 * Tested with ISR attached directly to int 10. To attach to "ueng"
 * Interrupt change one line in .cpp file
 * 
 * *****************  Version 6  *****************
 * User: Skornfel     Date: 3/25/99    Time: 7:25p
 * Updated in $/development/driver
 * adopted to UCOS
 * 
 * *****************  Version 2  *****************
 * User: Skornfel     Date: 3/16/99    Time: 2:38p
 * Updated in $/development/driver
 * . New for BL5:  HoldTxToEngine(int engine) and ResumeTxToEngine(int
 * engine)
 * . lots of small changes by other developers broke the Unit testing -
 * Fixed
 * . "multi chip" configuration changes made by other developers will not
 * really work(just for info) 
 * 
 * 
 * *****************  Version 1  *****************
 * User: Skornfel     Date: 3/16/99    Time: 2:25p
 * Created in $/development/driver
 * NEW lock sss for the driver
 * 
 * ---------------------------------------------------------------------
 */
#ifndef DRV_LOCKS
#define DRV_LOCKS


#include <iostream.h>
#include "TS.h" 
#include "MessageDstr.h"

class MsgStateMachinePtrs;
class MsgStateSemaphores; 
//---------------------------------------------------------------------------------------------------------------------------

//Resetable Lock to use in the FDriever
class ResetableLock{

public:
	ResetableLock(){LockCount=0;};

	~ResetableLock(){};

	//take a lock and increment lock counter
	inline void
		Take(){
			LockCount++;
#ifdef TEST_SIMPLE
			cout << endl << LockCount  << " ................... LockCount" << endl;
#endif
			Lock.Take();
		};


	//unlock (give) a lock number of times, recorded by Take()
	inline void
		Reset(){
		int LoopCount = LockCount;
			for (int i=0; i<LoopCount; i++){
#ifdef TEST_SIMPLE
			cout << endl << LockCount  << " ................... LockCount RESET" << endl;
#endif
				this->Give();
				LockCount--;
			}
		};

private:
	//simple give 
	inline void
		Give(){
			Lock.Give();
		};

private:
	Semaphore Lock;		//lock fron OS
	int LockCount;		//Just in case we need to store more than one Take()
};


//Engine Lock Management
class EngineLockManagement{

	enum Hold{
		HOLD_ENGINE,
		NORMAL_EXECUTION
	};


public:
	EngineLockManagement(unsigned int & ireg, 
		MsgStateMachinePtrs & arr_rx_disp_ptrs
//#ifndef ONE_RX_THREAD
		,MsgStateSemaphores & state_semaphores
//#endif //ONE_RX_THREAD	
		)
		:iReg(ireg),_RxMsgStateSemaphores(state_semaphores),_ArrRxDisp_prts(arr_rx_disp_ptrs){
		EngineHoldInicator[0]=NORMAL_EXECUTION;
		EngineHoldInicator[1]=NORMAL_EXECUTION;	
		EngineHoldInicator[2]=NORMAL_EXECUTION;	
		EngineHoldInicator[3]=NORMAL_EXECUTION;	
		EngineHoldInicator[4]=NORMAL_EXECUTION;	
		EngineHoldInicator[5]=NORMAL_EXECUTION;
		for (int c=0; c< 24; c++)ThreadIndicator[c]=NORMAL_EXECUTION;
		storedEvents=0;
	};

	~EngineLockManagement(){};

	inline void  //TX to uEngine 
		CheckForHold(int engine){
			if (EngineHoldInicator[engine] == HOLD_ENGINE){
				Lock[engine].Take();
			}
		};

	inline void  //RX from uEngines
		CheckForAccept(int thread){
			if (ThreadIndicator[thread] == HOLD_ENGINE){
				//clear ireg sourse
				iReg &= ~( 1 << thread);
				//store 
				storedEvents |= ( 1 << thread);
			}
		};

	inline void
		HoldEngine(int engine){
			EngineHoldInicator[engine]=HOLD_ENGINE;
			ThreadIndicator[(engine*4+0)]=HOLD_ENGINE;
			ThreadIndicator[(engine*4+1)]=HOLD_ENGINE;
			ThreadIndicator[(engine*4+2)]=HOLD_ENGINE;
			ThreadIndicator[(engine*4+3)]=HOLD_ENGINE;
		};

	inline void
		ReleaseEngine(int engine);
//#ifdef XXX	{
//			EngineHoldInicator[engine]=NORMAL_EXECUTION;
//			Lock[engine].Reset();//Give();
//			//restore IREGbits();
//		  /*iReg |= ( 1 << (engine +0));
//			iReg |= ( 1 << (engine +1));
//			iReg |= ( 1 << (engine +2));
//			iReg |= ( 1 << (engine +3));*/
//			//call unblocked states
//			//RxCnxtDispatcher
//			for(int bit=0; bit<24; bit++){
//
//				if ( storedEvents & (1 << bit)){
//					storedEvents = storedEvents & ~( 1<< bit); //Clear the bit
//#ifdef ONE_RX_THREAD
//                    (_ArrRxDisp_prts.ptr_RxMsgDptr[bit])->FBoxToCoreDispatch();
//#else  //ONE_RX_THREAD
//				    (_RxMsgStateSemaphores.ptr_Semaphore[bit])->Give();
//#endif //ONE_RX_THREAD
//
//				};
//			};
//		};
//#endif 

private:

	int EngineHoldInicator[6];					// holded engines
	int ThreadIndicator[24];					// holded threads
	int storedEvents;							// events to store for the processing after ReleaseEngine()
	unsigned int & iReg;						// Reference to Ireg (may have some issues check later)
	ResetableLock Lock[6];						// Locks. One per engine
//#ifdef ONE_RX_THREAD
	MsgStateMachinePtrs & _ArrRxDisp_prts;		// Arrays of state machines(pointes to)
//#else  //ONE_RX_THREAD
	MsgStateSemaphores & _RxMsgStateSemaphores;	// Semaphores to unlock the Rx Thread
//#endif //ONE_RX_THREAD
 
};









//-------------------------------------------------------------------------------------------------------------------------
#endif //DRV_LOCKS




