/* GBoxContext.h
 * Fdriver Rx Source Dispetcher file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: FDriver
 * author: sk, Feb 98 Initial version
 * $History: GboxContext.h $
 * 
 * *****************  Version 25  *****************
 * User: Dmwhite      Date: 5/09/00    Time: 9:34a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * more ostype cleanup
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 12/08/99   Time: 2:19p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Fixed for AMBAIO simulation mode
 * 
 * *****************  Version 20  *****************
 * User: Tclancyx     Date: 8/20/99    Time: 7:02p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Change reference to iostream::operator<<(void *) to (unsigned int)
 * since vxWorks doesn't seem to be able to print a pointer to void
 * 
 * *****************  Version 18  *****************
 * User: Skornfel     Date: 6/28/99    Time: 11:43a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Runs on 1200 if SA_1200 defined, use multithreaded model.
 * Tested with ISR attached directly to int 10. To attach to "ueng"
 * Interrupt change one line in .cpp file
 * 
 * *****************  Version 17  *****************
 * User: Skornfel     Date: 3/25/99    Time: 7:25p
 * Updated in $/development/driver
 * adopted to UCOS
 * 
 * *****************  Version 13  *****************
 * User: Skornfel     Date: 3/16/99    Time: 2:38p
 * Updated in $/development/driver
 * . New for BL5:  HoldTxToEngine(int engine) and ResumeTxToEngine(int
 * engine)
 * . lots of small changes by other developers broke the Unit testing -
 * Fixed
 * . "multi chip" configuration changes made by other developers will not
 * really work(just for info) 
 * 
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 2/12/99    Time: 6:10p
 * Updated in $/development/driver
 * Updated for multi-chip/auto_ambio related changes
 * 
 * *****************  Version 10  *****************
 * User: Hqianx       Date: 9/17/98    Time: 4:28p
 * Updated in $/base_level3/driver
 * Added AMBA IO interface.
 * 
 * *****************  Version 9  *****************
 * User: Skornfel     Date: 7/23/98    Time: 1:53p
 * Updated in $/base_level3/driver
 * Init context data to 0x0
 * 
 * *****************  Version 8  *****************
 * User: Skornfel     Date: 5/21/98    Time: 2:21p
 * Updated in $/base_level2/Driver
 * COPYRIGHT (c)  1998 BY  INTEL  CORPORATION
 * 
 * *****************  Version 7  *****************
 * User: Skornfeld    Date: 5/04/98    Time: 12:46p
 * Updated in $/base_level2/Driver
 * The same functionality compare to
 * the previous Check In. Fixed locks
 * and Queue. No output to the console.
 * 
 * *****************  Version 6  *****************
 * User: Skornfeld    Date: 4/17/98    Time: 10:52a
 * Updated in $/base_level2/Driver
 * BL2 Driver  
 * 
 * *****************  Version 5  *****************
 * User: Skornfeld    Date: 3/10/98    Time: 6:50p
 * Updated in $/base_level1/Driver
 * Digial Headers, Code cleaup, comments 
 * ---------------------------------------------------------------------
 */


#ifndef GBOX_CONTEXT
#define GBOX_CONTEXT

#include "hal_sram.h"
#include "MessageHead.h"
#include "DriverLocks.h"
#include "ostype.h"

//24 bits in CSR valid
enum BitsInCsr{ BITS_IN_CSR =24 };
enum GBox { NUMBER_OF_GBOXES =6, CONTEXTS_PER_GBOX=4 };



//FOR TRANSACTOR
// Class encapsulates the Context Registers  / Mbox in SRAM or SDRAM
// Context can be instantiated as Read or Write and will be used in 
// Class Context 
//
#ifndef SA_1200
class CnxtMbox{
public:
	CnxtMbox(int context, int base){
		Context=context;
		Base=base;
		Set0(0);Set1(0);Set2(0);Set3(0);
	};
	 
	~CnxtMbox(){};

	inline void Set0(unsigned int  r0)  { Word0 = r0; SRAM_WRITE((Base+(Context*STEP)), Word0);}
	inline void Set1(unsigned int  r1)  { Word1 = r1; SRAM_WRITE((Base+(Context*STEP)+1), Word1);}
	inline void Set2(unsigned int  r2)  { Word2 = r2; SRAM_WRITE((Base+(Context*STEP)+2), Word2);}
	inline void Set3(unsigned int  r3)  { Word3 = r3; SRAM_WRITE((Base+(Context*STEP)+3), Word3);}

	unsigned int  inline Get0()  { SRAM_READ((Base+Context*STEP),   &Word0);return Word0;}
	unsigned int  inline Get1()  { SRAM_READ((Base+Context*STEP+1), &Word1);return Word1;}
	unsigned int  inline Get2()  { SRAM_READ((Base+Context*STEP+2), &Word2);return Word2;}
	unsigned int  inline Get3()  { SRAM_READ((Base+Context*STEP+3), &Word3);return Word3;}


	void Show(void){
		cout
#if (OS!=UCOS)	
		<< hex	
#endif
		<< ((unsigned int)&Word0) << " Word0=" << Word0 << " Word1=" << Word1 << " Word2=" << Word2 << " Word3=" << Word3 << "\n";
	};

	inline void 
	Set(long w0, long w1=0, long w2=0, long w3=0){
		Word0 = w0;	Word1 = w1;	Word2 = w2;	Word3 = w3;
	}
private:
	unsigned int  Word0;
	unsigned int  Word1;
	unsigned int  Word2;
	unsigned int  Word3;
	//TXT to calculate offsets 
	int Context;		//context number 
	int Base;			//base adress of the area in SRAM/SDRAm
};
#endif //


//------------------------------------------------------------------------------------------
#ifdef SA_1200
class CnxtMbox{
public:
static void * operator new(size_t size, void * ptr ){ return ptr; }
static void   operator delete( void * ptr){ return; }

public:
	CnxtMbox(/*dbg*/int context){ Set(context); cout << " CTOR CnxtMbox @ "  << ((unsigned int)this) << endl;};
	 
	~CnxtMbox(){};

	inline void Set0(unsigned int  w0)  { Word0 = w0; }
	inline void Set1(unsigned int  w1)  { Word1 = w1; }
	inline void Set2(unsigned int  w2)  { Word2 = w2; }
	inline void Set3(unsigned int  w3)  { Word3 = w3; }

	unsigned int  inline Get0() const { return Word0;}
	unsigned int  inline Get1() const { return Word1;}
	unsigned int  inline Get2() const { return Word2;}
	unsigned int  inline Get3() const { return Word3;}

	void Show(void){cout
#if (OS!=UCOS)	
		<< hex	
#endif
		<< ((unsigned int)&Word0) << " Word0=" << Word0 << " Word1=" << Word1 << " Word2=" << Word2 << " Word3=" << Word3 << "\n";
	};

	inline void 
	Set(long w0, long w1=0, long w2=0, long w3=0){ 
		Word0 = w0;	Word1 = w1;	Word2 = w2;	Word3 = w3;
	}

private:
	unsigned int  Word0;
	unsigned int  Word1;
	unsigned int  Word2;
	unsigned int  Word3;
};

#endif //A_1200_HW





// Class encapsulates the Context (one Fengine / _uCode thread)
// with Read and Write Registers or Locations in SRAM / DRAM 
// for   Read and Write Mboxes
//

class GboxCnxt{

public:
	GboxCnxt(
		     CoreToFboxCsr & csr, EngineLockManagement & elm, int thrd, int engn)
		: _SignalCsr(csr),_ELM(elm)
		{
		thread = thrd;
		engine = engn;
		u_thread = engine*4 + thread;
#ifndef SA_1200
		_ReadMbox =  new CnxtMbox(engn*CONTEXTS_PER_GBOX + thrd, CORE_TO_UTHREAD_MAILBOXES);  
		_WriteMbox = new CnxtMbox(engn*CONTEXTS_PER_GBOX + thrd, UTHREAD_TO_CORE_MAILBOXES);  
#else  //#ifdef SA_1200
		int context=engn*CONTEXTS_PER_GBOX + thrd;
		//cout << "CTOR GboxCnxt CONTEXT   "<< context << endl;
		const int SRAM_START = 0x10000000;
		_ReadMbox =  new((void *)(SRAM_START + (CORE_TO_UTHREAD_MAILBOXES+(context*STEP))*4) ) CnxtMbox(context);
		_WriteMbox = new((void *)(SRAM_START + (UTHREAD_TO_CORE_MAILBOXES+(context*STEP))*4) ) CnxtMbox(context);  
#endif //SA_1200
		
		
		SignalThreadLock.Give();
	//cout << "after GboxCnxt CTOR" << endl;
	};
	~GboxCnxt(){
		delete _ReadMbox;
		delete _WriteMbox;
	};

	// Lets just get the reference
	//
	inline CnxtMbox 
	GetMessage(void){return *_WriteMbox;};

	// Send the massage to the BOX / _uCode thred
	//
	///
	inline void
    PutMessage(MessageHeader header){

		//Check if Engine on Hold 
		_ELM.CheckForHold(engine);

		//_ReadMbox->Set0((long)header);
		_ReadMbox->Set0(header.GetHeader());
		_ReadMbox->Set1(header.GetDataWord1());
		_ReadMbox->Set2(header.GetDataWord2());
		_ReadMbox->Set3(header.GetDataWord3());
		//
		//Show();
		SignalThreadLock.Take();
		_SignalCsr.SignalThread( thread, engine); //
		SignalThreadLock.Give();
	}; 

	void Show(void){
		cout << "ReadMbox  "<< thread << ":" << engine << " ";_ReadMbox->Show();
	    cout << "WriteMbox "<< thread << ":" << engine << " ";_WriteMbox->Show();
	};

	// get reference for "read/write tx reg" 
    //
	inline CnxtMbox 
    * GetRdMbox(void){ return _ReadMbox;}; 
    
	inline CnxtMbox 
	* GetWrMbox(void){ return _WriteMbox;}; 
	 
	int u_thread;
private:
	CnxtMbox * _ReadMbox;			// pointer to the ReadMbox;
    CnxtMbox * _WriteMbox;			// pointer to the WriteMbox;
	CoreToFboxCsr & _SignalCsr;		// reference of the Signal Reg (core _> to-> _uCode) 
	EngineLockManagement & _ELM;	// reference to EngineLockManagement
	int thread;						// Thread number
	int engine;						// Engine number
	Semaphore SignalThreadLock; 
		// to calculate offsets 
	int Context;		//context number 
	int Base;			//base adress of the area in SRAM/SDRAm

};


#endif //GBOX_CONTEXT
