/* MessageDstr.h
 * Fdriver Rx Message Dispatcher file 
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 * system: SA1200
 * subsystem: FDriver
 * author: sk, Feb 98 Initial version
 * $History: MessageDstr.h $
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 12/06/99   Time: 10:51a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * 
 * *****************  Version 20  *****************
 * User: Skornfel     Date: 10/13/99   Time: 10:53a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Corrected to support OS type and IOSTYLE 
 * 
 * *****************  Version 18  *****************
 * User: Skornfel     Date: 6/28/99    Time: 11:43a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Runs on 1200 if SA_1200 defined, use multithreaded model.
 * Tested with ISR attached directly to int 10. To attach to "ueng"
 * Interrupt change one line in .cpp file
 * 
 * *****************  Version 17  *****************
 * User: Skornfel     Date: 3/25/99    Time: 7:25p
 * Updated in $/development/driver
 * adopted to UCOS
 * 
 * *****************  Version 13  *****************
 * User: Skornfel     Date: 3/16/99    Time: 2:38p
 * Updated in $/development/driver
 * . New for BL5:  HoldTxToEngine(int engine) and ResumeTxToEngine(int
 * engine)
 * . lots of small changes by other developers broke the Unit testing -
 * Fixed
 * . "multi chip" configuration changes made by other developers will not
 * really work(just for info) 
 * 
 * 
 * *****************  Version 11  *****************
 * User: Hqianx       Date: 9/28/98    Time: 4:04p
 * Updated in $/base_level3/driver
 * Commented line "OneWayRequestSignalDone.Give()" out.
 * 
 * *****************  Version 10  *****************
 * User: Drjohns1     Date: 6/17/98    Time: 2:49p
 * Updated in $/base_level3/Driver
 * Linux/GNU port
 * 
 * *****************  Version 9  *****************
 * User: Skornfel     Date: 5/21/98    Time: 2:21p
 * Updated in $/base_level2/Driver
 * COPYRIGHT (c)  1998 BY  INTEL  CORPORATION
 * 
 * *****************  Version 8  *****************
 * User: Skornfeld    Date: 5/19/98    Time: 2:53p
 * Updated in $/base_level2/Driver
 * 2way message may be sent to the driver's queue interface
 * and it will be returned to the Dlib's queue
 * 
 * *****************  Version 7  *****************
 * User: Skornfeld    Date: 5/08/98    Time: 3:23p
 * Updated in $/base_level2/Driver
 * fix in CoreTOFBox(...) - unlock moved to be before return :( 
 * 
 * *****************  Version 6  *****************
 * User: Skornfeld    Date: 5/04/98    Time: 12:46p
 * Updated in $/base_level2/Driver
 * The same functionality compare to
 * the previous Check In. Fixed locks
 * and Queue. No output to the console.
 * 
 * *****************  Version 5  *****************
 * User: Skornfeld    Date: 4/17/98    Time: 10:52a
 * Updated in $/base_level2/Driver
 * BL2 Driver  
 * 
 * *****************  Version 4  *****************
 * User: Skornfeld    Date: 3/10/98    Time: 6:50p
 * Updated in $/base_level1/Driver
 * Digial Headers, Code cleaup, comments 
 * ---------------------------------------------------------------------
 */


#ifndef MESSAGE_DSPTR
#define MESSAGE_DSPTR


class MsgStateMachinePtrs;
class MsgStateSemaphores;
class EngineLockManagement;


#include "TS.h"
#include "SignalCsr.h"
#include "GboxContext.h"
#include "MessageHead.h"




extern void CallFunc(MessageHeader & message_to_core);
extern "C" int LibD_Invoke (unsigned int * msg );
extern "C" int LibD_RouteMsg(unsigned int *msg);

extern "C" int LibD_InvokeE (unsigned int * msg, int thread );  //for testing without libD
extern "C" int LibD_PutMsgQueE (void *msg, int thread );


//dj:
#if defined(__DECCXX)
typedef enum {false=0, true} bool;
#endif


//-----------------------------------------------------------------------------
// Abstract Message class (Java)
// forces inherited classes to have 
// "ProcessIt" method
//
class Message{

public:
	Message(){}; 
	
	virtual 
	~Message (){};

    //Messages have to supply the following methods:
	virtual void ProcessIt(void)=0; 
	virtual void Show(void)=0;
};

//------------------------------------------------------------------------------

//ACK message Nothing to do Just unblock the State machine
//

class MessageAck:public Message{

public:
	MessageAck(GboxCnxt & context, Semaphore  & signal):_Context(context), _OneWayRequestSignalDone(signal){
        //cout << "MsgAck ctor " ;
	};
	virtual ~MessageAck(){};

	virtual void
	Show(void){
		cout << ":-) I am MessageAck Message" << "\n" ;
	};

	virtual void
	ProcessIt(){

			RxHead.SetHeader((_Context.GetMessage()).Get0());
			//RxHead.Show();
#ifdef DEBUG_SOK
			RxHead.SockShow(_Context.u_thread);    //print to Socket
#endif		
			//Release LOCK _OneWayRequestSignalDone
			_OneWayRequestSignalDone.Give();

		cout << "### ACK from uThread --- Done" << "\n";
	};


private:
	MessageHeader RxHead;					// Lets do NOT do it on the stack
	GboxCnxt & _Context;					// reference to the context only RxRegs Used
	Semaphore  & _OneWayRequestSignalDone;  // reference to the Semaphore to give
};

//------------------------------------------------------------------------------

// MessageReply - Two way reply -  message "Nothing to do" Unblock the State machine..
// ...... THE message came back from _uCdode - pass IT back to Core
//

class MessageReply:public Message{

public:
	
	MessageReply(GboxCnxt & context, Semaphore & signal):_Context(context),_TwoWayRequestSignal(signal){
        //cout << "MsgReply ctor " ;
	};
	virtual ~MessageReply(){};

	virtual void
	Show(void){
		cout << ":-) I am MessageReply Message" << "\n" ;
	};

	virtual void
	ProcessIt(){
		//Release LOCK  _TwoWayRequest
		_TwoWayRequestSignal.Give();
		cout << "### REPLY from uThread --- Done" << "\n";
	};


private:
	GboxCnxt & _Context;                    // reference to the context onlly RxRegs Used
	Semaphore & _TwoWayRequestSignal;		// reference to the Semaphore to Give
};


//------------------------------------------------------------------------------

// OneWayRequest - One way request -  message 
// Call the core do not expect the massage back
// (with queue, put on the queue of libD or Host or ....)
// send the ACK message to _ucode 
// 

class OneWayRequest:public Message{


public:
	
	OneWayRequest(GboxCnxt & context):_Context(context){
        //cout << "MsgOneWayReq ctor " ;
	};
	virtual ~OneWayRequest(){};

	virtual void
	Show(void){
		cout << ":-) I am OneWayRequest Message" << endl;
	};

	virtual void
	ProcessIt(){

	//Rx
		MessageHeader RxHead;
		RxHead.SetHeader((_Context.GetMessage()).Get0()); 
		RxHead.SetDataWord1((_Context.GetMessage()).Get1()); 
		RxHead.SetDataWord2((_Context.GetMessage()).Get2()); 
		RxHead.SetDataWord3((_Context.GetMessage()).Get3()); 
		
		//RxHead.Show();
#ifdef DEBUG_SOK
		RxHead.SockShow(_Context.u_thread);  //to the Socket
#endif		
		// CALL get the message and RETURN WITH NEW MESSAGE
		//
#ifdef NO_LIB
		LibD_PutMsgQueE((void *) &RxHead, _Context.u_thread );  //for testing without libD
#else
		LibD_RouteMsg((unsigned int *) &RxHead );
#endif
		//Tx
		TxHead.SetDriverAck();               //Ack back to _uCode 
#ifdef DEBUG_SOK
		TxHead.SockShow(_Context.u_thread); //to Socket
#endif
		_Context.PutMessage(TxHead);		//Ack IT now!!!
		cout << "### ONE WAY Request from uThread --- Done" << "\n";

	};


private:
	MessageHeader TxHead;		//Lets do not do it on the stack 
	GboxCnxt & _Context;        // RxTxRegs Used
};

//-----------------------------------------------------------------------------

// TwoWayRequest - Two way request -  message 
// Call the core EXPECT the message back
// forward the Core Message to the _uCode
// 

class TwoWayRequest:public Message{

public:
	TwoWayRequest(GboxCnxt & context):_Context(context){
         //cout << "MsgTwoWayReq ctor";
	};
	virtual ~TwoWayRequest(){};

	virtual void
	Show(void){
		cout << ":-) I am TwoWayRequest Message" << endl;
	};


	virtual void
	ProcessIt(){

			//Rx
			MessageHeader RxHead;
			RxHead.SetHeader((_Context.GetMessage()).Get0()); 
			RxHead.SetDataWord1((_Context.GetMessage()).Get1()); 
			RxHead.SetDataWord2((_Context.GetMessage()).Get2()); 
			RxHead.SetDataWord3((_Context.GetMessage()).Get3()); 
		
			//SK print it on RX 
			//RxHead.Show();
#ifdef DEBUG_SOK
			RxHead.SockShow(_Context.u_thread);   //to socket
#endif		
			//CALL AND RETURN WITH NEW MESSAGE
#ifdef NO_LIB
			LibD_InvokeE((unsigned int *) &RxHead, _Context.u_thread  );
#else
			LibD_Invoke((unsigned int *) &RxHead );
#endif
		
			//RxHead.Show();
#ifdef DEBUG_SOK
			RxHead.SockShow(_Context.u_thread);    //to Socket
#endif		
			//Tx
			_Context.PutMessage(RxHead);
			cout << "### TWO WAY Request from uThread --- Done" << "\n";
	};

private:
	//RxMessageHeader RxHead;				//SK problem !!!!  I will do it on the stack!!!
	GboxCnxt  & _Context;					//Rx and Tx Regs Used
};


//-----------------------------------------------------------------------------
//enum MsgType { MSG_ONEWAY_REQ, MSG_DRIVER_ACK, MSG_TWOWAY_REQ, MSG_TWOWAY_REPLY };
enum MsgTypes { MSG_TYPE_NUM = 4 };


// RxMsgDispatcher class, instance of this class will 
// dispatch messages on Rx from _uCode
// 
class RxMsgDispatcher{

public:
	RxMsgDispatcher(CoreToFboxCsr & signal_reg, EngineLockManagement & ELM, int thrd, int engn)
		:_SignalReg(signal_reg){

		//OneWayRequestSignalDone.Give();  ///SK I do not know yet if this is a left over or BUG or something else
		CoreToFBoxLOCK.Give();
		
		_Context = new GboxCnxt( _SignalReg, ELM, thrd, engn);
		
		//_Context->Show();

		p_msg[MSG_DRIVER_ACK]   = new MessageAck    (*_Context, OneWayRequestSignalDone);
		p_msg[MSG_TWOWAY_REPLY] = new MessageReply  (*_Context, TwoWayRequestSignal);
		p_msg[MSG_ONEWAY_REQ]   = new OneWayRequest (*_Context);
		p_msg[MSG_TWOWAY_REQ]   = new TwoWayRequest (*_Context);

		//cout << "Created Message Dispatcher for Thread " << thrd  << ", Engine " << engn << endl;
		cout << " MsgDisp Th:" << thrd  << ", En:" << engn;
	};

	virtual ~RxMsgDispatcher(){

		delete _Context;
		delete p_msg[MSG_DRIVER_ACK];
		delete p_msg[MSG_TWOWAY_REPLY];
		delete p_msg[MSG_ONEWAY_REQ];
		delete p_msg[MSG_TWOWAY_REQ];
	};
	

	//Method for the Fdriver to dispetch Rx messages
	inline void
	FBoxToCoreDispatch(){
cout << "MESSAGE CAME FROM Ucode !!!!!!!!!!!!!!!!!!!!!!" << endl;
		MessageHeader RxMessHd((_Context->GetMessage()).Get0());

			//Check the Memory located Async message	
			//		if ( RxMessHd.GetAckMem() == 1){ // ACK from Async. Memory located message ?
			//			cout << "ProcessAsync();\n" ;//3 ACK "Async - MEM" 
			//			return;
			//		};
		
		msg_head_temp = RxMessHd.GetTriger();  // GetTriger(); 

			//print the triger
			//cout << "* rx triger* " << hex << msg_head_temp << " ****" << endl;

		//do "IT" 
		if ( (msg_head_temp >= 0) &&  (msg_head_temp < MSG_TYPE_NUM)){	//check if valid  
			p_msg[msg_head_temp]->ProcessIt();							//
		};
	}; 




	//Method  to deal with messages comming from the Core
	inline void 
	CoreToFBox(MessageHeader & header, bool queued = false ){
	
	unsigned int one_way_return = header.GetPend();

	CoreToFBoxLOCK.Take();//L O C K

		_Context->PutMessage( header );    //Give it to Ucode
			

  	    //QUEUE msg queued?? ACK will delete the queue element
	    //If one way message return from call but do not take ..................
	    
		// O N E  W A Y
		//
		if ( one_way_return == ONE_WAY) {
			OneWayRequestSignalDone.Take(); //Wait for Ucode to consume message (for Ucode Ack)
			
	CoreToFBoxLOCK.Give();//U N L O C K

			return;		//out of one way message
		};
		
	
		//OR T W O W A Y
		//

		//check if message is queued if... return it through the Queue

		if ( queued ) { 
			
		TwoWayRequestSignal.Take();        //Wait for Ucode to Reply to the Core 
		
		//Prepare the header to return
		//
		header.SetHeader(_Context->GetMessage().Get0());
		header.SetDataWord1(_Context->GetMessage().Get1());
		header.SetDataWord2(_Context->GetMessage().Get2());
		header.SetDataWord3(_Context->GetMessage().Get3());
		//send it through the QUEUE back

#ifdef NO_LIB
		LibD_PutMsgQueE((void *) &header, header.GetSource() );  //for testing without libD
#else
		LibD_RouteMsg((unsigned int *) &header );
#endif

	CoreToFBoxLOCK.Give();//U N L O C K

		return;		//out of queued two way message
		}


		// ....or TWO WAY NOT QUEUEd
		//
				
		TwoWayRequestSignal.Take();			//Wait for Ucode to Reply to the Core
		
		//Prepare the header to return
		//
		header.SetHeader(_Context->GetMessage().Get0());
		header.SetDataWord1(_Context->GetMessage().Get1());
		header.SetDataWord2(_Context->GetMessage().Get2());
		header.SetDataWord3(_Context->GetMessage().Get3());

		//SK two ways to print
		//header.Show();
		//_Context->GetMessage().Show();
		//header.SockShow(_Context->u_thread);	//to Socket

		//out of CoreToFBox
		
	CoreToFBoxLOCK.Give();//U N L O C K

		return;		//out of two way message
	};



	//Get the context
	//
	inline GboxCnxt *
	GetContext(void){
			return _Context;
	};

	inline void MboxShow(void){_Context->Show();}


private:
	Message    * p_msg[MSG_TYPE_NUM];	// how many types of messages 
	GboxCnxt   * _Context;				// pointer to the Context;
	CoreToFboxCsr & _SignalReg;			// reference to the CoreToFbox  Reg 
	Semaphore  TwoWayRequestSignal;     // Two Way Request Signal
	Semaphore  OneWayRequestSignalDone; // One Way Request Done Signal 
	Semaphore  CoreToFBoxLOCK;			// LOCK the CoreToFBox()
	int msg_head_temp;					// Temp - out of the stack
};

//---------------------------------------------------------------------------

class MsgStateMachinePtrs {
public:
	RxMsgDispatcher * ptr_RxMsgDptr[NUMBER_OF_GBOXES * CONTEXTS_PER_GBOX];
	};

class MsgStateSemaphores {
public:
	Semaphore * ptr_Semaphore[NUMBER_OF_GBOXES * CONTEXTS_PER_GBOX];
	};

#endif //MESSAGE_DSPTR

