/* MessageHead.h
 * Fdriver Rx Message Dispatcher file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: FDriver
 * author: sk, Feb 98 Initial version
 * $History: MessageHead.h $
 * 
 * *****************  Version 19  *****************
 * User: Dmwhite      Date: 5/09/00    Time: 9:34a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * more ostype cleanup
 * 
 * *****************  Version 14  *****************
 * User: Skornfel     Date: 6/28/99    Time: 11:43a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Runs on 1200 if SA_1200 defined, use multithreaded model.
 * Tested with ISR attached directly to int 10. To attach to "ueng"
 * Interrupt change one line in .cpp file
 * 
 * *****************  Version 13  *****************
 * User: Skornfel     Date: 3/25/99    Time: 7:25p
 * Updated in $/development/driver
 * adopted to UCOS
 * 
 * *****************  Version 9  *****************
 * User: Skornfel     Date: 3/16/99    Time: 2:39p
 * Updated in $/development/driver
 * . New for BL5:  HoldTxToEngine(int engine) and ResumeTxToEngine(int
 * engine)
 * . lots of small changes by other developers broke the Unit testing -
 * Fixed
 * . "multi chip" configuration changes made by other developers will not
 * really work(just for info) 
 * 
 * 
 * *****************  Version 7  *****************
 * User: Skornfel     Date: 5/21/98    Time: 2:21p
 * Updated in $/base_level2/Driver
 * COPYRIGHT (c)  1998 BY  INTEL  CORPORATION
 * 
 * *****************  Version 6  *****************
 * User: Skornfeld    Date: 5/04/98    Time: 12:46p
 * Updated in $/base_level2/Driver
 * The same functionality compare to
 * the previous Check In. Fixed locks
 * and Queue. No output to the console.
 * 
 * *****************  Version 5  *****************
 * User: Skornfeld    Date: 4/17/98    Time: 10:52a
 * Updated in $/base_level2/Driver
 * BL2 Driver  
 * 
 * *****************  Version 4  *****************
 * User: Skornfeld    Date: 3/10/98    Time: 6:50p
 * Updated in $/base_level1/Driver
 * Digial Headers, Code cleaup, comments 
 * ---------------------------------------------------------------------
 */





#ifndef MESSAGE_HEAD
#define MESSAGE_HEAD
#include "ostype.h"

//enum MsgType { NOMSG, MSG_ACK=0x1, MSG_REPLY=0x0, MSG_IP=(0x1<<1), MSG_SDISP=(0x2<<1), MSG_ATM=(0x3<<1) };

enum Pend { ONE_WAY, TWO_WAY };
enum MsgType { MSG_ONEWAY_REQ, MSG_DRIVER_ACK, MSG_TWOWAY_REQ, MSG_TWOWAY_REPLY };

class MessageHeader{

	enum Shifts{
		TRIGER_MSK_SH =30, // "PEND_MSK + ACK_REPLY_MSK" Shift
		PEND_SH =31,       // pend 
		ACK_REPLY_SH =30,
		FORMAT_SH =28,
		SOURCE_SH =16,
		DEST_SH =8,
		FUNCTION_SH =0,
		TRANS_ID_SH =24
	};

	enum Masks{
		TRIGER_MSK =0x3,// PEND_MSK + ACK_REPLY_MSK
		PEND_MSK =0x1,
		ACK_REPLY_MSK = 0x1,
		FORMAT_MSK = 0x3,
		SOURCE_MSK = 0xff,
		DEST_MSK = 0xff,
		FUNCTION_MSK = 0xff,
	    TRANS_ID_MSK = 0xf
	};
public:		
	MessageHeader(){ Header = 0;};
	MessageHeader(int reg0){ Header = reg0;};

	~MessageHeader(){};
 
	operator int() {return Header;};

	inline unsigned int GetHeader()    const { return Header; };
	inline unsigned int GetDataWord1() const { return DataWord1; };
	inline unsigned int GetDataWord2() const { return DataWord2; };
	inline unsigned int GetDataWord3() const { return DataWord3; };

	inline unsigned int GetTriger()    const { return ( Header >> TRIGER_MSK_SH) & TRIGER_MSK; };
	inline unsigned int GetPend()      const { return ( Header >> PEND_SH) & PEND_MSK; };
	inline unsigned int GetAck()       const { return ( Header >> ACK_REPLY_SH) & ACK_REPLY_MSK;};
	inline unsigned int GetFormat()    const { return ( Header >> FORMAT_SH) & FORMAT_MSK;};
	inline unsigned int GetSource()    const { return ( Header >> SOURCE_SH) & SOURCE_MSK;};
	inline unsigned int GetDest()      const { return ( Header >> DEST_SH) & DEST_MSK;};
	inline unsigned int GetFunction()  const { return ( Header >> FUNCTION_SH) & FUNCTION_MSK;};
	inline unsigned int GetTransId()   const { return  (Header >> TRANS_ID_SH) & TRANS_ID_MSK;};

//
	inline void SetHeader(int header)   { Header = header; };
	inline void SetDataWord1(int data)  { DataWord1 = data; };
	inline void SetDataWord2(int data)  { DataWord2 = data; };
	inline void SetDataWord3(int data)  { DataWord3 = data; };


	inline void SetDriverAck(void){
		SetAck(1);
		SetPend(ONE_WAY); 
	};

	inline void SetTwoWayReply(void){
		SetAck(1);
		SetPend(TWO_WAY); 
	};

	inline void SetOneWayRequest(void){
		SetAck(0);
		SetPend(ONE_WAY); 
	};

	inline void SetTwoWayRequest(void){
		SetAck(0);
		SetPend(TWO_WAY); 
	};


	inline void SetPend(int pend){
		Header &= ~(PEND_MSK<<PEND_SH); Header |= (pend<<PEND_SH); 
	};
	inline void SetAck(int ack){
		Header &= ~(ACK_REPLY_MSK<<ACK_REPLY_SH); Header |= (ack<<ACK_REPLY_SH); 
	};
	inline void SetFormat(int format){
		Header &= ~(FORMAT_MSK<<FORMAT_SH); Header |= (format<<FORMAT_SH); 
	};
	inline void SetSource(int src){
		Header &= ~(SOURCE_MSK<<SOURCE_SH); Header |= (src<<SOURCE_SH); 
	};
	inline void SetDest(int dst){
		Header &= ~(DEST_MSK<<DEST_SH); Header |= (dst<<DEST_SH); 
	};
	inline void SetFunction(int func){
		Header &= ~(FUNCTION_MSK<<FUNCTION_SH); Header |= (func<<FUNCTION_SH); 
	};
	inline void SetTransId(int tr_id){
		Header &= ~(TRANS_ID_MSK<<TRANS_ID_SH); Header |= (tr_id<<TRANS_ID_SH);
	};

	inline void SockShow(int uthread){
#ifdef DEBUG_SOK
		ostrstream RetStream; 
	    
		RetStream    << hex << endl <<
		"Header "    << GetHeader() << endl <<
		"GetPend "   << GetPend() << endl <<
		"Ack "	     << GetAck() << endl <<   
		"Format "	 << GetFormat() << endl <<
		"GetSource " << GetSource() << endl <<
		"GetDest "   << GetDest() << endl <<
		"Function "	 << GetFunction() << endl <<
		"GetTransId "<< GetTransId() << endl <<
		"Word1 "   << GetDataWord1() << endl <<
		"Word2 "   << GetDataWord2() << endl <<
		"Word3 "   << GetDataWord3() << endl ;

		Sockets[uthread]->Send(RetStream.str(),RetStream.pcount());
#endif
	};

	inline void Show(void){

		cout
#if (OS!=UCOS)			
		<< hex
#endif  //UCOS		
		<< endl <<
		"Header "   << GetHeader() << endl <<
		"GetPend "   << GetPend() << endl <<
		"Ack "	     << GetAck() << endl <<   
		"Format "	 << GetFormat() << endl <<
		"GetSource " << GetSource() << endl <<
		"GetDest "   << GetDest() << endl <<
		"Function "	 << GetFunction() << endl <<
		"GetTransId "<< GetTransId() << endl <<
		"Word1 "   << GetDataWord1() << endl <<
		"Word2 "   << GetDataWord2() << endl <<
		"Word3 "   << GetDataWord3() << endl ;
/**/	};

private:
	int Header;
	int DataWord1;
	int DataWord2;
	int DataWord3;

};




#endif //MESSAGE_HEAD
