/* SignalCsr.h
 * Fdriver Rx Message Dispatcher file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: FDriver
 * author: sk, Feb 98 Initial version
 * $History: SignalCsr.h $
 * 
 * *****************  Version 24  *****************
 * User: Dmwhite      Date: 5/09/00    Time: 9:34a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * more ostype cleanup
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 2/15/00    Time: 11:34a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Fixed SCR# 848...also removed reading from inter_th_sig csr --
 * write-only reg.
 * 
 * *****************  Version 21  *****************
 * User: Drjohns1     Date: 1/14/00    Time: 11:47a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Modified CoreToFboxCsr::SignalThread to use WRITE_LWORD /READ_LWORD
 * macros
 * 
 * *****************  Version 19  *****************
 * User: Drjohns1     Date: 12/08/99   Time: 2:19p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Fixed for AMBAIO simulation mode
 * 
 * *****************  Version 17  *****************
 * User: Skornfel     Date: 10/13/99   Time: 10:54a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Corrected to support OS type and IOSTYLE
 * 
 * *****************  Version 16  *****************
 * User: Tclancyx     Date: 8/20/99    Time: 7:02p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Change reference to iostream::operator<<(void *) to (unsigned int)
 * since vxWorks doesn't seem to be able to print a pointer to void
 * 
 * *****************  Version 14  *****************
 * User: Skornfel     Date: 6/28/99    Time: 11:43a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Runs on 1200 if SA_1200 defined, use multithreaded model.
 * Tested with ISR attached directly to int 10. To attach to "ueng"
 * Interrupt change one line in .cpp file
 * 
 * *****************  Version 13  *****************
 * User: Qabuild      Date: 3/26/99    Time: 11:04a
 * Updated in $/development/driver
 * SignalCsr #ifdef please revisit later
 * 
 * *****************  Version 12  *****************
 * User: Skornfel     Date: 3/25/99    Time: 7:25p
 * Updated in $/development/driver
 * adopted to UCOS
 * 
 * *****************  Version 8  *****************
 * User: Skornfel     Date: 3/16/99    Time: 2:38p
 * Updated in $/development/driver
 * . New for BL5:  HoldTxToEngine(int engine) and ResumeTxToEngine(int
 * engine)
 * . lots of small changes by other developers broke the Unit testing -
 * Fixed
 * . "multi chip" configuration changes made by other developers will not
 * really work(just for info) 
 * 
 * 
 * *****************  Version 6  *****************
 * User: Skornfel     Date: 5/21/98    Time: 2:21p
 * Updated in $/base_level2/Driver
 * COPYRIGHT (c)  1998 BY  INTEL  CORPORATION
 * 
 * *****************  Version 5  *****************
 * User: Skornfeld    Date: 5/04/98    Time: 12:46p
 * Updated in $/base_level2/Driver
 * The same functionality compare to
 * the previous Check In. Fixed locks
 * and Queue. No output to the console.
 * 
 * *****************  Version 4  *****************
 * User: Skornfeld    Date: 4/17/98    Time: 10:52a
 * Updated in $/base_level2/Driver
 * BL2 Driver  
 * 
 * *****************  Version 3  *****************
 * User: Skornfeld    Date: 3/10/98    Time: 6:50p
 * Updated in $/base_level1/Driver
 * Digial Headers, Code cleaup, comments 
 * ---------------------------------------------------------------------
 */



#ifndef SIGNAL_CSR
#define SIGNAL_CSR
#ifdef DEBUG_SOK
extern Semaphore uCodeWaitForCoreTest[]; //semaphore for testing
extern Socket * Sockets[];//socks for testing
#endif

#include "hal_fbi.h"
#include "core_io.h"


// CoreToFboxCsr - the register the Core SW uses to "signal" Fenine Context
// Core can signal one context per register write.
class CoreToFboxCsr{

	enum Shifts{
		THREAD_SH =0,
		F_ENGINE_SH =2,
	};

	enum Masks{
		THREAD_MSK =0x3,
		F_ENGINE_MSK =0x7,
	};
public:		
	CoreToFboxCsr(){fbiUnit = FBI_Attach(); sigCsr = 0xbadbaddd;}
	~CoreToFboxCsr(){};

	//Signal Thread Syncronized at the time of call -  CAN be SET ONLY by ONE TREAD:)
	inline void SignalThread( int thread, int engine){

		unsigned int tempSignalCsr=0;
		tempSignalCsr = (thread << THREAD_SH);
		tempSignalCsr |= (engine << F_ENGINE_SH);
		fbiUnit->setINTER_THD_SIG(tempSignalCsr);
		cout << ((unsigned int)this) << " AFTER  signaling, tempSignalCsr ....0x" << hex << tempSignalCsr  << endl; 

#ifdef NO_LIB
#ifdef DEBUG_SOK
		SockShow(thread + engine*4);
		uCodeWaitForCoreTest[(thread + 4*engine)].Give();
#endif  
#endif
	};

	inline void Show(){
/* dj: the INTER_TH_SIG csr is write-only
		cout << endl << "Signal uThread, RegSignal: 0x" 
#if (OS!=UCOS)
		<< hex 
#endif  //UCOS
			 << sigCsr
			 << endl << " uThread " <<  ( (sigCsr >> THREAD_SH) & THREAD_MSK )
			 << endl << " uEngine " <<  ( (sigCsr >> F_ENGINE_SH) & F_ENGINE_MSK )
			 << endl << endl;
*/
	};

#ifdef DEBUG_SOK //
#if (OS!=UCOS)
	inline void SockShow(int uthread){
		ostrstream RetStream; 
/* dj: the INTER_TH_SIG csr is write-only
	    RetStream
		     << endl << "Signal uThread, RegSignal: 0x" << hex << sigCsr
			 << endl << " uThread " <<  ( (sigCsr >> THREAD_SH) & THREAD_MSK )
			 << endl << " uEngine " <<  ( (sigCsr >> F_ENGINE_SH) & F_ENGINE_MSK )
			 << endl << endl;
*/
		Sockets[uthread]->Send(RetStream.str(),RetStream.pcount());
		return ;//RetStream;
	};
#endif  //UCOS
#endif


private:
	unsigned int sigCsr;
	FbiUnit *fbiUnit;
};



#endif //SIGNAL_CSR
