/* Driver.h
 * Fdriver Rx Source Dispetcher file 
 *
 * ---------------------------------------------------------------------        
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: FDriver
 * author: sk, Feb 98 Initial version
 * $History: UcommDriver.h $
 * 
 * *****************  Version 40  *****************
 * User: Ewalker      Date: 12/04/00   Time: 9:04a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Changed ChipName to XactIoGetChipName function call.
 * 
 * *****************  Version 38  *****************
 * User: Dmwhite      Date: 4/05/00    Time: 2:55p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * no more  U_COS macro, changed to (OS==UCOS)
 * 
 * *****************  Version 36  *****************
 * User: Drjohns1     Date: 2/15/00    Time: 11:35a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Fixed SCR# 848...also removed reading from inter_th_sig csr --
 * write-only reg.
 * 
 * *****************  Version 33  *****************
 * User: Skornfel     Date: 10/13/99   Time: 10:55a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Corrected to support OS type and IOSTYLE
 * 
 * *****************  Version 31  *****************
 * User: Qabuild      Date: 7/14/99    Time: 4:02p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Change UCOS to U_COS avoiding  the conflict with UCOS=2 which is
 * defined in ostype.h
 * 
 * *****************  Version 30  *****************
 * User: Qabuild      Date: 7/07/99    Time: 6:15p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * added #ifndefs SA_1200 to take out dead code
 * 
 * *****************  Version 29  *****************
 * User: Skornfel     Date: 6/28/99    Time: 11:43a
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Runs on 1200 if SA_1200 defined, use multithreaded model.
 * Tested with ISR attached directly to int 10. To attach to "ueng"
 * Interrupt change one line in .cpp file
 * 
 * *****************  Version 28  *****************
 * User: Skornfel     Date: 4/29/99    Time: 5:28p
 * Updated in $/dev_1200/SA1_CoreLibs/driver
 * Added #define ONE_RX_THREAD it will make driver to work in sequential
 * way,  and use three threads instead of 26
 * 
 * *****************  Version 27  *****************
 * User: Drjohns1     Date: 4/22/99    Time: 1:45p
 * Updated in $/development/driver
 * Fixed problem with clearing interrupt on IREG
 * 
 * *****************  Version 26  *****************
 * User: Drjohns1     Date: 4/16/99    Time: 1:28p
 * Updated in $/development/driver
 * Changed invalid/temporary scheme for referencing IREG
 * 
 * *****************  Version 25  *****************
 * User: Skornfel     Date: 3/25/99    Time: 7:25p
 * Updated in $/development/driver
 * adopted to UCOS
 * 
 * *****************  Version 21  *****************
 * User: Skornfel     Date: 3/16/99    Time: 2:38p
 * Updated in $/development/driver
 * . New for BL5:  HoldTxToEngine(int engine) and ResumeTxToEngine(int
 * engine)
 * . lots of small changes by other developers broke the Unit testing -
 * Fixed
 * . "multi chip" configuration changes made by other developers will not
 * really work(just for info) 
 * 
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 2/12/99    Time: 6:10p
 * Updated in $/development/driver
 * Updated for multi-chip/auto_ambio related changes
 * 
 * *****************  Version 19  *****************
 * User: Lhui         Date: 11/25/98   Time: 10:01a
 * Updated in $/development/driver
 * Change CHIP_NAME to ChipName
 * 
 * *****************  Version 17  *****************
 * User: Hqianx       Date: 9/22/98    Time: 2:41p
 * Updated in $/base_level3/driver
 * Added line #include "ambaio.h"
 * 
 * *****************  Version 16  *****************
 * User: Hqianx       Date: 9/17/98    Time: 4:21p
 * Updated in $/base_level3/driver
 * Added AMBA IO Interface.
 * 
 * *****************  Version 15  *****************
 * User: Skornfel     Date: 5/21/98    Time: 2:21p
 * Updated in $/base_level2/Driver
 * COPYRIGHT (c)  1998 BY  INTEL  CORPORATION
 * 
 * *****************  Version 14  *****************
 * User: Skornfeld    Date: 5/19/98    Time: 3:23p
 * Updated in $/base_level2/Driver
 * 2way message may be sent to the driver's queue interface
 * and it will be returned to the Dlib's queue
 * (henry's change deleted) 
 * 
 * *****************  Version 12  *****************
 * User: Skornfeld    Date: 5/05/98    Time: 2:35p
 * Updated in $/base_level2/Driver
 * can't make templatized queue work with ccarm sorry.. 
 * 
 * *****************  Version 11  *****************
 * User: Skornfeld    Date: 5/05/98    Time: 12:45p
 * Updated in $/base_level2/Driver
 * for VxWorks ->  template class CircQueue<RxMessageHeader>;//explicitly 
 * 
 * *****************  Version 10  *****************
 * User: Skornfeld    Date: 5/04/98    Time: 12:46p
 * Updated in $/base_level2/Driver
 * The same functionality compare to
 * the previous Check In. Fixed locks
 * and Queue. No output to the console.
 * 
 * *****************  Version 8  *****************
 * User: Skornfeld    Date: 4/17/98    Time: 10:52a
 * Updated in $/base_level2/Driver
 * BL2 Driver  
 * 
 * *****************  Version 7  *****************
 * User: Skornfeld    Date: 3/10/98    Time: 6:50p
 * Updated in $/base_level1/Driver
 * Digial Headers, Code cleaup, comments 
 * 
 * *****************  Version 6  *****************
 * User: Skornfeld    Date: 3/09/98    Time: 5:34p
 * Updated in $/base_level1/Driver
 * ---------------------------------------------------------------------
 */



#ifndef DRIVER
#define DRIVER

#include "version.h"
#include <iostream.h>
#include <new.h>


#ifdef WIN32
#include "DbgSock.h"
#endif



#include "MessageDstr.h"
#include "ContextDstr.h"
#include "GboxContext.h"
#include "InQueue.h"


#include "MessageHead.h"
#include "SignalCsr.h"
#include "TS.h"
#include "DriverLocks.h"

#include "ambaio.h"

//SK template class CircQueue<RxMessageHeader>;

// The 1200 Driver Wrapper class . The instance of the driver creates 
// all sub components of the driver. It provides external methods (interfaces)
// to other components of the system     
// 




#ifdef DEBUG_SOK
Socket * Sockets[25]; //socks for debuging......!!! cock25 -inQueue
#endif


class Driver{


public:

//UGLY
static Driver * This;
static MsgStateMachinePtrs  S;   //Array of driver message dipetchers (pointers)
                         //  0 1 2 3   4 5 6 7   8 9 0 1 .....
                         // [0,1,2,3] [0,1,2,3] [0,1,2,3] [0,1,2,3] [0,1,2,3]
static int ThreadRun;


	Driver(){

		cout << "####### Driver CTOR" << endl;
		ThreadRun = 1;
		CtoFCsr = new CoreToFboxCsr();
		ELM = new EngineLockManagement(CSR, S, RxMsgSemaphores);
#ifdef ONE_RX_THREAD
		RxDsCo = new RxCnxtDispatcher(*((unsigned int *)0xb00401e0), *ELM, S);      //Context Dispetcher
#else //ONE_RX_THREAD
		RxDsCo = new RxCnxtDispatcher(*((unsigned int *)0xb00401e0), *ELM, S, RxMsgSemaphores); //Context Dispetcher
#endif //ONE_RX_THREAD


		//Create Instances
		for ( int engn=0; engn< NUMBER_OF_GBOXES ; engn++ ){
			for ( int thrd=0; thrd< CONTEXTS_PER_GBOX ; thrd++ ){
				//cout << "IN S.ptr_RxMsgDptr LOOP " << endl;
				S.ptr_RxMsgDptr[engn*CONTEXTS_PER_GBOX + thrd] = new RxMsgDispatcher(*CtoFCsr, *ELM, thrd ,engn);
			}
		};

		//THD Create Thread & Signal
		This = this;

		TxMessageLock.Give(); 
#if (OS!=UCOS)
#ifndef SA_1200
		RxThread = new Thread(RxThreadFunc);
#endif  //SA_1200
		FwTxThread = new Thread(FwTxMessage);
#else //UCOS
#ifndef SA_1200
		RxThread = new Thread(RxThreadFunc,44);
#endif  //SA_1200
		FwTxThread = new Thread(FwTxMessage,45);
#endif //UCOS


#ifdef ONE_RX_THREAD
		RxMsgSemaphore= new Semaphore;
		RxMessageThread = new Thread(RxMessageThreadFunc);

#else //#ifndef ONE_RX_THREAD
		for(int conxt=0;conxt<(CONTEXTS_PER_GBOX*NUMBER_OF_GBOXES);++conxt)
			RxMsgSemaphores.ptr_Semaphore[conxt]= new Semaphore;
#if (OS!=UCOS)
		RxMsgThread[0] = new Thread(RxMSGThreadFunc0);
		RxMsgThread[1] = new Thread(RxMSGThreadFunc1);
		RxMsgThread[2] = new Thread(RxMSGThreadFunc2);
		RxMsgThread[3] = new Thread(RxMSGThreadFunc3);
		RxMsgThread[4] = new Thread(RxMSGThreadFunc4);
		RxMsgThread[5] = new Thread(RxMSGThreadFunc5);
		RxMsgThread[6] = new Thread(RxMSGThreadFunc6);
		RxMsgThread[7] = new Thread(RxMSGThreadFunc7);
		RxMsgThread[8] = new Thread(RxMSGThreadFunc8);
		RxMsgThread[9] = new Thread(RxMSGThreadFunc9);
		RxMsgThread[10] = new Thread(RxMSGThreadFunc10);
		RxMsgThread[11] = new Thread(RxMSGThreadFunc11);
		RxMsgThread[12] = new Thread(RxMSGThreadFunc12);
		RxMsgThread[13] = new Thread(RxMSGThreadFunc13);
		RxMsgThread[14] = new Thread(RxMSGThreadFunc14);
		RxMsgThread[15] = new Thread(RxMSGThreadFunc15);
		RxMsgThread[16] = new Thread(RxMSGThreadFunc16);
		RxMsgThread[17] = new Thread(RxMSGThreadFunc17);
		RxMsgThread[18] = new Thread(RxMSGThreadFunc18);
		RxMsgThread[19] = new Thread(RxMSGThreadFunc19);
		RxMsgThread[20] = new Thread(RxMSGThreadFunc20);
		RxMsgThread[21] = new Thread(RxMSGThreadFunc21);
		RxMsgThread[22] = new Thread(RxMSGThreadFunc22);
		RxMsgThread[23] = new Thread(RxMSGThreadFunc23);
#else //UCOS
		RxMsgThread[0] = new Thread(RxMSGThreadFunc0,20);
		RxMsgThread[1] = new Thread(RxMSGThreadFunc1,21);
		RxMsgThread[2] = new Thread(RxMSGThreadFunc2,22);
		RxMsgThread[3] = new Thread(RxMSGThreadFunc3,23);
		RxMsgThread[4] = new Thread(RxMSGThreadFunc4,24);
		RxMsgThread[5] = new Thread(RxMSGThreadFunc5,25);
		RxMsgThread[6] = new Thread(RxMSGThreadFunc6,26);
		RxMsgThread[7] = new Thread(RxMSGThreadFunc7,27);
		RxMsgThread[8] = new Thread(RxMSGThreadFunc8,28);
		RxMsgThread[9] = new Thread(RxMSGThreadFunc9,29);
		RxMsgThread[10] = new Thread(RxMSGThreadFunc10,30);
		RxMsgThread[11] = new Thread(RxMSGThreadFunc11,31);
		RxMsgThread[12] = new Thread(RxMSGThreadFunc12,32);
		RxMsgThread[13] = new Thread(RxMSGThreadFunc13,33);
		RxMsgThread[14] = new Thread(RxMSGThreadFunc14,34);
		RxMsgThread[15] = new Thread(RxMSGThreadFunc15,35);
		RxMsgThread[16] = new Thread(RxMSGThreadFunc16,36);
		RxMsgThread[17] = new Thread(RxMSGThreadFunc17,37);
		RxMsgThread[18] = new Thread(RxMSGThreadFunc18,38);
		RxMsgThread[19] = new Thread(RxMSGThreadFunc19,39);
		RxMsgThread[20] = new Thread(RxMSGThreadFunc20,40);
		RxMsgThread[21] = new Thread(RxMSGThreadFunc21,41);
		RxMsgThread[22] = new Thread(RxMSGThreadFunc22,42);
		RxMsgThread[23] = new Thread(RxMSGThreadFunc23,43);
#endif //UCOS

#endif //ONE_RX_THREAD

#ifdef DEBUG_SOK
//#define SOCKET_SERVER ARMADA_LPTOP//serge
		Sockets[0]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE00,1024);
		Sockets[1]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE01,1024);
		Sockets[2]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE02,1024);
		Sockets[3]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE03,1024);

		Sockets[4]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE10,1024);
		Sockets[5]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE11,1024);
		Sockets[6]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE12,1024);
		Sockets[7]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE13,1024);

		Sockets[8]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE20,1024);
		Sockets[9]  = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE21,1024);
		Sockets[10] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE22,1024);
		Sockets[11] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE23,1024);

		Sockets[12] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE30,1024);
		Sockets[13] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE31,1024);
		Sockets[14] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE32,1024);
		Sockets[15] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE33,1024);


		Sockets[16] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE40,1024);
		Sockets[17] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE41,1024);
		Sockets[18] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE42,1024);
		Sockets[19] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE43,1024);

		Sockets[20] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE50,1024);
		Sockets[21] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE51,1024);
		Sockets[22] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE52,1024);
		Sockets[23] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE53,1024);

		Sockets[24] = new Socket("armada_lptop" /*"16.128.128.47"*/,0xFE0,1024);




#endif  //DEBUG_SOK

/*( TEST OF the queue lock/signaling
RxMessageHeader H;
H.SetDest(0); 		InMsgQue.Qstore(H);
H.SetDest(1); 		InMsgQue.Qstore(H);
H.SetDest(2);		InMsgQue.Qstore(H);
H.SetDest(3);		InMsgQue.Qstore(H);
H.SetDest(4);		InMsgQue.Qstore(H);
H.SetDest(5);		InMsgQue.Qstore(H);
H.SetDest(14);		InMsgQue.Qstore(H);
H.SetDest(15);		InMsgQue.Qstore(H);
H.SetDest(16);		InMsgQue.Qstore(H);
H.SetDest(17);		InMsgQue.Qstore(H);
H.SetDest(18);		InMsgQue.Qstore(H);
H.SetDest(19);		InMsgQue.Qstore(H);
H.SetDest(21);		InMsgQue.Qstore(H);
H.SetDest(22);		InMsgQue.Qstore(H);
H.SetDest(23);		InMsgQue.Qstore(H);
//TEST )*/
	cout << "Driver CTOR ok" << endl;
	};
	
	~Driver(){

		//get out of the thread's loops before killing the 
		//thread's handlers( may not be nessasary - so can be removed)
		ThreadRun = 0;
		MessageHeader *CoreToFboxHeader = NULL;
		TxMessageQue(0, *CoreToFboxHeader);
		
		for ( int engn=0; engn< NUMBER_OF_GBOXES ; engn++ ){
			for ( int thrd=0; thrd< CONTEXTS_PER_GBOX ; thrd++ )
				RxMsgSemaphores.ptr_Semaphore[engn*CONTEXTS_PER_GBOX + thrd]->Give();
		};

		for ( engn=0; engn< NUMBER_OF_GBOXES ; engn++ ){
			for ( int thrd=0; thrd< CONTEXTS_PER_GBOX ; thrd++ )
				delete 	RxMsgThread[engn*CONTEXTS_PER_GBOX + thrd];
		};

		//onethread
	    //delete
		//delete
		delete FwTxThread;
		delete RxThread;

		for ( engn=0; engn< NUMBER_OF_GBOXES ; engn++ ){
			for ( int thrd=0; thrd< CONTEXTS_PER_GBOX ; thrd++ )
				delete S.ptr_RxMsgDptr[engn*CONTEXTS_PER_GBOX + thrd];
		};

		delete RxDsCo;
		delete ELM;
		delete CtoFCsr;
	
	};

	//THD TEST
	//
	inline void
	RxUnblock(void){
		RxSemaphore.Give();
	};
	

#ifndef SA_1200
	//THD Create Thread To Rx from 1200
	//
	static inline void 
	RxThreadFunc(void){
		for (;;){ //cout << "Thred" ;
			This->RxSemaphore.Take();
			if(ThreadRun)This->RxDispatch();
		};
	};
#endif //SA_1200


#ifdef SA_1200
	static inline void Isr(void){ This->RxDsCo->Dispatch(); };
#endif //#ifdef SA_1200
	
#ifndef SA_1200
	//handle interrupts Rx from 1200 _uThreads path
	//Dispetch the message
	//
	inline void 
	RxDispatch(void){
#ifndef NO_LIB
#ifdef USE_AMBAIO	
        AMBAIO_ReadIREG(XactIoGetChipName(),&CSR);
        AMBAIO_WriteIREG(XactIoGetChipName(),0xff000000 & CSR);	// clear fbox interrupt
#else
		//hack for transactor integration
		XactIoReadIREG(&CSR);
		XactIoWriteIREG(0xff000000 & CSR);	// clear fbox interrupt

#endif
#endif  //NO_LIB 


		//CSR has an shadow of ireg		
		//call State mahines  
		RxDsCo->Dispatch();

#ifndef NO_LIB
        ///???XACT_amba_clearing_interrupt(XactIoGetChipName(), 10);
#endif
	};
#endif  // SA_1200

	//Send Message From Core To 1200 _uThread Directly
	//
	inline void 
	TxMessage(int context, MessageHeader & message){
		RxDsCo->GetInstance(context)->CoreToFBox(message);
	};

	//Send Message From Core To 1200 _uThread Queued
	//
	inline void 
	TxMessageQue(int context, MessageHeader & message){
		//bl2RxDsCo->GetInstance(context)->CoreToFBox(message);
		TxMessageLock.Take();      // LOCK
		InMsgQue.Qstore(message);
		TxMessageLock.Give();      //Un LOCK
	};

	//get out of queue, Forward message To 1200 _uThread
	//Thread it !
	static inline void 
	FwTxMessage(){
		static int err;
		static MessageHeader msg;
		for (;;){
			This->InMsgQue.WaitForMessage();
			if(!ThreadRun)return;
			msg = This->InMsgQue.Qretrive(err);
			//cout << "QRetrive e e e e e e e e e e e e e e e e e e e " << msg.GetDest() << endl;
#ifdef DEBUG_SOK
{			ostrstream STRM;
			STRM << "QUEUE RETRIVED To the Destonation " << msg.GetDest() << endl <<" " << endl;Sockets[24]->SendSrting(STRM);
}
#endif
			if(msg.GetTriger() == MSG_TWOWAY_REQ){

				This->RxDsCo->GetInstance(msg.GetDest())->CoreToFBox(msg, true );

			}else{/*MSG_ONEWAY_REQ  */ 
				This->RxDsCo->GetInstance(msg.GetDest())->CoreToFBox(msg);
			}
		};
	};


    // Hold Driver from sending messages to UCode Engine
	//
	inline void 
	HoldTxToEngine(int engine){
		ELM->HoldEngine(engine);
	};

    // Resume Driver to send messages to UCode Engine
	//
	inline void 
	ResumeTxToEngine(int engine){
		ELM->ReleaseEngine(engine);
	};

//TESTING
	// Put Message into the "mailbox" for the driver to receive
	// 
	inline void //For Test 
	PutMessageForDriverToRx(int contxt,int r0,int r1=0,int r2=0,int r3=0 ){
		
		((RxDsCo->GetInstance(contxt))->GetContext())->GetWrMbox()->Set(r0, r1, r2, r3);	
	};


	// print Message from the "mailbox" for the UCode to receive
	// 
	inline void //For Test 
	GetAndPrintMessageFor_uTread(int contxt){
		
		MessageHeader * p_temp;
		p_temp = (MessageHeader *)((RxDsCo->GetInstance(contxt))->GetContext())->GetRdMbox();
		p_temp->SockShow(contxt);
	};


	// Get Message from the "mailbox" for the UCode to receive
	// 
	inline void * 
	GetMessageFor_uTread(int contxt){
		
		MessageHeader * p_temp;
		p_temp = (MessageHeader *)((RxDsCo->GetInstance(contxt))->GetContext())->GetRdMbox();
		return (void *)p_temp; // ? serge what are you doing ? 
	};


#ifdef ONE_RX_THREAD
static inline void RxMessageThreadFunc(){
//	for (;;){	This->RxMsgSemaphore.Take();
//			for (int thread=0; thread<24; thread++){
//			         if()S.ptr_RxMsgDptr[thread]->FBoxToCoreDispatch();
//			};
//	};
};


#else  //#ifndef ONE_RX_THREAD
//looks VERY Ugly, but I cannot find better way do do it at this point :(

//THD Create Thread To run MSG logic on
//

static inline void RxMSGThreadFunc0(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[0]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[0]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc1(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[1]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[1]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc2(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[2]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[2]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc3(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[3]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[3]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc4(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[4]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[4]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc5(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[5]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[5]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc6(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[6]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[6]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc7(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[7]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[7]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc8(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[8]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[8]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc9(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[9]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[9]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc10(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[10]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[10]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc11(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[11]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[11]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc12(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[12]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[12]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc13(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[13]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[13]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc14(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[14]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[14]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc15(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[15]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[15]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc16(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[16]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[16]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc17(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[17]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[17]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc18(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[18]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[18]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc19(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[19]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[19]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc20(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[20]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[20]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc21(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[21]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[21]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc22(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[22]->Take();
	                    if(ThreadRun)S.ptr_RxMsgDptr[22]->FBoxToCoreDispatch(); 		};
};
static inline void RxMSGThreadFunc23(){
	for (;;){			This->RxMsgSemaphores.ptr_Semaphore[23]->Take();
			            if(ThreadRun)S.ptr_RxMsgDptr[23]->FBoxToCoreDispatch(); 		};
};
//............. it is soo bad :(
#endif ONE_RX_THREAD

	inline void MboxAllShow(void){
		for ( int engn=0; engn< NUMBER_OF_GBOXES ; engn++ ){
			for ( int thrd=0; thrd< CONTEXTS_PER_GBOX ; thrd++ )
				S.ptr_RxMsgDptr[engn*CONTEXTS_PER_GBOX + thrd]->MboxShow();
		};//MboxShow
	};


	inline void MboxShow(int box){
		S.ptr_RxMsgDptr[box]->MboxShow();
	};

//SK to try public:
private:


 
//MsgSateMachinePtrs  S;   //Array of driver message dipetchers (pointers)
                         //  0 1 2 3   4 5 6 7   8 9 0 1 .....
                         // [0,1,2,3] [0,1,2,3] [0,1,2,3] [0,1,2,3] [0,1,2,3]

RxCnxtDispatcher *RxDsCo;			 // Pointer to the Rx Context Dispetcher
CoreToFboxCsr    *CtoFCsr;           //"SignalReg" placement NEW or Referense for now
//THD
Thread * RxThread;					 // Thread to run driver on, Rx from uCode
Thread * FwTxThread;				 // Thread to pass messages to uCode from Queue

#ifdef ONE_RX_THREAD
Semaphore * RxMsgSemaphore;
Thread * RxMessageThread;
#else //#ifndef ONE_RX_THREAD
Thread * RxMsgThread[NUMBER_OF_GBOXES*CONTEXTS_PER_GBOX];//Threads to run instances of RxMsg  State machines
MsgStateSemaphores RxMsgSemaphores;
#endif //ONE_RX_THREAD
Semaphore RxSemaphore;			 // Driver's semaphore to Wait on RX from _uCode
Semaphore TxMessageLock;		 // Incomming queueu lock/mutex	
//template class CircQueue<RxMessageHeader>;
//SK CircQueue<class RxMessageHeader> InMsgQue;
CircQueue InMsgQue;
EngineLockManagement * ELM;

};

#endif //DRIVER
