/* hal_fbi.cpp
 * SA1200 Hardware Absraction Library -- FBI
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: HAL
 *  author: dfh, Mar 2, 99  Initial version
 *  revisions:
 *  $History: hal_fbi.cpp $
 * 
 * *****************  Version 8  *****************
 * User: Dhooper      Date: 7/19/99    Time: 9:57a
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * compile in FBI_Reset always
 * 
 * *****************  Version 7  *****************
 * User: Dhooper      Date: 7/15/99    Time: 4:38p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * fbi reset to enable stop then restart reference design
 * 
 * *****************  Version 6  *****************
 * User: Dhooper      Date: 6/02/99    Time: 4:27p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * add scratch read, write
 * 
 * *****************  Version 5  *****************
 * User: Dhooper      Date: 5/14/99    Time: 9:25a
 * Updated in $/BL5_Plus/SA1_CoreLibs/hal_1200
 * change SRAM, SDRAM, FBI_Init to SRAM, SDRAM, FBI_Attach
 * 
 * *****************  Version 4  *****************
 * User: Dhooper      Date: 5/10/99    Time: 12:42p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * fbi init function
 * 
 * *****************  Version 2  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:41p
 * Updated in $/development/hal_1200
 * update COPYRIGHT
 * 
 * *****************  Version 1  *****************
 * User: Dhooper      Date: 3/02/99    Time: 4:21p
 * Created in $/development/hal_1200
 * fbi hardware abstraction lib
 * 
 * ---------------------------------------------------------------------
 */
#include "hal_fbi.h"
#include "hal_pci.h"

static FbiUnit *fbi = (FbiUnit *)FBI_CSR_BASE;
static PciCSRs *Pci;

/*-----------------------------------------------------------------
	FBI_Attach
	Return a pointer to the FBI instance
	returns: FbiUnit *
	uses:
	modifies:
  -----------------------------------------------------------------*/
FbiUnit *FBI_Attach()
{
	return(fbi);
}



/*-----------------------------------------------------------------
	FBI_Reset
	Assert, then de-assert reset to FBI.
	This should only be done when the full system is quiesced, ports stopped, etc.
	It may be useful for a restart without reboot.
	returns: 
	uses:
	modifies:
  -----------------------------------------------------------------*/
void FBI_Reset()
{
	unsigned int data;
	int lock_token;

	// disable all interrupts
	lock_token = int_lock();

	// set the reset bit
	Pci = (PciCSRs *)RESET_ENABLE_REG;	// starting at address 0x42000000 + 0x7c
	Pci->getResetReg(&data);
	data |= FBI_RESET_MASK;
	Pci->setResetReg(data);

	// now clear the reset bit
	data &= ~FBI_RESET_MASK;
	Pci->setResetReg(data);

	// enable interrupts
	int_unlock(lock_token);

	return;
}


/*-----------------------------------------------------------------
	Scratch_Write
	Write val to sram longword  address addr
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
void Scratch_Write(unsigned int lwaddr, unsigned int val)
{
	fbi->scratch_pad_write(lwaddr, val);
	return;
}


/*-----------------------------------------------------------------
	Scratch_Read
	Read val from sram longword  address addr
	returns: data from memory location
	uses:
	modifies: sram
  -----------------------------------------------------------------*/
unsigned int Scratch_Read(unsigned int lwaddr)
{
	unsigned int val;
	fbi->scratch_pad_read(lwaddr, &val);
	return(val);
}

