/* hal_sram.cpp
 * SA1200 Hardware Absraction Library -- SRAM
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem:  HAL
 *  author: dfh, Feb 24, 99  Initial version
 *  revisions:
 *  $History: hal_sram.cpp $
 * 
 * *****************  Version 17  *****************
 * User: Drjohns1     Date: 6/13/00    Time: 12:16p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * Fixed bug in test_and_set_bits and test_and_clear_bits functions --
 * long-word addressable
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 6/06/00    Time: 2:38p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * Fixed bug in test_and_set_bits and test_and_clear_bits functions
 * 
 * *****************  Version 15  *****************
 * User: Qabuild      Date: 5/12/00    Time: 2:02p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * Merge with IAL's change
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 4/20/00    Time: 4:37p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * Fix build errors
 * 
 * *****************  Version 13  *****************
 * User: Qabuild      Date: 4/18/00    Time: 5:19p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * update from IAL zip file 4/12/00
 * 
 * *****************  Version 9  *****************
 * User: Dhooper      Date: 5/14/99    Time: 9:25a
 * Updated in $/BL5_Plus/SA1_CoreLibs/hal_1200
 * change SRAM, SDRAM, FBI_Init to SRAM, SDRAM, FBI_Attach
 * 
 * *****************  Version 8  *****************
 * User: Dhooper      Date: 5/11/99    Time: 5:24p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * don't initialize val to 0 on read and write
 * 
 * *****************  Version 7  *****************
 * User: Dhooper      Date: 5/11/99    Time: 12:18p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * include iostyle
 * 
 * *****************  Version 6  *****************
 * User: Dhooper      Date: 5/10/99    Time: 12:35p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * init function call
 * 
 * *****************  Version 4  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:41p
 * Updated in $/development/hal_1200
 * update COPYRIGHT
 * 
 * *****************  Version 3  *****************
 * User: Dhooper      Date: 3/08/99    Time: 6:40p
 * Updated in $/development/hal_1200
 * add console read write functions
 * 
 * *****************  Version 2  *****************
 * User: Dhooper      Date: 3/02/99    Time: 6:51p
 * Updated in $/development/hal_1200
 * add boilerplate
 * 
 * ---------------------------------------------------------------------
 */
#include "ostype.h"
#include "hal_sram.h"

#if (OS == VXWORKS)
#include "taskLib.h"
#include "intLib.h"
#endif /* OS == VXWORKS */

static SramUnit *sram = (SramUnit *)SRAM_CSR_BASE;

/*-----------------------------------------------------------------
	SDRAM_Attach
	Return a pointer to the SDRAM instance
	returns: SdramUnit	*
	uses:
	modifies:
  -----------------------------------------------------------------*/
SramUnit *SRAM_Attach()
{
	return(sram);
}


/*-----------------------------------------------------------------
	SRAM_Write
	Write val to sram longword  address addr
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
void SRAM_Write(unsigned int lwaddr, unsigned int val)
{
	SRAM_WRITE(lwaddr, val);
	return;
}


/*-----------------------------------------------------------------
	SRAM_Read
	Read val from sram longword  address addr
	returns: data from memory location
	uses:
	modifies: sram
  -----------------------------------------------------------------*/
unsigned int SRAM_Read(unsigned int lwaddr)
{
	unsigned int val;
	SRAM_READ(lwaddr, &val);
	return(val);
}


#if ((IOSTYLE==HARDWARE)||(IOSTYLE==AMBAIO))

/*-----------------------------------------------------------------
	SramUnit::test_and_set_bits
	Atomic test and set at sram unit. It is not atomic inside core,
	therefore interrupts are disabled to prevent another test and set/clear
	write before the read has been accomplished.

	returns: read data from memory location getval, set bits of setval
	assumes: hal_1200 is always used for this sram controller operation
	modifies: sram
  -----------------------------------------------------------------*/
void SramUnit::test_and_set_bits(unsigned int lwaddr, unsigned int setval, unsigned int *getval)
{
// tbd disable interrupts
	WRITE_LWORD(SRAM_TEST_AND_SET_BITS + (lwaddr << 2), setval);
	getSRAM_TEST_MOD(getval);
// tbd enable interrupts

	return;
}

/*-----------------------------------------------------------------
	SramUnit::test_and_clear_bits
	Atomic test and clear at sram unit. It is not atomic inside core,
	therefore interrupts are disabled to prevent another test and set/clear
	write before the read has been accomplished.

	returns: read data from memory location getval, set bits of setval
	assumes: hal_1200 is always used for this sram controller operation
	modifies: sram
  -----------------------------------------------------------------*/
void SramUnit::test_and_clear_bits(unsigned int lwaddr, unsigned int setval, unsigned int *getval)
{
// tbd disable interrupts
	WRITE_LWORD(SRAM_TEST_AND_CLEAR_BITS + (lwaddr << 2), setval);
	getSRAM_TEST_MOD(getval);
// tbd enable interrupts

	return;
}

/*-----------------------------------------------------------------
        read_lock_spin	
	Obtain and read a lock from the SRAM CAM unit, but this
	routine will block out other tasks from obtaining a
	read_lock and will spin until the lock is obtained
	returns: the value of the address locked down 
	uses:
	modifies:
        comments: only works with VxWorks
  -----------------------------------------------------------------*/

int SramUnit::read_lock_spin(unsigned int lwaddr, unsigned int *val) {
#if (OS == VXWORKS)

    unsigned int csr;
    //int interruptLock;
    int failSafe = 0x10000000;

    if (OK == taskLock()) {
        //interruptLock = intLock(); 
        this->read_lock(lwaddr, val);
        this->getSRAM_CSR(&csr);
        if (!(csr & 0x1)) {
            do {
                this->getSRAM_CSR(&csr);
	    } while (!(csr & 0x2) && (--failSafe > 0));
	    if (0 == failSafe) {
                logMsg("%s: failSafe is zero!!!!!!!\n", (int) __FUNCTION__,
                       2, 3, 4, 5, 6);
		taskUnlock();
                return -1;
            }
	    this->setSRAM_CSR(csr | 0x2);
        }
	//intUnlock(interruptLock);
        SRAM_READ(lwaddr, val);
        taskUnlock();
	return 0;
    }
#endif //VXWORKS
    return -1;
}

#if 0
// An alternative approach for read_lock_spin
#include "sem.h"
static sem_T readLockSem = Sem_Init(1);
int SramUnit::read_lock_spin(unsigned int lwaddr, unsigned int *val) {
    unsigned int csr;
    
    // Sanity check
    if (NULL == readLockSem) {
        return -1;
    }
    if (SEM_SUCCESS != Sem_Wait(readLockSem, 60*60)) {
        return -1;
    }
    this->read_lock(lwaddr, val);
    this->getSRAM_CSR(&csr);
    if (!(csr & 0x1)) {
        do {
            this->getSRAM_CSR(&csr);
        } while (!(csr & 0x2));
        this->setSRAM_CSR(csr | 0x2);
        SRAM_READ(lwaddr, val);
    }
    Sem_Post(readLockSem);
    return 0;
}
#endif // 0

#endif		//((IOSTYLE==HARDWARE)||(IOSTYLE==AMBAIO))
