#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#if (OS == WIN_32)
#include <winsock.h>
#else
#include <netinet/in.h>
#endif

#include "ostype.h"
#include "core_io.h"
#include "utils.h"
#include "uclo.h"
#include "hal_sdram.h"
#include "hal_sram.h"
#include "ueng.h"
#include "xact.h"
#include "config_1200.h"


#include "hdbm.h"
#include "hdbm_testMap.h"

typedef unsigned long uint32;
void seedMT(uint32 seed);
uint32 randomMT(void);


static SysConf *SystemConf;
#ifdef HDBM_EXT
static HashDbmExt *Hdb = NULL;
#else
static HashDBM *Hdb = NULL;
#endif
static void *ObjHandle;			// micro code object handle
static unsigned int RecNum=1;
static unsigned int PriTableBase, SramFreeLstBase, SdramFreeLstBase;
static FreeLst *SramFreeLst, *SdramFreeLst;

static int		pLog;
static char		*pFnam;
static FILE		*pFp;


extern "C" int printEle(HdbmKey_T *key, void *dataEntry, void *userData)
{
	printf("recNum=%d, dbKey=0x%08x%08x, data=0x%x\n",
		RecNum++, key->B7654, key->B3210, (unsigned int)dataEntry);
	return(0);
}


int openLog(char *fnam){
	if(!(pFp = fopen(fnam, "w+"))){
		perror(fnam); 
		return(-1);
	}
	fprintf(pFp, "printf(\"Creating Hdbm test database...\\n\");\n");
	pLog=1;
	return(0);
}

void closeLog(){
	printf("closing file...\n");
	fprintf(pFp, "printf(\"end of database creation\\n\");\n");
	fclose(pFp);
	pLog=0;
}

void logSram(unsigned int addr, unsigned int val)
{
	if(!pLog) return;
	fprintf(pFp, "dep/s sram[0x%x] = 0x%x\n", addr, val);
	return;
}


void logSdram(int full64, unsigned int addr, unsigned int val1, unsigned int val2)
{
	if(!pLog) return;
	fprintf(pFp, "dep/s sdram[0x%x]<31:0> = 0x%x\n", addr, val1);
	if(full64) fprintf(pFp, "dep/s sdram[0x%x]<63:32> = 0x%x\n", addr, val2);
	return;
}



#ifdef HDBM_DEBUG
int analyzeDb(void)
{	
	return(Hdb->analyze());
}

//dj: debug
int dumpColLst(void)
{
	SramFreeLst->dumpLst();
	return(0);
}
int dumpKeyLst(void)
{
	SdramFreeLst->dumpLst();
	return(0);
}
#endif


int addDb(int keyLo, int keyHi, int data)
{
	HdbmKey_T key;
	int status;

	key.B3210 = keyLo;
	key.B7654 = keyHi;

	switch((status = Hdb->Add(&key, (void *)data))){
		case HDBM_SUCCESS: break;
		case HDBM_DUPL_ENTRY:
			printf("Error: Duplicate entry, key=0x%08x%08x\n",
					key.B7654, key.B3210);
			break;
		case HDBM_ERROR:
			printf("Error: Failed to add entry, key=0x%08x%08x\n",
					key.B7654, key.B3210);
			break;
		case HDBM_BAD_SDADDR:
			printf("Error: Bad SDRAM address, key=0x%08x%08x\n",
					key.B7654, key.B3210);
			break;
		case HDBM_LOCK_ERROR:
			printf("Error: Database locked, key=0x%08x%08x\n",
					key.B7654, key.B3210);
			break;
		case HDBM_EMPTY_COL_LST:
			printf("Error: Collision table SRAM freelist empty, key=0x%08x%08x\n",
					key.B7654, key.B3210);
			break;
		case HDBM_EMPTY_KEYDATA_LST:
			printf("Error: KeyData SDRAM freelist empty, key=0x%08x%08x\n",
					key.B7654, key.B3210);
			break;
		default:
			printf("Error: Unknown failure, key=0x%08x%08x\n",
				key.B7654, key.B3210);
			break;
	}
	return(status);
}


int lookupDb(int keyLo, int keyHi)
{
	HdbmKey_T key;
	int status, data;

	key.B3210 = keyLo;
	key.B7654 = keyHi;

	if((status = Hdb->Lookup(&key, (void *)&data)) != HDBM_SUCCESS){
		printf("Lookup failed: status = %d\n", status);
	}
	return(data);
}


int listDb(void)
{
	RecNum=1;
	Hdb->Traverse(printEle, NULL);
	return(0);
}

int deleDb(void)
{
//	printf("dj: before dele\n");
//	SdramFreeLst->chkLst();
	Hdb->RemoveAllEntries();
//	printf("dj: after dele\n");
//	SdramFreeLst->chkLst();
	return(0);
}

int createDb(void)
{
	HdbmKey_T key;
	unsigned int dataEle, dataEle2, dataEle3, dataEle4, dataEle5, dataEle6, dataEleX;

	if(!Hdb){
		printf("Hdbm is not initialized\n");
		return(1);
	}

	dataEle  = 0x1111;
	dataEle2 = GOOD_DATA;
	dataEle3 = 0x3333;
	dataEle4 = 0x4444;
	dataEle5 = 0x5555;
	dataEle6 = 0x6666;


	// add new entry
    key.B3210 = BAD_KEY_LO;
    key.B7654 = BAD_KEY_HI;
	printf("Adding: key=0x%08x%08x, data=0x%x\n", key.B7654, key.B3210, dataEle);
	if(Hdb->Add(&key, (void *)dataEle) != HDBM_SUCCESS){
		printf("Failure: Error adding entry\n");
		return(1);
	}

	//Note:- the hash multiplier, in hdbm.cpp,  must be set to 1 in order
	//for the following keys to collide

	// add entry that collides with previous entry
	key.B3210 = GOOD_KEY_LO;
	key.B7654 = GOOD_KEY_HI;
	printf("Adding: key=0x%08x%08x, data=0x%x\n", key.B7654, key.B3210, dataEle2);
	if(Hdb->Add(&key, (void *)dataEle2) != HDBM_SUCCESS){
		printf("Failure: Error adding entry\n");
		return(1);
	}

	// add entry that collides with previous entry
	key.B3210 = 0x11223355;
	key.B7654 = 0x55667788;
	printf("Adding: key=0x%08x%08x, data=0x%x\n", key.B7654, key.B3210, dataEle5);
	if(Hdb->Add(&key, (void *)dataEle5) != HDBM_SUCCESS){
		printf("Failure: Error adding entry\n");
		return(1);
	}

	// add entry that collides with previous entry
	key.B3210 = 0x11223455;
	key.B7654 = 0x55667788;
	printf("Adding: key=0x%08x%08x, data=0x%x\n", key.B7654, key.B3210, dataEle6);
	if(Hdb->Add(&key, (void *)dataEle6) != HDBM_SUCCESS){
		printf("Failure: Error adding entry\n");
		return(1);
	}

	// add duplicate entry
    key.B3210 = BAD_KEY_LO;
    key.B7654= BAD_KEY_HI;
	printf("Adding: key=0x%08x%08x, data=0x%x\n", key.B7654, key.B3210, dataEle3);
	if(Hdb->Add(&key, (void *)dataEle3) != HDBM_DUPL_ENTRY){
		printf("Failure: Error adding entry\n");
		return(1);
	}

	// add new entry
	key.B3210 = 0x94EB549E;
	key.B7654 = 0x0060;
	printf("Adding: key=0x%08x%08x, data=0x%x\n", key.B7654, key.B3210, dataEle4);
	if(Hdb->Add(&key, (void *)dataEle4) != HDBM_SUCCESS){
		printf("Failure: Error adding entry\n");
		return(1);
	}

	// list entries
	printf("\nFive entries should be listed...\n");
	Hdb->Traverse(printEle, NULL);

	// search for existing key
	key.B3210 = BAD_KEY_LO;
	key.B7654 = BAD_KEY_HI;
	printf("Lookup: key=0x%08x%08x\n", key.B7654, key.B3210);
	if(Hdb->Lookup(&key, (void *)&dataEleX) != HDBM_SUCCESS){
		printf("Failure: Key 0x%08x%08x not found\n", key.B7654, key.B3210);
		return(1);
	}
	else if(dataEleX != dataEle){
		printf("Failure: Invalid data 0x%08x != %08x\n", dataEleX, dataEle);
		return(1);
	}


	// delete an existing collision level1 key
	key.B3210 = BAD_KEY_LO;
	key.B7654 = BAD_KEY_HI;
	printf("Remove: key=0x%08x%08x\n", key.B7654, key.B3210);
	if(Hdb->Remove(&key) != HDBM_SUCCESS){
		printf("Failure: Key 0x%08x%08x not deleted\n", key.B7654, key.B3210);
		return(1);
	}

	// search for non-existing key
	key.B3210 = BAD_KEY_LO;
	key.B7654 = BAD_KEY_HI;
	printf("Lookup: key=0x%08x%08x\n", key.B7654, key.B3210);
	if(Hdb->Lookup(&key, (void *)&dataEleX) != HDBM_NO_ENTRY){
		printf("Failure: found non-exixting key: ");
		printEle(&key, (void *)dataEleX, (void *)NULL);
		return(1);
	}


	// delete an existing collision level2 key
	key.B3210 = 0x11223455;
	key.B7654 = 0x55667788;
	printf("Remove: key=0x%08x%08x\n", key.B7654, key.B3210);
	if(Hdb->Remove(&key) != HDBM_SUCCESS){
		printf("Failure: Key 0x%08x%08x not deleted\n", key.B7654, key.B3210);
		//return(1);
	}

	// search for non-existing key
	key.B3210 = 0x11223455;
	key.B7654 = 0x55667788;
	printf("Lookup: key=0x%08x%08x\n", key.B7654, key.B3210);
	if(Hdb->Lookup(&key, (void *)&dataEleX) != HDBM_NO_ENTRY){
		printf("Failure: found non-exixting key: ");
		printEle(&key, (void *)dataEleX, (void *)NULL);
		return(1);
	}

	return(0);
}

int addRandomDb(int numEntry)
{
	unsigned int keyHi, keyLo, added=0;
	clock_t start, finish;
	double  duration;

	keyLo = 0xdeaf;
	keyHi = 0xbead;
	start = clock();
	printf("\nAdding: %d entries\n", numEntry);
	for(int i = 0; i < numEntry; i++){
		keyLo = (unsigned int) randomMT();
		keyHi = (unsigned int) randomMT();

		//dj: debug
//		printf("dj: %d before the add\n", i);
//		if(SdramFreeLst->chkLst()){
//			printf("dj: addRandomDb() failed on the %dth iteration\n", i);
//			return(1);
//		}

		if(addDb(keyLo, keyHi, i + 10) == HDBM_SUCCESS) added++;

		//dj: debug
//		printf("dj: %d after the add\n", i);
//		if(SdramFreeLst->chkLst()){
//			printf("dj: addRandomDb() failed on the %dth iteration\n", i);
//			return(1);
//		}
	}

	finish = clock();
	duration = (double)(finish - start) / CLOCKS_PER_SEC;
	printf( "Addition of %d entries -- elapsed time: %2.1f seconds\n\n", added, duration );

	return(0);
}



#ifdef WB_SVR
/*-----------------------------------------------------------------
	uofLoaded
	Gets called whenever UOF is loaded by the workbench.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int uofLoaded(void *arg)
{
	if(!(ObjHandle = SystemConf->getUcloHandle())){ 
		printf("UOF not loaded\n");
		return -1;
	}
	return(0);
}
#endif


int initTest(char *uofName, int maxCollTable, int maxDataEle)
{
#if ((IOSTYLE != HARDWARE) && defined(STANDALONE))
	XactIoInit();
#endif

#if WB_SVR
	int WbSvr_init();

	/* initialize the loader and ueng libs */
	SystemConf = SysConf_Attach();
	SystemConf->Init();							// init libs

	UcLo_InitLib();

	// call me back when the UOF has been loaded
	SystemConf->SetUofCb(uofLoaded, NULL);
	WbSvr_init();	// start workbench remote services

	SystemConf->Reset(0, ALL_PORTS_MASK);		// don't reset microengines
	SystemConf->TakeControl();
	SystemConf->nextSymbol = 0;					// start with new import_vars
#endif

#if 0			// for now, no need to load the ucode
	/* Load the uCode object file */
	if(UcLo_LoadObjFile(&ObjHandle, uofName)){
		printf("Error; loading file %s\n", uofName);
		return(-1);
	}

	/* write uof to uStore */
	if(UcLo_WriteUimageAll(ObjHandle)){
	   printf("Error; failed to write Image to uStore\n");
	   return(-3);
	}
#endif


	// define the max of coll and data/key free list elements
//	unsigned int maxColEle = (0x80000 - COL_TABLE_BASE) / (1 << COL_SHF_BITS);
//	unsigned int keyDataPoolLen = 49152;			// max num of dataKey entries
	unsigned int maxColEle = maxCollTable;
	unsigned int keyDataPoolLen = maxDataEle;


	PriTableBase = PRIM_TABLE_BASE;
	SramFreeLstBase = COL_TABLE_BASE;
	SdramFreeLstBase = KEYDATA_SD_BASE;

	// create global free lists
	printf("dj: creating SRAM free list for %d %d-word elements; base=0x%x\n", maxColEle, (1<<COL_SHF_BITS), SramFreeLstBase);
	SramFreeLst = new FreeLst(HDBM_SWQ, FREELST_SRAM, SramFreeLstBase, (1<<COL_SHF_BITS), maxColEle);

#ifdef HDBM_EXT
	unsigned int keyPoolLen =  HDBM_INDEX_MASK;		// max num of key entries
	// create freelist of 2-word entries
	printf("dj: creating SDRAM free list for %d %d-quadword elements; base=0x%x\n", keyPoolLen, HDBM_KEY_QWORDS, SdramFreeLstBase);
	SdramFreeLst = new FreeLst(HDBM_SWQ, FREELST_SDRAM, SdramFreeLstBase, HDBM_KEY_QWORDS, keyPoolLen);
#else

	// create freelist of 3-word entries
	printf("dj: creating SDRAM free list for %d %d-word elements; base=0x%x\n", keyDataPoolLen, HDBM_KEYDATA_QWORDS, SdramFreeLstBase);
	SdramFreeLst = new FreeLst(HDBM_SWQ, FREELST_SDRAM, SdramFreeLstBase, HDBM_KEYDATA_QWORDS, keyDataPoolLen);
#endif


	HdbmPriTableDesc_T priTable;
	HdbmColTableDesc_T colTable;
	priTable.baseAddr = PriTableBase;
	priTable.indxStartBit = PRIM_INDX_START_BIT;
	priTable.numIndxBits = PRIM_INDX_BIT_LEN;

	colTable.indxStartBit = COL_INDX_START_BIT;
	colTable.numIndxBits = COL_INDX_BIT_LEN;
	colTable.colShiftBits = COL_SHF_BITS;
	colTable.freeLst = SramFreeLst;
	
	if(Hdb){
		deleDb();
		delete(Hdb);
	}
	printf("dj: creating hdbm object\n");
#ifdef HDBM_EXT
	Hdb = new HashDbmExt(HASH64, HASH_MULT_HI, HASH_MULT_LO, &priTable, &colTable, SdramFreeLst);
#else
	Hdb = new HashDBM(HASH48, HASH_MULT_HI, HASH_MULT_LO, &priTable, &colTable, SdramFreeLst);
#endif

    seedMT(4357U);	// seed random number generator
    return(0);
}


#if (IOSTYLE != HARDWARE)

int hdbm_model_initialize()
{
	XACT_printf("Calling hdbm_model_initialize...\n");
	if (!XACT_register_console_function("createDb", createDb, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() createDb failed\n");
			return(0);
	}
	if (!XACT_register_console_function("initTest", initTest, 1, 2) ){
            fprintf(stderr,"XACT_register_console_function() initTest failed\n");
			return(0);
	}
	if (!XACT_register_console_function("addDb", addDb, 0, 3) ){
            fprintf(stderr,"XACT_register_console_function() addDb failed\n");
			return(0);
	}
	if (!XACT_register_console_function("addRandomDb", addRandomDb, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() addRandomDb failed\n");
			return(0);
	}
	if (!XACT_register_console_function("lookupDb", lookupDb, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() lookupDb failed\n");
			return(0);
	}
	if (!XACT_register_console_function("openLog", openLog, 1, 0) ){
            fprintf(stderr,"XACT_register_console_function() openLog failed\n");
			return(0);
	}
	if (!XACT_register_console_function("closeLog", closeLog, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() closeLog failed\n");
			return(0);
	}
	if (!XACT_register_console_function("listDb", listDb, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() listDb failed\n");
			return(0);
	}
	if (!XACT_register_console_function("deleDb", deleDb, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() deleDb failed\n");
			return(0);
	}
#ifdef HDBM_DEBUG
	if (!XACT_register_console_function("analyzeDb", analyzeDb, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() analyzeDb failed\n");
			return(0);
	}
	if (!XACT_register_console_function("dumpColLst", dumpColLst, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() dumpColLst failed\n");
			return(0);
	}
	if (!XACT_register_console_function("dumpKeyLst", dumpKeyLst, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() dumpKeyLst failed\n");
			return(0);
	}
#endif
	return(1);
}


#ifdef HDBM_STANDALONE
extern "C" {
int foreign_model_initialize()
{
	hdbm_model_initialize();
	return(1);
}


/* this routine will be called prior to each transactor simulation event */
/* It can be used to deposit state values into the transactor model prior */
/* to simulating the next event.  Returning 0 results in an error. */
int foreign_model_pre_sim()
{
	return(1);
}



/* this routine will be called subsequent to each transactor simulation event. */
/* It can be used to query transactor simulation state, in order to copy it */
/* into the foreign model simulator. */
int foreign_model_post_sim()
{
	return(1);
}

/* this routine will be called just prior to exiting the simulator. */
/* The routine allows the foreign model to clean up, close files, etc */
/* before shutting down the program. */
int foreign_model_exit()
{
    return(1);
}

/* this routine will be called just after reseting the simulator. */
/* The routine allows the foreign model to reset itself to stay */
/* in sync with the simulator. */
int foreign_model_reset()
{
	return(1);
}

}			// extern "C"

#if defined(BUILD_DLL)
/*-----------------------------------------------------------------
    GetForeignModelFunctions

    This function is exported as the sole entry point into the DLL version
    of this package.  The Developer Workbench calls it in order to get the
    foreign model entry points for the transactor.  The Workbench thens
    registers these entry points with the transactor.
    
     returns: 
        uses: 
    modifies: 
*/
__declspec(dllexport) void __cdecl
GetForeignModelFunctions(
    int (**ForeignModelInitialize)(),
    int (**ForeignModelPreSim)(),
    int (**ForeignModelPostSim)(),
    int (**ForeignModelExit)(),
    int (**ForeignModelReset)())
{
    *ForeignModelInitialize = foreign_model_initialize;
    *ForeignModelPreSim = foreign_model_pre_sim;
    *ForeignModelPostSim = foreign_model_post_sim;
    *ForeignModelExit = foreign_model_exit;
    *ForeignModelReset = foreign_model_reset;
}
#endif      // #if defined(BUILD_DLL)

#endif		// #if HDBM_STANDALONE
#endif		// OS!=HARDWARE

