/* libd.cpp
 * Diapatcher Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Ucode Dispatcher Library
 * author: drj Sep 2, 99 Initial version
 * revisions:
 * $History: libd.cpp $
 * 
 * *****************  Version 6  *****************
 * User: Drjohns1     Date: 8/02/00    Time: 6:46p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Fixed init bug when using the C++ interface.
 * 
 * *****************  Version 5  *****************
 * User: Drjohns1     Date: 3/06/00    Time: 11:30a
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Bug fixes...
 * 
 * *****************  Version 3  *****************
 * User: Drjohns1     Date: 2/02/00    Time: 9:51a
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Corrected some comments
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 12/03/99   Time: 1:11p
 * Created in $/dev_1200/SA1_CoreLibs/libd
 *
 * ---------------------------------------------------------------------
 */

#include "libd_ver.h"
#include "libd.h"
#include "libd.hxx"


static const char __copyright[] =
"Copyright Intel Corp, 1997-2000. All rights reserved";

void autoInit(void);

#ifdef __cplusplus
extern "C" {
#endif



/*-----------------------------------------------------------------------------
   Function: 	LibD_RegisterCallback
   Description:  Register the callback of a function that can be invoked by a
   		 message from the ucomm driver. The library will maintain tables of
		 libraries and related functions that will be used by the LibD_Invoke
		 function to invoke the function.

   Returns:	 LIBD_SUCCESS, LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_RegisterCallback(void *libdHandle, unsigned char funcId,
			  int (*funcPtr) (void *inArg, void *outArg),
			  int inArgBytes, int outArgBytes)
{
   CoreAppPort *appPort = (CoreAppPort *)libdHandle;

   if(appPort)
	   return(appPort->RegCoreFunc(funcId, funcPtr, inArgBytes, outArgBytes));

   return (LIBD_ERROR);
}



/*-----------------------------------------------------------------------------
   Function: 	LibD_RegAlienCallout
   Description: Registers uEng function to be called by the core-application
   Returns:	pointer to the callout function or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
LibD_Callout_T *LibD_RegAlienCallout(void *libdHandle, unsigned char funcId, LibD_FuncType funcType,
			char *funcName, int inArgBytes, int outArgBytes)
{
   CoreAppPort *appPort = (CoreAppPort *)libdHandle;
   LibD_Callout_T *callout = new(LibD_Callout_T);

   if(callout && appPort){
      if((callout->reservedObjPtr = appPort->RegAlienCallout(funcId, funcType,
								inArgBytes, outArgBytes))) return(callout);
	  delete(callout);
   }
   return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_RegLocalCallout
   Description: Registers uEng function to be called by the core-application
   Returns:		A pointer to the callout function or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
LibD_Callout_T *LibD_RegLocalCallout(void *libdHandle, char *funcName)
{
   CoreAppPort *appPort = (CoreAppPort *)libdHandle;
   LibD_Callout_T *callout = new(LibD_Callout_T);

   if(callout && appPort){
      if((callout->reservedObjPtr = appPort->RegUofCallout(funcName))) return(callout);
	  delete(callout);
   }
   return(NULL);
}



/*-----------------------------------------------------------------------------
   Function: 	LibD_CallOut
   Description: This function will be call by the core-application to invoke
                a uEngine function.  It will create message from the arguments
				and send the message to the uEng driver via either a message
				queue or directly.  The driver doesn't support 2-way queued
				message, therefore, they will be sent as 1-way queued and the
				transaction ID will be tracked to determine the associated
				returned message.

   Returns:		LIBD_SUCCESS, LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_CallOut(LibD_Callout_T *funcHandle, void *inArg, void *outArg)
{
   if(!funcHandle || !funcHandle->reservedObjPtr) return(LIBD_ERROR);
   return(((Callout *)funcHandle->reservedObjPtr)->Invoke(funcHandle->destId, funcHandle->sync, &funcHandle->txId, inArg, outArg));
}



/*-----------------------------------------------------------------------------
   Function: 	LibD_Invoke
   Description: This function will invoke a core-application function that
                was previously	registered with this library. LibD_Invoke
				is typically called by the Ucomm driver library to provide
				a synchronous execution of a core-function on the local system.

   Returns:  LIBD_SUCCESS, LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_Invoke(unsigned int *msg)
{
   CoreAppPort *coreApp;
   unsigned char destId = GET_DEST_ID(msg[LIBD_MSGPREAMBLE]);
   if((coreApp = LibD_GetCoreApp(destId)))
	   return(coreApp->InvokeCoreFunc(msg));

  return (LIBD_ERROR);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_Init
   Description: Associates an application to a destination ID.
   Returns:		LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void *LibD_Init(void *uofHandle, unsigned char myId)
{
   CoreAppPort *appsPort = new CoreAppPort(myId, LIBD_DEFMSGQSIZE, uofHandle);
   LibD_AssignAppDestId(appsPort, myId);
   return((void *)appsPort);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_SetUofHandle
   Description: Associates the UOF handle with the dispatcher library.
   Returns:  LIBD_SUCCESS, LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_SetUofHandle(void *libdHandle, void *uofHandle)
{
   CoreAppPort *coreApp = (CoreAppPort *)libdHandle;

   if(!coreApp) return(LIBD_ERROR);
   coreApp->SetUofHandle(uofHandle);
   return(LIBD_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_Inquire
   Description: Checks if the reply of an asynchronous queued message is completed.
                If completed, then the func return-values will be copied to the
		outArg value that was passed in with LibD_Callout and also to
		the "output" parameter to this function, if it is not NULL.
   Returns:	LIBD_ERROR, LIBD_SUCCESS, LIBD_INCOMPLTE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_Inquire(LibD_Callout_T *funcHandle, void *outArg)
{
   if(funcHandle && funcHandle->reservedObjPtr){
      Callout *callout;
	  callout = (Callout *)funcHandle->reservedObjPtr;
	  return(callout->Inquire(funcHandle->destId, funcHandle->txId, outArg));
   }
   return(LIBD_ERROR);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_GetReply
   Description: Waits on the completion reply of an asynchronous queued message.
                If completed, then the msg return values will be copied to the
				outArg value.
   Returns:	LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_GetReply(LibD_Callout_T *funcHandle, void *outArg, unsigned int timeout)
{
   if(funcHandle && funcHandle->reservedObjPtr){
      Callout *callout;
	  callout = (Callout *)funcHandle->reservedObjPtr;
	  return(callout->GetReply(funcHandle->destId, funcHandle->txId, outArg, timeout));
   }
   return(LIBD_ERROR);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_Wait
   Description: Waits on the completion reply of an asynchronous queued message.
                If completed, then the msg return values will be copied to the
				outArg value.
   Returns:	LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_Wait(LibD_Callout_T *funcHandle, void *outArg)
{
   if(funcHandle && funcHandle->reservedObjPtr){
      Callout *callout;
	  callout = (Callout *)funcHandle->reservedObjPtr;
	  return(callout->WaitOnReply(funcHandle->destId, funcHandle->txId, outArg));
   }
   return(LIBD_ERROR);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_DeleHandle
   Description: Deletes the dispatcher library handle.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void LibD_DeleHandle(void *libdHandle)
{
   CoreAppPort *coreApp = (CoreAppPort *)libdHandle;

   if(coreApp) delete(coreApp);
   return;
}

/*-----------------------------------------------------------------------------
   Function: 	LibD_ProcessMsgQ
   Description: Process the application message queue by executing core
		or micro-engine functions.
   Returns:  LIBD_SUCCESS, LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_ProcessMsgQ(void *libdHandle, int wait)
{
   CoreAppPort *coreApp = (CoreAppPort *)libdHandle;

   if(!coreApp) return(LIBD_ERROR);
   coreApp->ProcessReqMsgQ(wait);
   return(LIBD_SUCCESS);
}


#ifdef __cplusplus
}
#endif
