/* libd.hxx
 * Dispatcher Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: IXP1200
 *  subsystem: Dispatcher Library
 *  author: drj Sept 2, 99  Initial version
 *  revisions:
 *  $History: libd.hxx $
 * 
  * ---------------------------------------------------------------------
 */
#ifndef __LIBD_HXX
#define __LIBD_HXX
#include "sem.h"
#include "thread.h"
#include "MsgQ.h"
#include "libd.h"
#include "ostype.h"


#define LIBD_MAXFUNCID	    	256
#define LIBD_MAXCOREFUNC    	LIBD_MAXFUNCID
#define LIBD_MAXUENGFUNC    	LIBD_MAXFUNCID
#define LIBD_MAXDEST			256
#define LIBD_MAXMSGTXID	    	7
#define LIBD_MAXMSGWRDLEN		100
#define LIBD_MAXMSGBYTLEN		4 * LIBD_MAXMSGWRDLEN
#define LIBD_MSGPREAMBLE		0
#define LIBD_MSGARGS			1
#define LIBD_MAXUTHREAD			24
#define LIBD_MAX_VARARG_BYTES	LIBD_MAXMSGBYTLEN - 12

#define LIBD_DEFMSGQSIZE		LIBD_MAXMSGBYTLEN

#if __cplusplus
extern "C"{
#endif
int LibD_GetMsgFormat(unsigned int argBytes);
#if __cplusplus
}
#endif

/* info pertaining to queued callout messages */
typedef struct LibD_AppQMsg_S {
   unsigned short txId;
   unsigned short srcId;
   unsigned short destId;
   unsigned int msg[LIBD_MAXMSGWRDLEN];
}LibD_appQMsg_T;

class Callout{
public:
   Callout (unsigned char srcId, unsigned char funcId, LibD_FuncType funcType, int format, int (*funcPtr)(void *inArg, void *outArg), int inArgBytes, int outArgBytes);
   int Invoke(unsigned char destId, int sync, short *txId, void *inArg, void *outArg);
   int InvokeMsgFunc(unsigned int *msg);
   int Inquire(unsigned char destId, int txId, void *outArg);
   int WaitOnReply(unsigned char destId, int txId, void *outArg);
   int GetReply(unsigned char destId, int txId, void *outArg, unsigned int timeout);
   int AsyncReply(void *msg);
   inline int inArgFormat(void){return(LibD_GetMsgFormat(inArgBytes));}
   inline int outArgFormat(void){return(LibD_GetMsgFormat(outArgBytes));}

private:
   unsigned int inArgBytes;						/* number of input bytes */ 
   unsigned int outArgBytes;					/* number of output bytes */
   LibD_appQMsg_T asyncMsg[LIBD_MAXMSGTXID];	/* array of outstanding messages */
   LibD_FuncType type;							/* ONEWAY or TWOWAY */
   int preamble;								/* message preamble */
   sem_T msgSync;								/* msg sync semaphore */
   int   msgWait;								/* msg sync condition variable */
   int (*funcPtr) (void *inArg, void *outArg);	/* pointer to non-ueng function */

   int getTxId(unsigned char destId);
};


//typedef struct CoreFunc_S {
//  short funcId;
//   char inMsgFormat;
//   char outMsgFormat;
//   int inArgBytes;
//   int outArgBytes;
//   int (*funcPtr) (void *inArg, void *outArg);
//}CoreFunc_T;


class CoreAppPort{
public:
   CoreAppPort(unsigned char myId, unsigned int msgQueSize, void *objHandle=NULL);
   ~CoreAppPort();
   Callout *RegAlienCallout(unsigned char funcId, LibD_FuncType funcType, int inArgBytes, int outArgBytes);
   Callout *RegUofCallout(char *funcName);
   inline int IsValid(void){return(valid);}
   inline void SetUofHandle(void *objHandle){uofHandle = (void *)objHandle;}

   int RegCoreFunc(unsigned char funcId,  int (*funcPtr) (void *inArg, void *outArg), int inArgBytes, int outArgBytes);
   int PutMsg(void *msg);
   int ProcessReqMsgQ(int wait);
   int InvokeCoreFunc(unsigned int *msg);

private:
   unsigned char myId;
   unsigned int numOfCoreFunc, valid;
   void *uofHandle;
   sem_T objSem;
   //MsgQ_Handle msgQHandle;
   MsgQ msgQ;
   Callout *uengFuncs[LIBD_MAXUENGFUNC];
//   CoreFunc_T *coreFuncs[LIBD_MAXCOREFUNC];
};

CoreAppPort *LibD_GetCoreApp(unsigned char destId);
int LibD_GetMsgBytesLen(unsigned int *msg);


#if __cplusplus
extern "C"{
#endif
int LibD_RouteMsg(unsigned int *msg);
void FDriver_PutMsg(int *msg, int threadNum);
void FDriver_PutMsgQue(int *msg, int threadNum);

int LibD_AssignAppDestId(CoreAppPort *objHandle, unsigned char myId);
#define LibD_PutMsgQue(msg) LibD_RouteMsg(msg)

#if __cplusplus
}
#endif

#endif		// __LIBD_HXX

