#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "utils.h"
#include "uclo.h"
#include "libd.h"
#include "thread.h"
#include "libdTest.h"
#include "ostype.h"

static void *LibdHandle = NULL;
static LibD_Callout_T *RemoteUofLoad = NULL;
static LibD_Callout_T *RemoteAdd = NULL;
static LibD_Callout_T *RemoteEcho = NULL;
static unsigned int Timeout = 5;		// 5 sec

   unsigned int HexUof_libdtest[] = {
      0xc6c2,0xc3cc,0x10001,0x5f464f55,0x534a424f,0x5104,0x20,0xf0,0xffffffff,0xf0000,
      0x30003,0x0,0x0,0x5f464f55,0x44495447,0x44,0x14,0x5f464f55,0x54525453,0x58,
      0x8,0x5f464f55,0x47414d49,0x60,0x90,0x646c6355,0x20202020,0xb,0x0,0x0,
      0x4,0x706f68,0x0,0xe00c0004,0x0,0x0,0xffffffff,0xf,0x3f,0x10000,
      0x500004,0x4,0x0,0xe8,0x98,0xec,0x12,0x0,0xdc000000,0xdc200080,
      0xdc400100,0xdc600180,0xdc800200,0xdca00280,0x40100be,0xe00c0104,0x42110be,0xe00c0104,0x44120be,0xe00c0104,
      0x46130be,0xe00c0104,0x48140be,0xe00c0104,0x4a150be,0xe00c0104,0x0,0x0};
     unsigned int HexUofSize_libdtest = sizeof(HexUof_libdtest);


int remoteEcho(char *txt)
{
    static unsigned int outArg[ECHOMAXLEN], inArg[ECHOMAXLEN], readOnly = 0;
	shrimpArgDesc_T *argDesc, *outArgDesc;

	argDesc = (shrimpArgDesc_T *)inArg;
	outArgDesc = (shrimpArgDesc_T *)outArg;
	RemoteEcho->destId = SRV_DESTID;
	RemoteEcho->sync = ASYNC;
	argDesc->addr = txt;
	argDesc->len = strlen(txt) + 1;


    if(LibD_CallOut(RemoteEcho, argDesc, &outArg) != LIBD_SUCCESS){
       printf("remoteEcho: echo failed\n"); fflush(stdout);
       return(-1);
    }

	if(RemoteEcho->sync == ASYNC){
		printf("remoteEcho: Waiting for completion\n"); fflush(stdout);
		if(LibD_GetReply(RemoteEcho, (void **)&outArg, Timeout) != LIBD_SUCCESS){
		   printf("remoteEcho: Error, RemoteEcho not completed\n"); fflush(stdout);
		   return(-1);
		}
	}
    if(outArgDesc->len < ECHOMAXLEN)
		printf("%.*s\n", outArgDesc->len, outArgDesc->addr);
	return(0);
}


int remoteLoadUof(void)
{
   unsigned int outArg[3], readOnly = 0;
   shrimpArgDesc_T argDesc;

	RemoteUofLoad->destId = SRV_DESTID;
	RemoteUofLoad->sync = ASYNC;
	argDesc.addr = (char *)HexUof_libdtest;
	argDesc.len = HexUofSize_libdtest;
    if(LibD_CallOut(RemoteUofLoad, &argDesc, &outArg) != LIBD_SUCCESS){
       printf("remoteLoadUof: LibD_CallOut failed\n"); fflush(stdout);
       return(-1);
    }

	printf("remoteLoadUof: Waiting for completion\n"); fflush(stdout);
    if(LibD_GetReply(RemoteUofLoad, (void **)&outArg, Timeout*4) != LIBD_SUCCESS){
       printf("remoteLoadUof: Error, RemoteUofLoad not completed\n"); fflush(stdout);
       return(-1);
    }
	printf("UOF successfully loaded\n");
	return(0);
}


int remoteAdd(unsigned int op1, unsigned int op2)
{
   unsigned int inArg[4], outArg;

	RemoteAdd->destId = SRV_DESTID;
	RemoteAdd->sync = ASYNC;
	inArg[0] = op1;
	inArg[1] = op2;
    
	if(LibD_CallOut(RemoteAdd, inArg, &outArg) != LIBD_SUCCESS){
       printf("remoteAdd: LibD_CallOut failed\n"); fflush(stdout);
       return(-1);
    }

	if(RemoteAdd->sync == ASYNC){
		printf("remoteAdd: Waiting for completion\n"); fflush(stdout);
		if(LibD_GetReply(RemoteAdd, (void *)&outArg, Timeout) != LIBD_SUCCESS){
		   printf("remoteAdd: Error, RemoteAdd not completed\n"); fflush(stdout);
		   return(-1);
		}
	}
	printf("remoteAdd: %d + %d = %d\n", op1, op2, outArg); fflush(stdout);
	return(0);
}


int testLib_init(int appId)
{
	unsigned int begUthread=0, numOfUthread=24;

    /* initialize the dispatcher library -- set the source ID */
    LibdHandle = LibD_Init(NULL, appId);

#ifdef LIBD_PCI
	int myPciDevNum, extDevNum = 0;
	myPciDevNum = LibD_GetLocalPciDevNum();
	if(myPciDevNum == extDevNum) extDevNum = 1;

	printf("MyPciDevId = %d, ExtPciDev = %d\n", myPciDevNum, extDevNum);
	// route these destination through the PCI device. The deviceNum is ignored.
    if(LibD_AssignExtDestId(LIBD_DEST_PCI, extDevNum, SRV_BEG_DESTID,
		SRV_NUMDESTID) != LIBD_SUCCESS){
		printf("Fail: cannot assign external PCI device %d\n", extDevNum);
		return(-1);
	}
#else
	int extDevNum = 1;	
	printf("External COM device ID = %d\n", extDevNum); fflush(stdout);
    if(LibD_AssignExtDestId(LIBD_DEST_SERIAL, extDevNum, SRV_BEG_DESTID,
		SRV_NUMDESTID) != LIBD_SUCCESS){
		printf("Fail: cannot assign external Serial device %d\n", extDevNum);
		return(-1);
	}
#endif

    RemoteUofLoad = LibD_RegAlienCallout(LibdHandle, UOF_FUNCID, TWOWAY,
			"uofLoad", HexUofSize_libdtest, FRMT_ARG_NONE);
    RemoteAdd = LibD_RegAlienCallout(LibdHandle, ADD_FUNCID, TWOWAY,"add", 8, 4);

    if(!(RemoteEcho = LibD_RegAlienCallout(LibdHandle, ECHO_FUNCID, TWOWAY,
			"echo", ECHOMAXLEN, ECHOMAXLEN))) printf("failed to register remoteEcho\n");

    return(0);    
}


void initClnt()
{
	static int once = 0;
    if(once) return;

	if((testLib_init(TSTAPP_DESTID)))
		printf("Init failed...\n");
	else once++;
	return;
}


#if (OS == UCOS)
void main_srv(void);
static int Argc;
static char **Argv;

int main(int argc, char** argv)
{
    Argc = argc;
    Argv = argv;

    /* Start uC/OS and run userInit in root task
     * with highest priority.  After userInit()
     * returns, root task commits suicide.
     */
    ucosStart(main_clnt, NO_CONSOLE);

    /* ucosStart() doesn't return. */
   return 0;
}

void main_clnt(void)
{
   int devNum = 1;
   if(Argc > 1) devNum = atoi(Argv[1]);

   initClnt(devNum);

   return;
}
#endif


#if (OS == WIN_32) || defined(WIN32)
extern "C" int getch(void);
int main(int argc, char **argv)
{
   int c, quit = 0, devNum = 1;
   if(argc > 1) devNum = atoi(argv[1]);
   initClnt();
   while(!quit){
	   printf("que> ");
	   c = getch();
	   switch(c){
		case 'a': remoteAdd(12, 43); break;
		case 'e': remoteEcho("Victory is mine, victory today is mine!"); break;
		case 'l': remoteLoadUof(); break;
		case 'q': quit++; break;
	   }
   }
   return(0);
}
#endif
