/* libd_exec.c
 * Diapatcher Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Ucode Dispatcher Library
 * author: drj Sep 2, 99 Initial version
 * revisions:
 * $History: libd_exec.cpp $
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 9/25/00    Time: 4:45p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Fixed LibdTest problem
 * 
 * *****************  Version 10  *****************
 * User: Drjohns1     Date: 8/02/00    Time: 6:46p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Fixed init bug when using the C++ interface.
 * 
 * *****************  Version 9  *****************
 * User: Drjohns1     Date: 4/26/00    Time: 4:57p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Renamed pciDriverAPI.h to pciCommAPI.h
 * ---------------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#ifdef LIBD_PCI
#include "pciCommAPI.h"
#endif
#ifndef NO_UCLO
#include "uclo.h"
#endif
#include "libd.hxx"
#include "sem.h"
#include "thread.h"
#include "MsgQMgr.h"
#include "libd_ver.h"


static const char __copyright[] =
"Copyright Intel Corp, 1997-2000. All rights reserved";


typedef struct LibD_LibHandle_S {
   void *objHandle;
   int   sourceId;
} LibD_LibHandle_T;


//sem_T Callback_sem, Callout_sem;

#ifdef __cplusplus
extern "C" {
#endif

/*-----------------------------------------------------------------------------
   Function: 	LibD_StrLowcase
   Description: Set string to lowercase characters
   Returns:	pointer to the string
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *LibD_StrLowcase(char *s)
{
   char *cs;
   for(cs = s; cs && *cs; cs++)
      *cs = tolower(*cs);
   return(s);
}

/*-----------------------------------------------------------------------------
   Function: 	LibD_PrintMsg
   Description: Prints the components of a SHRIMP msg
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void LibD_PrintMsg(unsigned int *msg)
{
   char transId, format, sync, ack, destId, sourceId;
   unsigned char funcId, i;

   if (!msg) return;

   /* get the func id and input arg, then call the related function */
   sync = GET_SYNC(msg[LIBD_MSGPREAMBLE]);
   ack = GET_ACK_REQ(msg[LIBD_MSGPREAMBLE]);
   format = GET_FORMAT(msg[LIBD_MSGPREAMBLE]);
   sourceId = GET_SOURCE_ID(msg[LIBD_MSGPREAMBLE]);
   destId = GET_DEST_ID(msg[LIBD_MSGPREAMBLE]);
   funcId = GET_FUNC_ID(msg[LIBD_MSGPREAMBLE]);
   transId = GET_TRANS_ID(msg[LIBD_MSGPREAMBLE]);
   printf("Msg: syn=%d, ack=%d, format=%d, source=%d, dest=%d, funcId=%d, transId=%d\n",
	  sync, ack, format, sourceId, destId, funcId, transId);
   printf("	args=");
   for (i = LIBD_MSGARGS; i <= format; i++)
      printf("0x%x ", msg[i]);
   printf("\n");
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_GetMsgFormat
   Description: Determines the message format based on the number of arg bytes.
   Returns:	    0-3 valid format, -1 invalid format
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_GetMsgFormat(unsigned int argBytes)
{
   if (argBytes == 0) return (FRMT_ARG_NONE);				// no argument
   if (argBytes < 9) return ((argBytes - 1) / 4 + 1);		// 1 thru 8 arg bytes
   return(FRMT_ARG_DESC);									// arg descriptor
}

/*-----------------------------------------------------------------------------
   Function: 	LibD_GetMsgArgBytesLen
   Description: Determine the length of the SHRIMP message arguments in bytes.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_GetMsgArgBytesLen(unsigned int *msg)
{
   int msgLen = LibD_GetMsgBytesLen(msg);
   if(msgLen < 0) return(0);			
   return(msgLen-4);
}


#ifdef __cplusplus
}
#endif



/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::CoreAppPort
   Description: Constructs a libd application thread object -- the member function
				'IsValid' should be called to determine it the object was properly
				created.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
CoreAppPort::CoreAppPort(unsigned char id, unsigned int msgQueSize, void *objHandle) : msgQ(0, msgQueSize, LIBD_MAXMSGBYTLEN)
{
   unsigned int i;

   void initRouter(void);
   initRouter();

   uofHandle = objHandle;
   objSem = Sem_Init(1);
   numOfCoreFunc = 0;
   myId = id;

   for(i = 0; i < LIBD_MAXUENGFUNC; i++) uengFuncs[i] = NULL;
   valid = 1;
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::~CoreAppPort
   Description: Destructs a libd application thread object.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
CoreAppPort::~CoreAppPort()
{
   for(unsigned int i = 0; i < LIBD_MAXCOREFUNC; i++)
	   if(uengFuncs[i]) delete(uengFuncs[i]);

   LibD_DeAssignDestId(myId);
   Sem_Destroy(objSem);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::RegAlienCallout
   Description: 
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Callout *CoreAppPort::RegAlienCallout(unsigned char funcId, LibD_FuncType funcType,
									  int inArgBytes, int outArgBytes)
{
   int format;

   // if funcId is negative or greater than the max funcId then error
   if(funcId == LIBD_NOFUNCID || funcId > LIBD_MAXFUNCID ||
       (funcType != ONEWAY && funcType != TWOWAY) ||
	   (uengFuncs[funcId])) return (NULL);

   if((format = LibD_GetMsgFormat(inArgBytes)) < 0) return (NULL);
   
   return((uengFuncs[funcId] = new Callout(myId, funcId, funcType, format, NULL, inArgBytes, outArgBytes)));
}


/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::RegUofCallout
   Description: 
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Callout *CoreAppPort::RegUofCallout(char *funcName)
{
#ifndef NO_UCLO
   UcLo_FuncInfo_T funcInfo;

   if(!uofHandle) return(NULL);		// uof object not loaded/specified
   LibD_StrLowcase(funcName);
   if (UcLo_GetFuncInfo(uofHandle, funcName, &funcInfo) != UCLO_SUCCESS)
	   return (NULL);

   return(RegAlienCallout((char)funcInfo.funcAddress, (LibD_FuncType)funcInfo.funcType,
						funcInfo.inArgBytes, funcInfo.outArgBytes));
#else
   return(NULL);
#endif
}

/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::RegCoreFunc
   Description: 
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int CoreAppPort::RegCoreFunc(unsigned char funcId,  int (*funcPtr)(void *inArg, void *outArg),
		         int in_argBytes, int out_argBytes)
{
   int inMsgFormat, outMsgFormat;

   /* is the array big enough to index by funcId? */
   if (funcId >= LIBD_MAXCOREFUNC || !funcPtr) return (LIBD_ERROR);

   /* get the in and out msg format */
   if((inMsgFormat = LibD_GetMsgFormat(in_argBytes)) < 0) return (LIBD_ERROR);
   if((outMsgFormat = LibD_GetMsgFormat(out_argBytes)) < 0) return (LIBD_ERROR);
   uengFuncs[funcId] = new Callout(myId, funcId, TWOWAY, inMsgFormat, funcPtr, in_argBytes, out_argBytes);
   return(LIBD_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::PutMsg
   Description: Handles incomming messages for the core-application. The message
				could either to be a request or a reply to an outstanding request.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int CoreAppPort::PutMsg(void *msg)
{
   unsigned int *m = (unsigned int *)msg, funcId, destId, ack;

   //printf("PutMsg: msg(0x%x 0x%x 0x%x 0x%x) into appl queue\n", m[3], m[2], m[1], m[0]); fflush(stdout); //dj:

   // ignore msg if it's not for us or invalid function id
   destId = GET_DEST_ID(m[LIBD_MSGPREAMBLE]);
   funcId = GET_FUNC_ID(m[LIBD_MSGPREAMBLE]);
   if(funcId >= LIBD_MAXCOREFUNC) return(LIBD_ERROR);
   if((destId != myId) || (!uengFuncs[funcId])) return(LIBD_ERROR);

   // if it's an ack, then place in app's context
   if((ack = GET_ACK_REQ(m[LIBD_MSGPREAMBLE])) == ACK)
      return(uengFuncs[funcId]->AsyncReply(m));

   // it's not a reply message--place message onto the app's queue
   int msgLen;
   unsigned int *mq;
   if((msgLen = LibD_GetMsgBytesLen(m)) < 0) return(LIBD_ERROR);
   if(!(mq = (unsigned int *)msgQ.AllocMsgBuf(LIBD_MAXMSGBYTLEN))) return(LIBD_ERROR);

 
   memmove(mq, m, msgLen);
   if(GET_FORMAT(m[LIBD_MSGPREAMBLE]) == FRMT_ARG_DESC){
	  // change the desc data address to be relative to the msgQ buf
	  mq[LIBD_MSGARGS+1] = (unsigned int)&mq[LIBD_MSGARGS+2];
	  memmove((void*)mq[LIBD_MSGARGS+1], (void *)m[LIBD_MSGARGS+1], m[LIBD_MSGARGS]);
   }

   //printf("PutMsg: Putting msg(0x%x 0x%x 0x%x 0x%x) into appl queue\n", mq[3], mq[2], mq[1], mq[0]); fflush(stdout); 
   if(msgQ.Put(mq, NOWAIT) != msgQ.MSGQ_TRUE) return(LIBD_ERROR);

   return (LIBD_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::ProcessMsgQ
   Description: Retrieve the message from the coreApp queue and execute the
				specified core function.  Only asynchornous request messages
				should be placed on this que -- not ACK or SYNC request.
   Returns:	LIBD_SUCCESS or LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int CoreAppPort::ProcessReqMsgQ(int wait)
{
   Wait w = WAIT;
   int status, funcId;

   if(!wait) w = NOWAIT;
   unsigned int *msg;

   //printf("ProcessMsgQ: Waiting for msg from appl queue...\n"); fflush(stdout);

   if(msgQ.Get((void **)&msg, w) != msgQ.MSGQ_TRUE) return(LIBD_ERROR);

   //printf("ProcessMsgQ: Got msg(0x%x 0x%x 0x%x 0x%x) from appl queue\n", msg[3], msg[2], msg[1], msg[0]); fflush(stdout);

   funcId = GET_FUNC_ID(msg[LIBD_MSGPREAMBLE]);
   if(uengFuncs[funcId]) status = uengFuncs[funcId]->InvokeMsgFunc(msg);

   //printf("ProcessMsgQ: InvokeMsgFunc returned %d\n", status); fflush(stdout);

   msgQ.FreeMsgBuf(msg, LIBD_MAXMSGBYTLEN);	/* return msg resource back to MsgQ */
   return(status);
}



/*-----------------------------------------------------------------------------
   Function: 	CoreAppPort::InvokeCoreFunc
   Description: This function will invoke a core-application function that
                was previously	registered with this library. InvokeCoreFunc
				is typically called by the Ucomm driver library.

   Returns:  LIBD_SUCCESS, LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int CoreAppPort::InvokeCoreFunc(unsigned int *msg)
{
   int funcId = GET_FUNC_ID(msg[LIBD_MSGPREAMBLE]);

   // verify the func ID
   if ((funcId > LIBD_MAXFUNCID) || !uengFuncs[funcId]) {
      SET_ERROR(msg[LIBD_MSGPREAMBLE], MSG_FAIL);	// set error flag
      return (LIBD_ERROR);							// func not defined
   }
   return(uengFuncs[funcId]->InvokeMsgFunc(msg));
}



/*-----------------------------------------------------------------------------
   Function: 	Callout::Callout
   Description: Constructs a callout object
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Callout::Callout(unsigned char srcId, unsigned char funcId, LibD_FuncType funcType, int format,
				 int (*functionPtr)(void *inArg, void *outArg),
				 int in_argBytes, int out_argBytes)
{
   /* DEST will be set when the function is invoked; others are set below */
   preamble = 0;
   SET_ERROR(preamble, MSG_OK);				/* NO ERROR */
   SET_ACK_REQ(preamble, REQ);				/* REQUEST */
   SET_FUNC_ID(preamble, funcId);
   SET_SOURCE_ID(preamble, srcId);

   /* Note: the sync bit is really the PEND bit which indicates the function type */
   if(funcType == TWOWAY){
       SET_SYNC(preamble, SYNC);			/* non-pended message */
   }
   else{
       SET_SYNC(preamble, ASYNC);			/* pend message */
   }

   type = funcType;
   inArgBytes = in_argBytes;
   outArgBytes = out_argBytes;
   funcPtr = functionPtr;

   SET_FORMAT(preamble, format);
   msgSync =  Sem_Init(0);
   msgWait = 0;

   for(int i = 0; i < LIBD_MAXMSGTXID; i++){
      asyncMsg[i].msg[LIBD_MSGPREAMBLE] = 0;
      asyncMsg[i].txId = 0;
      asyncMsg[i].srcId = (short)-1;
      asyncMsg[i].destId = (short)-1;
   }

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	Callout::InvokeMsgFunc
   Description: Execute the indicated registered core function.  If the function
				is a two-way function, then the result will returned either in
				the input argument msg, (for sync type messages); or queued to the
				sourceId.  If bad argument or if the function failed, then the
				result message will be returned without arguments -- shrimp message
				format = FRMT_ARG_NONE -- and the shrimp message err bit set.

   Returns:	LIBD_SUCCESS or LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Callout::InvokeMsgFunc(unsigned int *msg)
{
   int sync;
   unsigned char tmpId, destId;
   unsigned int outArgs[LIBD_MAXMSGWRDLEN], *replyHdr, *replyArg;

   sync = GET_SYNC(msg[LIBD_MSGPREAMBLE]);
   if(sync == SYNC){
	   replyHdr = msg;
	   replyArg = &msg[LIBD_MSGARGS];
   }
   else{
	  outArgs[LIBD_MSGPREAMBLE] = msg[LIBD_MSGPREAMBLE];
      replyHdr = outArgs;
	  replyArg = &outArgs[LIBD_MSGARGS];
   }

   // prepare reply msg header
   if(type == TWOWAY){
      tmpId = GET_SOURCE_ID(replyHdr[LIBD_MSGPREAMBLE]);
      destId = GET_DEST_ID(replyHdr[LIBD_MSGPREAMBLE]);
      SET_SOURCE_ID(replyHdr[LIBD_MSGPREAMBLE], destId);
      SET_DEST_ID(replyHdr[LIBD_MSGPREAMBLE], tmpId);
      SET_ACK_REQ(replyHdr[LIBD_MSGPREAMBLE], ACK);
      //printf("InvokeMsgFunc: reply preamble=0x%x\n", msg[LIBD_MSGPREAMBLE]); fflush(stdout);
   }

   if(!funcPtr || (inArgFormat() != (int)GET_FORMAT(replyHdr[LIBD_MSGPREAMBLE]))){
      SET_ERROR(replyHdr[LIBD_MSGPREAMBLE], MSG_FAIL);
	  SET_FORMAT(replyHdr[LIBD_MSGPREAMBLE], FRMT_ARG_NONE);
      //printf("InvokeMsgFunc: Error NULL functPtr\n"); fflush(stdout);
   }
   else{
      if(outArgFormat() == FRMT_ARG_DESC){
		 shrimpArgDesc_T *inArgDesc = (shrimpArgDesc_T *)&msg[LIBD_MSGARGS];
		 if(inArgDesc->len && (inArgDesc->len < LIBD_MAX_VARARG_BYTES)){
		 	inArgDesc->addr = (char *)((unsigned int)inArgDesc + sizeof(shrimpArgDesc_T));	// re-adjust the pointer
		 }
		 else inArgDesc->addr = NULL;
	  }

	  //printf("InvokeMsgFunc: executing funcPtr\n"); fflush(stdout);
      if((funcPtr)((void **)&msg[LIBD_MSGARGS], (void **)replyArg)){
         SET_ERROR(replyHdr[LIBD_MSGPREAMBLE], MSG_FAIL);
	     SET_FORMAT(replyHdr[LIBD_MSGPREAMBLE], FRMT_ARG_NONE);
		 //printf("InvokeMsgFunc: Error, funcPtr failed\n"); fflush(stdout);
	  }
	  else{
	     // change the reply format
		 int argFrmt = outArgFormat();
	     SET_FORMAT(replyHdr[LIBD_MSGPREAMBLE], argFrmt);
         SET_ERROR(replyHdr[LIBD_MSGPREAMBLE], MSG_OK);

		 // copy the returned descriptor and data to result
		 if(argFrmt == FRMT_ARG_DESC){
			shrimpArgDesc_T *rplyDesc = (shrimpArgDesc_T *)replyArg;
			if(rplyDesc->len && (rplyDesc->len < LIBD_MAX_VARARG_BYTES)){
			   char *dPtr = (char *)((unsigned int)&replyArg + sizeof(shrimpArgDesc_T));
			   memmove(dPtr, rplyDesc->addr, rplyDesc->len);
		 	   rplyDesc->addr = dPtr;	// re-adjust the pointer
			}
			else rplyDesc->addr = NULL;
		 }
	  }
   }

   if(type == TWOWAY) LibD_RouteMsg(replyHdr);

   return(LIBD_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Callout::GetReply
   Description: Check if the reply of an asynchronous queued message is completed.
                If completed, then the func return-values will be copied to the
				outArg value.

   Returns:		LIBD_ERROR, LIBD_SUCCESS, LIBD_ERR_COMPLTED, LIBD_INCOMPLTE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Callout::GetReply(unsigned char destId, int txId, void *outArg, unsigned int timeout)
{
   int stat = LIBD_INCOMPLETE;
   time_t endTime, elapsedTime;

   time(&endTime);
   endTime += timeout;

   while((stat == LIBD_INCOMPLETE) && (endTime > time(&elapsedTime))){
	  if((stat = Inquire(destId, txId, outArg)) == LIBD_SUCCESS) break;
	  if((stat == LIBD_ERROR) || (stat == LIBD_ERR_COMPLETED)) break;

	  /* set waiting for msg flag */
	  msgWait = 1;
	  Sem_Wait(msgSync, 1000);	// semaphore is "given" in AsyncReply
	  msgWait = 0;
   }

   return(stat);
}


/*-----------------------------------------------------------------------------
   Function: 	Callout::WaitOnReply
   Description: Check if the reply of an asynchronous queued message is completed.
                If completed, then the func return-values will be copied to the
				outArg value.

   Returns:		LIBD_ERROR, LIBD_SUCCESS, LIBD_ERR_COMPLTED, LIBD_INCOMPLTE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Callout::WaitOnReply(unsigned char destId, int txId, void *outArg)
{
   int stat = LIBD_INCOMPLETE;

   while(stat == LIBD_INCOMPLETE){
	  if((stat = Inquire(destId, txId, outArg)) == LIBD_SUCCESS) break;
	  if((stat == LIBD_ERROR) || (stat == LIBD_ERR_COMPLETED)) break;

	  /* set waiting for msg flag */
	  msgWait = 1;
	  Sem_Wait(msgSync, SEM_WAITFOREVER);	// semaphore is "given" in AsyncReply
	  msgWait = 0;
   }

   return(stat);
}


/*-----------------------------------------------------------------------------
   Function: 	Callout::AsyncReply
   Description: This function will store the reply message in the context of the
				caller that initiated the queued callout message.

   Returns:	LIBD_SUCCESS or LIBD_ERROR
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Callout::AsyncReply(void *msg)
{
   unsigned int *m = (unsigned int *)msg;
   unsigned int txId, srcId, destId, funcId;
   int msgLen;

   //printf("Callout::AsyncReply: obj=0x%x, msgLIBD_MSGPREAMBLE]=0x%x\n", (unsigned int)this, m[LIBD_MSGPREAMBLE]); fflush(stdout); //dj:
   if((msgLen = LibD_GetMsgBytesLen(m)) < 0) return(LIBD_ERROR);

   if((txId = GET_TRANS_ID(m[LIBD_MSGPREAMBLE])) >= LIBD_MAXMSGTXID){
	  //printf("Callout::AsyncReply: Error, txId=%d >= LIBD_MAXMSGTXID\n", txId); fflush(stdout);
      return(LIBD_ERROR);
   }

   srcId = GET_SOURCE_ID(m[LIBD_MSGPREAMBLE]);
   destId = GET_DEST_ID(m[LIBD_MSGPREAMBLE]);
   funcId = GET_FUNC_ID(m[LIBD_MSGPREAMBLE]);


   // place the message in the context of the caller that initiated the request
   if(asyncMsg[txId].msg[LIBD_MSGPREAMBLE] == 0 &&
      asyncMsg[txId].txId == txId &&
      asyncMsg[txId].srcId == srcId &&
      asyncMsg[txId].destId == destId){

	  if(outArgFormat() == FRMT_ARG_DESC){
		 // copy the header, msgDescriptor, and the message
		 memmove(asyncMsg[txId].msg, m, sizeof(int) + sizeof(shrimpArgDesc_T));		// save the header and desc
		 asyncMsg[txId].msg[LIBD_MSGARGS+1] = (unsigned int)&asyncMsg[txId].msg[LIBD_MSGARGS+2];
         memmove((void *)asyncMsg[txId].msg[LIBD_MSGARGS+1], (void *)m[LIBD_MSGARGS+1], m[LIBD_MSGARGS]);
	  }
	  else{
	     memmove(asyncMsg[txId].msg, m, msgLen);	// copy the shrimp message
	  }

	  //printf("Callout::AsyncReply: msglen = %d, asyncMsg[txId].msg(0x%x 0x%x 0x%x 0x%x)\n", msgLen, asyncMsg[txId].msg[3], asyncMsg[txId].msg[2], asyncMsg[txId].msg[1], asyncMsg[txId].msg[0]); fflush(stdout); //dj:
      if(msgWait) Sem_Post(msgSync);
      return(LIBD_SUCCESS);
   }
   return(LIBD_ERROR);
}


/*-----------------------------------------------------------------------------
   Function: 	Callout::Inquire
   Description: Checks if the reply of an asynchronous queued message is completed.
                If completed, then the func return-values will be copied to the
		outArg value that was passed in with LibD_Callout and also to
		the "output" parameter to this function, if it is not NULL.
   Returns:	LIBD_ERROR, LIBD_SUCCESS, LIBD_INCOMPLTE, LIBD_ERRCOMPLTE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Callout::Inquire(unsigned char destId, int txId, void *outArg)
{
   int funcStatus;

   //printf("Callout::Inquire: Obj=0x%x, destId=%d txId=%d\n", (unsigned int)this, destId, txId); fflush(stdout);

   if(txId == 0 || txId >= LIBD_MAXMSGTXID || (!outArg && outArgBytes)){
     //printf("Callout::Inquire: Invalid txId=%d\n", txId); fflush(stdout);
     return(LIBD_ERROR);
   }

   //Sem_Wait(Callout_sem, SEM_WAITFOREVER);
   if(asyncMsg[txId].msg[LIBD_MSGPREAMBLE] != 0 &&
      (asyncMsg[txId ].txId == txId) && (asyncMsg[txId ].srcId == destId)){
	  funcStatus = GET_ERROR(asyncMsg[txId].msg[LIBD_MSGPREAMBLE]);

      //Sem_Post(Callout_sem);

      if(funcStatus == MSG_FAIL){
         //printf("Callout::Inquire: Completed with error\n"); fflush(stdout);
	     asyncMsg[txId].msg[LIBD_MSGPREAMBLE] = 0;
         return(LIBD_ERR_COMPLETED);		/* completed with error */
      }

	  if(outArgBytes){
		 int msgArgLen = LibD_GetMsgArgBytesLen(asyncMsg[txId].msg);
		 memmove(outArg, &asyncMsg[txId].msg[LIBD_MSGARGS], msgArgLen);
		 if(outArgFormat() == FRMT_ARG_DESC){
			unsigned int *oArg = (unsigned int *)outArg;
			// change desc addr to caller's
		    shrimpArgDesc_T *argDesc = (shrimpArgDesc_T *)&oArg[LIBD_MSGARGS];
		    argDesc->addr = (char *)&oArg[LIBD_MSGARGS+2];
		 }
	  }

      //printf("Callout::Inquire: Completed\n"); fflush(stdout);
	  asyncMsg[txId].msg[LIBD_MSGPREAMBLE] = 0;
      return(LIBD_SUCCESS);
    }

   //Sem_Post(Callout_sem);
   return(LIBD_INCOMPLETE);
}


/*-----------------------------------------------------------------------------
   Function: 	Callout::GetTxId
   Description: GetTxId is used to store a value/structure that can be used
   		to determine a reply to an outstanding message.
   Returns:	The transaction ID, or MAXMSGTXID
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Callout::getTxId(unsigned char destId)
{
   int i;

   for(i = 1; i < LIBD_MAXMSGTXID; i++)
      if(asyncMsg[i].msg[LIBD_MSGPREAMBLE] == 0){
         asyncMsg[i].srcId = destId;
         asyncMsg[i].destId = GET_SOURCE_ID(preamble);
         asyncMsg[i].txId = i;
	     break;
      }
   return(i);
}



/*-----------------------------------------------------------------------------
   Function: 	Callout::Invoke
   Description: Invoke an Application or micro-engine function locally or remotely.
				A transaction identifier value between 1-7 is returned in txId to
				which could be used to query the completion of an asynchronous
				transaction. Synchronous transactions identifier is always 0.
				Remote functions can only be invoked asynchronously.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Callout::Invoke(unsigned char destId, int sync, short *txId, void *inArg, void *outArg)
{
   unsigned int msg[LIBD_MAXMSGWRDLEN];
   int localDev, status, argLen;

   if((localDev = LibD_IsLocalDev(destId)) == -1) return(LIBD_ERROR);
   if((!inArg && inArgBytes) || (!outArg && outArgBytes)) return(LIBD_ERROR);

   msg[LIBD_MSGPREAMBLE] = preamble;
   SET_DEST_ID(msg[LIBD_MSGPREAMBLE], destId);

   if(inArgFormat() == FRMT_ARG_DESC){
	  shrimpArgDesc_T *argDesc = (shrimpArgDesc_T *)inArg;

	  if(((argLen = argDesc->len)) > LIBD_MAX_VARARG_BYTES) return(LIBD_ERROR);

	  if(argDesc->len){
		 shrimpArgDesc_T *msgDesc = (shrimpArgDesc_T *)&msg[LIBD_MSGARGS];

		 //dj: This needs to be changed whenever Fdriver and svLib-ucode are changed!!
		 //dj: FDriver cannot handle FRMT_ARG_DESC message type...so pass the first 4 long-words
	     if(localDev && !funcPtr ){
		    if(argDesc->len > 12) return(LIBD_ERROR);
            memmove(&msg[LIBD_MSGARGS], argDesc->addr, argDesc->len);
		 }
		 else{

		    // construct the shrimp descriptor message
            memmove(msgDesc, (void*)argDesc, sizeof(shrimpArgDesc_T));	// copy the desc
		    msgDesc->addr = (char *)((unsigned int)msgDesc + sizeof(shrimpArgDesc_T));
            memmove(msgDesc->addr, argDesc->addr, argDesc->len);		// copy the data
		 }
	  }
   }
   else{
      memmove(&msg[LIBD_MSGARGS], inArg, inArgBytes);		// copy the input args
   }

   // if local device and sync, call function and wait; else place msg on que and return
   if (localDev && (sync == SYNC)){

	  // determine if it's a micro-engine or core function
	  if(!funcPtr){
         // execute the micro-engine function
	     int uThreaNum;

         SET_TRANS_ID(msg[LIBD_MSGPREAMBLE], 0);	// transaction ID = 0 for SYNC mesages
         if((uThreaNum = LibD_GetUThreadNum(destId)) < 0) return(LIBD_ERROR);

	     FDriver_PutMsg((int *)msg, uThreaNum);		    // call uComm driver with synchronous message

         // extract the output arg from the message
		 if(type == TWOWAY){
			// return fail if the function failed
			if(GET_ERROR(msg[LIBD_MSGPREAMBLE] == MSG_FAIL)) return(LIBD_ERROR);

			// copy the output -- no more than the expected number of bytes
			if(outArgFormat() == FRMT_ARG_DESC){

//dj: Remove whenever Fdriver and svLib-ucode accept FRMT_ARG_DESC message type!!
#if 1
			   int *outPtr = (int *)outArg;
		       shrimpArgDesc_T *outDesc = (shrimpArgDesc_T *)outPtr[LIBD_MSGARGS];
			   outPtr[LIBD_MSGPREAMBLE] = msg[LIBD_MSGPREAMBLE];
		       outDesc->len = outArgBytes;
			   outDesc->addr = (char *)((unsigned int)outArg) + sizeof(shrimpArgDesc_T);
               memmove(outDesc->addr, outDesc->addr, outDesc->len);		// copy the data
		       return(LIBD_SUCCESS);
#else
               memmove(&msg[LIBD_MSGARGS], argDesc->addr, argDesc->len);
			   if((argLen = LibD_GetMsgArgBytesLen(msg)) > LIBD_MAXMSGBYTLEN) return(LIBD_ERROR);
#endif
			}
			else argLen = outArgBytes;
			memmove(outArg, &msg[LIBD_MSGARGS], argLen);
		    return(LIBD_SUCCESS);
		 }
		 return(LIBD_ERROR);
	  }
	  else{
         // execute the core function
		 return((status = (funcPtr)(inArg, outArg)));
	  }
   }

   if(sync == SYNC) return(LIBD_ERROR);  // no remote syn messages
   int xid = 0;
   // Is an Async message; so, if TWOWAY message, get transaction ID
   if(type == TWOWAY){
	  if((xid = getTxId(destId)) == LIBD_MAXMSGTXID) return(LIBD_ERROR);
   }

   SET_TRANS_ID(msg[LIBD_MSGPREAMBLE], xid);	// set transaction identification
   if(txId) *txId = xid;
   return(LibD_RouteMsg(msg));
}

