/* libd_router.c
 * Diapatcher Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Ucode Dispatcher Library
 * author: drj Dec 31, 97 Initial version
 * revisions:
 * $History: libd_router.cpp $
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 10/30/00   Time: 2:43p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Replaced cacheDFlush with cacheFlush -- fix SCR #1179 
 * 
 * *****************  Version 15  *****************
 * User: Drjohns1     Date: 9/25/00    Time: 4:45p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Fixed LibdTest problem
 * 
 * *****************  Version 13  *****************
 * User: Drjohns1     Date: 8/02/00    Time: 6:46p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Fixed init bug when using the C++ interface.
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 4/26/00    Time: 4:57p
 * Updated in $/dev_1200/SA1_CoreLibs/libd
 * Renamed pciDriverAPI.h to pciCommAPI.h
 * 
 * 
 *---------------------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>

#ifdef LIBD_PCI
#include "pciCommAPI.h"
#endif

#ifdef LIBD_SERIAL
#include "scomm.h"
#endif

#include "utils.h"
#include "uclo.h"
#include "libd.hxx"
#include "libd_ver.h"

#if(OS == VXWORKS)
#include <cacheLib.h>
#endif

static const char __copyright[] =
"Copyright Intel Corp, 1997-2000. All rights reserved";

#ifdef __cplusplus
extern "C" {
#endif


#if __cplusplus
}
#endif



#if (OS == UCOS) || defined(NO_UCOMM)
// Waiting for FDriver being ported
void FDriver_PutMsg(int *msg, int uthreadNum){return;};
void FDriver_PutMsgQue(int *msg, int threadNum){return;};
#else
#endif

/*****************************************************************************/

#if (OS == UCOS)
#define SERIAL_TH_PRIORITY MID_PRIORITY + 1
#define PCI_TH_PRIORITY MID_PRIORITY + 2
#else
#define SERIAL_TH_PRIORITY MID_PRIORITY
#define PCI_TH_PRIORITY MID_PRIORITY
#endif


typedef enum{
	LIBD_DEVICE_PCI,
	LIBD_DEVICE_SERIAL
}DevType_T;


typedef struct{
	int destType;
	union{
		int uthreadNum;
		int devNum;
		CoreAppPort *coreApp;
	};
}DestTable_T;

typedef struct {
	void     *handle;
	int		 active;
    thread_T threadId;
}LibdDevice_T;


static const unsigned int LIBD_CHANNEL = 4, LIBD_MAX_PCI_DEVICES = 8, LIBD_MAX_DEVNAME = 64;
static const unsigned int LIBD_MAX_SERIAL_DEVICES = 2;

static LibdDevice_T PciDevices[LIBD_MAX_PCI_DEVICES];
static LibdDevice_T SerialDevices[LIBD_MAX_SERIAL_DEVICES];

static DestTable_T DestTable[LIBD_MAXDEST];
static sem_T DispatchSem, PciSem, PciAckSem;
static unsigned int AckMsg[4];


void LibD_GetPciMsg(void *devHandle);
void LibD_GetSerialMsg(void *sConnId);


void initRouter(void)
{
   unsigned int i;
   static int once = 0;

   if(once) return;


   for(i = 0; i < LIBD_MAXDEST; i++) DestTable[i].destType = LIBD_DEST_NONE;
   DispatchSem = Sem_Init(1);

 #ifdef LIBD_PCI
   PciSem = Sem_Init(1);
   PciAckSem = Sem_Init(1);
   Sem_Wait(PciAckSem, SEM_WAITFOREVER);
   AckMsg[0] = 0xf0000000;
   AckMsg[1] = 0xffffffff;
   AckMsg[2] = 0xffffffff;
   
   for(i = 0; i < LIBD_MAX_PCI_DEVICES; i++){
      PciDevices[i].handle = NULL;
	  PciDevices[i].active = 0;
   }
   Pci_Init();

#endif
#ifdef LIBD_SERIAL
   for(i = 0; i < LIBD_MAX_SERIAL_DEVICES; i++){
	  SerialDevices[i].handle = NULL;
	  SerialDevices[i].active = 0;
   }
#endif
   once = 1;
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_GetMsgBytesLen
   Description: Determine the length of the SHRIMP message in bytes.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_GetMsgBytesLen(unsigned int *msg)
{
   int msgLen;
   shrimpArgDesc_T *argDesc;

   switch(GET_FORMAT(msg[LIBD_MSGPREAMBLE])){
	  case FRMT_ARG_NONE: msgLen = 4;  break;			// header only
	  case FRMT_ARG_1WRD: msgLen = 8;  break;			// header + 1 word
	  case FRMT_ARG_2WRD: msgLen = 12; break;			// header + 2 words message
	  case FRMT_ARG_DESC:
		  argDesc = (shrimpArgDesc_T *)&msg[LIBD_MSGARGS];
		  msgLen = 12 + argDesc->len; break;			// header + 2 word (len, addr) + len bytes
   }
   if(msgLen > LIBD_MAXMSGBYTLEN) return(-1);
   return(msgLen);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_EnableDevice
   Description: Open device channel and spawn task to read and dispatch messages.
   Returns:	LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_EnableDevice(DevType_T devType, unsigned int devNum)
{
#if defined(LIBD_PCI) || defined(LIBD_SERIAL)
	char devName[LIBD_MAX_DEVNAME];
#endif

	switch(devType){

	   case LIBD_DEVICE_PCI:
#ifdef LIBD_PCI
	      if(PciDevices[0].active) return(LIBD_ERROR);
		  sprintf(devName, "pci%d", devNum+1);
		  PciDevices[0].handle = (void *)devNum;

		  // spawn thread to handle input from the channel
		  PciDevices[0].threadId = Thread_Create(PCI_TH_PRIORITY, (threadFunc_T)LibD_GetPciMsg, (void *)&PciDevices[0]);
#endif
		  break;
	   case LIBD_DEVICE_SERIAL:
#ifdef LIBD_SERIAL
		  if(SerialDevices[devNum].active) return(LIBD_ERROR);
		  sprintf(devName, "com%d", devNum+1);
		  SComm_InstallDev(devName);
		  SerialDevices[devNum].handle = (void *)SComm_Open(devName, LIBD_CHANNEL);

		  // spawn thread to handle input from the channel
		  SerialDevices[devNum].threadId = Thread_Create(SERIAL_TH_PRIORITY, (threadFunc_T)LibD_GetSerialMsg, (void *)&SerialDevices[devNum]);
#endif
		  break;
	   default: return(LIBD_ERROR);
	}

	return(LIBD_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_AssignUThreadId
   Description: Assign destId to micro-engine threads
   Returns:	LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_AssignUThreadId(unsigned char destId, unsigned int begUthread=0,
					     unsigned int numOfUthread=24)
{
	if(((destId + numOfUthread) >= LIBD_MAXDEST) ||
		((begUthread + numOfUthread) > LIBD_MAXUTHREAD)) return(LIBD_ERROR);

    Sem_Wait(DispatchSem, SEM_WAITFOREVER);
	for(unsigned int i = 0; i < numOfUthread; i++, destId++){
		DestTable[destId].uthreadNum = begUthread++;
		DestTable[destId].destType = (char)LIBD_DEST_UENG;
	}
    Sem_Post(DispatchSem);
	return(LIBD_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	LibD_AssignAppDestId
   Description: Associates an application with a destination ID.
   Returns:	LIBD_SUCCESS -- success
			LIBD_ERROR   -- failure
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_AssignAppDestId(CoreAppPort *coreApp, unsigned char destId)
{
   if(destId >= LIBD_MAXDEST) return(LIBD_ERROR);

   Sem_Wait(DispatchSem, SEM_WAITFOREVER);
   DestTable[destId].destType = LIBD_DEST_APP;
   DestTable[destId].coreApp = coreApp;
   Sem_Post(DispatchSem);

   return(LIBD_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_AssignExtDestId
   Description: Define the external device that the destination ID's
				are associated with.  Therefore, the associated ID's
				will be routed through the specified device.
   Returns:	LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_AssignExtDestId(PortType_T portType, unsigned int devNum, unsigned int begDestId,
					  unsigned int numOfDestId)
{
#if defined(LIBD_PCI) || defined(LIBD_SERIAL)
    unsigned int i;
#endif

	if((begDestId + numOfDestId) > LIBD_MAXDEST) return(LIBD_ERROR);

	//printf("LibD_AssignExtDestId: devNum=%d, begDestId=%d, numOfDestId=%d\n", devNum, begDestId, numOfDestId); fflush(stdout);

	switch(portType){
	   case LIBD_DEST_SERIAL:
#ifdef LIBD_SERIAL
		  if((devNum == 0) || (devNum > LIBD_MAX_SERIAL_DEVICES)) return(LIBD_ERROR);
          Sem_Wait(DispatchSem, SEM_WAITFOREVER);
		  devNum--;
		  if(!SerialDevices[devNum].active){ 
			  if(LibD_EnableDevice(LIBD_DEVICE_SERIAL, devNum)){
				  Sem_Post(DispatchSem);
				  return(LIBD_ERROR);
			  }
              SerialDevices[devNum].active = 1;
		  }

		  for(i = 0; i < numOfDestId; begDestId++, i++){
		     DestTable[begDestId].destType = (char)portType;
		     DestTable[begDestId].devNum = (short)devNum;
		  }
          Sem_Post(DispatchSem);
#endif
		  break;
	   
	   case LIBD_DEST_PCI:
#ifdef LIBD_PCI
		  // ignore the specified device num -- we only need to know about one
          Sem_Wait(DispatchSem, SEM_WAITFOREVER);
		  if(!PciDevices[0].active){
			  if(LibD_EnableDevice(LIBD_DEVICE_PCI, LibD_GetLocalPciDevNum())){
				  Sem_Post(DispatchSem);
				  return(LIBD_ERROR);
			  }
			  PciDevices[0].active = 1;
		  }
		  for(i = 0; i < numOfDestId; begDestId++, i++){
		     DestTable[begDestId].destType = (char)portType;
		     DestTable[begDestId].devNum = (short)devNum;
		  }
          Sem_Post(DispatchSem);
#endif
	   break;

	   default: return(LIBD_ERROR);
	   break;
	}
	
	return(LIBD_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_DeAssignDestId
   Description: Deassign the destination ID.
   Returns:	LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_DeAssignDestId(unsigned char destId)
{
   if(destId < LIBD_MAXDEST){
      Sem_Wait(DispatchSem, SEM_WAITFOREVER);
      DestTable[destId].destType = LIBD_DEST_NONE;
      Sem_Post(DispatchSem);
	  return(LIBD_SUCCESS);
   }
   return(LIBD_ERROR);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_GetUThreadNum
   Description: Returns the micro-engine thread number
   Returns:	the micro-thread number or -1 if error
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_GetUThreadNum(unsigned char destId)
{
	if(destId < LIBD_MAXDEST) return(DestTable[destId].uthreadNum);
	return(-1);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_GetCoreApp
   Description: Returns a pointer to the core application with the specified Id.
   Returns:	pointer to object or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
CoreAppPort *LibD_GetCoreApp(unsigned char destId)
{
	if(destId < LIBD_MAXDEST) return(DestTable[destId].coreApp);
	return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_GetDevNum
   Description: Returns the device number
   Returns:	    the device number or -1 if error
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_GetDevNum(unsigned char destId)
{
	if(destId < LIBD_MAXDEST) return(DestTable[destId].devNum);
	return(-1);
}


/*-----------------------------------------------------------------------------
   Function: 	LibD_IsLocalDev
   Description: Determine if the destination is located on the local device 
   Returns:	    1=local, 0=remote, -1=error
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_IsLocalDev(unsigned char destId)
{
	if((destId >= LIBD_MAXDEST) || (DestTable[destId].destType == LIBD_DEST_NONE)) return(-1);

	if((DestTable[destId].destType == LIBD_DEST_UENG) ||
	   (DestTable[destId].destType == LIBD_DEST_APP)) return(1);

	return(0);
}



/*-----------------------------------------------------------------------------
   Function: 	LibD_RouteMsg                
   Description: Dispatch SHRIMP message to the appropriate destination.  The
				destination could be an application port, micro-engine thread,
				or an external device through via either PCI or Serial channel.
   Returns:		LIBD_ERROR, LIBD_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_RouteMsg(unsigned int *msg)
 {
   unsigned char destId, srcId, timeout=5;
   int devNum, msgLen;

   //printf("LibD_RouteMsg: msg[LIBD_MSGPREAMBLE]=0x%x\n",msg[LIBD_MSGPREAMBLE]);  fflush(stdout); //dj:

   destId = GET_DEST_ID(msg[LIBD_MSGPREAMBLE]);
   srcId = GET_SOURCE_ID(msg[LIBD_MSGPREAMBLE]);
   if((destId >= LIBD_MAXDEST) || (DestTable[destId].devNum == -1) || (destId == srcId)){
      //printf("Routing to invalid destid=%d, devNum=%d\n", destId, DestTable[destId].devNum);  fflush(stdout); //dj:
      return(LIBD_ERROR);
   }

   devNum = DestTable[destId].devNum;
   switch(DestTable[destId].destType){
#ifdef LIBD_SERIAL
	  case LIBD_DEST_SERIAL:
		 if((devNum >= LIBD_MAX_SERIAL_DEVICES) || !SerialDevices[devNum].active) return(LIBD_ERROR);
         if((msgLen = LibD_GetMsgBytesLen(msg)) < 0) return(LIBD_ERROR);
		 if(SComm_Send((int)SerialDevices[devNum].handle, LIBD_CHANNEL, msg, msgLen,
			      timeout) != msgLen) return(LIBD_ERROR);
	     break;
#endif

#ifdef LIBD_PCI
	  case LIBD_DEST_PCI:
		  {
			 unsigned int ackTimeout = SEM_WAITFOREVER;
			 ShrimpMessage pciMsg;

			 if((devNum >= LIBD_MAX_PCI_DEVICES) || !PciDevices[0].active) return(LIBD_ERROR);
			 if((msgLen = LibD_GetMsgBytesLen(msg)) < 0) return(LIBD_ERROR);
			 //printf("Routing via PCI\n");  fflush(stdout); //dj: debug

			 if(GET_FORMAT(msg[LIBD_MSGPREAMBLE]) == FRMT_ARG_DESC){
				 // convert SHRIMP msg descriptor to a PCI msg descriptor
				 pciMsg.word_0 = msg[LIBD_MSGPREAMBLE];			// shrimp header
				 pciMsg.word_1 = msg[LIBD_MSGARGS];				// msgLen
				 pciMsg.word_2 = (unsigned int)msg[LIBD_MSGARGS+1];			// msgAddr
				 pciMsg.word_3 = (int)PciDevices[0].handle;	// my pci device num

				//printf("dj: LibD_RouteMsg: hdr=0x%x, len=%d, PciAddr=0x%x, devNum=%d\n", pciMsg.word_0, pciMsg.word_1, pciMsg.word_2, pciMsg.word_3);
				//printf("dj: addr=0x%x, data='%.*s'\n", (char*)pciMsg.word_2, pciMsg.word_1 , (char*)pciMsg.word_2);

				// only one message desc can be sent at time
				Sem_Wait(PciSem, SEM_WAITFOREVER);
				AckMsg[3] = devNum;

#if(OS == VXWORKS)
				// flush the buffer
				cacheFlush(DATA_CACHE, pciMsg.word_2, pciMsg.word_1 - 1);
#endif
				// send shrimp message to destination device
				Pci_PutMsg(devNum, (ShrimpMessage *)&pciMsg, PCI_CALL_UNBLOCK);

				// wait for the remote LibD_GetPciMsg to complete Pci_ReadMemBlock
				Sem_Wait(PciAckSem, ackTimeout);		// released by an ACK in LibD_GetPciMsg
			    Sem_Post(PciSem);
			 }
			 else{
			    // send message to destination device
			    Pci_PutMsg((int)devNum, (ShrimpMessage *)msg, PCI_CALL_UNBLOCK);
			 }
		  }
	     break;
#endif

	  case LIBD_DEST_APP:
	     //printf("Routing to App->PutMsg(), dest=0x%x, msgHdr=0x%x, arg1=0x%x, arg2=0x%x, arg3=0x%x\n", destId, msg[0], msg[1], msg[2], msg[3]);  fflush(stdout);
         if((msgLen = LibD_GetMsgBytesLen(msg)) < 0) return(LIBD_ERROR);
	     if(DestTable[destId].coreApp) DestTable[destId].coreApp->PutMsg(msg);
	     break;

	  case LIBD_DEST_UENG:
	     //printf("Routing to FDriver_PutMsgQue\n");  fflush(stdout);
         if((msgLen = LibD_GetMsgBytesLen(msg)) < 0) return(LIBD_ERROR);

		 //dj: ucommDriver will only copy the 1st 4 long-words
         FDriver_PutMsgQue((int *)msg, (int)DestTable[destId].uthreadNum);
	     break;

	  default:
	     //printf("Routing to Invalid dest=%d, type=%d\n", destId, DestTable[destId].destType);  fflush(stdout); //dj:
         return(LIBD_ERROR);
	     break;
   }
   return(LIBD_SUCCESS);
}


#ifdef LIBD_PCI
/*-----------------------------------------------------------------------------
   Function: 	LibD_GetLocalPciDevNum
   Description: Get the local PCI device number.
   Returns:		The device number
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int LibD_GetLocalPciDevNum()
{
	return(Pci_DeviceID);
}



/*-----------------------------------------------------------------------------
   Function: 	LibD_GetPciMsg
   Description: Receive SHRIMP message from PCI port and placed it on the appropriate
				queue.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void LibD_GetPciMsg(void *devPtr)
{
   ShrimpMessage shrimpMsg;
   char buf[LIBD_MAX_VARARG_BYTES];
   LibdDevice_T *dev = (LibdDevice_T *)devPtr;

   // PCI messages descriptor: SHRIMP_HEADER, msgLen, PCIaddr, PCIdevNum.
   // This is a fix until the PCI message unit can handle variable length SHRIMP message.

   while(1){
	  // read the 4-word shrimp message
      if(Pci_GetMsg(&shrimpMsg, PCI_CALL_BLOCK) != PCI_SUCCESS) continue;

	  //printf("dj: LibD_RouteMsg: hdr=0x%x, len=%d, PciAddr=0x%x, devNum=%d\n", shrimpMsg.word_0, shrimpMsg.word_1, shrimpMsg.word_2, shrimpMsg.word_3); fflush(stdout);

	  // determine the message type
	  if(GET_FORMAT(shrimpMsg.word_0) == FRMT_ARG_DESC){
		    unsigned int deviceNum, msgLen, pciAddr;

			// Check if it's an ack for the pci_ReadMemBlock completion.
			// Kludge: if the length is invalid, and the device number matches
			// the expected ack device, then consider it an ack.
		    if((shrimpMsg.word_3 == AckMsg[LIBD_MSGARGS+2]) &&
					(shrimpMsg.word_1 == AckMsg[LIBD_MSGARGS])){
				 // let router know that we receive an ack
				//printf("LibD_GetPciMsg: got ack, len=%d, dev=%d...informing router\n", shrimpMsg.word_1, shrimpMsg.word_3); //dj:
				Sem_Post(PciAckSem);
				continue;
			}

			// it's a PCI message descriptor (shrimp_hdr, len, pciAddr, pciDev)
			if((msgLen =  shrimpMsg.word_1) > LIBD_MAX_VARARG_BYTES){
				//printf("LibD_GetPciMsg: message too long %d > %d bytes\n",msgLen, LIBD_MAX_VARARG_BYTES); //dj:
				continue;
			}
			pciAddr = shrimpMsg.word_2;
			deviceNum = shrimpMsg.word_3;

			// message descriptor...read the message into SHRIMP message descriptor
	        //printf("dj: Reading %d bytes from PCI addr=0x%x to bufAddr=0x%x\n", msgLen, (char *)pciAddr, (char *)buf); fflush(stdout);
            Pci_ReadMemBlock(deviceNum, (char *)pciAddr, buf, msgLen);
			//printf("dj: stringData = '%.*s'\n", msgLen, buf); fflush(stdout);

			// transform pciDesc to shrimpDesc
			shrimpMsg.word_2 = (unsigned int)buf;

			// inform peer that we have copied the message from PCI memory
			AckMsg[3] = PciDevices[0].handle;
			Pci_PutMsg(deviceNum, (ShrimpMessage *)&AckMsg, PCI_CALL_BLOCK);
	  }

	  //printf("dj: LibD_RouteMsg: hdr=0x%x, len=%d, bufAddr=0x%x, devNum=%d\n", shrimpMsg.word_0, shrimpMsg.word_1, shrimpMsg.word_2, shrimpMsg.word_3); fflush(stdout);
      LibD_RouteMsg((unsigned int *)&shrimpMsg);

   }
   return;
}

#endif

#ifdef LIBD_SERIAL
/*-----------------------------------------------------------------------------
   Function: 	LibD_GetSerialMsg
   Description: Receive SHRIMP message from Serial port and placed it on the appropriate
				queue.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void LibD_GetSerialMsg(void *devPtr)
{
   unsigned int msg[LIBD_MAXMSGWRDLEN];
   int sConnId, format, timeout = 1;
   LibdDevice_T *dev = (LibdDevice_T *)devPtr;
   sConnId = (int)dev->handle;

   while(1){
	  // read shrimp header
      if(SComm_Recv(sConnId, &msg[LIBD_MSGPREAMBLE], 4, SCOMM_PEND) != 4) continue;

	  // determine the number of remaining byes to read
	  switch((format = GET_FORMAT(msg[LIBD_MSGPREAMBLE]))){
	     case FRMT_ARG_NONE:					// header only
			break;
		 case FRMT_ARG_1WRD:					// header + 1 word
		    if(SComm_Recv(sConnId, &msg[LIBD_MSGARGS], 4, timeout) != 4) continue;
		    break;
		 case FRMT_ARG_2WRD:					// header + 2 words message
		    if(SComm_Recv(sConnId, &msg[LIBD_MSGARGS], 8, timeout) != 8) continue;
		    break;
		 case FRMT_ARG_DESC:					// header + 2 words (len, addr) + len bytes
			shrimpArgDesc_T *argDesc = (shrimpArgDesc_T *)&msg[LIBD_MSGARGS];

		    if(SComm_Recv(sConnId, &msg[LIBD_MSGARGS], sizeof(shrimpArgDesc_T), timeout) != sizeof(shrimpArgDesc_T)) continue;
			if(argDesc->len && (argDesc->len <= LIBD_MAX_VARARG_BYTES)){
			   argDesc->addr = (char *)((unsigned int)argDesc + sizeof(shrimpArgDesc_T));
		       if(SComm_Recv(sConnId, argDesc->addr, argDesc->len, timeout) != (int)argDesc->len) continue;
			}
			else argDesc->addr = NULL;
			break;
	  }

      LibD_RouteMsg(msg);
   }
   return;
}
#endif

