#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "uclo.h"
#include "libd.h"
#include "thread.h"


/* Destination IDs */
#define TEST_APP	30      /* App ID */
#define UENG_DEST	2       /* uEng ID */

/* uEng function IDs */
#define BTESTSET_ID 0
#define BZERO_ID	1
#define BCOPY_ID	2
#define GATHER_ID	3
#define SCATTER_ID	4


/* test function IDs that are can be call by the uEng */
#define TEST_FUNC0	0
#define TEST_FUNC1	1
#define TEST_FUNC2	2
#define TEST_FUNC3	3

#define MAX_FUNC_HANDLES 256
#define MAX_ARG_LWORDS	 3


typedef	struct calloutArg_S{
	LibD_Callout_T  *fHandle;
	int				inArg[MAX_ARG_LWORDS];
	int				outArg[MAX_ARG_LWORDS];
}calloutArg_T;


static void *ObjHandle = NULL, *LibdHandle = NULL;
LibD_Callout_T *FuncHandles[MAX_FUNC_HANDLES];
#ifdef __cplusplus
{
#endif


#ifdef STANDALONE
//int LibD_PutMsgQue(void *msg);
int LibD_RouteMsg(void *msg);
void LibD_PrintMsg(unsigned int *msg);

void FDriver_PutMsg(unsigned int *msg, int threadNum)
{
   printf("(DEBUG)FDriver_PutMsg: entering...\n"); fflush(stdout);
   LibD_PrintMsg(msg);

   /* modify the args -- the ACK should be changed but...it will work */
   msg[1] = 0x11111111;
   msg[2] = 0x22222222;
   msg[3] = 0x33333333;

   return;
}

void FDriver_PutMsgQue(unsigned int *msg, int threadNum)
{
   int srcId;

   printf("(DEBUG)FDriver_PutMsgQue: entering...\n");  fflush(stdout);
   LibD_PrintMsg(msg);

   srcId = GET_SOURCE_ID(msg[0]);
   SET_SOURCE_ID(msg[0], GET_DEST_ID(msg[0]));
   SET_DEST_ID(msg[0], srcId);
   SET_ACK_REQ(msg[0], ACK);

   /* modify the args -- the ACK should be changed but...it will work */
   msg[1] = 0x11111111;
   msg[2] = 0x22222222;
   msg[3] = 0x33333333;

   //LibD_PutMsgQue(msg);
   LibD_RouteMsg(msg);
   return;
}

#define XACT_SUCCESS 0

int XactIoRead(int storage,		     /* IN: either SRAM, SDRAM or USTORE */ 
               char MEngineMask,     /* IN: set if storage = USTORE, 0 otherwise */
               int from,		     /* IN: from memory location within SRAM or SDRAM */
               unsigned int *element)/* OUT: to CORE memory */
{
   return XACT_SUCCESS;
}                		



int XactIoWrite(int  storage,	        /* IN: SRAM, SDRAM or USTORE */
                char MEngineMask,       /* IN: set if storage = USTORE, 0 otherwise */
                unsigned int element,	/* IN: data value */
                int  to)                /* IN: to buffer within SRAM or SDRAM or USTORE */
{
   return XACT_SUCCESS;
}


int XactIoInit()
{
   return XACT_SUCCESS;
}

#else
int testLib_consoleCmd();
int XACT_register_console_function(char *function_name, void *function_ptr,								   int num_char_ptr_args, int num_uint_args );
#endif			/* STANDALONE */


int testLib_getFHandle()
{
	int i;

	for(i = 0; i < MAX_FUNC_HANDLES; i++)
		if(!FuncHandles[i]) return(i);

	return(-1);
}


void testLib_freeFHandle(int funcHandle)
{
	if((funcHandle < MAX_FUNC_HANDLES) || (funcHandle < 0))
		printf("testLib_freeFuncHandle: Invalid function handle %d\n", funcHandle);
	
	FuncHandles[funcHandle] = NULL;

	return;
}


/* 
   This function accepts 1 too three as input and will
   return an output with the same length as the input
   modified to a fixed pattern.
*/
int doSomeWork(int argsLen, void *inArg, void *outArg)
{
   int *arg = (int *)inArg, i;
   int outPat[3] = {0x11111111, 0x22222222, 0x33333333};

   /* do something with the input and modify output args */
   printf("input args... ");
   for(i=0; i < argsLen; i++) printf("0x%0x ", arg[i]);

   /* modify and print the out argument */
   arg = (int *)outArg;
   printf("\noutput args... ");
   for(i=0; i < argsLen; i++){
      arg[i] = outPat[i];
      printf("0x%0x ", arg[i]);
   }

   return(0);
}


/* 
   Function to be call from uEng with ID 1.  It accepts
   no inputs and returns none.
*/
int func0(void *inArg, void *outArg)
{
   printf("func0: entering...\n");
   printf("No input and no output arguments...\n");
   return(0);
}


/* 
   Function to be call from uEng with ID 1.  It accepts
   a long-word as input and returns a long-word.
*/
int func1(void *inArg, void *outArg)
{
   printf("func1: entering...\n");
   return(doSomeWork(1, inArg, outArg));
}


/* 
   Function to be call from uEng with ID 2.  It accepts
   two long-words as input and returns two long-words.
*/
int func2(void *inArg, void *outArg)
{
   printf("func2: entering...\n");
   return(doSomeWork(2, inArg, outArg));
}



/* 
   Function to be call from uEng with ID 1.  It accepts
   three long-word as input and returns three long-word.
*/
int func3(void *inArg, void *outArg)
{
   printf("func3: entering...\n");
   return(doSomeWork(3, inArg, outArg));
}


int simUengCall()
{
   unsigned int msg[4], i;

   SET_SYNC(msg[0], SYNC);				/* PEND */
   SET_ACK_REQ(msg[0], REQ);			/* REQUEST */
   SET_FORMAT(msg[0], 3);				/* FORMAT -- 3 long-words */
   SET_SOURCE_ID(msg[0], UENG_DEST);	/* SOURCE -- ueng */
   SET_DEST_ID(msg[0], TEST_APP);		/* DESTINATION -- this app */
   SET_FUNC_ID(msg[0], TEST_FUNC3);		/* FUNCTION -- 3 */
   SET_ERROR(msg[0], MSG_OK);			/* ERROR -- 0 */
   SET_TRANS_ID(msg[0], 0);				/* TRANSACTION -- 0 */
   msg[1] = 0x10000000;                 /* 1st word of argument */
   msg[2] = 0x02000000;                 /* 2nd word of argument */
   msg[3] = 0x00300000;                 /* 3rd word of argument */

   printf("simUengCall: sending msg to core... ");
   for(i = 0; i < 4; i++) printf("0x%x ", msg[i]);
   printf("\n");

   if(LibD_Invoke(msg)) printf("simUengCall: LibD_Invoke failed\n");

   printf("simUengCall: msg from core... ");
   for(i = 0; i < 4; i++) printf("0x%x ", msg[i]);
   printf("\n");
   return(0);
}



int testLib_writeImages()
{
   /* write all images to uStore */
   return(UcLo_WriteUimageAll(ObjHandle));
}


int testLib_bindSym(char *imageName, char *symName, int value)
{
	if(!ObjHandle){
		printf("testLib_bindSym: No image is loaded\n");
		return(1);
	}

	return(UcLo_BindSymbol(ObjHandle, imageName, symName, value));
}


int testLib_regAlienFunc(char *funcLabel, int funcId, int funcType, int inArgBytes, int outArgBytes)
{
	int fHandle;
	LibD_FuncType fType = TWOWAY;

	printf("(debug: dj) Registering func %s\n", funcLabel);

	if((fHandle = testLib_getFHandle()) < 0){
		printf("testLib_regAlienFunc: Error out of function handles\n");
	    return(fHandle);
	}

	/*
	  register the callout functions -- if funcId is neg or greater than max func id,
      then get the function label from the object.
	*/
	if(fType != funcType) fType = ONEWAY;
	FuncHandles[fHandle] = LibD_RegAlienCallout(LibdHandle, funcId, fType, funcLabel,
                             inArgBytes, outArgBytes);
	return(fHandle);
}


int testLib_regLocalFunc(char *funcLabel)
{
	int fHandle;

	printf("(debug: dj) Registering func %s\n", funcLabel);

	if((fHandle = testLib_getFHandle()) < 0){
		printf("testLib_regLocalFunc: Error out of function handles\n");
	    return(fHandle);
	}

	/*
	  register local callout function -- get the function label from the object.
	*/
	FuncHandles[fHandle] = LibD_RegLocalCallout(LibdHandle, funcLabel);
	return(fHandle);
}


void testLib_callout(void *arg)
{
   calloutArg_T *coArg = (calloutArg_T *)arg;
   int status, i;

   /* call ueng function */
	if(!(status = LibD_CallOut(coArg->fHandle, (void *)&coArg->inArg,
		                       (void *)&coArg->outArg))){

		/* if the magic pattern not set, then assume something was returned */
		printf("testLib_callout: outArgs = "); fflush(stdout);
		for(i = 0; i < 3; i++){
			if(coArg->outArg[i] != 0xfbaddadf){			
				/* print the output arguments */
				printf("0x%x ", coArg->outArg[i]); fflush(stdout);
			}
			else break;
		}
		printf("\n"); fflush(stdout);
	}
	if(arg) free(arg);
	return;
}


int testLib_execUengFunc(char *sync, int destId, int funcHandle,
						 int arg1, int arg2, int arg3)
{
	/* int inArg[3], outArg[3]; */
	int status = 0;
	LibD_Callout_T   *fHandle;
	thread_T thread;
	calloutArg_T *calloutArg;

	printf("(debug: dj) Executing uEng function; funcHandle=%d, destId=%d\n",
		funcHandle, destId);  fflush(stdout);

	if((funcHandle < 0) || (funcHandle >= MAX_FUNC_HANDLES) ||
		(!(fHandle = FuncHandles[funcHandle]))){
		printf("testLib_execUengFunc: Invalid function handle %d\n",
		       funcHandle); fflush(stdout);
		return(-1);
	}

	if(!strcmp(sync, "SYNC")) fHandle->sync = SYNC;
	else fHandle->sync = ASYNC;

/*	fHandle->destId = atoi(destId); */
	fHandle->destId = destId;
	
	/* pack the input arguments and spawn thread to execute request */
	calloutArg = (calloutArg_T *)malloc(sizeof(calloutArg_T));
	calloutArg->fHandle = fHandle;
    calloutArg->inArg[0] = arg1;
	calloutArg->inArg[1] = arg2;
	calloutArg->inArg[2] = arg3;

    calloutArg->outArg[0] = calloutArg->outArg[1] = calloutArg->outArg[2] = 0xfbaddadf;

	/* create thread to process the request.  The thread should free calloutArg */
	thread = Thread_Create(MID_PRIORITY, (threadFunc_T)testLib_callout,
		                   (void *)calloutArg);

	return(status);
}

int testLib_wait(int funcHandle)
{
	int outArg[3];
	int status = 0;
	LibD_Callout_T   *fHandle;

	printf("(debug: dj) testLib_wait: funcHandle=%d\n", funcHandle);  fflush(stdout);

	if((funcHandle < 0) || (funcHandle >= MAX_FUNC_HANDLES) ||
		(!(fHandle = FuncHandles[funcHandle]))){
		printf("testLib_wait: Invalid function handle %d\n",
		       funcHandle); fflush(stdout);
		return(-1);
	}

    status = LibD_Wait(fHandle, (void **)&outArg);
	printf("testLib_wait: outArg = 0x%x 0x%x 0x%x \n", outArg[2],  outArg[1],  outArg[0]);

	return(status);
}


int testLib_init(char *filename, int appId)
{
	int i;
	unsigned int begUthread=0, numOfUthread=24;
	
	if(ObjHandle)
		/* trash the image */
		UcLo_DeleObj(ObjHandle);

	UcLo_InitLib();
	for(i = 0; i < MAX_FUNC_HANDLES; i++) FuncHandles[i] = NULL;

	/* load the object */
	if(UcLo_LoadObjFile(&ObjHandle, filename)){
		printf("ERROR: UcLo_LoadObjFile failed\n");
		return(0);
	}

   /* initialize the dispatcher library -- set the source ID */
   LibdHandle = LibD_Init(ObjHandle, appId);

   /* define the uEngs threadId's */
   LibD_AssignUThreadId(appId, begUthread, numOfUthread);


   /* register callback functions -- uEng accessable functions */
   LibD_RegisterCallback(LibdHandle, TEST_FUNC0	, func0, 0, 0);         /* no args */
   LibD_RegisterCallback(LibdHandle, TEST_FUNC1	, func1, 4, 4);         /* 1 long-word arg */
   LibD_RegisterCallback(LibdHandle, TEST_FUNC2	, func2, 8, 8);         /* 2 long-word arg */
   LibD_RegisterCallback(LibdHandle, TEST_FUNC3	, func3, 12, 12);       /* 3 long-word arg */
 
  return(0);    
}


int testLib_closeObj()
{
	/*trash the image */
	return(UcLo_DeleObj(ObjHandle));
}

int BTestSet(int mask, int addr)
{
	static int fHandle = -1;

	/* register function is not already registered */
	if(fHandle == -1)
		fHandle = testLib_regAlienFunc("BTestSet", BTESTSET_ID, TWOWAY, 8, 4);

	return(testLib_execUengFunc("SYNC", 0, fHandle, mask, addr, 0));
}

int BZero(int len, int data)
{
	static int fHandle = -1;

	/* register function is not already registered */
	if(fHandle == -1)
		fHandle = testLib_regAlienFunc("BZero", BZERO_ID, TWOWAY, 8, 0);

	return(testLib_execUengFunc("SYNC", 0, fHandle, len, data, 0));
}


int BCopy(int len, int from, int to)
{
	static int fHandle = -1;

	/* register function is not already registered */
	if(fHandle == -1)
		fHandle = testLib_regAlienFunc("BCopy", BCOPY_ID, TWOWAY, 12, 0);

	return(testLib_execUengFunc("SYNC", 0, fHandle, len, from, to));
}


int Gather(int count, int from, int to)
{
	static int fHandle = -1;

	/* register function is not already registered */
	if(fHandle == -1)
		fHandle = testLib_regAlienFunc("Gather", GATHER_ID, TWOWAY, 12, 0);

	return(testLib_execUengFunc("SYNC", 0, fHandle, count, from, to));
}


int Scatter(int count, int from, int to)
{
	static int fHandle = -1;

	/* register function is not already registered */
	if(fHandle == -1)
		fHandle = testLib_regAlienFunc("Scatter", SCATTER_ID, TWOWAY, 12, 0);

	return(testLib_execUengFunc("SYNC", 0, fHandle, count, from, to));
}


#ifdef STANDALONE
/* this routine will be called to initialize the foreign model after */
/* the transactor "init" command has successfully executed.  Returning 0 */
/* will result in a transactor error. */
int foreign_model_initialize()
{
   return(1);
}

/* this routine will be called prior to each transactor simulation event */
/* It can be used to deposit state values into the transactor model prior */
/* to simulating the next event.  Returning 0 results in an error. */
extern int foreign_model_pre_sim()
{
	return(1);
}


/* this routine will be called subsequent to each transactor simulation event. */
/* It can be used to query transactor simulation state, in order to copy it */
/* into the foreign model simulator. */
extern int foreign_model_post_sim()
{
	return(1);
}

/* this routine will be called just prior to exiting the simulator. */
/* The routine allows the foreign model to clean up, close files, etc */
/* before shutting down the program. */
extern int foreign_model_exit()
{
    return(1);
}


int main(int argc, char **argv)
{
   int		bzeroHandle, bcopyHandle;
   int		inArg[3] = {999, 888, 777}, outArg[3] = {0, 0,0};
   char		*uof_filename = "libdTest.uof", *imageName = "libdTest";
   char		*rbuf = "rbuf", *sendBuf = "sendBuf";
   char		*MemSv_BZero = "MemSv_BZero#", *MemSv_BCopy = "MemSv_BCopy#";

   if(argc > 1) uof_filename = argv[1];

	/* load the object from file and initialize the dispatcher
	   library -- set the source ID
	*/
	if((testLib_init(uof_filename, TEST_APP)))	return(-1);

	/* bind and resolve import variables */
	testLib_bindSym(imageName, rbuf, 3333);
	testLib_bindSym(imageName, sendBuf, 5555);


	/* 
      register the local callout functions -- functions that are in the object (uof).
	*/
	if((bzeroHandle = testLib_regLocalFunc(MemSv_BZero)) == -1){
		printf("testLib_regLocalFunc: Error registering func '%s'\n", MemSv_BZero);
	}

	if((bcopyHandle = testLib_regLocalFunc(MemSv_BCopy)) == -1){
		printf("testLib_regLocalFunc: Error registering func '%s'\n",MemSv_BCopy);
	}


	/* write all images to uStore */
	testLib_writeImages();

	/* execute function "test" in uEng 6 context 4 */
	printf("\nCalling uEng func with input arg = 0x%x\n", inArg);
	testLib_execUengFunc("SYNC", 23, bzeroHandle, inArg[0], inArg[1], inArg[2]);

	/* execute function "test2" in uEng 5 */
	printf("\nCalling uEng func with input arg = 0x%x\n", inArg);
	testLib_execUengFunc("ASYNC", 5, bcopyHandle, inArg[0], inArg[1], inArg[2]);
    testLib_wait(bcopyHandle);

	/* simulate the calling up from uEng for func0 */
	printf("\nSimulation call from uEng\n");
	if(simUengCall()) printf("simUengCall failed\n");

	/* trash the image */
	testLib_closeObj();

   return(0);
}

#else
int testLib_consoleCmd()
{
	if (!XACT_register_console_function("UcLoInit", testLib_init, 1, 1) ){
        fprintf(stderr,"XACT_register_console_function() UclLoInit failed\n");
		return(0);
	}

	if (!XACT_register_console_function("BindSym", testLib_bindSym, 2, 1) ){
        fprintf(stderr,"XACT_register_console_function() BindSym failed\n");
		return(0);
	}

	if (!XACT_register_console_function("RegAlienFunc", testLib_regAlienFunc, 1, 4) ){
        fprintf(stderr,"XACT_register_console_function() RegAlienFunc failed\n");
		return(0);
	}

	if (!XACT_register_console_function("RegLocalFunc", testLib_regLocalFunc, 1, 0) ){
        fprintf(stderr,"XACT_register_console_function() RegLocalFunc failed\n");
		return(0);
	}

	if (!XACT_register_console_function("WriteImages", testLib_writeImages, 0, 0) ){
        fprintf(stderr,"XACT_register_console_function() WriteImages failed\n");
		return(0);
	}

	if (!XACT_register_console_function("ExecUengFunc", testLib_execUengFunc, 1, 5) ){
        fprintf(stderr,"XACT_register_console_function() ExecUengFunc failed\n");
		return(0);
	}

	if (!XACT_register_console_function("CloseObj", testLib_closeObj, 0, 0) ){
		fprintf(stderr,"XACT_register_console_function() CloseObj failed\n");
		return(0);
	}


	if (!XACT_register_console_function("BTestSet", BTestSet, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() BTestSet failed\n");
		return(0);
	}

	if (!XACT_register_console_function("BZero", BZero, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() BZero failed\n");
		return(0);
	}

	if (!XACT_register_console_function("BCopy", BCopy, 0, 3) ){
        fprintf(stderr,"XACT_register_console_function() BCopy failed\n");
		return(0);
	}

	if (!XACT_register_console_function("Gather", Gather, 0, 3) ){
        fprintf(stderr,"XACT_register_console_function() Gather failed\n");
		return(0);
	}

	if (!XACT_register_console_function("Scatter", Scatter, 0, 3) ){
        fprintf(stderr,"XACT_register_console_function() Scatter failed\n");
		return(0);
	}

	return(1);
}
#ifdef __cplusplus
}
#endif

#endif			/* STANDALONE */


