/* MsgQ.cpp: implementation of the MsgQ class.
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Message Queue
 * author: Lai-Wah Hui 2/12/98
 * revisions:
 * $History: MsgQ.cpp $
 * 
 * *****************  Version 13  *****************
 * User: Ewalker      Date: 8/11/00    Time: 10:10
 * Updated in $/dev_1200/SA1_CoreLibs/msgq
 * Removed obsolete VXWORKS-specific initialization code  which was
 * already excluded anyway.
 * 
 * *****************  Version 8  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:42p
 * Updated in $/development/msgq
 * update COPYRIGHT
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 5/20/98    Time: 6:27p
 * Updated in $/base_level2/msgq
 * Replace Digital copyright with Intel copyright
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 4/29/98    Time: 2:46p
 * Updated in $/base_level2/msgq
 * Add MSGQ_EXIST error code
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 4/08/98    Time: 2:28p
 * Updated in $/base_level2/msgq
 * Remove MsgQ.h from MsgQMgr.h
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 3/26/98    Time: 7:08p
 * Updated in $/base_level2/msgq
 * Add AllocMsgBuffer and FreeMsgBuffer functions
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 2/24/98    Time: 12:53p
 * Created in $/base_level2/msgq
 * init. check-in
 * 
 *
 *---------------------------------------------------------------------
 */
#include "MsgQ.h"
#include "MsgQMgr.h"



/*-------------------------------------------------------------------------
    MsgQ::MsgQ
    Construction
    return: none
   	uses:       
	modifies:   
  ------------------------------------------------------------------------*/


MsgQ::MsgQ(unsigned int MsgQueue, unsigned int elements, size_t Msgsize)
{
    CS = new Semaphore(1);
    hFull = new Semaphore(1);
    hEmpty = new Semaphore;
    hMemPool = new PoolMgr(Msgsize, elements);   
    QSize = elements;
    Reference = 0;
    Buf = new int [elements+1];
    _TailPh = &Buf[0];
    _HeadPh = &Buf[QSize];
    Valid = MSGQ_TRUE;
	MsgQID = MsgQueue;
    Set_FullFlag(MSGQ_FALSE);
    Set_EmptyFlag(MSGQ_TRUE);
}

    
/*-------------------------------------------------------------------------
    MsgQ::MsgQ
    deconstruction
    return: none
   	uses:       
	modifies:   

  ------------------------------------------------------------------------*/



MsgQ::~MsgQ()
{
    delete CS;
    delete hMemPool;
    delete hFull; 
    delete hEmpty;
	delete Buf;
}


MsgQ::Status MsgQ::Put(void *Msg, int wait)	
{
    CS->Take();
	if(Get_FullFlag() == MSGQ_TRUE){
		CS->Give();
		if(wait == NOWAIT)
			return MSGQ_FALSE;
        // Wait for a message
        hFull->Take();  
		hFull->Give();  // reset -- need re-work
        CS->Take();
    }

  
    *_TailPh = (int)Msg;

		// set full signal
    if(IsFull() == MSGQ_TRUE){
        hFull->Take();
        Set_FullFlag(MSGQ_TRUE);
    }
	_TailPh++;
	if (_TailPh==&Buf[QSize])
			_TailPh =&Buf[0]; //loop back

	// reset empty signal
    if(Get_EmptyFlag() == MSGQ_TRUE){
        hEmpty->Give();
        Set_EmptyFlag(MSGQ_FALSE);
    }


    CS->Give();
    return MSGQ_TRUE;
}


MsgQ::Status MsgQ::Get(void **Msg, int wait)	
{


    CS->Take();

	if(Get_EmptyFlag() == MSGQ_TRUE){
		CS->Give();
		if(wait == NOWAIT)
			return MSGQ_FALSE;
        hEmpty->Take();
		hEmpty->Give(); // need re-work --reseting semphore
		CS->Take();
    }

	if (_HeadPh==&Buf[QSize])
	    _HeadPh =&Buf[0]; //loop back

    *Msg = (void *)*_HeadPh++;

          // reset full flag
    if(Get_FullFlag() == MSGQ_TRUE){
        hFull->Give();
        Set_FullFlag(MSGQ_FALSE);
    }

    if(IsEmpty() == MSGQ_TRUE){
        Set_EmptyFlag(MSGQ_TRUE);
        hEmpty->Take();
    }

    CS->Give();
    return MSGQ_TRUE;
}
