/* MsgQ.h: 
 *      interface for the MsgQ class.
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Message Queue
 * author: Lai-Wah Hui 2/12/98
 * revisions:
 * $History: MsgQ.h $
 * 
 * *****************  Version 9  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:42p
 * Updated in $/development/msgq
 * update COPYRIGHT
 * 
 * *****************  Version 7  *****************
 * User: Drjohns1     Date: 6/19/98    Time: 1:37p
 * Updated in $/base_level3/msgq
 * Changed "ts.h" to "TS.h"
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 5/20/98    Time: 6:27p
 * Updated in $/base_level2/msgq
 * Replace Digital copyright with Intel copyright
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 4/29/98    Time: 2:46p
 * Updated in $/base_level2/msgq
 * Add MSGQ_EXIST error code
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 4/08/98    Time: 2:28p
 * Updated in $/base_level2/msgq
 * Remove MsgQ.h from MsgQMgr.h
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 3/26/98    Time: 7:08p
 * Updated in $/base_level2/msgq
 * Add AllocMsgBuffer and FreeMsgBuffer functions
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 2/24/98    Time: 12:54p
 * Created in $/base_level2/msgq
 * init. check-in
 * 
 * 
 *---------------------------------------------------------------------
 */ 

#if !defined(AFX_MSGQ_H__C581CC09_A327_11D1_86AA_0000F80373DD__INCLUDED_)
#define AFX_MSGQ_H__C581CC09_A327_11D1_86AA_0000F80373DD__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000
#include <iostream.h>
#include <TS.h>
#include "poolmgr.h"

class MsgQ  {
public:
    enum Status {MSGQ_TRUE,MSGQ_FALSE};
    enum LockStatus {UNLOCK, LOCK};
	MsgQ(unsigned int MsgQueue, unsigned int elements, size_t bufsize);
    ~MsgQ();
    Status Put(void *Msg, int wait);
    Status Get(void **Msg, int wait);
    Status IsEmpty() const {
	       return (_HeadPh == _TailPh || ((_HeadPh == &Buf[QSize])&&_TailPh == &Buf[0])) ? MSGQ_TRUE : MSGQ_FALSE;};
    Status IsFull() const { 
          return ((_TailPh+1 == _HeadPh) || ( _TailPh+1 == &Buf[QSize] && _HeadPh == &Buf[0])) ? MSGQ_TRUE : MSGQ_FALSE;}

    Status IsValid() const {return Valid;};
    Status IsReferenced() const {return (Reference>0) ? MSGQ_TRUE:MSGQ_FALSE;};
    Status Get_FullFlag() const {return FullFlag;};
    Status Get_EmptyFlag() const {return EmptyFlag;};
    void Set_FullFlag(const Status flag)  {FullFlag = flag;};
    void Set_EmptyFlag(const Status flag) {EmptyFlag = flag;};

    unsigned int GetMsgQID() const {return MsgQID;};
    PoolMgr *hMemPool;
    void *AllocMsgBuf(size_t n) {
                void *buf;
                CS->Take();
                buf = hMemPool->alloc(n);
                CS->Give();
                return buf;
    }           
    void FreeMsgBuf(void *buf,size_t n) {
                CS->Take();
                hMemPool->free(buf, n);
                CS->Give();
    }           

    void QReset() {
                CS->Take();
                _TailPh= &Buf[0];
                _HeadPh = &Buf[QSize];
			    Set_FullFlag(MSGQ_FALSE);
				Set_EmptyFlag(MSGQ_TRUE);
			    delete hFull; 
                delete hEmpty;
				hFull = new Semaphore(1);
				hEmpty = new Semaphore;
                CS->Give();
    }
    void RefInc() {
                CS->Take();
                Reference++;  
                CS->Give();
    };
    void RefDec() {
                CS->Take();
                Reference--;  
                CS->Give();
    };

private:
    Semaphore* CS;
    Semaphore*  hFull;
    Semaphore* hEmpty;
    int QSize;
    int Reference;
    unsigned int MsgQID;
    Status FullFlag;
    Status EmptyFlag;
    Status Valid;
    int *Buf;
    int *_TailPh; 
    int *_HeadPh; 
};
 

#endif // !defined(AFX_MSGQ_H__C581CC09_A327_11D1_86AA_0000F80373DD__INCLUDED_)
