/* MsgQMgr.cpp: implementation of the message queue application interface
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Message Queue
 * author: Lai-Wah Hui 2/12/98
 * revisions:
 * $History: MsgQMgr.cpp $
 * 
 * *****************  Version 11  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:42p
 * Updated in $/development/msgq
 * update COPYRIGHT
 * 
 * *****************  Version 10  *****************
 * User: Hqianx       Date: 3/15/99    Time: 5:14p
 * Updated in $/development/msgq
 * Added #include <string.h> for memset() to make ARM compiler happy.
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 5/20/98    Time: 6:27p
 * Updated in $/base_level2/msgq
 * Replace Digital copyright with Intel copyright
 * 
 * *****************  Version 6  *****************
 * User: Qabuild      Date: 5/05/98    Time: 11:43a
 * Updated in $/base_level2/msgq
 * Change MSGQ_INIT == NULL to MSGQ_INIT == 0 to prevent VxWork compile
 * error
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 4/29/98    Time: 2:46p
 * Updated in $/base_level2/msgq
 * Add MSGQ_EXIST error code
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 4/17/98    Time: 5:38p
 * Updated in $/base_level2/msgq
 * Change input type for MsgQ_Create() & MsgQ_Open()
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 4/08/98    Time: 2:28p
 * Updated in $/base_level2/msgq
 * Remove MsgQ.h from MsgQMgr.h
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 3/26/98    Time: 7:08p
 * Updated in $/base_level2/msgq
 * Add AllocMsgBuffer and FreeMsgBuffer functions
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 2/24/98    Time: 12:54p
 * Created in $/base_level2/msgq
 * init. check-in
 * 
 *
 *---------------------------------------------------------------------
 */
#include <string.h>
#include "MsgQ.h"
#include "MsgQMgr.h"

/*-------------------------------------------------------------------------
    MsgQ_Create
    Create a circular FIFO queue referenced by the argument msgq_id, sized 
    by argument elements. It also pre-allocats message buffer pool.
    return: MSGQ_SUCCESS or MSGQ_ERROR
   	uses:       
	modifies:   
  ------------------------------------------------------------------------*/
MsgQ *MsgQMgr[MaxQueues];

static int MSGQ_INIT = 0;

int MsgQ_Create(MsgQID Qid,   /* IN: Message Queue ID */
                size_t elements,    /* IN: Size of message queue */
                size_t bufsize)     /* IN: Size of message buffer */  
{

    if(MSGQ_INIT == 0){
        memset(MsgQMgr, NULL, MaxQueues);
        MSGQ_INIT = 1;
    }
    if(MsgQMgr[Qid] == NULL){
	    MsgQMgr[Qid] = new MsgQ(Qid, elements, bufsize);
        return (MsgQMgr[Qid]->IsValid() == MSGQ_SUCCESS) ? MSGQ_SUCCESS : MSGQ_ERROR;
    }
    else   
        return (MsgQMgr[Qid]->IsValid() == MSGQ_SUCCESS) ? MSGQ_EXIST : MSGQ_ERROR;
   
}

/*-------------------------------------------------------------------------
    MsgQ_Open
    Opens the circular FIFO message queue created by the MsgQ_Create() function. 
    After a successful MsgQ_Open() call, the task may send or receive a message 
    using the MsgQ_Handle returned from MsgQ_Open().  
    return: MsgQ_Handle or INVALID_MSGQ_HANDLE 
    uses:       
	modifies:   
  ------------------------------------------------------------------------*/
MsgQ_Handle MsgQ_Open(MsgQID Qid)   /* IN: Message queue ID */
{


	if((MsgQMgr[Qid] == NULL) || (MsgQMgr[Qid]->IsValid()) != MsgQMgr[Qid]->MSGQ_TRUE)
		return (MsgQ_Handle)INVALID_MSGQ_HANDLE;
    
    MsgQMgr[Qid]->RefInc();
    return MsgQMgr[Qid];

}

/*-------------------------------------------------------------------------
    MsgQ_Send
    Sends a message to the message queue referenced by MsgQ_Handle.  If 
    the message queue is full, the input parameter wait have the values NOWAIT 
    (0) or WAIT(1). NOWAIT means the routine should return immediately; 
    WAIT means the routine should wait for ever.
    return: MSGQ_SUCCESS, MSGQ_ERROR, MSGQ_FULL
   	uses:       
	modifies:   
  ------------------------------------------------------------------------*/

int MsgQ_Send(MsgQ_Handle phMsgQ,   /* IN: Message queue handle */ 
              void *msg,            /* IN: Message */
              int wait)             /* IN: NOWAIT or WAIT*/
{
    MsgQ *hMsgQ = (MsgQ *)phMsgQ;

    if(hMsgQ->IsValid() == hMsgQ->MSGQ_FALSE)
        return MSGQ_ERROR;
    return (hMsgQ->Put(msg, wait) == hMsgQ->MSGQ_FALSE) ? MSGQ_FULL : MSGQ_SUCCESS;


}

/*-------------------------------------------------------------------------
    MsgQ_Receive
    Receives a message from the message queue referenced by MsgQ_Handle. 
    If the message queue is empty, the input parameter wait have the values NOWAIT 
    (0) or WAIT(1). NOWAIT means the routine should return immediately; 
    WAIT means the routine should wait for ever for a message.

    return: MSGQ_SUCCESS, MSGQ_ERROR, MSGQ_EMPTY
    uses:       
    modifies:   
  ------------------------------------------------------------------------*/
int MsgQ_Receive(MsgQ_Handle phMsgQ,           /* IN: Message queue handle */  
                 void **msg,                  /* IN: Poiner to a message buffer*/
                 int wait)                    /* IN: NOWAIT or WAIT */
{
    MsgQ *hMsgQ = (MsgQ *)phMsgQ;

    if(hMsgQ->IsValid() ==  hMsgQ->MSGQ_FALSE)
        return MSGQ_ERROR;

    return (hMsgQ->Get(&*msg, wait) == hMsgQ->MSGQ_FALSE) ? MSGQ_EMPTY : MSGQ_SUCCESS;

}


/*-------------------------------------------------------------------------
    MsgQ_AllocMsgBuf
    Message buffer allocation
    return: pointer to a message buffer
    uses:       
    modifies:   
  ------------------------------------------------------------------------*/
void *MsgQ_AllocMsgBuf(MsgQ_Handle phMsgQ,       /* IN: Message queue handle */
                       size_t BufSize)          /* IN: Buffer size */
{
    MsgQ *hMsgQ = (MsgQ *)phMsgQ;

    return hMsgQ->AllocMsgBuf(BufSize);

}

/*-------------------------------------------------------------------------
    MsgQ_FreeMsgBuf
    Message buffer deallocation
    return: none
    uses:       
    modifies:   
  ------------------------------------------------------------------------*/
void MsgQ_FreeMsgBuf(MsgQ_Handle phMsgQ,         /* IN: Message queue handle */ 
                     void *buf,                 /* IN: Message buffer  */
                     size_t BufSize)            /* IN: Message buffer size */
{
    MsgQ *hMsgQ = (MsgQ *)phMsgQ;

    hMsgQ->FreeMsgBuf(buf, BufSize);

}


/*-------------------------------------------------------------------------
    MsgQ_Close
    Deallocates the message handle indicated by hMsgQ.  Upon close of the last 
    message queue reference, the state of the message queue will be reset.
    return: MSGQ_SUCCESS or MSGQ_ERROR
   	uses:       
	modifies:   
  ------------------------------------------------------------------------*/
int MsgQ_Close(MsgQ_Handle phMsgQ)               /* IN: Message queue handle */ 
{
    MsgQ *hMsgQ = (MsgQ *)phMsgQ;


    if(!hMsgQ->IsValid())
        return MSGQ_ERROR;

    hMsgQ->RefDec();
    if(hMsgQ->IsReferenced())
        return MSGQ_SUCCESS;
    
    hMsgQ->QReset();
    return MSGQ_SUCCESS;

}

