/* MsgQTest.cpp: Message Queue Test Application 
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Message Queue
 * author: Lai-Wah Hui 2/12/98

 * revisions:
 * $History: MsgQTest.cpp $
 * 
 * *****************  Version 10  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:42p
 * Updated in $/development/msgq
 * update COPYRIGHT
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 5/20/98    Time: 6:27p
 * Updated in $/base_level2/msgq
 * Replace Digital copyright with Intel copyright
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 4/30/98    Time: 9:38a
 * Updated in $/base_level2/msgq
 * Change INVALID_MSGQ_HANDLE value from -1 to NULL to prevent a compile
 * warning message.
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 4/29/98    Time: 2:46p
 * Updated in $/base_level2/msgq
 * Add MSGQ_EXIST error code
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 4/17/98    Time: 5:38p
 * Updated in $/base_level2/msgq
 * Change input type for MsgQ_Create() & MsgQ_Open()
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 4/08/98    Time: 2:28p
 * Updated in $/base_level2/msgq
 * Remove MsgQ.h from MsgQMgr.h
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 3/26/98    Time: 7:08p
 * Updated in $/base_level2/msgq
 * Add AllocMsgBuffer and FreeMsgBuffer functions
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 2/24/98    Time: 12:54p
 * Created in $/base_level2/msgq
 * init. check-in
 * 
 * 
 *
 *---------------------------------------------------------------------
 */
#include <stdio.h>
#include <iostream.h>
#include <assert.h>
#include "MsgQMgr.h"
#include "ts.h"
#include "MsgQTest.h"

#ifdef WIN32
int main(int argc, char **argv)
#else
int msgq_main()
#endif
{
    int status;
    status = MsgQ_Create(DownQueue, MaxData, BufSize);
    if (status!= MSGQ_SUCCESS)
        cout << "MsgQ_Create(DownQueue) Fail" << status << endl;


    status = MsgQ_Create(UpQueue, MaxData, BufSize);
    if (status!= MSGQ_SUCCESS)
        cout << "MsgQ_Create(UpQueue) Fail" << status << endl;

    status = MsgQ_Create(ToHostQueue, MaxData, BufSize);
    if (status!= MSGQ_SUCCESS)
        cout << "MsgQ_Create(ToHostQueue) Fail" << status << endl;

    
    status = MsgQ_Create(FromHostQueue, MaxData,BufSize);
    if (status!= MSGQ_SUCCESS)
        cout << "MsgQ_Create(FromHostQueue) Fail" << status << endl;
   
    //
    status = MsgQ_Create(DownQueue, MaxData, BufSize);
    if (status!= MSGQ_EXIST)
        cout << "MsgQ_Create(DownQueue) Fail" << status << endl;


    status = MsgQ_Create(UpQueue, MaxData, BufSize);
    if (status!= MSGQ_EXIST)
        cout << "MsgQ_Create(UpQueue) Fail" << status << endl;

    status = MsgQ_Create(ToHostQueue, MaxData, BufSize);
    if (status!= MSGQ_EXIST)
        cout << "MsgQ_Create(ToHostQueue) Fail" << status << endl;
    status = MsgQ_Create(FromHostQueue, MaxData,BufSize);
    if (status!= MSGQ_EXIST)
        cout << "MsgQ_Create(FromHostQueue) Fail" << status << endl;

    
    cout << "Start Test 1 " << endl;
    MsgQ_Test_1();
    cout << "Start Test 2 " << endl;
    MsgQ_Test_2();
    cout << "Start Test 3 " << endl;
    MsgQ_Test_3();
   cout << "Start Test 4 " << endl;
    MsgQ_Test_4();


    return 1;

}

int MsgQ_Test_1()
{
    int status;
    int *testdata;
    int *receiveData;
    int index;
    MsgQ_Handle hDownQueue;
    MsgQ_Handle hUpQueue;
    MsgQ_Handle hToHostQueue;
    MsgQ_Handle hFromHostQueue;


    hDownQueue = MsgQ_Open(DownQueue);
    if(hDownQueue == INVALID_MSGQ_HANDLE){
       cout << "Can't open DownQueue" << endl;
       return (1);
    }

    hUpQueue = MsgQ_Open(UpQueue);
    if(hUpQueue == INVALID_MSGQ_HANDLE){
       cout << "Can't open hUpQueue" << endl;
       return (1);
    }
    hToHostQueue = MsgQ_Open(ToHostQueue);
    if(hToHostQueue == INVALID_MSGQ_HANDLE){
       cout << "Can't open hToHostQueue" << endl;
       return (1);
    }
    hFromHostQueue = MsgQ_Open(FromHostQueue);
    if(hFromHostQueue  == INVALID_MSGQ_HANDLE){
       cout << "Can't open hFromHostQueue " << endl;
       return (1);
    }

    for (index = 0; index <= MaxTest; index++){
        testdata = (int *)MsgQ_AllocMsgBuf(hDownQueue,BufSize);
        *testdata = index + (int)hDownQueue; 
        cout << hex << "DownQueue: testdata=" << *testdata << " ;" << endl;
        status = MsgQ_Send(hDownQueue, (void *)testdata, NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
              cout << "Test Success: return status = (MsgQ_Send(hDownQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Send(hDownQueue))" << status << endl;
            MsgQ_FreeMsgBuf(hDownQueue, testdata, BufSize);
        }
    }
	for (index = 0; index <= MaxTest; index++){
        testdata = (int *)MsgQ_AllocMsgBuf(hUpQueue, BufSize);
        *testdata = index + (int)hUpQueue; 
        cout << hex << "UpQueue: testdata=" << *testdata << " ;" << endl;
        status = MsgQ_Send(hUpQueue, (void *)testdata, NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
              cout << "Test Success: return status = (MsgQ_Send(hUpQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Send(hUpQueue))" << status << endl;
            MsgQ_FreeMsgBuf(hUpQueue,testdata,BufSize);
        }
  

    }
    for (index = 0; index <= MaxTest; index++){
        testdata = (int *)MsgQ_AllocMsgBuf(hToHostQueue, BufSize);
        *testdata = index + (int)hToHostQueue; 
        cout << hex << "ToHostQueue: testdata=" << *testdata << " ;" << endl;
        status = MsgQ_Send(hToHostQueue, (void *)testdata, NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
              cout << "Test Success: return status = (MsgQ_Send(hToHostQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Send(hToHostQueue))" << status << endl;
            MsgQ_FreeMsgBuf(hToHostQueue,testdata,BufSize);
        }

     
    }
    for (index = 0; index <= MaxTest; index++){
        testdata = (int *)MsgQ_AllocMsgBuf(hFromHostQueue, BufSize);
        *testdata = index + (int)hFromHostQueue; 
        cout << hex << "FromHostQueue: testdata=" << *testdata << " ;" << endl;
        status = MsgQ_Send(hFromHostQueue, (void *)testdata, NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
              cout << "Test Success: return status = (MsgQ_Send(hFromHostQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Send(hFromHostQueue))" << status << endl;
            MsgQ_FreeMsgBuf(hFromHostQueue, testdata,BufSize);
        }

    }
	cout << "Receiving data" << endl;

    for (index = 0; index <= MaxTest; index++){
    	status = MsgQ_Receive(hDownQueue, (void **)(&receiveData), NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
              cout << "Test Success: return status = (MsgQ_Receive(hdownQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Receive(hdownQueue))" << status << endl;
        }          
        else{
 //           assert(*receiveData == (index + (int)hDownQueue));
            cout << "Success: hDownQueue: receivedata =" << *receiveData << endl;
            assert(*receiveData == (index + (int)hDownQueue));
            MsgQ_FreeMsgBuf(hDownQueue,&*receiveData,BufSize);

        }
    }
    for (index = 0; index <= MaxTest; index++){
    	status = MsgQ_Receive(hUpQueue, (void **)(&receiveData), NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
               cout << "Test Success: return status = (MsgQ_Receive(hUpQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Receive(hUpQueue))" << status << endl;
        }
        else{
            assert(*receiveData == (index + (int)hUpQueue));
            cout << "Success: hUpQueue: receivedata =" << *receiveData << endl;
            MsgQ_FreeMsgBuf(hUpQueue,&*receiveData,BufSize);

        }
    }

    for (index = 0; index <= MaxTest; index++){
    	status = MsgQ_Receive(hToHostQueue, (void **)(&receiveData), NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
               cout << "Test Success: return status = (MsgQ_Receive(hToHostQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Receive(hToHostQueue))" << status << endl;
        }
        else{
            assert(*receiveData == (index + (int)hToHostQueue));
            cout << "SUCCESS: hToHostQueue: receivedata =" << *receiveData << endl;
            MsgQ_FreeMsgBuf(hToHostQueue,&*receiveData,BufSize);

        }
    }

    for (index = 0; index <= MaxTest; index++){
    	status = MsgQ_Receive(hFromHostQueue, (void **)(&receiveData), NOWAIT);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
               cout << "Test Success: return status = (MsgQ_Receive(hFromHostQueue))" << status << endl;
            else
                cout << "Test Fail: return status = (MsgQ_Receive(hFromHostQueue))" << status << endl;
        }

        else{
            assert(*receiveData == (index + (int)hFromHostQueue));
            cout << "Success: hFromHostQueue: receivedata =" << *receiveData << endl;
            MsgQ_FreeMsgBuf(hFromHostQueue,&*receiveData,BufSize);

        }
     }

    assert(MsgQ_Close(hDownQueue));
    assert(MsgQ_Close(hUpQueue));
    assert(MsgQ_Close(hToHostQueue));
    assert(MsgQ_Close(hFromHostQueue));

    return 1;

}

#define wait WAIT
#define QID DownQueue


static void MsgQ_Test_Send(void)
{

    int status;
    int *testdata;
    int index;
    MsgQ_Handle hQueue;

    hQueue = MsgQ_Open(QID);
 
    for (index = 0; index <= MaxTest; index++){
        testdata = (int *)MsgQ_AllocMsgBuf(hQueue, BufSize);
        *testdata = index + (int)hQueue; 

        cout <<  "Send: index= " << index 
			 << "testdata="  << *testdata
			 << endl;
        status = MsgQ_Send(hQueue, (void *)testdata, wait);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
              cout << "Test Success:QID= 1"  << "return status = (MsgQ_Send)" << status << endl;
            else
                cout << "Test Fail: QID="  << "return status = (MsgQ_Send)" << status << endl;
            MsgQ_FreeMsgBuf(hQueue,testdata,BufSize);
        }
#ifdef WIN32
        Sleep(5);
#else
	taskDelay(1);	
#endif

    }

    assert(MsgQ_Close(hQueue));

 
}


static void MsgQ_Test_Receive(void)
{

    int status;
    int index;
    int *receiveData;
    MsgQ_Handle hQueue;

    hQueue = MsgQ_Open(QID);
 
    
    for (index = 0; index <= MaxTest; index++){
		status = MsgQ_Receive(hQueue, (void **)(&receiveData), wait);
        if(status != MSGQ_SUCCESS){
            if(index == MaxTest)
              cout << "Test Success: QID=" << "return status = MsgQ_Receive()" << status << endl;
            else
                cout << "Test Fail:QID="  << "return status = MsgQ_Receive()" << status << endl;
        }          
        else{
            assert(*receiveData == (index + (int)hQueue));
			cout << "Success: Rec index =" << index << "receivedata =" << *receiveData << endl;
            MsgQ_FreeMsgBuf(hQueue,&*receiveData,BufSize);
        }
#ifdef WIN32

        Sleep(5);
#else
	taskDelay(1);	
#endif
    }

    assert(MsgQ_Close(hQueue));
  
}

int MsgQ_Test_2()
{
    Thread* sendTh;
    Thread* RecTh;
    sendTh = new  Thread(MsgQ_Test_Send);
    RecTh = new  Thread(MsgQ_Test_Receive);
#ifdef WIN32
    WaitForSingleObject(sendTh->GetThreadHandle(), INFINITE); 
    WaitForSingleObject(RecTh->GetThreadHandle(), INFINITE);
#else
    taskDelay(100);	

#endif
    return 1;
}


static void Asyn_MsgQ_Test_Send(void)
{

    int status;
    int *testdata;
    int index;
    MsgQ_Handle hQueue;

    hQueue = MsgQ_Open(QID);
 
    for (index = 0; index <= MaxTest; index++){
        testdata = (int *)MsgQ_AllocMsgBuf(hQueue,BufSize);
        *testdata = index + (int)hQueue; 

        cout <<  "Send: index= " << index 
			 << "testdata="  << *testdata
			 << endl;
        for(;;){
			if((status = MsgQ_Send(hQueue, (void *)testdata, NOWAIT)) == MSGQ_SUCCESS)
				break;
			if(status == MSGQ_FULL){
				cout << "send queue full" << endl;
#ifdef WIN32
				Sleep(10);
#else
	taskDelay(1);	
#endif
			}
			else{
				cout << "MsgQ_Send - error status " << status << endl;
		        MsgQ_FreeMsgBuf(hQueue, testdata,BufSize);
  				break;
			}
		}
#ifdef WIN32
        Sleep(5);
#else
	taskDelay(1);	
#endif

    }

    assert(MsgQ_Close(hQueue));

 
}


static void Asyn_MsgQ_Test_Receive(void)
{

    int status;
    int index;
    int *receiveData;
    MsgQ_Handle hQueue;

    hQueue = MsgQ_Open(QID);
 
    
    for (index = 0; index <= MaxTest; index++){
		for(;;){
			status = MsgQ_Receive(hQueue, (void **)(&receiveData), NOWAIT);
			if(status == MSGQ_SUCCESS)
				break;
			if(status == MSGQ_EMPTY){
				cout << "Receive queue empty" << endl;
#ifdef WIN32
				Sleep(10);
#else
	taskDelay(1);	
#endif

			}
		}
    assert(*receiveData == (index + (int)hQueue));
	cout << "Success: Rec index =" << index << "receivedata =" << *receiveData << endl;
    MsgQ_FreeMsgBuf(hQueue,&*receiveData,BufSize);

#ifdef WIN32
    Sleep(5);
#else
	taskDelay(1);	
#endif

    }

    assert(MsgQ_Close(hQueue));
  
}
int MsgQ_Test_3()
{
    Thread* sendTh;
    Thread* RecTh;
    sendTh = new  Thread(Asyn_MsgQ_Test_Send);
    RecTh = new  Thread(Asyn_MsgQ_Test_Receive);
#ifdef WIN32
    WaitForSingleObject(sendTh->GetThreadHandle(), INFINITE); 
    WaitForSingleObject(RecTh->GetThreadHandle(), INFINITE);
    cout << "goto sleep" << endl;
#else
    taskDelay(1000);
   
#endif
    return 1;
}


int MsgQ_Test_4()
{
    int status;
    int *testdata;
    int *receiveData;
    int index;
    int loop;
    MsgQ_Handle hDownQueue;


    hDownQueue = MsgQ_Open(DownQueue);
    for(loop =0; loop < 5; loop++){
        for (index = 0; index <= MaxTest; index++){
            testdata = (int *)MsgQ_AllocMsgBuf(hDownQueue,BufSize);
            *testdata = index + (int)hDownQueue; 
            cout << hex << "DownQueue: testdata=" << *testdata << " ;" << endl;
            status = MsgQ_Send(hDownQueue, (void *)testdata, NOWAIT);
            if(status != MSGQ_SUCCESS){
                if(index == MaxTest)
                  cout << "Test Success: return status = (MsgQ_Send(hDownQueue))" << status << endl;
                else
                    cout << "Test Fail: return status = (MsgQ_Send(hDownQueue))" << status << endl;
                MsgQ_FreeMsgBuf(hDownQueue, testdata,BufSize);
            }
        }
    	cout << "Receiving data" << endl;

        for (index = 0; index <= MaxTest; index++){
    	    status = MsgQ_Receive(hDownQueue, (void **)(&receiveData), NOWAIT);
            if(status != MSGQ_SUCCESS){
                if(index == MaxTest)
                  cout << "Test Success: return status = (MsgQ_Receive(hdownQueue))" << status << endl;
                else
                    cout << "Test Fail: return status = (MsgQ_Receive(hdownQueue))" << status << endl;
            }          
            else{
                assert(*receiveData == (index + (int)hDownQueue));
                cout << "Success: hDownQueue: receivedata =" << *receiveData << endl;
                MsgQ_FreeMsgBuf(hDownQueue,&*receiveData,BufSize);

            }
        }
    }
    assert(MsgQ_Close(hDownQueue));

    return 1;


}
