/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* fdrive.h
 * f bus mac receive side
 *
 *
 * system: SA1200
 * subsystem: packet_gen
 * author: Don Hooper 3/10/99
 * revisions:
 * $History fdrive.h $
 * 
 * 
 */
#ifndef FDRIVE_H
#define FDRIVE_H

#include "fdevice.h"
//#include <math.h>


#define MAX_PACKETS_IN_POOL 512
#define POOL_SIZE 65536								// unsigned int count for sum of all packets in pool
#define BUF_SIZE_GIG_PORT	2048					// mac buffer size, bytes
#define BUF_SIZE_SLOW_PORT	256						// mac buffer size, bytes
#define INTER_PACKET_BYTES 20						// IPG + preamble + SFD + SPD

#define RCV_OK_STATUS 0x100

enum fdrive_returns{FDRIVE_XACT_FAIL = 0, FDRIVE_SUCCESS = 1, FDRIVE_POOL_FULL, 
						FDRIVE_NO_SUCH_PACKET, BAD_RATE };

#define DEFAULT_RDY_THRESHOLD 64

// a set of known packets here can be used to feed the model
class PacketPool{
public:
	PacketPool();
	int PkPoolInit();
	int Show(int n);								// show a specific packet in the pool
	int Add(void **buf, int len);
	unsigned int pksource[POOL_SIZE];				// buffer holding all packet data
	unsigned short pkendbuf;						// index into pksource: where next packet can be added
	unsigned short pkend;							// number of next packet to be added
	unsigned short pkcount;							// number of packets in pool
	unsigned int pkstarts[MAX_PACKETS_IN_POOL];		// start index into pksource: of each packet
	int pksizes[MAX_PACKETS_IN_POOL];				// byte count of each packet
private:
};


class RxPort{
public:
	void Init();
	void Enable();
	void Disable(unsigned int pkcount);
	void UpdateOutptr();
	int DriveFbeLo();
	int DriveFbeHi();
	int DriveFBus();
	void NoDriveFBus();
	int SetFirst(unsigned int n);
	int SetLast(unsigned int n);
	int SetCurrent(unsigned int n);
	int ShowCurrent();
	int rem_size;						// remaining byte count on current packet
	unsigned char enabled;
	unsigned char rdy;
	int my_id;

	unsigned int current_pksource;
	unsigned int first_pksource;
	unsigned int last_pksource;
	unsigned int current_packet;
	unsigned int first_packet;
	unsigned int last_packet;

	// wire level parameters and methods
	unsigned char fast_port;			// 1 if fast_port 1, 2 if fast_port 2. 0 if not
	int buf_size;
	int rdy_threshold;
	double wire_bytes_per_clock;
	double buf_bytes;					// bytes in mac buffer
	void BufBytesPut();					// simulate putting bytes from wire into MAC buffer
	void BufBytesTake(int n);			// sim taking bytes from MAC buffer for fbus drive cycle

private:
	enum rxport_state{ IN_PACKET = 0, EOP_SENT, STATUS1_SENT, STATUS2_SENT, IDLE };
	unsigned char prep_to_disable;
	unsigned int pk_rem_count;
	unsigned int state;

	// wire level parameters and methods
	double buf_pk_bytes;				// bytes from wire for current wire_packet
	double inter_packet_wait;
	unsigned int hardware_buf_size;
	unsigned int packets_in_buf;		// how many eops in mac buffer
	unsigned char overflow;
	unsigned char underflow;
	double current_wire_packet;			// packet number
};


class RxMAC :public FDevice{
public:
	void Init();						// set rdy_vector to 0, clear all pksource data
	int SetRate(int Mbits_per_sec, int MFClocks_per_sec);
	void ShowBufs();					// print buffer counts
	int Cycle();						// run MAC logic
	RxPort ports[PORTS_PER_MAC];
	unsigned char enabled_vector;		// 8 bit ports enabled vector (bit 0 = port 0, etc.)
	unsigned char rdy_vector;			// 8 bit ports ready vector (bit 0 = port 0, etc.)
	
private:
	void UpdatePortVectors();
	void DecodeMACSel();
	void DecodeMACRdySel();
	int rx_mac_sel;
	int rx_mac_rdy_sel;
};


class FDrive :public PacketPool{

public:
	XACT_HANDLE fdat_lo_bus_handle;
	XACT_HANDLE fdat_hi_bus_handle;
	XACT_HANDLE fbe_lo_l_bus_handle;
	XACT_HANDLE fbe_hi_l_bus_handle;
	XACT_HANDLE rdybus_bus_handle;
	XACT_HANDLE sop_bus_handle;
	XACT_HANDLE eop_bus_handle;
	XACT_HANDLE rxfail_bus_handle;
	XACT_HANDLE fast_rx1_handle;
	XACT_HANDLE fast_rx2_handle;

	int initialized;
	int rx_count;				// count of packets received
	int rx_stop_count;			// when to stop the simulator

	RxMAC macs[MAX_MACS];

	void Init();
	int Go(int pkcount);
	int	AddBusClients();
	int Cycle();

private:

};

	extern "C" int FDrive_Help();
	extern FDrive fd;

#endif		// FDRIVE_H