/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* fileio.cpp
 * fileio for use by transactor command line functions
 *
 *
 * system: SA1200
 * subsystem: packet_gen
 * author: Don Hooper 3/18/99
 * revisions:
 * $History fileio.cpp $
 * 
 * 
 */ 

#include <stdio.h>
#include "fileio.h"



/*-----------------------------------------------------------------
	FileIo:: FileIo
	description: construct FileIo
	uses:
	modifies:
  -----------------------------------------------------------------*/
FileIo::FileIo()
{
	int i;

	for(i = 0; i<MAX_FILEPTRS; i++){
		outfptr_valid[i] = 0;
	}
	next_fptr_index = 0;
}


/*-----------------------------------------------------------------
	FileIo:: FOpen
	description: Open a file, as in fopen().
	returns: index to file (like a file pointer, acceptable to transactor)
	uses:
	modifies:
  -----------------------------------------------------------------*/
int FileIo::FOpen(char *fname, char *mode)
{
	FILE *infptr;
	if (next_fptr_index == MAX_FILEPTRS) return(FILEIO_FILEPTR_LIMIT);
		
		int this_index = next_fptr_index;

    if ( (infptr = fopen(fname, mode ) ) == NULL ) {
        perror("Problem in creating output file");
        return(FILEIO_FOPEN_FAIL);
    }
	open_files[this_index] = infptr;
	outfptr_valid[this_index] = 1;
	next_fptr_index++;

	return(this_index); 
}


/*-----------------------------------------------------------------
	FileIo:: FClose
	description: Close a file, as in fclose().
		fptr is index to file (like a file pointer, but acceptable to transactor)
	returns: PKGEN_SUCCESS
	uses:
	modifies:
  -----------------------------------------------------------------*/
int FileIo::FClose(int fptr)
{
	FILE *infptr;

	if (outfptr_valid[fptr]){
		infptr = open_files[fptr];
		return(fclose(infptr));
	}
	return(FILEIO_SUCCESS);
}
