/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

// fmon.cpp
// fbus monitor

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "packet.h"
#include "fmon.h"
#include "fdrive.h"

// instantiate a FMon
FMon fm;



/*-----------------------------------------------------------------
	TxPort::Init
	description: Initialize port variables.
	returns:
	uses:
	modifies: enabled, ready, left
  -----------------------------------------------------------------*/
void TxPort::Init()
{
	int i;

	enabled = 0;
	rdy = 1;
	left = 1;
	state = IDLE;
	buf_inptr = 0;
	buf_outptr = 0;
	buf_bytes = 0;
	packets_in_buf = 0;
	buf_size = TX_BUF_SIZE_GIG_PORT;
	rdy_threshold = TX_DEFAULT_RDY_THRESHOLD;
	send_threshold = TX_DEFAULT_SEND_THRESHOLD;
	packet_started = 0;	

	eop = (unsigned char *)malloc(buf_size * sizeof(unsigned char));
	for(i=0;i<buf_size;i++){
		eop[i] = 0;
	}
	return;
}


/*-----------------------------------------------------------------
	TxPort::Enable
	returns: FDRIVE_SUCCESS or FDRIVE_FAIL
	enable sinking packets for this port
	uses:
	modifies: enabled
  -----------------------------------------------------------------*/
int TxPort::Enable()
{
	enabled = 1;
	return(FMON_SUCCESS);
}

extern "C" int tport_on(unsigned int mac, unsigned int port)
{
	fm.macs[mac].enabled_vector = fm.macs[mac].enabled_vector | (1<<port);
	return(fm.macs[mac].ports[port].Enable());
}


/*-----------------------------------------------------------------
	TxPort::Disable
	description: Disable sinking the packets for this port.
	returns: FDRIVE_SUCCESS or FDRIVE_FAIL
	uses:
	modifies: enabled
  -----------------------------------------------------------------*/
int TxPort::Disable()
{
	enabled = 0;
	return(FMON_SUCCESS);
}

extern "C" int tport_off(unsigned int mac, unsigned int port)
{
	fm.macs[mac].enabled_vector = fm.macs[mac].enabled_vector & ~(1<<port);
	return(fm.macs[mac].ports[port].Disable());
}


extern "C" int tport_fopen(char *fname, unsigned int mac, unsigned int port)
{
	return(fm.macs[mac].ports[port].outfptr = fm.macs[mac].ports[port].FOpen(fname, "w+"));
}

/*-----------------------------------------------------------------
	TxPort::BufBytesPut
	description:
		If transmitting on fbus, increment the buf_bytes by bytes sent.
		If (buf_size - buf_bytes) > rdy_threshold, set port ready, else clear port ready.
		If buf_bytes > buf_size	print overflow.
		
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
void TxPort::BufBytesPut(int n)
{
	unsigned int halt_val, int_outptr, int_inptr;
	int prev_buf_bytes = buf_bytes;

	buf_inptr += n;
	if (buf_inptr >= buf_size) buf_inptr -= buf_size;		// wrap

	int_outptr = (unsigned int) buf_outptr;
	int_inptr = (unsigned int) buf_inptr;


	if (int_outptr <= int_inptr){
		buf_bytes = int_inptr - int_outptr;
	} else if ((int_inptr +1) != int_outptr){	// deal with rounding error
	buf_bytes = int_inptr + buf_size - int_outptr;
	}

	if(((buf_size - buf_bytes) > rdy_threshold)&(packets_in_buf < 2)){
		rdy = 1;
		if (fm.pr_enabled) XACT_printf("READY port %d\n", my_id);
	} else {
		rdy = 0;
		if (fm.pr_enabled) XACT_printf("NOT READY port %d\n", my_id);
	}

	if(prev_buf_bytes > buf_bytes){
		XACT_printf("Transmit OverFlow, mac: %d \tport: %d\n", my_id >> 3, my_id &0x7);
		overflow = 1;
		halt_val = 1;
		XACT_set_state_value(fm.sim_halt_handle, &halt_val);
	} else overflow = 0;

	return;
}


/*-----------------------------------------------------------------
	TxPort::SendToWire
	description: look for eop
		If wire bytes per clock < 1, check eop and increment buffer output pointer.
		If "" > 1,  
	
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/

void TxPort::SendToWire()
{
	int have_eop = 0;
	double temp_outptr = buf_outptr;
	double finish_outptr = buf_outptr + wire_bytes_per_clock;
	double bytes_to_send = wire_bytes_per_clock;

	// e.g. if slow port has .18 wire_bytes_per_clock, this will test 1 eop slot.
	//		if fast port has 1.8 wire_bytes_per_clock, this will test 2 eop slots.
	while(finish_outptr > temp_outptr){
		if (eop[(unsigned int)temp_outptr]){
			// if we hit eop before finish, subtract remaining clock time from interpacket wait				
			inter_packet_wait = (INTER_PACKET_BYTES - bytes_to_send) / wire_bytes_per_clock;
			buf_outptr = temp_outptr;
			eop[(unsigned int)temp_outptr] = 0;
			have_eop = 1;
			packet_started = 0;
			if (packets_in_buf > 0) packets_in_buf--;
			//printf("||||Send to wire: port %d, have eop, outptr: %d, packets_in_buf: %d\n", my_id, have_eop, (unsigned int)temp_outptr, packets_in_buf); 
		}
		(bytes_to_send < 1) ? temp_outptr = finish_outptr : temp_outptr+=1;
		bytes_to_send = finish_outptr - temp_outptr;							
	}
	if(!have_eop) buf_outptr += wire_bytes_per_clock;

	if (temp_outptr >= buf_size){
		buf_outptr = temp_outptr - buf_size;			//wrap
		if (eop[(unsigned int)buf_outptr]){
			// if we hit eop before finish, subtract remaining clock time from interpacket wait				
			inter_packet_wait = (INTER_PACKET_BYTES - bytes_to_send) / wire_bytes_per_clock;
			eop[(unsigned int)buf_outptr] = 0;
			have_eop = 1;
			packet_started = 0;
			if (packets_in_buf > 0) packets_in_buf--;
		}
	}	
	return;
}


/*-----------------------------------------------------------------
	TxPort::BufBytesTake
	description:
		if interpacket_wait, decrement interpacket_wait value
		else if at or above xmit_send_threshold and packet_started==0,
			set packet_started = 1, take bytes_per_cycle bytes.
		else if packet_started ==1, take bytes_per_cycle bytes.
		if sending eop, set interpacket_wait
		if (buf_size - buf_bytes) > xmit_rdy_threshold, set port ready, else clear port ready.
		if buf_bytes < 0	print underflow.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
void TxPort::BufBytesTake()
{
	unsigned int halt_val, int_outptr, int_inptr;
	int prev_buf_bytes = buf_bytes;

	if(inter_packet_wait > 0){
		inter_packet_wait--;
	// start a new packet
	}else if(((buf_bytes > send_threshold)|(packets_in_buf>0)) && !packet_started && (inter_packet_wait <= 0)){
		packet_started = 1;	
		SendToWire();
	} else if(packet_started){
		SendToWire();
	}

	int_outptr = (unsigned int) buf_outptr;
	int_inptr = (unsigned int) buf_inptr;

	if (int_outptr <= int_inptr){
		buf_bytes = int_inptr - int_outptr;
	} else {
		buf_bytes = int_inptr + buf_size - int_outptr;
	}
	// deal with rounding error
	if (buf_bytes == buf_size-1) buf_bytes = 0;
	
	// if packet fits in buffer, buf_bytes = 0 = no packet
	if ((buf_size>1500)&(buf_bytes == 0)){
		packet_started = 0;
		packets_in_buf = 0;
	}
		
	if(((buf_size - buf_bytes) > rdy_threshold)&(packets_in_buf < 2)){
		rdy = 1;
		if (fm.pr_enabled) XACT_printf("READY port %d\n", my_id);
	} else {
		rdy = 0;
		if (fm.pr_enabled) XACT_printf("NOT READY port %d\n", my_id);
	}

	if(prev_buf_bytes < buf_bytes){
		XACT_printf("Transmit UnderFlow, mac: %d \tport: %d\n", my_id >> 3, my_id &0x7);
		underflow = 1;
		halt_val = 1;
		XACT_set_state_value(fm.sim_halt_handle, &halt_val);
	}

	return;
}

/*-----------------------------------------------------------------
	QWordByteCount
	description: convert 8 byte enables to byte count
	returns: byte count
	uses:
	modifies:
  -----------------------------------------------------------------*/
int QWordByteCount(unsigned int behi, unsigned int belo)
{
	int be = ((behi & 0xf)<<4) | (belo & 0xf);
	int res;

	if (fm.fbus_big_endian){
		switch(be){
		case 0xf7:	
			res = 1;
			break;
		case 0xf3:
			res = 2;
			break;
		case 0xf1:
			res = 3;
			break;
		case 0xf0:
			res = 4;
		case 0x70:
			res = 5;
		case 0x30:
			res = 6;
		case 0x10:
			res = 7;
		default:
			res = 8;
		}
	} else {
		switch(be){
		case 0xfe:	
			res = 1;
			break;
		case 0xfc:
			res = 2;
			break;
		case 0xf8:
			res = 3;
			break;
		case 0xf0:
			res = 4;
		case 0xe0:
			res = 5;
		case 0xc0:
			res = 6;
		case 0x80:
			res = 7;
		default:
			res = 8;
		}

	}
	return(res);
}


/*-----------------------------------------------------------------
	LWordByteCount
	description: convert 4 byte enables to byte count
	returns: byte count
	uses:
	modifies:
  -----------------------------------------------------------------*/
int LWordByteCount(unsigned int be)
{
	int res;

	if (fm.fbus_big_endian){
		switch(be){
		case 0x7:	
			res = 1;
			break;
		case 0x3:
			res = 2;
			break;
		case 0x1:
			res = 3;
			break;
		default:
			res = 4;
		}
	} else {
		switch(be){
		case 0xe:	
			res = 1;
			break;
		case 0xc:
			res = 2;
			break;
		case 0x8:
			res = 3;
			break;
		default:
			res = 4;
		}

	}
	return(res);
}



/*-----------------------------------------------------------------
	TxPort::SinkFBus
	description: Sink the packets for this port (write them to a file)
	returns:	FMON_SUCCESS if data written
				FMON_BAD_FILEPTR if output file not valid
				FMON_STATUS_CYCLE if status on fbus
				FMON_IDLE_CYCLE if not in a packet
	uses:
	modifies: state, left
  -----------------------------------------------------------------*/
int TxPort::SinkFBus()
{
	unsigned int halt_val;
	FILE *fptr;

	// will print data while state is SOP_RECEIVED
	if(((state == IDLE)||(state== EOP_RECEIVED))&&(fm.fbus_bidirectional ? fm.sop_val : fm.sop32_xmit_val)){
		state = SOP_RECEIVED;
	}

	// ignore status written to fbus
	if(state == EOP_RECEIVED){
		state = IDLE;
		return(FMON_STATUS_CYCLE);
	}
	
	// EOP
	if ((state == SOP_RECEIVED)&&(fm.fbus_bidirectional ? fm.eop_val : fm.eop32_xmit_val)){
		state = EOP_RECEIVED;
		if (fm.tx_stop_count > 0) fm.tx_count++;
	}


	if(state == IDLE){
		if (fm.pr_enabled) XACT_printf("Not in packet and not status1\n");
		return(FMON_IDLE_CYCLE);
	}

	// put the bytes in the MAC buffer and insert eop flag at buffer byte location
	if (wire_bytes_per_clock){ 
		// 21440 has 256B buffer and a limit of 2 packets in transmit buffer
// gig mac spec doc error discovered 5/15/00 
//		// IXF1002 has same limit. If 2 packets in buffer and selected, stop
		if (packets_in_buf > 1){
			XACT_printf("Transmit Overflow. Too many packets in transmit buffer, mac: %d \tport: %d\n", my_id >> 3, my_id &0x7);
			overflow = 1;
			halt_val = 1;
			XACT_set_state_value(fm.sim_halt_handle, &halt_val);
		}
		if(fm.fbus_bidirectional){
			if(fm.eop_val){
				packets_in_buf++;
				//printf("|| Send to IX: port %d, buf_inptr %d, Incrementing packets_in_buf %d\n", my_id, (unsigned int)buf_inptr, packets_in_buf);
				eop[(unsigned int)buf_inptr] = 1;
			}
			BufBytesPut(QWordByteCount(fm.belo_val, fm.behi_val));
		} else {
			if(fm.eop32_xmit_val){
				packets_in_buf++;
				eop[(unsigned int)buf_inptr] = 1;
			}
			BufBytesPut(LWordByteCount(fm.behi_val));
		}
	}

	if(!outfptr_valid[outfptr]) return(FMON_BAD_FILEPTR);	// not necessarily an error
	fptr = open_files[outfptr];

	// now print to output file if there is one
	switch(fm.mode){
	case BI2MAC:
	case BI4MAC:
		fprintf(fptr, "0x%.8x\t", fm.fdatal_val);			// bytes 4-7 print to right
		fprintf(fptr, "0x%.8x\n", fm.fdatah_val);			// bytes 0-3 print to left
		if (fm.eop_val){
				fprintf(fptr, "\n");
		}
		break;
	case UNI2MAC:
	case UNI4MAC:
		// print only the xmit data
		if (left == 1) {
			// print word on left
			if (fm.eop32_xmit_val){
				if (fm.tx_stop_count > 0) fm.tx_count++;
				fprintf(fptr, "0x%.8x\t0x%.8x\n\n", fm.fdatah_val, 0);
				left = 1;
			} else {
				fprintf(fptr, "0x%.8x\t", fm.fdatah_val);
				left = 0;
			}
			
		} else {
			// print word on right
			fprintf(fptr, "0x%.8x\n", fm.fdatah_val);
			left = 1;
			if (fm.eop32_xmit_val){
				if (fm.tx_stop_count > 0) fm.tx_count++;
				fprintf(fptr, "\n");
			}
		}
		break;
	default:
		break;
	}
	// while selected force ready off
	rdy = 0;

			
	return(FMON_SUCCESS);		
}

/*-----------------------------------------------------------------
	TxPort::ShowCurrent
	description: Print port status information
	returns: FDRIVE_SUCCESS
	uses:
	modifies:
  -----------------------------------------------------------------*/
void TxPort::ShowCurrent()
{

	XACT_printf("Transmit status for port \t%d\n\n", my_id);
	XACT_printf("\tBuffer bytes:   \t%d\n", buf_bytes);
	XACT_printf("\tBuffer size:    \t%d\n", buf_size);
	XACT_printf("\tReady:          \t%d\n", rdy);
	XACT_printf("\tPackets in buffer\t%d\n", packets_in_buf);

	XACT_printf("\n");

	return;
}
extern "C" int port_show(int mac_id, int port_id)
{
	XACT_printf("Transmit side:\n");
	fm.macs[mac_id].ports[port_id].ShowCurrent();
	
	XACT_printf("Receive side:\n");
	fd.macs[mac_id].ports[port_id].ShowCurrent();

	return(FMON_SUCCESS);
}

/*-----------------------------------------------------------------
	TxMAC::Init
	description: Set ready and enabled to 0. Initialize all ports.
	returns:
	uses:
	modifies: enabled_vector, rdy_vector
  -----------------------------------------------------------------*/
void TxMAC::Init()
{
	int j;

	enabled_vector = 0;
	rdy_vector = 0;

	for (j=0;j<PORTS_PER_MAC; j++){
		ports[j].Init();
		ports[j].my_id = (my_id<<3) + j;
	}

	return;
}

/*-----------------------------------------------------------------
	TxMAC::SetRate
	description: Set transmit rate for MAC
	returns: 
	uses:
	modifies:
	see also: xact.h
  -----------------------------------------------------------------*/
int TxMAC::SetRate(int Mbits_per_sec, int MFClocks_per_sec)
{
	int j;
	double temp = (double)Mbits_per_sec / (double)(MFClocks_per_sec*8);

	if(Mbits_per_sec >= 600){
		for (j=0;j<MAX_FAST_PORTS_PER_MAC; j++){
			ports[j].wire_bytes_per_clock = temp;
			ports[j].buf_size = TX_BUF_SIZE_GIG_PORT;
		}

	} else{
		for (j=0;j<PORTS_PER_MAC; j++){
			ports[j].wire_bytes_per_clock = temp;
			ports[j].buf_size = TX_BUF_SIZE_SLOW_PORT;
		}
	}

	return(FDRIVE_SUCCESS);
}

extern "C" int tmac_set_rate(int mac, int Mbits_per_sec, int MFClocks_per_sec)
{
	return(fm.macs[mac].SetRate(Mbits_per_sec, MFClocks_per_sec));
}

extern "C" int tmac_set_rdy_threshold(int mac, int threshold)
{
	int j;

	for (j=0;j<PORTS_PER_MAC; j++){
		fm.macs[mac].ports[j].rdy_threshold = threshold;
	}
	return(FDRIVE_SUCCESS);
}

extern "C" int tmac_set_send_threshold(int mac, int threshold)
{
	int j;

	for (j=0;j<PORTS_PER_MAC; j++){
		fm.macs[mac].ports[j].send_threshold = threshold;
	}
	return(FDRIVE_SUCCESS);
}
/*-----------------------------------------------------------------
	TxMAC::UpdatePortVectors
	description: Poll ports to determined if they are enabled and ready.
	returns:
	uses:
	modifies: enabled_vector, rdy_vector
  -----------------------------------------------------------------*/
void TxMAC::UpdatePortVectors()
{
	int i;

	enabled_vector = rdy_vector = 0;
	for(i=0;i<PORTS_PER_MAC;i++){
		enabled_vector = enabled_vector | (ports[i].enabled<<i); 
		rdy_vector = rdy_vector | (ports[i].rdy<<i); 
	}
	return;
}


/*-----------------------------------------------------------------
	TxMAC::DecodeMACSel
	description: Determine which Tx MAC has been selected.
	uses:
	modifies: tx_mac_sel
  -----------------------------------------------------------------*/
void TxMAC::DecodeMACSel()
{
	switch(fm.mode){
	case BI2MAC:
		switch(port_ctl_l_dly2){
		case 0xB: tx_mac_sel = 0; break;
		case 0x7: tx_mac_sel = 1; break;
		default: tx_mac_sel = 8;		// no select
		}
		break;
	case BI4MAC:
		switch(port_ctl_l_dly2){
		case 0x0: tx_mac_sel = 0; break;
		case 0x2: tx_mac_sel = 1; break;
		case 0x4: tx_mac_sel = 2; break;
		case 0x6: tx_mac_sel = 3; break;
		case 0x8: tx_mac_sel = 4; break;
		case 0xA: tx_mac_sel = 5; break;
		case 0xC: tx_mac_sel = 6; break;
		default: tx_mac_sel = 8;		// no select
		}
		break;
	case UNI2MAC:
		switch((port_ctl_l_dly2>>2)&0x3){
		case 0x2: tx_mac_sel = 0; break;
		case 0x1: tx_mac_sel = 1; break;
		default: tx_mac_sel = 8;		// no select
		}
		break;
	case UNI4MAC:
		if ((gpio_dly2 & 1) == 0){		// if gpio<0> = 1, no select
			tx_mac_sel = 8;				// no select
		} else {
			switch((port_ctl_l_dly2>>2) & 0x3){
			case 0x0: tx_mac_sel = 0; break;
			case 0x1: tx_mac_sel = 1; break;
			case 0x2: tx_mac_sel = 2; break;
			case 0x3: tx_mac_sel = 3; break;
			default: tx_mac_sel = 8;	// no select
			}
		}
	}

	return;
}


/*-----------------------------------------------------------------
	TxMAC::DecodeMACRdySel
	description: Determine which TxMAC has been seleced to drive rdybus.
	returns:
	uses:
	modifies: tx_mac_rdy_sel
  -----------------------------------------------------------------*/
void TxMAC::DecodeMACRdySel()
{
	switch(fm.mode){
	case BI2MAC:
	case UNI2MAC:
		switch(rdyctl_l_dly3){
		case 0xD: tx_mac_rdy_sel = 0; break;
		case 0x7: tx_mac_rdy_sel = 1; break;
		default: tx_mac_rdy_sel = 8;	// no select
		}
		break;
	case BI4MAC:
		switch(rdyctl_l_dly3 | (rdyctl_4_l_dly3<<4)){
		case 0x17: tx_mac_rdy_sel = 0; break;
		case 0x16: tx_mac_rdy_sel = 1; break;
		case 0x15: tx_mac_rdy_sel = 2; break;
		case 0x14: tx_mac_rdy_sel = 3; break;
		case 0x07: tx_mac_rdy_sel = 4; break;
		case 0x06: tx_mac_rdy_sel = 5; break;
		case 0x05: tx_mac_rdy_sel = 6; break;
		default: tx_mac_rdy_sel = 8;	// no select
		}
		break;
	case UNI4MAC: 	
		switch(rdyctl_l_dly3){
		case 0x7: tx_mac_rdy_sel = 0; break;
		case 0x6: tx_mac_rdy_sel = 1; break;
		case 0x5: tx_mac_rdy_sel = 2; break;
		case 0x4: tx_mac_rdy_sel = 3; break;
		default: tx_mac_rdy_sel = 8;	// no select
		}
		break;
	default:
		tx_mac_rdy_sel = 8;	// no select
	}

	return;

}

/*-----------------------------------------------------------------
	TxMAC::ShowBufs
	description: Print port buffer byte counts.
	returns: FDRIVE_SUCCESS
	uses:
	modifies:
  -----------------------------------------------------------------*/
void TxMAC::ShowBufs()
{
	int i;

	XACT_printf("TxMAC %d bytes: ", my_id);
	for (i=0;i<PORTS_PER_MAC;i++){
		if (ports[i].enabled){
			XACT_printf("p%d: %.4d  ", i, (int) ports[i].buf_bytes);
		}
	}
	XACT_printf("\n");

	return;
}
extern "C" int mac_show(int id)
{
	XACT_printf("Transmit side:\n");
	fm.macs[id].ShowBufs();
	
	XACT_printf("Receive side:\n");
	fd.macs[id].ShowBufs();

	return(FMON_SUCCESS);
}


/*-----------------------------------------------------------------
	TxMAC::Cycle
	description: Execute TxMAC logic. Called every fbus cycle
		if FMon has been initialized via FMon_Init
	returns: FMON_SUCCESS or XACT fail
	uses:
	modifies: FDevice pin delay variables, port_sel_dly3, ready_vector, enabled_vector
				tx_mac_sel. tx_mac_rdy_sel
	see also: xact.h
  -----------------------------------------------------------------*/
int TxMAC::Cycle()
{
	unsigned int val, i;
	int retval = FMON_SUCCESS;

	PinDelayChains();							// set FDevice pin delay variables

	if (fm.fbus_bidirectional == 1){
		port_sel_dly2 = fps_dly2 & 0x7;
	} else port_sel_dly2 = (gpio_dly2>>1) & 0x7;

	UpdatePortVectors();						// set ready_vector, enabled_vector



	// simulate taking bytes from MAC buffer and sending to wire
	for (i=0;i<PORTS_PER_MAC;i++){
		if((ports[i].enabled)&&(ports[i].wire_bytes_per_clock)) ports[i].BufBytesTake();
	}
	if ((fm.pr_enabled)&&(ports[0].wire_bytes_per_clock)){
		ShowBufs();								// print buffer counts
	}
	DecodeMACSel();								// set tx_mac_sel to selected MAC

	
	if (tx_mac_sel == my_id){					// if this mac selected
		if (fm.pr_enabled) XACT_printf("Tx-------USING IXBUS, my_id: %d, mac: %d, port: %d\n", my_id, tx_mac_sel, port_sel_dly2);
		ports[port_sel_dly2].SinkFBus();
	}


	DecodeMACRdySel();							// set tx_mac_rdy_sel to selected MAC
	// tell the selected port to drive the rdybus
	if (tx_mac_rdy_sel == my_id){
		val = rdy_vector & enabled_vector;
		if (fm.pr_enabled) XACT_printf("Tx DRIVING RDYBUS, mac: %d\n", my_id);
		retval &= XACT_set_state_value(fd.rdybus_bus_handle, &val);
	}

	return(retval);
}


/*-----------------------------------------------------------------
	FMon::FMon
	Construct and initialize FMon
	uses:
	modifies: initialized, cycle, enabled
  -----------------------------------------------------------------*/
FMon::FMon()
{
	initialized = 0;
	cycle = 0;
	pr_enabled = 0;
}


/*-----------------------------------------------------------------
	FMon::Cycle
	description: Get pin states from transactor and cycle each TxMAC.
	returns: FMON_SUCCESS or XACT_set_state_value fail
	uses:
	modifies: cycle, tx_stop_count
  -----------------------------------------------------------------*/
int FMon::Cycle()
{
	int i;
	unsigned int val;
	int retval = FMON_SUCCESS;

	for(i=0;i<fbus_mac_count;i++){
			retval &= macs[i].Cycle();
	}
	cycle++;

	if ((tx_stop_count)&&(tx_count == tx_stop_count)){
		val = 1;
		retval &= XACT_set_state_value(sim_halt_handle, &val);
		tx_stop_count = 0;
	}
	return(retval);
}


/*-----------------------------------------------------------------
	FMon_Clock
	description: Function for transactor to call every fbus cycle.
	uses:
	modifies:
  -----------------------------------------------------------------*/
FMon_Clock(XACT_HANDLE h, void* state)
{
    if (fm.initialized){
		fm.GetPinVals();
		fm.PrintPinVals();
		fm.Cycle();
	}
	if (fd.initialized) fd.Cycle();

	return(FMON_SUCCESS);
}

#if defined(__cplusplus)
extern "C"
#endif
void FMon_Reset()
{
    fm.initialized = 0;
    fd.initialized = 0;
}

/*-----------------------------------------------------------------
	FMon::XDefineCycleCallBack
	description: Cause transactor to call FMon_Clock every sim.fbus_clk_cycle.
	uses:
	modifies:
  -----------------------------------------------------------------*/
int FMon::XDefineCycleCallBack()
{

	int (*mfp)(int, void *) = FMon_Clock;		// member functon pointer

	XACT_HANDLE handle = XACT_get_handle("sim.fbus_clk_cycle", -1);
    if (handle == INVALID_XACT_HANDLE)
        return(FMON_XACT_FAIL);

    int retval = XACT_Define_Cancel_Callback_State_Transition(handle, mfp);
    if (retval == FALSE)
        return(FMON_XACT_FAIL);

	retval = XACT_Define_Callback_State_Transition(handle, 0, mfp, "");
    if (retval == FALSE)
        return(FMON_XACT_FAIL);

    return(FMON_SUCCESS);
}


/*-----------------------------------------------------------------
	FMon::GetPinVals
	description: Get transactor states.
	uses:
	modifies:
  -----------------------------------------------------------------*/
void FMon::GetPinVals()
{
	GetPortCtl();
	GetRdyBus();
	GetRdyCtl();
	GetRdyCtl4();
	GetGPIO();
	GetPortSel();
	GetFDataH();
	GetFDataL();
	GetByteEnHi();
	GetByteEnLo();
	GetSOP();
	GetEOP();
	GetSOPXmit();
	GetEOPXmit();
	GetXmitV();
	GetFBusCy();
	if(fm.fbuscy_val ==0){
		GetTime();
	}
	
	return;
}


/*-----------------------------------------------------------------
	FMon::PrintPinVals
	description: Print fbus pin values.
	uses:
	modifies:
  -----------------------------------------------------------------*/
void FMon::PrintPinVals()
{
	if(pr_enabled){
		XACT_printf("FCy: %.4d ", cycle);
		XACT_printf("pctl=%x, fps=%x, rdyctl_l=%x%x, gpio=%x, ",fm.pctl_val,fm.fps_val,fm.rdyctl_4_l_val,fm.rdyctl_l_val,gpio_val);
		XACT_printf("rdybus=");
		if(fm.rdybus_val == 0xDEADBEEF) XACT_printf("XX, "); else XACT_printf("%.2x, ", fm.rdybus_val);
		XACT_printf("eopsop=");
		if(fm.eop_val == 0xDEADBEEF) XACT_printf("X"); else XACT_printf("%x", fm.eop_val);
		if(fm.sop_val == 0xDEADBEEF) XACT_printf("X, "); else XACT_printf("%x, ", fm.sop_val);
		XACT_printf("eopsop32tx=");
		if(fm.eop32_xmit_val == 0xDEADBEEF) XACT_printf("X"); else XACT_printf("%x", fm.eop32_xmit_val);
		if(fm.sop32_xmit_val == 0xDEADBEEF) XACT_printf("X, "); else XACT_printf("%x, ", fm.sop32_xmit_val);
		XACT_printf("be_l=");
		if(fm.behi_val == 0xDEADBEEF) XACT_printf("X"); else XACT_printf("%x", fm.behi_val);
		if(fm.belo_val == 0xDEADBEEF) XACT_printf("X, "); else XACT_printf("%x, ", fm.belo_val);
		XACT_printf("data=%.8x %.8x\n",fm.fdatah_val,fm.fdatal_val);
	}
	return;
}


/*-----------------------------------------------------------------
	FMon::RegisterStates
	description: Register pin states with transactor.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
void FMon::RegisterStates()
{
	fps_handle = XACT_get_handle("fps", -1);
	behi_handle = XACT_get_handle("fbe_hi_l", -1);
	belo_handle = XACT_get_handle("fbe_lo_l", -1);
	sop_handle = XACT_get_handle("sop", -1);
	eop_handle = XACT_get_handle("eop", -1);
	sop32_xmit_handle = XACT_get_handle("sop32_xmit", -1);
	eop32_xmit_handle = XACT_get_handle("eop32_xmit", -1);
	fdatah_handle = XACT_get_handle("fdat_hi", -1);
	fdatal_handle = XACT_get_handle("fdat_lo", -1);
	time_handle = XACT_get_handle("fbi.cycle_count_incr_result", -1);
	fbuscy_handle = XACT_get_handle("sim.fbus_clk_cycle", -1);
	busmode_handle = XACT_get_handle("fbi.fbus.rdybus_template_ctl", -1);
	xmitv_handle = XACT_get_handle("fbi.xmit_valid_bits", -1);
	pctl_handle = XACT_get_handle("portctl_l", -1);
	rdybus_handle = XACT_get_handle("rdybus", -1);
	rdyctl_handle = XACT_get_handle("rdyctl_l", -1);
	rdyctl4_handle = XACT_get_handle("rdyctl_4_l", -1);
	gpio_handle = XACT_get_handle("gpio", -1);
	sop32_xmit_handle = XACT_get_handle("sop32_xmit", -1);
	eop32_xmit_handle = XACT_get_handle("eop32_xmit", -1);
	sim_halt_handle = XACT_get_handle("sim.halt", -1);

	return;
}


/*-----------------------------------------------------------------
	FMon::FClose
	description: Close all open TxPort files.
	returns: FMON_SUCCESS
	uses:
	modifies:
  -----------------------------------------------------------------*/
extern "C" int fm_fclose()
{
	int i, j;
	
	for(i=0; i<fm.fbus_mac_count; i++){
		for(j=0;j<PORTS_PER_MAC;j++){
			fm.macs[i].ports[j].FClose(fm.macs[i].ports[j].outfptr);
		}
	}

	return (FMON_SUCCESS);
}


/*-----------------------------------------------------------------
	FMon::Init
	description: Initialize all Tx MACs and their ports.
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
void FMon::Init()
{
	int i;

	cycle = 0;
	fbus_big_endian = 1;				// default to big


	for(i=0; i<fm.fbus_mac_count; i++){
		macs[i].my_id = i;
		fm.macs[i].Init();
	}
	return;
}


/*-----------------------------------------------------------------
	FMon::Go
	description: Go until last EOP of pkcount transmit packets
	returns: FDRIVE_SUCCESS or XACT fail return value
	uses:
	modifies:
	see also: xact.h
  -----------------------------------------------------------------*/
int FMon::Go(int pkcount)
{
	unsigned int val;

	tx_count = 0;
	tx_stop_count = pkcount;
	val = 0;
	return(XACT_set_state_value(sim_halt_handle, &val));
}

extern "C" int fm_go(int pkcount)
{
	return(fm.Go(pkcount));
}


/*-----------------------------------------------------------------
	fb_print
	description: Enable printing fbus info to command line.
	returns: 1
	uses:
	modifies:
  -----------------------------------------------------------------*/
extern "C" int fb_print()
{

	fm.pr_enabled = 1;

	return(1);
}


/*-----------------------------------------------------------------
	fb_noprint
	description: Disable printing fbus info to command line.
	returns: 1
	uses:
	modifies:
  -----------------------------------------------------------------*/
extern "C" int fb_noprint()
{
	fm.pr_enabled = 0;

	return(1);
}


/*-----------------------------------------------------------------
	fb_init
	description: Initialize the Fbus configuration parameters.
	returns: FBUS_SUCCESS or FBUS_ILLEGAL_BUS_SIZE or FBUS_WRONG_NUMBER_OF_MACS
	uses:
	modifies:
  -----------------------------------------------------------------*/
extern "C" int fb_init(int bus_size, int mac_count)
{
	if ((bus_size != 32) && (bus_size != 64)){
		XACT_printf("fb_init illegal bus size, usage:\n \t fb_init(32or64, mac_count)\n");
		return(FBUS_ILLEGAL_BUS_SIZE);
	}
	if ((mac_count == 0) || (mac_count>6)){
		XACT_printf("fb_init wrong number of macs, usage:\n \t fb_init(32or64, mac_count)\n");
		return(FBUS_WRONG_NUMBER_OF_MACS);
	}

	if (fm.initialized == 0){
		fm.initialized = 1;
		fm.RegisterStates();
		fm.XDefineCycleCallBack();
	}

	fm.GetBusMode();
	switch((0x3 & (fm.busmode_val >> 9)))
	{
	case 0x0:
		fm.mode = UNI4MAC;
		break;
	case 0x1:
		fm.mode = BI4MAC;
		break;
	case 0x2:
		fm.mode = UNI2MAC;
		break;
	case 0x3:
		fm.mode = BI2MAC;
		break;
	}
	fm.fbus_no_status = (fm.busmode_val >>6) & 1;

	fm.fbus_mac_count = mac_count;
	if (bus_size == 64){
		fm.fbus_bidirectional = 1;
		// if (mac_count > 2) fm.mode = BI4MAC; else fm.mode = BI2MAC;
	} else{
		fm.fbus_bidirectional = 0;
		// if (mac_count > 2) fm.mode = UNI4MAC; else fm.mode = UNI2MAC;
	}
	fm.Init();
	fd.Init();
	return(FBUS_SUCCESS);
}

/*-----------------------------------------------------------------
	fb_big
	description: Initialize the Fbus to big or little endian. if 0 little, else big.
		big is default, so you don't need to set it if you want big endian.
	returns: FBUS_SUCCESS
	uses:
	modifies:
  -----------------------------------------------------------------*/
extern "C" int fb_big(int big)
{
	fm.fbus_big_endian = big;

	return(FBUS_SUCCESS);
}


/*-----------------------------------------------------------------
	FMon_Help
	description: list FMon functions
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int FMon_Help()
{
	XACT_printf("\n");
	XACT_printf("*** FMon command-line functions:\n\n");
	XACT_printf("fb_init(bus_size, mac_count);     // initialize IX bus 32 or 64 bits, 1-6 macs\n");
	XACT_printf("fb_big(boolean);                  // set endian mode 0 = little, !0 = big(default)\n");
	XACT_printf("fb_print();                       // enable printing to command line/log\n");
	XACT_printf("fb_noprint();                     // disable printing\n");
	XACT_printf("tport_on(mac, port);              // enable a transmit port\n");
	XACT_printf("tport_off(mac, port);             // disable a transmit port\n");
	XACT_printf("tport_fopen(fname, mac, port);	   // open file for writing transmit-side packets\n");
	XACT_printf("port_show(mac, port);             // print port status\n");
	XACT_printf("tmac_set_rate(mac, Mbits_per_sec, MFClocks_per_sec);      // set TxMAC rate\n");
	XACT_printf("tmac_set_rdy_threshold(mac, rdy); // set available buf bytes at which TxMAC becomes ready\n");
	XACT_printf("tmac_set_send_threshold(mac, rdy);// set buf_bytes filled at which TxMAC starts to send\n");
	XACT_printf("mac_show(mac);                    // print mac information, buf byte counts, other status\n");
	XACT_printf("fm_go(n);                         // transmit n packets then stop (see also FDrive_Go)\n");
	XACT_printf("fm_fclose();                      // close all transmit side output files\n");
	return(1);
}


/*-----------------------------------------------------------------
	fb_help;
	description: list PkGen, FDrive and FMon functions
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int fb_help()
{
	PkGen_Help();
	FDrive_Help();
	FMon_Help();

	return(1);
}


extern fb_break()
{
	FDrive *thisfd = &fd;
	FMon *thisfm = &fm;
	return(1);
}



/*-----------------------------------------------------------------
	fmon_model_initialize
	description: Called by foreign_model_initialize to register console functions.
	returns: 1
	uses:
	modifies:
	see also: xact.h
  -----------------------------------------------------------------*/
extern "C" int fmon_model_initialize()
{
	if (!XACT_register_console_function("fb_help", fb_help, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() fb_help failed\n");
			return(0);
	}
	if (!XACT_register_console_function("fb_break", fb_break, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() fb_break failed\n");
			return(0);
	}
	if (!XACT_register_console_function("fb_init", fb_init, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() fb_init failed\n");
			return(0);
	}
	if (!XACT_register_console_function("fb_big", fb_big, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() fb_big failed\n");
			return(0);
	}
	if (!XACT_register_console_function("fb_print", fb_print, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() fb_print failed\n");
			return(0);
	}
	if (!XACT_register_console_function("fb_noprint", fb_noprint, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() fb_noprint failed\n");
			return(0);
	}
	if (!XACT_register_console_function("tport_on", tport_on, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() tport_on failed\n");
			return(0);
	}
	if (!XACT_register_console_function("tport_off", tport_off, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() tport_off failed\n");
			return(0);
	}
	if (!XACT_register_console_function("tport_fopen", tport_fopen, 1, 2) ){
            fprintf(stderr,"XACT_register_console_function() tport_fopen failed\n");
			return(0);
	}
	if (!XACT_register_console_function("port_show", port_show, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() port_show failed\n");
			return(0);
	}
	if (!XACT_register_console_function("tmac_set_rate", tmac_set_rate, 0, 3) ){
            fprintf(stderr,"XACT_register_console_function() tmac_set_rate failed\n");
			return(0);
	}
	if (!XACT_register_console_function("tmac_set_rdy_threshold", tmac_set_rdy_threshold, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() tmac_set_rdy_threshold failed\n");
			return(0);
	}
	if (!XACT_register_console_function("tmac_set_send_threshold", tmac_set_send_threshold, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() tmac_set_send_threshold failed\n");
			return(0);
	}
	if (!XACT_register_console_function("mac_show", mac_show, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() mac_show\n");
			return(0);
	}
	if (!XACT_register_console_function("fm_go", fm_go, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() fm_go failed\n");
			return(0);
	}
	if (!XACT_register_console_function("fm_fclose", fm_fclose, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() fm_fclose failed\n");
			return(0);
	}
	return(1);
}

