/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

// fmon.h
// fbus monitor

#ifndef FMON_H
#define FMON_H

#include "fileio.h"
#include "xact.h"
#include "fdevice.h"

enum fmon_returns{ FMON_SUCCESS = 1, FMON_BAD_FILEPTR, FMON_XACT_FAIL, FMON_STATUS_CYCLE, 
					FMON_IDLE_CYCLE };



//  port and mac count limits
#define MAX_MACS 7
#define MAX_FAST_PORTS_PER_MAC 2
// defaults for octal 21440 MAC 
#define PORTS_PER_MAC 8
#define TX_BUF_SIZE_SLOW_PORT 256
#define TX_DEFAULT_RDY_THRESHOLD 64
#define TX_DEFAULT_SEND_THRESHOLD 124
// for dual Gig MAC
#define TX_BUF_SIZE_GIG_PORT 2048


const int MAX_PORT_FILES = PORTS_PER_MAC * MAX_MACS;


// configurations of FBUS and MACs for FMon::mode
// = decode of <10:9> of fbi.fbus.rdybus_template_ctl
#define UNI4MAC 0
#define BI4MAC 1
#define UNI2MAC 2
#define BI2MAC 3



class TxPort :public FileIo{
public:
	void Init();
	int Enable();
	int Disable();
	int SinkFBus();
	void ShowCurrent();					// print status
	char left;							// for 32 bit mode write, left data value does not cr
	int enabled;
	int rdy;
	int my_id;
	int outfptr;
// wire level methods and parameters
	void TxPort::SendToWire();			// update buf_outptr, check for eop when sending
	void BufBytesPut(int n);			// simulate putting bytes from fbus into MAC buffer
	void BufBytesTake();				// sim taking bytes from MAC buffer to wire
	int buf_size;						// capacity of transmit buffer
	int rdy_threshold;					// port is ready if (buf_size - buf_bytes) >= this threshold
	int send_threshold;					// send to wire if buf_bytes >= this threshold
	double wire_bytes_per_clock;		// bytes to transmit per fbus cycle
	int buf_bytes;						// bytes in mac buffer

private:
	enum txport_state{ IDLE = 0, SOP_RECEIVED, EOP_RECEIVED };
	unsigned int state;

// wire level parameters and methods
	double inter_packet_wait;			// time delay between packets
	unsigned int packets_in_buf;		// how many eops in mac buffer
	unsigned char packet_started;		// packet send to wire has started
	unsigned char overflow;				// 1 if overflow happened	
	unsigned char underflow;			// 1 if underflow happened
	double buf_inptr;					// fbus side pointer
	double buf_outptr;					// wire side pointer
	unsigned char *eop;					// array of EOP flags, where in buffer there is EOP

};


class TxMAC :public FDevice{
public:
	void Init();						// set rdy_vector to 0, clear all pksource data
	int SetRate(int Mbits_per_sec, int MFClocks_per_sec);
	void ShowBufs();					// print buffer counts
	int Cycle();						// run MAC logic
	TxPort ports[PORTS_PER_MAC];
	unsigned char enabled_vector;		// 8 bit ports enabled vector (bit 0 = port 0, etc.)
	unsigned char rdy_vector;			// 8 bit ports ready vector (bit 0 = port 0, etc.)

private:
	void UpdatePortVectors();
	void DecodeMACSel();
	void DecodeMACRdySel();
	int tx_mac_sel;
	int tx_mac_rdy_sel;
};


class FMon {
  public:
	XACT_HANDLE pctl_handle;
	XACT_HANDLE rdybus_handle;
	XACT_HANDLE rdyctl_handle;
	XACT_HANDLE rdyctl4_handle;
	XACT_HANDLE fps_handle;
	XACT_HANDLE gpio_handle;
	XACT_HANDLE fdatah_handle;
	XACT_HANDLE fdatal_handle;
	XACT_HANDLE behi_handle;
	XACT_HANDLE belo_handle;
	XACT_HANDLE sop_handle;
	XACT_HANDLE eop_handle;
	XACT_HANDLE sop32_xmit_handle;
	XACT_HANDLE eop32_xmit_handle;
	XACT_HANDLE busmode_handle;
	XACT_HANDLE time_handle;
	XACT_HANDLE fbuscy_handle;
	XACT_HANDLE xmitv_handle;
	XACT_HANDLE sim_halt_handle;
	unsigned int pctl_val;
	unsigned int rdybus_val;
	unsigned int rdyctl_l_val;
	unsigned int rdyctl_4_l_val;
	unsigned int fps_val;
	unsigned int rxfps_val;
	unsigned int gpio_val;
	unsigned int gpio_val_dly;
	unsigned int fdatah_val;
	unsigned int fdatal_val;
	unsigned int behi_val;
	unsigned int belo_val;
	unsigned int sop_val;
	unsigned int eop_val;
	unsigned int sop32_xmit_val;
	unsigned int eop32_xmit_val;
	unsigned int busmode_val;
	unsigned int fbuscy_val;
	unsigned int time_val;
	unsigned int xmitv_val;

	int initialized;						// busmode initialized
	int pr_enabled;							// enable print to command line
	int mode;								// BI2MAC, BI4MAC, UNI2MAC, UNI4MAC
	int fbus_mac_count;
	int fbus_bidirectional;
	int fbus_no_status;
	int fbus_big_endian;					// 1 if big endian byte enables
	int enabled_vector;
	int rdy_vector;
	int cycle;
	int tx_count;							// count of packets transmitted
	int tx_stop_count;						// when to stop the simulator

	TxMAC macs[MAX_MACS];

	FMon();
	void Init();
	int Go(int pkcount);

	int XDefineCycleCallBack();
	void RegisterStates();
	int Cycle();
	int GetPortSel(){return(XACT_get_state_value(fps_handle, &fps_val));}
	int GetByteEnHi(){return(XACT_get_state_value(behi_handle, &behi_val));}
	int GetByteEnLo(){return(XACT_get_state_value(belo_handle, &belo_val));}
	int GetSOP(){return(XACT_get_state_value(sop_handle, &sop_val));}
	int GetEOP(){return(XACT_get_state_value(eop_handle, &eop_val));}
	int GetSOPXmit(){return(XACT_get_state_value(sop32_xmit_handle, &sop32_xmit_val));}
	int GetEOPXmit(){return(XACT_get_state_value(eop32_xmit_handle, &eop32_xmit_val));}
	int GetFBusCy(){return(XACT_get_state_value(fbuscy_handle, &fbuscy_val));}
	int GetFDataH(){return(XACT_get_state_value(fdatah_handle, &fdatah_val));}
	int GetFDataL(){return(XACT_get_state_value(fdatal_handle, &fdatal_val));}
	int GetTime(){return(XACT_get_state_value(time_handle, &time_val));}
	int GetBusMode(){return(XACT_get_state_value(busmode_handle, &busmode_val));}
	int GetXmitV(){return(XACT_get_state_value(xmitv_handle, &xmitv_val));}
	int GetPortCtl(){return(XACT_get_state_value(pctl_handle, &pctl_val));}
	int GetRdyBus(){return(XACT_get_state_value(rdybus_handle, &rdybus_val));}
	int GetRdyCtl(){return(XACT_get_state_value(rdyctl_handle, &rdyctl_l_val));}
	int GetRdyCtl4(){return(XACT_get_state_value(rdyctl4_handle, &rdyctl_4_l_val));}
	int GetGPIO(){return(XACT_get_state_value(gpio_handle, &gpio_val));}

	void GetPinVals();
	void PrintPinVals();
	int OpenOutPort(char* fname, int p);
	int QWordByteCount(unsigned int belo, unsigned int behi);
	int LWordByteCount(unsigned int be);
	int SinkFBus();

  private:
};

	extern "C" int FMon_Help();
	extern FMon fm;

#endif		// FMON_H
