/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* packet.cpp
 * packet generation routines
 *
 *
 * system: SA1200
 * subsystem: packet_gen
 * author: Don Hooper 3/25/98
 * revisions:
 * $History packet.cpp $
 * 
 * 
 */ 


#include <stdlib.h>				// where rand() is

#include "endian.h"
#include "packet.h"
#include "pkgen_fields.h"
#include "xact.h"



IPPacket P;			// global current packet


// see fileio.cpp

extern "C" int pk_fopen(char *fname, char *mode)
{
	return(P.FOpen(fname, mode));
}


extern "C" int pk_fclose(int fptr)
{
	return(P.FClose(fptr));
}


/*-----------------------------------------------------------------
	Packet::Packet
	description: Construct a packet class instance.
	uses:
	modifies:
  -----------------------------------------------------------------*/
Packet::Packet()
{
	fill_byte = 0;
	fill_insn = USE_INCR_FILL;
	count = 0;
	next_fptr_index = 0;
	print_format = 0;
}


/*-----------------------------------------------------------------
	Packet::SetMACField
	description: Insert val into field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::SetMACField(int id, int val)
{
	switch(id)
	{
	case MAC_DA_BYTES_03:
		pk_buf[0] = (unsigned short) (val >> 16);
		pk_buf[1] = (unsigned short)(val & 0xffff);
		break;
	case MAC_DA_BYTES_45:
		pk_buf[2] = (unsigned short) (val & 0xffff);
		break;
	case MAC_SA_BYTES_03:
		pk_buf[3] = (unsigned short) (val >>16);
		pk_buf[4] = (unsigned short) (val & 0xffff);
		break;
	case MAC_SA_BYTES_45:
		pk_buf[5] = (unsigned short) val;
		break;
	case MAC_PROTOCOL_LEN:
		pk_buf[6] = (unsigned short) val;
		break;

	default:
		return(PKGEN_NO_SUCH_FIELD);
	}
	return(PKGEN_SUCCESS);
}


/*-----------------------------------------------------------------
	Packet::GetMACField
	description: Return field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::GetMACField(int id)
{

	switch(id)
	{
	case MAC_DA_BYTES_03:
		return((pk_buf[0] << 16) | pk_buf[1]);
		break;
	case MAC_DA_BYTES_45:
		return(pk_buf[2]);
		break;
	case MAC_SA_BYTES_03:
		return((pk_buf[3] <<16) | pk_buf[4]);
		break;
	case MAC_SA_BYTES_45:
		return(pk_buf[5]);
		break;
	case MAC_PROTOCOL_LEN:
		return(pk_buf[6]);
		break;
	case L3_HEADER_START:
		return(pk_buf[L3_HEADER_START]);

	default:
		return(PKGEN_NO_SUCH_FIELD);
	}
	return(PKGEN_SUCCESS);
}


/*-----------------------------------------------------------------
	Packet::SetField
	description: Insert val into field identified by field-id id.
		ids are defined in pkgen_fields.h
	returns: PKGEN_SUCCESS or PKGEN_NO_SUCH_FIELD
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::SetField(int id, int val)
{
	if (SetMACField(id, val) == PKGEN_NO_SUCH_FIELD){
		XACT_printf("SetField: No such field\n");
		return(PKGEN_NO_SUCH_FIELD);
	}
	return(PKGEN_SUCCESS);
}


/*-----------------------------------------------------------------
	Packet::GetField
	description: Return field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::GetField(int id)
{
	int temp = GetMACField(id);
	if (temp == PKGEN_NO_SUCH_FIELD){
		XACT_printf("GetField: No such field\n");
	}
	return(temp);
}

/*-----------------------------------------------------------------
	IPPacket::SetMacAddr
	description: Insert val into field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int IPPacket::SetMACAddr(int id, unsigned int b03, unsigned short b45)
{
	switch(id){
	case MAC_DA_ADDR:
		pk_buf[0] = (short)(b03 >> 16);
		pk_buf[1] = (short)(b03 & 0xffff);
		pk_buf[2] = (b45 & 0xffff);
		break;
	case MAC_SA_ADDR:
		pk_buf[3] = (short)(b03 >> 16);
		pk_buf[4] = (short)(b03 & 0xffff);
		pk_buf[5] = (b45 & 0xffff);
		break;
	default:
		return(PKGEN_NO_SUCH_FIELD);
	}
	return(PKGEN_SUCCESS);
}

extern "C" int pk_set_mac_addr(int id, int b03, int b45)
{
	return(P.SetMACAddr(id, (unsigned int)b03, (unsigned short)b45));
}


/*-----------------------------------------------------------------
	IPPacket::SetIPField
	description: Insert val into field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int IPPacket::SetIPField(int id, int val)
{
	unsigned int temp;

	switch(id)
	{
	case IP_VERSION:
		pk_buf[ip_header_start] = val;
		break;
	case IP_TOTAL_LEN:
		pk_buf[ip_header_start+1] = val;
		break;
	case IP_ID:
		pk_buf[ip_header_start+2] = val;
		break;
// tbd: flags and fragmentation
	case IP_TIME_TO_LIVE:
		temp = pk_buf[ip_header_start+4];
		pk_buf[ip_header_start+4] = (temp & 0x00ff) | (val << 8);
		break;
	case IP_PROTOCOL:
		temp = pk_buf[ip_header_start+4];
		pk_buf[ip_header_start+4] = (temp & 0xff00) | val;
		break;
	case IP_CHECKSUM:
		pk_buf[ip_header_start+5] = val;
		break;
	case IP_SA:
		pk_buf[ip_header_start+6] = val >> 16;
		pk_buf[ip_header_start+7] = (val & 0xffff);
		break;
	case IP_DA:
		pk_buf[ip_header_start+8] = val >> 16;
		pk_buf[ip_header_start+9] = (val & 0xffff);
		break;
	default:
		return(PKGEN_NO_SUCH_FIELD);
	}
	GenChecksum();						// recalc and insert checksum

	return(PKGEN_SUCCESS);
}


/*-----------------------------------------------------------------
	IPPacket::GetIPField
	description: Return field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int IPPacket::GetIPField(int id)
{

	switch(id)
	{
	case IP_VERSION:
		return((int)(pk_buf[ip_header_start]));
	case IP_ID:
		return(pk_buf[ip_header_start+2]);
// tbd: flags and fragmentation
	case IP_TOTAL_LEN:
		return((int)pk_buf[ip_header_start+1]);
	case IP_TIME_TO_LIVE:
		return((int) pk_buf[ip_header_start+4] >> 8);
	case IP_PROTOCOL:
		return((int) pk_buf[ip_header_start+4] & 0xff);
	case IP_CHECKSUM:
		return(pk_buf[ip_header_start+5]);
	case IP_SA:
		return((pk_buf[ip_header_start+6] <<16) | pk_buf[ip_header_start+7]);
	case IP_DA:
		return((pk_buf[ip_header_start+8] <<16) | pk_buf[ip_header_start+9]);
		break;
	default:
		return(PKGEN_NO_SUCH_FIELD);
	}

	return(PKGEN_SUCCESS);
}


/*-----------------------------------------------------------------
	IPPacket::SetField
	description: Insert val into field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int IPPacket::SetField(int id, int val)
{
	if (SetMACField(id, val) == PKGEN_NO_SUCH_FIELD){
		if (SetIPField(id, val) == PKGEN_NO_SUCH_FIELD){
			XACT_printf("SetField: No such field\n");
			return(PKGEN_NO_SUCH_FIELD);
		}
	}
	return(PKGEN_SUCCESS);
}

extern "C" int pk_set_field(int id, int val)
{
	return(P.SetField(id, val));
}


/*-----------------------------------------------------------------
	IPPacket::GetField
	description: Return field identified by field-id id.
		ids are defined in pkgen_fields.h
	uses:
	modifies:
  -----------------------------------------------------------------*/
int IPPacket::GetField(int id)
{
	int temp = GetMACField(id);

	if (temp == PKGEN_NO_SUCH_FIELD){
		temp = GetIPField(id);
		if (temp == PKGEN_NO_SUCH_FIELD){
			XACT_printf("GetField: No such field\n");
		}
	}
	return(temp);
}

extern "C" int pk_get_field(int id)
{
	return(P.GetField(id));
}


/*-----------------------------------------------------------------
	Packet::Populate
	description: Populate a packet with fill bytes.
	fill_insn determines type of fill: replicated, incremented or random.
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::Populate()
{
	int i;
	int full_hwords = size >>1;

	// clear out the 4 bytes just beyoud the packet
	for (i=full_hwords;i<(full_hwords + 2); i++) {
		pk_buf[i] = 0;
	}

	switch (fill_insn)
	{
	case USE_FILL:
		for(i=0;i<full_hwords;i++) {
			pk_buf[i] = (unsigned short) fill_byte | (fill_byte << 8);
		}
		if (size % 2) {									// if odd, there is one more byte
			pk_buf[i] =  (unsigned short)(fill_byte << 8);
		}
		break;
	case USE_INCR_FILL:
		fill_byte++;
		if (fill_byte == 0){
			fill_byte = 1;
		}
		for(i=0;i<full_hwords;i++) {
			pk_buf[i] = (unsigned short) fill_byte | (fill_byte << 8);
		}
		if (size % 2) {									// if odd, there is one more byte
			pk_buf[i] =  (unsigned short)(fill_byte << 8);
		}
		break;
	case USE_RANDOM:
		for(i=0;i<full_hwords;i++) {
			pk_buf[i] = (unsigned short) rand() % 0xffff;
		}
		if (size % 2) {									// if odd, there is one more byte
			pk_buf[i] =  (unsigned short) (rand() % 0xff) <<8;
		}
		break;
	default:
		return(PKGEN_NO_SUCH_FILL);
	}

	return(PKGEN_SUCCESS);
}


/*-----------------------------------------------------------------
	Packet::ZeroHeader
	description: Zero the first num_hwords halfwords starting at the halfword index pk_index
	uses:
	modifies:
  -----------------------------------------------------------------*/
void Packet::ZeroHeader(int pk_index, int num_hwords)
{
	int i;

	for (i = 0; i<num_hwords; i++) {
		pk_buf[pk_index+i] = 0;
	}
	return;
}

extern "C" void pk_zero(int pk_index, int num_hwords)
{
	P.ZeroHeader(pk_index, num_hwords);
	return;
}


/*-----------------------------------------------------------------
	Packet::OtherEndian
	description: swap bytes in the halfwords of packets
		Note: packet starts out as big-endian.
	uses:
	modifies:
  -----------------------------------------------------------------*/
void Packet::OtherEndian()
{
	int i;
	int hwords = (size >>1)+1;		// cover possible remaining odd byte

	for (i=0; i<hwords; i++) {
		pk_buf[i] = swap2(pk_buf[i]);
	}

	return;
}


/*-----------------------------------------------------------------
	pk_endian_swap
	description: change endian-ness of the packet
		Note: packet starts out as big-endian.
	uses:
	modifies:
  -----------------------------------------------------------------*/
extern "C" void pk_endian_swap()
{
	int i;
	unsigned short temp;
	int hwords = (P.size >>1)+1;		// cover possible remaining odd byte
	for (i=0; i<hwords; i+=2) {
		temp = P.pk_buf[i];
		P.pk_buf[i] = swap2(P.pk_buf[i+1]);
		P.pk_buf[i+1] = swap2(temp);
	}
	return;
}


/*-----------------------------------------------------------------
	Packet::Create
	description: Generate a packet with default parameters of size s
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::Create(int s)
{
	size = s;										// store size in object
	count++;

	Populate();										// put something in every byte
	SetMACField(MAC_PROTOCOL_LEN, 1501);			// ethernet not 802.3

	return(PKGEN_SUCCESS);
}


extern "C" int pk_create(int size)
{
	return(P.Create(size));
}


/*-----------------------------------------------------------------
	Packet::Show
	description: Display a packet in big-endian order.
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::Show()
{
	int i;

	int qw = size >> 3;
	int rem_qw_bytes = size & 0x7;

	if (rem_qw_bytes) {
		qw++;
	}
	XACT_printf("bytes 0-3 \tbytes 4-7\n");
	for (i = 0; i < qw*4; i = i+4) {
			XACT_printf("0x%.4x%.4x\t0x%.4x%.4x\n", pk_buf[i], pk_buf[i+1], pk_buf[i+2], pk_buf[i+3]);

	}
	XACT_printf("size: %d bytes\n\n", size);

	return(PKGEN_SUCCESS);
}

extern "C" int pk_show()
{
	return(P.Show());
}


/*-----------------------------------------------------------------
	IPPacket::Create
	description: Generate a packet with default parameters of size s.
	uses:
	modifies:
  -----------------------------------------------------------------*/
int IPPacket::Create(int s)
{

// store parameters in object
	size = s;										// store size in object
	count++;
	ip_header_start = L3_HEADER_START;
	Populate();										// put something in every byte
	ZeroHeader(ip_header_start, 10);				// zero the IP header 20 bytes
	SetMACField(MAC_PROTOCOL_LEN, 1501);			// ethernet not 802.3
	SetIPField(IP_VERSION, 0x4500);
	SetIPField(IP_TOTAL_LEN, size - 18);			// 18 = MAC + FCS
	SetIPField(IP_TIME_TO_LIVE, 4);					// time to live
	SetIPField(IP_PROTOCOL, 0x06);					// tcp
	SetIPField(IP_DA, 0x02020202);

	// note: checksum is automatically generated by SetIPField

	return(PKGEN_SUCCESS);
}


/*-----------------------------------------------------------------
	IPPacket::GenChecksum
	description: Generate and insert the ip checksum.
	Assumes the packet data is big-endian. 
	uses:
	modifies:
  -----------------------------------------------------------------*/
IPPacket::GenChecksum()
{
	unsigned short i;
	unsigned int ck = 0;

	for (i = 7; i < 17; i++) {
		if (i != 12)	ck+= (unsigned int)pk_buf[i];
	}
	pk_buf[12] = (unsigned short) 0xffff & (~ ((ck >> 16) + (ck & 0xffff)));
	return(pk_buf[12]);
}


/*-----------------------------------------------------------------
	Packet:: Write
	description: Write a packet to a file.
	returns: PKGEN_SUCCESS
	uses:
	modifies:
  -----------------------------------------------------------------*/
int Packet::Write(int fptr)
{
	int i;
	FILE *infptr;

	infptr = open_files[fptr];

	int qw = size >> 3;
	int rem_qw_bytes = size & 0x7;

	if (rem_qw_bytes) {
		qw++;
	}

	if (print_format == VERBOSE){
		fprintf(infptr, "packet id: %d\n", count);
		fprintf(infptr, "size: %d\n", size);
		fprintf(infptr, "data bytes 0-7:\n");
	}
	
	for (i = 0; i < qw*4; i = i+4) {
			fprintf(infptr, "0x%.4x%.4x\t0x%.4x%.4x\n", pk_buf[i], pk_buf[i+1], pk_buf[i+2], pk_buf[i+3]);

	}
	fprintf(infptr, "\n");

	return(PKGEN_SUCCESS);
}

extern "C" int pk_write(int fptr)
{
	return(P.Write(fptr));
}


/*-----------------------------------------------------------------
	PkGen_Help
	description: list PkGen  functions
	returns:
	uses:
	modifies:
  -----------------------------------------------------------------*/
int PkGen_Help()
{
	XACT_printf("\n");
	XACT_printf("***PkGen command-line functions:\n\n");
	XACT_printf("pk_create(byte_size);               // create an IP packet with good checksum\n");
	XACT_printf("pk_show();                          // show packet just created\n");
	XACT_printf("pk_zero(start_hw, count_hw);        // zero count_hw halfwords\n");
	XACT_printf("pk_endian_swap();                   // endian swap all ints in packet\n");
	XACT_printf("pk_set_field(field_id, value);      // set field value\n");
	XACT_printf("\tField Ids are defined by @pkgen_fields.h\n");
	XACT_printf("\tvalid fields:\n");
	XACT_printf("\t\tMAC_DA_BYTES_01, MAC_DA_BYTES_25, MAC_SA_BYTES_01, MAC_SA_BYTES_25,\n");
	XACT_printf("\t\tMAC_PROTOCOL_LEN, ETHER_PROTOCOL,\n");
	XACT_printf("\t\tIP_VERSION, IP_TOTAL_LEN, IP_ID, IP_FLAGS, IP_FRAG, IP_TIME_TO_LIVE,\n");
	XACT_printf("\t\tIP_PROTOCOL, IP_CHECKSUM, IP_SA, IP_DA, ZP_PROTOCOL\n");
	XACT_printf("\tnote: each time an IP field is changed, a new checksum is generated\n");
	XACT_printf("pk_get_field(field_id);             // return field value\n");
	XACT_printf("pk_fopen(fname, mode);              // open a file, return an int file pointer\n");
	XACT_printf("pk_fclose(fileptr);                 // close file\n");
	XACT_printf("pk_write(fileptr);                  // write a packet to a file\n");
	
	return(1);
}





extern "C" int pkgen_model_initialize()
{

	if (!XACT_register_console_function("pk_create", pk_create, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() pk_create failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_show", pk_show, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() pk_show failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_zero", pk_zero, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() pk_zero failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_endian_swap", pk_endian_swap, 0, 0) ){
            fprintf(stderr,"XACT_register_console_function() pk_endian_swap failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_set_field", pk_set_field, 0, 2) ){
            fprintf(stderr,"XACT_register_console_function() pk_set_field failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_set_mac_addr", pk_set_mac_addr, 0, 3) ){
            fprintf(stderr,"XACT_register_console_function() pk_set_mac_addr failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_get_field", pk_get_field, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() pk_get_field failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_fopen", pk_fopen, 2, 0) ){
            fprintf(stderr,"XACT_register_console_function() pk_fopen failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_fclose", pk_fclose, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() pk_fclose failed\n");
			return(0);
	}
	if (!XACT_register_console_function("pk_write", pk_write, 0, 1) ){
            fprintf(stderr,"XACT_register_console_function() pk_write failed\n");
			return(0);
	}

	return(1);
}
