/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-99 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* packet.h
 * packet generation class definitions
 *
 *
 * system: SA1200
 * subsystem: packet_gen
 * author: Don Hooper 3/25/98
 * revisions:
 * $History packet.h $
 * 
 * 
 */ 
#ifndef PACKET_H
#define PACKET_H


#include "fileio.h"

// link layer
//-------------------------------------
// minimum frame size = 64, composed of
// bytes	field
// 6		mac_sa
// 6		mac_da
// 2		protocol_len
// 46-1500	link_payload
// 4		fcs
// -------
// 64-1518



// ip layer (in link_payload)
//-------------------------------------
//
// bytes
// 20-24	ip_header
// 0-1480	ip_payload


enum pkgen_returns { PKGEN_SUCCESS = 1, PKGEN_NO_SUCH_FIELD, PKGEN_NO_SUCH_FILL };


#define MIN_FRAME				64
#define MAX_FRAME				1518
#define MIN_LINK_PAYLOAD		46
#define MAX_LINK_PAYLOAD		1500
#define MAX_IP_HEADER			24
#define MIN_IP_PAYLOAD			22
#define MAX_IP_PAYLOAD			1480
#define MAX_TCP_HEADER			24
#define MAX_TCP_PAYLOAD			1456

#define USE_FILL				1
#define USE_INCR_FILL			2
#define USE_RANDOM				3

#define L3_HEADER_START			7

#define VERBOSE					1			// verbose packet file format


// base packet class that covers mac layer
//
class Packet :public FileIo{
public:
	int size;
	int outport;
	unsigned short pk_buf[MAX_FRAME / 2];

	int defaults[64];					// defaults field settings
	char use_defaults[64];				// use defaults if 1
	unsigned char fill_insn;			// use fill byte, random, etc
	unsigned char fill_byte;			// byte to fill packet with
	int count;							// number of packets created
	int print_format;					// packet format to file,
										//		raw data is in hex bytes: 00112233 44556677
										// DEFAULT: 
										//		<raw data> 
										//		at EOP, CR/linefeed
										// VERBOSE pretty prints 
										//		packet id: <id>
										//		size: <bytes>
										//		data bytes 0-7:
										//		<raw data>
										//		at EOP, CR/linefeed
	Packet();
	int Create(int size);
	int SetAutoFill(int insn, int byte);
	int Populate();
	void ZeroHeader(int pk_index, int num_hwords);
	void OtherEndian();
	inline int GetSize() { return(size); };
	int SetMACField(int id, int val);
	int GetMACField(int id);
	int SetField(int id, int val);
	int GetField(int id);
	int Show();

	int StoreRawData(int sdram_loc);
	int StoreStatus(int sram_loc, int port);
	int Write(int fptr);
	int Packet::XSDRAMWriteQWord(int sdram_index, int pk_index);
  private:
};


// packet class that covers ip layer
//
class IPPacket : public Packet{
  public:

	int Create(int size);
	int SetIPField(int id, int val);
	int GetIPField(int id);
	int SetField(int id, int val);
	int GetField(int id);
	int StoreData(int sdram_loc);
	int GenChecksum();
	int SetMACAddr(int id, unsigned int b03, unsigned short b45);

  private:
	int ip_header_start;
	int ip_header_len;

};



// packet class that covers tcp layer
//
class TCPPacket : public IPPacket{
  public:
	int Create(int size);
	int SetTCPField(int id, int val);
	int GetTCPField(int id);
	int SetField(int field_id, int val);
	int GetField(int field_id);
  private:
	int idx_tcp_payload_start;		// index in pk_buf
};


extern "C" int PkGen_Help();

extern IPPacket P;



#endif		// PACKET_H
