/* ---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *----------------------------------------------------------------------------*/

//////////////////////////////////////////////////////////////////////////////
// $Source: /mnt/bigpine/src/oem/IXP1200/CoreLibs/rtm-mvr/Attic/PortManager.h,v $
// $Revision: 2 $
// $Date: 5/02/00 3:35p $
//
// Description:
//     This file declares the PortManager class.  It is used by the
//     RouteTable class to associate routing tables with particular ports.
//     This association means that packets coming in on ports will be routed
//     using their associated routing table.
//
//////////////////////////////////////////////////////////////////////////////

#ifndef _PORTMANAGER_H_
#define _PORTMANAGER_H_

#include "Trie.h"
#include "RTMException.h"

/////////////////////////////////////////////////////////////////////////////
//  Name: PortManager - Manages a set of ports with respect to their associated
//                      routing table.
//
//  Description:
//      This class allows clients to associate ports with routing table
//      structures so that multiple routing tables can be used within an
//      installation.
//
/////////////////////////////////////////////////////////////////////////////
class PortManager {
public:
    //////////////////////////////////////////////////////////////////////////
    //  Method: PortManager::PortManager
    //
    //  Description:
    //      This is the constructor for the PortManager class.  It initializes
    //      data members and zeroes the memory for the table of first tries in
    //      SRAM.
    //////////////////////////////////////////////////////////////////////////
    PortManager(SRAM_ADDRESS location, unsigned int numPorts);
    virtual ~PortManager();

    //////////////////////////////////////////////////////////////////////////
    //  Method: PortManager::SetPortFirstTrie
    //
    //  Description:
    //      Changes the first trie to be used in route lookups for packets
    //      coming in on a particular ingress port.
    //////////////////////////////////////////////////////////////////////////
    void SetPortFirstTrie(PORT_NUMBER port, const class Trie& firstTrie)
        throw(InvalidPortException);

    //////////////////////////////////////////////////////////////////////////
    //  Method: PortManager::ClearPortFirstTrie
    //
    //  Description:
    //      Changes the first trie to be used in route lookups for packets
    //      coming in on a particular ingress port to NULL.  This will cause 
    //      all incoming packets to this port to be dropped.
    //////////////////////////////////////////////////////////////////////////
    void ClearPortFirstTrie(PORT_NUMBER port)
        throw(InvalidPortException);

    //////////////////////////////////////////////////////////////////////////
    //  Method: PortManager::RemoveFirstTrieFromAll
    //
    //  Description:
    //      Searches the list of ports and sets the first trie to NULL if its
    //      current first trie matches the specified one.
    //////////////////////////////////////////////////////////////////////////
    void RemoveFirstTrieFromAll(const class Trie& firstTrie);

    SRAM_ADDRESS GetLocation() const { return m_location; }
    unsigned int GetNumPorts() const { return m_numPorts; }

protected:
    SRAM_ADDRESS m_location;
    unsigned int m_numPorts;
};

#endif // _PORTMANAGER_H_
