/* ---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *----------------------------------------------------------------------------*/

//////////////////////////////////////////////////////////////////////////////
// $Source: /mnt/bigpine/src/oem/IXP1200/CoreLibs/rtm-mvr/Attic/TrieManager.h,v $
// $Revision: 2 $
// $Date: 5/02/00 3:35p $
//
// Description:
//      This file declares the TrieManager class.  This class acts as a
//      Factory for Trie objects.  When a TrieManager is created, it is
//      assigned a block of memory from which it can allocate trie blocks.
//////////////////////////////////////////////////////////////////////////////

#ifndef _TRIEMANAGER_H_
#define _TRIEMANAGER_H_

#include "rtm_mvr.h"
#include "Trie.h"
#include "RTMException.h"
#include <hal_sram.h>
#include <list>

#ifndef NONAMESPACES
using namespace std;
#endif

/////////////////////////////////////////////////////////////////////////////
//  Name: TrieManager
//
//  Description:
//      This class acts as a Factory for Trie objects.  When a TrieManager is 
//      created, it is assigned a block of memory from which it can allocate 
//      trie blocks.
/////////////////////////////////////////////////////////////////////////////
class TrieManager {
public:
    //////////////////////////////////////////////////////////////////////////
    //  Method: TrieManager::TrieManager
    //
    //  Description:
    //      This is the constructor for the TrieManager class.  It 
    //      initializes data members and zeroes the SRAM to be used by tries.
    //      It also creates a freelist of tries to use for allocating and 
    //      freeing them.
    //////////////////////////////////////////////////////////////////////////
    TrieManager(SRAM_ADDRESS trieBase, unsigned int maxNumTries);
    virtual ~TrieManager();

    //////////////////////////////////////////////////////////////////////////
    //  Method: TrieManager::GetTrieBase
    //
    //  Description:
    //      Retrieves the base address for all tries managed by this 
    //      TrieManager
    //////////////////////////////////////////////////////////////////////////
    SRAM_ADDRESS GetTrieBase();

    //////////////////////////////////////////////////////////////////////////
    //  Method: TrieManager::AllocateTrie
    //
    //  Description:
    //      This method creates a Trie and assigns a section of SRAM to it.
    //////////////////////////////////////////////////////////////////////////
    class Trie* AllocateTrie() throw (OutOfTriesException);
    //////////////////////////////////////////////////////////////////////////
    //  Method: TrieManager::FreeTrie
    //
    //  Description:
    //      This method destroys a trie block and makes it's SRAM available 
    //      for future use.
    //////////////////////////////////////////////////////////////////////////
    void FreeTrie(class Trie* pTrie);

    //////////////////////////////////////////////////////////////////////////
    //  Method: TrieManager::GetNumFreeTries
    //
    //  Description:
    //      Retrieves the number of free tries.
    //////////////////////////////////////////////////////////////////////////
    unsigned int GetNumFreeTries();

protected:
    SRAM_ADDRESS m_trieBase;
    unsigned int m_numTries;
    list<unsigned int> m_trieFreelist;
};

#endif //_TRIEMANAGER_H_
