/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* rtm.c
 *
 *
 * system: SA1200
 * subsystem: Xact_IO
 * author: Don Hooper 1/19/98
 * revisions:
 * $History: rtm.cpp $
 * 
 * *****************  Version 1  *****************
 * User: Qabuild      Date: 4/18/00    Time: 5:02p
 * Created in $/dev_1200/SA1_CoreLibs/rtm-mvr
 * added from IAL zip file 4/12/00
 * 
 * *****************  Version 29  *****************
 * User: Dhooper      Date: 1/21/00    Time: 3:12p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * add, lookup, delete now return int
 * 
 * *****************  Version 28  *****************
 * User: Dhooper      Date: 1/20/00    Time: 11:39a
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * clean up the API syntax and comments
 * 
 * *****************  Version 26  *****************
 * User: Dhooper      Date: 1/13/00    Time: 7:48p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * support route_delete
 * 
 * *****************  Version 25  *****************
 * User: Drjohns1     Date: 1/05/00    Time: 3:45p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * Removed simulation-related references for IOSTYLE=HARDWARE
 * 
 * *****************  Version 24  *****************
 * User: Dhooper      Date: 12/20/99   Time: 12:11p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * fixed tracker prob 807 doesn't find route for "41.41.41.32"
 * 
 * *****************  Version 23  *****************
 * User: Dhooper      Date: 12/14/99   Time: 2:59p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * fix mask 255.255.0.0 bug
 * 
 * *****************  Version 22  *****************
 * User: Dhooper      Date: 12/01/99   Time: 6:33p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * use 8 bit 256 table for full longest prefix match of 31:24
 * 
 * *****************  Version 21  *****************
 * User: Dhooper      Date: 12/01/99   Time: 11:47a
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * fix lost short match
 * 
 * *****************  Version 20  *****************
 * User: Dhooper      Date: 11/29/99   Time: 7:00p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * save prev longest match for use in trie walk
 * 
 * *****************  Version 19  *****************
 * User: Dhooper      Date: 9/02/99    Time: 6:22p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * write mask into trie mask tables
 * 
 * *****************  Version 17  *****************
 * User: DHooper     Date: 7/14/99    Time: 6:58p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * fix bug where end_bit -=4 should have been ned_bit +=4, causing route
 * table to be incorrect for vxworks. somehow it worked for nt
 * 
 */ 

#include "rtm_mvr.h"
#include "rtm_cfg.h"
#include "endian.h"
#if (IOSTYLE != HARDWARE)
#include "Xact_API.h"
#include "xact.h"
#endif
#include "uclo.h"
#include "mem_map.h"
#include "version.h"

#include "hal_sram.h"
#include "hal_sdram.h"
#include "TrieManager.h"
#include "Trie.h"
#include "PortManager.h"
#include "RouteTable.h"

// pointers to sdram
SramUnit*  g_pSram;
SdramUnit* g_pSdram;

#ifndef STANDALONE
#include "config_1200.h"
#endif //not STANDALONE

/*
    RTM_config is used to hold the RTM configuration, addresses, limits. etc.
 */

RTM_ConfigInfo RTM_config;
TrieManager* g_pTrieManager = 0;
PortManager* g_pPortManager = 0;



/*-----------------------------------------------------------------
        RTM_Init
        Initialize data structures for RTM.
        returns: RTM_SUCCESS or RTM_FAIL
        uses:
        modifies: microengine microcode storage
  -----------------------------------------------------------------*/
int    // RTM_SUCCESS or RTM_RTM_FAIL
RTM_Init()
{
    int i;
    unsigned int baseaddr, val;

    // get sdram pointers
    g_pSdram = SDRAM_Attach();
    g_pSram = SRAM_Attach();
    g_pTrieManager = new TrieManager(SRAM_DYNAMIC_LWD_START, NUM_TRIE_SETS);
    g_pPortManager = new PortManager(SRAM_FIRST_TRIES, 16);


/*
    Initialize the route table.
    This is an SDRAM array of route entries containing the information about 
    the actual routes. At present each route entry consist eight long 
    words.
*/
    RTM_config.route_table_base = baseaddr = SDRAM_ROUTE_FWD_BASE*2;

#if (IOSTYLE != HARDWARE)
    // if no hardware, zero this buffer space quickly through XactIo
    if (XactIo_ZeroSDBuffer(NUM_ROUTES * SIZE_OF_ROUTE, baseaddr) 
        != XACT_SUCCESS) {
        perror("RTM_Init(): fatal error, unable to initialize route table in "
               "SDRAM\n");
        return(RTM_FAIL);
    }
#else
    for (i=0; i < NUM_ROUTES * SIZE_OF_ROUTE ; i++) {
        g_pSdram->write(baseaddr+i, 0);
    }
#endif
    // set default location to no route (-1)
    val = -1;
    g_pSdram->write(baseaddr, val);

    RTM_config.num_free_routes = NUM_ROUTES;

    // initialize route freelist
    // start at 1, 0 is reserved as default
    RTM_config.next_free_rt = 1;
    for(i=1;i<(NUM_ROUTES-1);i++){
        RTM_config.rt_flinks[i].next = i+1;
        RTM_config.rt_flinks[i].index = i;
    }
    RTM_config.rt_flinks[i].next = 0;
    RTM_config.rt_flinks[i].index = i;

#ifndef STANDALONE
    // bind symbols in system config
    RTM_DelayBindSymbols("receive0");
    RTM_DelayBindSymbols("receive1");
    RTM_DelayBindSymbols("receive2");
#endif

    return(RTM_SUCCESS);
} // end of RTM_Init()


#ifndef STANDALONE
/*-----------------------------------------------------------------
        RTM_DelayBindSymbols
        Delay BindSymbols (microcode import variables)for an image in system 
        configuration sysConf, so that each subsequent microcode load will bind 
        the symbols then.
        returns: RTM_SUCCESS or RTM_FAIL
        uses:
        modifies: symbol values in microengine microcode image copy
  -----------------------------------------------------------------*/
int RTM_DelayBindSymbols(char *image_name)
{
    unsigned int baseaddr = SRAM_DYNAMIC_LWD_START;

    SysConf *sysConf = SysConf_Attach();
    if (sysConf->BindSymbol(image_name, "trie_base", baseaddr)){
        printf("Error; binding symbol 'trie_base'\n");
        return(RTM_FAIL);
    }

    baseaddr = SDRAM_ROUTE_FWD_BASE;    // from mem_map.h shared with 
                                        // microcode(ucode/system)

    /*RTM_RT_Table */
    if (sysConf->BindSymbol(image_name, "route_table_base", baseaddr)){
        printf("Error; binding symbol 'route_table_base'\n");
        return(RTM_FAIL);
    }

    baseaddr = SRAM_FIRST_TRIES;    // from mem_map.h shared with 
                                        // microcode(ucode/system)

    /*RTM_RT_Table */
    if (sysConf->BindSymbol(image_name, "first_tries", baseaddr)){
        printf("Error; binding symbol 'first_tries'\n");
        return(RTM_FAIL);
    }

    return(RTM_SUCCESS);

}
#endif //not STANDALONE


/*-----------------------------------------------------------------
        RTM_BindSymbols
        BindSymbols for an image
        returns: RTM_SUCCESS or RTM_FAIL
        uses:
        modifies: symbol values in microengine microcode image copy
  -----------------------------------------------------------------*/
int RTM_BindSymbols(void *objHandle, char *image_name)
{
    unsigned int baseaddr;

    /* Bind trie_table Import Variable */
    baseaddr = SRAM_DYNAMIC_LWD_START;

    if((UcLo_BindSymbol(objHandle, image_name, "trie_base",baseaddr )) 
        != UCLO_SUCCESS) {
        printf("Error; binding symbol 'trie_base'\n");
        return(RTM_FAIL);
    }

    baseaddr = SDRAM_ROUTE_FWD_BASE;                        // from mem_map.h shared with microcode(ucode/system)

    /*RTM_RT_Table */
    if((UcLo_BindSymbol(objHandle, image_name, "route_table_base",baseaddr)) 
        != UCLO_SUCCESS) {
        printf("Error; binding symbol 'route_table_base'\n");
        return(RTM_FAIL);
    }

    baseaddr = SRAM_FIRST_TRIES;                        // from mem_map.h shared with microcode(ucode/system)

    /*RTM_RT_Table */
    if((UcLo_BindSymbol(objHandle, image_name, "first_tries",baseaddr)) 
        != UCLO_SUCCESS) {
        printf("Error; binding symbol 'first_tries'\n");
        return(RTM_FAIL);
    }

    return(RTM_SUCCESS);
}

/*-----------------------------------------------------------------
        RTM_InitULookup
        Read microcode image file, bind shared address symbols and write
        image to microengines.
        returns: RTM_SUCCESS or RTM_FAIL
        uses:
        modifies: microengine microcode storage
  -----------------------------------------------------------------*/
int                                                                     // RTM_SUCCESS or RTM_FAIL
RTM_InitULookup(char *fnam)
{
    void *objHandle;

    UcLo_InitLib();
        
    /* Load the uCode image from file */
    if((UcLo_LoadObjFile(&objHandle, fnam)) != UCLO_SUCCESS) {
        printf("Error; loading microcode object file %s\n", fnam);
        return(RTM_FAIL);
    }

    RTM_BindSymbols(objHandle, "receive0");
    RTM_BindSymbols(objHandle, "receive1");
    RTM_BindSymbols(objHandle, "receive2");

    /* copy microcode images to fboxes */
    if((UcLo_WriteUimageAll(objHandle)) != UCLO_SUCCESS) {
        printf("Error; failed to write image to uStore\n");
        return(RTM_FAIL);
    }

    return(RTM_SUCCESS);
}

/*-----------------------------------------------------------------
        RTM_AllocRoute
        description: get index to next free route entry
        uses: RTM_config
        modifies: pointer to RTM_config.next_free_rt
        returns: index of new route entry, or RTM_NO_ROUTE_FREE
  -----------------------------------------------------------------*/
int                 // index to new trie block or NO_TRIE_BLOCK_FREE
RTM_AllocRoute()
{
    int new_route;
        
    if (new_route = RTM_config.next_free_rt){
        RTM_config.num_free_routes--;
        RTM_config.next_free_rt = RTM_config.rt_flinks[new_route].next;
        return(new_route*SIZE_OF_ROUTE);
    } else return(RTM_NO_ROUTE_FREE);
}


/*-----------------------------------------------------------------
        RTM_Free_Route
        Free a Route entry and place a Flink with route's index at head of 
        RTM_config.next_free_rt
        returns: RTM_SUCCESS
        uses:
        modifies: RTM_config.next_free_rt
  -----------------------------------------------------------------*/
int                 // RTM_SUCCESS or RTM_FAIL
RTM_FreeRoute(unsigned int route_index)
{
    int i;
    int prev_free, new_free;

    prev_free = RTM_config.next_free_rt;
    new_free = route_index/SIZE_OF_ROUTE;

    RTM_config.rt_flinks[new_free].next = prev_free;
    RTM_config.next_free_rt = new_free;

    for(i = 0; i < SIZE_OF_ROUTE; i++){
        g_pSdram->write(RTM_config.route_table_base+route_index+i, 0);
    }

    RTM_config.num_free_routes++;


    return(RTM_SUCCESS);
}


/*-----------------------------------------------------------------
        RTM_RtLookup
        Get a route from the route table using sram reads and no
        side structure usage.
        returns: pointer to the route entry
        uses:
        modifies:
  -----------------------------------------------------------------*/
int         // pointer to route entry
RTM_RtLookup(char *destination, unsigned int inPort)      // string ip destination, e.g. "1.1.1.1"
{
    uint dest = inet_addr(destination);
    //convert to little-endian
    dest = ntohl(dest);

    unsigned int currentNibble = 0;
    unsigned int memContents = 0;
    unsigned int routePtr = 0;
    unsigned int currentTrie = 0;
    unsigned int offset = SRAM_DYNAMIC_LWD_START;

    // First we find the correct first trie for this ingress port
    g_pSram->read(SRAM_FIRST_TRIES + inPort, &currentTrie);
    for(int i = 7; i >= 0; i--)
    {
        currentNibble = (dest >> (i * 4)) & 0xf;

        g_pSram->read(offset + currentTrie + currentNibble, &memContents);
        if(memContents >> 16) {
            routePtr = memContents >> 16;
        }
        currentTrie = memContents & 0xffff;
        if(!currentTrie) {
            break;
        }
    }

    // route_loc is in bits 31:16 of the entry
    // if this field is 0, it is the default route, first entry of rt table
    return(RTM_config.route_table_base + routePtr);
}

