/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* rtm_cfg.h
 *
 *
 *
 * system: SA1200
 * subsystem: Route Table Manager
 * author: Don Hooper
 * revisions:
 * $History: rtm_cfg.h $
 * 
 * *****************  Version 2  *****************
 * User: Qabuild      Date: 5/02/00    Time: 3:35p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm-mvr
 * updated from IAL hudson4-28-00.zip
 * 
 * *****************  Version 7  *****************
 * User: Dhooper      Date: 1/20/00    Time: 11:39a
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * clean up the API syntax and comments
 * 
 * *****************  Version 6  *****************
 * User: Dhooper      Date: 1/13/00    Time: 7:48p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * support route_delete
 * 
 * *****************  Version 5  *****************
 * User: Dhooper      Date: 12/01/99   Time: 6:33p
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * use 8 bit 256 table for full longest prefix match of 31:24
 * 
 * *****************  Version 3  *****************
 * User: Dhooper      Date: 5/07/99    Time: 10:55a
 * Updated in $/dev_1200/SA1_CoreLibs/rtm
 * use latest IOSTYLE convention
 * 
 * *****************  Version 1  *****************
 * User: Dhooper      Date: 3/26/99    Time: 4:51p
 * Created in $/development/rtm
 * move rtm internal defs here from rtm.h
 * 
 * 
 */ 
#ifndef RTM_CFG_H
#define RTM_CFG_H
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#ifdef STANDALONE
#include "iostyle.h"                    // core to rest of chip interface style
#endif

#ifdef WIN32
#include <winsock.h>                    // to get inet_addr()
#else
#include <sys/socket.h>
#endif


#define uint    unsigned int
#define uchar   unsigned char

#ifndef WIN32
#define CLEAR "clear"
#else
#define CLEAR "cls"
#endif

#define TRUE    1
#define FALSE   0

/* default number of routes */
#define NUM_ROUTES   512
/* default number of trie entries */
#define NUM_TRIE_SETS    1024
/* Number of entries in trie set = number of long words */
#define SIZE_OF_TRIE_SET 16


#ifdef WIN32
#ifndef bzero
#define bzero(a,b)   memset( (a), 0, (b) )
#endif
#endif

/* DATA STRUCTURES  */



// Flink type to support freelists      
struct Flink{
    int next;
    int index;
};

struct RTM_ConfigInfo{
    // route entries
    unsigned int route_table_base;            // array of route entries. in IO 
                                              // space, shared with microengines
    unsigned int next_free_rt;                // head of freelist
    Flink rt_flinks[NUM_ROUTES];              // static allocation of nodes 
                                              // for freelist
    unsigned int num_free_routes;             // number of free routes 
                                              // entry locations
};




extern RTM_ConfigInfo RTM_config;
extern class SramUnit*  g_pSram;
extern class SdramUnit* g_pSdram;
extern class TrieManager* g_pTrieManager;
extern class PortManager* g_pPortManager;


#endif /* RTM_CFG_H */
