/* ------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------------------------*/

/* rtm_console.c
 * register console functions so they can be called at the transactor cmd line
 *
 *
 * system: SA1200
 * subsystem: RTM
 * author: Don Hooper 1/29/98
 * revisions:
 * $History rtm_console.c $
 * 
 * 
 */ 

/*=================================================================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "rtm_mvr.h"
#include "rtm_test.h"
#include "rtm_cfg.h"
#include "endian.h"
#include "rtm_console.h"
#include "hal_sram.h"
#include "hal_sdram.h"
#include "packet_gen.h"
#include "ambaio.h"
#include "mem_map.h"

#if defined(BUILD_DLL)
#include "xact_dll.h"
#else
#include "xact.h"
#endif

#include "RouteTable.h"

#if (IOSTYLE!=HARDWARE)
extern unsigned int AMBAIO_InitDone;
#endif


// Change printf to a macro depending on IOSTYLE
#if (IOSTYLE!=HARDWARE)
#define PRINTF XACT_printf
#else
#define PRINTF printf
#endif

int rt_init()
{

// note: because ueng driver must run AMBAIO, we init the AMBAIO here
    /*
#if (IOSTYLE != HARDWARE)
// initialize ambaio

    AMBAIO_InitDone = 0;

        ambaIoInit();
        char *spin="|/-|\\";
        int i =0;
        while(AMBAIO_InitDone == 0){
                if(i > 4) i = 0;
        printf("                Init AmbaIO:    %c\n", spin[i++]);
        XACT_ExecuteCommandStr("go 1");
    }
#endif
*/
        // get sdram pointer
    return(RTM_Init());
}

int rt_load_ucfile(char *fnam)
{
    return(RTM_InitULookup(fnam));
}

HROUTETABLE rt_create_table() {
    return new RouteTable(*g_pTrieManager, *g_pPortManager);
}

int rt_delete_table(HROUTETABLE table) {
    delete (RouteTable*)table;
    return RTM_SUCCESS;
}

int rt_add_port(HROUTETABLE table, unsigned int port) {
    ((RouteTable*)table)->AddPort(port);
    return RTM_SUCCESS;
}

int rt_remove_port(HROUTETABLE table, unsigned int port) {
    ((RouteTable*)table)->RemovePort(port);
    return RTM_SUCCESS;
}

/*-----------------------------------------------------------------
        route_add
        Add a route to the route table, using string and int argument types 
        returns: return value of RTM_RtAdd
        modifies: Route Table
        see also: rtm.cpp RTM_RtAdd
  -----------------------------------------------------------------*/
int rt_add_full(
             char *dest,            // string ip destination, e.g. "1.1.1.1"
             char *netmask,         // string netmask, e.g., "255.255.0.0"
             char *gateway,         // string next hop gateway, e.g., "255.255.0.0"
             int itf,               // physical interface id (output        port number)
             int gateway_da_hi32,   // next hop MAC DA 48:16
             int gateway_da_lo16,   // next hop MAC DA 15:0
             HROUTETABLE table)
{
    int i;
    struct RouteTable::RouteInfo add_info;
        
    add_info.itf = itf;
    add_info.nextHopDA.hi32 = gateway_da_hi32;
    add_info.nextHopDA.lo16 = gateway_da_lo16;
    add_info.destination = inet_addr(dest);
    add_info.netmask = inet_addr(netmask);
    add_info.gateway = inet_addr(gateway);
    
    // set rest of route info to 0
    for (i = 1; i<SIZE_OF_REST_ROUTE_ENTRY;i++){
            add_info.rest[i] = 0;
    }

    // add_info.rt_info[1] = add_info.gateway;

    try {
        ((RouteTable*)table)->AddRoute(add_info);
    }
    catch(...) {
        return RTM_FAIL;
    }

    return RTM_SUCCESS;
}
// same as route_add, but gateway is 0, for testing pruposes
int rt_add(char *dest, char *netmask, int itf, HROUTETABLE table)
{
    return rt_add_full(dest, netmask, "", itf, 0, 0, table);
}


/*-----------------------------------------------------------------
        route_lookup
        Get a route from the route table. 
        This can be called from an .ind file.
        returns: physical interface number
        see also: rtm.cpp
        modifies:
  -----------------------------------------------------------------*/
int                                                                     // interface number
rt_lookup(char *destination,            // string ip destination, e.g. "1.1.1.1"
          unsigned int inPort)      // ingress port for this lookup
{
    uint route_loc;
    uint val;
    
    route_loc = RTM_RtLookup(destination, inPort);
    g_pSdram->read(route_loc, &val);

    return(val);                                        // first word in route entry is the interface
}

/*-----------------------------------------------------------------
        rt_lookup_all
        Get a route from the route table for each input port
  -----------------------------------------------------------------*/
void
rt_lookup_all(char *destination,     // string ip destination, e.g. "1.1.1.1"
              unsigned int numPorts) // number of ports (0 - numPorts-1 to
                                     // lookup
{
    uint route_loc;
    uint val;
    
    printf("In\tOut\n");
    for (int i=0; i < numPorts; ++i) {
        route_loc = RTM_RtLookup(destination, i);
        g_pSdram->read(route_loc, &val);
        printf("%d\t%d\n", i, val);
    }
}

/*-----------------------------------------------------------------
        route_delete
        Delete a route from the route table, using simple argument types 
        returns: return value of RTM_RtDelete
        modifies: Route Table
        see also: rtm.cpp RTM_RtDelete
  -----------------------------------------------------------------*/
int rt_delete(
             char *dest,            // string ip destination, e.g. "1.1.1.1"
             char *netmask,         // string netmask, e.g., "255.255.0.0"
             HROUTETABLE table)
{
    unsigned int destination = inet_addr(dest);
    unsigned int mask = inet_addr(netmask);

    try {
        ((RouteTable*)table)->DeleteRoute(destination, mask);
    }
    catch(...) {
        return RTM_FAIL;
    }

    return RTM_SUCCESS;
}

int rt_trie_show(HROUTETABLE table)
{
#ifdef DEBUG
    ((RouteTable*)table)->TrieShow();
#endif
    return(RTM_SUCCESS);
}

int rt_ent_show(int route_table_index)
{
    uint route_loc = RTM_config.route_table_base + route_table_index;
    uint itf, dest, mask, gateway, mac_hi32, mac_lo16;

    g_pSdram->read(route_loc, &itf);
    g_pSdram->read(route_loc+1, &mac_hi32);
    g_pSdram->read(route_loc+2, &mac_lo16);
    g_pSdram->read(route_loc+3, &dest);
    g_pSdram->read(route_loc+4, &mask);
    g_pSdram->read(route_loc+5, &gateway);

    uint mac1 = mac_hi32 >> 24;
    uint mac2 = (mac_hi32 >> 16) & 0xff;
    uint mac3 = (mac_hi32 >> 8) & 0xff;
    uint mac4 = mac_hi32 & 0xff;
    uint mac5 = mac_lo16 >> 8;
    uint mac6 = mac_lo16 & 0xff;
    PRINTF("Route Entry at 0x%08x:\n", route_table_index);
    PRINTF("---------------------------\n");
    PRINTF("Port #   Dest IP      Dest Mask    Dest MAC Addr\n");
    PRINTF("----------------------------------------------------\n");
    PRINTF("0x%x      0x%08x   0x%08x   %02x:%02x:%02x:%02x:%02x:%02x\n",
           itf, dest, mask, mac1, mac2, mac3, mac4, mac5, mac6);
    return(RTM_SUCCESS);
}

int rt_ent_info(char *destination, unsigned int inPort)
{
    uint route_loc;

    route_loc = RTM_RtLookup(destination, inPort);

    return(rt_ent_show(route_loc - RTM_config.route_table_base));
}

int rt_trie_show_real(unsigned int trieIndex, unsigned int recurse)
{
    if (!g_pTrieManager) {
        return -1;
    }

    unsigned int trieBase = g_pTrieManager->GetTrieBase();

    PRINTF("Trie Contents for Trie at 0x%x\n", trieIndex);
    PRINTF("------------------------------------\n");
    PRINTF("Entry #   Next Trie Addr    Route Entry Addr\n");
    PRINTF("--------------------------------------------\n");

    for (unsigned int i = 0; i < 16 ; i++) {
        unsigned int entry = 0;
        g_pSram->read(trieBase + trieIndex + i, &entry);

        unsigned int routeEntry = entry >> 16;
        unsigned int nextTrie = entry & 0xffff;

        PRINTF("0x%x       0x%08x        0x%08x\n", 
               i, nextTrie, routeEntry);
    }

    if (recurse) {
        for (unsigned int i = 0; i < 16 ; i++) {
            unsigned int entry = 0;
            g_pSram->read(trieBase + trieIndex + i, &entry);

            unsigned int routeEntry = entry >> 16;
            unsigned int nextTrie = entry & 0xffff;

            if (nextTrie) {
                rt_trie_show_real(nextTrie, recurse);
            }
            if (routeEntry) {
                rt_ent_show(routeEntry);
            }
        }
    }

    return RTM_SUCCESS;
}

/*-----------------------------------------------------------------
        rt_help
        description: list RTM functions
        returns:
        uses:
        modifies:
  -----------------------------------------------------------------*/
int rt_help()
{
    PRINTF("\n");
    PRINTF("*** RTM command-line functions:\n\n");
    PRINTF("rt_init();\n");
    PRINTF("    Initializes RTM table structures.\n\n");
    PRINTF("rt_load_ucfile(char *uof_file);\n");
    PRINTF("    Loads microcode file and inserts shared variables\n\n");
    PRINTF("rt_create_table();\n");
    PRINTF("    Creates new route table. HROUTETABLE returned.\n\n");
    PRINTF("rt_delete_table(HROUTETABLE table);\n");
    PRINTF("    Deletes route table.\n\n");
    PRINTF("rt_add_port(HROUTETABLE table, int inport);\n");
    PRINTF("    Adds ingress port to specified route table.\n\n");
    PRINTF("rt_remove_port(HROUTETABLE table, int inport);\n");
    PRINTF("    Removes ingress port from specified route table.\n\n");
    PRINTF("rt_add(char *ipda, char *netmask, int fwd_port, HROUTETABLE table);\n");
    PRINTF("    Inserts route entry.\n\n");
    PRINTF("rt_add_full(char *ipda, char *netmask, char* next_hop, int fwd_port,\n");
    PRINTF("            int da_hi32, int da_lo16, HROUTETABLE table);\n");
    PRINTF("    Inserts complete route entry.\n\n");
    PRINTF("rt_delete(char *ipda, char *netmask, HROUTETABLE table);\n");
    PRINTF("    Deletes route entry.\n\n");
    PRINTF("rt_lookup(char *ipda, int inport);\n");
    PRINTF("    Looks up forwarding port in route table for specified port.\n\n");
    PRINTF("rt_ent_info(char *ipda, int port);\n");
    PRINTF("    Shows route entry for specified address and port.\n\n");
    PRINTF("rt_ent_show(int route_table_index);\n");
    PRINTF("    Shows route entry at specified index.\n\n");
    PRINTF("rt_trie_show(HROUTETABLE table);\n");
    PRINTF("    Shows trie structure for specified table.\n\n");
    PRINTF("rt_trie_show_real(int index, int recurse);\n");
    PRINTF("    Shows trie structure for specified table.  Uses only SRAM entries.\n");
    PRINTF("    recurse indicated whether or not to recurse (1 = yes)\n\n");
    PRINTF("rt_port_show();\n");
    PRINTF("    Shows first trie addresses for each port.\n\n");
    PRINTF("rt_test();\n");
    PRINTF("    Runs prescripted tests.\n\n");
    return(1);
}

/* FOREIGN MODEL INITIALIZE

  instructions
        1. define ext_model_port as a compiler switch when compiling safe_xact
        2. "                                                                                                       " rtm_registration
 */

/* functions called by transactor to foreign model */

int IO_Read(unsigned int addr)
{
    unsigned int *val = 0;
#if (IOSTYLE != HARDWARE)
    AMBAIO_AutoRead32("",addr, val);
#endif

    return(*val);
}

int IO_Write(unsigned int addr, unsigned int val)
{
#if (IOSTYLE != HARDWARE)
    AMBAIO_AutoWrite32("",(unsigned int)addr, (unsigned int)val);
#endif
    return(val);
}

int rt_port_show()
{
    if (!g_pPortManager) {
        return -1;
    }

    unsigned int location = g_pPortManager->GetLocation();
    unsigned int numPorts = g_pPortManager->GetNumPorts();

    PRINTF("First tries for ingress ports:\n");
    PRINTF("------------------------------\n");
    PRINTF("Port #     First Trie Address\n");
    PRINTF("-----------------------------\n");

    for (unsigned int i = 0; i < numPorts; i++) {
        unsigned int firstTrie = 0;
        g_pSram->read(location + i, &firstTrie);

        PRINTF("0x%x        0x%08x\n", i, firstTrie);
    }

    return RTM_SUCCESS;
}


/* this routine will be called to initialize the foreign model after */
/* the transactor "init" command has successfully executed.  Returning 0 */
/* will result in a transactor error. */
#if (IOSTYLE != HARDWARE)
#ifndef STANDALONE
int rtm_model_initialize()
#else
int foreign_model_initialize()
#endif
{
    if (!XACT_register_console_function("rt_help", rt_help, 0, 0) ){
        fprintf(stderr,"XACT_register_console_function() rt_help failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_init", rt_init, 0, 0) ){
        fprintf(stderr,"XACT_register_console_function() rt_init failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_load_ucfile", rt_load_ucfile, 1, 0) ){
        fprintf(stderr,"XACT_register_console_function() rt_load_ucfile failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_add", rt_add, 2, 2) ){
        fprintf(stderr,"XACT_register_console_function() rt_add failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_add_full", rt_add_full, 3, 4) ){
        fprintf(stderr,"XACT_register_console_function() rt_add_full failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_lookup", rt_lookup, 1, 1) ){
        fprintf(stderr,"XACT_register_console_function() rt_lookup failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_delete", rt_delete, 2, 1) ){
        fprintf(stderr,"XACT_register_console_function() rt_delete failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_trie_show", rt_trie_show, 0, 1) ){
        fprintf(stderr,"XACT_register_console_function() rt_trie_show failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_trie_show_real", rt_trie_show_real, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() rt_trie_show_real failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_port_show", rt_port_show, 0, 0) ){
        fprintf(stderr,"XACT_register_console_function() rt_port_show failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_test", rt_test, 0, 0) ){
        fprintf(stderr,"XACT_register_console_function() rt_test failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_ent_info", rt_ent_info, 1, 1) ){
        fprintf(stderr,"XACT_register_console_function() rt_ent_info failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_ent_show", rt_ent_show, 0, 1) ){
        fprintf(stderr,"XACT_register_console_function() rt_ent_show failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_create_table", rt_create_table, 0, 0) ){
        fprintf(stderr,"XACT_register_console_function() rt_create_table failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_delete_table", rt_delete_table, 0, 1) ){
        fprintf(stderr,"XACT_register_console_function() rt_delete_table failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_add_port", rt_add_port, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() rt_add_port failed\n");
        return(0);
    }
    if (!XACT_register_console_function("rt_remove_port", rt_remove_port, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() rt_remove_port failed\n");
        return(0);
    }
#ifdef STANDALONE
    // from hal_1200/hal_sram


    if (!XACT_register_console_function("SRAM_Write", SRAM_Write, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() SRAM_Write failed\n");
        return(0);
    }
    if (!XACT_register_console_function("SRAM_Read", SRAM_Read, 0, 1) ){
        fprintf(stderr,"XACT_register_console_function() SRAM_Read failed\n");
        return(0);
    }
    // from hal_1200/hal_sdram
    if (!XACT_register_console_function("SDRAM_Write", SDRAM_Write, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() SDRAM_Write failed\n");
        return(0);
    }
    if (!XACT_register_console_function("SDRAM_Read", SDRAM_Read, 0, 1) ){
        fprintf(stderr,"XACT_register_console_function() SDRAM_Read failed\n");
        return(0);
    }
    if (!XACT_register_console_function("IO_Write", IO_Write, 0, 2) ){
        fprintf(stderr,"XACT_register_console_function() IO_Write failed\n");
        return(0);
    }
    if (!XACT_register_console_function("IO_Read", IO_Read, 0, 1) ){
        fprintf(stderr,"XACT_register_console_function() IO_Read failed\n");
        return(0);
    }

    pkgen_model_initialize();
    fdrive_model_initialize();
    fmon_model_initialize();

#endif  // STANDALONE
    return(1);
}


/* this routine will be called prior to each transactor simulation event */
/* It can be used to deposit state values into the transactor model prior */
/* to simulating the next event.  Returning 0 results in an error. */
#ifdef STANDALONE 
int foreign_model_pre_sim()
{
    return(1);
}

/* this routine will be called subsequent to each transactor simulation event. */
/* It can be used to query transactor simulation state, in order to copy it */
/* into the foreign model simulator. */
int foreign_model_post_sim()
{
    return(1);
}

/* this routine will be called just prior to exiting the simulator. */
/* The routine allows the foreign model to clean up, close files, etc */
/* before shutting down the program. */
int foreign_model_exit()
{
    return(1);
}

/* this routine will be called just after reseting the simulator. */
/* The routine allows the foreign model to reset itself to stay */
/* in sync with the simulator. */
int foreign_model_reset()
{
    AMBAIO_ResetAllChips();
    return(1);
}


#if defined(BUILD_DLL)
/*-----------------------------------------------------------------
    GetForeignModelFunctions

    This function is exported as the sole entry point into the DLL version
    of this package.  The Developer Workbench calls it in order to get the
    foreign model entry points for the transactor.  The Workbench thens
    registers these entry points with the transactor.
    
     returns: 
        uses: 
    modifies: 
*/
#if defined(__cplusplus)
extern "C"
#endif
__declspec(dllexport) void __cdecl
GetForeignModelFunctions(
    int (**ForeignModelInitialize)(),
    int (**ForeignModelPreSim)(),
    int (**ForeignModelPostSim)(),
    int (**ForeignModelExit)(),
    int (**ForeignModelReset)())
{
    *ForeignModelInitialize = foreign_model_initialize;
    *ForeignModelPreSim = foreign_model_pre_sim;
    *ForeignModelPostSim = foreign_model_post_sim;
    *ForeignModelExit = foreign_model_exit;
    *ForeignModelReset = foreign_model_reset;
}
#endif      // #if defined(BUILD_DLL)
#endif          // STANDALONE
#endif          // IOSTYLE != HARDWARE
