/* ---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *----------------------------------------------------------------------------*/

//////////////////////////////////////////////////////////////////////////////
// $Source: /mnt/bigpine/src/oem/IXP1200/CoreLibs/rtm-mvr/Attic/rtm_test.cpp,v $
// $Revision: 2 $
// $Date: 5/02/00 3:35p $
//
// Description:
//      This file implements some testing functions for the route table code.
//////////////////////////////////////////////////////////////////////////////

#include "TrieManager.h"
#include "rtm_mvr.h"
#include "rtm_cfg.h"
#include "stdio.h"

typedef struct {
    char*        dest;
    char*        mask;
    unsigned int port;
    HROUTETABLE  table;
} rt_test_route;

typedef struct {
    char*        dest;
    int          inport;
    int          expected;
} rt_test_lookup;

class rt_test_exception {
public:
    rt_test_exception(char* msg) {m_message = msg;}
    char* m_message;
};

void rt_test_add_routes(rt_test_route* routes, 
                        unsigned int numRoutes) {
    rt_test_route* pRoute = NULL;

    for(unsigned int i = 0; i < numRoutes; i++) {
        pRoute = routes + i;

        if(1 != rt_add(pRoute->dest, pRoute->mask, pRoute->port, pRoute->table)) {
            char message[128];
            sprintf(message, "Adding route failed!  dest = %s mask = %s\n",
                    pRoute->dest, pRoute->mask);
            throw rt_test_exception(message);
        }
    }
}

void rt_test_delete_routes(rt_test_route* routes, 
                           unsigned int numRoutes) {
    rt_test_route* pRoute = NULL;

    for(unsigned int i = 0; i < numRoutes; i++) {
        pRoute = routes + i;

        if(1 != rt_delete(pRoute->dest, pRoute->mask, pRoute->table)) {
            char message[128];
            sprintf(message, "Deleting route failed!  dest = %s mask = %s\n",
                    pRoute->dest, pRoute->mask);
            throw rt_test_exception(message);
        }
    }
}

void rt_test_lookup_routes(rt_test_lookup* lookups, unsigned int numRoutes)
{
    rt_test_lookup* pLookup = NULL;

    for(unsigned int i = 0; i < numRoutes; i++) {
        pLookup = lookups + i;

        if(pLookup->expected != rt_lookup(pLookup->dest, pLookup->inport)) {
            char message[128];
            sprintf(message, "Route lookup failed!  dest = %s, inport = %d\n", 
                    pLookup->dest, pLookup->inport);
            throw rt_test_exception(message);
        }
    }
}

int rt_test_one_table() {

    try {
        // First we check and see how many free tries and free route entries
        // there are so we can know if we leak.
        unsigned int freeTries = g_pTrieManager->GetNumFreeTries();
        unsigned int freeRoutes = RTM_config.num_free_routes;

        printf("Creating one route table...\n");
        
        HROUTETABLE table = rt_create_table();
        if(!table) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the route table...\n");

        if(1 != rt_add_port(table, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding 32 routes...\n");

        rt_test_route routes[] = {
            { "24.128.0.0", "255.224.0.0", 11, table },
            { "24.135.116.0", "255.255.252.0", 22, table },
            { "24.128.0.0", "255.248.0.0", 13, table },
            { "24.132.0.0", "255.252.0.0", 14, table },
            { "0.0.0.0", "128.0.0.0", 1, table },
            { "24.135.96.0", "255.255.224.0", 19, table },
            { "24.135.112.0", "255.255.240.0", 20, table },
            { "0.0.0.0", "224.0.0.0", 3, table },
            { "24.135.118.32", "255.255.255.224", 27, table },
            { "24.135.118.48", "255.255.255.240", 28, table },
            { "24.135.118.0", "255.255.254.0", 23, table },
            { "24.135.118.0", "255.255.255.0", 24, table },
            { "24.135.118.0", "255.255.255.128", 25, table },
            { "0.0.0.0", "192.0.0.0", 2, table },
            { "24.134.0.0", "255.254.0.0", 15, table },
            { "24.135.64.0", "255.255.192.0", 18, table },
            { "24.135.0.0", "255.255.128.0", 17, table },
            { "24.128.0.0", "255.240.0.0", 12, table },
            { "24.135.112.0", "255.255.248.0", 21, table },
            { "24.135.118.50", "255.255.255.255", 32, table },
            { "16.0.0.0", "240.0.0.0", 4, table },
            { "24.0.0.0", "255.0.0.0", 8, table },
            { "24.135.118.0", "255.255.255.192", 26, table },
            { "24.135.0.0", "255.255.0.0", 16, table },
            { "24.0.0.0", "252.0.0.0", 6, table },
            { "24.0.0.0", "254.0.0.0", 7, table },
            { "24.135.118.48", "255.255.255.252", 30, table },
            { "24.135.118.50", "255.255.255.254", 31, table },
            { "24.128.0.0", "255.128.0.0", 9, table },
            { "24.128.0.0", "255.192.0.0", 10, table },
            { "24.135.118.48", "255.255.255.248", 29, table },
            { "24.0.0.0", "248.0.0.0", 5, table },
        };

        rt_test_add_routes(routes, 32);

        printf("Performing lookups...\n");

        rt_test_lookup lookups[] = {
            { "128.0.0.0", 0, -1 },
            { "64.0.0.0", 0, 1 },
            { "32.0.0.0", 0, 2 },
            { "0.0.0.0", 0, 3 },
            { "16.0.0.0", 0, 4 },
            { "28.0.0.0", 0, 5 },
            { "26.0.0.0", 0, 6 },
            { "25.0.0.0", 0, 7 },
            { "24.0.0.0", 0, 8 },
            { "24.192.0.0", 0, 9 },
            { "24.160.0.0", 0, 10 },
            { "24.144.0.0", 0, 11 },
            { "24.136.0.0", 0, 12 },
            { "24.128.0.0", 0, 13 },
            { "24.132.0.0", 0, 14 },
            { "24.134.0.0", 0, 15 },
            { "24.135.128.0", 0, 16 },
            { "24.135.0.0", 0, 17 },
            { "24.135.64.0", 0, 18 },
            { "24.135.96.0", 0, 19 },
            { "24.135.120.0", 0, 20 },
            { "24.135.112.0", 0, 21 },
            { "24.135.116.0", 0, 22 },
            { "24.135.119.0", 0, 23 },
            { "24.135.118.128", 0, 24 },
            { "24.135.118.64", 0, 25 },
            { "24.135.118.0", 0, 26 },
            { "24.135.118.32", 0, 27 },
            { "24.135.118.56", 0, 28 },
            { "24.135.118.52", 0, 29 },
            { "24.135.118.48", 0, 30 },
            { "24.135.118.51", 0, 31 },
            { "24.135.118.50", 0, 32 },
        };

        rt_test_lookup_routes(lookups, 33);

        printf("Deleting some routes...\n");

        rt_test_route deletes[] = {
            { "24.135.118.48", "255.255.255.248", 29, table },
            { "24.0.0.0", "248.0.0.0", 5, table },
            { "24.135.0.0", "255.255.128.0", 17, table },
            { "0.0.0.0", "128.0.0.0", 1, table },
            { "24.135.118.50", "255.255.255.255", 32, table },
            { "16.0.0.0", "240.0.0.0", 4, table },
            { "24.128.0.0", "255.192.0.0", 10, table },
            { "24.135.118.0", "255.255.255.128", 25, table },
            { "0.0.0.0", "192.0.0.0", 2, table },
            { "24.135.112.0", "255.255.248.0", 21, table },
            { "24.135.116.0", "255.255.252.0", 22, table },
            { "24.128.0.0", "255.248.0.0", 13, table },
            { "24.132.0.0", "255.252.0.0", 14, table },
            { "24.134.0.0", "255.254.0.0", 15, table },
            { "24.135.64.0", "255.255.192.0", 18, table },
            { "24.128.0.0", "255.224.0.0", 11, table },
            { "24.128.0.0", "255.240.0.0", 12, table },
            { "24.135.118.0", "255.255.255.0", 24, table },
        };

        rt_test_delete_routes(deletes, 18);

        printf("Performing lookups...\n");

        rt_test_lookup lookups2[] = {
            { "128.0.0.0", 0, -1 },
            { "64.0.0.0", 0, -1 },
            { "32.0.0.0", 0, -1 },
            { "0.0.0.0", 0, 3 },
            { "16.0.0.0", 0, 3 },
            { "28.0.0.0", 0, 3 },
            { "26.0.0.0", 0, 6 },
            { "25.0.0.0", 0, 7 },
            { "24.0.0.0", 0, 8 },
            { "24.192.0.0", 0, 9 },
            { "24.160.0.0", 0, 9 },
            { "24.144.0.0", 0, 9 },
            { "24.136.0.0", 0, 9 },
            { "24.128.0.0", 0, 9 },
            { "24.132.0.0", 0, 9 },
            { "24.134.0.0", 0, 9 },
            { "24.135.128.0", 0, 16 },
            { "24.135.0.0", 0, 16 },
            { "24.135.64.0", 0, 16 },
            { "24.135.96.0", 0, 19 },
            { "24.135.120.0", 0, 20 },
            { "24.135.112.0", 0, 20 },
            { "24.135.116.0", 0, 20 },
            { "24.135.119.0", 0, 23 },
            { "24.135.118.128", 0, 23 },
            { "24.135.118.64", 0, 23 },
            { "24.135.118.0", 0, 26 },
            { "24.135.118.32", 0, 27 },
            { "24.135.118.56", 0, 28 },
            { "24.135.118.52", 0, 28 },
            { "24.135.118.48", 0, 30 },
            { "24.135.118.51", 0, 31 },
            { "24.135.118.50", 0, 31 },
        };

        rt_test_lookup_routes(lookups2, 33);

        printf("Deleting remaining routes...\n");

        rt_test_route deletes2[] = {
            { "24.135.118.0", "255.255.255.192", 26, table },
            { "24.0.0.0", "255.0.0.0", 8, table },
            { "24.135.118.32", "255.255.255.224", 27, table },
            { "24.0.0.0", "254.0.0.0", 7, table },
            { "24.135.0.0", "255.255.0.0", 16, table },
            { "24.135.118.0", "255.255.254.0", 23, table },
            { "24.135.118.48", "255.255.255.252", 30, table },
            { "24.0.0.0", "252.0.0.0", 6, table },
            { "24.135.112.0", "255.255.240.0", 20, table },
            { "0.0.0.0", "224.0.0.0", 3, table },
            { "24.135.118.50", "255.255.255.254", 31, table },
            { "24.128.0.0", "255.128.0.0", 9, table },
            { "24.135.118.48", "255.255.255.240", 28, table },
            { "24.135.96.0", "255.255.224.0", 19, table },
        };

        rt_test_delete_routes(deletes2, 14);

        printf("Performing lookups...\n");

        rt_test_lookup lookups3[] = {
            { "128.0.0.0", 0, -1 },
            { "64.0.0.0", 0, -1 },
            { "32.0.0.0", 0, -1 },
            { "0.0.0.0", 0, -1 },
            { "16.0.0.0", 0, -1 },
            { "28.0.0.0", 0, -1 },
            { "26.0.0.0", 0, -1 },
            { "25.0.0.0", 0, -1 },
            { "24.0.0.0", 0, -1 },
            { "24.192.0.0", 0, -1 },
            { "24.160.0.0", 0, -1 },
            { "24.144.0.0", 0, -1 },
            { "24.136.0.0", 0, -1 },
            { "24.128.0.0", 0, -1 },
            { "24.132.0.0", 0, -1 },
            { "24.134.0.0", 0, -1 },
            { "24.135.128.0", 0, -1 },
            { "24.135.0.0", 0, -1 },
            { "24.135.64.0", 0, -1 },
            { "24.135.96.0", 0, -1 },
            { "24.135.120.0", 0, -1 },
            { "24.135.112.0", 0, -1 },
            { "24.135.116.0", 0, -1 },
            { "24.135.119.0", 0, -1 },
            { "24.135.118.128", 0, -1 },
            { "24.135.118.64", 0, -1 },
            { "24.135.118.0", 0, -1 },
            { "24.135.118.32", 0, -1 },
            { "24.135.118.56", 0, -1 },
            { "24.135.118.52", 0, -1 },
            { "24.135.118.48", 0, -1 },
            { "24.135.118.51", 0, -1 },
            { "24.135.118.50", 0, -1 },
        };

        rt_test_lookup_routes(lookups3, 33);

        printf("Deleting route table...\n");

        if(1 != rt_delete_table(table)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Test successful!!\n");

    }
    catch (rt_test_exception e) {
        printf(e.m_message);
        return 0;
    }
    catch(...) {
        printf("Unhandled exception!!\n");
        return 0;
    }

    return 1;
}

int rt_test_mvr1() {

    try {
        // First we check and see how many free tries and free route entries
        // there are so we can know if we leak.
        unsigned int freeTries = g_pTrieManager->GetNumFreeTries();
        unsigned int freeRoutes = RTM_config.num_free_routes;

        printf("Creating first route table...\n");
        
        HROUTETABLE table1 = rt_create_table();
        if(!table1) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating second route table...\n");
        
        HROUTETABLE table2 = rt_create_table();
        if(!table2) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the first route table...\n");

        if(1 != rt_add_port(table1, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding port 1 to the first route table...\n");

        if(1 != rt_add_port(table1, 1)) {
            throw rt_test_exception("Could not add port 1 to the route "
                                    "table!!\n");
        }

        printf("Adding port 2 to the second route table...\n");

        if(1 != rt_add_port(table2, 2)) {
            throw rt_test_exception("Could not add port 2 to the route "
                                    "table!!\n");
        }

        printf("Adding port 3 to the second route table...\n");

        if(1 != rt_add_port(table2, 3)) {
            throw rt_test_exception("Could not add port 3 to the route "
                                    "table!!\n");
        }

        printf("Adding 6 routes...\n");

        rt_test_route routes[] = {
            { "10.0.1.0", "255.255.255.0", 0, table1 },
            { "10.0.11.0", "255.255.255.0", 1, table1 },
            { "10.0.2.0", "255.255.255.0", 2, table2 },
            { "10.0.11.0", "255.255.255.0", 3, table2 },
            { "10.0.1.0", "255.255.255.0", 3, table2 },
            { "10.0.2.0", "255.255.255.0", 1, table1 },
        };

        rt_test_add_routes(routes, 6);

        printf("Performing lookups...\n");

        rt_test_lookup lookups[] = {
            { "10.0.1.10", 1, 0 },
            { "10.0.1.10", 2, 3 },
            { "10.0.2.10", 0, 1 },
            { "10.0.2.10", 3, 2 },
            { "10.0.11.1", 2, 3 },
            { "10.0.11.2", 0, 1 },
            { "10.0.11.2", 4, -1 },
        };

        rt_test_lookup_routes(lookups, 7);

        printf("Deleting routes...\n");

        rt_test_delete_routes(routes, 6);

        printf("Performing lookups...\n");

        rt_test_lookup lookups1[] = {
            { "10.0.1.10", 1, -1 },
            { "10.0.1.10", 2, -1 },
            { "10.0.2.10", 0, -1 },
            { "10.0.2.10", 3, -1 },
            { "10.0.11.1", 2, -1 },
            { "10.0.11.2", 0, -1 },
            { "10.0.11.2", 4, -1 },
        };

        rt_test_lookup_routes(lookups1, 6);

        printf("Deleting first route table...\n");

        if(1 != rt_delete_table(table1)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting second route table...\n");

        if(1 != rt_delete_table(table2)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Test successful!!\n");

    }
    catch (rt_test_exception e) {
        printf(e.m_message);
        return 0;
    }
    catch(...) {
        printf("Unhandled exception!!\n");
        return 0;
    }

    return 1;
}

int rt_test_mvr2() {

    try {
        // First we check and see how many free tries and free route entries
        // there are so we can know if we leak.
        unsigned int freeTries = g_pTrieManager->GetNumFreeTries();
        unsigned int freeRoutes = RTM_config.num_free_routes;

        printf("Creating first route table...\n");
        
        HROUTETABLE table1 = rt_create_table();
        if(!table1) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating second route table...\n");
        
        HROUTETABLE table2 = rt_create_table();
        if(!table2) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the first route table...\n");

        if(1 != rt_add_port(table1, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding port 1 to the first route table...\n");

        if(1 != rt_add_port(table1, 1)) {
            throw rt_test_exception("Could not add port 1 to the route "
                                    "table!!\n");
        }

        printf("Adding port 2 to the second route table...\n");

        if(1 != rt_add_port(table2, 2)) {
            throw rt_test_exception("Could not add port 2 to the route "
                                    "table!!\n");
        }

        printf("Adding port 3 to the second route table...\n");

        if(1 != rt_add_port(table2, 3)) {
            throw rt_test_exception("Could not add port 3 to the route "
                                    "table!!\n");
        }

        printf("Adding 8 routes...\n");

        rt_test_route routes[] = {
            { "10.0.1.0", "255.255.255.0", 0, table1 },
            { "10.0.11.0", "255.255.255.0", 1, table1 },
            { "10.0.2.0", "255.255.255.0", 2, table2 },
            { "10.0.12.0", "255.255.255.0", 3, table2 },
            { "10.0.1.0", "255.255.255.0", 3, table2 },
            { "10.0.11.0", "255.255.255.0", 3, table2 },
            { "10.0.2.0", "255.255.255.0", 1, table1 },
            { "10.0.12.0", "255.255.255.0", 1, table1 },
        };

        rt_test_add_routes(routes, 8);

        printf("Performing lookups...\n");

        rt_test_lookup lookups[] = {
            { "10.0.1.10", 1, 0 },
            { "10.0.1.10", 2, 3 },
            { "10.0.2.10", 0, 1 },
            { "10.0.2.10", 3, 2 },
            { "10.0.11.1", 2, 3 },
            { "10.0.12.1", 0, 1 },
            { "10.0.11.2", 0, 1 },
            { "10.0.12.2", 2, 3 },
            { "10.0.12.2", 4, -1 },
        };

        rt_test_lookup_routes(lookups, 9);

        printf("Deleting routes...\n");

        rt_test_delete_routes(routes, 8);

        printf("Performing lookups...\n");

        rt_test_lookup lookups1[] = {
            { "10.0.1.10", 1, -1 },
            { "10.0.1.10", 2, -1 },
            { "10.0.2.10", 0, -1 },
            { "10.0.2.10", 3, -1 },
            { "10.0.11.1", 2, -1 },
            { "10.0.12.1", 0, -1 },
            { "10.0.11.2", 0, -1 },
            { "10.0.12.2", 2, -1 },
            { "10.0.12.2", 4, -1 },
        };

        rt_test_lookup_routes(lookups1, 6);

        printf("Deleting first route table...\n");

        if(1 != rt_delete_table(table1)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting second route table...\n");

        if(1 != rt_delete_table(table2)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Test successful!!\n");

    }
    catch (rt_test_exception e) {
        printf(e.m_message);
        return 0;
    }
    catch(...) {
        printf("Unhandled exception!!\n");
        return 0;
    }

    return 1;
}

int rt_test_mvr3() {

    try {
        // First we check and see how many free tries and free route entries
        // there are so we can know if we leak.
        unsigned int freeTries = g_pTrieManager->GetNumFreeTries();
        unsigned int freeRoutes = RTM_config.num_free_routes;

        printf("Creating first route table...\n");
        
        HROUTETABLE table1 = rt_create_table();
        if(!table1) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating second route table...\n");
        
        HROUTETABLE table2 = rt_create_table();
        if(!table2) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating third route table...\n");
        
        HROUTETABLE table3 = rt_create_table();
        if(!table3) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating fourth route table...\n");
        
        HROUTETABLE table4 = rt_create_table();
        if(!table4) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the first route table...\n");

        if(1 != rt_add_port(table1, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding port 1 to the first route table...\n");

        if(1 != rt_add_port(table1, 1)) {
            throw rt_test_exception("Could not add port 1 to the route "
                                    "table!!\n");
        }

        printf("Adding port 2 to the second route table...\n");

        if(1 != rt_add_port(table2, 2)) {
            throw rt_test_exception("Could not add port 2 to the route "
                                    "table!!\n");
        }

        printf("Adding port 3 to the second route table...\n");

        if(1 != rt_add_port(table2, 3)) {
            throw rt_test_exception("Could not add port 3 to the route "
                                    "table!!\n");
        }

        printf("Adding port 7 to the third route table...\n");

        if(1 != rt_add_port(table3, 7)) {
            throw rt_test_exception("Could not add port 4 to the route "
                                    "table!!\n");
        }

        printf("Adding port 6 to the third route table...\n");

        if(1 != rt_add_port(table3, 6)) {
            throw rt_test_exception("Could not add port 5 to the route "
                                    "table!!\n");
        }

        printf("Adding port 5 to the fourth route table...\n");

        if(1 != rt_add_port(table4, 5)) {
            throw rt_test_exception("Could not add port 6 to the route "
                                    "table!!\n");
        }

        printf("Adding port 4 to the fourth route table...\n");

        if(1 != rt_add_port(table4, 4)) {
            throw rt_test_exception("Could not add port 7 to the route "
                                    "table!!\n");
        }

        printf("Adding 12 routes...\n");

        rt_test_route routes[] = {
            { "10.0.10.0", "255.255.255.0", 0, table1 },
            { "10.0.1.0", "255.255.255.0", 1, table1 },
            { "10.0.11.0", "255.255.255.0", 1, table1 },
            { "10.0.10.0", "255.255.255.0", 2, table2 },
            { "10.0.1.0", "255.255.255.0", 2, table2 },
            { "10.0.11.0", "255.255.255.0", 3, table2 },
            { "10.0.1.0", "255.255.255.0", 7, table3 },
            { "10.0.10.0", "255.255.255.0", 6, table3 },
            { "10.0.11.0", "255.255.255.0", 6, table3 },
            { "10.0.1.0", "255.255.255.0", 5, table4 },
            { "10.0.10.0", "255.255.255.0", 5, table4 },
            { "10.0.11.0", "255.255.255.0", 4, table4 },
        };

        rt_test_add_routes(routes, 12);

        printf("Performing lookups...\n");

        rt_test_lookup lookups[] = {
            { "10.0.11.2", 0, 1 },
            { "10.0.11.2", 2, 3 },
            { "10.0.10.2", 1, 0 },
            { "10.0.10.2", 3, 2 },
            { "10.0.1.2", 0, 1 },
            { "10.0.1.1", 3, 2 },
            { "10.0.1.2", 6, 7 },
            { "10.0.1.2", 4, 5 },
            { "10.0.11.2", 5, 4 },
            { "10.0.11.2", 7, 6 },
            { "10.0.10.2", 7, 6 },
            { "10.0.10.1", 4, 5 },
        };

        rt_test_lookup_routes(lookups, 12);

        printf("Deleting routes...\n");

        rt_test_delete_routes(routes, 12);

        printf("Performing lookups...\n");

        rt_test_lookup lookups1[] = {
            { "10.0.11.2", 0, -1 },
            { "10.0.11.2", 2, -1 },
            { "10.0.10.2", 1, -1 },
            { "10.0.10.2", 3, -1 },
            { "10.0.1.2", 0, -1 },
            { "10.0.1.1", 3, -1 },
            { "10.0.1.2", 6, -1 },
            { "10.0.1.2", 4, -1 },
            { "10.0.11.2", 5, -1 },
            { "10.0.11.2", 7, -1 },
            { "10.0.10.2", 7, -1 },
            { "10.0.10.1", 4, -1 },
        };

        rt_test_lookup_routes(lookups1, 12);

        printf("Deleting first route table...\n");

        if(1 != rt_delete_table(table1)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting second route table...\n");

        if(1 != rt_delete_table(table2)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting third route table...\n");

        if(1 != rt_delete_table(table3)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting fourth route table...\n");

        if(1 != rt_delete_table(table4)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Test successful!!\n");

    }
    catch (rt_test_exception e) {
        printf(e.m_message);
        return 0;
    }
    catch(...) {
        printf("Unhandled exception!!\n");
        return 0;
    }

    return 1;
}

int rt_test_mvr4() {

    try {
        // First we check and see how many free tries and free route entries
        // there are so we can know if we leak.
        unsigned int freeTries = g_pTrieManager->GetNumFreeTries();
        unsigned int freeRoutes = RTM_config.num_free_routes;

        printf("Creating first route table...\n");
        
        HROUTETABLE table1 = rt_create_table();
        if(!table1) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating second route table...\n");
        
        HROUTETABLE table2 = rt_create_table();
        if(!table2) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating third route table...\n");
        
        HROUTETABLE table3 = rt_create_table();
        if(!table3) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating fourth route table...\n");
        
        HROUTETABLE table4 = rt_create_table();
        if(!table4) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the first route table...\n");

        if(1 != rt_add_port(table1, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding port 1 to the first route table...\n");

        if(1 != rt_add_port(table1, 1)) {
            throw rt_test_exception("Could not add port 1 to the route "
                                    "table!!\n");
        }

        printf("Adding port 2 to the second route table...\n");

        if(1 != rt_add_port(table2, 2)) {
            throw rt_test_exception("Could not add port 2 to the route "
                                    "table!!\n");
        }

        printf("Adding port 3 to the second route table...\n");

        if(1 != rt_add_port(table2, 3)) {
            throw rt_test_exception("Could not add port 3 to the route "
                                    "table!!\n");
        }

        printf("Adding port 7 to the third route table...\n");

        if(1 != rt_add_port(table3, 7)) {
            throw rt_test_exception("Could not add port 7 to the route "
                                    "table!!\n");
        }

        printf("Adding port 6 to the third route table...\n");

        if(1 != rt_add_port(table3, 6)) {
            throw rt_test_exception("Could not add port 6 to the route "
                                    "table!!\n");
        }

        printf("Adding port 5 to the fourth route table...\n");

        if(1 != rt_add_port(table4, 5)) {
            throw rt_test_exception("Could not add port 5 to the route "
                                    "table!!\n");
        }

        printf("Adding port 4 to the fourth route table...\n");

        if(1 != rt_add_port(table4, 4)) {
            throw rt_test_exception("Could not add port 4 to the route "
                                    "table!!\n");
        }

        printf("Adding 12 routes...\n");

        rt_test_route routes[] = {
            { "10.0.2.0", "255.255.255.0", 1, table1 },
            { "10.0.1.0", "255.255.255.0", 0, table1 },
            { "10.0.10.0", "255.255.255.0", 1, table1 },
            { "10.0.2.0", "255.255.255.0", 3, table2 },
            { "10.0.1.0", "255.255.255.0", 2, table2 },
            { "10.0.10.0", "255.255.255.0", 2, table2 },
            { "10.0.1.0", "255.255.255.0", 7, table3 },
            { "10.0.10.0", "255.255.255.0", 6, table3 },
            { "10.0.2.0", "255.255.255.0", 6, table3 },
            { "10.0.1.0", "255.255.255.0", 5, table4 },
            { "10.0.10.0", "255.255.255.0", 5, table4 },
            { "10.0.2.0", "255.255.255.0", 4, table4 },
        };

        rt_test_add_routes(routes, 12);

        printf("Performing lookups...\n");

        rt_test_lookup lookups[] = {
            { "10.0.1.10", 1, 0 },
            { "10.0.1.10", 3, 2 },
            { "10.0.2.10", 0, 1 },
            { "10.0.2.10", 2, 3 },
            { "10.0.10.2", 0, 1 },
            { "10.0.10.1", 3, 2 },
            { "10.0.1.10", 6, 7 },
            { "10.0.1.10", 4, 5 },
            { "10.0.2.10", 5, 4 },
            { "10.0.2.10", 7, 6 },
            { "10.0.10.3", 7, 6 },
            { "10.0.10.3", 4, 5 },
            { "10.0.2.1", 5, 4 },
            { "10.0.2.1", 7, 6 },
            { "10.0.10.1", 4, 5 },
            { "10.0.10.2", 7, 6 },
        };

        rt_test_lookup_routes(lookups, 16);

        printf("Deleting routes...\n");

        rt_test_delete_routes(routes, 12);

        printf("Performing lookups...\n");

        rt_test_lookup lookups1[] = {
            { "10.0.1.10", 1, -1 },
            { "10.0.1.10", 3, -1 },
            { "10.0.2.10", 0, -1 },
            { "10.0.2.10", 2, -1 },
            { "10.0.10.2", 0, -1 },
            { "10.0.10.1", 3, -1 },
            { "10.0.1.10", 6, -1 },
            { "10.0.1.10", 4, -1 },
            { "10.0.2.10", 5, -1 },
            { "10.0.2.10", 7, -1 },
            { "10.0.10.3", 7, -1 },
            { "10.0.10.3", 4, -1 },
            { "10.0.2.1", 5, -1 },
            { "10.0.2.1", 6, -1 },
            { "10.0.10.1", 4, -1 },
            { "10.0.10.2", 7, -1 },
        };

        rt_test_lookup_routes(lookups1, 16);

        printf("Deleting first route table...\n");

        if(1 != rt_delete_table(table1)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting second route table...\n");

        if(1 != rt_delete_table(table2)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting third route table...\n");

        if(1 != rt_delete_table(table3)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting fourth route table...\n");

        if(1 != rt_delete_table(table4)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Test successful!!\n");

    }
    catch (rt_test_exception e) {
        printf(e.m_message);
        return 0;
    }
    catch(...) {
        printf("Unhandled exception!!\n");
        return 0;
    }

    return 1;
}

int rt_test_mvr5() {

    try {
        // First we check and see how many free tries and free route entries
        // there are so we can know if we leak.
        unsigned int freeTries = g_pTrieManager->GetNumFreeTries();
        unsigned int freeRoutes = RTM_config.num_free_routes;

        printf("Creating first route table...\n");
        
        HROUTETABLE table1 = rt_create_table();
        if(!table1) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating second route table...\n");
        
        HROUTETABLE table2 = rt_create_table();
        if(!table2) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Creating third route table...\n");
        
        HROUTETABLE table3 = rt_create_table();
        if(!table3) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the first route table...\n");

        if(1 != rt_add_port(table1, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding port 1 to the first route table...\n");

        if(1 != rt_add_port(table1, 1)) {
            throw rt_test_exception("Could not add port 1 to the route "
                                    "table!!\n");
        }

        printf("Adding port 2 to the second route table...\n");

        if(1 != rt_add_port(table2, 2)) {
            throw rt_test_exception("Could not add port 2 to the route "
                                    "table!!\n");
        }

        printf("Adding port 3 to the second route table...\n");

        if(1 != rt_add_port(table2, 3)) {
            throw rt_test_exception("Could not add port 3 to the route "
                                    "table!!\n");
        }

        printf("Adding port 4 to the third route table...\n");

        if(1 != rt_add_port(table3, 4)) {
            throw rt_test_exception("Could not add port 4 to the route "
                                    "table!!\n");
        }

        printf("Adding port 5 to the third route table...\n");

        if(1 != rt_add_port(table3, 5)) {
            throw rt_test_exception("Could not add port 5 to the route "
                                    "table!!\n");
        }

        printf("Adding 12 routes...\n");

        rt_test_route routes[] = {
            { "10.0.1.0", "255.255.255.0", 0, table1 },
            { "10.0.10.0", "255.255.255.0", 1, table1 },
            { "10.0.11.0", "255.255.255.0", 1, table1 },
            { "10.0.2.0", "255.255.255.0", 1, table1 },
            { "10.0.1.0", "255.255.255.0", 2, table2 },
            { "10.0.10.0", "255.255.255.0", 2, table2 },
            { "10.0.11.0", "255.255.255.0", 3, table2 },
            { "10.0.2.0", "255.255.255.0", 3, table2 },
            { "10.0.1.0", "255.255.255.0", 4, table3 },
            { "10.0.10.0", "255.255.255.0", 4, table3 },
            { "10.0.11.0", "255.255.255.0", 4, table3 },
            { "10.0.2.0", "255.255.255.0", 5, table3 },
        };

        rt_test_add_routes(routes, 12);

        printf("Performing lookups...\n");

        rt_test_lookup lookups[] = {
            { "10.0.10.3", 0, 1 },
            { "10.0.11.2", 0, 1 },
            { "10.0.2.10", 0, 1 },
            { "10.0.1.10", 1, 0 },
            { "10.0.10.3", 3, 2 },
            { "10.0.11.2", 2, 3 },
            { "10.0.2.10", 2, 3 },
            { "10.0.1.10", 3, 2 },
            { "10.0.10.3", 5, 4 },
            { "10.0.11.2", 5, 4 },
            { "10.0.2.10", 4, 5 },
            { "10.0.1.10", 5, 4 },
            { "10.0.1.10", 6, -1 },
        };

        rt_test_lookup_routes(lookups, 13);

        printf("Deleting routes...\n");

        rt_test_delete_routes(routes, 12);

        printf("Performing lookups...\n");

        rt_test_lookup lookups1[] = {
            { "10.0.10.3", 0, -1 },
            { "10.0.11.2", 0, -1 },
            { "10.0.2.10", 0, -1 },
            { "10.0.1.10", 1, -1 },
            { "10.0.10.3", 3, -1 },
            { "10.0.11.2", 2, -1 },
            { "10.0.2.10", 2, -1 },
            { "10.0.1.10", 3, -1 },
            { "10.0.10.3", 5, -1 },
            { "10.0.11.2", 5, -1 },
            { "10.0.2.10", 4, -1 },
            { "10.0.1.10", 5, -1 },
            { "10.0.1.10", 6, -1 },
        };

        rt_test_lookup_routes(lookups1, 13);

        printf("Deleting first route table...\n");

        if(1 != rt_delete_table(table1)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting second route table...\n");

        if(1 != rt_delete_table(table2)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Deleting third route table...\n");

        if(1 != rt_delete_table(table3)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Test successful!!\n");

    }
    catch (rt_test_exception e) {
        printf(e.m_message);
        return 0;
    }
    catch(...) {
        printf("Unhandled exception!!\n");
        return 0;
    }

    return 1;
}

int rt_test_leak() {

    try {
        // First we check and see how many free tries and free route entries
        // there are so we can know if we leak.
        unsigned int freeTries = g_pTrieManager->GetNumFreeTries();
        unsigned int freeRoutes = RTM_config.num_free_routes;

        printf("Creating route table...\n");
        
        HROUTETABLE table = rt_create_table();
        if(!table) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the route table...\n");

        if(1 != rt_add_port(table, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding overlapping routes...\n");

        rt_test_route routes[] = {
            { "10.0.64.0", "255.255.240.0", 0, table },
            { "10.0.64.0", "255.255.240.0", 1, table },
        };

        rt_test_add_routes(routes, 2);

        printf("Performing lookups...\n");

        rt_test_lookup lookups[] = {
            { "10.0.64.3", 0, 1 },
            { "10.0.1.10", 0, -1 },
        };

        rt_test_lookup_routes(lookups, 2);

        printf("Deleting routes...\n");

        rt_test_route deletes[] = {
            { "10.0.64.0", "255.255.242.0", 1, table },
        };

        rt_test_delete_routes(routes, 1);

        printf("Deleting route table...\n");

        if(1 != rt_delete_table(table)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Creating route table...\n");
        
        table = rt_create_table();
        if(!table) {
            throw rt_test_exception("Could not create route table!!\n");
        }

        printf("Adding port 0 to the route table...\n");

        if(1 != rt_add_port(table, 0)) {
            throw rt_test_exception("Could not add port 0 to the route "
                                    "table!!\n");
        }

        printf("Adding overlapping routes...\n");

        rt_test_route routes1[] = {
            { "10.0.64.0", "255.255.240.0", 0, table },
            { "10.0.64.0", "255.255.240.0", 1, table },
        };

        rt_test_add_routes(routes1, 2);

        printf("Deleting route table...\n");

        if(1 != rt_delete_table(table)) {
            throw rt_test_exception("Could not delete route table!!\n");
        }

        printf("Checking for memory leaks...\n");

        if(freeTries != g_pTrieManager->GetNumFreeTries()) {
            throw rt_test_exception("Leaking Tries!!\n");
        }
        if(freeRoutes != RTM_config.num_free_routes) {
            throw rt_test_exception("Leaking Route Entries!!\n");
        }

        printf("Test successful!!\n");
    }
    catch (rt_test_exception e) {
        printf(e.m_message);
        return 0;
    }
    catch(...) {
        printf("Unhandled exception!!\n");
        return 0;
    }

    return 1;
}

int rt_test() {

    struct test {
        int(*testfunc)(void);
        char*   name;
        bool    passed;
    };

    struct test tests[] = {
        { rt_test_one_table, "single route table test", false },
        { rt_test_mvr1, "MVR test 1", false },
        { rt_test_mvr2, "MVR test 2", false },
        { rt_test_mvr3, "MVR test 3", false },
        { rt_test_mvr4, "MVR test 4", false },
        { rt_test_mvr5, "MVR test 5", false },
        { rt_test_leak, "memory leak test", false },
    };

    bool allPassed = true;
    for(int i = 0; i < sizeof(tests) / sizeof(test); i++) {
        printf("********** Performing %s...\n", tests[i].name);
        if(1 == tests[i].testfunc()) {
            tests[i].passed = true;
        }
        else {
            allPassed = false;
        }

        printf("\n");
    }

    if(allPassed) {
        printf("All tests passed!!\n");
    }
    else {
        for(i = 0; i < sizeof(tests) / sizeof(test); i++) {
            if(!tests[i].passed) {
                printf("Test #%d (%s) failed!\n", i, tests[i].name);
            }
        }
    }

    return 1;
}
