/* hashsv.cpp
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: memory service library functions
 * author: Henry Qian 4/4/98
 * revisions:
 * $History: hashsv.cpp $
 * 
 * *****************  Version 13  *****************
 * User: Drjohns1     Date: 2/04/00    Time: 6:31p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 1/27/00    Time: 6:31p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * Fixed for libD changes
 * 
 * *****************  Version 11  *****************
 * User: Drjohns1     Date: 1/14/00    Time: 2:00p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * Assigned destId's to uthreads
 * 
 * *****************  Version 8  *****************
 * User: Dhooper      Date: 6/17/99    Time: 4:50p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * use lo32 instead of lo15 of hsv_Data
 * 
 * *****************  Version 7  *****************
 * User: Dhooper      Date: 6/16/99    Time: 12:22p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * hsv_Data48 and hsv_Data64 are now hsv_Data
 * 
 * *****************  Version 6  *****************
 * User: Hqianx       Date: 4/27/99    Time: 5:49p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * Build for the new tree.
 * 
 * *****************  Version 4  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 4:18p
 * Updated in $/development/sv_libs
 * update COPYRIGHT
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 5/07/98    Time: 11:09a
 * Updated in $/base_level2/sv_libs
 * Add Intel copyright 
 * 
 * *****************  Version 2  *****************
 * User: C-hqian      Date: 5/12/98    Time: 4:22p
 * Updated in $/base_level2/sv_libs
 * Delete LibD handle when library is closed.
 * Converted input/output arg formats when call LibD_CallOut().
 * 
 * *****************  Version 1  *****************
 * User: C-hqian      Date: 4/30/98    Time: 3:27p
 * Created in $/base_level2/sv_libs
 * Hash service library routines.
 * 
 * *****************  Version 1  *****************
 * User: C-hqian      Date: 4/28/98    Time: 1:52p
 * Created in $/base_level2/sv_libs
 * Memory service routine library.
 * 
 */
// #include "uclo.h"
#include <stdio.h>
#include <stdlib.h>
#include <new.h>
#include "libd.h"
#include "svlibs.h"
#include "hashsv.h"

struct HashSvLibHandleExt {       // common to all Lib
    int tid;                // thread id
    SvCallModes flag;    // PEND, QUEUE_ASYNC, or QUEUE_SYNC
    LibD_Callout_T* ldMult48Handle;
    LibD_Callout_T* ldMult64Handle;
    LibD_Callout_T* exec48Handle;
    LibD_Callout_T* exec64Handle;
    void* libdHandle;
};

/*------------------------------------------------------------------------
    MemSv_Init
    return:
  ------------------------------------------------------------------------*/
LibHandle* HashSv_Init(
    void* objHandle,    // a handle of uof file
    int appSrcId        // in multi apps, it identifies a specific app
)
{
    if ((appSrcId < 0) || (appSrcId > 255))
        return((LibHandle*)HASHSV_BAD_HANDLE);

    //
    // library dispatch initialization
    //
    void* libdHandle = LibD_Init(objHandle, appSrcId);
    if (libdHandle == 0)
        return((LibHandle*)HASHSV_BAD_HANDLE);

    //
	// dj: need to be changed -- destId's are now global across devices
	// assign destId 0 thru 23 to uthreads 0 thru 23
    //
	LibD_AssignUThreadId(0, 0, 24);


    //
    // allocate service library handle
    //
    HashSvLibHandleExt* hashSvLibHandleExt = new(HashSvLibHandleExt);
    if (hashSvLibHandleExt == 0)
        return((LibHandle*)HASHSV_BAD_HANDLE);

    //
    // save libdHandle for deletion at the end of this thread
    //
    hashSvLibHandleExt->libdHandle = libdHandle;

    //
    // get () function handle
    //
    hashSvLibHandleExt->ldMult48Handle =
        LibD_RegLocalCallout(libdHandle, "hashsv_ldmult48#");
    if (hashSvLibHandleExt->ldMult48Handle == 0)
        return((LibHandle*)HASHSV_BAD_HANDLE);

    //
    // get () function handle
    //
    hashSvLibHandleExt->ldMult64Handle =
        LibD_RegLocalCallout(libdHandle, "hashsv_ldmult64#");
    if (hashSvLibHandleExt->ldMult64Handle == 0)
        return((LibHandle*)HASHSV_BAD_HANDLE);

    //
    // get () function handle
    //
    hashSvLibHandleExt->exec48Handle = 
        LibD_RegLocalCallout(libdHandle, "hashsv_exec48#");
    if (hashSvLibHandleExt->exec48Handle == 0)
        return((LibHandle*)HASHSV_BAD_HANDLE);

    //
    // get () function handle
    //
    hashSvLibHandleExt->exec64Handle = 
        LibD_RegLocalCallout(libdHandle, "hashsv_exec64#");
    if (hashSvLibHandleExt->exec64Handle == 0)
        return((LibHandle*)HASHSV_BAD_HANDLE);

    return((LibHandle*)hashSvLibHandleExt);
};

/*------------------------------------------------------------------------
    HashSv_LdMult48
        Load the 48 bit multiplier in the hardware hash unit

    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - HASHSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int HashSv_LdMult48(
    LibHandle* libHandle,
    hsv_Data* inArg
)
{
    LibD_Callout_T* ldMult48Handle = 
        ((HashSvLibHandleExt*)libHandle)->ldMult48Handle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (ldMult48Handle == 0)
        return(HASHSV_FAIL);

    //
    // clear transaction ID
    //
    ldMult48Handle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < HASHSV_MAX_THREADS))
        ldMult48Handle->destId = libHandle->tid;
    else
        return(HASHSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            ldMult48Handle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            ldMult48Handle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            ldMult48Handle->sync = ASYNC;
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };

    // dumb parameter
    //
    unsigned int outArg;

    // call micro code function
    //
    int retval = LibD_CallOut(  ldMult48Handle,
                                (void*)inArg,
                                (void*)&outArg);
    if (retval != LIBD_SUCCESS)
            return(HASHSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(ldMult48Handle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(ldMult48Handle, 0);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(ldMult48Handle->txId);
            else
                return(HASHSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(ldMult48Handle->txId);
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };

    return(HASHSV_SUCCESS);
};

/*------------------------------------------------------------------------
    HashSv_LdMult64
        Load the 64 bit multiplier in the hardware hash unit

    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - HASHSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int HashSv_LdMult64(
    LibHandle* libHandle,
    hsv_Data* inArg
)
{
    LibD_Callout_T* ldMult64Handle = 
        ((HashSvLibHandleExt*)libHandle)->ldMult64Handle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (ldMult64Handle == 0)
        return(HASHSV_FAIL);

    //
    // clear transaction ID
    //
    ldMult64Handle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < HASHSV_MAX_THREADS))
        ldMult64Handle->destId = libHandle->tid;
    else
        return(HASHSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            ldMult64Handle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            ldMult64Handle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            ldMult64Handle->sync = ASYNC;
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };

    //
    // call micro code function
    //
    int outArg; // dumb parameter
    int retval = LibD_CallOut(  ldMult64Handle,
                                (void*)inArg,
                                (void*)&outArg);
    if (retval != LIBD_SUCCESS)
            return(HASHSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(ldMult64Handle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(ldMult64Handle, 0);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(ldMult64Handle->txId);
            else
                return(HASHSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(ldMult64Handle->txId);
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };

    return(HASHSV_SUCCESS);
};

/*------------------------------------------------------------------------
    HashSv_LdMult48
        Load the 48 bit multiplier in the hardware hash unit

    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - HASHSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int HashSv_Exec48(
    LibHandle* libHandle,
    hsv_Data* inArg,
    hsv_Data* outArg
)
{
    LibD_Callout_T* exec48Handle = 
        ((HashSvLibHandleExt*)libHandle)->exec48Handle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (exec48Handle == 0)
        return(HASHSV_FAIL);

    //
    // clear transaction ID
    //
    exec48Handle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < HASHSV_MAX_THREADS))
        exec48Handle->destId = libHandle->tid;
    else
        return(HASHSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            exec48Handle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            exec48Handle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            exec48Handle->sync = ASYNC;
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };


    // call micro code function
    //
    int retval = LibD_CallOut(  exec48Handle,
                                (void*)inArg,
                                (void*)outArg);
    if (retval != LIBD_SUCCESS)
            return(HASHSV_FAIL);


    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(exec48Handle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(exec48Handle, (void*)outArg);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(exec48Handle->txId);
            else
                return(HASHSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(exec48Handle->txId);
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };


    return(HASHSV_SUCCESS);
};

/*------------------------------------------------------------------------
    HashSv_LdMult64
        Load the 64 bit multiplier in the hardware hash unit

    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - HASHSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int HashSv_Exec64(
    LibHandle* libHandle,
    hsv_Data* inArg,
    hsv_Data* outArg
)
{
    LibD_Callout_T* exec64Handle = 
        ((HashSvLibHandleExt*)libHandle)->exec64Handle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (exec64Handle == 0)
        return(HASHSV_FAIL);

    //
    // clear transaction ID
    //
    exec64Handle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < HASHSV_MAX_THREADS))
        exec64Handle->destId = libHandle->tid;
    else
        return(HASHSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            exec64Handle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            exec64Handle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            exec64Handle->sync = ASYNC;
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };

    //
    // call micro code function
    //
    int retval = LibD_CallOut(  exec64Handle,
                                (void*)inArg,
                                (void*)outArg);
    if (retval != LIBD_SUCCESS)
            return(HASHSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(exec64Handle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(exec64Handle, (void*)outArg);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(exec64Handle->txId);
            else
                return(HASHSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(exec64Handle->txId);
            break;

        default:
            return(HASHSV_FAIL); // libHandle->flag has a wrong value
    };

    return(HASHSV_SUCCESS);
};

void HashSv_Close(
    LibHandle* libHandle
)
{
    LibD_DeleHandle(((HashSvLibHandleExt*)libHandle)->libdHandle);
    delete((HashSvLibHandleExt*)libHandle);
}

// end of file

