/* memsv.cpp
 *
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 * system: SA1200
 * subsystem: memory service library functions
 * author: Henry Qian 4/4/98
 * revisions:
 * $History: memsv.cpp $
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 1/27/00    Time: 6:31p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * Fixed for libD changes
 * 
 * *****************  Version 11  *****************
 * User: Drjohns1     Date: 1/14/00    Time: 2:00p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * Assigned destId's to uthreads
 * 
 * *****************  Version 9  *****************
 * User: Drjohns1     Date: 12/08/99   Time: 5:46p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * Updated for libD/Shrimp changes
 * 
 * *****************  Version 7  *****************
 * User: Hqianx       Date: 4/27/99    Time: 5:49p
 * Updated in $/dev_1200/SA1_CoreLibs/sv_libs
 * Build for the new tree.
 * 
 * *****************  Version 5  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 4:18p
 * Updated in $/development/sv_libs
 * update COPYRIGHT
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 5/07/98    Time: 11:09a
 * Updated in $/base_level2/sv_libs
 * Add Intel copyright 
 * 
 * *****************  Version 3  *****************
 * User: C-hqian      Date: 5/12/98    Time: 4:09p
 * Updated in $/base_level2/sv_libs
 * Deleted LibD handle when library closes.
 * 
 * *****************  Version 2  *****************
 * User: C-hqian      Date: 4/30/98    Time: 3:25p
 * Updated in $/base_level2/sv_libs
 * Code development.
 * 
 * *****************  Version 1  *****************
 * User: C-hqian      Date: 4/28/98    Time: 1:52p
 * Created in $/base_level2/sv_libs
 * Memory service routine library.
 * 
 */
// #include "uclo.h"
#include <stdio.h>
#include <stdlib.h>
#include <new.h>
#include "libd.h"
#include "svlibs.h"
#include "memsv.h"

struct MemSvLibHandleExt {       // common to all Lib
    int tid;                // thread id
    SvCallModes flag;       // PEND, QUEUE_ASYNC, or QUEUE_SYNC
    LibD_Callout_T* testSetHandle;
    LibD_Callout_T* bZeroHandle;
    LibD_Callout_T* bCopyHandle;
    LibD_Callout_T* gatherHandle;
    LibD_Callout_T* scatterHandle;
    void* libdHandle;
};

/*------------------------------------------------------------------------
    MemSv_Init
    return:
  ------------------------------------------------------------------------*/
LibHandle* MemSv_Init(
    void* objHandle,    // a handle of uof file
    int appSrcId        // in multi apps, it identifies a specific app
)
{
    if ((appSrcId < 0) || (appSrcId > 255))
        return((LibHandle*)MEMSV_BAD_HANDLE);

    //
    // library dispatch initialization
    //
    void* libdHandle = LibD_Init(objHandle, appSrcId);
    if (libdHandle == 0)
        return((LibHandle*)MEMSV_BAD_HANDLE);

    //
	// dj: need to be changed -- destId's are now global across devices
	// assign destId 0 thru 23 to uthreads 0 thru 23.
    //
	LibD_AssignUThreadId(0, 0, 24);

    
	//
    // allocate service library handle
    //
    MemSvLibHandleExt* memSvLibHandleExt = new(MemSvLibHandleExt);
    if (memSvLibHandleExt == 0)
        return((LibHandle*)MEMSV_BAD_HANDLE);

    //
    // save libdHandle for deletion at the end of this thread
    //
    memSvLibHandleExt->libdHandle = libdHandle;

    //
    // get bZero() function handle
    //
    memSvLibHandleExt->bZeroHandle =
        LibD_RegLocalCallout(libdHandle, "memsv_bzero#");
    if (memSvLibHandleExt->bZeroHandle == 0)
        return((LibHandle*)MEMSV_BAD_HANDLE);

    //
    // get testSet() function handle
    //
    memSvLibHandleExt->testSetHandle =
        LibD_RegLocalCallout(libdHandle, "memsv_testandset#");
    if (memSvLibHandleExt->testSetHandle == 0)
        return((LibHandle*)MEMSV_BAD_HANDLE);

    //
    // get bCopy() function handle
    //
    memSvLibHandleExt->bCopyHandle = 
        LibD_RegLocalCallout(libdHandle, "memsv_bcopy#");
    if (memSvLibHandleExt->bCopyHandle == 0)
        return((LibHandle*)MEMSV_BAD_HANDLE);

    //
    // get gather() function handle
    //
    memSvLibHandleExt->gatherHandle = 
        LibD_RegLocalCallout(libdHandle, "memsv_gather#");
    if (memSvLibHandleExt->gatherHandle == 0)
        return((LibHandle*)MEMSV_BAD_HANDLE);

    //
    // get scatter() function handle
    //
    memSvLibHandleExt->scatterHandle = 
        LibD_RegLocalCallout(libdHandle, "memsv_scatter#");
    if (memSvLibHandleExt->scatterHandle == 0)
        return((LibHandle*)MEMSV_BAD_HANDLE);

    return((LibHandle*)memSvLibHandleExt);
};

/*------------------------------------------------------------------------
    MemSv_TestAndSet
        Atomic test and set bits at an SRAM location.
        Write msvTASIn->st bits at SRAM memory location
        msvTASIn->at.  Return the value of data at location
        msvTASIn->at prior to the set.
    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - MEMSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int MemSv_TestAndSet(
    LibHandle* libHandle,
    msvTASIn* inArg,
    int* outArg
)
{
    LibD_Callout_T* testSetHandle = 
        ((MemSvLibHandleExt*)libHandle)->testSetHandle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (testSetHandle == 0)
        return(MEMSV_FAIL);

    //
    // clear transaction ID
    //
    testSetHandle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < MEMSV_MAX_THREADS))
        testSetHandle->destId = libHandle->tid;
    else
        return(MEMSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            testSetHandle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            testSetHandle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            testSetHandle->sync = ASYNC;
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    //
    // call micro code function
    //
    int retval = LibD_CallOut(  testSetHandle,
                                (void*)inArg,
                                (void*)outArg);
    if (retval != LIBD_SUCCESS)
            return(MEMSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(testSetHandle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(testSetHandle, (void*)outArg);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(testSetHandle->txId);
            else
                return(MEMSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(testSetHandle->txId);
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    return(MEMSV_SUCCESS);
};

/*------------------------------------------------------------------------
    MemSv_BZero
        Zero a memory buffer.
        Zero msvBuf->len bytes of data starting at memory
        location specified by msvBuf->data.
    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - MEMSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int MemSv_BZero(
    LibHandle* libHandle,
    msvBuf* inArg
)
{
    LibD_Callout_T* bZeroHandle = 
        ((MemSvLibHandleExt*)libHandle)->bZeroHandle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (bZeroHandle == 0)
        return(MEMSV_FAIL);

    //
    // clear transaction ID
    //
    bZeroHandle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < MEMSV_MAX_THREADS))
        bZeroHandle->destId = libHandle->tid;
    else
        return(MEMSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            bZeroHandle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            bZeroHandle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            bZeroHandle->sync = ASYNC;
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    //
    // call micro code function
    //
    int outArg; // dumb parameter
    int retval = LibD_CallOut(  bZeroHandle,
                                (void*)inArg,
                                (void*)&outArg);
    if (retval != LIBD_SUCCESS)
            return(MEMSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(bZeroHandle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(bZeroHandle, 0);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(bZeroHandle->txId);
            else
                return(MEMSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(bZeroHandle->txId);
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    return(MEMSV_SUCCESS);
};


/*------------------------------------------------------------------------
    MemSv_BCopy
        Copy a buffer from one memory location to another.
        Copy msvBCopyIn->len bytes of data from one memory
        location specified by msvBCopyIn->from to another
        memory location specified by msvBCopyIn->to.
    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - MEMSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int MemSv_BCopy(
    LibHandle* libHandle,
    msvBCopyIn* inArg
)
{
    LibD_Callout_T* bCopyHandle = 
        ((MemSvLibHandleExt*)libHandle)->bCopyHandle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (bCopyHandle == 0)
        return(MEMSV_FAIL);

    //
    // clear transaction ID
    //
    bCopyHandle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < MEMSV_MAX_THREADS))
        bCopyHandle->destId = libHandle->tid;
    else
        return(MEMSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            bCopyHandle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            bCopyHandle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            bCopyHandle->sync = ASYNC;
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    //
    // call micro code function
    //
    int outArg; // dumb parameter
	shrimpArgDesc_T inputArg;
	inputArg.len = sizeof(msvBCopyIn);
	inputArg.addr = (char *)inArg;
    int retval = LibD_CallOut(  bCopyHandle,
                                (void*)&inputArg,
                                (void*)&outArg);
    if (retval != LIBD_SUCCESS)
            return(MEMSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(bCopyHandle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(bCopyHandle, 0);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(bCopyHandle->txId);
            else
                return(MEMSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(bCopyHandle->txId);
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    return(MEMSV_SUCCESS);
};


/*------------------------------------------------------------------------
    MemSv_Gather
        Gather multiple memory buffers into one memory buffer.
        Gather data described by an array[msvGatherIn->count]
        of msvGatherIn->from.  Concatenate and place in one
        memory location specified by msvGatherIn->to.
    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - MEMSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int MemSv_Gather(
    LibHandle* libHandle,
    msvGatherIn* inArg
)
{
    LibD_Callout_T* gatherHandle = 
        ((MemSvLibHandleExt*)libHandle)->gatherHandle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (gatherHandle == 0)
        return(MEMSV_FAIL);

    //
    // clear transaction ID
    //
    gatherHandle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < MEMSV_MAX_THREADS))
        gatherHandle->destId = libHandle->tid;
    else
        return(MEMSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            gatherHandle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            gatherHandle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            gatherHandle->sync = ASYNC;
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    //
    // call micro code function
    //
    int outArg; // dumb parameter
	shrimpArgDesc_T inputArg;
	inputArg.len = sizeof(msvGatherIn);
	inputArg.addr = (char *)inArg;
    int retval = LibD_CallOut(  gatherHandle,
                                (void*)&inputArg,
                                (void*)&outArg);
    if (retval != LIBD_SUCCESS)
            return(MEMSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(gatherHandle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(gatherHandle, 0);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(gatherHandle->txId);
            else
                return(MEMSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(gatherHandle->txId);
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    return(MEMSV_SUCCESS);
};


/*------------------------------------------------------------------------
    MemSv_scatter
        Scatter one memory buffer to multiple memory buffers.
        Read data from one location specified by msvScatterIn->from,
        section and place at lengths/addresses described by
        array[msvScatterIn->count] of msvScatterIn-to.
    return:
        success 0 - 7:  If asynchronous, the request succeded.
                        If synchronous, the operation succeded.
        success 0 - 7:  Transaction Id for asynchronous operation.
        fail    8 - 15: If asynchronous, the request failed.
                        If synchronous, the operation failed.
        fail    (return - MEMSV_FAIL)
                        Transaction Id for failed asynchronous
                        request.
  ------------------------------------------------------------------------*/
int MemSv_scatter(
    LibHandle* libHandle,
    msvScatterIn* inArg
)
{
    LibD_Callout_T* scatterHandle = 
        ((MemSvLibHandleExt*)libHandle)->scatterHandle;

    //
    // check if library is initialized
    // and the function is registered
    //
    if (scatterHandle == 0)
        return(MEMSV_FAIL);

    //
    // clear transaction ID
    //
    scatterHandle->txId = 0;

    //
    // specify the micro engin thread,
    // in which the function stays.
    //
    if ((libHandle->tid >= 0) && (libHandle->tid < MEMSV_MAX_THREADS))
        scatterHandle->destId = libHandle->tid;
    else
        return(MEMSV_FAIL);

    //
    // specify the function call mode is either
    // asynchronous or synchronous.
    //
    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            scatterHandle->sync = SYNC;
            break;

        case SVLIB_SYNC_CALL:
            scatterHandle->sync = ASYNC;
            break;

        case SVLIB_ASYNC_CALL:
            scatterHandle->sync = ASYNC;
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    //
    // call micro code function
    //
    int outArg; // dumb parameter
	shrimpArgDesc_T inputArg;
	inputArg.len = sizeof(msvScatterIn);
	inputArg.addr = (char *)inArg;
    int retval = LibD_CallOut(  scatterHandle,
                                (void*)&inputArg,
                                (void*)&outArg);
    if (retval != LIBD_SUCCESS)
            return(MEMSV_FAIL);

    switch (libHandle->flag)
    {
        case SVLIB_PEND_CALL:
            return(scatterHandle->txId);
            break;

        case SVLIB_SYNC_CALL:
            retval = LibD_Wait(scatterHandle, 0);
            if ((retval == LIBD_SUCCESS) && (outArg != 0))
                return(scatterHandle->txId);
            else
                return(MEMSV_FAIL);

            break;

        case SVLIB_ASYNC_CALL:
            return(scatterHandle->txId);
            break;

        default:
            return(MEMSV_FAIL); // libHandle->flag has a wrong value
    };

    return(MEMSV_SUCCESS);
}

void MemSv_Close(
    LibHandle* libHandle
)
{
    LibD_DeleHandle(((MemSvLibHandleExt*)libHandle)->libdHandle);
    delete((MemSvLibHandleExt*)libHandle);
}

// end of file

