/* uclo.c
 * Ucode Object File Loader facilities
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Ucode Linker/Loader
 * author: drj Dec 31, 97 Initial version
 * revisions:
 * $History: uclo.c $
 * 
 * *****************  Version 71  *****************
 * User: Drjohns1     Date: 12/16/00   Time: 3:23p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Changed __cpu_revision to __chip_revision
 * 
 * *****************  Version 70  *****************
 * User: Drjohns1     Date: 12/07/00   Time: 8:38p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Corrected return value comment.
 * 
 * *****************  Version 69  *****************
 * User: Drjohns1     Date: 12/05/00   Time: 6:40p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Removed temporary fix for Vxworks/enum problem
 * 
 * *****************  Version 68  *****************
 * User: Ewalker      Date: 12/05/00   Time: 1:11p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Fixed SDRAM_WRITEx bug causing transactor to crash.
 * 
 * *****************  Version 67  *****************
 * User: Drjohns1     Date: 12/01/00   Time: 2:25p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Temporary fix for SDRAM_NON_PREFETCH_BEGIN being set to zero in VxWorks
 * 
 * *****************  Version 66  *****************
 * User: Drjohns1     Date: 11/30/00   Time: 8:26p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * 
 * *****************  Version 65  *****************
 * User: Drjohns1     Date: 11/29/00   Time: 6:39p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * 
 * *****************  Version 64  *****************
 * User: Drjohns1     Date: 11/28/00   Time: 6:57p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Adjusted sdram long-word address to write to the correct quad-word
 * location
 * 
 * *****************  Version 63  *****************
 * User: Ewalker      Date: 11/27/00   Time: 3:53p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Modified cpu revision checking to conform to new  numbering convention
 * 
 * *****************  Version 62  *****************
 * User: Drjohns1     Date: 11/21/00   Time: 1:26p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Added retrieval of variables memory segments values
 * 
 * *****************  Version 61  *****************
 * User: Drjohns1     Date: 11/17/00   Time: 5:30p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Added _chip_revision, __chip_id, and __eungine_id support
 * 
 * *****************  Version 60  *****************
 * User: Qabuild      Date: 11/16/00   Time: 4:25p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Eliminated unused local variable warnings
 * 
 * *****************  Version 59  *****************
 * User: Qabuild      Date: 11/16/00   Time: 3:11p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Disable xactio macros
 * 
 * *****************  Version 58  *****************
 * User: Ewalker      Date: 11/16/00   Time: 2:53p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Added configurations for uclo_xact.lib so that transactor can implement
 * a new load_uof_file command.
 * 
 * *****************  Version 57  *****************
 * User: Drjohns1     Date: 11/15/00   Time: 3:45p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Added uC support
 * 
 * *****************  Version 56  *****************
 * User: Drjohns1     Date: 10/30/00   Time: 11:35a
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Fixed SCR #1201 -- ucode fill pattern
 * 
 * 
 * ---------------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <errno.h>
#include "utils.h"
#include "uclo_dev.h"
#include "uclo.h"
#include "core_io.h"

#include "hal_sram.h"
#include "hal_sdram.h"



#ifndef UCLO_WB_LIB
#include "hal_fbi.h"
#endif

#if defined(UCLO_WB_LIB)
#include "ueng_constants.h"
#elif !defined(UCLO_XACT_LIB)
#include "ueng.h"
#endif  /* #if defined(UCLO_WB_LIB) */

#include "uclo_ver.h"
#include "uclo_utl.h"

#ifdef __ARMCC_VERSION
#define VALUE(x) x.__lo
#else
#define VALUE(x) x
#endif

#define ALL_CPU_TYPE 0xff

static const char __copyright[]=
"Copyright Intel Corp, 1997-2000. All rights reserved";

/* define GetCpuRev macro and local version of sysCpuIDGet function to always specific values */
#if defined(UCLO_WB_LIB)
#define GetCpuRev() (ALL_CPU_TYPE)
static int XACT_get_ID_CHIP(void){return(0x6901c120);}
#else
#define GetCpuRev() (0x0f & sysCpuIDGet())
#endif  /* UCLO_WB_LIB */


/*-----------------------------------------------------------------------------
   Function: UcLo_WrSram	
   Description: Write a specified number of lonwords to SRAM
   Returns: 
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLo_WrSram(unsigned int lwaddr, unsigned int *val, unsigned int count)
{
	unsigned int i=0;

	/*printf("(debug dj:) SRAM addr=0x%x, writing %d words, value[0]=0x%x\n", lwaddr, count, val[0]);  fflush(stdout);*/

#ifndef UCLO_WB_LIB
	for(i; i < count; i++){
		SRAM_WRITE(lwaddr + i, val[i]);
	}
#endif
	return;
}



/*-----------------------------------------------------------------------------
   Function: UcLo_WrSdram	
   Description: Write a specified number of lonwords to SDRAM
   Returns: 
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLo_WrSdram(unsigned int lwaddr, unsigned int *val, unsigned int count)
{
	unsigned int i=0;

	/*printf("(debug dj:) SDRAM addr=0x%x, writing %d words, value[0]=0x%x\n", lwaddr, count, val[0]);  fflush(stdout);*/

#ifndef UCLO_WB_LIB
	for(i=0; i < count; i++){
		SDRAM_WRITE((lwaddr + i), val[i]);
	}
#endif
	return;
}

/*-----------------------------------------------------------------------------
   Function: UcLo_WrScratch	
   Description: Write a specified number of lonwords to SCRATCH PAD MEMORY
   Returns: 
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLo_WrScratch(unsigned int lwaddr, unsigned int *val, unsigned int count)
{
	unsigned int i = 0;

	/*printf("(debug dj:) SCRATCH addr=0x%x, writing %d words, value[0]=0x%x\n", lwaddr, count, val[0]); fflush(stdout);*/

#ifndef UCLO_WB_LIB
	for(i; i < count; i++){
		SCRATCH_WRITE(lwaddr + i, val[i]);
	}
#endif
	return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_InitMemory
   Description: 
   Returns: UCLO_SUCCESS, UCLO_BADARG, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_InitMemory(uclo_objHandle_T *objHandle)
{
  unsigned int *values, i;
   uof_initMem_T *initMem;
   uof_initMemTab_T *initMemTab;

   if(!objHandle) return(UCLO_BADARG);
   initMemTab = &objHandle->initMemTab;

   for(i = 0; i < initMemTab->numEntries; i++){
	   initMem = &(initMemTab->initMem[i]);
	   values = (unsigned int *)(initMem->valOffset + objHandle->fBuf);
	   switch(initMem->region){
	      case 0: UcLo_WrSram(initMem->addr, values, initMem->numLwords); break;
	      case 1: UcLo_WrSdram(initMem->addr >> 1, values, initMem->numLwords); break;
	      case 2: UcLo_WrScratch(initMem->addr, values, initMem->numLwords); break;
		  default: return(UCLO_BADOBJ);
	   }
   }
   return(UCLO_SUCCESS);
}



/*-----------------------------------------------------------------------------
   Function: UcLo_StrLowcase	
   Description: Sets string to lowercase.
   Returns: pointer to the string
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *UcLo_StrLowcase(char *s)
{
   char *cs;
   for(cs = s; cs && *cs; cs++)
      *cs = tolower(*cs);
   return(s);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_getString
   Description: Gets string from the string table.
   Returns: pointer to the string or NULL if not found
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *UcLo_getString(uof_strTab_T *strTable, int strOffset)
{
   if(strOffset > strTable->tableLen) return(NULL);
   return((char *)(strTable->strings + strOffset));
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_findChunkHdr
   Description: Locates UOF chunk header matching the type specified by the
                input parameter 'chunkId'.
   Returns: pointer to the chunk header or NULL if not found
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_chunkHdr_T *UcLo_findChunkHdr(uof_encapUofObj_T *encapUofObj, char *chunkId,
                                  void *cur)
{
   int i;

   for(i = 0; i < encapUofObj->objHdr->numChunks; i++)
      if(!(strncmp(encapUofObj->chunkHdr[i].chunkId, chunkId, UOF_OBJID_LEN)) &&
	  (cur < (void *)&encapUofObj->chunkHdr[i])) return(&encapUofObj->chunkHdr[i]);
   return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_extractObj
   Description: Creates a new file header and copies the indicated object 
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *UcLo_copyObj(FILE *fp, char *chunkId, unsigned int *objSize)
{
   unsigned int 		*chunkFid, oSize;
   int					i;
   uof_fileHdr_T 		fileHdr, *newFileHdr;
   uof_fileChunkHdr_T  	fileChunk, *newFileChunk;
   char		      		*buf, *chunk;
   fpos_t				filePos;


   /* read the file header */ 
   if(fread(&fileHdr, 1, sizeof(uof_fileHdr_T), fp) !=
   		sizeof(uof_fileHdr_T)) return(NULL);

   /* verify file id */
   chunkFid = (unsigned int *)fileHdr.fileId;
   if(chunkFid[0] != UOF_CFID_0 || chunkFid[1] != UOF_CFID_1) return(NULL);

   for(i = 0; i < fileHdr.numChunks; i++){
	  /* read the all the primary chunks */
      if(fread(&fileChunk, 1, sizeof(uof_fileChunkHdr_T), fp) !=
   		sizeof(uof_fileChunkHdr_T)) return(NULL);

	  if(!(strncmp(fileChunk.chunkId, chunkId, UOF_OBJID_LEN))){

		 /* alloc space for new file header and object */
		 oSize = sizeof(uof_fileHdr_T) + sizeof(uof_fileChunkHdr_T) + fileChunk.size;
		 if(!(buf = (char *)malloc(oSize))) return(NULL);

		 /* init the new file header */
		 newFileHdr = (uof_fileHdr_T *)buf;
		 newFileHdr->numChunks = 1;
		 newFileHdr->maxChunks = 1;
		 memmove(&newFileHdr->fileId, &fileHdr.fileId, UOF_OBJID_LEN);


		 /* init the new file chunk header */
		 newFileChunk = (uof_fileChunkHdr_T *)(buf + sizeof(uof_fileHdr_T));
		 memmove(newFileChunk, &fileChunk, sizeof(uof_fileChunkHdr_T));
		 newFileChunk->offset = sizeof(uof_fileHdr_T) + sizeof(uof_fileChunkHdr_T);

		 /* read the object */
         VALUE(filePos) = fileChunk.offset;
         fsetpos(fp, &filePos);
		 chunk = (char *)(buf + newFileChunk->offset);
         if(fread(chunk, 1, fileChunk.size, fp) != (size_t)fileChunk.size){
	        free(buf);
	        break;
		 }

		 /* verify the checksum */
	     if(fileChunk.checksum != Crc_StrChecksum((char*)chunk, fileChunk.size)){
	        free(buf);
	        break;
		 }

		 /* return the size of the new object */
		 if(objSize) *objSize = oSize;
         return(buf);
      }
   }

   return(NULL);
}

/*-----------------------------------------------------------------------------
   Function: 	Uclo_GetUofUword
   Description: Given a valid uword address, return the associated uword from
				the UOF. The function will fail if uAddr is an invalid or unused
				address.
   Returns: pointer to the uword or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
unsigned int *Uclo_GetUofUword(uof_encapPage_T *page, unsigned int uAddr)
{
	if(!page) return(NULL);

	if(uAddr >= UOF_MAX_UWORDS_SEG){
	   uAddr = uAddr - UOF_MAX_UWORDS_SEG + page->numMicroWords;
	   if(uAddr >= (unsigned)(page->numMicroWords + page->numMicroWords_2)) return(NULL);
	}
	else{
		if(uAddr >= (unsigned)page->numMicroWords) return(NULL);
	}
	return(&page->microWords[uAddr]);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_readFileHdr
   Description: Reads the file header -- must be the first entry in the file.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void *UcLo_readFileHdr(FILE *fp, char *chunkId, unsigned int *chunkSize)
{
   unsigned int 		*chunkFid;
   int					i;
   uof_fileHdr_T 		fileHdr;
   uof_fileChunkHdr_T  	fileChunk;
   void		      		*chunk;
   fpos_t				filePos;


   /* read the file header */ 
   if(fread(&fileHdr, 1, sizeof(uof_fileHdr_T), fp) !=
   		sizeof(uof_fileHdr_T)) return(NULL);

   /* verify file id */
   chunkFid = (unsigned int *)fileHdr.fileId;
   if(chunkFid[0] != UOF_CFID_0 || chunkFid[1] != UOF_CFID_1) return(NULL);

   for(i = 0; i < fileHdr.numChunks; i++){
      /* read the all the primary chunks */
      if(fread(&fileChunk, 1, sizeof(uof_fileChunkHdr_T), fp) !=
   		sizeof(uof_fileChunkHdr_T)) return(NULL);
      if(!(strncmp(fileChunk.chunkId, chunkId, UOF_OBJID_LEN))){

	 /* alloc space and read chunk */
	 if(!(chunk = malloc(fileChunk.size + 512))) return(NULL);
         VALUE(filePos) = fileChunk.offset;
         fsetpos(fp, &filePos);
         if(fread(chunk, 1, fileChunk.size, fp) != (size_t)fileChunk.size){
	    free(chunk);
	    break;
	 }

	 if(fileChunk.checksum != Crc_StrChecksum((char*)chunk, fileChunk.size)){
	    free(chunk);
	    break;
	 }
		if(chunkSize) *chunkSize = fileChunk.size;
         return(chunk);
      }
   }

   return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_mapFileHdr
   Description: Maps the file header -- must be the first entry in the buffer.
   Returns:		Pointer to the chunk or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void *UcLo_mapFileHdr(char *buf, char *chunkId)
{
   unsigned int 		*chunkFid;
   int					i;
   uof_fileHdr_T 		*fileHdr;
   uof_fileChunkHdr_T  	*fileChunk;
   void					*chunk;

   /* map the file header */
   if(!(fileHdr = (uof_fileHdr_T *)buf)) return(NULL);

   /* verify file id */
   chunkFid = (unsigned int *)fileHdr->fileId;
   if(chunkFid[0] != UOF_CFID_0 || chunkFid[1] != UOF_CFID_1) return(NULL);
   fileChunk = (uof_fileChunkHdr_T *)(buf + sizeof(uof_fileHdr_T));

   /* find the chunk, verify it, return a pointer to it */
   for(i = 0; i < fileHdr->numChunks; i++){
      if(!(strncmp(fileChunk->chunkId, chunkId, UOF_OBJID_LEN))){

		  /* verify chunk checksum */
		  chunk = buf + fileChunk->offset;
	     if(fileChunk->checksum != Crc_StrChecksum((char*)chunk, fileChunk->size)) break;
	     return(chunk);
 	  }
      fileChunk++;
   }
   return(NULL);
}



/*-----------------------------------------------------------------------------
   Function: 	UcLo_isFengSet
   Description: Determines if an Uengine is assigned
   Returns: 1 if assigned, 0 otherwise
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_isFengSet(short imageAttrib, int fEngine)
{
   unsigned int attr;

   attr = imageAttrib;
   return(isBitSet(attr, fEngine));
}



/*-----------------------------------------------------------------------------
   Function: 	UcLo_mapImage
   Description: Map the uof image
   Returns: number of images, otherwise 0 if none or failure
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_mapImage(uof_encapUofObj_T *encapUofObj, uof_encapFeng_T *encapImage,
	    int maxImages, int readOnly)
{
	int a = 0, p, i;
	uof_chunkHdr_T *chunkHdr = NULL;
	uof_Image_T    *image;
	uof_codePage_T *codePage;

	for(i = 0; i < maxImages; i++){
		encapImage[i].images = NULL;
		encapImage[i].pages = NULL;
	}


	for(a = 0; a < maxImages; a++){
		/* get IMAGE chunk header */
		if(!(chunkHdr = UcLo_findChunkHdr(encapUofObj, UOF_IMAG, (void *)chunkHdr))) break;

		image = (uof_Image_T *)(encapUofObj->begUof + chunkHdr->offset);
		encapImage[a].images = image;

		codePage = (uof_codePage_T *)((char *)image + sizeof(uof_Image_T));
		if(!(encapImage[a].pages = (uof_encapPage_T *)calloc(image->numOfPage,
					sizeof(uof_encapPage_T)))){
			return(0);
		}


		/* assign the pages */
		for(p = 0; p < image->numOfPage; p++){
			uof_ucVarTab_T        *ucVarTab;
			uof_impVarTab_T        *impVarTab;
			uof_expFuncTab_T      *expFuncTab;
			uof_codeArea_T         *codeArea;

			/* assign uC variables */
			ucVarTab = (uof_ucVarTab_T *)(encapUofObj->begUof + codePage[p].ucVarTabOffset);
			encapImage[a].pages[p].numUcVar = ucVarTab->numEntries;
			encapImage[a].pages[p].ucVar = (uof_ucVar_T *)((char *)ucVarTab + sizeof(uof_ucVarTab_T));

			/* assign import variables */
			impVarTab = (uof_impVarTab_T *)(encapUofObj->begUof + codePage[p].impVarTabOffset);
			encapImage[a].pages[p].numImpVar = impVarTab->numEntries;
			encapImage[a].pages[p].importVar = (uof_importVar_T *)((char *)impVarTab + sizeof(uof_impVarTab_T));

			/* assign export functions */
			expFuncTab = (uof_expFuncTab_T *)(encapUofObj->begUof + codePage[p].expFuncTabOffset);
			encapImage[a].pages[p].numExpFunc = expFuncTab->numEntries;
			encapImage[a].pages[p].exportFunc = (uof_exportFunc_T *)((char *)expFuncTab + sizeof(uof_expFuncTab_T));

			/* assign code area */
			codeArea = (uof_codeArea_T *)(encapUofObj->begUof + codePage[p].codeAreaOffset);
			encapImage[a].pages[p].numMicroWords = codeArea->numMicroWords;
			encapImage[a].pages[p].numMicroWords_2 = codeArea->numMicroWords_2;

			/* if there are uC or import variables to be updated, then copy the microwords--in case we are mapping from ROM. */
			if((encapImage[a].pages[p].numImpVar || encapImage[a].pages[p].numUcVar) && readOnly){
				if(!(encapImage[a].pages[p].microWords = (int *)calloc(encapImage[a].pages[p].numMicroWords +
							encapImage[a].pages[p].numMicroWords_2, sizeof(int)))){
				   return(0);
				}
				memmove(encapImage[a].pages[p].microWords,
					  ((char *)codeArea + sizeof(uof_codeArea_T)),
					  (encapImage[a].pages[p].numMicroWords + encapImage[a].pages[p].numMicroWords_2) * sizeof(int));
			}
			else{
					encapImage[a].pages[p].microWords = (int *)((char *)codeArea + sizeof(uof_codeArea_T));
			}
		}
	}

   return(a);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_mapFeng
   Description: Maps Uengines in UOF to the handle
   Returns: UCLO_SUCCESS
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_mapFeng(uclo_objHandle_T *objHandle, int maxFeng)
{
   int i, a;

   for(i = 0; i < maxFeng; i++){
      objHandle->encapFengImage[i].images = NULL;
      objHandle->encapFengImage[i].pages = NULL;

      /* traverse the images and check if uEng is assigned */
      for(a = 0; a < objHandle->numUniqueImage; a++){
	 if(UcLo_isFengSet(objHandle->uniqueFengImage[a].images->imageAttrib, i)){
	    objHandle->encapFengImage[i].images = objHandle->uniqueFengImage[a].images;
	    objHandle->encapFengImage[i].pages = objHandle->uniqueFengImage[a].pages;
	    break;
	 }
      }
   }

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_mapStrTable
   Description: Maps the UOF string table
   Returns: pointer to the string table or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_strTab_T *UcLo_mapStrTable(uof_encapUofObj_T *encapUofObj, uof_strTab_T *strTable)
{
   uof_chunkHdr_T *chunkHdr = NULL;

   /* get the first string table -- should be only one */
   if((chunkHdr = UcLo_findChunkHdr(encapUofObj, UOF_STRT, NULL))){
      memmove(&strTable->tableLen, (encapUofObj->begUof + chunkHdr->offset), sizeof(strTable->tableLen));
      strTable->strings = (char *)(encapUofObj->begUof + chunkHdr->offset + sizeof(strTable->tableLen));
   }
   return(strTable);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_mapInitMemTable
   Description: Maps the UOF init memory table
   Returns: pointer to the initMem table or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_initMemTab_T *UcLo_mapInitMemTable(uof_encapUofObj_T *encapUofObj, uof_initMemTab_T *initMemTab)
{
	uof_chunkHdr_T *chunkHdr = NULL;

   /* get the first string table -- should be only one */
   if((chunkHdr = UcLo_findChunkHdr(encapUofObj, UOF_IMEM, NULL))){
      memmove(&initMemTab->numEntries, (encapUofObj->begUof + chunkHdr->offset), sizeof(unsigned int));
      if(initMemTab->numEntries)
		  initMemTab->initMem = (uof_initMem_T *)(encapUofObj->begUof + chunkHdr->offset + sizeof(unsigned int));
   }
   else{
	  initMemTab->numEntries = 0;
	  initMemTab->initMem = NULL;
   }
   return(initMemTab);
}

/*-----------------------------------------------------------------------------
   Function: 	UcLo_mapVarMemSeg
   Description: Maps the uC memory segments
   Returns: pointer to the uof_uCmemSeg_T or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_varMemSeg_T *UcLo_mapVarMemSeg(uof_encapUofObj_T *encapUofObj)
{
	uof_chunkHdr_T *chunkHdr = NULL;
	uof_varMemSeg_T *varMemSeg = NULL;
	unsigned int numEntries;

   /* get the first string table -- should be only one */
   if((chunkHdr = UcLo_findChunkHdr(encapUofObj, UOF_MSEG, NULL))){
	  /* numEntries should be one  -- allows us to entend the number of uof_varMemSeg_T without
	     affecting the UOF format */
      memmove(&numEntries, (encapUofObj->begUof + chunkHdr->offset), sizeof(unsigned int));
      varMemSeg = (uof_varMemSeg_T *)(encapUofObj->begUof + chunkHdr->offset + sizeof(numEntries));
   }
   return(varMemSeg);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_mapGenToolId
   Description: Map the generating tool's ID chunk header
   Returns: pointer to the gen too chnuk header or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_GTID_T *UcLo_mapGenToolId(uof_encapUofObj_T *encapUofObj)
{
   uof_chunkHdr_T *chunkHdr = NULL;
   uof_GTID_T *gtid = NULL;

   /* get the first generator tool id -- should be only one */
   if((chunkHdr = UcLo_findChunkHdr(encapUofObj, UOF_GTID, NULL)))
      gtid = (uof_GTID_T *)(encapUofObj->begUof + chunkHdr->offset);
   return(gtid);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_setValue
   Description: Assigns 'value' to a group of bits, (sub-field), as described
                by the 'field' input parameter.
   Returns: the updated word
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
unsigned int UcLo_setValue(unsigned int uword, char *field, unsigned int value)
{
   int i;

   /* validate the sub-fields */
   for(i=0; i < 12;){
      unsigned int val;
      char msb, lsb, begBitPos, len, rShtValue;

      if((msb = field[i++]) > 31) break;
      lsb = field[i++];
      begBitPos = lsb;
      len = (msb - lsb + 1);

      rShtValue = field[i++];
      val = value >> rShtValue; 
      uword = setField(uword, begBitPos, len, val);
   }

   return(uword);
}



/*-----------------------------------------------------------------------------
   Function: 	UcLo_getFileSize
   Description: Determines the byte size of an opened file
   Returns: number of bytes in the file
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_getFileSize(FILE *fp)
{
   fpos_t	endPos;
   long		curFpos;

   curFpos = ftell(fp);
   fseek(fp, 0L, SEEK_END);
   VALUE(endPos) = ftell(fp);
   fseek(fp, curFpos, SEEK_SET);
   return((int)VALUE(endPos));
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_getObjHandle
   Description: Creates an UOF object handle
   Returns: pointer to handle, otherwise NULL if failed
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uclo_objHandle_T *UcLo_getObjHandle()
{
   uclo_objHandle_T *objHandle;

   if(!(objHandle = (uclo_objHandle_T*)calloc(1, sizeof(uclo_objHandle_T))))
      return(NULL);

   objHandle->readOnly = 1;
   return(objHandle);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_GetEncapImage
   Description: Locates an UOF image by name as specified by the 'ucodeImageName'
                parameter.
   Returns: pointer to the encapsulated image or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_encapFeng_T *UcLo_GetEncapImage(uclo_objHandle_T *objHandle, char *ucodeImageName)
{
   int i;

   if(!ucodeImageName || !objHandle || !objHandle->fBuf) return(NULL);

   /* find the image */
   for(i = 0; i < objHandle->numUniqueImage; i++)
      if(!strcmp(ucodeImageName, UcLo_getString(&objHandle->strTable,
                 objHandle->uniqueFengImage[i].images->imageName)))
	 return(&objHandle->uniqueFengImage[i]);

   return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_GetImage
   Description: Locates an UOF image by name as specified by the 'ucodeImageName'
                parameter.
   Returns: pointer to the image or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_Image_T *UcLo_GetImage(uclo_objHandle_T *objHandle, char *ucodeImageName)
{
   uof_encapFeng_T *encapImage;

   if((encapImage = UcLo_GetEncapImage(objHandle, ucodeImageName)))
      return(encapImage->images);

   return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_checkCompat
   Description: Check if the UOF objects are compatible with the chip.
   Returns: UCLO_SUCCESS, or UCLO_UOFINCOMPAT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_checkCompat(uclo_objHandle_T *objHandle)
{
	unsigned int cpuType = ALL_CPU_TYPE;
	int cpuVer;

	cpuVer = GetCpuRev();  /* uses XACT_ or uEng_ function defined by core_io.h */

#ifdef _DBG_PRINT
	printf("Checking UOF compatibility - cpuVer=%d, minCpuVer=%d, maxCpuVer=%d\n",
			cpuVer, 
			objHandle->encapUofObj.objHdr->minCpuVer,
			objHandle->encapUofObj.objHdr->maxCpuVer);
#endif

	/* check the min and max cpu version required against the chip's */
	if(cpuVer >= ALL_CPU_TYPE) return(UCLO_SUCCESS);	/* any version */
	if((objHandle->encapUofObj.objHdr->maxCpuVer < cpuVer) ||
	   (objHandle->encapUofObj.objHdr->minCpuVer > cpuVer)) return(UCLO_UOFINCOMPAT);

	return(UCLO_SUCCESS);
}


#define GLOBAL_SCOPE 0
#define LOCAL_SCOPE 1


/*-----------------------------------------------------------------------------
   Function: 	UcLo_fixupGlobalConst
   Description:	Set the __CHIP_ID, __CHIP_REVISION constants to the appropriate
				values in the micro-instructions.
   Returns: UCLO_SUCCESS, or UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_fixupGlobalConst(uof_strTab_T *strTable, uof_encapPage_T *page)
{
	unsigned int *uword, uaddr, uwrdBefore;
	int iv;
	char *fields;
	unsigned int cpuRev, cpuId, value;
	char *chipRev = "__chip_revision";
	char *chipId = "__chip_id";
	uof_importVar_T *importVar;

	cpuRev = GetCpuRev();  /* uses XACT_ or uEng_ function defined by core_io.h */
	cpuId = sysCpuIDGet() >> 4;
	for(iv = 0; iv < page->numImpVar; iv++){

		if(!strncmp(UcLo_getString(strTable, page->importVar[iv].varName), chipRev, strlen(chipRev))){
			importVar = &page->importVar[iv];
			value = cpuRev;
		}
		else if(!strncmp(UcLo_getString(strTable, page->importVar[iv].varName), chipId, strlen(chipId))){
			importVar = &page->importVar[iv];
			value = cpuId;
		}
		else continue;

		fields = importVar->fieldAttrs;
		uaddr = ADDR_ATTR_UWORD(importVar->uwordAddress);

		/* bind the global constant */
		if(!(uword = Uclo_GetUofUword(page, uaddr))) return(UCLO_FAILURE);
		uwrdBefore = *uword;
		*uword = UcLo_setValue(*uword, fields, value);
		/*printf("(debug dj:) UcLo_fixupGlobalConst: Addr=%d, value=0x%x, uword=0x%x -> 0x%x\n", uaddr, value, uwrdBefore, *uword);*/
	}

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_fixupLocalConst
   Description:	Set the __UENGINE_ID constants to the appropriate
				values in the micro-instructions.
   Returns: UCLO_SUCCESS, or UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_fixupLocalConst(uof_strTab_T *strTable, uof_encapPage_T *page, char ueng)
{
	unsigned int *uword, uaddr, uwrdBefore;
	int iv;
	char *fields;
	char *uengine_id = "__uengine_id";
	uof_importVar_T *importVar;


	for(iv = 0; iv < page->numImpVar; iv++){

		if(!strncmp(UcLo_getString(strTable, page->importVar[iv].varName), uengine_id, strlen(uengine_id))){
			importVar = &page->importVar[iv];
		}
		else continue;

		fields = importVar->fieldAttrs;
		uaddr = ADDR_ATTR_UWORD(importVar->uwordAddress);

		/* bind the global constant */
		if(!(uword = Uclo_GetUofUword(page, uaddr))) return(UCLO_FAILURE);
		uwrdBefore = *uword;
		*uword = UcLo_setValue(*uword, fields, ueng);
		/*printf("(debug dj:) UcLo_fixupLocalConst: Addr=%d, value=0x%x, uword=0x%x -> 0x%x\n", uaddr, ueng, uwrdBefore, *uword);*/
	}

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_fixupLocalUcVar
   Description: Fixup the local variables for the specified uengine in the UOF
   Returns: UCLO_SUCCESS, or UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_fixupLocalUcVar(uof_strTab_T *strTable, uof_encapPage_T *page, char ueng)
{
	unsigned int *uword, uaddr, uwrdBefore;
	int ucv;
	char *fields;
	char prefix[8];

    LOCAL_NAME(prefix, "", ueng);	/* format the local variable prefix */
	for(ucv = 0; ucv < page->numUcVar; ucv++){
		if((page->ucVar[ucv].scope == LOCAL_SCOPE) &&
				!strncmp(UcLo_getString(strTable, page->ucVar[ucv].symName), prefix, strlen(prefix))){
			fields = page->ucVar[ucv].fieldAttrs;
			uaddr = ADDR_ATTR_UWORD(page->ucVar[ucv].uwordAddress);

			/* bind the local variable */
			if(!(uword = Uclo_GetUofUword(page, uaddr))) return(UCLO_FAILURE);
			uwrdBefore = *uword;
			*uword = UcLo_setValue(*uword, fields, page->ucVar[ucv].value);
			/*printf("(debug dj:) UcLo_fixupLocalUcVar: Addr=%d, value=0x%x, uword=0x%x -> 0x%x\n", uaddr, page->ucVar[ucv].value, uwrdBefore, *uword);*/
		}
	}

   return(UCLO_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	UcLo_fixupGlobalUcVar
   Description: Fixup the global variables in the UOF
   Returns: UCLO_SUCCESS, or UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_fixupGlobalUcVar(uof_encapPage_T *page)
{
	unsigned int *uword, uaddr, uwrdBefore;
	int ucv;
	char *fields;

	for(ucv = 0; ucv < page->numUcVar; ucv++){

		if(page->ucVar[ucv].scope == GLOBAL_SCOPE){
			fields = page->ucVar[ucv].fieldAttrs;
			uaddr = ADDR_ATTR_UWORD(page->ucVar[ucv].uwordAddress);

			/* bind the global variable */
			if(!(uword = Uclo_GetUofUword(page, uaddr))) return(UCLO_FAILURE);
			uwrdBefore = *uword;
			*uword = UcLo_setValue(*uword, fields, page->ucVar[ucv].value);
			/*printf("(debug dj:) UcLo_fixupGlobalUcVar: Addr=%d, value=0x%x, uword=0x%x -> 0x%x\n", uaddr, page->ucVar[ucv].value, uwrdBefore, *uword);*/
		}
	}

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_fixupLocals
   Description: Fixup local variables and constants in the UOF for the specified
				microengine.
   Returns: UCLO_SUCCESS, or UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_fixupLocals(uof_strTab_T *strTable, uof_encapPage_T *page, char ueng)
{
	if(UcLo_fixupLocalUcVar(strTable, page, ueng)) return(UCLO_FAILURE);
	if(UcLo_fixupLocalConst(strTable, page, ueng)) return(UCLO_FAILURE);
	return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_fixupGlobals
   Description: Fixup global variables and constants in the UOF for the specified
				microengine.
   Returns: UCLO_SUCCESS, or UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_fixupGlobals(uof_strTab_T *strTable, uof_encapPage_T *page)
{
	if(page->numUcVar)
		if(UcLo_fixupGlobalUcVar(page)) return(UCLO_FAILURE);
	if(UcLo_fixupGlobalConst(strTable, page)) return(UCLO_FAILURE);
	return(UCLO_SUCCESS);
}



/*-----------------------------------------------------------------------------
   Function: 	UcLo_overlayObj
   Description: Overlays the UOF objects
   Returns: UCLO_SUCCESS, or UCLO_BADOBJ
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_overlayObj(uclo_objHandle_T *objHandle, int readOnly)
{
   int i;

   /* set the UOF object offset */
   objHandle->encapUofObj.begUof = objHandle->fBuf;
   objHandle->encapUofObj.objHdr = (uof_objHdr_T *)objHandle->fBuf;
   objHandle->encapUofObj.chunkHdr = (uof_chunkHdr_T *)(objHandle->fBuf + sizeof(uof_objHdr_T));

   /* map the string table */
   UcLo_mapStrTable(&objHandle->encapUofObj, &objHandle->strTable);

   /* map and initialize the memory segments */
   UcLo_mapInitMemTable(&objHandle->encapUofObj, &objHandle->initMemTab);
   if(objHandle->initMemTab.numEntries)
	   if(UcLo_InitMemory(objHandle)) return(UCLO_BADOBJ);

   objHandle->encapUofObj.varMemSeg = UcLo_mapVarMemSeg(&objHandle->encapUofObj);

   /* map the images */
   if((objHandle->numUniqueImage = UcLo_mapImage(&objHandle->encapUofObj,
   				 objHandle->uniqueFengImage,
				 UOF_MAX_FENGINES, readOnly))){
	  if(UcLo_checkCompat(objHandle)) return(UCLO_UOFINCOMPAT);
      if(UcLo_mapFeng(objHandle, UOF_MAX_FENGINES)) return(UCLO_BADOBJ);
   }

   /* bind the uC global variables -- local variable will done on-the-fly */
   for(i = 0; i < objHandle->numUniqueImage; i++){
      int p;

      for(p = 0; p < objHandle->uniqueFengImage[i].images->numOfPage; p++){
		  if(objHandle->uniqueFengImage[i].pages[p].microWords){
			  if(UcLo_fixupGlobals(&objHandle->strTable,
				  &objHandle->uniqueFengImage[i].pages[p]) != UCLO_SUCCESS) return(UCLO_FAILURE);
		  }
	  }
   }
   

   objHandle->readOnly = readOnly;
   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_WriteUword
   Description: Writes a word to the specified uEngine
   Returns: UCLO_SUCCESS, UCLO_BADARG, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_WriteUword(unsigned int uEngMask, unsigned int uAddr, unsigned int uWord)
{
#ifndef UCLO_WB_LIB
	unsigned char eng;

	for(eng = 0; eng < UOF_MAX_FENGINES; eng++){
	   if(IS_BIT_SET(uEngMask, eng))
#ifdef UCLO_XACT_LIB
		   if(XactIoWrite(USTORE, (char)(1 << (eng & 0xff)), uWord, uAddr) != XACT_SUCCESS) return(UCLO_FAILURE);
#else
           if(uEng_setNumUword(eng, uAddr, 1, (unsigned int *)&uWord) != UENG_SUCCESS) return(UCLO_FAILURE);
#endif  /* UCLO_XACT_LIB */
	}

    return(UCLO_SUCCESS);
#else
    return(UCLO_FAILURE);
#endif  /* UCLO_WB_LIB */
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_ReadUword
   Description: Reads a word from the specified uEngine
   Returns: UCLO_SUCCESS, UCLO_BADARG, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_ReadUword(unsigned char uEngNum, unsigned int uAddr, unsigned int *uWord)
{
#ifndef UCLO_WB_LIB
   if(uEngNum >= UOF_MAX_FENGINES || uAddr >= UOF_MAX_UWORDS) return(UCLO_BADARG);

#ifdef UCLO_XACT_LIB
   if(XactIoRead(USTORE, (char)(1 << (uEngNum & 0xff)), uAddr, (unsigned int *)uWord) != XACT_SUCCESS) return(UCLO_FAILURE);
#else
   if(uEng_getNumUword(uEngNum, uAddr, 1, (unsigned int *)uWord) != UENG_SUCCESS) return(UCLO_FAILURE);
#endif  /* UCLO_XACT_LIB */

   return(UCLO_SUCCESS);
#else
   return(UCLO_FAILURE);
#endif  /* UCLO_WB_LIB */
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_VerifyEncapUimage
   Description: Compares the encapsulated images to images that are loaded in
                the Uengines.
   Returns: UCLO_SUCCESS, UCLO_FAILURE, or UCLO_BADARG, 
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_VerifyEncapUimage(uof_strTab_T *strTable, uof_encapFeng_T *encapImage, char uEngNum)
{
	uof_encapPage_T      *pages;
	uof_Image_T          *image;
	int                  p;

	if(!encapImage) return(UCLO_BADARG);

	image = encapImage->images;
	pages = encapImage->pages;

	/* traverse the pages */
	for(p = 0; p < image->numOfPage; p++){
		unsigned int fEngUword, addr, *uword;

		if(UcLo_fixupLocals(strTable, &pages[p], uEngNum)) return(UCLO_FAILURE);

		/* get and compare the first uword segment */
		for(addr = 0; addr < (unsigned int)pages[p].numMicroWords; addr++){
			if(UcLo_ReadUword((unsigned char)uEngNum, addr, &fEngUword))
				return(UCLO_FAILURE);
			if(!(uword = Uclo_GetUofUword(pages, addr))) return(UCLO_FAILURE);
			if(fEngUword != *uword) return(UCLO_FAILURE);
		}

		/* get and compare the second uword segment */
		for(addr = UOF_MAX_UWORDS_SEG; addr < (unsigned int)(UOF_MAX_UWORDS_SEG + pages[p].numMicroWords_2); addr++){
			if(UcLo_ReadUword((unsigned char)uEngNum, addr, &fEngUword)) return(UCLO_FAILURE);
			if(!(uword = Uclo_GetUofUword(pages, addr))) return(UCLO_FAILURE);
			if(fEngUword != *uword) return(UCLO_FAILURE);
		}
	}

	return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_MapObjAddr
   Description: Maps the UOF objects from the memory location described by the
                'addrPtr', and 'memSize' input parameters.  Reference to the
		objects is returned in the 'handle' parameter. The 'readOnly'
		parameter indicates that the mapped memory is read-only and any
		object that has to be updated will be copied by the this library.
   Returns: UCLO_SUCCESS, UCLO_BADARG, UCLO_MEMFAIL, UCLO_BADOBJ
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_MapObjAddr(void **handle, void *addrPtr, int memSize, int readOnly)
{
   uclo_objHandle_T     *objHdl;
   int                  status;

   if(!handle || !addrPtr || memSize < UCLO_MIN_UOF_SIZE) return(UCLO_BADARG);
   if(!(objHdl = UcLo_getObjHandle())) return(UCLO_MEMFAIL);
   
   /* get the UOF object chunk  */
   if(!(objHdl->fBuf = (char*)UcLo_mapFileHdr((char *)addrPtr, UOF_OBJS))){
      free(objHdl);
      return(UCLO_BADOBJ);
   }

   if((status = UcLo_overlayObj(objHdl, readOnly))){
      free(objHdl);
      return(status);
   }
 
   objHdl->objLoaded = 0;
   *handle = (void *)objHdl;

   return(status);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_LoadObjFile
   Description: Loads the UOF objects from the file specified by 'fileName' and
                return a reference in 'handle' parameter.
   Returns: UCLO_SUCCESS, UCLO_BADARG, UCLO_FILEFAIL, UCLO_MEMFAIL, UCLO_BADOBJ
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_LoadObjFile(void **handle, char *fileName)
{
   uclo_objHandle_T     *objHdl;
   int                  status, chunkSize;
   FILE                 *fp;   

   if(!handle || !fileName) return(UCLO_BADARG);
   if(!(fp = fopen(fileName, "rb"))) return(UCLO_FILEFAIL);

   if(!(objHdl = UcLo_getObjHandle())){
      fclose(fp);
      return(UCLO_MEMFAIL);
   }

   /* get the file header  */
   if(!(objHdl->fBuf = (char*)UcLo_readFileHdr(fp, UOF_OBJS, (unsigned int*)&chunkSize))){
      fclose(fp);
      free(objHdl);
      return(UCLO_FILEFAIL);
   }
   fclose(fp);

   /* map image to buffer and indicate writable -- avoid copying ucode */
   if((status = UcLo_overlayObj(objHdl, 0))){
      free(objHdl->fBuf);
      free(objHdl);
      return(status);
   }
   objHdl->objLoaded = 1;
   *handle = (void *)objHdl;
   return(status);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_CopyObjFile
   Description: Copys the UOF object from the file specified by 'fileName' to
				a buffer.  If UCLO_SUCCESS is returned; then a buffer will
				be allocated and its format will be an UOF containing one object;
				the size of the object will be returned in the variable chunkSize.
				It is the responsibility of the caller to delete the buffer.
   Returns: UCLO_SUCCESS, UCLO_BADARG, UCLO_FILEFAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_CopyObjFile(char *fileName, char **objBuf, unsigned int *chunkSize)
{
   FILE *fp;

   if(!fileName || !objBuf | !chunkSize) return(UCLO_BADARG);
   if(!(fp = fopen(fileName, "rb"))) return(UCLO_FILEFAIL);

   /* get the file header  */
   if(!(*objBuf = (char*)UcLo_copyObj(fp, UOF_OBJS, chunkSize))){
      fclose(fp);
      return(UCLO_FILEFAIL);
   }
   fclose(fp);
   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_DeleObj
   Description: Removes and free all resources references associated with the handle.
   Returns: UCLO_SUCCESS, UCLO_NOOBJ
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_DeleObj(void *handle)
{
   int  		a;
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

   /* check if an object is loaded or mapped */
   if(!objHandle || !objHandle->fBuf) return(UCLO_NOOBJ);

   /* free resource that were allocated by the library */
   for(a = 0; a < objHandle->numUniqueImage; a++){
      int p;

      for(p = 0; p < objHandle->uniqueFengImage[a].images->numOfPage; p++){

	 /* free microWords if they were allocated */
	 if(objHandle->uniqueFengImage[a].pages[p].numImpVar &&
	    objHandle->uniqueFengImage[a].pages[p].microWords && objHandle->readOnly)
	    free(objHandle->uniqueFengImage[a].pages[p].microWords);

	 /* free encapsulated pages */
	 if(objHandle->uniqueFengImage[a].pages) free(objHandle->uniqueFengImage[a].pages);
      }
   }

   if(objHandle->objLoaded) free(objHandle->fBuf);

   free(objHandle);

   /* delete string table, and other tables */
   /*
   objLoaded->strTable.tableLen = 0;
   objLoaded->strTable.strings = NULL;
   */

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_BindSymbol
   Description: Binds a value to the Ucode symbol in the UOF image
   Returns: UCLO_SUCCESS, UCLO_NOOBJ, UCLO_BADARG, UCLO_IMGNOTFND, UCLO_SYMNOTFND
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_BindSymbol(void *handle, char *ucodeImageName, char *ucodeSymName, int value)
{
	uof_encapFeng_T      *encapImage;
	uof_Image_T          *image;
	uof_encapPage_T      *pages;
	int                  p, validSym;
	uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

	if(!objHandle || !objHandle->fBuf) return(UCLO_NOOBJ);
	if(!ucodeImageName || !ucodeSymName) return(UCLO_BADARG);

	utl_strLowcase(ucodeImageName);
	utl_strLowcase(ucodeSymName);

	if(!(encapImage = UcLo_GetEncapImage(objHandle, ucodeImageName)))
	  return(UCLO_IMGNOTFND);

	/* the image exists */
	image = encapImage->images;
	pages = encapImage->pages;

   /* traverse the pages */
	for(p = 0; p < image->numOfPage; p++){
		uof_importVar_T   *impVar;
		int               v;

		/* update all instance of the variable */
		for(v = 0; v < pages[p].numImpVar; v++){
			impVar = &pages[p].importVar[v];

			if(!strcmp(ucodeSymName, UcLo_getString(&objHandle->strTable, impVar->varName))){
				char        *fields;
				int         uaddr, *uword;

				/* found one... update it */
				uaddr = ADDR_ATTR_UWORD(impVar->uwordAddress);
				fields = impVar->fieldAttrs;

				/* check that the address is within range */
				if(!(uword = Uclo_GetUofUword(&pages[p], uaddr))) return(UCLO_BADOBJ);

				/* printf("(debug dj:) UcLo_BindSymbol: Image ='%s', Sym=%s, Addr=%d\n", ucodeImageName, ucodeSymName, uaddr); */
				*uword = UcLo_setValue(*uword, fields, value);
				validSym = 1;
			}
		}
	}
	if(!validSym) return(UCLO_SYMNOTFND);
	return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_GetExpFunc
   Description: Locates an export function label in the UOF image
   Returns: pointer to the export function or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
uof_exportFunc_T *UcLo_GetExpFunc(void *handle, char *ucodeImageName, char *ucodeFuncName)
{
	uof_encapFeng_T      *encapImage;
	uof_Image_T           *image;
	uof_encapPage_T      *pages;
	int                  p;
	uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

	if(!objHandle || !objHandle->fBuf || !ucodeImageName || !ucodeFuncName) return(NULL);
	if(!(encapImage = UcLo_GetEncapImage(objHandle, ucodeImageName))) return(NULL);

	image = encapImage->images;
	pages = encapImage->pages;

	/* traverse the pages */
	for(p = 0; p < image->numOfPage; p++){
		uof_exportFunc_T *expFunc;
		int               f;

		/* search the pages for the function label */
		for(f = 0; f < pages[p].numExpFunc; f++){
			expFunc = &pages[p].exportFunc[f];
			if(!strcmp(ucodeFuncName, UcLo_getString(&objHandle->strTable, expFunc->funcName)))
			return(expFunc);
		}
	}
	return(NULL);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_GetFuncInfo
   Description: Locates an export function by searching all the images and updates
                the "funcInfo" struct with the appropriate information.
   Returns:     UCLO_SUCCESS, UCLO_BADARG, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_GetFuncInfo(void *objHandle, char *uFuncName, UcLo_FuncInfo_T *funcInfo)
{
   uclo_objHandle_T 	*handle = (uclo_objHandle_T *)objHandle;
   uof_exportFunc_T *exportFunc;
   int i;

   if(!funcInfo || !uFuncName || !handle || !handle->fBuf) return(UCLO_BADARG);

   /* traverse the images to find the function */
   for(i = 0; i < handle->numUniqueImage; i++)
      if((exportFunc = UcLo_GetExpFunc(handle, UcLo_getString(&handle->strTable,
                 handle->uniqueFengImage[i].images->imageName), uFuncName))){

         funcInfo->funcName = UcLo_getString(&handle->strTable, exportFunc->funcName);
	     funcInfo->funcAddress = exportFunc->funcAddress;

	     /* dj: exportFunc->funcType to be done */
	     funcInfo->funcType = GET_EXPFUNC_TYP(exportFunc->attr);
	     funcInfo->threadIds =  GET_EXPFUNC_TID(exportFunc->attr);
	     funcInfo->inArgBytes = exportFunc->inArgLen * 4;
	     funcInfo->outArgBytes = exportFunc->outArgLen * 4;
	     return(UCLO_SUCCESS);
	  }
      
   return(UCLO_FAILURE);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_BindFunction
   Description: Associates a name with an exported uCode function label in the
                UOF image.
   Returns: UCLO_SUCCESS, UCLO_TABLEFULL, UCLO_NOOBJ, UCLO_REDEFINE, UCLO_IMGNOTFND,
            UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_BindFunction(void *handle, char *bindName, char *ucodeImageName, char *ucodeFuncName)
{
   int          	i;
   uof_Image_T  	*image;
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;


   if(!objHandle || !objHandle->fBuf) return(UCLO_NOOBJ);
   if(objHandle->numBindFunc >= UCLO_MAXFUNC) return(UCLO_TABLEFULL);

   utl_strLowcase(ucodeImageName);
   utl_strLowcase(ucodeFuncName);

   for(i = 0; i < objHandle->numBindFunc; i++)
      if(!strcmp(bindName, objHandle->bindFuncTab[i].bindName)) return(UCLO_REDEFINE);

   if(!(objHandle->bindFuncTab[objHandle->numBindFunc].expFunc = 
             UcLo_GetExpFunc(objHandle, ucodeImageName, ucodeFuncName)))
      return(UCLO_FAILURE);

   if(!(image = UcLo_GetImage(objHandle, ucodeImageName))) return(UCLO_IMGNOTFND);
   objHandle->bindFuncTab[objHandle->numBindFunc].uEngFlag = 
           (short)IMAGE_MICRO_ENGINE(image->imageAttrib);
   objHandle->bindFuncTab[objHandle->numBindFunc].bindName = 
           (char *)malloc(strlen(bindName)+1);
   strcpy(objHandle->bindFuncTab[objHandle->numBindFunc].bindName, bindName);

   objHandle->numBindFunc++;
   
   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_WriteUimagePage
   Description: Writes the specified page in the UOF image to the assigned uEngines.
   Returns: UCLO_SUCCESS, UCLO_NOOBJ, UCLO_IMGNOTFND, UCLO_BADARG, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_WriteUimagePage(void *handle, char *ucodeImageName, int pageNum)
{
   int					i=0;
   uof_encapFeng_T      *encapImage;
   uof_encapPage_T      *pages;
   uof_Image_T          *image;
   unsigned int         uAddr = 0;
   unsigned char		eng;
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

   utl_strLowcase(ucodeImageName);

   if(!objHandle || !objHandle->fBuf) return(UCLO_NOOBJ);
   if(!(encapImage = UcLo_GetEncapImage(objHandle, ucodeImageName))) return(UCLO_IMGNOTFND);

   image = encapImage->images;
   pages = encapImage->pages;

   /* verify the page number */
   if(pageNum >= image->numOfPage) return(UCLO_BADARG);

   /* load the page starting a ustore address zero */
   for(eng = 0; eng < UOF_MAX_FENGINES; eng++)
      if(IS_BIT_SET(IMAGE_MICRO_ENGINE(image->imageAttrib), eng)){

	  if(UcLo_fixupLocals(&objHandle->strTable, &pages[pageNum], eng) != UCLO_SUCCESS)
			 return(UCLO_FAILURE);

#ifndef UCLO_WB_LIB	/* Workbench doesn't need these functionality */

		/* Write to micro-store and set the uPCs to the entry-point address */
#if (IOSTYLE==HARDWARE)

		 if(uEng_setNumUword(eng, uAddr, (unsigned short)pages[pageNum].numMicroWords,
			 (unsigned int *)&pages[pageNum].microWords[uAddr]) != UENG_SUCCESS){
			 return(UCLO_FAILURE);
		 }

		 /* set the second segment uwords */
		 if(pages[pageNum].numMicroWords_2){
			 if(uEng_setNumUword(eng, UOF_MAX_UWORDS_SEG, (unsigned short)pages[pageNum].numMicroWords_2,
								 (unsigned int *)&pages[pageNum].microWords[pages[pageNum].numMicroWords])
								 != UENG_SUCCESS){
				 return(UCLO_FAILURE);
			 }
		 }
		 if(uEng_setPC(eng, UENG_ALL_CTX, image->entryAddress) != UENG_SUCCESS){
			 return(UCLO_FAILURE);
		 }

#else
		 for(i = 0; i < pages[pageNum].numMicroWords; i++){
		    if(XactIoWrite(USTORE, (char)(1 << (eng & 0xff)), (unsigned int)pages[pageNum].microWords[i], i) != XACT_SUCCESS) return(UCLO_FAILURE);
		 }

		 if(pages[pageNum].numMicroWords_2){
			 uAddr = UOF_MAX_UWORDS_SEG;
			 for(i = 0; i < pages[pageNum].numMicroWords_2; i++){
				if(XactIoWrite(USTORE, (char)(1 << (eng & 0xff)), 
					      (unsigned int)pages[pageNum].microWords[pages[pageNum].numMicroWords+i],
					      uAddr++) != XACT_SUCCESS) return(UCLO_FAILURE);
			 }
		 }
		 /* dj: there are currently no XACT API to read/write the uPCs */

#endif

		 
		 /* fill unused uwords with pattern */
		 if(IMAGE_FILL_PATTERN(image->imageAttrib)){
#if (IOSTYLE==HARDWARE)

			/* fill the bottom unused portion of the first segment */
		    for(i = pages[pageNum].numMicroWords; i < UOF_MAX_UWORDS_SEG; i++){
		       if(uEng_setUword(eng, i, (unsigned int)image->fillPattern) != UENG_SUCCESS)
				   return(UCLO_FAILURE);
			}

			/* fill the bottom unused portion of the second segment */
			if(pages[pageNum].numMicroWords_2){
				for(i = UOF_MAX_UWORDS_SEG + pages[pageNum].numMicroWords_2; i < UOF_MAX_UWORDS; i++){
				   if(uEng_setUword(eng, i, (unsigned int)image->fillPattern) != UENG_SUCCESS)
					   return(UCLO_FAILURE);
				}
			}
#else
			/* fill the bottom unused portion of the first segment */
		    for(i = pages[pageNum].numMicroWords; i < UOF_MAX_UWORDS_SEG; i++){
		       if(XactIoWrite(USTORE, (char)(1 << (eng & 0xff)), (unsigned int)image->fillPattern, i) != XACT_SUCCESS) return(UCLO_FAILURE);
			}

			/* fill the bottom unused portion of the second segment */
			if(pages[pageNum].numMicroWords_2){
				for(i = UOF_MAX_UWORDS_SEG + pages[pageNum].numMicroWords_2; i < UOF_MAX_UWORDS; i++){
				   if(XactIoWrite(USTORE, (char)(1 << (eng & 0xff)), (unsigned int)image->fillPattern, i) != XACT_SUCCESS) return(UCLO_FAILURE);
				}
			}
#endif
		 }

#ifndef UCLO_XACT_LIB
         /* let the ueng lib know the number of useful micro words */
         if(uEng_setUstoreSegFreeMem(eng, pages[pageNum].numMicroWords,
								UOF_MAX_UWORDS_SEG - pages[pageNum].numMicroWords,
								UOF_MAX_UWORDS_SEG + pages[pageNum].numMicroWords_2,
								UOF_MAX_UWORDS_SEG - pages[pageNum].numMicroWords_2) != UENG_SUCCESS)
		    return(UCLO_FAILURE);
#endif

#endif		/* #ifndef UCLO_WB_LIB */
	  }

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_InitLib
   Description: Initializes the Loader Library
   Returns: 
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLo_InitLib()
{
#if !defined(UCLO_WB_LIB) && !defined(UCLO_XACT_LIB)
   static int onlyOnce = 0;

   if(onlyOnce) return;
   onlyOnce = 1;
   uEng_initUengs();
#endif  /* not UCLO_XACT_LIB or UCLO_WB_LIB */
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_WriteUimage
   Description: Writes the first page of the specified UOF image to the
                assigned uEngines.
   Returns: UCLO_SUCCESS, UCLO_NOOBJ, UCLO_IMGNOTFND, UCLO_BADARG, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_WriteUimage(void *handle, char *ucodeImageName)
{
   char pageNum = 0;
   if(!handle) return(UCLO_NOOBJ);
   return(UcLo_WriteUimagePage(handle, ucodeImageName, pageNum));
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_WriteUimageAll
   Description: Writes all of the UOF images to the assigned uEngines.
   Returns: UCLO_SUCCESS, UCLO_NOOBJ, UCLO_IMGNOTFND, UCLO_BADARG, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_WriteUimageAll(void *handle)
{
   int 			i, stat;
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

   if(!objHandle || !objHandle->fBuf){
	   return(UCLO_NOOBJ);
   }
   for(i = 0; i < objHandle->numUniqueImage; i++){
      if((stat = UcLo_WriteUimagePage(objHandle, UcLo_getString(&objHandle->strTable,
		  objHandle->uniqueFengImage[i].images->imageName), 0)) != UCLO_SUCCESS){
		return(UCLO_FAILURE);
	  }
	}

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_VerifyUimage
   Description: Compares the UOF image to the content of the assigned uEngines
   Returns: UCLO_SUCCESS, UCLO_NOOBJ, UCLO_IMGNOTFND, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_VerifyUimage(void *handle, char *ucodeImageName)
{
   uof_encapFeng_T      *encapImage;
   uof_Image_T          *image;
   char                 ueng;
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

   if(!objHandle || !objHandle->fBuf) return(UCLO_NOOBJ);

   utl_strLowcase(ucodeImageName);
   if(!(encapImage = UcLo_GetEncapImage(objHandle, ucodeImageName))) return(UCLO_IMGNOTFND);

   /* compare all assigned uEngine images */
   image = encapImage->images;
   for(ueng = 0; ueng < UOF_MAX_FENGINES; ueng++){
      if(UcLo_isFengSet(image->imageAttrib, ueng)){
	 if(UcLo_VerifyEncapUimage(&objHandle->strTable, encapImage, ueng)) return(UCLO_FAILURE);
      }
   }

   return(UCLO_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_VerifyUengine
   Description: Compares the content of the specified uEngine to the UOF image
   Returns: UCLO_SUCCESS, UCLO_NOOBJ, UCLO_IMGNOTFND, UCLO_FAILURE
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_VerifyUengine(void *handle, int uEngNum)
{
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

   if(!objHandle || !objHandle->fBuf) return(UCLO_NOOBJ);
   if(!objHandle->encapFengImage[uEngNum].pages ||
      !objHandle->encapFengImage[uEngNum].images) return(UCLO_IMGNOTFND);
   return(UcLo_VerifyEncapUimage(&objHandle->strTable, &objHandle->encapFengImage[uEngNum],
	      (unsigned char)uEngNum));
}



/*-----------------------------------------------------------------------------
   Function: 	UcLo_GetUfuncId
   Description: Gets the identification of the specified ucode function.
   Returns: the uAddress or UCLO_BADFUNCID
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_GetUfuncId(void *handle, char *ucodeFuncName)
{
   int i;
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

   if(!objHandle || !objHandle->fBuf || !ucodeFuncName) return(UCLO_BADFUNCID);

   utl_strLowcase(ucodeFuncName);

   /* find function address regardless of scope -- return the first match.
      Later we may want to return more than just the address. ie. context */
   for(i = 0; i < objHandle->numUniqueImage; i++){
      int j;

      /* search the pages of the image */
      for(j = 0; j < objHandle->uniqueFengImage[i].pages->numExpFunc; j++)
         if(!strcmp(UcLo_getString(&objHandle->strTable,
	                      objHandle->uniqueFengImage[i].pages->exportFunc[j].funcName),
			      ucodeFuncName))
	    return(objHandle->uniqueFengImage[i].pages->exportFunc[j].funcAddress);
   }

   return(UCLO_BADFUNCID);
}


/*-----------------------------------------------------------------------------
   Function: 	UcLo_GetVarMemSegs
   Description: Get the memory segment usage.
   Returns: UCLO_SUCCESS, UCLO_NOOBJ, UCLO_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLo_GetVarMemSegs(void *handle, UcLo_VarMemSeg_T *varMemSegs)
{
   uclo_objHandle_T 	*objHandle = (uclo_objHandle_T *)handle;

   if(!objHandle || !objHandle->fBuf) return(UCLO_NOOBJ);
   if(!varMemSegs) return(UCLO_BADARG);
   varMemSegs->sramBaseAddr = objHandle->encapUofObj.varMemSeg->sramBase;
   varMemSegs->sramSize = objHandle->encapUofObj.varMemSeg->sramSize;
   varMemSegs->sdramBaseAddr = objHandle->encapUofObj.varMemSeg->sdramBase;
   varMemSegs->sdramSize = objHandle->encapUofObj.varMemSeg->sdramSize;
   varMemSegs->scratchBaseAddr = objHandle->encapUofObj.varMemSeg->scratchBase;
   varMemSegs->scratchSize = objHandle->encapUofObj.varMemSeg->scratchSize;
   return(UCLO_SUCCESS);
}


