/* uclo_exam.c
 * An example of using the Ucode Object Loader Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Ucode Linker/Loader
 * author: drj Dec 31, 97 Initial version
 * revisions:
 * $History: uclo_exam.c $
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 11/30/00   Time: 5:01p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 8/31/00    Time: 3:13p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Updated for C0 changes
 *
 * ---------------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ueng.h"
#include "uclo.h"

static const char __copyright[]=
"Copyright Intel Corp, 1997-1998. All rights reserved";


/* uclo_exam.c:
	This code is an example of using the Ucode Loader Library (uclo.lib).
	Although it uses .\exam1.uc and .\exam2.uc; it
	does nothing useful with them and only serves as a desmonstration.

    This example illustrates the:
		1) loading of uof file
		2) initialization of ucode import variables
		3) obtaining ucode function identifications
		4) loading of UOF images to uStore
		5) comparing uof image against ustore image
		6) writing and reading uwords
		7) releasing uof object

    The ucode image files must have been created by the ucode linker, ucld, using
	the .\exam1.uc and .\exam2.uc files.

    To create the object file example.uof:
		$> uca exam1.uc exam2.uc
		$> ucld -u 012 exam1.list -u 345 exam2.list -o example.uof

	To run this program:
		$> ucloExam example.uof
*/

int Hi256_base = 0xff00;
int Hi64k_base = 0x00ff;
int Route_table_base = 0xf0f0;
int Trie_base = 0x0000;

#if(OS == VXWORKS)
int ucloExam(char *uofFile)
#else
int main(int argc, char **argv)
#endif
{
   int		i;
   unsigned int	bcopyId, bzeroId, value;
   char		*fnam = "example.uof";
   char		*image1 = "exam1", *image2 = "exam2";
   void		*objHandle;

#if(OS == VXWORKS)
   if(uofFile) fnam = uofFile;
#else
   for(i = 1; i < argc; i++){
      if(!strcmp(argv[i], "-h")){
         printf("Usage: %s [uof_file]\n", argv[0]);
         printf("   Where uof_file is the uCode image file that was created by UCLD.\n");
		 printf("   The default uof_file name is 'example.uof'\n");
		 exit(0);
	  }
      fnam = argv[i];   /* assume it's the input file name */
   }
#endif


   /* initialize the loader and ueng libs */
   UcLo_InitLib();


   /* Load the uCode object file */
   if(UcLo_LoadObjFile(&objHandle, fnam)){
      printf("Error; loading file %s\n", fnam);
      exit(-1);
   }

	/* stop uengines */
   for(i=0; i < 6; i++) uEng_disableCtx(i, 0xf);

   /* Bind and resolve Import Variables in image1 */
   if(UcLo_BindSymbol(objHandle, image1, "hi256_base", Hi256_base))
	   printf("Error; binding symbol 'hi256_base' in Image %s\n", image1);

   if(UcLo_BindSymbol(objHandle, image1, "hi64k_base", Hi64k_base))
	   printf("Error; binding symbol 'hi64k_base' in Image %s\n", image1);

   if(UcLo_BindSymbol(objHandle, image1, "route_table_base", Route_table_base))
	   printf("Error; binding symbol 'route_table_base' in Image %s\n", image1);

   if(UcLo_BindSymbol(objHandle, image1, "trie_base", Trie_base))
	   printf("Error; binding symbol 'trie_base' in Image %s\n", image1);


   /* write images1 & image2 to uStore */
   if(UcLo_WriteUimageAll(objHandle))
	   printf("Error; failed to write Image to uStore\n");

   /* compare image1 to the content of the assigned micro engine(s).  */
   if(UcLo_VerifyUimage(objHandle, image1) != UCLO_SUCCESS)
	   printf("Error; failed to verify image %s\n", image1);


   /* compare image2 to the content of the assigned micro engine(s).  */
   if(UcLo_VerifyUimage(objHandle, image2) != UCLO_SUCCESS)
	   printf("Error; failed to verify image %s\n", image2);

   /* set the last ustore location of ueng 0 to 0xf0f0f0f0 */
   if(UcLo_WriteUword(0x1, 1023, 0xf0f0f0f0) != UCLO_SUCCESS)
      printf("Error; writing to uEng0 address 1023\n");

   /* read the last ustore location of ueng 0 */
   if(UcLo_ReadUword(0x0, 1023, &value) != UCLO_SUCCESS)
	   printf("Error; reading from uEng0 address 1023\n");
   if(value != 0xf0f0f0f0)
	  printf("Error; incorrect value(0x%x) read from uEng0 address 1023\n", value);


   /* 
      get function ids for functions exam_func0# & exam_func1#
      A global search (search both images) will done to locate the function,
	  therefore, function names must be unique across images.
   */
   if((bcopyId = UcLo_GetUfuncId(objHandle, "exam_func0#")) < 0)
	   printf("Error; cannot get ID for function exam_func0#\n");

   if((bzeroId = UcLo_GetUfuncId(objHandle, "exam_func1#")) < 0)
	   printf("Error; cannot get ID for function exam_func1#\n");

   /* start the micro-engines */
  for(i = 0; i < MAX_UENGS; i++){
      if(uEng_start(i, 0, UENG_ALL_CTX) != UENG_SUCCESS)
	     printf("Error; cannot start micro-engine %d\n", i);
   }


   /* to stop the micro engines... */
   for(i = 0; i < MAX_UENGS; i++)
		uEng_disableCtx(i, UENG_ALL_CTX);

 
   /* trash the object and free allocated resources */
   UcLo_DeleObj(objHandle);
   printf("UcLoExam: tests completed\n");
   return(0);
}
