/* uclo_utl.c
 * Ucode Object File Loader utilities
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Ucode Linker/Loader
 * author: drj Dec 31, 97 Initial version
 * revisions:
 * $History: uclo_utl.c $
 * 
 * *****************  Version 23  *****************
 * User: Drjohns1     Date: 11/29/00   Time: 7:14p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 11/28/00   Time: 6:55p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Changed local variable prefix
 * 
 * *****************  Version 21  *****************
 * User: Drjohns1     Date: 11/21/00   Time: 1:26p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Added retrieval of variables memory segments values
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 11/15/00   Time: 3:45p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Added uC support
 * 
 * *****************  Version 18  *****************
 * User: Drjohns1     Date: 9/06/00    Time: 6:41p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Fixed CPU rev checking.
 * 
 * *****************  Version 17  *****************
 * User: Drjohns1     Date: 9/05/00    Time: 7:09p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Added CPU rev checking.
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 8/31/00    Time: 3:13p
 * Updated in $/dev_1200/SA1_CoreLibs/uclo
 * Updated for C0 changes
 * 
 * ---------------------------------------------------------------------
 */
#include <stdio.h>
#include <string.h>
#include "uclo.h"
#include "uclo_utl.h"
#include "utl_bits.h"
#include "uclo_ver.h"


static const char __copyright[]=
"Copyright Intel Corp, 1997-1998. All rights reserved";


unsigned int *Uclo_GetUofUword(uof_encapPage_T *page, unsigned int uAddr);


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnChunkHdr
   Description: Prints the UOF header
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnChunkHdr(FILE *fp, uof_chunkHdr_T *chunkHdr)
{
   fprintf(fp, "chunkId = %.*s, offset = %d, size = %d\n", 
	  UOF_OBJID_LEN, chunkHdr->chunkId, chunkHdr->offset, chunkHdr->size);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnUofObj
   Description: Prints the UOF file header
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnUofObj(FILE *fp, uof_objHdr_T *objHdr, uof_chunkHdr_T *chunkHdr)
{
   int i;

   fprintf(fp, "max_chunk = %d, num_chunks = %d\n", objHdr->maxChunks,
   	   objHdr->numChunks);

   for(i = 0; i < objHdr->numChunks; i++)
      fprintf(fp, "chunkId = %.*s, offset = %d, size = %d\n", 
	     UOF_OBJID_LEN, chunkHdr[i].chunkId, chunkHdr[i].offset, chunkHdr[i].size);

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnGenToolId
   Description: Prints the ID of the tool that generated the UOF
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnGenToolId(FILE *fp, uof_GTID_T *genToolId)
{
   fprintf(fp, "toolId=%.*s, toolVersion=%d\n",
	  UOF_OBJID_LEN, genToolId->toolId, genToolId->toolVersion);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnInitMemTable
   Description: Prints the content of a UOF init memory table
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnInitMemTable(FILE *fp, uclo_objHandle_T *objHandle)
{
   char *memRegion;
   unsigned int *values, i;
   short ii;
   uof_initMem_T *initMem;

   for(i = 0; i < objHandle->initMemTab.numEntries; i++){
	  initMem = &objHandle->initMemTab.initMem[i];
	  switch(initMem->region){
		  case 0: memRegion = "SRAM"; break;
		  case 1: memRegion = "SDRAM"; break;
		  case 2: memRegion = "SCRATCH"; break;
		  default: memRegion = "UNKNOWN"; break;
	  }
	  values = (unsigned int *)(objHandle->fBuf + initMem->valOffset);
      fprintf(fp, "%s, address=0x%x =[", memRegion, initMem->addr);
	  for(ii = 0; ii < initMem->numLwords; ii++)  fprintf(fp, " 0x%x", values[ii]);

	  fprintf(fp, "]\n");
   }

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnVarMemSeg
   Description: Prints the content of a UOF init memory table
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnVarMemSeg(FILE *fp, uclo_objHandle_T *objHandle)
{
   unsigned int i, numEntries = 1;
   uof_varMemSeg_T *varMemSeg;

   for(i = 0; i < numEntries; i++){
	  if(!(varMemSeg = &objHandle->encapUofObj.varMemSeg[i])) continue;
      fprintf(fp, "Sram: 0x%x:0x%x, Sdram: 0x%x:0x%x, Scratch: 0x%x:0x%x\n",
		  varMemSeg->sramBase, varMemSeg->sramSize,
		  varMemSeg->sdramBase, varMemSeg->sdramSize,
		  varMemSeg->scratchBase, varMemSeg->scratchSize);
   }

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnStrTable
   Description: Prints the content of a UOF string table
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnStrTable(FILE *fp, uclo_objHandle_T *objHandle)
{
   char *cPtr;

   for(cPtr = objHandle->strTable.strings; cPtr <
              (objHandle->strTable.strings +
	      objHandle->strTable.tableLen); cPtr += strlen(cPtr)+1)
      fprintf(fp, "String='%s'\n", cPtr);

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_uEngToStr
   Description: Converts the  uEngFlags flags to a string representation--uengine
                that are assigned are represented as a series of char digits.
   Returns: Null terminated string via the 'uEngStr' parameter.
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_uEngToStr(short uEngFlags, char *uEngStr)
{
   char   *digits = "0123456789";
   int    i, u = 0;

   for(i = 0; i < UOF_MAX_FENGINES; i++){
      if(isBitSet(uEngFlags, i))
	 uEngStr[u++] = digits[i];
   }
   uEngStr[u] = '\0';

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_uThreadTypToStr
   Description: Converts the Uthread types to a string representation--thread types
                are presented as a series of char digits.
   Returns: Null terminated string via the 'srv' and 'auton' parameters for Service
            and Autonomous type threads respectively.
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_uThreadTypToStr(short uThreadTypes, char *srv, char *auton)
{
   char   *digits = "0123456789";
   int    i, a = 0, s = 0;

   for(i = 0; i < UOF_MAX_THREADS; i++){
      if(isBitSet((unsigned int)uThreadTypes, i))
	 srv[s++] = digits[i];
      else auton[a++] = digits[i];
   }
   srv[s] = '\0';
   auton[a] = '\0';

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_uThreadToStr
   Description: Converts the Uthread number to a string representation--thread
                are presented as a series of char digits.
   Returns: Null terminated string via the 'uThreadsStr' parameter.
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_uThreadToStr(short uThreads, char *uThreadsStr)
{
   char   *digits = "0123456789";
   int    i, u = 0;

   for(i = 0; i < UOF_MAX_THREADS; i++){
      if(isBitSet(uThreads, i))
	 uThreadsStr[u++] = digits[i];
   }
   uThreadsStr[u] = '\0';

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnImage
   Description: Prints the contents of an UOF image.
   Returns: 
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnImage(FILE *fp, uof_strTab_T *strTable, uof_Image_T *image)
{
   char *image_name = "Unspecified";
   char uEngStr[UOF_MAX_FENGINES+1];
   char srvTypeStr[UOF_MAX_FENGINES+1];
   char autonTypeStr[UOF_MAX_FENGINES+1];
   char fillPat[16];

	if(IMAGE_FILL_PATTERN(image->imageAttrib))
		sprintf(fillPat, "0x%.8x", image->fillPattern);
	else strcpy(fillPat, "xxxxxxxx");

   if(image->imageName > -1) image_name = UcLo_getString(strTable, image->imageName);
   fprintf(fp, "ImageName=%s, numOfPage=%d, fillPattern=%s, entryPage=%d, entryAddress=%d, numOfPage=%d\n",
      image_name, image->numOfPage, fillPat, image->entryPage, image->entryAddress, image->numOfPage);

   UcLoUtl_uEngToStr((short)(IMAGE_MICRO_ENGINE(image->imageAttrib)), uEngStr);
   UcLoUtl_uThreadTypToStr((short)(IMAGE_THREAD_TYPE(image->imageAttrib)), srvTypeStr,
				     autonTypeStr);
   fprintf(fp, "uEng=(%s), ServiceThreads=(%s), AutonomousThreads=(%s),\n\
	  CpuType=0x%x, maxVer=0x%x, minVer=0x%x\n",
	  uEngStr, srvTypeStr, autonTypeStr, image->cpuType, image->maxVer, image->minVer);

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnEncapUofObj
   Description: Prints an encapsulated chunk file header
   Returns: 
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnEncapUofObj(FILE *fp, uclo_objHandle_T *objHandle)
{
   UcLoUtl_prnUofObj(fp, objHandle->encapUofObj.objHdr,
   			objHandle->encapUofObj.chunkHdr);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnUcVar
   Description: Prints the uC variables
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnUcVar(FILE *fp, uof_strTab_T *strTable, uof_ucVar_T *ucVar)
{
   int i, x, uwrdAddr;

   uwrdAddr = ADDR_ATTR_UWORD(ucVar->uwordAddress);

   fprintf(fp, "\t\t%s_var '%s' @ 0x%x, uAddr=%d, value=0x%x, fieldAttr=<",
	  (ucVar->scope) ? "Local" : "Global", UcLo_getString(strTable, ucVar->symName),
	  ucVar->value, uwrdAddr);

   for(i = 0, x=0; i < UOF_FIELD_POS_SIZE/3; i++, x += 3){
      if((ucVar->fieldAttrs[x] > 31) ||
	 (ucVar->fieldAttrs[x+1] > 31) ||
	 (ucVar->fieldAttrs[x+2] > 31)) break;
      fprintf(fp, " %d:%d:%d ", ucVar->fieldAttrs[x],
	     ucVar->fieldAttrs[x+1], ucVar->fieldAttrs[x+2]);
   }
   fprintf(fp, ">\n");

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnImpVar
   Description: Prints the import variables
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnImpVar(FILE *fp, uof_strTab_T *strTable, uof_importVar_T *importVar)
{
   int i, x, uwrdAddr;

   uwrdAddr = ADDR_ATTR_UWORD(importVar->uwordAddress);

   fprintf(fp, "\t\tName=%s, Address=%d, fieldAttr=<",
	  UcLo_getString(strTable, importVar->varName), uwrdAddr);

   for(i = 0, x=0; i < UOF_FIELD_POS_SIZE/3; i++, x += 3){
      if((importVar->fieldAttrs[x] > 31) ||
	 (importVar->fieldAttrs[x+1] > 31) ||
	 (importVar->fieldAttrs[x+2] > 31)) break;
      fprintf(fp, " %d:%d:%d ", importVar->fieldAttrs[x],
	     importVar->fieldAttrs[x+1], importVar->fieldAttrs[x+2]);
   }
   fprintf(fp, ">\n");

   return;
}



/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnExpFunc
   Description: Prints the export function labels
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnExpFunc(FILE *fp, uof_strTab_T *strTable, uof_exportFunc_T *exportFunc)
{
   char tidStr[UOF_MAX_THREADS+1];

   UcLoUtl_uThreadToStr((short)GET_EXPFUNC_TID(exportFunc->attr), tidStr);
   fprintf(fp, "\t\tName=%s, Type=%s, Id=%d, Threads=(%s), InArgLen=%d, OutArgLen=%d\n",
	  UcLo_getString(strTable, exportFunc->funcName),
	  	(GET_EXPFUNC_TYP(exportFunc->attr) == UOF_ONEWAY) ? "ONEWAY" : "TWOWAY",
	  	exportFunc->funcAddress, tidStr,
	        exportFunc->inArgLen, exportFunc->outArgLen);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnUwords
   Description: Prints a specified number of the micro-words starting a the
				location micro-address given by addr. If numUwrds is equal to
				zero,then the uwords within the range of the addr and the last
				valid uword will be printed. The addr and numUwrds
				will be adjusted if they result in an out of range address.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnNUwords(FILE *fp, uof_encapPage_T *encapPage,
						unsigned int addr, unsigned int numUwrds)
{
	unsigned int i, uWords, *uInst;

	if(!encapPage) return;

	if(encapPage->numMicroWords_2){
	   uWords = UOF_MAX_UWORDS;
	}
	else{
	   uWords = UOF_MAX_UWORDS_SEG;
	}

	if(addr >= uWords) addr = uWords - 1;

	if(numUwrds == 0 || (numUwrds + addr) > uWords)
	   numUwrds = uWords - addr;

	fprintf(fp, "\tMicroWords:\n");
	for(i = 0; i < numUwrds; i++){
	   if((uInst = Uclo_GetUofUword(encapPage, addr)))
			fprintf(fp, "\taddress=%d,\tuword=0x%.8x\n", addr, *uInst);
	   addr++;
	}
	return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnEncapPage
   Description: Prints encapsulated page
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnEncapPage(FILE *fp, uof_strTab_T *strTable,
						  uof_encapPage_T *encapPage, unsigned int numUwrds)
{
   int i;

   if(!encapPage) return;
   /* print the uwords starting at address zero */
   UcLoUtl_prnNUwords(fp, encapPage, 0, numUwrds);

   /* print the uC variables */
   fprintf(fp, "\tuC Variables:\n");
   for(i = 0; i < encapPage->numUcVar; i++)
      UcLoUtl_prnUcVar(fp, strTable, &encapPage->ucVar[i]);

   /* print the import variables */
   fprintf(fp, "\tImport Variables:\n");
   for(i = 0; i < encapPage->numImpVar; i++)
      UcLoUtl_prnImpVar(fp, strTable, &encapPage->importVar[i]);

   /* print the export functions */
   fprintf(fp, "\tExport Functions:\n");
   for(i = 0; i < encapPage->numExpFunc; i++)
      UcLoUtl_prnExpFunc(fp, strTable, &encapPage->exportFunc[i]);

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_prnEncapImage
   Description: Prints an encapsulated image
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void UcLoUtl_prnEncapImage(FILE *fp, uclo_objHandle_T *objHandle,
						   unsigned int numUwrds)
{
	int i;

	if(!objHandle) return;

	fprintf(fp, "\nUOF Object: CpuType=0x%x, minCpuVer=%d, maxCpuVer=%d\n",
		objHandle->encapUofObj.objHdr->cpuType,
		objHandle->encapUofObj.objHdr->minCpuVer,
		objHandle->encapUofObj.objHdr->maxCpuVer);

	for(i = 0; i < objHandle->numUniqueImage; i++){
		if(objHandle->uniqueFengImage[i].images){
			int ii;
			UcLoUtl_prnImage(fp, &objHandle->strTable, objHandle->uniqueFengImage[i].images);

			for(ii = 0; ii < objHandle->uniqueFengImage[i].images->numOfPage; ii++){
				fprintf(fp, "\tPage %d:\n", ii);
				UcLoUtl_prnEncapPage(fp, &objHandle->strTable, &objHandle->uniqueFengImage[i].pages[ii],
										numUwrds);
			}
		}
	}
	return;
}


/*-----------------------------------------------------------------------------
   Function: 	UcLoUtl_numOfChunkHdrType
   Description: Get the number of chunk types that are in the header
   Returns: The number of chunk matching the chunkId
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int UcLoUtl_numOfChunkHdrType(uof_encapUofObj_T *encapUofObj, char *chunkId)
{
   int i, chunks = 0;
 
   for(i = 0; i < encapUofObj->objHdr->numChunks; i++)
      if(!(strncmp(encapUofObj->chunkHdr[i].chunkId, chunkId, UOF_OBJID_LEN))) chunks++;
   return(chunks);
}
