/* ueng.cpp
 * Ucode Ueng Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: drj, June 16,98  Initial version
 *  revisions:
 *  $History: ueng.cpp $
 * 
 * *****************  Version 53  *****************
 * User: Ewalker      Date: 12/15/00   Time: 3:18p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Fixed problem where uEng_clrIregTh was incorrectly clearing breakpoint
 * interrupt bits along with ueng msg bits.
 * 
 * *****************  Version 52  *****************
 * User: Ewalker      Date: 11/27/00   Time: 2:28p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * 
 * *****************  Version 51  *****************
 * User: Ewalker      Date: 11/27/00   Time: 12:40p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Renamed uEng_getDeviceType to uEng_getCpuRev
 * 
 * *****************  Version 50  *****************
 * User: Ewalker      Date: 11/17/00   Time: 2:35p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Using MAX_USTORE_PER_SEG instead of UENG_MAX_USTORE for consistency.
 * Also improved argument checking for segUstoreFreeMem functions.
 * 
 * *****************  Version 49  *****************
 * User: Drjohns1     Date: 11/16/00   Time: 4:45p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Included core_io.h
 * 
 * *****************  Version 48  *****************
 * User: Ewalker      Date: 11/16/00   Time: 2:56p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Renamed getDeviceType function to uEng_getDeviceType so that core_io.h
 * can  define GetDeviceType to be either uEng_getDeviceType or
 * XACT_get_REV_CHIP depending on the value of IOSTYLE.
 * 
 * *****************  Version 47  *****************
 * User: Ewalker      Date: 11/09/00   Time: 10:52a
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Fixed SCR/bug #1242 - uengine::start was not shifting the starting
 * context into bit positions 2:3 when writing the CTX_ARB_CNTL register,
 * so context 0 was always starting instead of the desired context.
 * 
 * *****************  Version 46  *****************
 * User: Drjohns1     Date: 11/02/00   Time: 12:11p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Fixed set/get NumUword to read/write contigious ustore across segments
 * 
 * ---------------------------------------------------------------------
 */
#include <iostream.h>
#include <time.h>
#include "mem_map.h"
#include "core_io.h"
#include "utils.h"
#include "hal_fbi.h"
#include "hal_sdram.h"
#include "hal_pci.h"
#include "ueng.h"
#include "ueng_ver.h"


typedef struct{
   ISR_FUNC		funcPtr;
   int			param;
   unsigned int mask;
}UengInt_T;


static const char __copyright[] =
"Copyright Intel Corp, 1998. All rights reserved";


static const unsigned int TmpUaddr = 0;

static FbiUnit *Fbi;
static PciCSRs *Pci;
static UengInt_T UengInts[UENG_NUM_INTS];
static unsigned int UnhandledMask;
static sem_T GlobalSem;
static Uengine *Sa1200[MAX_UENGS];
static int CpuRev = IXP1200_REV_B0;

#define BITS_IN_WORD 32
static const unsigned int clrMasks[BITS_IN_WORD] ={
	0xfffffffe, 0xfffffffd, 0xfffffffb, 0xfffffff7,
	0xffffffef, 0xffffffdf, 0xffffffbf, 0xffffff7f,
	0xfffffeff, 0xfffffdff, 0xfffffbff, 0xfffff7ff,
	0xffffefff, 0xffffdfff, 0xffffbfff, 0xffff7fff,
	0xfffeffff, 0xfffdffff, 0xfffbffff, 0xfff7ffff,
	0xffefffff, 0xffdfffff, 0xffbfffff, 0xff7fffff,
	0xfeffffff, 0xfdffffff, 0xfbffffff, 0xf7ffffff,
	0xefffffff, 0xdfffffff, 0xbfffffff, 0x7fffffff};

static const unsigned int setMasks[BITS_IN_WORD] ={
	0x00000001, 0x00000002, 0x00000004, 0x00000008,
	0x00000010, 0x00000020, 0x00000040, 0x00000080,
	0x00000100, 0x00000200, 0x00000400, 0x00000800,
	0x00001000, 0x00002000, 0x00004000, 0x00008000,
	0x00010000, 0x00020000, 0x00040000, 0x00080000,
	0x00100000, 0x00200000, 0x00400000, 0x00800000,
	0x01000000, 0x02000000, 0x04000000, 0x08000000,
	0x10000000, 0x20000000, 0x40000000, 0x80000000};

#define SETBIT(value, bit) (setMasks[bit] | value)
#define CLRBIT(value, bit) (clrMasks[bit] & value)


enum {GPR_A = 0, GPR_B, SRAM_XFER, SDRAM_XFER, NODES};

enum {IN_RESET = 0, OUTOF_RESET =1};

#define UENG_IREG_BITS		0x07ffffff

enum{
	UPC_MASK			= 0x3ff,
	MSK_IREG_THD		= 0x00ffffff,
	CLR_IREG_THD		= 0xf8ffffff,
	MSK_IREG_BKPT		= 0x07000000,
	CLR_IREG_BKPT		= 0xf7000000,

	CLR_IREG_UEFIQ		= 0x70000000,
	SET_IREG_UEFIQ		= 0x80000000,
	CLR_IREG_UEIRQ		= 0xb0000000,
	SET_IREG_UEIRQ		= 0x40000000,

	CLR_IREG_CINTFIQ	= 0xd0000000,
	SET_IREG_CINTFIQ	= 0x20000000,
	CLR_IREG_CINTIRQ	= 0xe0000000,
	SET_IREG_CINTIRQ	= 0x10000000,

    MSK_IREG_ENABLES	= 0xf8000000
};

typedef struct{
	unsigned int addr1;
	unsigned int len1;
	unsigned int addr2;
	unsigned int len2;
}UengAddrSeg_T;


#define INIT_CTX_ARB_VALUE			0x0
#define INIT_PC_VALUE				0x0
#define INIT_WAKEUP_EVENTS_VALUE	0x1
#define INIT_SIG_EVENTS_VALUE		0x0

#define RST_CSR_FBOX_LSB			0
#define RST_CSR_FBOX_MSK			0x3f
#define RST_CSR_FBOX_MSK_SET	~(RST_CSR_FBOX_MSK << RST_CSR_FBOX_LSB)
#define GET_CSR_RST_FBOX(regVal) ((regVal >> RST_CSR_FBOX_LSB) & RST_CSR_FBOX_MSK)
#define SET_RESET_CSR_FBOX(regVal, fbMsk) regVal |= (fbMsk << RST_CSR_FBOX_LSB)
#define CLR_RESET_CSR_FBOX(regVal, fbMsk) regVal &= ~(fbMsk & RST_CSR_FBOX_MSK)

#ifdef _DEBUG
#define VERIFY_LIB(uengCsr) if(!uengCsr)return(UENG_BADLIB)
#define VERIFY_ARGPTR(arg) if(!arg)return(UENG_BADARG)
#else
#define VERIFY_LIB(uengCsr)
#define VERIFY_ARGPTR(arg)
#endif


/*-----------------------------------------------------------------------------
   Function: 	uEng_getCpuRev
   Description: Gets the IXP1200 device revision from the OS by masking out 
                all but the least significant 4 bits of the CPU ID. The
                expected revisions are currently: A3=0, B0=1, C0=3, D0=5
                (see IxpDevRev_T values)
   Returns:	
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int uEng_getCpuRev(void)
{
	return(0x0f & sysCpuIDGet());
}


/*-----------------------------------------------------------------------------
   Function: 	uEng_intConnect
   Description: Interrupt Service Routine for the micro-engines.
   Returns:	
   Modifies:
-----------------------------------------------------------------------------*/
int uEng_intConnect(int index, unsigned int intMask, ISR_FUNC isr, int param)
{
   int oldLevel, i;

   // is intMask within the bit we are servicing?
   if((intMask & ~UENG_IREG_BITS) || index >= UENG_NUM_INTS) return(UENG_BADARG);

   // check for multiple overlapping masks
   for(i = 0; i < UENG_NUM_INTS; i++)
	  if((i != index) && (UengInts[i].mask & intMask)) return(UENG_BADARG);

   oldLevel = int_lock();
   UengInts[index].funcPtr = isr;
   UengInts[index].mask = intMask;
   UengInts[index].param = param;
   UnhandledMask = UENG_IREG_BITS;

   for(i = 0; i < UENG_NUM_INTS; i++) UnhandledMask &= ~UengInts[i].mask;
   int_unlock(oldLevel);

   return(UENG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	uEng_disableCINTFIQ
   Description: Clear the external fast interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_disableCINTFIQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg &= CLR_IREG_CINTFIQ;
   Fbi->setIREG(ireg);
   return;
}



/*-----------------------------------------------------------------------------
   Function: 	enableCINTFIQ
   Description: Set the external fast interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_enableCINTFIQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg |= SET_IREG_CINTFIQ;
   Fbi->setIREG(ireg);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	uEng_disableCINTIRQ
   Description: Clear the external regular interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_disableCINTIRQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg &= CLR_IREG_CINTIRQ;
   Fbi->setIREG(ireg);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	uEng_enableCINTIRQ
   Description: Set the external regular interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_enableCINTIRQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg |= SET_IREG_CINTIRQ;
   Fbi->setIREG(ireg);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	uEng_disableUEFIQ
   Description: Clear the uengine fast interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_disableUEFIQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg &= CLR_IREG_UEFIQ;
   Fbi->setIREG(ireg);
   return;
}



/*-----------------------------------------------------------------------------
   Function: 	enableUEFIQ
   Description: Set the uengine fast interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_enableUEFIQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg |= SET_IREG_UEFIQ;
   Fbi->setIREG(ireg);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	uEng_disableUEIRQ
   Description: Clear the uengine regular interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_disableUEIRQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg &= CLR_IREG_UEIRQ;
   Fbi->setIREG(ireg);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	uEng_enableUEIRQ
   Description: Set the uengine regular interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_enableUEIRQ()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg |= SET_IREG_UEIRQ;
   Fbi->setIREG(ireg);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	clrIregBp
   Description: Clear the breakpoint interupt bit in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_clrIregBp()
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg &= CLR_IREG_BKPT;	// write one to clear bits 24-26
   Fbi->setIREG(ireg);
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	clrIregTh
   Description: Clear the uengine thread interupt bits in IREG.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_clrIregTh(unsigned int thMask)
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);
   ireg &= MSK_IREG_ENABLES | (thMask & CLR_IREG_THD);	// write one to clear bits 0-23
   Fbi->setIREG(ireg);
   return;
}

/*-----------------------------------------------------------------------------
   Function: 	uengIsr
   Description: Interrupt Service Routine for the micro-engines.
   Returns:	
   Modifies:
-----------------------------------------------------------------------------*/
void uengIsr(int param)
{
   unsigned int ireg;

   Fbi->getIREG(&ireg);

   for(int i = 0; i < UENG_NUM_INTS; i++)
	 if(ireg & UengInts[i].mask) (UengInts[i].funcPtr)(UengInts[i].param);

   // clear unassigned interrupts -- write one to clear

   if(ireg & UnhandledMask) Fbi->setIREG((MSK_IREG_ENABLES & ireg) | 
					(ireg & (UENG_IREG_BITS & UnhandledMask)));
   return;
}


/*-----------------------------------------------------------------------------
   Function: 	getAddrSeg
   Description: Returns the segments address and lengths to access the specified
				uwords.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void getAddrSeg(unsigned int uAddr, unsigned short numWords, UengAddrSeg_T *addrSeg)
{
	if(uAddr < MAX_USTORE_PER_SEG){
		// starts in seg1
		addrSeg->addr1 = uAddr;
		if((numWords + addrSeg->addr1) < MAX_USTORE_PER_SEG){
			addrSeg->len1 = numWords;
			addrSeg->len2 = 0;
		}
		else{
			//spawns two segments
			addrSeg->len1 = MAX_USTORE_PER_SEG - addrSeg->addr1;
			addrSeg->addr2 = MAX_USTORE_PER_SEG;
			addrSeg->len2 = numWords - addrSeg->len1;
		}
	}
	else{
		//starts in seg2
		addrSeg->len1 = 0;
		addrSeg->addr2 = uAddr;
		addrSeg->len2 = numWords;
	}

    if(addrSeg->len1 > 0)
		addrSeg->addr1 = SETBIT(addrSeg->addr1, ADDR_EN_BITPOS);	// enable address bit

	if(addrSeg->len2 > 0){
		addrSeg->addr2 = CLRBIT(addrSeg->addr2, USTORE_SEGMENT_BITPOS);
		addrSeg->addr2 = SETBIT(addrSeg->addr2, USTORE_2K_EN_BITPOS);			// enable 2k address bit
		addrSeg->addr2 = SETBIT(addrSeg->addr2, ADDR_EN_BITPOS);				// enable address bit
	}
	return;
}



/*-----------------------------------------------------------------------------
   Function: 	initUengs
   Description: Initialize the micro-engines and return a pointer to the
				objects.
   Returns:		An 8-bit register address with the upper two bit set.
   Modifies:
-----------------------------------------------------------------------------*/
Uengine **initUengs()
{
   static unsigned init = 0;

   if(!init){
      int i;
      const int isrParam = 0;

	  Fbi = FBI_Attach();
	  Pci = (PciCSRs *)RESET_ENABLE_REG;	// starting at address 0x42000000 + 0x7c

	  // init and attach the uengine ISR
	  UnhandledMask = UENG_IREG_BITS;
	  for(i = 0; i < UENG_NUM_INTS; i++){
		 UengInts[i].funcPtr = NULL;
		 UengInts[i].mask = 0;
	  }

	  // create semaphore to protect global variables
	  GlobalSem = Sem_Init(1);

	  // create uengine objects
	  for(i = 0; i < MAX_UENGS; i++) Sa1200[i] = new Uengine(i);

      uEng_clrResetUengs(UENG_ALL_UENG);	// take engines out of reset

	  for(i = 0; i < MAX_UENGS; i++){
		 // The hardware should powerup to these default values, but we'll set them
		 // anyway...just to make sure we're starting from a known point
		 Sa1200[i]->disableCtx(UENG_ALL_CTX);			// disable contexts
		 Sa1200[i]->setPC(UENG_ALL_CTX, INIT_PC_VALUE);	// initialize the PCs
		 Sa1200[i]->setCtxArb(INIT_CTX_ARB_VALUE);		// init the ctx_arb
		 Sa1200[i]->setCtxWakeupEvents(UENG_ALL_CTX, INIT_WAKEUP_EVENTS_VALUE);
		 Sa1200[i]->setCtxSigEvents(UENG_ALL_CTX, INIT_SIG_EVENTS_VALUE);
	  }

	  // connect to system interrupt, and enable uengine irq
	  if(int_connect(SAINT_UENG, uengIsr, isrParam)) return(NULL);
	  uEng_enableUEIRQ();

      init = 1;
   }

   return(Sa1200);
}

/*-----------------------------------------------------------------------------
   Function: 	uEng_resetUengs
   Description: Reset the specified micro-engines and conditionally clear the
				registers to their power-up state.
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_resetUengs(unsigned int uEngMask, int clrReg)
{
   unsigned int data;

   Sem_Wait(GlobalSem, SEM_WAITFOREVER);

   Pci->getResetReg(&data);
   SET_RESET_CSR_FBOX(data, uEngMask);
   Pci->setResetReg(data);

   for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++){
	   if((uEngMask & setMasks[ueng])){

		   if(clrReg ){
			  Sa1200[ueng]->disableCtx(UENG_ALL_CTX);			// disable contexts
			  Sa1200[ueng]->setPC(UENG_ALL_CTX, INIT_PC_VALUE);	// initialize the PCs
			  Sa1200[ueng]->setCtxArb(INIT_CTX_ARB_VALUE);		// init the ctx_arb
			  Sa1200[ueng]->setCtxWakeupEvents(UENG_ALL_CTX, INIT_WAKEUP_EVENTS_VALUE);
			  Sa1200[ueng]->setCtxSigEvents(UENG_ALL_CTX, INIT_SIG_EVENTS_VALUE);
		   }

		   Sa1200[ueng]->setRst();   // set the reset-state indicator
	   }
   }

   Sem_Post(GlobalSem);
   return;
}

/*-----------------------------------------------------------------------------
   Function: 	clrResetUengs
   Description: Take the specified micro-engines out of reset
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_clrResetUengs(unsigned int uEngMask)
{
   unsigned int data;

   Sem_Wait(GlobalSem, SEM_WAITFOREVER);

   Pci->getResetReg(&data);
   CLR_RESET_CSR_FBOX(data, uEngMask);
   Pci->setResetReg(data);

   // clear the reset indicator
   for(unsigned int ueng = 0; ueng < MAX_UENGS; ueng++)
      if((uEngMask & setMasks[ueng])) Sa1200[ueng]->clrRst();

   // re-establish UENG IRQ -- just in case a fbi-reset did occurred
   uEng_enableUEIRQ();

   Sem_Post(GlobalSem);
   return;
}

/*-----------------------------------------------------------------------------
   Function: 	getReg6bitAddr
   Description: Get the 6-bit address of the specified register number.
   Returns:		An 8-bit register address with the upper two bit set.
   Uses:		type: 0: GPR-A
				      1: GPR-B
					  2: SRAM
			          3: SDRAM
			  		  4: NODES
   Modifies:
-----------------------------------------------------------------------------*/
unsigned char getReg6bitAddr(unsigned char type, unsigned short regNum)
{
   unsigned char regAddr = 0xc0;

   if(type == GPR_A || type == GPR_B){				// GPR-A or GPR-B
      regAddr |= (type & 0x3) << 4;
	  regAddr |= regNum & 0xf;
   }
   else if(type == SRAM_XFER || type == SDRAM_XFER){	// SRAM or SDRAM
	  unsigned char bits3to5 = 4;
	  if(type == SDRAM_XFER) bits3to5 = 5;
	  regAddr |= (unsigned char)(bits3to5 << 3) | regNum & 0x7;
   }
   else	regAddr = 0xff;						// NODEST

   return(regAddr);
}


/*-----------------------------------------------------------------------------
   Function: 	getReg5bitAddr
   Description: Get the 5-bit address of the specified register number.
   Returns:		An 8-bit register address with the upper 3 bits set.
   Uses:		type: 0: GPR-A
				      1: GPR-B
					  2: SRAM
			          3: SDRAM
   Modifies:
-----------------------------------------------------------------------------*/
unsigned char getReg5bitAddr(unsigned char type, unsigned short regNum)
{
   unsigned char regAddr = 0;

   if(type == GPR_A || type == GPR_B){					// GPR-A or GPR-B
	  regAddr |= regNum & 0xf;
   }
   else if(type == SRAM_XFER || type == SDRAM_XFER){	// SRAM or SDRAM
	  unsigned char bits3to4 = 2;
	  if(type == SDRAM_XFER) bits3to4 = 3;
	  regAddr |= (unsigned char)(bits3to4 << 3) | (regNum & 0x7);
   }

   return(regAddr);
}


/*-----------------------------------------------------------------------------
   Function: 	getReg8bitAbsSrcAddr
   Description: Get the 8-bit absolute source address of the register number.
   Returns:		An 8-bit register address
   Uses:		type: 0: GPR-A
				      1: GPR-B
					  2: SRAM
			          3: SDRAM
   Modifies:
-----------------------------------------------------------------------------*/
inline unsigned char getReg8bitAbsSrcAddr(unsigned char type, unsigned short ctx,
										  unsigned short regNum)
{
   return(((ctx & 0x3) << 5) | getReg5bitAddr(type, regNum));
}

/*-----------------------------------------------------------------------------
   Function: 	getReg8bitRelSrcAddr
   Description: Get the 8-bit relative source address of the register number.
   Returns:		An 8-bit register address
   Uses:		type: 0: GPR-A
				      1: GPR-B
					  2: SRAM
			          3: SDRAM
   Modifies:
-----------------------------------------------------------------------------*/
inline unsigned char getReg8bitRelSrcAddr(unsigned char type, unsigned short regNum)
{
   return(((0x4 & 0x3) << 5) | getReg5bitAddr(type, regNum));
}


/*-----------------------------------------------------------------------------
   Function: 	getReg8bitAddr
   Description: Get the 8-bit address of the specified register number.
   Returns:		An 8-bit register address
   Uses:		type: 0: GPR-A
				      1: GPR-B
					  2: SRAM
			          3: SDRAM
			  		  4: NODES
   Modifies:
-----------------------------------------------------------------------------*/
unsigned char getReg8bitAddr(unsigned char type, unsigned short ctx, unsigned short regNum)
{
   unsigned char regAddr = 0;

   if(type == GPR_A || type == GPR_B){					  // GPR-A or GPR-B
      regAddr = (type & 0x3) << 6;
      regAddr |= (ctx & 0x3) << 4;
	  regAddr |= regNum & 0xf;
   }
   else if(type == SRAM_XFER || type == SDRAM_XFER){	// SRAM or SDRAM
	  unsigned char bit3 = 0;
	  if(type == SDRAM_XFER) bit3 = 1;
      regAddr = (unsigned char)(0x2 << 6);
      regAddr |= (ctx & 0x3) << 4;
	  regAddr |= (bit3 << 3) | (regNum & 0x7);
   }
   else regAddr = 0xff;							// no destination

   return(regAddr);
}



/*-----------------------------------------------------------------------------
   Function: 	Uengine::Uengine
   Description: Uengine class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
Uengine::Uengine(unsigned char uEngNum)
{
	switch(uEngNum){
	  case 0:
		 uEngCsr = (UengCSRs *)UENG0_CSR_BASE_ADDR;
		 sramXferReg[0] = (SramXferReg *)UENG0_CTX0_SRAMXFER_BASE_ADDR;
		 sramXferReg[1] = (SramXferReg *)UENG0_CTX1_SRAMXFER_BASE_ADDR;
		 sramXferReg[2] = (SramXferReg *)UENG0_CTX2_SRAMXFER_BASE_ADDR;
		 sramXferReg[3] = (SramXferReg *)UENG0_CTX3_SRAMXFER_BASE_ADDR;
		 break;
	  case 1:
		 uEngCsr = (UengCSRs *)UENG1_CSR_BASE_ADDR;
		 sramXferReg[0] = (SramXferReg *)UENG1_CTX0_SRAMXFER_BASE_ADDR;
		 sramXferReg[1] = (SramXferReg *)UENG1_CTX1_SRAMXFER_BASE_ADDR;
		 sramXferReg[2] = (SramXferReg *)UENG1_CTX2_SRAMXFER_BASE_ADDR;
		 sramXferReg[3] = (SramXferReg *)UENG1_CTX3_SRAMXFER_BASE_ADDR;
		 break;
	  case 2:
		 uEngCsr = (UengCSRs *)UENG2_CSR_BASE_ADDR;
		 sramXferReg[0] = (SramXferReg *)UENG2_CTX0_SRAMXFER_BASE_ADDR;
		 sramXferReg[1] = (SramXferReg *)UENG2_CTX1_SRAMXFER_BASE_ADDR;
		 sramXferReg[2] = (SramXferReg *)UENG2_CTX2_SRAMXFER_BASE_ADDR;
		 sramXferReg[3] = (SramXferReg *)UENG2_CTX3_SRAMXFER_BASE_ADDR;
		 break;
	  case 3:
		 uEngCsr = (UengCSRs *)UENG3_CSR_BASE_ADDR;
		 sramXferReg[0] = (SramXferReg *)UENG3_CTX0_SRAMXFER_BASE_ADDR;
		 sramXferReg[1] = (SramXferReg *)UENG3_CTX1_SRAMXFER_BASE_ADDR;
		 sramXferReg[2] = (SramXferReg *)UENG3_CTX2_SRAMXFER_BASE_ADDR;
		 sramXferReg[3] = (SramXferReg *)UENG3_CTX3_SRAMXFER_BASE_ADDR;
		 break;
	  case 4:
		 uEngCsr = (UengCSRs *)UENG4_CSR_BASE_ADDR;
		 sramXferReg[0] = (SramXferReg *)UENG4_CTX0_SRAMXFER_BASE_ADDR;
		 sramXferReg[1] = (SramXferReg *)UENG4_CTX1_SRAMXFER_BASE_ADDR;
		 sramXferReg[2] = (SramXferReg *)UENG4_CTX2_SRAMXFER_BASE_ADDR;
		 sramXferReg[3] = (SramXferReg *)UENG4_CTX3_SRAMXFER_BASE_ADDR;
		 break;
	  case 5:
		 uEngCsr = (UengCSRs *)UENG5_CSR_BASE_ADDR;
		 sramXferReg[0] = (SramXferReg *)UENG5_CTX0_SRAMXFER_BASE_ADDR;
		 sramXferReg[1] = (SramXferReg *)UENG5_CTX1_SRAMXFER_BASE_ADDR;
		 sramXferReg[2] = (SramXferReg *)UENG5_CTX2_SRAMXFER_BASE_ADDR;
		 sramXferReg[3] = (SramXferReg *)UENG5_CTX3_SRAMXFER_BASE_ADDR;
		 break;
	  default: uEngCsr = NULL; break;
	}
	thisUengNum = uEngNum;
	freeAddr = 0;											// set available ustores
	freeSize = MAX_USTORE_PER_SEG;

	CpuRev = uEng_getCpuRev();
	if(CpuRev == IXP1200_REV_C0){
		uStoreSize = MAX_C0_USTORE;
		freeAddr2 = MAX_USTORE_PER_SEG;						// set available ustores
		freeSize2 = MAX_USTORE_PER_SEG;
	}
	else{
		uStoreSize = MAX_PRE_C0_USTORE;
		freeAddr2 = MAX_C0_USTORE ;
		freeSize2 = 0;
	}

	uEngSem = Sem_Init(1);
	semLock = LOW;
	return;
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::reset
   Description: Reset the micro-engine
   Returns:	
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void Uengine::reset(int clrReg)
{
   uEng_resetUengs(setMasks[thisUengNum], clrReg);
   return;
}

/*-----------------------------------------------------------------------------
   Function: 	Uengine::clrReset
   Description: Take the micro-engine out of reset
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void Uengine::clrReset()
{
   uEng_clrResetUengs(setMasks[thisUengNum]);
   rst = 0;

   return;
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::start
   Description: Start micro-engine from PC zero
   Returns:		UENG_SUCCESS, UENG_BADARG,UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::start(unsigned char startCtx, unsigned int ctxEnMask)
{
   VERIFY_LIB(uEngCsr);
   if(rst) clrReset();

   // verify that the starting context will be enabled
   if(!(ctxEnMask & setMasks[startCtx])) return(UENG_BADARG);

   const unsigned int sigEvents = 0x0, voluntaryWakeupEvent = 0x1, ccEnable= 0x2fff;
   setCtxArb((startCtx << 2) & 0xC);    							// set start context
   setCtxWakeupEvents(setMasks[startCtx], voluntaryWakeupEvent);	// wakeup voluntary
   setCtxSigEvents(setMasks[startCtx], sigEvents);					// set sig events
   setCondCodeEn(ccEnable);											// enable cc
   enableCtx(ctxEnMask);											// enable context

   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setUstoreFreeMem
   Description: Define the ustore free region of both ustore segments
   Returns:		UENG_SUCCESS, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setUstoreFreeMem(unsigned int seg1FreeAddr, unsigned int seg1FreeSize,
							  unsigned int seg2FreeAddr, unsigned int seg2FreeSize)
{
	if((seg1FreeAddr > MAX_USTORE_PER_SEG) || 
       ((seg1FreeAddr + seg1FreeSize) > MAX_USTORE_PER_SEG))
		return(UENG_BADARG);
	freeAddr = seg1FreeAddr;
	freeSize = seg1FreeSize;

	if(CpuRev == IXP1200_REV_C0){
		if((seg2FreeAddr < MAX_USTORE_PER_SEG) || (seg2FreeAddr > MAX_C0_USTORE) || 
           ((seg2FreeAddr + seg2FreeSize) > MAX_C0_USTORE))
			return(UENG_BADARG);
	   freeAddr2 = seg2FreeAddr;
	   freeSize2 = seg2FreeSize;
	}
	return(UENG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Uengine::getUstoreFreeMem
   Description: Get the ustore free region
   Returns:		UENG_SUCCESS, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getUstoreFreeMem(unsigned int *seg1FreeAddr, unsigned int *seg1FreeSize,
							  unsigned int *seg2FreeAddr, unsigned int *seg2FreeSize)
{
   if(!seg1FreeAddr || !seg1FreeSize || !seg2FreeAddr || !seg2FreeSize) return(UENG_BADARG);
   *seg1FreeAddr = freeAddr;
   *seg1FreeSize = freeSize;
   *seg2FreeAddr = freeAddr2;
   *seg2FreeSize = freeSize2;
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setUstoreFreeMem
   Description: Define the ustore free region
   Returns:		UENG_SUCCESS, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setUstoreFreeMem(unsigned int begFreeAddr, unsigned int size)
{
   if((begFreeAddr > MAX_USTORE_PER_SEG) ||
      ((begFreeAddr + size) > MAX_USTORE_PER_SEG))
	   return(UENG_BADARG);
   freeAddr = begFreeAddr;
   freeSize = size;
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::getUstoreFreeMem
   Description: Get the ustore free region
   Returns:		UENG_SUCCESS, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getUstoreFreeMem(unsigned int *begFreeAddr, unsigned int *size)
{
   if(!begFreeAddr || !size) return(UENG_BADARG);
   *begFreeAddr = freeAddr;
   *size = freeSize;
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::disableCtx
   Description: Disable the specified micro-thread(s).
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::disableCtx(unsigned int ctxMask)
{
   unsigned int ctxEn, wrCtxEn;
   int key = lockAccess(HIGH);

   if(rst){
      unlockAccess(key);
	  return(UENG_RESET);
   }
   // get the context that are enabled
   getCtxEnables(&ctxEn);

   wrCtxEn = (~(ctxMask << CTX_ENABLES_BITPOS)) & ctxEn;	// set the selected bits
   uEngCsr->setCTX_ENABLES(wrCtxEn);							// write csr

   unlockAccess(key);
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::enableCtx
   Description: Enable the specified micro-thread(s) -- those that were
				already enabled, will remain enabled.
   Returns:		UENG_SUCCESS, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::enableCtx(unsigned int ctxMask)
{
   unsigned int ctxEn, wrCtxEn;
   int key = lockAccess(HIGH);

   if(rst){
      unlockAccess(key);
	  return(UENG_RESET);
   }

   // get the context that are enabled
   getCtxEnables(&ctxEn);

   wrCtxEn = ((ctxMask & 0xf) << CTX_ENABLES_BITPOS) | ctxEn; // set selected bits high
   uEngCsr->setCTX_ENABLES(wrCtxEn);

   unlockAccess(key);
   return(UENG_SUCCESS);
}


									   
/*-----------------------------------------------------------------------------
   Function: 	Uengine::getPC
   Description: Get the program counter of the specified context.
   Returns:		UENG_SUCCESS, UENG_FAIL,UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getPC(unsigned char ctx, unsigned int *upc)
{
   unsigned int ctxStatus;

   VERIFY_LIB(uEngCsr);
   VERIFY_ARGPTR(upc);
   switch(ctx){
      case 0: uEngCsr->getCTX_0_STS(&ctxStatus); break;
      case 1: uEngCsr->getCTX_1_STS(&ctxStatus); break;
      case 2: uEngCsr->getCTX_2_STS(&ctxStatus); break;
      case 3: uEngCsr->getCTX_3_STS(&ctxStatus); break;
	  default: return(UENG_BADARG);
   }
   *upc = ctxStatus & UPC_MASK;
   if(ctxStatus & CTX_X_USTORE_SEGMENT_MASK){
	   *upc = SETBIT(*upc, USTORE_SEGMENT_BITPOS);
   }
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setPC
   Description: Set the indicated threads to a specified uPC.
   Returns:		UENG_SUCCESS, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setPC(unsigned int ctxMask, unsigned int upc)
{
	VERIFY_LIB(uEngCsr);
	unsigned int val;

	val = upc & UPC_MASK;
	if(upc & (1 << USTORE_SEGMENT_BITPOS)){
		val = SETBIT(val, CTX_X_USTORE_SEGMENT_BITPOS);		// enable 2k address bit
	}

	for(int ctx = 0; ctx < MAX_CTX; ctx++){
	  if(ctxMask & setMasks[ctx]){
		 switch(ctx){
			case 0: uEngCsr->setCTX_0_STS(val); break;
			case 1: uEngCsr->setCTX_1_STS(val); break;
			case 2: uEngCsr->setCTX_2_STS(val); break;
			case 3: uEngCsr->setCTX_3_STS(val); break;
		 }
	  }
	}

   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setCtxStatus
   Description: Set the context status of the specified context.
   Returns:		UENG_SUCCESS, UENG_BADARG,UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setCtxStatus(unsigned int ctxMask, unsigned int ctxStatus)
{
   VERIFY_LIB(uEngCsr);

   for(int ctx = 0; ctx < MAX_CTX; ctx++){
      // save signaled events
      if(ctxMask & setMasks[ctx]){

	     switch(ctx){
			case 0: uEngCsr->setCTX_0_STS(ctxStatus); break;
			case 1: uEngCsr->setCTX_1_STS(ctxStatus); break;
			case 2: uEngCsr->setCTX_2_STS(ctxStatus); break;
			case 3: uEngCsr->setCTX_3_STS(ctxStatus); break;
			default: return(UENG_BADARG);
		 }
	  }
   }
   return(UENG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	Uengine::getCtxStatus
   Description: Get the context status of the specified context.
   Returns:		UENG_SUCCESS, UENG_BADARG,UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getCtxStatus(unsigned char ctx, unsigned int *ctxStatus)
{
   VERIFY_LIB(uEngCsr);
   VERIFY_ARGPTR(ctxStatus);
   switch(ctx){
      case 0: uEngCsr->getCTX_0_STS(ctxStatus); break;
      case 1: uEngCsr->getCTX_1_STS(ctxStatus); break;
      case 2: uEngCsr->getCTX_2_STS(ctxStatus); break;
      case 3: uEngCsr->getCTX_3_STS(ctxStatus); break;
	  default: return(UENG_BADARG);
   }
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::getCtxSigEvents
   Description: Get the state of the sig-event signals.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getCtxSigEvents(unsigned char ctx, unsigned int *events)
{
   VERIFY_LIB(uEngCsr);
   VERIFY_ARGPTR(events);

   switch(ctx){
      case 0: uEngCsr->getCTX_0_SIG_EVENTS(events); break;
      case 1: uEngCsr->getCTX_1_SIG_EVENTS(events); break;
      case 2: uEngCsr->getCTX_2_SIG_EVENTS(events); break;
      case 3: uEngCsr->getCTX_3_SIG_EVENTS(events); break;
	  default: return(UENG_BADARG);
   }

   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setCtxSigEvents
   Description: Write the state of the sig-event signals.
   Returns:		UENG_SUCCESS, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setCtxSigEvents(unsigned int ctxMask, unsigned int events)
{
   VERIFY_LIB(uEngCsr);

   for(int ctx = 0; ctx < MAX_CTX; ctx++){
      // save signaled events
      if(ctxMask & setMasks[ctx]){
         switch(ctx){
            case 0: uEngCsr->setCTX_0_SIG_EVENTS(events); break;
            case 1: uEngCsr->setCTX_1_SIG_EVENTS(events); break;
            case 2: uEngCsr->setCTX_2_SIG_EVENTS(events); break;
            case 3: uEngCsr->setCTX_3_SIG_EVENTS(events); break;
		 }
	  }
   }

   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::getCtxWakeupEvents
   Description: Save the state of the wakeup-event signals.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getCtxWakeupEvents(unsigned char ctx, unsigned int *events)
{
   VERIFY_LIB(uEngCsr);
   VERIFY_ARGPTR(events);

   switch(ctx){
      case 0: uEngCsr->getCTX_0_WAKEUP_EVENTS(events); break;
      case 1: uEngCsr->getCTX_1_WAKEUP_EVENTS(events); break;
      case 2: uEngCsr->getCTX_2_WAKEUP_EVENTS(events); break;
      case 3: uEngCsr->getCTX_3_WAKEUP_EVENTS(events); break;
	  default: return(UENG_BADARG);
   }

   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setCtxWakeupEvents
   Description: Restore the state of the wakeup-event signals.
   Returns:		UENG_SUCCESS, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setCtxWakeupEvents(unsigned int ctxMask, unsigned int events)
{
   VERIFY_LIB(uEngCsr);

   if(rst) return(UENG_RESET);

   for(int ctx = 0; ctx < MAX_CTX; ctx++){
      // save signaled events
      if(ctxMask & setMasks[ctx]){
         switch(ctx){
            case 0: uEngCsr->setCTX_0_WAKEUP_EVENTS(events); break;
            case 1: uEngCsr->setCTX_1_WAKEUP_EVENTS(events); break;
            case 2: uEngCsr->setCTX_2_WAKEUP_EVENTS(events); break;
            case 3: uEngCsr->setCTX_3_WAKEUP_EVENTS(events); break;
		 }
	  }
   }

   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::isUengActive
   Description: Determine if the Uengine is active -- if bit 2 of ctx_active_sts
				is set.
   Returns:		UENG_ACTIVE,UENG_INACTIVE,UENG_BADLIB,UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::isUengActive()
{
   unsigned int csr;
   VERIFY_LIB(uEngCsr);

   if(rst) return(UENG_RESET);
   uEngCsr->getACTIVE_CTX_STS(&csr);
   if(csr & CTX_ACTIVE_AB_MASK) return(UENG_ACTIVE);

   return(UENG_INACTIVE);
}



/*-----------------------------------------------------------------------------
   Function: 	Uengine::getUword
   Description: Read a long-word from micro-store. The micro engine must be
				inactive before reading from micro-store.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_ACTIVE, UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getUword(unsigned int uAddr, unsigned int *uWord)
{
	return(getNumUword(uAddr, 1, uWord));
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setUword
   Description: Write the a long-word to micro-store.  The micro engine must be
				inactive, possible under reset, before writing to micro-store.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_ACTIVE, UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setUword(unsigned int uAddr, unsigned int uWord)
{
	return(setNumUword(uAddr, 1, &uWord));
}



/*-----------------------------------------------------------------------------
   Function: 	Uengine::getNumUword
   Description: Read a number oflong-words from micro-store. The micro engine must be
				inactive before reading from micro-store.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_ACTIVE, UENG_BADARG, UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getNumUword(unsigned int uAddr, unsigned short numWords, unsigned int *uWord)
{
	int status;
	unsigned int i, uwIndx = 0;
	UengAddrSeg_T addrSeg; 

	if(!uWord || (uAddr + numWords) > uStoreSize) return(UENG_BADARG);

	int key = lockAccess(HIGH);
	if((status = isUengActive()) != UENG_INACTIVE){
		unlockAccess(key);
		return(status);
	}

	getAddrSeg(uAddr, numWords, &addrSeg);
	for(i = 0; i < addrSeg.len1; i++){
		uEngCsr->setUSTORE_ADDRESS(addrSeg.addr1++);
		uEngCsr->getUSTORE_DATA(&uWord[uwIndx++]);
	}
	for(i = 0; i < addrSeg.len2; i++){
	   uEngCsr->setUSTORE_ADDRESS(addrSeg.addr2++);
	   uEngCsr->getUSTORE_DATA(&uWord[uwIndx++]);
	}


	uEngCsr->setUSTORE_ADDRESS(0);						// clear addr enable bit
	unlockAccess(key);

	return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setNumUword
   Description: Write a number of long-words to micro-store.  The micro engine must be
				inactive, possible under reset, before writing to micro-store.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_ACTIVE, UENG_BADARG, UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setNumUword(unsigned int uAddr, unsigned short numWords, unsigned int *uWord)
{
	int status;
	unsigned int i, uwIndx = 0; 
	UengAddrSeg_T addrSeg; 

	if(!uWord || (uAddr + numWords) > uStoreSize) return(UENG_BADARG);

	int key = lockAccess(HIGH);
	if((status = isUengActive()) != UENG_INACTIVE){
		unlockAccess(key);
		return(status);
	}

	getAddrSeg(uAddr, numWords, &addrSeg);
	for(i = 0; i < addrSeg.len1; i++){
		uEngCsr->setUSTORE_ADDRESS(addrSeg.addr1++);
		uEngCsr->setUSTORE_DATA(uWord[uwIndx++]);
	}
	for(i = 0; i < addrSeg.len2; i++){
		uEngCsr->setUSTORE_ADDRESS(addrSeg.addr2++);
		uEngCsr->setUSTORE_DATA(uWord[uwIndx++]);
	}

	uEngCsr->setUSTORE_ADDRESS(0);						// clear addr enable bit
	unlockAccess(key);

	return(UENG_SUCCESS);
}



/*-----------------------------------------------------------------------------
   Function: 	Uengine::getCtxArb
   Description: Read the micro-engine context arbitrition-control register.
   Returns:		UENG_SUCCESS, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::getCtxArb(int unsigned *ctxArbCtl)
{
   VERIFY_LIB(uEngCsr);
   uEngCsr->getCTX_ARB_CNTL(ctxArbCtl);
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::setCtxArb
   Description: Write the micro-engine context arbitrition-control register.
   Returns:		UENG_SUCCESS, UENG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::setCtxArb(int unsigned ctxArbCtl)
{
   VERIFY_LIB(uEngCsr);
   uEngCsr->setCTX_ARB_CNTL(ctxArbCtl);
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::execMicroInst
   Description: Execute a list of micro-instructions, in the specified context,
				and then restore the state to the previous context. The code
				to be executed must perform a ctx_arb in order for this function
				to terminate sucessfully.
   Returns:		UENG_SUCCESS, UENG_BADARG, UENG_ACTIVE, UENG_RESET, UENG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::execMicroInst(unsigned char ctx, unsigned int *microInst,
						   unsigned int numInst, int condCodeOff,
						   unsigned int waitForMs)
{
   unsigned int savCC, wakeupEvents, savCtx, savPc, savUwords[MAX_EXEC_INST];
   int status = UENG_SUCCESS;

   if((numInst > MAX_EXEC_INST) || !microInst) return(UENG_BADARG);
   if(rst) return(UENG_RESET);

   // save current states:
   if((status = getNumUword(TmpUaddr, numInst, savUwords)) != UENG_SUCCESS)
	   return(status);						// instructions at loc 0 thru 2

   getCtxWakeupEvents(ctx, &wakeupEvents);	// save wakeup-events
   getPC(ctx, &savPc);						// save PC

   int key = lockAccess(HIGH);
   uEngCsr->getCC_ENABLE(&savCC);			// save conditional-code
   uEngCsr->getACTIVE_CTX_STS(&savCtx);		// save current context
   unlockAccess(key);
   
   setNumUword(TmpUaddr, numInst, microInst);// copy instructions to ustore
   setPC(setMasks[ctx], TmpUaddr);			// set PC

   key= lockAccess(HIGH);
   uEngCsr->setACTIVE_CTX_STS(ctx & 0x3);	// change the active context

   if(condCodeOff)
      uEngCsr->setCC_ENABLE(0x0);			// disable conditional-code
   unlockAccess(key);

   setCtxWakeupEvents(setMasks[ctx],0x1);	// wakeup-event voluntary
   enableCtx(setMasks[ctx]);				// enable context

   // wait for it to finish
   key= lockAccess(HIGH);
   waitForMs = waitForMs / DELAYMS_RES;
   while(isUengActive() == UENG_ACTIVE){
	   if(waitForMs--){
		  status = UENG_FAIL;
		  break;
	   }
	   delayMs(DELAYMS_RES);				// delay 1 system clock-tick							// wait 1msec
   }
   unlockAccess(key);

   // retore to previous state:
   disableCtx(setMasks[ctx]);						// disable context
   setNumUword(TmpUaddr, numInst, savUwords);		// instructions
   setCtxWakeupEvents(setMasks[ctx], wakeupEvents);	// wakeup-events
   setPC(setMasks[ctx], savPc);						// PC

   key = lockAccess(HIGH);
   uEngCsr->setCC_ENABLE(savCC);						// conditional-code
   uEngCsr->setACTIVE_CTX_STS(savCtx & 0x3);		// restore the context
   unlockAccess(key);

   return(status);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::readRelSramWrXfer
   Description: Read a long-word from SRAM write-xfer register.  Reading and
				writing to/from SRAM transfer registers are relative from the
				micro-engine's perspective.  Therefore, when the core reads
				from the SRAM xfer register it's actually reading the
				write-transfer registers.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::readRelSramWrXfer(unsigned char ctx, unsigned short regNum, unsigned int *val)
{
   VERIFY_LIB(uEngCsr);

   int key= lockAccess(HIGH);
   switch(ctx){
	  case 0: sramXferReg[ctx]->getSramXferReg(regNum, val); break;
	  case 1: sramXferReg[ctx]->getSramXferReg(regNum, val); break;
	  case 2: sramXferReg[ctx]->getSramXferReg(regNum, val); break;
	  case 3: sramXferReg[ctx]->getSramXferReg(regNum, val); break;
	  default: unlockAccess(key); return(UENG_BADARG);
   }
   unlockAccess(key);
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::writeRelSramRdXfer
   Description: Write a long-word to SRAM read-xfer register
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::writeRelSramRdXfer(unsigned char ctx, unsigned short regNum, unsigned int val)
{
   VERIFY_LIB(uEngCsr);

   int key = lockAccess(HIGH);
   switch(ctx){
	  case 0: sramXferReg[ctx]->setSramXferReg(regNum, val); break;
	  case 1: sramXferReg[ctx]->setSramXferReg(regNum, val); break;
	  case 2: sramXferReg[ctx]->setSramXferReg(regNum, val); break;
	  case 3: sramXferReg[ctx]->setSramXferReg(regNum, val); break;
	  default: unlockAccess(key); return(UENG_BADARG);
   }
   unlockAccess(key);
   return(UENG_SUCCESS);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::readRelReg
   Description: Read a long-word from either an absolute GRP, SRAM, or SDRAM
				register.  Type is one of:- 0=GPR-A, 1=GPR-B, 2=SRAM, 3=SDRAM.
				Ctx = 0-3.

   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::readRelReg(unsigned int type, unsigned char ctx, unsigned short regNum,
						unsigned int *data)
{
   unsigned int inst, savCtx, savUword, tmpUaddr;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);

   unsigned char regAddr = getReg5bitAddr(type, regNum);

   // ALU/Shift instruction
   //if(type == 0 || type == 2 || type == 3){		// A rel source
   if(type == GPR_A){			// A rel source
      inst = 0x26000000 | ((regAddr & 0x1f) << 12);
   } else{											// B rel source
      inst = 0x06000000 | ((regAddr & 0x1f) << 7);
   }

   int key = lockAccess(HIGH);
   // read current context
   uEngCsr->getACTIVE_CTX_STS(&savCtx);

   if (ctx != (savCtx & 0x3)) {
      uEngCsr->setACTIVE_CTX_STS(ctx & 0x3);		// change the context
   }

   tmpUaddr = SETBIT(TmpUaddr, ADDR_EN_BITPOS);		// enable address bit
   uEngCsr->setUSTORE_ADDRESS(tmpUaddr);
   uEngCsr->getUSTORE_DATA(&savUword);

   uEngCsr->setUSTORE_DATA(inst);					// execute inst by writing to CSdata reg
   uEngCsr->getALU_OUTPUT(data);					// read ALU output

   uEngCsr->setUSTORE_DATA(savUword);
   uEngCsr->setUSTORE_ADDRESS(0);					// clear addr enable bit
   if (ctx != (savCtx & 0x3)) {
      uEngCsr->setACTIVE_CTX_STS(savCtx & 0x3);		// restore the context
   }
   unlockAccess(key);

   return(UENG_SUCCESS);
}



/*-----------------------------------------------------------------------------
   Function: 	Uengine::writeRelReg
   Description: Write a long-word to either an absolute GRP, SRAM
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
				UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::writeRelReg(unsigned int type, unsigned char ctx, unsigned short regNum,
						 unsigned int data)
{
   unsigned int i, regAddr;
   unsigned int microInst[] = {
	  0xC0100000,		// immed_w0[reg,val32_lo]
	  0xC0140000,		// immed_w1[reg,val32_hi]

	  0xE00C0004		// ctx_arb[kill]
   };
   const int num_inst = sizeof(microInst)/sizeof(microInst[0]), condCodeOff = 1;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);

   // get the 8-bit address of the register
   regAddr = getReg8bitAddr(type, ctx, regNum);

   // modify the first two instructions
   for(i = 0; i < 2; i++){
	  unsigned int val;
	  val = data >> (16 * i);
      microInst[i] |= (val & 0x3ff) << 7;		// data bits 0-9 = reg bits 7-16
	  microInst[i] |= (val >> 10) & 0x3f;		// data bits 10-15 = reg bits 0-5
	  microInst[i] |= (regAddr & 0xff) << 21;	// reg bits 21-28
   }  
   return(execMicroInst(ctx, microInst, num_inst, condCodeOff));  
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::writeRelSramWrXfer
   Description: Write a long-word to a relative SRAM write-xfer reg.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
				UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::writeRelSramWrXfer(unsigned char ctx, unsigned short regNum, unsigned int data)
{
   unsigned int i, regAddr, gprVal, gprAddrSrc, gprAddrDest;
   int status = UENG_SUCCESS;
   unsigned int microInst[] = {
	  0xC0100000,		// immed_w0[gpr,data_lo]
	  0xC0140000,		// immed_w1[gpr,data_hi]

	  0x80000000,		// alu_[xferReg,--,b, reg]		; assign gpr to xfer reg

	  0xC0100000,		// immed_w0[gpr,regOldVal_lo]	; restore old value
	  0xC0140000,		// immed_w1[gpr,regOldVal_hi]

	  0xE00C0004		// ctx_arb[kill]
   };
   const int num_inst = sizeof(microInst)/sizeof(microInst[0]), gprNum = 0, condCodeOff = 1;

   if(!uEngCsr)return(UENG_BADLIB);
   if(rst) return(UENG_RESET);
   if(regNum > MAX_XFER_REG) return(UENG_BADARG);

   // get the value of a b-bank gpr
   readRelReg(GPR_B, ctx, gprNum, &gprVal);

   // get the 8-bit address of the src gpr & dest xfer register
   gprAddrDest = getReg8bitAddr(GPR_B, ctx, gprNum);
   gprAddrSrc = getReg8bitAbsSrcAddr(GPR_B, ctx, gprNum);
   regAddr = getReg8bitAddr(SRAM_XFER, ctx, regNum);

   // install the value and registers in the instructions
   for(i = 0; i < 2; i++){
	  unsigned int val, oldVal;
	   val = data >> (16 * i);
	   oldVal = gprVal >> (16 * i);
      microInst[i] |= (val & 0x3ff) << 7;			// data bits 0-9 = reg bits 7-16
	  microInst[i] |= (val >> 10) & 0x3f;			// data bits 10-15 = reg bits 0-5
	  microInst[i] |= (gprAddrDest & 0xff) << 21;	// reg bits 21-28

      microInst[i+3] |= (oldVal & 0x3ff) << 7;		// data bits 0-9 = reg bits 7-16
	  microInst[i+3] |= (oldVal >> 10) & 0x3f;		// data bits 10-15 = reg bits 0-5
	  microInst[i+3] |= (gprAddrDest  & 0xff) << 21;// reg bits 21-28
   }  
   microInst[2] |= (gprAddrSrc & 0x1f) << 7;		// reg bits 7-11
   microInst[2] |= ((gprAddrSrc >> 5) & 0x7) << 4;	// reg bits 4-6
   microInst[2] |= (regAddr & 0xff) << 21;			// reg bits 21-28

   status = execMicroInst(ctx, microInst, num_inst, condCodeOff);  
   writeRelReg(GPR_B, ctx, gprNum, gprVal);
   return(status);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::writeRelSdramWrXfer
   Description: Write a quad-word to a relative SDRAM write-xfer reg.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
                UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::writeRelSdramWrXfer(unsigned char ctx, unsigned short regNum, 
								 unsigned int *data)
{
   unsigned int i, reg1Addr, reg2Addr, gpr1Val, gpr2Val;
   unsigned int gpr1AddrDest, gpr1AddrSrc, gpr2AddrDest, gpr2AddrSrc;
   int status = UENG_SUCCESS;
   unsigned int microInst[] = {
		0xC0100000,		// immed_w0[reg1,val1_lo]
		0xC0140000,		// immed_w1[reg1,val1_hi]

		0xC0100000,		// immed_w0[reg2,val2_lo]
		0xC0140000,		// immed_w1[reg2,val2_hi]

		0x80000000,		// alu_[xferReg1,--,b, reg1]
		0x80000000,		// alu_[xferReg2,--,b, reg2]

		0xC0100000,		// immed_w0[reg1,gpr1Val_lo]
		0xC0140000,		// immed_w1[reg1,gpr1Val_hi]

		0xC0100000,		// immed_w0[reg2,gpr2Val_lo]
		0xC0140000,		// immed_w1[reg2,gpr2Val_hi]

		0xE00C0004		// ctx_arb[kill]
   };
   const int num_inst = sizeof(microInst)/sizeof(microInst[0]);
   const unsigned int gpr1 = 0, gpr2 = 1, condCodeOff = 1;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);
   if(regNum >= MAX_XFER_REG) return(UENG_BADARG);

   // get the value of a B gpr's
   readRelReg(GPR_B, ctx, gpr1, &gpr1Val);
   readRelReg(GPR_B, ctx, gpr2, &gpr2Val);

   // get the 8-bit address of the src gpr & dest xfer register
   gpr1AddrDest = getReg8bitAddr(GPR_B, ctx, gpr1);
   gpr1AddrSrc = getReg8bitAbsSrcAddr(GPR_B, ctx, gpr1);
   reg1Addr = getReg8bitAddr(SDRAM_XFER, ctx, regNum);
   gpr2AddrDest = getReg8bitAddr(GPR_B, ctx, gpr2);
   gpr2AddrSrc = getReg8bitAbsSrcAddr(GPR_B, ctx, gpr2);
   reg2Addr = getReg8bitAddr(SDRAM_XFER, ctx, regNum+1);

   // install the value and registers in the instructions
   for(i = 0; i < 2; i++){
      unsigned int val_h, val_l;
	  val_l = data[0] >> (16 * i);
	  val_h = data[1] >> (16 * i);
      microInst[i] |= (val_l & 0x3ff) << 7;		    // data bits 0-9 = reg bits 7-16
	  microInst[i] |= (val_l >> 10) & 0x3f;			// data bits 10-15 = reg bits 0-5
	  microInst[i] |= (gpr1AddrDest & 0xff) << 21;	// reg bits 21-28

      microInst[i+2] |= (val_h & 0x3ff) << 7;		// data bits 0-9 = reg bits 7-16
	  microInst[i+2] |= (val_h >> 10) & 0x3f;		// data bits 10-15 = reg bits 0-5
	  microInst[i+2] |= (gpr2AddrDest & 0xff) << 21;// reg bits 21-28

	  val_l = gpr1Val >> (16 * i);
	  val_h = gpr2Val >> (16 * i);
      microInst[i+6] |= (val_l & 0x3ff) << 7;		// data bits 0-9 = reg bits 7-16
	  microInst[i+6] |= (val_l >> 10) & 0x3f;		// data bits 10-15 = reg bits 0-5
	  microInst[i+6] |= (gpr1AddrDest & 0xff) << 21;// reg bits 21-28

      microInst[i+8] |= (val_h & 0x3ff) << 7;		// data bits 0-9 = reg bits 7-16
	  microInst[i+8] |= (val_h >> 10) & 0x3f;		// data bits 10-15 = reg bits 0-5
	  microInst[i+8] |= (gpr2AddrDest & 0xff) << 21;// reg bits 21-28
   }  
   microInst[4] |= (gpr1AddrSrc & 0x1f) << 7;		// reg bits 7-11
   microInst[4] |= ((gpr1AddrSrc >> 5) & 0x7) << 4;	// reg bits 4-6
   microInst[4] |= (reg1Addr & 0xff) << 21;			// reg bits 21-28

   microInst[5] |= (gpr2AddrSrc & 0x1f) << 7;		// reg bits 7-11
   microInst[5] |= ((gpr2AddrSrc >> 5) & 0x7) << 4;	// reg bits 4-6
   microInst[5] |= (reg2Addr & 0xff) << 21;			// reg bits 21-28

   status = execMicroInst(ctx, microInst, num_inst, condCodeOff);

   // restore the gprs value
   writeRelReg(GPR_B, ctx, gpr1, gpr1Val);
   writeRelReg(GPR_B, ctx, gpr2, gpr2Val);
   return(status);
}

/*-----------------------------------------------------------------------------
   Function: 	Uengine::readRelSdramWrXfer
   Description: Read a quad-word from a relative SDRAM read-xfer reg.
      procedure to read write-xfer registers:
      1) save the contents of two sdram locations
	  2) save the contents of two a-gprs
	  3) make sdram locations differ: sdram[sdramAddr+1] = ~sdram[sdramAddr+1]
	  4) execute:
		   immed[gpr0, sdramAddr]
		   immed[gpr1, sdramAddr+1]
		   mem[sdram_write, $$reg, gpr0]
		   mem[sdram_write, $$reg, gpr1]
		   ctx_arb[kill]
	  5) check sdram write completion -- compare the two sdram locations until equal or timeout
	  7) restore the contents of sdram locations
	  8) resore the contents of the gpr
	  9) return the value
 
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
				UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::readRelSdramWrXfer(unsigned char ctx, unsigned short regNum, 
								unsigned int *data)
{
   unsigned int microInst[] = {
		0xC0100000,		// immed_w0[reg1,sdramAddr1_lo]
		0xC0140000,		// immed_w1[reg1,sdramAddr1_hi]

		0xC0100000,		// immed_w0[reg2,sdramAddr2_lo]
		0xC0140000,		// immed_w1[reg2,sdramAddr2_hi]

		0xA0000031,		// mem[sdram_write, xferReg1,reg1,0,0]
		0xA0000031,		// mem[sdram_write, xferReg1,reg2,0,0]

		0xE00C0004		// ctx_arb[kill]
   };
   unsigned int i, gpr0Val,gpr1Val, gpr2Val, gpr3Val;
   unsigned int gpr0AddrDest, gpr0AddrSrc, gpr2AddrDest, gpr2AddrSrc;
   int status = UENG_SUCCESS;
   const int num_inst = sizeof(microInst)/sizeof(microInst[0]);
   const unsigned int gpr0 = 0, gpr1 = 1, gpr2 = 2, gpr3 = 3, condCodeOff=1;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);
   if(!data || (regNum >= MAX_XFER_REG)) return(UENG_BADARG);

   // get the value of four A-bank gprs
   readRelReg(GPR_A, ctx, gpr0, &gpr0Val);
   readRelReg(GPR_A, ctx, gpr1, &gpr1Val);
   readRelReg(GPR_A, ctx, gpr2, &gpr2Val);
   readRelReg(GPR_A, ctx, gpr3, &gpr3Val);


   // get the source and destination gpr addresses
   gpr0AddrSrc = getReg5bitAddr(GPR_A, gpr0);
   gpr0AddrDest = getReg8bitAddr(GPR_A, ctx, gpr0);
   gpr2AddrSrc = getReg5bitAddr(GPR_A, gpr2);
   gpr2AddrDest = getReg8bitAddr(GPR_A, ctx, gpr2);


   // install the sdram address in the gprs
   unsigned int sdVal1[2], sdVal2[2], sdAddr1, sdAddr2, val1;
   const unsigned int fbox_sdAddr1 = SDRAM_UDEBUG_TMPADR, fbox_sdAddr2 = SDRAM_UDEBUG_TMPADR + 1;
   const unsigned int core_sdAddr1 = fbox_sdAddr1 << 1, core_sdAddr2 = fbox_sdAddr2 << 1;

   for(i = 0; i < 2; i++){
	  sdAddr1 = fbox_sdAddr1 >> (16 * i);
	  sdAddr2 = fbox_sdAddr2 >> (16 * i);
      microInst[i] |= (sdAddr1 & 0x3ff) << 7;				// data bits 0-9 = reg bits 7-16
	  microInst[i] |= (sdAddr1 >> 10) & 0x3f;				// data bits 10-15 = reg bits 0-5
	  microInst[i] |= (gpr0AddrDest & 0xff) << 21;			// reg bits 21-28

      microInst[i+2] |= (sdAddr2 & 0x3ff) << 7;				// data bits 0-9 = reg bits 7-16
	  microInst[i+2] |= (sdAddr2 >> 10) & 0x3f;				// data bits 10-15 = reg bits 0-5
	  microInst[i+2] |= (gpr2AddrDest & 0xff) << 21;		// reg bits 21-28
   }  

   microInst[4] |= (gpr0AddrSrc & 0x1f) << 12;				// reg bits 12-16
   microInst[4] |= (regNum & 0x7) << 20;					// reg bits 20-22

   microInst[5] |= (gpr2AddrSrc & 0x1f) << 12;				// reg bits 12-16
   microInst[5] |= (regNum & 0x7) << 20;					// reg bits 20-22


   // save the content of the sdram locations
   sdVal1[0] = SDRAM_Read(core_sdAddr1);
   sdVal1[1] = SDRAM_Read(core_sdAddr1+1);
   sdVal2[0] = SDRAM_Read(core_sdAddr2);
   sdVal2[1] = SDRAM_Read(core_sdAddr2+1);


   SDRAM_Write(core_sdAddr2,  ~sdVal1[0]);
   status = execMicroInst(ctx, microInst, num_inst, condCodeOff);	// execute the instructions

   // poll sdram locations to determine when the sdram write has completed
   unsigned int times = 0;
   while(status == UENG_SUCCESS){
      data[0] = SDRAM_Read(core_sdAddr1);
      val1 = SDRAM_Read(core_sdAddr2);

	  if(data[0] == val1){						// is the operation completed?
         data[1] = SDRAM_Read(core_sdAddr1+1);
		 break;
	  }

	  delayMs(DELAYMS_RES);						// delay 1 system clock-tick;							// wait 1 clock-tick
	  if((++times) > 5){
	     status = UENG_FAIL;
		 break;
	  }
   }

   // restore the gprs and sdram locations
   writeRelReg(0, ctx, gpr0, gpr0Val);
   writeRelReg(0, ctx, gpr1, gpr1Val);
   writeRelReg(0, ctx, gpr2, gpr2Val);
   writeRelReg(0, ctx, gpr3, gpr3Val);
   SDRAM_Write(core_sdAddr1, sdVal1[0]);
   SDRAM_Write(core_sdAddr1+1, sdVal1[1]);
   SDRAM_Write(core_sdAddr2, sdVal2[0]);
   SDRAM_Write(core_sdAddr2+1, sdVal2[1]);

  return(status);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::writeRelSdramRdXfer
   Description: Write a quad-word to a relative SDRAM read-xfer reg.
      procedure to write read-xfer registers:
      1) save the contents of a sdram location
	  2) save the contents of a gpr
	  3) store the data in the sdram location
	  4) execute:
		   immed[gpr0, sdramAddr0]
		   sdram[read, $$reg, gpr0]
		   ctx_arb[kill]
	  5) wait an arbituary amount of time for the SDRAM write to be completed
	  6) restore the contents sdram locations
	  7) resore the contents of the gprs
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
				UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::writeRelSdramRdXfer(unsigned char ctx, unsigned short regNum, 
								 unsigned int *data)
{
   unsigned int i, gprVal1, gprVal2, gprAddrDest, gprAddrSrc, status = UENG_SUCCESS;
   unsigned int microInst[] = {
		0xC0100000,		// immed_w0[reg1,sdramAddr1_lo]
		0xC0140000,		// immed_w1[reg1,sdramAddr1_hi]
		0xA0000030,		// sdram[read, xferReg1,reg1,0,0]
		0xE00C0004		// ctx_arb[kill]
   };
   const int num_inst = sizeof(microInst)/sizeof(microInst[0]);
   const unsigned int gpr0=0, gpr1=1, condCodeOff=1;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);
   if(regNum >= MAX_XFER_REG) return(UENG_BADARG);

   // save the value of two consecetive A-bank gprs
   readRelReg(GPR_A, ctx, gpr0, &gprVal1);
   readRelReg(GPR_A, ctx, gpr1, &gprVal2);

   // get the source and destination gpr addresses
   gprAddrSrc = getReg5bitAddr(GPR_A, gpr0);
   gprAddrDest = getReg8bitAddr(GPR_A, ctx, gpr0);

   unsigned int sdVal[2], sdAddr;
   const unsigned int fbox_sdAddr1 = SDRAM_UDEBUG_TMPADR;
   const unsigned int sdAddr1 = fbox_sdAddr1 << 1;


   // install the sdram address in the gprs
   for(i = 0; i < 2; i++){
	  sdAddr = fbox_sdAddr1 >> (16 * i);
      microInst[i] |= (sdAddr & 0x3ff) << 7;				// data bits 0-9 = reg bits 7-16
	  microInst[i] |= (sdAddr >> 10) & 0x3f;				// data bits 10-15 = reg bits 0-5
	  microInst[i] |= (gprAddrDest & 0xff) << 21;			// reg bits 21-28
   }  
   microInst[2] |= (gprAddrSrc & 0x1f) << 12;				// reg bits 12-16
   microInst[2] |= (regNum & 0x7) << 20;					// reg bits 20-22

   // save the content of the two sdram locations
   sdVal[0] = SDRAM_Read(sdAddr1);
   sdVal[1] = SDRAM_Read(sdAddr1+1);

   // write the data sdram
   SDRAM_Write(sdAddr1, data[0]);
   SDRAM_Write(sdAddr1+1, data[1]);

   status = execMicroInst(ctx, microInst, num_inst, condCodeOff);	// execute the instructions
   delayMs(DELAYMS_RES);		// delay 1 system clock-tick


   // restore the gprs and sdram locations
   writeRelReg(0, ctx, gpr0, gprVal1);
   writeRelReg(0, ctx, gpr1, gprVal2);
   SDRAM_Write(sdAddr1, sdVal[0]);
   SDRAM_Write(sdAddr1+1, sdVal[1]);

   return(status);
}

/*-----------------------------------------------------------------------------
   Function: 	Uengine::readRelSdramRdXfer
   Description: Read a quad-word from a relative SDRAM read-xfer reg. 
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
				UENG_RESET
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::readRelSdramRdXfer(unsigned char ctx, unsigned short regNum, 
								 unsigned int *data)
{
   int status = UENG_SUCCESS;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);
   if(!data || (regNum >= MAX_XFER_REG)) return(UENG_BADARG);

   if((status = readRelReg(SDRAM_XFER, ctx, regNum, &data[0])) != UENG_SUCCESS)
		 return(status);
   if((status = readRelReg(SDRAM_XFER, ctx, regNum+1, &data[1])) != UENG_SUCCESS)
		 return(status);

   return(status);
}


/*-----------------------------------------------------------------------------
   Function: 	Uengine::readRxFifo
   Description: Read a quad-word from a receive-FIFO address. 
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
				UENG_RESET
   Uses:		Context zero to execute the memory reference
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::readRxFifo(unsigned short fifoAddr, unsigned int *data)
{
   unsigned int microInst[] = {
		0xC0100000,		// immed_w0[gpr0,sdAddr1_lo]
		0xC0140000,		// immed_w1[gpr0,sdAddr1_hi]

		0xC0100000,		// immed_w0[gpr1,sdAddr2_lo]
		0xC0140000,		// immed_w1[gpr1,sdAddr2_hi]

		0xC0100000,		// immed_w0[gpr2,fifoAddr_lo]
		0xC0140000,		// immed_w1[gpr2,fifoAddr_hi]

		0x9E000000,		// alu[--, --, b, gpr2]
		0xA0000073,		// sdram[r_fifo_rd, --, sdaddr1, 0, word_cnt_1], indirect_ref

		0x9E000000,		// alu[--, --, b, gpr2]
		0xA0000073,		// sdram[r_fifo_rd, --, sdaddr2, 0, word_cnt_1], indirect_ref
		0xE00C0004		// ctx_arb[kill]
   };


   unsigned int i, gpr0Val, gpr1Val, gpr2Val;
   unsigned int gpr0AddrDest, gpr0AddrSrc, gpr1AddrDest, gpr1AddrSrc;
   unsigned int gpr2AddrDest, gpr2AddrSrc;
   int status = UENG_SUCCESS;
   const int num_inst = sizeof(microInst)/sizeof(microInst[0]);
   const unsigned int ctx=0, gpr0 = 0, gpr1 = 1, gpr2 = 2, gpr3 = 3, condCodeOff=1;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);
   if(!data || (fifoAddr >= MAX_FIFO_QWADDR)) return(UENG_BADARG);

   // save the values of two A-bank gprs
   readRelReg(GPR_A, ctx, gpr0, &gpr0Val);
   readRelReg(GPR_A, ctx, gpr1, &gpr1Val);
   readRelReg(GPR_B, ctx, gpr2, &gpr2Val);


   // get the source & destination gpr addresses
   gpr0AddrSrc = getReg5bitAddr(GPR_A, gpr0);
   gpr0AddrDest = getReg8bitAddr(GPR_A, ctx, gpr0);
   gpr1AddrSrc = getReg5bitAddr(GPR_A, gpr1);
   gpr1AddrDest = getReg8bitAddr(GPR_A, ctx, gpr1);

   gpr2AddrSrc = getReg8bitAbsSrcAddr(GPR_B, ctx, gpr2);
   gpr2AddrDest = getReg8bitAddr(GPR_B, ctx, gpr2);

   // install the sdram address in the gprs
   unsigned int sdVal1[2], sdVal2[2], sdAddr1, sdAddr2, val1, tmp, fAddr = (unsigned int)fifoAddr << 4;
   const unsigned int fbox_sdAddr1 = SDRAM_UDEBUG_TMPADR, fbox_sdAddr2 = SDRAM_UDEBUG_TMPADR + 1;
   const unsigned int core_sdAddr1 = fbox_sdAddr1 << 1, core_sdAddr2 = fbox_sdAddr2 << 1;

   for(i = 0; i < 2; i++){
	  sdAddr1 = fbox_sdAddr1 >> (16 * i);
	  sdAddr2 = fbox_sdAddr2 >> (16 * i);
	  tmp = fAddr >> (16 * i);

      microInst[i] |= (sdAddr1 & 0x3ff) << 7;				// data bits 0-9 = reg bits 7-16
	  microInst[i] |= (sdAddr1 >> 10) & 0x3f;				// data bits 10-15 = reg bits 0-5
	  microInst[i] |= (gpr0AddrDest & 0xff) << 21;			// reg bits 21-28

      microInst[i+2] |= (sdAddr2 & 0x3ff) << 7;				// data bits 0-9 = reg bits 7-16
	  microInst[i+2] |= (sdAddr2 >> 10) & 0x3f;				// data bits 10-15 = reg bits 0-5
	  microInst[i+2] |= (gpr1AddrDest & 0xff) << 21;		// reg bits 21-28

      microInst[i+4] |= (tmp & 0x3ff) << 7;					// data bits 0-9 = reg bits 7-16
	  microInst[i+4] |= (tmp >> 10) & 0x3f;					// data bits 10-15 = reg bits 0-5
	  microInst[i+4] |= (gpr2AddrDest & 0xff) << 21;		// reg bits 21-28
   }  

   microInst[6] |= (gpr2AddrSrc & 0x1f) << 7;				// reg bits 7-11
   microInst[6] |= ((gpr2AddrSrc >> 5) & 0x7) << 4;			// reg bits 4-6
   microInst[7] |= (gpr0AddrSrc & 0x1f) << 12;				// reg bits 12-16

   microInst[8] |= (gpr2AddrSrc & 0x1f) << 7;				// reg bits 7-11
   microInst[8] |= ((gpr2AddrSrc >> 5) & 0x7) << 4;			// reg bits 4-6
   microInst[9] |= (gpr1AddrSrc & 0x1f) << 12;				// reg bits 12-16

   // save the content of four sdram locations
   sdVal1[0] = SDRAM_Read(core_sdAddr1);
   sdVal1[1] = SDRAM_Read(core_sdAddr1+1);
   sdVal2[0] = SDRAM_Read(core_sdAddr2);
   sdVal2[1] = SDRAM_Read(core_sdAddr2+1);

   SDRAM_Write(core_sdAddr2,  ~sdVal1[0]);		// make sdaddr1 & sdaddr2 different
   status = execMicroInst(ctx, microInst, num_inst, condCodeOff);	// execute the instructions

   // poll sdram locations to determine when the sdram write has completed
   unsigned int times = 0;
   while(status == UENG_SUCCESS){
      data[0] = SDRAM_Read(core_sdAddr1);
      val1 = SDRAM_Read(core_sdAddr2);

	  if(data[0] == val1){						// is the operation completed?
         data[1] = SDRAM_Read(core_sdAddr1+1);
		 break;
	  }

	  delayMs(DELAYMS_RES);						// delay 1 system clock-tick;							// wait 1 clock-tick
	  if((++times) > 5){
	     status = UENG_FAIL;
		 break;
	  }
   }

   // restore the gprs and sdram locations
   writeRelReg(GPR_A, ctx, gpr0, gpr0Val);
   writeRelReg(GPR_A, ctx, gpr1, gpr1Val);
   writeRelReg(GPR_B, ctx, gpr2, gpr2Val);

   SDRAM_Write(core_sdAddr1, sdVal1[0]);
   SDRAM_Write(core_sdAddr1+1, sdVal1[1]);
   SDRAM_Write(core_sdAddr2, sdVal2[0]);
   SDRAM_Write(core_sdAddr2+1, sdVal2[1]);

  return(status);
}



/*-----------------------------------------------------------------------------
   Function: 	Uengine::writeTxFifo
   Description: Write a quad-word to a transmit-FIFO address. The setValid value
				indicates that the 64-bit data in the tfifo is vallid and can be
				transmitted.
   Returns:		UENG_SUCCESS, UENG_BADLIB, UENG_BADARG, UENG_ACTIVE, UENG_FAIL,
				UENG_RESET
   Uses:		Context zero to execute the memory reference
   Modifies:
-----------------------------------------------------------------------------*/
int Uengine::writeTxFifo(unsigned short fifoAddr, unsigned int *data, int setValid)
{
   unsigned int i, gpr0Val, gpr1Val, gpr0AddrDest, gpr0AddrSrc, gpr1AddrDest, gpr1AddrSrc;
   unsigned int status = UENG_SUCCESS;
   unsigned int microInst[] = {
		0xC0100000,		// immed_w0[gpr0,sdramAddr1_lo]
		0xC0140000,		// immed_w1[gpr0,sdramAddr1_hi]

		0xC0100000,		// immed_w0[gpr1,fifoAddr_lo]
		0xC0140000,		// immed_w1[gpr1,fifoAddr_hi]

		0x06000030,		// alu[--, --, b, gpr1]
		0xA0000072,		// sdram[t_fifo_wr, --, sdaddr1, 0, word_cnt_1], indirect_ref

		0xE00C0004		// ctx_arb[kill]
   };
   const int num_inst = sizeof(microInst)/sizeof(microInst[0]);
   const unsigned int ctx=0, gpr0=0, gpr1=1, condCodeOff=1;

   VERIFY_LIB(uEngCsr);
   if(rst) return(UENG_RESET);
   if(fifoAddr >= MAX_FIFO_QWADDR) return(UENG_BADARG);

   // save the gpr's
   readRelReg(GPR_A, ctx, gpr0, &gpr0Val);
   readRelReg(GPR_B, ctx, gpr1, &gpr1Val);

   // get the source and destination gpr addresses
   gpr0AddrSrc = getReg5bitAddr(GPR_A, gpr0);
   gpr0AddrDest = getReg8bitAddr(GPR_A, ctx, gpr0);

   gpr1AddrSrc = getReg8bitAbsSrcAddr(GPR_B, ctx, gpr1);
   gpr1AddrDest = getReg8bitAddr(GPR_B, ctx, gpr1);

   unsigned int sdVal[2], sdAddr, tmp, ind = (unsigned int)fifoAddr << 4;
   const unsigned int fbox_sdAddr1 = SDRAM_UDEBUG_TMPADR;
   const unsigned int sdAddr1 = fbox_sdAddr1 << 1;

   if(setValid) ind |= 1 << 15;								// reg bits 15

   // install the sdram address in the gprs
   for(i = 0; i < 2; i++){
	  sdAddr = fbox_sdAddr1 >> (16 * i);
	  tmp = ind >> (16 * i);

      microInst[i] |= (sdAddr & 0x3ff) << 7;				// data bits 0-9 = reg bits 7-16
	  microInst[i] |= (sdAddr >> 10) & 0x3f;				// data bits 10-15 = reg bits 0-5
	  microInst[i] |= (gpr0AddrDest & 0xff) << 21;			// reg bits 21-28

      microInst[i+2] |= (tmp & 0x3ff) << 7;					// data bits 0-9 = reg bits 7-16
	  microInst[i+2] |= (tmp >> 10) & 0x3f;					// data bits 10-15 = reg bits 0-5
	  microInst[i+2] |= (gpr1AddrDest & 0xff) << 21;		// reg bits 21-28
   }  
   microInst[4] |= (gpr1AddrSrc & 0x1f) << 7;				// reg bits 7-11
   microInst[4] |= ((gpr1AddrSrc >> 5) & 0x7) << 4;			// reg bits 4-6
   microInst[5] |= (gpr0AddrSrc & 0x1f) << 12;				// reg bits 12-16

   // save the content of the two sdram locations
   sdVal[0] = SDRAM_Read(sdAddr1);
   sdVal[1] = SDRAM_Read(sdAddr1+1);

   // write the data sdram
   SDRAM_Write(sdAddr1, data[0]);
   SDRAM_Write(sdAddr1+1, data[1]);

   status = execMicroInst(ctx, microInst, num_inst, condCodeOff);	// execute the instructions
   delayMs(DELAYMS_RES);		// delay 1 system clock-tick


   // restore the gprs and sdram locations
   writeRelReg(GPR_A, ctx, gpr0, gpr0Val);
   writeRelReg(GPR_B, ctx, gpr1, gpr1Val);
   SDRAM_Write(sdAddr1, sdVal[0]);
   SDRAM_Write(sdAddr1+1, sdVal[1]);
   return(status);
}


//dj:  for debuging
void Uengine::dump(void)
{
	unsigned int ctxArbCtl, ctxEn,activeCtxStatus,ccEn;
	unsigned int ctx0SigEv, ctx1SigEv, ctx2SigEv, ctx3SigEv;
	unsigned int ctx0Status,ctx1Status,ctx2Status,ctx3Status;
	unsigned int ctx0WkupEv, ctx1WkupEv, ctx2WkupEv, ctx3WkupEv;
	unsigned int ireg, reset;

    Pci->getResetReg(&reset);
    Fbi->getIREG(&ireg);

	uEngCsr->getCTX_ARB_CNTL(&ctxArbCtl);
	uEngCsr->getCTX_ENABLES(&ctxEn);
	uEngCsr->getACTIVE_CTX_STS(&activeCtxStatus);
	uEngCsr->getCC_ENABLE(&ccEn);

    uEngCsr->getCTX_0_SIG_EVENTS(&ctx0SigEv);
    uEngCsr->getCTX_1_SIG_EVENTS(&ctx1SigEv);
    uEngCsr->getCTX_2_SIG_EVENTS(&ctx2SigEv);
    uEngCsr->getCTX_3_SIG_EVENTS(&ctx3SigEv);

    uEngCsr->getCTX_0_WAKEUP_EVENTS(&ctx0WkupEv);
    uEngCsr->getCTX_1_WAKEUP_EVENTS(&ctx1WkupEv);
    uEngCsr->getCTX_2_WAKEUP_EVENTS(&ctx2WkupEv);
    uEngCsr->getCTX_3_WAKEUP_EVENTS(&ctx3WkupEv);

	uEngCsr->getCTX_0_STS(&ctx0Status); 
	uEngCsr->getCTX_1_STS(&ctx1Status); 
    uEngCsr->getCTX_2_STS(&ctx2Status);
    uEngCsr->getCTX_3_STS(&ctx3Status);

	printf("\nUeng%d: resetReg=0x%x, ireg=0x%x\n",
		thisUengNum, reset, ireg);

	printf("ctxArbCtl=0x%x, ctxEn=0x%x, actCtxStat=0x%x, ccEn=0x%x\n",
			ctxArbCtl, ctxEn, activeCtxStatus, ccEn);

	printf("ctx0SigEv=0x%x, ctx1SigEv=0x%x, ctx2SigEv=0x%x, ctx3SigEv=0x%x\n",
			ctx0SigEv,ctx1SigEv,ctx2SigEv,ctx3SigEv);

	printf("ctx0WkupEv=0x%x, ctx1WkupEv=0x%x, ctx2WkupEv=0x%x, ctx3WkupEv=0x%x\n",
			ctx0WkupEv,ctx1WkupEv,ctx2WkupEv,ctx3WkupEv);

	printf("ctx0Status=0x%x,ctx1Status=0x%x,ctx2Status=0x%x,ctx3Status=0x%x\n",
			ctx0Status, ctx1Status, ctx2Status, ctx3Status);

	printf("ctx0PC=%d,ctx1PC=%d,ctx2PC=%d,ctx3PC=%d\n",
			(ctx0Status & 0x3ff) | ((ctx0Status & CTX_X_USTORE_SEGMENT_MASK) >> 4),
			(ctx1Status & 0x3ff) | ((ctx1Status & CTX_X_USTORE_SEGMENT_MASK) >> 4),
			(ctx2Status & 0x3ff) | ((ctx2Status & CTX_X_USTORE_SEGMENT_MASK) >> 4),
			(ctx3Status & 0x3ff) | ((ctx3Status & CTX_X_USTORE_SEGMENT_MASK) >> 4));

	fflush(stdout);
		
	return;
}


void SramXferReg::getSramXferReg(unsigned short regNum, unsigned int *val)
{
	switch(regNum){
	   case 0: READ_LWORD(&xfer_reg_0, val); break;
	   case 1: READ_LWORD(&xfer_reg_1, val); break;
	   case 2: READ_LWORD(&xfer_reg_2, val); break;
	   case 3: READ_LWORD(&xfer_reg_3, val); break;
	   case 4: READ_LWORD(&xfer_reg_4, val); break;
	   case 5: READ_LWORD(&xfer_reg_5, val); break;
	   case 6: READ_LWORD(&xfer_reg_6, val); break;
	   case 7: READ_LWORD(&xfer_reg_7, val); break;
	}
	return;
}

void SramXferReg::setSramXferReg(unsigned short regNum, unsigned int val){
	switch(regNum){
	   case 0: WRITE_LWORD(&xfer_reg_0, val); break;
	   case 1: WRITE_LWORD(&xfer_reg_1, val); break;
	   case 2: WRITE_LWORD(&xfer_reg_2, val); break;
	   case 3: WRITE_LWORD(&xfer_reg_3, val); break;
	   case 4: WRITE_LWORD(&xfer_reg_4, val); break;
	   case 5: WRITE_LWORD(&xfer_reg_5, val); break;
	   case 6: WRITE_LWORD(&xfer_reg_6, val); break;
	   case 7: WRITE_LWORD(&xfer_reg_7, val); break;
	}
	return;
}


// end of file