/* ueng_clib.cpp
 * Ucode Uengine Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Debug Library
 *  author: drj, Jan 11,99  Initial version
 *  revisions:
 *  $History: ueng_clib.cpp $
 * 
 * *****************  Version 14  *****************
 * User: Ewalker      Date: 11/09/00   Time: 10:04a
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Fixed typo -  Turned one of the two uEng_getCtxArb function definitions
 * into uEng_setCtxArb
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 8/31/00    Time: 3:14p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * Updated for C0 changes
 * 
 * *****************  Version 9  *****************
 * User: Drjohns1     Date: 9/22/99    Time: 4:31p
 * Updated in $/dev_1200/SA1_CoreLibs/ueng
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 1/29/99    Time: 11:25a
 * Created in $/development/ueng
 * 
 * ---------------------------------------------------------------------
 */
#include <stdio.h>
#include "ueng.h"


static const char __copyright[] =
"Copyright Intel Corp, 1998. All rights reserved";

static Uengine **uEngs;

/*-----------------------------------------------------------------------------
   Function: 	uDbg_InitLib
   Description: Creates and initialise a Debug library object.

   Returns:		Debug library object handle or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void uEng_initUengs()
{
   uEngs = initUengs();
   return;
}


int uEng_setUstoreFreeMem(unsigned char uEng, unsigned int begFreeAddr, unsigned int size)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setUstoreFreeMem(begFreeAddr, size));
}


int uEng_getUstoreFreeMem(unsigned char uEng, unsigned int *begFreeAddr, unsigned int *size)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getUstoreFreeMem(begFreeAddr, size));
}

// dj: This function is for supporting C0 and is subject ot change
int uEng_setUstoreSegFreeMem(unsigned char uEng,
							 unsigned int seg1FreeAddr, unsigned int seg1FreeSize,
							 unsigned int seg2FreeAddr, unsigned int seg2FreeSize)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setUstoreFreeMem(seg1FreeAddr, seg1FreeSize, seg2FreeAddr, seg2FreeSize));
}

// dj: This function is for supporting C0 and is subject ot change
int uEng_getUstoreSegFreeMem(unsigned char uEng,
							 unsigned int *seg1FreeAddr, unsigned int *seg1FreeSize,
							 unsigned int *seg2FreeAddr, unsigned int *seg2FreeSize)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getUstoreFreeMem(seg1FreeAddr, seg1FreeSize, seg2FreeAddr, seg2FreeSize));
}

int uEng_getCtxEnables(unsigned char uEng, unsigned int *ctxEnables)
{
	if(!uEngs) return(UENG_BADARG);
	uEngs[uEng]->getCtxEnables(ctxEnables);
	return(UENG_SUCCESS);
}

int uEng_getActiveStatus(unsigned char uEng, unsigned int *activeCtxStatus)
{
	if(!uEngs) return(UENG_BADARG);
	uEngs[uEng]->getActiveStatus(activeCtxStatus);
	return(UENG_SUCCESS);
}

int uEng_getCsrWriteEn(unsigned char uEng, unsigned int *csrWriteEn)
{
	if(!uEngs) return(UENG_BADARG);
	uEngs[uEng]->getCsrWriteEn(csrWriteEn);
	return(UENG_SUCCESS);
}


int uEng_getSramJouEn(unsigned char uEng, unsigned int *sramJouEn)
{
	if(!uEngs) return(UENG_BADARG);
	uEngs[uEng]->getSramJouEn(sramJouEn);
	return(UENG_SUCCESS);
}

int uEng_getCondCodeEn(unsigned char uEng, unsigned int *ccEn)
{
	if(!uEngs) return(UENG_BADARG);
	uEngs[uEng]->getCondCodeEn(ccEn);
	return(UENG_SUCCESS);
}

int uEng_setCondCodeEn(unsigned char uEng, unsigned int ccEn)
{
	if(!uEngs) return(UENG_BADARG);
	uEngs[uEng]->setCondCodeEn(ccEn);
	return(UENG_SUCCESS);
}

int uEng_disableCtx(unsigned char uEng, unsigned int ctxMask)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->disableCtx(ctxMask));
}

int uEng_enableCtx(unsigned char uEng, unsigned int ctxMask)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->enableCtx(ctxMask));
}

int uEng_getCtxStatus(unsigned char uEng, unsigned char ctx, unsigned int *status)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getCtxStatus(ctx, status));
}

int uEng_setCtxStatus(unsigned char uEng, unsigned int ctxMask, unsigned int status)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setCtxStatus(ctxMask, status));
}


int uEng_getCtxSigEvents(unsigned char uEng, unsigned char ctx, unsigned int *events)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getCtxSigEvents(ctx, events));
}

int uEng_setCtxSigEvents(unsigned char uEng, unsigned int ctxMask, unsigned int events)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setCtxSigEvents(ctxMask, events));
}

int uEng_getCtxWakeupEvents(unsigned char uEng, unsigned char ctx, unsigned int *events)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getCtxWakeupEvents(ctx, events));
}

int uEng_setCtxWakeupEvents(unsigned char uEng, unsigned int ctxMask, unsigned int events)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setCtxWakeupEvents(ctxMask, events));
}


int uEng_getPC(unsigned char uEng, unsigned char ctx, unsigned int *upc)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getPC(ctx, upc));
}

int uEng_setPC(unsigned char uEng, unsigned int ctxMask, unsigned int upc)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setPC(ctxMask, upc));
}

int uEng_isUengActive(unsigned char uEng)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->isUengActive());
}


int uEng_resume(unsigned char uEng)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->resume());
}

int uEng_start(unsigned char uEng, unsigned char startCtx, unsigned int ctxEnMask)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->start(startCtx, ctxEnMask));
}

int uEng_getNumUword(unsigned char uEng, unsigned int uAddr,
					 unsigned short numWords, unsigned int *uWord)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getNumUword(uAddr, numWords, uWord));
}


int uEng_setNumUword(unsigned char uEng, unsigned int uAddr,
					 unsigned short numWords, unsigned int *uWord)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setNumUword(uAddr, numWords, uWord));
}

int uEng_getUword(unsigned char uEng, unsigned int uAddr, unsigned int *uWord)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getUword(uAddr, uWord));
}

int uEng_setUword(unsigned char uEng, unsigned int uAddr, unsigned int uWord)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setUword(uAddr, uWord));
}



int uEng_getCtxArb(unsigned char uEng, int unsigned *ctxArbCtl)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->getCtxArb(ctxArbCtl));
}

int uEng_setCtxArb(unsigned char uEng, int unsigned ctxArbCtl)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->setCtxArb(ctxArbCtl));
}


int uEng_readRelGpr(unsigned char uEng, unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->readRelGpr(type, ctx, regNum, data));
}

int uEng_writeRelGpr(unsigned char uEng, unsigned int type, unsigned char ctx, unsigned short regNum, unsigned int data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->writeRelGpr(type, ctx, regNum, data));
}

int uEng_readRelSramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->readRelSramRdXfer(ctx, regNum, data));
}


int uEng_readRelSramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->readRelSramWrXfer(ctx, regNum, data));
}

int uEng_writeRelSramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->writeRelSramRdXfer(ctx, regNum, data));
}

int uEng_writeRelSramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->writeRelSramWrXfer(ctx, regNum, data));
}

int uEng_writeRelSdramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->writeRelSdramWrXfer(ctx, regNum, data));
}

int uEng_writeRelSdramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->writeRelSdramRdXfer(ctx, regNum, data));
}

int uEng_readRelSdramWrXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->readRelSdramWrXfer(ctx, regNum, data));
}

int uEng_readRelSdramRdXfer(unsigned char uEng, unsigned char ctx, unsigned short regNum, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->readRelSdramRdXfer(ctx, regNum, data));
}

int uEng_readRxFifo(unsigned char uEng, unsigned short fifoAddr, unsigned int *data)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->readRxFifo(fifoAddr, data));
}

int uEng_writeTxFifo(unsigned char uEng, unsigned short fifoAddr, unsigned int *data, int setValid)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->writeTxFifo(fifoAddr, data, setValid));
}

int uEng_execMicroInst(unsigned char uEng, unsigned char ctx, unsigned int *microInst,
					   unsigned int numInst, int condCodeOff, unsigned int waitForMs)
{
	if(!uEngs) return(UENG_BADARG);
	return(uEngs[uEng]->execMicroInst(ctx, microInst, numInst, condCodeOff, waitForMs));
}
