/* rs_clnt.c: implementation of client support functions
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 7/9/98
 * revisions:
 * $History: rs_clnt.c $
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 4/13/99    Time: 8:15p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_Define_Automatic_Sim_Halt
 * 
 * *****************  Version 13  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:47p
 * Updated in $/development/RemoteServices
 * update COPYRIGHT
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:22p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_get_fifo_handle & XACT_Define_Callback_Ref_Status
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:02p
 * Updated in $/development/RemoteServices
 * Add support for xact_get|set_fifo_state_value
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:20p
 * Updated in $/development/RemoteServices
 * Add support for new transactor functions
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 10/09/98   Time: 10:11p
 * Updated in $/base_level3/RemoteServices
 * Fix a problem with thread termination
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 10/02/98   Time: 6:19p
 * Updated in $/base_level3/RemoteServices
 * Terminate a thread when close_connection is called
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 9/30/98    Time: 5:20p
 * Updated in $/base_level3/RemoteServices
 * Add pmap_unset() to reset the connection
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 9/22/98    Time: 5:30p
 * Updated in $/base_level3/RemoteServices
 * Fix bugs
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 9/14/98    Time: 6:01p
 * Updated in $/base_level3/RemoteServices
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 8/26/98    Time: 3:47p
 * Updated in $/base_level3/RemoteServices
 * Fix bugs
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 8/19/98    Time: 3:52p
 * Updated in $/base_level3/RemoteServices
 * Add "close_connection" function
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:11a
 * Updated in $/base_level3/RemoteServices
 * Add comments
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 7/24/98    Time: 5:29p
 * Created in $/base_level3/RemoteServices
 * Init. check-in
 */ 

#include <stdio.h>
#include <OncRpc32.h>
#include <thread.h>
#include <pmap_clnt.h>
#include "rs_xact.h"
#include "rs_control.h"
#include "remote_services.h"
#include "thread.h"
 		
/* this is not thread safe -- To be fixed */
tpRs_Client pRs_Client=NULL;
thread_T XactTh_CK;
/*
 * function: 	rs_gethostname_1
 * description: Send local host name to the server
 * return:      status from the server		
 */

int *
rs_gethostname_1(argp, clnt)
	char **argp;
	CLIENT *clnt;
{
	static int res;
    static struct timeval TIMEOUT = { 25, 0 };

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_GETHOSTNAME, xdr_wrapstring, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function: 	Open_Connection
 * description: create a connection and register the remote program
 *              with the remote server. 
 * return:      status from the server		
 */

RS_API Open_Connection(char *server,            /* remote server */
                       unsigned int RemoteProg) /* remote program */
{
    char hostname[100];
	int *result, status, retry;
    char *pHname = &hostname[0];
	if (server == NULL) 
		return RS_UNKNOWNSERVER; 


	OncRpc32Init();

    pRs_Client = malloc(sizeof(tRs_Client));
	if(pRs_Client == NULL)
		return RS_NOMEMORY;

	memset(pRs_Client, 0, sizeof(tRs_Client));

	pRs_Client->pCk_functions = malloc(sizeof(tCk_functions));
	if(pRs_Client->pCk_functions == NULL){
		free(pRs_Client);
		return RS_NOMEMORY;
	}
    memcpy(pRs_Client->server, server, strlen(server)); 

    switch (RemoteProg){
    case RS_XACT:
 		(void)pmap_unset(XACT_CALLBK, SA1200VERSION);
  
        /*
         * Create client "handle" used for calling SA1200PROG on the
         * server designated on the command line. We tell the rpc package
         * to use the "tcp" protocol when contacting the server.
         */
         pRs_Client->cl = clnt_create(pRs_Client->server, RS_XACT, SA1200VERSION, "tcp");
         if (pRs_Client->cl == NULL) {
		    /*
		     * Couldn't establish connection with server.
		     * Print error message and return.
		     */
		    clnt_pcreateerror(pRs_Client->server);

		    return RS_ERROR;
	    }
        /*
         * send host name to target system
         */
        if(gethostname(hostname, sizeof(hostname)) != 0){
            OncRpc32Cleanup();
            return RS_ERROR;
        }

        result = rs_gethostname_1(&pHname, pRs_Client->cl);
        status = Rs_GetStatus(result);
        if(status != RS_SUCCESS)
            OncRpc32Cleanup();
    
         break;
    case RsSimExt_Client:
    	(void)pmap_unset(XACT_CALLBK, SA1200VERSION);
        /*
         * Create client "handle" used for calling SA1200PROG on the
         * server designated on the command line. We tell the rpc package
         * to use the "tcp" protocol when contacting the server.
         */
        for(retry=0; retry<5; retry++){
            printf("retry=%d,server=%s \n", retry, pRs_Client->server);
            pRs_Client->cl = clnt_create(pRs_Client->server, RS_XACT, SA1200VERSION, "tcp");
            printf("return from clnt_create\n");
            if (pRs_Client->cl != NULL) 
                break;
            else
		        clnt_pcreateerror(pRs_Client->server);
        }
        if(retry == 5){
            /*
		     * Couldn't establish connection with server.
		     * Print error message and return.
		     */
		    clnt_pcreateerror(pRs_Client->server);
		    /*
		     * Couldn't establish connection with server.
		     * Print error message and return.
		     */

		    return RS_ERROR;

        }
        break;
    default:
        break;
    }
    return RS_SUCCESS;
}


/*
 * function: 	Close_Connection
 * description: Close a connection and register the remote program
 *              with the remote server. 
 * return:      status from the server		
 */

RS_API Close_Connection(char *server,            /* remote server */
                       unsigned int RemoteProg) /* remote program */
{
	if (server == NULL) 
		return RS_UNKNOWNSERVER; 

    OncRpc32Cleanup();
    free(pRs_Client);
    pRs_Client = NULL;

    return RS_SUCCESS;
}

/*
 * function: 	CreateXactCallBackServer
 * description: Create a server thread for accepting transactor
 *              call back requests
 * return:      status from the server		
 */

CreateXactCallBackServer()
{
	thread_T XactTh;
    XactTh = Thread_Create(MID_PRIORITY,(void *)callback_main,0); 
    return (RS_SUCCESS);
   
}
/*
 * function: 	Rs_GetStatus
 * description: return status from remote procedur call
 */

int Rs_GetStatus(int *result)
{

    if (result == NULL) {
    /*
     * An error occurred while calling the server. 
     */
	    return RS_SYSTEMERROR;
    }
    /*
     * Okay, we successfully called the remote procedure.
     */
    if (*result == 0) {
	    /*
	     * server is unable to call the remote procedure
         */
	    return RS_PROCUNAVAIL;
    } 
    return RS_SUCCESS;
}
