/* rs_proc.c: implementation of the server stubs for remote procedure transactor 
 *            functions
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 7/8/98
 * revisions:
 * $History: rs_proc.c $
 * 
 * *****************  Version 18  *****************
 * User: Qabuild      Date: 9/09/99    Time: 5:08p
 * Updated in $/dev_1200/Share/RemoteServices
 * added includes
 * 
 * *****************  Version 17  *****************
 * User: Drjohns1     Date: 8/06/99    Time: 5:06p
 * Updated in $/dev_1200/Share/RemoteServices
 * Added delay param to Sem_Wait
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 7/22/99    Time: 7:45p
 * Updated in $/dev_1200/share/RemoteServices
 * Fix bug #669 -  Remote Simulation doe not make connection
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 5/14/99    Time: 10:14a
 * Updated in $/dev_1200/Share/RemoteServices
 * Add support for xact_clock_with_remainder(),
 * xact_define_callback_default_go_clock_domain and
 * xact_stop_execution_at_clk 
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 4/13/99    Time: 8:15p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_Define_Automatic_Sim_Halt
 * 
 * *****************  Version 11  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:47p
 * Updated in $/development/RemoteServices
 * update COPYRIGHT
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:22p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_get_fifo_handle & XACT_Define_Callback_Ref_Status
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:02p
 * Updated in $/development/RemoteServices
 * Add support for xact_get|set_fifo_state_value
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:20p
 * Updated in $/development/RemoteServices
 * Add support for new transactor functions
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 10/09/98   Time: 9:55p
 * Updated in $/base_level3/RemoteServices
 * Fix a hang problem with termination
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 9/22/98    Time: 5:30p
 * Updated in $/base_level3/RemoteServices
 * Fix bugs
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 9/15/98    Time: 12:03p
 * Updated in $/base_level3/RemoteServices
 * Fix a bug for xact_find_wildcard_state_name function
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 9/14/98    Time: 5:55p
 * Updated in $/base_level3/RemoteServices
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 8/25/98    Time: 11:56a
 * Updated in $/base_level3/RemoteServices
 * fix a problem with re-init
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:11a
 * Updated in $/base_level3/RemoteServices
 * Add comments
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 7/24/98    Time: 5:29p
 * Created in $/base_level3/RemoteServices
 * Init. check-in
 *************************************************************/
#include <stdio.h>
#ifdef WIN32
#include "OncRpc32.h"
#include "pmap_clnt.h"
#include "xact_gui_dll.h"
#else
#include <rpc/rpc.h>
#include "xact_gui.h"
#endif

#include "rs_xact.h"
#include "remote_services.h"
#include <thread.h>
#include "sem.h"

#define  MaxTry 5

/* Default timeout can be changed using clnt_control() */
static struct timeval TIMEOUT = { 999999, 0 };
static CLIENT *Callback_cl = NULL;
sem_T hSem_Ck;
/* remote host name */
static char remote_host[100];
extern int exit_cmd;			/* exit flag */


/*
 * function: 	CreateClientConnection
 * description: Create a client connection with remote
 *              host for Xact callback functions
 *		
 */

CreateClientConnection(char *rs_host, 
                       unsigned int Prog)
{
    int retry;
    int status=RS_SUCCESS;
    Callback_cl=NULL;
    for(retry = 0; retry<MaxTry; retry++){
        Callback_cl = clnt_create(rs_host, Prog, SA1200VERSION, "tcp");
        if(Callback_cl != NULL)
        break;
    }
    if(retry ==MaxTry){ 
    /*
     * Couldn't establish connection with remote host.
     * Print error message and die.
     */
        printf("cann't creat a connection\n");
        clnt_pcreateerror(remote_host);
        status = RS_ERROR;
    }
	if(status != RS_ERROR)
		hSem_Ck = Sem_Init(1);
    return status;
}
/*
 * function: 	Xact_Thread
 * description: Create a client connection with remote host for Xact 
 *              callback functions and then start the transactor console
 *		
 */

void XACT_Thread()
{
#ifndef WIN32
    rpcTaskInit ();      
#endif
    if(CreateClientConnection(remote_host, XACT_CALLBK)!= RS_SUCCESS)
#ifdef WIN32
		ExitThread(0x1);
#else
		return;
#endif

    for(;;){
        if(exit_cmd == TRUE)
            break;  
        XACT_start_console();
		/* inform workbench that transactor is exiting */
		rs_xact_exit_1();
		break;
    }
#if 0
#ifdef WIN32
	ExitThread(0x1);
#else
	return;
#endif
#endif
	exit(1);
}

/*
 * function: 	XactOutputCallback
 * description: Send Xact output to client
 *		
 */

void XactOutputCallback(
     const char *pMessage)      /* int: output message from Xact */
{
    
    static int res;
    int status;
    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);
    if(Callback_cl != NULL){
        status = clnt_call(Callback_cl,RS_CK_SET_OUTPUT_MESSAGE, xdr_wrapstring, &pMessage, xdr_void, &res, TIMEOUT);

 	if (status != RPC_SUCCESS) 
		exit_cmd = TRUE;	
    }
	Sem_Post(hSem_Ck);
    return;
}

/*
 * function: 	XactPromptCallback
 * description: Send the Xact command prompt to the client
 *		
 */

void XactPromptCallback(
        char *pPrompt)          /* in:   command prompt from Xact */ 
{

    int status;
    static int res;
    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);
    if(Callback_cl != NULL){
	    status  = clnt_call(Callback_cl,RS_CK_SET_PROMPT, xdr_wrapstring, &pPrompt, xdr_void, &res, TIMEOUT);
/*        if ((status != RPC_SUCCESS) || (res == 0)) 	  */
		if(status != RPC_SUCCESS)
		   exit_cmd = TRUE;
    }
	Sem_Post(hSem_Ck);

    return;
}

/*
 * function: 	XactCmdCallback
 * description: Receive Xact command prompt from the remote client
 * return:      Xact console command
 */

char*
XactCmdCallback(void *argp)
{


    /* Default timeout can be changed using clnt_control() */
    static struct timeval TIMEOUT_CMD = {999999, 0 };

    static char *res;
    int status;
    bzero((char *)&res, sizeof(res));
    if(Callback_cl != NULL){
/*			Sem_Wait(hSem_Ck, SEM_WAITFOREVER); */
			status = clnt_call(Callback_cl,RS_CK_GET_CONSOLE_INPUT, xdr_void, argp, xdr_wrapstring, &res, TIMEOUT_CMD); 
/*			Sem_Post(hSem_Ck); */

			if (status != RPC_SUCCESS) {
                if(exit_cmd == TRUE) 	 
                    return NULL;
            }

    }
    return res;

}
/*
 * function: 	XactCreateChipCallback
 * description: Call callback when a chip of the specified name 
 *              has been called.  Send chip name to the remote host
 */

void XactCreateChipCallback(
                char *pChip)            /* in: chip name */
{
    static int res;
    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
        if (clnt_call(Callback_cl,RS_CK_CREATE_CHIP, xdr_wrapstring, &pChip, xdr_void, &res, TIMEOUT) != RPC_SUCCESS) {
            printf("RS_CK_CREATE_CHIP call failure\n");
            clnt_perror(Callback_cl, remote_host);
        }
    }
	Sem_Post(hSem_Ck);

    return;
}


/*
 * function: 	XactInitSimCallback
 * description: call a client callback when the simulation has been instantiated
 *              via the "init" command 
 */

void XactInitSimCallback()
{
    static int res;
    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
    	if (clnt_call(Callback_cl, RS_CK_INIT_SIM, xdr_void, NULL, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
            printf("RS_CK_INIT_SIM call failure\n");
            clnt_perror(Callback_cl, remote_host);
        }
    }
	Sem_Post(hSem_Ck);

    return;
}


/*
 * function: 	XactSimResetCallback
 * description: call a client callback when the simulation has been resetted
 *              via the "sim_reset" command 
 */

void XactSimResetCallback()
{
    static int res;
    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
    	if (clnt_call(Callback_cl, RS_CK_SIM_RESET, xdr_void, NULL, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
            printf("RS_CK_SIM_RESET call failure\n");
            clnt_perror(Callback_cl, remote_host);
        }
    }
	Sem_Post(hSem_Ck);

    return;
}

/*
 * function: 	XactRestoreCallback
 * description: call a client callback when the simulation has been reloaded
 *              via the "restore" command 
 */

void XactRestoreCallback()
{
    static int res;
    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
    	if (clnt_call(Callback_cl, RS_CK_RESTORE, xdr_void, NULL, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
            printf("RS_CK_RESTORE call failure\n");
            clnt_perror(Callback_cl, remote_host);
        }
    }
	Sem_Post(hSem_Ck);

    return;
}

/*
 * function: 	XactRefStatusCallback
 * description: call a client callback for ref status
 */

void XactRefStatusCallback(Meta_tag_ref *ref_data)
{
	static rs_meta_tag_ref argp;
	static char chipname[MAXBUFLEN];
	int res;


	argp.chip_name = &chipname[0];

	strcpy(argp.chip_name,ref_data->chip_name); 

	argp.dest_unit= ref_data->dest_unit;
	argp.cmd_type = ref_data->cmd_type;
	argp.sig_type = ref_data->sig_type;
	argp.fbox_pc = ref_data->fbox_pc;
	argp.fbox = ref_data->fbox;
	argp.ctx = ref_data->ctx;
	argp.fbox_to_be_signaled = ref_data->fbox_to_be_signaled;
	argp.ctx_to_be_signaled = ref_data->ctx_to_be_signaled;
	argp.word_cnt = ref_data->word_cnt;
	argp.ending_addr = ref_data->ending_addr;
	argp.rmw = ref_data->rmw;
	argp.creation_time = ref_data->creation_time;
	argp.update_type = ref_data->update_type;
	argp.qin_cycle = ref_data->qin_cycle;
	argp.qout_cycle = ref_data->qout_cycle;
	argp.process_cycle = ref_data->process_cycle;
	argp.signal_cycle = ref_data->signal_cycle;
	argp.wakeup_cycle = ref_data->wakeup_cycle;


	bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
	if (clnt_call(Callback_cl, RS_CK_REF_STATUS, xdr_rs_meta_tag_ref, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
            printf("RS_CK_REF_STATUS call failure\n");
            clnt_perror(Callback_cl, remote_host);
        }
    }
	Sem_Post(hSem_Ck);
    return;
}



/*
 * function:    XactLoadUcodeCallback
 * description: Call back when ucode file has been loaded into specified 
 *              chip/fbox  If the ucode_file name is NULL, this means that the 
 *              associated control store has been reset and no longer corresponds 
 *              to a ucode_file
 */

XactLoadUcodeCallback(char *pChip,          /* chip name */
                      int fbox_num,         /* F-thread  */
                      char *ucode_file)     /* ucode file name */
{
    static int res;
    static ucodeload argp;
    static char nullfile[1];
    nullfile[0]='\0';
    argp.chip_name = pChip;
    argp.fbox_num = fbox_num;
    if(ucode_file == NULL)
        argp.ucode_file = nullfile;
    else
       argp.ucode_file = ucode_file;

    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
        if (clnt_call(Callback_cl,RS_CK_LOAD_UCODE, xdr_ucodeload, &argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
            printf("RS_CK_LOAD_UCODE call failure\n");
		    clnt_perror(Callback_cl, remote_host);
        }
    }
	Sem_Post(hSem_Ck);

    return TRUE;

}

/*
 * function:    XactBreakpointCallback
 * description: Callback when break point is defined
 */


XactBreakpointCallback(char *pChip,        
                       int fbox_num,     
                       int pc, 
                       int contextmask)
{
    static int res;
    static breakevent argp;
    argp.chip_name = pChip;
    argp.fbox_num = fbox_num;
    argp.pc = pc;
    argp.contextmask = contextmask;

    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
	    if (clnt_call(Callback_cl,RS_CK_BREAKPOINT, xdr_breakevent, &argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
            printf("RS_CK_BREAKPOINT call failure\n");
            clnt_perror(Callback_cl, remote_host);
        }
    }
	Sem_Post(hSem_Ck);

    return TRUE;

}

/*
 * function:    XactBreakpointEventCallback
 * description: Callback whenever a breakpoint has been reached
 */


XactBreakpointEventCallback(char *pChip,
                            int fbox_num, 
                            int pc, 
                            int contextnum)
{
    static int res;
    int status;
    static breakevent argp;
    argp.chip_name = pChip;
    argp.fbox_num = fbox_num;
    argp.pc = pc;
    argp.contextmask = contextnum;

    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
	status = clnt_call(Callback_cl,RS_CK_BREAKPOINT_EVENT, xdr_breakevent, &argp, xdr_int, &res, TIMEOUT);
 /*  if ((status != RPC_SUCCESS) || (res == 0))  */
	if(status != RPC_SUCCESS)
		exit_cmd = TRUE;
    }
	Sem_Post(hSem_Ck);

    return TRUE;

}

/*
 * function:    XactStateTransitionCallback
 * description: callback to be invoked when the specified state makes a 
 *              transition
 */

XactStateTransitionCallback(XACT_HANDLE transitioning_state,
                            void *usercontext)
{
    static int res;
    int status;
    static state_transition_ck argp;
    argp.handle = transitioning_state;
    argp.usercontext = (unsigned int)usercontext;

    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
	    status = clnt_call(Callback_cl,RS_CK_STATE_TRANSITION, xdr_state_transition_ck, &argp, xdr_int, &res, TIMEOUT);
		if(status != RPC_SUCCESS)

		exit_cmd = TRUE;
    }
	Sem_Post(hSem_Ck);

    return TRUE;

}

/*
 * function:    XactCancelStateTransitionCallback
 * description: Callback when the transactor deletes the state associated 
 *              with the HANDLE. 
 */


XactCancelStateTransitionCallback(XACT_HANDLE x_handle,
                                  void *user_data)

{
    static int res;
    static state_transition_ck argp;
    int status;
    argp.handle = x_handle;
    argp.usercontext = (unsigned int)user_data;

    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
	    status = clnt_call(Callback_cl,RS_XACT_CANCEL_STATE_TRANSITION_CALLBACK, xdr_state_transition_ck, &argp, xdr_int, &res, TIMEOUT);
		if(status != RPC_SUCCESS)
		exit_cmd = TRUE;
    }
	Sem_Post(hSem_Ck);

   return TRUE;

}

/*
 * function:    XactSimHaltCallback
 * description: Callback when the simulator has prematurely halted model
 *              for the reason specified by the input argument 
 */

XactSimHaltCallback(int halt_status){

    static int res;

    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
	    res = clnt_call(Callback_cl,RS_CK_SIM_HALT,xdr_int, &halt_status, xdr_int, &res, TIMEOUT);
		if(res != RPC_SUCCESS)
		    exit_cmd = TRUE;
    
    }
	Sem_Post(hSem_Ck);

   return TRUE;

}


/*
 * function:    XactGoClockDomain
 * description: Callback to be invoked whenever default clock domain for "go" 
 *               simulation changes
 */

XactGoClockDomainCallback(RS_CLK_TYPE argp){

    static int res;

    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);

    if(Callback_cl != NULL){
	    res = clnt_call(Callback_cl,RS_CK_GO_CLOCK_DOMAIN, xdr_RS_CLK_TYPE, &argp, xdr_int, &res, TIMEOUT);
	
		if(res != RPC_SUCCESS)
		    exit_cmd = TRUE;
    
    }
	Sem_Post(hSem_Ck);

   return TRUE;

}

/*
 * function:    *rs_xact_initialize_1
 * description: call XACT_initialize 
 * return:      return status to the remote host
 */


int *rs_xact_initialize_1()
{

	static int result; /* must be static! */
	result = XACT_initialize();
	return (&result);

}

/*
 * function:    *rs_xact_start_console_1
 * description: create a transactor console thread
 * return:      return status to the remote host
 */

int *rs_xact_start_console_1()
{
	static thread_T XactTh=NULL;
	static int result=1; /* must be static! */

	if(XactTh != NULL);
		XactTh = Thread_Create(MID_PRIORITY,(void *)XACT_Thread,(void*)0); 
	return (&result);

}


/*
 * function:    *rs_xact_define_callback_set_prompt_1
 * description: call XACT_Define_Callback_Set_Prompt
 * return:      return status to the remote host
 */

int *rs_xact_define_callback_set_prompt_1()
{
	static int result; /* must be static! */
	result = XACT_Define_Callback_Set_Prompt(XactPromptCallback);
	return (&result);
}


/*
 * function:    *rs_xact_define_callback_ref_status_1
 * description: call xact_define_callback_ref_status
 * return:      return status to the remote host
 */

int *rs_xact_define_callback_ref_status_1()
{
	static int result; /* must be static! */
	result = XACT_Define_Callback_Ref_Status(XactRefStatusCallback);
	return (&result);
}


/*
 * function:    rs_xact_define_automatic_sim_halt_1
 * description: call XACT_Define_Automatic_Sim_Halt
 * return:      return status to the remote host
 */

int *rs_xact_define_auto_sim_halt_1()
{
	static int result; /* must be static! */
	result = XACT_Define_Automatic_Sim_Halt(XactSimHaltCallback);
	return (&result);
}


/*
 * function:    rs_ck_go_clock_domain_1
 * description: call XACT_Define_Callback_Default_Go_Clock_Domain
 * return:      return status to the remote host
 */

int *rs_ck_go_clock_domain_1()
{
	static int result; /* must be static! */
	result = XACT_Define_Callback_Default_Go_Clock_Domain(XactGoClockDomainCallback);
	return (&result);
}


/*
 * function:   rs_xact_define_callback_def_go_clock_domain_1
 * description: call XACT_Define_Callback_Default_Go_Clock_Domain
 * return:      return status to the remote host
 */

int *rs_xact_define_callback_def_go_clock_domain_1()
{
	static int result; /* must be static! */
	result = XACT_Define_Callback_Default_Go_Clock_Domain(XactGoClockDomainCallback);
	return (&result);
}



/*
 * function:    *rs_xact_define_callback_set_output_message_1
 * description: call XACT_Define_Callback_Output_Message
 * return:      return status to the remote host
 */

int *rs_xact_define_callback_set_output_message_1()
{

	static int result; /* must be static! */
	result=XACT_Define_Callback_Output_Message(XactOutputCallback);
	return (&result);

}

/*
 * function:    *rs_xact_define_callback_get_console_input_1
 * description: call XACT_Define_Callback_Get_Console_Input
 * return:      return status to the remote host
 */

int *rs_xact_define_callback_get_console_input_1(){
	static int result; /* must be static! */
	result = XACT_Define_Callback_Get_Console_Input(XactCmdCallback);
	return (&result);

}


/*
 * function:    *rs_xact_get_handle_1
 * description: call XACT_get_handle
 * return:      return status to the remote host
 */

int *
rs_xact_get_handle_1(handle *argp)

{
	static int res;
	res = XACT_get_handle( argp->name, argp->array_index);
	return (&res);
}


/*
 * function:    *rs_xact_get_fifo_handle_1
 * description: call XACT_get_fifo_handle
 * return:      return status to the remote host
 */

int *
rs_xact_get_fifo_handle_1(handle *argp)

{
	static int res;
	res = XACT_get_fifo_handle( argp->name, argp->array_index);
	return (&res);
}



/*
 * function:    *rs_xact_delete_handle_1
 * description: call XACT_delete_handle
 * return:      return status to the remote host
 */

int *
rs_xact_delete_handle_1(int *argp)
{
	static int res;
    res = XACT_delete_handle(*argp );
	return (&res);
}

/*
 * function:    *rs_xact_get_state_value_1
 * description: call XACT_get_state_value
 * return:      return state value to the remote host
 */


struct varintarr *
rs_xact_get_state_value_1(int *argp)
{

	static struct varintarr res;
    static unsigned int statevalue[MAXBUFLEN];

    res.pdata.pdata_val = &statevalue[0];
 	XACT_get_state_value( *argp, statevalue);
    res.pdata.pdata_len=MAXBUFLEN;
	return (&res);
}

/*
 * function:    *rs_xact_start_of_cycle_1
 * description: call XACT_start_of_cycle
 * return:      return status to the remote host
 */

int *
rs_xact_start_of_cycle_1(RS_CLK_TYPE *argp)
{
	static int res;
	res = XACT_start_of_cycle(*argp);
	return (&res);
}

/*
 * function:    *rs_xact_start_of_cycle_1
 * description: call XACT_clock_cycle
 * return:      return status to the remote host
 */


int *
rs_xact_clock_cycle_1(RS_CLK_TYPE *argp)
{
    static int res;
       
    res = XACT_clock_cycle(*argp);
    return (&res);
}


/*
 * function:    rs_xact_clk_cycle_with_remainder_1
 * description: call XACT_clock_cycle
 * return:      return status to the remote host
 */

varintarr *
rs_xact_clk_cycle_with_remainder_1(RS_CLK_TYPE *argp)
{
    static varintarr res;
	static double remainder;
	
       
    XACT_clock_cycle_with_remainder(*argp,&remainder);
	res.pdata.pdata_len = 2;
	res.pdata.pdata_val = (unsigned int *)&remainder;
    return (&res);
}

/*
 * function:    *rs_xact_define_callback_create_chip_1
 * description: call XACT_Define_Callback_Create_Chip
 * return:      return status to the remote host
 */


int *
rs_xact_define_callback_create_chip_1()
{

	static int result; /* must be static! */
	result = XACT_Define_Callback_Create_Chip(XactCreateChipCallback);
	return (&result);
}


/*
 * function:    *rs_xact_define_callback_load_ucode_1
 * description: call XACT_Define_Callback_Load_Ucode
 * return:      return status to the remote host
 */

int *
rs_xact_define_callback_load_ucode_1()
{

	static int result; /* must be static! */
	result = XACT_Define_Callback_Load_Ucode(XactLoadUcodeCallback);
	return (&result);

}


/*
 * function:    rs_xact_define_callback_init_sim_1
 * description: call XACT_Define_Callback_Init_Sim
 * return:      return status to the remote host
 */

int *
rs_xact_define_callback_init_sim_1()
{

	static int result; /* must be static! */
	result = XACT_Define_Callback_Init_Sim(XactInitSimCallback);
	return (&result);

}


/*
 * function:    rs_xact_define_callback_sim_reset_1
 * description: call XACT_Define_Callback_Sim_Reset
 * return:      return status to the remote host
 */

int *
rs_xact_define_callback_sim_reset_1()
{

	static int result; /* must be static! */
	result = XACT_Define_Callback_Sim_Reset(XactSimResetCallback);
	return (&result);

}


/*
 * function:    rs_xact_define_callback_restore_1
 * description: call XACT_Define_Callback_Restore
 * return:      return status to the remote host
 */

int *
rs_xact_define_callback_restore_1()
{

	static int result; /* must be static! */
	result = XACT_Define_Callback_Restore(XactRestoreCallback);
	return (&result);

}

/*
 * function:    *rs_xact_define_callback_breakpoint_1
 * description: call XACT_Define_Callback_Breakpoint
 * return:      return status to the remote host
 */

int *
rs_xact_define_callback_breakpoint_1()
{
	static int result; /* must be static! */
	result =  XACT_Define_Callback_Breakpoint(XactBreakpointCallback);
	return (&result);

}

/*
 * function:    *rs_xact_define_callback_breakpoint_event_1
 * description: call XACT_Define_Callback_Breakpoint_Event
 * return:      return status to the remote host
 */

int *
rs_xact_define_callback_breakpoint_event_1()
{
	static int result; /* must be static! */
	result =  XACT_Define_Callback_Breakpoint_Event(XactBreakpointEventCallback);
	return (&result);
}


/*
 * function:    *rs_xact_define_callback_state_transition_1
 * description: call XACT_Define_Callback_State_Transition
 * return:      return status to the remote host
 */

int *
rs_xact_define_callback_state_transition_1(state_transition_ck *argp)
{

	static int result; /* must be static! */
	result =  XACT_Define_Callback_State_Transition(argp->handle,
                argp->callback_priority,
                XactStateTransitionCallback,(void *)argp->usercontext);
	return (&result);

}

/*
 * function:    *rs_xact_cancel_state_transition_callback_1
 * description: call XACT_Define_Cancel_Callback_State_Transition
 * return:      return status to the remote host
 */

int *
rs_xact_cancel_state_transition_callback_1(int *argp)
{

	static int result; /* must be static! */
	result =  XACT_Define_Cancel_Callback_State_Transition(*argp,
                  XactCancelStateTransitionCallback);

	return (&result);

}

/*
 * function:    *rs_xact_define_cancel_callback_state_transition_1
 * description: call XACT_Define_Cancel_Callback_State_Transition
 * return:      return status to the remote host
 */


int *
rs_xact_define_cancel_callback_state_transition_1(int *argp)
{
	static int result; /* must be static! */
	result =  XACT_Define_Cancel_Callback_State_Transition(*argp,
                  XactCancelStateTransitionCallback);
	return (&result);

}


/*
 * function:    *rs_xact_get_state_info_1
 * description: call XACT_get_state_info
 * return:      return information to the remote host about the state 
 *              referenced by the specified handle:  
 *      		state_name:    name of state
 *              width:         width of state 
 *		        array_length:  indicates length of array ranging from 0 to 
 *                             array_length-1.  If this value is -1, the 
 *                             state is a non-array state. 
 */

state_info *
rs_xact_get_state_info_1(int *argp)
{
	static state_info res;
    static char s_name[MAXBUFLEN];
    int status;


    res.state_name=&s_name[0];
    memset(s_name,0, MAXBUFLEN);
    status= XACT_get_state_info(*argp, s_name, &res.width, &res.array_length);
    return (&res);
}



stat_char *
rs_xact_get_stat_characteristic_1(int *argp)

{
	static stat_char res;
	static Stat_characteristic stat_char_buf;

	memset(&stat_char_buf, 0, sizeof(stat_char_buf));
    res.res= XACT_get_stat_characteristic(*argp, &stat_char_buf );

	if(res.res == FALSE)
		return(&res);


 	res.min_sample = stat_char_buf.min_sample;
	res.max_sample = stat_char_buf.max_sample;
	res.num_samples = stat_char_buf.num_samples;
	res.sampled_sum = stat_char_buf.sampled_sum;
	res.min_bucket_value = stat_char_buf.min_bucket_value;
	res.max_bucket_value = stat_char_buf.max_bucket_value;
	res.bucket_size = stat_char_buf.bucket_size;
	res.num_buckets = stat_char_buf.num_buckets;
	res.titled_buckets = stat_char_buf.titled_buckets;
    res.stat_title.name = &stat_char_buf.stat_title.name[0];
    res.unit_specifier.name = &stat_char_buf.unit_specifier.name[0];

	return(&res);

}



get_bucket_data_OArg *
rs_xact_get_stat_bucket_data_1(get_bucket_data_IArg *argp)
{
	static get_bucket_data_OArg res;
	static Stat_str name_buf;
	memset(name_buf.name, 0, sizeof(name_buf.name));
	res.bucket_title.name = &name_buf.name[0];

    res.res = XACT_get_stat_bucket_data(argp->stat_handle, argp->bucket_index, &res.num_bucket_samples, &name_buf);

	return (&res);
}

int *
rs_xact_stop_execution_1(void *argp)
{

	static int res;
	res = XACT_stop_execution();
	return &res;
}

int *
rs_xact_stop_exe_at_clk_1(RS_CLK_TYPE *argp)
{

	static int res;
	res = XACT_stop_execution_at_clk(*argp);
	return &res;
}

/*
 * function:    *rs_xact_get_array_state_value_1
 * description: call XACT_get_array_state_value
 * return:      return array state value to the remote host
 */

varintarr *
rs_xact_get_array_state_value_1(state_handle *argp)
{

    static struct varintarr res;
    static unsigned int statevalue[MAXBUFLEN];
    res.pdata.pdata_val = &statevalue[0];
    XACT_get_array_state_value( argp->array_handle, argp->array_index,statevalue);
    res.pdata.pdata_len=MAXBUFLEN;
    return (&res);

}

/*
 * function:    *rs_xact_get_fifo_state_value_1
 * description: call XACT_get_fifo_state_value
 * return:      return array state value to the remote host
 */

varintarr *
rs_xact_get_fifo_state_value_1(state_handle *argp)
{

    static struct varintarr res;
    static unsigned int statevalue[MAXBUFLEN];
    res.pdata.pdata_val = &statevalue[0];
    XACT_get_fifo_state_value( argp->array_handle, argp->array_index,statevalue);
    res.pdata.pdata_len=MAXBUFLEN;
    return (&res);

}

/*
 * function:    *rs_xact_set_state_value_1
 * description: call XACT_set_state_value
 * return:      return status to the remote host
 */

int *
rs_xact_set_state_value_1(set_state_value *argp)
{

   	static int res;
    res =  XACT_set_state_value(argp->state_handle,
               argp->value.value_val);
	return (&res);

}

/*
 * function:    *rs_xact_set_array_state_value_1
 * description: call XACT_set_array_state_value
 * return:      return status to the remote host
 */


int *
rs_xact_set_array_state_value_1(set_array_state_value *argp)
{
	static int res;
    res =  XACT_set_array_state_value(argp->array_handle.array_handle,
              argp->array_handle.array_index,
              argp->value.value_val);
	return (&res);
}


/*
 * function:    *rs_xact_set_fifo_state_value_1
 * description: call XACT_set_array_state_value
 * return:      return status to the remote host
 */


int *
rs_xact_set_fifo_state_value_1(set_array_state_value *argp)
{
	static int res;
    res =  XACT_set_fifo_state_value(argp->array_handle.array_handle,
              argp->array_handle.array_index,
              argp->value.value_val);
	return (&res);
}


/*
 * function:    rs_xact_find_wildcard_state_name_1
 * description: call XACT_find_wildcard_state_name
 * return:      returns status to remote client:    
 *              0:   no match found                                             
 *	            1:   match found                                                
 *              -1:  match found but not returned due to size of rtn_name_buf   
 */

wildcard_state *
rs_xact_find_wildcard_state_name_1(char  **argp)
{
    static wildcard_state res;
    static char statename[MAXBUFLEN];
    res.rtn_name_buf= &statename[0];
    if(strlen(*argp) == 0)
      res.status = XACT_find_wildcard_state_name(NULL,statename, MAXBUFLEN);
    else
        res.status = XACT_find_wildcard_state_name(*argp,statename, MAXBUFLEN);


    return (&res);
}




/*
 * function:    *rs_executecommandstr_1
 * description: call rs_executecommandstr_1
 * return:      return status to the remote host
 */

int *
rs_executecommandstr_1(char **argp)
{

	static int result; /* must be static! */
	result =  XACT_ExecuteCommandStr(*argp);
	return (&result);

}

/*
 * function:    *rs_gethostname_1
 * description: get the remote host name
 * return:      return status to the remote host
 */

int *
rs_gethostname_1(char **argp)
{
 
    static int result = 1; /* must be static! */
    memset(remote_host, 0, sizeof(remote_host));
    strcpy(remote_host, *argp);
	return (&result);

}

/*
 *  send exit command to workbench
 */
int * rs_xact_exit_1()
{
	static struct timeval EXITTIMEOUT = { 0, 5 };

    static int res;
    bzero((char *)&res, sizeof(res));
	Sem_Wait(hSem_Ck, SEM_WAITFOREVER);
    if(Callback_cl != NULL){
		clnt_call(Callback_cl, RS_XACT_EXIT, xdr_void, NULL, xdr_int, &res, EXITTIMEOUT);
		res = 1;
	}
	Sem_Post(hSem_Ck);
    
    return (&res);
}


/*
 * function:    *rs_xact_init_gui_console_1
 * description: call XACT_init_gui_console 
 * return:      return status to the remote host
 */


int *rs_xact_init_gui_console_1()
{

	static int result; /* must be static! */
	result = XACT_init_gui_console();
	return (&result);

}


/*
 * function:    *rs_xact_init_gui_console_1
 * description: call XACT_init_gui_console 
 * return:      return status to the remote host
 */


int *rs_xact_gui_execute_command_1(char **argp)
{

	static int result; /* must be static! */
	result = XACT_gui_execute_command(*argp);
	return (&result);

}


