/* rs_xact_callback.c: implementation of callback functions
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 7/9/98
 * revisions:
 * $History: rs_xact_callback.c $
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 5/14/99    Time: 10:14a
 * Updated in $/dev_1200/Share/RemoteServices
 * Add support for xact_clock_with_remainder(),
 * xact_define_callback_default_go_clock_domain and
 * xact_stop_execution_at_clk 
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 4/13/99    Time: 8:15p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_Define_Automatic_Sim_Halt
 * 
 * *****************  Version 8  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:47p
 * Updated in $/development/RemoteServices
 * update COPYRIGHT
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:23p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_get_fifo_handle & XACT_Define_Callback_Ref_Status
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:02p
 * Updated in $/development/RemoteServices
 * Add support for xact_get|set_fifo_state_value
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:20p
 * Updated in $/development/RemoteServices
 * Add support for new transactor functions
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 10/09/98   Time: 10:11p
 * Updated in $/base_level3/RemoteServices
 * Fix a problem with thread termination
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 9/14/98    Time: 5:55p
 * Updated in $/base_level3/RemoteServices
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:11a
 * Updated in $/base_level3/RemoteServices
 * Add comments
 */

#include <stdio.h>
#ifdef WIN32
#include <OncRpc32.h>
#else
#include <rpc/rpc.h>
#endif
#include "rs_xact.h"
#include "rs_control.h"
#include "thread.h"
#include "xact.h"
extern tpRs_Client pRs_Client;
char ConsoleInputBuf[200];
#ifdef WIN32
HANDLE hCommandAvailable;
HANDLE hGoGetCommand;
thread_T thConsoleInput=NULL;
#endif
static char *pInputCommand;

/* Default timeout can be changed using clnt_control() */
static struct timeval TIMEOUT = { 25, 0 };

/*
 * function:    xact_callbk_1
 * description: this function is invoked by the RPC system whenever a 
 *              transactor server calls a callback function
 *		
 */

void xact_callbk_1(rqstp, transp)
	struct svc_req *rqstp;
	SVCXPRT *transp;
{
	union {
		char *rs_ck_set_prompt_1_arg;
		char *rs_ck_set_output_message_1_arg;
		char *rs_ck_create_chip_1_arg;
		ucodeload rs_ck_load_ucode_1_arg;
		breakevent rs_ck_breakpoint_1_arg;
		breakevent rs_ck_breakpoint_event_1_arg;
		state_transition_ck rs_ck_state_transition_1_arg;
		state_transition_ck rs_ck_cancel_state_transition_1_arg;
		rs_meta_tag_ref rs_ck_ref_status_1_arg;
        int rs_ck_sim_halt_1_arg;

	} argument;
	char *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	char *(*local)();

	switch (rqstp->rq_proc) {
	case NULLPROC:
		(void)svc_sendreply(transp, xdr_void, (char *)NULL);
		return;

	case RS_CK_SET_PROMPT:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_void;
		local = (char *(*)()) rs_ck_set_prompt_1;
		break;

	case RS_CK_SET_OUTPUT_MESSAGE:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_void;
		local = (char *(*)()) rs_ck_set_output_message_1;
		break;

	case RS_CK_GET_CONSOLE_INPUT:
		xdr_argument = xdr_void;
		xdr_result = xdr_wrapstring;
		local = (char *(*)()) rs_ck_get_console_input_1;
		break;

	case RS_CK_CREATE_CHIP:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_create_chip_1;
		break;

	case RS_CK_LOAD_UCODE:
		xdr_argument = xdr_ucodeload;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_load_ucode_1;
		break;

	case RS_CK_BREAKPOINT:
		xdr_argument = xdr_breakevent;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_breakpoint_1;
		break;

	case RS_CK_BREAKPOINT_EVENT:
		xdr_argument = xdr_breakevent;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_breakpoint_event_1;
		break;

	case RS_CK_STATE_TRANSITION:
		xdr_argument = xdr_state_transition_ck;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_state_transition_1;
		break;

	case RS_CK_CANCEL_STATE_TRANSITION:
		xdr_argument = xdr_state_transition_ck;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_cancel_state_transition_1;
		break;
	case RS_CK_INIT_SIM:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_init_sim_1;
		break;

	case RS_CK_SIM_RESET:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_sim_reset_1;
		break;

	case RS_CK_RESTORE:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_restore_1;
		break;

	case RS_XACT_EXIT:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_exit_1;
		break;

	case RS_CK_REF_STATUS:
		xdr_argument = xdr_rs_meta_tag_ref;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_ref_status_1;
        break;

	case RS_CK_SIM_HALT:
		xdr_argument = xdr_int;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_sim_halt_1;
		break;

	case RS_CK_GO_CLOCK_DOMAIN:
		xdr_argument = xdr_RS_CLK_TYPE;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_ck_go_clock_domain_1;
		break;

	
	default:
		svcerr_noproc(transp);
		return;
	}
	bzero((char *)&argument, sizeof(argument));
	if (!svc_getargs(transp, xdr_argument, &argument)) {
		svcerr_decode(transp);
		return;
	}
	result = (*local)(&argument, rqstp);
	if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
		svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, xdr_argument, &argument)) {
		(void)fprintf(stderr, "unable to free arguments\n");
		exit(1);
	}
}


/*
 * function: 	rs_ck_set_output_message_1
 * description: pass remote transactor output strings to local callback 
 *              function
 */


int *
rs_ck_set_output_message_1(char **argp)
{
	static int result = 1; /* must be static! */
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Output_Message != NULL))
	    pRs_Client->pCk_functions->Xact_Callback_Output_Message(*argp);
    else
        result = 0;
	return (&result);
}
/*
 * function: GetConsoleInput
 * description: fetch local console input from an external source 
 */
#ifdef WIN32
int GetConsoleInput()
{
  pInputCommand = &ConsoleInputBuf[0];
for(;;){
    WaitForSingleObject(hGoGetCommand, INFINITE);
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Get_Console_Input != NULL))
	    pInputCommand = pRs_Client->pCk_functions->Xact_Callback_Get_Console_Input();
	SetEvent(hCommandAvailable);
	ResetEvent(hGoGetCommand);

}
	return(1);
}
#endif
/*
 * function: 	rs_ck_get_console_input_1
 * description: fetch local console input from an external source 
 * return:      return the local console input string to the remote 
 *              transactor
 */

char **
rs_ck_get_console_input_1()
{

    static char buf[2], *res;
    res = &buf[0]; 
    buf[0] = '\0';

#ifdef WIN32	
	if(thConsoleInput == NULL)
		thConsoleInput = Thread_Create(MID_PRIORITY,(void *)GetConsoleInput,0); 
	/* go get next command */
    SetEvent(hGoGetCommand);

//	if(WaitForSingleObject(hCommandAvailable, 5000) != WAIT_OBJECT_0) {
	if(WaitForSingleObject(hCommandAvailable, 0xffffffff) != WAIT_OBJECT_0) {
		/* no input command */
		return (&res);
	}
	else {
		ResetEvent(hCommandAvailable);
 		return (&pInputCommand);
	}
#endif

}

/*
 * function: 	rs_ck_init_sim_1(
 * description: call callback when the simulation has been instantiated
 *              and initialized via the "init" command 
 */

int *
rs_ck_init_sim_1()
{
	static int res=1; /* must be static! */
  
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Init_Sim != NULL))
	    pRs_Client->pCk_functions->Xact_Callback_Init_Sim();
    else
        res=0;

    return (&res);

}

/*
 * function: 	rs_xact_exit_1
 * description: 
 */

int *
rs_xact_exit_1()
{
	static int res=1; /* must be static! */
#ifdef WIN32
	ExitThread(0x1);
#else
	exit(1);
#endif
	return(&res);
}

/*
 * function: 	rs_ck_sim_reset_1(
 * description: call callback when the simulation has been reset
 *              via the "sim_reset" command 
 */

int *
rs_ck_sim_reset_1()
{
	static int res=1; /* must be static! */
  
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Sim_Reset != NULL))
	    pRs_Client->pCk_functions->Xact_Callback_Sim_Reset();
    else
        res=0;

    return (&res);

}

/*
 * function: 	rs_ck_restore_1(
 * description: call callback when the simulation state has been 
 *              reloaded via the "restore" command 
 */

int *
rs_ck_restore_1()
{
	static int res=1; /* must be static! */
  
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Restore != NULL))
        pRs_Client->pCk_functions->Xact_Callback_Restore();
    else
        res=0;

    return (&res);

}



/*
 * function: 	rs_ck_set_prompt_1
 * description: pass the remote transctor command prompt to local 
 *              console input
 * return:      return status to the remote transactor
 */

int *
rs_ck_set_prompt_1(char **argp)
{

	static int result=1; /* must be static! */
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Set_Prompt!= NULL))
	    pRs_Client->pCk_functions->Xact_Callback_Set_Prompt(*argp);
    else
        result = 0;
	return (&result);

}

/*
 * function: 	rs_ck_create_chip_1
 * description: pass the chip name to the local client callback function 
 * return:      return status to the remote transactor
 */

int *
rs_ck_create_chip_1(char **argp)
{
	static int res=1;
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Create_Chip != NULL))
        pRs_Client->pCk_functions->Xact_Callback_Create_Chip(*argp);
    else
        res = 0;
	return (&res);

}

/*
 * function:    rs_ck_load_ucode_1
 * description: pass the load ucode information to the local host
 * return:      return status to the remote transactor
 */

int *
rs_ck_load_ucode_1(ucodeload *argp)
{
	static int res=1;
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Load_Ucode != NULL))
        pRs_Client->pCk_functions->Xact_Callback_Load_Ucode
        (argp->chip_name, argp->fbox_num, argp->ucode_file);
    else
        res = 0;
	return (&res);

}

/*
 * function: 	rs_ck_breakpoint_1
 * description: pass the break point information to local host 
 * return:      return status to the remote transactor
 */

int *
rs_ck_breakpoint_1(breakevent *argp)
{
	static int res=1;
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Breakpoint!= NULL))
        pRs_Client->pCk_functions->Xact_Callback_Breakpoint
        (argp->chip_name, argp->fbox_num, argp->pc, argp->contextmask);
    else
        res = 0;
	return (&res);
}

/*
 * function: 	rs_ck_breakpoint_event_1
 * description: pass the break point event to local host 
 * return:      return status to the remote transactor
 */

int *
rs_ck_breakpoint_event_1(breakevent *argp)
{
	static int res=1;

    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Breakpoint_Event != NULL))
    pRs_Client->pCk_functions->Xact_Callback_Breakpoint_Event
        (argp->chip_name, argp->fbox_num, argp->pc, argp->contextmask);
    else
        res = 0;


    return (&res);
}
/*
 * function: 	rs_ck_state_transition_1
 * description: invoke local call back function
 * return:      return status to the remote transactor
 */


int *
rs_ck_state_transition_1(state_transition_ck *argp)
{
	static int res=1;

    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_State_Transition != NULL))
    pRs_Client->pCk_functions->Xact_Callback_State_Transition
        (argp->handle, argp->usercontext);
    else
        res = 0;
    return (&res);
}

/*
 * function: 	rs_ck_cancel_state_transition_1
 * description: invoke local call back function
 * return:      return status to the remote transactor
 */
int *
rs_ck_cancel_state_transition_1(state_transition_ck *argp)
{
	static int res=1;
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Cancel_State_Transition!= NULL))
    pRs_Client->pCk_functions->Xact_Callback_Cancel_State_Transition
        (argp->handle, argp->usercontext);
    else
        res=0;
    
    return (&res);
}


/*
 * function: 	rs_ck_ref_status_1
 * description: invoke local call back function
 * return:      return status to the remote transactor
 */

int *rs_ck_ref_status_1(rs_meta_tag_ref *argp)
{
	static int res=1;
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Load_Ucode != NULL))
        pRs_Client->pCk_functions->Xact_Callback_RefStatus((struct Meta_tag_ref *)argp);
    else
        res = 0;
	return (&res);
}

int *
rs_ck_sim_halt_1(int *argp)
{
	static int res=1;
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Sim_Halt != NULL))
        pRs_Client->pCk_functions->Xact_Callback_Sim_Halt((HALT_STATUS)*argp);
    else
        res = 0;
	return (&res);
}


int *
rs_ck_go_clock_domain_1(RS_CLK_TYPE *argp)
{
	static int res=1;
    if((pRs_Client != NULL) && (pRs_Client->pCk_functions->Xact_Callback_Go_Clk_Domain != NULL))
        pRs_Client->pCk_functions->Xact_Callback_Go_Clk_Domain( (enum CLK_TYPE)(*argp));
    else
        res = 0;
	return (&res);

}

