/* rs_xact_clnt.c: implementation of the client stubs for remote procedure transactor functions
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 7/9/98
 * revisions:
 * $History: rs_xact_clnt.c $
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 7/22/99    Time: 7:45p
 * Updated in $/dev_1200/share/RemoteServices
 * Fix bug #669 -  Remote Simulation doe not make connection
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 5/14/99    Time: 10:14a
 * Updated in $/dev_1200/Share/RemoteServices
 * Add support for xact_clock_with_remainder(),
 * xact_define_callback_default_go_clock_domain and
 * xact_stop_execution_at_clk 
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 4/13/99    Time: 8:15p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_Define_Automatic_Sim_Halt
 * 
 * *****************  Version 10  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:47p
 * Updated in $/development/RemoteServices
 * update COPYRIGHT
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:23p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_get_fifo_handle & XACT_Define_Callback_Ref_Status
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:02p
 * Updated in $/development/RemoteServices
 * Add support for xact_get|set_fifo_state_value
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:20p
 * Updated in $/development/RemoteServices
 * Add support for new transactor functions
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 10/09/98   Time: 9:55p
 * Updated in $/base_level3/RemoteServices
 * Fix a hang problem with termination
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 9/14/98    Time: 6:01p
 * Updated in $/base_level3/RemoteServices
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 8/26/98    Time: 3:47p
 * Updated in $/base_level3/RemoteServices
 * Fix bugs
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 7/31/98    Time: 7:13p
 * Updated in $/base_level3/RemoteServices
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:11a
 * Updated in $/base_level3/RemoteServices
 * Add comments
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 7/24/98    Time: 5:29p
 * Created in $/base_level3/RemoteServices
 * Init. check-in
 */ 

#include <stdio.h>
#include <OncRpc32.h>
#include <thread.h>
#include "rs_control.h"
#include "rs_xact.h"
#include "remote_services.h"
 		
/* Default timeout can be changed using clnt_control() */
static struct timeval TIMEOUT = { 30, 0 };

/*
 * function:    rs_xact_initialize_1
 * description: call rs_xact_initialize_1 on server
 */

int *
rs_xact_initialize_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_INITIALIZE, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_start_console_1
 * description: call rs_xact_start_console_1 on server
 */

int *
rs_xact_start_console_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_START_CONSOLE, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


/*
 * function:    rs_xact_define_callback_set_prompt_1
 * description: call rs_xact_define_callback_set_prompt_1 on server
 */

int *
rs_xact_define_callback_set_prompt_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_SET_PROMPT, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_set_output_message_1
 * description: call rs_xact_define_callback_set_output_message_1 on server
 */

int *
rs_xact_define_callback_set_output_message_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_SET_OUTPUT_MESSAGE, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_get_console_input_1
 * description: call rs_xact_define_callback_get_console_input_1 on server
 */

int *
rs_xact_define_callback_get_console_input_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_GET_CONSOLE_INPUT, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


/*
 * function:    rs_xact_get_handle_1
 * description: call rs_xact_get_handle_1 on server
 */

int *
rs_xact_get_handle_1(argp, clnt)
	handle *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_HANDLE, xdr_handle, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_delete_handle_1
 * description: call rs_xact_delete_handle_1 on server
 */

int *
rs_xact_delete_handle_1(argp, clnt)
	int *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DELETE_HANDLE, xdr_int, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_initialize_1
 * description: call rs_xact_initialize_1 on server
 */

varintarr *
rs_xact_get_state_value_1(argp, clnt)
	int *argp;
	CLIENT *clnt;
{
	static varintarr res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_STATE_VALUE, xdr_int, argp, xdr_varintarr, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}

	return (&res);
}

/*
 * function:    rs_xact_get_state_info_1
 * description: call rs_xact_get_state_info_1 on server
 */

state_info *
rs_xact_get_state_info_1(argp, clnt)
	int *argp;
	CLIENT *clnt;
{
	static state_info res;
	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_STATE_INFO, xdr_int, argp, xdr_state_info, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}

	return (&res);
}

/*
 * function:    rs_xact_get_array_state_value_1
 * description: call rs_xact_get_array_state_value_1 on server
 */


varintarr *
rs_xact_get_array_state_value_1(argp, clnt)
	state_handle *argp;
	CLIENT *clnt;
{
	static varintarr res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_ARRAY_STATE_VALUE, xdr_state_handle, argp, xdr_varintarr, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_start_of_cycle_1
 * description: call rs_xact_start_of_cycle_1 on server
 */

int *
rs_xact_start_of_cycle_1(argp, clnt)
	RS_CLK_TYPE *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_START_OF_CYCLE, xdr_RS_CLK_TYPE, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_clock_cycle_1
 * description: call rs_xact_clock_cycle_1 on server
 */

int *
rs_xact_clock_cycle_1(argp, clnt)
	RS_CLK_TYPE *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_CLOCK_CYCLE, xdr_RS_CLK_TYPE, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_create_chip_1
 * description: call rs_xact_define_callback_create_chip_1 on server
 */

int *
rs_xact_define_callback_create_chip_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_CREATE_CHIP, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_load_ucode_1
 * description: call rs_xact_define_callback_load_ucode_1 on server
 */

int *
rs_xact_define_callback_load_ucode_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_LOAD_UCODE, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_breakpoint_1
 * description: call rs_xact_define_callback_breakpoint_1 on server
 */

int *
rs_xact_define_callback_breakpoint_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_BREAKPOINT, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_breakpoint_event_1
 * description: call rs_xact_define_callback_breakpoint_event_1 on server
 */

int *
rs_xact_define_callback_breakpoint_event_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_BREAKPOINT_EVENT, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_state_transition_1
 * description: call rs_xact_define_callback_state_transition_1 on server
 */

int *
rs_xact_define_callback_state_transition_1(argp, clnt)
	state_transition_ck *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_STATE_TRANSITION, xdr_state_transition_ck, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_cancel_callback_state_transition_1
 * description: call rs_xact_define_cancel_callback_state_transition_1 on server
 */

int *
rs_xact_define_cancel_callback_state_transition_1(argp, clnt)
	int *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CANCEL_CALLBACK_STATE_TRANSITION, xdr_int, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
         return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_cancel_state_transition_callback_1
 * description: call rs_xact_cancel_state_transition_callback_1 on server
 */

int *
rs_xact_cancel_state_transition_callback_1(argp, clnt)
	int *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_CANCEL_STATE_TRANSITION_CALLBACK, xdr_int, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}
varintarr *
rs_xact_clk_cycle_with_remainder_1(argp, clnt)
	RS_CLK_TYPE *argp;
	CLIENT *clnt;
{
	static varintarr res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_CLK_CYCLE_WITH_REMAINDER, xdr_RS_CLK_TYPE, argp, xdr_varintarr, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_xact_define_callback_def_go_clock_domain_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_DEF_GO_CLOCK_DOMAIN, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_xact_stop_exe_at_clk_1(argp, clnt)
	RS_CLK_TYPE *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_STOP_EXE_AT_CLK, xdr_RS_CLK_TYPE, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


/*
 * function:    rs_xact_set_state_value_1
 * description: call rs_xact_set_state_value_1 on server
 */

int *
rs_xact_set_state_value_1(argp, clnt)
	set_state_value *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_SET_STATE_VALUE, xdr_set_state_value, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_set_array_state_value_1
 * description: call rs_xact_set_array_state_value_1 on server
 */

int *
rs_xact_set_array_state_value_1(argp, clnt)
	set_array_state_value *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_SET_ARRAY_STATE_VALUE, xdr_set_array_state_value, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_executecommandstr_1
 * description: call rs_executecommandstr_1 on server
 */

int *
rs_executecommandstr_1(argp, clnt)
	char **argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_EXECUTECOMMANDSTR, xdr_wrapstring, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}



/*
 * function:    rs_xact_find_wildcard_state_name_1
 * description: call rs_xact_find_wildcard_state_name_1 on server
 */

wildcard_state *
rs_xact_find_wildcard_state_name_1(argp, clnt)
	char **argp;
	CLIENT *clnt;
{
	static wildcard_state res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_FIND_WILDCARD_STATE_NAME, xdr_wrapstring, argp, xdr_wildcard_state, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}



/*
 * function:    rs_xact_define_callback_init_sim_1
 * description: call rs_xact_define_callback_init_sim_1 on server
 */

int *
rs_xact_define_callback_init_sim_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_INIT_SIM, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


/*
 * function:    rs_xact_define_callback_sim_reset_1
 * description: call rs_xact_define_callback_sim_reset_1 on server
 */

int *
rs_xact_define_callback_sim_reset_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_SIM_RESET, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/*
 * function:    rs_xact_define_callback_restore_1
 * description: call rs_xact_define_callback_restore_1 on server
 */

int *
rs_xact_define_callback_restore_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_RESTORE, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

stat_char *
rs_xact_get_stat_characteristic_1(argp, clnt)
	int *argp;
	CLIENT *clnt;
{
	static stat_char res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_STAT_CHARACTERISTIC, xdr_int, argp, xdr_stat_char, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


get_bucket_data_OArg *
rs_xact_get_stat_bucket_data_1(argp, clnt)
	get_bucket_data_IArg *argp;
	CLIENT *clnt;
{
	static get_bucket_data_OArg res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_STAT_BUCKET_DATA, xdr_get_bucket_data_IArg, argp, xdr_get_bucket_data_OArg, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_xact_stop_execution_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_STOP_EXECUTION, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

varintarr *
rs_xact_get_fifo_state_value_1(argp, clnt)
	state_handle *argp;
	CLIENT *clnt;
{
	static varintarr res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_FIFO_STATE_VALUE, xdr_state_handle, argp, xdr_varintarr, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_xact_set_fifo_state_value_1(argp, clnt)
	set_array_state_value *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_SET_FIFO_STATE_VALUE, xdr_set_array_state_value, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_xact_get_fifo_handle_1(argp, clnt)
	handle *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GET_FIFO_HANDLE, xdr_handle, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_xact_define_callback_ref_status_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_CALLBACK_REF_STATUS, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_xact_define_auto_sim_halt_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_DEFINE_AUTO_SIM_HALT, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_xact_init_gui_console_1(argp, clnt)
	void *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_INIT_GUI_CONSOLE, xdr_void, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_xact_gui_execute_command_1(argp, clnt)
	char **argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_GUI_EXECUTE_COMMAND, xdr_wrapstring, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}