/* rs_xact_dll.c: 
 *      remote SA1200 transactor functions 
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 7/9/98
 * revisions:
 * $History: rs_xact_dll.c $
 * 
 * *****************  Version 16  *****************
 * User: Lhui         Date: 7/22/99    Time: 7:45p
 * Updated in $/dev_1200/share/RemoteServices
 * Fix bug #669 -  Remote Simulation doe not make connection
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 5/14/99    Time: 10:14a
 * Updated in $/dev_1200/Share/RemoteServices
 * Add support for xact_clock_with_remainder(),
 * xact_define_callback_default_go_clock_domain and
 * xact_stop_execution_at_clk 
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 4/13/99    Time: 8:15p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_Define_Automatic_Sim_Halt
 * 
 * *****************  Version 12  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:47p
 * Updated in $/development/RemoteServices
 * update COPYRIGHT
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:23p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_get_fifo_handle & XACT_Define_Callback_Ref_Status
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:02p
 * Updated in $/development/RemoteServices
 * Add support for xact_get|set_fifo_state_value
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:20p
 * Updated in $/development/RemoteServices
 * Add support for new transactor functions
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 10/09/98   Time: 10:24p
 * Updated in $/base_level3/RemoteServices
 * Add closehandle
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 10/09/98   Time: 9:55p
 * Updated in $/base_level3/RemoteServices
 * Fix a hang problem with termination
 * 
 * *****************  Version 6  *****************
 * User: Rdmurato     Date: 10/01/98   Time: 5:27p
 * Updated in $/base_level3/RemoteServices
 * Fix bug in get_state_info function; need to handle null state name
 * pointer.
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 9/15/98    Time: 12:03p
 * Updated in $/base_level3/RemoteServices
 * Fix a bug for xact_find_wildcard_state_name function
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 9/14/98    Time: 5:55p
 * Updated in $/base_level3/RemoteServices
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 8/26/98    Time: 3:47p
 * Updated in $/base_level3/RemoteServices
 * Fix bugs
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:11a
 * Updated in $/base_level3/RemoteServices
 * Add comments
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 7/24/98    Time: 5:29p
 * Created in $/base_level3/RemoteServices
 * Init. check-in
 */ 

#include <stdio.h>
#include <OncRpc32.h>
#include <thread.h>
#include <pmap_clnt.h>
#include "rs_control.h"
#include "rs_xact.h"
#include "rsxact_dll.h"
#include "remote_services.h"
 		
extern tpRs_Client pRs_Client;
/* need to re-work after BL3 */
#ifdef WIN32
HANDLE hCommandAvailable;
HANDLE hGoGetCommand;
thread_T thConsoleInput;
#endif

/*
 *  Create a connection for transactor callback functions
 */
callback_main()
{
   SVCXPRT *transp;
 
   (void)pmap_unset(XACT_CALLBK, SA1200VERSION);

	transp = svcudp_create(RPC_ANYSOCK);
	if (transp == NULL) {
		(void)fprintf(stderr, "cannot create udp service.\n");
		return RS_SYSTEMERROR;
	}
	if (!svc_register(transp, XACT_CALLBK, SA1200VERSION, xact_callbk_1, IPPROTO_UDP)) {
		(void)fprintf(stderr, "unable to register XACT_CALLBK, SA1200VERSION, udp).\n");
		return RS_UDPCANTREGPROG;
	}

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		(void)fprintf(stderr, "cannot create tcp service.\n");
		return RS_SYSTEMERROR;
	}
	if (!svc_register(transp, XACT_CALLBK, SA1200VERSION, xact_callbk_1, IPPROTO_TCP)) {
		(void)fprintf(stderr, "unable to registerXACT_CALLBK, SA1200VERSION, tcp).\n");
		return RS_TCPCANTREGPROG;
	}

	svc_run();
	(void)fprintf(stderr, "svc_run returned\n");
	return RS_SYSTEMERROR;
}

/*
 * function: 	RS_XACT_initialize
 * description: call the remote procedure "xact_start_init" on the server
 */

RS_XACTAPI RS_XACT_initialize()
{
	int *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    
	result = rs_xact_initialize_1("", pRs_Client->cl);
	return Rs_GetStatus(result);
}

/*
 * function: 	RS_XACT_start_console
 * description: call the remote procedure "xact_start_console" on the server
 */

RS_XACTAPI RS_XACT_start_console()
{
	int *result;
    int status;
	thread_T XactTh_CK=NULL;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	XactTh_CK = Thread_Create(MID_PRIORITY,(void *)callback_main,0); 
/* need to re-work after BL3 */
#ifdef WIN32
    hCommandAvailable = CreateEvent(NULL, FALSE, FALSE, NULL);
    hGoGetCommand = CreateEvent(NULL, FALSE, FALSE, NULL);
	Sleep(1000);
#else
	taskDelay(200);	
#endif

    result = rs_xact_start_console_1("", pRs_Client->cl);
    if((status = Rs_GetStatus(result)) != RS_SUCCESS)
        Thread_Cancel(XactTh_CK);
#ifdef WIN32
	else{
	   /* waiting for the xact exit command */
		WaitForSingleObject(XactTh_CK,INFINITE);
		Thread_Cancel(thConsoleInput);
		CloseHandle(XactTh_CK);
		CloseHandle(hCommandAvailable);
		CloseHandle(hGoGetCommand);

	}
#endif

    return status;

}

/*
 * function: 	RS_XACT_Define_Callback_Set_Prompt
 * description: call the remote procedure "XACT_Define_Callback_Set_Prompt" 
 *              on the server
 */

/***********************************************************************/
RS_XACTAPI RS_XACT_Define_Callback_Set_Prompt( void(*fp)( char *prompt_str ) )
	{
	/* this routine registers the callback to pass the transactor prompt */
	/* to an external command line.  It returns TRUE if successful, and */
	/* FALSE otherwise. */

	int *result;
	int status;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;
	result = rs_xact_define_callback_set_prompt_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Set_Prompt = fp;

	return status;
	}

/*
 * function: 	RS_XACT_Define_Callback_Output_Message
 * description: call the remote procedure "XACT_Define_Callback_Output_Message" 
 *              on the server
 */

RS_XACTAPI RS_XACT_Define_Callback_Output_Message( void(*fp)(const char *message) )
	{
	int *result;
	int status;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


	result = rs_xact_define_callback_set_output_message_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Output_Message = fp;

	return status;
	}

/*
 * function: 	RS_XACT_Define_Automatic_Sim_Halt
 * description: call the remote procedure "XACT_Define_Automatic_Sim_Halt" 
 *              on the server
 */

RS_XACTAPI RS_XACT_Define_Automatic_Sim_Halt( void(*fp)(HALT_STATUS halt_status) )
	{
	int *result;
	int status;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


	result = rs_xact_define_auto_sim_halt_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Sim_Halt = fp;

	return status;
	}


/*
 * function: 	RS_XACT_Define_callback_default_go_clock_domain
 * description: call the remote procedure "XACT_Define_callback_default_go_clock_domain" 
 *              on the server
 */

RS_XACTAPI RS_XACT_Define_Callback_Default_Go_Clock_Domain( void(*fp)(enum CLK_TYPE clk_type) )
	{
	int *result;
	int status;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


	result =rs_xact_define_callback_def_go_clock_domain_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Go_Clk_Domain = fp;

	return status;
	}


/*
 * function: 	RS_XACT_Define_Callback_Get_Console_Input
 * description: call the remote procedure "XACT_Define_Callback_Get_Console_Input" 
 *              on the server
 */

RS_XACTAPI RS_XACT_Define_Callback_Get_Console_Input( char *(*console_input)() )
	{
	/* this routine defines the callback necessary when the external program */
	/* supplies console input to the transactor.  It returns TRUE if successful, */
	/* and FALSE otherwise. */
	int *result;
    int status;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	result = rs_xact_define_callback_get_console_input_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
 		pRs_Client->pCk_functions->Xact_Callback_Get_Console_Input = console_input;

	return status;
	}


/*
 * function: 	RS_XACT_get_handle
 * description: call the remote procedure "XACT_get_handle" on the server
 */

RS_XACTAPI_NORET RS_XACT_HANDLE RS_XACT_get_handle( 
                                  char *state_name, 
                                  int array_index )
{
    int *result_handle;
    handle  handle_info;
    handle_info.name = state_name;
    handle_info.array_index = array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result_handle = rs_xact_get_handle_1(&handle_info, pRs_Client->cl);

    if (result_handle == NULL){ 
		clnt_perror(pRs_Client->cl, "testtarget");
        return RS_INVALID_XACT_HANDLE;
    }
    
    return *result_handle;
}


/*
 * function: 	RS_XACT_get_fifo_handle
 * description: call the remote procedure "RS_XACT_get_fifo_handle" on the server
 */

RS_XACTAPI_NORET RS_XACT_HANDLE RS_XACT_get_fifo_handle( 
                                  char *state_name, 
                                  int array_index )
{
    int *result_handle;
    handle  handle_info;
    handle_info.name = state_name;
    handle_info.array_index = array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result_handle = rs_xact_get_fifo_handle_1(&handle_info, pRs_Client->cl);

    if (result_handle == NULL){ 
		clnt_perror(pRs_Client->cl, "testtarget");
        return RS_INVALID_XACT_HANDLE;
    }
    
    return *result_handle;
}



/*
 * function: 	RS_XACT_delete_handle
 * description: call the remote procedure "XACT_delete_handle" on the server
 */

RS_XACTAPI RS_XACT_delete_handle( RS_XACT_HANDLE this_handle )
{
    int *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_delete_handle_1(&this_handle, pRs_Client->cl);
	return Rs_GetStatus(result);

}

/*
 * function: 	RS_XACT_get_state_value
 * description: call the remote procedure "XACT_get_state_value" on the server
 */

RS_XACTAPI RS_XACT_get_state_value( 
                RS_XACT_HANDLE state_handle, 
                unsigned int *value ) 
{
    varintarr *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


    result = rs_xact_get_state_value_1(&state_handle, pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;
    *value = *result->pdata.pdata_val;
    return RS_SUCCESS; 

}

/*
 * function: 	RS_XACT_start_of_cycle
 * description: call the remote procedure "RS_XACT_start_of_cycle" on the server
 */

RS_XACTAPI RS_XACT_start_of_cycle( enum CLK_TYPE clk_type )
{

    int *result;    

   if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_start_of_cycle_1(&clk_type, pRs_Client->cl);
    return Rs_GetStatus(result);

  
}

/*
 * function: 	RS_XACT_clock_cycle
 * description: call the remote procedure "RS_XACT_clock_cycle" on the server
 */

RS_XACTAPI RS_XACT_clock_cycle( enum CLK_TYPE clk_type)
{
    int *result;    

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;
    

    result = rs_xact_clock_cycle_1(&clk_type, pRs_Client->cl);
    if (result == NULL) 
	    return -1;
    return *result;

}


/*
 * function: 	RS_XACT_clock_with_remainder
 * description: call the remote procedure "XACT_clock_with_remainder" on the server
 */

RS_XACTAPI RS_XACT_clock_cycle_with_remainder( enum CLK_TYPE clk_type, double *percent_remainder)
{
    varintarr *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;
    

    result = rs_xact_clk_cycle_with_remainder_1(&clk_type, pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;
	*percent_remainder = *result->pdata.pdata_val; 

	return RS_SUCCESS;


}


/*
 * function: 	RS_XACT_Define_Callback_Create_Chip
 * description: call the remote procedure "XACT_Define_Callback_Create_Chip" on the server
 */

RS_XACTAPI RS_XACT_Define_Callback_Create_Chip( void(*fp)( char *chip_name ) )
{
	int *result;
	int status;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	result = rs_xact_define_callback_create_chip_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Create_Chip = fp;

	return status;

}


/*
 * function: 	RS_XACT_Define_Callback_Init_Sim
 * description: call the remote procedure "XACT_Define_Callback_Init_Sim" on the server
 */
RS_XACT_Define_Callback_Init_Sim( void(*fp)() )
{
	int *result;
	int status;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	result = rs_xact_define_callback_init_sim_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Init_Sim = fp;

	return status;

}

/*
 * function: 	RS_XACT_Define_Callback_Sim_Reset
 * description: call the remote procedure "XACT_Define_Callback_Sim_Reset" on the server
 */
RS_XACT_Define_Callback_Sim_Reset( void(*fp)() )
{
	int *result;
	int status;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	result = rs_xact_define_callback_sim_reset_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Sim_Reset = fp;

	return status;

}


/*
 * function: 	RS_XACT_Define_Callback_Restore
 * description: call the remote procedure "XACT_Define_Callback_Restore" on the server
 */
RS_XACT_Define_Callback_Restore( void(*fp)() )
{
	int *result;
	int status;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	result = rs_xact_define_callback_restore_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Restore = fp;

	return status;

}


/*
 * function: 	RS_XACT_Define_Callback_Ref_Status
 * description: call the remote procedure "XACT_Define_Callback_Ref_Status" on the server
 */
RS_XACT_Define_CallbackRef_Status( void(*fp)(Meta_tag_ref *ref_data))
{
	int *result;
	int status;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	result = rs_xact_define_callback_ref_status_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_RefStatus = fp;

	return status;

}


/*
 * function: 	RS_XACT_Define_Callback_Load_Ucode
 * description: call the remote procedure "XACT_Define_Callback_Load_Ucode" on the server
 */

RS_XACTAPI RS_XACT_Define_Callback_Load_Ucode( void(*fp)( char *chip_name,
												   int fbox_num,
												   char *ucode_file ) )
{
	int *result;
	int status;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


	result = rs_xact_define_callback_load_ucode_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Load_Ucode = fp;

	return status;

}

/*
 * function: 	RS_XACT_Define_Callback_Breakpoint
 * description: call the remote procedure "XACT_Define_Callback_Breakpoint" on the server
 */


RS_XACTAPI RS_XACT_Define_Callback_Breakpoint( 
            void (*fp)( char *chip_name, int fbox_num, int PC, int ContextMask ) ) 

{
	int *result;
	int status;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


	result = rs_xact_define_callback_breakpoint_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Breakpoint = fp;

	return status;

}

/*
 * function: 	RS_XACT_Define_Callback_Breakpoint_Event
 * description: call the remote procedure "XACT_Define_Callback_Breakpoint_Event" on the server
 */

RS_XACTAPI RS_XACT_Define_Callback_Breakpoint_Event( void (*fp)
                   ( char *chip_name, int fbox_num, int PC, int ContextNum ) )

{
	int *result;
	int status;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


	result = rs_xact_define_callback_breakpoint_event_1("", pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Breakpoint_Event = fp;

	return status;

}

/*
 * function: 	RS_XACT_Define_Callback_State_Transition
 * description: call the remote procedure "XACT_Define_Callback_State_Transition" on the server
 */

RS_XACTAPI RS_XACT_Define_Callback_State_Transition( 
                    RS_XACT_HANDLE transitioning_state,
                    int callback_priority,
			        int(*fp)( RS_XACT_HANDLE transitioning_state, void *user_context ),
					void *user_context )
{
	int *result;
	int status;
    state_transition_ck arg; 

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    arg.handle = transitioning_state;
    arg.callback_priority = callback_priority;
    arg.usercontext = (unsigned int)user_context;

	result = rs_xact_define_callback_state_transition_1(&arg, pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_State_Transition = fp;

	return status;

}

/*
 * function: 	RS_XACT_Define_Cancel_Callback_State_Transition
 * description: call the remote procedure "XACT_Define_Cancel_Callback_State_Transition" 
 *              on the server
 */

RS_XACTAPI RS_XACT_Define_Cancel_Callback_State_Transition(
                                 RS_XACT_HANDLE state,
                                 int (*fp)( RS_XACT_HANDLE handle, void *user_data)
                                 )
{
	int *result;
	int status;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;



	result = rs_xact_define_cancel_callback_state_transition_1(&state, pRs_Client->cl); 
    if((status = Rs_GetStatus(result)) == RS_SUCCESS)
    	pRs_Client->pCk_functions->Xact_Callback_Cancel_State_Transition = fp;

	return status;

}
/*
 * function: 	RS_XACT_Cancel_State_Transition_Callback
 * description: call the remote procedure "XACT_Cancel_State_Transition_Callback" 
 *              on the server
 */

RS_XACTAPI RS_XACT_Cancel_State_Transition_Callback( RS_XACT_HANDLE state )

{
    int *result;    
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


    result = rs_xact_cancel_state_transition_callback_1(&state, pRs_Client->cl);
    return Rs_GetStatus(result);
}

/*
 * function: 	RS_XACT_get_array_state_value
 * description: call the remote procedure "XACT_get_array_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_get_array_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    varintarr *result;
    state_handle handle_info;
    handle_info.array_handle=s_handle;
    handle_info.array_index=array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_get_array_state_value_1(&handle_info, pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;
    *value = *result->pdata.pdata_val;
    return RS_SUCCESS; 

}


/*
 * function: 	RS_XACT_get_fifo_state_value
 * description: call the remote procedure "XACT_get_fifo_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_get_fifo_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    varintarr *result;
    state_handle handle_info;
    handle_info.array_handle=s_handle;
    handle_info.array_index=array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_get_fifo_state_value_1(&handle_info, pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;
    *value = *result->pdata.pdata_val;
    return RS_SUCCESS; 

}

/*
 * function: 	RS_XACT_find_wildcard_state_name
 * description: call the remote procedure "XRS_XACT_find_wildcard_state_name" 
 *              on the server
 */
RS_XACTAPI RS_XACT_find_wildcard_state_name(
        char *wildcard_name_spec, 
        char *rtn_name_buf,
        unsigned int rtn_name_buf_length )
{
 
    
	wildcard_state *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_find_wildcard_state_name_1(&wildcard_name_spec,  pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;

    strcpy(rtn_name_buf, result->rtn_name_buf);

    return result->status; 

}


/*
 * function: 	RS_XACT_get_state_info
 * description: call the remote procedure "XACT_get_state_info" on the server
 */

RS_XACTAPI RS_XACT_get_state_info( RS_XACT_HANDLE s_handle, 
                                  char *state_name, int *width, 
                                  int *array_length )
{
    state_info *result;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_get_state_info_1(&s_handle,pRs_Client->cl);
    if(result == NULL)
        return RS_SYSTEMERROR;
    // Caller can pass null pointer to state name buffer is he/she doesn't want
    // the name.
    if (state_name != NULL)
        strcpy(state_name, result->state_name);
    *width = result->width;
    *array_length = result->array_length;

    return RS_SUCCESS;
}

/*
 * function: 	RS_XACT_set_array_state_value
 * description: call the remote procedure "XACT_set_array_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_set_array_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    int *result;
    set_array_state_value argp;
    argp.array_handle.array_handle=s_handle;
    argp.array_handle.array_index=array_index;
    argp.value.value_val=value;
    argp.value.value_len=strlen((char *)value);
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


    result = rs_xact_set_array_state_value_1(&argp, pRs_Client->cl);
    return Rs_GetStatus(result);

}


/*
 * function: 	RS_XACT_set_fifo_state_value
 * description: call the remote procedure "XACT_set_fifo_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_set_fifo_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    int *result;
    set_array_state_value argp;
    argp.array_handle.array_handle=s_handle;
    argp.array_handle.array_index=array_index;
    argp.value.value_val=value;
    argp.value.value_len=strlen((char *)value);
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


    result = rs_xact_set_fifo_state_value_1(&argp, pRs_Client->cl);
    return Rs_GetStatus(result);

}

/*
 * function: 	RS_XACT_set_state_value
 * description: call the remote procedure "XACT_set_state_value" on the server
 */

RS_XACTAPI RS_XACT_set_state_value( RS_XACT_HANDLE s_handle, 
                                    unsigned int *value )
{

    int *result;
    set_state_value argp;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    argp.state_handle=s_handle;
    argp.value.value_val=value;
    argp.value.value_len=strlen((char *)value);
    result = rs_xact_set_state_value_1(&argp, pRs_Client->cl);
    return Rs_GetStatus(result);

}

/*
 * function: 	RS_XACT_ExecuteCommandStr
 * description: call the remote procedure "XACT_ExecuteCommandStr" 
 *              on the server
 */

RS_XACTAPI RS_XACT_ExecuteCommandStr(char *cmd_str)
{


    int *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_executecommandstr_1(&cmd_str, pRs_Client->cl);
    return Rs_GetStatus(result);

}

/*
 * function: 	RS_XACT_ExecuteCommandStr
 * description: call the remote procedure "XACT_gui_execute_command" 
 *              on the server
 */

RS_XACTAPI RS_XACT_gui_execute_command(char *command_line)
{


    int *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_gui_execute_command_1(&command_line, pRs_Client->cl);
    return Rs_GetStatus(result);

}

/*
 * function: 	RS_XACT_init_gui_console
 * description: call the remote procedure "XACT_init_gui_console" 
 *              on the server
 */

RS_XACTAPI RS_XACT_init_gui_console()
{
 int *result;
 if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

 result = rs_xact_init_gui_console_1(NULL, pRs_Client->cl);

 if(result == NULL)
	 return RS_SYSTEMERROR;

 return *result;


}

RS_XACTAPI RS_XACT_get_stat_characteristic( RS_XACT_HANDLE stat_handle, Stat_characteristic *stat_char_buf )
{

    stat_char *result;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0) || (stat_char_buf == NULL))
		return FALSE;

    result = rs_xact_get_stat_characteristic_1(&stat_handle,pRs_Client->cl);
    if(result == NULL)
        return RS_SYSTEMERROR;
	if(result->res == FALSE)
		return RS_ERROR;
	

	stat_char_buf->min_sample = result->min_sample;
	stat_char_buf->max_sample = result->max_sample;
	stat_char_buf->num_samples = result->num_samples;
	stat_char_buf->sampled_sum = result->sampled_sum;
	stat_char_buf->min_bucket_value = result->min_bucket_value;
	stat_char_buf->max_bucket_value = result->max_bucket_value;
	stat_char_buf->bucket_size = result->bucket_size;
	stat_char_buf->num_buckets = result->num_buckets;
	stat_char_buf->titled_buckets = result->titled_buckets;
    strcpy(&stat_char_buf->stat_title.name[0], result->stat_title.name);
    strcpy(&stat_char_buf->unit_specifier.name[0], result->unit_specifier.name);

    return RS_SUCCESS;

}
RS_XACTAPI RS_XACT_get_stat_bucket_data( RS_XACT_HANDLE stat_handle, int bucket_index, int *num_bucket_samples, Stat_str *bucket_title )
{
	get_bucket_data_IArg  argp;
	get_bucket_data_OArg  *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	argp.stat_handle = stat_handle;
	argp.bucket_index = bucket_index;

    result = rs_xact_get_stat_bucket_data_1(&argp, pRs_Client->cl);
    if(result == NULL)
        return RS_SYSTEMERROR;

   *num_bucket_samples = result->num_bucket_samples;
   strcpy(&bucket_title->name[0], result->bucket_title.name);
   return RS_SUCCESS;


}

RS_XACTAPI RS_XACT_stop_execution()
{
 int *result;
 if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

 result = rs_xact_stop_execution_1(NULL, pRs_Client->cl);

 if(result == NULL)
	 return RS_SYSTEMERROR;

 return *result;


}



RS_XACTAPI RS_XACT_stop_execution_at_clk(enum CLK_TYPE clk_type)
{
 int *result;
 if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

 result = rs_xact_stop_exe_at_clk_1(&clk_type, pRs_Client->cl);

 if(result == NULL)
	 return RS_SYSTEMERROR;

 return *result;


}


