/* rs_xact_svc.c: implementation of dispatching client transactor commands
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 7/8/98
 * revisions:
 * $History: rs_xact_svc.c $
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 7/22/99    Time: 7:45p
 * Updated in $/dev_1200/share/RemoteServices
 * Fix bug #669 -  Remote Simulation doe not make connection
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 5/14/99    Time: 10:14a
 * Updated in $/dev_1200/Share/RemoteServices
 * Add support for xact_clock_with_remainder(),
 * xact_define_callback_default_go_clock_domain and
 * xact_stop_execution_at_clk 
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 4/13/99    Time: 8:15p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_Define_Automatic_Sim_Halt
 * 
 * *****************  Version 8  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:47p
 * Updated in $/development/RemoteServices
 * update COPYRIGHT
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:23p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_get_fifo_handle & XACT_Define_Callback_Ref_Status
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:03p
 * Updated in $/development/RemoteServices
 * Add support for xact_get|set_fifo_state_value
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:20p
 * Updated in $/development/RemoteServices
 * Add support for new transactor functions
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 9/14/98    Time: 6:01p
 * Updated in $/base_level3/RemoteServices
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 8/25/98    Time: 11:56a
 * Updated in $/base_level3/RemoteServices
 * fix a problem with re-init
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:11a
 * Updated in $/base_level3/RemoteServices
 * Add comments
 */ 

#include <stdio.h>
#ifdef WIN32
#include <xact_dll.h>
#include <Oncrpc32.h>
#include "pmap_clnt.h"

#else
#include <rpc/rpc.h>
#include <xact.h>
#endif
#include "rs_xact.h"
#include <thread.h>

int exit_cmd = FALSE;

static void sa1200prog_1();

#ifdef WIN32
main()
#else
void xact_server()
#endif
{
	SVCXPRT *transp;
#ifdef WIN32
  	OncRpc32Init();
#endif
#ifdef VX_WORKS
	rpcTaskInit();
#endif

    (void)pmap_unset(SA1200PROG, SA1200VERSION);
#ifndef WIN32
    /* reset transactor */
    XACT_ExecuteCommandStr("sim_reset");
#endif
    exit_cmd = FALSE;
	transp = svcudp_create(RPC_ANYSOCK);
	if (transp == NULL) {
		(void)fprintf(stderr, "cannot create udp service.\n");
		exit(1);
	}
	if (!svc_register(transp, SA1200PROG, SA1200VERSION, sa1200prog_1, IPPROTO_UDP)) {
		(void)fprintf(stderr, "unable to register (SA1200PROG, SA1200VERSION, udp).\n");
		exit(1);
	}

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		(void)fprintf(stderr, "cannot create tcp service.\n");
		exit(1);
	}
	if (!svc_register(transp, SA1200PROG, SA1200VERSION, sa1200prog_1, IPPROTO_TCP)) {
		(void)fprintf(stderr, "unable to register (SA1200PROG, SA1200VERSION, tcp).\n");
		exit(1);
	}

	svc_run();
       
	(void)fprintf(stderr, "svc_run returned\n");
     exit_cmd = TRUE;
#ifndef WIN32
	exit(1);
#else
    return (1);
#endif
}

/*
 * function:    sa1200prog_1
 * description: this function is invoked by the RPC system whenever a 
 *              transactor client calls a transactor remote procedure
 *		
 */

static void
sa1200prog_1(rqstp, transp)
	struct svc_req *rqstp;
	SVCXPRT *transp;
{
	union {
		handle rs_xact_get_handle_1_arg;
		int rs_xact_delete_handle_1_arg;
		int rs_xact_get_state_value_1_arg;
		RS_CLK_TYPE rs_xact_start_of_cycle_1_arg;
		RS_CLK_TYPE rs_xact_clock_cycle_1_arg;
		state_transition_ck rs_xact_define_callback_state_transition_1_arg;
		int rs_xact_define_cancel_callback_state_transition_1_arg;
		int rs_xact_cancel_state_transition_callback_1_arg;
		int rs_xact_get_state_info_1_arg;
		state_handle rs_xact_get_array_state_value_1_arg;
		set_state_value rs_xact_set_state_value_1_arg;
		set_array_state_value rs_xact_set_array_state_value_1_arg;
		char *rs_executecommandstr_1_arg;
		char *rs_gethostname_1_arg;
		char *rs_xact_find_wildcard_state_name_1_arg;
		int rs_xact_get_stat_characteristic_1_arg;
		get_bucket_data_IArg rs_xact_get_stat_bucket_data_1_arg;
		state_handle rs_xact_get_fifo_state_value_1_arg;
		set_array_state_value rs_xact_set_fifo_state_value_1_arg;
		handle rs_xact_get_fifo_handle_1_arg;
		RS_CLK_TYPE rs_xact_clk_cycle_with_remainder_1_arg;
		RS_CLK_TYPE rs_xact_stop_exe_at_clk_1_arg;
		char *rs_xact_gui_execute_command_1_arg;

	} argument;

	char *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	char *(*local)();

	switch (rqstp->rq_proc) {
	case NULLPROC:
		(void)svc_sendreply(transp, xdr_void, (char *)NULL);
		return;

	case RS_XACT_INITIALIZE:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_initialize_1;
		break;

	case RS_XACT_START_CONSOLE:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_start_console_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_SET_PROMPT:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_set_prompt_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_SET_OUTPUT_MESSAGE:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_set_output_message_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_GET_CONSOLE_INPUT:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_get_console_input_1;
		break;

	case RS_XACT_GET_HANDLE:
		xdr_argument = xdr_handle;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_get_handle_1;
		break;

	case RS_XACT_DELETE_HANDLE:
		xdr_argument = xdr_int;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_delete_handle_1;
		break;

	case RS_XACT_GET_STATE_VALUE:
		xdr_argument = xdr_int;
		xdr_result = xdr_varintarr;
		local = (char *(*)()) rs_xact_get_state_value_1;
		break;

	case RS_XACT_START_OF_CYCLE:
		xdr_argument = xdr_RS_CLK_TYPE;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_start_of_cycle_1;
		break;

	case RS_XACT_CLOCK_CYCLE:
		xdr_argument = xdr_RS_CLK_TYPE;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_clock_cycle_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_CREATE_CHIP:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_create_chip_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_LOAD_UCODE:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_load_ucode_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_BREAKPOINT:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_breakpoint_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_BREAKPOINT_EVENT:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_breakpoint_event_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_STATE_TRANSITION:
		xdr_argument = xdr_state_transition_ck;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_state_transition_1;
		break;

	case RS_XACT_DEFINE_CANCEL_CALLBACK_STATE_TRANSITION:
		xdr_argument = xdr_int;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_cancel_callback_state_transition_1;
		break;

	case RS_XACT_CANCEL_STATE_TRANSITION_CALLBACK:
		xdr_argument = xdr_int;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_cancel_state_transition_callback_1;
		break;

	case RS_XACT_GET_STATE_INFO:
		xdr_argument = xdr_int;
		xdr_result = xdr_state_info;
		local = (char *(*)()) rs_xact_get_state_info_1;
		break;

	case RS_XACT_GET_ARRAY_STATE_VALUE:
		xdr_argument = xdr_state_handle;
		xdr_result = xdr_varintarr;
		local = (char *(*)()) rs_xact_get_array_state_value_1;
		break;

	case RS_XACT_SET_STATE_VALUE:
		xdr_argument = xdr_set_state_value;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_set_state_value_1;
		break;

	case RS_XACT_SET_ARRAY_STATE_VALUE:
		xdr_argument = xdr_set_array_state_value;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_set_array_state_value_1;
		break;

	case RS_EXECUTECOMMANDSTR:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_executecommandstr_1;
		break;

	case RS_GETHOSTNAME:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_gethostname_1;
		break;

	case RS_XACT_FIND_WILDCARD_STATE_NAME:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_wildcard_state;
		local = (char *(*)()) rs_xact_find_wildcard_state_name_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_INIT_SIM:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_init_sim_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_SIM_RESET:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_sim_reset_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_RESTORE:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_restore_1;
		break;

	case RS_XACT_GET_STAT_CHARACTERISTIC:
		xdr_argument = xdr_int;
		xdr_result = xdr_stat_char;
		local = (char *(*)()) rs_xact_get_stat_characteristic_1;
		break;

	case RS_XACT_GET_STAT_BUCKET_DATA:
		xdr_argument = xdr_get_bucket_data_IArg;
		xdr_result = xdr_get_bucket_data_OArg;
		local = (char *(*)()) rs_xact_get_stat_bucket_data_1;
		break;

	case RS_XACT_STOP_EXECUTION:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_stop_execution_1;
		break;

	case RS_XACT_GET_FIFO_STATE_VALUE:
		xdr_argument = xdr_state_handle;
		xdr_result = xdr_varintarr;
		local = (char *(*)()) rs_xact_get_fifo_state_value_1;
		break;

	case RS_XACT_SET_FIFO_STATE_VALUE:
		xdr_argument = xdr_set_array_state_value;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_set_fifo_state_value_1;
		break;


	case RS_XACT_GET_FIFO_HANDLE:
		xdr_argument = xdr_handle;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_get_fifo_handle_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_REF_STATUS:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_ref_status_1;
		break;

    case RS_XACT_DEFINE_AUTO_SIM_HALT:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)())rs_xact_define_auto_sim_halt_1;
		break;

		case RS_XACT_CLK_CYCLE_WITH_REMAINDER:
		xdr_argument = xdr_RS_CLK_TYPE;
		xdr_result = xdr_varintarr;
		local = (char *(*)()) rs_xact_clk_cycle_with_remainder_1;
		break;

	case RS_XACT_DEFINE_CALLBACK_DEF_GO_CLOCK_DOMAIN:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_define_callback_def_go_clock_domain_1;
		break;

	case RS_XACT_STOP_EXE_AT_CLK:
		xdr_argument = xdr_RS_CLK_TYPE;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_stop_exe_at_clk_1;
		break;

	case RS_XACT_INIT_GUI_CONSOLE:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_init_gui_console_1;
		break;

	case RS_XACT_GUI_EXECUTE_COMMAND:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_gui_execute_command_1;
		break;

	
	default:
		svcerr_noproc(transp);
		return;
	}
	bzero((char *)&argument, sizeof(argument));
	if (!svc_getargs(transp, xdr_argument, &argument)) {
		svcerr_decode(transp);
		return;
	}
	result = (*local)(&argument, rqstp);
	if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
		svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, xdr_argument, &argument)) {
		(void)fprintf(stderr, "unable to free arguments\n");
		exit(1);
	}
}
