/* rs_xacttest.c: implementation of remote transactor unit test program
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 6/12/98
 * revisions:
 * $History: rs_xacttest.c $
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 5/14/99    Time: 10:14a
 * Updated in $/dev_1200/Share/RemoteServices
 * Add support for xact_clock_with_remainder(),
 * xact_define_callback_default_go_clock_domain and
 * xact_stop_execution_at_clk 
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:19p
 * Updated in $/development/qa-test/rsxact_tests
 * Add test cases for rs_xact_get_fifo handle 
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:03p
 * Updated in $/development/qa-test/rsxact_tests
 * Add test for xact_get|set_fifo_state value
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:18p
 * Updated in $/development/qa-test/rsxact_tests
 * Add test cases for new transactor functions
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 9/15/98    Time: 12:04p
 * Updated in $/base_level3/qa-test/rsxact_tests
 * Add test cases for rs_xact_find_wildcard_state_name function
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 9/14/98    Time: 5:56p
 * Updated in $/base_level3/qa-test/rsxact_tests
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 8/25/98    Time: 5:59p
 * Updated in $/base_level3/qa-test/rsxact_tests
 * Allow command line input 
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 7/27/98    Time: 3:34p
 * Updated in $/base_level3/qa-test/rsxact_tests
 * Add more test cases
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:10a
 * Updated in $/base_level3/qa-test/rsxact_tests
 * Add comments
 * 
 */ 

#include <stdio.h>
#include <thread.h>
#include "rsxact_dll.h"
#include "remote_services.h"

int state_context;
int array_state_context;


char buf[200];
int ind_file = FALSE;

void XACT_TestThread();

void XactOutputCallback(const char *outstr)
{
	printf("%s",outstr);
}


void XactRefStatusCallback(const Meta_tag_ref *argp)
{
	printf("enter RefStatus Callback \n");

}

int XactPromptCallback(char *outstr)
{
	printf("%s", outstr);

    return RS_SUCCESS;
}

char *XactCmdCallback()
{
    static char *s;
   

    s = &buf[0];

    /* check for input ind file */

    if(ind_file == TRUE)
        ind_file = FALSE;
    else
        fgets( s, 200, stdin );


	return s;

}


void
XactCreateChipCallback(
    char *pChipName)
{
    printf("enter CreateChipCallback\n");
	printf("chip name = %s\n", pChipName);

}


void
XactInitSimCallback()
{
    int status;
    char buf[64];

    printf("enter XactSimInitCallback\n");

    printf("test find_wildcard_state_name \n");
    status = RS_XACT_find_wildcard_state_name("*", buf, 64);
    if(status == 1){ 
            printf("return buf=%s, status = %d\n", buf, status);
            for(;;){
               status = RS_XACT_find_wildcard_state_name(NULL, buf, 64);
               if(status == 1){ 
                    printf("return buf=%s, status = %d\n", buf, status);
                    continue;
               }
               else{
                   printf("return status = %d\n", status);
                   break;
               }
            }

    }
    else
        printf("status = %d\n", status);
return;
}
void
XactSimResetCallback()
{
    printf("enter XactResetSimCallback\n");

}


void
XactSimHaltCallback(HALT_STATUS halt_status)
{
    printf("enter XactSimHaltCallback\n");
    printf("halt status = %x\n", halt_status);

}

void
XactGoClockCallback(enum CLK_TYPE clk_type)
{
    printf("enter XactGoClockCallback\n");
    printf("clk type = %x\n", clk_type);

}


void
XactRestoreCallback()
{
    printf("enter XactRestoreCallback\n");

}

/* call callback when ucode file has been loaded into specified chip/fbox */
/* If the ucode_file name is NULL, this means that the associated control */
/* store has been reset and no longer corresponds to a ucode_file */
void
XactUcodeLoadedCallback(
    char *pChipName,
    int MicroEngineNumber,
    char *pUcodeFilePath)
{
	printf("enter UcodeLoadedCallback\n");
    printf("chip name = %s F-Eng= %d,UcodeFile=%s\n", pChipName, MicroEngineNumber,pUcodeFilePath);

}

/* NOTE:  *ContextMask == 0 means that the breakpoint has been cleared */
void
XactBreakpointSetCallback(
    char *pChipName, 
    int MicroEngineNumber, 
    int PC, 
    int ContextMask)
{
	printf("XactBreakpointSetCallback\n");
    printf("chip name = %s F-Eng= %d, PC=%d, CondextMask=%d\n",
        pChipName, MicroEngineNumber,PC, ContextMask);
}

void
XactBreakpointHitCallback(
    char *pChipName, 
    int MicroEngineNumber, 
    int PC, 
    int ContextNumber)
{
	printf("XactBreakpointHitCallback\n");
    printf("chip name = %s F-Eng= %d, PC=%d, CondextMask=%d\n",
        pChipName, MicroEngineNumber,PC, ContextNumber);

    XACT_TestThread();
}

int XactCallbackState_tran(int handle, void *user_context)
{
   int *temp;
   temp = (int *)user_context;
   printf("enter XactCallbackState_tran\n");
   printf("user_context = %d\n", *temp);
   return 1;

}


int XactCancelCallbackState_tran(int handle, void *user_context)
{
    int *temp;
    temp = (int *)user_context;
    
   printf("enter XactCancelCallbackState_tran\n");
   printf("user_context = %d\n", *temp);
   return 1;

}

void XACT_TestThread()
{
    RS_XACT_HANDLE TestHandle, Fifo_TestHandle, hSimTime;
    int i, status;
    unsigned int Value = 0;
    unsigned int SetValue;
    char state_name[100];
    int width, array_length;
	Stat_characteristic stat_char_buf;	
	int num_bucket_samples;
	Stat_str bucket_title;
	double remainder;

    printf("test callback state transition\n");
    hSimTime = RS_XACT_get_handle("sim.time", -1);
    printf("hSimTime=%x\n", hSimTime);
    if(hSimTime == RS_INVALID_XACT_HANDLE)
        printf("ErrorError - RS_XACT_get_handle failed\n");

    printf("call RS_XACT_Define_Cancel_Callback_state_Transition\n");
    status = RS_XACT_Define_Cancel_Callback_State_Transition(
        hSimTime, XactCancelCallbackState_tran);

    if(status != RS_SUCCESS)
        printf("RS_XACT_Define_Cancel_Callback_state error status=%d\n", status);

 
    status = RS_XACT_Define_Callback_State_Transition(hSimTime,0,
        XactCallbackState_tran, &state_context);
  	

    
	state_context = 0xAA;
    if(status != RS_SUCCESS)
        printf("RS_XACT_Define_Callback_state error status=%d\n", status);

    status = RS_XACT_Define_CallbackRef_Status(XactRefStatusCallback);

    if(status != RS_SUCCESS)
        printf("XACT_Define_Callback_Ref_Status=%d\n", status);


    state_context = 0xAA;


    for(i=0; i <3; i++){
        printf("*** get sram[0] handle ***\n");
        TestHandle = RS_XACT_get_handle("sram", 0);
        printf("TestHandle=%x\n", TestHandle);
        if(TestHandle == RS_INVALID_XACT_HANDLE)
            continue;

        SetValue = i+50;
        printf("RS_XACT_set_state_value - value=%x\n", SetValue);
        status = RS_XACT_set_state_value(TestHandle, &SetValue);
        if(status != RS_SUCCESS)
            printf("RS_XACT_set_state_value status=%d\n", status);
   
        status = RS_XACT_get_state_value(TestHandle, &Value);
        printf("RS_XACT_get_state_value: status = %x sram[0]=%x\n",status,Value);
        if(Value != SetValue)
            printf("ERRORERROR: RS_XACT_get_state_vaule failure: expected value = %d, actual = %d\n", SetValue, SetValue);
        printf("RS_XACT_state_info\n");
        status=RS_XACT_get_state_info(TestHandle, state_name, &width, &array_length);
        printf("state info: state_name=%s, width=%d, array_length=%d\n",
                state_name, width, array_length);

        RS_XACT_delete_handle(TestHandle);


        printf("***** array_state testing *****\n");
        TestHandle = RS_XACT_get_handle("sram", -1);
        printf("TestHandle=%x\n", TestHandle);
        if(TestHandle == RS_INVALID_XACT_HANDLE)
            continue;

        SetValue=i+100;
        status = RS_XACT_set_array_state_value(TestHandle, i+100, &SetValue);
        printf("RS_XACT_set_array_state_value: status = %d, value=%x\n", status, SetValue);
        status = RS_XACT_get_array_state_value(TestHandle, i+100, &Value);
        printf("RS_XACT_get_array_state_value: status = %d sram[%d]=%x\n",status,i+100,Value);
     
   
        RS_XACT_delete_handle(TestHandle);
        printf("***** fifo_state testing *****\n");
        TestHandle = RS_XACT_get_handle("fbi.getq", -1);
        printf("TestHandle=%x\n", TestHandle);
        if(TestHandle == RS_INVALID_XACT_HANDLE)
            continue;

        SetValue=1;
        status = RS_XACT_set_fifo_state_value(TestHandle, 0, &SetValue);
        printf("RS_XACT_set_fifo_state_value: status = %d, value=%x\n",status, SetValue);
        status = RS_XACT_get_fifo_state_value(TestHandle, 0, &Value);
        printf("RS_XACT_get_fifo_state_value: status = %d fifo[0]=%x\n",status,Value);
     
   
        RS_XACT_delete_handle(TestHandle);
 

        printf("test clock cycle\n");
        printf("FBOX cycle = %d\n",RS_XACT_clock_cycle(CLK_FBOX));
        printf("FBUS cycle = %d\n",RS_XACT_start_of_cycle(CLK_FBUS));
        printf("PCI cycle = %d\n",RS_XACT_start_of_cycle(CLK_PCI));

    }
	printf("call RS_XACT_clock_cycle_with_remainder\n");
	RS_XACT_clock_cycle_with_remainder(CLK_FBOX, &remainder);
    printf("FBOX: clock remainder = %f\n", remainder);

	printf("call RS_XACT_get_stat_characteristic\n");
	Fifo_TestHandle = RS_XACT_get_fifo_handle("stat.sbus_push_bandwidth", 1);
	printf("handle returned from get_fifo_handle=%d\n", TestHandle);


	TestHandle = RS_XACT_get_handle("dc.stat.cmd_cycles", -1);

	printf("handle returned from get_handle=%d\n", TestHandle);
	status = RS_XACT_get_stat_characteristic( TestHandle, &stat_char_buf );
	if(status == RS_SUCCESS){
	  printf("minimum value sampled = %d\n", stat_char_buf.min_sample );
	  printf("maximum value sampled  = %d\n", stat_char_buf.max_sample );
	  printf("number of samples recorded = %d\n",stat_char_buf.num_samples );

	  printf("sum of sample values  = %d\n", stat_char_buf.sampled_sum );
	  printf("value of smallest allocated bucket = %d\n", stat_char_buf.min_bucket_value );
	  printf("value of largest allocated bucket = %d\n", stat_char_buf.max_bucket_value);

	  printf("range of contiguous int values per bucket = %d\n", stat_char_buf.bucket_size);
	  printf("number of buckets to capture samples = %d\n", stat_char_buf.max_bucket_value);
	  printf("number of buckets to capture samples = %d\n", stat_char_buf.num_buckets);
	  printf("TRUE if each bucket is given a name = %d\n", stat_char_buf.titled_buckets);
	  printf("stat_title = %s\n", stat_char_buf.stat_title.name);
	  printf("unit_specifier = %s\n", stat_char_buf.unit_specifier.name);
}
	printf("call RS_XACT_get_stat_bucket_data\n");
	for(i=0; i < stat_char_buf.num_buckets; i++){
		status = RS_XACT_get_stat_bucket_data( TestHandle, i, &num_bucket_samples, &bucket_title )	;
		printf("i=%d, number_bucket_samples=%d, ucket_title=%s\n", i, num_bucket_samples, bucket_title.name);
	}

	RS_XACT_delete_handle(TestHandle);

    RS_XACT_delete_handle(hSimTime);

	RS_XACT_stop_execution_at_clk(CLK_FBOX);


	RS_XACT_stop_execution();

	 
    return;

}


main(argc, argv)
	int argc;
	char *argv[];
{
	int status;
	int i;

    if (argc < 2) {
		fprintf(stderr, "usage: %s host\n", argv[0]);
		exit(1);
	}
    /* check for any ind files specified on the command line */
    for( i = 1; i < argc; i++ )
	{
		if ( strcmp( argv[i], "-i") == 0 )
		{
			i++;
		    sprintf( buf, "@%s", argv[i] );
            ind_file = TRUE;
            break;
        }
    }

    if(Open_Connection(argv[1], RS_XACT) != RS_SUCCESS){
        printf("open_connection failed\n");
        exit(1);
    }

	/*
	 * Call the remote procedure "xact_start_init" on the server
	 */
    if((status = RS_XACT_initialize()) != RS_SUCCESS){
        printf("xact init. error %d\n", status);
        exit(1);
    }
    status = RS_XACT_Define_Callback_Output_Message(XactOutputCallback);
    status = RS_XACT_Define_Callback_Set_Prompt(XactPromptCallback);
	status = RS_XACT_Define_Callback_Get_Console_Input(XactCmdCallback);

    status = RS_XACT_Define_Callback_Create_Chip(XactCreateChipCallback);
    status = RS_XACT_Define_Callback_Load_Ucode(XactUcodeLoadedCallback);
    status = RS_XACT_Define_Callback_Breakpoint(XactBreakpointSetCallback);
    status = RS_XACT_Define_Callback_Breakpoint_Event(XactBreakpointHitCallback);


   status = RS_XACT_Define_Callback_Init_Sim(XactInitSimCallback);
   status = RS_XACT_Define_Callback_Sim_Reset(XactSimResetCallback);
   status = RS_XACT_Define_Callback_Restore(XactRestoreCallback);

   status = RS_XACT_Define_Callback_Default_Go_Clock_Domain(XactGoClockCallback);
   status = RS_XACT_Define_Automatic_Sim_Halt(XactSimHaltCallback);

	/*
	 * Call the remote procedure "xact_start_console" on the server
	 */
   RS_XACT_start_console();

   Close_Connection(argv[1], RS_XACT);
 return 1;
}


