/* sample_clnt.c: implementation of client sample functions
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Simulation Extension DLL example
 * author: Lai-Wah Hui 8/4/98
 * revisions:
 * $History: client_foreign_model.c $
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 4/09/99    Time: 5:38p
 * Created in $/development/Samples/RemoteSimExt
 * 
 * *****************  Version 6  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 4:16p
 * Updated in $/development/Samples/SimExtensionDLL
 * update COPYRIGHT
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 10/12/98   Time: 4:16p
 * Updated in $/base_level3/Samples/SimExtensionDLL
 * Add comments
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 10/12/98   Time: 9:51a
 * Updated in $/base_level3/Samples/SimExtensionDLL
 * Add ..\common\sim.c into the project
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 10/05/98   Time: 3:34p
 * Updated in $/base_level3/Samples/SimExtensionDLL
 * Code cleanup
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 9/18/98    Time: 9:54p
 * Updated in $/base_level3/Samples/SimExtensionDLL
 * Add support for remote transactor calls
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 8/07/98    Time: 8:01p
 * Created in $/base_level3/Samples/SimExtensionDLL
 * Add remote supports
 */ 

#include <stdio.h>
#include <OncRpc32.h>
#include <pmap_clnt.h>
#include "remote_services.h"
#include "RemoteSample.h"
#include "rs_xact.h"
#include "thread.h"

static CLIENT *client=NULL;		
thread_T XactTh;

void *XACT_Server();

/*
 * RemoteInit
 *
 * Creates a client "handle" used for calling remote program on the
 * server.  This function must be called prior to invoking any 
 * remote functions.
 * return:      		
 */

RemoteInit(char *server,                     /* IN: remote server */
           unsigned int RemoteProgName)      /* IN: remote program */    

{
    char hostname[100];
	int *result, status;
    char *pHname = &hostname[0];
    
    if (server == NULL) 
		return 1; 


    /*
     *  init. Win32 socket library
     */
	OncRpc32Init();

    /*
     * Create a client "handle" used for calling remote program on the
     * server designated on the command line. We tell the rpc package
     * to use the "tcp" protocol when contacting the server.
     */

    client = clnt_create(server, RemoteProgName, SA1200VERSION, "tcp");
    if (client == NULL) {
		/*
		 * Couldn't establish connection with server.
		 * Print error message and return.
		 */
		clnt_pcreateerror(server);

		return 1;
	}

    /*
     * Create a xact server 
     */
    XactTh = Thread_Create(MID_PRIORITY,(void *)XACT_Server,(void*)0); 
    /*
     * Wait until the XACT_Server is in a running mode
     */
    Sleep(1000);  
    /*
     * send host name to target system
     */
    if(gethostname(hostname, sizeof(hostname)) != 0){
        OncRpc32Cleanup();
        return 1;
    }

    result = rs_foreign_model_initialize_1(&pHname,client);
    status = Rs_GetStatus(result);
    if(status != RS_SUCCESS){
        OncRpc32Cleanup();
        return 1;
    }

    return 1;
}


/*
 * function: 	Rs_GetStatus
 * description: return status from remote procedur call
 */

int Rs_GetStatus(int *result)
{

    if (result == NULL) {
    /*
     * An error occurred while calling the server. 
     */
	    return RS_SYSTEMERROR;
    }
    /*
     * Okay, we successfully called the remote procedure.
     */
    if (*result == 0) {
	    /*
	     * server is unable to call the remote procedure
         */
	    return RS_PROCUNAVAIL;
    } 
    return RS_SUCCESS;

}


/* rs_foreign_model_pre_sim
 *
 * This function calls a local rs_foreign_model_pre_sim_1 stub function which then calls
 * clnt_call(), a OncRpc32 library function to send a request to the remote foreign
 * model application to execute the actual foreign_model_pre_sim() function.
 */

rs_foreign_model_pre_sim(){
 
    int *result;
    char buf[100];
    char *pbuf=buf;
    if(client == NULL)
        return (1);
    result = rs_foreign_model_pre_sim_1(&pbuf, client);
    return Rs_GetStatus(result);
}


/* foreign_model_post_sim
 *
 * This function calls a local rs_foreign_model_post_sim_1 stub function which then
 * calls clnt_call(), an OncRpc32 library function to send a request to the remote
 * foreign model application to execute the actual foreign_model_post_sim() function.
 */

int rs_foreign_model_post_sim(){

    int *result;
    char buf[100];
    char *pbuf=buf;
    if(client == NULL)
        return (1);
    result = rs_foreign_model_post_sim_1(&pbuf, client);
    return Rs_GetStatus(result);

}

/* rs_foreign_model_exit
 *
 * This function calls a local rs_foreign_model_exit_1 stub function which then calls
 * clnt_call(), a OncRpc32 library function to send a request to the remote foreign
 * model application to execute the actual foreign_model_exit_sim() function.
 */

int rs_foreign_model_exit(){
    int *result;
    char buf[100];
    char *pbuf=buf;
    if(client == NULL)
        return (1);
 
    result = rs_foreign_model_exit_1(&pbuf, client);
	/*
	 * terminates the callback server 
	 */
    Thread_Cancel(XactTh);
	/*
	 * clean up sockets
	 */
    OncRpc32Cleanup();   
    return Rs_GetStatus(result);
}