/* connect.c: implementation of xact support functions
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 4/9/99
 * revisions:
 * $History: connect.c $
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 4/09/99    Time: 7:44p
 * Updated in $/development/Samples/RemoteSimExt
 * Ported to Solaris
 */
#include <stdio.h>
#ifdef WIN32
#include <OncRpc32.h>
#include <pmap_clnt.h>
#else
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#endif
#include "rs_xact.h"
#include "rs_control.h"
#include "remote_services.h"
#include "thread.h"
 		
/* this is not thread safe -- need to be fixed */
tpRs_Client pRs_Client=NULL;


int ConnectToXact(char *server)            /* remote server */
{
	int retry;
	if (server == NULL) 
		return RS_UNKNOWNSERVER; 

#ifdef WIN32
	OncRpc32Init();
#endif
    pRs_Client = malloc(sizeof(tRs_Client));
	if(pRs_Client == NULL)
		return RS_NOMEMORY;

	memset(pRs_Client, 0, sizeof(tRs_Client));
    memcpy(pRs_Client->server, server, strlen(server)); 

   /*
    * Create client "handle" used for calling SA1200PROG on the
    * server designated on the command line. We tell the rpc package
    * to use the "tcp" protocol when contacting the server.
    */
    for(retry=0; retry<5; retry++){
        printf("retry=%d,server=%s \n", retry, pRs_Client->server);
        pRs_Client->cl = clnt_create(pRs_Client->server, RS_XACT, SA1200VERSION, "tcp");
        printf("return from clnt_create\n");
        if (pRs_Client->cl != NULL) 
            break;
        else
		    clnt_pcreateerror(pRs_Client->server);
    }
    if(retry == 5){
        /*
		 * Couldn't establish connection with server.
		 * Print error message and return.
		 */
		clnt_pcreateerror(pRs_Client->server);
		/*
		 * Couldn't establish connection with server.
		 * Print error message and return.
		 */
		return RS_ERROR;
	}
return RS_SUCCESS;
}


/*
 * function: 	DisconnectFromXact
 * description: Close a connection and free up memeory
 * return:      status from the server		
 */

int DisconnecFromXact()
{
#ifdef WIN32
    OncRpc32Cleanup();
#endif
    free(pRs_Client);
    pRs_Client = NULL;

    return RS_SUCCESS;
}

/*
 * function: 	Rs_GetStatus
 * description: return status from remote procedur call
 */

int Rs_GetStatus(int *result)
{

    if (result == NULL) {
    /*
     * An error occurred while calling the server. 
     */
	    return RS_SYSTEMERROR;
    }
    /*
     * Okay, we successfully called the remote procedure.
     */
    if (*result == 0) {
	    /*
	     * server is unable to call the remote procedure
         */
	    return RS_PROCUNAVAIL;
    } 
    return RS_SUCCESS;
}
