/* xact_client.c: 
 *      remote SA1200 transactor functions 
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 4/8/99
 * revisions:
 * $History: xact_client.c $
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 4/09/99    Time: 7:44p
 * Updated in $/development/Samples/RemoteSimExt
 * Ported to Solaris
 */ 

#include <stdio.h>
#ifdef WIN32
#include <OncRpc32.h>
#include <pmap_clnt.h>
#else
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#endif
#include "thread.h"
#include "rs_control.h"
#include "rs_xact.h"
#include "rsxact_dll.h"
#include "remote_services.h"
 		
extern tpRs_Client pRs_Client;

/*
 * function: 	RS_XACT_get_handle
 * description: call the remote procedure "XACT_get_handle" on the server
 */

RS_XACTAPI_NORET RS_XACT_HANDLE RS_XACT_get_handle( 
                                  char *state_name, 
                                  int array_index )
{
    int *result_handle;
    handle  handle_info;
    handle_info.name = state_name;
    handle_info.array_index = array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result_handle = rs_xact_get_handle_1(&handle_info, pRs_Client->cl);

    if (result_handle == NULL){ 
		clnt_perror(pRs_Client->cl, "testtarget");
        return RS_INVALID_XACT_HANDLE;
    }
    
    return *result_handle;
}


/*
 * function: 	RS_XACT_get_fifo_handle
 * description: call the remote procedure "RS_XACT_get_fifo_handle" on the server
 */

RS_XACTAPI_NORET RS_XACT_HANDLE RS_XACT_get_fifo_handle( 
                                  char *state_name, 
                                  int array_index )
{
    int *result_handle;
    handle  handle_info;
    handle_info.name = state_name;
    handle_info.array_index = array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result_handle = rs_xact_get_fifo_handle_1(&handle_info, pRs_Client->cl);

    if (result_handle == NULL){ 
		clnt_perror(pRs_Client->cl, "testtarget");
        return RS_INVALID_XACT_HANDLE;
    }
    
    return *result_handle;
}



/*
 * function: 	RS_XACT_delete_handle
 * description: call the remote procedure "XACT_delete_handle" on the server
 */

RS_XACTAPI RS_XACT_delete_handle( RS_XACT_HANDLE this_handle )
{
    int *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_delete_handle_1(&this_handle, pRs_Client->cl);
	return Rs_GetStatus(result);

}

/*
 * function: 	RS_XACT_get_state_value
 * description: call the remote procedure "XACT_get_state_value" on the server
 */

RS_XACTAPI RS_XACT_get_state_value( 
                RS_XACT_HANDLE state_handle, 
                unsigned int *value ) 
{
    varintarr *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


    result = rs_xact_get_state_value_1(&state_handle, pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;
    *value = *result->pdata.pdata_val;
    return RS_SUCCESS; 

}

/*
 * function: 	RS_XACT_start_of_cycle
 * description: call the remote procedure "RS_XACT_start_of_cycle" on the server
 */

RS_XACTAPI RS_XACT_start_of_cycle( enum CLK_TYPE clk_type )
{

    int *result;    

   if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_start_of_cycle_1(&clk_type, pRs_Client->cl);
    return Rs_GetStatus(result);

  
}

/*
 * function: 	RS_XACT_clock_cycle
 * description: call the remote procedure "RS_XACT_clock_cycle" on the server
 */

RS_XACTAPI RS_XACT_clock_cycle( enum CLK_TYPE clk_type)
{
    int *result;    

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;
    

    result = rs_xact_clock_cycle_1(&clk_type, pRs_Client->cl);
    if (result == NULL) 
	    return -1;
    return *result;

}


/*
 * function: 	RS_XACT_get_array_state_value
 * description: call the remote procedure "XACT_get_array_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_get_array_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    varintarr *result;
    state_handle handle_info;
    handle_info.array_handle=s_handle;
    handle_info.array_index=array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_get_array_state_value_1(&handle_info, pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;
    *value = *result->pdata.pdata_val;
    return RS_SUCCESS; 

}


/*
 * function: 	RS_XACT_get_fifo_state_value
 * description: call the remote procedure "XACT_get_fifo_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_get_fifo_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    varintarr *result;
    state_handle handle_info;
    handle_info.array_handle=s_handle;
    handle_info.array_index=array_index;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_get_fifo_state_value_1(&handle_info, pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;
    *value = *result->pdata.pdata_val;
    return RS_SUCCESS; 

}

/*
 * function: 	RS_XACT_find_wildcard_state_name
 * description: call the remote procedure "XRS_XACT_find_wildcard_state_name" 
 *              on the server
 */
RS_XACTAPI RS_XACT_find_wildcard_state_name(
        char *wildcard_name_spec, 
        char *rtn_name_buf,
        unsigned int rtn_name_buf_length )
{
 
    
	wildcard_state *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_find_wildcard_state_name_1(&wildcard_name_spec,  pRs_Client->cl);
    if (result == NULL) 
	    return RS_SYSTEMERROR;

    strcpy(rtn_name_buf, result->rtn_name_buf);

    return result->status; 

}


/*
 * function: 	RS_XACT_get_state_info
 * description: call the remote procedure "XACT_get_state_info" on the server
 */

RS_XACTAPI RS_XACT_get_state_info( RS_XACT_HANDLE s_handle, 
                                  char *state_name, int *width, 
                                  int *array_length )
{
    state_info *result;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    result = rs_xact_get_state_info_1(&s_handle,pRs_Client->cl);
    if(result == NULL)
        return RS_SYSTEMERROR;
    // Caller can pass null pointer to state name buffer is he/she doesn't want
    // the name.
    if (state_name != NULL)
        strcpy(state_name, result->state_name);
    *width = result->width;
    *array_length = result->array_length;

    return RS_SUCCESS;
}

/*
 * function: 	RS_XACT_set_array_state_value
 * description: call the remote procedure "XACT_set_array_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_set_array_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    int *result;
    set_array_state_value argp;
    argp.array_handle.array_handle=s_handle;
    argp.array_handle.array_index=array_index;
    argp.value.value_val=value;
    argp.value.value_len=strlen((char *)value);
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


    result = rs_xact_set_array_state_value_1(&argp, pRs_Client->cl);
    return Rs_GetStatus(result);

}


/*
 * function: 	RS_XACT_set_fifo_state_value
 * description: call the remote procedure "XACT_set_fifo_state_value" 
 *              on the server
 */

RS_XACTAPI RS_XACT_set_fifo_state_value( RS_XACT_HANDLE s_handle, 
                                         int array_index, unsigned int *value )
{

    int *result;
    set_array_state_value argp;
    argp.array_handle.array_handle=s_handle;
    argp.array_handle.array_index=array_index;
    argp.value.value_val=value;
    argp.value.value_len=strlen((char *)value);
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;


    result = rs_xact_set_fifo_state_value_1(&argp, pRs_Client->cl);
    return Rs_GetStatus(result);

}

/*
 * function: 	RS_XACT_set_state_value
 * description: call the remote procedure "XACT_set_state_value" on the server
 */

RS_XACTAPI RS_XACT_set_state_value( RS_XACT_HANDLE s_handle, 
                                    unsigned int *value )
{

    int *result;
    set_state_value argp;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

    argp.state_handle=s_handle;
    argp.value.value_val=value;
    argp.value.value_len=strlen((char *)value);
    result = rs_xact_set_state_value_1(&argp, pRs_Client->cl);
    return Rs_GetStatus(result);

}

RS_XACTAPI RS_XACT_get_stat_characteristic( RS_XACT_HANDLE stat_handle, Stat_characteristic *stat_char_buf )
{

    stat_char *result;
    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0) || (stat_char_buf == NULL))
		return FALSE;

    result = rs_xact_get_stat_characteristic_1(&stat_handle,pRs_Client->cl);
    if(result == NULL)
        return RS_SYSTEMERROR;
	if(result->res == FALSE)
		return RS_ERROR;
	

	stat_char_buf->min_sample = result->min_sample;
	stat_char_buf->max_sample = result->max_sample;
	stat_char_buf->num_samples = result->num_samples;
	stat_char_buf->sampled_sum = result->sampled_sum;
	stat_char_buf->min_bucket_value = result->min_bucket_value;
	stat_char_buf->max_bucket_value = result->max_bucket_value;
	stat_char_buf->bucket_size = result->bucket_size;
	stat_char_buf->num_buckets = result->num_buckets;
	stat_char_buf->titled_buckets = result->titled_buckets;
    strcpy(&stat_char_buf->stat_title.name[0], result->stat_title.name);
    strcpy(&stat_char_buf->unit_specifier.name[0], result->unit_specifier.name);

    return RS_SUCCESS;

}
RS_XACTAPI RS_XACT_get_stat_bucket_data( RS_XACT_HANDLE stat_handle, int bucket_index, int *num_bucket_samples, Stat_str *bucket_title )
{
	get_bucket_data_IArg  argp;
	get_bucket_data_OArg  *result;

    if ((pRs_Client == NULL ) || (pRs_Client->cl == 0))
		return FALSE;

	argp.stat_handle = stat_handle;
	argp.bucket_index = bucket_index;

    result = rs_xact_get_stat_bucket_data_1(&argp, pRs_Client->cl);
    if(result == NULL)
        return RS_SYSTEMERROR;

   *num_bucket_samples = result->num_bucket_samples;
   strcpy(&bucket_title->name[0], result->bucket_title.name);
   return RS_SUCCESS;


}



