/* xact_interface.c: implementation of the server stubs for remote procedure transactor 
 *            functions
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 7/8/98
 * revisions:
 * $History: xact_interface.c $
 * 
 * *****************  Version 8  *****************
 * User: Elee1x       Date: 10/16/00   Time: 5:16p
 * Updated in $/dev_1200/Share/Samples/RemoteSimInterface
 * Added support for a couple more calls needed by the remote PLI
 * interface.
 * 
 * *****************  Version 4  *****************
 * User: Qabuild      Date: 9/10/99    Time: 9:49a
 * Updated in $/dev_1200/Share/Samples/RemoteSimInterface
 * changed includes to xact_gui.h & xact_gui_dll.h
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 4/09/99    Time: 5:39p
 * Created in $/development/Samples/RemoteSimExt
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 2/01/99    Time: 7:22p
 * Updated in $/development/RemoteServices
 * Add supports for XACT_get_fifo_handle & XACT_Define_Callback_Ref_Status
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 12/18/98   Time: 4:02p
 * Updated in $/development/RemoteServices
 * Add support for xact_get|set_fifo_state_value
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 12/14/98   Time: 6:20p
 * Updated in $/development/RemoteServices
 * Add support for new transactor functions
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 10/09/98   Time: 9:55p
 * Updated in $/base_level3/RemoteServices
 * Fix a hang problem with termination
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 9/22/98    Time: 5:30p
 * Updated in $/base_level3/RemoteServices
 * Fix bugs
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 9/15/98    Time: 12:03p
 * Updated in $/base_level3/RemoteServices
 * Fix a bug for xact_find_wildcard_state_name function
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 9/14/98    Time: 5:55p
 * Updated in $/base_level3/RemoteServices
 * Add support for 4 new transactor functions:
 * XACTAPI XACT_find_wildcard_state_name()
 * XACTAPI XACT_Define_Callback_Init_Sim();
 * XACTAPI XACT_Define_Callback_Sim_Reset();
 * XACTAPI XACT_Define_Callback_Restore( );
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 8/25/98    Time: 11:56a
 * Updated in $/base_level3/RemoteServices
 * fix a problem with re-init
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/27/98    Time: 10:11a
 * Updated in $/base_level3/RemoteServices
 * Add comments
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 7/24/98    Time: 5:29p
 * Created in $/base_level3/RemoteServices
 * Init. check-in
 *************************************************************/
#include <stdio.h>
#ifdef WIN32
#include "OncRpc32.h"
#include "pmap_clnt.h"
#include <xact_gui_dll.h>
#else
#include <rpc/rpc.h>
#include <xact_gui.h>
#endif

#include "rs_xact.h"
#include "remote_services.h"
#include <thread.h>
#include "sem.h"

/* Default timeout can be changed using clnt_control() */
static struct timeval TIMEOUT = { 50, 0 };
/* remote host name */
static char remote_host[100];

/*
 * function:    *rs_xact_get_handle_1
 * description: call XACT_get_handle
 * return:      return status to the remote host
 */

int *
rs_xact_get_handle_1(handle *argp)

{
	static int res;
	res = XACT_get_handle( argp->name, argp->array_index);
	return (&res);
}


/*
 * function:    *rs_xact_get_fifo_handle_1
 * description: call XACT_get_fifo_handle
 * return:      return status to the remote host
 */

int *
rs_xact_get_fifo_handle_1(handle *argp)

{
	static int res;
	res = XACT_get_fifo_handle( argp->name, argp->array_index);
	return (&res);
}



/*
 * function:    *rs_xact_delete_handle_1
 * description: call XACT_delete_handle
 * return:      return status to the remote host
 */

int *
rs_xact_delete_handle_1(int *argp)
{
	static int res;
    res = XACT_delete_handle(*argp );
	return (&res);
}

/*
 * function:    *rs_xact_get_state_value_1
 * description: call XACT_get_state_value
 * return:      return state value to the remote host
 */


struct varintarr *
rs_xact_get_state_value_1(int *argp)
{

	static struct varintarr res;
    static unsigned int statevalue[MAXBUFLEN];

    res.pdata.pdata_val = &statevalue[0];
 	XACT_get_state_value( *argp, statevalue);
    res.pdata.pdata_len=MAXBUFLEN;
	return (&res);
}

/*
 * function:    *rs_xact_start_of_cycle_1
 * description: call XACT_start_of_cycle
 * return:      return status to the remote host
 */

int *
rs_xact_start_of_cycle_1(RS_CLK_TYPE *argp)
{
	static int res;
	res = XACT_start_of_cycle(*argp);
	return (&res);
}

/*
 * function:    *rs_xact_start_of_cycle_1
 * description: call XACT_clock_cycle
 * return:      return status to the remote host
 */


int *
rs_xact_clock_cycle_1(RS_CLK_TYPE *argp)
{
    static int res;
       
    res = XACT_clock_cycle(*argp);
    return (&res);
}

/*
 * function:    *rs_xact_bus_data_validity_1
 * description: call XACT_bus_data_validity
 * return:      return bus validity to the remote host
 */


int *
rs_xact_bus_data_validity_1(int* argp)
{
    static int res;
    XACT_bus_data_validity(*argp,&res);
    return (&res);
}



/*
 * function:    *rs_xact_get_state_info_1
 * description: call XACT_get_state_info
 * return:      return information to the remote host about the state 
 *              referenced by the specified handle:  
 *      		state_name:    name of state
 *              width:         width of state 
 *		        array_length:  indicates length of array ranging from 0 to 
 *                             array_length-1.  If this value is -1, the 
 *                             state is a non-array state. 
 */

state_info *
rs_xact_get_state_info_1(int *argp)
{
	static state_info res;
    static char s_name[MAXBUFLEN];
    int status;


    res.state_name=&s_name[0];
    memset(s_name,0, MAXBUFLEN);
    status= XACT_get_state_info(*argp, s_name, &res.width, &res.array_length);
    return (&res);
}



stat_char *
rs_xact_get_stat_characteristic_1(int *argp)

{
	static stat_char res;
	static Stat_characteristic stat_char_buf;

	memset(&stat_char_buf, 0, sizeof(stat_char_buf));
    res.res= XACT_get_stat_characteristic(*argp, &stat_char_buf );

	if(res.res == FALSE)
		return(&res);

 	res.min_sample = stat_char_buf.min_sample;
	res.max_sample = stat_char_buf.max_sample;
	res.num_samples = stat_char_buf.num_samples;
	res.sampled_sum = stat_char_buf.sampled_sum;
	res.min_bucket_value = stat_char_buf.min_bucket_value;
	res.max_bucket_value = stat_char_buf.max_bucket_value;
	res.bucket_size = stat_char_buf.bucket_size;
	res.num_buckets = stat_char_buf.num_buckets;
	res.titled_buckets = stat_char_buf.titled_buckets;
    res.stat_title.name = &stat_char_buf.stat_title.name[0];
    res.unit_specifier.name = &stat_char_buf.unit_specifier.name[0];

	return(&res);

}



get_bucket_data_OArg *
rs_xact_get_stat_bucket_data_1(get_bucket_data_IArg *argp)
{
	static get_bucket_data_OArg res;
	static Stat_str name_buf;
	memset(name_buf.name, 0, sizeof(name_buf.name));
	res.bucket_title.name = &name_buf.name[0];

    res.res = XACT_get_stat_bucket_data(argp->stat_handle, argp->bucket_index, &res.num_bucket_samples, &name_buf);

	return (&res);
}


/*
 * function:    *rs_xact_get_array_state_value_1
 * description: call XACT_get_array_state_value
 * return:      return array state value to the remote host
 */

varintarr *
rs_xact_get_array_state_value_1(state_handle *argp)
{

    static struct varintarr res;
    static unsigned int statevalue[MAXBUFLEN];
    res.pdata.pdata_val = &statevalue[0];
    XACT_get_array_state_value( argp->array_handle, argp->array_index,statevalue);
    res.pdata.pdata_len=MAXBUFLEN;
    return (&res);

}

/*
 * function:    *rs_xact_get_fifo_state_value_1
 * description: call XACT_get_fifo_state_value
 * return:      return array state value to the remote host
 */

varintarr *
rs_xact_get_fifo_state_value_1(state_handle *argp)
{

    static struct varintarr res;
    static unsigned int statevalue[MAXBUFLEN];
    res.pdata.pdata_val = &statevalue[0];
    XACT_get_fifo_state_value( argp->array_handle, argp->array_index,statevalue);
    res.pdata.pdata_len=MAXBUFLEN;
    return (&res);

}

/*
 * function:    *rs_xact_set_state_value_1
 * description: call XACT_set_state_value
 * return:      return status to the remote host
 */

int *
rs_xact_set_state_value_1(set_state_value *argp)
{
   	static int res;
    res =  XACT_set_state_value(argp->state_handle,
               argp->value.value_val);
	return (&res);

}

/*
 * function:    *rs_xact_set_array_state_value_1
 * description: call XACT_set_array_state_value
 * return:      return status to the remote host
 */


int *
rs_xact_set_array_state_value_1(set_array_state_value *argp)
{
	static int res;
    res =  XACT_set_array_state_value(argp->array_handle.array_handle,
              argp->array_handle.array_index,
              argp->value.value_val);
	return (&res);
}

/*
 * function:    *rs_xact_exit_transactor_1
 * description: call XACT_exit_transactor
 * return:      return status to the remote host
 */


int *
rs_xact_exit_transactor_1(void *argp)
{
	static int res;
    res =  XACT_exit_transactor();
	return (&res);
}


/*
 * function:    rs_xact_find_wildcard_state_name_1
 * description: call XACT_find_wildcard_state_name
 * return:      returns status to remote client:    
 *              0:   no match found                                             
 *	            1:   match found                                                
 *              -1:  match found but not returned due to size of rtn_name_buf   
 */

wildcard_state *
rs_xact_find_wildcard_state_name_1(char  **argp)
{
    static wildcard_state res;
    static char statename[MAXBUFLEN];
    res.rtn_name_buf= &statename[0];
    if(strlen(*argp) == 0)
      res.status = XACT_find_wildcard_state_name(NULL,statename, MAXBUFLEN);
    else
        res.status = XACT_find_wildcard_state_name(*argp,statename, MAXBUFLEN);


    return (&res);
}



/*
 * function:    *rs_gethostname_1
 * description: get the remote host name
 * return:      return status to the remote host
 */

int *
rs_gethostname_1(char **argp)
{
 
    static int result = 1; /* must be static! */
    memset(remote_host, 0, sizeof(remote_host));
    strcpy(remote_host, *argp);
	return (&result);

}

/*
 * function:    *rs_xact_set_fifo_state_value_1
 * description: call XACT_set_array_state_value
 * return:      return status to the remote host
 */


int *
rs_xact_set_fifo_state_value_1(set_array_state_value *argp)
{
	static int res;
    res =  XACT_set_fifo_state_value(argp->array_handle.array_handle,
              argp->array_handle.array_index,
              argp->value.value_val);
	return (&res);
}


