#define MAXBUFLEN 64
#define MAXPATH 255

enum RS_META_TAG_UNIT_TYPE {
	RS_META_TAG_UNIT__SRAM = 0,
	RS_META_TAG_UNIT__SDRAM = 1,
	RS_META_TAG_UNIT__RFIFO = 2,
	RS_META_TAG_UNIT__TFIFO = 3,
	RS_META_TAG_UNIT__HASH = 4,
	RS_META_TAG_UNIT__PCI = 5,
	RS_META_TAG_UNIT__CSR = 6,
};
typedef enum RS_META_TAG_UNIT_TYPE RS_META_TAG_UNIT_TYPE;
bool_t xdr_RS_META_TAG_UNIT_TYPE();


enum RS_META_TAG_CMD_TYPE {
	RS_META_TAG_CMD__NO_CMD = 0,
	RS_META_TAG_CMD__SRAM_READ = 1,
	RS_META_TAG_CMD__SRAM_READ_LOCK = 2,
	RS_META_TAG_CMD__SRAM_WRITE = 3,
	RS_META_TAG_CMD__SRAM_WRITE_UNLOCK = 4,
	RS_META_TAG_CMD__SRAM_UNLOCK = 5,
	RS_META_TAG_CMD__SRAM_BIT_WRITE = 6,
	RS_META_TAG_CMD__SRAM_PUSH = 7,
	RS_META_TAG_CMD__SRAM_POP = 8,
	RS_META_TAG_CMD__SDRAM_READ = 9,
	RS_META_TAG_CMD__SDRAM_RMW = 10,
	RS_META_TAG_CMD__SDRAM_WRITE = 11,
	RS_META_TAG_CMD__SDRAM_RD_TO_TFIFO = 12,
	RS_META_TAG_CMD__SDRAM_WR_FR_RFIFO = 13,
};
typedef enum RS_META_TAG_CMD_TYPE RS_META_TAG_CMD_TYPE;
bool_t xdr_RS_META_TAG_CMD_TYPE();


enum RS_META_TAG_SIG_TYPE {
	RS_META_TAG_SIG__NONE = 0,
	RS_META_TAG_SIG__SRAM = 1,
	RS_META_TAG_SIG__SDRAM = 2,
	RS_META_TAG_SIG__FBI = 3,
	RS_META_TAG_SIG__PCI = 4,
	RS_META_TAG_SIG__INTER_THREAD = 5,
	RS_META_TAG_SIG__AUTOPUSH = 6,
	RS_META_TAG_SIG__SEQ_NUM1 = 7,
	RS_META_TAG_SIG__SEQ_NUM2 = 8,
	RS_META_TAG_SIG__REC_CTL = 9,
};
typedef enum RS_META_TAG_SIG_TYPE RS_META_TAG_SIG_TYPE;
bool_t xdr_RS_META_TAG_SIG_TYPE();


enum RS_META_TAG_UPDATE_TYPE {
	RS_META_TAG_UPDATE__REF_CREATE = 0,
	RS_META_TAG_UPDATE__QIN = 1,
	RS_META_TAG_UPDATE__QOUT = 2,
	RS_META_TAG_UPDATE__PROCESS = 3,
	RS_META_TAG_UPDATE__SET_SIGNAL = 4,
	RS_META_TAG_UPDATE__WAKEUP = 5,
};
typedef enum RS_META_TAG_UPDATE_TYPE RS_META_TAG_UPDATE_TYPE;
bool_t xdr_RS_META_TAG_UPDATE_TYPE();


struct rs_meta_tag_ref {
	char *chip_name;
	RS_META_TAG_UNIT_TYPE dest_unit;
	RS_META_TAG_CMD_TYPE cmd_type;
	RS_META_TAG_SIG_TYPE sig_type;
	int fbox_pc;
	int fbox;
	int ctx;
	int fbox_to_be_signaled;
	int ctx_to_be_signaled;
	int word_cnt;
	int ending_addr;
	int rmw;
	u_int creation_time;
	RS_META_TAG_UPDATE_TYPE update_type;
	int qin_cycle;
	int qout_cycle;
	int process_cycle;
	int signal_cycle;
	int wakeup_cycle;
};
typedef struct rs_meta_tag_ref rs_meta_tag_ref;
bool_t xdr_rs_meta_tag_ref();


enum RS_CLK_TYPE {
	RS_CLK_FBOX = 0,
	RS_CLK_FBUS = 1,
	RS_CLK_PCI = 2,
};
typedef enum RS_CLK_TYPE RS_CLK_TYPE;
bool_t xdr_RS_CLK_TYPE();


struct statevalue {
	struct {
		u_int value_len;
		int *value_val;
	} value;
};
typedef struct statevalue statevalue;
bool_t xdr_statevalue();


struct statedata {
	int handle;
	struct {
		u_int value_len;
		int *value_val;
	} value;
};
typedef struct statedata statedata;
bool_t xdr_statedata();


struct varintarr {
	struct {
		u_int pdata_len;
		u_int *pdata_val;
	} pdata;
};
typedef struct varintarr varintarr;
bool_t xdr_varintarr();


struct handle {
	char *name;
	int array_index;
};
typedef struct handle handle;
bool_t xdr_handle();


struct wildcard_state {
	char *rtn_name_buf;
	int status;
};
typedef struct wildcard_state wildcard_state;
bool_t xdr_wildcard_state();


struct state_info {
	char *state_name;
	int width;
	int array_length;
};
typedef struct state_info state_info;
bool_t xdr_state_info();


struct state_handle {
	u_int array_handle;
	u_int array_index;
};
typedef struct state_handle state_handle;
bool_t xdr_state_handle();


struct set_array_state_value {
	state_handle array_handle;
	struct {
		u_int value_len;
		u_int *value_val;
	} value;
};
typedef struct set_array_state_value set_array_state_value;
bool_t xdr_set_array_state_value();


struct set_state_value {
	int state_handle;
	struct {
		u_int value_len;
		u_int *value_val;
	} value;
};
typedef struct set_state_value set_state_value;
bool_t xdr_set_state_value();


struct rs_stat_str {
	char *name;
};
typedef struct rs_stat_str rs_stat_str;
bool_t xdr_rs_stat_str();


struct get_bucket_data_IArg {
	int stat_handle;
	int bucket_index;
};
typedef struct get_bucket_data_IArg get_bucket_data_IArg;
bool_t xdr_get_bucket_data_IArg();


struct get_bucket_data_OArg {
	int num_bucket_samples;
	rs_stat_str bucket_title;
	int res;
};
typedef struct get_bucket_data_OArg get_bucket_data_OArg;
bool_t xdr_get_bucket_data_OArg();


struct stat_char {
	int min_sample;
	int max_sample;
	int num_samples;
	int sampled_sum;
	int min_bucket_value;
	int max_bucket_value;
	int bucket_size;
	int num_buckets;
	int titled_buckets;
	rs_stat_str stat_title;
	rs_stat_str unit_specifier;
	int res;
};
typedef struct stat_char stat_char;
bool_t xdr_stat_char();


#define SA1200PROG ((u_long)1689)
#define SA1200VERSION ((u_long)1)
#define RS_XACT_GET_HANDLE ((u_long)1)
extern int *rs_xact_get_handle_1();
#define RS_XACT_DELETE_HANDLE ((u_long)2)
extern int *rs_xact_delete_handle_1();
#define RS_XACT_GET_STATE_VALUE ((u_long)3)
extern varintarr *rs_xact_get_state_value_1();
#define RS_XACT_START_OF_CYCLE ((u_long)4)
extern int *rs_xact_start_of_cycle_1();
#define RS_XACT_CLOCK_CYCLE ((u_long)5)
extern int *rs_xact_clock_cycle_1();
#define RS_XACT_GET_STATE_INFO ((u_long)6)
extern state_info *rs_xact_get_state_info_1();
#define RS_XACT_GET_ARRAY_STATE_VALUE ((u_long)7)
extern varintarr *rs_xact_get_array_state_value_1();
#define RS_XACT_SET_STATE_VALUE ((u_long)8)
extern int *rs_xact_set_state_value_1();
#define RS_XACT_SET_ARRAY_STATE_VALUE ((u_long)9)
extern int *rs_xact_set_array_state_value_1();
#define RS_GETHOSTNAME ((u_long)11)
extern int *rs_gethostname_1();
#define RS_XACT_FIND_WILDCARD_STATE_NAME ((u_long)12)
extern wildcard_state *rs_xact_find_wildcard_state_name_1();
#define RS_XACT_GET_STAT_CHARACTERISTIC ((u_long)13)
extern stat_char *rs_xact_get_stat_characteristic_1();
#define RS_XACT_GET_STAT_BUCKET_DATA ((u_long)14)
extern get_bucket_data_OArg *rs_xact_get_stat_bucket_data_1();
#define RS_XACT_GET_FIFO_STATE_VALUE ((u_long)15)
extern varintarr *rs_xact_get_fifo_state_value_1();
#define RS_XACT_SET_FIFO_STATE_VALUE ((u_long)16)
extern int *rs_xact_set_fifo_state_value_1();
#define RS_XACT_GET_FIFO_HANDLE ((u_long)17)
extern int *rs_xact_get_fifo_handle_1();
#define RS_XACT_BUS_DATA_VALIDITY ((u_long)18)
extern int *rs_xact_bus_data_validity_1();
#define RS_XACT_EXIT_TRANSACTOR ((u_long)19)
extern int *rs_xact_exit_transactor_1();
