/* xact_server.x
 *      transactor remote sample functions
 *
 * ------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services
 * author: Lai-Wah Hui 4/8/99
 * revisions:
 * $History: xact.x
 */ 

const MAXBUFLEN  = 64;
const MAXPATH = 255;

/* definitions for fbox reference status updates */
enum RS_META_TAG_UNIT_TYPE
	{
	RS_META_TAG_UNIT__SRAM,
	RS_META_TAG_UNIT__SDRAM,
	RS_META_TAG_UNIT__RFIFO,
	RS_META_TAG_UNIT__TFIFO,
	RS_META_TAG_UNIT__HASH,
	RS_META_TAG_UNIT__PCI,
	RS_META_TAG_UNIT__CSR
	};


enum RS_META_TAG_CMD_TYPE
	{
	RS_META_TAG_CMD__NO_CMD,
	RS_META_TAG_CMD__SRAM_READ,
	RS_META_TAG_CMD__SRAM_READ_LOCK,
	RS_META_TAG_CMD__SRAM_WRITE,
	RS_META_TAG_CMD__SRAM_WRITE_UNLOCK,
	RS_META_TAG_CMD__SRAM_UNLOCK,
	RS_META_TAG_CMD__SRAM_BIT_WRITE,
	RS_META_TAG_CMD__SRAM_PUSH,
	RS_META_TAG_CMD__SRAM_POP,
	RS_META_TAG_CMD__SDRAM_READ,
	RS_META_TAG_CMD__SDRAM_RMW,
	RS_META_TAG_CMD__SDRAM_WRITE,
	RS_META_TAG_CMD__SDRAM_RD_TO_TFIFO,
	RS_META_TAG_CMD__SDRAM_WR_FR_RFIFO
	};


enum RS_META_TAG_SIG_TYPE
	{
	RS_META_TAG_SIG__NONE,
	RS_META_TAG_SIG__SRAM,
	RS_META_TAG_SIG__SDRAM,
	RS_META_TAG_SIG__FBI,
	RS_META_TAG_SIG__PCI,
	RS_META_TAG_SIG__INTER_THREAD,
	RS_META_TAG_SIG__AUTOPUSH,
	RS_META_TAG_SIG__SEQ_NUM1,
	RS_META_TAG_SIG__SEQ_NUM2,
	RS_META_TAG_SIG__REC_CTL
	};

enum RS_META_TAG_UPDATE_TYPE
	{
	RS_META_TAG_UPDATE__REF_CREATE,
	RS_META_TAG_UPDATE__QIN,
	RS_META_TAG_UPDATE__QOUT,
	RS_META_TAG_UPDATE__PROCESS,
	RS_META_TAG_UPDATE__SET_SIGNAL,
	RS_META_TAG_UPDATE__WAKEUP
	};


struct rs_meta_tag_ref
	{
	string chip_name<MAXBUFLEN>;	 	 /* name of chip associated with this ref */
	RS_META_TAG_UNIT_TYPE dest_unit;	/* destination unit that processes the ref */
	RS_META_TAG_CMD_TYPE cmd_type;/* type of ref  */
	RS_META_TAG_SIG_TYPE sig_type; /* type of signal  */
	int fbox_pc;				/* fbox pc at which ref was issued */
	int fbox;					/* fbox corresponding to ref issue */
	int ctx;					/* fbox ctx corresponding to ref issue */
	int fbox_to_be_signaled;	/* fbox corresponding to fbox to be signaled  */
	int ctx_to_be_signaled;		/* fbox ctx corresponding ctx to be signaled  */
	int word_cnt;				/* num of words bursted in ref	 */
	int ending_addr;			/* ending addr of ref  */
	int rmw;					/* indicates that this is a read-mod-write ref */

	unsigned int creation_time;	/* creation time of ref:  NOTE that qin_cycles, qout_cycles, process_cycles,   */
								/* and wakeup_cycles all specify cycle counts relative to the cycle	  */
								/* value specified in creation_time	  */

	RS_META_TAG_UPDATE_TYPE update_type;	/* specifies 1 of 5 values below that has been updated */
	int qin_cycle;				/* core clk cycle when ref was loaded in dest queue (-1 ==> uninitialized) */
	int qout_cycle;				/* core clk cycle when ref was unloaced from dest queue (-1 ==> uninitialized) */
	int process_cycle;			/* core clk cycle when ref was read/written (-1 ==> uninitialized)	*/
	int signal_cycle;			/* core clk cycle when signal was sent to fbox	  */
	int wakeup_cycle;			/* core clk cycle when fbox thread arbitor "consumes" a signal event  */
								/* and thus, wakes up a thread if it was previously swapped out		  */
 	};



enum RS_CLK_TYPE 
    {
    RS_CLK_FBOX,
    RS_CLK_FBUS,
    RS_CLK_PCI
    } ;

struct statevalue
{
	int value<MAXBUFLEN>;
};

struct statedata
{
    int handle;
    int value<>;
};
struct  varintarr
{
   unsigned int pdata<MAXBUFLEN>;
};

struct handle
{
	string name<MAXBUFLEN>;
	int    array_index;
};


struct wildcard_state
{
	string rtn_name_buf<MAXBUFLEN>;
	int    status;
};


struct state_info
{
	string state_name<MAXBUFLEN>;
        int    width;
        int    array_length;
};

struct  state_handle
{
	unsigned int array_handle;
	unsigned int array_index;
};

struct  set_array_state_value
{
        state_handle array_handle;
        unsigned int value<>;
};


struct  set_state_value
{
        int      state_handle;
        unsigned int value<>;
};

struct rs_stat_str
{
	string name<>;
};

struct  get_bucket_data_IArg
{
        int      stat_handle;
        int      bucket_index;
};

struct  get_bucket_data_OArg
{
        int      num_bucket_samples;
        rs_stat_str   bucket_title;
	int res;
};


struct stat_char
{
	int min_sample;					
	int max_sample;					
	int num_samples;				
	int sampled_sum;				
	int min_bucket_value;			
	int max_bucket_value;			
	int bucket_size;				
	int num_buckets;				
	int titled_buckets;				
	rs_stat_str stat_title;		
	rs_stat_str unit_specifier;	
	int res;
};	


program SA1200PROG {
	version SA1200VERSION {
		int RS_XACT_GET_HANDLE(handle)=1;
		int RS_XACT_DELETE_HANDLE(int)=2;
        varintarr RS_XACT_GET_STATE_VALUE(int) = 3;
		int RS_XACT_START_OF_CYCLE(RS_CLK_TYPE)=4;
		int RS_XACT_CLOCK_CYCLE( RS_CLK_TYPE )=5;
        state_info RS_XACT_GET_STATE_INFO(int)=6; 
        varintarr RS_XACT_GET_ARRAY_STATE_VALUE(state_handle)=7;
        int RS_XACT_SET_STATE_VALUE(set_state_value) = 8;
        int RS_XACT_SET_ARRAY_STATE_VALUE(set_array_state_value)=9;
        int RS_GETHOSTNAME(string)=11;
		wildcard_state RS_XACT_FIND_WILDCARD_STATE_NAME(string)=12;
		stat_char RS_XACT_GET_STAT_CHARACTERISTIC(int)=13;
		get_bucket_data_OArg RS_XACT_GET_STAT_BUCKET_DATA(get_bucket_data_IArg)=14;
	    varintarr RS_XACT_GET_FIFO_STATE_VALUE(state_handle)=15;
        int RS_XACT_SET_FIFO_STATE_VALUE(set_array_state_value)=16;
		int RS_XACT_GET_FIFO_HANDLE(handle)=17;

	} = 1;
} = 1689;

