#include <stdio.h>
#ifdef WIN32
#include <OncRpc32.h>
#include <pmap_clnt.h>
#else
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#endif
#include "xact_server.h"

static void sa1200prog_1();

void XACT_Server()
{
	SVCXPRT *transp;
#ifdef WIN32
  	OncRpc32Init();
#endif


	(void)pmap_unset(SA1200PROG, SA1200VERSION);

	transp = svcudp_create(RPC_ANYSOCK);
	if (transp == NULL) {
		(void)fprintf(stderr, "cannot create udp service.\n");
		exit(1);
	}
	if (!svc_register(transp, SA1200PROG, SA1200VERSION, sa1200prog_1, IPPROTO_UDP)) {
		(void)fprintf(stderr, "unable to register (SA1200PROG, SA1200VERSION, udp).\n");
		exit(1);
	}

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		(void)fprintf(stderr, "cannot create tcp service.\n");
		exit(1);
	}
	if (!svc_register(transp, SA1200PROG, SA1200VERSION, sa1200prog_1, IPPROTO_TCP)) {
		(void)fprintf(stderr, "unable to register (SA1200PROG, SA1200VERSION, tcp).\n");
		exit(1);
	}
	svc_run();
	(void)fprintf(stderr, "svc_run returned\n");
	exit(1);
}

static void
sa1200prog_1(rqstp, transp)
	struct svc_req *rqstp;
	SVCXPRT *transp;

{

	union {
		handle rs_xact_get_handle_1_arg;
		int rs_xact_delete_handle_1_arg;
		int rs_xact_get_state_value_1_arg;
		RS_CLK_TYPE rs_xact_start_of_cycle_1_arg;
		RS_CLK_TYPE rs_xact_clock_cycle_1_arg;
		int rs_xact_get_state_info_1_arg;
		state_handle rs_xact_get_array_state_value_1_arg;
		set_state_value rs_xact_set_state_value_1_arg;
		set_array_state_value rs_xact_set_array_state_value_1_arg;
		char *rs_gethostname_1_arg;
		char *rs_xact_find_wildcard_state_name_1_arg;
		int rs_xact_get_stat_characteristic_1_arg;
		get_bucket_data_IArg rs_xact_get_stat_bucket_data_1_arg;
		state_handle rs_xact_get_fifo_state_value_1_arg;
		set_array_state_value rs_xact_set_fifo_state_value_1_arg;
		handle rs_xact_get_fifo_handle_1_arg;
		int *rs_xact_bus_data_validity_1_arg;
		void *rs_xact_exit_transactor_1_arg;
	} argument;
	char *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	char *(*local)();

	switch (rqstp->rq_proc) {
	case NULLPROC:
		(void)svc_sendreply(transp, xdr_void, (char *)NULL);
		return;

	case RS_XACT_GET_HANDLE:
		xdr_argument = xdr_handle;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_get_handle_1;
		break;

	case RS_XACT_DELETE_HANDLE:
		xdr_argument = xdr_int;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_delete_handle_1;
		break;

	case RS_XACT_GET_STATE_VALUE:
		xdr_argument = xdr_int;
		xdr_result = xdr_varintarr;
		local = (char *(*)()) rs_xact_get_state_value_1;
		break;

	case RS_XACT_START_OF_CYCLE:
		xdr_argument = xdr_RS_CLK_TYPE;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_start_of_cycle_1;
		break;

	case RS_XACT_CLOCK_CYCLE:
		xdr_argument = xdr_RS_CLK_TYPE;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_clock_cycle_1;
		break;

	case RS_XACT_GET_STATE_INFO:
		xdr_argument = xdr_int;
		xdr_result = xdr_state_info;
		local = (char *(*)()) rs_xact_get_state_info_1;
		break;

	case RS_XACT_GET_ARRAY_STATE_VALUE:
		xdr_argument = xdr_state_handle;
		xdr_result = xdr_varintarr;
		local = (char *(*)()) rs_xact_get_array_state_value_1;
		break;

	case RS_XACT_SET_STATE_VALUE:
		xdr_argument = xdr_set_state_value;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_set_state_value_1;
		break;

	case RS_XACT_SET_ARRAY_STATE_VALUE:
		xdr_argument = xdr_set_array_state_value;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_set_array_state_value_1;
		break;

	case RS_GETHOSTNAME:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_gethostname_1;
		break;

	case RS_XACT_FIND_WILDCARD_STATE_NAME:
		xdr_argument = xdr_wrapstring;
		xdr_result = xdr_wildcard_state;
		local = (char *(*)()) rs_xact_find_wildcard_state_name_1;
		break;

	case RS_XACT_GET_STAT_CHARACTERISTIC:
		xdr_argument = xdr_int;
		xdr_result = xdr_stat_char;
		local = (char *(*)()) rs_xact_get_stat_characteristic_1;
		break;

	case RS_XACT_GET_STAT_BUCKET_DATA:
		xdr_argument = xdr_get_bucket_data_IArg;
		xdr_result = xdr_get_bucket_data_OArg;
		local = (char *(*)()) rs_xact_get_stat_bucket_data_1;
		break;

	case RS_XACT_GET_FIFO_STATE_VALUE:
		xdr_argument = xdr_state_handle;
		xdr_result = xdr_varintarr;
		local = (char *(*)()) rs_xact_get_fifo_state_value_1;
		break;

	case RS_XACT_SET_FIFO_STATE_VALUE:
		xdr_argument = xdr_set_array_state_value;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_set_fifo_state_value_1;
		break;

	case RS_XACT_GET_FIFO_HANDLE:
		xdr_argument = xdr_handle;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_get_fifo_handle_1;
		break;

	case RS_XACT_BUS_DATA_VALIDITY:
		xdr_argument = xdr_int;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_bus_data_validity_1;
		break;

	case RS_XACT_EXIT_TRANSACTOR:
		xdr_argument = xdr_void;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_xact_exit_transactor_1;
		break;

	default:
		svcerr_noproc(transp);
		return;
	}
	bzero((char *)&argument, sizeof(argument));
	if (!svc_getargs(transp, xdr_argument, &argument)) {
		svcerr_decode(transp);
		return;
	}

	result = (*local)(&argument, rqstp);
	if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
		svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, xdr_argument, &argument)) {
		(void)fprintf(stderr, "unable to free arguments\n");
		exit(1);
	}
}

