/* rsSc_udebug_svc.c
 * Remote Services Debug Library - Implements Serial server entry points
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rsSC_udebug_svc.c $
 * 
 * *****************  Version 39  *****************
 * User: Ewalker      Date: 9/29/00    Time: 2:32p
 * Updated in $/dev_1200/Share/rs_udebug
 * Changed server receive loop timeout back to infinite
 * 
 * *****************  Version 38  *****************
 * User: Ewalker      Date: 9/29/00    Time: 12:27p
 * Updated in $/dev_1200/Share/rs_udebug
 * Change infinite delay in server receive loop to 60 seconds
 * 
 * *****************  Version 36  *****************
 * User: Ewalker      Date: 9/14/00    Time: 10:04a
 * Updated in $/dev_1200/Share/rs_udebug
 * Fixed Rs serial server receive loop error handing
 * 
 * *****************  Version 35  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ostype.h"
#include "scomm.h"
#include "RS_ScMsg.h"
#include "rs_udebug_svc.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

#ifdef __cplusplus
extern "C" {
#endif
extern char SerialDebugDevName[];
void Rs_SC_server();
#ifdef __cplusplus
}
#endif

static int sa1200SC_target(Sc_Msg_T *Msg, int sConnId);

static int ScommSvc_SendResults(Sc_CLIENT *clnt, 
				   bool_t (*ResultRoutine)(Sc_CLIENT *clnt, void *result, int Flag),
				   void *result, int Flag)
{
    bool_t res;
    Sc_Msg_T Rpc_ID;
    int len;
    
    /* send result to the remote client function */
    res = (ResultRoutine)(clnt, result, Flag);
    if (!res) return RS_ERROR; 
    
    /* get ack from client */
    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
    
    if((len == sizeof(Sc_Msg_T)) && 
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
        res = TRUE;
    else res = FALSE;
    
    return (res ? RS_SUCCESS : RS_ERROR);
}

	
static int ScommSvc_GetArgs(Sc_CLIENT *clnt, 
			   int (*ArgRoutine)(Sc_CLIENT *clnt, void *argp, int Flag), 
			   void *argp, int Flag)
{
	int res;
	Sc_Msg_T Rpc_ID;
    int len;

	Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;
	
    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
    /*if (len != sizeof(Sc_Msg_T)) return RS_ERROR;*/

	if((res = (ArgRoutine)(clnt, argp, Flag)) != TRUE) return RS_ERROR;

	Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
    /*if (len != sizeof(Sc_Msg_T)) return RS_ERROR;*/

	return RS_SUCCESS;
}

/* free arguments */
static int Scomm_freeargs(unsigned FuncID, void *arg)
{
	switch (FuncID) {
		case RS_CNTL_INITUOF:
                {
                rs_CNTL_InitUof_T *Tmp;
                Tmp = (rs_CNTL_InitUof_T *)arg;
   				if (Tmp->UofData.UofData_val != NULL) 
					free ((void *)(Tmp->UofData.UofData_val));
                }
				break;
#if 0
        /* values are allocated statically - no free required */
   	    case RS_UDBG_GETUWORD:
	    case RS_UDBG_SETUWORD:
  	    case RS_UDBG_GETNUMUWORD:
        case RS_UDBG_SETNUMUWORD:
                {
		        rs_uDbg_uWord_T *objp;
				objp = (rs_uDbg_uWord_T *)arg;
				free ((void *)(objp->u_Word.u_Word_val));
                }
                break;
 	   case RS_UDBG_WRITEMEM32:
	   case RS_UDBG_READMEM32:
	   case RS_UDBG_WRITEMEM64:
	   case RS_UDBG_READMEM64:
                {
		        rs_uDbg_Mem_T *objp;
				objp = (rs_uDbg_Mem_T *)arg;
				free ((void *)(objp->data.data_val));
                }
				break;
	   case RS_UDBG_WRITEFIFO_TX:
	   case RS_UDBG_READFIFO_RX:
                {
		        rs_uDbg_FiFo_T *objp;
				objp = (rs_uDbg_FiFo_T *)arg;
				free ((void *)(objp->data64.data64_val));	   
                }
				break;
      case RS_UDBG_GETCSR:
      case RS_UDBG_SETCSR:
                {
		        rs_uDbg_CSR_T *objp;
				objp = (rs_uDbg_CSR_T *)arg;
				free ((void *)(objp->csrData.csrData_val));	
                }
				break;
	  case RS_UDBG_PUTRELGPR:
 	  case RS_UDBG_GETRELGPR:
	  case RS_UDBG_PUTRELSRAM_WRXFER:
	  case RS_UDBG_PUTRELSDRAM_WRXFER:
	  case RS_UDBG_PUTRELSRAM_RDXFER:
	  case RS_UDBG_PUTRELSDRAM_RDXFER:
	  case RS_UDBG_GETRELSRAM_WRXFER:
	  case RS_UDBG_GETRELSDRAM_WRXFER:
	  case RS_UDBG_GETRELSRAM_RDXFER:
	  case RS_UDBG_GETRELSDRAM_RDXFER:
                {
		        rs_uDbg_Reg_T *objp;
				objp = (rs_uDbg_Reg_T *)arg;
				free ((void *)(objp->regData.regData_val));	 
                }
				break;
#endif
	  default: break;
	}	

	return RS_SUCCESS;
}


#if (OS == WIN_32)
#ifdef RS_UDEBUGTEST
void Rs_SC_server()
#else
void main()
#endif
#else
void Rs_SC_server()
#endif
{
    int sConnId;
    int res;
	int len;
    char *dev = SerialDebugDevName;
	Sc_Msg_T Msg;

#ifdef _DBG_PRINT
    printf("\nStarting serial debug server using device %s\n", dev);
#endif

    if ((res = SComm_InstallDev(dev)) != SCOMM_SUCCESS){  /* typically called at OS startup */
#ifdef _DBG_PRINT
		printf("Exiting serial debug server... return status from SComm_InstallDev=%d\n", res);
#endif
        return;

    }	
    
    sConnId = SComm_Open(dev, TargetComPort_1);
    if (sConnId < 0){
#ifdef _DBG_PRINT
		printf("Exiting serial debug server... return status from SComm_Open=%d\n", sConnId);
#endif
        return;
    }
    
    while(1){
	    /* ready to receive */
		len = SComm_Recv(sConnId, &Msg, sizeof(Sc_Msg_T), ScommInfiniteTimeout);
		if(len < 0){
#ifdef _DBG_PRINT
			printf("Exiting serial debug server... return status from SComm_Recv=%d\n", len);
#endif
			return;
		}

        if(len != sizeof(Sc_Msg_T) || Msg.MsgType != RS_FUNCTION)
			continue;

        if((res = sa1200SC_target(&Msg, sConnId)) != RS_SUCCESS) {
            /* send error reply */
            Msg.MsgType = RS_REPLY;
			Msg.Arg.Reply = res;
            len = SComm_Send(sConnId, HostComPort_1, &Msg, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
		}
	}
}

static int
sa1200SC_target(Sc_Msg_T *Msg, Sc_CLIENT sConnId)
{
	union {
		rs_uDbg_uStoreFree_T rs_udbg_initlib_1_arg;
		rs_uDbg_Handle_T rs_udbg_delelib_1_arg;
		rs_uDbg_Two_uint_T rs_udbg_resetueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_clrresetueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_startueng_1_arg;
		rs_uDbg_Two_uint_T rs_udbg_pauseueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_resumeueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_isuengactive_1_arg;
		rs_uDbg_One_uint_T rs_udbg_getactiveuengs_1_arg;
		rs_uDbg_GetThPC_T rs_udbg_getuthreadpc_1_arg;
		rs_uDbg_Three_uint_T rs_udbg_setuthreadspc_1_arg;
		rs_uDbg_Mem_T rs_udbg_setuengpc_1_arg;
		rs_uDbg_One_uint_T rs_udbg_getuengpc_1_arg;
		rs_uDbg_uWord_T rs_udbg_getuword_1_arg;
		rs_uDbg_uWord_T rs_udbg_setuword_1_arg;
		rs_uDbg_uWord_T rs_udbg_getnumuword_1_arg;
		rs_uDbg_uWord_T rs_udbg_setnumuword_1_arg;
		rs_uDbg_Mem_T rs_udbg_writemem32_1_arg;
		rs_uDbg_Mem_T rs_udbg_readmem32_1_arg;
		rs_uDbg_Mem_T rs_udbg_writemem64_1_arg;
		rs_uDbg_Mem_T rs_udbg_readmem64_1_arg;
		rs_uDbg_BreakPoint_T rs_udbg_setbreakpoint_1_arg;
		rs_uDbg_One_uint_T rs_udbg_showbreakpoints_1_arg;
		rs_uDbg_One_uint_T rs_udbg_clearallbreakpoints_1_arg;
		rs_uDbg_One_uint_T rs_udbg_clearbreakpoint_1_arg;
		rs_uDbg_CSR_T rs_udbg_getcsr_1_arg;
		rs_uDbg_CSR_T rs_udbg_setcsr_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelgpr_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelgpr_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsdram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsram_rdxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsdram_rdxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsdram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsram_rdxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsdram_rdxfer_1_arg;
		rs_uDbg_One_uint_T rs_udbg_getuengstatus_1_arg;
		rs_uDbg_Two_uint_T rs_udbg_hopueng_1_arg;
		rs_CNTL_InitUof_T rs_cntl_inituof_1_arg;
		rs_uDbg_Three_uint_T rs_cntl_reset_1_arg;
		rs_uDbg_Three_uint_T rs_cntl_pause_1_arg;
		rs_uDbg_Two_uint_T rs_cntl_resume_1_arg;
		rs_uDbg_Two_uint_T rs_cntl_start_1_arg;
		rs_uDbg_Three_uint_T rs_cntl_hop_1_arg;
		rs_uDbg_Three_uint_T rs_udbg_definefreeustore_1_arg;
		rs_uDbg_One_uint_T rs_udbg_updatefreeustore_1_arg;
		rs_uDbg_FiFo_T rs_udbg_writefifo_tx_1_arg;
		rs_uDbg_FiFo_T rs_udbg_readfifo_rx_1_arg;
		rs_uDbg_Handle_T rs_udbg_getsysmemsize_1_arg;


	} argument;
	void *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	LOCAL_FUNC local;

    switch (Msg->Arg.FunctionID) {

	case RS_UDBG_INITLIB: 
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_uStoreFree_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_initlib_1;
		break;

	case RS_UDBG_DELELIB:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Handle_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_delelib_1;
		break;

	case RS_UDBG_DEFINEFREEUSTORE:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Three_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC)rs_udbg_definefreeustore_1;
		break;

	case RS_UDBG_UPDATEFREEUSTORE:
		xdr_argument =  (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC)rs_udbg_updatefreeustore_1;
		break;

	case RS_UDBG_RESETUENG:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Two_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_resetueng_1;
		break;

	case RS_UDBG_CLRRESETUENG: 
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_clrresetueng_1;
		break;

	case RS_UDBG_STARTUENG:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_startueng_1;
		break;

	case RS_UDBG_PAUSEUENG:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Two_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_pauseueng_1;
		break;

	case RS_UDBG_RESUMEUENG:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_resumeueng_1;
		break;

	case RS_UDBG_ISUENGACTIVE:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_isuengactive_1;
		break;

	case RS_UDBG_GETACTIVEUENGS:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_getactiveuengs_1;
		break;

	case RS_UDBG_GETUTHREADPC:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_GetThPC_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getuthreadpc_1;
		break;

	case RS_UDBG_SETUTHREADSPC:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Three_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_setuthreadspc_1;
		break;

	case RS_UDBG_SETUENGPC:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Mem_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_setuengpc_1;
		break;

	case RS_UDBG_GETUENGPC:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getuengpc_1;
		break;

	case RS_UDBG_GETUWORD:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_uWord_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getuword_1;
		break;

	case RS_UDBG_SETUWORD:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_uWord_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_setuword_1;
		break;

	case RS_UDBG_GETNUMUWORD:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_uWord_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getnumuword_1;
		break;

	case RS_UDBG_SETNUMUWORD:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_uWord_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_setnumuword_1;
		break;

	case RS_UDBG_WRITEMEM32:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Mem_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_writemem32_1;
		break;

	case RS_UDBG_READMEM32:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Mem_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_readmem32_1;
		break;

	case RS_UDBG_WRITEMEM64:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Mem_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_writemem64_1;
		break;

	case RS_UDBG_READMEM64:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Mem_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_readmem64_1;
		break;

	case RS_UDBG_SETBREAKPOINT:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_BreakPoint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_setbreakpoint_1;
		break;

	case RS_UDBG_SHOWBREAKPOINTS:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_ShowBreakpts_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_showbreakpoints_1;
		break;

	case RS_UDBG_CLEARALLBREAKPOINTS:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_clearallbreakpoints_1;
		break;

	case RS_UDBG_CLEARBREAKPOINT:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_clearbreakpoint_1;
		break;

	case RS_UDBG_GETCSR:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_CSR_GetArg_T;
		xdr_result =(RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getcsr_1;
		break;

	case RS_UDBG_SETCSR:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_CSR_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_setcsr_1;
		break;

	case RS_UDBG_PUTRELGPR:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_putrelgpr_1;
		break;

	case RS_UDBG_GETRELGPR:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getrelgpr_1;
		break;

	case RS_UDBG_PUTRELSRAM_WRXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_putrelsram_wrxfer_1;
		break;

	case RS_UDBG_PUTRELSDRAM_WRXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_putrelsdram_wrxfer_1;
		break;

	case RS_UDBG_PUTRELSRAM_RDXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_putrelsram_rdxfer_1;
		break;

	case RS_UDBG_PUTRELSDRAM_RDXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_udbg_putrelsdram_rdxfer_1;
		break;

	case RS_UDBG_GETRELSRAM_WRXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getrelsram_wrxfer_1;
		break;

	case RS_UDBG_GETRELSDRAM_WRXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getrelsdram_wrxfer_1;
		break;

	case RS_UDBG_GETRELSRAM_RDXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getrelsram_rdxfer_1;
		break;

	case RS_UDBG_GETRELSDRAM_RDXFER:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Reg_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getrelsdram_rdxfer_1;
		break;

	case RS_UDBG_GETUENGSTATUS:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_One_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_GetuEngStatus_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_getuengstatus_1;
		break;

	case RS_UDBG_HOPUENG:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Two_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_udbg_hopueng_1;
		break;

	case RS_CNTL_INITUOF:
		xdr_argument = (RTN_T)Sc_xdr_rs_CNTL_InitUof_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_cntl_inituof_1;
		break;

	case RS_CNTL_RESET:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Three_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_cntl_reset_1;
		break;

	case RS_CNTL_PAUSE:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Three_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_cntl_pause_1;
		break;

	case RS_CNTL_RESUME:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Two_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_cntl_resume_1;
		break;

	case RS_CNTL_START:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Two_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC) rs_cntl_start_1;
		break;

	case RS_CNTL_HOP:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Three_uint_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC) rs_cntl_hop_1;
		break;

	case RS_UDBG_WRITEFIFO_TX:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_FiFo_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC)rs_udbg_writefifo_tx_1;
		break;

	case RS_UDBG_READFIFO_RX:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_FiFo_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_Status_SetRes_T;
		local = (LOCAL_FUNC)rs_udbg_readfifo_rx_1;
		break;

	case RS_UDBG_GETSYSMEMSIZE:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Handle_GetArg_T;;
		xdr_result = (RTN_T)Sc_xdr_rs_uDbg_GetMemSize_SetRes_T;
		local = (LOCAL_FUNC)rs_udbg_getsysmemsize_1;
		break;

   	default:
#ifdef _DBG_PRINT
        printf("Unrecognized function %d in sa1200SC_target\n", Msg->Arg.FunctionID);
#endif
		return(RS_BADPARAM);
	}

	bzero((char *)&argument, sizeof(argument));

    if (ScommSvc_GetArgs(&sConnId, (ARGRTN_T)xdr_argument, &argument, 0) != RS_SUCCESS) {
		return(RS_BADPARAM);
	}

	result = local(&argument); 

	ScommSvc_SendResults(&sConnId, (ARGRTN_T)xdr_result, result, 0); 

    Scomm_freeargs(Msg->Arg.FunctionID, &argument);
	
    return (RS_SUCCESS);
}

