/* rs_ScuDbg_xdr.c
 * Remote Services Debug Library - Serial communication argument packaging
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rs_ScUdbg_xdr.c $
 * 
 * *****************  Version 27  *****************
 * User: Ewalker      Date: 9/07/00    Time: 16:25
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 26  *****************
 * User: Ewalker      Date: 9/05/00    Time: 16:36
 * Updated in $/dev_1200/Share/rs_udebug
 * Better handshake on CNTL_InitUof
 * 
 * *****************  Version 25  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:39
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "scomm.h"
#include "debug_1200.h"
#include "rs_ScMsg.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

#define MAX_NAME 40

#define RS_SCOMM_BUFSIZE 1024


/****** Client functions ***********/


bool_t
Sc_Get_xdr_rs_uDbg_uEngStatus_T(Sc_CLIENT *clnt, rs_uDbg_uEngStatus_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

    /* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_uEngStatus_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) && 
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

    return (res);
}




bool_t
Sc_Get_xdr_rs_uDbg_GetuEngStatus_T(Sc_CLIENT *clnt, rs_uDbg_GetuEngStatus_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* waiting for result from remote call */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_GetuEngStatus_T), ScommRecvResultsTimeout);
    if (len != sizeof(rs_uDbg_GetuEngStatus_T)) return(FALSE);

    Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, TargetComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_Handle_T(Sc_CLIENT *clnt, rs_uDbg_Handle_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_Handle_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

    return (res);
}


bool_t
Sc_xdr_rs_uDbg_Bkpt_T(Sc_CLIENT *clnt, rs_uDbg_Bkpt_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_Bkpt_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_BPoint_T(Sc_CLIENT *clnt, rs_uDbg_BPoint_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_BPoint_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_BreakPoint_T(Sc_CLIENT *clnt, rs_uDbg_BreakPoint_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_BreakPoint_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

    return (res);
}


bool_t
Sc_Get_xdr_int(Sc_CLIENT *clnt, void *Argp, int Flag)
{
	bool_t res = TRUE;
    Sc_Msg_T Rpc_ID;
    int len;
    unsigned int *objp = (unsigned int *)Argp;

    /* waiting for result from remote call */
	len = SComm_Recv(*clnt,  objp, sizeof(unsigned int), ScommRecvResultsTimeout);
    if (len != sizeof(unsigned int)) return(FALSE);

    Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, TargetComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

    return (res);
}


bool_t
Sc_xdr_rs_uDbg_uStoreFree_T(Sc_CLIENT *clnt, rs_uDbg_uStoreFree_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;
	char pCName[MAX_NAME];

	bzero(pCName, sizeof(pCName));
	strncpy(pCName, objp->ClientName, sizeof(pCName) - 1);
    pCName[sizeof(pCName) - 1] = '\0';

    /* send rs_uDbg_uStoreFree_T */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_uStoreFree_T), ScommSendArgsTimeout);
    /* wait for ACK */
    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        /* send ClientName  */
        len = SComm_Send(*clnt, TargetComPort_1, pCName, MAX_NAME, ScommSendArgsTimeout);
        /* wait for ACK */
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
        
        if ((len == sizeof(Sc_Msg_T)) &&
            (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
            if(objp->freeUpc.freeUpc_len > 0){
                /* send data buffer */
                len = SComm_Send(*clnt, TargetComPort_1, objp->freeUpc.freeUpc_val, 
                    objp->freeUpc.freeUpc_len, ScommSendArgsTimeout);
                len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
                
                if((len == sizeof(Sc_Msg_T)) &&
                    (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
                    /* send data buffer */
                    len = SComm_Send(*clnt, TargetComPort_1, objp->freeSize.freeSize_val, 
                        objp->freeSize.freeSize_len, ScommSendArgsTimeout);
                    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
                    if((len == sizeof(Sc_Msg_T)) &&
                        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
                        res = TRUE;
                }
            }

            else res = TRUE;  /* no freeUpc values to send */
        }
    }
    
	return (res);
}




bool_t
Sc_xdr_rs_uDbg_One_uint_T(Sc_CLIENT *clnt, rs_uDbg_One_uint_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send rs_uDbg_One_uint_T */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_One_uint_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

	return (res);
}




bool_t
Sc_xdr_rs_uDbg_Two_uint_T(Sc_CLIENT *clnt, rs_uDbg_Two_uint_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send rs_uDbg_One_uint_T */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_Two_uint_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_Three_uint_T(Sc_CLIENT *clnt, rs_uDbg_Three_uint_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

    /* send rs_uDbg_One_uint_T */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_Three_uint_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_uWord_T(Sc_CLIENT *clnt, rs_uDbg_uWord_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

    /* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_uWord_T), ScommSendArgsTimeout);

   	len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        if(objp->u_Word.u_Word_len > 0){
            len = SComm_Send(*clnt, TargetComPort_1, objp->u_Word.u_Word_val,
                objp->u_Word.u_Word_len*sizeof(int), ScommSendArgsTimeout);
            
            len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

            if((len == sizeof(Sc_Msg_T)) &&
                (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
                res = TRUE;
        }

		else res = TRUE;
    }

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_Mem_T(Sc_CLIENT *clnt, rs_uDbg_Mem_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_Mem_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        len = SComm_Send(*clnt, TargetComPort_1, objp->data.data_val,
            objp->data.data_len*sizeof(int), ScommSendArgsTimeout);
        
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
        
        if((len == sizeof(Sc_Msg_T)) &&
			(Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
            res = TRUE;
    }
    
    return (res);
}


bool_t
Sc_xdr_rs_uDbg_FiFo_T(Sc_CLIENT *clnt, rs_uDbg_FiFo_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_FiFo_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        len = SComm_Send(*clnt, TargetComPort_1, objp->data64.data64_val,
            objp->data64.data64_len*sizeof(int), ScommSendArgsTimeout);
        
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
        
        if((len == sizeof(Sc_Msg_T)) &&
			(Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
            res = TRUE;
    }
    
    return (res);
}


bool_t
Sc_xdr_rs_string_T(Sc_CLIENT *clnt, rs_string_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

    /* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_string_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        len = SComm_Send(*clnt, TargetComPort_1, objp->msg.msg_val, 
                       objp->msg.msg_len, ScommSendArgsTimeout);
        
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

        if((len == sizeof(Sc_Msg_T)) &&
            (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
            res = TRUE;
    }

	return (res);
}




bool_t
Sc_xdr_rs_uDbg_CSR_T(Sc_CLIENT *clnt, rs_uDbg_CSR_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

    /* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_CSR_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        len = SComm_Send(*clnt, TargetComPort_1, objp->csrData.csrData_val,
            objp->csrData.csrData_len*sizeof(int), ScommSendArgsTimeout);
        
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
        
        if((len == sizeof(Sc_Msg_T)) &&
            (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
            res = TRUE;
    }

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_Reg_T(Sc_CLIENT *clnt, rs_uDbg_Reg_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

    /* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_Reg_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        len = SComm_Send(*clnt, TargetComPort_1, objp->regData.regData_val,
            objp->regData.regData_len*sizeof(int), ScommSendArgsTimeout);
        
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
        
        if((len == sizeof(Sc_Msg_T)) &&
            (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
            res = TRUE;
    }

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_GetThPC_T(Sc_CLIENT *clnt, rs_uDbg_GetThPC_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

    /* send argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_uDbg_GetThPC_T), ScommSendArgsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        len = SComm_Send(*clnt, TargetComPort_1, objp->regData.regData_val,
            objp->regData.regData_len*sizeof(int), ScommSendArgsTimeout);
        
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
        
        if((len == sizeof(Sc_Msg_T)) &&
            (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
            res = TRUE;
    }
    
    return (res);
}

bool_t Sc_Get_xdr_rs_uDbg_GetMemSize_T(Sc_CLIENT *clnt, rs_uDbg_GetMemSize_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;

    len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_GetMemSize_T), ScommRecvResultsTimeout);
    if (len != sizeof(rs_uDbg_GetMemSize_T)) return(FALSE);

    Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, TargetComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

	return (res);
}


bool_t
Sc_Get_xdr_rs_uDbg_ShowBreakpts_T(Sc_CLIENT *clnt, rs_uDbg_ShowBreakpts_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
	static rs_uDbg_Bkpt_T resBkpt[UDBG_MAX_BREAKPOINTS];

    /* waiting for result from remote call */

    len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_ShowBreakpts_T), ScommRecvResultsTimeout);
	if (len != sizeof(rs_uDbg_ShowBreakpts_T)) return(FALSE);

    Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, TargetComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

    bzero((char *)resBkpt, sizeof(rs_uDbg_Bkpt_T)*UDBG_MAX_BREAKPOINTS);

    /* receive actual number of break points */
    len = SComm_Recv(*clnt, &resBkpt[0], objp->BkptArray.len*sizeof(rs_uDbg_Bkpt_T), ScommRecvResultsTimeout);
    if ((unsigned int)len != (objp->BkptArray.len*sizeof(rs_uDbg_Bkpt_T))) return(FALSE);

    Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;
	
    len = SComm_Send(*clnt, TargetComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
	
    objp->BkptArray.bkpts.bkpts_val = (rs_uDbg_Bkpt_T *)&resBkpt[0];  
    
    return (res);
}

bool_t
Sc_Get_xdr_rs_uDbg_Status_T(Sc_CLIENT *clnt, rs_uDbg_Status_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int resData_len = 0;
    static int *resData = NULL;

    /* initialize resData */
    if (resData_len == 0 || resData == NULL){
        resData_len = 256;
        resData = (int *) malloc(resData_len*sizeof(int));
		if (resData == NULL) return(FALSE);
    }

	/* waiting for result from remote call */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Status_T), ScommRecvResultsTimeout);
    if (len != sizeof(rs_uDbg_Status_T)) return(FALSE);
	
    Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, TargetComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

    /* dynamically allocate enough space for memory being received */
    if(objp->resData.resData_len > resData_len){
        if (resData != NULL) free((void *) resData);
        resData_len = objp->resData.resData_len;
        resData = (int *) malloc(resData_len*sizeof(int));
		if (resData == NULL) return(FALSE);
    }

    objp->resData.resData_val = resData;
    
	len = SComm_Recv(*clnt, resData, objp->resData.resData_len*sizeof(int), ScommRecvResultsTimeout);
    if ((unsigned int)len != (objp->resData.resData_len*sizeof(int))) return(FALSE);

	Rpc_ID.MsgType = RS_REPLY;
    Rpc_ID.Arg.Reply = RS_SUCCESS;
	
    len = SComm_Send(*clnt, TargetComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
	
    return (res);
}

bool_t
Sc_xdr_rs_CNTL_InitUof_T(Sc_CLIENT *clnt, rs_CNTL_InitUof_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* get argument structure */
	len = SComm_Send(*clnt, TargetComPort_1, objp, sizeof(rs_CNTL_InitUof_T), ScommSendArgsTimeout);
	
    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

 	if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
		unsigned int TotalLoop = objp->UofData.UofData_len / RS_SCOMM_BUFSIZE;
		unsigned int send_len = objp->UofData.UofData_len % RS_SCOMM_BUFSIZE;

		unsigned int loop; 
		unsigned int ByteSend = 0;
		
		char *BufPtr = objp->UofData.UofData_val;
		for(loop = 0; loop < TotalLoop; loop++){
			BufPtr = BufPtr + ByteSend;
			ByteSend = SComm_Send(*clnt, TargetComPort_1, BufPtr, RS_SCOMM_BUFSIZE, ScommSendArgsTimeout);

            len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

            if((len != sizeof(Sc_Msg_T)) ||
                (Rpc_ID.MsgType != RS_REPLY) || (Rpc_ID.Arg.Reply != RS_SUCCESS)) return(FALSE);
        }

		BufPtr = BufPtr + ByteSend;	

		ByteSend = SComm_Send(*clnt, TargetComPort_1, BufPtr, send_len, ScommSendArgsTimeout);
		
        len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

        if((len == sizeof(Sc_Msg_T)) &&
            (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		    res = TRUE;
	}

    return (res);
}




/****** Server functions ***********/

bool_t
Sc_xdr_rs_uDbg_uEngStatus_SetRes_T(Sc_CLIENT *clnt, rs_uDbg_uEngStatus_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
	int len;

    /* send argument structure */
	len = SComm_Send(*clnt, HostComPort_1, (void *)objp, sizeof(rs_uDbg_uEngStatus_T), ScommSendResultsTimeout);

    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;
	
	return (res);
}



bool_t
Sc_xdr_rs_uDbg_GetuEngStatus_SetRes_T(Sc_CLIENT *clnt, rs_uDbg_GetuEngStatus_T *objp, int Flag)
{
	bool_t res = TRUE;
	int len;

    /* send argument structure */
	len = SComm_Send(*clnt, HostComPort_1, (void *)objp, sizeof(rs_uDbg_GetuEngStatus_T), ScommSendResultsTimeout);
	
	return (res);	
}




bool_t
Sc_xdr_rs_uDbg_Handle_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_Handle_T *objp, int Flag)
{
	bool_t res = TRUE;
   	Sc_Msg_T Rpc_ID;
    int len;

    Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

	/* Get argument  */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Handle_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_Handle_T)) return(FALSE);

	return (res);
}


bool_t
Sc_xdr_rs_uDbg_Bkpt_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_Bkpt_T *objp, int Flag)
{
	bool_t res = TRUE;
   	Sc_Msg_T Rpc_ID;
    int len;

    Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Bkpt_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_Bkpt_T)) return(FALSE);

    return (res);
}



bool_t
Sc_xdr_rs_uDbg_BPoint_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_BPoint_T *objp, int Flag)
{
	bool_t res = TRUE;
   	Sc_Msg_T Rpc_ID;
    int len;

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_BPoint_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_BPoint_T)) return(FALSE);

    return (res);
}



bool_t
Sc_xdr_rs_uDbg_BreakPoint_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_BreakPoint_T *objp, int Flag)
{
	bool_t res = TRUE;
   	Sc_Msg_T Rpc_ID;
    int len;

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_BreakPoint_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_BreakPoint_T)) return(FALSE);

    return (res);
}


bool_t
Sc_xdr_SetRes_int(Sc_CLIENT *clnt, unsigned int *objp, int Flag)
{
	bool_t res = TRUE;
    int len;

	/* send argument structure */
	len = SComm_Send(*clnt, HostComPort_1, (void *)objp, sizeof(int), ScommSendResultsTimeout);
	
    return (res);
}


bool_t
Sc_xdr_rs_uDbg_uStoreFree_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_uStoreFree_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
	static char ClientName[MAX_NAME];
	static u_short freeUpc[MAX_UENGS];
	static u_short freeSize[MAX_UENGS];

    len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_uStoreFree_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_uStoreFree_T)) return(FALSE);

    Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
	
	/* get client name */
	len = SComm_Recv(*clnt, ClientName, MAX_NAME, ScommRecvArgsTimeout);
    if (len != MAX_NAME) return(FALSE);
	
    if(objp->freeUpc.freeUpc_len > 0){    	
		Rpc_ID.MsgType = RS_REPLY;
		Rpc_ID.Arg.Reply = RS_SUCCESS;

		len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
	
        /* get freeUpc */
	    len = SComm_Recv(*clnt, freeUpc, objp->freeUpc.freeUpc_len, ScommRecvArgsTimeout);
        if ((unsigned int)len != objp->freeUpc.freeUpc_len) return(FALSE);
		
        Rpc_ID.MsgType = RS_REPLY;
	    Rpc_ID.Arg.Reply = RS_SUCCESS;

        len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

	    /* get freeSize */
	    len = SComm_Recv(*clnt, freeSize, objp->freeSize.freeSize_len, ScommRecvArgsTimeout);
	    if ((unsigned int)len != objp->freeSize.freeSize_len) return(FALSE);

        objp->freeUpc.freeUpc_val = freeUpc;
	    objp->freeSize.freeSize_val = freeSize;
    }

    objp->ClientName = ClientName;
    
	return (res);
}




bool_t
Sc_xdr_rs_uDbg_One_uint_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_One_uint_T *objp, int Flag)
{
	bool_t res = TRUE;
    int len;

	/* get rs_uDbg_One_uint_T */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_One_uint_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_One_uint_T)) return(FALSE);

	return (res);
}




bool_t
Sc_xdr_rs_uDbg_Two_uint_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_Two_uint_T *objp, int Flag)
{
	bool_t res = TRUE;
    int len;

 	/* get rs_uDbg_One_uint_T */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Two_uint_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_Two_uint_T)) return(FALSE);
	
    return (res);
}




bool_t
Sc_xdr_rs_uDbg_Three_uint_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_Three_uint_T *objp, int Flag)
{
	bool_t res = TRUE;
    int len;

	/* get rs_uDbg_One_uint_T */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Three_uint_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_Three_uint_T)) return(FALSE);

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_uWord_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_uWord_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int uWordBuf_len = 0;
    static int *uWordBuf = NULL;

    /* initialize uWordBuf */
    if (uWordBuf_len == 0 || uWordBuf == NULL){
        uWordBuf_len = 256;
        uWordBuf = (int *) malloc(uWordBuf_len*sizeof(int));
		if (uWordBuf == NULL) return(FALSE);
    }

	/* get argument structure */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_uWord_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_uWord_T)) return(FALSE);

    /* dynamically allocate enough space for uwords being received */
    if(objp->u_Word.u_Word_len > uWordBuf_len){
        if (uWordBuf != NULL) free((void *) uWordBuf);
        uWordBuf_len = objp->u_Word.u_Word_len;
        uWordBuf = (int *) malloc(uWordBuf_len*sizeof(int));
		if (uWordBuf == NULL) return(FALSE);
    }

    objp->u_Word.u_Word_val = uWordBuf;

    if(objp->u_Word.u_Word_len > 0){	    
		Rpc_ID.MsgType = RS_REPLY;
		Rpc_ID.Arg.Reply = RS_SUCCESS;
		
		len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
		
        len = SComm_Recv(*clnt, objp->u_Word.u_Word_val, 
            objp->u_Word.u_Word_len*sizeof(int), ScommRecvArgsTimeout);
        if ((unsigned int)len != (objp->u_Word.u_Word_len*sizeof(int))) return(FALSE);
    }

    return (res);
}




bool_t
Sc_xdr_rs_uDbg_Mem_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_Mem_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int MemBuf_len = 0;
    static int *MemBuf = NULL;

    /* initialize MemBuf */
    if (MemBuf_len == 0 || MemBuf == NULL){
        MemBuf_len = 256;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

	/* get argument structure */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Mem_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_Mem_T)) return(FALSE);

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

    /* dynamically allocate enough space for memory being received */
    if(objp->data.data_len > MemBuf_len){
        if (MemBuf != NULL) free((void *) MemBuf);
        MemBuf_len = objp->data.data_len;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

    objp->data.data_val = MemBuf;

    len = SComm_Recv(*clnt, objp->data.data_val,
        objp->data.data_len*sizeof(int), ScommRecvArgsTimeout);
    if ((unsigned int)len != (objp->data.data_len*sizeof(int))) return(FALSE);

    return (res);
}

bool_t
Sc_xdr_rs_uDbg_FiFo_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_FiFo_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int MemBuf_len = 0;
    static int *MemBuf = NULL;

    /* initialize MemBuf */
    if (MemBuf_len == 0 || MemBuf == NULL){
        MemBuf_len = 256;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

	/* get argument structure */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_FiFo_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_FiFo_T)) return(FALSE);

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

    /* dynamically allocate enough space for memory being received */
    if(objp->data64.data64_len > MemBuf_len){
        if (MemBuf != NULL) free((void *) MemBuf);
        MemBuf_len = objp->data64.data64_len;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

    objp->data64.data64_val = MemBuf;

    len = SComm_Recv(*clnt, objp->data64.data64_val,
        objp->data64.data64_len*sizeof(int), ScommRecvArgsTimeout);
    if ((unsigned int)len != (objp->data64.data64_len*sizeof(int))) return(FALSE);

    return (res);
}



bool_t
Sc_xdr_rs_string_GetArg_T(Sc_CLIENT *clnt, rs_string_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int MemBuf_len = 0;
    static char *MemBuf = NULL;

    /* initialize MemBuf */
    if (MemBuf_len == 0 || MemBuf == NULL){
        MemBuf_len = 256;
        MemBuf = (char *) malloc(MemBuf_len);
		if (MemBuf == NULL) return(FALSE);
    }

    /* get argument structure */
	len = SComm_Recv(*clnt, objp, sizeof(rs_string_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_string_T)) return(FALSE);

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);

    /* dynamically allocate enough space for memory being received */
    if(objp->msg.msg_len > MemBuf_len){
        if (MemBuf != NULL) free((void *) MemBuf);
        MemBuf_len = objp->msg.msg_len;
        MemBuf = (char *) malloc(MemBuf_len);
		if (MemBuf == NULL) return(FALSE);
    }

    objp->msg.msg_val = MemBuf;

    len = SComm_Recv(*clnt, objp->msg.msg_val, objp->msg.msg_len, ScommRecvArgsTimeout);
    if ((unsigned int)len != objp->msg.msg_len) return(FALSE);

    return (res);
}


bool_t
Sc_xdr_rs_uDbg_CSR_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_CSR_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int MemBuf_len = 0;
    static int *MemBuf = NULL;

    /* initialize MemBuf */
    if (MemBuf_len == 0 || MemBuf == NULL){
        MemBuf_len = 256;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

	/* get argument structure */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_CSR_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_CSR_T)) return(FALSE);

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
	
    /* dynamically allocate enough space for memory being received */
    if(objp->csrData.csrData_len > MemBuf_len){
        if (MemBuf != NULL) free((void *) MemBuf);
        MemBuf_len = objp->csrData.csrData_len;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

    objp->csrData.csrData_val = MemBuf;
	
    len = SComm_Recv(*clnt, objp->csrData.csrData_val,
        objp->csrData.csrData_len*sizeof(int), ScommRecvArgsTimeout);
    if ((unsigned int)len != (objp->csrData.csrData_len*sizeof(int))) return(FALSE);
	
    return (res);
}


bool_t
Sc_xdr_rs_uDbg_Reg_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_Reg_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int MemBuf_len = 0;
    static int *MemBuf = NULL;

    /* initialize MemBuf */
    if (MemBuf_len == 0 || MemBuf == NULL){
        MemBuf_len = 256;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

	/* get argument structure */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Reg_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_Reg_T)) return(FALSE);

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
	
    /* dynamically allocate enough space for memory being received */
    if(objp->regData.regData_len > MemBuf_len){
        if (MemBuf != NULL) free((void *) MemBuf);
        MemBuf_len = objp->regData.regData_len;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

    objp->regData.regData_val = MemBuf;
	
    len = SComm_Recv(*clnt, objp->regData.regData_val,
        objp->regData.regData_len*sizeof(int), ScommRecvArgsTimeout);
    if ((unsigned int)len != (objp->regData.regData_len*sizeof(int))) return(FALSE);
	
    return (res);
}




bool_t
Sc_xdr_rs_uDbg_GetThPC_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_GetThPC_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
    static unsigned int MemBuf_len = 0;
    static int *MemBuf = NULL;

    /* initialize MemBuf */
    if (MemBuf_len == 0 || MemBuf == NULL){
        MemBuf_len = 256;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

	/* get argument structure */
	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_GetThPC_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_GetThPC_T)) return(FALSE);

	Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
	
    /* dynamically allocate enough space for memory being received */
    if(objp->regData.regData_len > MemBuf_len){
        if (MemBuf != NULL) free((void *) MemBuf);
        MemBuf_len = objp->regData.regData_len;
        MemBuf = (int *) malloc(MemBuf_len*sizeof(int));
		if (MemBuf == NULL) return(FALSE);
    }

    objp->regData.regData_val = MemBuf;
	
    len = SComm_Recv(*clnt, objp->regData.regData_val,
        objp->regData.regData_len*sizeof(int), ScommRecvArgsTimeout);
    if ((unsigned int)len != (objp->regData.regData_len*sizeof(int))) return(FALSE);

    return (res);
}


bool_t
Sc_xdr_rs_uDbg_ShowBreakpts_SetRes_T(Sc_CLIENT *clnt, rs_uDbg_ShowBreakpts_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send results structure */
	len = SComm_Send(*clnt, HostComPort_1, (void *)objp, sizeof(rs_uDbg_ShowBreakpts_T), ScommSendResultsTimeout);
	
    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
	
    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        /* send actual number of breakpoints to client */
        len = SComm_Send(*clnt, HostComPort_1, objp->BkptArray.bkpts.bkpts_val,
                         objp->BkptArray.len*sizeof(rs_uDbg_Bkpt_T), ScommSendResultsTimeout);       
        res = TRUE;
    }
    
    return (res);
}                                                                  
bool_t
Sc_xdr_rs_uDbg_Status_SetRes_T(Sc_CLIENT *clnt, rs_uDbg_Status_T *objp, int Flag)
{
	bool_t res = FALSE;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send results structure */
	len = SComm_Send(*clnt, HostComPort_1, (void *)objp, sizeof(rs_uDbg_Status_T), ScommSendResultsTimeout);
	
    len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);
	
    if((len == sizeof(Sc_Msg_T)) &&
        (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        len = SComm_Send(*clnt, HostComPort_1, objp->resData.resData_val,
            objp->resData.resData_len*sizeof(int), ScommSendResultsTimeout);

        res = TRUE;
    }
    
	return (res);
}

bool_t 
Sc_xdr_rs_uDbg_GetMemSize_SetRes_T(Sc_CLIENT *clnt,rs_uDbg_GetMemSize_T *objp, int Flag)
{
	bool_t res = TRUE;
    int len;

    len = SComm_Send(*clnt, HostComPort_1, (void *)objp, sizeof(rs_uDbg_GetMemSize_T), ScommSendResultsTimeout);

	return (res);
}


bool_t
Sc_xdr_rs_CNTL_InitUof_GetArg_T(Sc_CLIENT *clnt, rs_CNTL_InitUof_T *objp, int Flag)
{
	bool_t res = TRUE;
	Sc_Msg_T Rpc_ID;
    int len;
	static char *MemBuf, *BufPtr;
	unsigned int TotalLoop;
	unsigned int send_len;
	unsigned int loop; 
	unsigned int ByteReceived = 0;

	objp->UofData.UofData_val = NULL;

	len = SComm_Recv(*clnt, objp, sizeof(rs_CNTL_InitUof_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_CNTL_InitUof_T)) return(FALSE);
	
    Rpc_ID.MsgType = RS_REPLY;
	Rpc_ID.Arg.Reply = RS_SUCCESS;

    len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
    
    MemBuf = malloc(objp->UofData.UofData_len);
    if (MemBuf == NULL) return(FALSE);

	objp->UofData.UofData_val = MemBuf;

    TotalLoop = objp->UofData.UofData_len / RS_SCOMM_BUFSIZE;
    send_len = objp->UofData.UofData_len % RS_SCOMM_BUFSIZE;
    ByteReceived = 0;
 		
	BufPtr = MemBuf;

    for(loop = 0; loop < TotalLoop; loop++){
        BufPtr = BufPtr + ByteReceived;
        ByteReceived = SComm_Recv(*clnt, BufPtr, RS_SCOMM_BUFSIZE, ScommRecvArgsTimeout);
        if (ByteReceived != RS_SCOMM_BUFSIZE) return(FALSE);

        Rpc_ID.MsgType = RS_REPLY;
        Rpc_ID.Arg.Reply = RS_SUCCESS;
        
        len = SComm_Send(*clnt, HostComPort_1, &Rpc_ID, sizeof(Sc_Msg_T), ScommSendReplyTimeout);
    }
    
	BufPtr = BufPtr + ByteReceived;	
 	ByteReceived = SComm_Recv(*clnt, BufPtr, send_len, ScommRecvArgsTimeout);
    if (ByteReceived != send_len) return(FALSE);

	return (res);
}


/* event call back functions */
/* Target (Client) function */
bool_t
Sc_xdr_rs_uDbg_Bkpt_CK_T(Sc_CLIENT *clnt, void *Argp, int Flag)
{
	bool_t res = TRUE;
    int len;
    rs_uDbg_Bkpt_CK_T *objp = (rs_uDbg_Bkpt_CK_T *)Argp;

	/* send argument  */
	len = SComm_Send(*clnt, HostComPort_2, objp, sizeof(rs_uDbg_Bkpt_CK_T), ScommSendArgsTimeout);
    if (len != sizeof(rs_uDbg_Bkpt_CK_T)) res = FALSE;
    
    return (res); 
}

/* Host (Server) function */
bool_t
Sc_xdr_rs_uDbg_Bkpt_CK_GetArg_T(Sc_CLIENT *clnt, rs_uDbg_Bkpt_CK_T *objp, int Flag)
{
	bool_t res = TRUE;
    int len;

	len = SComm_Recv(*clnt, objp, sizeof(rs_uDbg_Bkpt_CK_T), ScommRecvArgsTimeout);
    if (len != sizeof(rs_uDbg_Bkpt_CK_T)) res = FALSE;

    return (res);
}

