/* rs_ScuDbg_events_svc.c
 * Remote Services Debug Library - Serial-based callback (event) services
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rs_ScuDbg_events_svc.c $
 * 
 * *****************  Version 17  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 16  *****************
 * User: Ewalker      Date: 9/29/00    Time: 2:33p
 * Updated in $/dev_1200/Share/rs_udebug
 * Added test for invalid breakpoint notification message size to event
 * server receive loop
 * 
 * *****************  Version 14  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:39
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 
  
#include <stdio.h>

#include "utils.h"
#include "scomm.h"
#include "rs_ScMsg.h"
#include "rs_uDbg_events.h"

#ifdef __cplusplus
extern "C" {
#endif
void *rs_udbg_bkpt_ck_1(rs_uDbg_Bkpt_CK_T *argp);
#ifdef __cplusplus
}
#endif

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

static int udbg_callbk_sc1();

static int sConnId = SCOMM_FAIL;

#if 0
static ScommEvent_SendResults(Sc_CLIENT *clnt, 
				   bool_t (*ResultRoutine)(Sc_CLIENT *clnt, void *result, int Flag),
				   void *result, int Flag)
{
	bool_t res;
	Sc_Msg_T Rpc_ID;
    int len;

	/* send result to the remote client function */
	res = (ResultRoutine)(clnt, result, Flag); 
    if (!res) return RS_ERROR; 

    /* get ack from client */
	len = SComm_Recv(*clnt, &Rpc_ID, sizeof(Sc_Msg_T), ScommRecvReplyTimeout);

	if((len == sizeof(Sc_Msg_T)) &&
       (Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS))
		res = TRUE;
    else res = FALSE;

    return (res ? RS_SUCCESS : RS_ERROR);
}
#endif

static int ScommEvent_GetArgs(Sc_CLIENT *clnt, 
			   int (*ArgRoutine)(Sc_CLIENT *clnt, void *argp, int Flag), 
			   void *argp, int Flag)
{
	int res;

	if((res = (ArgRoutine)(clnt, argp, Flag)) != TRUE) return RS_ERROR;

	return RS_SUCCESS;
}

int SC_Svc_Setup(char *DevName)
{        
    if (sConnId != SCOMM_FAIL) SComm_Close(sConnId);
    
    sConnId = SComm_Open(DevName, HostComPort_2);
    if(sConnId < 0){
#ifdef _DBG_PRINT
		printf("return status from SComm_Open=%d\n", sConnId);
#endif
        sConnId = SCOMM_FAIL;
		return RS_ERROR;
    }
	return RS_SUCCESS;
}

void ScDbgCK_main(int *shutdown_flag)
{
	Sc_Msg_T Msg;
	/* open driver */
	int len;
    int res;

#ifdef _DBG_PRINT
    printf("enter ScDbgCK_main\n");
#endif

    if (sConnId < 0){
#ifdef _DBG_PRINT
        printf("exit... SComm_Open previously failed\n");
#endif
        ExitThread(1);
    }

	while(1){
        /* check if thread termination was requested */
        if (*shutdown_flag) ExitThread(1);

	    /* ready to receive */
		len = SComm_Recv(sConnId, &Msg, sizeof(Sc_Msg_T), 1 /* second */);
		if(len < 0){
#ifdef _DBG_PRINT
			printf("exit... return status from SComm_Recv=%d\n", len);
#endif
			ExitThread(1);
		}

        if(len != sizeof(Sc_Msg_T) || Msg.MsgType != RS_FUNCTION)
			continue;

        res = udbg_callbk_sc1(&Msg, sConnId);
	}
}

static int
udbg_callbk_sc1(Sc_Msg_T *Msg, int sConnId)
{
	union {
		rs_uDbg_Bkpt_CK_T rs_udbg_bkpt_ck_1_arg;
	} argument;
	char *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	LOCAL_FUNC local;

#ifdef _DBG_PRINT
  printf("enter udbg_callbk_sc1: FunctionID=%d\n", Msg->Arg.FunctionID);
#endif
	switch (Msg->Arg.FunctionID) {

    case RS_uDbg_Bkpt_CK:
		xdr_argument = (RTN_T)Sc_xdr_rs_uDbg_Bkpt_CK_GetArg_T;
		xdr_result = (RTN_T)Sc_xdr_SetRes_int;
		local = (LOCAL_FUNC)rs_udbg_bkpt_ck_1;
		break;

   	default:
		return(RS_BADPARAM);
	}
	bzero((char *)&argument, sizeof(argument));

    if (ScommEvent_GetArgs(&sConnId, (ARGRTN_T)xdr_argument, &argument, 0) != RS_SUCCESS) {
        return(RS_BADPARAM); 
    }
 
	result = (*local)(&argument);

#if 0       /* the callback service is a one-way - no results sent */
	ScommEvent_SendResults(&sConnId, (ARGRTN_T)xdr_result, result, 0); 
#endif

    return (RS_SUCCESS);
}
