/* rs_connection.cpp
 * Remote Services Debug Library - Debug connection open and close routines
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Services Debug Library
 *  author: lwh,  2/19/99  Initial version
 *  revisions:
 *  $History: rs_connection.cpp $
 * 
 * *****************  Version 30  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 28  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:39
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 27  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:54
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 26  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:15
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 24  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 1/12/00    Time: 4:14p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 19  *****************
 * User: Lhui         Date: 1/07/00    Time: 5:58p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 1/06/00    Time: 6:07p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port 
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 4/22/99    Time: 5:41p
 * Updated in $/development/rs_udebug
 * Add a delay parameter to  rs_udebug/control  hop functions. 
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:08p
 * Updated in $/development/rs_udebug
 * Fix bugs in rs_uDbg_GetUengPCs and rs_uDbg_InitLib 
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 3/29/99    Time: 6:04p
 * Updated in $/development/rs_udebug
 * Add supports for debug control functions (start & hop)
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 3/23/99    Time: 6:36p
 * Updated in $/development/rs_udebug
 * Code cleanup and bug fixing
 * 
 * *****************  Version 11  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:47p
 * Updated in $/development/rs_udebug
 * update COPYRIGHT
 * 
 * *****************  Version 10  *****************
 * User: Qabuild      Date: 3/19/99    Time: 4:12p
 * Updated in $/development/rs_udebug
 * update makefile dependencies
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 3/19/99    Time: 1:17p
 * Updated in $/development/rs_udebug
 * Add supports for Scomm.lib
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 3/18/99    Time: 5:29p
 * Updated in $/development/rs_udebug
 * Add supports for SA1200 debug control functions
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 3/12/99    Time: 2:04p
 * Updated in $/development/rs_udebug
 * Create an  include file for external interface
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 3/09/99    Time: 6:23p
 * Updated in $/development/rs_udebug
 * bug fixing
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 3/01/99    Time: 8:42p
 * Updated in $/development/rs_udebug
 * Fix prototypes
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 2/25/99    Time: 8:52p
 * Updated in $/development/rs_udebug
 * Updated for the latest version of udebug related changes
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 2/23/99    Time: 9:26a
 * Created in $/development/rs_udebug
 * Add supports for multiple Core libraries
 */ 
 
#include <iostream.h>
#include <time.h> 

#include "rs_target.h"
#include "rs_connection.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";


/*-----------------------------------------------------------------------------
   Function: 	Open_EtherNet_Connection
   Description: Creates and initialise a target object and establishes an
                RPC connection with a remote target. The TargetName is an
                IP address or nodename of the remote debug server node.

   Returns:		Target object handle or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Open_EtherNet_Connection(char *TargetName, hRS_Conn_T *hTarget)
{
	RS_Target *TargetObj;
	
    if (RS_Target::GetObject() != NULL){
        *hTarget = 0;
        return RS_ERROR;  /* a connection is already open */
    }

    TargetObj = new RS_Target(TargetName);
	if(TargetObj->IsValid()){
		*hTarget = TargetObj;
		return RS_SUCCESS;
	}
	
    delete TargetObj;
	*hTarget = 0;
	
    return RS_ERROR;
}

/*-----------------------------------------------------------------------------
   Function: 	Open_Serial_Connection
   Description: Creates and initialize a target object and establishes a
                serial connection with a remote target. The TargetName is
                a COM port name (i.e. "com1") on the local node which is 
                connected via serial cable to the remote debug server node.

   Returns:		Target object handle or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Open_Serial_Connection(char *TargetName, hRS_Conn_T *hTarget)
{
	char name[20];
    RS_Target *TargetObj;

    if (RS_Target::GetObject() != NULL){
        *hTarget = 0;
        return RS_ERROR;  /* a connection is already open */
    }

    /* convert name to lower case and clip if necessary */
    for (int i = 0; (i < (sizeof(name) - 1)) && (TargetName[i] != '\0'); i++){
        name[i] = tolower(TargetName[i]);
    }
    name[i] = '\0';

    TargetObj = new RS_Target(name, HostComPort_1);
	if(TargetObj->IsValid()){
		*hTarget = TargetObj;
		return RS_SUCCESS;
	}
	
    delete TargetObj;
	*hTarget = 0;

    return RS_ERROR;
}

/*-----------------------------------------------------------------------------
   Function: 	Open_PCI_Connection
   Description: Creates and initialize a target object and establishes a
                PCI connection with a remote target. The TargetName is an
                integer device number string between "0" and "9" representing
                a unique remote debug target "node" on the PCI bus.

   Returns:		Target object handle or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Open_PCI_Connection(char *TargetName, hRS_Conn_T *hTarget)
{
	char name[20];
    RS_Target *TargetObj;

    if (RS_Target::GetObject() != NULL){
        *hTarget = 0;
        return RS_ERROR;  /* a connection is already open */
    }

    if (TargetName[0] < '0' || TargetName[0] > '9' || TargetName[1] != '\0'){
        *hTarget = 0;
        return RS_ERROR;  /* invalid device number */
    }

    /* generate device name */
    sprintf(name, "ixphpc%c0", TargetName[0]);

    TargetObj = new RS_Target(name, HostComPort_1);
	if(TargetObj->IsValid()){
		*hTarget = TargetObj;
		return RS_SUCCESS;
	}
	
    delete TargetObj;
	*hTarget = 0;
	
    return RS_ERROR;
}

/*-----------------------------------------------------------------------------
   Function: 	Open_VxWorks_Connection
   Description: Creates and initialise a target object and establishes a
                WTX connection with a remote VxWorks target. The TargetName is
                the IP address or nodename of the remote debug server node.

   Returns:		Target object handle or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Open_VxWorks_Connection(char *TargetName, hRS_Conn_T *hTarget)
{
#ifdef VXWORKS_SUPPORT
	RS_Target *TargetObj;

    if (RS_Target::GetObject() != NULL){
        *hTarget = 0;
        return RS_ERROR;  /* a connection is already open */
    }

    TargetObj = new RS_Target(TargetName, "VxWorks");
	if(TargetObj->IsValid()){
		*hTarget = TargetObj;
		return RS_SUCCESS;
	}

    delete TargetObj;
	*hTarget = 0;

    return RS_ERROR;
#else
	*hTarget = 0;
	return RS_ERROR;  // NO VXWORKS WTX support
#endif
}

/*-----------------------------------------------------------------------------
   Function: 	Close_EtherNet_Connection
   Description: Close an RPC connection with a remote target.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Close_EtherNet_Connection(hRS_Conn_T hTarget)
{
	RS_Target *TargetObj = (RS_Target *)hTarget;
    if(!RS_Target::ValidObject(TargetObj)) return RS_ERROR;
    delete TargetObj;
	return RS_SUCCESS;
}

/*-----------------------------------------------------------------------------
   Function: 	Close_Serial_Connection
   Description: Close a serial connection with a remote target.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Close_Serial_Connection(hRS_Conn_T hTarget)
{
	RS_Target *TargetObj = (RS_Target *)hTarget;
    if(!RS_Target::ValidObject(TargetObj)) return RS_ERROR;
    delete TargetObj;
	return RS_SUCCESS;
}

/*-----------------------------------------------------------------------------
   Function: 	Close_PCI_Connection
   Description: Close a PCI connection with a remote target.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Close_PCI_Connection(hRS_Conn_T hTarget)
{
	RS_Target *TargetObj = (RS_Target *)hTarget;
    if(!RS_Target::ValidObject(TargetObj)) return RS_ERROR;
    delete TargetObj;
	return RS_SUCCESS;
}

/*-----------------------------------------------------------------------------
   Function: 	Close_VXWORKS_Connection
   Description: Close a WTX connection with a remote VxWorks target.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_API Close_VxWorks_Connection(hRS_Conn_T hTarget)
{
#ifdef VXWORKS_SUPPORT
	RS_Target *TargetObj = (RS_Target *)hTarget;
    if(!RS_Target::ValidObject(TargetObj)) return RS_ERROR;
    delete TargetObj;
	return RS_SUCCESS;
#else
    return RS_ERROR;  // NO VXWORKS WTX support
#endif
}
