/* rs_target.cpp
 * Remote Services Debug Library - Debug Target class implementation
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Services Debug Library
 *  author: lwh,  2/19/99  Initial version
 *  revisions:
 *  $History: rs_target.cpp $
 * 
 * *****************  Version 32  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 31  *****************
 * User: Ewalker      Date: 9/26/00    Time: 8:26a
 * Updated in $/dev_1200/Share/rs_udebug
 * Better protection against invalid target object handle passing
 * 
 * *****************  Version 29  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 28  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:54
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 27  *****************
 * User: Ewalker      Date: 8/29/00    Time: 17:42
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 26  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:15
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 25  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:49p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 24  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:47p
 * Updated in $/Beta_4/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 21  *****************
 * User: Lhui         Date: 1/06/00    Time: 6:07p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port 
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 10/25/99   Time: 6:08p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix a showstopper bug SCR#770
 * 
 * *****************  Version 19  *****************
 * User: Lhui         Date: 10/15/99   Time: 5:51p
 * Updated in $/dev_1200/share/rs_udebug
 * Fix the problem with breakpoint callback after re-open the WB project
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 10/13/99   Time: 5:55p
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 4/22/99    Time: 5:41p
 * Updated in $/development/rs_udebug
 * Add a delay parameter to  rs_udebug/control  hop functions. 
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:08p
 * Updated in $/development/rs_udebug
 * Fix bugs in rs_uDbg_GetUengPCs and rs_uDbg_InitLib 
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 3/29/99    Time: 6:04p
 * Updated in $/development/rs_udebug
 * Add supports for debug control functions (start & hop)
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 3/23/99    Time: 6:37p
 * Updated in $/development/rs_udebug
 * Code cleanup and bug fixing
 * 
 * *****************  Version 11  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:48p
 * Updated in $/development/rs_udebug
 * update COPYRIGHT
 * 
 * *****************  Version 10  *****************
 * User: Qabuild      Date: 3/19/99    Time: 4:12p
 * Updated in $/development/rs_udebug
 * update makefile dependencies
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 3/19/99    Time: 1:17p
 * Updated in $/development/rs_udebug
 * Add supports for Scomm.lib
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 3/18/99    Time: 5:48p
 * Updated in $/development/rs_udebug
 * Add supports for SA1200 debug control functions
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 3/12/99    Time: 2:04p
 * Updated in $/development/rs_udebug
 * Create an  include file for external interface
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 3/09/99    Time: 6:23p
 * Updated in $/development/rs_udebug
 * bug fixing
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 3/01/99    Time: 8:42p
 * Updated in $/development/rs_udebug
 * Fix prototypes
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 2/25/99    Time: 8:52p
 * Updated in $/development/rs_udebug
 * Updated for the latest version of udebug related changes
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 2/23/99    Time: 9:26a
 * Created in $/development/rs_udebug
 * Add supports for multiple Core libraries
 */ 
 
#include <iostream.h>
#include <time.h> 

#ifdef WIN32
#ifdef VXWORKS_SUPPORT
#include <host.h> 
#include <wtx.h>
#endif
#include <OncRpc32.h>
#include <pmap_clnt.h>
#else
#include <rpc/rpc.h>
#endif

#include "scomm.h"
#include "rs_udbg.h"
#include "rs_udbg_rpc.h"
#include "rs_udbg_serial.h"
#include "rs_udbg_wtx.h"
#include "rs_udbg_lib.h"
#include "rs_target.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

extern "C" {
CLIENT *create_hClient(char *Server, int RemoteProg, int Ver, char *protocol);
void destroy_hClient(CLIENT *clnt);
}

RS_Target* RS_Target::TheTargetObj = NULL;

/*-----------------------------------------------------------------------------
   Function: 	RS_Target::RS_Target
   Description: RPC Target class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_Target::RS_Target(char *Server)
{
    /* delete the previous target if user forgot since only one is allowed at a time */
    if (TheTargetObj != NULL) delete TheTargetObj;

    TheTargetObj = this;

    valid = 0;
    
    TargetInit();
    
    config = RPC_Target;
    
    if (Server == NULL) return;

    /*
    * Create client "handle" used for calling SA1200Target on the
    * server designated on the command line. We tell the rpc package
    * to use the "tcp" protocol when contacting the server.
    */
    cl = (void *)create_hClient(Server, SA1200TARGET, SA1200VERSION, "tcp");
    if(cl == NULL) return;
    
    /* save host node name */
    if(gethostname(hostname, (sizeof(hostname) - 1)) != 0) return;
    hostname[sizeof(hostname) - 1] = '\0';
    
    valid = 1;

    return;
}

#ifdef VXWORKS_SUPPORT
/*-----------------------------------------------------------------------------
   Function: 	RS_Target::RS_Target
   Description: VxWorks WTX Target class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_Target::RS_Target(char *Server, char *RTOS)
{
    /* delete the previous target if user forgot since only one is allowed at a time */
    if (TheTargetObj != NULL) delete TheTargetObj;

    TheTargetObj = this;

    valid = 0;
    
    TargetInit();
    
    config = VxWorks_WTX_Target;
    
    if (Server == NULL) return;
    
    // Load the remote services debug client dll.
    //        CString WtxDllPath = theApp.GetDefaultDirPath() + "wtxapi.dll";
    m_hWtx_Dll = LoadLibrary("wtxapi.dll");
    if (m_hWtx_Dll == NULL) return;
    
    // Get the addresses of all the exported wtx functions that we use.
    FARPROC ProcAddr;
    ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxInitialize");
    if (ProcAddr == NULL) return;
    Fn_wtxInitialize = (T_wtxInitialize)ProcAddr;
    
    ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxToolAttach");
    if (ProcAddr == NULL) return;
    Fn_wtxToolAttach = (T_wtxToolAttach)ProcAddr;
    
    ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxToolDetach");
    if (ProcAddr == NULL) return;
    Fn_wtxToolDetach = (T_wtxToolDetach)ProcAddr;
    
    ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxTerminate");
    if (ProcAddr == NULL) return;
    Fn_wtxTerminate = (T_wtxTerminate)ProcAddr;
    
    ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxSymFind");
    if (ProcAddr == NULL) return;
    Fn_wtxSymFind = (T_wtxSymFind)ProcAddr;
    
    ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxResultFree");
    if (ProcAddr == NULL) return;
    Fn_wtxResultFree = (T_wtxResultFree)ProcAddr;
    
    if (Fn_wtxInitialize(&wtxh) != OK){
		wtxh = NULL;
		return;
	}
    
    /* attach to target server */ 
    if (Fn_wtxToolAttach(wtxh, Server, "wtxapp") != OK){
		Fn_wtxTerminate(wtxh);
		wtxh = NULL;
		return;
	}
    
    /* remote entry table init */
    if((RemoteEntryInit()) != RS_SUCCESS){
        Fn_wtxToolDetach(wtxh);
		Fn_wtxTerminate(wtxh);
		wtxh = NULL;
		return;
	}

    valid = 1;
    
    return;
}
#endif

/*-----------------------------------------------------------------------------
   Function: 	RS_Target::RS_Target
   Description: Serial Target class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/

RS_Target::RS_Target(char *DevName, int PortNum)
{
    int res, ComId;

    /* delete the previous target if user forgot since only one is allowed at a time */
    if (TheTargetObj != NULL) delete TheTargetObj;

    TheTargetObj = this;

    valid = 0;
    
    if (DevName == NULL) return;

    TargetInit();
    
    config = Serial_Target;
    
    /* save serial communication device name */
    strncpy(PortName, DevName, sizeof(PortName) - 1);
    PortName[sizeof(PortName) - 1] = '\0';

    /* reinstalling same device is no problem */
    /* any device installation error will be caught by SComm_Open */
    res = SComm_InstallDev(PortName);
    
    ComId = SComm_Open(PortName, PortNum);  
    if(ComId < 0) return;
    
    SvrComId = ComId;
    cl = &SvrComId;
    
    valid = 1;

    return;
}

RS_Target::~RS_Target()
{
    valid = 0;

	if(config == RPC_Target){
        if(rpc){
	       delete rpc;
           rpc = NULL;
        }

        if (cl != NULL){
            destroy_hClient((CLIENT *)cl);
            cl = NULL;
        }
    }
 
	else if(config == Serial_Target){
        if(serial){
			delete serial;
            serial = NULL;
        }

        if (cl != NULL){
			SComm_Close(SvrComId);
		}
	}

#ifdef VXWORKS_SUPPORT
    else if(config == VxWorks_WTX_Target){
        if(wtx){
           delete wtx;
           wtx = NULL;
        }

        if(wtxh){
            Fn_wtxToolDetach(wtxh);  /* detach from target server */ 
            Fn_wtxTerminate(wtxh);   /* terminate WTX session */ 
        }

        if(m_hWtx_Dll != NULL){
            FreeLibrary(m_hWtx_Dll);
        }        
    }
#endif

	TheTargetObj = NULL;

    return;
}

#ifdef VXWORKS_SUPPORT
int RS_Target::RemoteEntryInsert(unsigned int index, char *FuncName)
{
    WTX_SYMBOL *pSymbol;

    if ((pSymbol = Fn_wtxSymFind (GethWTX(), FuncName, NULL, 0, 0, 0)) == NULL)  
		return RS_ERROR;     
	Entry[index] = pSymbol->value;
	Fn_wtxResultFree (wtxh, pSymbol);
	return RS_SUCCESS;
}
#endif

#ifdef VXWORKS_SUPPORT
int RS_Target::RemoteEntryInit()
{

	if((RemoteEntryInsert(RS_UDBG_INITLIB,"uDbg_InitLib")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_DELELIB,"uDbg_DeleLib")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_RESETUENG,"uDbg_ResetUeng")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_STARTUENG,"uDbg_StartUeng")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_PAUSEUENG,"uDbg_PauseUeng")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_RESUMEUENG,"uDbg_ResumeUeng")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_ISUENGACTIVE,"uDbg_IsUengActive")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETACTIVEUENGS,"uDbg_GetActiveUengs")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETUTHREADPC,"uDbg_GetUthreadPC")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_SETUTHREADSPC,"uDbg_SetUthreadsPC")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_SETUENGPC,"uDbg_SetUengPCs")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETUENGPC,"uDbg_GetUengPCs")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETUWORD,"uDbg_GetUword")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_SETUWORD,"uDbg_SetUword")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETNUMUWORD,"uDbg_GetNumUword")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_SETNUMUWORD,"uDbg_SetNumUword")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_WRITEMEM32,"uDbg_WriteMem32")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_READMEM32,"uDbg_ReadMem32")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_WRITEMEM64,"uDbg_WriteMem64")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_READMEM64,"uDbg_ReadMem64")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_SETBREAKPOINT,"WTX_uDbg_SetBreakpoint")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_SHOWBREAKPOINTS,"uDbg_ShowBreakpoints")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_CLEARALLBREAKPOINTS,"uDbg_ClearAllBreakpoints")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_CLEARBREAKPOINT,"uDbg_ClearBreakpoint")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_SETCSR,"uDbg_SetCSR")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETCSR,"uDbg_GetCSR")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_PUTRELGPR,"uDbg_PutRelGPR")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETRELGPR,"uDbg_GetRelGPR")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_PUTRELSRAM_WRXFER,"uDbg_PutRelSRAM_WrXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_PUTRELSDRAM_WRXFER,"uDbg_PutRelSDRAM_WrXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_PUTRELSRAM_RDXFER,"uDbg_PutRelSRAM_RdXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_PUTRELSDRAM_RDXFER,"uDbg_PutRelSDRAM_RdXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETRELSRAM_WRXFER,"uDbg_GetRelSRAM_WrXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETRELSDRAM_WRXFER,"uDbg_GetRelSDRAM_WrXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETRELSRAM_RDXFER,"uDbg_GetRelSRAM_RdXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETRELSDRAM_RDXFER,"uDbg_GetRelSDRAM_RdXfer")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETUENGSTATUS,"uDbg_GetUengStatus")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_HOPUENG,"uDbg_HopUeng")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_CNTL_INITUOF,"CNTL_InitUof")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_CNTL_RESET,"CNTL_Reset")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_CNTL_PAUSE,"CNTL_Pause")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_CNTL_RESUME,"CNTL_Resume")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_CNTL_START,"CNTL_Start")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_CNTL_HOP,"CNTL_Hop")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_CLRRESETUENG,"uDbg_ClrResetUeng")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_DEFINEFREEUSTORE,"uDbg_DefineFreeUstore")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_UPDATEFREEUSTORE,"uDbg_UpdateFreeUstore")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_WRITEFIFO_TX,"uDbg_WriteFIFO_Tx")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_READFIFO_RX,"uDbg_ReadFIFO_Rx")) == RS_ERROR)
		return RS_ERROR;

	if((RemoteEntryInsert(RS_UDBG_GETSYSMEMSIZE,"uDbg_GetSysMemSize")) == RS_ERROR)
		return RS_ERROR;

	return RS_SUCCESS;

}
#endif


