/* rs_target.h
 * Remote Services Debug Library - Debug Target class definitions
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui 2/03/99
 * revisions:
 * $History: rs_target.h $
 * 
 * *****************  Version 33  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 31  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 30  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:54
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 29  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:15
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 28  *****************
 * User: Ewalker      Date: 8/07/00    Time: 9:39
 * Updated in $/dev_1200/Share/rs_udebug
 * Eliminated compile time bzero and bcopy macro redefinition warnings by
 * including host.h prior to oncrpc32.h.
 * 
 * *****************  Version 27  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:49p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 26  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:47p
 * Updated in $/Beta_4/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 24  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 23  *****************
 * User: Lhui         Date: 1/14/00    Time: 6:39p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * 
 * *****************  Version 22  *****************
 * User: Lhui         Date: 1/12/00    Time: 6:16p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * Increase buffer size
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 1/06/00    Time: 6:07p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port 
 * 
 * *****************  Version 19  *****************
 * User: Lhui         Date: 10/15/99   Time: 5:51p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix the problem with breakpoint callback after re-open the WB project
 * 
 * *****************  Version 16  *****************
 * User: Lhui         Date: 4/22/99    Time: 5:41p
 * Updated in $/development/rs_udebug
 * Add a delay parameter to  rs_udebug/control  hop functions. 
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:08p
 * Updated in $/development/rs_udebug
 * Fix bugs in rs_uDbg_GetUengPCs and rs_uDbg_InitLib 
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 4/02/99    Time: 2:47p
 * Updated in $/development/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 3/29/99    Time: 6:04p
 * Updated in $/development/rs_udebug
 * Add supports for debug control functions (start & hop)
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 3/23/99    Time: 6:37p
 * Updated in $/development/rs_udebug
 * Code cleanup and bug fixing
 * 
 * *****************  Version 11  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:48p
 * Updated in $/development/rs_udebug
 * update COPYRIGHT
 * 
 * *****************  Version 10  *****************
 * User: Qabuild      Date: 3/19/99    Time: 4:12p
 * Updated in $/development/rs_udebug
 * update makefile dependencies
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 3/19/99    Time: 1:17p
 * Updated in $/development/rs_udebug
 * Add supports for Scomm.lib
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 3/18/99    Time: 5:48p
 * Updated in $/development/rs_udebug
 * Add supports for SA1200 debug control functions
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 3/12/99    Time: 2:04p
 * Updated in $/development/rs_udebug
 * Create an  include file for external interface
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 3/09/99    Time: 6:23p
 * Updated in $/development/rs_udebug
 * bug fixing
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 3/01/99    Time: 8:42p
 * Updated in $/development/rs_udebug
 * Fix prototypes
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 2/25/99    Time: 8:52p
 * Updated in $/development/rs_udebug
 * Updated for the latest version of udebug related changes
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 2/23/99    Time: 9:26a
 * Created in $/development/rs_udebug
 * Add supports for multiple Core libraries
 */ 
#ifndef RS_TARGET_H
#define RS_TARGET_H	

#ifdef WIN32
#ifdef VXWORKS_SUPPORT
#include <host.h> 
#include <wtx.h>
#endif
#endif

#include "rs_udbg.h"
#include "rs_udbg_rpc.h"
#include "rs_udbg_serial.h"
#include "rs_udbg_wtx.h"

enum{
    RPC_Target = 0 			/* use RPC communication with target */
   ,Serial_Target			/* use Serial Communication with target */
#ifdef VXWORKS_SUPPORT
   ,VxWorks_WTX_Target      /* use VxWorks WTX communication with target */
#endif
};


#ifdef __cplusplus
class RS_Target{
public: 
   static ValidObject(RS_Target *obj){return(obj != NULL && obj == TheTargetObj);}
   static RS_Target *GetObject(void){return(TheTargetObj);}
   RS_Target(char *Server);                     /* RPC Target */
#ifdef VXWORKS_SUPPORT
   RS_Target(char *Server, char *RTOS);         /* WTX Target */
#endif
   RS_Target(char *DevName, int PortNum);       /* Serial Target */
   ~RS_Target();
   RS_Debug	*debug;
   RS_uDbg_SERIAL *serial;
   RS_uDbg_RPC *rpc;
#ifdef VXWORKS_SUPPORT
   RS_uDbg_WTX *wtx;
#endif
   char *GetHostName() { return &hostname[0]; };
   char *GetPortName() { return &PortName[0]; };
   void *GetClient()		    {return cl;};
   const int IsValid()			const{return valid;};
   const int GetConfig()		const{return config;};
#ifdef VXWORKS_SUPPORT
   const HWTX GethWTX()			const{return wtxh;};
   const HINSTANCE GethWtx_Dll()  const{return m_hWtx_Dll;}; 
#endif

   void TargetInit()			
   {
       debug = NULL;
       rpc = NULL;
       serial = NULL;

       cl = NULL;
       SvrComId = 0;

       hostname[0] = '\0';
       PortName[0] = '\0';

#ifdef VXWORKS_SUPPORT
       wtx = NULL;
       wtxh = 0;
       for (int i = 0; i < RS_MAX_FUNCTIONS; i++) Entry[i] = 0;
       m_hWtx_Dll = NULL;

       Fn_wtxInitialize = NULL;
       Fn_wtxSymFind = NULL;
       Fn_wtxResultFree = NULL;
       Fn_wtxToolAttach = NULL;
       Fn_wtxToolDetach = NULL;
       Fn_wtxTerminate = NULL;
#endif
   };
   
#ifdef VXWORKS_SUPPORT
   RemoteEntryInit();

   TGT_ADDR_T TargetGetEntry(int FunctionID)	
	{if ((FunctionID <= 0) || (FunctionID > RS_MAX_FUNCTIONS))
		 return 0;
	 else
		 return Entry[FunctionID];
	}; 

   RemoteEntryInsert(unsigned int index, char *FuncName); 
#endif

private:
   static RS_Target *TheTargetObj;  /* only a single target is allowed at a time*/
   int valid;
   int config;
   void *cl;  
   int SvrComId;
   char hostname[100];
   char PortName[50];
#ifdef VXWORKS_SUPPORT
   HWTX  wtxh;                   /* WTX API handle */ 
   TGT_ADDR_T		 Entry[RS_MAX_FUNCTIONS];
   HINSTANCE m_hWtx_Dll;

   typedef STATUS (__cdecl *T_wtxInitialize)(HWTX *);
   T_wtxInitialize Fn_wtxInitialize; 

   typedef WTX_SYMBOL* (__cdecl *T_wtxSymFind)(HWTX,  const char *, TGT_ADDR_T, BOOL32, UINT8, UINT8);
   T_wtxSymFind Fn_wtxSymFind; 

   typedef STATUS (__cdecl *T_wtxResultFree)(HWTX, void *);
   T_wtxResultFree Fn_wtxResultFree; 

   typedef STATUS (__cdecl *T_wtxToolAttach)(HWTX, const char *, const char *);
   T_wtxToolAttach Fn_wtxToolAttach; 

   typedef STATUS (__cdecl *T_wtxToolDetach)(HWTX);
   T_wtxToolDetach  Fn_wtxToolDetach; 

   typedef STATUS (__cdecl *T_wtxTerminate)(HWTX);
   T_wtxTerminate Fn_wtxTerminate; 
#endif 
};

#endif		/* __cplusplus */

#endif      /* RS_TARGET_H */



