/* rs_uDbg_events_clnt.c
 * Remote Services Debug Library - Implements RPC callback (event) stubs
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rs_uDbg_events_clnt.c $
 * 
 * *****************  Version 17  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 

#include <stdio.h>

#include "rs_uDbg_events.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

#ifdef __cplusplus
extern "C" {
#endif
int *rs_udbg_bkpt_ck(rs_uDbg_Bkpt_CK_T *argp, CLIENT *clnt);
CLIENT *create_ck_hClient(char *Server);
void destroy_ck_hClient(CLIENT *clnt);
#ifdef __cplusplus
}
#endif

/* Default timeout can be changed using clnt_control() */
static struct timeval CKTIMEOUT = { 0, 10 }; /* 10 microseconds */

int *
rs_udbg_bkpt_ck(argp, clnt)
	rs_uDbg_Bkpt_CK_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_uDbg_Bkpt_CK, xdr_rs_uDbg_Bkpt_CK_T, argp, xdr_void, &res, CKTIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

/* 
 * Function Name: create_ck_hClient
 * Description:   The function create a client handle which uses by the
 *                breakpoint callback
 */

CLIENT *create_ck_hClient(char *Server)
{
 	CLIENT *cl;

#ifdef WIN32
	OncRpc32Init();
#else
	rpcTaskInit();
#endif
	
	cl = clnt_create(Server, uDbg_CALLBK, SA1200VERSION, "tcp");

    if (cl == NULL) {
	/*
	 * Couldn't establish connection with server.
	 * Print error message and return.
	 */
	  clnt_pcreateerror(Server);
      return NULL;
	}
	return cl;
}

/* 
 * Function Name: destroy_ck_hClient
 * Description:   The function destroy a client handle which uses by the
 *                breakpoint callback
 */

void destroy_ck_hClient(CLIENT *clnt)
{
    if (clnt != NULL) clnt_destroy(clnt);
    return;
}
