/* rs_uDbg_events_svc.c
 * Remote Services Debug Library - RPC-based callback (event) services
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rs_uDbg_events_svc.c $
 * 
 * *****************  Version 17  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 15  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 
  
#include <stdio.h>

#include "OncRpc32.h"
#include <errno.h>

#include "utils.h"
#include "rs_udbg_events.h"

#ifdef __cplusplus
extern "C" {
#endif
void *rs_udbg_bkpt_ck_1(rs_uDbg_Bkpt_CK_T *argp);
#ifdef __cplusplus
}
#endif

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

static void udbg_callbk_1();
static SVCXPRT *transp_tcp = NULL;  /* only create transport once */

int Svc_Setup()
{
    svc_unregister(uDbg_CALLBK, SA1200VERSION);  /* in case previously registered */
	pmap_unset(uDbg_CALLBK, SA1200VERSION);

    if (transp_tcp == NULL){
        transp_tcp = svctcp_create(RPC_ANYSOCK, 0, 0);
        if (transp_tcp == NULL) return RS_ERROR;
    }
    
    if (!(svc_register(transp_tcp, uDbg_CALLBK, SA1200VERSION, udbg_callbk_1, IPPROTO_TCP)))
        return RS_ERROR;
    
    return RS_SUCCESS;
}

void DbgCK_main(int *shutdown_flag)
{
#ifdef FD_SETSIZE
	fd_set readfds;
#else
    int readfds;
#endif /* def FD_SETSIZE */

    struct timeval wait_per_try = { 0, 500000 };  /* wait for 500ms */

    for (;;) {
        /* check if thread termination was requested */
        if (*shutdown_flag) ExitThread(1);

        /* process services requests */
#ifdef FD_SETSIZE
		readfds = svc_fdset;
#else
		readfds = svc_fds;
#endif /* def FD_SETSIZE */

        switch (select(_rpc_dtablesize(), &readfds, NULL, NULL,
			       &wait_per_try)) {
		case -1:
			if (errno == EINTR) continue;
			return;  /* severe error occured */
		case 0:
			continue;
		default:
			svc_getreqset(&readfds);
		}
	}
}

static void
udbg_callbk_1(rqstp, transp)
	struct svc_req *rqstp;
	SVCXPRT *transp;
{
	union {
		rs_uDbg_Bkpt_CK_T rs_udbg_bkpt_ck_1_arg;
	} argument;
	char *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	char *(*local)();

	switch (rqstp->rq_proc) {
	case NULLPROC:
		(void)svc_sendreply(transp, xdr_void, (char *)NULL);
		return;

	case RS_uDbg_Bkpt_CK:
		xdr_argument = xdr_rs_uDbg_Bkpt_CK_T;
		xdr_result = xdr_void;
		local = (char *(*)()) rs_udbg_bkpt_ck_1;
		break;

	default:
		svcerr_noproc(transp);
		return;
	}
	bzero((char *)&argument, sizeof(argument));
	if (!svc_getargs(transp, xdr_argument, &argument)) {
		svcerr_decode(transp);
		return;
	}
	result = (*local)(&argument, rqstp);
	if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
		svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, xdr_argument, &argument)) {
		(void)fprintf(stderr, "unable to free arguments\n");
		ExitThread(1);
	}
}

