/* rs_udbg_TestCases.cpp
 * Remote Services Debug Library - Test case routines
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug library
 * author: Lai-Wah Hui 12/2/98
 * revisions:
 * $History: rs_udbg_TestCases.cpp $
 * 
 * *****************  Version 33  *****************
 * User: Ewalker      Date: 12/11/00   Time: 12:07p
 * Updated in $/dev_1200/Share/rs_udebug
 * Pause before trying to clear breakpoints
 * 
 * *****************  Version 32  *****************
 * User: Ewalker      Date: 12/11/00   Time: 11:34a
 * Updated in $/dev_1200/Share/rs_udebug
 * Added pause and clrreset so that program can be rerun more reliably
 * 
 * *****************  Version 31  *****************
 * User: Ewalker      Date: 11/02/00   Time: 9:03a
 * Updated in $/dev_1200/Share/rs_udebug
 * Fixed SCR/Bug #1222. The serial form of rs_udbg_resetueng was crashing
 * the  target. A typographical error present since pre-V1.0 was
 * corrected.
 * 
 * *****************  Version 30  *****************
 * User: Ewalker      Date: 10/30/00   Time: 11:49a
 * Updated in $/dev_1200/Share/rs_udebug
 * Added some tests and cleaned up some logging 
 * 
 * *****************  Version 28  *****************
 * User: Ewalker      Date: 9/20/00    Time: 8:02a
 * Updated in $/dev_1200/Share/rs_udebug
 * Added performance/throughput test to test cases
 * 
 * *****************  Version 27  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 26  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:16
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 24  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 23  *****************
 * User: Lhui         Date: 1/17/00    Time: 10:46a
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * 
 * *****************  Version 23  *****************
 * User: Lhui         Date: 1/17/00    Time: 10:45a
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 21  *****************
 * User: Lhui         Date: 10/04/99   Time: 4:42p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for the new dbg_reset prototype
 * 
 * *****************  Version 19  *****************
 * User: Lhui         Date: 6/28/99    Time: 7:33p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 6/25/99    Time: 7:40p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 17  *****************
 * User: Lhui         Date: 6/18/99    Time: 6:27p
 * Updated in $/dev_1200/Share/rs_udebug
 * Update Control API
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 4/22/99    Time: 5:40p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add a delay parameter to  rs_udebug/control  hop functions. 
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:10p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases for uDbg_InitLib and uDbg_GetUengPC
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 4/01/99    Time: 6:25p
 * Updated in $/development/qa-test/rsdbg_tests
 * Fix bugs
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 3/29/99    Time: 6:07p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 7  *****************
 * User: Qabuild      Date: 3/22/99    Time: 11:57a
 * Updated in $/development/qa-test/rsdbg_tests
 * 
 * *****************  Version 6  *****************
 * User: Qabuild      Date: 3/22/99    Time: 11:19a
 * Updated in $/development/qa-test/rsdbg_tests
 * Updated debug.h to debug_1200.h
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 3/19/99    Time: 1:21p
 * Updated in $/development/qa-test/rsdbg_tests
 * Update for the new debug SA1200 path
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 3/18/99    Time: 5:52p
 * Updated in $/development/qa-test/rsdbg_tests
 * change "udbg_lib.h" to "debug.h"
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 3/15/99    Time: 12:07p
 * Updated in $/development/qa-test/rsdbg_tests
 * Link with rs_udbg_client_dll
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 3/09/99    Time: 6:25p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 3/09/99    Time: 10:45a
 * Created in $/development/qa-test/rsdbg_tests
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 3/01/99    Time: 8:43p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 2/25/99    Time: 8:54p
 * Updated in $/development/qa-test/rsdbg_tests
 * Updated for the latest version of udebug related changes
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 2/22/99    Time: 7:54p
 * Updated in $/development/qa-test/rsdbg_tests
 * Updated for udebug related changes
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 1/28/99    Time: 6:31p
 * Created in $/development/qa-test/rsdbg_tests
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 12/10/98   Time: 11:42a
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 12/07/98   Time: 3:04p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add more test cases
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 12/03/98   Time: 10:18p
 * Updated in $/development/qa-test/rsdbg_tests
 * Add test cases
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 12/03/98   Time: 5:54p
 * Created in $/development/qa-test/rsdbg_tests
 * Init. check-in
 */

#include <stdio.h>
#include <iostream.h>
#include <string.h>
#include <assert.h>
#include <time.h>
#include "debug_1200.h"
#include "ueng.h"
#include "rs_udbg_lib.h"
#include "rs_udbg_test.h"
#include "rs_connection.h"

#define CTX0_SIG_EVENTS			0x44
#define CTX1_SIG_EVENTS			0x48
#define CTX2_SIG_EVENTS			0x4C
#define CTX3_SIG_EVENTS			0x50

#define UDEBUG_CTX_0			0x1
#define UDEBUG_CTX_1			0x2
#define UDEBUG_CTX_2			0x4
#define UDEBUG_CTX_3			0x8

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

/*
 * -----------------------------
 */
//void bkPtCallback_1(unsigned short breakpointId, unsigned char ueng, unsigned char ctx, void *usrData)
//{
//	printf("\n\n *****BREAK: BreakpointId=%d, Micro-engine=%d, Context %d stopped, usrData=%x\n",
//		breakpointId, ueng, ctx, *((unsigned int *)usrData));
//	return;
//}

extern "C" void cmdThread(void *);
void bkPtCallback(unsigned short breakpointId, unsigned char ctx, void *usrData);


uDbg_BkptArray_T BkptArray;
uDbg_Bkpt_T Bkpt;
unsigned int callbk_data; 
void bkPtCallback(unsigned short breakpointId, unsigned char ueng, unsigned char ctx, void *usrData)
{
	printf("\n\n@@@@@@ BREAKPOINT: BreakpointId=%d, Micro-engine=%d, Context %d stopped, usrData=%x @@@@@@\n",
		breakpointId, ueng, ctx, *((unsigned int *)usrData));
	fflush(stdout);
	return;
}

void prnData32(unsigned int *data, unsigned int cnt){
	unsigned int *ptr, i;
    for(i=0, ptr=data; i<cnt; i++, ptr++)
        printf("%s0x%08x", (i>0?"|":""), *ptr);
	printf("\n");
}

void lstBreak()
{
   uDbg_Bkpt_T *bp;
   for(int i = 0; i < BkptArray.len; i++){
      bp = &BkptArray.bkpts[i];
	  printf(" Breakpoint: id=%d, uEng=%d, activeCtxMask=0x%x, uAddr=0x%x\n",
	          bp->id, bp->uEng, (bp->ctx & 0xf), bp->uAddr);
   }
   return;
}



int stopat(hRS_Conn_T handle, int ueng, int activeCtxMask, int breakOnCtx, int upc)
{
   uDbg_CTX_T breakIf;
   int rc;
   int stpUengMask = 0x3f;			// stop all uengines
   callbk_data = 88;
   switch(breakOnCtx){
      case 0: breakIf = UDBG_CTX0; break;
      case 1: breakIf = UDBG_CTX1; break;
      case 2: breakIf = UDBG_CTX2; break;
      case 3: breakIf = UDBG_CTX3; break;
      default: breakIf = UDBG_CTXANY; break;
   }
   Bkpt.callback = bkPtCallback;
   Bkpt.uEng = (unsigned char)ueng;
   Bkpt.usrData = &callbk_data;
   Bkpt.ctx = (unsigned short)(activeCtxMask & 0xf);
   Bkpt.uAddr = (unsigned short)upc;
   Bkpt.breakIfCtx = breakIf;
   Bkpt.stpOnBrkUengMask = stpUengMask;
   printf("\nuDbg_SetBreakpoint: uEng=%d, ctxMask=0x%x, OnCtx=0x%x uAddr=0x%x\n", ueng, activeCtxMask, breakOnCtx, upc);
   rc = RS_uDbg_SetBreakpoint(handle,&Bkpt);
   return rc;
}

// showbreaks
int showbkpt(hRS_Conn_T handle, unsigned int uEngMask)
{
   int rc;   
   printf("\nuDbg_ShowBreakPoints: uEngMask=0x%x\n", uEngMask);
   BkptArray.len = 0;
   rc = RS_uDbg_ShowBreakpoints(handle, uEngMask, &BkptArray);
   if (rc == UDBG_SUCCESS) lstBreak();
   return rc;
}

int clrallbreaks(hRS_Conn_T handle, unsigned int uEngMask)
{
   printf("\nuDbg_ClearAllBreakPoints: uEngMask=0x%x\n", uEngMask);
   return RS_uDbg_ClearAllBreakpoints(handle, uEngMask);
}

int clrbreak(hRS_Conn_T handle, short id)
{
   printf("\nuDbg_ClearBreakPoint: id=%d\n", id);
   return RS_uDbg_ClearBreakpoint(handle, id);
}


// test start engine
int start(hRS_Conn_T handle, unsigned int uEngMask)
{
   printf("\nuDbg_StartUeng: uEngMask=0x%x\n", uEngMask);
   return RS_uDbg_StartUeng(handle, uEngMask);
}

int isActive(hRS_Conn_T handle, unsigned int uEngMask)
{
   int active;	
   printf("\nuDbg_IsUengActive: uEngMask=0x%x\n", uEngMask);
   active = RS_uDbg_IsUengActive(handle, uEngMask);
   printf("uDbg_IsUengActive returned 0x%x - ", active);	
   if (active == UDBG_UENGACT){
      printf("Uengine(s) are active\n");
	  return 1;
   }
   printf("Uengine(s) are NOT active\n");
   return(0);
}


int GetActive(hRS_Conn_T handle, unsigned int uEngMask)
{
   int active;
   printf("\nuDbg_GetActiveUengs: uEngMask=0x%x\n", uEngMask);
   active = RS_uDbg_GetActiveUengs(handle, uEngMask);
   printf("uDbg_GetActiveUengs returned 0x%x\n", active);	
   return(active);
}

int resume(hRS_Conn_T handle, unsigned int uEngMask)
{
   printf("\nuDbg_ResumeUeng: uEngMask=0x%x\n", uEngMask);
   return RS_uDbg_ResumeUeng(handle, uEngMask);
}

int pause(hRS_Conn_T handle, unsigned int uEngMask, unsigned int delay)
{
   printf("\nuDbg_PauseUeng: uEngMask=0x%x, delay=%d\n", uEngMask, delay);
   return RS_uDbg_PauseUeng(handle, uEngMask, delay);
}

int hopuEng(hRS_Conn_T handle, unsigned int uEngMask, unsigned int delay)
{
   printf("\nuDbg_HopUeng: uEngMask=0x%x, delay=%d\n", uEngMask, delay);
   return RS_uDbg_HopUeng(handle, &uEngMask, delay);
}

int CntlHop(hRS_Conn_T handle, unsigned int uEngMask, unsigned int portMask, unsigned int delay)
{
   printf("\nCNTL_Hop: uEngMask=0x%x, portMask=0x%x, delay=%d\n", uEngMask, portMask, delay);
   return RS_CNTL_Hop(handle, &uEngMask, portMask, delay);
}

int SetUthreadsPC(hRS_Conn_T handle, unsigned int uEngMask, unsigned int ctx, unsigned int upc) 
{
   printf("\nuDbg_SetUthreadsPC: uEngMask=0x%x, ctxMask=0x%x, upc=%d\n", uEngMask, ctx, upc);		 	
   return RS_uDbg_SetUthreadsPC(handle, uEngMask, ctx, upc);
}

int GetUthreadPC(hRS_Conn_T handle, unsigned char uEng, unsigned char ctx, unsigned int *upc) 
{
   int rc;
   printf("\nuDbg_GetUthreadPC: uEng=%d, ctx=%d\n", uEng, ctx);		 	
   rc = RS_uDbg_GetUthreadPC(handle, uEng, ctx, upc);
   if (rc == UDBG_SUCCESS)
       printf("uDbg_GetUthreadPC got upc=%d\n", *upc);
   return rc;			
}

int GetUengPCs(hRS_Conn_T handle, unsigned char uEng, unsigned int *upc) 
{
   int rc;
   printf("\nuDbg_GetUengPCs: uEng=%d\n", uEng);		
   rc = RS_uDbg_GetUengPCs(handle, uEng, upc);
   if (rc == UDBG_SUCCESS)
       printf("uDbg_GetUengPCs got upc=%d|%d|%d|%d\n", upc[0], upc[1], upc[2], upc[3]);
   return rc;
}

int SetUengPCs(hRS_Conn_T handle, unsigned char uEng, unsigned int *upc) 
{
   printf("\nuDbg_SetUengPCs: uEng=%d, upc=%d|%d|%d|%d\n", uEng, upc[0], upc[1], upc[2], upc[3]);		
   return RS_uDbg_SetUengPCs(handle, uEng, upc);
}

int readUword(hRS_Conn_T handle, unsigned char uEng, unsigned int uWordAddr, unsigned int *udata)
{
   int rc;
   printf("\nuDbg_GetUword: uEng=%d, addr=0x%x\n", uEng, uWordAddr);
   rc = RS_uDbg_GetUword(handle, uEng, uWordAddr, udata);
   if (rc == UDBG_SUCCESS)
       printf("uDbg_GetUword got data=0x%x\n", *udata);
   return rc;
}

int writeUword(hRS_Conn_T handle, unsigned int uEngMask, unsigned int uWordAddr, int udata)
{
   printf("\nuDbg_SetUword: uEngMask=0x%x, addr=0x%x, data=0x%08x\n", uEngMask, uWordAddr, udata);
   return RS_uDbg_SetUword(handle, uEngMask, uWordAddr, udata);
}


int readNumUword(hRS_Conn_T handle, unsigned char uEng, unsigned int uWordAddr, unsigned int *udata, int cnt)
{
   int rc;
   printf("\nuDbg_GetNumUword: uEng=%d, addr=0x%x, cnt=%d\n", uEng, uWordAddr, cnt);
   rc = RS_uDbg_GetNumUword(handle, uEng, uWordAddr, cnt, udata);
   if (rc == UDBG_SUCCESS){
       printf("uDbg_GetNumUword got data=");
       prnData32(udata, cnt);
   }
   return rc;    
}

int writeNumUword(hRS_Conn_T handle, unsigned int uEngMask, unsigned int uWordAddr, unsigned int *udata, int cnt)
{
   printf("\nuDbg_SetNumUword: uEngMask=0x%x, addr=0x%x, cnt=%d, data=", uEngMask, uWordAddr, cnt);
   prnData32(udata, cnt);
   return RS_uDbg_SetNumUword(handle, uEngMask, uWordAddr, (unsigned short)cnt, udata);
}

int readCsr(hRS_Conn_T handle, int csrAddr, unsigned int *buf)
{
   int rc;
   printf("\nuDbg_GetCSR: addr=0x%x\n", csrAddr);
   rc = RS_uDbg_GetCSR(handle, csrAddr, buf);
   if (rc == UDBG_SUCCESS)
       printf("uDbg_GetCSR got data=0x%08x\n", *buf);
   return rc;
}

int writeCsr(hRS_Conn_T handle, int csrAddr, unsigned int data)
{
   printf("\nuDbg_SetCSR: addr=0x%x, data=0x%08x\n", csrAddr, data);
   return RS_uDbg_SetCSR(handle, csrAddr, data);
}

int readMem32(hRS_Conn_T handle, int addr, unsigned int *buf, unsigned int len)
{
  int rc;
  printf("\nuDbg_ReadMem32: addr=0x%x, len=%d\n", addr, len);
  rc = RS_uDbg_ReadMem32(handle, addr, buf, len);
  if (rc == UDBG_SUCCESS){
      printf("uDbg_ReadMem32 got data=");
      prnData32(buf, len);
  }
  return rc;
}

int writeMem32(hRS_Conn_T handle, int addr, unsigned int *buf, unsigned int len)
{
  printf("\nuDbg_WriteMem32: addr=0x%x, len=%d, data=", addr, len);
  prnData32(buf, len);
  return RS_uDbg_WriteMem32(handle, addr, buf, len);
}

int readMem64(hRS_Conn_T handle, int addr, unsigned int *buf, unsigned int len)
{
  int rc;
  printf("\nuDbg_ReadMem64: addr=0x%x, len=%d\n", addr, len);
  rc = RS_uDbg_ReadMem64(handle, addr, buf, len);
  if (rc == UDBG_SUCCESS){
      printf("uDbg_ReadMem64 got data=");
      prnData32(buf, len*2);
  }
  return rc;
}

int writeMem64(hRS_Conn_T handle, int addr, unsigned int *buf, unsigned int len)
{
  printf("\nuDbg_WriteMem64: addr=0x%x, len=%d, data=", addr, len);
  prnData32(buf, len*2);
  return RS_uDbg_WriteMem64(handle, addr, buf, len);
}

int readMem64_t(hRS_Conn_T handle, int addr, unsigned int *buf, unsigned int len)
{
  int rc;
//  printf("\nuDbg_ReadMem64: addr=0x%x, len=%d\n", addr, len);
  rc = RS_uDbg_ReadMem64(handle, addr, buf, len);
//  if (rc == UDBG_SUCCESS){
//      printf("uDbg_ReadMem64 got data=");
//      prnData32(buf, len*2);
//  }
  return rc;
}

int writeMem64_t(hRS_Conn_T handle, int addr, unsigned int *buf, unsigned int len)
{
//  printf("\nuDbg_WriteMem64: addr=0x%x, len=%d, data=", addr, len);
//  prnData32(buf, len*2);
  return RS_uDbg_WriteMem64(handle, addr, buf, len);
}

int reset(hRS_Conn_T handle, unsigned int uEngMask)
{
   printf("\nuDbg_ResetUeng: uEngMask=0x%x\n", uEngMask);
   return RS_uDbg_ResetUeng(handle, uEngMask, 0);
}

int clrreset(hRS_Conn_T handle, unsigned int uEngMask)
{
   printf("\nuDbg_ClrResetUeng: uEngMask=0x%x\n", uEngMask);
   return RS_uDbg_ClrResetUeng(handle, uEngMask);
}

int writeRelGpr(uDbg_Bank_T aBank, hRS_Conn_T handle, unsigned char ueng, unsigned char ctx, 
                unsigned int regNum, unsigned int *data, int cnt)
{
   printf("\nuDbg_PutRelGpr: aBank=%d, uEng=%d, ctx=%d, regNum=0x%x, cnt=%d, data=",
         aBank, ueng, ctx, regNum, cnt);
   prnData32(data, cnt);
   return RS_uDbg_PutRelGPR(handle,ueng, ctx, aBank,regNum, data,cnt);
}

int readRelGpr(uDbg_Bank_T aBank, hRS_Conn_T handle, unsigned char ueng, unsigned char ctx, 
               unsigned int regNum, unsigned int *data, int cnt)
{
   int rc;
   printf("\nuDbg_GetRelGPR: aBank=%d, uEng=%d, ctx=%d, regNum=0x%x, cnt=%d\n",
         aBank, ueng, ctx, regNum, cnt);
   rc = RS_uDbg_GetRelGPR(handle, ueng, ctx, aBank, regNum, data, cnt);
   if (rc == UDBG_SUCCESS){
       printf("uDbg_GetRelGPR got data=");
       prnData32(data, cnt);
   }
   return rc;
}

int writeRelSramWx(hRS_Conn_T handle, unsigned char ueng, unsigned char ctx, 
                   unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   printf("\nuDbg_PutRelSRAM_WrXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d, data=",
          ueng, ctx, regNum, cnt);
   prnData32(data, cnt);
   return RS_uDbg_PutRelSRAM_WrXfer(handle, ueng, ctx, regNum, data, cnt);
}

int readRelSramWx(hRS_Conn_T handle, unsigned char ueng, unsigned char ctx, 
                  unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   int rc;	
   printf("\nuDbg_GetRelSRAM_WrXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d\n", ueng, ctx, regNum, cnt);
   rc = RS_uDbg_GetRelSRAM_WrXfer(handle, ueng, ctx, regNum, data, cnt);
   if (rc == UDBG_SUCCESS){
       printf("uDbg_GetRelSRAM_WrXfer got data=");
       prnData32(data, cnt);
   }
   return rc;
}

int writeRelSramRx(hRS_Conn_T handle, unsigned char ueng, unsigned char ctx, 
                   unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   printf("\nuDbg_PutRelSRAM_RdXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d, data=",
          ueng, ctx, regNum, cnt);
   prnData32(data, cnt);
   return RS_uDbg_PutRelSRAM_RdXfer(handle, ueng, ctx, regNum, data, cnt);
}

int readRelSramRx(hRS_Conn_T handle, unsigned char ueng, unsigned char ctx,
                  unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   int rc;	
   printf("\nuDbg_GetRelSRAM_RdXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d\n", ueng, ctx, regNum, cnt);
   rc = RS_uDbg_GetRelSRAM_RdXfer(handle, ueng, ctx, regNum, data,cnt);
   if (rc == UDBG_SUCCESS){
       printf("uDbg_GetRelSRAM_RdXfer got data=");
       prnData32(data, cnt);
   }
   return rc;
}

int writeRelSdramWx(hRS_Conn_T handle, unsigned char ueng,
                    unsigned char ctx, unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   printf("\nuDbg_PutRelSDRAM_WrXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d, data=",
          ueng, ctx, regNum, cnt);
   prnData32(data, cnt);
   return RS_uDbg_PutRelSDRAM_WrXfer(handle, ueng, ctx, regNum, data, cnt);
}

int readRelSdramWx(hRS_Conn_T handle, unsigned char ueng, unsigned char ctx,
                   unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   int rc;	
   printf("\nuDbg_GetRelSDRAM_WrXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d\n", ueng, ctx, regNum, cnt);
   rc = RS_uDbg_GetRelSDRAM_WrXfer(handle, ueng, ctx, regNum, data, cnt);
   if (rc == UDBG_SUCCESS){
       printf("uDbg_GetRelSDRAM_WrXfer got data=");
       prnData32(data, cnt);
   }
   return rc;
}

int writeRelSdramRx(hRS_Conn_T handle, unsigned char ueng, unsigned char ctx,
                    unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   printf("\nuDbg_PutRelSDRAM_RdXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d, data=",
          ueng, ctx, regNum, cnt);
   prnData32(data, cnt);
   return RS_uDbg_PutRelSDRAM_RdXfer(handle, ueng, ctx, regNum, data, cnt);
}

int readRelSdramRx(hRS_Conn_T handle, unsigned char ueng, unsigned char ctx,
                   unsigned int regNum, unsigned int *data, unsigned int cnt)
{
   int rc;	
   printf("\nuDbg_GetRelSDRAM_RdXfer: uEng=%d, ctx=%d, regNum=0x%x, cnt=%d\n", ueng, ctx, regNum, cnt);
   rc = RS_uDbg_GetRelSDRAM_RdXfer(handle, ueng, ctx, regNum, data,cnt);
   if (rc == UDBG_SUCCESS){
       printf("uDbg_GetRelSDRAM_RdXfer got data=");
       prnData32(data,cnt);
   }
   return rc;
}
//**********************************************
//
// Remote uDebug Test Cases
//
//
//
//*******************************************
//
// SRAM Read/Write Tests
//
//*******************************************
int RW_SRAM_test(hRS_Conn_T dbgH)
{

unsigned int wx_save_val, rx_save_val, set_val, get_val;
unsigned int cnt = 1;
int Ret;
int any_fail = FALSE;

printf("\nRead/Write SRAM tests start...\n");

// pause all
Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
} else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
}

printf("\nR/W SRAM_WrXfer testing start...\n");
// first read and save value
Ret = readRelSramWx(dbgH, 2, 1, 5, &wx_save_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetRelSRAM_WrXfer\n");
} else {
	printf("FAIL: uDbg_GetRelSRAM_WrXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

set_val = 0xCAFECAFE; 
// write sram read xfer reg
Ret = writeRelSramWx(dbgH, 2, 1, 5, &set_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelSRAM_WrXfer\n");
} else {
	printf("FAIL: uDbg_PutRelSRAM_WrXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readRelSramWx(dbgH, 2, 1, 5, &get_val, cnt);	
if ((Ret == UDBG_SUCCESS) && (get_val == set_val)) {
	printf("PASS: uDbg_GetRelSRAM_WrXfer\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetRelSRAM_WrXfer returned %d\n", Ret);
	else printf("FAIL: uDbg_GetRelSRAM_WrXfer mismatched\n");
    any_fail = TRUE;
}	

// restore original value
Ret = writeRelSramWx(dbgH, 2, 1, 5, &wx_save_val, cnt);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelSRAM_WrXfer\n");
} else {
	printf("FAIL: uDbg_PutRelSRAM_WrXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

printf("\nR/W SRAM_RdXfer testing start...\n");
// first read and save value
Ret = readRelSramRx(dbgH, 2, 1, 5, &rx_save_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetRelSRAM_RdXfer\n");
} else {
	printf("FAIL: uDbg_GetRelSRAM_RdXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

set_val =  0xABCDABCD; 
// write sram read xfer reg
Ret = writeRelSramRx(dbgH, 2, 1, 5, &set_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelSRAM_RdXfer\n");
} else {
	printf("FAIL: uDbg_PutRelSRAM_RdXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readRelSramRx(dbgH, 2, 1, 5, &get_val, cnt);	
if ((Ret == UDBG_SUCCESS) && (get_val == set_val)) {
	printf("PASS: uDbg_GetRelSRAM_RdXfer\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetRelSRAM_RdXfer returned %d\n", Ret);
	else printf("FAIL: uDbg_GetRelSRAM_RdXfer mismatched\n");
    any_fail = TRUE;
}	


// restore original value
Ret = writeRelSramRx(dbgH, 2, 1, 5, &rx_save_val, cnt);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelSRAM_RdXfer\n");
} else {
	printf("FAIL: uDbg_PutRelSRAM_RdXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = resume(dbgH, UDEBUG_UENG_ALL);

if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// now make sure that the routines fail when uengine is active

Ret = readRelSramWx(dbgH, 2, 1, 5, &get_val, 1);	
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_GetRelSRAM_WrXfer uengact test\n");
} else {
	printf("FAIL: uDbg_GetRelSRAM_WrXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = writeRelSramWx(dbgH, 2, 1, 5, &wx_save_val, 1);
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_PutRelSRAM_WrXfer uengact test\n");
} else {
	printf("FAIL: uDbg_PutRelSRAM_WrXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readRelSramRx(dbgH, 2, 1, 5, &get_val, 1);	
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_GetRelSRAM_RdXfer uengact test\n");
} else {
	printf("FAIL: uDbg_GetRelSRAM_RdXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = writeRelSramRx(dbgH, 2, 1, 5, &rx_save_val, 1);
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_PutRelSRAM_RdXfer uengact test\n");
} else {
	printf("FAIL: uDbg_PutRelSRAM_RdXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

 return (any_fail);
}

//*******************************************
//
// SDRAM Read/Write Tests
//
//*******************************************
int RW_SDRAM_test(hRS_Conn_T dbgH)
{

unsigned int wx_save_val, rx_save_val, set_val, get_val;
unsigned int  cnt = 1;
int i;
int Ret;
int any_fail = FALSE;
unsigned int command_buf[8];

printf("\nRead/Write SDRAM tests start...\n");

// pause all
Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
} else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
}

// first read and save value
Ret = readRelSdramWx(dbgH, 1, 0, 0, &wx_save_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetRelSDRAM_WrXfer\n");
} else {
	printf("FAIL: uDbg_GetRelSDRAM_WrXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readRelSdramRx(dbgH, 1, 0, 0, &rx_save_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetRelSDRAM_RdXfer\n");
} else {
	printf("FAIL: uDbg_GetRelSDRAM_RdXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

printf("\nR/W SDRAM_WrXfer testing start...\n");

for(i=0; i < 8; i++)
	command_buf[i] = 0xcc;

Ret = writeRelSdramWx(dbgH, 1, 0, 0, &command_buf[0], 8);	
if (Ret == UDBG_SUCCESS) {
 		printf("PASS: init Sdram\n");
} else {
 		printf("FAIL: init Sdram\n");
		any_fail = TRUE;
}

// write sdram read xfer reg
set_val = 0xCAFECAFE; 
Ret = writeRelSdramWx(dbgH, 1, 0, 0, &set_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelSDRAM_WrXfer\n");
} else {
	printf("FAIL: uDbg_PutRelSDRAM_WrXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readRelSdramWx(dbgH, 1, 0, 0, &get_val, cnt);	
if ((Ret == UDBG_SUCCESS) && (get_val == set_val)) {
	printf("PASS: uDbg_GetRelSDRAM_RdXfer\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetRelSDRAM_RdXfer returned %d\n", Ret);
	else printf("FAIL: uDbg_GetRelSDRAM_RdXfer mismatched\n");
    any_fail = TRUE;
}	


printf("\nR/W SDRAM_RdXfer testing start...\n");
// write sdram read xfer reg
set_val = 0xABCDABCD; 
Ret = writeRelSdramRx(dbgH, 1, 1, 6, &set_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetRelSDRAM_RdXfer\n");
} else {
	printf("FAIL: uDbg_GetRelDSRAM_RdXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readRelSdramRx(dbgH, 1, 1, 6, &get_val, cnt);	
if ((Ret == UDBG_SUCCESS) && (get_val == set_val)) {
	printf("PASS: uDbg_GetRelSDRAM_RdXfer\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetRelSDRAM_RdXfer returned %d\n", Ret);
	else printf("FAIL: uDbg_GetRelSDRAM_RdXfer mismatched\n");
    any_fail = TRUE;
}	

Ret = writeRelSdramWx(dbgH, 1, 0, 0, &wx_save_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelSDRAM_WrXfer\n");
} else {
	printf("FAIL: uDbg_PutRelSDRAM_WrXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = writeRelSdramRx(dbgH, 1, 0, 0, &rx_save_val, cnt);	
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelSDRAM_RdXfer\n");
} else {
	printf("FAIL: uDbg_PutRelSDRAM_RdXfer returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = resume(dbgH,UDEBUG_UENG_ALL);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// now make sure that the routines fail when uengine is active

Ret = readRelSdramWx(dbgH, 1, 0, 0, &get_val, 1);	
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_GetRelSDRAM_WrXfer uengact test\n");
} else {
	printf("FAIL: uDbg_GetRelSDRAM_WrXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = writeRelSdramWx(dbgH, 1, 0, 0, &wx_save_val, 1);	
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_PutRelSDRAM_WrXfer uengact test\n");
} else {
	printf("FAIL: uDbg_PutRelSDRAM_WrXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readRelSdramRx(dbgH, 1, 0, 0, &get_val, 1);	
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_GetRelSDRAM_RdXfer uengact test\n");
} else {
	printf("FAIL: uDbg_GetRelSDRAM_RdXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = writeRelSdramRx(dbgH, 1, 0, 0, &rx_save_val, 1);	
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_PutRelSDRAM_RdXfer uengact test\n");
} else {
	printf("FAIL: uDbg_PutRelSDRAM_RdXfer uengact test returned %d\n", Ret);
	any_fail = TRUE;
}	

 return (any_fail);
}

//*******************************************
//
// BreakPoints Tests
//
//*******************************************

int BreakPoints_test(hRS_Conn_T dbgH)
{
int Ret;
int any_fail = FALSE;

printf("\nBreakPoint tests start...\n");

Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
 } else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
 }

// uDbg_SetBreakPoint
Ret = stopat(dbgH, 2, UDBG_CTXALL, 0x4, 13);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetBreakPoint\n");
} else {
	printf("FAIL: uDbg_SetBreakPoint returned %d\n", Ret);
	any_fail = TRUE;
}

// uDbg_SetBreakPoint
Ret = stopat(dbgH, 3, UDBG_CTXALL, 0x4, 5);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetBreakPoint\n");
} else {
	printf("FAIL: uDbg_SetBreakPoint returned %d\n", Ret);
	any_fail = TRUE;
}


// uDbg_SetBreakPoint
Ret = stopat(dbgH, 4, UDBG_CTXALL, 0x4, 10);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetBreakPoint\n");
} else {
	printf("FAIL: uDbg_SetBreakPoint returned %d\n", Ret);
	any_fail = TRUE;
}

// uDbg_SetBreakPoint
// Same breakpoint - return = UDBG_DUPBKP
Ret = stopat(dbgH, 4, UDBG_CTXALL, 0x4, 10);
if (Ret == UDBG_DUPBKP) {
	printf("PASS: uDbg_SetBreakPoint duplicate\n");
} else {
	printf("FAIL: uDbg_SetBreakPoint returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = resume(dbgH, UDEBUG_UENG_ALL);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

Ret = showbkpt(dbgH, UDEBUG_UENG_ALL);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }

// show breakpoint for each microengine

Ret = showbkpt(dbgH, UDEBUG_UENG_0);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }


Ret = showbkpt(dbgH, UDEBUG_UENG_1);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }


Ret = showbkpt(dbgH, UDEBUG_UENG_2);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }


Ret = showbkpt(dbgH, UDEBUG_UENG_3);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }


Ret = showbkpt(dbgH, UDEBUG_UENG_4);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }


Ret = showbkpt(dbgH, UDEBUG_UENG_5);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }

Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
 } else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
 }

Ret = clrallbreaks(dbgH, UDEBUG_UENG_2);  // clears breakpoint number 0
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ClearAllBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ClearAllBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }

Ret = showbkpt(dbgH, UDEBUG_UENG_ALL);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }

Ret = clrbreak(dbgH, 2);  // clears breakpoint number 2
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ClearBreakPoint\n");
 } else {
 	printf("FAIL: uDbg_ClearBreakPoint returned %d\n", Ret);
 	any_fail = TRUE;
 }

 Ret = showbkpt(dbgH, UDEBUG_UENG_ALL);  // should be only one breakpoint left (number 1)
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }

Ret = clrallbreaks(dbgH, UDEBUG_UENG_ALL);  // clears all breakpoints
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ClearAllBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ClearAllBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }
 
Ret = resume(dbgH, UDEBUG_UENG_ALL);

if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// now make sure that set fails when uengine is active
Ret = stopat(dbgH, 5, UDBG_CTXALL, 0x4, 5);
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_SetBreakPoint uengact test\n");
} else {
	printf("FAIL: uDbg_SetBreakPoint uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

 Ret = showbkpt(dbgH, UDEBUG_UENG_ALL);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ShowBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ShowBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }


Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
 } else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
 }

Ret = clrallbreaks(dbgH, UDEBUG_UENG_ALL);  // clears all breakpoints
 if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ClearAllBreakPoints\n");
 } else {
 	printf("FAIL: uDbg_ClearAllBreakPoints returned %d\n", Ret);
 	any_fail = TRUE;
 }
 
Ret = resume(dbgH, UDEBUG_UENG_ALL);

if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

 return (any_fail);

}

//*******************************************
//
// R/W GPR Tests
//
//*******************************************

#define GPR_Cnt 4

// test R/W GPR
int RW_GPR_test(hRS_Conn_T dbgH)
{
unsigned int save_val[GPR_Cnt], set_val, get_val;
unsigned int get_buf[GPR_Cnt], set_buf[GPR_Cnt];
int i;
int Ret;
int any_fail = FALSE;


printf("\nRead/Write GPR tests start...\n");

// pause all
Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
} else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
}


Ret = readRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, save_val,GPR_Cnt);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetRelGPR\n");
} else {
	printf("FAIL: uDbg_GetRelGPR returned %d\n", Ret);
	any_fail = TRUE;
}

set_val = 0xffffffff;
Ret = writeRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, &set_val,1);

if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelGPR\n");
} else {
	printf("FAIL: uDbg_PutRelGPR returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, &get_val,1);
if ((Ret == UDBG_SUCCESS) && (get_val == set_val)) {
	printf("PASS: uDbg_GetRelGPR\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetRelGPR returned %d\n", Ret);
    else printf("FAIL: uDbg_GetRelGPR mismatched\n");
    any_fail = TRUE;
}


// restore the saved value
Ret = writeRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, &save_val[0],1);

if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelGPR\n");
} else {
	printf("FAIL: uDbg_PutRelGPR returned %d\n", Ret);
	any_fail = TRUE;
}

for (i=0; i < GPR_Cnt; i++)
	set_buf[i]= i;

// set multiple values
Ret = writeRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, set_buf, GPR_Cnt);

if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelGPR\n");
} else {
	printf("FAIL: uDbg_PutRelGPR returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, get_buf, GPR_Cnt);
if (Ret == UDBG_SUCCESS) {
	for(i=0; i < GPR_Cnt; i++){
		if(get_buf[i] != set_buf[i])
			break;
	}
	if (i < GPR_Cnt){
	 printf("FAIL: uDbg_GetRelGPR mismatched\n");
	 any_fail = TRUE;
	}
	else
 	  printf("PASS: uDbg_GetRelGPR\n");
} else {
	printf("FAIL: uDbg_GetRelGPR returned %d\n", Ret);
	any_fail = TRUE;
}

// restore values
Ret = writeRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, save_val, GPR_Cnt);

if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PutRelGPR\n");
} else {
	printf("FAIL: uDbg_PutRelGPR returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = resume(dbgH,UDEBUG_UENG_ALL);

if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
} else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// make sure that the routines fail when uengine is active

Ret = readRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, get_buf, GPR_Cnt);
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_GetRelGPR uengact test\n");
} else {
	printf("FAIL: uDbg_GetRelGPR uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = writeRelGpr(UDBG_GPR_A_BANK, dbgH, 0, 0, 0, save_val, GPR_Cnt);
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_PutRelGPR uengact test\n");
} else {
	printf("FAIL: uDbg_PutRelGPR uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

return (any_fail);
}


//*******************************************
//
// Get uEngStatus Tests
//
//*******************************************

int Get_uEngStatus_test(hRS_Conn_T dbgH)
{

int Ret;
int any_fail = FALSE;
uDbg_uEngStatus_T uEngStatus;

// pause all
printf("\nGet uEngStatus tests start...\n");

Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
} else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
}

Ret = RS_uDbg_GetUengStatus(dbgH, 0, &uEngStatus);
printf("\nuDbg_GetUengStatus: UENG_0 state=0x%x, ctx=0x%x, enables=0x%x, breakId=0x%x, pc=0x%x\n", 
  	uEngStatus.state, uEngStatus.ctx, uEngStatus.enables,
	uEngStatus.breakId,
	uEngStatus.pc);
if (Ret == UDBG_SUCCESS)
	printf("PASS: RS_uDbg_GetUengStatus\n");
else {
	printf("FAIL: RS_uDbg_GetUengStatus\n");
	any_fail = TRUE;
}

Ret = RS_uDbg_GetUengStatus(dbgH, 1, &uEngStatus);
printf("\nuDbg_GetUengStatus: UENG_1 state=0x%x, ctx=0x%x, enables=0x%x, breakId=0x%x, pc=0x%x\n", 
  	uEngStatus.state, uEngStatus.ctx, uEngStatus.enables,
	uEngStatus.breakId,
	uEngStatus.pc);
if (Ret == UDBG_SUCCESS)
	printf("PASS: RS_uDbg_GetUengStatus\n");
else {
	printf("FAIL: RS_uDbg_GetUengStatus\n");
	any_fail = TRUE;
}

Ret = RS_uDbg_GetUengStatus(dbgH, 2, &uEngStatus);
printf("\nuDbg_GetUengStatus: UENG_2 state=0x%x, ctx=0x%x, enables=0x%x, breakId=0x%x, pc=0x%x\n", 
  	uEngStatus.state, uEngStatus.ctx, uEngStatus.enables,
	uEngStatus.breakId,
	uEngStatus.pc);
if (Ret == UDBG_SUCCESS)
	printf("PASS: RS_uDbg_GetUengStatus\n");
else {
	printf("FAIL: RS_uDbg_GetUengStatus\n");
	any_fail = TRUE;
}

Ret = resume(dbgH,UDEBUG_UENG_ALL);

if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

return (any_fail);
}


//*******************************************
//
// Set/Get uWord Tests
//
//*******************************************
int Set_uWord_test(hRS_Conn_T dbgH)
{

int Ret;
unsigned int uData, save_uData;
int any_fail = FALSE;

printf("\nSet uWord tests start...\n");

// pause all
Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
//Ret = pause(dbgH, 0x1, 0xf);

if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
} else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
}


Ret = readUword(dbgH, 0, 1023, &save_uData);
if ((Ret == UDBG_SUCCESS) && (save_uData == 0xe00c0004))  // pattern from cliptest
    printf("PASS: uDbg_GetUWord\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUWord returned %d\n", Ret);
    else printf("FAIL: uDbg_GetUWord fill pattern mismatched\n");
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_0, 1023, 0xCAFECAFE);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readUword(dbgH, 0, 1023, &uData);
if ((Ret == UDBG_SUCCESS) && (uData == 0xcafecafe))
    printf("PASS: uDbg_GetUWord after write\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUWord after write returned %d\n", Ret);
	else printf("FAIL: uDbg_GetUWord after write mismatched\n");
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_0, 1023, save_uData);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readUword(dbgH, 0, 1022, &save_uData);
if ((Ret == UDBG_SUCCESS) && (save_uData == 0xe00c0004))  // pattern from cliptest
    printf("PASS: uDbg_GetUWord\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUWord returned %d\n", Ret);
    else printf("FAIL: uDbg_GetUWord fill pattern mismatched\n");
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_0, 1022, 0xCAFECAFE);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readUword(dbgH, 0, 1022, &uData);
if ((Ret == UDBG_SUCCESS) && (uData == 0xcafecafe))
    printf("PASS: uDbg_GetUWord after write\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUWord after write returned %d\n", Ret);
	else printf("FAIL: uDbg_GetUWord after write mismatched\n");
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_0, 1022, save_uData);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readUword(dbgH, 3, 0, &save_uData);
if (Ret == UDBG_SUCCESS)
    printf("PASS: uDbg_GetUWord\n");
else {
    printf("FAIL: uDbg_GetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_3, 0, 0xfaddadd0);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readUword(dbgH, 3, 0, &uData);
if ((Ret == UDBG_SUCCESS) && (uData == 0xfaddadd0))
    printf("PASS: uDbg_GetUWord after write\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUWord after write returned %d\n", Ret);
	else printf("FAIL: uDbg_GetUWord after write mismatched\n");
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_3, 0, save_uData);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readUword(dbgH, 3, 1, &save_uData);
if (Ret == UDBG_SUCCESS)
    printf("PASS: uDbg_GetUWord\n");
else {
    printf("FAIL: uDbg_GetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_3, 1, 0xcafecafe);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = readUword(dbgH, 3, 1, &uData);
if ((Ret == UDBG_SUCCESS) && (uData == 0xcafecafe))
    printf("PASS: uDbg_GetUWord after write\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUWord after write returned %d\n", Ret);
	else printf("FAIL: uDbg_GetUWord after write mismatched\n");
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_3, 1, save_uData);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUWord\n");
} else {
	printf("FAIL: uDbg_SetUWord returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = resume(dbgH,UDEBUG_UENG_ALL);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// make sure that the routines fail when uengine is active

Ret = readUword(dbgH, 3, 1, &uData);
if (Ret == UDBG_UENGACT)
    printf("PASS: uDbg_GetUWord uengact test\n");
else {
	printf("FAIL: uDbg_GetUWord uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = writeUword(dbgH, UDEBUG_UENG_3, 1, save_uData);
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_SetUWord uengact test\n");
} else {
	printf("FAIL: uDbg_SetUWord uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

return (any_fail);
}

//*******************************************
//
// Set/Get NumuWord Tests
//
//*******************************************

#define uWordCnt 30
// GetSet uWord tests
int NumuWord_test(hRS_Conn_T dbgH)
{

unsigned int databuf[uWordCnt], save_data[uWordCnt], i;
int Ret;
int any_fail = FALSE;

printf("\nSet/Get NumUWord tests start...\n");

// pause all
Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
} else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
}


// Read a  Uword
Ret = readNumUword(dbgH, 0, 1022, &save_data[0],1);
if ((Ret == UDBG_SUCCESS) && (save_data[0] == 0xe00c0004))  // pattern from cliptest
    printf("PASS: uDbg_GetNumUWord\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetNumUWord returned %d\n", Ret);
    else printf("FAIL: uDbg_GetNumUWord fill pattern mismatched\n");
	any_fail = TRUE;
}

/**** write one uWord ***/
databuf[0] = 0xCAFECAFE;
Ret = writeNumUword(dbgH, UDEBUG_UENG_0, 1022, &databuf[0],1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetNumUWord\n");
} else {
	printf("FAIL: uDbg_SetNumUWord returned %d\n", Ret);
	any_fail = TRUE;
}

// Read a  Uword
Ret = readNumUword(dbgH, 0, 1022, &databuf[1],1);
if ((Ret == UDBG_SUCCESS) && (databuf[0] == databuf[1]))
	printf("PASS: uDbg_GetNumUWord after write\n");
else {
    if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetNumUWord after write returned %d\n", Ret);
	else printf("FAIL: uDbg_GetNumUWord after write mismatched\n");
	any_fail = TRUE;
}

// restore original value
Ret = writeNumUword(dbgH, UDEBUG_UENG_0, 1022, &save_data[0],1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetNumUWord\n");
} else {
	printf("FAIL: uDbg_SetNumUWord returned %d\n", Ret);
	any_fail = TRUE;
}


// Read some Uwords
Ret = readNumUword(dbgH, 0, 0, &save_data[0],uWordCnt);
if (Ret == UDBG_SUCCESS){
	printf("PASS: uDbg_GetNumUWord\n");
} else {
	printf("FAIL: uDbg_GetNumUWord returned %d\n", Ret);
	any_fail = TRUE;
}

/* write some uWords *****/
for(i=0; i< uWordCnt; i++)
	databuf[i] = i;

Ret = writeNumUword(dbgH, UDEBUG_UENG_0, 0, &databuf[0],uWordCnt);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetNumUWord\n");
} else {
	printf("FAIL: uDbg_SetNumUWord returned %d\n", Ret);
	any_fail = TRUE;
}

memset(databuf, 0, sizeof(databuf));

// Read some Uwords
Ret = readNumUword(dbgH, 0, 0, &databuf[0], uWordCnt);
if ((Ret == UDBG_SUCCESS) && (databuf[uWordCnt-1] == uWordCnt-1)) {
	printf("PASS: uDbg_GetNumUWord after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetNumUWord after write returned %d\n", Ret);
    else printf("FAIL: uDbg_GetNumUWord after write mismatched\n");
	any_fail = TRUE;
}

// restore original values
Ret = writeNumUword(dbgH, UDEBUG_UENG_0, 0, &save_data[0], uWordCnt);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetNumUWord\n");
} else {
	printf("FAIL: uDbg_SetNumUWord returned %d\n", Ret);
	any_fail = TRUE;
}

 
Ret = resume(dbgH,UDEBUG_UENG_ALL);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// make sure that the routines fail when uengine is active

Ret = readNumUword(dbgH, 0, 0, &databuf[0], uWordCnt);
if (Ret == UDBG_UENGACT){
	printf("PASS: uDbg_GetNumUWord uengact test\n");
} else {
	printf("FAIL: uDbg_GetNumUWord uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = writeNumUword(dbgH, UDEBUG_UENG_0, 0, &save_data[0], uWordCnt);
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_SetNumUWord uengact test\n");
} else {
	printf("FAIL: uDbg_SetNumUWord uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

return (any_fail);
}

//*******************************************
//
// R/W Mem64 Tests
//
//*******************************************

int RW_mem64_test(hRS_Conn_T dbgH)
{
unsigned int buf[10];
int Ret;
int any_fail = FALSE;
printf("\nRead/Write mem64 tests start...\n");

Ret = readMem64(dbgH, UENG0_CSR_BASE_ADDR + CTX_ENABLES,&buf[0], 1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_readMem64\n");
} else {
	printf("FAIL: uDbg_readMem64 returned %d\n", Ret);
	any_fail = TRUE;
}	
buf[2]=0xA00;
buf[3]=0x00003000;
Ret = writeMem64(dbgH, UENG0_CSR_BASE_ADDR + CTX_ENABLES, &buf[2], 1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_WriteMem64\n");
} else {
	printf("FAIL: uDbg_WriteMem64 returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readMem64(dbgH, UENG0_CSR_BASE_ADDR + CTX_ENABLES, &buf[4], 1);
if ((Ret == UDBG_SUCCESS) && (buf[4] == buf[2]) && (buf[5] == buf[3])) {
	printf("PASS: uDbg_readMem64 after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_readMem64 after write returned %d\n", Ret);
    else printf("FAIL: uDbg_readMem64 after write mismatched\n");
	any_fail = TRUE;
}	

return (any_fail);
}
//*******************************************
//
// R/W Mem32 Tests
//
//*******************************************

int RW_mem32_test(hRS_Conn_T dbgH)
{
unsigned int buf[10];
int Ret;
int any_fail = FALSE;

printf("\nRead/Write Mem32 tests start...\n");

Ret = readMem32(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, &buf[0], 1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_readMem32\n");
} else {
	printf("FAIL: uDbg_readMem32 returned %d\n", Ret);
	any_fail = TRUE;
}	
buf[1]=0xAA;
Ret = writeMem32(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, &buf[1], 1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_WriteMem32\n");
} else {
	printf("FAIL: uDbg_WriteMem32 returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readMem32(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, &buf[2], 1);
if ((Ret == UDBG_SUCCESS) && (buf[2] == buf[1])) {
	printf("PASS: uDbg_readMem32 after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_readMem32 after write returned %d\n", Ret);
    else printf("FAIL: uDbg_readMem32 after write mismatched\n");
	any_fail = TRUE;
}	

Ret = writeMem32(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, &buf[0], 1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_WriteMem32\n");
} else {
	printf("FAIL: uDbg_WriteMem32 returned %d\n", Ret);
	any_fail = TRUE;
}	

 return (any_fail);
}
//*******************************************
//
// R/W CSR tests
//
//*******************************************
int RW_csr_test(hRS_Conn_T dbgH)
{ 
int Ret;
unsigned int buf;
unsigned int testdata = 0x58, save_data;
int any_fail = FALSE;
printf("\nRead/Write CSR tests start...\n");

// pause all
Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_PauseUeng\n");
} else {
 	printf("FAIL: uDbg_PauseUeng\n");
 	any_fail = TRUE;
}

Ret = readCsr(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, &save_data);
if (Ret == UDBG_SUCCESS){
	printf("PASS: uDbg_GetCSR\n");
} else {
	printf("FAIL: uDbg_GetCSR returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = writeCsr(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, testdata);
if (Ret == UDBG_SUCCESS){
	printf("PASS: uDbg_SetCSR\n");
} else {
	printf("FAIL: uDbg_SetCSR returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readCsr(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, &buf);
if ((Ret == UDBG_SUCCESS) && (buf == testdata)){
	printf("PASS: uDbg_GetCSR after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetCSR after write returned %d\n", Ret);
    else printf("FAIL: uDbg_GetCSR after write mismatched\n");
	any_fail = TRUE;
}	

Ret = writeCsr(dbgH, UENG0_CSR_BASE_ADDR + CTX0_SIG_EVENTS, save_data);
if (Ret == UDBG_SUCCESS){
	printf("PASS: uDbg_SetCSR\n");
} else {
	printf("FAIL: uDbg_SetCSR returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = resume(dbgH,UDEBUG_UENG_ALL);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

return (any_fail);
}

//*******************************************
//
// Control Engine (Resume/Pause/Start) Tests
//
//*******************************************
int ControlEngine_test(hRS_Conn_T dbgH)
{


int Ret;
int any_fail = FALSE;
printf("\nStart/Pause/Resume tests start...\n");


Ret = start(dbgH, UDEBUG_UENG_1);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_StartUeng\n");
} else {
	printf("FAIL: uDbg_StartUeng returned %d\n", Ret);
	any_fail = TRUE;
}

// are engines active?
printf("\nChecking pause and activity status...\n");

if (isActive(dbgH, UDEBUG_UENG_1)) {
	printf("PASS: Microengine 1 is active\n");
} else {
	printf("FAIL: Microengine 1 is stopped - should be running\n");
	any_fail = TRUE;
}

// pause engine
Ret = pause(dbgH, UDEBUG_UENG_1, 1200);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PauseUeng\n");
} else {
	printf("FAIL: uDbg_PauseUeng returned %d\n", Ret);
	any_fail = TRUE;
}

// Is this microengine paused?

if (isActive(dbgH, UDEBUG_UENG_1) == 0) {
	printf("PASS: Microengine 1 is stopped\n");
} else {
	printf("FAIL: Microengine 1 is running - should be stopped\n");
	any_fail = TRUE;
}

// Resume microengine from the current PC
printf("\nChecking resume and activity status...\n");
Ret = resume(dbgH, UDEBUG_UENG_1);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// Is this microengine now active?
if (isActive(dbgH, UDEBUG_UENG_1)) {
	printf("PASS: Microengine 1 is active\n");
} else {
	printf("FAIL: Microengine 1 is stopped - should be running\n");
	any_fail = TRUE;
}
// is UDEBUG_UENG_1 active?

if ((GetActive(dbgH,  UDEBUG_UENG_1)) == UDEBUG_UENG_1)
	printf("PASS: Microengine 1 is active\n");
 else {
	printf("FAIL: Microengine 1 is stopped - should be running\n");
	any_fail = TRUE;
}
 

return (any_fail);
}


//*******************************************
//
// Set/Get PC Tests
//
//*******************************************
int SG_PCtest(hRS_Conn_T dbgH)
{

int Ret;
int any_fail = FALSE;
unsigned int Thread_pc_sav;
unsigned int uEng_pc_sav[4];
unsigned int new_pc_array[4];
unsigned int upc;

printf("\nSet/Get PC & ThreadPC tests start...\n");

Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PauseUeng\n");
} else {
	printf("FAIL: uDbg_PauseUeng\n");
	any_fail = TRUE;
}


Ret = GetUthreadPC(dbgH, 1, 0, &Thread_pc_sav);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetUThreadPC\n");
} else {
	printf("FAIL: uDbg_GetUThreadPC returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = GetUengPCs(dbgH, 1,  &uEng_pc_sav[0]);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetUengPCs\n");
} else {
	printf("FAIL: uDbg_GetUengPCs returned %d\n", Ret);
	any_fail = TRUE;
}

// change PC values

Ret = SetUthreadsPC(dbgH, UDEBUG_UENG_1, UDEBUG_CTX_0, 8);	 
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUthreadsPC\n");
} else {
	printf("FAIL: uDbg_SetUthreadsPC returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = GetUthreadPC(dbgH, 1, 0, &upc);
if ((Ret == UDBG_SUCCESS) && (upc == 8)) {
	printf("PASS: uDbg_GetUThreadPC after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUThreadPC after write returned %d\n", Ret);
    else printf("FAIL: uDbg_GetUThreadPC after write mismatched\n");
	any_fail = TRUE;
}

Ret = GetUengPCs(dbgH, 1, new_pc_array);
if ((Ret == UDBG_SUCCESS) && (new_pc_array[0] == 8)) {
	printf("PASS: uDbg_GetUengPCs after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUengPCs after write returned %d\n", Ret);
    else printf("FAIL: uDbg_GetUengPCs after write mismatched\n"); 
	any_fail = TRUE;
}

new_pc_array[0]=2;
new_pc_array[1]=3;
new_pc_array[2]=4;
new_pc_array[3]=5;

Ret = SetUengPCs(dbgH, 1, new_pc_array);	 
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUengPCs\n");
} else {
	printf("FAIL: uDbg_SetPCs returned %d\n", Ret);
	any_fail = TRUE;
}

memset(new_pc_array, 0, sizeof(new_pc_array));

Ret = GetUengPCs(dbgH, 1,  new_pc_array);
if ((Ret == UDBG_SUCCESS) && (new_pc_array[0] == 2) && (new_pc_array[1] == 3) &&
                             (new_pc_array[2] == 4) && (new_pc_array[3] == 5))   {
	printf("PASS: uDbg_GetUengPCs after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUengPCs after write returned %d\n", Ret);
    else printf("FAIL: uDbg_GetUengPCs after write mismatched\n"); 
	any_fail = TRUE;
}

Ret = GetUthreadPC(dbgH, 1, 0, &upc);
if ((Ret == UDBG_SUCCESS) && (upc == 2)) {
	printf("PASS: uDbg_GetUThreadPC after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_GetUThreadPC after write returned %d\n", Ret);
    else printf("FAIL: uDbg_GetUThreadPC after write mismatched\n");
	any_fail = TRUE;
}

// restore values 	

Ret = SetUthreadsPC(dbgH, UDEBUG_UENG_1, UDEBUG_CTX_0, Thread_pc_sav);	 
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUthreadsPC\n");
} else {
	printf("FAIL: uDbg_SetUthreadsPC returned %d\n", Ret);
	any_fail = TRUE;
}


Ret = SetUengPCs(dbgH, 1, &uEng_pc_sav[0]);	 
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_SetUengPCs\n");
} else {
	printf("FAIL: uDbg_SetUengPCs returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = resume(dbgH, UDEBUG_UENG_ALL);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

// make sure sets fail when uengine is active

Thread_pc_sav = 3;  // should kill the thread based on the cliptest microcode
Ret = SetUthreadsPC(dbgH, UDEBUG_UENG_1, UDEBUG_CTX_0, Thread_pc_sav);	 
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_SetUthreadsPC uengact test\n");
} else {
	printf("FAIL: uDbg_SetUthreadsPC uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

uEng_pc_sav[0] = 3;  // should kill the thread based on the cliptest microcode
Ret = SetUengPCs(dbgH, 1, &uEng_pc_sav[0]);	 
if (Ret == UDBG_UENGACT) {
	printf("PASS: uDbg_SetUengPCs uengact test\n");
} else {
	printf("FAIL: uDbg_SetUengPCs uengact test returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_PauseUeng\n");
} else {
	printf("FAIL: uDbg_PauseUeng\n");
	any_fail = TRUE;
}

Ret = GetUthreadPC(dbgH, 1, 0, &Thread_pc_sav);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetUThreadPC\n");
} else {
	printf("FAIL: uDbg_GetUThreadPC returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = GetUengPCs(dbgH, 1,  &uEng_pc_sav[0]);
if (Ret == UDBG_SUCCESS) {
	printf("PASS: uDbg_GetUengPCs\n");
} else {
	printf("FAIL: uDbg_GetUengPCs returned %d\n", Ret);
	any_fail = TRUE;
}

Ret = resume(dbgH, UDEBUG_UENG_ALL);
if (Ret == UDBG_SUCCESS) {
 	printf("PASS: uDbg_ResumeUeng\n");
 } else {
 	printf("FAIL: uDbg_ResumeUeng\n");
 	any_fail = TRUE;
 }

return (any_fail);
}

//*******************************************
//
// Throughput Tests
//
//*******************************************

int Throughput_test(hRS_Conn_T dbgH)
{
unsigned int buf[10];
int Ret;
int any_fail = FALSE;
time_t startTime, endTime;

printf("\nThroughput test started...\n");

startTime = time(&startTime);

// pause all
//Ret = pause(dbgH, UDEBUG_UENG_ALL, 1200);
//if (Ret == UDBG_SUCCESS) {
// 	printf("PASS: uDbg_PauseUeng\n");
//} else {
// 	printf("FAIL: uDbg_PauseUeng\n");
// 	any_fail = TRUE;
//}

for (int i = 0;i < 1000; i++)
{
if (i % 100 == 0) printf("Completed %d\n", i);

Ret = readMem64_t(dbgH, UENG0_CSR_BASE_ADDR + CTX_ENABLES,&buf[0], 1);
if (Ret == UDBG_SUCCESS) {
//	printf("PASS: uDbg_readMem64\n");
} else {
	printf("FAIL: uDbg_readMem64 returned %d\n", Ret);
	any_fail = TRUE;
}	
buf[2]=0xA00;
buf[3]=0x00003000;
Ret = writeMem64_t(dbgH, UENG0_CSR_BASE_ADDR + CTX_ENABLES, &buf[2], 1);
if (Ret == UDBG_SUCCESS) {
//	printf("PASS: uDbg_WriteMem64\n");
} else {
	printf("FAIL: uDbg_WriteMem64 returned %d\n", Ret);
	any_fail = TRUE;
}	

Ret = readMem64_t(dbgH, UENG0_CSR_BASE_ADDR + CTX_ENABLES, &buf[4], 1);
if ((Ret == UDBG_SUCCESS) && (buf[4] == buf[2]) && (buf[5] == buf[3])) {
//	printf("PASS: uDbg_readMem64 after write\n");
} else {
	if (Ret != UDBG_SUCCESS) printf("FAIL: uDbg_readMem64 after write returned %d\n", Ret);
    else printf("FAIL: uDbg_readMem64 after write mismatched\n");
	any_fail = TRUE;
}	

}
//Ret = resume(dbgH,UDEBUG_UENG_ALL);

//if (Ret == UDBG_SUCCESS) {
// 	printf("PASS: uDbg_ResumeUeng\n");
// } else {
// 	printf("FAIL: uDbg_ResumeUeng\n");
// 	any_fail = TRUE;
// }

endTime = time(&endTime);

if (!any_fail) printf("Throughput test took %d seconds\n", endTime - startTime);

return (any_fail);
}
