/* rs_udbg_clib.cpp
 * Remote Services Debug Library - Implementation of client C entry points
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Services Debug Library
 *  author: lwh, Jan 27,99  Initial version
 *  revisions:
 *  $History: rs_udbg_clib.cpp $
 * 
 * *****************  Version 42  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 41  *****************
 * User: Ewalker      Date: 10/31/00   Time: 4:15p
 * Updated in $/dev_1200/Share/rs_udebug
 * Define segment 2 free ustore in RS_uDbg_InitLib using DefineFreeUstore
 * if uStoreFree was passed
 * 
 * *****************  Version 40  *****************
 * User: Ewalker      Date: 10/31/00   Time: 12:11p
 * Updated in $/dev_1200/share/rs_udebug
 * C0 2k ustore support - made InitLib backwards compatible, supporting
 * only segment 1 settings. Apps must use DefineFreeUstore to set segment
 * 2 values if UpdateFreeUstore is not used.  
 * 
 * *****************  Version 39  *****************
 * User: Ewalker      Date: 9/26/00    Time: 8:32a
 * Updated in $/dev_1200/share/rs_udebug
 * 
 * *****************  Version 38  *****************
 * User: Ewalker      Date: 9/25/00    Time: 8:30a
 * Updated in $/dev_1200/Share/rs_udebug
 * Better protection against Workbench passing bad target object handle
 * 
 * *****************  Version 36  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 35  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:54
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 34  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:15
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 31  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 29  *****************
 * User: Lhui         Date: 1/12/00    Time: 4:14p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 28  *****************
 * User: Lhui         Date: 1/06/00    Time: 6:15p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port
 * 
 * *****************  Version 27  *****************
 * User: Lhui         Date: 10/19/99   Time: 5:16p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for uDbg_GetSysMemSize()
 * 
 * *****************  Version 26  *****************
 * User: Lhui         Date: 10/13/99   Time: 5:55p
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup
 * 
 * *****************  Version 25  *****************
 * User: Lhui         Date: 10/06/99   Time: 4:21p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for new fifo read/write functions
 * 
 * *****************  Version 24  *****************
 * User: Lhui         Date: 10/04/99   Time: 4:32p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for the enhanced dbg_reset API
 * 
 * *****************  Version 23  *****************
 * User: Rdmurato     Date: 9/30/99    Time: 12:52p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add stubs for new fifo read/write functions
 * 
 * *****************  Version 21  *****************
 * User: Lhui         Date: 7/16/99    Time: 4:15p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for uDbg_UpdateFreeUstore & uDbg_DefineFreeUstore
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 6/29/99    Time: 7:17p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for uDbg_ClrReset
 * 
 * *****************  Version 19  *****************
 * User: Lhui         Date: 6/18/99    Time: 6:27p
 * Updated in $/dev_1200/Share/rs_udebug
 * Update Control API
 * 
 * *****************  Version 17  *****************
 * User: Lhui         Date: 4/22/99    Time: 5:42p
 * Updated in $/development/rs_udebug
 * Add a delay parameter to  rs_udebug/control  hop functions. 
 * 
 * *****************  Version 16  *****************
 * User: Lhui         Date: 4/08/99    Time: 5:51p
 * Updated in $/development/rs_udebug
 * Change rs_cntl_pause prototypes
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:08p
 * Updated in $/development/rs_udebug
 * Fix bugs in rs_uDbg_GetUengPCs and rs_uDbg_InitLib 
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 3/29/99    Time: 6:04p
 * Updated in $/development/rs_udebug
 * Add supports for debug control functions (start & hop)
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 3/23/99    Time: 6:37p
 * Updated in $/development/rs_udebug
 * Code cleanup and bug fixing
 * 
 * *****************  Version 12  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:48p
 * Updated in $/development/rs_udebug
 * update COPYRIGHT
 * 
 * *****************  Version 11  *****************
 * User: Qabuild      Date: 3/19/99    Time: 4:12p
 * Updated in $/development/rs_udebug
 * update makefile dependencies
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 3/19/99    Time: 1:17p
 * Updated in $/development/rs_udebug
 * Add supports for Scomm.lib
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 3/18/99    Time: 5:48p
 * Updated in $/development/rs_udebug
 * Add supports for SA1200 debug control functions
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 3/15/99    Time: 3:52p
 * Updated in $/development/rs_udebug
 * Increase the timeout value 
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 3/15/99    Time: 12:06p
 * Updated in $/development/rs_udebug
 * Create a DLL version of rs_udbg_client
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 3/12/99    Time: 2:04p
 * Updated in $/development/rs_udebug
 * Create an  include file for external interface
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 3/09/99    Time: 6:23p
 * Updated in $/development/rs_udebug
 * bug fixing
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 3/01/99    Time: 8:42p
 * Updated in $/development/rs_udebug
 * Fix prototypes
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 2/25/99    Time: 8:52p
 * Updated in $/development/rs_udebug
 * Updated for the latest version of udebug related changes
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 2/22/99    Time: 7:51p
 * Updated in $/development/rs_udebug
 * Add supports for multiple Core components
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 1/28/99    Time: 6:24p
 * Created in $/development/rs_udebug
 * Add supports for "C" interface and new udebug functions
 *
 */ 

#include <stdio.h>

#include "rs_target.h"
#include "rs_connection.h"
#include "rs_udbg.h"
#include "rs_udebug.h"
#include "rs_udbg_rpc.h"
#include "rs_udbg_serial.h"
#include "rs_udbg_wtx.h"
#include "rs_udbg_lib.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

extern "C" {
void *rs_udbg_bkpt_ck_1(rs_uDbg_Bkpt_CK_T *argp);
}

/*-----------------------------------------------------------------------------
   Function: 	rs_udbg_bkpt_ck_1
   Description: Invokes callback for this break
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/

void *rs_udbg_bkpt_ck_1(rs_uDbg_Bkpt_CK_T *argp)
{
    RS_Target *targetObj = RS_Target::GetObject();  // lookup the single target object

    if (targetObj && targetObj->IsValid() &&
        targetObj->debug && targetObj->debug->IsValid()){
		if (targetObj->debug->GetCkRtn(argp->bkptId))
			targetObj->debug->GetCkRtn(argp->bkptId)->callback(argp->bkptId, argp->ueng, argp->ctx, (void *)argp->usrData); 
	}
	return NULL;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_InitLib
   Description: Creates and initialise a Debug library object.

   Returns:		Debug library object handle or NULL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_InitLib(hRS_Conn_T hTarget, uDbg_uStoreFree_T *uStoreFree)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
    unsigned int i;
	if(!RS_Target::ValidObject(targetObj)) return(UDBG_BADARG);
    if (targetObj->debug) return(UDBG_BADARG);
	void *hCl = targetObj->GetClient();
	switch (targetObj->GetConfig()){
	case RPC_Target:
		targetObj->rpc = new RS_uDbg_RPC(hCl, targetObj->GetConfig(), targetObj->GetHostName(), uStoreFree);
		targetObj->debug = targetObj->rpc;
		break;
	case Serial_Target: 	
		targetObj->serial = new RS_uDbg_SERIAL(hCl, targetObj->GetConfig(), targetObj->GetPortName(), uStoreFree);
		targetObj->debug = targetObj->serial;
		break;
#ifdef VXWORKS_SUPPORT
    case VxWorks_WTX_Target:
		hCl= targetObj->GethWTX();
		targetObj->wtx = new RS_uDbg_WTX(hCl, targetObj->GetConfig(), targetObj->GethWtx_Dll());
		targetObj->debug = targetObj->wtx;
		break;
#endif
	default: return UDBG_FAIL;  // unsupported connection/config type
	}

    if(targetObj->debug->IsValid()){
		targetObj->debug->SetMyTarget((void *)targetObj);
#ifdef VXWORKS_SUPPORT
        if(targetObj->wtx != NULL){
		    if ((targetObj->wtx->Udbg_InitLib(uStoreFree)) == RS_SUCCESS)
				return UDBG_SUCCESS;
		}
#else  // no VXWORKS WTX support
        if (0){
        }
#endif
        else{ // not WTX
            // define segment 2 free ustore if uStoreFree was passed
            if (uStoreFree != NULL){
                for(i = 0; i < MAX_UENGS; i++){
                    if ((targetObj->debug->DefineFreeUstore(((unsigned int)(1 << i)), 
                                                            uStoreFree->seg2FreeUpc[i],
                                                            uStoreFree->seg2FreeSize[i])) != RS_SUCCESS) break;
                }

                if (i == MAX_UENGS) return UDBG_SUCCESS;  // all were defined successfully
            }

            else return UDBG_SUCCESS;  // uStoreFree was not passed
        }
	}

    // debug object is not valid

    if(targetObj->rpc)
		delete targetObj->rpc;
	targetObj->rpc = NULL;

    if(targetObj->serial)
		delete targetObj->serial;
	targetObj->serial = NULL;

#ifdef VXWORKS_SUPPORT
	if(targetObj->wtx)
		delete targetObj->wtx;
	targetObj->wtx = NULL;
#endif

    targetObj->debug = NULL;

    return UDBG_FAIL;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_DeleLib
   Description: Deletes a Debug library object.

   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_DeleLib(hRS_Conn_T hTarget)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);

    if(targetObj->rpc)
		delete targetObj->rpc;
	targetObj->rpc = NULL;

    if(targetObj->serial)
		delete targetObj->serial;
	targetObj->serial = NULL;

#ifdef VXWORKS_SUPPORT
	if(targetObj->wtx)
		delete targetObj->wtx;
	targetObj->wtx = NULL;
#endif

    targetObj->debug = NULL;

	return UDBG_SUCCESS;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_UpdateFreeUstore
   Description: Update the micro-store free regions with values from the 
				uengine-driver.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_UpdateFreeUstore(hRS_Conn_T hTarget, unsigned int uEngMask)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->UpdateFreeUstore(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_DefineFreeUstore
   Description: Define the micro-store free region for the specified micro-engines.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_DefineFreeUstore(hRS_Conn_T hTarget, unsigned int uEngMask, 
						            unsigned int freeUaddr, unsigned int freeSize)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->DefineFreeUstore(uEngMask, freeUaddr, freeSize));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ResetUeng
   Description: Resets the micro-engine specified in uengMask.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ResetUeng(hRS_Conn_T hTarget, unsigned int uEngMask, int clrReg)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ResetUeng(uEngMask, clrReg));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ClrResetUeng
   Description: Resets the micro-engine specified in uengMask.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ClrResetUeng(hRS_Conn_T hTarget, unsigned int uEngMask)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ClrResetUeng(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_StartUeng
   Description: Starts all threads of the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_StartUeng(hRS_Conn_T hTarget, unsigned int uEngMask)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->StartUeng(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_PauseUeng
   Description: Pause the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_PauseUeng(hRS_Conn_T hTarget, unsigned int uEngMask, unsigned int delay)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->PauseUeng(uEngMask, delay));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ResumeUeng
   Description: Resume the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ResumeUeng(hRS_Conn_T hTarget, unsigned int uEngMask)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ResumeUeng(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_IsUengActive
   Description: Determine if any of the selected micro-engines are active.

   Returns:		UDBG_UENGACT, UDBG_UENGINACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_IsUengActive(hRS_Conn_T hTarget, unsigned int uEngMask)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->IsUengActive(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	uDbg_GetActiveUengs
   Description: Determine which of the specified micro-engines are active.

   Returns:		A mask of the active micro-engines
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetActiveUengs(hRS_Conn_T hTarget, unsigned int uEngMask)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetActiveUengs(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_SetUthreadsPC
   Description: Set the program-counter of the selected micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_SetUthreadsPC(hRS_Conn_T hTarget, unsigned int uEngMask, 
                                 unsigned int ctxMask, unsigned int upc)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->SetUthreadsPC(uEngMask, ctxMask, upc));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetUthreadPC
    Description: Get the program-counters of the specified micro-engine.

   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetUthreadPC(hRS_Conn_T hTarget, unsigned char uEng, 
                                unsigned char ctx, unsigned int *upc)

{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetUthreadPC(uEng, ctx, upc));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_SetUword
   Description: Write uWord to a micro-store address.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_SetUword(hRS_Conn_T hTarget, unsigned int uEngMask, 
                            unsigned int uWordAddr, unsigned int uWord)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->SetUword(uEngMask, uWordAddr, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetUword
   Description: Read uWord from a micro-store location.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetUword(hRS_Conn_T hTarget, unsigned char uEng, 
                            unsigned int uWordAddr, unsigned int *uWord)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetUword(uEng, uWordAddr, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_SetNumUword
   Description: Write a number of uWords to micro-store.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_SetNumUword(hRS_Conn_T hTarget, unsigned int uEngMask,
                               unsigned int uWordAddr, unsigned short numWords, 
                               unsigned int *uWord)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->SetNumUword(uEngMask, uWordAddr, numWords, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetNumUword
   Description: Read a number of uWords from micro-store.

   Returns:		UDBG_SUCCESS, UDBG_UENGACT, UDBG_BADLIB, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetNumUword(hRS_Conn_T hTarget, unsigned char uEng, 
                               unsigned int uWordAddr, unsigned short numWords, 
                               unsigned int *uWord)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetNumUword(uEng, uWordAddr, numWords, uWord));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ReadMem32
   Description: Read a long-word from a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ReadMem32(hRS_Conn_T hTarget, unsigned int addr, unsigned int *data,
							 unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ReadMem32(addr, data, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WriteMem32
   Description: Write a quad-word to a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_WriteMem32(hRS_Conn_T hTarget, unsigned int addr, unsigned int *data,
							  unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->WriteMem32(addr, data, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WriteMem64
   Description: Write a quad-word to a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_WriteMem64(hRS_Conn_T hTarget, unsigned int addr, unsigned int *data,
							  unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->WriteMem64(addr, data, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ReadMem64
   Description: Read a quad-word from a memory.

   Returns:		UDBG_SUCCESS, UDBG_BADADDR,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ReadMem64(hRS_Conn_T hTarget, unsigned int addr, unsigned int *data,
							 unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ReadMem64(addr, data, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_SetBreakpoint
   Description: Set breakpoint in the specified micro-engines.

   Returns:		UDBG_SUCCESS, UDBG_NOUSTORE,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_SetBreakpoint(hRS_Conn_T hTarget, uDbg_Bkpt_T *breakpoint)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->SetBreakpoint(breakpoint));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ShowBreakpoints
   Description: Show all breakpoints.

   Returns:		UDBG_SUCCESS,UDBG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ShowBreakpoints(hRS_Conn_T hTarget, unsigned int uEngMask,
                                   uDbg_BkptArray_T *breakpoints)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ShowBreakpoints(uEngMask, breakpoints));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ClearBreakpoint
   Description: Remove the breakpoint associated with breakpointId.

   Returns:		UDBG_SUCCESS,UDBG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ClearBreakpoint(hRS_Conn_T hTarget, short breakpointId)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ClearBreakpoint(breakpointId));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ClearAllBreakpoints
   Description: Remove all breakpoints in the specified micro-engines.

   Returns:		UDBG_SUCCESS,UDBG_FAIL
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_ClearAllBreakpoints(hRS_Conn_T hTarget, unsigned int uEngMask)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ClearAllBreakpoints(uEngMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_SetCSR
   Description: Write a long-word to a CSR.

   Returns:		UDBG_SUCCESS,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_SetCSR(hRS_Conn_T hTarget, unsigned int csr, unsigned int data)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	targetObj->debug->SetCSR(csr, data);
	return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetCSR
   Description: Read a long-word from a CSR.

   Returns:		UDBG_SUCCESS,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetCSR(hRS_Conn_T hTarget, unsigned int csr, unsigned int *data)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	targetObj->debug->GetCSR(csr, data);
	return(UDBG_SUCCESS);
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_PutRelGPR
   Description: Write a long-word to a relative GPR.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_PutRelGPR(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
							 uDbg_Bank_T bankType, unsigned int regNum, 
							 unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->PutRelGPR(uEng, ctx, bankType, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetRelGPR
   Description: Read a long-word from a relative GPR.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetRelGPR(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
							 uDbg_Bank_T bankType, unsigned int regNum, 
							 unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetRelGPR(uEng, ctx, bankType, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_PutRelSRAM_WrXfer
   Description: Write a long-word to a relative SRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_PutRelSRAM_WrXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
									 unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->PutRelSRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetRelSRAM_WrXfer
   Description: Read a long-word from a relative SRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetRelSRAM_WrXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
									 unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetRelSRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_PutRelSRAM_RdXfer
   Description: Write a long-word to a relative SRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_PutRelSRAM_RdXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
									 unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->PutRelSRAM_RdXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetRelSRAM_RdXfer
   Description: Read a long-word from a relative SRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetRelSRAM_RdXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
									 unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetRelSRAM_RdXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_PutRelSDRAM_WrXfer
   Description: Write a quad-word to a relative SDRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_PutRelSDRAM_WrXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
									  unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->PutRelSDRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetRelSDRAM_WrXfer
   Description: Read a quad-word from a relative SDRAM write-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetRelSDRAM_WrXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx,
									  unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetRelSDRAM_WrXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_PutRelSDRAM_RdXfer
   Description: Write a quad-word to a relative SDRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_PutRelSDRAM_RdXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx,
									  unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->PutRelSDRAM_RdXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetRelSDRAM_RdXfer
   Description: Read a long-word from a relative SDRAM read-xfer register.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetRelSDRAM_RdXfer(hRS_Conn_T hTarget, unsigned char uEng, unsigned char ctx, 
									  unsigned int regNum, unsigned int *regData, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetRelSDRAM_RdXfer(uEng, ctx, regNum, regData, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetUengStatus
   Description: Get the micro-engine status.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetUengStatus(hRS_Conn_T hTarget, unsigned char uEng, uDbg_uEngStatus_T *uEngStatus)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetUengStatus(uEng, uEngStatus));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_HopUeng
   Description: Hop the specified microengines.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_HopUeng(hRS_Conn_T hTarget, unsigned int *uEngMask, unsigned int delay)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->HopUeng(uEngMask, delay));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_SetUengPCs
   Description: Set the micro-engine PCs.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_SetUengPCs(hRS_Conn_T hTarget, unsigned char uEng, unsigned int *upc)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->SetUengPCs(uEng, upc));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetUengPCs
   Description: Get the micro-engine PCs.

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_GetUengPCs(hRS_Conn_T hTarget, unsigned char uEng, unsigned int *upc)
{  
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetUengPCs(uEng, upc));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WriteFIFO_Tx
   Description: Write long-word(s) to transmit-FIFO.
   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_uDbg_WriteFIFO_Tx(hRS_Conn_T hTarget, unsigned char uEng, unsigned int fifoAddr,
                                unsigned int *data64, unsigned int count, int setValid)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->WriteFIFO_Tx(uEng, fifoAddr, data64, count, setValid));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_ReadFIFO_Rx
   Description: Read long-word(s) from receive-FIFO.
   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_ReadFIFO_Rx(hRS_Conn_T hTarget, unsigned char uEng, unsigned int fifoAddr,
                        unsigned int *data64, unsigned int count)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->ReadFIFO_Rx(uEng, fifoAddr, data64, count));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_GetSysMemSize
   Description: This routine returns sizes of virtual memory regions in the following
				pointers:
					sys     -- Size of memory mapped at zero for OS
					sys_all -- Size of memory mapped at zero and usable by the OS
					sdram   -- Size of memory mapped at C0000000 and usable by HW
					sram    -- Size of sram
					flash   -- Size of Flash

   Returns:		UDBG_SUCCESS,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_GetSysMemSize(hRS_Conn_T hTarget, unsigned int* sys, unsigned int* sys_all,
                          unsigned int* sdram, unsigned int* sram,unsigned int* flash)
{
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->GetSysMemSize(sys, sys_all, sdram, sram, flash));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_CNTL_RESET
   Description: reset SA1200

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_CNTL_Reset(hRS_Conn_T hTarget, unsigned int engineMask, unsigned int portMask,int clrReg)
{  
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->CNTL_Reset(engineMask, portMask, clrReg));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_CNTL_PAUSE
   Description: pause SA1200

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_CNTL_Pause(hRS_Conn_T hTarget, unsigned int engineMask, unsigned int portMask, unsigned int delay)
{  
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
    if(engineMask==0) return UDBG_SUCCESS;
	return(targetObj->debug->CNTL_Pause(engineMask, portMask, delay));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_CNTL_RESUME
   Description: resume SA1200

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_CNTL_Resume(hRS_Conn_T hTarget, unsigned int engineMask, unsigned int portMask)
{  
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->CNTL_Resume(engineMask, portMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_CNTL_Hop
   Description: Hop 

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_CNTL_Hop(hRS_Conn_T hTarget, unsigned int *engineMask, unsigned int portMask, unsigned int delay)
{  
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->CNTL_Hop(engineMask, portMask, delay));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_CNTL_START
   Description: start SA1200

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_CNTL_Start(hRS_Conn_T hTarget, unsigned int engineMask, unsigned int portMask)
{  
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->CNTL_Start(engineMask, portMask));
}

/*-----------------------------------------------------------------------------
   Function: 	RS_CNTL_InitUof
   Description: load ucode SA1200

   Returns:		UDBG_SUCCESS,UDBG_BADLIB,UDBG_UENGACT,UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbgAPI RS_CNTL_InitUof(hRS_Conn_T hTarget, char* buf, int buflen)
{  
	RS_Target *targetObj = (RS_Target *)hTarget;
	if(!RS_Target::ValidObject(targetObj) || (!targetObj->debug))return(UDBG_BADARG);
	return(targetObj->debug->CNTL_InitUof(buf, buflen));
}

