/* rs_udbg_init.cpp
 * Remote Services Debug Library - Remote Debug Server Initialization 
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui 6/29/98
 * revisions:
 * $History: rs_udbg_init.cpp $
 * 
 * *****************  Version 19  *****************
 * User: Ewalker      Date: 9/11/00    Time: 10:31
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 18  *****************
 * User: Ewalker      Date: 9/08/00    Time: 17:44
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 17  *****************
 * User: Ewalker      Date: 9/08/00    Time: 17:41
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 16  *****************
 * User: Ewalker      Date: 9/08/00    Time: 17:37
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 15  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 14  *****************
 * User: Ewalker      Date: 8/29/00    Time: 17:43
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 13  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:15
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 4/17/00    Time: 2:34p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bug# 925
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 1/06/00    Time: 6:15p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port
 * 
 * *****************  Version 7  *****************
 * User: Drjohns1     Date: 7/20/99    Time: 5:58p
 * Updated in $/dev_1200/Share/rs_udebug
 * Make a function call instead of creating a thread
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 7/13/99    Time: 12:45p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add thread id for New Thread ()
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 7/01/99    Time: 7:09p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add Init_ucloLib
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 7/01/99    Time: 12:50p
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 6/30/99    Time: 3:20p
 * Created in $/dev_1200/Share/rs_udebug
 * Add init function
 *
 */

#include <stdio.h>
#include <iostream.h>
#include <assert.h>
#include <time.h>

#include "ostype.h"
#include "osApi.h"
#include "ts.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

extern "C" 
{
#if (OS == VXWORKS)
int UseSerialDebugConnection = 0;          // use RPC by default
char SerialDebugDevName[20] = "ixphpc00";  // use PCI device when serial
#elif (OS == UCOS)
int UseSerialDebugConnection = 1;          // use Serial by default
char SerialDebugDevName[20] = "com2";      // use COM2 device when serial
#elif (OS == WIN_32)
int UseSerialDebugConnection = 0;          // use RPC by default
char SerialDebugDevName[20] = "com2";      // use COM2 device when serial
#else
int UseSerialDebugConnection = 0;          // use RPC by default
char SerialDebugDevName[20] = "ixphpc00";  // use PCI device when serial
#endif

#if (OS != UCOS)
void Rs_udbg_server();
#endif
void Rs_SC_server();
}

static Thread* rs_udbgServer = NULL;
static Thread* heartbeatLED_thread = NULL;

#if (OS == UCOS)
#ifdef __cplusplus
extern "C" {
#endif
unsigned int OSTimeGet(void);
#ifdef __cplusplus
}
#endif
static time_t Rs_OS_time(time_t *t)
{
    if (t != NULL) return(*t=(time_t)(OSTimeGet()/100));
    return((time_t)(OSTimeGet()/100));
}
#else  /* not UCOS */
static time_t Rs_OS_time(time_t *t)
{
    return(time(t));
}
#endif

#define IXP1200_LED_ADDR   (0x38400000+0x108000)  /* Address of LED display */
static void heartbeat_LED(void)
{
#if (OS != WIN_32)
    time_t t, last_t;
    last_t = 0;

    while(1){
        t = Rs_OS_time(NULL);
        if (t != last_t){
            last_t = t;
            t = t % 100;
            *(unsigned int*)(IXP1200_LED_ADDR) = ((t / 10) << 4) | (t % 10);
        }
        delayMs(200);
    }
#endif
}

int WbSvr_start_heartbeatLED()
{
#if (OS != WIN_32)
    if (heartbeatLED_thread != NULL) delete (heartbeatLED_thread);
#if (OS == UCOS)
    heartbeatLED_thread = new Thread(heartbeat_LED, 6);
#else
    heartbeatLED_thread = new Thread(heartbeat_LED);
#endif
#endif
    return 0;
}

int WbSvr_stop_heartbeatLED()
{
#if (OS != WIN_32)
    if (heartbeatLED_thread != NULL){
        delete (heartbeatLED_thread);
        heartbeatLED_thread = NULL;
    }
#endif
    return 0;
}

int WbSvr_init()
{
    if (rs_udbgServer != NULL) return 1;  // server thread already started

/*    WbSvr_start_heartbeatLED();*/

    if (UseSerialDebugConnection){
#if (OS == UCOS)
//	    rs_udbgServer = new Thread(Rs_SC_server, 6);
        Rs_SC_server();  // UCOS server server runs directly
#else
	    rs_udbgServer = new Thread(Rs_SC_server);
#endif
    }

    else{
#if (OS != UCOS)
        rs_udbgServer = new Thread(Rs_udbg_server);
#else
        return 1;  // UCOS does not support RPC
#endif
    }

    if(rs_udbgServer == NULL) return 1;

	return 0;
}

void WbSvr_exit()
{
    if(rs_udbgServer != NULL) delete (rs_udbgServer);
    rs_udbgServer = NULL;

/*    WbSvr_stop_heartbeatLED();*/
}

