/* rs_udbg_interface.cpp
 * Remote Services Debug Library - Remote debug server functions
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui 12/1/98
 * revisions:
 * $History: rs_udbg_interface.cpp $
 * 
 * *****************  Version 77  *****************
 * User: Ewalker      Date: 12/11/00   Time: 4:31p
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup serial communication channel after link closure.
 * 
 * *****************  Version 76  *****************
 * User: Ewalker      Date: 12/04/00   Time: 2:11p
 * Updated in $/dev_1200/Share/rs_udebug
 * Enforce a single Debug object in the rs_udbg server.
 * 
 * *****************  Version 75  *****************
 * User: Ewalker      Date: 11/30/00   Time: 6:38p
 * Updated in $/dev_1200/Share/rs_udebug
 * Better protect against invalid uDbg object handle passed from
 * workbench.
 * 
 * *****************  Version 74  *****************
 * User: Ewalker      Date: 11/28/00   Time: 10:43a
 * Updated in $/dev_1200/Share/rs_udebug
 * Fixed DefineFreeUstore to return BADARG when appropriate
 * 
 * *****************  Version 73  *****************
 * User: Ewalker      Date: 10/31/00   Time: 12:11p
 * Updated in $/dev_1200/Share/rs_udebug
 * C0 2k ustore support - made InitLib backwards compatible, supporting
 * only segment 1 settings. Apps must use DefineFreeUstore to set segment
 * 2 values if UpdateFreeUstore is not used.  
 * 
 * *****************  Version 72  *****************
 * User: Ewalker      Date: 10/27/00   Time: 3:09p
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup some DBG_PRINT statements
 * 
 * *****************  Version 70  *****************
 * User: Ewalker      Date: 9/14/00    Time: 10:28p
 * Updated in $/dev_1200/Share/rs_udebug
 * Needed to include stdlib.h for malloc.
 * 
 * *****************  Version 69  *****************
 * User: Ewalker      Date: 9/14/00    Time: 7:57p
 * Updated in $/dev_1200/Share/rs_udebug
 * Better debug print statements. Also removed some fixed size read
 * buffers.
 * 
 * *****************  Version 68  *****************
 * User: Ewalker      Date: 9/13/00    Time: 3:02p
 * Updated in $/dev_1200/Share/rs_udebug
 * Improved debug printing to help with debugging
 * 
 * *****************  Version 67  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 66  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:54
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 65  *****************
 * User: Ewalker      Date: 8/29/00    Time: 17:43
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 64  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:15
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 63  *****************
 * User: Lhui         Date: 4/05/00    Time: 4:49p
 * Updated in $/dev_1200/Share/rs_udebug
 * Use new OS type
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 12/03/98   Time: 1:56p
 * Created in $/users/Hui/SA-1200/rs_udebug
 */ 

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "ostype.h"
#include "debug_1200.h"
#include "debug_1200.hxx"
#include "scomm.h"
#if (OS == VXWORKS)
#include <wtx.h>    // only VXWORKS support WTX
#endif
#include "rs_udebug_svc.h"
#include "rs_uDbg_events.h"
#include "rs_ScMsg.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

// UCOS does not support RPC
#if (OS != UCOS)
static CLIENT *hckClient = NULL;
static char ckServer[100];
#endif

// all OS's support serial (through either PCI or COM)
static Sc_CLIENT ScommID = SCOMM_FAIL;

// used for breakpoint callbacks
static uDbg_Handle_T Save_dbgH = NULL;

extern "C" 
{
extern int UseSerialDebugConnection;
extern char SerialDebugDevName[20];
#if (OS == VXWORKS)
int WTX_uDbg_SetBreakpoint(uDbg_Handle_T dbgH, uDbg_Bkpt_T *breakpoint);
#endif
#if (OS != UCOS)
int *rs_udbg_bkpt_ck(rs_uDbg_Bkpt_CK_T *argp, CLIENT *clnt);
CLIENT *create_ck_hClient(char *ckServer);
void destroy_ck_hClient(CLIENT *clnt);
#endif
int *rs_udbg_bkpt_ck_sc(rs_uDbg_Bkpt_CK_T *argp,Sc_CLIENT *clnt);
}

#if (OS == VXWORKS)
void WTX_BKCallback(unsigned short bkptId, unsigned char ueng, unsigned char ctx, void *usrData)
{
	char MSGBuf[128];
#ifdef _DBG_PRINT
	printf("Enter WTX_BKCallback: ID=%d, ueng=%d, ctx=%d, usrData=%x\n", bkptId, ueng, ctx, usrData);
#endif
	memset(MSGBuf, 0, sizeof(MSGBuf));
	sprintf(MSGBuf,"IXP1200^:%d;%d;%x;%x;\n",bkptId,ueng,ctx,usrData);
	if (Save_dbgH != NULL) CNTL_Pause(Save_dbgH, 0, 0xffff, 1); //HECK pause all the ports
    wdbUserEvtPost((char*)MSGBuf);
#ifdef _DBG_PRINT
	printf("Return from WTX_BKCallback\n");
#endif

}
#endif




void BKCallback(unsigned short bkptId, unsigned char ueng, unsigned char ctx, void *usrData)
{
	static rs_uDbg_Bkpt_CK_T Args;

    Args.bkptId = bkptId;
    Args.ueng = ueng;
	Args.ctx = ctx;
	Args.usrData = (int)usrData;

#ifdef _DBG_PRINT
	printf("BKCallback: BkptId=%d, ueng=%d, ctx=%x, usrData=%x\n", bkptId, ueng, ctx, usrData);
#endif

    // serial connection for debug supported universally (either through PCI or COM)
    if (UseSerialDebugConnection){
        if(ScommID == SCOMM_FAIL){
	     if((ScommID = SComm_Open(SerialDebugDevName, TargetComPort_2)) < 0){
             ScommID = SCOMM_FAIL;
			 return;
             }
        }

        rs_udbg_bkpt_ck_sc(&Args, &ScommID);
    }

    else{
#if (OS != UCOS)    // RPC connection for debug not supported on UCOS
        if(hckClient == NULL){
		    if((hckClient = create_ck_hClient(ckServer)) == NULL)
			    return;
        }
        rs_udbg_bkpt_ck(&Args, hckClient);
    	if (Save_dbgH != NULL) CNTL_Pause(Save_dbgH, 0, 0xffff, 1); //HECK pause all the ports
#endif
    }

    return;
 }




int *
rs_udbg_initlib_1(rs_uDbg_uStoreFree_T *argp)
{
	static int res = 0;
	unsigned int i;
    Debug *dbgObj;

    res = 0;

#ifdef _DBG_PRINT
	printf("enter rs_udbg_initlib_1 - ClientName=%s\n", argp->ClientName);
#endif

    // close serial breakpoint callback connection if open
    if(ScommID != SCOMM_FAIL){
        SComm_Close(ScommID);
        ScommID = SCOMM_FAIL;
    }

    // close RPC callback connection is open
#if (OS != UCOS)    // RPC connection for debug not supported on UCOS
    if (hckClient != NULL){
        destroy_ck_hClient(hckClient);
        hckClient = NULL;
    }
#endif   

    Save_dbgH = NULL;

    // create the debug object
    dbgObj = new Debug(NULL);

    for(i=0; i < argp->freeUpc.freeUpc_len && i < argp->freeSize.freeSize_len && i < MAX_UENGS; i++){
#ifdef _DBG_PRINT
	    printf("  freeUpc[%d]=%x\n", i, argp->freeUpc.freeUpc_val[i]);
		printf("  freeSize[%d]=%x\n", i, argp->freeSize.freeSize_val[i]);
#endif
        dbgObj->DefineFreeUstore(((unsigned int)(1 << i)), 
                                    argp->freeUpc.freeUpc_val[i],
                                    argp->freeSize.freeSize_val[i]);
    }

    res = (int)dbgObj;

#if (OS != UCOS)
	// save remote host name which is passed in the client name field
    strncpy(ckServer, argp->ClientName, sizeof(ckServer) - 1);
    ckServer[sizeof(ckServer) - 1] = '\0';
#endif

#ifdef _DBG_PRINT
	printf("leave rs_udbg_initlib_1 - obj=%x\n", res);
#endif

    return (&res);
}


int *
rs_udbg_delelib_1(rs_uDbg_Handle_T *argp)
{
	static int res = 1;
	Debug *puDbg = (Debug *)argp->uDbg_Handle; 

    res = 1;

#ifdef _DBG_PRINT
	printf("enter rs_udbg_delelib_1 - obj=%x\n", puDbg);
#endif

    // close serial breakpoint callback connection if open
    if(ScommID != SCOMM_FAIL){
        SComm_Close(ScommID);
        ScommID = SCOMM_FAIL;
    }

    // close RPC callback connection is open
#if (OS != UCOS)    // RPC connection for debug not supported on UCOS
    if (hckClient != NULL){
        destroy_ck_hClient(hckClient);
        hckClient = NULL;
    }
#endif

    Save_dbgH = NULL;

    if (Debug::ValidObject(puDbg)) delete puDbg;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_delelib_1\n");
#endif    

    return (&res);
}

int *
rs_udbg_updatefreeustore_1(rs_uDbg_One_uint_T *argp)
{
	static int res = UDBG_SUCCESS;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_updatefreeustore_1 - engMask=%x\n", argp->arg1);
#endif

    res = UDBG_SUCCESS;

    if(Debug::ValidObject(puDbg))
        puDbg->UpdateFreeUstore(argp->arg1);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_updatefreeustore_1 - res=%x\n", res);
#endif

    return (&res);
}

int *
rs_udbg_definefreeustore_1(rs_uDbg_Three_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_definefreeustore_1 - engMask=%x, freeUaddr=%x, freeSize=%x\n", argp->arg1, argp->arg2, argp->arg3);
#endif

    if(Debug::ValidObject(puDbg))
	     res = puDbg->DefineFreeUstore(argp->arg1, argp->arg2, argp->arg3);

    else res = UDBG_BADARG;
	
#ifdef _DBG_PRINT
	printf("leave rs_udbg_definefreeustore_1 - res=%x\n", res);
#endif

    return (&res);
}



int *
rs_udbg_resetueng_1(rs_uDbg_Two_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_resetueng_1 - engMask=%x, clrReg=%x\n", argp->arg1, argp->arg2);
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->ResetUeng(argp->arg1, argp->arg2);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_resetueng_1 - res=%x\n", res);
#endif

	return (&res);
}

int *
rs_udbg_clrresetueng_1(rs_uDbg_One_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_clrresetueng_1 - engMask=%x\n", argp->arg1);
#endif

    if(Debug::ValidObject(puDbg))
		res = puDbg->ClrResetUeng(argp->arg1);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_clrresetueng_1 - res=%x\n",res);
#endif

	return (&res);
}



int *
rs_udbg_startueng_1(rs_uDbg_One_uint_T *argp)
{
	static int res = UDBG_BADARG;
    Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle;

#ifdef _DBG_PRINT
	printf("enter rs_udbg_startueng_1 - engMask=%x\n", argp->arg1);
#endif

 	if(Debug::ValidObject(puDbg))
		res = puDbg->StartUeng(argp->arg1);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_startueng_1 - res=%x\n", res);
#endif

	return (&res);
}




int *
rs_udbg_pauseueng_1(rs_uDbg_Two_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_pauseueng_1 - engMask=%x, delay=%x\n", argp->arg1, argp->arg2);
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->PauseUeng(argp->arg1, argp->arg2);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_pauseueng_1 - res=%x\n", res);
#endif

	return (&res);
}


int *
rs_udbg_resumeueng_1(rs_uDbg_One_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_resumeueng_1 - engMask=%x\n", argp->arg1);
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->ResumeUeng(argp->arg1);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_resumeueng_1 - res=%x\n", res);
#endif

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_hopueng_1(rs_uDbg_Two_uint_T *argp)
{
	static rs_uDbg_Status_T res;
   	static unsigned int val;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
    val = argp->arg1; 
	res.resData.resData_val = &val;
	res.resData.resData_len = 1;

#ifdef _DBG_PRINT
	printf("enter rs_udbg_hopueng_1 - engMask=%x, delay=%x\n", val, argp->arg2);
#endif

    if(Debug::ValidObject(puDbg))
		res.res = puDbg->HopUeng(&val, argp->arg2);

    else res.res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udgb_hopueng_1 - res=%x\n", res.res);
#endif
	
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getuthreadpc_1(rs_uDbg_GetThPC_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int val;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getuthreadpc_1 - eng=%d, ctx=%d\n", argp->uEng, argp->ctx);
#endif

	res.resData.resData_val = &val;
	res.resData.resData_len = 1;

	if(Debug::ValidObject(puDbg))
		res.res = puDbg->GetUthreadPC(argp->uEng, argp->ctx, res.resData.resData_val);

    else res.res = UDBG_BADARG;

    if (res.res != UDBG_SUCCESS) val = 0;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getuthreadpc_1 - uThreadPC=%x, res=%x\n", val, res.res);
#endif

	return (&res);
}


int *
rs_udbg_setuthreadspc_1(rs_uDbg_Three_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_setuthreadspc_1 - engMask=%x, ctxMask=%x, upc=%x\n", argp->arg1, argp->arg2, argp->arg3);
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->SetUthreadsPC(argp->arg1, argp->arg2, argp->arg3);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_setuthreadspc_1 - res=%x\n", res);
#endif
	
	return (&res);
}


int *
rs_udbg_isuengactive_1(rs_uDbg_One_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_isuengactive_1 - engMask=%x\n", argp->arg1);
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->IsUengActive(argp->arg1);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_isuengactive_1 - res=%x\n", res);
#endif

	return (&res);
}

int *
rs_udbg_getactiveuengs_1(rs_uDbg_One_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getactiveuengs_1 - engMask=%x\n", argp->arg1);
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->GetActiveUengs(argp->arg1);

    else res = UDBG_BADARG;
	
#ifdef _DBG_PRINT
	printf("leave rs_udbg_getactiveuengs_1 - res=%x\n", res);
#endif

	return (&res);
}

rs_uDbg_Status_T *
rs_udbg_getuword_1(rs_uDbg_uWord_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int val;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

	res.resData.resData_val = &val;
	res.resData.resData_len = 1;

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getuword_1 - eng=%d, uWordAddr=%x\n", argp->uEng, argp->uWordAddr);
#endif

	if(Debug::ValidObject(puDbg))
		res.res = puDbg->GetUword(argp->uEng, argp->uWordAddr, res.resData.resData_val);

    else res.res = UDBG_BADARG;

    if (res.res != UDBG_SUCCESS) val = 0;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getuword_1 - uData=%x, res=%x\n", val, res.res);
#endif

	return (&res);
}


int *
rs_udbg_setuword_1(rs_uDbg_uWord_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_setuword_1 - engMask=%x, uWordAddr=%x, uData=%x\n",
		   argp->uEng, argp->uWordAddr, *argp->u_Word.u_Word_val);
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->SetUword(argp->uEng, argp->uWordAddr, *argp->u_Word.u_Word_val);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_setuword_1 - res=%x\n", res);
#endif

	return (&res);

}


rs_uDbg_Status_T *
rs_udbg_getnumuword_1(rs_uDbg_uWord_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getnumuword_1 - eng=%d, uWordAddr=%x, numWords=%d\n",
		   argp->uEng, argp->uWordAddr, argp->numWords);
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->numWords || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->numWords;
		if (val_buf_len < 1) val_buf_len = 1;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

	res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->numWords;

	if(val_buf != NULL && argp->numWords > 0 && Debug::ValidObject(puDbg))
		res.res = puDbg->GetNumUword(argp->uEng, argp->uWordAddr, argp->numWords, val_buf);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->numWords; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getnumuword_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("uData=");
        for(unsigned int i = 0; i < res.resData.resData_len; i++) 
			printf("%s%x", (i == 0 ? "" : "|"), val_buf[i]);
		printf(", ");
	}
	printf("res=%x\n", res.res);
#endif

	return (&res);
}


int *
rs_udbg_setnumuword_1(rs_uDbg_uWord_T *argp)

{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	unsigned int *val = argp->u_Word.u_Word_val;
	printf("enter rs_udbg_setnumuword_1 - engMask=%x, uWordAddr=%x, numWords=%d, uData=",
		   argp->uEng, argp->uWordAddr, argp->numWords);
    for(int i = 0; i < argp->numWords; i++, val++) printf("%s%x", (i == 0 ? "" : "|"), *val);
	printf("\n");
#endif

	if(Debug::ValidObject(puDbg))
		res = puDbg->SetNumUword(argp->uEng, argp->uWordAddr, argp->numWords, argp->u_Word.u_Word_val);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_setnumuword_1 - res=%x\n", res);
#endif

    return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getcsr_1(rs_uDbg_CSR_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int val;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getcsr_1 - csr=%x\n", argp->csrAddr);
#endif

	res.resData.resData_val = &val;
	res.resData.resData_len = 1;
    
    res.res = UDBG_SUCCESS;

    if(Debug::ValidObject(puDbg))
		puDbg->GetCSR(argp->csrAddr, res.resData.resData_val);

    else res.res = UDBG_BADARG;

    if (res.res != UDBG_SUCCESS) val = 0;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getcsr_1 - csrval=%x, res=%x\n", val, res.res);
#endif

	return (&res);
}


int *
rs_udbg_setcsr_1(rs_uDbg_CSR_T *argp)
{
	static int res = UDBG_SUCCESS;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_setcsr_1 - csr=%x, csrval=%x\n", argp->csrAddr, *argp->csrData.csrData_val);
#endif

	res = UDBG_SUCCESS;

	if(Debug::ValidObject(puDbg))
		puDbg->SetCSR(argp->csrAddr, *argp->csrData.csrData_val);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_setcsr_1 - res=%x\n", res);
#endif

    return (&res);
}


rs_uDbg_Status_T *
rs_udbg_setbreakpoint_1(rs_uDbg_BreakPoint_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int val;
	struct uDbg_Bkpt *pBpoint = (struct uDbg_Bkpt *)&argp->pBpoint;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_setbreakpoint_1\n");
#endif

    pBpoint->callback = BKCallback;

    val = 0;
    res.resData.resData_val = &val;
	res.resData.resData_len = 1;

    if(Debug::ValidObject(puDbg)){
        Save_dbgH = puDbg;
		res.res = puDbg->SetBreakpoint((struct uDbg_Bkpt *)&argp->pBpoint);
    }

    else res.res = UDBG_BADARG;

	*res.resData.resData_val = argp->pBpoint.id;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_setbreakpoint_1 - id=%x, uAddr=%x, eng=%d, ", pBpoint->id, pBpoint->uAddr, pBpoint->uEng);
	printf("usrData=%x, breakIfCtx=%x, onBrkUengMask=%x, res=%x\n", pBpoint->usrData, pBpoint->breakIfCtx, pBpoint->stpOnBrkUengMask, res.res);
#endif

    return (&res);
}


rs_uDbg_ShowBreakpts_T *
rs_udbg_showbreakpoints_1(rs_uDbg_One_uint_T *argp)
	
{
	static rs_uDbg_ShowBreakpts_T res;
	static uDbg_BkptArray_T resBkpt;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_showbreakpoint_1 - engMask=%x\n", argp->arg1);
#endif

    bzero((char *)&res, sizeof(res));
    bzero((char *)&resBkpt, sizeof(resBkpt));

    res.BkptArray.bkpts.bkpts_val = (rs_uDbg_Bkpt_T *)&resBkpt.bkpts[0];  
	res.BkptArray.bkpts.bkpts_len = UDBG_MAX_BREAKPOINTS;

	if(Debug::ValidObject(puDbg)) {
		res.res = puDbg->ShowBreakpoints(argp->arg1, (struct uDbg_BkptArray *)&resBkpt);
		res.BkptArray.len = resBkpt.len;
	}

    else res.res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_showbreakpoint_1 - res=%x\n", res.res);
#endif

    return (&res);
}


int *
rs_udbg_clearallbreakpoints_1(rs_uDbg_One_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_clearallbreakpoints_1 - engMask=%x\n", argp->arg1);
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->ClearAllBreakpoints(argp->arg1);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_clearallbreakpoints_1 - res=%x\n", res);
#endif

	return (&res);
}


int *
rs_udbg_clearbreakpoint_1(rs_uDbg_One_uint_T *argp)
	
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_clearbreakpoint_1 - id=%d\n", argp->arg1);
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->ClearBreakpoint(argp->arg1);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_clearbreakpoint_1 - res=%x\n", res);
#endif

	return (&res);
}


int * 
rs_udbg_writemem32_1(rs_uDbg_Mem_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_writemem32_1 - addr=%x, len=%d, val=", argp->addr, argp->data.data_len);
	for(unsigned int i = 0; i < argp->data.data_len; i++) 
		printf("%s%x", (i == 0 ? "" : "|"), argp->data.data_val[i]);
	printf("\n");
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->WriteMem32(argp->addr, argp->data.data_val, argp->data.data_len);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_writemem32_1 - res=%x\n", res);
#endif

	return (&res);
}


rs_uDbg_Status_T 
*rs_udbg_readmem32_1(rs_uDbg_Mem_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_readmem32_1 - addr=%x, len=%d\n", argp->addr, argp->data.data_len); 
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->data.data_len || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->data.data_len;
		if (val_buf_len < 1) val_buf_len = 1;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

    res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->data.data_len;

	if(val_buf != NULL && argp->data.data_len > 0 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->ReadMem32(argp->addr, val_buf, argp->data.data_len);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->data.data_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_readmem32_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
		for(unsigned int i = 0; i < res.resData.resData_len; i++) 
			printf("%s%x", (i == 0 ? "" : "|"), val_buf[i]);
	    printf(", ");
	}
	printf("res=%x\n", res.res);
#endif
	
	return (&res);
}


int *
rs_udbg_writemem64_1(rs_uDbg_Mem_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_writemem64_1 - addr=%x, len=%d, val=", argp->addr, argp->data.data_len/2);
	for(unsigned int i = 0; i < argp->data.data_len; i++)
        printf("%s%08x", ((((i % 2) == 0) && i != 0) ? "|" : ""), argp->data.data_val[i]);
	printf("\n");
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->WriteMem64(argp->addr, argp->data.data_val, argp->data.data_len/2);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_writemem64_1 - res=%x\n", res);
#endif

	return (&res);
}


rs_uDbg_Status_T 
*rs_udbg_readmem64_1(rs_uDbg_Mem_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_readmem64_1 - addr=%x, len=%d\n", argp->addr, argp->data.data_len/2); 
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->data.data_len+1 || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->data.data_len+1;
		if (val_buf_len < 2) val_buf_len = 2;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

    res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->data.data_len;

    if(val_buf != NULL && argp->data.data_len >= 2 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->ReadMem64(argp->addr, val_buf, argp->data.data_len/2);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->data.data_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_readmem64_1 - "); 
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
	    for(unsigned int i = 0; i < res.resData.resData_len; i++)
            printf("%s%08x", ((((i % 2) == 0) && i != 0) ? "|" : ""), val_buf[i]);
        printf(", ");
	}
	printf("res=%x\n", res.res);
#endif
	
	return (&res);
}


int *
rs_udbg_putrelgpr_1(rs_uDbg_Reg_T *argp)
	
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_putrelgpr_1 - eng=%d, ctx=%d, bankType=%x, regAddr=%x, value[0]=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->bankType, argp->regAddr, *(argp->regData.regData_val), argp->regData.regData_len);
#endif

    if(Debug::ValidObject(puDbg)) 
		res = puDbg->PutRelGPR(argp->uEng, argp->ctx, (uDbg_Bank_T)argp->bankType, argp->regAddr, argp->regData.regData_val, argp->regData.regData_len);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_putrelgpr_1 - res=%x\n", res);
#endif

    return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelgpr_1(rs_uDbg_Reg_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_getrelgpr_1 - eng=%d, ctx=%d, bankType=%x, regAddr=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->bankType, argp->regAddr, argp->regData.regData_len);
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->regData.regData_len || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->regData.regData_len;
		if (val_buf_len < 1) val_buf_len = 1;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

    res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->regData.regData_len;

	if(val_buf != NULL && argp->regData.regData_len > 0 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->GetRelGPR(argp->uEng, argp->ctx, (uDbg_Bank_T)argp->bankType, argp->regAddr, val_buf, argp->regData.regData_len);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->regData.regData_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getrelgpr_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
	    for(unsigned int i = 0; i < res.resData.resData_len; i++)
			printf("%s%x", (i == 0 ? "" : "|"), val_buf[i]);
        printf(", ");
	}
	printf("res=%x\n", res.res);
#endif

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsram_wrxfer_1(rs_uDbg_Reg_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_getrelsram_wrxfer_1 - eng=%d, ctx=%d, regAddr=%x, len=%d\n",
            argp->uEng, argp->ctx, argp->regAddr, argp->regData.regData_len);
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->regData.regData_len || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->regData.regData_len;
		if (val_buf_len < 1) val_buf_len = 1;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

    res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->regData.regData_len;

    if(val_buf != NULL && argp->regData.regData_len > 0 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->GetRelSRAM_WrXfer(argp->uEng, argp->ctx, argp->regAddr, val_buf, argp->regData.regData_len);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->regData.regData_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getrelsram_wrxfer_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
	    for(unsigned int i = 0; i < res.resData.resData_len; i++)
			printf("%s%x", (i == 0 ? "" : "|"), val_buf[i]);
        printf(", ");
	}
	printf("res=%x\n", res.res);
#endif

  return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsdram_wrxfer_1(rs_uDbg_Reg_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_getrelsdram_wrxfer_1 - eng=%d, ctx=%d, regAddr=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->regAddr, argp->regData.regData_len);
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->regData.regData_len || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->regData.regData_len;
		if (val_buf_len < 1) val_buf_len = 1;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

    res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->regData.regData_len;

	if(val_buf != NULL && argp->regData.regData_len > 0 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->GetRelSDRAM_WrXfer(argp->uEng, argp->ctx, argp->regAddr, val_buf, argp->regData.regData_len);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->regData.regData_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getrelsdram_wrxfer_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
	    for(unsigned int i = 0; i < res.resData.resData_len; i++)
			printf("%s%x", (i == 0 ? "" : "|"), val_buf[i]);
        printf(", ");
	}
	printf("res=%x\n", res.res);
#endif

	return (&res);
}


int *
rs_udbg_putrelsram_wrxfer_1(rs_uDbg_Reg_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle;
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_putrelsram_wrxfer_1 - eng=%d, ctx=%d, regAddr=%x, value[0]=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->regAddr, *(argp->regData.regData_val), argp->regData.regData_len);
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->PutRelSRAM_WrXfer(argp->uEng, argp->ctx,argp->regAddr, (unsigned int *)(argp->regData.regData_val), argp->regData.regData_len);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_putrelsram_wrxfer_1 - res=%x\n", res);
#endif

	return (&res);
}


int *
rs_udbg_putrelsdram_wrxfer_1(rs_uDbg_Reg_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_putrelsdram_wrxfer_1 - eng=%d, ctx=%d, regAddr=%x, value[0]=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->regAddr, *(argp->regData.regData_val), argp->regData.regData_len);
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->PutRelSDRAM_WrXfer(argp->uEng, argp->ctx, argp->regAddr, argp->regData.regData_val, argp->regData.regData_len);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_putrelsdram_wrxfer_1 - res=%x\n", res);
#endif

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsram_rdxfer_1(rs_uDbg_Reg_T *argp)
	
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_getrelsram_rdxfer_1 - eng=%d, ctx=%d, regAddr=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->regAddr, argp->regData.regData_len);
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->regData.regData_len || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->regData.regData_len;
		if (val_buf_len < 1) val_buf_len = 1;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

    res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->regData.regData_len;

	if(val_buf != NULL && argp->regData.regData_len > 0 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->GetRelSRAM_RdXfer(argp->uEng, argp->ctx, argp->regAddr, val_buf, argp->regData.regData_len);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->regData.regData_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getrelsram_rdxfer_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
	    for(unsigned int i = 0; i < res.resData.resData_len; i++)
			printf("%s%x", (i == 0 ? "" : "|"), val_buf[i]);
        printf(", ");
	}
	printf("res=%x\n", res.res);
#endif

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsdram_rdxfer_1(rs_uDbg_Reg_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_getrelsdram_rdxfer_1 - eng=%d, ctx=%d, regAddr=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->regAddr, argp->regData.regData_len);
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->regData.regData_len || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->regData.regData_len;
		if (val_buf_len < 1) val_buf_len = 1;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

    res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->regData.regData_len;

	if(val_buf != NULL && argp->regData.regData_len > 0 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->GetRelSDRAM_RdXfer(argp->uEng, argp->ctx, argp->regAddr, val_buf, argp->regData.regData_len);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->regData.regData_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getrelsdram_rdxfer_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
	    for(unsigned int i = 0; i < res.resData.resData_len; i++)
			printf("%s%x", (i == 0 ? "" : "|"), val_buf[i]);
        printf(", ");
	}
	printf("res=%x\n", res.res);
#endif
	
	return (&res);
}


int *
rs_udbg_putrelsram_rdxfer_1(rs_uDbg_Reg_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_putrelsram_rdxfer_1 - eng=%d, ctx=%d, regAddr=%x, value[0]=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->regAddr, *(argp->regData.regData_val), argp->regData.regData_len);
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->PutRelSRAM_RdXfer(argp->uEng, argp->ctx, argp->regAddr, argp->regData.regData_val, argp->regData.regData_len);

    else res = UDBG_BADARG;
    
#ifdef _DBG_PRINT
	printf("leave rs_udbg_putrelsram_rdxfer_1 - res=%x\n", res);
#endif
	
    return (&res);
}


int *
rs_udbg_putrelsdram_rdxfer_1(rs_uDbg_Reg_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_putrelsdram_rdxfer_1 - eng=%d, ctx=%d, regAddr=%x, value[0]=%x, len=%d\n",
           argp->uEng, argp->ctx, argp->regAddr, *(argp->regData.regData_val), argp->regData.regData_len);
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->PutRelSDRAM_RdXfer(argp->uEng, argp->ctx, argp->regAddr, argp->regData.regData_val, argp->regData.regData_len);

    else res = UDBG_BADARG;
    
#ifdef _DBG_PRINT
	printf("leave rs_udbg_putrelsdram_rdxfer_1 - res=%x\n", res);
#endif
    
    return (&res);
}





rs_uDbg_GetuEngStatus_T *rs_udbg_getuengstatus_1(rs_uDbg_One_uint_T *argp)
{
	static rs_uDbg_GetuEngStatus_T res;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getuengstatus_1 - eng=%d\n", argp->arg1);
#endif

	if(Debug::ValidObject(puDbg)) 
		res.res = puDbg->GetUengStatus(argp->arg1, (uDbg_uEngStatus_T *)&res.uEngStatus);

    else res.res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getuengstatus_1 - ");
    if (res.res == UDBG_SUCCESS){
        printf("state=%x, ctx=%x, enables=%x, breakId=%x, pc=%x ",
            res.uEngStatus.state, res.uEngStatus.activeCtx, res.uEngStatus.enables,
            res.uEngStatus.breakId, res.uEngStatus.activePC);
    }
    printf("res=%x\n", res.res);
#endif

    return (&res);
}



int *rs_udbg_setuengpc_1(rs_uDbg_Mem_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
    
#ifdef _DBG_PRINT
	unsigned int *ptr;
	ptr = argp->data.data_val;
	printf("enter rs_udbg_setuengpc_1 - eng=%d, pc=%x|%x|%x|%x\n", argp->addr, ptr[0], ptr[1], ptr[2], ptr[3]);
#endif
  
	if(Debug::ValidObject(puDbg)) 
		res = puDbg->SetUengPCs((char)argp->addr, argp->data.data_val);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_setuengpc_1 - res=%x\n", res);
#endif
    
	return (&res);
}
	
rs_uDbg_Status_T *rs_udbg_getuengpc_1(rs_uDbg_One_uint_T *argp)
{
	static rs_uDbg_Status_T res;
   	static unsigned int val[MAX_CTX];	
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getuengpc_1 - eng=%d\n", argp->arg1);
#endif

	res.resData.resData_val = &val[0];
	res.resData.resData_len = MAX_CTX;

	if(Debug::ValidObject(puDbg)) 
		res.res = puDbg->GetUengPCs(argp->arg1, res.resData.resData_val);

    else res.res = UDBG_BADARG;

    if (res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < MAX_CTX; i++) val[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getuengpc_1 - Uengpc=%x|%x|%x|%x, res=%x\n",
           val[0], val[1], val[2], val[3], res.res);
#endif

    return (&res);
}

int *rs_udbg_writefifo_tx_1(rs_uDbg_FiFo_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_writefifo_tx_1 - eng=%d, fifoAddr=%x, data64=%x, count=%x, setValid=%d\n",
           argp->uEng, argp->fifoAddr, *(argp->data64.data64_val), argp->count, argp->setValid);
#endif

	if(Debug::ValidObject(puDbg)) 
		res = puDbg->WriteFIFO_Tx(argp->uEng, argp->fifoAddr, argp->data64.data64_val, argp->count, argp->setValid);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_writefifo_tx_1 - res=%x\n", res);
#endif

    return (&res);
}


rs_uDbg_Status_T *rs_udbg_readfifo_rx_1(rs_uDbg_FiFo_T *argp)
{
	static rs_uDbg_Status_T res;
	static unsigned int *val_buf = NULL;
	static unsigned int val_buf_len = 0;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
#ifdef _DBG_PRINT
	printf("enter rs_udbg_readfifo_rx_1 - eng=%d, fifoAddr=%x, count=%x\n", argp->uEng, argp->fifoAddr, argp->count);
#endif

	/* reuse last buffer if possible */
    if (val_buf_len < argp->data64.data64_len+1 || val_buf == NULL){
        if (val_buf != NULL) free((void*) val_buf);
		val_buf_len = argp->data64.data64_len+1;
		if (val_buf_len < 2) val_buf_len = 2;
		val_buf = (unsigned int*) malloc(val_buf_len*sizeof(unsigned int));
		if (val_buf == NULL) val_buf_len = 0;
	}

	res.resData.resData_val = val_buf;
	res.resData.resData_len = argp->data64.data64_len;

	if(val_buf != NULL && argp->data64.data64_len > 0 && Debug::ValidObject(puDbg)) 
		res.res = puDbg->ReadFIFO_Rx(argp->uEng, argp->fifoAddr, val_buf, argp->count);

    else{
		res.res = UDBG_BADARG;
		res.resData.resData_len = 0;
	}

    if (val_buf != NULL && res.res != UDBG_SUCCESS){
        for (unsigned int i = 0; i < argp->data64.data64_len; i++) val_buf[i] = 0;
    }

#ifdef _DBG_PRINT
	printf("leave rs_udbg_readfifo_rx_1 - ");
	if (res.resData.resData_len > 0 && res.res == UDBG_SUCCESS){
		printf("val="); 
	    for(unsigned int i = 0; i < res.resData.resData_len; i++)
            printf("%s%08x", ((((i % 2) == 0) && i != 0) ? "|" : ""), val_buf[i]);
        printf(", ");
	}
	printf("res=%x\n", res.res);
#endif

	return (&res);
}

rs_uDbg_GetMemSize_T *rs_udbg_getsysmemsize_1(rs_uDbg_Handle_T *argp)
{
	static rs_uDbg_GetMemSize_T res;
	Debug *puDbg = (Debug *)argp->uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_udbg_getsysmemsize_1\n");
#endif

	bzero((char *)&res, sizeof(res));

    if(Debug::ValidObject(puDbg))
		res.res = puDbg->GetSysMemSize(&res.sys, &res.sys_all, &res.sdram, &res.sram, &res.flash);

    else res.res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_udbg_getsysmemsize_1 - sys=%d, sys_all=%d, sdram=%d, sram=%d, flash=%d, res=%x\n",
           res.sys, res.sys_all, res.sdram, res.sram, res.flash, res.res);
#endif

	return (&res);
}


int *
rs_cntl_reset_1(rs_uDbg_Three_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_cntl_reset_1 - engMask=%x, portMask=%x, clreng=%x\n", argp->arg1, argp->arg2, argp->arg3);
#endif

    if(Debug::ValidObject(puDbg))
	    res = CNTL_Reset((Debug *)argp->hRs_uDbg.uDbg_Handle, argp->arg1, argp->arg2, argp->arg3);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_cntl_reset_1 - res=%x\n", res);
#endif

    return (&res);
}

int *
rs_cntl_pause_1(rs_uDbg_Three_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_cntl_pause_1 - engMask=%x, portMask=%x, delay=%x\n", argp->arg1, argp->arg2, argp->arg3);
#endif

    if(Debug::ValidObject(puDbg))
	    res = CNTL_Pause((Debug *)argp->hRs_uDbg.uDbg_Handle, argp->arg1, argp->arg2, argp->arg3);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_cntl_pause_1 - res=%x\n", res);
#endif

	return (&res);
}

int *
rs_cntl_resume_1(rs_uDbg_Two_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_cntl_resume_1 - engMask=%x, portMask=%x\n", argp->arg1, argp->arg2);
#endif

    if(Debug::ValidObject(puDbg))
	    res = CNTL_Resume((Debug *)argp->hRs_uDbg.uDbg_Handle, argp->arg1, argp->arg2);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_cntl_resume_1 - res=%x\n", res);
#endif

	return (&res);
}

int *
rs_cntl_start_1(rs_uDbg_Two_uint_T *argp)
{
	static int res = UDBG_BADARG;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 

#ifdef _DBG_PRINT
	printf("enter rs_cntl_start_1 - engMask=%x, portMask=%x\n", argp->arg1, argp->arg2);
#endif

    if(Debug::ValidObject(puDbg))
    	res = CNTL_Start((Debug *)argp->hRs_uDbg.uDbg_Handle, argp->arg1, argp->arg2);

    else res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_cntl_start_1 - res=%x\n", res);
#endif

	return (&res);
}

rs_uDbg_Status_T *
rs_cntl_hop_1(rs_uDbg_Three_uint_T *argp)
{
	static rs_uDbg_Status_T res;
   	static unsigned int val;
	Debug *puDbg = (Debug *)argp->hRs_uDbg.uDbg_Handle; 
	
    val = argp->arg1; 
	res.resData.resData_val = &val;
	res.resData.resData_len = 1;

#ifdef _DBG_PRINT
	printf("enter rs_cntl_hop_1 - engMask=%x, portMask=%x, delay=%x\n", val, argp->arg2, argp->arg3);
#endif

    if(Debug::ValidObject(puDbg))
	    res.res = CNTL_Hop((Debug *)argp->hRs_uDbg.uDbg_Handle, &val, argp->arg2, argp->arg3);

    else res.res = UDBG_BADARG;

#ifdef _DBG_PRINT
	printf("leave rs_cntl_hop_1 - engMask=%x, res=%x\n", val, res.res);
#endif
	
	return (&res);
}

int *
rs_cntl_inituof_1(rs_CNTL_InitUof_T *argp)
{
	static int res = 1;

#ifdef _DBG_PRINT
	printf("enter rs_cntl_inituof_1 - length=%d\n", argp->UofData.UofData_len);
#endif

    res = CNTL_InitUof(argp->UofData.UofData_val, argp->UofData.UofData_len);

#ifdef _DBG_PRINT
	printf("leave rs_cntl_inituof_1 - res=%x\n", res);
#endif

    return (&res);
}


#if (OS == VXWORKS)
int WTX_uDbg_SetBreakpoint(uDbg_Handle_T dbgH, uDbg_Bkpt_T *breakpoint)
{
	Debug *dbgObj = (Debug *)dbgH;

    if(!Debug::ValidObject(dbgObj)) return(UDBG_BADARG);

#ifdef _DBG_PRINT
	printf("WTX_uDbg_SetBreakpoint\n");
#endif

    breakpoint->callback = WTX_BKCallback;

    Save_dbgH = dbgH;

    return(dbgObj->SetBreakpoint(breakpoint));
}
#endif
