/* rs_udbg_lib.cpp
 * Remote Services Debug Library - Remote Debug base class implementation
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Services Debug Library
 *  author: lwh,  12/02/98  Initial version
 *  revisions:
 *  $History: rs_udbg_lib.cpp $
 * 
 * *****************  Version 48  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 47  *****************
 * User: Ewalker      Date: 9/25/00    Time: 8:31a
 * Updated in $/dev_1200/Share/rs_udebug
 * Better protection against using invalid debug object handle
 * 
 * *****************  Version 45  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 44  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:55
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 43  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:16
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 40  *****************
 * User: Lhui         Date: 1/10/00    Time: 10:13a
 * Updated in $/dev_1200/Share/rs_udebug
 * Add serial port support
 * 
 * *****************  Version 39  *****************
 * User: Lhui         Date: 10/25/99   Time: 6:07p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix a showstopper bug SCR#770
 * 
 * *****************  Version 38  *****************
 * User: Lhui         Date: 10/22/99   Time: 2:33p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix a bug in GetMemSize function
 * 
 * *****************  Version 37  *****************
 * User: Lhui         Date: 10/19/99   Time: 6:24p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix the problem with the invalid counter value 
 * 
 * *****************  Version 36  *****************
 * User: Lhui         Date: 10/19/99   Time: 5:16p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for uDbg_GetSysMemSize()
 * 
 * *****************  Version 35  *****************
 * User: Lhui         Date: 10/15/99   Time: 5:51p
 * Updated in $/dev_1200/share/rs_udebug
 * Fix the problem with breakpoint callback after re-open the WB project
 * 
 * *****************  Version 34  *****************
 * User: Lhui         Date: 10/06/99   Time: 4:21p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for new fifo read/write functions
 * 
 * *****************  Version 32  *****************
 * User: Lhui         Date: 7/23/99    Time: 8:31p
 * Updated in $/dev_1200/share/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 30  *****************
 * User: Lhui         Date: 6/29/99    Time: 7:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for uDbg_ClrReset
 * 
 * *****************  Version 29  *****************
 * User: Lhui         Date: 6/18/99    Time: 6:27p
 * Updated in $/dev_1200/Share/rs_udebug
 * Update Control API
 * 
 * *****************  Version 27  *****************
 * User: Lhui         Date: 4/22/99    Time: 5:42p
 * Updated in $/development/rs_udebug
 * Add a delay parameter to  rs_udebug/control  hop functions. 
 * 
 * *****************  Version 26  *****************
 * User: Lhui         Date: 4/08/99    Time: 5:51p
 * Updated in $/development/rs_udebug
 * Change rs_cntl_pause prototypes
 * 
 * *****************  Version 25  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:08p
 * Updated in $/development/rs_udebug
 * Fix bugs in rs_uDbg_GetUengPCs and rs_uDbg_InitLib 
 * 
 * *****************  Version 24  *****************
 * User: Lhui         Date: 4/02/99    Time: 2:47p
 * Updated in $/development/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 23  *****************
 * User: Lhui         Date: 4/01/99    Time: 6:25p
 * Updated in $/development/rs_udebug
 * Fixed bugs in ReadNumuWords() and GetMem64()
 * 
 * *****************  Version 22  *****************
 * User: Lhui         Date: 3/29/99    Time: 6:04p
 * Updated in $/development/rs_udebug
 * Add supports for debug control functions (start & hop)
 * 
 * *****************  Version 21  *****************
 * User: Lhui         Date: 3/23/99    Time: 6:37p
 * Updated in $/development/rs_udebug
 * Code cleanup and bug fixing
 * 
 * *****************  Version 20  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:48p
 * Updated in $/development/rs_udebug
 * update COPYRIGHT
 * 
 * *****************  Version 19  *****************
 * User: Qabuild      Date: 3/19/99    Time: 4:12p
 * Updated in $/development/rs_udebug
 * update makefile dependencies
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 3/19/99    Time: 1:17p
 * Updated in $/development/rs_udebug
 * Add supports for Scomm.lib
 * 
 * *****************  Version 17  *****************
 * User: Lhui         Date: 3/18/99    Time: 5:48p
 * Updated in $/development/rs_udebug
 * Add supports for SA1200 debug control functions
 * 
 * *****************  Version 14  *****************
 * User: Lhui         Date: 3/12/99    Time: 2:04p
 * Updated in $/development/rs_udebug
 * Create an  include file for external interface
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 3/09/99    Time: 6:23p
 * Updated in $/development/rs_udebug
 * bug fixing
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 3/01/99    Time: 8:42p
 * Updated in $/development/rs_udebug
 * Fix prototypes
 * 
 * *****************  Version 11  *****************
 * User: Lhui         Date: 2/25/99    Time: 8:52p
 * Updated in $/development/rs_udebug
 * Updated for the latest version of udebug related changes
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 2/22/99    Time: 7:51p
 * Updated in $/development/rs_udebug
 * Add supports for multiple Core components
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 1/28/99    Time: 6:22p
 * Updated in $/development/rs_udebug
 * Add supports for "C" interface and new udebug functions
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 1/19/99    Time: 5:35p
 * Updated in $/development/rs_udebug
 * update for udebug Get/SetCSR prototype
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 12/18/98   Time: 5:23p
 * Updated in $/development/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 12/09/98   Time: 7:17p
 * Updated in $/development/rs_udebug
 * Modify udebug calls to reflect udebug's API changes
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 12/07/98   Time: 3:04p
 * Updated in $/development/rs_udebug
 * Fix misc. bugs
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 12/03/98   Time: 10:18p
 * Updated in $/development/rs_udebug
 * fix bugs
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 12/03/98   Time: 5:49p
 * Updated in $/users/Hui/SA-1200/rs_udebug
 * Move files to development area
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 12/03/98   Time: 1:56p
 * Created in $/users/Hui/SA-1200/rs_udebug
 */
 
#include <iostream.h>
#include <time.h>

#include "utils.h"
#include "rs_udbg.h"
#include "rs_udbg_events.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

/*-----------------------------------------------------------------------------
   Function: 	RS_Debug::RS_Debug
   Description: Debug class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_Debug::RS_Debug(int configuration)
{
    for (int i = 0; i < UDBG_MAX_BREAKPOINTS; i++) bkCkPts[i].callback = NULL;
    uDbg_Handle_svr = 0;
    bkPtThread_shutdown_flag = 0;
    bkPtThreadId = NULL;
    Status = RS_SUCCESS;
    myTarget = NULL;
	config = configuration;
	Valid = 1;
    return;
}

RS_Debug::~RS_Debug()
{
    if(bkPtThreadId != NULL){
        Thread_Cancel(bkPtThreadId);
    }

    return;
}

