/* rs_udbg_rpc.cpp
 * Remote Services Debug Library - RPC support
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Sevices Debug Library
 *  author: lwh,  12/15/99  Initial version
 *  revisions:
 *  $History: rs_udbg_rpc.cpp $
 * 
 * *****************  Version 16  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 15  *****************
 * User: Ewalker      Date: 10/31/00   Time: 4:11p
 * Updated in $/dev_1200/share/rs_udebug
 * 
 * *****************  Version 14  *****************
 * User: Ewalker      Date: 10/31/00   Time: 12:11p
 * Updated in $/dev_1200/share/rs_udebug
 * C0 2k ustore support - made InitLib backwards compatible, supporting
 * only segment 1 settings. Apps must use DefineFreeUstore to set segment
 * 2 values if UpdateFreeUstore is not used.  
 * 
 * *****************  Version 13  *****************
 * User: Ewalker      Date: 10/27/00   Time: 3:52p
 * Updated in $/dev_1200/share/rs_udebug
 * Result buffer pointers didn't need to be static. Also only copy actual
 * number of
 * array values returned
 * 
 * *****************  Version 11  *****************
 * User: Ewalker      Date: 9/22/00    Time: 8:55a
 * Updated in $/dev_1200/Share/rs_udebug
 * Removed some obsolete code (already commented out)
 * 
 * *****************  Version 10  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:40
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 9  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:16
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 5/05/00    Time: 8:01p
 * Updated in $/dev_1200/share/rs_udebug
 * Fix the crash problem with "stop debug, close project, open project and
 * then start debug again" 
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:49p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:47p
 * Updated in $/Beta_4/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 1/12/00    Time: 3:54p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * cleanup
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 1/06/00    Time: 6:07p
 * Updated in $/dev_1200/share/rs_udebug
 * Add support for VxWorks serial port 
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 12/15/99   Time: 4:51p
 * Created in $/dev_1200/share/rs_udebug
 * 
 */
 
#include <iostream.h>
#include <time.h>

#include "utils.h"
#include "debug_1200.h"
#include "rs_udbg.h"
#include "rs_udbg_rpc.h"
#include "rs_udebug_clnt.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

extern "C" {
int Svc_Setup();
void DbgCK_main();
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::RS_uDbg_RPC
   Description: Debug class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbg_RPC::RS_uDbg_RPC(void *rs_cl, int config, char *hostname, uDbg_uStoreFree_T *puStoreFree)
: RS_Debug(config)

{
	rs_uDbg_uStoreFree_T uStoreFree;
    int *result;

    SetuDbg_Handle(NULL);
	SetValid(0);

    uStoreFree.ClientName = hostname;

    if(puStoreFree != NULL){ 
	    uStoreFree.freeUpc.freeUpc_len = MAX_UENGS;
	    uStoreFree.freeUpc.freeUpc_val = &puStoreFree->freeUpc[0];
	    uStoreFree.freeSize.freeSize_len= MAX_UENGS;
	    uStoreFree.freeSize.freeSize_val = &puStoreFree->freeSize[0];
        // no support for setting segment 2 through rs_udbg_initlib
    }
    else {
	    uStoreFree.freeUpc.freeUpc_len = 0;
	    uStoreFree.freeUpc.freeUpc_val = 0;
	    uStoreFree.freeSize.freeSize_len= 0;
	    uStoreFree.freeSize.freeSize_val = 0;
        // no support for setting segment 2 through rs_udbg_initlib
   }

    if (Svc_Setup() != RS_SUCCESS) return;

    cl = (CLIENT *)rs_cl;
	result = rs_udbg_initlib_c(&uStoreFree, cl);

	if(CheckStatus(result) != RS_SUCCESS)
		return;

	// save server debugger handle 
	SetuDbg_Handle(*result);

    // create event service thread
    bkPtThread_shutdown_flag = 0;
    SetBKThreadID(Thread_Create(MID_PRIORITY, (threadFunc_T)DbgCK_main, &bkPtThread_shutdown_flag));

    SetValid(1);
    return;
}

RS_uDbg_RPC::~RS_uDbg_RPC()
{    
    rs_uDbg_Handle_T hSvr;

    // delete remote debug object
    if (IsValid()){        
        hSvr.uDbg_Handle = GetuDbg_Handle();
        if(hSvr.uDbg_Handle != NULL)
            rs_udbg_delelib_c(&hSvr, cl);

        // shutdown breakpoint thread waiting for at most 2 seconds
        if (GetBKThreadID()){
            bkPtThread_shutdown_flag = 1;
            if (WaitForSingleObject(GetBKThreadID(), 2000) == WAIT_OBJECT_0)
                SetBKThreadID(0);
        }
    }

    return;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetUthreadPC
   Description: Get the program-counter of the specified thread.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetUthreadPC(unsigned char uEng, unsigned char ctx, unsigned int *upc)
{
    rs_uDbg_GetThPC_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
    Args.ctx = ctx;
    Args.regData.regData_len = 0;

    result = rs_udbg_getuthreadpc_c(&Args,cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    *upc = *result->resData.resData_val; 

    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetUword
   Description: Read micro-engine control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetUword(unsigned char uEng, unsigned int uWordAddr, unsigned int *uWord)
{
    rs_uDbg_uWord_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
    Args.uWordAddr = uWordAddr;
    Args.numWords = 0;
    Args.u_Word.u_Word_len = 0;

    result = rs_udbg_getuword_c(&Args,cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    *uWord = *result->resData.resData_val; 

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::SetUword
   Description: Write to micro-engines control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::SetUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned int uWord)
{
    rs_uDbg_uWord_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEngMask;
    Args.uWordAddr = uWordAddr;
    Args.numWords = 0;
    Args.u_Word.u_Word_len = sizeof(int);
    Args.u_Word.u_Word_val = &uWord;

    result = rs_udbg_setuword_c(&Args,cl);

	if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetNumUword
   Description: Read micro-engine control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetNumUword(unsigned char uEng, unsigned int uWordAddr,
					   unsigned short numWords, unsigned int *uWord)
{
    rs_uDbg_uWord_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
    Args.uWordAddr = uWordAddr;
    Args.numWords = numWords;
    Args.u_Word.u_Word_len = 0;

    result = rs_udbg_getnumuword_c(&Args,cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(uWord, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int));

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::SetNumUword
   Description: Write to micro-engines control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::SetNumUword(unsigned int uEngMask, unsigned int uWordAddr,
					unsigned short numWords, unsigned int *uWord)
{
    rs_uDbg_uWord_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEngMask;
    Args.uWordAddr = uWordAddr;
    Args.numWords = numWords;
    Args.u_Word.u_Word_len = numWords;
    Args.u_Word.u_Word_val = uWord;
 
    result = rs_udbg_setnumuword_c(&Args,cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::SetBreakpoint
   Description: Set a breakpoint and return the breakpoint identification.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::SetBreakpoint(uDbg_Bkpt_T *bp)
{
    rs_uDbg_BreakPoint_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.pBpoint.id = 0;
    Args.pBpoint.callback = 0;
    Args.pBpoint.ctx = bp->ctx;
    Args.pBpoint.uAddr = bp->uAddr;
    Args.pBpoint.uEng = bp->uEng;
	Args.pBpoint.userData = (unsigned int)bp->usrData;
	Args.pBpoint.breakIfCtx = bp->breakIfCtx;
	Args.pBpoint.stpOnBrkUengMask = bp->stpOnBrkUengMask;

    result = rs_udbg_setbreakpoint_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    bp->id = (short)*result->resData.resData_val;

    SetCkRtn(bp->id, bp->callback);

    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::ShowBreakpoints
   Description: Return the defined breakpoints.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::ShowBreakpoints(unsigned int uEngMask, uDbg_BkptArray_T *breakpoints)
{
    rs_uDbg_One_uint_T Args;
    rs_uDbg_ShowBreakpts_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.arg1 = uEngMask;

    result = rs_udbg_showbreakpoints_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    breakpoints->len = result->BkptArray.len;

    memmove(breakpoints->bkpts, (void *)result->BkptArray.bkpts.bkpts_val, result->BkptArray.bkpts.bkpts_len*sizeof(rs_uDbg_Bkpt_T));
 
    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::WriteMem64
   Description: Write a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::WriteMem64(unsigned int addr, unsigned int *data, unsigned int count)
{
    rs_uDbg_Mem_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.addr = addr;
    Args.data.data_len = count*2;
    Args.data.data_val = data;

    result = rs_udbg_writemem64_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return(*result);
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::ReadMem64
   Description: read a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::ReadMem64(unsigned int addr, unsigned int *data, unsigned int count)
{
    rs_uDbg_Mem_T Args;
    rs_uDbg_Status_T *result;
 
    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.addr = addr;
    Args.data.data_len = count*2;
    Args.data.data_val = data;

    result = rs_udbg_readmem64_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(data, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::WriteMem32
   Description: Write a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::WriteMem32(unsigned int addr, unsigned int *data, unsigned int count)
{
    rs_uDbg_Mem_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.addr = addr;
    Args.data.data_len = count;
    Args.data.data_val = data;

    result = rs_udbg_writemem32_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return(*result);
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::ReadMem32
   Description: read a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::ReadMem32(unsigned int addr, unsigned int *data, unsigned int count)
{
    rs_uDbg_Mem_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.addr = addr;
    Args.data.data_len = count;
    Args.data.data_val = data;

    result = rs_udbg_readmem32_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(data, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int));

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::PutRelGPR
   Description: Write a long-word to a micro-engine GPR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::PutRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = bankType;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;
 
    result = rs_udbg_putrelgpr_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return(*result);
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetRelGPR
   Description: Read a long-word from a micro-engine GPR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned count)
{
    rs_uDbg_Reg_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = bankType;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_getrelgpr_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(regData, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetRelSRAM_WrXfer
   Description: Read a long-word from a micro-engine SRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/

int RS_uDbg_RPC::GetRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_getrelsram_wrxfer_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(regData, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 
 
    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::PutRelSRAM_WrXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::PutRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_putrelsram_wrxfer_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetRelSDRAM_WrXfer
   Description: Read a long-word from a micro-engine SDRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_getrelsdram_wrxfer_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(regData, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::PutRelSDRAM_WrXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::PutRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_putrelsdram_wrxfer_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetRelSRAM_RdXfer
   Description: Read a long-word from a micro-engine SRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_getrelsram_rdxfer_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(regData, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::PutRelSRAM_RdXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::PutRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_putrelsram_rdxfer_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetRelSDRAM_RdXfer
   Description: Read a long-word from a micro-engine SDRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_getrelsdram_rdxfer_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(regData, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::PutRelSDRAM_RdXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::PutRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData, unsigned int count)
{
    rs_uDbg_Reg_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.ctx = ctx;
	Args.bankType = 0;
	Args.regAddr = regNum;
	Args.regData.regData_len = count;
	Args.regData.regData_val = regData;

    result = rs_udbg_putrelsdram_rdxfer_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::SetUengPCs
   Description: Set ueng PCs
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::SetUengPCs(unsigned char uEng, unsigned int *upc)
{
    rs_uDbg_Mem_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.addr = (unsigned int)uEng;
    Args.data.data_len = MAX_CTX;
    Args.data.data_val = upc;

    result = rs_udbg_setuengpc_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return(*result);
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetUengPCs
   Description: Read CSR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetUengPCs(unsigned char uEng, unsigned int *upc)
{
    rs_uDbg_One_uint_T Args;
    rs_uDbg_Status_T *result;
 
    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.arg1 = uEng;

    result = rs_udbg_getuengpc_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(upc, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 

    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetCSR
   Description: Read CSR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetCSR(unsigned int csrAddr, unsigned int *csrData)
{
    rs_uDbg_CSR_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.csrAddr = csrAddr;
	Args.csrData.csrData_val = 0;
	Args.csrData.csrData_len = 0;

    result = rs_udbg_getcsr_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    *csrData = *result->resData.resData_val; 

    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::SetCSR
   Description: Write to CSR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::SetCSR(unsigned int csrAddr, unsigned int csrData)
{
    rs_uDbg_CSR_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.csrAddr = csrAddr;
	Args.csrData.csrData_val = &csrData;
	Args.csrData.csrData_len = 1;

    result = rs_udbg_setcsr_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::GetUengStatus
   Description: Retreive the status of the specified micro-engine.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetUengStatus(unsigned char uEng, uDbg_uEngStatus_T *uEngStatus)
{
    rs_uDbg_One_uint_T Args;
    rs_uDbg_GetuEngStatus_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.arg1 = uEng;

    result = rs_udbg_getuengstatus_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    uEngStatus->state = result->uEngStatus.state;
	uEngStatus->ctx = result->uEngStatus.activeCtx;
	uEngStatus->enables = result->uEngStatus.enables;
	uEngStatus->breakId = result->uEngStatus.breakId;
	uEngStatus->pc = result->uEngStatus.activePC;

    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::HopUeng
   Description: Hop microengine
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::HopUeng(unsigned int *uEngMask, unsigned int delay)

{
    rs_uDbg_Two_uint_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.arg1 = *uEngMask;
    Args.arg2 = delay;

    result = rs_udbg_hopueng_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    *uEngMask = (unsigned int)*result->resData.resData_val;

    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::WriteFIFO_Tx
   Description: Write a number of quad-words to micro-engine transfer-fifo.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::WriteFIFO_Tx(unsigned char uEng, unsigned int fifoAddr,
						unsigned int *data64, unsigned int count, int setValid)
{
    rs_uDbg_FiFo_T Args;
    int *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.fifoAddr = fifoAddr;
	Args.data64.data64_len = count*2;
	Args.data64.data64_val = data64;
	Args.count = count;

    result = rs_udbg_writefifo_tx_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::ReadFIFO_Rx
   Description: Read a number of quad-words from micro-engine receive-fifo.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::ReadFIFO_Rx(unsigned char uEng, unsigned int fifoAddr,
						unsigned int *data64, unsigned int count)
{
    rs_uDbg_FiFo_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	Args.uEng = uEng;
	Args.fifoAddr = fifoAddr;
	Args.data64.data64_len = count*2;
	Args.data64.data64_val = data64;
	Args.count = count;

    result = rs_udbg_readfifo_rx_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    memmove(data64, (void *)result->resData.resData_val, result->resData.resData_len*sizeof(int)); 

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	Debug::GetSysMemSize
   Description: This routine returns sizes of virtual memory regions in the following
				pointers:
					sys     -- Size of memory mapped at zero for OS
					sys_all -- Size of memory mapped at zero and usable by the OS
					sdram   -- Size of memory mapped at C0000000 and usable by HW
					sram    -- Size of sram
					flash   -- Size of Flash
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_TIMEOUT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::GetSysMemSize(unsigned int* sys, unsigned int* sys_all, unsigned int* sdram,
							unsigned int* sram,unsigned int* flash)
{
	rs_uDbg_Handle_T Args;
    rs_uDbg_GetMemSize_T *result;

    Args.uDbg_Handle = GetuDbg_Handle();

    result = rs_udbg_getsysmemsize_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    *sys = result->sys;
	*sys_all = result->sys_all;
	*sdram = result->sdram;
	*sram = result->sram;
	*flash = result->flash;

    return result->res;  
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::CNTL_Hop
   Description: Hop microengine
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::CNTL_Hop(unsigned int *uEngMask, unsigned int portMask, unsigned int delay)

{
    rs_uDbg_Three_uint_T Args;
    rs_uDbg_Status_T *result;

    Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
    Args.arg1 = *uEngMask;
	Args.arg2 = portMask;
    Args.arg3 = delay;

    result = rs_cntl_hop_c(&Args, cl);

    if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();

    *uEngMask = (unsigned int)*result->resData.resData_val;

    return result->res;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_RPC::CNTL_InitUof
   Description: Load uCode
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_RPC::CNTL_InitUof(char* buf, int buflen)
{
    rs_CNTL_InitUof_T Args;
    int *result;

    Args.UofData.UofData_len = buflen;
	Args.UofData.UofData_val = buf;

    result = rs_cntl_inituof_c(&Args, cl);

    if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

    return *result;  
}
