/* rs_udbg_serial.h
 * Remote Services Debug Library - Serial support
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Services Debug Library
 *  author: lwh, 12/12/99  Initial version
 *  revisions:
 *  $History: rs_udbg_serial.h $
 * 
 * *****************  Version 10  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 8  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 7  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:55
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 6  *****************
 * User: Ewalker      Date: 8/30/00    Time: 14:10
 * Updated in $/dev_1200/Share/rs_udebug
 * About to change name to rs_udbg_serial.h
 * 
 * *****************  Version 5  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:16
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 1/07/00    Time: 9:57a
 * Created in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port
 */ 

#ifndef RS_UDBG_SERIAL_H
#define RS_UDBG_SERIAL_H

#include "debug_1200.h"
#include "rs_udbg_status.h"
#include "rs_udbg.h"
#include "rs_ScMsg.h"

#ifdef __cplusplus
class RS_uDbg_SERIAL : public RS_Debug {
public:
   RS_uDbg_SERIAL(void *rs_cl, int config, char *devnam, uDbg_uStoreFree_T *puStoreFree);
   ~RS_uDbg_SERIAL();

    /* virtual functions */
	virtual int SetUengPCs(unsigned char uEng, unsigned int *upc);
	virtual int GetUengPCs(unsigned char uEng, unsigned int *upc);
	virtual int GetUword(unsigned char uEng, unsigned int uWordAddr, unsigned int *uWord);
	virtual int SetUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned int uWord);
	virtual int GetNumUword(unsigned char uEng, unsigned int uWordAddr, unsigned short numWords,unsigned int *uWord);
	virtual int SetNumUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned short numWords, unsigned int *uWord);
	virtual int GetCSR(unsigned int csrAddr, unsigned int *csrData);
	virtual int SetCSR(unsigned int csrAddr, unsigned int csrData);
	virtual int SetBreakpoint(uDbg_Bkpt_T *breakpoint);
	virtual int ShowBreakpoints(unsigned int uEngMask, uDbg_BkptArray_T *breakpoints);
	virtual int WriteMem32(unsigned int addr, unsigned int *data, unsigned count);
	virtual int ReadMem32(unsigned int addr, unsigned int *data, unsigned count);
	virtual int WriteMem64(unsigned int addr, unsigned int *data, unsigned count);
	virtual int ReadMem64(unsigned int addr, unsigned int *data, unsigned count);
	virtual int PutRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int HopUeng(unsigned int *uEngMask, unsigned int delay);
	virtual int GetUengStatus(unsigned char uEng, uDbg_uEngStatus_T *uEngStatus);
	virtual int WriteFIFO_Tx(unsigned char uEng, unsigned int fifoAddr,unsigned int *regData, unsigned int count, int setValid);
	virtual int ReadFIFO_Rx(unsigned char uEng, unsigned int fifoAddr, unsigned int *regData, unsigned int count);
	virtual int GetSysMemSize(unsigned int* sys, unsigned int* sys_all, unsigned int* sdram,
				 unsigned int* sram,unsigned int* flash);
	virtual int CNTL_InitUof(char* buf, int buflen);
	virtual int CNTL_Hop(unsigned int *uEngMask, unsigned int PortMask, unsigned int delay);
	virtual int GetUthreadPC(unsigned char uEng, unsigned char ctx, unsigned int *upc);



    /* inline functions */
   	virtual inline int ResetUeng(unsigned int uEngMask, int clrReg)
    {
	      return CommandToSvr_Arg_2(rs_udbg_resetueng_sc, uEngMask, clrReg);
	};

   	virtual inline int DefineFreeUstore(unsigned int uEngMask, unsigned int freeUaddr, unsigned int freeSize)
    {
	      return CommandToSvr_Arg_3(rs_udbg_definefreeustore_sc, uEngMask, freeUaddr, freeSize);
    };

   	virtual inline int UpdateFreeUstore(unsigned int uEngMask)
    {
	      return CommandToSvr_Arg_1(rs_udbg_updatefreeustore_sc, uEngMask);
    };
 
   	virtual inline int ClrResetUeng(unsigned int uEngMask)
    {
 	      return CommandToSvr_Arg_1(rs_udbg_clrresetueng_sc, uEngMask);
    };

   	virtual inline int StartUeng(unsigned int uEngMask)
	{
		return CommandToSvr_Arg_1(rs_udbg_startueng_sc, uEngMask);
    };

   	virtual inline int PauseUeng(unsigned int uEngMask, unsigned int delay)
    {
       return CommandToSvr_Arg_2(rs_udbg_pauseueng_sc, uEngMask, delay);
    };

   	virtual inline int ResumeUeng(unsigned int uEngMask)
    {
       return CommandToSvr_Arg_1(rs_udbg_resumeueng_sc, uEngMask);
    };

   	virtual inline int GetActiveUengs(unsigned int uEngMask)
    {
	   return CommandToSvr_Arg_1(rs_udbg_getactiveuengs_sc, uEngMask);
    };
   	virtual inline int SetUthreadsPC(unsigned int uEngMask, unsigned int ctxMask, unsigned int upc)
    {
       return CommandToSvr_Arg_3(rs_udbg_setuthreadspc_sc, uEngMask, ctxMask, upc);
    };

    virtual inline int IsUengActive(unsigned int uEngMask)
    {
      return CommandToSvr_Arg_1(rs_udbg_isuengactive_sc, uEngMask);
    };
   	virtual int ClearAllBreakpoints(unsigned int uEngMask)
    {
 		return CommandToSvr_Arg_1(rs_udbg_clearallbreakpoints_sc, uEngMask);
    };
   	virtual inline int ClearBreakpoint(short breakPointId)
    {
	      return CommandToSvr_Arg_1(rs_udbg_clearbreakpoint_sc, breakPointId);
    };
 
   	virtual inline int CNTL_Reset(unsigned int uEngMask,unsigned int PortMask, int clrReg)
    {
	      return CommandToSvr_Arg_3(*rs_cntl_reset_sc, uEngMask, PortMask, clrReg);
    };

   	virtual inline int CNTL_Pause(unsigned int uEngMask, unsigned int PortMask, unsigned int delay)
    {
       return CommandToSvr_Arg_3(*rs_cntl_pause_sc, uEngMask,PortMask, delay);
    };
    virtual inline int CNTL_Resume(unsigned int uEngMask,unsigned int PortMask)
    {
      return CommandToSvr_Arg_2(*rs_cntl_resume_sc, uEngMask,PortMask);
    };
    virtual inline int CNTL_Start(unsigned int uEngMask,unsigned int PortMask)
    {
		return CommandToSvr_Arg_2(*rs_cntl_start_sc, uEngMask,PortMask);
    };
   
   int  CommandToSvr_Arg_1(int *(uDbgRtn)(rs_uDbg_One_uint_T *argp, Sc_CLIENT *clnt), const int arg1) 
   {
       static int *result;
       rs_uDbg_One_uint_T Args;
	   Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	   Args.arg1 = arg1;
       result = uDbgRtn(&Args, scl);
       if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();
	
	    /* return status from debugger server */
	    return *result;
   };

 
    int CommandToSvr_Arg_1_Oval(rs_uDbg_Status_T *(uDbgRtn)(rs_uDbg_One_uint_T *argp, Sc_CLIENT *clnt), const int arg1, unsigned int *Oval) 
    {
       static rs_uDbg_Status_T *result;
       rs_uDbg_One_uint_T Args;
	   Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	   Args.arg1 = arg1;
       result = uDbgRtn(&Args, scl);
       if(CheckStatus((int *)result) != RS_SUCCESS)
		  return GetStatus();
	
	    /* return status from debugger server */
		memmove(Oval, result->resData.resData_val, result->resData.resData_len);
	    return result->res;
    };

    int  CommandToSvr_Arg_2(int *(uDbgRtn)(rs_uDbg_Two_uint_T *argp, Sc_CLIENT *clnt), const int arg1, const int arg2) 
    {
       static int *result;
       rs_uDbg_Two_uint_T Args;
	   Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	   Args.arg1 = arg1;
	   Args.arg2 = arg2;

       result = uDbgRtn(&Args, scl);
       if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();
	
	   /* return status from debugger server */
	   return *result;
    };

    int  CommandToSvr_Arg_3(int *(uDbgRtn)(rs_uDbg_Three_uint_T *argp,Sc_CLIENT *clnt), const int arg1, const int arg2, const int arg3) 
    {
       static int *result;
       rs_uDbg_Three_uint_T Args;
	   Args.hRs_uDbg.uDbg_Handle = GetuDbg_Handle();
	   Args.arg1 = arg1;
	   Args.arg2 = arg2;
	   Args.arg3 = arg3;
       result = uDbgRtn(&Args, scl);
       if(CheckStatus(result) != RS_SUCCESS)
		  return GetStatus();

       /* return status from debugger server */
	   return *result;
    };
private:
	   Sc_CLIENT   *scl;
};


#endif          /* __cplusplus */
#endif			/* RS_UDBG_SERIAL_H */
