/* rs_udbg_wtx.cpp
 * Remote Services Debug Library - VxWorks WTX support
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Services Debug Library
 *  author: lwh,  12/15/99  Initial version
 *  revisions:
 *  $History: rs_udbg_wtx.cpp $
 * 
 * *****************  Version 17  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 16  *****************
 * User: Ewalker      Date: 10/31/00   Time: 12:11p
 * Updated in $/dev_1200/share/rs_udebug
 * C0 2k ustore support - made InitLib backwards compatible, supporting
 * only segment 1 settings. Apps must use DefineFreeUstore to set segment
 * 2 values if UpdateFreeUstore is not used.  
 * 
 * *****************  Version 15  *****************
 * User: Ewalker      Date: 9/26/00    Time: 8:32a
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 13  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 12  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:55
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 11  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:16
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:49p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:47p
 * Updated in $/Beta_4/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 1/17/00    Time: 10:46a
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 1/17/00    Time: 10:45a
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 1/12/00    Time: 4:14p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 1/06/00    Time: 6:07p
 * Updated in $/dev_1200/share/rs_udebug
 * Add support for VxWorks serial port 
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 12/21/99   Time: 5:15p
 * Created in $/dev_1200/share/rs_udebug
 * 
 */
 
#ifdef VXWORKS_SUPPORT

#include <iostream.h>
#include <time.h>
#include <string.h>

#include <host.h>
#include <wtx.h>

#include "utils.h"
#include "debug_1200.h"
#include "rs_udbg.h"
#include "rs_udbg_wtx.h"
#include "rs_udebug_clnt.h"
#include "rs_uDbg_events.h"

#include "rs_target.h"  // this is a bit circular with rs_udbg_*.h

extern "C" {
void *rs_udbg_bkpt_ck_1(rs_uDbg_Bkpt_CK_T *argp);
}

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

/* these statics are used by the eventHookRtn */
static RS_uDbg_WTX *wtxObj = NULL;
static HWTX ActiveHWTX = NULL;
#define MaxBuf 250

static FUNCPTR wtx_eventHookRtn(
      WTX_EVENT_DESC * event				/* Event received from the target server */
)
{
    char *pp;
	char evtbuf[MaxBuf];
	int index;
	int Ptmp;	
	rs_uDbg_Bkpt_CK_T arg;

	if (wtxObj == NULL || ActiveHWTX == NULL) return 0;

	int eventlen = strlen(event->event);
	if (event->event != NULL){
// check if it is break point event
		for(pp=event->event, index =0; index < eventlen; index++, pp++){
			if((*pp == ':'))
				break;
		}
		// unknow event type
		if(index >= eventlen)
			return 0;
        //change ';' to '\0' 
		pp++;
// start conversion
		for(Ptmp = 0; Ptmp <= 3; Ptmp++){
			for(index=0; index<MaxBuf; pp++,index++){
				if(*pp == ';'){
					evtbuf[index]='\0';
					break;
				}
				evtbuf[index]=*pp;
			}
			switch (Ptmp){
			case 0:
				arg.bkptId = wtxObj->Fn_wtxStrToInt32(ActiveHWTX,evtbuf);
				break;
			case 1:
				arg.ueng = wtxObj->Fn_wtxStrToInt32(ActiveHWTX,evtbuf);
				break;
			case 2:
				arg.ctx = wtxObj->Fn_wtxStrToInt32(ActiveHWTX,evtbuf);
				break;
			case 3:
				arg.usrData = wtxObj->Fn_wtxStrToInt32(ActiveHWTX,evtbuf);
				break;
			default:
				break;
			}
			pp++; // advance pointer
		}


	rs_udbg_bkpt_ck_1(&arg);
	}
	return 0;   

}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::RS_uDbg_WTX
   Description: Debug class Constructor
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
RS_uDbg_WTX::RS_uDbg_WTX(void *hCl, int config,  void *hWtx_Dll)
: RS_Debug(config)

{
    SetValid(0);

    wtxh = (HWTX)hCl; 
    m_hWtx_Dll = (HINSTANCE)hWtx_Dll;
	TargetBuf = 0;

	if(m_hWtx_Dll != NULL){
		 // Get the addresses of all the exported wtx functions that we use.
        FARPROC ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxResultFree");
        if (ProcAddr == NULL) return;
        Fn_wtxResultFree = (T_wtxResultFree)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxStrToInt32");
        if (ProcAddr == NULL) return;
        Fn_wtxStrToInt32 = (T_wtxStrToInt32)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxMemAlloc");
        if (ProcAddr == NULL) return;
        Fn_wtxMemAlloc = (T_wtxMemAlloc)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxMemWrite");
        if (ProcAddr == NULL) return;
        Fn_wtxMemWrite = (T_wtxMemWrite)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxDirectCall");
        if (ProcAddr == NULL) return;
        Fn_wtxDirectCall = (T_wtxDirectCall)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxRegisterForEvent");
        if (ProcAddr == NULL) return;
        Fn_wtxRegisterForEvent = (T_wtxRegisterForEvent)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxAsyncNotifyEnable");
        if (ProcAddr == NULL) return;
        Fn_wtxAsyncNotifyEnable = (T_wtxAsyncNotifyEnable)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxMemFree");
        if (ProcAddr == NULL) return;
        Fn_wtxMemFree = (T_wtxMemFree)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxAsyncNotifyDisable");
        if (ProcAddr == NULL) return;
        Fn_wtxAsyncNotifyDisable = (T_wtxAsyncNotifyDisable)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxUnregisterForEvent");
        if (ProcAddr == NULL) return;
        Fn_wtxUnregisterForEvent = (T_wtxUnregisterForEvent)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxMemRead");
        if (ProcAddr == NULL) return;
        Fn_wtxMemRead = (T_wtxMemRead)ProcAddr;

        ProcAddr = GetProcAddress(m_hWtx_Dll, "wtxResultFree");
        if (ProcAddr == NULL) return;
        Fn_wtxResultFree = (T_wtxResultFree)ProcAddr;
	}
	if (blockSize > 0){
		if((TargetBuf = Fn_wtxMemAlloc(wtx_GetHWTX(), blockSize)) == (TGT_ADDR_T) 0) return;
	}

    SetValid(1);
    return;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::Udbg_InitLib
   Description: Call uDbb
   Returns:		
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::Udbg_InitLib(uDbg_uStoreFree_T *puStoreFree)					  
{
     int RetVal=RS_ERROR, ret=RS_ERROR;
     TGT_ADDR_T RemoteFunction;

     SetValid(0);

	 if( (RemoteFunction = wtx_GetEntry(RS_UDBG_INITLIB)) == 0)
		return ret;
	 
	 if(puStoreFree != NULL){
	        int size = sizeof(uDbg_uStoreFree_T);
            TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
			Fn_wtxMemWrite(wtx_GetHWTX(), puStoreFree, tmpTargetBuf, size);
			if((ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal,tmpTargetBuf,
				0,0,0,0,0,0,0,0,0)) != WTX_OK){
                 wtx_FreeTargetBuf(tmpTargetBuf, size);
				 return ret;
			}
		 }
	 else{
			if((ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal,NULL,
				 0,0,0,0,0,0,0,0,0)) != WTX_OK)
				 return ret;
		 }

    /* register ucode breakpoint events*/
	ActiveHWTX = wtx_GetHWTX();
	wtxObj = this;
	Fn_wtxRegisterForEvent(wtx_GetHWTX(), "IXP1200.*");
    Fn_wtxAsyncNotifyEnable(wtx_GetHWTX(), (FUNCPTR)wtx_eventHookRtn);

	// save server debugger handle 
	SetuDbg_Handle(RetVal);
    SetValid(1);
    return RS_SUCCESS;
}

RS_uDbg_WTX::~RS_uDbg_WTX()
{
    if (IsValid()){
        CommandToSvr_Arg_1(RS_UDBG_DELELIB, GetuDbg_Handle());
        Fn_wtxAsyncNotifyDisable(wtx_GetHWTX());
        Fn_wtxUnregisterForEvent(wtx_GetHWTX(), "IXP1200.*");
    }

    if (TargetBuf) Fn_wtxMemFree (wtx_GetHWTX(), TargetBuf); 

	// zero module statics
	wtxObj = NULL;
	ActiveHWTX = NULL;

    return;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetUthreadPC
   Description: Get the program-counter of the specified thread.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetUthreadPC(unsigned char uEng, unsigned char ctx, unsigned int *upc)
{
 
	int ret;
	int size=sizeof(int);
    TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	ret = CommandToSvr_Arg_3(RS_UDBG_GETUTHREADPC, uEng, ctx, tmpTargetBuf); 

   if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, upc, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
   }
   wtx_FreeTargetBuf(tmpTargetBuf, size);
   return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetUword
   Description: Read micro-engine control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetUword(unsigned char uEng, unsigned int uWordAddr, unsigned int *uWord)
{
	int ret;

	int size=sizeof(int);
    TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_3(RS_UDBG_GETUWORD, uEng, uWordAddr, tmpTargetBuf); 

   if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, uWord, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
   }

   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;
}



/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::SetUword
   Description: Write to micro-engines control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::SetUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned int uWord)
{
	int ret;

	ret = CommandToSvr_Arg_3(RS_UDBG_SETUWORD, uEngMask, uWordAddr, uWord); 

    return ret;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetNumUword
   Description: Read micro-engine control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetNumUword(unsigned char uEng, unsigned int uWordAddr,
					   unsigned short numWords, unsigned int *uWord)
{
	int ret;
	int size=numWords*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_4(RS_UDBG_GETNUMUWORD, uEng, uWordAddr, numWords, tmpTargetBuf); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, uWord, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }

   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;
}



/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::SetNumUword
   Description: Write to micro-engines control-store.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::SetNumUword(unsigned int uEngMask, unsigned int uWordAddr,
					unsigned short numWords, unsigned int *uWord)
{
	int ret;
	int size=numWords*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), uWord, tmpTargetBuf, size)) == WTX_ERROR){
	    wtx_FreeTargetBuf(tmpTargetBuf, size);
		return RS_NOMEMORY;
	}

	ret = CommandToSvr_Arg_4(RS_UDBG_SETNUMUWORD, uEngMask, uWordAddr, numWords, tmpTargetBuf); 

	wtx_FreeTargetBuf(tmpTargetBuf, size);
    return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::SetBreakpoint
   Description: Set a breakpoint and return the breakpoint identification.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::SetBreakpoint(uDbg_Bkpt_T *bp)
{

	int ret;
	uDbg_Bkpt_T tmpBp;
	int size=sizeof(uDbg_Bkpt_T);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), bp, tmpTargetBuf, size)) == WTX_ERROR)
		return RS_NOMEMORY;

    ret = CommandToSvr_Arg_1(RS_UDBG_SETBREAKPOINT, tmpTargetBuf);
	if(ret != RS_SUCCESS){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);
	   return ret;
	}
	if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, &tmpBp, size)) == WTX_ERROR){
		   wtx_FreeTargetBuf(tmpTargetBuf, size);
		   return RS_NOMEMORY; 
	}
   bp->id = tmpBp.id;
   SetCkRtn(bp->id, bp->callback);
   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;


}



/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::ShowBreakpoints
   Description: Return the defined breakpoints.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::ShowBreakpoints(unsigned int uEngMask, uDbg_BkptArray_T *breakpoints)
{
	int ret;

	int size=sizeof(uDbg_BkptArray_T);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_2(RS_UDBG_SHOWBREAKPOINTS, uEngMask, tmpTargetBuf); 
    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, breakpoints, size)) == WTX_ERROR)
		   ret= RS_NOMEMORY;
    }
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;

}





/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::WriteMem64
   Description: Write a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::WriteMem64(unsigned int addr, unsigned int *data, unsigned int count)
{
	int ret;
	int size=count*sizeof(int)*2;
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), data, tmpTargetBuf, size )) == WTX_ERROR){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);
		return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_3(RS_UDBG_WRITEMEM64, addr, tmpTargetBuf, count); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::ReadMem64
   Description: read a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::ReadMem64(unsigned int addr, unsigned int *data, unsigned int count)
{
	int ret;

	int size=count*sizeof(int)*2;
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_3(RS_UDBG_READMEM64, addr, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, data, size)) == WTX_ERROR)
		   ret= RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);


   return ret;
 
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::WriteMem32
   Description: Write a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::WriteMem32(unsigned int addr, unsigned int *data, unsigned int count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), data, tmpTargetBuf, size )) == WTX_ERROR){
 	    wtx_FreeTargetBuf(tmpTargetBuf, size);
		return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_3(RS_UDBG_WRITEMEM32, addr, tmpTargetBuf, count); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::ReadMem32
   Description: read a long-word to memory.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::ReadMem32(unsigned int addr, unsigned int *data, unsigned int count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_3(RS_UDBG_READMEM32, addr, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, data, size)) == WTX_ERROR)
		   ret= RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::PutRelGPR
   Description: Write a long-word to a micro-engine GPR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::PutRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned int count)
{

	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), regData, tmpTargetBuf, size )) == WTX_ERROR){
 	    wtx_FreeTargetBuf(tmpTargetBuf, size);
		return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_6(RS_UDBG_PUTRELGPR, uEng, ctx, bankType, regNum, tmpTargetBuf, count); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetRelGPR
   Description: Read a long-word from a micro-engine GPR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
				  unsigned int regNum, unsigned int *regData, unsigned count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_6(RS_UDBG_GETRELGPR, uEng, ctx, bankType, regNum, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, regData, size)) == WTX_ERROR)
		   ret= RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetRelSRAM_WrXfer
   Description: Read a long-word from a micro-engine SRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/

int RS_uDbg_WTX::GetRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,unsigned int count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_5(RS_UDBG_GETRELSRAM_WRXFER, uEng, ctx, regNum, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, regData, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::PutRelSRAM_WrXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::PutRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData,unsigned int count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), regData, tmpTargetBuf, size )) == WTX_ERROR){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);
 	   return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_5(RS_UDBG_PUTRELSRAM_WRXFER, uEng, ctx, regNum, tmpTargetBuf, count); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}



/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetRelSDRAM_WrXfer
   Description: Read a long-word from a micro-engine SDRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,unsigned int count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_5(RS_UDBG_GETRELSDRAM_WRXFER, uEng, ctx, regNum, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, regData, size)) == WTX_ERROR)
		   ret= RS_NOMEMORY;
    }
  wtx_FreeTargetBuf(tmpTargetBuf, size);


   return ret;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::PutRelSDRAM_WrXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::PutRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData,unsigned int count)
{

	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), regData, tmpTargetBuf, size )) == WTX_ERROR){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);
	   return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_5(RS_UDBG_PUTRELSDRAM_WRXFER, uEng, ctx, regNum, tmpTargetBuf, count); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetRelSRAM_RdXfer
   Description: Read a long-word from a micro-engine SRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,unsigned int count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_5(RS_UDBG_GETRELSRAM_RDXFER, uEng, ctx, regNum, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, regData, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);


   return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::PutRelSRAM_RdXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::PutRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData,unsigned int count)
{
	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), regData, tmpTargetBuf, size )) == WTX_ERROR){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);
		return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_5(RS_UDBG_PUTRELSRAM_RDXFER, uEng, ctx, regNum, tmpTargetBuf, count); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}



/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetRelSDRAM_RdXfer
   Description: Read a long-word from a micro-engine SDRAM xfer reg.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
				  unsigned int regNum, unsigned int *regData,unsigned int count)
{
	int ret;

	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	ret = CommandToSvr_Arg_5(RS_UDBG_GETRELSDRAM_RDXFER, uEng, ctx, regNum, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, regData, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }

   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;
}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::PutRelSDRAM_RdXfer
   Description: Write regData to a micro-engine write transfer register
				located at regAddr. The caller may need to translate from a
				symbolic name to register number prior to using this function.
				Legal values for regNum are 0-7.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::PutRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx, unsigned int regNum,
						   unsigned int *regData,unsigned int count)
{

	int ret;
	int size=count*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	if((Fn_wtxMemWrite(wtx_GetHWTX(), regData, tmpTargetBuf, size )) == WTX_ERROR){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);

		return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_5(RS_UDBG_PUTRELSDRAM_RDXFER, uEng, ctx, regNum, tmpTargetBuf, count); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}
/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::SetUengPCs
   Description: Set ueng PCs
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::SetUengPCs(unsigned char uEng, unsigned int *upc)
{
	int ret;
	int size=sizeof(int)*MAX_CTX;
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	if((Fn_wtxMemWrite(wtx_GetHWTX(), upc, tmpTargetBuf, size )) == WTX_ERROR){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);
	   return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_2(RS_UDBG_SETUENGPC, uEng, tmpTargetBuf); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetUengPCs
   Description: Read CSR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetUengPCs(unsigned char uEng, unsigned int *upc)
{
	int ret;

	int size=MAX_CTX*sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_2(RS_UDBG_GETUENGPC, uEng,tmpTargetBuf); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, upc, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetCSR
   Description: Read CSR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetCSR(unsigned int csrAddr, unsigned int *csrData)
{
	int ret;
	int size=sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	ret = CommandToSvr_Arg_2(RS_UDBG_GETCSR, csrAddr, tmpTargetBuf); 
    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, csrData, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;
}



/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::SetCSR
   Description: Write to CSR.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::SetCSR(unsigned int csrAddr, unsigned int csrData)
{
	int ret;

	ret = CommandToSvr_Arg_2(RS_UDBG_SETCSR, csrAddr, csrData); 

    return ret;
}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::GetUengStatus
   Description: Retreive the status of the specified micro-engine.

   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::GetUengStatus(unsigned char uEng, uDbg_uEngStatus_T *uEngStatus)
{
	int ret;
	uDbg_uEngStatus_T result;
	int size=sizeof(uDbg_uEngStatus_T);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	ret = CommandToSvr_Arg_2(RS_UDBG_GETUENGSTATUS, uEng, tmpTargetBuf); 
    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, &result, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);

  	uEngStatus->state = result.state;
	uEngStatus->ctx=result.ctx;
	uEngStatus->enables=result.enables;
	uEngStatus->breakId=result.breakId;
	uEngStatus->pc=result.pc;

    return ret;
}
/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::HopUeng
   Description: Hop microengine
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::HopUeng(unsigned int *uEngMask, unsigned int delay)

{
	int ret;
	int size=sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	if((Fn_wtxMemWrite(wtx_GetHWTX(), uEngMask, tmpTargetBuf, size )) == WTX_ERROR){
	   wtx_FreeTargetBuf(tmpTargetBuf, size);
	   return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_2(RS_UDBG_HOPUENG, tmpTargetBuf, delay); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;

}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::WriteFIFO_Tx
   Description: Write a number of quad-words to micro-engine transfer-fifo.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::WriteFIFO_Tx(unsigned char uEng, unsigned int fifoAddr,
						unsigned int *data64, unsigned int count, int setValid)
{

	int ret;
	int size=count*sizeof(int)*2;
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);
	if((Fn_wtxMemWrite(wtx_GetHWTX(), data64, tmpTargetBuf, size )) == WTX_ERROR){
	    wtx_FreeTargetBuf(tmpTargetBuf, size);
		return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_5(RS_UDBG_WRITEFIFO_TX, uEng, fifoAddr,tmpTargetBuf, count, setValid); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

    return ret;

}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::ReadFIFO_Rx
   Description: Read a number of quad-words from micro-engine receive-fifo.
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::ReadFIFO_Rx(unsigned char uEng, unsigned int fifoAddr,
						unsigned int *data64, unsigned int count)
{

	int ret;
	int size=count*sizeof(int)*2;
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_4(RS_UDBG_READFIFO_RX, uEng, fifoAddr, tmpTargetBuf, count); 

    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf, data64, size)) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);

   return ret;

}


/*-----------------------------------------------------------------------------
   Function: 	Debug::GetSysMemSize
   Description: This routine returns sizes of virtual memory regions in the following
				pointers:
					sys     -- Size of memory mapped at zero for OS
					sys_all -- Size of memory mapped at zero and usable by the OS
					sdram   -- Size of memory mapped at C0000000 and usable by HW
					sram    -- Size of sram
					flash   -- Size of Flash
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_TIMEOUT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
#define rs_sys     0
#define rs_sys_all 4
#define rs_sdram   8
#define rs_sram 12
#define rs_flash 16




int RS_uDbg_WTX::GetSysMemSize(unsigned int* sys, unsigned int* sys_all, unsigned int* sdram,
							unsigned int* sram,unsigned int* flash)
{
	int ret;
	int size=100;
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	ret = CommandToSvr_Arg_5(RS_UDBG_GETSYSMEMSIZE, tmpTargetBuf+rs_sys, tmpTargetBuf+rs_sys_all,
		tmpTargetBuf+rs_sdram,tmpTargetBuf+rs_sram,tmpTargetBuf+rs_flash);


    if(ret == UDBG_SUCCESS){
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf+rs_sys, sys, sizeof(int))) == WTX_ERROR)
		   ret = RS_NOMEMORY;
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf+rs_sys_all, sys_all, sizeof(int))) == WTX_ERROR)
		   ret = RS_NOMEMORY;
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf+rs_sdram,sdram, sizeof(int))) == WTX_ERROR)
		   ret = RS_NOMEMORY;
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf+rs_sram, sram, sizeof(int))) == WTX_ERROR)
		   ret = RS_NOMEMORY;
	   if((Fn_wtxMemRead(wtx_GetHWTX(), tmpTargetBuf+rs_flash, flash, sizeof(int))) == WTX_ERROR)
		   ret = RS_NOMEMORY;
    }
   wtx_FreeTargetBuf(tmpTargetBuf, size);


   return ret;

}

/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::CNTL_Hop
   Description: Hop microengine
   Returns:		UDBG_SUCCESS, UDBG_BADARG
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::CNTL_Hop(unsigned int *uEngMask, unsigned int portMask, unsigned int delay)

{
	int ret;
	int size=sizeof(int);
	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if((Fn_wtxMemWrite(wtx_GetHWTX(), uEngMask, tmpTargetBuf, size )) == WTX_ERROR){
	    wtx_FreeTargetBuf(tmpTargetBuf, size);
		return RS_NOMEMORY;
	}
	ret = CommandToSvr_Arg_3(RS_CNTL_HOP, tmpTargetBuf, portMask,delay); 
    wtx_FreeTargetBuf(tmpTargetBuf, size);

	return ret;

}


/*-----------------------------------------------------------------------------
   Function: 	RS_uDbg_WTX::CNTL_InitUof
   Description: Load uCode
   Returns:		UDBG_SUCCESS, UDBG_BADARG, UDBG_BADLIB, UDBG_UENGACT
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int RS_uDbg_WTX::CNTL_InitUof(char* buf, int buflen)
{
	int ret = RS_ERROR;
    int RetVal;
    TGT_ADDR_T RemoteFunction;
	int size=buflen;

	TGT_ADDR_T tmpTargetBuf = wtx_GetTargetBuf(size);

	if( (RemoteFunction = wtx_GetEntry(RS_CNTL_INITUOF)) == 0)
		return ret;


	Fn_wtxMemWrite(wtx_GetHWTX(),buf, tmpTargetBuf,size);

    if((ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal,tmpTargetBuf,
				 buflen,0,0,0,0,0,0,0,0)) != WTX_OK)
				 return RS_ERROR;
		
    wtx_FreeTargetBuf(tmpTargetBuf, size);


	return RetVal;  
}

int RS_uDbg_WTX::wtx_GetEntry(int RemoteID){
	RS_Target *ThisTarget = (RS_Target *)GetMyTarget();
    return (int)ThisTarget->TargetGetEntry(RemoteID);
}

#endif      /* VXWORKS_SUPPORT */


