/* rs_udbg_wtx.h
 * Remote Services Debug Library - VxWorks WTX support
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: Remote Services Debug Library
 *  author: lwh, 12/12/99  Initial version
 *  revisions:
 *  $History: rs_udbg_wtx.h $
 * 
 * *****************  Version 15  *****************
 * User: Ewalker      Date: 11/30/00   Time: 7:06p
 * Updated in $/dev_1200/share/rs_udebug
 * Trying to allow multiple concurrent hardware connections. Not there
 * yet.
 * 
 * *****************  Version 14  *****************
 * User: Ewalker      Date: 10/31/00   Time: 12:11p
 * Updated in $/dev_1200/share/rs_udebug
 * C0 2k ustore support - made InitLib backwards compatible, supporting
 * only segment 1 settings. Apps must use DefineFreeUstore to set segment
 * 2 values if UpdateFreeUstore is not used.  
 * 
 * *****************  Version 13  *****************
 * User: Ewalker      Date: 9/26/00    Time: 8:32a
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 11  *****************
 * User: Ewalker      Date: 9/14/00    Time: 4:45p
 * Updated in $/dev_1200/share/rs_udebug
 * DefineFreeUstore and UpdateFreeUstore functions return void from udbg
 * library
 * so must always return RS_SUCCESS to Workbench under WTX debugging.
 * 
 * *****************  Version 10  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 9  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:16
 * Updated in $/dev_1200/share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 8  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:49p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 3/01/00    Time: 5:47p
 * Updated in $/Beta_4/Share/rs_udebug
 * Fix bug #838
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 1/17/00    Time: 10:46a
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * 
 * *****************  Version 4  *****************
 * User: Qabuild      Date: 1/14/00    Time: 6:56p
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 3  *****************
 * User: Lhui         Date: 1/14/00    Time: 6:39p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * 
 * *****************  Version 2  *****************
 * User: Lhui         Date: 1/12/00    Time: 4:14p
 * Updated in $/Beta4_Interim/Share/rs_udebug
 * Fix bugs
 * 
 * *****************  Version 1  *****************
 * User: Lhui         Date: 1/07/00    Time: 9:57a
 * Created in $/dev_1200/Share/rs_udebug
 * Add support for VxWorks serial port
 */ 

#ifndef RS_UDBG_WTX_H
#define RS_UDBG_WTX_H

#ifdef VXWORKS_SUPPORT

#include <host.h>
#include <wtx.h>

#include "debug_1200.h"
#include "rs_udbg_status.h"
#include "rs_udbg.h"
#include "rs_udebug_clnt.h"

/*#define blockSize 20480*/
#define blockSize 256

#ifdef __cplusplus
class RS_uDbg_WTX : public RS_Debug {
public:
    RS_uDbg_WTX(void *rs_cl, int config, void *hWtx_Dll);

	~RS_uDbg_WTX();

    /* virtual functions */
	virtual int SetUengPCs(unsigned char uEng, unsigned int *upc);
	virtual int GetUengPCs(unsigned char uEng, unsigned int *upc);
	virtual int GetUword(unsigned char uEng, unsigned int uWordAddr, unsigned int *uWord);
	virtual int SetUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned int uWord);
	virtual int GetNumUword(unsigned char uEng, unsigned int uWordAddr, unsigned short numWords,unsigned int *uWord);
	virtual int SetNumUword(unsigned int uEngMask, unsigned int uWordAddr, unsigned short numWords, unsigned int *uWord);
	virtual int GetCSR(unsigned int csrAddr, unsigned int *csrData);
	virtual int SetCSR(unsigned int csrAddr, unsigned int csrData);
	virtual int SetBreakpoint(uDbg_Bkpt_T *breakpoint);
	virtual int ShowBreakpoints(unsigned int uEngMask, uDbg_BkptArray_T *breakpoints);
	virtual int WriteMem32(unsigned int addr, unsigned int *data, unsigned count);
	virtual int ReadMem32(unsigned int addr, unsigned int *data, unsigned count);
	virtual int WriteMem64(unsigned int addr, unsigned int *data, unsigned count);
	virtual int ReadMem64(unsigned int addr, unsigned int *data, unsigned count);
	virtual int PutRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelGPR(unsigned char uEng, unsigned char ctx, uDbg_Bank_T bankType,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSDRAM_RdXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int PutRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int GetRelSDRAM_WrXfer(unsigned char uEng, unsigned char ctx,
			  unsigned int regNum, unsigned int *regData, unsigned count);
	virtual int HopUeng(unsigned int *uEngMask, unsigned int delay);
	virtual int GetUengStatus(unsigned char uEng, uDbg_uEngStatus_T *uEngStatus);
	virtual int WriteFIFO_Tx(unsigned char uEng, unsigned int fifoAddr,unsigned int *regData, unsigned int count, int setValid);
	virtual int ReadFIFO_Rx(unsigned char uEng, unsigned int fifoAddr, unsigned int *regData, unsigned int count);
	virtual int GetSysMemSize(unsigned int* sys, unsigned int* sys_all, unsigned int* sdram,
				 unsigned int* sram,unsigned int* flash);
	virtual int CNTL_InitUof(char* buf, int buflen);
	virtual int CNTL_Hop(unsigned int *uEngMask, unsigned int PortMask, unsigned int delay);
	virtual int GetUthreadPC(unsigned char uEng, unsigned char ctx, unsigned int *upc);
	int Udbg_InitLib(uDbg_uStoreFree_T *puStoreFree);

    /* inline functions */
   	virtual inline int ResetUeng(unsigned int uEngMask, int clrReg)
    {
	      return CommandToSvr_Arg_2(RS_UDBG_RESETUENG, uEngMask, clrReg);
	};

   	virtual inline int DefineFreeUstore(unsigned int uEngMask, unsigned int freeUaddr, unsigned int freeSize)
    {
	      CommandToSvr_Arg_3(RS_UDBG_DEFINEFREEUSTORE, uEngMask, freeUaddr, freeSize);
	      return RS_SUCCESS;  /* DefineFreeUstore returns void - assume success */
	};

   	virtual inline int UpdateFreeUstore(unsigned int uEngMask)
    {
	      CommandToSvr_Arg_1(RS_UDBG_UPDATEFREEUSTORE, uEngMask);
	      return RS_SUCCESS;  /* UpdateFreeUstore returns void - assume success */
    };
 
   	virtual inline int ClrResetUeng(unsigned int uEngMask)
    {
 	      return CommandToSvr_Arg_1(RS_UDBG_CLRRESETUENG, uEngMask);
    };

   	virtual inline int StartUeng(unsigned int uEngMask)
	{
		return CommandToSvr_Arg_1(RS_UDBG_STARTUENG, uEngMask);
    };

   	virtual inline int PauseUeng(unsigned int uEngMask, unsigned int delay)
    {
      return CommandToSvr_Arg_2(RS_UDBG_PAUSEUENG, uEngMask, delay);
 
	};

   	virtual inline int ResumeUeng(unsigned int uEngMask)
    {
       return CommandToSvr_Arg_1(RS_UDBG_RESUMEUENG, uEngMask);
    };

   	virtual inline int GetActiveUengs(unsigned int uEngMask)
    {
	   return CommandToSvr_Arg_1(RS_UDBG_GETACTIVEUENGS, uEngMask);
    };
   	virtual inline int SetUthreadsPC(unsigned int uEngMask, unsigned int ctxMask, unsigned int upc)
    {
       return CommandToSvr_Arg_3(RS_UDBG_SETUTHREADSPC, uEngMask, ctxMask, upc);
    };

    virtual inline int IsUengActive(unsigned int uEngMask)
    {
      return CommandToSvr_Arg_1(RS_UDBG_ISUENGACTIVE, uEngMask);
    };
   	virtual int ClearAllBreakpoints(unsigned int uEngMask)
    {
 		return CommandToSvr_Arg_1(RS_UDBG_CLEARALLBREAKPOINTS, uEngMask);
    };
   	virtual inline int ClearBreakpoint(short breakPointId)
    {
	      return CommandToSvr_Arg_1(RS_UDBG_CLEARBREAKPOINT, breakPointId);
    };
 
   	virtual inline int CNTL_Reset(unsigned int uEngMask,unsigned int PortMask, int clrReg)
    {
	      return CommandToSvr_Arg_3(RS_CNTL_RESET, uEngMask, PortMask, clrReg);
    };

   	virtual inline int CNTL_Pause(unsigned int uEngMask, unsigned int PortMask, unsigned int delay)
    {

		return CommandToSvr_Arg_3(RS_CNTL_PAUSE, uEngMask,PortMask, delay);
    };
    virtual inline int CNTL_Resume(unsigned int uEngMask,unsigned int PortMask)
    {
      return CommandToSvr_Arg_2(RS_CNTL_RESUME, uEngMask,PortMask);
    };
    virtual inline int CNTL_Start(unsigned int uEngMask,unsigned int PortMask)
    {
		return CommandToSvr_Arg_2(RS_CNTL_START, uEngMask,PortMask);
    };
   
	typedef STATUS (__cdecl *T_wtxResultFree)(HWTX, void *);
	T_wtxResultFree Fn_wtxResultFree; 

	typedef INT32 (__cdecl *T_wtxStrToInt32)(HWTX, const char *);
	T_wtxStrToInt32 Fn_wtxStrToInt32; 

	typedef TGT_ADDR_T (__cdecl *T_wtxMemAlloc)(HWTX, UINT32);
	T_wtxMemAlloc Fn_wtxMemAlloc; 

	typedef UINT32 (__cdecl *T_wtxMemWrite)(HWTX, void *, TGT_ADDR_T,  UINT32);
	T_wtxMemWrite Fn_wtxMemWrite; 

	typedef STATUS (__cdecl *T_wtxDirectCall)(HWTX, TGT_ADDR_T, void *,TGT_ARG_T, TGT_ARG_T, TGT_ARG_T, TGT_ARG_T,
	   TGT_ARG_T, TGT_ARG_T, TGT_ARG_T, TGT_ARG_T, TGT_ARG_T, TGT_ARG_T);
	T_wtxDirectCall Fn_wtxDirectCall; 

	typedef STATUS (__cdecl *T_wtxRegisterForEvent)(HWTX, const char *);
	T_wtxRegisterForEvent Fn_wtxRegisterForEvent; 

	typedef STATUS (__cdecl *T_wtxAsyncNotifyEnable)(HWTX,FUNCPTR);
	T_wtxAsyncNotifyEnable Fn_wtxAsyncNotifyEnable; 

	typedef STATUS (__cdecl *T_wtxMemFree)(HWTX,TGT_ADDR_T);
	T_wtxMemFree Fn_wtxMemFree; 

	typedef STATUS (__cdecl *T_wtxAsyncNotifyDisable)(HWTX);
	T_wtxAsyncNotifyDisable Fn_wtxAsyncNotifyDisable; 

	typedef STATUS (__cdecl *T_wtxUnregisterForEvent)(HWTX, char *);
	T_wtxUnregisterForEvent Fn_wtxUnregisterForEvent; 

	typedef int (__cdecl *T_wtxMemRead)(HWTX, TGT_ADDR_T, void *, UINT32);
	T_wtxMemRead Fn_wtxMemRead; 

   int  CommandToSvr_Arg_1(int fID, const int arg1) 
   {
       int RetVal=RS_ERROR, ret=0;
       TGT_ADDR_T RemoteFunction;
	   
	   if( (RemoteFunction = wtx_GetEntry(fID)) != 0){
			ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal, GetuDbg_Handle(),
				  arg1,0,0,0,0,0,0,0,0);
	   }
		return RetVal;
   };

   int  CommandToSvr_Arg_2(int fID, const int arg1, const int arg2) 
   {
       int RetVal=RS_ERROR, ret=0;
       TGT_ADDR_T RemoteFunction;
	   
	   if( (RemoteFunction = wtx_GetEntry(fID)) != 0){
			ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal, GetuDbg_Handle(),
				  arg1,arg2,0,0,0,0,0,0,0);
	   }
		return RetVal;
   };

   int  CommandToSvr_Arg_3(int fID, const int arg1, const int arg2, const int arg3) 
   {
       int RetVal=RS_ERROR, ret=0;
       TGT_ADDR_T RemoteFunction;
	   
	   if( (RemoteFunction = wtx_GetEntry(fID)) != 0){
			ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal, GetuDbg_Handle(),
				  arg1,arg2,arg3,0,0,0,0,0,0);
	   }
		return RetVal;
   };

   int  CommandToSvr_Arg_4(int fID, const int arg1, const int arg2, const int arg3, const int arg4) 
   {
       int RetVal=RS_ERROR, ret=0;
       TGT_ADDR_T RemoteFunction;
	   
	   if( (RemoteFunction = wtx_GetEntry(fID)) != 0){
			ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal, GetuDbg_Handle(),
				  arg1,arg2,arg3,arg4,0,0,0,0,0);
	   }
		return RetVal;
   };
 
   int  CommandToSvr_Arg_5(int fID, const int arg1, const int arg2, const int arg3, const int arg4, const int arg5) 
   {
       int RetVal=RS_ERROR, ret=0;
       TGT_ADDR_T RemoteFunction;
	   
	   if( (RemoteFunction = wtx_GetEntry(fID)) != 0){
			ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal, GetuDbg_Handle(),
				  arg1,arg2,arg3,arg4,arg5,0,0,0,0);
	   }
		return RetVal;
   };

   int  CommandToSvr_Arg_6(int fID, const int arg1, const int arg2, const int arg3, const int arg4,
	   const int arg5, const int arg6) 
   {
       int RetVal=RS_ERROR, ret=0;
       TGT_ADDR_T RemoteFunction;
	   
	   if( (RemoteFunction = wtx_GetEntry(fID)) != 0){
			ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal, GetuDbg_Handle(),
				  arg1,arg2,arg3,arg4,arg5,arg6,0,0,0);
	   }
		return RetVal;
   };

   int  CommandToSvr_Arg_7(int fID, const int arg1, const int arg2, const int arg3, const int arg4,
	   const int arg5, const int arg6, const int arg7) 
   {
       int RetVal=RS_ERROR, ret=0;
       TGT_ADDR_T RemoteFunction;
	   
	   if( (RemoteFunction = wtx_GetEntry(fID)) != 0){
			ret = Fn_wtxDirectCall(wtx_GetHWTX(), RemoteFunction, &RetVal, GetuDbg_Handle(),
				  arg1,arg2,arg3,arg4,arg5,arg6,arg7,0,0);
	   }
		return RetVal;
   };

   HWTX wtx_GetHWTX()		{return wtxh;};

   TGT_ADDR_T wtx_GetTargetBuf(int size) {
		if(size > blockSize){
			return (Fn_wtxMemAlloc (wtx_GetHWTX(), size));
		}
		else
		    return TargetBuf;
   };

   int wtx_FreeTargetBuf(TGT_ADDR_T buf, int size) {
		if(size > blockSize){
			return (Fn_wtxMemFree(wtx_GetHWTX(), buf));
		}
		else
		    return RS_SUCCESS;

   };

private:
    HWTX             wtxh; 
    TGT_ADDR_T		 TargetBuf;
    HINSTANCE m_hWtx_Dll;
    int wtx_GetEntry(int remoteID);

};
#endif          /* __cplusplus */
#endif          /* VXWORKS_SUPPORT */
#endif			/* RS_UDBG_WTX_H */
