/* rs_udebug.h
 * Remote Services Debug Library - Definitions common to client and server
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rs_udebug.h $
 * 
 * *****************  Version 31  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 

#ifndef RS_UDEBUG_H
#define RS_UDEBUG_H

#include "ostype.h"
#include "rs_udbg_status.h"

#if (OS == WIN_32)
#include <OncRpc32.h>
#include <pmap_clnt.h>
#else   /* not WIN_32 */
#if (OS != UCOS)
#include <rpc/rpc.h>
#else   /* UCOS only */
#define u_int32 unsigned int
#define	bool_t	int
#define u_char unsigned char 
#define u_int unsigned int
#define u_short unsigned short
#define u_long unsigned int
#ifndef FALSE
#define FALSE (0)
#define TRUE  (1)
#endif
#endif  /* UCOS only */
#endif  /* not WIN_32 */

#if (OS != UCOS)
#define SA1200TARGET ((u_long)1691)
#ifndef SA1200VERSION
#define SA1200VERSION ((u_long)1)
#endif
#endif

struct rs_string_T {
	struct {
		u_int msg_len;
		char *msg_val;
	} msg;
};
typedef struct rs_string_T rs_string_T;
bool_t xdr_rs_string_T();


struct rs_uDbg_uEngStatus_T {
	char state;
	char activeCtx;
	char enables;
	char breakId;
	short activePC;
};
typedef struct rs_uDbg_uEngStatus_T rs_uDbg_uEngStatus_T;
bool_t xdr_rs_uDbg_uEngStatus_T();


struct rs_uDbg_GetuEngStatus_T {
	int res;
	rs_uDbg_uEngStatus_T uEngStatus;
};
typedef struct rs_uDbg_GetuEngStatus_T rs_uDbg_GetuEngStatus_T;
bool_t xdr_rs_uDbg_GetuEngStatus_T();


struct rs_uDbg_GetMemSize_T {
	int res;
	u_int sys;
	u_int sys_all;
	u_int sdram;
	u_int sram;
	u_int flash;
};
typedef struct rs_uDbg_GetMemSize_T rs_uDbg_GetMemSize_T;
bool_t xdr_rs_uDbg_GetMemSize_T();


struct rs_uDbg_Handle_T {
	int uDbg_Handle;
};
typedef struct rs_uDbg_Handle_T rs_uDbg_Handle_T;
bool_t xdr_rs_uDbg_Handle_T();


struct rs_uDbg_Bkpt_T {
	int callback;
	u_short id;
	u_short ctx;
	u_short uAddr;
	u_char uEng;
	u_int userData;
	u_int breakIfCtx;
	u_int stpOnBrkUengMask;
};
typedef struct rs_uDbg_Bkpt_T rs_uDbg_Bkpt_T;
bool_t xdr_rs_uDbg_Bkpt_T();


struct rs_uDbg_BPoint_T {
	int callback;
	u_int usrData;
	u_int uword;
	short id;
	u_short upc;
	u_short ctxBreak;
	u_short brUpc;
	u_char state;
	u_char ueng;
	u_int breakIfCtx;
	u_int stpUengMask;
};
typedef struct rs_uDbg_BPoint_T rs_uDbg_BPoint_T;
bool_t xdr_rs_uDbg_BPoint_T();


struct rs_uDbg_BreakPoint_T {
	rs_uDbg_Handle_T hRs_uDbg;
	rs_uDbg_Bkpt_T pBpoint;
};
typedef struct rs_uDbg_BreakPoint_T rs_uDbg_BreakPoint_T;
bool_t xdr_rs_uDbg_BreakPoint_T();


struct rs_uDbg_BkptArray_T {
	u_short len;
	struct {
		u_int bkpts_len;
		rs_uDbg_Bkpt_T *bkpts_val;
	} bkpts;
};
typedef struct rs_uDbg_BkptArray_T rs_uDbg_BkptArray_T;
bool_t xdr_rs_uDbg_BkptArray_T();


struct rs_uDbg_uStoreFree_T {
	char *ClientName;
	struct {
		u_int freeUpc_len;
		u_short *freeUpc_val;
	} freeUpc;
	struct {
		u_int freeSize_len;
		u_short *freeSize_val;
	} freeSize;
};
typedef struct rs_uDbg_uStoreFree_T rs_uDbg_uStoreFree_T;
bool_t xdr_rs_uDbg_uStoreFree_T();


struct rs_uDbg_One_uint_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_int arg1;
};
typedef struct rs_uDbg_One_uint_T rs_uDbg_One_uint_T;
bool_t xdr_rs_uDbg_One_uint_T();


struct rs_uDbg_Two_uint_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_int arg1;
	u_int arg2;
};
typedef struct rs_uDbg_Two_uint_T rs_uDbg_Two_uint_T;
bool_t xdr_rs_uDbg_Two_uint_T();


struct rs_uDbg_Three_uint_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_int arg1;
	u_int arg2;
	u_int arg3;
};
typedef struct rs_uDbg_Three_uint_T rs_uDbg_Three_uint_T;
bool_t xdr_rs_uDbg_Three_uint_T();


struct rs_uDbg_uWord_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_int uEng;
	u_int uWordAddr;
	u_short numWords;
	struct {
		u_int u_Word_len;
		u_int *u_Word_val;
	} u_Word;
};
typedef struct rs_uDbg_uWord_T rs_uDbg_uWord_T;
bool_t xdr_rs_uDbg_uWord_T();


struct rs_uDbg_Mem_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_int addr;
	struct {
		u_int data_len;
		u_int *data_val;
	} data;
};
typedef struct rs_uDbg_Mem_T rs_uDbg_Mem_T;
bool_t xdr_rs_uDbg_Mem_T();


struct rs_uDbg_CSR_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_int csrAddr;
	struct {
		u_int csrData_len;
		u_int *csrData_val;
	} csrData;
};
typedef struct rs_uDbg_CSR_T rs_uDbg_CSR_T;
bool_t xdr_rs_uDbg_CSR_T();


struct rs_uDbg_Reg_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_char uEng;
	u_char ctx;
	u_int bankType;
	u_int regAddr;
	struct {
		u_int regData_len;
		u_int *regData_val;
	} regData;
};
typedef struct rs_uDbg_Reg_T rs_uDbg_Reg_T;
bool_t xdr_rs_uDbg_Reg_T();


struct rs_uDbg_GetThPC_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_char uEng;
	u_char ctx;
	struct {
		u_int regData_len;
		u_int *regData_val;
	} regData;
};
typedef struct rs_uDbg_GetThPC_T rs_uDbg_GetThPC_T;
bool_t xdr_rs_uDbg_GetThPC_T();


struct rs_uDbg_Status_T {
	int res;
	struct {
		u_int resData_len;
		u_int *resData_val;
	} resData;
};
typedef struct rs_uDbg_Status_T rs_uDbg_Status_T;
bool_t xdr_rs_uDbg_Status_T();


struct rs_uDbg_ShowBreakpts_T {
	int res;
	rs_uDbg_BkptArray_T BkptArray;
};
typedef struct rs_uDbg_ShowBreakpts_T rs_uDbg_ShowBreakpts_T;
bool_t xdr_rs_uDbg_ShowBreakpts_T();

struct rs_CNTL_InitUof_T {
	struct {
		u_int UofData_len;
		char *UofData_val;
	} UofData;
};
typedef struct rs_CNTL_InitUof_T rs_CNTL_InitUof_T;
bool_t xdr_rs_CNTL_InitUof_T();

struct rs_uDbg_FiFo_T {
	rs_uDbg_Handle_T hRs_uDbg;
	u_char uEng;
	u_int fifoAddr;
	struct {
		u_int data64_len;
		u_int *data64_val;
	} data64;
	u_int count;
	u_int setValid;
};
typedef struct rs_uDbg_FiFo_T rs_uDbg_FiFo_T;
bool_t xdr_rs_uDbg_FiFo_T();


/* Client/Server function identifiers */

#define RS_UDBG_INITLIB ((u_long)1)
#define RS_UDBG_DELELIB ((u_long)2)
#define RS_UDBG_RESETUENG ((u_long)3)
#define RS_UDBG_STARTUENG ((u_long)4)
#define RS_UDBG_PAUSEUENG ((u_long)5)
#define RS_UDBG_RESUMEUENG ((u_long)6)
#define RS_UDBG_ISUENGACTIVE ((u_long)7)
#define RS_UDBG_GETACTIVEUENGS ((u_long)8)
#define RS_UDBG_GETUTHREADPC ((u_long)9)
#define RS_UDBG_SETUTHREADSPC ((u_long)10)
#define RS_UDBG_SETUENGPC ((u_long)11)
#define RS_UDBG_GETUENGPC ((u_long)12)
#define RS_UDBG_GETUWORD ((u_long)13)
#define RS_UDBG_SETUWORD ((u_long)14)
#define RS_UDBG_GETNUMUWORD ((u_long)15)
#define RS_UDBG_SETNUMUWORD ((u_long)16)
#define RS_UDBG_WRITEMEM32 ((u_long)17)
#define RS_UDBG_READMEM32 ((u_long)18)
#define RS_UDBG_WRITEMEM64 ((u_long)19)
#define RS_UDBG_READMEM64 ((u_long)20)
#define RS_UDBG_SETBREAKPOINT ((u_long)21)
#define RS_UDBG_SHOWBREAKPOINTS ((u_long)22)
#define RS_UDBG_CLEARALLBREAKPOINTS ((u_long)23)
#define RS_UDBG_CLEARBREAKPOINT ((u_long)24)
#define RS_UDBG_GETCSR ((u_long)25)
#define RS_UDBG_SETCSR ((u_long)26)
#define RS_UDBG_PUTRELGPR ((u_long)27)
#define RS_UDBG_GETRELGPR ((u_long)28)
#define RS_UDBG_PUTRELSRAM_WRXFER ((u_long)29)
#define RS_UDBG_PUTRELSDRAM_WRXFER ((u_long)30)
#define RS_UDBG_PUTRELSRAM_RDXFER ((u_long)31)
#define RS_UDBG_PUTRELSDRAM_RDXFER ((u_long)32)
#define RS_UDBG_GETRELSRAM_WRXFER ((u_long)33)
#define RS_UDBG_GETRELSDRAM_WRXFER ((u_long)34)
#define RS_UDBG_GETRELSRAM_RDXFER ((u_long)35)
#define RS_UDBG_GETRELSDRAM_RDXFER ((u_long)36)
#define RS_UDBG_GETUENGSTATUS ((u_long)37)
#define RS_UDBG_HOPUENG ((u_long)38)
#define RS_XACT_COMMAND ((u_long)39)
#define RS_CNTL_INITUOF ((u_long)40)
#define RS_CNTL_RESET ((u_long)41)
#define RS_CNTL_PAUSE ((u_long)42)
#define RS_CNTL_RESUME ((u_long)43)
#define RS_CNTL_START ((u_long)44)
#define RS_CNTL_HOP ((u_long)45)
#define RS_UDBG_CLRRESETUENG ((u_long)46)
#define RS_UDBG_DEFINEFREEUSTORE ((u_long)47)
#define RS_UDBG_UPDATEFREEUSTORE ((u_long)48)
#define RS_UDBG_WRITEFIFO_TX ((u_long)49)
#define RS_UDBG_READFIFO_RX ((u_long)50)
#define RS_UDBG_GETSYSMEMSIZE ((u_long)51)
#define RS_MAX_FUNCTIONS ((u_long)52)

#endif          /* RS_UDEBUG_H */
