/* rs_udebug_clnt.c
 * Remote Services Debug Library - Implements remote debug RPC client stubs
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rs_udebug_clnt.c $
 * 
 * *****************  Version 33  *****************
 * User: Ewalker      Date: 9/07/00    Time: 16:24
 * Updated in $/dev_1200/Share/rs_udebug
 * Change timeout to 15 seconds
 * 
 * *****************  Version 32  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 

#include <stdio.h>

#include "rs_udebug.h"
#include "rs_udebug_clnt.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

/* Default timeout can be changed using clnt_control() */
#ifdef _DEBUG
static struct timeval TIMEOUT = {3600,0};  /* one hour */
#else
static struct timeval TIMEOUT = {15,00};  /* 15 seconds */
#endif

#ifdef __cplusplus
extern "C" {
CLIENT *create_hClient(char *Server, int RemoteProg, int Ver, char *protocol);
void destroy_hClient(CLIENT *clnt);
}
#endif

int *
rs_udbg_initlib_c(argp, clnt)
	rs_uDbg_uStoreFree_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_INITLIB, xdr_rs_uDbg_uStoreFree_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_delelib_c(argp, clnt)
	rs_uDbg_Handle_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_DELELIB, xdr_rs_uDbg_Handle_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_udbg_definefreeustore_c(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_DEFINEFREEUSTORE, xdr_rs_uDbg_Three_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_updatefreeustore_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_UPDATEFREEUSTORE, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}



int *
rs_udbg_resetueng_c(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_RESETUENG, xdr_rs_uDbg_Two_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_udbg_clrresetueng_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_CLRRESETUENG, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}



int *
rs_udbg_startueng_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_STARTUENG, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_pauseueng_c(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_PAUSEUENG, xdr_rs_uDbg_Two_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_resumeueng_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_RESUMEUENG, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_isuengactive_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_ISUENGACTIVE, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_getactiveuengs_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETACTIVEUENGS, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getuthreadpc_c(argp, clnt)
	rs_uDbg_GetThPC_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETUTHREADPC, xdr_rs_uDbg_GetThPC_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_setuthreadspc_c(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_SETUTHREADSPC, xdr_rs_uDbg_Three_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_setuengpc_c(argp, clnt)
	rs_uDbg_Mem_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_SETUENGPC, xdr_rs_uDbg_Mem_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getuengpc_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETUENGPC, xdr_rs_uDbg_One_uint_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getuword_c(argp, clnt)
	rs_uDbg_uWord_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETUWORD, xdr_rs_uDbg_uWord_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_setuword_c(argp, clnt)
	rs_uDbg_uWord_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_SETUWORD, xdr_rs_uDbg_uWord_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getnumuword_c(argp, clnt)
	rs_uDbg_uWord_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETNUMUWORD, xdr_rs_uDbg_uWord_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_setnumuword_c(argp, clnt)
	rs_uDbg_uWord_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_SETNUMUWORD, xdr_rs_uDbg_uWord_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_writemem32_c(argp, clnt)
	rs_uDbg_Mem_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_WRITEMEM32, xdr_rs_uDbg_Mem_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_readmem32_c(argp, clnt)
	rs_uDbg_Mem_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_READMEM32, xdr_rs_uDbg_Mem_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_writemem64_c(argp, clnt)
	rs_uDbg_Mem_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_WRITEMEM64, xdr_rs_uDbg_Mem_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_readmem64_c(argp, clnt)
	rs_uDbg_Mem_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_READMEM64, xdr_rs_uDbg_Mem_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_setbreakpoint_c(argp, clnt)
	rs_uDbg_BreakPoint_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_SETBREAKPOINT, xdr_rs_uDbg_BreakPoint_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_ShowBreakpts_T *
rs_udbg_showbreakpoints_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_ShowBreakpts_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_SHOWBREAKPOINTS, xdr_rs_uDbg_One_uint_T, argp, xdr_rs_uDbg_ShowBreakpts_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_clearallbreakpoints_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_CLEARALLBREAKPOINTS, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_clearbreakpoint_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_CLEARBREAKPOINT, xdr_rs_uDbg_One_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getcsr_c(argp, clnt)
	rs_uDbg_CSR_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETCSR, xdr_rs_uDbg_CSR_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_setcsr_c(argp, clnt)
	rs_uDbg_CSR_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_SETCSR, xdr_rs_uDbg_CSR_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_putrelgpr_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_PUTRELGPR, xdr_rs_uDbg_Reg_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelgpr_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETRELGPR, xdr_rs_uDbg_Reg_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_putrelsram_wrxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_PUTRELSRAM_WRXFER, xdr_rs_uDbg_Reg_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_putrelsdram_wrxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_PUTRELSDRAM_WRXFER, xdr_rs_uDbg_Reg_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_putrelsram_rdxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_PUTRELSRAM_RDXFER, xdr_rs_uDbg_Reg_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_putrelsdram_rdxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_PUTRELSDRAM_RDXFER, xdr_rs_uDbg_Reg_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsram_wrxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETRELSRAM_WRXFER, xdr_rs_uDbg_Reg_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsdram_wrxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETRELSDRAM_WRXFER, xdr_rs_uDbg_Reg_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsram_rdxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETRELSRAM_RDXFER, xdr_rs_uDbg_Reg_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsdram_rdxfer_c(argp, clnt)
	rs_uDbg_Reg_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETRELSDRAM_RDXFER, xdr_rs_uDbg_Reg_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_GetuEngStatus_T *
rs_udbg_getuengstatus_c(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_GetuEngStatus_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETUENGSTATUS, xdr_rs_uDbg_One_uint_T, argp, xdr_rs_uDbg_GetuEngStatus_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_hopueng_c(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_HOPUENG, xdr_rs_uDbg_Two_uint_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_xact_command_c(argp, clnt)
	rs_string_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_XACT_COMMAND, xdr_rs_string_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_cntl_inituof_c(argp, clnt)
	rs_CNTL_InitUof_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_CNTL_INITUOF, xdr_rs_CNTL_InitUof_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_cntl_reset_c(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_CNTL_RESET, xdr_rs_uDbg_Three_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_cntl_pause_c(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_CNTL_PAUSE, xdr_rs_uDbg_Three_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_cntl_resume_c(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_CNTL_RESUME, xdr_rs_uDbg_Two_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_cntl_start_c(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_CNTL_START, xdr_rs_uDbg_Two_uint_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_cntl_hop_c(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_CNTL_HOP, xdr_rs_uDbg_Three_uint_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_writefifo_tx_c(argp, clnt)
	rs_uDbg_FiFo_T *argp;
	CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_WRITEFIFO_TX, xdr_rs_uDbg_FiFo_T, argp, xdr_int, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_readfifo_rx_c(argp, clnt)
	rs_uDbg_FiFo_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_READFIFO_RX, xdr_rs_uDbg_FiFo_T, argp, xdr_rs_uDbg_Status_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

rs_uDbg_GetMemSize_T *
rs_udbg_getsysmemsize_c(argp, clnt)
	rs_uDbg_Handle_T *argp;
	CLIENT *clnt;
{
	static rs_uDbg_GetMemSize_T res;

	bzero((char *)&res, sizeof(res));
	if (clnt_call(clnt, RS_UDBG_GETSYSMEMSIZE, xdr_rs_uDbg_Handle_T, argp, xdr_rs_uDbg_GetMemSize_T, &res, TIMEOUT) != RPC_SUCCESS) {
		return (NULL);
	}
	return (&res);
}



CLIENT *create_hClient(char *Server, int RemoteProg, int Ver, char *protocol)
{
 	CLIENT *cl;
 
	OncRpc32Init();

    cl = clnt_create(Server, RemoteProg, Ver, protocol);
    if (cl == NULL) {
	/*
	 * Couldn't establish connection with server.
	 * Print error message and return.
	 */
	  clnt_pcreateerror(Server);
      return NULL;
	}
	return cl;
}

void destroy_hClient(CLIENT *clnt)
{
    if (clnt != NULL) clnt_destroy(clnt);
    return;
}



