/* rs_udebug_comm.c
 * Remote Services Debug Library - Serial comm client and server stubs
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui 2/1/99
 * revisions:
 * $History: rs_udebug_comm.c $
 * 
 * *****************  Version 29  *****************
 * User: Ewalker      Date: 12/11/00   Time: 4:32p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fixed status returned from rs_udbg_bkpt_ck_sc.
 * 
 * *****************  Version 28  *****************
 * User: Ewalker      Date: 11/02/00   Time: 9:03a
 * Updated in $/dev_1200/Share/rs_udebug
 * Fixed SCR/Bug #1222. The serial form of rs_udbg_resetueng was crashing
 * the  target. A typographical error present since pre-V1.0 was
 * corrected.
 * 
 * *****************  Version 27  *****************
 * User: Ewalker      Date: 9/29/00    Time: 4:25p
 * Updated in $/dev_1200/Share/rs_udebug
 * Needed error return in Scomm_call when handshake error occurs
 * 
 * *****************  Version 25  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 * *****************  Version 24  *****************
 * User: Ewalker      Date: 8/30/00    Time: 17:55
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 23  *****************
 * User: Ewalker      Date: 8/28/00    Time: 16:16
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup and added support for PCI debug connection
 * 
 * *****************  Version 20  *****************
 * User: Lhui         Date: 1/27/00    Time: 4:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Rewrite CNTL_InitUof_T function to allow sending multiple packets to
 * SCOMM
 * 
 * *****************  Version 18  *****************
 * User: Lhui         Date: 10/22/99   Time: 6:32p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix misc. bugs
 * 
 * *****************  Version 17  *****************
 * User: Lhui         Date: 10/22/99   Time: 2:33p
 * Updated in $/dev_1200/Share/rs_udebug
 * Fix a bug in GetMemSize function
 * 
 * *****************  Version 16  *****************
 * User: Lhui         Date: 10/11/99   Time: 7:00p
 * Updated in $/dev_1200/Share/rs_udebug
 * Cleanup and fix a callback problem with breakpoint
 * 
 * *****************  Version 15  *****************
 * User: Lhui         Date: 10/04/99   Time: 4:42p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for the new dbg_reset prototype
 * 
 * *****************  Version 13  *****************
 * User: Lhui         Date: 7/20/99    Time: 7:36p
 * Updated in $/dev_1200/Share/rs_udebug
 * Clean up
 * 
 * *****************  Version 12  *****************
 * User: Lhui         Date: 7/16/99    Time: 4:15p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for uDbg_UpdateFreeUstore & uDbg_DefineFreeUstore
 * 
 * *****************  Version 10  *****************
 * User: Lhui         Date: 6/29/99    Time: 7:18p
 * Updated in $/dev_1200/Share/rs_udebug
 * Add support for uDbg_ClrReset
 * 
 * *****************  Version 9  *****************
 * User: Lhui         Date: 6/18/99    Time: 6:27p
 * Updated in $/dev_1200/Share/rs_udebug
 * Update Control API
 * 
 * *****************  Version 7  *****************
 * User: Lhui         Date: 4/22/99    Time: 5:42p
 * Updated in $/development/rs_udebug
 * Add a delay parameter to  rs_udebug/control  hop functions. 
 * 
 * *****************  Version 6  *****************
 * User: Lhui         Date: 4/08/99    Time: 5:51p
 * Updated in $/development/rs_udebug
 * Change rs_cntl_pause prototypes
 * 
 * *****************  Version 5  *****************
 * User: Lhui         Date: 4/07/99    Time: 5:08p
 * Updated in $/development/rs_udebug
 * Fix bugs in rs_uDbg_GetUengPCs and rs_uDbg_InitLib 
 * 
 * *****************  Version 4  *****************
 * User: Lhui         Date: 4/02/99    Time: 2:47p
 * Updated in $/development/rs_udebug
 * Fix bugs
 *
 */ 

#include <stdio.h>
#include "scomm.h"
#include "rs_ScMsg.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

/* scomm_call performs a serial communication-based RPC call */
static int Scomm_call(Sc_CLIENT *clnt, int Rs_Call, 
			   bool_t (*ArgRoutine)(Sc_CLIENT *clnt, void *argp, int Flag), void *argp,
			   bool_t (*ResRoutine)(Sc_CLIENT *clnt, void *argp, int Flag), void *resp, 
               int override_timeout, int Flag)
{
	Sc_Msg_T Rpc_ID;
	int len;
    bool_t res;

	Rpc_ID.MsgType = RS_FUNCTION;
	Rpc_ID.Arg.FunctionID = Rs_Call;

    len = SComm_Send(*clnt, TargetComPort_1, (void *)&Rpc_ID, sizeof(Sc_Msg_T), 
        (override_timeout != 0 ? override_timeout : ScommSendFunctionTimeout));
	if (len != sizeof(Sc_Msg_T)) return RS_ERROR;  /* not all bytes sent */

    len = SComm_Recv(*clnt, (void *)&Rpc_ID, sizeof(Sc_Msg_T), 
        (override_timeout != 0 ? override_timeout : ScommRecvReplyTimeout));
	if (len != sizeof(Sc_Msg_T)) return RS_ERROR;  /* not all bytes received */

	if((Rpc_ID.MsgType == RS_REPLY) && (Rpc_ID.Arg.Reply == RS_SUCCESS)){
        /* send arguments to remote function */
		res = (ArgRoutine)(clnt, argp, Flag);
        if (!res) return RS_ERROR;

        /* receive result from remote function */
		res = (ResRoutine)(clnt, resp, Flag); 
        if (!res) return RS_ERROR;

        return RS_SUCCESS;
	}

	return RS_ERROR;
}

/* client stubs */

int *
rs_udbg_initlib_sc(argp, clnt)
	rs_uDbg_uStoreFree_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_INITLIB, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_uStoreFree_T, (void *)argp, 
		            (ARGRTN_T)Sc_Get_xdr_int, (void *)&res, ScommConnectTimeout, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
	
}


int *
rs_udbg_delelib_sc(argp, clnt)
	rs_uDbg_Handle_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_DELELIB, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Handle_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_resetueng_sc(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_RESETUENG, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Two_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_udbg_clrresetueng_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_CLRRESETUENG, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_updatefreeustore_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_UPDATEFREEUSTORE, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_udbg_definefreeustore_sc(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_DEFINEFREEUSTORE, 
		            (ARGRTN_T)Sc_xdr_rs_uDbg_Three_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res,	0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_startueng_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_STARTUENG, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_pauseueng_sc(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_PAUSEUENG, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Two_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


int *
rs_udbg_resumeueng_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_RESUMEUENG, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_isuengactive_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_ISUENGACTIVE, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_getactiveuengs_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETACTIVEUENGS, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getuthreadpc_sc(argp, clnt)
	rs_uDbg_GetThPC_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETUTHREADPC, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_GetThPC_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_setuthreadspc_sc(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_SETUTHREADSPC, 
		            (ARGRTN_T)Sc_xdr_rs_uDbg_Three_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_setuengpc_sc(argp, clnt)
	rs_uDbg_Mem_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_SETUENGPC, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Mem_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getuengpc_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETUENGPC, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getuword_sc(argp, clnt)
	rs_uDbg_uWord_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETUWORD, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_uWord_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_setuword_sc(argp, clnt)
	rs_uDbg_uWord_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_SETUWORD, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_uWord_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getnumuword_sc(argp, clnt)
	rs_uDbg_uWord_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETNUMUWORD, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_uWord_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_setnumuword_sc(argp, clnt)
	rs_uDbg_uWord_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_SETNUMUWORD, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_uWord_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_writemem32_sc(argp, clnt)
	rs_uDbg_Mem_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_WRITEMEM32, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Mem_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_readmem32_sc(argp, clnt)
	rs_uDbg_Mem_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_READMEM32, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Mem_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_writemem64_sc(argp, clnt)
	rs_uDbg_Mem_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_WRITEMEM64, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Mem_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_readmem64_sc(argp, clnt)
	rs_uDbg_Mem_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_READMEM64, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Mem_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_setbreakpoint_sc(argp, clnt)
	rs_uDbg_BreakPoint_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_SETBREAKPOINT, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_BreakPoint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_ShowBreakpts_T *
rs_udbg_showbreakpoints_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_ShowBreakpts_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_SHOWBREAKPOINTS, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_ShowBreakpts_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_clearallbreakpoints_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_CLEARALLBREAKPOINTS, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_clearbreakpoint_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_CLEARBREAKPOINT, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getcsr_sc(argp, clnt)
	rs_uDbg_CSR_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETCSR, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_CSR_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_setcsr_sc(argp, clnt)
	rs_uDbg_CSR_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_SETCSR, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_CSR_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_putrelgpr_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));


	if (Scomm_call(clnt, RS_UDBG_PUTRELGPR, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelgpr_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETRELGPR, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_putrelsram_wrxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_PUTRELSRAM_WRXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_putrelsdram_wrxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_PUTRELSDRAM_WRXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_putrelsram_rdxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_PUTRELSRAM_RDXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_udbg_putrelsdram_rdxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_PUTRELSDRAM_RDXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsram_wrxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETRELSRAM_WRXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsdram_wrxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETRELSDRAM_WRXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsram_rdxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETRELSRAM_RDXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_getrelsdram_rdxfer_sc(argp, clnt)
	rs_uDbg_Reg_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETRELSDRAM_RDXFER, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Reg_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_GetuEngStatus_T *
rs_udbg_getuengstatus_sc(argp, clnt)
	rs_uDbg_One_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_GetuEngStatus_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_GETUENGSTATUS, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_One_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_GetuEngStatus_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_hopueng_sc(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	Sc_CLIENT *clnt;
{

	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_UDBG_HOPUENG, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Two_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_xact_command_sc(argp, clnt)
	rs_string_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_XACT_COMMAND, 
                    (ARGRTN_T)Sc_xdr_rs_string_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

int *
rs_udbg_writefifo_tx_sc(argp, clnt)
	rs_uDbg_FiFo_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));
	if (Scomm_call(clnt, RS_UDBG_WRITEFIFO_TX, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_FiFo_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}


rs_uDbg_Status_T *
rs_udbg_readfifo_rx_sc(argp, clnt)
	rs_uDbg_FiFo_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));
	if (Scomm_call(clnt, RS_UDBG_READFIFO_RX, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_FiFo_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}

rs_uDbg_GetMemSize_T *
rs_udbg_getsysmemsize_sc(argp, clnt)
	rs_uDbg_Handle_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_GetMemSize_T res;

	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_UDBG_GETSYSMEMSIZE, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Handle_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_GetMemSize_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);
}



int *
rs_cntl_inituof_sc(argp, clnt)
	rs_CNTL_InitUof_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;
	
	bzero((char *)&res, sizeof(res));

    if (Scomm_call(clnt, RS_CNTL_INITUOF, 
                    (ARGRTN_T)Sc_xdr_rs_CNTL_InitUof_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}
	return (&res);

}


int *
rs_cntl_reset_sc(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_CNTL_RESET, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Three_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_cntl_pause_sc(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_CNTL_PAUSE, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Three_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


int *
rs_cntl_resume_sc(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_CNTL_RESUME, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Two_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);

}

int *
rs_cntl_start_sc(argp, clnt)
	rs_uDbg_Two_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static int res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_CNTL_START, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Two_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_int, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}
rs_uDbg_Status_T *
rs_cntl_hop_sc(argp, clnt)
	rs_uDbg_Three_uint_T *argp;
	Sc_CLIENT *clnt;
{
	static rs_uDbg_Status_T res;

	bzero((char *)&res, sizeof(res));

	if (Scomm_call(clnt, RS_CNTL_HOP, 
                    (ARGRTN_T)Sc_xdr_rs_uDbg_Three_uint_T, argp, 
                    (ARGRTN_T)Sc_Get_xdr_rs_uDbg_Status_T, &res, 0, 0) != RS_SUCCESS) {
		return (NULL);
	}

	return (&res);
}


/* client callback (event) stubs */

int *
rs_udbg_bkpt_ck_sc(argp, clnt)
	rs_uDbg_Bkpt_CK_T *argp;
    Sc_CLIENT *clnt;

{
    static int res = RS_SUCCESS;
	Sc_Msg_T Rpc_ID;
	int len;

	Rpc_ID.MsgType =  RS_FUNCTION;
	Rpc_ID.Arg.FunctionID = RS_uDbg_Bkpt_CK;

    len = SComm_Send(*clnt, HostComPort_2, (void *)&Rpc_ID, sizeof(Sc_Msg_T), ScommSendFunctionTimeout);
	if (len != sizeof(Sc_Msg_T)) return (NULL);
    
    /* sending parameters */
	if (!Sc_xdr_rs_uDbg_Bkpt_CK_T(clnt, argp, 0)) res = RS_ERROR;
    else res = RS_SUCCESS;

    return (&res);
}


