/* rs_udebug_svc.c
 * Remote Services Debug Library - Implements RPC server entry points
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2000 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Remote Services Debug Library
 * author: Lai-Wah Hui
 * revisions:
 * $History: rs_udebug_svc.c $
 * 
 * *****************  Version 35  *****************
 * User: Ewalker      Date: 11/30/00   Time: 6:38p
 * Updated in $/dev_1200/Share/rs_udebug
 * Better protect against invalid uDbg object handle passed from
 * workbench.
 * 
 * *****************  Version 33  *****************
 * User: Ewalker      Date: 9/08/00    Time: 14:33
 * Updated in $/dev_1200/Share/rs_udebug
 * 
 * *****************  Version 32  *****************
 * User: Ewalker      Date: 9/07/00    Time: 16:24
 * Updated in $/dev_1200/Share/rs_udebug
 * Print RTC time when service was invoked
 * 
 * *****************  Version 31  *****************
 * User: Ewalker      Date: 9/05/00    Time: 11:41
 * Updated in $/dev_1200/Share/rs_udebug
 * Add missing copyright comment block and static declaration
 * 
 */ 

#include <stdio.h>
#include <time.h>

#include "ostype.h"
#include "rs_udebug_svc.h"

static const char __copyright[] =
"Copyright Intel Corp, 1998-2000. All rights reserved";

static void sa1200target_1();

#if (OS == WIN_32)
#ifdef RS_UDEBUGTEST
#ifdef __cplusplus
extern "C" {
#endif
void Rs_udbg_server()
#ifdef __cplusplus
}
#endif
#else
void main()
#endif
#else
#ifdef __cplusplus
extern "C" {
#endif
void Rs_udbg_server()
#ifdef __cplusplus
}
#endif
#endif
{
	SVCXPRT *transp;

#if (OS == WIN_32)
	OncRpc32Init();
#else
	rpcTaskInit();
#endif

	pmap_unset(SA1200TARGET, SA1200VERSION);

	transp = svctcp_create(RPC_ANYSOCK, 0, 0);
	if (transp == NULL) {
		(void)fprintf(stderr, "cannot create tcp service.\n");
		exit(1);
	}

    if (!svc_register(transp, SA1200TARGET, SA1200VERSION, sa1200target_1, IPPROTO_TCP)) {
		(void)fprintf(stderr, "unable to register (SA1200TARGET, SA1200VERSION, tcp).\n");
		exit(1);
	}

    svc_run();
	(void)fprintf(stderr, "svc_run returned\n");
	exit(1);
}

static void
sa1200target_1(rqstp, transp)
	struct svc_req *rqstp;
	SVCXPRT *transp;
{
	union {
		rs_uDbg_uStoreFree_T rs_udbg_initlib_1_arg;
		rs_uDbg_Handle_T rs_udbg_delelib_1_arg;
		rs_uDbg_Two_uint_T rs_udbg_resetueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_clrresetueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_startueng_1_arg;
		rs_uDbg_Two_uint_T rs_udbg_pauseueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_resumeueng_1_arg;
		rs_uDbg_One_uint_T rs_udbg_isuengactive_1_arg;
		rs_uDbg_One_uint_T rs_udbg_getactiveuengs_1_arg;
		rs_uDbg_GetThPC_T rs_udbg_getuthreadpc_1_arg;
		rs_uDbg_Three_uint_T rs_udbg_setuthreadspc_1_arg;
		rs_uDbg_Mem_T rs_udbg_setuengpc_1_arg;
		rs_uDbg_One_uint_T rs_udbg_getuengpc_1_arg;
		rs_uDbg_uWord_T rs_udbg_getuword_1_arg;
		rs_uDbg_uWord_T rs_udbg_setuword_1_arg;
		rs_uDbg_uWord_T rs_udbg_getnumuword_1_arg;
		rs_uDbg_uWord_T rs_udbg_setnumuword_1_arg;
		rs_uDbg_Mem_T rs_udbg_writemem32_1_arg;
		rs_uDbg_Mem_T rs_udbg_readmem32_1_arg;
		rs_uDbg_Mem_T rs_udbg_writemem64_1_arg;
		rs_uDbg_Mem_T rs_udbg_readmem64_1_arg;
		rs_uDbg_BreakPoint_T rs_udbg_setbreakpoint_1_arg;
		rs_uDbg_One_uint_T rs_udbg_showbreakpoints_1_arg;
		rs_uDbg_One_uint_T rs_udbg_clearallbreakpoints_1_arg;
		rs_uDbg_One_uint_T rs_udbg_clearbreakpoint_1_arg;
		rs_uDbg_CSR_T rs_udbg_getcsr_1_arg;
		rs_uDbg_CSR_T rs_udbg_setcsr_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelgpr_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelgpr_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsdram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsram_rdxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_putrelsdram_rdxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsdram_wrxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsram_rdxfer_1_arg;
		rs_uDbg_Reg_T rs_udbg_getrelsdram_rdxfer_1_arg;
		rs_uDbg_One_uint_T rs_udbg_getuengstatus_1_arg;
		rs_uDbg_Two_uint_T rs_udbg_hopueng_1_arg;
		rs_CNTL_InitUof_T rs_cntl_inituof_1_arg;
		rs_uDbg_Three_uint_T rs_cntl_reset_1_arg;
		rs_uDbg_Three_uint_T rs_cntl_pause_1_arg;
		rs_uDbg_Two_uint_T rs_cntl_resume_1_arg;
		rs_uDbg_Two_uint_T rs_cntl_start_1_arg;
		rs_uDbg_Three_uint_T rs_cntl_hop_1_arg;
		rs_uDbg_Three_uint_T rs_udbg_definefreeustore_1_arg;
		rs_uDbg_One_uint_T rs_udbg_updatefreeustore_1_arg;
		rs_uDbg_FiFo_T rs_udbg_writefifo_tx_1_arg;
		rs_uDbg_FiFo_T rs_udbg_readfifo_rx_1_arg;
		rs_uDbg_Handle_T rs_udbg_getsysmemsize_1_arg;


	} argument;
	char *result;
	bool_t (*xdr_argument)(), (*xdr_result)();
	char *(*local)();

	switch (rqstp->rq_proc) {
	case NULLPROC:
		(void)svc_sendreply(transp, xdr_void, (char *)NULL);
		return;

	case RS_UDBG_INITLIB:
		xdr_argument = xdr_rs_uDbg_uStoreFree_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_initlib_1;
		break;

	case RS_UDBG_DELELIB:
		xdr_argument = xdr_rs_uDbg_Handle_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_delelib_1;
		break;

	case RS_UDBG_DEFINEFREEUSTORE:
		xdr_argument = xdr_rs_uDbg_Three_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_definefreeustore_1;
		break;

	case RS_UDBG_UPDATEFREEUSTORE:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_updatefreeustore_1;
		break;
	
	case RS_UDBG_RESETUENG:
		xdr_argument = xdr_rs_uDbg_Two_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_resetueng_1;
		break;

	case RS_UDBG_CLRRESETUENG:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_clrresetueng_1;
		break;

	case RS_UDBG_STARTUENG:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_startueng_1;
		break;

	case RS_UDBG_PAUSEUENG:
		xdr_argument = xdr_rs_uDbg_Two_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_pauseueng_1;
		break;

	case RS_UDBG_RESUMEUENG:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_resumeueng_1;
		break;

	case RS_UDBG_ISUENGACTIVE:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_isuengactive_1;
		break;

	case RS_UDBG_GETACTIVEUENGS:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_getactiveuengs_1;
		break;

	case RS_UDBG_GETUTHREADPC:
		xdr_argument = xdr_rs_uDbg_GetThPC_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getuthreadpc_1;
		break;

	case RS_UDBG_SETUTHREADSPC:
		xdr_argument = xdr_rs_uDbg_Three_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_setuthreadspc_1;
		break;

	case RS_UDBG_SETUENGPC:
		xdr_argument = xdr_rs_uDbg_Mem_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_setuengpc_1;
		break;

	case RS_UDBG_GETUENGPC:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getuengpc_1;
		break;

	case RS_UDBG_GETUWORD:
		xdr_argument = xdr_rs_uDbg_uWord_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getuword_1;
		break;

	case RS_UDBG_SETUWORD:
		xdr_argument = xdr_rs_uDbg_uWord_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_setuword_1;
		break;

	case RS_UDBG_GETNUMUWORD:
		xdr_argument = xdr_rs_uDbg_uWord_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getnumuword_1;
		break;

	case RS_UDBG_SETNUMUWORD:
		xdr_argument = xdr_rs_uDbg_uWord_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_setnumuword_1;
		break;

	case RS_UDBG_WRITEMEM32:
		xdr_argument = xdr_rs_uDbg_Mem_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_writemem32_1;
		break;

	case RS_UDBG_READMEM32:
		xdr_argument = xdr_rs_uDbg_Mem_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_readmem32_1;
		break;

	case RS_UDBG_WRITEMEM64:
		xdr_argument = xdr_rs_uDbg_Mem_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_writemem64_1;
		break;

	case RS_UDBG_READMEM64:
		xdr_argument = xdr_rs_uDbg_Mem_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_readmem64_1;
		break;

	case RS_UDBG_SETBREAKPOINT:
		xdr_argument = xdr_rs_uDbg_BreakPoint_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_setbreakpoint_1;
		break;

	case RS_UDBG_SHOWBREAKPOINTS:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_rs_uDbg_ShowBreakpts_T;
		local = (char *(*)()) rs_udbg_showbreakpoints_1;
		break;

	case RS_UDBG_CLEARALLBREAKPOINTS:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_clearallbreakpoints_1;
		break;

	case RS_UDBG_CLEARBREAKPOINT:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_clearbreakpoint_1;
		break;

	case RS_UDBG_GETCSR:
		xdr_argument = xdr_rs_uDbg_CSR_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getcsr_1;
		break;

	case RS_UDBG_SETCSR:
		xdr_argument = xdr_rs_uDbg_CSR_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_setcsr_1;
		break;

	case RS_UDBG_PUTRELGPR:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_putrelgpr_1;
		break;

	case RS_UDBG_GETRELGPR:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getrelgpr_1;
		break;

	case RS_UDBG_PUTRELSRAM_WRXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_putrelsram_wrxfer_1;
		break;

	case RS_UDBG_PUTRELSDRAM_WRXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_putrelsdram_wrxfer_1;
		break;

	case RS_UDBG_PUTRELSRAM_RDXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_putrelsram_rdxfer_1;
		break;

	case RS_UDBG_PUTRELSDRAM_RDXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_putrelsdram_rdxfer_1;
		break;

	case RS_UDBG_GETRELSRAM_WRXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getrelsram_wrxfer_1;
		break;

	case RS_UDBG_GETRELSDRAM_WRXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getrelsdram_wrxfer_1;
		break;

	case RS_UDBG_GETRELSRAM_RDXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getrelsram_rdxfer_1;
		break;

	case RS_UDBG_GETRELSDRAM_RDXFER:
		xdr_argument = xdr_rs_uDbg_Reg_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_getrelsdram_rdxfer_1;
		break;

	case RS_UDBG_GETUENGSTATUS:
		xdr_argument = xdr_rs_uDbg_One_uint_T;
		xdr_result = xdr_rs_uDbg_GetuEngStatus_T;
		local = (char *(*)()) rs_udbg_getuengstatus_1;
		break;

	case RS_UDBG_HOPUENG:
		xdr_argument = xdr_rs_uDbg_Two_uint_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_hopueng_1;
		break;

	case RS_CNTL_INITUOF:
		xdr_argument = xdr_rs_CNTL_InitUof_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_cntl_inituof_1;
		break;

	case RS_CNTL_RESET:
		xdr_argument = xdr_rs_uDbg_Three_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_cntl_reset_1;
		break;

	case RS_CNTL_PAUSE:
		xdr_argument = xdr_rs_uDbg_Three_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_cntl_pause_1;
		break;

	case RS_CNTL_RESUME:
		xdr_argument = xdr_rs_uDbg_Two_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_cntl_resume_1;
		break;

	case RS_CNTL_START:
		xdr_argument = xdr_rs_uDbg_Two_uint_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_cntl_start_1;
		break;

	case RS_CNTL_HOP:
		xdr_argument = xdr_rs_uDbg_Three_uint_T;
		xdr_result =xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_cntl_hop_1;
		break;

	case RS_UDBG_WRITEFIFO_TX:
		xdr_argument = xdr_rs_uDbg_FiFo_T;
		xdr_result = xdr_int;
		local = (char *(*)()) rs_udbg_writefifo_tx_1;
		break;

	case RS_UDBG_READFIFO_RX:
		xdr_argument = xdr_rs_uDbg_FiFo_T;
		xdr_result = xdr_rs_uDbg_Status_T;
		local = (char *(*)()) rs_udbg_readfifo_rx_1;
		break;

	case RS_UDBG_GETSYSMEMSIZE:
		xdr_argument = xdr_rs_uDbg_Handle_T;
		xdr_result = xdr_rs_uDbg_GetMemSize_T;
		local = (char *(*)()) rs_udbg_getsysmemsize_1;
		break;

	default:
		svcerr_noproc(transp);
		return;
	}
	bzero((char *)&argument, sizeof(argument));
	if (!svc_getargs(transp, xdr_argument, &argument)) {
		svcerr_decode(transp);
		return;
	}
	result = (*local)(&argument, rqstp);
	if (result != NULL && !svc_sendreply(transp, xdr_result, result)) {
		svcerr_systemerr(transp);
	}
	if (!svc_freeargs(transp, xdr_argument, &argument)) {
		(void)fprintf(stderr, "unable to free arguments\n");
		exit(1);
	}
}
