/* ebsa285ComDev.cpp
 * Digital EBSA-285 serial port communication
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: EBSA-285
 *  subsystem: Serial Communication Library
 *  author: drj March 1, 99 Initial version
 *  revisions:
 *  $History: ebsa285ComDev.cpp $
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:20p
 * Created in $/dev_1200/Share/scomm
 * 
 * ---------------------------------------------------------------------
 */

#include "ebsa285ComDev.h"

Ebsa285ComDev::Ebsa285ComDev(char *devName){
   if(!strcmp(devName, "com1")){
          uart = (UartReg *)0x42000160;
//dj: test	  uart = (UartReg *)Uart;
	  status = 1;
   }
}

int Ebsa285ComDev::getx(char *buf, unsigned int len)
{
   unsigned int i, stat;
   for(i = 0; i < len;){
	   if(uart->rxData()){
		 buf[i] = uart->getChar();
		 stat = uart->getRxStat();
		 if((stat & 0x3) == 0) i++;
	  }
   }
   return(i);
}

int Ebsa285ComDev::get(char *c, unsigned int len)
{
   unsigned int stat;
   //assume rxdata is true -- interrupt driven
   *c = uart->getChar();
   stat = uart->getRxStat();
   if((stat & 0x3) == 0) return(1);
   return(0);
}


int Ebsa285ComDev::put(char *buf, unsigned int len)
{
   unsigned int i, t=0;
   for(i = 0; i < len;){
	  if(uart->txFifoReady()){
		 uart->putChar(buf[i++]);
	  }
   }
   return(i);
}


int Ebsa285ComDev::setBaud(int baudrate)
{
   int oldRate = getBaud();

   switch(baudrate){
	  case BR_300: uart->setBRD(BAUDRATE_300); break;
	  case BR_1200: uart->setBRD(BAUDRATE_1200); break;
	  case BR_2400: uart->setBRD(BAUDRATE_2400); break;
			case BR_4800: uart->setBRD(BAUDRATE_4800); break;
			case BR_9600: uart->setBRD(BAUDRATE_9600); break;
			case BR_14400: uart->setBRD(BAUDRATE_14400); break;
			case BR_19200: uart->setBRD(BAUDRATE_19200); break;
			case BR_38400: uart->setBRD(BAUDRATE_38400); break;
			case BR_57600: uart->setBRD(BAUDRATE_57600); break;
			case BR_115200: uart->setBRD(BAUDRATE_115200); break;
			case BR_230400: uart->setBRD(BAUDRATE_230400); break;
			default: 		setBaudrate(baudrate); break;
   }
   return(oldRate);
}

int Ebsa285ComDev::getBaud(void)
{
   switch(uart->getBRD()){
	  case BAUDRATE_300: return(BR_300);
	  case BAUDRATE_1200: return(BR_1200);
			case BAUDRATE_2400: return(BR_1200);
			case BAUDRATE_4800: return(BR_4800);
			case BAUDRATE_9600: return(BR_9600);
			case BAUDRATE_14400: return(BR_14400);
			case BAUDRATE_19200: return(BR_19200);
			case BAUDRATE_38400: return(BR_38400);
			case BAUDRATE_57600: return(BR_57600);
			case BAUDRATE_115200: return(BR_115200);
			case BAUDRATE_230400: return(BR_230400);
   }
   return(BR_OTHER);
}
	

int Ebsa285ComDev::setDevCtlBlk(DevCtlBlk_T *devCtlBlk)
{
   if(!devCtlBlk) return(-1);

   uart->setBreak(1);
   uart->setUartEn(0);		// disable uart

   uart->setDataSizeSelect(devCtlBlk->byteSize);

   if(devCtlBlk->parity == ODD_PARITY){
	  uart->setParityEnable(1);
	  uart->setOddEvenSelect(0);
   }
   else if(devCtlBlk->parity == EVEN_PARITY){
	  uart->setParityEnable(1);
	  uart->setOddEvenSelect(1);
   }
   else uart->setParityEnable(0);

   //uart->setEnableFIFO(1);
   uart->setStopBitSelect(devCtlBlk->stopBits);

   setBaud(devCtlBlk->baudRate);

   uart->setBreak(0);
   uart->setUartEn(devCtlBlk->enableDev);	// enable/disable uart

   return(0);
}


int Ebsa285ComDev::getDevCtlBlk(DevCtlBlk_T *devCtlBlk)
{
   if(!devCtlBlk) return(-1);

   switch(uart->getBRD()){
	  case BAUDRATE_300: devCtlBlk->baudRate = BR_300; break;
	  case BAUDRATE_1200: devCtlBlk->baudRate = BR_1200; break;
	  case BAUDRATE_2400: devCtlBlk->baudRate = BR_2400; break;
	  case BAUDRATE_4800: devCtlBlk->baudRate = BR_4800; break;
	  case BAUDRATE_9600: devCtlBlk->baudRate = BR_9600; break;
	  case BAUDRATE_14400: devCtlBlk->baudRate = BR_14400; break;
	  case BAUDRATE_19200: devCtlBlk->baudRate = BR_19200; break;
	  case BAUDRATE_38400: devCtlBlk->baudRate = BR_38400; break;
	  case BAUDRATE_57600: devCtlBlk->baudRate = BR_57600; break;
	  case BAUDRATE_115200: devCtlBlk->baudRate = BR_115200; break;
	  case BAUDRATE_230400: devCtlBlk->baudRate = BR_230400; break;
	  default: devCtlBlk->baudRate = uart->getBRD(); break;
   }

   devCtlBlk->byteSize = uart->getDataSizeSelect();
   if(uart->getParityEnable()) devCtlBlk->parity = uart->getOddEvenSelect();
   else devCtlBlk->parity = NO_PARITY;

   devCtlBlk->stopBits = uart->getStopBitSelect();
   devCtlBlk->enableDev = uart->getUartEn();

   return(0);
}
