/* ebsa285ComDev.h
 * Digital EBSA-285 serial port communication
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: SA1200
 *  subsystem: Serial Communication Library
 *  author: drj March 1, 99 Initial version
 *  revisions:
 *  $History: ebsa285ComDev.h $
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 15  *****************
 * User: Drjohns1     Date: 3/27/00    Time: 5:12p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 12  *****************
 * User: Drjohns1     Date: 9/13/99    Time: 10:39a
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 11  *****************
 * User: Drjohns1     Date: 9/07/99    Time: 12:51p
 * Updated in $/dev_1200/Share/scomm
 * 
 * ---------------------------------------------------------------------
 */

#ifndef __EBSA285COMDEV_H
#define __EBSA285COMDEV_H
#include <stdio.h>
#include <string.h>

#define CONTROL_STATUS_BASE             0x42000000
#define UART1_BASE                      CONTROL_STATUS_BASE + 0x160
#define UART3_BASE                      0x80050000

const unsigned int FCLK_IN = 5000000;
const unsigned int BAUD_CLK = (FCLK_IN / 4);
const unsigned int BRD_CONT = 16;

// ESB-285 Baudrate divisors (based on 50Mhz fclk_in):  (baud_clk / 16 * baud_rate) - 1
const unsigned int BAUDRATE_230400	= 1;
const unsigned int BAUDRATE_115200	= 6;
const unsigned int BAUDRATE_57600	= 13;
const unsigned int BAUDRATE_38400   = 19;
const unsigned int BAUDRATE_19200	= 40;
const unsigned int BAUDRATE_14400	= 53;
const unsigned int BAUDRATE_9600	= 80;
const unsigned int BAUDRATE_4800	= 162;
const unsigned int BAUDRATE_2400	= 325;
const unsigned int BAUDRATE_1200	= 650;
const unsigned int BAUDRATE_300		= 2603;


const unsigned int RFIFO_SIZE = 16;		// serial device receive FIFO size


const unsigned int BRK_MASK	= 0x1;
const unsigned int PE_MASK	= 0x2;
const unsigned int OES_MASK	= 0x4;
const unsigned int SBS_MASK	= 0x8;
const unsigned int EF_MASK	= 0x10;
const unsigned int DSS_MASK	= 0x60;
const unsigned int H_BRD_MASK	= 0xf;
const unsigned int L_BRD_MASK	= 0xff;
const unsigned int UE_MASK	= 0x1;
const unsigned int SHS_MASK	= 0x2;
const unsigned int RIDT_MASK	= 0x4;
const unsigned int TBY_MASK	= 0x8;
const unsigned int RFS_MASK	= 0x10;
const unsigned int TFS_MASK	= 0x20;


#include "scommdev.h"


class UartReg{
public:
	UartReg(){}
	~UartReg(){}

	inline void putChar(char c){uartdr = c;}
	inline char getChar(void){return((uartdr & 0xff));}

	inline int getRxStat(void){return(rxstat);}
	inline unsigned int txBusy(void){return(uartflg & 0x8);}

	inline unsigned int txFifoReady(void){return((!(uartflg & 0x20)) && (uartcon & 0x1));}
	inline unsigned int rxData(void){return(!(uartflg & 0x10));}

 	inline void setUartEn(char state){uartcon &= ~UE_MASK; uartcon |= state & UE_MASK;}  // this also clears S & I
	inline unsigned int getUartEn(void){return(uartcon & UE_MASK);}

	inline void setBreak(char state){h_ubrlcr &= ~BRK_MASK; h_ubrlcr |= 0x1 & state;}
	inline unsigned int getBreak(void){return(h_ubrlcr & 0x1);}

	inline void setParityEnable(char state){h_ubrlcr &= ~PE_MASK; h_ubrlcr |= (0x1 & state) << 1;}
	inline unsigned int getParityEnable(void){return((h_ubrlcr >> 1) & 0x1);}

	inline void setOddEvenSelect(char state){h_ubrlcr &= ~OES_MASK; h_ubrlcr |= (0x1 & state) << 2;}
	inline unsigned int getOddEvenSelect(void){return((h_ubrlcr >> 2) & 0x1);}

	inline void setStopBitSelect(char state){h_ubrlcr &= ~SBS_MASK; h_ubrlcr |= (0x1 & state) << 3;}
	inline unsigned int getStopBitSelect(void){return((h_ubrlcr >> 3) & 0x1);}

	inline void setEnableFIFO(char state){h_ubrlcr &= ~EF_MASK; h_ubrlcr |= (0x1 & state) << 4;}
	inline unsigned int getEnableFIFO(void){return((h_ubrlcr >> 4) & 0x1);}

	inline void setDataSizeSelect(char dataSize){h_ubrlcr &= ~DSS_MASK; h_ubrlcr |= (0x3 & dataSize) << 5;}
	inline unsigned int getDataSizeSelect(void){return((h_ubrlcr >> 5) & 0x3);}

	inline void setBRD(unsigned int baud){m_ubrlcr = (baud >> 8) & 0xf; l_ubrlcr = baud & 0xff; return;}
	inline unsigned int getBRD(){return(((m_ubrlcr & 0xf) << 8) | (l_ubrlcr & 0xff));}

void show(void){
   printf("\nuartdr = 0x%x+, rxstat=0x%x, h_ubrlcr=0x%x(0x%x), m_ubrlcr=0x%x(0x%x), l_ubrlcr=0x%x(0x%x), uartcon=0x%x(0x%x), uartflg=0x%x(0x%x)\n",
		&uartdr, &rxstat, &h_ubrlcr, h_ubrlcr,
		&m_ubrlcr, m_ubrlcr, &l_ubrlcr, l_ubrlcr, &uartcon, uartcon,
		&uartflg, uartflg);
   printf("uEn=%d, break=%d, parityEn=%d, oddEvenSel=%d, stopBitSel=%d, fifoEn=%d, dataSizeSel=%d, brd=%d\n",
          getUartEn(), getBreak(), getParityEnable(),getOddEvenSelect(),
          getStopBitSelect(), getEnableFIFO(), getDataSizeSelect(), getBRD());
   fflush(stdout);
   return;
}


private:
	unsigned int uartdr;				// 0
	unsigned int rxstat;				// 4
	unsigned int h_ubrlcr;				// 8
	unsigned int m_ubrlcr;				// 12
	unsigned int l_ubrlcr;				// 16
	unsigned int uartcon;				// 20
	unsigned int uartflg;				// 24
};

//dj: debug
//static unsigned int Uart[] = {0x1d, 0x0, 0x60, 0x0, 0x28, 0x1, 0x38};
extern "C" int SComm_Print(char *format, ...);

class Ebsa285ComDev : public Device{
public:
	Ebsa285ComDev(char *devName);
	~Ebsa285ComDev(){}

	int get(char *c, unsigned int len);
	int getx(char *buf, unsigned int len);
	int put(char *buf, unsigned int len);
	int setBaud(int baudrate);
	int getBaud(void);


	inline int created(){return((status >= 0));}
	inline void enableDev(){uart->setUartEn(1);}
	inline void disableDev(){uart->setUartEn(0);}
    int setDevCtlBlk(DevCtlBlk_T *devCtlBlk);
	int getDevCtlBlk(DevCtlBlk_T *devCtlBlk);

    inline void clearInterrupt(){}
	inline void enableIntTx(){*(unsigned int *)0x42000188 = 0x8;}
	inline void enableIntRx(){*(unsigned int *)0x42000188 = 0x4;}

	//disable the IRQ rx and tx.  Angel is also disabled by disabling the FIQ tx & rx 
	inline void disableIntTx(){*(unsigned int *)0x4200028c = 0x8; *(unsigned int *)0x4200018c = 0x8;}
	inline void disableIntRx(){*(unsigned int *)0x4200028c = 0x4; *(unsigned int *)0x4200018c = 0x4;}

	inline void sanity(void){uart->show();}

private:
	inline void setBaudrate(int baudrate){uart->setBRD((BAUD_CLK/(BRD_CONT * baudrate))-1);}
	int status;
	UartReg *uart;
};

#endif			//__EBSA285COMDEV_H
