/* win32HpcDev.cpp
 * Win32 HPC Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Serial Communication Library
 * author: drj Mar 16, 99 Initial version
 * revisions:
 * $History: ixp1200ComDev.cpp $
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 6/28/00    Time: 2:17p
 * Created in $/dev_1200/Share/scomm
 * 
 * ---------------------------------------------------------------------
 */

#include "ixp1200ComDev.h"


ixp1200ComDev::ixp1200ComDev(char *devName){
   uart = new UartReg();
   status = 1;
}


ixp1200ComDev::~ixp1200ComDev()
{
	delete(uart);
}


int ixp1200ComDev::setBaud(int baudrate)
{
   int oldRate = getBaud();

   switch(baudrate){
	  case BR_300: uart->setBRD(BAUDRATE_300); break;
	  case BR_1200: uart->setBRD(BAUDRATE_1200); break;
	  case BR_2400: uart->setBRD(BAUDRATE_2400); break;
			case BR_4800: uart->setBRD(BAUDRATE_4800); break;
			case BR_9600: uart->setBRD(BAUDRATE_9600); break;
			case BR_14400: uart->setBRD(BAUDRATE_14400); break;
			case BR_19200: uart->setBRD(BAUDRATE_19200); break;
			case BR_38400: uart->setBRD(BAUDRATE_38400); break;
			case BR_57600: uart->setBRD(BAUDRATE_57600); break;
			case BR_115200: uart->setBRD(BAUDRATE_115200); break;
			default: 		setBaudrate(baudrate); break;
   }
   return(oldRate);
}


int ixp1200ComDev::getBaud(void)
{
   switch(uart->getBRD()){
	  case BAUDRATE_300: return(BR_300);
	  case BAUDRATE_1200: return(BR_1200);
			case BAUDRATE_2400: return(BR_2400);
			case BAUDRATE_4800: return(BR_4800);
			case BAUDRATE_9600: return(BR_9600);
			case BAUDRATE_14400: return(BR_14400);
			case BAUDRATE_19200: return(BR_19200);
			case BAUDRATE_38400: return(BR_38400);
			case BAUDRATE_57600: return(BR_57600);
			case BAUDRATE_115200: return(BR_115200);
   }
   return(BR_OTHER);
}
	

int ixp1200ComDev::setDevCtlBlk(DevCtlBlk_T *devCtlBlk)
{
   if(!devCtlBlk) return(-1);

   uart->setBreak(ENABLE);
   uart->setUartEn(DISABLE);		/* disable uart */

   uart->setDataSizeSelect(devCtlBlk->byteSize);

   if(devCtlBlk->parity == ODD_PARITY){
	  uart->setParityEnable(ENABLE);
	  uart->setOddEvenSelect(ODD_SELECT);
   }
   else if(devCtlBlk->parity == EVEN_PARITY){
	  uart->setParityEnable(ENABLE);
	  uart->setOddEvenSelect(EVEN_SELECT);
   }
   else uart->setParityEnable(DISABLE);

   uart->setStopBitSelect(devCtlBlk->stopBits);

   setBaud(devCtlBlk->baudRate);

   uart->setBreak(DISABLE);

   // enable uart if requested, otherwise leave disabled
   if(devCtlBlk->enableDev)
	  uart->setUartEn(devCtlBlk->enableDev);	/* enable uart */

   return(0);
}


int ixp1200ComDev::getDevCtlBlk(DevCtlBlk_T *devCtlBlk)
{
   if(!devCtlBlk) return(-1);

   switch(uart->getBRD()){
	  case BAUDRATE_300: devCtlBlk->baudRate = BR_300; break;
	  case BAUDRATE_1200: devCtlBlk->baudRate = BR_1200; break;
	  case BAUDRATE_2400: devCtlBlk->baudRate = BR_2400; break;
	  case BAUDRATE_4800: devCtlBlk->baudRate = BR_4800; break;
	  case BAUDRATE_9600: devCtlBlk->baudRate = BR_9600; break;
	  case BAUDRATE_14400: devCtlBlk->baudRate = BR_14400; break;
	  case BAUDRATE_19200: devCtlBlk->baudRate = BR_19200; break;
	  case BAUDRATE_38400: devCtlBlk->baudRate = BR_38400; break;
	  case BAUDRATE_57600: devCtlBlk->baudRate = BR_57600; break;
	  case BAUDRATE_115200: devCtlBlk->baudRate = BR_115200; break;
	  default: devCtlBlk->baudRate = uart->getBRD(); break;
   }

   devCtlBlk->byteSize = uart->getDataSizeSelect();
   if(uart->getParityEnable()) devCtlBlk->parity = uart->getOddEvenSelect();
   else devCtlBlk->parity = NO_PARITY;

   devCtlBlk->stopBits = uart->getStopBitSelect();
   devCtlBlk->enableDev = uart->getUartEn();

   return(0);
}
