/* ixp1200DevCom.h
 * Intel IXP1200 serial port communication
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c) 1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: SA1200
 *  subsystem: Serial Communication Library
 *  author: drj April 29, 99 Initial version
 *  revisions:
 *  $History: ixp1200ComDev.h $
 * 
 * *****************  Version 10  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 9  *****************
 * User: Drjohns1     Date: 3/27/00    Time: 5:12p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 6  *****************
 * User: Drjohns1     Date: 12/03/99   Time: 1:14p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 5  *****************
 * User: Drjohns1     Date: 8/02/99    Time: 6:02p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 3  *****************
 * User: Drjohns1     Date: 7/15/99    Time: 5:37p
 * Updated in $/dev_1200/Share/scomm
 * Updated for A1
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 7/07/99    Time: 7:46p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 1  *****************
 * User: Qabuild      Date: 4/30/99    Time: 12:22p
 * Created in $/dev_1200/Share/scomm
 * 
 * 
 * ---------------------------------------------------------------------
 */

#ifndef __IXP1200COMDEV_H
#define __IXP1200COMDEV_H
#include <stdio.h>
#include <string.h>

#define UART_CR		0x90003800
#define UART_SR		0x90003400
#define UART_DR		0x90003c00

#define FIQ		0x90001000
#define IRQ		0x90001400



//BAUD_RATE_DIVISOR = (FCLK_IN/16/BAUD_RATE)-1 
#if SA1200_A0

/* SA1200 baudrate divisors -- 100000000 clock */
const unsigned int FCLK_IN = 100000000;
const unsigned int BRD_CONST = (FCLK_IN / 32 / 16);

const unsigned int BAUDRATE_115200	= 0x0;
const unsigned int BAUDRATE_57600	= 0x2;
const unsigned int BAUDRATE_38400   = 0x4;
const unsigned int BAUDRATE_19200	= 0x9;
const unsigned int BAUDRATE_14400	= 0xc;
const unsigned int BAUDRATE_9600	= 0x13;
const unsigned int BAUDRATE_4800	= 0x27;
const unsigned int BAUDRATE_2400	= 0x50;
const unsigned int BAUDRATE_1200	= 0xa1;
const unsigned int BAUDRATE_300		= 0x28a;
#else
/* SA1200 baudrate divisors  -- 3686400 clock */
const unsigned int FCLK_IN = 3686400;
const unsigned int BRD_CONST = (FCLK_IN / 16);

const unsigned int BAUDRATE_230400	= 0x000;
const unsigned int BAUDRATE_115200	= 0x001;
const unsigned int BAUDRATE_57600	= 0x003;
const unsigned int BAUDRATE_38400	= 0x005;
const unsigned int BAUDRATE_19200	= 0x00B;
const unsigned int BAUDRATE_14400	= 0x00F;
const unsigned int BAUDRATE_9600	= 0x017;
const unsigned int BAUDRATE_4800	= 0x2F;
const unsigned int BAUDRATE_2400	= 0x05F;
const unsigned int BAUDRATE_1200	= 0x0BF;
const unsigned int BAUDRATE_300		= 0x2FF;
#endif

const unsigned int RFIFO_SIZE = 8;		/* serial device receive FIFO size */


const unsigned int DATA_MASK	= 0xff;
const unsigned int RX_STAT_MASK = 0x3;
const unsigned int UE_MASK		= 0x80;
const unsigned int TXR_MASK		= 0x4;
const unsigned int TXF_MASK		= 0x8;
const unsigned int RXR_MASK		= 0x10;
const unsigned int BRK_MASK		= 0x1;
const unsigned int PE_MASK		= 0x2;
const unsigned int PS_MASK		= 0x4;
const unsigned int SBS_MASK		= 0x8;
const unsigned int DSS_MASK		= 0x60;
const unsigned int BRD_MASK		= 0x3ff0000;
const unsigned int XIE_MASK		= 0x100;
const unsigned int RIE_MASK		= 0x10;
const unsigned int UIC_MASK		= 0x200;


#include "scommdev.h"


class UartReg{
public:
	UartReg(){
		cr = (unsigned int *)UART_CR;
		sr = (unsigned int *)UART_SR;
		dr = (unsigned int *)UART_DR;
	}
	~UartReg(){}

	inline void putChar(char c){(*dr) = (DATA_MASK & c);}
	inline char getChar(void){return(((*dr) & DATA_MASK));}

	inline int getRxStat(void){return(((*sr) >> 2) & RX_STAT_MASK);}
	inline unsigned int txBusy(void){return((*sr) & TXF_MASK);}

	inline unsigned int txFifoReady(void){return(((*sr) & TXR_MASK));}
	inline unsigned int rxData(void){return(((*sr) & RXR_MASK));}

 	inline void setUartEn(char state){(*cr) &= ~UE_MASK; (*cr) |= (state & 0x1) << 7;}
	inline unsigned int getUartEn(void){return(((*cr) >> 7) & 0x1);}

	inline void setBreak(char state){(*cr) &= ~BRK_MASK; (*cr) |= 0x1 & state;}
	inline unsigned int getBreak(void){return((*cr) & BRK_MASK);}

	inline void setParityEnable(char state){(*cr) &= ~PE_MASK; (*cr) |= (0x1 & state) << 1;}
	inline unsigned int getParityEnable(void){return((*cr) & PE_MASK);}

	inline void setOddEvenSelect(char state){(*cr) &= ~PS_MASK; (*cr) |= (0x1 & state) << 2;}
	inline unsigned int getOddEvenSelect(void){return(((*cr) >> 2) & 0x1);}

	inline void setStopBitSelect(char state){(*cr) &= ~SBS_MASK; *cr |= (0x1 & state) << 3;}
	inline unsigned int getStopBitSelect(void){return(((*cr) >> 3) & 0x1);}

	inline void setDataSizeSelect(char dataSize){(*cr) &= ~DSS_MASK; (*cr) |= (0x3 & dataSize) << 5;}
	inline unsigned int getDataSizeSelect(void){return(((*cr) >> 5) & 0x3);}

	inline void setBRD(unsigned int baud){(*cr) &= ~BRD_MASK; (*cr) |= (baud << 16) & BRD_MASK; return;}
	inline unsigned int getBRD(){return(((*cr) & BRD_MASK) >> 16);}

	inline void enableIntTx(){(*cr) |= XIE_MASK; return;};
	inline void enableIntRx(){(*cr) |= RIE_MASK; return;};
	inline void disableIntTx(){(*cr) &= ~XIE_MASK; return;};
	inline void disableIntRx(){(*cr) &= ~RIE_MASK; return;};
	inline void clearInt(){(*cr) |= UIC_MASK; return;}

	inline int get(char *c, unsigned int len){
		int cRead = 0;
		unsigned int datar;

		while((cRead < len) && rxData()){
			datar = *dr;
		    /* if framing or parity error, ignore char */
	        if(!((datar >> 8) & 0x3))  c[cRead++] = datar & DATA_MASK;
		}
 	    return(cRead);
	}

	inline int put(char *buf, unsigned int len){
	   unsigned int i;
	   for(i = 0; i < len;){
		  if(txFifoReady()){
		     putChar(buf[i++]);
		  }
	   }
	   return(i);
	}


void show(void){
   printf("\nsr = 0x%x+, cr=0x%x, dr=0x%x\n",	*sr, *cr, *dr);
   printf("uEn=%d, break=%d, parityEn=%d, oddEvenSel=%d, stopBitSel=%d, dataSizeSel=%d, brd=%d\n",
          getUartEn(), getBreak(), getParityEnable(),getOddEvenSelect(),
          getStopBitSelect(), getDataSizeSelect(), getBRD());
   fflush(stdout);
   return;
}


private:
	unsigned int *sr;				/* status  register */
	unsigned int *cr;				/* control register */
	unsigned int *dr;				/* data register */
};

extern "C" int SComm_Print(char *format, ...);

#define ENABLE		1
#define DISABLE		0
#define ODD_SELECT	1
#define EVEN_SELECT 0

class ixp1200ComDev : public Device{
public:
	ixp1200ComDev(char *devName);
	~ixp1200ComDev();

	inline int get(char *c, unsigned int len){return(uart->get(c, len));}
	inline int put(char *c, unsigned int len){return(uart->put(c, len));}
	inline int created(){return((status >= 0));}
	inline void enableDev(){uart->setUartEn(ENABLE);}
	inline void disableDev(){uart->setUartEn(DISABLE);}

	int setBaud(int baudrate);
	int getBaud(void);
	int setDevCtlBlk(DevCtlBlk_T *devCtlBlk);
	int getDevCtlBlk(DevCtlBlk_T *devCtlBlk);


    inline void clearInterrupt(){uart->clearInt();}
	inline void enableIntTx(){uart->enableIntTx();};
	inline void enableIntRx(){uart->enableIntRx();};
	inline void disableIntTx(){uart->disableIntTx();};
	inline void disableIntRx(){uart->disableIntRx();};

	inline void sanity(void){uart->show();}


private:
	inline void setBaudrate(int baudrate){uart->setBRD((BRD_CONST / baudrate)-1);}
	int status;
	UartReg *uart;
};

#endif			/* __IXP1200COMDEV_H */

