#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>

#ifdef WIN32
#define _AFXDLL
#include <afx.h>
#endif

#include "ostype.h"
#include "utils.h"


//const unsigned int SRAM_BASE = 0x10000000;
const unsigned int OUR_SIGNATURE = 0xfeedcafe;
static sem_T MemLogSem;
static char *LastAddr, *FirstAddr;
static char   *bufPtr;
static unsigned int MaxMsgLen;


typedef struct {
	unsigned int maxLen;
	unsigned int numMsg;
	unsigned int maxMsg;
	unsigned int sign;
	unsigned int spin;
	char *head, *tail;
}MemLog_T;

MemLog_T *MemLog;

#define INCR_PTR(ptr) \
  if((ptr) == LastAddr) (ptr) = FirstAddr; \
  else (ptr) = (ptr) + MaxMsgLen

char *allocMsg()
{
   char *cptr = NULL;

   Sem_Wait(MemLogSem, SEM_WAITFOREVER);
   if(MemLog->numMsg < MemLog->maxMsg){
	  cptr = MemLog->tail;
      MemLog->numMsg++;
      INCR_PTR(MemLog->tail);
   }
   else if(MemLog->spin){
	  cptr = MemLog->tail;
      INCR_PTR(MemLog->head);
      INCR_PTR(MemLog->tail);
   }
   Sem_Post(MemLogSem);
   return(cptr);
}

#ifdef __cplusplus
extern "C" {
#endif

int memLog(char *format,...)
{
   static char *msg;
   int msgLen = 0;
   va_list ap;

#if ((OS == UCOS) || (OS == VXWORKS))
   if(!(msg = allocMsg())) return(0);
   va_start(ap, format);
   msgLen = vsprintf(msg, format, ap);
   va_end(ap);
#else
   char msgx[256];
   va_start(ap, format);
   msgLen = vsprintf(msgx, format, ap);
   TRACE("%s\n", msgx);
   va_end(ap);
#endif
   return(msgLen);
}

void memDumpLog(void)
{
	unsigned int i;
   Sem_Wait(MemLogSem, SEM_WAITFOREVER);

	char *s = MemLog->head;
    if(MemLog->numMsg > MemLog->maxMsg){
		printf("memDumpLog: Invalid values numMsg(%d) > maxMsg(%d)\n",
			MemLog->numMsg, MemLog->maxMsg);
		return;
	}
	for(i=0; i < MemLog->numMsg; i++){
	   printf("%s\n", s);
	   INCR_PTR(s);
	}
    Sem_Post(MemLogSem);
    return;
}

void initMemLog(char *memBase, int maxMsg, int maxMsglen, int circular)
{
	MemLog = (MemLog_T *)memBase;
    MemLogSem = Sem_Init(1);
#ifdef WIN32
	return;
#endif
	if((MemLog->sign == OUR_SIGNATURE) && MemLog->numMsg){
		MaxMsgLen = MemLog->maxLen;
		FirstAddr = (char *)MemLog->head;
		LastAddr = (char *)((unsigned int)FirstAddr + (MemLog->maxMsg * MemLog->maxLen));
		memDumpLog();
	}

	MemLog->maxLen = maxMsglen;
	MemLog->maxMsg = maxMsg;
	MemLog->numMsg = 0;
    MemLog->sign = OUR_SIGNATURE;
	MemLog->spin = circular;
	MaxMsgLen =  MemLog->maxLen;
	FirstAddr = (char *)((unsigned int)memBase + sizeof(MemLog_T));
	LastAddr = (char *)((unsigned int)FirstAddr + (MemLog->maxMsg * MaxMsgLen));
	MemLog->head = MemLog->tail = FirstAddr;
    return;
}
#ifdef __cplusplus
}
#endif
