#include <windows.h>
#include <winioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ostype.h"
#include "osApi.h"
#include "core_io.h"
#include "IXPHPC_shared.h"

static HANDLE Hpc_fh;
OVERLAPPED AsyncRead, AsyncWrite;
char *Msg = "abcdefghijklmnopqrstuvwxyz";
static unsigned int WrError = 0, RdError = 0;
static const unsigned int BufSize = 256;
static unsigned int BytesRecv = 0, BytesSent = 0;

// reset the ixp1200 board
void __cdecl resetixp(HANDLE hDriver)
{
	unsigned long dwErr, cbReturned =0;

    //
    // Try to reset
    //
    if (DeviceIoControl (hDriver,
                         (DWORD) IOCTL_IXPHPC_RESET_IXP1200,
                         0,0,0,0,&cbReturned,0) )
    {
        printf ("DeviceIoControl RESET succeeded\n");
    } else {

        dwErr = GetLastError();
		printf ("FAIL: the error returned from GetLastError: %d\n", dwErr);
		if (dwErr == 6) printf("FAIL: Invalid file handle.\n");
		printf ("FAIL: resetixp() DeviceIoControl failed\n");
		printf ("FAIL: cbReturned = %d\n", cbReturned);
		
	}
	return;
} // end resetixp


int recvBytes(char *buf, unsigned int bytesToRead)
{
	int fReadStat, dwError, dwEvtMask = 0, retry = 100;
	unsigned long bytesRead = 0;
	
	if(!(fReadStat = ReadFile(Hpc_fh, buf, (unsigned long)bytesToRead, (unsigned long *)&bytesRead, &AsyncRead))){
		if((dwError = GetLastError()) == ERROR_IO_PENDING){
			if(!GetOverlappedResult(Hpc_fh, &AsyncRead, (unsigned long *)&bytesRead, TRUE))
				printf("***GetOverlappedResult failed, err=0x%x\n", GetLastError());
			//delayMs(40);
		}
	}
	return(bytesRead);
}


int sendBytes(char *buf, unsigned int bytesToWrite)
{
	int fWriteStat, dwError;
	unsigned int bytesSent=0, retry=100;

	//	printf("Writing %d bytes to hpc device\n", bytesToWrite);
	if(!(fWriteStat = WriteFile(Hpc_fh, buf, (unsigned long)bytesToWrite, (unsigned long *)&bytesSent, &AsyncWrite))){
		if((dwError = GetLastError()) == ERROR_IO_PENDING){
			while(retry && !GetOverlappedResult(Hpc_fh, &AsyncWrite, (unsigned long *)&bytesSent,FALSE)){
				dwError = GetLastError();
				if(dwError == ERROR_IO_INCOMPLETE){
					// normal result if not finished
					//retry--;
					//delayMs(1);
					continue;
				}
				else{
					printf("***Error writing to device, error=%d\n", dwError); fflush(stdout);
					if(bytesSent < bytesToWrite) continue;
					break;
				}
			}
		}
		else{
			printf("***Error writing to device, error=%d\n",dwError); fflush(stdout);
		}

	}

	if(!bytesSent && !retry){
		if(!CancelIo(Hpc_fh))
			printf("CancelIo failed, error=%d\n", GetLastError());
		printf("Timeout trying to write to hpc device\n");
	}
	return(bytesSent);
}

void hpcRecv(void *)
{
	int bytesRead = 0;
	char buf[BufSize];
	unsigned int msgNum = 0, nextMsgNum = 0;

	unsigned int bytesToRead = strlen(Msg) + 7;
	
	while(1){
		bytesRead = 0;
		if((bytesRead = recvBytes(buf, bytesToRead))){

			if(!strlen(buf)){
				printf("*** data not ready -- not copied by hpc\n");
				continue;
			}

			msgNum = atoi(buf);
			if(msgNum != nextMsgNum){
				printf("**expecMsgNum=%d, msgNum=%d, ", nextMsgNum, msgNum);
				nextMsgNum = msgNum+1;
			}
			else nextMsgNum++;

			printf("%d bytes Msg = '%s'\n",bytesRead, buf);
			BytesRecv += bytesRead;
			buf[0] = '\0';
		}else printf("####read zero bytes\n");
	}
	return;
}


void hpcSend(void *)
{
	int bytesSent=0, bytesToWrite;
	unsigned int msgNum = 0;
	char buf[BufSize];


	while(1){
		sprintf(buf, "%05d %s", msgNum++, Msg);
		bytesToWrite = strlen(buf) + 1;
		if((bytesSent = sendBytes(buf, bytesToWrite)) != bytesToWrite) WrError++;
		else BytesSent += bytesSent;
		//delayMs(100);
	}
	return;
}


void hpcBoth(void)
{
	Thread_Create(MID_PRIORITY, (threadFunc_T)hpcSend, NULL);
	hpcRecv(NULL);
	return;
}

void hpcEcho(void)
{
	char buf[BufSize];
	int bytesRead, bytesSent;

	while(1){
		if((bytesRead = recvBytes(buf, BufSize))){
			if(bytesRead > 0){
				BytesRecv += bytesRead;
				if((bytesSent = sendBytes(buf, bytesRead)) != bytesRead) WrError++;
				else BytesSent += bytesSent;
			}
		}
		else{
			RdError++;
			delayMs(100);
		}
	}
	return;
}


int main(int argc, char **argv)
{
	char *dev = "\\\\.\\ixphpc00";

	if((Hpc_fh = CreateFile(dev, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED, NULL)) == INVALID_HANDLE_VALUE){
		printf("Device::Device: Error opening %s, error=%d\n", dev, GetLastError());
		return(-1);
	}

	AsyncRead.Internal = 0;
	AsyncRead.InternalHigh = 0;
	AsyncRead.hEvent = NULL;
	AsyncRead.Offset = 0 ;
	AsyncRead.OffsetHigh = 0 ;

	AsyncWrite.hEvent = NULL;
	AsyncWrite.Offset = 0 ;
	AsyncWrite.OffsetHigh = 0 ;

	if((AsyncRead.hEvent = CreateEvent(
		NULL,    // no security attributes 
		TRUE,    // manual reset event 
		FALSE,   // not-signalled 
		NULL)) == NULL){
		printf("Device::Device: Error creating read event, error=%d\n", GetLastError());
		fflush(stdout);
	}

	if((AsyncWrite.hEvent = CreateEvent(
		NULL,    // no security attributes 
		TRUE,    // manual reset event 
		FALSE,   // not-signalled 
		NULL)) == NULL){
		printf("Device::Device: Error creating write event, error=%d\n", GetLastError());
		fflush(stdout);
	}

	if(argc > 1)
		if(!strcmp(argv[1], "send")) hpcSend(NULL);
		else if(!strcmp(argv[1], "recv")) hpcRecv(NULL);
		else if(!strcmp(argv[1], "echo")) hpcEcho();
		else if(!strcmp(argv[1], "reset")) resetixp(Hpc_fh);
		else if(!strcmp(argv[1], "both")) hpcBoth();
		else printf("Invalid command\n");

	CloseHandle(Hpc_fh);
	CloseHandle(AsyncRead.hEvent);
	CloseHandle(AsyncWrite.hEvent);
	return(0);
}
