#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include "ostype.h"
#include "scomm.h"
#include "utils.h"

static const int MAX_MSGLEN = 256;
typedef struct msgDesc_S{
	int		msgLen;
	char	msg[MAX_MSGLEN];
}msgDesc_T;

static const int MSG_FIXEDLEN = sizeof(msgDesc_T) - MAX_MSGLEN;
static int Console = SCOMM_INVLDPORT, ConsolePort = 127, Stop=1;
static    thread_T Thread;
static sem_T ScommPrintSem;

int SComm_Print(char *format,...)
{
   msgDesc_T txtMsg;
   va_list ap;

   va_start(ap, format);
   txtMsg.msgLen = vsprintf(txtMsg.msg, format, ap);
   va_end(ap);
   if(txtMsg.msgLen < 0) return(txtMsg.msgLen);

#if (OS == UCOS)
   if(Stop) return(0);		// ignore if print server is not running
   return(SComm_Send(Console, ConsolePort, &txtMsg, MSG_FIXEDLEN + txtMsg.msgLen, SCOMM_PEND) - MSG_FIXEDLEN);
#else
   int len;
   len = printf("%.*s", txtMsg.msgLen, txtMsg.msg);
   fflush(stdout);
   return(len);
#endif
}


#if (OS == WIN_32)
void printSrv(void *arg)
{
   msgDesc_T txtMsg;
   int len, scomId = (int)arg;

   Stop = 0;
   while(!Stop){
      if((len = SComm_Recv(scomId , &txtMsg, MSG_FIXEDLEN, SCOMM_PEND)) == MSG_FIXEDLEN)
		  if((len = SComm_Recv(scomId , &txtMsg.msg, txtMsg.msgLen, SCOMM_PEND)) == txtMsg.msgLen){
             printf("%.*s", txtMsg.msgLen, txtMsg.msg);
             fflush(stdout);
		 }
   }
   Stop++;
   return;
}

void SComm_StopPrintSrv(void)
{
   int t = 10;

   if(SCOMM_INVLDPORT == Console) return;	// wasn't started

   Stop = 1;
   while(Stop == 1 && t--) delayMs(DELAYMS_RES);
   Thread_Cancel(Thread);
   SComm_Close(Console);
   Sem_Destroy(ScommPrintSem);
   Console = SCOMM_INVLDPORT;
   return;
}
#endif

int SComm_StartPrintSrv(char *deviceName, int portNum)
{
   Stop = 1;
   ConsolePort = portNum;

   if((Console = SComm_Open(deviceName, ConsolePort)) <0) return(1);
   ScommPrintSem = Sem_Init(1);

#if (OS == WIN_32)
   Thread = Thread_Create(LOW_PRIORITY, (threadFunc_T)printSrv, (void *)Console);
   while(Stop) delayMs(DELAYMS_RES);
#else
   Stop = 0;
#endif

   return(0);
}
