#include <stdlib.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include <iostream.h>
#include "ostype.h"
#include "scomm.h"
#include "utils.h"
#include "scommTest.h"

static unsigned int Iterations = 0, Quit=0;

void processMsg(int portNum, msgHdr_T *msgHdr)
{
	if(!msgHdr) return;
	switch(msgHdr->attrib){
		case MSG_TEXT:
			printf("portNum=%d, pkt #%d: msg=%.*s\n",portNum, msgHdr->recNum, msgHdr->msgLen,msgHdr->msg); fflush(stdout);
			break;
		 case MSG_BINARY:
			printf("portNum=%d, pkt #%d: msg=", portNum, msgHdr->recNum);
			for(int i = 0; i < msgHdr->msgLen; i++) printf("0x%x ", msgHdr->msg[i]);
			printf("\n");
			fflush(stdout);
			break;
	}

	return;
}


void procRecv(void *arg)
{
	tArg_T *tArg = (tArg_T *)arg;
	msgHdr_T	msgHdr;
	int len, timeout = 5, retry=2, nextRec=0;

	while(1){
		if((len = SComm_Recv(tArg->sCommId, &msgHdr, MSG_FIXEDLEN, SCOMM_PEND)) != MSG_FIXEDLEN){
			SComm_Print("recvTh: sCommId=%d, port=%d, Error reading from %s device, %d\n", tArg->sCommId, tArg->portNum, tArg->com, len); fflush(stdout);
			if(!retry--){
				Quit = 1;  // stop auto send
				return;
			}
			continue;
		}

		if(msgHdr.msgLen >= MAX_MSGLEN){
			SComm_Print("recvTh: commId=%d, port=%d, Invalid msg len %d\n", tArg->sCommId, tArg->portNum, msgHdr.msgLen);
			continue;
		}

		if((len = SComm_Recv(tArg->sCommId, &msgHdr.msg, msgHdr.msgLen, timeout)) != msgHdr.msgLen){
			SComm_Print("recvTh: sCommId=%d, port=%d, Error reading from %s device, %d\n", tArg->sCommId, tArg->portNum, tArg->com, len); fflush(stdout);
			continue;
		}

		// print packet if not in auto mode, out-of-order, or every 100 packets
		if(!Iterations || (msgHdr.recNum != nextRec) || !(msgHdr.recNum % 100)){
		   processMsg(tArg->portNum, &msgHdr);
		   nextRec = msgHdr.recNum;	// resync the rec number
		}
		nextRec++;
		retry=5;
	}
	return;
}



void procSend(void *arg)
{
	tArg_T *tArg = (tArg_T *)arg;
	msgHdr_T txtMsg;
	int len, timeout=5;

	txtMsg.attrib = MSG_TEXT;
	txtMsg.recNum = 0;

	while(1){
		printf("scomm_test> "); fflush(stdout);
		if(fgets(txtMsg.msg, MAX_MSGLEN, stdin) == NULL){
			printf("Bye!!\n");
			fflush(stdout);
			break;
		}
		txtMsg.msgLen = strlen(txtMsg.msg) - 1;
		if(!(txtMsg.msgLen)) continue;
		txtMsg.msg[txtMsg.msgLen] = '\0';	// to get rid of \n

		if((len = SComm_Send(tArg->sCommId, tArg->portNum, &txtMsg, MSG_FIXEDLEN + txtMsg.msgLen, timeout)) != MSG_FIXEDLEN + txtMsg.msgLen){
			SComm_Print("procSend: commId=%d, port=%d, Error writing to %s device, %d\n",
				tArg->sCommId, tArg->portNum, tArg->com, len);
			fflush(stdout);
			continue;
		}
		txtMsg.recNum++;
	}
	return;
}

void autoSend(void *arg)
{
	tArg_T *tArg = (tArg_T *)arg;
	msgHdr_T txtMsg;
	int len;
	unsigned int t = 0;

	txtMsg.attrib = MSG_TEXT;
	txtMsg.recNum = 0;
	sprintf(txtMsg.msg,"The quick brown fox jumps over the lazy dog.");
    txtMsg.msgLen = strlen(txtMsg.msg);

	while((t++ < Iterations) && !Quit){
		if((len = SComm_Send(tArg->sCommId, tArg->portNum, &txtMsg, MSG_FIXEDLEN + txtMsg.msgLen, SCOMM_PEND)) != MSG_FIXEDLEN + txtMsg.msgLen){
			printf("procSend: commId=%d, port=%d, Error writing to %s device, %d\n",
			        tArg->sCommId, tArg->portNum, tArg->com, len);
			SComm_Print("procSend: commId=%d, port=%d, Error writing to %s device, %d\n",
			        tArg->sCommId, tArg->portNum, tArg->com, len);
		}
 	    else txtMsg.recNum++;

//		if(!(txtMsg.recNum % 3))
//			delayMs(1);		// slow it down a bit -- only needed in SCOMM_SOCK mode
	}
	if(Quit){
		printf("The receive thread stopped\n");
		fflush(stdout);
	}
	delayMs(2000);		// wait for receive buffer to drain
}


void printUsage(void)
{
   printf("Usage: scommTest_clnt -tp testPortNum -pp PrintSrvPortNum -dev comDevName -auto numIterations\n");
   fflush(stdout);
   exit(1);
}


#if (OS == UCOS)
void main_scomm(void);
static int Argc;
static char **Argv;

int main(int argc, char** argv)
{
    Argc = argc;
    Argv = argv;

    /* Start uC/OS and run userInit in root task
     * with highest priority.  After userInit()
     * returns, root task commits suicide.
     */
    ucosStart(main_scomm, NO_CONSOLE);

    /* ucosStart() doesn't return. */
   return 0;
}


void main_scomm(void)
{
	int argc = Argc;
	char **argv = Argv;
	char *devName="com2";

#else
#if (OS == VXWORKS)
static int Argc = 0;
static char **Argv;

void scommTstClnt(void)
{
	int argc = Argc;
	char **argv = Argv;
	char *devName="ixphpc00";
#else

void main(int argc, char **argv)
{
	char *devName="com2";
#endif
#endif
	tArg_T tArg[4];
	int sCommId, portNum = 2, printPort = 1;
	int i, automatic = 0;
    clock_t start, finish;
    double  duration;

    for(i = 1; i < argc;){
		if(!strcmp(argv[i], "-tp")){
			if((++i) < argc) portNum = atoi(argv[i++]);
			else printUsage();
		}
		else if(!strcmp(argv[i], "-pp")){
			if((++i) < argc) printPort = atoi(argv[i++]);
			else printUsage();
		}
		else if(!strcmp(argv[i], "-dev")){
	       if((++i) < argc) devName = argv[i++];
		   else printUsage();
		}
		else if(!strcmp(argv[i], "-auto")){
	       if((++i) < argc) Iterations  = atoi(argv[i++]);
		   else printUsage();
		}
		else printUsage();
	}
    if(printPort == portNum){
		SComm_Print("Error; TestPortNum and PrintSrvPortNum must be different!\n");
		fflush(stdout);
		printUsage();
	}

	const int numMsg = 100, maxMsglen = 48;
	char *memBase = new char[numMsg * maxMsglen];
    initMemLog(memBase, numMsg, maxMsglen, 1);

	if(SComm_InstallDev(devName) != SCOMM_SUCCESS){  // typically called at OS startup
		SComm_Print("Error installing communication device %s\n",devName);
		return;
	}	

#ifdef WIN32
	SComm_StartPrintSrv(devName, printPort);	// init print server -- uses port 1
#endif
    if((sCommId = SComm_Open(devName, portNum)) <0){
		SComm_Print("commId=%d, port=%d, Error opening communication channel\n",sCommId, portNum);
		return;
	}
	tArg[0].sCommId = sCommId;
	tArg[0].portNum = portNum;
	tArg[0].com = devName;

    SComm_SendReset(sCommId, portNum);	// reset port target

    thread_T th1;
    th1 = Thread_Create(MID_PRIORITY, (threadFunc_T)procRecv, (void *)&tArg[0]);

    if(Iterations){
		start = clock();
		autoSend((void *)&tArg[0]);
		finish = clock();
		duration = (double)(finish - start) / CLOCKS_PER_SEC;
		printf( "\nElapsed time: %2.1f seconds\n", duration );
	}
    else procSend((void *)&tArg[0]);

#if (OS == WIN_32)
	void scommStats(void);
    scommStats();
    SComm_StopPrintSrv();
    initMemLog(memBase, numMsg, maxMsglen, 1);
#endif
    Thread_Cancel(th1);
    SComm_Close(sCommId);
	return;
}
