#include <stdlib.h>
#include <string.h>
#include <iostream.h>
#include "ostype.h"
#include "scomm.h"
#include "utils.h"
#include "scommTest.h"

//dj: debug
extern "C" int memLog(char *format,...);
extern "C" void initMemLog(char *memBase, int numMsg, int maxMsglen, int circular);

void procEcho(void *arg)
{
	tArg_T *tArg = (tArg_T *)arg;
	msgHdr_T	msgHdr;
	int len, nextRecNum=0, i=0;

	while(1){
		if((len = SComm_Recv(tArg->sCommId, &msgHdr, MSG_FIXEDLEN, SCOMM_PEND)) != MSG_FIXEDLEN){
			SComm_Print("recvTh: sCommId=%d, port=%d, Error reading from %s device, %d\n", tArg->sCommId, tArg->portNum, tArg->com, len);
			continue;
		}
		if(msgHdr.msgLen >= MAX_MSGLEN){
			SComm_Print("recvTh: commId=%d, port=%d, Invalid msg len %d\n", tArg->sCommId, tArg->portNum, msgHdr.msgLen);
			continue;
		}
		if((len = SComm_Recv(tArg->sCommId, &msgHdr.msg, msgHdr.msgLen, 1000)) != msgHdr.msgLen){
			SComm_Print("recvTh: sCommId=%d, port=%d, Error reading from %s device, %d\n", tArg->sCommId, tArg->portNum, tArg->com, len);
		    continue;
		}
		
		if(msgHdr.recNum != nextRecNum){
			SComm_Print("recvTh: Packet out of order, expect=%d, received=%d\n", nextRecNum, msgHdr.recNum);
			nextRecNum = msgHdr.recNum;
		}
		nextRecNum++;

		if((len = SComm_Send(tArg->sCommId, tArg->portNum, &msgHdr, MSG_FIXEDLEN + msgHdr.msgLen, SCOMM_PEND)) != MSG_FIXEDLEN + msgHdr.msgLen){
		    SComm_Print("recvTh: commId=%d, port=%d, Error writing to %s device, %d\n", tArg->sCommId, tArg->portNum, tArg->com, len);
		}
	}
	return;
}

void printUsage(void)
{
   printf("Usage: scommTest_srv -tp testPortNum -pp PrintSrvPortNum -dev comDevName\n");
   fflush(stdout);
   exit(1);
}



#if (OS == UCOS)
void main_scomm(void);
static int Argc;
static char **Argv;

int main(int argc, char** argv)
{
    Argc = argc;
    Argv = argv;

    /* Start uC/OS and run userInit in root task
     * with highest priority.  After userInit()
     * returns, root task commits suicide.
     */
    ucosStart(main_scomm, NO_CONSOLE);

    /* ucosStart() doesn't return. */
   return 0;
}


void main_scomm(void)
{
	int argc = Argc;
	char **argv = Argv;
	char *devName="com2";

#else
#if (OS == VXWORKS)
static int Argc = 0;
static char **Argv;

void scommTstSrv(char *dev)
{
	int argc = Argc;
	char **argv = Argv;
	char *devName="ixphpc00";
	if(dev) devName = dev;

#else
void main(int argc, char **argv)
{
	char *devName="com1";
#endif
#endif
	tArg_T tArg[4];
	int devNum = 1, sCommId, portNum = 2, printPort = 1, i;

    for(i = 1; i < argc;){
		if(!strcmp(argv[i], "-tp")){
			if((++i) < argc) portNum = atoi(argv[i++]);
			else printUsage();
		}
		else if(!strcmp(argv[i], "-pp")){
			if((++i) < argc) printPort = atoi(argv[i++]);
			else printUsage();
		}
		else if(!strcmp(argv[i], "-dev")){
	       if((++i) < argc) devName = argv[i++];
		   else printUsage();
		}
		else printUsage();
	}
//dj: debug
    initMemLog((char *)0x10000000, 100, 48, 1);
    memLog("Main: memLog is initialized...");

	if(SComm_InstallDev(devName) != SCOMM_SUCCESS){  // typically called at OS startup
		SComm_Print("Error installing communication device %s\n",devName);
		return;
	}	

	SComm_StartPrintSrv(devName, printPort);	// init print server -- uses port 1
    if((sCommId = SComm_Open(devName, portNum)) <0){
		SComm_Print("commId=%d, port=%d, Error opening communication channel\n",sCommId, portNum);
		return;
	}
	tArg[0].sCommId = sCommId;
	tArg[0].portNum = portNum;
	tArg[0].com = devName;

    procEcho((void *)&tArg[0]);			// never return
    SComm_Close(sCommId);

	return;
}


