/* scomm_llc.cpp
 * Serial Communication Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Serial Communication Library
 * author: drj Mar 16, 99 Initial version
 * revisions:
 * $History: scomm_llc.cpp $
 * 
 * *****************  Version 42  *****************
 * User: Ewalker      Date: 9/28/00    Time: 5:31p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 41  *****************
 * User: Ewalker      Date: 9/28/00    Time: 3:21p
 * Updated in $/dev_1200/Share/scomm
 * Fixed exclusive task looping in recvProcess
 * 
 * *****************  Version 39  *****************
 * User: Ewalker      Date: 9/20/00    Time: 11:49a
 * Updated in $/dev_1200/Share/scomm
 * Receive thread needed lower delay for performance
 * 
 * *****************  Version 38  *****************
 * User: Ewalker      Date: 9/20/00    Time: 9:58a
 * Updated in $/dev_1200/Share/scomm
 * Performance cleanup
 * 
 * *****************  Version 37  *****************
 * User: Drjohns1     Date: 9/19/00    Time: 12:18p
 * Updated in $/dev_1200/Share/scomm
 * Changed timeout in recvProcess
 * 
 * *****************  Version 36  *****************
 * User: Drjohns1     Date: 9/08/00    Time: 5:10p
 * Updated in $/dev_1200/Share/scomm
 * Re-implemented timex
 * 
 * *****************  Version 35  *****************
 * User: Drjohns1     Date: 9/07/00    Time: 7:16p
 * Updated in $/dev_1200/Share/scomm
 * Fixed RTC init
 * 
 * *****************  Version 34  *****************
 * User: Drjohns1     Date: 9/01/00    Time: 3:35p
 * Updated in $/dev_1200/Share/scomm
 * Changed sysCpuVerGet to sysCpuIDGet.
 * 
 * *****************  Version 33  *****************
 * User: Drjohns1     Date: 8/30/00    Time: 6:29p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 32  *****************
 * User: Drjohns1     Date: 8/30/00    Time: 2:06p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 30  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 29  *****************
 * User: Drjohns1     Date: 4/19/00    Time: 12:54p
 * Updated in $/dev_1200/Share/scomm
 * Fixed for B0 chip -- UART ISR
 * 
 * *****************  Version 28  *****************
 * User: Drjohns1     Date: 4/17/00    Time: 12:24p
 * Updated in $/dev_1200/Share/scomm
 * Included ostype.h
 * 
 * *****************  Version 27  *****************
 * User: Drjohns1     Date: 4/04/00    Time: 6:43p
 * Updated in $/dev_1200/Share/scomm
 * OS type check and cleanup
 * 
 * ---------------------------------------------------------------------
 */
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "ostype.h"
#include "utils.h"
#include "core_io.h"
#include "scomm.h"
#include "scomm_llc.h"

static const char __copyright[] =
"Copyright Intel Corp, 1999. All rights reserved";

enum {
    STATE_BEG_DLE = 0,
	STATE_STX ,
	STATE_DATA,
	STATE_ETX
};

static unsigned int CharSnd=0, CharRcv=0, CharBuf=0, UnknState=0;
static unsigned int GoodFrames=0, BadFrame=0, BadDle=0, BadStx=0, BadEtx=0;

const int	STX = 0x2, DLE = 0x1, ETX = 0x3;
const unsigned int MV_PC_LR = 0xe1a0f00e, FrameBufSize=64, FrameMrkSize=2;
char FrameBuf[FrameBufSize];
char BegFrame[FrameMrkSize] = {DLE, STX}, EndFrame[FrameMrkSize] = {DLE, ETX};

#if (OS == UCOS)
// return from SWI -- avoid Angel interuptions
#define DISABLE_ANGEL()\
	*((volatile unsigned int *)0x08) = MV_PC_LR; \
	*((volatile unsigned int *)FIQ_EN_CLR) = (CONSOLE_INT_RX | CONSOLE_INT_TX)

#ifdef UCOS_DEBUG
extern "C" int _printf(const char * /*format*/, ...);
#endif


//dj: debug
extern "C" int memLog(char *format,...);
#endif


/*-----------------------------------------------------------------------------
   Function: 	llcStats
   Description: Print statistical information -- for debugging
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void llcStats(void)
{
	SComm_Print("\nllcStats: GoodFrames=%d, BadFrames=%d, CharSnd=%d, CharRcv=%d,\n\
	CharsBuffered=%d, BadDle=%d, BadStx=%d, BadEtx=%d, UnknState=%d\n",
		GoodFrames, BadFrame, CharSnd, CharBuf, CharRcv, BadDle, BadStx, BadEtx, UnknState);
	
	return;
}



/*-----------------------------------------------------------------------------
   Function: 	comRecvISR_revA
   Description: Serial device receive data interrupt handler for revA chip.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void comRecvISR_revA(int param)
{
	ComDev *comDevObj = (ComDev *)param;
	char  c[RCVBUF_SIZE];
	int rLen;

	if((rLen = comDevObj->getChars(c, RCVBUF_SIZE)))
		comDevObj->devBuf.put(c, rLen);
	comDevObj->clearInterrupt();
	CharRcv += rLen;

	return;
}

/*-----------------------------------------------------------------------------
   Function: 	comRecvISR
   Description: Serial device receive data interrupt handler for revB and newer
				chips.
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void comRecvISR(int param)
{
	ComDev *comDevObj = (ComDev *)param;
	char  c[RCVBUF_SIZE];
	int rLen;

	if((rLen = comDevObj->getChars(c, RCVBUF_SIZE)))
		comDevObj->devBuf.put(c, rLen);
	CharRcv += rLen;
	return;
}

void comRecvISRx(int param)
{
	ComDev *comDevObj = (ComDev *)param;
	char  c[RCVBUF_SIZE];
	int rLen;

	if((rLen = comDevObj->getChars(c, RCVBUF_SIZE)))
		CharSnd += comDevObj->putChars(c, rLen);
//	comDevObj->clearInterrupt();

	CharRcv += rLen;
	return;
}

#ifdef SCOMM_SOCK
/*-----------------------------------------------------------------------------
   Function: 	processConn
   Description: Waits for socket connections
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
extern "C" threadFunc_T processConnect(void *obj)
{
	Device *devObj= (Device *)obj;

	while(1){
      /* accept the connection */
      devObj->acceptConn();
	}
	return(0);
}
#endif

/*-----------------------------------------------------------------------------
   Function: 	recvProcess
   Description: Read char from device -- polls
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
threadFunc_T recvProcess(void *obj)
{
	ComDev	*comDevObj = (ComDev *)obj;
	char buf[RCVBUF_SIZE];
	unsigned int len, sLen;
//	unsigned int timeout;

	while(1){
		if((len = comDevObj->getChars(buf, RCVBUF_SIZE)) > 0){
			CharRcv += len;
			sLen = 0;
//			timeout = 5000 / DELAYMS_RES;  /* convert milliseconds to ticks */
			while((len - sLen)/* && timeout--*/){
				sLen += comDevObj->devBuf.put(&buf[sLen], len-sLen);
				if (sLen != len) delayTick(0);			// give other processes time to run
			}
			CharBuf += sLen;
		}
	    else delayTick(0);			// give other processes time to run
	}
	return(0);
}


/*-----------------------------------------------------------------------------
   Function: 	processFrame
   Description: Receive frame and place on destination
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
threadFunc_T processFrame(void *argPtr)
{
   ComDev *comDev = (ComDev *)argPtr;
   int dataLen, i;
   char frameBuf[MAX_FRAME_SIZE], ch, buf[FrameBufSize];
   unsigned int frameBufIndx = 0, state = STATE_BEG_DLE;

   while(1){
	 dataLen = comDev->devBuf.get(buf, FrameBufSize);

	 for(i = 0; i < dataLen; i++){
		// make sure we don't exceed the frame size
		if(frameBufIndx >= MAX_FRAME_SIZE){
		   // drop it and start over
		   state = STATE_BEG_DLE;
		   frameBufIndx = 0;
		   BadFrame++;
		}

        ch = buf[i];
		switch(state){
		   
		   case STATE_BEG_DLE:
			   if(ch == DLE) state = STATE_STX;
			   else BadDle++;
			   frameBufIndx = 0;
			   break;
		   
		   case STATE_STX:
			   if(ch == STX) state = STATE_DATA;
			   else{
				   state = STATE_BEG_DLE;
				   BadStx++;
			   }
			   break;

		   case STATE_DATA:
			   if(ch == DLE) state = STATE_ETX;
			   else frameBuf[frameBufIndx++] = ch;
			   break;

		   case STATE_ETX:
			   if(ch != ETX){
				   if(ch != DLE){
					  // framing error
					  state = STATE_BEG_DLE;
					  BadEtx++;
					  BadFrame++;
				   }
				   else{
					  // consecutive dle -- data
					  frameBuf[frameBufIndx++] = ch;
					  state = STATE_DATA;
				   }
			   }else{
				   // good frame -- send packet
				   GoodFrames++;
				  (comDev->recvPacket)(comDev->devId, frameBuf, frameBufIndx);
			      state = STATE_BEG_DLE;
			   }
			   break;

		   default:
			  state = STATE_BEG_DLE;
			  UnknState++;
			  break;
		}
	 }
   }
   return(0);
}


/*-----------------------------------------------------------------------------
   Function: 	ComDev::ComDev
   Description: ComDev Class constructor
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
ComDev::ComDev(char *deviceName, int deviceId, void (*recvPktFunc)(int devNum, char *data, int len)) : devBuf(RCVBUF_SIZE)
{
	writeSem = Sem_Init(1);
	device = NULL;

	strcpy(devNam, deviceName);
	utl_strLowcase(devNam);

#if (OS == WIN_32)
#ifdef SCOMM_HPC
	if(!strncmp("ixphpc", devNam, 6)){
		device = new WinHpcDev(devNam);
	}
#endif

#ifdef SCOMM_SOCK
	if(!strncmp("sock", devNam, 4)){
		device = new WinSockDev(devNam);
	}
#endif

#ifdef SCOMM_COM
	if(!strncmp("com", devNam, 3)){
		device = new Win32ComDev(devNam);
	}
#endif
#endif		// OS == WIN_32


#if (OS == VXWORKS)
#ifdef SCOMM_HPC
	if(!strncmp("ixphpc", devNam, 6)){
		device = new HpcDev(devNam);
	}
#endif

#ifdef SCOMM_SOCK
	if(!strncmp("sock", devNam, 4)){
		device = new VxSockDev(devNam);
	}
#endif
#endif		// OS == VXWORKS

#if (OS == UCOS)
	// only  serial port supported
//	if(!strncmp("com", devNam, 3)){
	device = new ixp1200ComDev(devNam);
//	}
#endif		// OS == UCOS

	if((!device) || (!device->created())) return;
	recvPacket = recvPktFunc;
	devId = deviceId;

	device->getDevCtlBlk(&devCtlBlk);
	devCtlBlk.byteSize = BYTE_SIZE8;
	devCtlBlk.parity = NO_PARITY;
    devCtlBlk.stopBits = ONE_STOPBIT;
	devCtlBlk.baudRate = DEFAULT_BAUD;
	devCtlBlk.enableDev = TRUE;
	device->setDevCtlBlk(&devCtlBlk);


#if (OS == UCOS)

#ifdef EBSA285
#define INT_UART 2
#else
#define INT_UART SAINT_UART
#endif

	device->disableIntTx();
	device->disableIntRx();

	// uCos -- ebs285 or IXP1200 -- install ISR to read device
    if((sysCpuIDGet() & 0xf) == 0)		// rev A chip
 	    int_connect(INT_UART, (ISR_FUNC)comRecvISR_revA, (int)this);
	else
 	    int_connect(INT_UART, (ISR_FUNC)comRecvISR, (int)this);

  SAr_OsUseUART();						// disable Angel

#ifdef UCOS_DEBUG
	// These are temp fixes to disable printing to the console while
	// the SComm library is initialized.  These fixes are not appropriate
	// if the program is being run out of ROM
	*(unsigned int *)printf = MV_PC_LR;		// disable printf
	*(unsigned int *)_printf = MV_PC_LR;	// disable printf
	*(unsigned int*)fputs = MV_PC_LR;		// disable cout
	*(unsigned int*)fflush = MV_PC_LR;		// disable cout
#endif
#endif


#if ((OS == WIN_32) || (OS == VXWORKS))
	// WIN95/NT or VxWorks: create thread to read device rxdata
	recvThread = Thread_Create(MID_PRIORITY, (threadFunc_T)recvProcess, (void *)this);
#endif

	Thread_Create(MID_PRIORITY, (threadFunc_T)processFrame, (void *)this);
	device->enableIntRx();					// enable receive interupt
	return;
}


/*-----------------------------------------------------------------------------
   Function: 	ComDev::~ComDev
   Description: ComDev Class destructor
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
ComDev::~ComDev()
{
	Thread_Cancel(recvThread);
	Sem_Destroy(writeSem);
	delete(device);
	return;
}


/*-----------------------------------------------------------------------------
   Function: 	ComDev::putv
   Description: scatter write to device
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int ComDev::putv(iovec_t *ioVec, int numIoVec)
{
	int i, len, nWrote = 0;
	for(i = 0; i < numIoVec; i++){
		len = device->put(ioVec[i].base, ioVec[i].len);
		nWrote += len;
		if(len != ioVec[i].len) break;
	}
	CharSnd += nWrote;
	return(nWrote);
}

/*-----------------------------------------------------------------------------
   Function: 	ComDev::getv
   Description: gather read from device
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int ComDev::getv(iovec_t *ioVec, int numIoVec)
{
	int i, len, nRead = 0;
	for(i = 0; i < numIoVec; i++){
		len = device->get(ioVec[i].base, ioVec[i].len);
		nRead += len;
		if(len != ioVec[i].len) break;
	}
	CharRcv += nRead;
	return(nRead);
}

/*-----------------------------------------------------------------------------
   Function: 	ComDev::findv
   Description: finds the iovec index and char position of the nth char
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int ComDev::findv(iovec_t *ioVec, int numIoVec, int nthChar, int *vecIndx, int *charIndx)
{
	int i, len = 0;
	// find the vec index
	for(i = 0; i < numIoVec; i++){
		len += ioVec[i].len;
		if(nthChar > len) continue;

		// return the index and char position
		*vecIndx = i;
		*charIndx = (ioVec[i].len - (len - nthChar)) - 1;
		return(0);
	}
	return(-1);
}



/*-----------------------------------------------------------------------------
   Function: 	ComDev::sendPacket
   Description: Package data and sen to device
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
int ComDev::sendPacket(iovec_t *ioVec, int numIoVec)
{
	int i, ii, obufLen=0, pktLen = 0, len;
	char *buf;

	Sem_Wait(writeSem, SEM_WAITFOREVER);
	CharSnd += device->put(BegFrame, FrameMrkSize);		// beg of frame marker
	for(ii = 0; (ii < numIoVec) && (pktLen < MAX_FRAME_SIZE); ii++){
		buf = ioVec[ii].base;
		len = ioVec[ii].len;

		// character stuffing
		for(i = 0; i < len; i++){

#ifdef SCOMM_NOOUTBUF
			CharSnd += device->put(&buf[i], 1);
			if(buf[i] == DLE) device->put(&buf[i], 1);;
#else
			// buffer up to the first DLE char, or up to the max buffer size
			if(buf[i] == DLE){
				if(obufLen+2 >= FrameBufSize){
					CharSnd += device->put(FrameBuf, obufLen);
					obufLen = 0;
				}
				FrameBuf[obufLen++] = DLE;
			}
			FrameBuf[obufLen++] = buf[i];
			if(obufLen >= FrameBufSize){
				CharSnd += device->put(FrameBuf, FrameBufSize);
				obufLen = 0;
			}
#endif
		}
		pktLen += len;
	}
	if(obufLen) CharSnd += device->put(FrameBuf, obufLen);
	CharSnd += device->put(EndFrame, FrameMrkSize);		// end of frame marker
	Sem_Post(writeSem);

	return(pktLen);
}

