/* scomm_llc.h
 * Serial port communication
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 - 1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *  system: SA1200
 *  subsystem: Serial Communication Library
 *  author: drj March 1, 99 Initial version
 *  revisions:
 *  $History: scomm_llc.h $
 * 
 * *****************  Version 22  *****************
 * User: Drjohns1     Date: 8/30/00    Time: 6:29p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 21  *****************
 * User: Drjohns1     Date: 8/30/00    Time: 2:06p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 20  *****************
 * User: Qabuild      Date: 8/24/00    Time: 5:35p
 * Updated in $/dev_1200/Share/scomm
 * Changed Scomm_InstallDev return type
 * 
 * *****************  Version 19  *****************
 * User: Drjohns1     Date: 8/17/00    Time: 4:38p
 * Updated in $/dev_1200/Share/scomm
 * Lowercased devices name
 * 
 * *****************  Version 18  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 17  *****************
 * User: Drjohns1     Date: 4/19/00    Time: 12:54p
 * Updated in $/dev_1200/Share/scomm
 * Fixed for B0 chip -- UART ISR
 * 
 * *****************  Version 16  *****************
 * User: Drjohns1     Date: 4/04/00    Time: 6:43p
 * Updated in $/dev_1200/Share/scomm
 * OS type check and cleanup
 * 
 * ---------------------------------------------------------------------
 */
#ifndef _SCOMM_LLC_H
#define _SCOMM_LLC_H

#include "ostype.h"
#include "thread.h"
#include "sem.h"
#include "utl_str.h"
#include "scomm_buff.h"


#if (OS == WIN_32)
#if (!defined(SCOMM_SOCK) && !defined(SCOMM_HPC) && !defined(SCOMM_COM))
#error atleast one of the following must defined: SCOMM_SOCK, SCOMM_HPC, SCOMM_COM
#endif

#ifdef SCOMM_SOCK
#include "win32SockDev.h"
#endif

#ifdef SCOMM_HPC
#include "win32HpcDev.h"
#endif

#ifdef SCOMM_COM
#include "win32ComDev.h"
#endif
#endif		// OS == WIN_32

#if (OS == UCOS)
#ifdef EBSA285
#include "ebsa285ComDev.h"
#else
#include "ixp1200ComDev.h"
#endif
#endif		// OS == UCOS

#if (OS == VXWORKS)
#ifdef SCOMM_HPC
#include "vxWrkHpcDev.h"
#endif
#ifdef SCOMM_SOCK
#include "vxSockDev.h"
#endif
#endif		// OS == VXWORKS 


#include "scommdev.h"

#define RCVBUF_SIZE		2048
const unsigned int MAX_FRAME_SIZE = 1280;

//#define DEFAULT_BAUD	BR_9600
//#define DEFAULT_BAUD	BR_19200
//#define DEFAULT_BAUD	BR_38400
#define DEFAULT_BAUD	BR_115200

typedef struct{
	char *base;
	int len;
}iovec_t;


class ComDev{
public:
	ComDev(char *devName, int deviceId, void (*recvPktFunc)(int devNum, char *data, int len));
	~ComDev();
	//void recvFrame(char *inChar, unsigned int len);
	inline void enableDevice(void){device->enableDev();}
	inline void disableDevice(void){device->disableDev();}
	int isDevReady(void);
	int isRxReady(void);
	int isTxReady(void);
	inline int getChars(char *c, unsigned int len){return(device->get(c, len));}
	inline int putChars(char *c, unsigned int len){return(device->put(c, len));}
	inline void clearInterrupt(){device->clearInterrupt();}
	int findv(iovec_t *ioVec, int numIoVec, int charPos, int *vecIndx, int *charIndx);
	int getv(iovec_t *ioVec, int numIoVec);
	int putv(iovec_t *ioVec, int numIoVec);
	int sendPacket(iovec_t *ioVec, int numIoVec);


private:
	char		devNam[MAX_DEVICE_NAME];
	Device		*device;
	DevCtlBlk_T devCtlBlk;
	FastBuf		devBuf;
	int			devId;
	unsigned int devStatus;		// rxReady, txReady, devEn
	sem_T		writeSem;
	void		(*recvPacket)(int devNum, char *data, int len);
	thread_T	recvThread;
	friend int SComm_InstallDev(char *DevName);
	friend int SComm_Open(char *devName, int portNum);
	friend void comRecvISR_revA(int param);
	friend void comRecvISR(int param);
	friend void comRecvISRx(int param);
	friend threadFunc_T processFrame(void *argPtr);
	friend threadFunc_T recvProcess(void *obj);
};

#endif			// _SCOMM_LLC_H
