#include "ostype.h"
#include "osApi.h"
#include "core_io.h"
#include "VxWrkHpcDev.h"

static int ChanNum = 0, Quit = 0, RdError = 0, WrError = 0;
static unsigned int BytesRecv = 0, BytesSent = 0;
HPC_IRP AsyncRead, AsyncWrite;
char *Msg = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
const unsigned int BufSize = 256, ReadTimeout = 1000, WriteTimeout = 1000;

#define SYNC_READ
//#define SYNC_WRITE

int recvBytes(char *buf, unsigned int bytesToRead)
{
	unsigned int bytesRead = 0;
	HPC_ERROR_CODE stat;

	
#ifdef SYNC_READ
	if((stat = hpcReadwrite(ChanNum, HPC_READ, buf, bytesToRead, &bytesRead, NULL)) != HPC_OK) return(0);
#else
	if((status = hpcReadwrite(ChanNum, HPC_READ, buf, bytesToRead, &bytesRead, &AsyncRead)) == HPC_PENDING){
		if(Sem_Wait(AsyncRead.done, ReadTimeout) == SEM_TIMEDOUT){
			printf("****Reading from hpc timed out\n"); fflush(stdout);
		}

		if((stat = hpcGetIrpStatus(&AsyncRead, &bytesRead, FALSE)) == HPC_PENDING){
			hpcCancelio(chanNum, HPC_READ);

			// get the actual number of bytes written
			hpcGetIrpStatus(&AsyncRead, &bytesRead, FALSE);
		}
	}
#endif
//printf("read completed, stat=0x%x, bytesRead=%d\n", stat, bytesRead);  fflush(stdout);
	return(bytesRead);
}



int sendBytes(char *buf, unsigned int bytesToWrite)
{
	unsigned int bytesSent = 0;
	HPC_ERROR_CODE stat, wstat;

#ifdef SYNC_WRITE
	if((stat = hpcReadwrite(ChanNum, HPC_WRITE, buf, bytesToWrite, &bytesSent, NULL)) != HPC_OK) return(0);
#else
//printf("writing to hpc, %d bytes\n", bytesToWrite); fflush(stdout);
	if((wstat = hpcReadwrite(ChanNum, HPC_WRITE, buf, bytesToWrite, &bytesSent, &AsyncWrite)) == HPC_PENDING){
		if(Sem_Wait(AsyncWrite.done, WriteTimeout) == SEM_TIMEDOUT){
			printf("****Write to hpc timed out\n"); fflush(stdout);
		}

		if((stat = hpcGetIrpStatus(&AsyncWrite, &bytesSent, FALSE)) == HPC_PENDING){
			hpcCancelio(ChanNum, HPC_WRITE);

			// get the actual number of bytes written
			hpcGetIrpStatus(&AsyncWrite, &bytesSent, FALSE);
		}
	}
#endif
//printf("write completed, wstat=0x%x, stat=0x%x, byteSent=%d\n", wstat, stat, bytesSent);  fflush(stdout);
	return(bytesSent);
}


/*
int recvBytes(char *buf, unsigned int bytesToRead)
{
	int bytesRead = 0, status, timeout = 1000;
	
#ifdef SYNC_READ
	status = hpcReadwrite(ChanNum, HPC_READ, buf, bytesToRead, &bytesRead, NULL);
#else
	if((status = hpcReadwrite(ChanNum, HPC_READ, buf, bytesToRead, &bytesRead, &AsyncRead)) == HPC_PENDING){
		while(timeout && (hpcGetIrpStatus(&AsyncRead, &bytesRead, FALSE) == HPC_PENDING)){
			printf("Read timeout, status = %d\n", status);
			timeout--;
			delayMs(10);
		}
	}
	else printf("**read from hpc failed; status = 0x%x\n", status);
#endif
	return(bytesRead);
}


int sendBytes(char *buf, unsigned int bytesToWrite)
{
	int bytesWritten = 0, status, timeout = 1000;

#ifdef SYNC_WRITE
	status = hpcReadwrite(ChanNum, HPC_WRITE, buf, bytesToWrite, &bytesWritten, NULL);
#else
	if((status = hpcReadwrite(ChanNum, HPC_WRITE, buf, bytesToWrite, &bytesWritten, &AsyncWrite)) == HPC_PENDING){
		while(timeout && (hpcGetIrpStatus(&AsyncWrite, &bytesWritten, FALSE) == HPC_PENDING)){
			timeout--;
			delayMs(1);
		}
	}
	else printf("**write to hpc failed; status = 0x%x\n", status);
#endif
	return(bytesWritten);
}
*/


void hpcRecv(void *)
{
	int bytesRead = 0, fReadStat;
	char buf[BufSize];
	unsigned int bytesToRead, msgNum = 0;
	
	bytesToRead = strlen(Msg) + 1;

	while(!Quit){
		bytesRead = recvBytes(buf, bytesToRead);
		if(bytesRead > 0){
			printf("MsgNum%d = '%s'\n", msgNum++, buf);
			BytesRecv += bytesRead;
		}
		delayMs(5);
	}
	return;
}


void hpcSend(void *)
{
	unsigned int bytesSent=0, bytesToWrite;

	bytesToWrite = strlen(Msg) + 1;

	while(!Quit){
		bytesSent = sendBytes(Msg, bytesToWrite);
		if(bytesSent > 0){
			BytesSent += bytesSent;
			if(bytesSent < bytesToWrite){
				printf("***Warning partial write to HPC device, %d, %d\n", bytesToWrite, bytesSent); fflush(stdout);
			}
		}
		delayMs(1000);
	}
	return;
}

void hpcBoth(void)
{
	Thread_Create(MID_PRIORITY, (threadFunc_T)hpcSend, NULL);
	hpcRecv(NULL);
	return;
}

void hpcEcho(void)
{
	char buf[BufSize];
	int bytesRead, bytesSent, status;

	while(!Quit){
		if((bytesRead = recvBytes(buf, BufSize)) > 0){
			BytesRecv += bytesRead;
			if((bytesSent = sendBytes(buf, bytesRead)) > 0){
				BytesSent += bytesSent;
			}
			else WrError++;
		}
		else{
			RdError++;
			delayMs(1000);
		}
	}
	return;
}



int hpcQuit(void)
{
	Quit = 1;
	delayMs(1000);		// give threads time to terminate
	if(hpcClose(ChanNum) != HPC_OK) return(-1);
	Quit = 0;
	return(0);
}


int tstHpc(char *tst)
{
	static init = 0;

	if(!init){
		if(hpcOpenW32(ChanNum) != HPC_OK) return(-1);
		hpcInitIrp(&AsyncRead);
		hpcInitIrp(&AsyncWrite);
		init = 1;
	}

	// set mode -- no indication on client partial read
	HPC_ERROR_CODE hpcSetmode(ChanNum, HPC_IB_NONE);

	if(!strcmp(tst, "send")) hpcSend(NULL);
	else if(!strcmp(tst, "recv")) hpcRecv(NULL);
	else if(!strcmp(tst, "quit")) return(hpcQuit());
	else if(!strcmp(tst, "both")) hpcBoth();
	else if(!strcmp(tst, "echo")) hpcEcho();
	else printf("Unknown command\n");

	return(0);
}

