/* vxWrkHpcDev.cpp
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Vxworks HPC Communication Library
 * author: drj Mar 16, 99 Initial version
 * revisions:
 * $History: vxWrkHpcDev.cpp $
 * 
 * *****************  Version 8  *****************
 * User: Ewalker      Date: 12/13/00   Time: 2:41p
 * Updated in $/V1.2/Share/scomm
 * Fixed a timeout problem with hardware debug over PCI.
 * 
 * *****************  Version 7  *****************
 * User: Ewalker      Date: 9/29/00    Time: 12:25p
 * Updated in $/dev_1200/Share/scomm
 * Made read asynchronous like write
 * 
 * *****************  Version 5  *****************
 * User: Ewalker      Date: 9/20/00    Time: 9:58a
 * Updated in $/dev_1200/Share/scomm
 * Performance cleanup
 * 
 * *****************  Version 4  *****************
 * User: Drjohns1     Date: 8/22/00    Time: 4:57p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 3  *****************
 * User: Qabuild      Date: 8/10/00    Time: 1:38p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 6/28/00    Time: 1:09p
 * Created in $/dev_1200/Share/scomm
 * ---------------------------------------------------------------------
 */

#include "utils.h"
#include "VxWrkHpcDev.h"

//#define SYNC_READ
//#define SYNC_WRITE



HpcDev::HpcDev(char *devName)
{
	devEn = 0;
	chanNum = getDevChan(devName);
	if(hpcOpenW32(chanNum) != HPC_OK) return;
	hpcInitIrp(&asyncRead);
	hpcInitIrp(&asyncWrite);

	// set mode -- no indication on client partial read
	HPC_ERROR_CODE hpcSetmode(chanNum, HPC_IB_NONE);

	// timeouts are in ticks
	readTimeout = 1000 / DELAYMS_RES;
	writeTimeout = 1000 / DELAYMS_RES;
	status = 0;

	return;
}

HpcDev::~HpcDev(){
	hpcClose(chanNum);
	hpcDeleteIrp(&asyncRead);
	hpcDeleteIrp(&asyncWrite);
}

int HpcDev::getDevChan(char *devName)
{
	// device name: hpcxx
	char *cptr;
	int chanNum = -1;

	if(devName){
		cptr = strstr(devName, "hpc");
		if(cptr && (strlen(cptr) > 4)) chanNum = atoi(cptr[4]);
	}

	return(chanNum);
}


int HpcDev::get(char *buf, unsigned int len)
{
	unsigned int bytesRead = 0;
	HPC_ERROR_CODE stat;
    
//    printf("Reading %d bytes from hpc device\n", len); fflush(stdout);

    if(len > 0){
#ifdef SYNC_READ
        if((stat = hpcReadwrite(chanNum, HPC_READ, buf, len, &bytesRead, NULL)) != HPC_OK){
            printf("***Error reading from hpc device\n"); fflush(stdout);
        }
#else
        if((stat = hpcReadwrite(chanNum, HPC_READ, buf, len, &bytesRead, &asyncRead)) == HPC_PENDING){
            if(Sem_Wait(asyncRead.done, readTimeout) == SEM_TIMEDOUT){
                printf("***Timeout reading from hpc device\n"); fflush(stdout);
            }
            
            if((stat = hpcGetIrpStatus(&asyncRead, &bytesRead, FALSE)) == HPC_PENDING){
                hpcCancelio(chanNum, HPC_READ);
                
                // get the actual number of bytes written
                hpcGetIrpStatus(&asyncRead, &bytesRead, FALSE);               
            }

            else if(stat != HPC_OK){
                printf("***Error reading from hpc device\n"); fflush(stdout);
            }
        }
        
        else if(stat != HPC_OK){
            printf("***Error reading from hpc device\n"); fflush(stdout);
        }
#endif
    }

//    printf("Read %d bytes from hpc device, stat=0x%x\n", bytesRead, stat); fflush(stdout);

    return(bytesRead);
}



int HpcDev::put(char *buf, unsigned int len){
	unsigned int bytesSent = 0;
	HPC_ERROR_CODE stat, wstat;

//    printf("Writing %d bytes to hpc device\n", len); fflush(stdout);

    if(len > 0){
#ifdef SYNC_WRITE
        if((wstat = hpcReadwrite(chanNum, HPC_WRITE, buf, len, &bytesSent, NULL)) != HPC_OK){
            printf("***Error writing to hpc device\n"); fflush(stdout);
            stat = 0;
        }
#else
        if((wstat = hpcReadwrite(chanNum, HPC_WRITE, buf, len, &bytesSent, &asyncWrite)) == HPC_PENDING){
            if(Sem_Wait(asyncWrite.done, writeTimeout) == SEM_TIMEDOUT){
                printf("***Timeout writing to hpc device\n"); fflush(stdout);
            }
            
            if((stat = hpcGetIrpStatus(&asyncWrite, &bytesSent, FALSE)) == HPC_PENDING){
                hpcCancelio(chanNum, HPC_WRITE);
                
                // get the actual number of bytes written
                hpcGetIrpStatus(&asyncWrite, &bytesSent, FALSE);
            }
            
            else if(stat != HPC_OK){
                printf("***Error writing to hpc device\n"); fflush(stdout);
            }
        }

        else if(wstat != HPC_OK){
            printf("***Error writing to hpc device\n"); fflush(stdout);
        }
#endif
    }

//    printf("Wrote %d bytes to hpc device, wstat=0x%x, stat=0x%x\n", bytesSent, wstat, stat); fflush(stdout);

    return(bytesSent);
}

