/* win32ComDev.cpp
 * Win32 Serial Communication Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Serial Communication Library
 * author: drj Mar 16, 99 Initial version
 * revisions:
 * $History: win32ComDev.cpp $
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 6/28/00    Time: 1:09p
 * Created in $/dev_1200/Share/scomm
 * ---------------------------------------------------------------------
 */

#include "win32ComDev.h"

static const unsigned int RXBUF_SIZE = 1024 * 30;
static const unsigned int TXBUF_SIZE = 1024;

Win32ComDev::Win32ComDev(char *devName)
{
	unsigned int eventMasks;

	devEn = 0;
	if((fh = CreateFile(devName, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING,
					FILE_FLAG_OVERLAPPED, NULL)) == INVALID_HANDLE_VALUE){
		printf("Device::Device: Error opening %s, error=%d\n", devName, GetLastError());
		status = -1;
		return;
	}

	if(!SetupComm(fh, RXBUF_SIZE, TXBUF_SIZE)){
		printf("Device::Device: Error setting up comm device %s, error=%d\n", devName, GetLastError());
		status = -1;
		CloseHandle(fh);
		return;
	}

	status = 0;
	asyncRead.Offset = 0 ;
	asyncRead.OffsetHigh = 0 ;
	asyncWrite.Offset = 0 ;
	asyncWrite.OffsetHigh = 0 ;

	// set read timeout to max of one second
	commTimeOuts.ReadIntervalTimeout = MAXWORD;
	commTimeOuts.ReadTotalTimeoutMultiplier = MAXWORD;
	commTimeOuts.ReadTotalTimeoutConstant = 1000;

	// no timeouts on write
	commTimeOuts.WriteTotalTimeoutMultiplier = 0;
	commTimeOuts.WriteTotalTimeoutConstant = 0;
	SetCommTimeouts(fh, &commTimeOuts);

	if((asyncRead.hEvent = CreateEvent(
		NULL,    // no security attributes 
		TRUE,    // manual reset event 
		FALSE,   // not-signalled 
		NULL)) == NULL){
		printf("Device::Device: Error creating read event, error=%d\n", GetLastError());
		fflush(stdout);
	}

	if((asyncWrite.hEvent = CreateEvent(
		NULL,    // no security attributes 
		TRUE,    // manual reset event 
		FALSE,   // not-signalled 
		NULL)) == NULL){
		printf("Device::Device: Error creating write event, error=%d\n", GetLastError());
		fflush(stdout);
	}

    PurgeComm(fh, PURGE_TXABORT | PURGE_RXABORT | PURGE_TXCLEAR | PURGE_RXCLEAR );
    EscapeCommFunction(fh, SETDTR);

	//eventMasks = EV_BREAK|EV_CTS|EV_DSR|EV_ERR|EV_RING|EV_RLSD|EV_RXCHAR|EV_RXFLAG|EV_TXEMPTY;		
	eventMasks = EV_RXCHAR;		
	if(!SetCommMask(fh, eventMasks)){
		printf("Device::Device: Error setting %s Events, error=%d\n", devName, GetLastError());
		CloseHandle(fh);
		return;
	}

	return;
}

Win32ComDev::~Win32ComDev(){
    EscapeCommFunction(fh, CLRDTR);
	CloseHandle(fh);
	CloseHandle(asyncRead.hEvent);
	CloseHandle(asyncWrite.hEvent);
}


int Win32ComDev::get(char *buf, unsigned int len)
{
	//printf("Device::get: reading from device\n"); fflush(stdout);
	int bytesRead = 0, bytesToRead, fReadStat, dwError, dwEvtMask = 0;
	DWORD	errorFlags;
	COMSTAT   comStat ;

	if(devEn == 0) return(0);	// device disabled -- not ready to receive

	if(!WaitCommEvent(fh, (unsigned long *)&dwEvtMask, NULL)){
		printf("Device::get: WaitCommEvent failed, error=%d\n", GetLastError());
		return(0);
	}

	if(dwEvtMask & EV_RXCHAR){
	   ClearCommError(fh, &errorFlags, &comStat);
       if((bytesToRead = min((DWORD)len, comStat.cbInQue)) > 0){

		  if(!(fReadStat = ReadFile(fh, buf, (unsigned long)bytesToRead, (unsigned long *)&bytesRead, &asyncRead))){
			 if((dwError = GetLastError()) == ERROR_IO_PENDING){
				 while(!GetOverlappedResult(fh, &asyncRead, (unsigned long *)&bytesRead, TRUE)){
					dwError = GetLastError();
					if(dwError == ERROR_IO_INCOMPLETE){
						// normal result if not finished
						continue;
					}
					else{
                        ClearCommError(fh, &errorFlags, &comStat) ;
						printf("Device::get error reading from device, error=%d\n", dwError); fflush(stdout);
						break;
					}
				 }
			 }
			 else{
				printf("Device::get error reading from device, error=%d\n", dwError); fflush(stdout);
			 }
		  }
	   }
	}

	//printf("Device::get: read %d (0x%x) bytes from device\n", bytesRead, buf[0]); fflush(stdout);
	return(bytesRead);
}



int Win32ComDev::put(char *buf, unsigned int len){
	int fWriteStat, dwError;
	unsigned int bytesSent = 0, bytesToWrite = 0, bytesWritten=0;
	DWORD  errorFlags;
	COMSTAT    comStat ;

	if(devEn == 0) return(0);	// device disabled -- not ready to send

	ClearCommError(fh, &errorFlags, &comStat);

	// prevent buffer over run
    if((bytesToWrite = min((DWORD)len, TXBUF_SIZE - comStat.cbOutQue)) > 0){

		//printf("Device::put: writing to device\n"); fflush(stdout);
		if(!(fWriteStat = WriteFile(fh, buf, (unsigned long)len, (unsigned long *)&bytesWritten, &asyncWrite))){
			if((dwError = GetLastError()) == ERROR_IO_PENDING){
				while(!GetOverlappedResult(fh, &asyncWrite, (unsigned long *)&bytesWritten,TRUE)){
					dwError = GetLastError();
					if(dwError == ERROR_IO_INCOMPLETE){
						// normal result if not finished
						bytesSent += bytesWritten;
						continue;
					}
					else{
						printf("Device::put error writing to device, error=%d\n", dwError); fflush(stdout);
						ClearCommError(fh, &errorFlags, &comStat);
						if(bytesSent < bytesWritten) continue;
						break;
					}
				}
			}
			else{
				printf("Device::put error writing to device, error=%d\n",dwError); fflush(stdout);
			}

		}
	}
	bytesSent += bytesWritten;
	if(!FlushFileBuffers(fh)){
	   printf("Device::put; FlushFileBuffers failed; error =%d\n",GetLastError()); fflush(stdout);
	}

	//printf("Device::put: wrote %d bytes to device\n", bytesSent); fflush(stdout);
	return(bytesSent);
}


int Win32ComDev::setBaud(int baudrate)
{
	int oldRate = getBaud();
	switch(baudrate){
		case BR_300: ctlBlk.BaudRate = BAUD_300; break;
		case BR_1200: ctlBlk.BaudRate = BAUD_1200; break;
		case BR_2400: ctlBlk.BaudRate = BAUD_2400; break;
		case BR_4800: ctlBlk.BaudRate = BAUD_4800; break;
		case BR_9600: ctlBlk.BaudRate = BAUD_9600; break;
		case BR_14400: ctlBlk.BaudRate = BAUD_14400; break;
		case BR_19200: ctlBlk.BaudRate = BAUD_19200; break;
		case BR_38400: ctlBlk.BaudRate = BAUD_38400; break;
		case BR_57600: ctlBlk.BaudRate = BAUD_57600; break;
		case BR_115200: ctlBlk.BaudRate = BAUD_115200; break;
		default: 		ctlBlk.BaudRate = baudrate; break;
	}
	SetCommState(fh, &ctlBlk) ;
	return(oldRate);
}

int Win32ComDev::getBaud(void)
{
	switch(ctlBlk.BaudRate){
		case BAUD_300: return(BR_300);
		case BAUD_1200: return(BR_1200);
		case BAUD_2400: return(BR_1200);
		case BAUD_4800: return(BR_4800);
		case BAUD_9600: return(BR_9600);
		case BAUD_14400: return(BR_14400);
		case BAUD_19200: return(BR_19200);
		case BAUD_38400: return(BR_38400);
		case BAUD_57600: return(BR_57600);
		case BAUD_115200: return(BR_115200);
	}
	return(BR_OTHER);
}


int Win32ComDev::setDevCtlBlk(DevCtlBlk_T *devCtlBlk)
{
	const unsigned char ASCII_XON = 0x17, ASCII_XOFF = 0x19;

	if(!devCtlBlk) return(-1);
	if(GetCommState(fh, &ctlBlk) == 0){
		printf("Device::Device: Error getting DCB, error=%d\n", GetLastError());
		CloseHandle(fh);
		return -1;
	}

	disableDev();		// disable device -- fake

	// setup hardware flow control -- DSR, DTR, RTS, and CTS are not supported on esb-21285 or IXP1200
	ctlBlk.fOutxDsrFlow = 0;
	if(ctlBlk.fOutxDsrFlow)
	  ctlBlk.fDtrControl = DTR_CONTROL_HANDSHAKE ;
	else
	  ctlBlk.fDtrControl = DTR_CONTROL_ENABLE ;

	ctlBlk.fOutxCtsFlow = 0;
	if(ctlBlk.fOutxCtsFlow)
	  ctlBlk.fRtsControl = RTS_CONTROL_HANDSHAKE ;
	else
	  ctlBlk.fRtsControl = RTS_CONTROL_ENABLE ;
	ctlBlk.fDsrSensitivity = FALSE;


	// setup software flow control -- no software control
	ctlBlk.fInX = ctlBlk.fOutX = FALSE;
	ctlBlk.fTXContinueOnXoff = TRUE;
	ctlBlk.fBinary = TRUE ;
	ctlBlk.fParity = FALSE ;
	ctlBlk.fAbortOnError = FALSE;
	ctlBlk.fErrorChar = FALSE ;
	ctlBlk.fNull = FALSE ;

	ctlBlk.XonChar = ASCII_XON ;
	ctlBlk.XoffChar = ASCII_XOFF ;
	ctlBlk.XonLim = 256 ;
	ctlBlk.XoffLim = 256 ;

	// other misc control
	ctlBlk.BaudRate = devCtlBlk->baudRate;
	ctlBlk.ByteSize = devCtlBlk->byteSize + 5;
	if(devCtlBlk->parity == ODD_PARITY) ctlBlk.Parity = ODDPARITY;
	   else if(devCtlBlk->parity == EVEN_PARITY) ctlBlk.Parity = EVENPARITY;
		  else ctlBlk.Parity = NOPARITY;
	if(devCtlBlk->stopBits == ONE_STOPBIT) ctlBlk.StopBits = ONESTOPBIT;
	   else ctlBlk.StopBits = TWOSTOPBITS;

	if(devCtlBlk->enableDev) enableDev();

	return(!SetCommState(fh, &ctlBlk));
}


int Win32ComDev::getDevCtlBlk(DevCtlBlk_T *devCtlBlk)
{
	if(!devCtlBlk) return(-1);

	if(GetCommState(fh, &ctlBlk) == 0){
		printf("Device::getDevCtlBlk: Error getting DCB, error=%d\n", GetLastError());
		return -1;
	}
	devCtlBlk->byteSize = ctlBlk.ByteSize - 5;
	if(ctlBlk.Parity == ODDPARITY) devCtlBlk->parity = ODD_PARITY;
	   else if(ctlBlk.Parity == EVENPARITY) devCtlBlk->parity = EVEN_PARITY;
		  else devCtlBlk->parity = NO_PARITY;

	if(ctlBlk.StopBits == ONESTOPBIT) devCtlBlk->stopBits = ONE_STOPBIT;
	   else devCtlBlk->stopBits = TWO_STOPBITS;
	devCtlBlk->enableDev = TRUE;
	devCtlBlk->baudRate = ctlBlk.BaudRate;
	return(0);
}

