/* win32HpcDev.cpp
 * Win32 HPC Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Serial Communication Library
 * author: drj June 26, 2000 Initial version
 * revisions:
 * $History: win32HpcDev.cpp $
 * 
 * *****************  Version 12  *****************
 * User: Ewalker      Date: 12/13/00   Time: 2:40p
 * Updated in $/dev_1200/Share/scomm
 * Fixed a timeout problem in hardware debug over PCI.
 * 
 * *****************  Version 11  *****************
 * User: Ewalker      Date: 9/29/00    Time: 12:23p
 * Updated in $/dev_1200/Share/scomm
 * Made async read code consistent with async write code
 * 
 * *****************  Version 9  *****************
 * User: Ewalker      Date: 9/20/00    Time: 9:58a
 * Updated in $/dev_1200/Share/scomm
 * Performance cleanup
 * 
 * *****************  Version 8  *****************
 * User: Drjohns1     Date: 8/30/00    Time: 2:06p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 7  *****************
 * User: Qabuild      Date: 8/24/00    Time: 5:35p
 * Updated in $/dev_1200/Share/scomm
 * Changed Scomm_InstallDev return type
 * 
 * *****************  Version 6  *****************
 * User: Drjohns1     Date: 8/22/00    Time: 4:57p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 5  *****************
 * User: Qabuild      Date: 8/15/00    Time: 5:13p
 * Updated in $/dev_1200/Share/scomm
 * Changed HPC device name to ixphpc
 * 
 * *****************  Version 4  *****************
 * User: Qabuild      Date: 8/10/00    Time: 1:38p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 3  *****************
 * User: Drjohns1     Date: 6/30/00    Time: 5:28p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 6/28/00    Time: 1:09p
 * Created in $/dev_1200/Share/scomm
 * ---------------------------------------------------------------------
 */

#include "win32HpcDev.h"


// device name should be ixphpc01 -- where the first digit indicates the device number (0 in this case),
// and the other digit is the channel number 


WinHpcDev::WinHpcDev(char *devName)
{
	char dev[MAX_DEVICE_NAME];


	devEn = 0;
	sprintf(dev, "\\\\.\\%s", devName);
	if((hpc_fh = CreateFile(dev, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING,
					FILE_ATTRIBUTE_NORMAL | FILE_FLAG_OVERLAPPED, NULL)) == INVALID_HANDLE_VALUE){
		printf("Device::Device: Error opening %s, error=%d\n", devName, GetLastError());
		status = -1;
		return;
	}

#ifdef dj_debug
#include <windows.h>
#include "IXPHPC_shared.h"

    unsigned long cbReturned;
	int msgMode = IXPHPC_MSG_STATUS_INCOMPLETE_OK;
    if(DeviceIoControl(hpc_fh, (DWORD) IOCTL_IXPHPC_SET_INCOMPLETE_STS,
                       &msgMode,4,0,0,&cbReturned,0)){
        printf ("DeviceIoControl SetMsgMode succeeded\n");
    }
    else
        printf ("DeviceIoControl SetMsgMode failed\n");
#endif

	status = 0;
	asyncRead.Internal = 0;
	asyncRead.InternalHigh = 0;
	asyncRead.hEvent = NULL;
	asyncRead.Offset = 0 ;
	asyncRead.OffsetHigh = 0 ;

	asyncWrite.hEvent = NULL;
	asyncWrite.Offset = 0 ;
	asyncWrite.OffsetHigh = 0 ;

	if((asyncRead.hEvent = CreateEvent(
		NULL,    // no security attributes 
		TRUE,    // manual reset event 
		FALSE,   // not-signalled 
		NULL)) == NULL){
		printf("WinHpcDev::WinHpcDev: Error creating read event, error=%d\n", GetLastError());
		fflush(stdout);
	}

	if((asyncWrite.hEvent = CreateEvent(
		NULL,    // no security attributes 
		TRUE,    // manual reset event 
		FALSE,   // not-signalled 
		NULL)) == NULL){
		printf("WinHpcDev::WinHpcDev: Error creating write event, error=%d\n", GetLastError());
		fflush(stdout);
	}

	// timeouts are in milliseconds
	readTimeout = 1000;
	writeTimeout = 1000;

	return;
}


WinHpcDev::~WinHpcDev(){
	CloseHandle(hpc_fh);
	CloseHandle(asyncRead.hEvent);
	CloseHandle(asyncWrite.hEvent);
}

#include "utils.h"


int WinHpcDev::put(char *buf, unsigned int bytesToWrite)
{
	int fWriteStat, dwError;
	unsigned int bytesSent=0;

//	printf("Writing %d bytes to hpc device\n", bytesToWrite); fflush(stdout);

    if(bytesToWrite > 0){
        if(!(fWriteStat = WriteFile(hpc_fh, buf, (unsigned long)bytesToWrite, (unsigned long *)&bytesSent, &asyncWrite))){
            if((dwError = GetLastError()) == ERROR_IO_PENDING){
                if (WaitForSingleObject(asyncWrite.hEvent, writeTimeout) == WAIT_TIMEOUT){
                   printf("***Timeout writing to hpc device\n"); fflush(stdout);
                }
                else{
                    if (!GetOverlappedResult(hpc_fh, &asyncWrite, (unsigned long *)&bytesSent, TRUE)){
                        printf("***Error writing to hpc device, error=%d\n", GetLastError()); fflush(stdout);
                    }
                }
            }
            else{
                printf("***Error writing to hpc device, error=%d\n", dwError); fflush(stdout);
            }            
        }
    }

//    printf("Wrote %d bytes to hpc device\n", bytesSent); fflush(stdout);

    return(bytesSent);
}

int WinHpcDev::get(char *buf, unsigned int bytesToRead)
{
	int bytesRead=0, fReadStat, dwError;
	
//	printf("Reading %d bytes from hpc device\n", bytesToRead); fflush(stdout);

    if(bytesToRead > 0){
        if(!(fReadStat = ReadFile(hpc_fh, buf, (unsigned long)bytesToRead, (unsigned long *)&bytesRead, &asyncRead))){
            if((dwError = GetLastError()) == ERROR_IO_PENDING){
//                if (WaitForSingleObject(asyncRead.hEvent, readTimeout) == WAIT_TIMEOUT){
//                    printf("***Timeout reading from hpc device\n"); fflush(stdout);
//                }
//                else{
                    if(!GetOverlappedResult(hpc_fh, &asyncRead, (unsigned long *)&bytesRead, TRUE)){
                        printf("***Error reading from hpc device, error=%d\n", GetLastError()); fflush(stdout);
                        return(0);
                    }
//                }
            }
        }
    }

//    printf("Read %d bytes from hpc device\n", bytesRead); fflush(stdout);

    return(bytesRead);
}

