/* win32SockDev.cpp
 * Win32 Socket Communication Library
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: SA1200
 * subsystem: Socket Communication Library
 * author: drj Mar 16, 99 Initial version
 * revisions:
 * $History: win32SockDev.cpp $
 * 
 * *****************  Version 3  *****************
 * User: Drjohns1     Date: 8/30/00    Time: 2:06p
 * Updated in $/dev_1200/Share/scomm
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 6/29/00    Time: 3:18p
 * Updated in $/dev_1200/Share/scomm
 * Updated for HPC and to handle multiple devices
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 6/28/00    Time: 1:09p
 * Created in $/dev_1200/Share/scomm
 * ---------------------------------------------------------------------
 */

#include "utils.h"
#include "win32SockDev.h"

#ifndef SD_BOTH
#define SD_BOTH         0x02
#endif


static const unsigned short PORTNUM = 988;
static const int SemCount = 2;

extern "C" threadFunc_T processConnect(void *obj);

WinSockDev::WinSockDev(char *devName)
{
	int  wVersionRequested, err, i, nodelay=1;
	WSADATA wsaData;
	char *host = NULL;

	// device name format: sock/hostname
	if(strlen(devName) > 5) host = &devName[5];

	status = devEn = 0;
	wVersionRequested = MAKEWORD(2, 0); 
	if((err = WSAStartup( wVersionRequested, &wsaData))){
		printf("Device::Device: Couldn't find a usable WinSock DLL\n");
		return;
	}

	connSem = Sem_Init(SemCount);
	for(i = 0; i < SemCount; i++) Sem_Wait(connSem, SEM_WAITFOREVER);

	memset((char *) &mySockAddr, 0, sizeof(struct sockaddr_in));
	mySockAddr.sin_family = AF_INET;
	mySockAddr.sin_port = htons(PORTNUM);;
	if(host){
		// client -- Get host address
		gethostname(hostName, MAXHOSTNAME);
		   
		if(!(hp = gethostbyname(host))){
			if(!(hp = gethostbyaddr(host, sizeof(struct sockaddr_in), AF_INET))){               
				fprintf(stderr,"Device::Device: Invalid host name %s\n", host);
				return;
			}
		}
		mySockAddr.sin_addr = *((struct in_addr *)hp->h_addr);
	}
	else mySockAddr.sin_addr.s_addr = htonl(INADDR_ANY);


	if(!host){
	   // Server
		/* Create socket */
		if((sd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == INVALID_SOCKET){
			printf("socket failed err=%d\n", WSAGetLastError());
		   return;
		}

		// set no delay
		if(setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (char *)&nodelay, sizeof(nodelay)) == INVALID_SOCKET){
		   printf("setsockopt failed err=%d\n", WSAGetLastError());
		   shutdown(sd, SD_BOTH);
		   closesocket(sd);
		   return;
		}

		if(bind(sd,(struct sockaddr *) &mySockAddr, sizeof(struct sockaddr_in)) == SOCKET_ERROR){
		  printf("bind failed err=%d\n", WSAGetLastError());
		   shutdown(sd, SD_BOTH);
		   closesocket(sd);
		   return;
		}
		//spawn process waiting of connections
		listen(sd, 1);
		Thread_Create(MID_PRIORITY, (threadFunc_T)processConnect, (void *)this);
	}
	else{
		//Client
		if((sconn = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP)) == INVALID_SOCKET){
			printf("socket failed err=%d\n", WSAGetLastError());
		   return;
		}
		// set no delay
		if(setsockopt(sconn, IPPROTO_TCP, TCP_NODELAY, (char *)&nodelay, sizeof(nodelay)) == INVALID_SOCKET){
			printf("setsockopt failed err=%d\n", WSAGetLastError());
		   shutdown(sconn, SD_BOTH);
		   closesocket(sconn);
		   return;
		}

		if(connect(sconn, (struct sockaddr *) &mySockAddr, sizeof(mySockAddr))){
		  shutdown(sconn, SD_BOTH);
		  closesocket(sconn);
		  return;
		}
		for(i =0; i < SemCount; i++) Sem_Post(connSem);
	}

	status = 1;
	return;
}


WinSockDev::~WinSockDev()
{
	shutdown(sd, SD_BOTH);
	closesocket(sd);
	shutdown(sconn, SD_BOTH);
	closesocket(sconn);
	Sem_Destroy(connSem);
}


void WinSockDev::acceptConn(void)
{
	int len;
	len = sizeof(fromSockAddr);
	if((sconn = accept(sd, (struct sockaddr *) &fromSockAddr, &len)) < 0){
	  if(errno != EINTR){
		  perror("accept");
		  return;
	  }
	}
	for(int i =0; i < SemCount; i++) Sem_Post(connSem);
	return;
}


int WinSockDev::get(char *buf, unsigned int len)
{
	int nmRead;

	Sem_Wait(connSem, SEM_WAITFOREVER);
	Sem_Post(connSem);

	if(!devEn) return(0);
	if((nmRead = recv(sconn, buf, len, 0)) == SOCKET_ERROR){
		int err = WSAGetLastError();
		if((err == WSAECONNRESET) || (err == WSAECONNABORTED)){
			// connection down -- wait for another connection
			for(int i =0; i < SemCount; i++) Sem_Wait(connSem, SEM_WAITFOREVER);
		}
printf("dj: recv failed, len=%d, err=%d\n", len, err); fflush(stdout);
		return(0);
	}
	return nmRead;
}


int WinSockDev::put(char *buf, unsigned int len)
{
	int bytes_written;

	Sem_Wait(connSem, SEM_WAITFOREVER);
	Sem_Post(connSem);
	if(!devEn) return(0);

	if((bytes_written = send(sconn, buf, len, 0)) == SOCKET_ERROR){
		int err = WSAGetLastError();
printf("dj: send failed, len=%d, err=%d\n", len, err); fflush(stdout);
		if((err == WSAECONNRESET) || (err == WSAECONNABORTED)){
			// connection down -- wait for another connection
			for(int i =0; i < SemCount; i++) Sem_Wait(connSem, SEM_WAITFOREVER);
		}
		return(0);
	}
	return(bytes_written);
}
