/*------------------------------------------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *
 *------------------------------------------------------------------------------------------------------------*/
/*
 * File = $Id: $
 *
 *
 * Edit History
 *
 * $Log: /dev_1200/Utils/lst_util.h $
 * 
 * 48    4/07/00 5:38p Mgfernan
 * fix "nested comment" warning
 * 
 * 45    12/07/99 10:40a Mgfernan
 * declare constant member functions as const
 * 
 * 44    9/22/99 9:42a Jdguilfo
 * Fix Bill's last fix: it was applied to wrong function
 * 
 * 43    9/21/99 2:42p Bwheeler
 * fix bug in find_elem
 * 
 * 42    9/08/99 8:25a Bwheeler
 * minor updates to lst_util.h
 * 
 * 41    9/08/99 8:20a Bwheeler
 * minor updates to lst_util.h
 * 
 * 38    3/23/99 3:20p C-tclancy
 * update COPYRIGHT
 * 
 * 37    3/23/99 3:18p C-tclancy
 * update COPYRIGHT
 * 
 * 36    2/18/99 12:20p Qabuild
 * update for comments
 * 
 * 35    2/12/99 6:46p Drjohns1
 * Updated for auto_ambio & multi-chip related changes.
 * 
 * 33    9/18/98 10:21a Bwheeler
 * make Dlist_ext<T>::list_elem return *T
 * 
 * 32    9/11/98 11:27a Bwheeler
 * 
 * 31    6/24/98 6:22p Bwheeler
 * updates to support sdram substitution mode:
 *     allow t-to-many signal relationships
 *     allow polarity inverts in signal relationships
 *     specify transactor tri-state enable in addition to v_model one
 * add pci signal to ctx_arb uword
 * 
 * 25    5/04/98 3:41p Bwheeler
 * fix more template compile bugs
 * 
 * 24    5/04/98 2:29p Bwheeler
 * fix unix compile errors/warnings
 * 
 * 23    5/04/98 11:15a Bwheeler
 * fix compile bug
 * 
 * 22    5/04/98 8:49a Bwheeler
 * change Iter<T> assignment constructor to fix compiler bug!
 * 
 * 20    5/01/98 1:56p Bwheeler
 * support generalized v_model integration
 * 
 * 19    4/30/98 3:55p Bwheeler
 * add Dlist_ext template (doubly linked lists working with iterators)
 * 
 * 17    2/13/98 4:35p Jguilford
 * Changes needed to compile under Unix
 * 
 * 15    1/30/98 4:20p Jguilford
 * Added Macros to make list creation easier
 * 
 * 13    12/12/97 1:22p Jguilford
 * Fix bug where swap would fail if elem2 was the only element in the list
 * and elem1 was not in the list
 * 
 * 12    12/09/97 3:04p Wheeler
 * fix memory leaks (not completely, but it's down to a trickle now)
 * 
 * 11    12/01/97 8:57a Wheeler
 * fix bugs around assembler integration with transactor
 * 
 * 10    11/17/97 8:38a Wheeler
 * fix bug in branch decision logic
 * 
 * 9     11/14/97 6:04a Wheeler
 * support uword addr field encodings used for Duane's model
 * 
 * 8     11/11/97 11:18a Wheeler
 * record label references for jump instructions
 * 
 * 7     11/11/97 10:15a Wheeler
 * add "dg" ifdef for Duane's kludges in code
 * 
 * 6     11/11/97 9:54a Wheeler
 * fix bugs in branch pipe determination logic
 * 
 * 5     9/05/97 1:49p Wheeler
 * add new immed variants corresponding to implementation
 * 
 * 4     9/05/97 6:28a Wheeler
 * eliminate immediate data option for ld_field
 * 
 * 3     8/28/97 11:16a Johnsondes
*/



#ifndef LST_UTIL_DEFINED
#define LST_UTIL_DEFINED 1

#include "assert.h"

#define TRUE	1
#define FALSE	0

#ifdef __cplusplus

template<class T> class Dlist
    {
    /* this template manages a doubly linked list container whose link */
    /* pointers are held directly in the objects being contained */
    /* This structure has a performance advantage over traditional list */
    /* container structures in that list searches only require one reference */
    /* per element, rather than two */

    protected:

    T *head_elem;
    T *tail_elem;
    int list_len;
    int next_offset;
    int back_offset;

    public:

    /* constructor */
    Dlist( T *p, T *&next, T *&back );
    Dlist( T *p, T **next, T **back );
    /* use DLIST as:
    Dlist<class> *var = new DLIST(class,next,back);*/
#define DLIST(class,next,back) Dlist<class>((class*)0,&(((class*)0)->next),&(((class*)0)->back))

    /* destructor */
    ~Dlist();

    /* function prototypes */
    void head_insert( T *elem );
    void tail_insert( T *elem );
    void insert_after( T *list_elem, T *insert_elem );
    void insert_before( T *list_elem, T *insert_elem );

    T *head_remove();
    T *tail_remove();
    T *remove( T *list_elem );
	void append( Dlist<T> *list );
    void remove_all();              /* remove all elements on list */
    void destroy_all();             /* remove and delete all elements on list */

    T *advance( T *cur, int n );    /* find ptr n elements past cur */
	T *list_elem( int n );          /* find the nth element on the list */
	T *list_elem( T *base_elem, int rel_offset );	/* find elem relative to base_elem*/
    int distance( T *from, T *to ); /* find distance between elements */
    int swap( T *elem1, T *elem2 ); /* swap elements on list */
    
    int contains( T *elem );        /* TRUE if list contains element */

    /* inline functions */
    T *head()
		{
		return head_elem;
		}

    T *tail()
		{
		return tail_elem;
		}

    T *next( T *cur )
		{
		/* return the next element on the list based on the current one */
		T **p;

		p = (T **) cur + next_offset;
		return *p;
		}

    void next( T *cur, T *assign_val )
		{
		/* update the value of the next field of the cur elem */
		T **p;

		p = (T **) cur + next_offset;
		*p = assign_val;
		}

    T *back(const T *cur ) const
		{
		/* return the previous element on the list based on the current one */
		T **p;

		p = (T **) cur + back_offset;
		return *p;
		}

    void back(const T *cur, T *assign_val ) const
		{
		/* update the value of the back field of the cur elem */
		T **p;

		p = (T **) cur + back_offset;
		*p = assign_val;
		}

    int len()
		{
		return list_len;    /* return length of entire list */
		}


    int len( T *cur )
		{
		/* return number of elements on list starting at cur */

		int i;

		for ( i = 0; cur; cur = next( cur ) )
			i++;

		return i;
		}
    };


template<class T> class Dnamelist : public Dlist<T>
    {
    /* this template is derived from the Dlist template and so manages */
    /* a doubly linked list container in the same way as Dlist<T>.  In */
    /* addition, this class maintains a character string field so that */
    /* the elements can be inserted alphabetically and searched based on */
    /* the name */

    private:
    int name_offset;

    public:
    Dnamelist( T *p, T *&next, T *&back, char *&name );
    Dnamelist( T *p, T **next, T **back, char **name );
#define DNAMELIST(class,next,back,name) Dnamelist<class>((class*)0,&(((class*)0)->next),&(((class*)0)->back),&(((class*)0)->name))

    int alpha_insert( T *elem );           /* insert elem alphabetically */
    T *find_name( char *search_name );     /* find elem by its name */
	T *find_name( char *search_name, T *&prev_name );

    char *name( T *cur )
		{
		/* return the string name associated with this element */
		char **p;

		p = (char **) cur + name_offset;

		return *p;
		}


    };

template<class T> class Dsortlist : public Dlist<T>
    {
    /* this is an extention of the Dlist template similar to Dnamelist
       which manages a doubly linked list container in the same way as
       Dlist<T>. In addition, this class allows elements to be inserted
       in sorted order.

       The class being contained must have a >= operator defined similar to

       class T {
       public:
           friend int operator>= (const T& first, const T& second);
           // this operator returns 0 if first==second in sorted order
           //                      >0 if first occurs after second in order
           //                      <0 if first occurs before second
     */
public:
    Dsortlist(T *p, T *&next, T *&back);
    Dsortlist(T *p, T **next, T **back);
#define DSORTLIST(class,next,back) Dsortlist<class>((class*)0,&(((class*)0)->next),&(((class*)0)->back))

    int sorted_insert( T *elem );
    };
    


/********************************************************************************/

/* constructor */
template<class T> Dlist<T>::Dlist( T *p, T *&next, T *&back )
    {
    head_elem = tail_elem = NULL;
    list_len = 0;

    /* find the offsets from the start of the object structure */
    /* for the next and back link pointers */
    next_offset = &next - (T **) p;
    back_offset = &back - (T **) p;
    }

template<class T> Dlist<T>::Dlist( T *p, T **next, T **back )
    {
    head_elem = tail_elem = NULL;
    list_len = 0;

    /* find the offsets from the start of the object structure */
    /* for the next and back link pointers */
    next_offset = next - (T **) p;
    back_offset = back - (T **) p;
    }

/* destructor */
template<class T> Dlist<T>::~Dlist()
    {
    remove_all();
    }

template<class T> void Dlist<T>::head_insert( T *elem )
    {
    /* add specified element to head of list */
    /* NOTE:  it is assumed that specified element is not already */
    /*        on a list (no checking is done to verify this) */

    next( elem, head_elem );     /* elem->next = head_elem */
    back( elem, NULL );     /* elem->back = NULL */

    if ( head_elem )
		back( head_elem, elem ); /* head_elem->back = elem */
    else tail_elem = elem;       /* first list element inserted */

    head_elem = elem;            /* new head_elem */

    list_len++;    
    }

template<class T> void Dlist<T>::tail_insert( T *elem )
    {
    /* add specified element to tail of list */
    /* NOTE:  it is assumed that specified element is not already */
    /*        on a list (no checking is done to verify this) */

    next( elem, NULL );     /* elem->next = NULL */
    back( elem, tail_elem );     /* elem->back = tail_elem */

    if ( tail_elem )
		next( tail_elem, elem ); /* tail_elem->next = elem */
    else head_elem = elem;       /* first list element inserted */

    tail_elem = elem;            /* new tail_elem */

    list_len++;    
    }

template<class T> void Dlist<T>::insert_after( T *list_elem, T *insert_elem )
    {
    /* insert "insert_elem" after the specified "list_elem".  If */
    /* "list_elem" == NULL, then element is inserted at head of list. */
    /* It is assumed that "insert_elem" is not on a list, and that */
    /* "list_elem" is on the list corresponding to Dlist<T> */

    if ( list_elem == NULL )
		head_insert( insert_elem );
    else if ( list_elem == tail_elem )
		tail_insert( insert_elem );
    else
		{
		/* insert in middle of list */
		next( insert_elem, next( list_elem ) ); /* insert_elem->next = list_elem->next */
		back( insert_elem, list_elem );         /* insert_elem->back = list_elem */

		back( next( list_elem ), insert_elem ); /* list_elem->next->back = insert_elem */
		next( list_elem, insert_elem );         /* list_elem->next = insert_elem */

		list_len++;
		}
    }

    
template<class T> void Dlist<T>::insert_before( T *list_elem, T *insert_elem )
    {
    /* insert "insert_elem" before the specified "list_elem".  If */
    /* "list_elem" == NULL, then element is inserted at tail of list. */
    /* It is assumed that "insert_elem" is not on a list, and that */
    /* "list_elem" is on the list corresponding to Dlist<T> */

    if ( list_elem == NULL )
		tail_insert( insert_elem );
    else if ( list_elem == head_elem )
		head_insert( insert_elem );
    else
		{
		/* insert in middle of list */
		back( insert_elem, back( list_elem ) ); /* insert_elem->back = list_elem->back */
		next( insert_elem, list_elem );         /* insert_elem->next = list_elem */

		next( back( list_elem ), insert_elem ); /* list_elem->back->next = insert_elem */
		back( list_elem, insert_elem );         /* list_elem->back = insert_elem */

		list_len++;
		}
    }

    
    
template<class T> T *Dlist<T>::remove( T *list_elem )
    {
    /* remove element from list */
    /* NOTE:  it is assumed that specified element is on */
    /*        the list corresponding to Dlist<T> (no checking */
    /*        is done to verify this) */

    T *prev_elem, *next_elem;

    if ( list_elem == head_elem )
		head_remove();
    else if ( list_elem == tail_elem )
		tail_remove();
    else
		{
		/* element is in middle of list */
		prev_elem = back( list_elem );
		next_elem = next( list_elem );
		next( prev_elem, next_elem );
		back( next_elem, prev_elem );

		next( list_elem, NULL );
		back( list_elem, NULL );

		list_len--;
		}

    return list_elem;
    }

template<class T> T *Dlist<T>::head_remove()
    {
    /* remove element from head of list */

    T *p;

    if ( head_elem == NULL )
		return NULL;    /* list is empty */

    if ( next( head_elem ) )
		{
		/* list has more than 1 element on it */
		p = head_elem;                  /* head elem to return */

		head_elem = next( head_elem );  /* set new head of list */
		back( head_elem, NULL );

		next( p, NULL );        /* unlink return elem from list */
		list_len--;
		}
    else
		{
		/* only 1 element on list */
		p = head_elem;
		head_elem = tail_elem = NULL;
		list_len = 0;
		}

    return p;
    }

template<class T> T *Dlist<T>::tail_remove()
    {
    /* remove element from tail of list */

    T *p;

    if ( tail_elem == NULL )
		return NULL;    /* list is empty */

    if ( back( tail_elem ) )
		{
		/* list has more than 1 element on it */
		p = tail_elem;                  /* tail elem to return */

		tail_elem = back( tail_elem );  /* set new head of list */
		next( tail_elem, NULL );

		back( p, NULL );        /* unlink return elem from list */
		list_len--;
		}
    else
		{
		/* only 1 element on list */
		p = tail_elem;
		head_elem = tail_elem = NULL;
		list_len = 0;
		}

    return p;
    }


template<class T> void Dlist<T>::append( Dlist<T> *list )
	{
	/* append the "list" to the end of the "this" list */

	if ( list->head_elem == NULL || this == list )
		return;

	if ( head_elem == NULL )
		{
		head_elem = list->head_elem;
		tail_elem = list->tail_elem;
		list_len = list->list_len;

		list->head_elem = NULL;
		list->tail_elem = NULL;
		list->list_len = 0;
		}

	list_len += list->len();
	list->head_elem->back( tail_elem );
	tail_elem->next( list->head_elem );
	tail_elem = list->tail_elem;

	list->head_elem = list->tail_elem = NULL;
	list->list_len = 0;
	}



template<class T> void Dlist<T>::remove_all()
    {
    /* remove all elements from list */

    while( head_elem )
		head_remove();
    }

template<class T> void Dlist<T>::destroy_all()
    {
    /* remove all elements from list and delete each element removed */

    T *p;

    while( head_elem )
		{
		p = head_remove();

		delete p;
		}
    }

template<class T> T *Dlist<T>::advance( T *cur, int n )
    {
    /* return a ptr to element on list "n" away from cur element */
    /* If n < 0, list is traversed backwards, otherwise, forwards. */
    /* Returning a NULL pointer means that specified element did not exist */

    if ( n >= 0 )
		{
		/* positive n means follow list forward */
		while( n && cur )
			{
			cur = next( cur );
			n--;
			}
		}
    else
		{
		/* negative n means follow list backward */
		while( n && cur )
			{
			cur = back( cur );
			n++;
			}
		}

    return cur;
    }

template<class T> T *Dlist<T>::list_elem( int n )
    {
	/* find the nth element on the list */

	T *elem;

	if ( n <= 0 )
		return NULL;

	for ( n--, elem = head(); n && elem; n--, elem = next( elem ) )
		;

	return elem;
    }

template<class T> T *Dlist<T>::list_elem( T *base_elem, int rel_offset )
    {
	/* find the nth element on the list, relative to the base_elem */
	/* A negative relative offset means that you search from the base_elem */
	/* towards the head of the list, otherwise, search toward the tail */

	T *elem;

	if ( rel_offset >= 0 )
		{
		for ( elem = base_elem; rel_offset && elem; rel_offset--, elem = next( elem ) )
			;
		}
	else
		{
		for ( elem = base_elem; rel_offset && elem; rel_offset++, elem = back( elem ) )
			;
		}

	return elem;
    }

template<class T> int Dlist<T>::distance( T *from, T *to )
    {
    /* return number representing distance between from and to elements */
    /* If return value < 0, then "to" was not found starting from "from"; */
    /* otherwise the return value represents the distance */

    int i;

    for ( i = 0; from != to && from; i++ )
		from = next( from );

    if ( from )
		return i;

    return -1;  /* to elem not found */
    }
    
template<class T> int Dlist<T>::contains( T *test_elem )
    {
    T *elem;

    for ( elem = head(); elem; elem = next( elem ) )
        {
        if (test_elem == elem)
            return 1;
        }
    return 0;
    }
    
template<class T> int Dlist<T>::swap( T *elem1, T *elem2 )
    {
    /* swap the 2 elements on the list.  This function successfully */
    /* performs the operation provided that:
    
	o) both elements are on same list
	o) one element is on a list and the other is not on a list

	otherwise it returns 0 without performing a swap
    */

    T *p, *temp;
    int n;


    if ( elem1 == elem2 )
		return 1;        /* same element specified */

    if ( next( elem1 ) == NULL && back( elem1 ) == NULL && head_elem != elem1 )
		{
		/* elem1 is not on a list */
		if ( next( elem2 ) == NULL && back( elem2 ) == NULL && head_elem != elem2 )
			return 0;  /* neither elem is on a list */

		/* elem2 is on a list */
		insert_after( elem2, elem1 );
		remove( elem2 );
		return 1;
		}

    /* elem1 is on a list */
    if ( next( elem2 ) == NULL && back( elem2 ) == NULL && head_elem != elem2 )
		{
		/* elem2 is not on a list */
		insert_after( elem1, elem2 );
		remove( elem1 );
		return 1;
		}

    /* both elements are on a list; check that it's the same list */
    n = distance( elem1, elem2 );
    if ( n < 0 )
		{
		temp = elem1;
		elem1 = elem2;
		elem2 = temp;
		n = distance( elem1, elem2 );
		}
    
    if ( n < 0 )
		return 0;  // elements not on same list

    /* swap elements on same list */
    remove( elem2 );

    insert_after( elem1, elem2 );
    remove( elem1 );

    p = advance( elem2, n-1 );
    insert_after( p, elem1 );

    return 1;
    }


/* constructor */
template<class T> Dnamelist<T>::Dnamelist( T *p, T *&next, T *&back, char *&name ) :
		  Dlist<T>( p, next, back )
    {
    /* find the offsets from the start of the object structure */
    /* for the next and back link pointers */
    name_offset = &name - (char **) p;
    }

/* constructor */
template<class T> Dnamelist<T>::Dnamelist( T *p, T **next, T **back, char **name ) :
		  Dlist<T>( p, next, back )
    {
    /* find the offsets from the start of the object structure */
    /* for the next and back link pointers */
    name_offset = name - (char **) p;
    }


template<class T> T *Dnamelist<T>::find_name( char *search_name )
    {
    /* find the first element on the list with the specified name */

    T *p;
    int i;

    for ( p = head(); p; p = next( p ) )
		{
		i = strcmp( name( p ), search_name );
		if ( i == 0 )
			return p;

		if ( i > 0 )
			break;
		}

    return NULL;    /* not found */
    }

template<class T> T *Dnamelist<T>::find_name( char *search_name, T *&prev_name )
    {
    /* Find and return the first element on the list with the specified name */
	/* Also pass back the element just prior to where the search element was found */
	/* or would have been found */

    T *p;
    int i;

	prev_name = NULL;
    for ( p = head(); p; p = next( p ) )
		{
		i = strcmp( name( p ), search_name );
		if ( i == 0 )
			return p;

		if ( i > 0 )
			break;

			prev_name = p;
		}

    return NULL;    /* not found */
    }

template<class T> int Dnamelist<T>::alpha_insert( T *elem )
    {
    /* insert the element into the list in the proper alphabetical position */
    /* return TRUE if the inserted name is unique on the list */

    T *p, *last_p;
    int i;

    last_p = NULL;
    for ( p = head(); p; p = next( p ) )
		{
		i = strcmp( name( p ), name( elem ) );
		if ( i >= 0 )
			break;

		last_p = p;
		}

    if ( last_p == NULL )
		head_insert( elem );
    else insert_after( last_p, elem );

    if ( p )
		return !!i;
    
    return 1;;
    }

/*******************************************************************************/

template<class T> Dsortlist<T>::Dsortlist(T *p, T *&next, T *&back) :
    Dlist<T>(p, next, back)
{}
template<class T> Dsortlist<T>::Dsortlist(T *p, T **next, T **back) :
    Dlist<T>(p, next, back)
{}

template<class T> int Dsortlist<T>::sorted_insert( T *elem )
    {
    /* insert the element into the list in the proper sorted order */
    /* return TRUE if the inserted name is unique on the list */

    T *p, *last_p;
    int i;

    last_p = NULL;
    for ( p = head(); p; p = next( p ) )
		{
		i = (*p >= *elem);
		if ( i >= 0 )
			break;

		last_p = p;
		}

    if ( last_p == NULL )
		head_insert( elem );
    else
        insert_after( last_p, elem );

    if ( p )
		return !!i;
    
    return 1;;
    }


/**********************************************************************************/
/******************************* EXTERNAL LIST ************************************/
/**********************************************************************************/

/* the following templates are used to contain lists of objects where the list pointers */
/* are held outside the actual objects on the list.  This means that traversing the */
/* list is slower than a Dlist<> list above, but it has the advantage that an object */
/* can be on multiple lists simultaneously without requiring that multiple list pointer */
/* fields be placed in the target object. */

/* The best way to manage this type of list is to use the iterator construct (also defined */
/* below).  Since an example is worth a 1000 words, here's how you could do it: */
#if 0
    Iter<class_type> p;    /* define an iterator for a list of class_type objects */

    /* allocate a list container for class_type objects */
    Dlist_ext<class_type> *list = new Dlist_ext<class_type>;

    list->head( p );    /* set iterator to head of list, which in this case, points */
                        /* to the NULL object because the list is empty */

    /* insert obj1 on list in position after p.  Because p points to the NULL object,
       obj1 will be inserted at the head of the list. */
    p.insert_after( obj1 );

    /* insert obj2 on list in position before p.  Because p points to the NULL object,
       obj2 will be inserted at the tail of the list. */
    p.insert_before( obj2 );

    /* advance p in the forward direction.  Because p points to the NULL object, p
       now points at obj1 (at head of list).  (note: p-- would make it point at
       tail of list (obj2) */
    p++;

    
    p.insert_after( obj3 );   /* list is now:  obj1 obj3 obj2       p=obj1 */
    p.insert_before( obj4 );  /* list is now:  obj4 obj1 obj3 obj2  p=obj1 */

    p1 = p - 1; /* set p1 iterator element before p */

    p+=2;   /* list is now:  obj4 obj1 obj3 obj2  p=obj2  p1=obj4 */

    /* swap elements corresponding to p and p1 */
    p.swap( p1 );   /* list is now:  obj2 obj1 obj3 obj4     p=obj4  p1=obj2 */

    /* remove obj p is pointing to; FALSE argument sets p to previous object
       A TRUE argument (which is the default argument) sets p to next object */
    p.remove( FALSE )      // list is now:  obj4 obj1 obj3  p=obj3

    class_type_obj_ptr = *p;     /* dereference p yields ptr to class_type object associated with iterator */

    /* a typical traversal of a loop */
    for ( list->head( p ); *p; p++ )
        {
        (*p)->some_class_type_function();
        }


    /* Dlist_ext<> allows the same sort of list operations as Dlist<>: */
    list->head_insert( obj );
    list->tail_insert( obj );
    list->remove_all()
    list->destroy_all();
    list->len();
    list->contains( obj );

#endif

template<class T> class Dlist_ext;

template<class T> class Lelem
    {
    /* this class represents the list element object which points to each user */
    /* object that the user wants contained on a list.  This class is used */
    /* in conjunction with the Dlist_ext class described below. */

    protected:

    Lelem *flink;
    Lelem *blink;
    T *obj_ptr;     // pointer to actual item on list

    public:

    Lelem( T *obj )
        {
        flink = NULL;
        blink = NULL;

        obj_ptr = obj;
        }

    ~Lelem()
        {
        /* flag error if this iterator is still on a list */
        assert( !flink );
        assert( !blink );
        }

    T *operator*()
        {
        return obj_ptr;
        }

    T *ptr()
        {
        return obj_ptr;
        }

    void set_ptr( T *p )
        {
        obj_ptr = p;
        }

    Lelem<T> *next()
        {
        return flink;
        }

    void next( Lelem<T> *next_p )
        {
        flink = next_p;
        }

    Lelem<T> *back()
        {
        return blink;
        }

    void back( Lelem<T> *prev_p )
        {
        blink = prev_p;
        }

    };

template<class T> class Iter
    {
    /* this template creates an iterator object that can iterate across */
    /* a list of Lelem<> objects managed by a Dlist_ext<> object */

    protected:
    Dlist_ext<T> *list_ptr;
    Lelem<T> *elem_ptr;


    public:
    Iter<T>()
        {
        list_ptr = NULL;
        elem_ptr = NULL;
        }

	/* copy constructor */
	Iter<T>( const Iter<T> &to_be_copied )
	    {
	    /* the copy constructor is called when:

		    1) object is copied for a call-by-value argument
		    2) returning an object from a function call
		    3) initializing an object from an object of the same type

	    NOTE:  "this" points to copied object

	    */

	    list_ptr = to_be_copied.list_ptr;
	    elem_ptr = to_be_copied.elem_ptr;
	    
	    }


    Lelem<T> *insert_after( T *insert_elem );
    Lelem<T> *insert_before( T *insert_elem );
    void swap( Iter<T> &i );
    T *remove( int advance_iter = TRUE );
    void operator+=( int adv_num );
    void operator-=( int adv_num );

    T *operator*()
        {
        /* return a pointer to the user object that corresponds to */
        /* Lelem object that this iterator is currently pointing to */

        if ( elem_ptr )
            {
			assert( list_ptr );     /* indicates this iterator is properly init'd */
			return elem_ptr->ptr();
			}

        return NULL;
        }

    void operator++()   /* prefix ++ incrementor */
        {
        if ( elem_ptr )
            elem_ptr = elem_ptr->next();
        else list_ptr->head( elem_ptr );   // point at head element
        }

    void operator++(int)   /* postfix ++ incrementor */
        {
        if ( elem_ptr )
            elem_ptr = elem_ptr->next();
        else list_ptr->head( elem_ptr );   /* point at head element */
        }

    void operator--()   /* prefix -- decrementor */
        {
        if ( elem_ptr )
            elem_ptr = elem_ptr->back();
        else list_ptr->tail( elem_ptr );   /* point at tail element */
        }

    void operator--(int)   /* postfix -- decrementor */
        {
        if ( elem_ptr )
            elem_ptr = elem_ptr->back();
        else list_ptr->tail( elem_ptr );   /* point at tail element */
        }

    void set_list_ptr( Dlist_ext<T> *pointer )
        {
        list_ptr = pointer;
        }

	Dlist_ext<T> *list()
		{
		return list_ptr;
		}

    void set_elem_ptr( Lelem<T> *pointer )
        {
        elem_ptr = pointer;
        }

    const Iter<T> &operator=( const Iter<T> &right_hand );
    Iter<T> operator+( int offset )
        {
        Iter<T> p;

        p = *this;
        p += offset;

        return p;
        }

    Iter<T> operator-( int offset )
        {
        Iter<T> p;

        p = *this;
        p -= offset;

        return p;
        }
    };


template<class T> const Iter<T> &Iter<T>::operator=( const Iter<T> &right_hand )
    {

    list_ptr = right_hand.list_ptr;
    elem_ptr = right_hand.elem_ptr;

    return *this;
    }

template<class T> void Iter<T>::operator+=(int adv_num )
    {
    /* advance iterator by specified amount */
    
    if ( adv_num < 0 )
        {
        /* negative count implies advancement in reverse direction */
        operator-=( -adv_num );
        return;
        }

    while( elem_ptr && adv_num-- )
        {
        elem_ptr = elem_ptr->next();
        }
    }

template<class T> void Iter<T>::operator-=(int adv_num )
    {
    /* move iterator backwards by specified amount */

    if ( adv_num < 0 )
        {
        /* negative count implies advancement */
        operator+=( -adv_num );
        return;
        }

    while( elem_ptr && adv_num-- )
        {
        elem_ptr = elem_ptr->back();
        }
    }

template<class T> Lelem<T> *Iter<T>::insert_after( T *insert_elem )
    {
    /* insert "insert_elem" after the specified list element, i.  If */
    /* "list_elem" == NULL, then element is inserted at head of list. */

    assert( list_ptr );

    return list_ptr->insert_after( elem_ptr, insert_elem );
    }

    
template<class T> Lelem<T> *Iter<T>::insert_before( T *insert_elem )
    {
    /* insert "insert_elem" before the specified list element, i.  If */
    /* "list_elem" == NULL, then element is inserted at head of list. */

    assert( list_ptr );

    return list_ptr->insert_before( elem_ptr, insert_elem );
    }

template<class T> T *Iter<T>::remove( int advance_iter )
    {
    /* remove and return list element; make iterator point to next in list if */
    /* advance_iter is TRUE; otherwise make iterator point to previous in list. */

    Lelem<T> *temp;

    assert( list_ptr );
    assert( elem_ptr );

    temp = elem_ptr;

    if ( advance_iter )
        this->operator++();
    else this->operator--();

    return list_ptr->remove( temp );
    }
    

template<class T> void Iter<T>::swap( Iter<T> &i )
    {
    /* swap object pointers for two iterators */

    T *temp;

	assert( elem_ptr );
	assert( i.elem_ptr );

    temp = elem_ptr->ptr();
    elem_ptr->set_ptr( *i );
    i.elem_ptr->set_ptr( temp );
    }

template<class T> class Dlist_ext
    {
    /* this template manages a doubly linked list container whose link */
    /* pointers are held externally in a separate linked list object. */
    /* This structure allows objects to be simultaneously held on multiple */
    /* lists since the pointers are held in separate list objects. */

    protected:

    Lelem<T> *head_elem;
    Lelem<T> *tail_elem;
    int list_len;

    public:

    /* constructor */
    Dlist_ext();

    /* destructor */
    ~Dlist_ext();

    /* function prototypes */
    Lelem<T> *head_insert( T *elem );
    Lelem<T> *tail_insert( T *elem );
    Lelem<T> *insert_after( Lelem<T> *list_elem, T *insert_elem );
    Lelem<T> *insert_before( Lelem<T> *list_elem, T *insert_elem );

    T *head_remove();
    void head_remove_wo_obj();	/* removes head without returning the obj */
    T *tail_remove();
    T *remove( Lelem<T> *&list_elem );
    T *remove( T *elem );
	void append( Dlist_ext<T> *list );
	int find_elem( Iter<T> &it, T *obj );
    void remove_all();              /* remove all elements on list */
    void destroy_all();             /* remove and delete all elements on list */

    T *advance( Lelem<T> *cur, int n );    /* find ptr n elements past cur */
	T *list_elem( int n );		           /* find the nth element on the list */
    int swap( Lelem<T> *elem1, Lelem<T> *elem2 ); /* swap elements on list */
    
    int contains( T *elem );        /* TRUE if list contains element */

    /* inline functions */
    int head( Lelem<T> *&elem )
		{
        if ( head_elem )
			{
			elem = head_elem;
		    return TRUE;
			}

		/* empty list */
		elem = NULL;
        return FALSE;
		}

    void head( Iter<T> &i )
		{

        i.set_list_ptr( this );
        i.set_elem_ptr( head_elem );
		}

	T *head()
		{
		if ( head_elem )
			return head_elem->ptr();

		return NULL;
		}

    int tail( Lelem<T> *&elem )
		{
        if ( tail_elem )
			{
			elem = tail_elem;
    		return TRUE;
			}

		/* empty list */
		elem = NULL;
        return FALSE;
		}

    void tail( Iter<T> &i )
		{

        i.set_list_ptr( this );
        i.set_elem_ptr( tail_elem );
		}

	T *tail()
		{
		if ( tail_elem )
			return tail_elem->ptr();

		return NULL;
		}

    int len()
		{
		return list_len;    /* return length of entire list */
		}


    int len( Lelem<T> *cur )
		{
		/* return number of elements on list starting at cur */

		int i;

		for ( i = 0; cur; cur = cur->next() )
			i++;

		return i;
		}
    };

#if 0
template<class T> class Dnamelist : public Dlist_ext<T>
    {
    /* this template is derived from the Dlist_ext template and so manages */
    /* a doubly linked list container in the same way as Dlist_ext<T>.  In */
    /* addition, this class maintains a character string field so that */
    /* the elements can be inserted alphabetically and searched based on */
    /* the name */

    private:
    int name_offset;

    public:
    Dnamelist( T *p, T *&next, T *&back, char *&name );
    Dnamelist( T *p, T **next, T **back, char **name );
#define DNAMELIST(class,next,back,name) Dnamelist<class>((class*)0,&(((class*)0)->next),&(((class*)0)->back),&(((class*)0)->name))

    int alpha_insert( T *elem );           // insert elem alphabetically
    T *find_name( char *search_name );     // find elem by its name
	T *find_name( char *search_name, T *&prev_name );

    char *name( T *cur )
		{
		/* return the string name associated with this element */
		char **p;

		p = (char **) cur + name_offset;

		return *p;
		}


    };

template<class T> class Dsortlist : public Dlist_ext<T>
    {
    /* this is an extention of the Dlist_ext template similar to Dnamelist
       which manages a doubly linked list container in the same way as
       Dlist_ext<T>. In addition, this class allows elements to be inserted
       in sorted order.

       The class being contained must have a >= operator defined similar to

       class T {
       public:
           friend int operator>= (const T& first, const T& second);
           // this operator returns 0 if first==second in sorted order
           //                      >0 if first occurs after second in order
           //                      <0 if first occurs before second
     */
public:
    Dsortlist(T *p, T *&next, T *&back);
    Dsortlist(T *p, T **next, T **back);
#define DSORTLIST(class,next,back) Dsortlist<class>((class*)0,&(((class*)0)->next),&(((class*)0)->back))

    int sorted_insert( T *elem );
    };
    
#endif

/********************************************************************************/

/* constructor */
template<class T> Dlist_ext<T>::Dlist_ext()
    {
    head_elem = tail_elem = NULL;
    list_len = 0;
    }

/* destructor */
template<class T> Dlist_ext<T>::~Dlist_ext()
    {
    remove_all();
    }

template<class T> Lelem<T> *Dlist_ext<T>::head_insert( T *elem )
    {
    /* add specified element to head of list */
    /* NOTE:  it is assumed that specified element is not already */
    /*        on a list (no checking is done to verify this) */

    Lelem<T> *list_elem;

    /* create and associate an iterator to the specified object */
    list_elem = new Lelem<T>( elem );

    list_elem->next( head_elem );
    list_elem->back( NULL );

    if ( head_elem )
		head_elem->back( list_elem );
    else tail_elem = list_elem;       // first list element inserted

    head_elem = list_elem;            // new head_elem

    list_len++;    

    return head_elem;
    }

template<class T> Lelem<T> *Dlist_ext<T>::tail_insert( T *elem )
    {
    /* add specified element to tail of list */
    /* NOTE:  it is assumed that specified element is not already */
    /*        on a list (no checking is done to verify this) */

    Lelem<T> *list_elem;

    /* create and associate an iterator to the specified object */
    list_elem = new Lelem<T>( elem );

    list_elem->next( NULL );
    list_elem->back( tail_elem );

    if ( tail_elem )
		tail_elem->next( list_elem );
    else head_elem = list_elem;       // first list element inserted

    tail_elem = list_elem;            // new tail_elem

    list_len++;    

    return tail_elem;
    }

template<class T> Lelem<T> *Dlist_ext<T>::insert_after( Lelem<T> *list_elem, T *insert_elem )
    {
    /* insert "insert_elem" after the specified list element, list_elem.  If */
    /* "list_elem" == NULL, then element is inserted at head of list. */

    Lelem<T> *insert_i;

    if ( list_elem == NULL )
		return head_insert( insert_elem );
    else if ( list_elem == tail_elem )
		return tail_insert( insert_elem );
    else
		{
		/* insert in middle of list */

        /* create and associate an iterator to the specified object */
        insert_i = new Lelem<T>( insert_elem );
		
        insert_i->next( list_elem->next() );
        insert_i->next()->back( insert_i );
        list_elem->next( insert_i );
        insert_i->back( list_elem );

		list_len++;
		}

    return insert_i;
    }

    
template<class T> Lelem<T> *Dlist_ext<T>::insert_before( Lelem<T> *list_elem, T *insert_elem )
    {
    /* insert "insert_elem" before the specified list element, list_elem.  If */
    /* "list_elem" == NULL, then element is inserted at tail of list. */

    Lelem<T> *insert_i;

    if ( list_elem == NULL )
		return tail_insert( insert_elem );
    else if ( list_elem == head_elem )
		return head_insert( insert_elem );
    else
		{
		/* insert in middle of list */

        /* create and associate an iterator to the specified object */
        insert_i = new Lelem<T>( insert_elem );
		
        insert_i->back( list_elem->back() );
        insert_i->back()->next( insert_i );
        list_elem->back( insert_i );
        insert_i->next( list_elem );

		list_len++;
		}

    return insert_i;
    }

    
template<class T> T *Dlist_ext<T>::remove( T *elem )
    {
    /* remove list_elem from list */
    /* NOTE:  it is assumed that specified element is on */
    /*        the list corresponding to Dlist_ext<T> (no checking */
    /*        is done to verify this) */

	Iter<T> it;

	assert( elem );

    find_elem( it, elem );

	assert( *it );	/* must be non-null */

	it.remove();

	return elem;
	}

template<class T> T *Dlist_ext<T>::remove( Lelem<T> *&list_elem )
    {
    /* remove list_elem from list */
    /* NOTE:  it is assumed that specified element is on */
    /*        the list corresponding to Dlist_ext<T> (no checking */
    /*        is done to verify this) */

    T *p;

    if ( list_elem == head_elem )
		return head_remove();
    else if ( list_elem == tail_elem )
		return tail_remove();
    else
		{
		/* element is in middle of list */
		list_elem->back()->next( list_elem->next() );
        list_elem->next()->back( list_elem->back() );

        list_elem->next( NULL );
        list_elem->back( NULL );
        p = list_elem->ptr();

        /* delete list element and NULL pointer */
        delete list_elem;
        list_elem = NULL;
        
		list_len--;
		}

    return p;
    }

template<class T> T *Dlist_ext<T>::head_remove()
    {
    /* remove element from head of list */

    T *p;
    Lelem<T> *list_elem;

    if ( head_elem == NULL )
		return NULL;    // list is empty

    if ( head_elem->next() )
		{
		/* list has more than 1 element on it */
		list_elem = head_elem;                  // head elem to return
        head_elem = head_elem->next();
        head_elem->back( NULL );

        list_elem->next( NULL );

        p = list_elem->ptr();
        delete list_elem;

        list_len--;
		}
    else
		{
		/* only 1 element on list */
		p = head_elem->ptr();
        delete head_elem;
		head_elem = tail_elem = NULL;
		list_len = 0;
		}

    return p;
    }

template<class T> void Dlist_ext<T>::head_remove_wo_obj()
    {
    /* remove element from head of list; don't return the corresponding object */

    Lelem<T> *list_elem;

    if ( head_elem == NULL )
		return;    /* list is already empty */

    if ( head_elem->next() )
		{
		/* list has more than 1 element on it */
		list_elem = head_elem;                  /* head elem to return */
        head_elem = head_elem->next();
        head_elem->back( NULL );

        list_elem->next( NULL );

        delete list_elem;

        list_len--;
		}
    else
		{
		/* only 1 element on list */
        delete head_elem;
		head_elem = tail_elem = NULL;
		list_len = 0;
		}

    return;
    }

template<class T> T *Dlist_ext<T>::tail_remove()
    {
    /* remove element from tail of list */

    T *p;
    Lelem<T> *list_elem;

    if ( tail_elem == NULL )
		return NULL;    /* list is empty */

    if ( tail_elem->back() )
		{
		/* list has more than 1 element on it */
		list_elem = tail_elem;                  /* head elem to return */
        tail_elem = tail_elem->back();
        tail_elem->next( NULL );

        list_elem->back( NULL );

        p = list_elem->ptr();
        delete list_elem;

        list_len--;
		}
    else
		{
		/* only 1 element on list */
		p = tail_elem->ptr();
        delete tail_elem;
		head_elem = tail_elem = NULL;
		list_len = 0;
		}

    return p;
    }


template<class T> int Dlist_ext<T>::find_elem( Iter<T> &it, T *obj )
	{
	/* set the iterator to the list elem containing obj */

	assert( obj );

	for ( head( it ); *it; it++ )
		{
		if ( *it == obj )
			return TRUE;
		}

	return FALSE;	/* obj not found on list */
	}



template<class T> void Dlist_ext<T>::append( Dlist_ext<T> *list )
	{
	/* append the "list" to the end of the "this" list */

	if ( list->head_elem == NULL || this == list )
		return;

	if ( head_elem == NULL )
		{
		head_elem = list->head_elem;
		tail_elem = list->tail_elem;
		list_len = list->list_len;

		list->head_elem = NULL;
		list->tail_elem = NULL;
		list->list_len = 0;

		return;
		}

	list_len += list->len();
	list->head_elem->back( tail_elem );
	tail_elem->next( list->head_elem );
	tail_elem = list->tail_elem;

	list->head_elem = list->tail_elem = NULL;
	list->list_len = 0;
	}



template<class T> void Dlist_ext<T>::remove_all()
    {
    /* remove all elements from list */

    while( head_elem )
		head_remove_wo_obj();
    }

template<class T> void Dlist_ext<T>::destroy_all()
    {
    /* remove all elements from list and delete each element removed */

    T *p;

    while( head_elem )
		{
		p = head_remove();

		delete p;
		}
    }

template<class T> T *Dlist_ext<T>::advance( Lelem<T> *cur, int n )
    {
    /* return a ptr to element on list "n" away from cur element */
    /* If n < 0, list is traversed backwards, otherwise, forwards. */
    /* Returning a NULL pointer means that specified element did not exist */

    if ( n >= 0 )
		{
		/* positive n means follow list forward */
		while( n && cur )
			{
			cur = cur->next();
			n--;
			}
		}
    else
		{
		/* negative n means follow list backward */
		while( n && cur )
			{
			cur = cur->back();
			n++;
			}
		}

    return cur->ptr;
    }

template<class T> T *Dlist_ext<T>::list_elem( int n )
    {
	/* find the nth element on the list */

	Lelem<T> *elem;

	if ( n <= 0 )
		return NULL;

	for ( n--, elem = head_elem; n && elem; n--, elem = elem->next() )
		;

	if ( elem )
        return elem->ptr();
    else
        return NULL;
    }

#if 0
template<class T> T *Dlist_ext<T>::list_elem( Lelem<T> *base_elem, int rel_offset )
    {
	/* find the nth element on the list, relative to the base_elem */
	/* A negative relative offset means that you search from the base_elem */
	/* towards the head of the list, otherwise, search toward the tail */

	Lelem<T> *elem;

	if ( rel_offset >= 0 )
		{
		for ( elem = base_elem; rel_offset && elem; rel_offset--, elem = elem->next() )
			;
		}
	else
		{
		for ( elem = base_elem; rel_offset && elem; rel_offset++, elem = elem->back() )
			;
		}

	return elem->ptr();
    }
#endif

template<class T> int Dlist_ext<T>::contains( T *test_elem )
    {
    Lelem<T> *elem;

    for ( elem = head_elem; elem; elem = elem->next() )
        {
        if ( test_elem == elem->ptr() )
            return 1;
        }
    return 0;
    }
    
template<class T> int Dlist_ext<T>::swap( Lelem<T> *elem1, Lelem<T> *elem2 )
    {
    /* swap the 2 elements on the list.  This function works */
    /* by swapping the pointers of the Lelem object...the Lelem objects */
    /* themselves are not swapped */

    T *temp;

    assert( elem1 );
    assert( elem2 );

    if ( elem1 == elem2 )
		return 1;        // same element specified

    temp = elem1->ptr;
    elem1->ptr = elem2->ptr;
    elem2->ptr = temp;

    return 1;
    }

#if 0
/* constructor */
template<class T> Dnamelist<T>::Dnamelist( T *p, T *&next, T *&back, char *&name ) :
		  Dlist_ext<T>( p, next, back )
    {
    /* find the offsets from the start of the object structure */
    /* for the next and back link pointers */
    name_offset = &name - (char **) p;
    }

/* constructor */
template<class T> Dnamelist<T>::Dnamelist( T *p, T **next, T **back, char **name ) :
		  Dlist_ext<T>( p, next, back )
    {
    /* find the offsets from the start of the object structure */
    /* for the next and back link pointers */
    name_offset = name - (char **) p;
    }


template<class T> T *Dnamelist<T>::find_name( char *search_name )
    {
    /* find the first element on the list with the specified name */

    T *p;
    int i;

    for ( p = head(); p; p = next( p ) )
		{
		i = strcmp( name( p ), search_name );
		if ( i == 0 )
			return p;

		if ( i > 0 )
			break;
		}

    return NULL;    // not found
    }

template<class T> T *Dnamelist<T>::find_name( char *search_name, T *&prev_name )
    {
    /* Find and return the first element on the list with the specified name */
	/* Also pass back the element just prior to where the search element was found */
	/* or would have been found */

    T *p;
    int i;

	prev_name = NULL;
    for ( p = head(); p; p = next( p ) )
		{
		i = strcmp( name( p ), search_name );
		if ( i == 0 )
			return p;

		if ( i > 0 )
			break;

			prev_name = p;
		}

    return NULL;    // not found
    }

template<class T> int Dnamelist<T>::alpha_insert( T *elem )
    {
    /* insert the element into the list in the proper alphabetical position */
    /* return TRUE if the inserted name is unique on the list */

    T *p, *last_p;
    int i;

    last_p = NULL;
    for ( p = head(); p; p = next( p ) )
		{
		i = strcmp( name( p ), name( elem ) );
		if ( i >= 0 )
			break;

		last_p = p;
		}

    if ( last_p == NULL )
		head_insert( elem );
    else insert_after( last_p, elem );

    if ( p )
		return !!i;
    
    return 1;;
    }

/*******************************************************************************/

template<class T> Dsortlist<T>::Dsortlist(T *p, T *&next, T *&back) :
    Dlist_ext<T>(p, next, back)
{}
template<class T> Dsortlist<T>::Dsortlist(T *p, T **next, T **back) :
    Dlist_ext<T>(p, next, back)
{}

template<class T> int Dsortlist<T>::sorted_insert( T *elem )
    {
    /* insert the element into the list in the proper sorted order */
    /* return TRUE if the inserted name is unique on the list */

    T *p, *last_p;
    int i;

    last_p = NULL;
    for ( p = head(); p; p = next( p ) )
		{
		i = (*p >= *elem);
		if ( i >= 0 )
			break;

		last_p = p;
		}

    if ( last_p == NULL )
		head_insert( elem );
    else
        insert_after( last_p, elem );

    if ( p )
		return !!i;
    
    return 1;;
    }

#endif
#endif			// __cplusplus

#endif			// LST_UTIL_DEFINED
