/* sram_man.cpp
 * SA1200 Hardware Absraction Library -- SRAM Manager
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 *
 *  system: SA1200
 *  subsystem: SRAM
 *  author: dfh, Mar 4,99  Initial version
 *  revisions:
 *  $History: sram_man.cpp $
 * 
 * *****************  Version 4  *****************
 * User: Dhooper      Date: 6/02/99    Time: 4:28p
 * Updated in $/dev_1200/Utils
 * add SRAN_BZero, Scratch_BZero
 * 
 * *****************  Version 3  *****************
 * User: Dhooper      Date: 5/17/99    Time: 1:38p
 * Updated in $/BL5_Plus/Utils
 * change SRAM_Init to SRAM_Attach
 * 
 * *****************  Version 2  *****************
 * User: Dhooper      Date: 5/13/99    Time: 11:26a
 * Updated in $/dev_1200/Utils
 * add sram_man and IOSTYLE for hardware and ambaio
 * 
 * *****************  Version 1  *****************
 * User: Dhooper      Date: 5/13/99    Time: 10:14a
 * Created in $/dev_1200/Utils
 * 
 * *****************  Version 7  *****************
 * User: Dhooper      Date: 5/11/99    Time: 5:23p
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * don't increment or decrement pointer, fix  to compare id to MAX_QUEUES,
 * not qty
 * 
 * *****************  Version 6  *****************
 * User: Dhooper      Date: 5/11/99    Time: 9:37a
 * Updated in $/dev_1200/SA1_CoreLibs/hal_1200
 * add UCODE_QUEUES_MASK
 * 
 * *****************  Version 4  *****************
 * User: C-tclancy    Date: 3/23/99    Time: 3:41p
 * Updated in $/development/hal_1200
 * update COPYRIGHT
 * 
 * *****************  Version 3  *****************
 * User: Dhooper      Date: 3/08/99    Time: 11:28a
 * Updated in $/development/hal_1200
 * intitalize ptr
 * 
 * *****************  Version 2  *****************
 * User: Dhooper      Date: 3/04/99    Time: 11:56a
 * Updated in $/development/hal_1200
 * size is in words
 * 
 * *****************  Version 1  *****************
 * User: Dhooper      Date: 3/02/99    Time: 6:51p
 * Created in $/development/hal_1200
 * sram manager, push pop freelist
 * 
 * ---------------------------------------------------------------------
 */
#include "sram_man.h"
#include "mem_map.h"
#include "hal_fbi.h"

/*-----------------------------------------------------------------------------
   Function:	SRAM_BZero
   Description: zero len sram memory longwords starting at loc
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void SRAM_BZero(int loc, int len)
{
	int i;

	for(i=0;i<len;i++){
		SRAM_Write(loc+i, 0);
	}

	return;
}

/*-----------------------------------------------------------------------------
   Function:	Scratch_BZero
   Description: zero len scratch memory longwords starting at loc
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
void Scratch_BZero(int loc, int len)
{
	int i;

	for(i=0;i<len;i++){
		Scratch_Write(loc+i, 0);
	}

	return;
}


#if ((IOSTYLE==HARDWARE)||(IOSTYLE==AMBAIO))

/*-----------------------------------------------------------------------------
   Function:	SFreeList::SFreeList
   Description: constructor
				set init_vector to 0
   Returns:
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
SFreeList::SFreeList()
{
	sram = SRAM_Attach();
	init_vector = 0;
}



/*-----------------------------------------------------------------------------
    Function:	 SFreeList::init
    Description: initialize a push pop queue by performing push qty times
				 thus creating a linked list of sram buffers of buf_size(in lwords)
				 starting at sram address base (lword offset from 0). The first word 
				 of each item on list is a pointer to the next item of the list. The first
				 word of the last item will be 0.
    Returns:	SF_SUCCESS if q was not previously initialized and hardware q exists by given id.
				Note: there are only 8 hardware queues, ids 0-7 in SA1200 first chip
				SF_Q_NOT_EXIST if id is greater than or equal to SF_MAX_QUEUES
				SF_Q_PREV_INIT if q was previously initialized by core
				SF_Q_UCODE_INIT if q is to be initialized by ucode
    Uses:		type: SRAM controller
				      AMBAIO interface if not HARDWARE
    Modifies:	push_pop_regs[id] and sram pointer locations
-----------------------------------------------------------------------------*/
int SFreeList::init(unsigned char id, unsigned int base, unsigned int buf_size, unsigned int qty)
{
	unsigned int i;
	unsigned char id_vector = 1<<id;

	if (id>=SF_MAX_QUEUES) return(SF_Q_NOT_EXIST);
	if ((init_vector & id_vector) >0) return(SF_Q_PREV_INIT);
	if ((UCODE_QUEUES_MASK & id_vector) >0) return(SF_Q_UCODE_INIT);

	for(i=base; i<qty; i=i+buf_size){		// setup the linked list
		sram->push(id, i);
	}
	sram->write(base, 0);				// terminate the list with a null
	init_vector = init_vector | id_vector;

	return(SF_SUCCESS);
}



/*-----------------------------------------------------------------------------
   Function:	SFreeList::alloc
   Description: pop a pointer to an sram buffer from a push-pop queue
   Returns:		sram address of start of buffer, or 0 if no buffer available
   Uses:		type: SRAM controller
				      AMBAIO interface
   Modifies:	push_pop_regs[id] to point to the next buffer
-----------------------------------------------------------------------------*/
unsigned int SFreeList::alloc(unsigned char id)
{
	unsigned int ptr;

	sram->pop(id, &ptr);
	return(ptr);
}



/*-----------------------------------------------------------------------------
   Function:	SFreeList::free
   Description: push a pointer to an sram buffer to a push-pop queue
   Returns:		SUCCESS
				if this fails, the sram controller hardware has crashed
   Uses:		type: SRAM controller
				      AMBAIO interface
   Modifies:	push_pop_regs[id] to point to the next buffer
-----------------------------------------------------------------------------*/
void SFreeList::free(unsigned char id, unsigned int ptr)
{
	sram->push(id, ptr);
	return;
}

#endif		//((IOSTYLE==HARDWARE)||(IOSTYLE==AMBAIO))
