/* utl_str.c
 * String Utility
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
 * system: IXP1200
 * subsystem: Utility Library
 * author: drj Aug 17, 2000 Initial version
 * revisions:
 * $History: utl_str.c $
 * 
 * *****************  Version 3  *****************
 * User: Drjohns1     Date: 8/30/00    Time: 2:10p
 * Updated in $/dev_1200/Utils
 * Added string utilities
 * 
 * *****************  Version 2  *****************
 * User: Drjohns1     Date: 8/18/00    Time: 10:42a
 * Updated in $/dev_1200/Utils
 * Added utl_str.c
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 8/17/00    Time: 2:25p
 * Created in $/dev_1200/Utils
 * Utility string functions
 * 
 * ---------------------------------------------------------------------
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "utl_str.h"


/*-----------------------------------------------------------------------------
   Function: utl_stripLeadBlanks	
   Description: Remove leading blanks or tabs from string
   Returns: pointer to the string
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *utl_stripLeadBlanks(char *str)
{
   char *c;

   for (c = str; c && *c == '\t' || *c == ' '; c++) ;
   return (c);
}


/*-----------------------------------------------------------------------------
   Function: utl_stripTrailBlanks	
   Description: Remove trailing blanks or tabs from string
   Returns: pointer to the string
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *utl_stripTrailBlanks(char *str)
{
   char *c;

   for (c = str + strlen(str); c != str && *c == '\t' || *c == ' '; c--) ;
   return (c);
}


/*-----------------------------------------------------------------------------
   Function: utl_stripTrailBlanks	
   Description: Remove leading and trailing blanks or tabs from string
   Returns: pointer to the string
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *utl_stripBlanks(char *str)
{
   return (utl_stripTrailBlanks(utl_stripLeadBlanks(str)));
}


/*-----------------------------------------------------------------------------
   Function: utl_strLowcase	
   Description: Sets string to lowercase.
   Returns: pointer to the string
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *utl_strLowcase(char *s)
{
   char *cs;
   for(cs = s; cs && *cs; cs++)
      *cs = tolower(*cs);
   return(s);
}

/*-----------------------------------------------------------------------------
   Function: utl_strUpcase	
   Description: Sets string to uppercase.
   Returns: pointer to the string
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *utl_strUpcase(char *s)
{
   char *cs;
   for(cs = s; cs && *cs; cs++)
      *cs = toupper(*cs);
   return(s);
}


/*-----------------------------------------------------------------------------
   Function: 	utl_strtoken
   Description: Extracts the token between the specified delimiters.
   Returns:	The pointer to the token, or NULL.
   Uses:
   Modifies:
-----------------------------------------------------------------------------*/
char *utl_strtoken(char *str, char *dlim, char **savPtr)
{
   char *begStr, *s;

   /* if str is NULL, then use the last reference pointer */
   if (!str) {
      if (savPtr && *savPtr)
	 str = *savPtr;
      else
	 return (NULL);
   }

   if (savPtr)
      *savPtr = NULL;

   /* locate the first byte of str that does not occur in dlim */
   for (begStr = s = str; *s; s++) {
      if (!(strchr(dlim, *s))) {
	 begStr = s;
	 break;
      }
   }

   /* locate the first byte of str that does occur in dlim */
   for (; *s; s++) {
      if ((strchr(dlim, *s))) {
	 *s = '\0';
	 if (savPtr)
	    *savPtr = s + 1;
	 break;
      }
   }

   return (begStr);
}

