/* TransactorInterface.cpp
 *
 * ---------------------------------------------------------------------	
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 */

static const char __copyright[]=
        "Copyright Intel Corp, 1997-1998. All rights reserved";

static const char __TransactorInterface_version__ []=
        "\nFILEINFO: $Workfile: XactInterface.cpp $ $Date: 3/23/99 4:16p $ $Revision: 4 $\n";

#include "stdafx.h"
#include "XactInterface.h"

// The one and only CXactInterface object
CXactInterface theXactInterface;

char*
XactCmdCallback()
{
    return theXactInterface.CommandCallback();
}

void
XactOutputCallback(
    const char *pMessage)
{
    theXactInterface.OutputCallback(pMessage);
}

void
XactPromptCallback(
    char *pPrompt)
{
    theXactInterface.PromptCallback(pPrompt);
}

DWORD WINAPI 
ThreadStub(
    CXactInterface *pTransactorInterface)
{
    return pTransactorInterface->ThreadFunction();
}

#if defined(__cplusplus)
extern "C" {
#endif

/*-----------------------------------------------------------------
    foreign_model_initialize

    This function will be called to initialize the foreign model after
    the transactor "init" command has successfully executed.  Returning 0
    will result in a transactor error.
    
     returns: 
        uses: 
    modifies: 
*/
int foreign_model_initialize()
{
	return(1);
}

/*-----------------------------------------------------------------
    foreign_model_pre_sim

    This function will be called prior to each transactor simulation event.
    It can be used to deposit state values into the transactor model prior
    to simulating the next event.  Returning 0 results in an error.
    
     returns: 
        uses: 
    modifies: 
*/
int foreign_model_pre_sim()
{
	return(1);
}

/*-----------------------------------------------------------------
    foreign_model_post_sim

    This function will be called subsequent to each transactor simulation event.
    It can be used to query transactor simulation state, in order to copy it
    into the foreign model simulator.
    
     returns: 
        uses: 
    modifies: 
*/
int foreign_model_post_sim()
{
	return(1);
}

/*-----------------------------------------------------------------
    foreign_model_exit

    This function will be called just prior to exiting the simulator.
    The routine allows the foreign model to clean up, close files, etc
    before shutting down the program.

     returns: 
        uses: 
    modifies: 
*/
int foreign_model_exit()
{
    return(1);
}

#if defined(__cplusplus)
}
#endif

///////////////////////////////////////////////////////////////////////////////
///

CXactInterface::CXactInterface() :
        m_Prompt(">")
{
    InitializeCriticalSection(&m_Critical);

    // Create event used to signal messages to the thread.
    m_hThreadSignal = CreateEvent(NULL,    // Security
                                  FALSE,   // FALSE == Automatic Reset
                                  FALSE,   // Initially not signaled
                                  NULL);   // Un-named
    if (m_hThreadSignal == NULL)
        AfxMessageBox("Could not create ThreadSignal Event");

    StartThread();
}

CXactInterface::~CXactInterface()
{
    DeleteCriticalSection(&m_Critical);
    CloseHandle( m_hThreadSignal );
    m_hThreadSignal = NULL;
}

bool 
CXactInterface::StartThread( 
    int ThreadPriority)
{
    DWORD ThreadID;
    m_hThread = CreateThread(NULL,
                             0,    // default stack
                             (LPTHREAD_START_ROUTINE) ThreadStub,
                             this,
                             0,    // creation flags
                             &ThreadID);
    if (m_hThread == NULL)
    {
        // Failed to create thread so give error return;
        AfxMessageBox("Failed to create thread");
        return false;
    }
    else 
    {
        // Success, now set priority
        SetThreadPriority(m_hThread, ThreadPriority);
        return true;
    }
}

void
CXactInterface::SetOutputWindow(
    HWND hOutputWindow)
{
    m_hOutputWindow = hOutputWindow;
}

void
CXactInterface::ExecuteCommand(
    const CString& CommandLine)
{
    EnterCriticalSection(&m_Critical);
    m_CommandLine = CommandLine;
    SetEvent(m_hThreadSignal);
    LeaveCriticalSection(&m_Critical);
}

DWORD
CXactInterface::ThreadFunction()
{
    XACT_initialize();
    XACT_Define_Callback_Output_Message(XactOutputCallback);
	XACT_Define_Callback_Set_Prompt(XactPromptCallback);
    XACT_Define_Callback_Get_Console_Input(XactCmdCallback);

    XACT_start_console();
    return 1;
}

char*
CXactInterface::CommandCallback()
{
    static bFirstTime = true;

    if (!bFirstTime)
        LeaveCriticalSection(&m_Critical);
    else
        bFirstTime = false;

    if (WAIT_OBJECT_0 != WaitForSingleObject(m_hThreadSignal, INFINITE))
        AfxMessageBox("WaitForSingleObject failed");

    EnterCriticalSection(&m_Critical);
    // Log the prompt and command in the output window.
    LogCommand(m_CommandLine);

    return m_CommandLine.GetBuffer(1);
}

void
CXactInterface::LogCommand(
    const CString& CommandLine)
{
    // Concatenate prompt and command and send result to be displayed in
    // output window.
    CString *pTemp = new CString(m_Prompt + CommandLine);
    // Make sure line ends in '\n'.
    if (pTemp->GetAt(pTemp->GetLength() - 1) != '\n')
        *pTemp += '\n';
    // Post the message rather than send it because message pump isn't running
    // because the other thread is waiting for the done signal.
    PostMessage(m_hOutputWindow, WM_REPORT_TRANSACTOR_OUTPUT, 
                (WPARAM)pTemp, (LPARAM)0);
}

// This method is called by the transactor when it changes the command line 
// prompt.  Therefore, it executes in the transactor execution thread.
void
CXactInterface::PromptCallback(
    char *pPrompt)
{
    m_Prompt = pPrompt;
    // Copy the prompt into a string that will get deleted by the function
    // that handles the message.
    CString *pTemp = new CString(m_Prompt);
    // Post the message rather than send it because message pump isn't running
    // because the other thread is waiting for the done signal.
    PostMessage(m_hOutputWindow, WM_REPORT_TRANSACTOR_PROMPT, 
                (WPARAM)pTemp, (LPARAM)0);
}

// This method is called indirectly by the transactor when it has command 
// execution output.  Therefore, it executes in the transactor execution 
// thread.
void
CXactInterface::OutputCallback(
    const char *pOutput)
{
    // Copy the output into a string that will get deleted by the function
    // that handles the message.
    CString *pTemp = new CString(pOutput);
    // Post the message rather than send it because message pump isn't running
    // because the other thread is waiting for the done signal.
    PostMessage(m_hOutputWindow, WM_REPORT_TRANSACTOR_OUTPUT, 
                (WPARAM)pTemp, (LPARAM)0);
}

