/*
 * ---------------------------------------------------------------------	
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 * ---------------------------------------------------------------------
 *
 */

#include "stdafx.h"
#include "XactInterfaceApp.h"
#include "XactInterfaceDlg.h"
#include "XactInterface.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CXactInterfaceDlg dialog


CXactInterfaceDlg::CXactInterfaceDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CXactInterfaceDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CXactInterfaceDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CXactInterfaceDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CXactInterfaceDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CXactInterfaceDlg, CDialog)
	//{{AFX_MSG_MAP(CXactInterfaceDlg)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
    ON_MESSAGE(WM_REPORT_TRANSACTOR_OUTPUT, OnReportTransactorOutput)
    ON_MESSAGE(WM_REPORT_TRANSACTOR_PROMPT, OnReportTransactorPrompt)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CXactInterfaceDlg message handlers

int CXactInterfaceDlg::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CDialog::OnCreate(lpCreateStruct) == -1)
		return -1;
	
    CRect Rect;
    GetClientRect(&Rect);
    int CommandEditHeight = 22;
    int OutputEditHeight = Rect.Height() - CommandEditHeight;
    int PromptWidth = 80;
    if (!m_CommandEdit.Create(ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_MULTILINE |
                              ES_WANTRETURN | ES_LEFT | WS_CHILD | WS_VISIBLE |
                              WS_BORDER,
                              CRect(PromptWidth + 2, OutputEditHeight, 
                              Rect.Width(), 
                              OutputEditHeight + CommandEditHeight), 
                              this, 
                              IDC_CMDLINE_EDIT))
        return 1;
    if (!m_OutputEdit.Create(WS_HSCROLL | WS_VSCROLL | ES_MULTILINE |
                             ES_READONLY | ES_LEFT | WS_CHILD | WS_VISIBLE |
                             WS_BORDER, 
                             CRect(0, 0, Rect.Width(), OutputEditHeight), 
                             this, 
                             IDC_CMDLINE_OUTPUT))
        return 1;
    if (!m_Prompt.Create(">0", SS_RIGHT | WS_CHILD | WS_VISIBLE, 
                         CRect(0, OutputEditHeight + 2, PromptWidth, 
                         OutputEditHeight + CommandEditHeight), 
                         this))
        return 1;
	
    theXactInterface.SetOutputWindow(GetSafeHwnd());

	return 0;
}

LRESULT
CXactInterfaceDlg::OnReportTransactorOutput(
    WPARAM  WParam, 
    LPARAM  LParam)
{
    // We need to convert any occurrences of '\n' by itself into '\r\n', as
    // required by the edit control.
    CString *pMsg = (CString*)WParam;
    int Length = pMsg->GetLength();
    // Assume worst case of all '\n' chars.  Also, if string has only one
    // char, allow for appending "\r\n\0".
    char *ConvertedMsg = new char[2*Length + 2];
    int j = 0;
    for (int i = 0; i < Length; i++) 
    {
        if (pMsg->GetAt(i) == '\n')
            if ((i == 0) || (pMsg->GetAt(i-1) != '\r'))
                ConvertedMsg[j++] = '\r';
        ConvertedMsg[j++] = pMsg->GetAt(i);
    }

    // Terminate the string.
    ConvertedMsg[j] = '\0';

    // Check if we've exceeded the maximum number of lines in the output
    // window.
    int ExcessLines = m_OutputEdit.GetLineCount() - 2000;
    if (ExcessLines > 0)
    {
        // Max is exceeded, so delete the excessive lines from the beginning
        // of the buffer.
        m_OutputEdit.SetSel(m_OutputEdit.LineIndex(0), 
                            m_OutputEdit.LineIndex(ExcessLines), 
                            TRUE);  // Don't scroll to the selected lines.
        m_OutputEdit.ReplaceSel("");
    }

    // Ensure that the insertion point is at the end of the current text so
    // that the new text gets inserted correctly.  Doing this is way too
    // cumbersome than it should be, but it's the only way I could figure
    // out how to do it.
    // Get the character index for the first character in the last line.
    int LastLineIndex = m_OutputEdit.GetLineCount() - 1;
    // Now compute the index for the last character.
    int CharsInLine = m_OutputEdit.LineLength(LastLineIndex);
    int LastCharIndex = m_OutputEdit.LineIndex(LastLineIndex) + CharsInLine;
    // Set the insertion point by selecting at the last character.
    m_OutputEdit.SetSel(LastCharIndex, LastCharIndex); 

    // Insert the new text.
    m_OutputEdit.ReplaceSel(ConvertedMsg);
    delete [] ConvertedMsg;
    delete pMsg;
	return (LRESULT)0;
}

LRESULT
CXactInterfaceDlg::OnReportTransactorPrompt(
    WPARAM  WParam, 
    LPARAM  LParam)
{
    CString *pPrompt = (CString *)WParam;

    // Display the new prompt.
    m_Prompt.SetWindowText(*pPrompt);
    delete pPrompt;
    return (LRESULT)0;
}

/////////////////////////////////////////////////////////////////////////////
// CCommandEdit

IMPLEMENT_DYNAMIC(CCommandEdit, CEdit);

CCommandEdit::CCommandEdit()
{
}

CCommandEdit::~CCommandEdit()
{
}


BEGIN_MESSAGE_MAP(CCommandEdit, CEdit)
	//{{AFX_MSG_MAP(CCommandEdit)
	ON_WM_CHAR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCommandEdit message handlers

void 
CCommandEdit::OnChar(
    UINT nChar, 
    UINT nRepCnt, 
    UINT nFlags) 
{
    if (nChar == VK_RETURN) {
        CString Command;
        GetWindowText(Command);
        if (Command.GetLength() > 0) {
            theXactInterface.ExecuteCommand(Command);
            SetWindowText("");
            return;
        }
    }

    CEdit::OnChar(nChar, nRepCnt, nFlags);
}

