/*----------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 ***************************************************************************
 * $Log: /dev_1200/osApi/iostyle.h $
 * 
 * 1     7/24/00 5:56p Drjohns1
 * 
 * 1     4/23/99 3:37p Jdguilfo
 * Initial version
*/
#ifndef IOSTYLE_H
#define IOSTYLE_H

/*************************************************************************
 * This file is used to define three IO styles:
 * XACTIO   : model magic
 * AMBAIO   : using the AMBA bus within the transactor
 * HARDWARE : talking to real hardware
 *
 * Files that do IO should include this file and then conditionalize their
 * code along the lines of:
 * 
 * #if (IOSTYLE == XACTIO)
 * ...
 * #elif (IOSTYLE == AMBAIO)
 * ...
 * #else // (IOSTYLE == HARDWARE)
 * ...
 * #endif
 *
 * Then the command line for the module should include something to 
 * the effect:
 *
 *      ... -DIOSTYLE=XACTIO ...
 * or   ... -DIOSTYLE=AMBAIO ...
 * etc.
 * 
 **********************************************************************/

#define XACTIO   1
#define AMBAIO   2
#define HARDWARE 3

#ifndef IOSTYLE
#error IOSTYLE must be defined on the command line
#endif
#if ((IOSTYLE != XACTIO) && (IOSTYLE != AMBAIO) && (IOSTYLE != HARDWARE))
#error "IOSTYLE" contains an invalid value: IOSTYLE
#endif

#endif // ifdef IOSTYLE_H
