/* thread.c
 * Thread utility
 *
 *---------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-1999 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *  system: SA1200
 *  subsystem: Common
 *  author: sk/drj Feb 27, 98  Initial version
 *  revisions:
 *  $History: thread.c $
 * 
 * *****************  Version 2  *****************
 * User: Ewalker      Date: 8/11/00    Time: 10:05
 * Updated in $/dev_1200/osApi
 * Changed "#ifdef WIN32" to "#if (OS == WIN_32)" for consistency with
 * sem.c
 * 
 * *****************  Version 1  *****************
 * User: Drjohns1     Date: 7/24/00    Time: 5:42p
 * Created in $/dev_1200/osApi
 * 
 * *****************  Version 21  *****************
 * User: Dmwhite      Date: 4/05/00    Time: 2:55p
 * Updated in $/dev_1200/Utils
 * no more  U_COS macro, changed to (OS==UCOS)
 * 
 * *****************  Version 20  *****************
 * User: Drjohns1     Date: 4/04/00    Time: 6:18p
 * Updated in $/dev_1200/Utils
 * OS type check and cleanup
 * 
 * ---------------------------------------------------------------------
 */

#include "ostype.h"
#include "thread.h"

#if (OS == WIN_32)

thread_T Thread_Create(int priority, void (* startFunc)(void *), threadArg_T *arg)
{
	thread_T thread;
	unsigned threadId;

	thread = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) startFunc,
						arg, 0,	&threadId);

	SetThreadPriority(thread, priority);
	return(thread);
}

int Thread_SetPriority(thread_T thread, int priority)
{
	return(SetThreadPriority(thread, priority));
}

int Thread_Cancel(thread_T thread)
{

    TerminateThread(thread, 0);
	CloseHandle(thread);
	return(0);
}
#endif		/* OS == WIN_32 */


#if (OS == VXWORKS)
thread_T Thread_Create(int priority, threadFunc_T startFunc, threadArg_T *arg)
{
   int threadId;
   thread_T thread;

   thread = taskSpawn(NULL,				    /* task name tn will be assigned */
					    priority,			/* Priority */
						0,			        /* option word */
						1024 * 32,			    /* stack size bytes */
						(FUNCPTR) startFunc,/* Thread to run */
						arg,0,0,0,0,0,0,0,0,0);	/* args */

  return(thread);
}

int Thread_SetPriority(thread_T thread, int priority)
{
	return(taskPrioritySet(thread, priority));
}

int Thread_Cancel(thread_T thread)
{
   return(taskDelete(thread));
}

#endif		/* OS == VXWORKS */


#if (OS == UCOS)
#include <stdlib.h>
#define STACKSIZE 0x400
#define STACKSIZE_BYTES 0x1000

static int nextUniqueTaskID = 10;

int getNextUniqueTaskID(void)
{
   nextUniqueTaskID++;
   return(nextUniqueTaskID);
}

thread_T Thread_Create(int priority, void(* startFunc)(void *) , threadArg_T *arg)
{
   unsigned int * Stack;

    Stack = (unsigned int*) malloc(STACKSIZE_BYTES);
    if (!Stack)
        return(-1);

   
   OSTaskCreate(   startFunc,
              arg,
              &Stack[STACKSIZE],
              priority,
              STACKSIZE*sizeof(int));


  return(priority);
}

int Thread_SetPriority(thread_T thread, int priority){
	/* not used for uCOS */
	return(0);
}


int Thread_Cancel(thread_T thread)
{
   /* later free stack kill task */
   return 0;
}

#endif		/* OS == UCOS */


#if (OS == UNIX)

thread_T Thread_Create(int priority, threadFunc_T startFunc, threadArg_T *arg)
{
   thread_T		thread;
   pthread_attr_t       thread_attr;
   struct sched_param   param;

   pthread_attr_init(&thread_attr);
   /* pthread_attr_setinheritsched(PTHREAD_EXPLICT_SCHED); */
   pthread_attr_setdetachstate(&thread_attr, PTHREAD_CREATE_DETACHED);
   param.sched_priority = priority;
   pthread_attr_setschedparam(&thread_attr, &param);

   pthread_create(&thread, &thread_attr, (pthread_startroutine_t)startFunc, (pthread_addr_t)arg);
   return(thread);
}

int Thread_SetPriority(thread_T thread, int priority)
{
   struct sched_param param;
   param.sched_priority = priority;
   return(pthread_setschedparam(thread, SCHED_OTHER, &param));
}

int Thread_Cancel(thread_T thread)
{
   return(pthread_cancel(thread));
}

#endif  /* OS == UNIX */
