/*
 * list.c
 * A package for managing doubly-linked lists.
 */
#include "list.h"
#include <stdlib.h>


/* ------------------------------------------------------------------------
 * Function: list_init
 */
List *list_init ()
{
  List    *p_list;

  /* Allocate a new List object. */
  p_list = (List *) malloc (sizeof(List));

  /* Set its fields to indicate and empty list. */
  p_list->p_head = NULL;
  p_list->size = 0;

  return (p_list);
}

List *getlist()
{
  return my_list;
}

void list_add (List *p_list, char *url)
{
  Element *p_elem;
  char *p_url;


  p_elem = list_find (p_list, url);

  if (p_elem != NULL) {
  printf("list_add: %s found\n",url);
  p_elem->value = p_elem->value + 1;
  } else {
  printf("list_add:%s not found\n",url);
  list_append (p_list, url);
  }

}

/* ------------------------------------------------------------------------
 * Function: list_append
 */
void list_append (List *p_list, char *url)
{
  Element *p_elem;
  Element *p_next;
  Element *p_last;
  char *p_url;
  int res;

  /* Sanity check: */
  if (p_list == NULL)
    exit(-1);

  /* Allocate a new Element object and set its fields. */
  p_elem = (Element *) malloc (sizeof(Element));
  p_url = (char *) malloc (sizeof(url));

  strcpy(p_url, url);

  p_elem->p_next = NULL;
  p_elem->p_url = p_url;
  p_elem->value = 1;

  printf("adding url=%s\n",p_url);

  p_next = p_list->p_head;
  p_last = NULL;
  while (p_next != NULL)
  {
     printf("skipping %s\n",p_next->p_url);
    /* Proceed to the next element. */
    p_last = p_next;
    p_next = p_next->p_next;
  }

   if (p_list->p_head == NULL)
      p_list->p_head = p_elem;
   else
      p_last->p_next = p_elem;

  /* Increment the list size. */
  p_list->size++;

  return;
}

/* ------------------------------------------------------------------------
 * Function: list_find
 */
const Element *list_find (List *p_list, char *url)
{
  Element *p_elem;
  int res;

  /* Sanity check: */
  if (p_list == NULL)
    exit(-1);

  /* Go over the list. */
  p_elem = p_list->p_head;
  while (p_elem != NULL)
  {
    /* Check if the current element contains the requested value. */

    printf("comparing %s=%s\n",p_elem->p_url,url);

    res = strcmp(p_elem->p_url, url);

    if (res == 0)
      return (p_elem);

    /* Proceed to the next element. */
    p_elem = p_elem->p_next;
  }

  /* If we reached here, the value is not found in the list. */
  printf("%s Not found\n", url);
  return (NULL);
}

/* ------------------------------------------------------------------------
 * Function: list_reset
 */
void list_reset (List *p_list)
{
  Element *p_elem;

  /* Sanity check: */
  if (p_list == NULL)
    exit(-1);

  while (p_list->p_head != NULL)
  {
    /* Keep track of the current list head. */
    p_elem = p_list->p_head;

    /* Make the second element the new list head. */
    p_list->p_head = p_list->p_head->p_next;

    /* Free memory. */
    free (p_elem);
  }

  /* Reset the rest of the fields. */
  p_list->size = 0;

  return;
}

