/*
 * list.h
 * A package for managing doubly-linked lists.
 */

/* Representation of a list element: */
struct _Element
{
  struct _Element  *p_prev;    /* Points to the previous element
                                 (or NULL in case of the list head). */
  char             *p_url;
  double           value;      /* The stored value. */
  struct _Element  *p_next;    /* Points to the next element
                                 (or NULL in case of the list tail). */
};

typedef struct _Element Element;

/* Representation of a doubly-linked list: */
struct _List
{
  Element          *p_head;    /* Points to the list head
                                 (or NULL if the list is empty). */
  int              size;       /* The list size. */
};

typedef struct _List List;

List        *my_list;

/* ------------------------------------------------------------------------
 * Function: list_init
 * Purpose:  Initialize a doubly-linked list.
 * Returns:  A pointer to an allocated list.
 *           Note that this pointer should eventually be freed using 
 *           list_reset() and then free().
 */
List *list_init ();

/* ------------------------------------------------------------------------
 * Function: list_prepend
 * Purpose:  Insert a value as the list head.
 * Input:    p_list - The list.
 *           x      - The inserted value.
 * Returns:  Nothing.
 */

void list_append (List *p_list, char *url);

/* ------------------------------------------------------------------------
 * Function: list_find
 * Purpose:  Find a value in the list.
 * Input:    p_list - The list.
 *           x      - The queried value.
 * Returns:  A pointer to the list element that contains the queried value,
 *           or NULL if the value is not found in the list.
 */
const Element *list_find (List *p_list, char *url);

/* ------------------------------------------------------------------------
 * Function: list_reset
 * Purpose:  Delete all objects from a list.
 * Input:    p_list - The list.
 * Returns:  Nothing.
 */
void list_reset (List *p_list);
