/*
** minimal SSLProxy server for content switch project
**			ganesh kumar godavari (gkgodava@archie.uccs.edu)
*/

/* general libraries */
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <errno.h>
#include <stdarg.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>
#include <gdbm/ndbm.h>		/* dbm library */ 

/* socket libraries*/
#include <sys/types.h>  	/* ULTRIX didn't like stat with types*/
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>  	/* for inet_ntoa */
#include <time.h>       	/* for ctime */
#include <signal.h>
#include <sys/wait.h>
#include <pwd.h>
#include <grp.h>
#include <fcntl.h>
#include <limits.h>
#include <sys/time.h>
#include <setjmp.h>
#include <sys/mman.h>

/* openssl libraries*/
#include <openssl/lhash.h>
#include <openssl/bn.h>
#include <openssl/err.h>
#include <openssl/pem.h>
#include <openssl/x509.h>
#include <openssl/ssl.h>
#include <openssl/rand.h>


/* rule matching structure */
struct ip_vs_cb_rule_field
{
  char  name[512];
  char  value[256];
};

typedef struct server
{
  char *name;
  u_long addr;
}server;

typedef struct front_server
{
  char     *name;
  u_long   addr;
  u_short  port;
}front_server;

static struct server server_list[] = 
{
  {"eca.uccs.edu",         0x80c6c0bc},  /*128.198.192.198*/
  {"frodo.uccs.edu",       0x80c6c0b7},  /*128.198.192.183*/
  {"reject", 		   0xffffffff} 
};

/*
** i need to remove this junk from chandra's code of xml parsing
*/
int 
   len_cr=2,
   len_lf=2,
   len_sp=1,
   len_la=1,
   len_ra=1,
   len_fs=1,
   len_ex=1,
   max_char_len=1;

char
   *c_cr="\r",
   *c_lf="\n",
   *c_sp=" ",
   *c_la="<",
   *c_ra=">",
   *c_fs="/",
   *c_ex="!";


struct ip_vs_cb_rule_field rule_fields[] =
{
{"purchase:1.totalAmount:1.", "" },
{"purchase:1.subTotal:1.", "" },
{"purchase:1.subTotal:2.", "" },
{"purchase:1.unitPrice:2.", "" }
};


int num_rule_fields = sizeof(rule_fields) / sizeof(struct ip_vs_cb_rule_field);

int num_servers = sizeof(server_list) / sizeof(struct server);

#define STICKY_ON_IP 		1
#define STICKY_ON_IP_PORT 	2
#define NON_STICKY		3
#define MIN(a,b) 		(a>b) ? b : a

typedef struct ip_sticky
{
 u_long source_ip;
 u_long dest_ip;
}ip_sticky;


/* Request structure */
typedef struct Request {
int content_length;
char method[10];
char *url;
char *content_type;
char *header;
char *body;
}Request;


/* connection structure */
typedef struct Connection {
u_short port;
struct front_server *cli_ser;
Request *req;
} Connection;


/* define constants -------- Editable Section */
#define SERVER_PORT 			443
#define CLIENT_TIMEOUT 			30
#define SERVER_ROOT			"/home/gkgodava/project/openssl-0.9.6a/apps"
#define LOG_FILE 			"log/ssl.log"
#define SESS_FILE			"cache/scache"
#define SERVER_IP 			0x80c6c0c3
#define SERVER_NAME			"oblib.uccs.edu"
#define CA_FILE     			"testssl/CA/cacert.pem"
#define CA_PATH     			"testssl/CA"
#define KEY_FILE    			"testssl/private/private.key"
#define CERT_FILE   			"testssl/cert/newcert.pem"
#define RAND_FILE   			"testssl/random/random.pem" /* random is a junk file that contains any data--ensure it is not repeated*/
#define SSL_SESSION_CACHE_TIMEOUT 	300
#define STICKY_SIZE 			20			    /* allocation for maximum # of the sticky connection */

/* End of  Editable Section */

/* define constants --------- Non Editable Section */
#define SSL_DBM_FILE_MODE   	(S_IRUSR | S_IWUSR)
#define SSL_MUTEX_LOCK_MODE   	(S_IRUSR | S_IWUSR)
#define IOBUFFSIZE	   	4096

/* End of Non Editable Section */

static SSL *ssl=(SSL *) NULL;
static int nMutexFD; 			/* for file locking stuff*/

/* contants definition for access from path */
char *log_file;	
char *sess_file;
char *ca_file; 
char *ca_path;
char *key_file;
char *cert_file;
char *rand_file;


union semun {
  int val;
  struct semid_ds *buf;
  ushort *array;
};

int semID;           			/* ID of the semaphore set.       	*/
union semun sem_val;  			/* semaphore value, for semctl(). 	*/
char* shm_addr;           		/* address of shared memory segment.  	*/
int shm_id;               		/* ID of the shared memory segment.   	*/
int *sticky_num;          		/* # of sticky entries in shared mem	*/
ip_sticky *sticky;

static int long_jumped=0,g_sock_fd;
static sigjmp_buf jmpbuf;
static volatile sig_atomic_t canjump;
static FILE *log_fp = (FILE *) NULL;

static int sslVerifyCallback(int ok,X509 *xs,X509 *xi,int depth,int error,char *arg);
static int ireadSSL(SSL *ssl,char *buf,int nbyte);
static int sockReadSSL(SSL *ssl,char *buf,size_t n);
static int sockGetsSSL(SSL *ssl,char *str,size_t count);
static int sockPutsSSL(SSL *ssl,char *str);
static int sockWriteSSL(SSL *ssl,char *str,size_t count);

static int serverSocket(u_short port,int max_server);
static int getConnection(int socket_type,u_short port,int *listener,int max_serv);
static int sockGets(int sockfd,char *str,size_t count);
static int sockPuts(int sockfd,char *str);
static int sockWrite(int sockfd,char *str,size_t count);
static int getPeerInfo(int sockfd,struct front_server *client);
static void serveRequest(int fd,struct front_server *remote_ip);
u_long rule_configure(char *url, u_long saddr,u_long daddr, u_short sport, u_short dport, u_short protocol); 
u_long ruleMatching(Connection *conn);
void ssl_mutex_sem_on(int sem_id);
void ssl_mutex_sem_off(int sem_id);

int get_tag_range(char *tag, int pos, char *hay_begin, char *hay_end, char **needle_begin, char **needle_end);
static int xml_tag_parser(char *buf);


/*=========================== socket code ===============================*/
/*
** iread()
** same as read() system call except for automatic
** recovery for EINTR errors
*/

int iread(int fd,char *buf,int nbyte)
{
  int
     maxfd,
     nready,
     ret=0;

  struct timeval  tv;
  fd_set   readfds;

  FD_ZERO(&readfds);
  maxfd=fd+1;
  FD_SET(fd,&readfds);

  tv.tv_sec=CLIENT_TIMEOUT;
  tv.tv_usec=0;

  nready=select(maxfd,&readfds,NULL,NULL,&tv);
  if (nready == 0)
   {
    (void) fprintf(stderr," *** read pid %d Timed out in %d seconds ***\n",getpid(),CLIENT_TIMEOUT);
    return -1;
   }
 /*
 ** get request
 */
 if (FD_ISSET(fd,&readfds))
  {
    while (1)
    {
      ret=read(fd,buf,nbyte);
      if ((ret < 0) && (errno == EINTR))
          continue;
       else
          return (ret);
    }
  }
    return (-1);   /* won't be here but for saftey */
}

int sockRead(int sockfd,char *buf,size_t n)
{
    int
        total_nbytes=0;

    int
        nbytes;

    int
        saved_errno;

    while (total_nbytes < n)
    {
        nbytes=iread(sockfd,&buf[total_nbytes],
            n-total_nbytes);
        if (nbytes < 0)
        {
            saved_errno=errno;
            if (saved_errno == ECONNRESET)
            {
                return (-3);
            }
            else
                return (-1);
        }
        else if (nbytes == 0)
            return (total_nbytes);
        else
           total_nbytes += nbytes;
    }
    return (total_nbytes);
}

/* a wrapper around getConnection() for simplicity 
** it's only for stream socket
** port is in host byte order
*/
static int serverSocket(u_short port,int max_server)
{
    int dummy=(-1);

    int sock_fd;  

    u_short nport;

    /*
    ** convert port to netword byte word 
    */
    nport=htons(port);
    sock_fd=getConnection(SOCK_STREAM,nport,&dummy,max_server);

    return (sock_fd);
}

int connectTo(char *netaddress, int port)
{
    struct hostent *host;
    struct sockaddr_in address;
    int sockOut, retval;

    if ((sockOut = socket(AF_INET, SOCK_STREAM, 0)) == -1)
     {
         return -1;
     }
    /*
    ** i tried to use gethostbyname_r but i failed badly, so i am using this technique of locking the process before call to gethostbyname
    */
   ssl_mutex_sem_on(semID);
   host =  gethostbyname(netaddress);
   ssl_mutex_sem_off(semID);
    if (host == (struct hostent *) NULL)
        return -1;


    bzero((char *) &address, sizeof(address));
    address.sin_family = AF_INET;
    address.sin_port = htons(port);
    bcopy((char *) host->h_addr, (char *) &address.sin_addr, host->h_length);
    retval = connect(sockOut, (struct sockaddr *) &address, sizeof(address));
 
    if(retval == -1)
     {
      (void) fprintf (stderr,"connect() failed!\n");
     close(sockOut);
      return -1;
     }

    return sockOut;         /* connection OK */
}

/*
** this function listens on a port and returns connecions.
** it forks returns off internally, so the calling funcion does not
** have to worry about that. 
** the function will create a new process for every incomming connection,
** so in the listening process, it will never return. Only when a connection
** comes in, and we create a process for it, will the function return.
** THE CALLING FUNCION SHOULD NOT LOOP
*/
static int getConnection(int socket_type,u_short port,int *listener,int max_serv)
{
    struct linger   li;

    struct sockaddr_in  address;

    int listeningSocket;

    int connectedSocket = -1;

    int newProcess;

    int one = 1;

    /*
    ** setup internet address information
    ** this is used with the bind () call
    */
    memset((char *) &address,0,sizeof(address));
    address.sin_family=AF_INET;
    address.sin_port=port;
    address.sin_addr.s_addr=htonl(INADDR_ANY);

    listeningSocket=socket(AF_INET,socket_type,0);
    if (listeningSocket < 0)
    {
        (void) fprintf (stderr,"Unable to open socket!\n");
        perror("socket");
        exit (1);
    }
    if (listener != (int *) NULL)
       *listener=listeningSocket;

    /*
    ** we will resuse port
    */
    setsockopt(listeningSocket,SOL_SOCKET,SO_REUSEADDR,(char *) &one,
        sizeof(int));

    /*
    ** we will set keep alive.  read Steven, Network Prog. 2nd edition, p186
    */
    one=1;
    setsockopt(listeningSocket,SOL_SOCKET,SO_KEEPALIVE,(char *) &one,
            sizeof(int));

    li.l_onoff=1;
    li.l_linger=30;
    setsockopt(listeningSocket,SOL_SOCKET,SO_LINGER,(char *) &li,
            sizeof(struct linger));

    if (bind(listeningSocket,(struct sockaddr *) &address,
        sizeof(address)) < 0)
    {
        (void) fprintf (stderr,"\n\nUnable to bind to socket at port: %d\n",
                        ntohs(port));
        (void) fprintf (stderr,"Probably the port is already in use!\n");
        (void) fprintf(stderr,"Or you do not have permission to bind!\n");
        close(listeningSocket);
        exit (1);
    }

    if (socket_type == SOCK_STREAM)
    {
        /*
        ** queue up max_serv connections before having them
        ** automatically rejected
        */
        if (listen(listeningSocket,max_serv) == 0)
        {
            (void) fprintf(stderr,"\nhttpd listening at port: %d\n",
                           ntohs(port));
        }
        else
        {
            perror("listen()");
        }

        while (connectedSocket < 0)
        {
            connectedSocket=accept(listeningSocket,NULL,NULL);
            if (connectedSocket < 0)
            {
                /*
                ** either a real error occured or blocking was 
                ** interrupted for some reason. only abort execution
                ** if a real error occured
                */
                if (errno != EINTR)
                {
                    (void) fprintf (stderr,"unable to accept!\n");
                    perror("accept");
                    close (listeningSocket);
                    exit (1);
                }
                else
                    continue; /* don't fork, do the accept again*/
            }
            newProcess=fork();
            if (newProcess < 0)
            {
                (void) fprintf (stderr,"failed to fork!\n");
                perror("fork");
                close(connectedSocket);
                connectedSocket=(-1);
            }
            else
            {
                /*
                ** we have a new process (child)
                */
                if (newProcess == 0)
                {
                    /*
                    ** this is the new process
                    ** close our copy of the socket
                    */
                    close(listeningSocket);

                    *listener=(-1); /* closed in this process, we are not
                                    ** responsible for it
                                    */
                }
                else
                {
                    /*
                    ** this is the main loop. close copy of connected 
                    ** socket, and continue loop
                    */
                    close (connectedSocket);
                    connectedSocket=(-1);
                }
            }
        }
        return (connectedSocket);
     }
     else
        return (listeningSocket);
}

int sockGets(int sockfd,char *str,size_t count)
{
  int
     bytesRead;

  int
     totalCount=0;

  char
     *currentPosition;

  char
     lastRead=0;

  currentPosition=str;
  while (lastRead != 10)
  {
    bytesRead=read(sockfd,&lastRead,1);
    if (bytesRead <= 0)
     {
       /*
       ** the other side may have closed unexpectedly
       */
         return (-1);
      }

     if ((totalCount < count) && (lastRead != 10) && (lastRead != 13))
     {
         *currentPosition=lastRead;
         currentPosition++;
         totalCount++;
      }
    }
 
    *currentPosition='\r';
    currentPosition++;
    *currentPosition='\n';
    currentPosition++;
    *currentPosition='\0';
    currentPosition++;

    if(count > 0)
      *currentPosition=0;

    return (totalCount);
}

/*
** this function writes a character string out to a socket.
** it returns -1 if the connection is closed while it is trying to
** write
*/
static int sockPuts(int sockfd,char *str)
{
    return (sockWrite(sockfd,str,strlen(str)));
}

static int sockWrite(int sockfd,char *str,size_t count)
{
    size_t
        bytesSent=0;

    int
        thisWrite;

    while (bytesSent < count)
    {
       do
          thisWrite=write(sockfd,str,count-bytesSent);
       while ((thisWrite < 0) && (errno == EINTR));

       if (thisWrite <= 0)
          return (thisWrite);

        bytesSent += thisWrite;
        str += thisWrite;
    }
    return (count);
}

/*
** i an currently retriving client port but it may not be of use!! got to wait and see -- gkgodava@archie.uccs.edu
*/
static int getPeerInfo(int sockfd,struct front_server *client)
{
    struct sockaddr_in  cs;
    int len;
    struct hostent   *ch;
    char temp[64];

    len=sizeof(cs);

    if (getpeername(sockfd,(struct sockaddr *) &cs,&len) < 0)
    {
        return (-1);
    }
    
    /*
    ** extract client IP
    */
    client->addr=(u_long) ntohl(cs.sin_addr.s_addr); /* network byte ordering */

    /*
    ** extract client_port
    */
    client->port=ntohs(cs.sin_port);

    /*
    ** extract cononical name of the client
    */
    ch=gethostbyaddr((char *) &cs.sin_addr,sizeof(cs.sin_addr),AF_INET); 

    if (ch == NULL)
    {
        (void) fprintf(stderr,"Unable to get cononical name of the client\n");
        return (-1);
    }
    else
    {
    	client->name = ch->h_name;
	client->name[strlen(client->name)] = '\0';
    }

    return (0);
}

/*=========================== socket code ===============================*/

static void daemonize(void)
{
    int i;

    if (fork())
    {
        _exit(0);   /* parent goes bye-bye */
    }

    /*
    ** detach from terminal
    */
#ifdef HAVE_SETSID
    setsid();   /* request a new session */
#else   
#ifdef TIOCNOTTY
    i=open("/dev/tty",O_RDWR);
    if (i >= 0)
    {
        ioctl(i, (int)TIOCNOTTY, (char *)NULL);
        close(i);
    }
#endif  /* TIO CNOTTY */
#endif  /* HAVE_SETSID */


    chdir("/");
    umask(0);

}

/*
** never core dump.
*/

/*
 * Signals that cause a core dump 
 *
 *    SIGQUIT      3    Core     Quit (see termio(7I))
 *    SIGILL       4    Core     Illegal Instruction
 *    SIGTRAP      5    Core     Trace/Breakpoint Trap
 *    SIGABRT      6    Core     Abort
 *    SIGEMT       7    Core     Emulation Trap
 *    SIGFPE       8    Core     Arithmetic Exception
 *    SIGBUS       10   Core     Bus Error
 *    SIGSEGV      11   Core     Segmentation Fault
 *    SIGSYS       12   Core     Bad System Call
 *    SIGXCPU      30   Core     CPU time limit exceeded (see getrlimit(2))
 *    SIGXFSZ      31   Core     File size limit exceeded (see getrlimit(2))
 */

static void noCoredump(void)
{
#ifdef SIGQUIT
    signal(SIGQUIT, SIG_IGN );
#endif /* SIGQUIT */

#ifdef SIGILL
    signal(SIGILL, SIG_IGN );
#endif /* SIGILL */

#ifdef SIGTRAP
    signal(SIGTRAP, SIG_IGN );
#endif /* SIGTRAP */

#ifdef SIGABRT
    signal(SIGABRT, SIG_IGN );
#endif /* SIGABRT */

#ifdef SIGEMT
    signal(SIGEMT, SIG_IGN );
#endif /* SIGEMT */

#ifdef SIGFPE
    signal(SIGFPE, SIG_IGN );
#endif /* SIGFPE */

#ifdef SIGBUS
    signal(SIGBUS, SIG_IGN );
#endif /* SIGBUS */

#ifdef SIGSEGV
    signal(SIGSEGV, SIG_IGN );
#endif /* SIGSEGV */

#ifdef SIGSYS
    signal(SIGSYS, SIG_IGN );
#endif /* SIGSYS */

#ifdef SIGXCPU
    signal(SIGXCPU, SIG_IGN );
#endif /* SIGXCPU */

#ifdef SIGXFSZ
    signal(SIGXFSZ, SIG_IGN );
#endif /* SIGXFSZ */


}

void fatalError(char *format,...)
{
    va_list  args;

    va_start(args,format);
    (void) fprintf (stderr,"Error: ");
    vfprintf(stderr,format,args);
    va_end(args);
    exit(-1);
}

/*
** write all the junk!!  which u think may be useful to Administrator
**
*/
void logPrint(FILE * log_fp,char * format,...)
{
    va_list  args;
    time_t   now;

 
   char   *time_string;

    if(log_fp == NULL) {
	(void) fprintf(stderr,"unable to identify how the log file %s closed \n",LOG_FILE);
	return;
    }
	
    now=time(NULL);

    time_string=ctime(&now);
    time_string[strlen(time_string)-1]='\0';

    va_start(args,format);

    (void) fprintf(log_fp,"[%s]",time_string);
    (void) vfprintf(log_fp,format,args);

    va_end(args);
    fflush(log_fp);
    return;

}

static void sigAlarm(int sigtype)
{
    if (canjump == 0)
    {
        return;
    }
    canjump=1;
    long_jumped=1;
    siglongjmp(jmpbuf,1);
}

/*
** duplicate a string
*/
static char *mystrdup (char *string)
{
    char *tmp;

    if (string == (char *) NULL)
        return ((char *) NULL); 

    tmp = (char *) malloc ((int) strlen(string) + 1);
    if (tmp == (char *) NULL)
    {
        perror("memory allocation problem mystrdup()");
        exit(0);
    }
    /* it's safe to copy this way */
    (void) strcpy(tmp,string);
    tmp[strlen(tmp)+1]='\0';
    return (tmp);
}


/*=========================== session handling code ===============================*/

void ssl_mutex_sem_on(int sem_id)
{
 struct sembuf lock_it;

 lock_it.sem_num = 0;
 lock_it.sem_op = -1;   
 lock_it.sem_flg = 0;
 semop(sem_id, &lock_it, 1);

 return;
}


void ssl_mutex_sem_off(int sem_id)
{
 struct sembuf unlock_it;

 unlock_it.sem_num = 0;
 unlock_it.sem_op = 1;  
 unlock_it.sem_flg = 0;
 semop(sem_id, &unlock_it, 1);

 return;
}

/*
** temporary function for checking if the shared memory stuff is working properly or not
** should be removed soon gkgodava@archie.uccs.edu
*/
void printDetails(int semID,ip_sticky *s,int *sticky_num)
{
  int i=0;

  for(i=0;i<(*sticky_num);i++)
   (void) fprintf(stderr,"\a\a\t==>invoked pid %d source address %x destination address is %x\n",getpid(),s[i].source_ip, s[i].dest_ip);
  
  return ;
}

/*
** generates a random number to remove the information from shared memory 
** should be invoked in case of an overflow
*/
int removeSticky(void)
{
 srand(time(NULL));
 return (rand()%STICKY_SIZE);
}

/*
** adds a sticky connection information into the shared memory
*/
void addSticky(u_long saddr,u_long daddr)
{
 int loc;

 ssl_mutex_sem_on(semID);
 loc = (*sticky_num);
 if((*sticky_num) >= STICKY_SIZE)
   {
   loc = removeSticky();
/*
   printf("\t\t==>call to removeSticky noticed removing %d\n",loc);
*/
   }
 else
    (*sticky_num)++;

    sticky[loc].source_ip = saddr;
    sticky[loc].dest_ip = daddr;
 ssl_mutex_sem_off(semID);
return;
}

/*
** checks for the connection information i.e sticky or not, in the shared memory
** if found returns destination address else 0 
*/
u_long findSticky(u_long saddr)
{
  int i=0;
  u_long addr=0;
/*
 (void) fprintf(stderr,"findsticky has been invoked by pid %d number of entries %d looking for saddr %x\n ",getpid(),(*sticky_num),saddr);
 */
  ssl_mutex_sem_on(semID);
  for(i=0;i<(*sticky_num);i++)
  {
   if(saddr == sticky[i].source_ip)
   {
     ssl_mutex_sem_off(semID);
/*
   (void) fprintf(stderr,"\t<==pid %d  saddr %x source address %x destination address is %x\n",getpid(),saddr,sticky[i].source_ip, sticky[i].dest_ip);
     (void) fprintf(stderr,"\t return address is going to be the  address of %x\n", sticky[i].dest_ip);
*/
     return sticky[i].dest_ip;
   }

  }
  ssl_mutex_sem_off(semID);
  return 0 ;
}

void ssl_mutex_file_open(void)
{
char *file;

file = mystrdup(sess_file);
strcat(file,".db");

 if((nMutexFD =(int) open(file,O_WRONLY|O_CREAT,SSL_MUTEX_LOCK_MODE)) <  0)
   {
     (void) fprintf(stderr,"child could not open SSLMutex lockFile %s errno is %d \n",SESS_FILE);
     exit(1);
   }

free(file);
  return;
}

unsigned int ssl_mutex_file_acquire(void)
{
  int rc = -1;
  struct flock lock_it;
    
  lock_it.l_whence = SEEK_SET; /* from current point */
  lock_it.l_start  = 0;        /* -"- */
  lock_it.l_len    = 0;        /* until end of file */
  lock_it.l_type   = F_WRLCK;  /* set exclusive/write lock */
  lock_it.l_pid    = 0;        /* pid not actually interesting */

  while (((rc = fcntl(nMutexFD, F_SETLKW, &lock_it)) < 0) && (errno == EINTR) )
      ;

  (void) fcntl(nMutexFD,F_GETLK,&lock_it);

  if (lock_it.l_type != F_UNLCK)
    return 0;
  else
    return 1;
}

unsigned int ssl_mutex_file_release(void)
{
 int rc = -1;
 struct flock unlock_it;

 unlock_it.l_whence = SEEK_SET; /* from current point */
 unlock_it.l_start  = 0;        /* -"- */
 unlock_it.l_len    = 0;        /* until end of file */
 unlock_it.l_type   = F_UNLCK;  /* unlock */
 unlock_it.l_pid    = 0;        /* pid not actually interesting */

 while (((rc = fcntl(nMutexFD, F_SETLKW, &unlock_it)) < 0) && (errno == EINTR) )
     ;

 (void) fcntl(nMutexFD,F_GETLK,&unlock_it);

 if(unlock_it.l_type == F_UNLCK)
   return 1;
 else
   return 0;
}

void ssl_mutex_on(void)
{
 unsigned int ok = 1;

 ok = ssl_mutex_file_acquire();

 if(!ok)
    (void) fprintf(stderr, "Failed to acquire mutex lock\n");

 return;
}

void ssl_mutex_off(void)
{
 unsigned int ok = 1;

 ok = ssl_mutex_file_release();

 if (!ok)
    (void) fprintf(stderr, "Failed to release mutex lock\n");

 return;
}

void ssl_scache_dbm_expire(void)
{
 DBM *dbm;
 datum dbmkey;
 datum dbmval;
 time_t tExpiresAt;
 int nElements = 0;
 int nDeleted = 0;
 int bDelete;
 datum *keylist;
 int keyidx;
 int i;
 static time_t tLast = 0;
 time_t tNow;

 /*
  * make sure the expiration for still not-accessed session
  * cache entries is done only from time to time
  */

  tNow = time(NULL);
  if (tNow < tLast + SSL_SESSION_CACHE_TIMEOUT)
      return;

  tLast = tNow;

  /*
   * Here we have to be very carefully: Not all DBM libraries are
   * smart enough to allow one to iterate over the elements and at the
   * same time delete expired ones. Some of them get totally crazy
   * while others have no problems. So we have to do it the slower but
   * more safe way: we first iterate over all elements and remember
   * those which have to be expired. Then in a second pass we delete
   * all those expired elements. Additionally we reopen the DBM file
   * to be really safe in state.
   */

  #define KEYMAX 1024
  ssl_mutex_on();
  for (;;) {
  /* 
  ** allocate memory for the key array
  **
  ** NOTE:- here i am not worried if i am running low on memory it does not cause the system to fail so i shall try in my next attempt 
  ** :-) go happy next time better luck :-)
  */

  if ((keylist = (datum *) malloc(sizeof(dbmkey)*KEYMAX)) == NULL)
   {
     break;
   }

  /* pass 1: scan DBM database */
  keyidx = 0;
  if ((dbm = dbm_open(sess_file, O_RDWR, SSL_DBM_FILE_MODE)) == NULL) 
   {
    (void) fprintf(stderr,"Cannot open SSLSessionCache DBM file `%s' for scanning",sess_file);
     break;
   }

  dbmkey = dbm_firstkey(dbm);

  while (dbmkey.dptr != NULL)
  {
    nElements++;
    bDelete = 0;
    dbmval = dbm_fetch(dbm, dbmkey);

    if(dbmval.dsize <= sizeof(time_t) || dbmval.dptr == NULL)
        bDelete = 1;
    else
    {
      memcpy(&tExpiresAt, dbmval.dptr, sizeof(time_t));
      if(tExpiresAt <= tNow)
         bDelete = 1;
    }

   if (bDelete)
   {
    if ((keylist[keyidx].dptr = (char *) malloc(dbmkey.dsize)) != NULL)
     {
       memcpy(keylist[keyidx].dptr, dbmkey.dptr, dbmkey.dsize);
       keylist[keyidx].dsize = dbmkey.dsize;
       keyidx++;

    if(keyidx == KEYMAX)
      break;
      }
   }

   dbmkey = dbm_nextkey(dbm);
  }

  dbm_close(dbm);

  /* pass 2: delete expired elements */
  if ((dbm = dbm_open(sess_file, O_RDWR, SSL_DBM_FILE_MODE)) == NULL)
   {
     (void) fprintf(stderr, "Cannot re-open SSLSessionCache DBM file `%s' for expiring",sess_file);
     break;
   }

  for (i = 0; i < keyidx; i++)
  {
   dbm_delete(dbm, keylist[i]);
   nDeleted++;
  }

  dbm_close(dbm);

  if (keyidx < KEYMAX)
     break;

 }

    ssl_mutex_off();
#if 0
    (void) fprintf(stderr, "Inter-Process Session Cache (DBM) Expiry: old: %d, new: %d, removed: %d", nElements, nElements-nDeleted, nDeleted);
#endif
    return;

}

void ssl_scache_remove(unsigned char *id, int idlen)
{
 DBM *dbm;
 datum dbmkey;

   /* create DBM key and values */
    dbmkey.dptr  = (char *)id;
    dbmkey.dsize = idlen;

    /* and delete it from the DBM file */
    ssl_mutex_on();
    if ((dbm = dbm_open(sess_file, O_RDWR, SSL_DBM_FILE_MODE)) == NULL)
    {
        (void) fprintf(stderr, "Cannot open SSLSessionCache DBM file `%s' for writing (delete)",sess_file);
        ssl_mutex_off();
        return;
     }

    dbm_delete(dbm, dbmkey);
    dbm_close(dbm);
    ssl_mutex_off();
    return;

}

SSL_SESSION *ssl_scache_retrieve(unsigned char *id, int idlen)
{
    DBM *dbm;
    datum dbmkey;
    datum dbmval;
    SSL_SESSION *sess = NULL;
    unsigned char *ucpData;
    int nData;
    time_t expiry;
    time_t now;

    /* allow the regular expiring to occur */
    ssl_scache_dbm_expire();

    /* create DBM key and values */
    dbmkey.dptr  = (char *)id;
    dbmkey.dsize = idlen;

    /* and fetch it from the DBM file */
    ssl_mutex_on();
    if ((dbm = dbm_open(sess_file, O_RDONLY, SSL_DBM_FILE_MODE)) == NULL)
    {
        (void) fprintf(stderr,"Cannot open SSLSessionCache DBM file `%s' for reading (fetch)",sess_file);

        ssl_mutex_off();
        return NULL;
    }

    dbmval = dbm_fetch(dbm, dbmkey);
    dbm_close(dbm);
    ssl_mutex_off();

    /* immediately return if not found */
    if (dbmval.dptr == NULL || dbmval.dsize <= sizeof(time_t))
        return NULL;

    /* parse resulting data */
    nData = dbmval.dsize-sizeof(time_t);
    ucpData = (unsigned char *)malloc(nData);
    if (ucpData == NULL) 
        return NULL;

    memcpy(ucpData, (char *)dbmval.dptr+sizeof(time_t), nData);
    memcpy(&expiry, dbmval.dptr, sizeof(time_t));


    /* make sure the stuff is still not expired */
    now = time(NULL);
    if (expiry <= now) {
        ssl_scache_remove(id, idlen);
        return NULL;
    }

    /* unstreamed SSL_SESSION */
    sess = d2i_SSL_SESSION(NULL, &ucpData, nData);

    return sess;
}


unsigned int ssl_scache_store(unsigned char *id, int idlen,time_t expiry,SSL_SESSION *sess)
{
  datum dbmkey;
  datum dbmval;
  unsigned char ucaData[1024*10];
  int nData;
  unsigned char *ucp;
  DBM  *dbm;

  /* streamline session of data */
  ucp = ucaData;
  nData = i2d_SSL_SESSION(sess, &ucp);

  /* be careful : try not to store too much bytes in a DBM file */
  if((idlen + nData) >= 950)
     return 0;

  /* create DBM key */
  dbmkey.dptr = (char *)id;
  dbmkey.dsize = idlen;


  /* create DBM value */
  dbmval.dsize = sizeof(time_t) + nData;
  dbmval.dptr = (char *) malloc(dbmval.dsize);

  if(dbmval.dptr == NULL)
    return 0;

  memcpy((char *)dbmval.dptr,&expiry,sizeof(time_t));
  memcpy((char *)dbmval.dptr+sizeof(time_t),ucaData,nData);

  /* store it to the DBM file */
  ssl_mutex_on();

  if((dbm = dbm_open(sess_file,O_RDWR|O_CREAT,SSL_DBM_FILE_MODE)) == NULL)
  {
    (void) fprintf(stderr, "cannot open SSL Session Cache DBM file %s for writing\n", SESS_FILE);
    ssl_mutex_off(); 
    free(dbmval.dptr);
    return 0;
  }

  if(dbm_store(dbm,dbmkey,dbmval,DBM_INSERT)<0)
  {
   (void) fprintf(stderr, "cannot store SSL Session to DBM file %s \n",SESS_FILE);
   ssl_mutex_off(); 
   free(dbmval.dptr);
   return 0;
  }

  dbm_close(dbm);
  ssl_mutex_off();

  /* free the temperary buffers */
  free(dbmval.dptr);

  /* allow the regular expiring to occur */
  ssl_scache_dbm_expire();

  return 1;
}


/*
** borrowed from apache
** temporarily -- gkgodava@archie.uccs.edu
*/
char *SSL_SESSION_id2sz(unsigned char *id, int idlen)
{
/*
** SSL_MAX_SSL_SESSION_ID_LENGTH is defined in ssl.h to be of 32
*/
    static char str[(SSL_MAX_SSL_SESSION_ID_LENGTH+1)*2];
    char *cp;
    int n;

    cp = str;

    for (n = 0; n < idlen && n < SSL_MAX_SSL_SESSION_ID_LENGTH; n++) {
        snprintf(cp, sizeof(str)-(cp-str), "%02X", id[n]);
        cp += 2;
    }

    *cp =(char) NULL;
    return str;
}

/*
** This callback function is executed by OPENSSL whenever a new SSL_sessiion is added to the internal
** session_cache. we use this hook to spread the SSL_session also to the inter processes disk-cache
** inorder to make share it with other processes
**/

int ssl_callback_NewSessionCacheEntry(SSL *ssl, SSL_SESSION  *pNew)
{
  long t;
  unsigned int flag;

  t = SSL_SESSION_CACHE_TIMEOUT;

  SSL_set_timeout(pNew,t);

  /*
  ** store the SSL_SESSION in the inter process cache with the same expire time, so that it expires automatically there too.
  */
  t = (SSL_get_time(pNew) + SSL_SESSION_CACHE_TIMEOUT);
  flag = ssl_scache_store(pNew->session_id, pNew->session_id_length, t, pNew);

  /*
  ** log the information
  */
  (void) fprintf(stderr,"\n\n new session created for ssl context value id is %s \n",SSL_SESSION_id2sz(pNew->session_id,pNew->session_id_length));

  return 0;
}


/*
** This callback function is executed by OPENSSL whenever a new SSL_sessiion is looked up in the internal OpenSSL
** session_cache and it is not found. we use this hook to lookup the SSL_session in to the inter processes disk-cache
** where it might be stored by other processes
**/
SSL_SESSION *ssl_callback_GetSessionCacheEntry(SSL *ssl, unsigned char *id, int idlen, int *pCopy)
{
  SSL_SESSION *pSession;

  pSession = ssl_scache_retrieve(id,idlen);

  /*
  ** log the information
  */

#if 0
  if(pSession != NULL)
    (void) fprintf(stderr,"\n\n old  session of ssl context value id is %s is being lookfor is found \n",SSL_SESSION_id2sz(id,idlen));
  else
    (void) fprintf(stderr,"\n\n old  session of ssl context value id is %s is being lookfor is not found \n",SSL_SESSION_id2sz(id,idlen));
#endif

  *pCopy=0;
  return pSession;
}

/*
** This callback function is executed by OPENSSL whenever a new SSL_sessiion is removed from internal OpenSSL
** session_cache. we use this hook to remove the SSL_session from inter processes disk-cache
**/
void ssl_callback_DelSessionCacheEntry(SSL_CTX *ctx, SSL_SESSION *pSession)
{
  /*
  ** log the information
  */
  ssl_scache_remove(pSession->session_id, pSession->session_id_length);
  (void) fprintf(stderr,"\n\n session dead removing it ssl value id is %s is being being deleted \n",
			SSL_SESSION_id2sz(pSession->session_id,pSession->session_id_length));

  return;
}

/*======================= End of session handling code ===========================*/

static u_long pow(int a, int b)
{
u_long temp=0;

if (b == 0) return 1;

if (b == 1) return a;

for ( ;b>1; b--)
  temp = temp + (a*a);

return temp;
}

/*
** function converts hexadecimal string to a decimal integer.
*/
static u_long Hex(char *s)
{
int pos=0;
u_long value=0;
int len = strlen(s)-2; /* skipping the \r\n characters here*/
#if 0
fprintf(stderr,"the string to be converted to decimal from hexadecimal is %s",s);
#endif

for(pos = 0, s += (len-1),value=0 ;pos<len ;s--,pos++)
{
  switch(*s)
  {
  case 'a':
  case 'A':
           value = value + (pow(16,pos) * 10);
           break;


  case 'b':
  case 'B':
           value = value + (pow(16,pos) * 11);
           break;

  case 'c':
  case 'C':
           value = value + (pow(16,pos) * 12);
           break;


  case 'd':
  case 'D':
           value = value + (pow(16,pos) * 13);
           break;

  case 'e':
  case 'E':
           value = value + (pow(16,pos) * 14);
           break;

  case 'f':
  case 'F':
           value = value + (pow(16,pos) * 15);
           break;

  case '0':
  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
           value = value + (pow(16,pos) * (*s - '0'));
           break;
 default:
         return value;
         break;
  }
}

#if 0
fprintf(stderr,"\n\n value to be returned is %d\n\n ",value);
#endif

return value;
}

void make_absolute_path(void)
{
if (
(log_file =
     (char *) malloc (sizeof (SERVER_ROOT) + sizeof (LOG_FILE) + 1)) == NULL
    ||
    (sess_file  = (char *) malloc( sizeof(SERVER_ROOT) + sizeof(SESS_FILE) + 1)) == NULL ||
    (ca_file    = (char *) malloc( sizeof(SERVER_ROOT) + sizeof(CA_FILE) + 1)) == NULL ||
    (ca_path    = (char *) malloc( sizeof(SERVER_ROOT) + sizeof(CA_PATH) + 1)) == NULL ||
    (key_file   = (char *) malloc( sizeof(SERVER_ROOT) + sizeof(KEY_FILE) + 1)) == NULL ||
    (cert_file  = (char *) malloc( sizeof(SERVER_ROOT) + sizeof(CERT_FILE) + 1)) == NULL ||
    (rand_file  = (char *) malloc( sizeof(SERVER_ROOT) + sizeof(RAND_FILE) + 1)) == NULL )
  {
   (void) fprintf(stderr," unable to allocate memory for converting relative paths to absolute paths");
   exit(1);
  }
(void) sprintf(log_file,"%s/%s",SERVER_ROOT,LOG_FILE);
(void) sprintf(sess_file,"%s/%s",SERVER_ROOT,SESS_FILE);
(void) sprintf(ca_file,"%s/%s",SERVER_ROOT,CA_FILE);
(void) sprintf(ca_path,"%s/%s",SERVER_ROOT,CA_PATH);
(void) sprintf(key_file,"%s/%s",SERVER_ROOT,KEY_FILE);
(void) sprintf(cert_file,"%s/%s",SERVER_ROOT,CERT_FILE);
(void) sprintf(rand_file,"%s/%s",SERVER_ROOT,RAND_FILE);
 return;
}

/*
** borrowed from mod_ssl
*/
int SSL_smart_shutdown(SSL *ssl)
{
    int i;
    int rc;

    /*
     * Repeat the calls, because SSL_shutdown internally dispatches through a
     * little state machine. Usually only one or two interation should be
     * needed, so we restrict the total number of restrictions in order to
     * avoid process hangs in case the client played bad with the socket
     * connection and OpenSSL cannot recognize it.
     */

    rc = 0;
    for (i = 0; i < 4 /* max 2x pending + 2x data = 4 */; i++) {
        if ((rc = SSL_shutdown(ssl)))
            break;
    }

    return rc;
}

int main(int argc, char **argv) 
{
    int  sock_fd,
         err,
	 rc,
         server_verify=SSL_VERIFY_NONE;

    char 
         szbuf[BUFSIZ],
         issuer[256];

    struct front_server szclient;

    SSL_CTX  *ctx;
    SSL_METHOD *method=SSLv23_server_method();

    X509 *ca_cert, *client_cert;

    FILE *ca_fp=(FILE *) NULL;
   
    SSL_load_error_strings();
    OpenSSL_add_ssl_algorithms();
    (void)make_absolute_path();     /* convert relative to absolute path */
    
    ctx=(SSL_CTX *) SSL_CTX_new(method);

    if (ctx == (SSL_CTX *) NULL)
    {
      fatalError("Unable to create new SSL CTX\n");
      goto ExitProcessing;
    }
     
    if (!SSL_CTX_load_verify_locations(ctx,ca_file,ca_path))
        fatalError("Failed in SSL_CTX_load_verify_locations()!\n");

    /*
    ** Load Private Key
    */

    err=SSL_CTX_use_RSAPrivateKey_file(ctx, key_file, SSL_FILETYPE_PEM);

    if (err == -1)
    {
       	(void) fprintf(stderr,"Error reading private key\n");
	SSL_CTX_free(ctx);
        exit(1);
    }


   /*
   ** Load Certificate
   */
    err=SSL_CTX_use_certificate_file(ctx, cert_file, SSL_FILETYPE_PEM);

    if (err == -1)
    {
        (void) fprintf(stderr,"Error reading certificate\n");
	SSL_CTX_free(ctx);
        exit(1);
    }

   /*
   ** check if the certificate and private key match
   */ 
   err = SSL_CTX_check_private_key(ctx);
 
    if (err == -1)
    {
       	(void) fprintf(stderr,"Error cerificate and private key donot match \n");
	SSL_CTX_free(ctx);
       	exit(1);
    }

    /*
    ** Load randomness
    */
    if (!RAND_load_file(rand_file,1024*1024))
    {
         (void) fprintf(stderr,"Unable to load Randomness for generating Entropy :-( \n");
    }

    /*
    ** read the ca certificate and save the issuer string, we'll compare
    ** the client's issuer with this one, if they match allow connection
    ** or zap him
    */
    ca_fp=fopen(ca_file,"r");
    if (ca_fp == (FILE *) NULL)
        fatalError("Failed to open Trusted CA certificate file: %s\n", ca_file);

    ca_cert=NULL;
    ca_cert=X509_new();
    if (!PEM_read_X509(ca_fp,&ca_cert,NULL,NULL))
        fatalError("Error reading trusted CA certificate fie: %s\n",ca_file);

     X509_NAME_oneline(X509_get_issuer_name(ca_cert),issuer,256);
    if (issuer == (char *) NULL)
        fatalError("No issuer for trusted CA certificate file!\n");


    if (ca_cert != NULL)
        X509_free(ca_cert);

   (void) fclose(ca_fp);


    /* open the log file */
    if((log_fp=fopen(log_file,"a+"))== NULL) 
	(void) fprintf(stderr, "unable to open the log file");

    shm_id = shmget(IPC_PRIVATE, (STICKY_SIZE * sizeof(ip_sticky)), IPC_CREAT | IPC_EXCL | 0600);

    if (shm_id == -1)
    {
        (void) fprintf(stderr,"Unable to allocate shared memory\n");
        exit(1);
    }

    /* attach the shared memory segment to our process's address space. */
    shm_addr = shmat(shm_id, NULL, 0);
    if (!shm_addr)  /* operation failed. */
    { 
        (void) fprintf(stderr,"Error: unable to attach the shared memory to our process's address space\n ");
        exit(1);
    }

    /* create a semaphore */
    semID = semget(IPC_PRIVATE, 1, IPC_CREAT | 0600);
    if (semID == -1) {
        (void) fprintf(stderr,"Error: unable to create a semaphore\n");
        exit(1);
    }

    /* intialize the semaphore to '1'. */
    sem_val.val = 1;
    rc = semctl(semID, 0, SETVAL, sem_val);
    if (rc == -1)
    {
        (void) fprintf(stderr,"Error: unable to initialize the semaphore");
        exit(1);
    } 

    sticky_num = (int*) shm_addr;
    *sticky_num = 0;
    sticky = (ip_sticky *) ((void*)shm_addr+sizeof(int));

    /*
    ** most cool Unix will take care of Zombies if we ignore SIGCHLD
    */
    signal(SIGCHLD,SIG_IGN);
    signal(SIGPIPE,SIG_IGN);
    signal(SIGALRM,sigAlarm);

    noCoredump(); 
    daemonize(); 

    /*
    ** open the server socket
    */
    sock_fd=serverSocket((u_short) SERVER_PORT ,1000);
    if (sock_fd == -1)
    {
        (void) fprintf(stderr,"Failed to create server\n");
        exit(1);
    }
    g_sock_fd=sock_fd;

    /*
    ** store the Session FIlename and file descriptors
    */
    ssl_mutex_file_open(); 

    /*
    ** get name, IP adress, and port number of the client
    */
    getPeerInfo(sock_fd,&szclient);
     
    alarm(CLIENT_TIMEOUT);

    sigsetjmp(jmpbuf,1);
    if (long_jumped == 1)
    {
        /*
        ** back from a long jump, alarm timer expired
        */
        long_jumped=0;
        canjump=1;
        goto ExitProcessing;
    }

     ssl=SSL_new(ctx);
     if (ssl == NULL)
         fatalError("Failed in SSL_new()!\n");

     SSL_set_fd(ssl,sock_fd);

    /*
    ** cleanup the structures 
    */
    SSL_clear(ssl);
    SSL_set_session(ssl,NULL);
    SSL_set_accept_state(ssl);

     /*
     ** session handling fucntions
     */ 
     SSL_CTX_flush_sessions(ctx,SSL_SESSION_CACHE_TIMEOUT);
     SSL_CTX_sess_set_new_cb(ctx, ssl_callback_NewSessionCacheEntry);
     SSL_CTX_sess_set_get_cb(ctx, ssl_callback_GetSessionCacheEntry);
     SSL_CTX_sess_set_remove_cb(ctx, ssl_callback_DelSessionCacheEntry);
 
     err=SSL_accept(ssl);

     if (SSL_get_error(ssl, err) == SSL_ERROR_ZERO_RETURN)
      {
         /*
         ** case where the connection was closed before any data was transferred
         */
          (void) fprintf(stderr, "SSL handshake stopped: connection was closed");
  	  goto ExitProcessing;
          return 0;
       }
     else if (ERR_GET_REASON(ERR_peek_error()) == SSL_R_HTTP_REQUEST)
          {
                /*
                ** case where OpenSSL has recognized a HTTP request => client speaks plain HTTP on our HTTPS
                */
		(void)fprintf(stderr, "SSL handshake failed: HTTP spoken on HTTPS port; ");
		goto ExitProcessing;
                return 0;
          }
     else if (SSL_get_error(ssl, err) == SSL_ERROR_SYSCALL)
          {
                if (errno > 0)
                    (void) fprintf(stderr,"SSL handshake interrupted by system "
                            "[Hint: Stop button pressed in browser?!]\n");
                else

                    (void) fprintf(stderr, "Spurious SSL handshake interrupt"
                            "[Hint: Usually just one of those OpenSSL confusions!?]\n");

		goto ExitProcessing;
		return 0;
          }
	else if (err == -1)
              {
	        fatalError("Error : unknown error in SSL_accept()\n");
		goto ExitProcessing;
		return 0;
	      }


     /* not verifying client at this time ?? */
     server_verify=SSL_VERIFY_PEER|SSL_VERIFY_FAIL_IF_NO_PEER_CERT|SSL_VERIFY_CLIENT_ONCE;

     (void) SSL_CTX_set_verify(ctx,server_verify,NULL);

     if(SSL_get_verify_result(ssl) != X509_V_OK)
         fatalError("Client %s certificate does not verify\n",szclient.name);
/*
     client_cert=SSL_get_peer_certificate(ssl);
     if (client_cert == NULL)
         fatalError("Client %s does not  have a certificate\n",szclient.name);
*/

    serveRequest(sock_fd,&szclient);


ExitProcessing:
    close(sock_fd);
    SSL_set_shutdown(ssl,SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
    SSL_smart_shutdown(ssl);
    SSL_free(ssl);
    exit(1);

/*
    SSL_CTX_free(ctx);
*/
}

/*
** here i am fixing the method and protocol name field to 10 bytes. i feel allocating  10 bytes is waste of "resource"
*/
static int parseRequest(char *req,char *method,char *url,char *proto,char *args,int len)
{
    char
        *chp,
        *m,
        *u,
        *p;

    if (req == NULL || *req == '\0')
        return (-1);

    /*
    ** Method
    */
    m=strtok(req,"\t ");
    if (m == NULL)
        return(-1);
    (void) strncpy(method,m,10);
    method[10]='\0';

    /*
    ** Url
    ** Url can't have any spaces
    */
    u=strtok(NULL,"\t\r ");
    if (u == NULL)
        return(-1);
    chp=strchr(u,'?');
    if (chp)
    {
        *chp++ = '\0';
        (void) strncpy(args,chp,len);
    }
 
    (void) strncpy(url,u,len);
    url[len]='\0';

    /*
    ** HTTP Protocol version
    */
    p=strtok(NULL,"\r ");
    if (p == NULL)
        return(-1);
    (void) strncpy(proto,p,10);
    proto[10]='\0';

    return (0); /* OK */
}


/*
** borrwed from thttpd
*/

static int hexDigit(char c)
{
    if (strchr("0123456789abcdefABCDEF",c) != (char *) NULL)
        return (1);

    return (0);
}

/*
** from thttpd source code
*/
static int hexIt(char c)
{
    if ((c >= '0') && (c <= '9'))
        return (c-'0');

    if ((c >= 'a') && (c <= 'f'))
        return (c-'a'+10);

    if ((c >= 'A') && (c <= 'F'))
        return (c-'A'+10);

    return (0);           /* shouldn't happen, but for saftey  */

}

/*
** from thttpd source code
*/
static char *strDecode(char *from)
{
char *to;
char *start=NULL;

    if (from  == (char *) NULL)
        return ((char *) NULL); 
 

    to = (char *) malloc ((int) strlen(from) + 1);

    if (to == (char *) NULL)
    {
        perror("memory allocation problem strDecode()");
	exit(0);
    }

    start = to;

    for (; *from != '\0'; ++to,++from)
    {
        if (*from == '%' && hexDigit(from[1]) &&
            hexDigit(from[2]))
        {
            *to=hexIt(from[1])*16+hexIt(from[2]);
            from += 2;
        }
        else
            *to = (*from=='+') ?' ': *from;	 /* inorder to replace + in post request with a space */
    }
    *to='\0';

return start;
}


/*
** find a substring from a string
** case insensitive
*/
int stristr(char *haystack,char *needle)
{
    int
        i, j, k, l;

    for (i = 0; haystack[i]; i++)
    {
        for (j = 0, l = k = i; haystack[k] && needle[j] &&
            tolower(haystack[k]) == tolower(needle[j]); j++, k++)
                ;
        if (needle[j] == '\0')
            return (i);
    }
    return -1;
}

char *getRealServerName(u_long saddr)
{
 int i=0;

 for(i=0; i<num_servers; i++)
 {
  if(server_list[i].addr == saddr)
     return server_list[i].name ;
 }

 return NULL;  /* should not happen but for saftey */
}


/*
** returns the formated text i.e replacing the text containing source with dest
** none of the arguments are modified
*/
char *replace(char *text, char *source, char *dest)
{
  char *header,*nh,newtext[1024];
  int start,end;

#if 0
(void) fprintf(stderr," the arguments to replace are text %s source %s dest %s",text,source,dest);
#endif


  header = mystrdup(text);

  if((start=stristr(header,source)) == -1)
  {
    (void) fprintf(stderr," unable to located substring %s in the string %s\n",text,source);
    return NULL;
  }

  if((nh=(char *) malloc(start+1)) == NULL)
  {
   (void) fprintf(stderr, "unable to allocate memory for the header\n");
   return 0;
  }

  strncpy(nh,header,start);
  nh[start]='\0';

#if 0
    (void) fprintf(stderr," new header %s\n",nh);
#endif

  end = strlen(source);


#if 0
  if((end=stristr((header+start),"\r\n")) == -1)
  {
    (void) fprintf(stderr," unable to located substring %s in the string %s\n",text,"\r\n");
    return NULL;
  }
#endif

#if 1
  if((text = (char *) malloc(strlen(nh) + strlen(dest) + strlen(header+start+end)+1))== NULL)
  {
   (void) fprintf(stderr, "unable to allocate memory for the header\n");
   return NULL;
  }
  sprintf(text,"%s%s%s\0",nh,dest,(header+start+end));

#endif

#if 0
strcpy(newtext,nh);
    (void) fprintf(stderr," new text %s\n",newtext);
strcat(newtext,dest);
    (void) fprintf(stderr," new text %s\n",newtext);
strcat(newtext,(header+start+end));
#endif

/*
** when i am freeing the memory occupied by headers and nh it is causing SIGSEGV(Segmentation Fault) only for I.E not for netscape 
** damn these browers / damn my coding as one of these is wrong
**  free(header);
**  free(nh);
*/

#if 0
(void) fprintf(stderr," the replaced text is %s",text);
#endif
 return text;

} 

void printRequest(Request *req)
{
  (void) fprintf(stderr, " Request details are headers  %s body  %s ", req->header, req->body);
  return;

}


/*
** test fuction to see everything required is there or not should not be in the prototype--gkgodava@archie.uccs.edu
*/
void printConnection(Connection *conn,u_long servid,char *comment)
{

  (void) fprintf(stderr, "pid %d to be routed to %x based on %s  Request details are Method %s URL %s content_type %s\n",getpid(),servid,comment, conn->req->method, conn->req->url,conn->req->content_type);
#if 0
  (void) fprintf(stderr, " Connection details remote ip %s, addr %x port %d\n",conn->cli_ser->name,conn->cli_ser->addr,conn->cli_ser->port);
  (void) fprintf(stderr, " Request details are Method %s URL %s content_type %s\n", conn->req->method, conn->req->url,conn->req->content_type);
  printRequest(conn->req);
#endif

}

void demoResponse(char *url)
{
 char body[1024];
 char content_length[64];
 sprintf(body," <!DOCTYPE HTML PUBLIC \"-//IETF//DTD HTML 2.0//EN\">
                   <HTML><HEAD>
                   <TITLE>404 Not Found</TITLE>
                   </HEAD><BODY>
                   <H1>Not Found</H1>
                   The requested URL %s was not found on this server.<P>
                   <HR>
                   <ADDRESS>Apache/1.3.14 Server at %s  Port 80</ADDRESS>
                   </BODY></HTML>",url,SERVER_NAME);

 sprintf(content_length,"Content-Length: %d\r\n",strlen(body));

 sockPutsSSL(ssl,"HTTP/1.0 404 OK\r\n");
 sockPutsSSL(ssl,"MIME-version: 1.0\r\n");
 sockPutsSSL(ssl,content_length);
 sockPutsSSL(ssl,"Content-Type: text/html\r\n\r\n");


 sockPutsSSL(ssl,body);

 return;
}

/*
** serverRequest is currently clumsy -- fix it and make it readable(gkgodava@archie.uccs.edu)
*/
static void serveRequest(int fd,struct front_server  *remote_ip)
{
  char
     header[1024],
     chunked_body[1024],
     *val,
     szcontent_type[BUFSIZ],
     szmethod[10],
     szurl[BUFSIZ],
     szproto[10],
     szargs[BUFSIZ],
     body[IOBUFFSIZE],
     TargetServer[128],
     szbuf[BUFSIZ];


  int
     maxfd,
     nready,
     close_connection,
     chunked,
     content_length,
     server_sock_fd,
     chunk_size,
     status_code,
     data_read;

  float version;


  struct timeval  tv;
  Connection *conn_rec;
  u_long ret_addr;
  fd_set   readfds;

KEEPALIVE:
  /* if the client does not respond within the timeout limit, zap it  */
   FD_ZERO(&readfds);

   maxfd=SSL_get_fd(ssl)+1;
   FD_SET(SSL_get_fd(ssl),&readfds);


  content_length=0;
  data_read=0;
  *szcontent_type='\0';
  *szmethod='\0';
  *szurl='\0';
  *szproto='\0';
  *szargs='\0';
  *szbuf='\0';

   tv.tv_sec=CLIENT_TIMEOUT;
   tv.tv_usec=0;

    nready=select(maxfd,&readfds,NULL,NULL,&tv);
    if (nready == 0)
    {
        (void) fprintf(stderr," *** pid %d Timed out in %d seconds ***\n",getpid(),CLIENT_TIMEOUT);
        goto  GracefulExit;
    }

 /*
 ** get request
 */
 if (FD_ISSET(SSL_get_fd(ssl),&readfds))
  {
     alarm(0);
    /* check whether the packet sent after the connection established has no data -- if so reject it */
     if((sockGetsSSL(ssl,szbuf,sizeof(szbuf)-1)) <= 0)
        goto  GracefulExit;
   }

  /*
  ** now we can allocate memory for the connection structue as we know it is not a junk request!!
  */
  if( ((conn_rec = (struct Connection *) malloc(sizeof(Connection))) == NULL) || 
       ((conn_rec->cli_ser = (struct front_server *) malloc(sizeof(struct front_server))) == NULL) || 
       ((conn_rec->req = (struct Request *) malloc(sizeof(Request)))  == NULL) ) 
   {
   	logPrint(log_fp,"Unable to create allocate memory for the connection structure !!\n");
        goto  GracefulExit;
   }

  memset(conn_rec,0,sizeof(conn_rec));

  conn_rec->req->header=NULL;

  strcpy(header, mystrdup(szbuf));

  if(parseRequest(szbuf,szmethod,szurl,szproto,szargs,BUFSIZ-1) != 0)
  {
	(void) fprintf(stderr," unable to process the request %s \n",szbuf);
        goto  GracefulExit;
  }

    logPrint(log_fp," request %s %s %s from %s \n",szmethod, szurl, szproto,remote_ip->name);

    while(sockGetsSSL(ssl,szbuf,sizeof(szbuf)-1) > 0)
	{
	strcat(header,szbuf);

        val=strchr(szbuf,':');
        if (val)
        {
            *val++ = '\0';
            if (strncasecmp("content-length",szbuf,strlen("content-length")) == 0)
            {
                content_length=atoi(val);
            }
            if (strncasecmp("content-type",szbuf,strlen("content-type")) == 0)
            {
                (void) strcpy(szcontent_type,val);
            }
        }
    }

   if((conn_rec->req->header = (char *) malloc(strlen(header)+1)) == NULL) 
    {
       (void) fprintf(stderr,"unable to allocate memory for conn->req->header\n");
       goto  GracefulExit;
    }
	
   (void) strcpy(conn_rec->req->header,header);

   if(content_length != 0)
   {
     if((conn_rec->req->body = (char *) malloc(content_length+1)) == NULL) 
       {
        (void) fprintf(stderr,"unable to allocate memory for conn->req->body\n");
        goto  GracefulExit;
       }
      sockReadSSL(ssl,conn_rec->req->body,content_length);
    }
     else
        conn_rec->req->body=NULL;

    /*
    ** store the connection information as i need to perform rule matching
    */
    memcpy(conn_rec->cli_ser,remote_ip,sizeof(struct front_server));
    strcpy(conn_rec->req->method,szmethod);
      
    if ((conn_rec->req->url = (char *) malloc(strlen(szurl)+1)) == NULL) 
     {
      (void) fprintf(stderr,"unable to allocate memory for conn->req->urlr\n");
      goto  GracefulExit;
     }
	
    (void) strcpy(conn_rec->req->url,szurl);

    if((conn_rec->req->content_type = (char *) malloc(strlen(szcontent_type)+1)) == NULL)
     {
       (void) fprintf(stderr,"unable to allocate memory for conn->req->content_type \r\n");
       goto  GracefulExit;
     }
	
    (void) strcpy(conn_rec->req->content_type,szcontent_type);


    /*
    ** performing the rule matching
    */
#if 0
    (void) fprintf(stderr, " Request to be served by %x\n", (ret_addr = ruleMatching(conn_rec)));
#endif
    ret_addr = ruleMatching(conn_rec);

    (void) strcpy(TargetServer,getRealServerName(ret_addr));

    if(szbuf == NULL)
    {
    (void) fprintf(stderr, "looks like server list has been modified\n");
    goto  GracefulExit;
    }
	
     conn_rec->req->header = mystrdup(replace(conn_rec->req->header,SERVER_NAME,TargetServer));

#if 0
    (void) printConnection(conn_rec);
    (void) fprintf(stderr, "server to which data is to be sent is %s\n",szbuf);
#endif

   if((server_sock_fd=connectTo(TargetServer,80 )) == -1)
   {
     (void) fprintf(stderr,"Failed to connect to %s\n",TargetServer);
     goto  GracefulExit;
   }

  /* send the request to the real server */
  sockPuts(server_sock_fd, conn_rec->req->header);
  sockPuts(server_sock_fd,"\r\n");

  if(strcmp(conn_rec->req->method, "POST") == 0)
     sockPuts(server_sock_fd,conn_rec->req->body);

  /* freeing & initializing the values*/
  content_length = 0;
  chunked = 0;
  close_connection = 0;
  status_code = 0;

 if((sockGets(server_sock_fd,szbuf,sizeof(szbuf)-1)) <= 0)
    goto  GracefulExit;

 /* check for the status code */
  sscanf(szbuf,"HTTP/%f %d \n",&version, &status_code);

  strcpy(header, mystrdup(szbuf));

  while(sockGets(server_sock_fd,szbuf,sizeof(szbuf)-1) > 0)
  {
   strcpy(szargs,szbuf);
   val=strchr(szbuf,':');

   if(val)
    {
     *val++ = '\0';
     if(strncasecmp("content-length",szbuf,strlen("content-length")) == 0)
       content_length=atoi(val);

     if(strncasecmp("Connection",szbuf,strlen("Connection")) == 0)
	if( strncasecmp(val," close",strlen(" close")) == 0)
          close_connection = 1;

    /*
    **  trying to make sure we follow rfc 2616 section 19.4.6 
    **  HTTP/1.1 introduces the Transfer-Encoding header field.
    **  Proxies/gateways MUST remove any transfer-coding prior to
    **  forwarding a message via a MIME-compliant protocol.
    */

     if(strncasecmp("Transfer-Encoding",szbuf,strlen("Transfer-Encoding")) == 0)
     {
      if( strncasecmp(val," chunked",strlen("chunked")) == 0)
       {
        chunked = 1;
        strcpy(szcontent_type,szargs);
       }
     }

    /*
    ** RFC 2616 sec 13.5.2
    ** A transparent proxy MUST NOT modify any of the following fields in a
    ** request or response, and it MUST NOT add any of these fields if not
    ** already present:
    **
    **  - Content-Location
    **
    **  - Content-MD5
    **
    **  - ETag
    **
    **  - Last-Modified
    **
    */
     /* i am violating this rule */
     if(strncasecmp("Location",szbuf,strlen("Location")) == 0)
     {
     strcpy(szargs,replace(szargs,TargetServer,SERVER_NAME));
     strcpy(szargs,replace(szargs,"http","https"));
     (void) fprintf(stderr,"\nthe new location bar is %s\n",szargs);
     }
    }
  
   strcat(header,szargs);
   }
#if 0
(void) fprintf(stderr,"\n\n the headers to be passed are %s content_type %s args %s",header,szcontent_type,szargs);
#endif

  if(status_code > 400 )
  {
   demoResponse(szurl);
  }
  else if(chunked == 1)
  {
   chunk_size=0;
   *chunked_body='\0';
   content_length=0; /* chunked had more preference than content_length */
   if ( sockGets(server_sock_fd,body,IOBUFFSIZE-1) < 0 )

      fprintf(stderr, "unable to read chunked data");
  
   while((chunk_size=Hex(body)) != 0)
   {
    fprintf(stderr, "szbuf %s chunck size %d \n",body,chunk_size);
    
    content_length = content_length + chunk_size;
  
    while ((chunk_size > 0) && ((data_read=sockRead(server_sock_fd,body,MIN(IOBUFFSIZE-1,chunk_size))) > 0 ))
    {
     chunk_size = chunk_size - data_read;
     strcat(chunked_body,body);
    }
   
    if (sockGets(server_sock_fd,body,IOBUFFSIZE-1) < 0 )
       fprintf(stderr, "unable to read chunked data\n"); 
   }

   /* making changes to the header */
   sprintf(szbuf,"Content-Length: %d\r\n",content_length);
   strcpy(szargs, replace(header,szcontent_type,szbuf));

   strcpy(header,szargs);

  }

 if(status_code < 400)
 {
  /* sending headers to the client */ 
  strcat(header,"\r\n\0");
  (void) fprintf(stderr," headers are %s\n",header);
  sockWriteSSL(ssl,header,strlen(header));

 /* sending the chunked data as body if the header contains Tansfer-Encoding: chunked\r\n*/
  if(chunked == 1)
  {
   sockWriteSSL(ssl,chunked_body,content_length);
   (void) fprintf(stderr," body is %s\n",chunked_body);
   content_length =0;
  }

  for(data_read=0; content_length > 0;data_read=0 )
  {
   if ((data_read = iread(server_sock_fd,body,IOBUFFSIZE-1)) < 0)
    {
      if (errno == ECONNRESET)
         (void) fprintf(stderr,"\nconnection was reset by the server\n");
      else
         (void) fprintf(stderr,"\nunable to read data from the socket\n");

      goto  disGracefulExit;
     }
     content_length -= data_read;
     if(data_read != sockWriteSSL(ssl,body,data_read))
     {
       (void) fprintf(stderr,"pid %d unable to pump more data on content length\n",getpid()); 
       goto  disGracefulExit; 
     }
  }

  (void) fprintf(stderr," pid %d data receive till now are  chunked %d close connection %d\n",getpid(),chunked,close_connection);
 }

  if(close_connection == 1)
    goto GracefulExit;
  else
  {
    sockPutsSSL(ssl,"\r\n");
    free(conn_rec->req->url);
    free(conn_rec->req->header);
    free(conn_rec->req->content_type);
    free(conn_rec->req->body);
    free(conn_rec->req);
    free(conn_rec->cli_ser->name);
    free(conn_rec->cli_ser);
    (void) close(server_sock_fd);
    goto KEEPALIVE;
  }

#if 0
   (void) demoResponse(); 
#endif

GracefulExit:
 sockPutsSSL(ssl,"\r\n");
disGracefulExit:
 free(conn_rec->req->url);
 free(conn_rec->req->header);
 free(conn_rec->req->content_type);
 free(conn_rec->req->body);
 free(conn_rec->req);
 free(conn_rec->cli_ser->name);
 free(conn_rec->cli_ser);
 (void) close(server_sock_fd);
 (void) close(SSL_get_fd(ssl)); /* closing the connection*/
 (void) fprintf(stderr,"\n\n closing the ssl context bye bye %d ..\n",getpid());
  return;
}



static int sslVerifyCallback(int ok,X509 *xs,X509 *xi,int depth,int error, char *arg)
{
    char
        *cip, *issuer_ca, s[64];

    X509_NAME_oneline(X509_get_subject_name(xs),s,64);
    if (s != (char *) NULL)
    {
        if (ok)
        {
            (void) fprintf(stderr,"ok depth=%d,subject=%s\n",depth,s);
        }
        else
            (void) fprintf (stderr,"depth=%d error=%d,subject=%s\n",
                            depth,error,s);
          free(s); 
    }
    X509_NAME_oneline(X509_get_issuer_name(xs),s,64);
    if (s == (char *) NULL)
    {
        (void) fprintf (stderr,"verify error\n");
        ok=0;
    }

    (void) fprintf(stderr,"Issuer=%s\n",s);
    ok=1;
    return (ok);
}


static int ireadSSL(SSL *ssl,char *buf,int nbyte)
{
    int
        ret=0;

    while (1)
    {
        ret=SSL_read(ssl,buf,nbyte);
        if ((ret < 0) && (errno == EINTR))
            continue;
        else
            return (ret);
    }

    return (ret);   /* won't be here */
}


static int sockReadSSL(SSL *ssl,char *buf,size_t n)
{
    int
        total_nbytes=0,
        nbytes,
        saved_errno;

    while (total_nbytes < n)
    {
        nbytes=ireadSSL(ssl,&buf[total_nbytes],
            n-total_nbytes);
        if (nbytes < 0)
        {
            saved_errno=errno;
            if (saved_errno == ECONNRESET)
            {
                return (-3);
            }
            else
                return (-1);
        }
        else if (nbytes == 0)
            return (total_nbytes);
        else
           total_nbytes += nbytes;
    }
    return (total_nbytes);
}


/*
** this function writes a character string out to a socket.
** it returns -1 if the connection is closed while it is trying to
** write
*/
static int sockPutsSSL(SSL *ssl,char *str)
{
    return (sockWriteSSL(ssl,str,strlen(str)));
}

static int sockWriteSSL(SSL *ssl,char *str,size_t count)
{
    size_t
        bytesSent=0;

    int
        thisWrite;

   if (!SSL_is_init_finished(ssl))
    {
      (void) fprintf(stderr,"SSL_is_init_finished(ssl) retuned that there is some problem");
      SSL_accept(ssl);
     }

    while (bytesSent < count)
    {
     do
      {
       thisWrite=SSL_write(ssl,str,count-bytesSent);
       switch (SSL_get_error(ssl,thisWrite))
       {
         case SSL_ERROR_NONE		   :
                              		    break;
         case SSL_ERROR_WANT_WRITE	   :
         case SSL_ERROR_WANT_READ	   :
         case SSL_ERROR_WANT_X509_LOOKUP   :
                                            fprintf(stderr,"pid %d ssl Write BLOCK\n",getpid());
                                            return -1;
                                            break;
         case SSL_ERROR_SYSCALL		   :
                                 	    fprintf(stderr," pid %d Peek_error %d SSL_ERROR_SYSCALL %d\n",getpid(),ERR_peek_error, ERR_get_error());
                                            return -1;
                                  	    break; 
         case SSL_ERROR_SSL		   :
                                 	    fprintf(stderr,"pid %d SSL_ERROR_SSL %d\n",getpid(), ERR_get_error());
                                            return -1;
                                  	    break; 
         case SSL_ERROR_ZERO_RETURN	   :
                                            fprintf(stderr,"pid %d SSL_ERROR_ZERO_RETURN %d \n",getpid(), ERR_get_error());
                                            return -1;
                                            break;
          }

        } while ((thisWrite < 0) && (errno == EINTR));

       if (thisWrite <= 0)
        {
         (void) fprintf(stderr,"pid %d unable to write to the socket\n",getpid());
          return (thisWrite);
        }

        bytesSent += thisWrite;
        str += thisWrite;
    }
    return (count);
}


static int sockGetsSSL(SSL *ssl,char *str,size_t count)
{
  int
     bytesRead,
     totalCount=0;

  char
     *currentPosition,
     lastRead=0;

  currentPosition=str;

  while (lastRead != 10)
  {
    bytesRead=SSL_read(ssl,&lastRead,1);
    if (bytesRead <= 0)
     {
      /*
      ** the other side may have closed unexpectedly
      */
       return (-1);
     }

    if ((totalCount < count) && (lastRead != 10) && (lastRead != 13))
     {
        *currentPosition=lastRead;
        currentPosition++;
        totalCount++;
     }
  }

   *currentPosition='\r';
   currentPosition++;
   *currentPosition='\n';
   currentPosition++;

   if(count > 0)
      *currentPosition=0;

    return (totalCount);
}

/*======================= Rule Matching Code =======================*/


void clear_rule_fld_values()
{
 int i = 0;
 for(i=0; i < num_rule_fields;i++)
   rule_fields[i].value[0] = '\0';

 return;
}

u_long route_to(char *server_name, int sticky_flag, u_long saddr)
{
 int i=0;
 u_long dest_addr = 0;

 for(i=0; i<num_servers; i++)
 {
  if(strcmp(server_name, server_list[i].name) == 0)
  {
    if(sticky_flag == STICKY_ON_IP)
    {
     addSticky(saddr,server_list[i].addr);
    }
    dest_addr = server_list[i].addr;
    break;
  }
 }
 return dest_addr;
}


u_long rule_configure(char *url, u_long saddr,u_long daddr, u_short sport, u_short dport, u_short protocol)
{

if (strstr(url,"cs522") != NULL) {
  (void) fprintf(stderr,"pid %d url %s match found with rule 1\n",getpid(), url);
  return route_to("frodo.uccs.edu",NON_STICKY,saddr);
}
if (strstr(url,"cs301") != NULL) {
  (void) fprintf(stderr,"pid %d url %s match found with rule 2\n",getpid(), url);
  return route_to("eca.uccs.edu",NON_STICKY,saddr);
}

if (strstr(url,"cs") != NULL) {
  (void) fprintf(stderr,"pdi %d url %s match found with rule 3\n",getpid(), url);
  return route_to("frodo.uccs.edu",NON_STICKY,saddr);
}

if (strstr(url,"keepalive") != NULL) {
  (void) fprintf(stderr,"pid %d url %s match found with rule 5\n",getpid(), url);
  return route_to("eca.uccs.edu",NON_STICKY,saddr);
}

if (strstr(url,"gkgodava") != NULL) {
  (void) fprintf(stderr,"pid %d url %s match found with rule 4\n",getpid(), url);
  return route_to("frodo.uccs.edu",NON_STICKY,saddr);
}


#if 0
if (strstr(url,"purchase.pl") != NULL) {
  return route_to("frodo.uccs.edu",NON_STICKY,saddr);
}

if (saddr == 0x80c6c0ca){
  (void) fprintf(stderr,"pid  %d match found with rule 6\n",getpid(), saddr);
  return route_to("reject",NON_STICKY,saddr);
}

if (saddr == 0x80c63c17){
  (void) fprintf(stderr,"pid %d  %x match found with rule 7\n",getpid(), saddr);
  return (u_long)  route_to("eca.uccs.edu",NON_STICKY,saddr);
}

if (saddr == 0x80c6a2d9){
  (void) fprintf(stderr," pid %d match found with rule 8\n",getpid());
  return route_to("eca.uccs.edu",NON_STICKY,saddr);
}
#endif

if ((atoi(rule_fields[1].value) > 0) && (atoi(rule_fields[1].value) <50000)){
  (void) fprintf(stderr," pid %d xml parsing match found --1 with rule 9\n",getpid());
  return route_to("frodo.uccs.edu",NON_STICKY,saddr);
}

if (atoi(rule_fields[1].value) > 50000) {
  (void) fprintf(stderr,"pid %d  xml parsing match found --2 with rule 10\n",getpid());
  return route_to("eca.uccs.edu",NON_STICKY,saddr);
}

if (strstr(url,"lcs1") != NULL) {
  (void) fprintf(stderr,"pid %d url %s match found with rule11 \n",getpid());
  return route_to("frodo.uccs.edu",NON_STICKY,saddr);
}

if (strstr(url,"lcs2") != NULL) {
  (void) fprintf(stderr,"pid %d url %s match found with rule12 \n",getpid());
  return route_to("eca.uccs.edu",NON_STICKY,saddr);
}

  (void) fprintf(stderr,"default rule match found \n");
  return route_to("frodo.uccs.edu",NON_STICKY,saddr);
}


char *mstrstr(char *source,  char *substr, size_t n)
{
  char *p = NULL;
  int  subStrLen = 0;

  p = strstr(source, substr);

  if(p == NULL)
  {
    subStrLen =  strlen(substr);
    if(!memcmp(source + n - subStrLen, substr, subStrLen))
      p = source + n - subStrLen;
  }
  return p;
}

int xml_tag_parser(char *buf)
{
  int   buf_len =strlen(buf);
  int   ret = 0;
  int   field_count = 0;
  short done_yet = 0;
  int   skip_len = 0;
  int   pos = 0;
  char  tag[64];
  char  pos_str[10];
  char  *p = NULL;
  char  *p1 = NULL;
  char  *fld_spec_beg = NULL;
  char  *fld_spec_end = NULL;
  char  *data_beg = NULL;
  char  *data_end = NULL;
  char  *tag_beg = NULL;
  char  *tag_end = NULL;


  for(field_count = 0; field_count < num_rule_fields; field_count++)
  { 
    fld_spec_beg = &(rule_fields[field_count].name[0]);
    fld_spec_end = &(rule_fields[field_count].name[strlen(rule_fields[field_count].name)]);
    data_beg = buf;
    data_end =(char *) (buf+buf_len);

    while(fld_spec_beg < fld_spec_end)
    {
      p = (char *)memchr(fld_spec_beg, ':', fld_spec_end - fld_spec_beg);

      if(!p)
      {
        (void) fprintf(stderr,"Invalid rule field specification '%s'\n", rule_fields[field_count].name);
        return -1;
      }

      memcpy(tag, fld_spec_beg, p - fld_spec_beg);
      tag[p - fld_spec_beg] = '\0';

      p += 1;
      p1 = (char *)memchr(p, '.', fld_spec_end - p);


      if(!p1)
      {
        (void) fprintf(stderr,"Invalid rule field specification '%s'\n", rule_fields[field_count].name);
        return -1;
      }

      memcpy(pos_str, p, p1 - p);

      pos_str[p1 - p] = '\0';
      fld_spec_beg = ++p1;

      pos = atoi(pos_str); 
    
      get_tag_range(tag, pos, data_beg, data_end, &tag_beg, &tag_end);
      if((!tag_beg) || (!tag_end))
      {
        (void) fprintf(stderr,"get_tag_range() returned NULL either for tag_beg or tag_end, "
               "tag_beg=%u, "
               "tag_end=%u, "
               "tag=%s, "
               "pos=%d ",
                ((tag_beg) ? (unsigned int) tag_beg : 0),
                ((tag_end) ? (unsigned int) tag_end : 0),
                tag,
                pos);

        return -1;
      }

      if(fld_spec_beg == fld_spec_end)
      {
        /* We have parsed all the sub fields of the field specified in the rule.
        ** Now parse data between tag_beg and tag_end and store in the field value
        ** after skipping white spaces, carriage returns and line feeds.
        */
        done_yet = 0;
        while((tag_beg < tag_end) && !done_yet)
        {
          done_yet = 1;
          skip_len = len_sp;
          if(!memcmp(tag_beg, c_sp, skip_len))
          {
            tag_beg += skip_len;
            done_yet = 0;
          }

          skip_len = len_cr;

          if(!memcmp(tag_beg, c_cr, skip_len))
          {
            tag_beg += skip_len;
            done_yet = 0;
          }

          skip_len = len_lf;
          if(!memcmp(tag_beg, c_lf, skip_len))
          {
            tag_beg += skip_len;
            done_yet = 0;
          }
        }

        done_yet = 0;

        while((tag_beg < tag_end) && !done_yet)
        {
          skip_len = len_sp;
          if(memcmp(tag_end - skip_len, c_sp, skip_len))
            skip_len = 0;

          if(!skip_len)
          {
            skip_len = len_cr;
            if(memcmp(tag_end - skip_len, c_cr, skip_len))
              skip_len = 0;
          }
         
          if(!skip_len)
          {
            skip_len = len_lf;
            if(memcmp(tag_end - skip_len, c_lf, skip_len))
              skip_len = 0;
          }
         
          if(skip_len)
            tag_end -= skip_len;
          else
            done_yet = 1;
        }

        if(tag_beg != tag_end)
        {
          if((tag_end - tag_beg) < sizeof(rule_fields[field_count].value))
          {
            memcpy(rule_fields[field_count].value, tag_beg, tag_end - tag_beg);
            rule_fields[field_count].value[tag_end - tag_beg] = '\0';
          }
          else
            (void) fprintf(stderr,"tag value size=%d is larger than its container size=%d for rule tag '%s' of field spec '%s'\n", (tag_end - tag_beg), sizeof(rule_fields[field_count].value), tag, rule_fields[field_count].name);
        }
        else
          (void) fprintf(stderr,"No value found for rule field spec '%s'\n", rule_fields[field_count].name);
      }
      else
      {
        data_beg = tag_beg;
        data_end = tag_end;
      }
    }
  } 
  return ret;
}



int get_tag_range(char *tag, int pos, char *hay_begin, char *hay_end, char **needle_begin, char **needle_end)
{
  char  temp_buf[67];
  int   curr_pos = 0;
  int   skip_len = 0;
  int   hay_len = 0;
  int   buf_len = 0;
  char  *p = NULL;
  char  *p_tmp = NULL;
  char  *p1 = NULL;
  char  *p_end = NULL;
  char  *p_beg = NULL;
  char  *tag_left = NULL;
  char  *tag_right = NULL;
  
  hay_len = hay_end - hay_begin;
  buf_len = hay_len;
  p = hay_begin;

  while(curr_pos < pos)
  {
    /*
    ** First Remove any leading comments
    */
    sprintf(temp_buf, "%s%s--", c_la, c_ex);


    if((p_tmp = mstrstr(p, temp_buf, hay_len)) != NULL)
    {
      /*
      ** Found the beginning of comment marker "<!--"
      ** Now find the end of comment marker "-->"
      */
       
	sprintf(temp_buf, "--%s", c_ra);


       if((p1 = mstrstr(p_tmp, temp_buf, buf_len - (p_tmp - hay_begin))) != NULL)
       {
         p = p1 + strlen(temp_buf);
         hay_len = buf_len - (p - hay_begin);
         continue;
       }
       else
       {
         (void) fprintf(stderr,"Bad xml data, no ending comment tag\n");
         break;
       }

    }

    sprintf(temp_buf, "%s%s", c_fs, tag);
    /*
    ** First look for end position "/tag" i.e., the vicinity of </tag> 
    */

    if((p_end = mstrstr(p, temp_buf, hay_len)) != NULL)
    {
      curr_pos++;
      if(curr_pos == pos)
      {
        /*
        ** Now look for begin position of the "tag" i.e., the vicinity of <tag>
        */

        if((p_beg = mstrstr(p, tag, hay_len)) != NULL)
        {
          /*
          ** First verify that it is a real tag 
          */

           tag_left = p_beg;
           tag_right = p_beg + strlen(tag);

          /*
          ** i.e, Below is a check for situations like <sometag> AtagB </sometag>
          */

           if(( (memcmp(tag_left - len_sp, c_sp, len_sp)) &&
                (memcmp(tag_left - len_ra, c_la, len_la)) &&
                (memcmp(tag_left - len_cr, c_cr, len_cr)) &&
                (memcmp(tag_left - len_lf, c_lf, len_lf)) ) || 
              ( (memcmp(tag_right, c_sp, len_sp)) &&
                (memcmp(tag_right, c_ra, len_ra)) &&
                (memcmp(tag_right, c_cr, len_cr)) &&
                (memcmp(tag_right, c_lf, len_lf)) ) )
           {
             p = tag_right;
             hay_len = buf_len - (p - hay_begin);
             --curr_pos;
             continue;
           }

           p_beg = tag_right;

           /*
           ** Find the position of '>' for p_beg
           */
           p = mstrstr(p_beg, c_ra, p_end - p_beg);

           if(p == NULL)
           {
             p_beg = NULL;
             p_end = NULL;
             break;
           }

           /*
           ** Find the position of '<' for beginning from p_beg
           */
           p_tmp = mstrstr(p_beg, c_la, p_end - p_beg);

           /*
           ** Below is a check for <sometag> tag </sometag>
           */

           if(p_tmp && (p_tmp < p))
           {
             hay_len = buf_len - (p - hay_begin);
             --curr_pos;
             continue;
           }

          /*
          ** Now eliminate all unwanted characters between p_beg and p_end
          ** such as white space, line feed, carriage return until until
          ** right and left anchors are seen respectively
          */

           if(p != NULL)
             p_beg = p + len_ra;
           else
           {
             /*
             ** We didn't find the  '>' for p_beg
             */
             p_beg = NULL;
             p_end = NULL;
             break;
           }

           /*
           ** Find the position of left anchor for p_end
           */

           while((p_end - max_char_len) > p_beg)
           {
             skip_len = len_la;
             if(!memcmp(p_end - skip_len, c_la, skip_len))
             {
               p_end -= skip_len;
               break;
             }

             skip_len = len_sp;
             if(!memcmp(p_end - skip_len, c_sp, skip_len))
               p_end -= skip_len;


             skip_len = len_cr;
             if(!memcmp(p_end - skip_len, c_cr, skip_len))
               p_end -= skip_len;

             skip_len = len_lf;
             if(!memcmp(p_end - skip_len, c_lf, skip_len))
               p_end -= skip_len;
           }
        }
      }
      else
      {
        /*
        ** Advance p for the next <tag>...data...</tag> block
        */
        p = p_end + strlen(temp_buf);
        hay_len = buf_len - (p - hay_begin);
      }
    }
    else
    /*
    ** No specified needle found in the hay
    */
      break;

  }

  *needle_begin = p_beg;
  *needle_end = p_end;
  
 return 0;

}

u_long ruleMatching(Connection *conn)
{
  u_long ret_addr = 0;
  int ret=0,i=0;

  if((ret_addr = findSticky(conn->cli_ser->addr)) > 0)
     return ret_addr;

  (void) fprintf(stderr, "the value returned by findSticky is %x \n",ret_addr);

  if(strcmp(conn->req->method, "POST") == 0)
  {
    if(strstr(conn->req->content_type,"x-www-form-urlencoded") != NULL)
       (void) fprintf(stderr, " encoded body when decoded =>%d\n",(ret=xml_tag_parser(strDecode(conn->req->body))));

    for(i=0;i < num_rule_fields;i++)
     (void) fprintf(stderr, "rule %s value %s\n ", rule_fields[i].name, rule_fields[i].value);
  }

  if(ret >= 0)
  {
     ret_addr = (u_long)  rule_configure(conn->req->url,conn->cli_ser->addr, (u_long) SERVER_IP, (u_short) conn->cli_ser->port, (u_short) SERVER_PORT,12);
     clear_rule_fld_values();
  }

  return ret_addr;

}


/*======================= Rule Matching Code =======================*/
