/*
ServerProbing,cpp

Author: Wen-Che Wu

Date : 2001/April


*/


#include <vxWorks.h>
#include <sockLib.h>
#include <inetLib.h>
#include <stdioLib.h>
#include <strLib.h>
#include <hostLib.h>
#include <ioLib.h>
#include <timers.h>
#include <stdlib.h>
/*copy from example*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <taskLib.h>
#include "npvs.h"
#include "npvs_sim.h"
#include "srvprb.h"
#define quo_dbg


struct prb_cnfg config[MAX_CLUSTER];

/* value = 0 for end of list, 1 for temporally remove the entry */
struct server_address srv_addr[NPVS_REAL_SERVER_MAX+1];


/* for server average response time
used for smoothing the integer time vlaue  */
double srv_rspnd_tm[NPVS_REAL_SERVER_MAX]; /* first 0 for end of cluster, -1 for temporaly remove from list */



void probe_test(void)
{
	int port_no = 80;
	config[0].prb_id = 0;
	config[0].prb_run = 1; /*use for start the probing code*/
	config[0].request_message = (char*)"GET HTTP/1.1\n";
	config[0].history_weight = 0.9;
	config[0].probing_function = server_probing;
	config[0].weight_calc = smooth_time_calc;
	
	/* initialize the original value*/
	for(int i = 0; i<NPVS_REAL_SERVER_MAX; i++)	
	{
		srv_addr[i].ip = 0;
		srv_addr[i].port = 0;
		srv_rspnd_tm[i] = 0;
		
	}
	/* set first one for testing */
	//srv_addr[0].ip = inet_addr("140.96.115.73");
	//srv_addr[1].ip= inet_addr("140.96.115.31");
	//srv_addr[2].ip= inet_addr("140.96.115.5");
	srv_addr[0].ip = inet_addr("128.198.60.20");
	srv_addr[1].ip= inet_addr("128.198.60.27");
	//srv_addr[1].ip= inet_addr("128.198.60.51");
	//srv_addr[2].ip= inet_addr("128.198.60.52");
	srv_addr[0].port = htons (80);
	srv_addr[1].port = htons (80);	
	//srv_addr[1].port = htons (80);
	//srv_addr[2].port = htons (80);
	//srv_ip_addr[2] = inet_addr("140.96.206.37");
	
	probe_main(&config[0], srv_rspnd_tm, srv_addr);

}





/*******************************************************************************
 *
 * prob_main - loop to probe all server in the cluster
 *
 * parameters:
 *	config: the configuration for the probing action
 *	srv_rspnd_tm : head of server response time array
 *	clstr_addr:	server address in the cluster,  
 *	port_no: the port to be probed
 */

void probe_main(struct prb_cnfg * config, double * srv_rspnd_tm, struct server_address * clstr_addr)
{
	SOCK_FD sock_fd;
	struct timespec time_diff;
	struct sockaddr_in srv_addr;
	int crnt_srv = 0;

#ifdef quo_dbg
	int count = 0;
#endif
	
	/* Initialize servers address */
	bzero ((char *)&srv_addr, sizeof(srv_addr));
	srv_addr.sin_family		= AF_INET;	
	/*srv_addr.sa_len = (u_char) sizeof(srv_addr);*/
	
	
	while(config->prb_run != 0)
	{
		if (clstr_addr[crnt_srv].ip == 0)
		{
			crnt_srv = 0; /* roll back to first one */
			continue;
		} else if (clstr_addr[crnt_srv].ip == 1)
		{
			crnt_srv = crnt_srv + 1;
			continue;
		}
		/* Create a socket */
		if ( (sock_fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 )
			error ("Socket failed");

		srv_addr.sin_addr.s_addr = clstr_addr[crnt_srv].ip;
		srv_addr.sin_port = clstr_addr[crnt_srv].port;

#ifdef quo_dbg
		printf("\nThe server ip is %s, port is %d\n",inet_ntoa(srv_addr.sin_addr.s_addr), ntohs(clstr_addr[crnt_srv].port));
#endif

		if (config->probing_function == NULL ||
		 config->probing_function(sock_fd, &srv_addr, config->request_message, &time_diff) == ERROR )
			error ("Probe failed");
		close(sock_fd);
#ifdef quo_dbg
		printf("Response time: %dsec + %d\n",time_diff.tv_sec, time_diff.tv_nsec);
#endif
		
		if (config->weight_calc(&srv_rspnd_tm[crnt_srv], &time_diff, config->history_weight) <0 )
			error ("Time index update failed");
		crnt_srv = crnt_srv + 1;
#ifdef quo_dbg
		count++;
		printf("Count = %d\n",count);
#endif
		
	}
	
}

/*
void sub_clstr_probing(struct prb_cnfg * config, unsigned long * clstr_addr, int clstr_len, char * request_message)
{	int crrnt_srv,i;
	crrnt_srv = 0;
	struct timespec start_time, finish_time;
	SOCK_FD sock[MAX_SOCKET_PER_PROCESS];
	struct sockaddr_in srv_addr;
	 Initialize servers address 
	bzero ((char *)&srv_addr, sizeof(srv_addr));
	srv_addr.sin_family		= AF_INET;	
	srv_addr.sin_port		= htons (port_no);
	
	
	for (i = 0; i < MAX_SOCKET_PER_PROCESS, i++)
	{	if ( (sock[i] = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 )
			error ("Socket failed");
	}
	for (i = 0; i < MAX_SOCKET_PER_PROCESS, i++)
	{	if ( crrnt_srv < clstr_len )
			crrnt_srv = crrnt_srv+1;
		else
			crrnt_srv = 0;
	}
	
} */



/***************************************************************************
* server_probing
* Give a socket and server address, it returns the response time for HTTP
*
* parameters:
* sock_fd:	generated socket id, int
* srv_addr:	structed server address
* request_message: the message used to test the response
* time_diff:	use for carry time_diff
*
*/

int server_probing(SOCK_FD sock_fd, struct sockaddr_in * srv_addr, char* request_message, struct timespec * time_diff )
{
	struct timespec start_time, finish_time;
	char response[MAX_RESPONSE_SIZE];
	int ix;

	/*get current system time*/
	clock_gettime(CLOCK_REALTIME, &start_time);
	/* Connect to server */
	if (connect(sock_fd, (struct sockaddr *) srv_addr, sizeof(struct sockaddr_in)) ==ERROR)
	{
		close(sock_fd);
		error ("conect failed");
	}

	if (write(sock_fd, request_message, strlen(request_message) + 1) < strlen (request_message) + 1)
	{
		close(sock_fd);
		error ("write failed");
	}
	if (read(sock_fd, response, MAX_RESPONSE_SIZE)<0)
	{				
		close(sock_fd);
		error ("read failed");
	}
#ifdef quo_dbg
		printf("Return message: %s\n", response);
#endif
	shutdown(sock_fd,2);

	clock_gettime(CLOCK_REALTIME, &finish_time);
	time_diff->tv_sec = finish_time.tv_sec - start_time.tv_sec;
	if((time_diff->tv_nsec = finish_time.tv_nsec - start_time.tv_nsec) < 0)
	{
		time_diff->tv_sec = time_diff->tv_sec - 1;
		time_diff->tv_nsec = (1000000000 + time_diff->tv_nsec);
	}
#ifdef quo_dbg
	printf("Finish ns =%d, Start ns = %d, Original ns value is %d\n", finish_time.tv_nsec, start_time.tv_nsec, time_diff->tv_nsec);
#endif
	return 1;
}



/************************************************************************
 *
 * error - print an error message and exit.
 *
 */
void error (char * pStr)
{
	perror (pStr);
	exit (1);
}

/************************************************************************
*	pserver - for testing socket program
*
*
*/
void pserver_init(void)
{
	SOCK_FD sock_fd,new_fd;
	struct timespec time_diff;
	unsigned long srv_ip_addr;
	struct sockaddr_in srv_addr, clnt_addr;
	char response[MAX_RESPONSE_SIZE];
	
	/*for testing*/
	srv_ip_addr = inet_addr(VIP);
	
	/* Initialize servers address */
	bzero ((char *)&srv_addr, sizeof(srv_addr));
	srv_addr.sin_family		= AF_INET;	
	srv_addr.sin_port		= htons (PORT_NO);
	srv_addr.sin_addr.s_addr = htonl(INADDR_ANY);
	
	/* Create a socket */
	if ( (sock_fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0 )
		error ("Socket failed");
	
	if ( bind(sock_fd, (struct sockaddr*) &srv_addr, sizeof(struct sockaddr_in)) == ERROR)
	{
		perror("BIND");
		close(sock_fd);
		exit(1);
	}
	
	if(listen(sock_fd, 1) == ERROR)
	{
		perror("listen");
		close(sock_fd);
		exit(1);
	}
	if((new_fd = accept(sock_fd, (struct sockaddr *) &clnt_addr, (int*)IPPROTO_TCP))==ERROR)
	{	perror("Accept");
		close(sock_fd);
		exit(1);
	}

	if (read(new_fd, response, MAX_RESPONSE_SIZE)<0)
	{				
		close (sock_fd);
		error ("read failed");
	}
	printf("Return message from server :%s\n", response);
	/*reuse reponse as sending out string*/
	if (write(new_fd, response, strlen(response) + 1) < strlen (response) + 1)
	{
		close (sock_fd);
		error ("write failed");
	}
	close(new_fd);
	close (sock_fd);
}	


/***************************************************************************
* probing_action
* Turn on or off the probing activity : 0->off, others->on
*
* parameters:
* id	: Probing group id
* new_value :   New value for control the probing action
*
*/

void probing_action(int id, int new_value)
{	
	config[id].prb_run = new_value;	
}

/***************************************************************************
* config_rqst_msg
* set the request message
*
* parameters:
* id	: Probing group id
* new_value :   New value for control the probing action
*
*/
void config_rqst_msg(int id, char * msg)
{	config[id].request_message = msg;
}

/***************************************************************************
* smooth_time_calc
* calculate the smoothed response time
*
* parameters:
* prev :   previous record of response time
* new_value : newest probing value of response time
* pre_weight: weight of the previous record
*
*/	
int smooth_time_calc(double * prev, struct timespec * new_value, double pre_weight)
{	
	double temp = 0;
	temp = (double)(new_value->tv_sec*1000 + new_value->tv_nsec/1000000);
	if( (*prev = *prev * pre_weight + temp * (1- pre_weight)) < 0 )
		return -1;
	return 0;
}


/***************************************************************************
* print_clstr_rspnd_tm
* --print the all value of repsonde time in one cluster
*
* parameters:
* rspnd_tm: the array for the response time of all server
*
*/	
void print_clstr_rspnd_tm(double * rspnd_tm)
{	int i = 0;
	for (i = 0; i<NPVS_REAL_SERVER_MAX; i++)
	{	if( rspnd_tm[i] != 0)
		{	printf("Srv_no = %3d, response time = %f\n", i, rspnd_tm[i]);
		}
	}
}

