#ifndef _NPVS_H
#define _NPVS_H

#define NPVS_CONN_CACHE_BITS  8
#define NPVS_CONN_CACHE_SIZE  (1 << NPVS_CONN_CACHE_BITS)
#define NPVS_CONN_CACHE_MASK  (NPVS_CONN_CACHE_SIZE - 1)

#define NPVS_ARP_CACHE_BITS 8
#define NPVS_ARP_CACHE_SIZE (1 << NPVS_ARP_CACHE_BITS)
#define NPVS_ARP_CACHE_MASK (NPVS_ARP_CACHE_SIZE - 1)

#define NPVS_REAL_SERVER_MAX 256

//functions defined in Npvsadm.cpp
extern void create_cluster(void);
extern void edit_cluster(void);
extern void delete_cluster(void);
extern void show_cluster(void);
extern void show_vs_node(struct vs_node *_vsnd);
extern void show_rs_node(struct rs_node *_rsnd, unsigned int _sch);
extern struct cluster *select_cluster_menu(unsigned int *_ch);
extern void create_virtual_service(struct cluster *_clstr);
extern void npvs_cmd_shell(void);

//functions defined in NpvsSched.cpp
extern void rr_scheduler(struct vs_node *_vsnd);
extern void wrr_scheduler(struct vs_node *_vsnd);
extern void lc_scheduler(struct vs_node *_vsnd);
extern void wlc_scheduler(struct vs_node *_vsnd);

//funnctions defined in NpvsMainProcess.cpp
extern unsigned char scan_all_cluster_for_matched_ip(unsigned long dest_ip,struct cluster **_clstr);
extern unsigned char scan_all_vs_node_for_matched_protocol_and_port(struct ip *_iphdr, struct vs_node *_vsnd);
extern void dr_process(struct ether_header *__ethdr, struct rs_node *_rsnd);
extern void npvs_main_process(struct ether_header *_ethdr);

extern unsigned int npvs_arp_hash_key(unsigned long __rsip);
extern unsigned char *npvs_arp_hash_lookup(unsigned long _rsip);
extern void npvs_arp_add_hash(unsigned long _rsip, unsigned char *_mac);
extern unsigned int npvs_conn_hash_key(unsigned char _proto, unsigned long _saddr, unsigned short _sport);
extern struct conn_cache *npvs_conn_hash_lookup(struct ip *_iphdr, struct conn_cache **_last);
extern void npvs_conn_add_hash(struct ether_header *__ethdr, struct rs_node *_rsnd);
extern void npvs_conn_remove_hash(struct conn_cache *_last, struct conn_cache *_remv);

//structures
struct rs_node
{
	unsigned long rs_ip;
	unsigned int redirect_port;	/* TCP or UDP port number */
	unsigned int redirect_type;	/* 0:NAT, 1:DR, 2:TUN */
	
	//under testing for direct routing
	unsigned char redirect_dmac[6];
	
	unsigned int wght;		/* 0:non-use */
	unsigned int wght_count;
	unsigned int connect_num;	
	struct rs_node *next;
};

struct vs_node
{
	//unsigned long vs_ip;
	unsigned int tcp_or_udp;	/* 0:tcp 1:udp */
	unsigned int vs_port;
	unsigned int sched_type;	/* 0:rr 1:wrr 2:lc 3:wlc Others:none */
	void (*vs_scheduler)(struct vs_node *);
	unsigned int rs_num;
	struct rs_node *rs_head;	/* pointer to the first rs_node */
	struct rs_node *rs_cur;
	struct vs_node *next;
};


struct cluster
{
	unsigned long vs_ip;
	struct vs_node *vs_head;
	struct cluster *next;
};

struct conn_cache
{
	unsigned char proto;
	unsigned long saddr;
	unsigned short sport;
	//corresponding to which real server
	struct rs_node *rs_nd;
	struct conn_cache *next;
};

struct arp_cache
{
	unsigned long a_ip;
	unsigned char a_mac[6];
	struct arp_cache *next;
};

//extern the global variable
extern struct  cluster *clstr_head;
//extern unsigned char arp_cache_update_flag = 0; //0: need to be update; 1: not

extern struct conn_cache *npvs_conn_cache[NPVS_CONN_CACHE_SIZE];
extern struct arp_cache *npvs_arp_cache[NPVS_ARP_CACHE_SIZE];

#else
#endif